#include "qlautoconf.h"

#include <stdio.h>
#include <unistd.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <time.h>
#include <signal.h>
#ifndef __APPLE__
#include <mntent.h>
#include <asm/types.h>
#endif // __APPLE__
#include <sys/types.h>
#include <errno.h>

#ifndef CONFIG_HOST_PLATFORM_USERSPACE
#include <linux/videodev2.h>
#else
#include "uservideodev2.h"
#endif

#include "qptypes.h"
#include "pldevioctl.h"
#include "qlstrm.h"
#include "mp4_file.h"

#define FTYP_SIZE   0x1c
#define MVHD_SIZE   0x6c
#define TKHD_SIZE   0x5c
#define ELST_SIZE   0x1c
#define EDTS_SIZE   ELST_SIZE + 0x08    // 0x24
#define MDHD_SIZE   0x20
#define HDLR_SIZE   0x34
#define VMHD_SIZE   0x14
#define SMHD_SIZE   0x10
#define URL_SIZE   0x0c
#define DREF_SIZE   URL_SIZE + 0x10     // 0x1c
#define DINF_SIZE   DREF_SIZE + 0x08    // 0x24
#define ESDS_SIZE   0x27
#define MP4A_SIZE   ESDS_SIZE + 0x24    //0x4b
#define STSD_SIZE   MP4A_SIZE + 0x10  //0x5b
#define STTS_SIZE   0x18
#define STSC_SIZE   0x1c

#if defined (__APPLE__)
#define FILEPOS_ASSIGN(x,y)   { x = y; }
#else
#define FILEPOS_ASSIGN(x,y)   { x.__pos = y; }
#endif

char QT_RECORD[2][5] =
{
    {0x1, 0x18, 0x1, 0x4, 0x1c},
    {0x1, 0x17, 0x1, /*0x5*/0x4, 0x1b},
};

unsigned char qpDecoderSR[4][4] =
{
    {0x05, 0x02, 0x12, 0x10},
    {0x05, 0x02, 0x11, 0x90},
    {0x05, 0x02, 0x12, 0x90},
    {0x05, 0x02, 0x15, 0x90},
};

struct ATOM_SIZE
{
    unsigned int addr;
    //unsigned int size;
};
//QT_TRANSCODE_INFO *pqtinfo_static= 0;

TQP_UINT32 qlputtag( unsigned char *buff, char *tag )
{
    if( !buff )
        return 0;
    unsigned int s = (unsigned int)strlen( tag );
    int t = s;
    while( s )
    {
        buff[t-s] = tag[t-s];
        s--;
    }
    return t;
}
TQP_UINT32 qlputbyte( unsigned char *buff, unsigned char b )
{
    if( !buff )
        return 0;
    *buff = b;
    return 1;
}
TQP_UINT32 qlput2byte( unsigned char *buff, unsigned short val )
{
    if( !buff )
        return 0;
    buff[0] = ((val & 0xff00) >> 8);
    buff[1] = val & 0xff;
    return 2;
}
TQP_UINT32 qlput4byte( unsigned char *buff, TQP_UINT32 val )
{
    if( !buff )
        return 0;
    buff[0] = ((val & 0xff000000) >> 24);
    buff[1] = ((val & 0x00ff0000) >> 16);
    buff[2] = ((val & 0x0000ff00) >> 8);
    buff[3] = val & 0xff;
    return 4;
}

QT_TRANSCODE_INFO * mp4_init(FILE * mp4_clip_file_handle,
                FILE * mp4_vstts, FILE * mp4_vstsz, FILE * mp4_vstco, FILE * mp4_vstsc,
                FILE * mp4_astts, FILE * mp4_astsz, FILE * mp4_astco, FILE * mp4_astsc,
                component_t *pComponent)
{
    QT_TRANSCODE_INFO *pqtinfo= malloc(sizeof(QT_TRANSCODE_INFO));
    FILE *pFile= mp4_clip_file_handle;
    TQP_UINT32 err = 0;
    int temp = 0xff;

    if(pqtinfo)
    {
        memset(pqtinfo, 0, sizeof(QT_TRANSCODE_INFO));
//        pqtinfo_static = pqtinfo;
        pqtinfo->vbuff.buff = (char *)malloc( QT_VBUFF );
		if (pqtinfo->vbuff.buff==NULL)
		{
			free(pqtinfo);
            printf( "no memory for pqtinfo->vbuff.buff\n" );
            return 0;
		}
        pqtinfo->vbuff.size = QT_VBUFF;
        pqtinfo->vbuff.offset = 0;
        pqtinfo->mp4_handle = mp4_clip_file_handle;

        pqtinfo->fp_vsttsp = mp4_vstts;
        pqtinfo->fp_vstszp = mp4_vstsz;
        pqtinfo->fp_vstcop = mp4_vstco;
        pqtinfo->fp_vstscp = mp4_vstsc;
        pqtinfo->fp_asttsp = mp4_astts;
        pqtinfo->fp_astszp = mp4_astsz;
        pqtinfo->fp_astcop = mp4_astco;
        pqtinfo->fp_astscp = mp4_astsc;

        pqtinfo->dAudio = 0;
        pqtinfo->dVideo = 0;
        pqtinfo->aChunkCnt = 0;
        pqtinfo->vChunkCnt = 0;
        pqtinfo->nvstts = 0;
        pqtinfo->nvstsz = 0;
        pqtinfo->nvstco = 0;
        pqtinfo->vstsc = 0;
        pqtinfo->nvstsc = 0;
        pqtinfo->nastts = 0;
        pqtinfo->nastsz = 0;
        pqtinfo->nastco = 0;
        pqtinfo->astsc = 0;
        pqtinfo->nastsc = 0;
		pqtinfo->data_buff = NULL;

        pqtinfo->avstart_address = 0x68;
        pqtinfo->file_offset = 0x00;

        temp = fseek( pFile, pqtinfo->file_offset, SEEK_SET );
        if( temp)
        {
            if (pqtinfo)
				free(pqtinfo);
            printf( "cannot seek to QMP_TOTAL\n" );
            return 0;
        }

        pqtinfo->vsttsp = malloc( QMP_S_STTSP );
        if( !pqtinfo->vsttsp )
        {
            printf( "not enough memory for stts\n" );
            err = 1;
        }
        pqtinfo->vstszp = malloc( QMP_S_STSZP );
        if( !pqtinfo->vstszp )
        {
            printf( "not enough memory for stsz\n" );
            err = 1;
        }
        pqtinfo->vstcop = malloc( QMP_S_STCOP );
        if( !pqtinfo->vstcop )
        {
            printf( "not enough memory for stco \n" );
            err = 1;
        }
        pqtinfo->vstscp = malloc( QMP_S_STSCP );
        if( !pqtinfo->vstscp )
        {
            printf( "not enough memory for stsc\n" );
            err = 1;
        }
        pqtinfo->asttsp = malloc( QMP_S_STTSP );
        if( !pqtinfo->asttsp )
        {
            printf( "not enough memory for stts\n" );
            err = 1;
        }
        pqtinfo->astszp = malloc( QMP_S_STSZP );
        if( !pqtinfo->astszp )
        {
            printf( "not enough memory for stsz\n" );
            err = 1;
        }
        pqtinfo->astcop = malloc( QMP_S_STCOP );
        if( !pqtinfo->astcop )
        {
            printf( "not enough memory for stco\n" );
            err = 1;
        }
        pqtinfo->astscp = malloc( QMP_S_STSCP );
        if( !pqtinfo->astscp )
        {
            printf( "not enough memory for stsc\n" );
            err = 1;
        }
        pqtinfo->nvsttsp = 0x00;
        pqtinfo->nvstszp = 0x00;
        pqtinfo->nvstcop = 0x00;
        pqtinfo->nvstscp = 0x00;
        pqtinfo->nasttsp = 0x00;
        pqtinfo->nastszp = 0x00;
        pqtinfo->nastcop = 0x00;
        pqtinfo->nastscp = 0x00;
    }
	else {
		err = 1;
	}

    if( err )
    {
        if(pqtinfo)
        {
            if( pqtinfo->vsttsp )
                free( pqtinfo->vsttsp );
            if( pqtinfo->vstszp )
                free( pqtinfo->vstszp );
            if( pqtinfo->vstcop )
                free( pqtinfo->vstcop );
            if( pqtinfo->vstscp )
                free( pqtinfo->vstscp );
            if( pqtinfo->asttsp )
                free( pqtinfo->asttsp );
            if( pqtinfo->astszp )
                free( pqtinfo->astszp );
            if( pqtinfo->astcop )
                free( pqtinfo->astcop );
            if( pqtinfo->astscp )
                free( pqtinfo->astscp );
            free(pqtinfo);
        }
        return 0;
    }

    pqtinfo->vcfg.creationtime= (TQP_UINT32)time(NULL);
    pqtinfo->vcfg.framerate= pComponent->inputControl.Artesa.out_frame_rate;
    pqtinfo->vcfg.minbitrate= pComponent->vbrControl.min_bit_rate;
    pqtinfo->vcfg.maxbitrate= pComponent->vbrControl.max_bit_rate;
    pqtinfo->vcfg.avgbitrate= pComponent->rateControl.Artesa.avg_bit_rate;
    pqtinfo->vcfg.width= pComponent->outResolution.out_pic_width;
    pqtinfo->vcfg.height= pComponent->outResolution.out_pic_height;
    pqtinfo->vcfg.gopsize= pComponent->gopLoopFltr.Artesa.intra_period;
    pqtinfo->vcfg.profile= pComponent->sysControl.Artesa.profile;
    pqtinfo->vcfg.level= pComponent->sysControl.Artesa.level;
    pqtinfo->vcfg.num_of_b_frames = 0;
    pqtinfo->vcfg.vid_block_set = 0;
    pqtinfo->dc = 0;
    pqtinfo->vcfg.profile= PROFILE_MAIN;
    pqtinfo->vcfg.level= LEVEL_3;

    pqtinfo->acfg.channelnum = pComponent->audControl.ch_num;
    pqtinfo->acfg.bitrate = pComponent->audControl.bit_rate*1000;
    switch (pComponent->audControl.sample_rate)
    {
        case 0x00:
            pqtinfo->acfg.samplingrate = 44100;
            break;
        case 0x02:
            pqtinfo->acfg.samplingrate = 32000;
            break;
        case 0x04:
            pqtinfo->acfg.samplingrate = 22050;
            break;
        case 0x05:
            pqtinfo->acfg.samplingrate = 24000;
            break;
        case 0x06:
            pqtinfo->acfg.samplingrate = 16000;
            break;
        case 0x08:
            pqtinfo->acfg.samplingrate = 8000;
            break;
        case 0x01:
        default:
            pqtinfo->acfg.samplingrate = 48000;
            break;
    }

    switch(pqtinfo->acfg.samplingrate)
    {
        case 44100:     pqtinfo->acfg.sr = 0; break;
        case 48000:     pqtinfo->acfg.sr = 1; break;
        case 32000:     pqtinfo->acfg.sr = 2; break;
        case 8000:      pqtinfo->acfg.sr = 3; break;
        default:        pqtinfo->acfg.sr = 0; break;
    }

    return pqtinfo;
}


int mp4_free(QT_TRANSCODE_INFO *pqtinfo)
{
    if (pqtinfo==NULL)
		return 0;
    if(pqtinfo->mpbuff)
        free(pqtinfo->mpbuff);
    //printf("Free pqtinfo->mpbuff\n");
    if(pqtinfo->astscp)
        free( pqtinfo->astscp );
    //printf("Free pqtinfo->astscp\n");
    if(pqtinfo->astcop)
        free( pqtinfo->astcop );
    //printf("Free pqtinfo->astsco\n");
    if(pqtinfo->astszp)
        free( pqtinfo->astszp );
    //printf("Free pqtinfo->astszp\n");
    if(pqtinfo->asttsp)
        free( pqtinfo->asttsp );
    //printf("Free pqtinfo->asttsp\n");
    if(pqtinfo->vstscp)
        free( pqtinfo->vstscp );
    //printf("Free pqtinfo->vstscp\n");
    if(pqtinfo->vstcop)
        free( pqtinfo->vstcop );
    //printf("Free pqtinfo->vstcop\n");
    if(pqtinfo->vstszp)
        free( pqtinfo->vstszp );
    //printf("Free pqtinfo->vstszp\n");
    if(pqtinfo->vsttsp)
        free( pqtinfo->vsttsp );
    //printf("Free pqtinfo->vsttsp\n");
    if( pqtinfo->vbuff.buff )
        free( pqtinfo->vbuff.buff );
    //printf("Free pqtinfo->vbuff.buff\n");

    //printf( "qt_fini 7.1, file_offset 0x%x \n", pqtinfo->file_offset );
    if(pqtinfo)
        free(pqtinfo);
//    if( truncate( pqtinfo->fname, pqtinfo->file_offset ) == -1 )
//    {
//        printf( "truncate file failed\n" );
//    }
	return 0;
}

long mp4_fini(QT_TRANSCODE_INFO * info)
{
    unsigned int where = 0, rt;
    unsigned int temp = 0, address = 0, i_frames;
    QT_TRANSCODE_INFO *pqtinfo=info;
    struct ATOM_SIZE Atom_Moov, Atom_Trak, Atom_Mdia, Atom_Minf, Atom_Stbl, Atom_Stsd, Atom_Avc1,Atom_AvcC;
    struct ATOM_SIZE Atom_Stts, Atom_Stsc, Atom_Stss, Atom_Stsz, Atom_Stco;
    fpos_t filepos;

	if (pqtinfo==NULL)
		return 0;

    if( fseek( pqtinfo->mp4_handle, 0x04, SEEK_SET ) )
    {
        printf( "cannot seek QMP_TOTAL\n" );
		mp4_free(pqtinfo);
        return 0;
    }
    rt = (unsigned int) fread( pqtinfo->vbuff.buff, 1, QT_VBUFF, pqtinfo->mp4_handle );
    if( rt < QT_VBUFF )
    {
        printf( "error in reading mp4\n" );
		mp4_free(pqtinfo);
        return 0;
    }

    //compute video parameter
    pqtinfo->vcfg.duration = (QT_TIME_SCALE/pqtinfo->vcfg.framerate)*pqtinfo->vcfg.frames;

    //compute audio parameter
    pqtinfo->acfg.duration = 1024*pqtinfo->acfg.frames;
    //printf( "mp4_fini:%x, \n",pqtinfo->acfg.duration);

    //write File header
    //printf("write File header: %x\n", pqtinfo->file_offset );
    pqtinfo->mpbuff = malloc( (QMP_S_STCOP<<2) );
    if( !pqtinfo->mpbuff )
    {
        printf( "not enough memory for QMP_S_STCOP\n" );
		mp4_free(pqtinfo);
		return -1;
    }

#if 1
    address = 0x00;
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x18);
    address += qlputtag( &pqtinfo->mpbuff[address], "ftyp" );
    #if 1
    address += qlputtag( &pqtinfo->mpbuff[address], "isom" );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x01 );
    address += qlputtag( &pqtinfo->mpbuff[address], "isom" );
    address += qlputtag( &pqtinfo->mpbuff[address], "avc1" );
    #else
    address += qlputtag( &pqtinfo->mpbuff[address], "M4V " );
    address += qlput4byte( &pqtinfo->mpbuff[address], 1 );
    address += qlputtag( &pqtinfo->mpbuff[address], "M4V " );
    address += qlputtag( &pqtinfo->mpbuff[address], "M4A " );
    address += qlputtag( &pqtinfo->mpbuff[address], "mp42" );
    address += qlputtag( &pqtinfo->mpbuff[address], "isom" );
    #endif
#else
    address += qlput4byte( &pqtinfo->mpbuff[address], FTYP_SIZE);
    address += qlputtag( &pqtinfo->mpbuff[address], "ftyp" );
    address += qlputtag( &pqtinfo->mpbuff[address], "isom" );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlputtag( &pqtinfo->mpbuff[address], "mp41");
    address += qlputtag( &pqtinfo->mpbuff[address], "avc1" );
    address += qlputtag( &pqtinfo->mpbuff[address], "qt  " );
#endif
    qlput4byte( &pqtinfo->mpbuff[0x00], address);

    address += qlput4byte( &pqtinfo->mpbuff[address], pqtinfo->avstart_address - address );
    address += qlputtag( &pqtinfo->mpbuff[address], "free" );
    memset( &pqtinfo->mpbuff[address], 0, pqtinfo->avstart_address - address );
    address = pqtinfo->avstart_address;
    address += qlput4byte( &pqtinfo->mpbuff[address], (TQP_UINT32)(pqtinfo->file_offset - pqtinfo->avstart_address));
    address += qlputtag( &pqtinfo->mpbuff[address], "mdat" );
    rewind( pqtinfo->mp4_handle );
    fwrite( pqtinfo->mpbuff, 1, address, pqtinfo->mp4_handle );

    where = (unsigned int)pqtinfo->file_offset;
    Atom_Moov.addr = where;
    //printf("write video header:%x\n",where);
    address = 0x00;
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x08 );
    address += qlputtag( &pqtinfo->mpbuff[address], "moov" );
    address += qlput4byte( &pqtinfo->mpbuff[address], MVHD_SIZE);
    address += qlputtag( &pqtinfo->mpbuff[address], "mvhd" );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], pqtinfo->vcfg.creationtime );
    address += qlput4byte( &pqtinfo->mpbuff[address], pqtinfo->vcfg.creationtime );
    address += qlput4byte( &pqtinfo->mpbuff[address], QT_TIME_SCALE );
    address += qlput4byte( &pqtinfo->mpbuff[address], pqtinfo->vcfg.duration ); //
    //address += qlput4byte( &pqtinfo->mpbuff[address], pqtinfo->acfg.samplingrate);
    //address += qlput4byte( &pqtinfo->mpbuff[address], pqtinfo->acfg.duration);
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x10000 );
    address += qlput2byte( &pqtinfo->mpbuff[address], 0x100 );
    address += qlput2byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x10000 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x10000 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x40000000 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 3 );

    //write video header
    Atom_Trak.addr = where + address;
    //printf("Video Track : %x\n", Atom_Trak.addr);
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x08);
    address += qlputtag( &pqtinfo->mpbuff[address], "trak" );
    address += qlput4byte( &pqtinfo->mpbuff[address], TKHD_SIZE);
    address += qlputtag( &pqtinfo->mpbuff[address], "tkhd" );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x01 );
    address += qlput4byte( &pqtinfo->mpbuff[address], pqtinfo->vcfg.creationtime );
    address += qlput4byte( &pqtinfo->mpbuff[address], pqtinfo->vcfg.creationtime );
    address += qlput4byte( &pqtinfo->mpbuff[address], 1 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], pqtinfo->vcfg.duration );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x10000 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x10000 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x40000000 );
    address += qlput4byte( &pqtinfo->mpbuff[address], (pqtinfo->vcfg.width<<16) );
    address += qlput4byte( &pqtinfo->mpbuff[address], (pqtinfo->vcfg.height<<16) );
#if 1
    address += qlput4byte( &pqtinfo->mpbuff[address], EDTS_SIZE);
    address += qlputtag( &pqtinfo->mpbuff[address], "edts" );
    address += qlput4byte( &pqtinfo->mpbuff[address], ELST_SIZE);
    address += qlputtag( &pqtinfo->mpbuff[address], "elst" );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 1 );
    address += qlput4byte( &pqtinfo->mpbuff[address], pqtinfo->vcfg.duration );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x10000 );
#endif
    Atom_Mdia.addr = where + address;
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x08);
    address += qlputtag( &pqtinfo->mpbuff[address], "mdia" );
    address += qlput4byte( &pqtinfo->mpbuff[address], MDHD_SIZE);
    address += qlputtag( &pqtinfo->mpbuff[address], "mdhd" );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], pqtinfo->vcfg.creationtime );
    address += qlput4byte( &pqtinfo->mpbuff[address], pqtinfo->vcfg.creationtime );
    address += qlput4byte( &pqtinfo->mpbuff[address], QT_TIME_SCALE );
    address += qlput4byte( &pqtinfo->mpbuff[address], pqtinfo->vcfg.duration );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x55c40000 );
    address += qlput4byte( &pqtinfo->mpbuff[address], HDLR_SIZE);
    address += qlputtag( &pqtinfo->mpbuff[address], "hdlr" );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlputtag( &pqtinfo->mpbuff[address], "vide" );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlputbyte( &pqtinfo->mpbuff[address], 0x13 );
    address += qlputtag( &pqtinfo->mpbuff[address], "Vide" );
    address += qlputtag( &pqtinfo->mpbuff[address], "o Me" );
    address += qlputtag( &pqtinfo->mpbuff[address], "dia " );
    address += qlputtag( &pqtinfo->mpbuff[address], "Hand" );
    address += qlputtag( &pqtinfo->mpbuff[address], "ler" );

    Atom_Minf.addr = where + address;
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x08);
    address += qlputtag( &pqtinfo->mpbuff[address], "minf" );
    address += qlput4byte( &pqtinfo->mpbuff[address], VMHD_SIZE);
    address += qlputtag( &pqtinfo->mpbuff[address], "vmhd" );
    address += qlput4byte( &pqtinfo->mpbuff[address], 1 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], DINF_SIZE);
    address += qlputtag( &pqtinfo->mpbuff[address], "dinf" );
    address += qlput4byte( &pqtinfo->mpbuff[address], DREF_SIZE);
    address += qlputtag( &pqtinfo->mpbuff[address], "dref" );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 1 );
    address += qlput4byte( &pqtinfo->mpbuff[address], URL_SIZE);
    address += qlputtag( &pqtinfo->mpbuff[address], "url " );
    address += qlput4byte( &pqtinfo->mpbuff[address], 1 );

    Atom_Stbl.addr = where + address;
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x08);
    address += qlputtag( &pqtinfo->mpbuff[address], "stbl" );

    Atom_Stsd.addr = address;
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x08);
    address += qlputtag( &pqtinfo->mpbuff[address], "stsd" );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 1 );

    Atom_Avc1.addr = address;
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x08);
    address += qlputtag( &pqtinfo->mpbuff[address], "avc1" );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 1 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput2byte( &pqtinfo->mpbuff[address], pqtinfo->vcfg.width );
    address += qlput2byte( &pqtinfo->mpbuff[address], pqtinfo->vcfg.height );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x480000 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x480000 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput2byte( &pqtinfo->mpbuff[address], 1 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput2byte( &pqtinfo->mpbuff[address], 0x18 );
    address += qlput2byte( &pqtinfo->mpbuff[address], 0xffff );

    Atom_AvcC.addr = address;
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x00);
    address += qlputtag( &pqtinfo->mpbuff[address], "avcC" );
    address += qlputbyte( &pqtinfo->mpbuff[address], 1 );
    address += qlputbyte( &pqtinfo->mpbuff[address], pqtinfo->vbuff.buff[1] );
    address += qlputbyte( &pqtinfo->mpbuff[address], pqtinfo->vbuff.buff[2] );
    address += qlputbyte( &pqtinfo->mpbuff[address], pqtinfo->vbuff.buff[3] );
    address += qlputbyte( &pqtinfo->mpbuff[address], 0xff );
    address += qlputbyte( &pqtinfo->mpbuff[address], 0xe1);
    rt = address;
    address += qlput2byte( &pqtinfo->mpbuff[address], 0x63 );
    for(temp = 0; temp < 0x70; temp++)
    {
        address += qlputbyte( &pqtinfo->mpbuff[address], pqtinfo->vbuff.buff[temp] );
        if((pqtinfo->vbuff.buff[temp+1] == 0x00)&&(pqtinfo->vbuff.buff[temp+2] == 0x00)&&(pqtinfo->vbuff.buff[temp+3] == 0x00)&&
        	(pqtinfo->vbuff.buff[temp+4] == 0x01)&&(pqtinfo->vbuff.buff[temp+5] == 0x68))
        {
            break;
        }
    }
    qlput2byte( &pqtinfo->mpbuff[rt], temp+1 );
    address += qlputbyte( &pqtinfo->mpbuff[address], 1 );
    address += qlput2byte( &pqtinfo->mpbuff[address], 0x05 );
    rt=temp+5;
    for(temp = 0; temp < 0x05; temp++)
    {
        address += qlputbyte( &pqtinfo->mpbuff[address], pqtinfo->vbuff.buff[rt+temp] );
    }
    qlput4byte( &pqtinfo->mpbuff[Atom_AvcC.addr], address - Atom_AvcC.addr );   //write AvcC size
    qlput4byte( &pqtinfo->mpbuff[Atom_Stsd.addr], address-Atom_Stsd.addr);      //write Video Stsd size
    qlput4byte( &pqtinfo->mpbuff[Atom_Avc1.addr], address-Atom_Avc1.addr);      //write Video Avc1 size
    rewind( pqtinfo->mp4_handle );
    FILEPOS_ASSIGN(filepos, where);
    if(fsetpos(pqtinfo->mp4_handle, &filepos))
    //if( fseek( pqtinfo->mp4_handle, where, SEEK_SET ) )
    {
        printf( "cannot seek Audio header\n" );
		mp4_free(pqtinfo);
        return -1;
    }
    fwrite( pqtinfo->mpbuff, 1, address, pqtinfo->mp4_handle );
    where += address;

    //write video stts
    Atom_Stts.addr = where;
    where += 0x10;
    rewind( pqtinfo->mp4_handle );
    FILEPOS_ASSIGN(filepos, where);
    if(fsetpos(pqtinfo->mp4_handle, &filepos))
    //if(fseek( pqtinfo->mp4_handle, where, SEEK_SET ))
    {
        printf( "cannot seek nvsttsp\n" );
		mp4_free(pqtinfo);
        return -1;
    }
    if(pqtinfo->nvsttsp)
    {
        pqtinfo->data_buff = (char *)malloc( QMP_S_STTSP );
		if (pqtinfo->data_buff==NULL)
		{
			printf( "cannot allocate memory for nvsttsp\n" );
			mp4_free(pqtinfo);
	        return -1;
		}
        rewind(pqtinfo->fp_vsttsp );
        while(pqtinfo->nvsttsp)
        {
            temp = QMP_S_STTSP;
            if(pqtinfo->nvsttsp < QMP_S_STTSP)
                temp = pqtinfo->nvsttsp;
            rt=(unsigned int) fread( pqtinfo->data_buff, 1, temp, pqtinfo->fp_vsttsp );
            if(rt!=temp)
            {
				mp4_free(pqtinfo);
                return -1;
            }
            fwrite(pqtinfo->data_buff, 1, temp, pqtinfo->mp4_handle);
            where += temp;
            pqtinfo->nvsttsp -= temp;
        }
        free(pqtinfo->data_buff);
		pqtinfo->data_buff = NULL;
    }
    if( pqtinfo->nvstts )
    {
        fwrite( pqtinfo->vsttsp, 1, pqtinfo->nvstts, pqtinfo->mp4_handle );
        where += pqtinfo->nvstts;
    }
    qlput4byte( &pqtinfo->vsttsp[0x00], 1 );
    qlput4byte( &pqtinfo->vsttsp[0x04], QT_TIME_SCALE / pqtinfo->vcfg.framerate);
    fwrite( pqtinfo->vsttsp, 1, 0x08, pqtinfo->mp4_handle );
    where += 8;
    qlput4byte( &pqtinfo->vsttsp[0x00], where - Atom_Stts.addr);
    qlputtag( &pqtinfo->vsttsp[0x04], "stts" );
    qlput4byte( &pqtinfo->vsttsp[0x08], 0 );
    qlput4byte( &pqtinfo->vsttsp[0x0c], (where - Atom_Stts.addr - 0x10)/8 );
    rewind( pqtinfo->mp4_handle );
    FILEPOS_ASSIGN(filepos, Atom_Stts.addr);
    if(fsetpos(pqtinfo->mp4_handle, &filepos))
    //if( fseek( pqtinfo->mp4_handle, Atom_Stts.addr, SEEK_SET ) )
    {
        printf( "cannot seek nvstszp\n" );
		mp4_free(pqtinfo);
        return -1;
    }
    fwrite( pqtinfo->vsttsp, 1, 0x10, pqtinfo->mp4_handle );

    //write video stsc
    if(pqtinfo->vstsc)
    {
        qlput4byte( &pqtinfo->vstscp[pqtinfo->nvstsc], pqtinfo->vChunkCnt);
        pqtinfo->nvstsc += 4;
        qlput4byte( &pqtinfo->vstscp[pqtinfo->nvstsc], pqtinfo->vstsc );
        pqtinfo->nvstsc += 4;
        qlput4byte( &pqtinfo->vstscp[pqtinfo->nvstsc], 1 );
        pqtinfo->nvstsc += 4;
    }
    Atom_Stsc.addr = where;
    where += 0x10;
    rewind( pqtinfo->mp4_handle );
    FILEPOS_ASSIGN(filepos, where);
    if(fsetpos(pqtinfo->mp4_handle, &filepos))
    //if(fseek( pqtinfo->mp4_handle, where, SEEK_SET ))
    {
        printf( "cannot seek nvstscp\n" );
		mp4_free(pqtinfo);
        return -1;
    }
    if( pqtinfo->nvstscp)
    {
        pqtinfo->data_buff = (char *)malloc( QMP_S_STSCP );
		if (pqtinfo->data_buff==NULL)
		{
			printf( "cannot allocate memory for nvstscp\n" );
			mp4_free(pqtinfo);
	        return -1;
		}
        rewind( pqtinfo->fp_vstscp );
        while(pqtinfo->nvstscp)
        {
            temp = QMP_S_STSCP;
            if(pqtinfo->nvstscp < QMP_S_STSCP)
                temp = pqtinfo->nvstscp;
            rt=(unsigned int) fread( pqtinfo->data_buff, 1, temp, pqtinfo->fp_vstscp );
            if(rt!=temp)
            {
				mp4_free(pqtinfo);
                return -1;
            }
            fwrite(pqtinfo->data_buff, 1, temp, pqtinfo->mp4_handle);
            where += temp;
            pqtinfo->nvstscp -= temp;
        }
        free(pqtinfo->data_buff);
		pqtinfo->data_buff = NULL;
    }
    if( pqtinfo->nvstsc )
    {
        fwrite( pqtinfo->vstscp, 1, pqtinfo->nvstsc, pqtinfo->mp4_handle );
        where += pqtinfo->nvstsc;
    }
    qlput4byte( &pqtinfo->vstscp[0x00], where - Atom_Stsc.addr);
    qlputtag( &pqtinfo->vstscp[0x04], "stsc" );
    qlput4byte( &pqtinfo->vstscp[0x08], 0 );
    pqtinfo->vChunkCnt = (where - Atom_Stsc.addr - 0x10)/12;
    qlput4byte( &pqtinfo->vstscp[0x0c], pqtinfo->vChunkCnt);
    rewind( pqtinfo->mp4_handle );
    FILEPOS_ASSIGN(filepos, Atom_Stsc.addr);
    if(fsetpos(pqtinfo->mp4_handle, &filepos))
    //if( fseek( pqtinfo->mp4_handle, Atom_Stsc.addr, SEEK_SET ) )
    {
        printf( "cannot seek astsz\n" );
		mp4_free(pqtinfo);
        return -1;
    }
    fwrite( pqtinfo->vstscp, 1, 0x10, pqtinfo->mp4_handle );

    // write video stss
    i_frames = pqtinfo->vcfg.frames / pqtinfo->vcfg.gopsize;
    if( pqtinfo->vcfg.frames % pqtinfo->vcfg.gopsize )
    {
        i_frames++;
    }
    address = 0x00;
    Atom_Stss.addr = address;
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x10);
    address += qlputtag( &pqtinfo->mpbuff[address], "stss" );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], i_frames );
    temp = 1;
    while( i_frames )
    {
        address += qlput4byte( &pqtinfo->mpbuff[address], temp );
        temp += pqtinfo->vcfg.gopsize;
        i_frames--;
    }
    qlput4byte( &pqtinfo->mpbuff[Atom_Stss.addr], address);
    rewind( pqtinfo->mp4_handle );
    FILEPOS_ASSIGN(filepos, where);
    if(fsetpos(pqtinfo->mp4_handle, &filepos))
    //if( fseek( pqtinfo->mp4_handle, where, SEEK_SET ) )
    {
        printf( "cannot seek Video header\n" );
		mp4_free(pqtinfo);
        return -1;
    }
    fwrite( pqtinfo->mpbuff, 1, address, pqtinfo->mp4_handle );
    where += address;

    //write video stsz
    //printf("video stsz start :%x\n", where);
    Atom_Stsz.addr = where;
    where += 0x14;
    rewind( pqtinfo->mp4_handle );
    FILEPOS_ASSIGN(filepos, where);
    if(fsetpos(pqtinfo->mp4_handle, &filepos))
    //if(fseek( pqtinfo->mp4_handle, where, SEEK_SET ))
    {
        printf( "cannot seek nvstszp\n" );
		mp4_free(pqtinfo);
        return -1;
    }
    if(pqtinfo->nvstszp)
    {
        pqtinfo->data_buff = (char *)malloc( QMP_S_STSZP );
		if (pqtinfo->data_buff==NULL)
		{
			printf( "cannot allocate memory for nvstszp\n" );
			mp4_free(pqtinfo);
	        return -1;
		}
        rewind( pqtinfo->fp_vstszp );
        while(pqtinfo->nvstszp)
        {
            temp = QMP_S_STSZP;
            if(pqtinfo->nvstszp < QMP_S_STSZP)
                temp = pqtinfo->nvstszp;
            rt=(unsigned int) fread( pqtinfo->data_buff, 1, temp, pqtinfo->fp_vstszp );
            if(rt!=temp)
            {
				mp4_free(pqtinfo);
                return -1;
            }
            fwrite(pqtinfo->data_buff, 1, temp, pqtinfo->mp4_handle);
            where += temp;
            pqtinfo->nvstszp -= temp;
        }
        free(pqtinfo->data_buff);
		pqtinfo->data_buff = NULL;
    }
    if( pqtinfo->nvstsz )
    {
        fwrite( pqtinfo->vstszp, 1, pqtinfo->nvstsz, pqtinfo->mp4_handle );
        where += pqtinfo->nvstsz;
    }
    qlput4byte( &pqtinfo->vstszp[0x00], where - Atom_Stsz.addr);
    qlputtag( &pqtinfo->vstszp[0x04], "stsz" );
    qlput4byte( &pqtinfo->vstszp[0x08], 0 );
    qlput4byte( &pqtinfo->vstszp[0x0c], 0 );
    qlput4byte( &pqtinfo->vstszp[0x10], pqtinfo->vcfg.frames);
    rewind( pqtinfo->mp4_handle );
    FILEPOS_ASSIGN(filepos, Atom_Stsz.addr);
    if(fsetpos(pqtinfo->mp4_handle, &filepos))
    //if( fseek( pqtinfo->mp4_handle, Atom_Stsz.addr, SEEK_SET ) )
    {
        printf( "cannot seek nvstszp\n" );
		mp4_free(pqtinfo);
        return -1;
    }
    fwrite( pqtinfo->vstszp, 1, 0x14, pqtinfo->mp4_handle );

    //write video stco
    //printf("video stco start :%x\n", where);
    Atom_Stco.addr = where;
    where += 0x10;
    rewind( pqtinfo->mp4_handle );
    FILEPOS_ASSIGN(filepos, where);
    if(fsetpos(pqtinfo->mp4_handle, &filepos))
    //if(fseek( pqtinfo->mp4_handle, where, SEEK_SET ))
    {
        printf( "cannot seek nvstcop\n" );
		mp4_free(pqtinfo);
        return -1;
    }
    if(pqtinfo->nvstcop)
    {
        pqtinfo->data_buff = (char *)malloc( QMP_S_STCOP );
		if (pqtinfo->data_buff==NULL)
		{
			printf( "cannot allocate memory for nvstcop\n" );
			mp4_free(pqtinfo);
	        return -1;
		}
        rewind( pqtinfo->fp_vstcop );
        while(pqtinfo->nvstcop)
        {
            temp = QMP_S_STCOP;
            if(pqtinfo->nvstcop < QMP_S_STCOP)
                temp = pqtinfo->nvstcop;
            rt=(unsigned int) fread( pqtinfo->data_buff, 1, temp, pqtinfo->fp_vstcop );
            if(rt!=temp)
            {
				mp4_free(pqtinfo);
                return -1;
            }
            fwrite(pqtinfo->data_buff, 1, temp, pqtinfo->mp4_handle);
            where += temp;
            pqtinfo->nvstcop -= temp;
        }
        free(pqtinfo->data_buff);
		pqtinfo->data_buff = NULL;
    }
    if(pqtinfo->nvstco)
    {
        fwrite( pqtinfo->vstcop, 1, pqtinfo->nvstco, pqtinfo->mp4_handle );
        where += pqtinfo->nvstco;
    }
    qlput4byte( &pqtinfo->vstcop[0x00], where - Atom_Stco.addr );
    qlputtag( &pqtinfo->vstcop[0x04], "stco" );
    qlput4byte( &pqtinfo->vstcop[0x08], 0 );
    qlput4byte( &pqtinfo->vstcop[0x0c], ((where - Atom_Stco.addr - 0x10)>>2) );
    rewind( pqtinfo->mp4_handle );
    FILEPOS_ASSIGN(filepos, Atom_Stco.addr);
    if(fsetpos(pqtinfo->mp4_handle, &filepos))
    //if( fseek( pqtinfo->mp4_handle, Atom_Stco.addr, SEEK_SET ) )
    {
        printf( "cannot seek Atom_Stco.addr\n" );
		mp4_free(pqtinfo);
        return -1;
    }
    fwrite( pqtinfo->vstcop, 1, 0x10, pqtinfo->mp4_handle );

    qlput4byte( &pqtinfo->vstcop[0x00], where - Atom_Stbl.addr);             //write Stbl Size
    rewind( pqtinfo->mp4_handle );
    FILEPOS_ASSIGN(filepos, Atom_Stbl.addr);
    if(fsetpos(pqtinfo->mp4_handle, &filepos))
    //if( fseek( pqtinfo->mp4_handle, Atom_Stbl.addr, SEEK_SET ) )
    {
        printf( "cannot seek Atom_Stbl.addr\n" );
		mp4_free(pqtinfo);
        return -1;
    }
    fwrite( pqtinfo->vstcop, 1, 0x04, pqtinfo->mp4_handle );

    qlput4byte( &pqtinfo->vstcop[0x00], where - Atom_Trak.addr);             //write Trak Size
    //printf( "track Size:%x, Po:%x\n",  where - Atom_Trak.addr, where);
    rewind( pqtinfo->mp4_handle );
    FILEPOS_ASSIGN(filepos, Atom_Trak.addr);
    if(fsetpos(pqtinfo->mp4_handle, &filepos))
    //if( fseek( pqtinfo->mp4_handle, Atom_Trak.addr, SEEK_SET ) )
    {
        printf( "cannot seek Atom_Trak.addr\n" );
		mp4_free(pqtinfo);
        return -1;
    }
    fwrite( pqtinfo->vstcop, 1, 0x04, pqtinfo->mp4_handle );

    qlput4byte( &pqtinfo->vstcop[0x00], where - Atom_Mdia.addr);             //write Mdia Size
    rewind( pqtinfo->mp4_handle );
    FILEPOS_ASSIGN(filepos, Atom_Mdia.addr);
    if(fsetpos(pqtinfo->mp4_handle, &filepos))
    //if( fseek( pqtinfo->mp4_handle, Atom_Mdia.addr, SEEK_SET ) )
    {
        printf( "cannot seek Atom_Mdia.addr\n" );
		mp4_free(pqtinfo);
        return -1;
    }
    fwrite( pqtinfo->vstcop, 1, 0x04, pqtinfo->mp4_handle );

    qlput4byte( &pqtinfo->vstcop[0x00], where - Atom_Minf.addr);             //write Minf Size
    rewind( pqtinfo->mp4_handle );
    FILEPOS_ASSIGN(filepos, Atom_Minf.addr);
    if(fsetpos(pqtinfo->mp4_handle, &filepos))
    //if( fseek( pqtinfo->mp4_handle, Atom_Minf.addr, SEEK_SET ) )
    {
        printf( "cannot seek Atom_Minf.addr\n" );
		mp4_free(pqtinfo);
        return -1;
    }
    fwrite( pqtinfo->vstcop, 1, 0x04, pqtinfo->mp4_handle );

    //write audio header
    //printf("Audio header start :%x\n", where);
    address = 0;
    Atom_Trak.addr = where;
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x08);
    address += qlputtag( &pqtinfo->mpbuff[address], "trak" );
    address += qlput4byte( &pqtinfo->mpbuff[address], TKHD_SIZE);
    address += qlputtag( &pqtinfo->mpbuff[address], "tkhd" );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x01 );
    address += qlput4byte( &pqtinfo->mpbuff[address], pqtinfo->vcfg.creationtime );
    address += qlput4byte( &pqtinfo->mpbuff[address], pqtinfo->vcfg.creationtime );
    address += qlput4byte( &pqtinfo->mpbuff[address], 2 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    temp = (QT_TIME_SCALE*1024/pqtinfo->acfg.samplingrate)*pqtinfo->acfg.frames;
    address += qlput4byte( &pqtinfo->mpbuff[address], temp );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x01000000 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x10000 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x10000 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x40000000 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    //address += qlput4byte( &pqtinfo->mpbuff[address], EDTS_SIZE);
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x30);
    address += qlputtag( &pqtinfo->mpbuff[address], "edts" );
    //address += qlput4byte( &pqtinfo->mpbuff[address], ELST_SIZE);
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x28);
    address += qlputtag( &pqtinfo->mpbuff[address], "elst" );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 2 );
    address += qlput4byte( &pqtinfo->mpbuff[address], pqtinfo->acfg.i_pts_start);
    address += qlput4byte( &pqtinfo->mpbuff[address], 0xffffffff );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x10000 );
    address += qlput4byte( &pqtinfo->mpbuff[address], temp );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x10000 );

    Atom_Mdia.addr = where + address;
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x08);
    address += qlputtag( &pqtinfo->mpbuff[address], "mdia" );
    address += qlput4byte( &pqtinfo->mpbuff[address], MDHD_SIZE);
    address += qlputtag( &pqtinfo->mpbuff[address], "mdhd" );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], pqtinfo->vcfg.creationtime );
    address += qlput4byte( &pqtinfo->mpbuff[address], pqtinfo->vcfg.creationtime );
    address += qlput4byte( &pqtinfo->mpbuff[address], pqtinfo->acfg.samplingrate );
    address += qlput4byte( &pqtinfo->mpbuff[address], pqtinfo->acfg.duration);
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x15c70000 );
    address += qlput4byte( &pqtinfo->mpbuff[address], HDLR_SIZE);
    address += qlputtag( &pqtinfo->mpbuff[address], "hdlr" );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlputtag( &pqtinfo->mpbuff[address], "soun" );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlputbyte( &pqtinfo->mpbuff[address], 0x13 );
    address += qlputtag( &pqtinfo->mpbuff[address], "Soun" );
    address += qlputtag( &pqtinfo->mpbuff[address], "d Me" );
    address += qlputtag( &pqtinfo->mpbuff[address], "dia " );
    address += qlputtag( &pqtinfo->mpbuff[address], "Hand" );
    address += qlputtag( &pqtinfo->mpbuff[address], "ler" );

    Atom_Minf.addr = where + address;
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x08 );
    address += qlputtag( &pqtinfo->mpbuff[address], "minf" );
    address += qlput4byte( &pqtinfo->mpbuff[address], SMHD_SIZE);
    address += qlputtag( &pqtinfo->mpbuff[address], "smhd" );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], DINF_SIZE);
    address += qlputtag( &pqtinfo->mpbuff[address], "dinf" );
    address += qlput4byte( &pqtinfo->mpbuff[address], DREF_SIZE);
    address += qlputtag( &pqtinfo->mpbuff[address], "dref" );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 1 );
    address += qlput4byte( &pqtinfo->mpbuff[address], URL_SIZE);
    address += qlputtag( &pqtinfo->mpbuff[address], "url " );
    address += qlput4byte( &pqtinfo->mpbuff[address], 1 );

    Atom_Stbl.addr = where + address;
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x08);
    address += qlputtag( &pqtinfo->mpbuff[address], "stbl" );
    address += qlput4byte( &pqtinfo->mpbuff[address], STSD_SIZE);
    address += qlputtag( &pqtinfo->mpbuff[address], "stsd" );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 1 );
    address += qlput4byte( &pqtinfo->mpbuff[address], MP4A_SIZE);
    address += qlputtag( &pqtinfo->mpbuff[address], "mp4a" );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 1 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput2byte( &pqtinfo->mpbuff[address], pqtinfo->acfg.channelnum);
    address += qlput2byte( &pqtinfo->mpbuff[address], 0x10 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0xfffe0000 );
    //address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], (pqtinfo->acfg.samplingrate<<16) );
    address += qlput4byte( &pqtinfo->mpbuff[address], ESDS_SIZE);
    address += qlputtag( &pqtinfo->mpbuff[address], "esds" );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x0319021f );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x00041140 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x15000000 );
    address += qlput4byte( &pqtinfo->mpbuff[address], pqtinfo->acfg.bitrate );
    address += qlput4byte( &pqtinfo->mpbuff[address], pqtinfo->acfg.bitrate );
    address += qlputbyte( &pqtinfo->mpbuff[address], qpDecoderSR[pqtinfo->acfg.sr][0]);
    address += qlputbyte( &pqtinfo->mpbuff[address], qpDecoderSR[pqtinfo->acfg.sr][1]);
    address += qlputbyte( &pqtinfo->mpbuff[address], qpDecoderSR[pqtinfo->acfg.sr][2]);
    address += qlputbyte( &pqtinfo->mpbuff[address], qpDecoderSR[pqtinfo->acfg.sr][3]);
    //address += qlput4byte( &pqtinfo->mpbuff[address], ql201_Swap4Bytes(*(TQP_UINT32 *)&qpDecoderSR[pqtinfo->acfg.sr]) );
    address += qlput2byte( &pqtinfo->mpbuff[address], 0x601 );
    address += qlputbyte( &pqtinfo->mpbuff[address], 2 );
    /*address += qlput4byte( &pqtinfo->mpbuff[address], STTS_SIZE);
    address += qlputtag( &pqtinfo->mpbuff[address], "stts" );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0 );
    address += qlput4byte( &pqtinfo->mpbuff[address], 1 );
    address += qlput4byte( &pqtinfo->mpbuff[address], pqtinfo->acfg.frames );
    address += qlput4byte( &pqtinfo->mpbuff[address], 0x400 );*/
    rewind( pqtinfo->mp4_handle );
    FILEPOS_ASSIGN(filepos, where);
    if(fsetpos(pqtinfo->mp4_handle, &filepos))
    //if( fseek( pqtinfo->mp4_handle, where, SEEK_SET ) )
    {
        printf( "cannot seek Audio header\n" );
		mp4_free(pqtinfo);
        return -1;
    }
    fwrite( pqtinfo->mpbuff, 1, address, pqtinfo->mp4_handle );
    where += address;
#if 1
    //write audio stts
    Atom_Stts.addr = where;
    where += 0x10;
    rewind( pqtinfo->mp4_handle );
    FILEPOS_ASSIGN(filepos, where);
    if(fsetpos(pqtinfo->mp4_handle, &filepos))
    //if(fseek( pqtinfo->mp4_handle, where, SEEK_SET ))
    {
        printf( "cannot seek nasttsp\n" );
		mp4_free(pqtinfo);
        return -1;
    }
    if(pqtinfo->nasttsp)
    {
        pqtinfo->data_buff = (char *)malloc( QMP_S_STTSP );
		if (pqtinfo->data_buff==NULL)
		{
			printf( "cannot allocate memory for nasttsp\n" );
			mp4_free(pqtinfo);
	        return -1;
		}
        rewind(pqtinfo->fp_asttsp );
        while(pqtinfo->nasttsp)
        {
            temp = QMP_S_STTSP;
            if(pqtinfo->nasttsp < QMP_S_STTSP)
                temp = pqtinfo->nasttsp;
            rt=(unsigned int) fread( pqtinfo->data_buff, 1, temp, pqtinfo->fp_asttsp );
            if(rt!=temp)
            {
				mp4_free(pqtinfo);
                return -1;
            }
            fwrite(pqtinfo->data_buff, 1, temp, pqtinfo->mp4_handle);
            where += temp;
            pqtinfo->nasttsp -= temp;
        }
        free(pqtinfo->data_buff);
		pqtinfo->data_buff = NULL;
    }
    if( pqtinfo->nastts )
    {
        fwrite( pqtinfo->asttsp, 1, pqtinfo->nastts, pqtinfo->mp4_handle );
        where += pqtinfo->nastts;
    }
    //qlput4byte( &pqtinfo->asttsp[0x00], 1 );
    //qlput4byte( &pqtinfo->asttsp[0x04], 0x400);
    //fwrite( pqtinfo->asttsp, 1, 0x08, pqtinfo->mp4_handle );
    //where += 8;
    qlput4byte( &pqtinfo->asttsp[0x00], where - Atom_Stts.addr);
    qlputtag( &pqtinfo->asttsp[0x04], "stts" );
    qlput4byte( &pqtinfo->asttsp[0x08], 0 );
    qlput4byte( &pqtinfo->asttsp[0x0c], (where - Atom_Stts.addr - 0x10)/8 );
    rewind( pqtinfo->mp4_handle );
    FILEPOS_ASSIGN(filepos, Atom_Stts.addr);
    if(fsetpos(pqtinfo->mp4_handle, &filepos))
    //if( fseek( pqtinfo->mp4_handle, Atom_Stts.addr, SEEK_SET ) )
    {
        printf( "cannot seek nvstszp\n" );
		mp4_free(pqtinfo);
        return -1;
    }
    fwrite( pqtinfo->asttsp, 1, 0x10, pqtinfo->mp4_handle );
#endif
    //write audio stsc
    //printf("Audio stsc start :%x\n", where);
    Atom_Stsc.addr = where;
    where += 0x10;
    rewind( pqtinfo->mp4_handle );
    FILEPOS_ASSIGN(filepos, where);
    if(fsetpos(pqtinfo->mp4_handle, &filepos))
    //if(fseek( pqtinfo->mp4_handle, where, SEEK_SET ))
    {
        printf( "cannot seek nastcop\n" );
		mp4_free(pqtinfo);
        return -1;
    }
    if( pqtinfo->nastscp)
    {
        pqtinfo->data_buff = (char *)malloc( QMP_S_STSCP );
		if (pqtinfo->data_buff==NULL)
		{
			printf( "cannot allocate memory for nastscp\n" );
			mp4_free(pqtinfo);
	        return -1;
		}
        rewind( pqtinfo->fp_astscp );
        while(pqtinfo->nastscp)
        {
            temp = QMP_S_STSCP;
            if(pqtinfo->nastscp < QMP_S_STSCP)
                temp = pqtinfo->nastscp;
            rt=(unsigned int) fread( pqtinfo->data_buff, 1, temp, pqtinfo->fp_astscp );
            if(rt!=temp)
            {
				mp4_free(pqtinfo);
                return -1;
            }
            fwrite(pqtinfo->data_buff, 1, temp, pqtinfo->mp4_handle);
            where += temp;
            pqtinfo->nastscp -= temp;
        }
        free(pqtinfo->data_buff);
		pqtinfo->data_buff = NULL;
    }
    if( pqtinfo->nastsc )
    {
        fwrite( pqtinfo->astscp, 1, pqtinfo->nastsc, pqtinfo->mp4_handle );
        where += pqtinfo->nastsc;
    }
    qlput4byte( &pqtinfo->astscp[0x00], where - Atom_Stsc.addr);
    qlputtag( &pqtinfo->astscp[0x04], "stsc" );
    qlput4byte( &pqtinfo->astscp[0x08], 0 );
    pqtinfo->aChunkCnt = (where - Atom_Stsc.addr - 0x10)/12;
    qlput4byte( &pqtinfo->astscp[0x0c], pqtinfo->aChunkCnt);
    rewind( pqtinfo->mp4_handle );
    FILEPOS_ASSIGN(filepos, Atom_Stsc.addr);
    if(fsetpos(pqtinfo->mp4_handle, &filepos))
    //if( fseek( pqtinfo->mp4_handle, Atom_Stsc.addr, SEEK_SET ) )
    {
        printf( "cannot seek astsz\n" );
		mp4_free(pqtinfo);
        return -1;
    }
    fwrite( pqtinfo->astscp, 1, 0x10, pqtinfo->mp4_handle );

    //write audio stsz
    Atom_Stsz.addr = where;
    //printf("Audio stsz start :%x\n", where);
    where += 0x14;
    rewind( pqtinfo->mp4_handle );
    FILEPOS_ASSIGN(filepos, where);
    if(fsetpos(pqtinfo->mp4_handle, &filepos))
    //if(fseek( pqtinfo->mp4_handle, where, SEEK_SET ))
    {
        printf( "cannot seek nastcop\n" );
		mp4_free(pqtinfo);
        return -1;
    }
    if( pqtinfo->nastszp)
    {
        pqtinfo->data_buff = (char *)malloc( QMP_S_STSZP );
		if (pqtinfo->data_buff==NULL)
		{
			printf( "cannot allocate memory for nastszp\n" );
			mp4_free(pqtinfo);
	        return -1;
		}
        rewind( pqtinfo->fp_astszp );
        while(pqtinfo->nastszp)
        {
            temp = QMP_S_STSZP;
            if(pqtinfo->nastszp < QMP_S_STSZP)
                temp = pqtinfo->nastszp;
            rt=(unsigned int) fread( pqtinfo->data_buff, 1, temp, pqtinfo->fp_astszp );
            if(rt!=temp)
            {
				mp4_free(pqtinfo);
                return -1;
            }
            fwrite(pqtinfo->data_buff, 1, temp, pqtinfo->mp4_handle);
            where += temp;
            pqtinfo->nastszp -= temp;
        }
        free(pqtinfo->data_buff);
		pqtinfo->data_buff = NULL;
    }
    if( pqtinfo->nastsz )
    {
        fwrite( pqtinfo->astszp, 1, pqtinfo->nastsz, pqtinfo->mp4_handle );
        where += pqtinfo->nastsz;
    }
    qlput4byte( &pqtinfo->astszp[0x00], where - Atom_Stsz.addr);
    qlputtag( &pqtinfo->astszp[0x04], "stsz" );
    qlput4byte( &pqtinfo->astszp[0x08], 0 );
    qlput4byte( &pqtinfo->astszp[0x0c], 0 );
    qlput4byte( &pqtinfo->astszp[0x10], pqtinfo->acfg.frames );
    rewind( pqtinfo->mp4_handle );
    FILEPOS_ASSIGN(filepos, Atom_Stsz.addr);
    if(fsetpos(pqtinfo->mp4_handle, &filepos))
    //if( fseek( pqtinfo->mp4_handle, Atom_Stsz.addr, SEEK_SET ) )
    {
        printf( "cannot seek astsz\n" );
		mp4_free(pqtinfo);
        return -1;
    }
    fwrite( pqtinfo->astszp, 1, 0x14, pqtinfo->mp4_handle );

    //write audio stco
    Atom_Stco.addr = where;
    //printf("Audio stco start :%x\n", where);
    where += 0x10;
    rewind( pqtinfo->mp4_handle );
    FILEPOS_ASSIGN(filepos, where);
    if(fsetpos(pqtinfo->mp4_handle, &filepos))
    //if(fseek( pqtinfo->mp4_handle, where, SEEK_SET ))
    {
        printf( "cannot seek nastcop\n" );
		mp4_free(pqtinfo);
        return -1;
    }
    if( pqtinfo->nastcop )
    {
        pqtinfo->data_buff = (char *)malloc( QMP_S_STCOP );
		if (pqtinfo->data_buff==NULL)
		{
			printf( "cannot allocate memory for nastcop\n" );
			mp4_free(pqtinfo);
	        return -1;
		}
        rewind( pqtinfo->fp_astcop );
        while(pqtinfo->nastcop)
        {
            temp = QMP_S_STCOP;
            if(pqtinfo->nastcop < QMP_S_STCOP)
                temp = pqtinfo->nastcop;
            rt=(unsigned int) fread( pqtinfo->data_buff, 1, temp, pqtinfo->fp_astcop );
            if(rt!=temp)
            {
				mp4_free(pqtinfo);
                return -1;
            }
            fwrite(pqtinfo->data_buff, 1, temp, pqtinfo->mp4_handle);
            where += temp;
            pqtinfo->nastcop -= temp;
        }
        free(pqtinfo->data_buff);
		pqtinfo->data_buff = NULL;
    }
    if( pqtinfo->nastco )
    {
        fwrite( pqtinfo->astcop, 1, pqtinfo->nastco, pqtinfo->mp4_handle );
        where += pqtinfo->nastco;
    }
    qlput4byte( &pqtinfo->astcop[0x00], where -Atom_Stco.addr );
    qlputtag( &pqtinfo->astcop[0x04], "stco" );
    qlput4byte( &pqtinfo->astcop[0x08], 0);
    qlput4byte( &pqtinfo->astcop[0x0c], (where - Atom_Stco.addr - 0x10)>>2);
    rewind( pqtinfo->mp4_handle );
    FILEPOS_ASSIGN(filepos, Atom_Stco.addr);
    if(fsetpos(pqtinfo->mp4_handle, &filepos))
    //if( fseek( pqtinfo->mp4_handle, Atom_Stco.addr, SEEK_SET ) )
    {
        printf( "cannot seek astsz\n" );
		mp4_free(pqtinfo);
        return -1;
    }
    fwrite( pqtinfo->astcop, 1, 0x10, pqtinfo->mp4_handle );

    qlput4byte( &pqtinfo->astcop[0x00], where -Atom_Trak.addr);         //write Trak Size
    //printf("Audio TRACK Size=%x\n",where -Atom_Trak.addr);
    rewind( pqtinfo->mp4_handle );
    FILEPOS_ASSIGN(filepos, Atom_Trak.addr);
    if(fsetpos(pqtinfo->mp4_handle, &filepos))
    //if( fseek( pqtinfo->mp4_handle, Atom_Trak.addr, SEEK_SET ) )
    {
        printf( "cannot seek Atom_Trak.addr\n" );
		mp4_free(pqtinfo);
        return -1;
    }
    fwrite( pqtinfo->astcop, 1, 0x04, pqtinfo->mp4_handle );

    qlput4byte( &pqtinfo->astcop[0x00], where -Atom_Mdia.addr);         //write Mdia Size
    //printf("Audio Mdia Size=%x\n",where -Atom_Mdia.addr);
    rewind( pqtinfo->mp4_handle );
    FILEPOS_ASSIGN(filepos, Atom_Mdia.addr);
    if(fsetpos(pqtinfo->mp4_handle, &filepos))
    //if( fseek( pqtinfo->mp4_handle, Atom_Mdia.addr, SEEK_SET ) )
    {
        printf( "cannot seek Atom_Mdia.addr\n" );
		mp4_free(pqtinfo);
        return -1;
    }
    fwrite( pqtinfo->astcop, 1, 0x04, pqtinfo->mp4_handle );

    qlput4byte( &pqtinfo->astcop[0x00], where -Atom_Minf.addr);         //write Minf Size
    //printf("Audio Minf Size=%x\n",where -Atom_Minf.addr);
    rewind( pqtinfo->mp4_handle );
    FILEPOS_ASSIGN(filepos, Atom_Minf.addr);
    if(fsetpos(pqtinfo->mp4_handle, &filepos))
    //if( fseek( pqtinfo->mp4_handle, Atom_Minf.addr, SEEK_SET ) )
    {
        printf( "cannot seek Atom_Minf.addr\n" );
		mp4_free(pqtinfo);
        return -1;
    }
    fwrite( pqtinfo->astcop, 1, 0x04, pqtinfo->mp4_handle );

    qlput4byte( &pqtinfo->astcop[0x00], where -Atom_Stbl.addr);         //write Stbl Size
    //printf("Audio STBL Size=%x\n",where -Atom_Stbl.addr);
    rewind( pqtinfo->mp4_handle );
    FILEPOS_ASSIGN(filepos, Atom_Stbl.addr);
    if(fsetpos(pqtinfo->mp4_handle, &filepos))
    //if( fseek( pqtinfo->mp4_handle, Atom_Stbl.addr, SEEK_SET ) )
    {
        printf( "cannot seek Atom_Stbl.addr\n" );
		mp4_free(pqtinfo);
        return -1;
    }
    fwrite( pqtinfo->astcop, 1, 0x04, pqtinfo->mp4_handle );

    //printf("udta start :%x\n", where);
    qlput4byte( &pqtinfo->astcop[0x00], 0x08 );
    qlputtag( &pqtinfo->astcop[0x04], "udta" );
    rewind( pqtinfo->mp4_handle );
    FILEPOS_ASSIGN(filepos, where);
    if(fsetpos(pqtinfo->mp4_handle, &filepos))
    //if( fseek( pqtinfo->mp4_handle, where, SEEK_SET ) )
    {
        printf( "cannot seek astsz\n" );
		mp4_free(pqtinfo);
        return -1;
    }
    fwrite( pqtinfo->astcop, 1, 0x08, pqtinfo->mp4_handle );
    where += 0x08;

    qlput4byte( &pqtinfo->astcop[0x00], where -Atom_Moov.addr);         //write Moov Size
    //printf("Data Length: %x\n", pqtinfo->file_offset);
    //printf("MOOV Size=%x\n",where -Atom_Moov.addr);
    rewind( pqtinfo->mp4_handle );
    FILEPOS_ASSIGN(filepos, Atom_Moov.addr);
    if(fsetpos(pqtinfo->mp4_handle, &filepos))
    //if( fseek( pqtinfo->mp4_handle, Atom_Moov.addr, SEEK_SET ) )
    {
        printf( "cannot seek Atom_Moov.addr\n" );
		mp4_free(pqtinfo);
        return -1;
    }
    fwrite( pqtinfo->astcop, 1, 0x04, pqtinfo->mp4_handle );
    pqtinfo->file_offset += (where -Atom_Moov.addr);
    //printf("File Length: %x\n", pqtinfo->file_offset);
	mp4_free(pqtinfo);
    return 0;
}

