/*************************************************************************/
/**      **/
/**            **/
/**                **/
/**              **/
/**                                                                     **/
/**                               **/
/**                                                  **/
/*************************************************************************/

#include "qlautoconf.h"
#include <stdio.h>
#include <unistd.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <time.h>
#include <signal.h>
#ifndef __APPLE__
#include <mntent.h>
#include <asm/types.h>
#endif // __APPLE__
#include <sys/types.h>
#include <errno.h>

#ifndef CONFIG_HOST_PLATFORM_USERSPACE
#include <linux/videodev2.h>
#else
#include "uservideodev2.h"
#endif

#include "qptypes.h"
#include "pldevioctl.h"
#include "qlstrm.h"

#ifdef CONFIG_HOST_PLATFORM_USERSPACE
#include "userpl330.h"
#define ioctl(dev, cmd, arg)  plv4l_ioctl(dev, cmd, arg)
#endif // CONFIG_HOST_PLATFORM_USERSPACE


#ifndef FIELD_OFFSET
#define FIELD_OFFSET(type, field)	((long)&(((type *)0)->field))
#endif //!FIELD_OFFSET


// encoder settings
QPCODEC_SYSTEM_CONTROL			sysControl;
QPCODEC_PICTURE_RESOLUTION		picResolution;
QPCODEC_INPUT_CONTROL			inputControl;
QPCODEC_RATE_CONTROL			rateControl;
QPCODEC_VBR_BIT_RATE			vbrControl;
QPCODEC_FILTER_CONTROL			fltrControl;
QPCODEC_GOP_LOOP_FILTER			gopLoopFltr;
QPCODEC_OUT_PICTURE_RESOLUTION	outResolution;
QPCODEC_AUDIO_CONTROL_PARAM		audControl;
QPCODEC_AUDIO_CONTROL_EX		audControlEx;
TQP_UINT32						mjpegQuality;
TQP_UINT32						mjpegBufNo;


// decoder settings
QPCODEC_DEC_STREAM_ID_EX		decStreamID;
TQP_UINT32						decOutputMode;
QPCODEC_PICTURE_RESOLUTION		decResolution;
QPCODEC_DEC_STOP_MODE			decStopMode;
QPCODEC_DEC_DISP_ATTRIB			decDispAttrib;
QPCODEC_DEC_DISP_MODE			decDispMode;
QPCODEC_DEC_AUDIO_CONTROL		decAudControl;

// misc settngs
QPCODEC_AOU_CONFIG			aouConfig;
QPCODEC_OSD_MEM_CONFIG		osdMem;
QPCODEC_DECIMATION_FACTOR	decimation;
TQP_UINT32 					deinterlacemode;
QPCODEC_ENC_CAP_MODE 		enccapmode;
QPCODEC_EXTERNAL_TRIGGER_MODE externaltriggermode;
QPCODEC_PTS_COUNTER_RESET 	ptscounterreset;
RATE_CONTROL_EX				rateControlEx;
TQP_UINT32 					largecompressbuffercontrol;
TQP_UINT32 					lowbitratemode;
TQP_UINT32 					avdiscardcontrolmode;
TQP_UINT32 					useswptsmode;

// codec settings
QPCODEC_SYS_FUNCTION			function;
QPCODEC_SYS_LINK				sys_link;



enum cmd_type_e {
    CMD_NONE = 0,
    CMD_GET,
    CMD_GETALL,
    CMD_SET,
    CMD_SETSCRIPT
};

char szDevName[32];

//static int ghDev = 0;

typedef struct _st_item_descript_table {
    int nValue;
    char *pszDescript;
} ST_ITEM_DESCRIPT_TABLE, *PST_ITEM_DESCRIPT_TABLE;

typedef struct _st_item_config_table {
    char *pszItemName;
    int nControlIdGet;
    int nControlIdSet;
	char *pConfig;
	int	offset;
    ST_ITEM_DESCRIPT_TABLE staDescript[32];
} ST_ITEM_CONFIG_TABLE, *PST_ITEM_CONFIG_TABLE;

/* refer to qpdevcntl.h */
ST_ITEM_CONFIG_TABLE gstaConfigTable[256] = {
    /* SystemControl */
    {"SystemControl-StreamType",
     PLDEV_CNTL_IOCTL_GET_SYSTEM_CONTROL,
     PLDEV_CNTL_IOCTL_SET_SYSTEM_CONTROL,
	 (char *)&sysControl,
	 FIELD_OFFSET(QPCODEC_SYSTEM_CONTROL, Artesa.stream_type),
     {{0, "ps"},
      {1, "ts"},
      {2, "mp4"},
      {3, "es"},
      {4, "raw"},
      {5, "special1"},
      {6, "jpeg"},
      {0, 0}}},
    {"SystemControl-StreamData",
     PLDEV_CNTL_IOCTL_GET_SYSTEM_CONTROL,
     PLDEV_CNTL_IOCTL_SET_SYSTEM_CONTROL,
	 (char *)&sysControl,
	 FIELD_OFFSET(QPCODEC_SYSTEM_CONTROL, Artesa.stream_data),
     {{1, "video"},
      {2, "audio"},
      {3, "video+audio"},
      {4, "vbi"},
      {8, "user"},
      {0, 0}}},
    {"SystemControl-Profile",
     PLDEV_CNTL_IOCTL_GET_SYSTEM_CONTROL,
     PLDEV_CNTL_IOCTL_SET_SYSTEM_CONTROL,
	 (char *)&sysControl,
	 FIELD_OFFSET(QPCODEC_SYSTEM_CONTROL, Artesa.profile),
     {{0, "baseline"},
      {1, "main"},
      {2, "extended"},
      {0, 0}}},
    {"SystemControl-Level",
     PLDEV_CNTL_IOCTL_GET_SYSTEM_CONTROL,
     PLDEV_CNTL_IOCTL_SET_SYSTEM_CONTROL,
	 (char *)&sysControl,
	 FIELD_OFFSET(QPCODEC_SYSTEM_CONTROL, Artesa.level),
     {{0, "1"},
      {1, "1.b"},
      {2, "1.1"},
      {3, "1.2"},
      {4, "1.3"},
      {5, "2"},
      {6, "2.1"},
      {7, "2.2"},
      {8, "3"},
      {9, "3.1"},
      {10, "3.2"},
      {11, "4"},
      {12, "4.1"},
      {13, "4.2"},
      {14, "5"},
      {15, "5.1"},
      {0, 0}}},
    {"SystemControl-FFMode",
     PLDEV_CNTL_IOCTL_GET_SYSTEM_CONTROL,
     PLDEV_CNTL_IOCTL_SET_SYSTEM_CONTROL,
	 (char *)&sysControl,
	 FIELD_OFFSET(QPCODEC_SYSTEM_CONTROL, Artesa.ff_mode),
     {{0, "auto"},
      {1, "frame"},
      {2, "field"},
      {0, 0}}},
    {"SystemControl-SpsrFreq",
     PLDEV_CNTL_IOCTL_GET_SYSTEM_CONTROL,
     PLDEV_CNTL_IOCTL_SET_SYSTEM_CONTROL,
	 (char *)&sysControl,
	 FIELD_OFFSET(QPCODEC_SYSTEM_CONTROL, Artesa.spsr_freq),
     {{0, 0}}},
    {"SystemControl-VMode",
     PLDEV_CNTL_IOCTL_GET_SYSTEM_CONTROL,
     PLDEV_CNTL_IOCTL_SET_SYSTEM_CONTROL,
	 (char *)&sysControl,
	 FIELD_OFFSET(QPCODEC_SYSTEM_CONTROL, Artesa.v_mode),
     {{0, "cavlc"},
      {1, "cabac"},
      {0, 0}}},
    {"SystemControl-CabacInitIdec",
     PLDEV_CNTL_IOCTL_GET_SYSTEM_CONTROL,
     PLDEV_CNTL_IOCTL_SET_SYSTEM_CONTROL,
	 (char *)&sysControl,
	 FIELD_OFFSET(QPCODEC_SYSTEM_CONTROL, Artesa.cabac_init_idc),
     {{0, 0}}},
    {"SystemControl-Version",
     PLDEV_CNTL_IOCTL_GET_SYSTEM_CONTROL,
     PLDEV_CNTL_IOCTL_SET_SYSTEM_CONTROL,
	 (char *)&sysControl,
	 0,
     {{0, 0}}},
    {"SystemControl-XferMode",
     PLDEV_CNTL_IOCTL_GET_SYSTEM_CONTROL,
     PLDEV_CNTL_IOCTL_SET_SYSTEM_CONTROL,
	 (char *)&sysControl,
	 0,
     {{0, "frame"},
      {1, "block"},
      {0, 0}}},

    /* PictureResolution */
    {"PictureResolution-InPicWidth",
     PLDEV_CNTL_IOCTL_GET_PICTURE_RESOLUTION,
     PLDEV_CNTL_IOCTL_SET_PICTURE_RESOLUTION,
	 (char *)&picResolution,
	 FIELD_OFFSET(QPCODEC_PICTURE_RESOLUTION, Artesa.in_pic_width),
     {{0, 0}}},
    {"PictureResolution-InPicHeight",
     PLDEV_CNTL_IOCTL_GET_PICTURE_RESOLUTION,
     PLDEV_CNTL_IOCTL_SET_PICTURE_RESOLUTION,
	 (char *)&picResolution,
	 FIELD_OFFSET(QPCODEC_PICTURE_RESOLUTION, Artesa.in_pic_height),
     {{0, 0}}},

    /* InputControl */
    {"InputControl-ScanFormat",
     PLDEV_CNTL_IOCTL_GET_INPUT_CONTROL,
     PLDEV_CNTL_IOCTL_SET_INPUT_CONTROL,
	 (char *)&inputControl,
	 FIELD_OFFSET(QPCODEC_INPUT_CONTROL, Artesa.scan_format),
     {{0, "interlace"},
      {1, "progressive"},
      {2, "sepcial1"},
      {3, "special2"},
      {4, "special3"},
      {0, 0}}},
    {"InputControl-SrcMode",
     PLDEV_CNTL_IOCTL_GET_INPUT_CONTROL,
     PLDEV_CNTL_IOCTL_SET_INPUT_CONTROL,
	 (char *)&inputControl,
	 FIELD_OFFSET(QPCODEC_INPUT_CONTROL, Artesa.src_mode),
     {{0, "656"},
      {1, "hdmi"},
      {2, "hiu"},
      {3, "hdimx2"},
      {0, 0}}},
    {"InputControl-SyncMode",
     PLDEV_CNTL_IOCTL_GET_INPUT_CONTROL,
     PLDEV_CNTL_IOCTL_SET_INPUT_CONTROL,
	 (char *)&inputControl,
	 FIELD_OFFSET(QPCODEC_INPUT_CONTROL, Artesa.sync_mode),
     {{0, "1920x1080i"},
      {1, "1920x1080p"},
      {2, "1440x1088i"},
      {3, "1440x576p"},
      {4, "1440x480p"},
      {5, "1280x720p"},
      {6, "720_1440x480i"},
      {7, "720_1440x576i"},
      {8, "720x480p"},
      {9, "704x576p"},
      {10, "720x480i"},
      {11, "704x576i"},
      {12, "640x480p"},
      {13, "640x480i"},
      {14, "176x188i"},
      {15, "2"},
      {16, "3"},
      {17, "0"},
      {18, "1"},
      {0, 0}}},
    {"InputControl-DataType",
     PLDEV_CNTL_IOCTL_GET_INPUT_CONTROL,
     PLDEV_CNTL_IOCTL_SET_INPUT_CONTROL,
	 (char *)&inputControl,
	 FIELD_OFFSET(QPCODEC_INPUT_CONTROL, Artesa.data_type),
     {{0, "h264"},
      {1, "mpeg2"},
      {2, "jpeg"},
      {3, "raw"},
      {0, 0}}},
    {"InputControl-InFrameRate",
     PLDEV_CNTL_IOCTL_GET_INPUT_CONTROL,
     PLDEV_CNTL_IOCTL_SET_INPUT_CONTROL,
	 (char *)&inputControl,
	 FIELD_OFFSET(QPCODEC_INPUT_CONTROL, Artesa.in_frame_rate),
     {{0, 0}}},
    {"InputControl-OutFrameRate",
     PLDEV_CNTL_IOCTL_GET_INPUT_CONTROL,
     PLDEV_CNTL_IOCTL_SET_INPUT_CONTROL,
	 (char *)&inputControl,
	 FIELD_OFFSET(QPCODEC_INPUT_CONTROL, Artesa.out_frame_rate),
     {{0, 0}}},
    {"InputControl-Fmt",
     PLDEV_CNTL_IOCTL_GET_INPUT_CONTROL,
     PLDEV_CNTL_IOCTL_SET_INPUT_CONTROL,
	 (char *)&inputControl,
	 FIELD_OFFSET(QPCODEC_INPUT_CONTROL, Artesa.fmt),
     {{0, "progressive"},
      {1, "interlace"},
      {0, 0}}},
    {"InputControl-Force60pIn",
     PLDEV_CNTL_IOCTL_GET_INPUT_CONTROL,
     PLDEV_CNTL_IOCTL_SET_INPUT_CONTROL,
	 (char *)&inputControl,
	 FIELD_OFFSET(QPCODEC_INPUT_CONTROL, Artesa.force_60p_in),
     {{0, "0"},
      {1, "1"},
      {0, 0}}},
    {"InputControl-PdEn",
     PLDEV_CNTL_IOCTL_GET_INPUT_CONTROL,
     PLDEV_CNTL_IOCTL_SET_INPUT_CONTROL,
	 (char *)&inputControl,
	 FIELD_OFFSET(QPCODEC_INPUT_CONTROL, Artesa.pd_en),
     {{0, 0}}},
    {"InputControl-CkEdge",
     PLDEV_CNTL_IOCTL_GET_INPUT_CONTROL,
     PLDEV_CNTL_IOCTL_SET_INPUT_CONTROL,
	 (char *)&inputControl,
	 FIELD_OFFSET(QPCODEC_INPUT_CONTROL, Artesa.ck_edge),
     {{0, "positive"},
      {1, "negative"},
      {0, 0}}},
    {"InputControl-ByteSwap",
     PLDEV_CNTL_IOCTL_GET_INPUT_CONTROL,
     PLDEV_CNTL_IOCTL_SET_INPUT_CONTROL,
	 (char *)&inputControl,
	 FIELD_OFFSET(QPCODEC_INPUT_CONTROL, Artesa.bytes_swap),
     {{0, "0"},
      {1, "1"},
      {0, 0}}},

    /* RateControl */
    {"RateControl-AvgBitRate",
     PLDEV_CNTL_IOCTL_GET_RATE_CONTROL,
     PLDEV_CNTL_IOCTL_SET_RATE_CONTROL,
	 (char *)&rateControl,
	 FIELD_OFFSET(QPCODEC_RATE_CONTROL, Artesa.avg_bit_rate),
     {{0, 0}}},
    {"RateControl-QpUpdateFreq",
     PLDEV_CNTL_IOCTL_GET_RATE_CONTROL,
     PLDEV_CNTL_IOCTL_SET_RATE_CONTROL,
	 (char *)&rateControl,
	 FIELD_OFFSET(QPCODEC_RATE_CONTROL, Artesa.qp_update_freq),
     {{0, 0}}},
    {"RateControl-Mode",
     PLDEV_CNTL_IOCTL_GET_RATE_CONTROL,
     PLDEV_CNTL_IOCTL_SET_RATE_CONTROL,
	 (char *)&rateControl,
	 FIELD_OFFSET(QPCODEC_RATE_CONTROL, Artesa.mode),
     {{0, "viu"},
      {1, "hiu"},
      {0, 0}}},
    {"RateControl-IntraRefresh",
     PLDEV_CNTL_IOCTL_GET_RATE_CONTROL,
     PLDEV_CNTL_IOCTL_SET_RATE_CONTROL,
	 (char *)&rateControl,
	 FIELD_OFFSET(QPCODEC_RATE_CONTROL, Artesa.intra_refresh_mode),
     {{0, "0"},
      {1, "1"},
      {0, 0}}},
    {"RateControl-Fixed",
     PLDEV_CNTL_IOCTL_GET_RATE_CONTROL,
     PLDEV_CNTL_IOCTL_SET_RATE_CONTROL,
	 (char *)&rateControl,
	 FIELD_OFFSET(QPCODEC_RATE_CONTROL, Artesa.fixed),
     {{0, "normal"},
      {1, "fixed"},
      {0, 0}}},
    {"RateControl-Vbr",
     PLDEV_CNTL_IOCTL_GET_RATE_CONTROL,
     PLDEV_CNTL_IOCTL_SET_RATE_CONTROL,
	 (char *)&rateControl,
	 FIELD_OFFSET(QPCODEC_RATE_CONTROL, Artesa.vbr),
     {{0, 0}}},

    /* VbrBitRate */
    {"VbrBitRate-MinBitRate",
     PLDEV_CNTL_IOCTL_GET_VBR_BIT_RATE,
     PLDEV_CNTL_IOCTL_SET_VBR_BIT_RATE,
	 (char *)&vbrControl,
	 FIELD_OFFSET(QPCODEC_VBR_BIT_RATE, min_bit_rate),
     {{0, 0}}},
    {"VbrBitRate-MaxBitRate",
     PLDEV_CNTL_IOCTL_GET_VBR_BIT_RATE,
     PLDEV_CNTL_IOCTL_SET_VBR_BIT_RATE,
	 (char *)&vbrControl,
	 FIELD_OFFSET(QPCODEC_VBR_BIT_RATE, max_bit_rate),
     {{0, 0}}},

    /* GopLoopFilter */
    {"GopLoopFilter-IntraPeriod",
     PLDEV_CNTL_IOCTL_GET_GOP_LOOP_FILTER,
     PLDEV_CNTL_IOCTL_SET_GOP_LOOP_FILTER,
	 (char *)&gopLoopFltr,
	 FIELD_OFFSET(QPCODEC_GOP_LOOP_FILTER, Artesa.intra_period),
     {{0, 0}}},
#if 1
    {"GopLoopFilter-LpFilter",
     PLDEV_CNTL_IOCTL_GET_GOP_LOOP_FILTER,
     PLDEV_CNTL_IOCTL_SET_GOP_LOOP_FILTER,
	 (char *)&gopLoopFltr,
	 FIELD_OFFSET(QPCODEC_GOP_LOOP_FILTER, Artesa.lp_filter),
     {{0, 0}}},
    {"GopLoopFilter-DisLfIdc",
     PLDEV_CNTL_IOCTL_GET_GOP_LOOP_FILTER,
     PLDEV_CNTL_IOCTL_SET_GOP_LOOP_FILTER,
	 (char *)&gopLoopFltr,
	 FIELD_OFFSET(QPCODEC_GOP_LOOP_FILTER, Artesa.dis_lf_idc),
     {{0, 0}}},
    {"GopLoopFilter-AlphaOffset",
     PLDEV_CNTL_IOCTL_GET_GOP_LOOP_FILTER,
     PLDEV_CNTL_IOCTL_SET_GOP_LOOP_FILTER,
	 (char *)&gopLoopFltr,
	 FIELD_OFFSET(QPCODEC_GOP_LOOP_FILTER, Artesa.alpha_offset),
     {{0, 0}}},
    {"GopLoopFilter-BetaOffset",
     PLDEV_CNTL_IOCTL_GET_GOP_LOOP_FILTER,
     PLDEV_CNTL_IOCTL_SET_GOP_LOOP_FILTER,
	 (char *)&gopLoopFltr,
	 FIELD_OFFSET(QPCODEC_GOP_LOOP_FILTER, Artesa.beta_offset),
     {{0, 0}}},
#endif
    {"GopLoopFilter-BNum",
     PLDEV_CNTL_IOCTL_GET_GOP_LOOP_FILTER,
     PLDEV_CNTL_IOCTL_SET_GOP_LOOP_FILTER,
	 (char *)&gopLoopFltr,
	 FIELD_OFFSET(QPCODEC_GOP_LOOP_FILTER, Artesa.B_num),
     {{0, 0}}},
    {"GopLoopFilter-T8x8",
     PLDEV_CNTL_IOCTL_GET_GOP_LOOP_FILTER,
     PLDEV_CNTL_IOCTL_SET_GOP_LOOP_FILTER,
	 (char *)&gopLoopFltr,
	 FIELD_OFFSET(QPCODEC_GOP_LOOP_FILTER, Artesa.t8x8),
     {{0, 0}}},
    {"GopLoopFilter-RefP",
     PLDEV_CNTL_IOCTL_GET_GOP_LOOP_FILTER,
     PLDEV_CNTL_IOCTL_SET_GOP_LOOP_FILTER,
	 (char *)&gopLoopFltr,
	 FIELD_OFFSET(QPCODEC_GOP_LOOP_FILTER, Artesa.ref_p),
     {{0, "off"},
      {1, "on"},
      {0, 0}}},
    {"GopLoopFilter-RefN",
     PLDEV_CNTL_IOCTL_GET_GOP_LOOP_FILTER,
     PLDEV_CNTL_IOCTL_SET_GOP_LOOP_FILTER,
	 (char *)&gopLoopFltr,
	 FIELD_OFFSET(QPCODEC_GOP_LOOP_FILTER, Artesa.ref_n),
     {{0, "all"},
      {1, "one"},
      {0, 0}}},
    {"GopLoopFilter-Idr",
     PLDEV_CNTL_IOCTL_GET_GOP_LOOP_FILTER,
     PLDEV_CNTL_IOCTL_SET_GOP_LOOP_FILTER,
	 (char *)&gopLoopFltr,
	 FIELD_OFFSET(QPCODEC_GOP_LOOP_FILTER, Artesa.IDR),
     {{0, "open"},
      {1, "close"},
      {0, 0}}},

#if 0
    /* EtControl */
    {"EtControl-EtThresholdOffset",
     QLENC_ET_CONTROL_G_ET_THRESHOLD_OFFSET,
     QLENC_ET_CONTROL_S_ET_THRESHOLD_OFFSET,
     {{0, 0}}},
    {"EtControl-EtThresholdMax",
     QLENC_ET_CONTROL_G_ET_THRESHOLD_MAX,
     QLENC_ET_CONTROL_S_ET_THRESHOLD_MAX,
     {{0, 0}}},
    {"EtControl-Et",
     QLENC_ET_CONTROL_G_ET,
     QLENC_ET_CONTROL_S_ET,
     {{0, 0}}},
    {"EtControl-EtSadThreshold",
     QLENC_ET_CONTROL_G_ET_SAD_THRESHOLD,
     QLENC_ET_CONTROL_S_ET_SAD_THRESHOLD,
     {{0, 0}}},
    {"EtControl-ChannelDataType",
     QLENC_ET_CONTROL_G_CHANNEL_DATA_TYPE,
     QLENC_ET_CONTROL_S_CHANNEL_DATA_TYPE,
     {{0, 0}}},
    {"EtControl-RefNum",
     QLENC_ET_CONTROL_G_REF_NUM,
     QLENC_ET_CONTROL_S_REF_NUM,
     {{0, 0}}},
#endif

    /* FilterControl */
    {"FilterControl-StartPixel",
     PLDEV_CNTL_IOCTL_GET_FILTER_CONTROL,
     PLDEV_CNTL_IOCTL_SET_FILTER_CONTROL,
	 (char *)&fltrControl,
	 FIELD_OFFSET(QPCODEC_FILTER_CONTROL, start_pixel),
     {{0, 0}}},
    {"FilterControl-FlEn",
     PLDEV_CNTL_IOCTL_GET_FILTER_CONTROL,
     PLDEV_CNTL_IOCTL_SET_FILTER_CONTROL,
	 (char *)&fltrControl,
	 FIELD_OFFSET(QPCODEC_FILTER_CONTROL, fl_en),
     {{0, 0}}},
    {"FilterControl-FilterStrength",
     PLDEV_CNTL_IOCTL_GET_FILTER_CONTROL,
     PLDEV_CNTL_IOCTL_SET_FILTER_CONTROL,
	 (char *)&fltrControl,
	 FIELD_OFFSET(QPCODEC_FILTER_CONTROL, filter_strength),
     {{0, 0}}},
    {"FilterControl-StartLine",
     PLDEV_CNTL_IOCTL_GET_FILTER_CONTROL,
     PLDEV_CNTL_IOCTL_SET_FILTER_CONTROL,
	 (char *)&fltrControl,
	 FIELD_OFFSET(QPCODEC_FILTER_CONTROL, start_line),
     {{0, 0}}},
    {"FilterControl-DisSkp",
     PLDEV_CNTL_IOCTL_GET_FILTER_CONTROL,
     PLDEV_CNTL_IOCTL_SET_FILTER_CONTROL,
	 (char *)&fltrControl,
	 FIELD_OFFSET(QPCODEC_FILTER_CONTROL, dis_skp),
     {{0, 0}}},

    /* OutPictureResolution */
    {"OutPictureResolution-OutPicWidth",
     PLDEV_CNTL_IOCTL_GET_OUT_PICTURE_RESOLUTION,
     PLDEV_CNTL_IOCTL_SET_OUT_PICTURE_RESOLUTION,
	 (char *)&outResolution,
	 FIELD_OFFSET(QPCODEC_OUT_PICTURE_RESOLUTION, out_pic_width),
     {{0, 0}}},
    {"OutPictureResolution-OutPicHeight",
     PLDEV_CNTL_IOCTL_GET_OUT_PICTURE_RESOLUTION,
     PLDEV_CNTL_IOCTL_SET_OUT_PICTURE_RESOLUTION,
	 (char *)&outResolution,
	 FIELD_OFFSET(QPCODEC_OUT_PICTURE_RESOLUTION, out_pic_height),
     {{0, 0}}},

    /* EncMjpegQuality, 0(fw internal control),1 to 15 */
    {"EncMjpeg-Quality",
     PLDEV_CNTL_IOCTL_GET_ENC_MJPEG_QUALITY,
     PLDEV_CNTL_IOCTL_SET_ENC_MJPEG_QUALITY,
	 (char *)&mjpegQuality,
	 0,
     {{0, 0}}},
    /* EncStillVideoInput,  0(fw internal control),1 to 30 */
    {"EncMjpeg-FrameBufNb",
     PLDEV_CNTL_IOCTL_GET_ENC_MJPEG_FRAME_BUF,
     PLDEV_CNTL_IOCTL_SET_ENC_MJPEG_FRAME_BUF,
	 (char *)&mjpegBufNo,
	 0,
     {{0, 0}}},


    /* AudioControlParam */
    {"AudioControlParam-BitRate",
     PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM,
     PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM,
	(char *)&audControl,
	FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_PARAM, bit_rate),
     {{0, 0}}},
    {"AudioControlParam-ChSel",
     PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM,
     PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM,
	(char *)&audControl,
	FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_PARAM, ch_sel),
     {{0, "mux"},
      {1, "left"},
      {2, "right"},
      {0, 0}}},
    {"AudioControlParam-SampleRate",
     PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM,
     PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM,
	(char *)&audControl,
	FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_PARAM, sample_rate),
     {{0, "44.1k"},
      {1, "48k"},
      {2, "32k"},
      {4, "22.5k"},
      {5, "24k"},
      {6, "16k"},
      {8, "8k"},
      {9, "88.2k"},
      {10, "96k"},
      {11, "64k"},
      {12, "176.4k"},
      {13, "192k"},
      {14, "128k"},
      {0, 0}}},
    {"AudioControlParam-AouPassEn",
     PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM,
     PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM,
	(char *)&audControl,
	FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_PARAM, aou_pass_en),
     {{0, "0"},
      {1, "1"},
      {0, 0}}},
    {"AudioControlParam-AiMsb",
     PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM,
     PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM,
	(char *)&audControl,
	FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_PARAM, ai_msb),
     {{0, "lsb"},
      {1, "msb"},
      {0, 0}}},
    {"AudioControlParam-LrclkI",
     PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM,
     PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM,
	(char *)&audControl,
	FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_PARAM, lrclk_i),
     {{0, "high"},
      {1, "low"},
      {0, 0}}},
    {"AudioControlParam-BclkI",
     PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM,
     PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM,
	(char *)&audControl,
	FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_PARAM, bclk_i),
     {{0, "rising"},
      {1, "falling"},
      {0, 0}}},
    {"AudioControlParam-AiI2s",
     PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM,
     PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM,
	(char *)&audControl,
	FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_PARAM, ai_i2s),
     {{0, "normal"},
      {1, "program"},
      {0, 0}}},
    {"AudioControlParam-AiRj",
     PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM,
     PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM,
	(char *)&audControl,
	FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_PARAM, ai_rj),
     {{0, "left"},
      {1, "right"},
      {0, 0}}},
    {"AudioControlParam-AiM",
     PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM,
     PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM,
	(char *)&audControl,
	FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_PARAM, ai_m),
     {{0, "slave"},
      {1, "master"},
      {0, 0}}},
    {"AudioControlParam-ChNum",
     PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM,
     PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM,
	(char *)&audControl,
	FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_PARAM, ch_num),
     {{0, "0"},
      {1, "1"},
      {2, "2"},
      {0, 0}}},
    {"AudioControlParam-Pts",
     PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM,
     PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM,
	(char *)&audControl,
	FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_PARAM, pts),
     {{0, 0}}},
    {"AudioControlParam-DisAi",
     PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM,
     PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM,
	(char *)&audControl,
	FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_PARAM, disAi),
     {{0, "0"},
      {1, "1"},
      {0, 0}}},
    {"AudioControlParam-Raw",
     PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM,
     PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM,
	(char *)&audControl,
	FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_PARAM, raw),
     {{0, 0}}},
    {"AudioControlParam-AudioType",
     PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM,
     PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM,
	(char *)&audControl,
	FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_PARAM, audio_type),
     {{0, "lpcm"},
      {1, "g726"},
      {2, "aac"},
      {3, "mp1"},
      {4, "mp2"},
      {5, "mp3"},
      {6, "ac3"},
      {7, "g723"},
      {8, "g711"},
      {9, "raw"},
      {10, "g729"},
      {11, "silk"},
      {0, 0}}},

    /* AudioControlEx */
    {"AudioControlEx-AacVer",
     PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX,
     PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_EX,
	(char *)&audControlEx,
	FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_EX, aac_ver),
     {{2, "mpeg2"},
      {4, "mpeg4"},
      {0, 0}}},
    {"AudioControlEx-HType",
     PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX,
     PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_EX,
	(char *)&audControlEx,
	FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_EX, h_type),
     {{0, "adts"},
      {1, "ga"},
      {0, 0}}},
    {"AudioControlEx-TNS",
     PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX,
     PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_EX,
	(char *)&audControlEx,
	FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_EX, TNS),
     {{0, 0}}},
    {"AudioControlEx-IS",
     PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX,
     PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_EX,
	(char *)&audControlEx,
	FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_EX, IS),
     {{0, 0}}},
    {"AudioControlEx-PNS",
     PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX,
     PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_EX,
	(char *)&audControlEx,
	FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_EX, PNS),
     {{0, 0}}},
    {"AudioControlEx-MS",
     PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX,
     PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_EX,
	(char *)&audControlEx,
	FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_EX, MS),
     {{0, 0}}},
    {"AudioControlEx-CutoffFreq",
     PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX,
     PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_EX,
	(char *)&audControlEx,
	FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_EX, cutoff_freq),
     {{0, 0}}},

    {"AudioControlEx-Mode",
     PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX,
     PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_EX,
	(char *)&audControlEx,
	FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_EX, mode),
     {{0, "a-law"},
      {1, "u-law"},
      {0, 0}}},
    {"AudioControlEx-BlockSize",
     PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX,
     PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_EX,
	(char *)&audControlEx,
	FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_EX, block_size),
     {{0, "80"},
      {1, "160"},
      {2, "240"},
      {3, "320"},
      {0, 0}}},

    {"AudioControlEx-SampleCnt",
     PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX,
     PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_EX,
	(char *)&audControlEx,
	FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_EX, sample_cnt),
     {{0, 0}}},

    {"AudioControlEx-RawType",
     PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX,
     PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_EX,
	(char *)&audControlEx,
	FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_EX, sample_cnt),
     {{0, "lpcm"},
      {1, "ac3"},
      {2, "aac"},
      {3, "aac2"},
      {4, "dts"},
     {0, 0}}},


    {"AudioControlEx-FrameLen",
     PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX,
     PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_EX,
	(char *)&audControlEx,
	FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_EX, frame_len),
     {{0, "20"},
      {1, "40"},
      {2, "60"},
      {0, 0}}},

    {"AudioControlEx-DownSample",
     PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX,
     PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_EX,
	(char *)&audControlEx,
	FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_EX, downsample),
     {{0, "0"},
      {1, "1"},
      {0, 0}}},

    {"AudioControlEx-LRSwap",
     PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX,
     PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_EX,
	(char *)&audControlEx,
	FIELD_OFFSET(QPCODEC_AUDIO_CONTROL_EX, lr_swap),
     {{0, "0"},
      {1, "1"},
      {0, 0}}},

#if 0
    /* Mp4VidNumBlock */
    {"Mp4VidNumBlock-Num",
     QLENC_MP4_VID_NUM_BLOCK_G_NUM,
     QLENC_MP4_VID_NUM_BLOCK_S_NUM,
     {{0, 0}}},
#endif

#if 0
    /* AudioEnhancement */
    {"AudioEnhancement-Gain1",
     QLENC_AUDIO_ENHANCEMENT_G_GAIN1,
     QLENC_AUDIO_ENHANCEMENT_S_GAIN1,
     {{0, 0}}},
    {"AudioEnhancement-Gain2",
     QLENC_AUDIO_ENHANCEMENT_G_GAIN2,
     QLENC_AUDIO_ENHANCEMENT_S_GAIN2,
     {{0, 0}}},
    {"AudioEnhancement-Add",
     QLENC_AUDIO_ENHANCEMENT_G_ADD,
     QLENC_AUDIO_ENHANCEMENT_S_ADD,
     {{0, 0}}},
    {"AudioEnhancement-Sub",
     QLENC_AUDIO_ENHANCEMENT_G_SUB,
     QLENC_AUDIO_ENHANCEMENT_S_SUB,
     {{0, 0}}},
    {"AudioEnhancement-Att1",
     QLENC_AUDIO_ENHANCEMENT_G_ATT1,
     QLENC_AUDIO_ENHANCEMENT_S_ATT1,
     {{0, 0}}},
    {"AudioEnhancement-Att2",
     QLENC_AUDIO_ENHANCEMENT_G_ATT2,
     QLENC_AUDIO_ENHANCEMENT_S_ATT2,
     {{0, 0}}},
    {"AudioEnhancement-LGain",
     QLENC_AUDIO_ENHANCEMENT_G_LGAIN,
     QLENC_AUDIO_ENHANCEMENT_S_LGAIN,
     {{0, 0}}},
    {"AudioEnhancement-RGain",
     QLENC_AUDIO_ENHANCEMENT_G_RGAIN,
     QLENC_AUDIO_ENHANCEMENT_S_RGAIN,
     {{0, 0}}},
#endif

#if 0
    /* EncStopMode */
    {"EncStopMode-StopMode",
     QLENC_ENC_STOP_MODE_G_STOP_MODE,
     QLENC_ENC_STOP_MODE_S_STOP_MODE,
     {{0, "immediately"},
      {1, "gop"},
      {2, "auto"},
      {0, 0}}},

    /* IndexCapFreq */
    {"IndexCapFreq-Freq",
     QLENC_INDEX_CAP_FREQ_G_FREQ,
     QLENC_INDEX_CAP_FREQ_S_FREQ,
     {{0, 0}}},

    /* EncVidPadding */
    {"EncVidPadding-Enable",
     QLENC_ENC_VID_PADDING_G_ENABLE,
     QLENC_ENC_VID_PADDING_S_ENABLE,
     {{0, 0}}},

    /* EncFreezeVideo */
    {"EncFreezeVideo-Enable",
     QLENC_ENC_FREEZE_VIDEO_G_ENABLE,
     QLENC_ENC_FREEZE_VIDEO_S_ENABLE,
     {{0, 0}}},

    /* EncStillVideoInput */
    {"EncStillVideoInput-Enable",
     QLENC_ENC_STILL_VIDEO_INPUT_G_ENABLE,
     QLENC_ENC_STILL_VIDEO_INPUT_S_ENABLE,
     {{0, 0}}},
#endif

    /* DecStreamId */
    {"DecStreamId-Type",
     PLDEV_CNTL_IOCTL_GET_DEC_STREAM_ID,
     PLDEV_CNTL_IOCTL_SET_DEC_STREAM_ID,
	 (char *)&decStreamID,
	 FIELD_OFFSET(QPCODEC_DEC_STREAM_ID_EX, read),
     {{0, "ps"},
      {1, "ts"},
      {2, "mp4"},
      {3, "es"},
      {4, "raw"},
      {5, "special1"},
      {6, "jpeg"},
      {0, 0}}},
    {"DecStreamId-Data",
     PLDEV_CNTL_IOCTL_GET_DEC_STREAM_ID,
     PLDEV_CNTL_IOCTL_SET_DEC_STREAM_ID,
	 (char *)&decStreamID,
	 FIELD_OFFSET(QPCODEC_DEC_STREAM_ID_EX, read),
     {{1, "video"},
      {2, "audio"},
      {3, "video+audio"},
      {4, "vbi"},
      {8, "user"},
      {0, 0}}},
    {"DecStreamId-XferMode",
     PLDEV_CNTL_IOCTL_GET_DEC_STREAM_ID,
     PLDEV_CNTL_IOCTL_SET_DEC_STREAM_ID,
	 (char *)&decStreamID,
	 FIELD_OFFSET(QPCODEC_DEC_STREAM_ID_EX, read),
     {{0, "frame"},
      {1, "block"},
      {0, 0}}},
    {"DecStreamId-ByteSwap",
     PLDEV_CNTL_IOCTL_GET_DEC_STREAM_ID,
     PLDEV_CNTL_IOCTL_SET_DEC_STREAM_ID,
	 (char *)&decStreamID,
	 FIELD_OFFSET(QPCODEC_DEC_STREAM_ID_EX, read),
     {{0, "0"},
      {1, "1"},
      {0, 0}}},

    /* DecOutputMode */
    {"DecOutputMode",
     PLDEV_CNTL_IOCTL_GET_DEC_OUTPUT_MODE,
     PLDEV_CNTL_IOCTL_SET_DEC_OUTPUT_MODE,
	 (char *)&decOutputMode,
	 0,
     {{0, "vou"},
      {1, "hiu"},
      {0, 0}}},

    /* DecResolution */
    {"DecResolution-InPicWidth",
     PLDEV_CNTL_IOCTL_GET_DEC_RESOLUTION,
     PLDEV_CNTL_IOCTL_SET_DEC_RESOLUTION,
	 (char *)&decResolution,
	 FIELD_OFFSET(QPCODEC_PICTURE_RESOLUTION, Artesa.in_pic_width),
     {{0, 0}}},
    {"DecResolution-InPicHeight",
     PLDEV_CNTL_IOCTL_GET_DEC_RESOLUTION,
     PLDEV_CNTL_IOCTL_SET_DEC_RESOLUTION,
	 (char *)&decResolution,
	 FIELD_OFFSET(QPCODEC_PICTURE_RESOLUTION, Artesa.in_pic_height),
     {{0, 0}}},

    /* DecStopMode */
    {"DecStopMode-StopMode",
     PLDEV_CNTL_IOCTL_GET_DEC_STOP_MODE,
     PLDEV_CNTL_IOCTL_SET_DEC_STOP_MODE,
	 (char *)&decStopMode,
	 FIELD_OFFSET(QPCODEC_DEC_STOP_MODE, stop_mode),
     {{0, "immediately"},
      {1, "eof"},
      {2, "at_pts"},
      {0, 0}}},
    {"DecStopMode-StopDispMode",
     PLDEV_CNTL_IOCTL_GET_DEC_STOP_MODE,
     PLDEV_CNTL_IOCTL_SET_DEC_STOP_MODE,
	 (char *)&decStopMode,
	 FIELD_OFFSET(QPCODEC_DEC_STOP_MODE, stop_disp_mode),
     {{0, "last_frame"},
      {1, "black"},
      {0, 0}}},
    {"DecStopMode-PTS1",
     PLDEV_CNTL_IOCTL_GET_DEC_STOP_MODE,
     PLDEV_CNTL_IOCTL_SET_DEC_STOP_MODE,
	 (char *)&decStopMode,
	 FIELD_OFFSET(QPCODEC_DEC_STOP_MODE, PTS1),
     {{0, 0}}},
    {"DecStopMode-PTS2",
     PLDEV_CNTL_IOCTL_GET_DEC_STOP_MODE,
     PLDEV_CNTL_IOCTL_SET_DEC_STOP_MODE,
	 (char *)&decStopMode,
	 FIELD_OFFSET(QPCODEC_DEC_STOP_MODE, PTS2),
     {{0, 0}}},

#if 0
    /* DecTrickMode */
    {"DecTrickMode-PlaySpeed",
     QLDEC_TRICK_MODE_G_PLAY_SPEED,
     QLDEC_TRICK_MODE_S_PLAY_SPEED,
     {{0, 0}}},
    {"DecTrickMode-PlayDirection",
     QLDEC_TRICK_MODE_G_PLAY_DIRECTION,
     QLDEC_TRICK_MODE_S_PLAY_DIRECTION,
     {{0, 0}}},
    {"DecTrickMode-Step",
     QLDEC_TRICK_MODE_G_STEP,
     QLDEC_TRICK_MODE_S_STEP,
     {{0, 0}}},
    {"DecTrickMode-PTS",
     QLDEC_TRICK_MODE_G_PTS,
     QLDEC_TRICK_MODE_S_PTS,
     {{0, 0}}},
    {"DecTrickMode-FieldPreference",
     QLDEC_TRICK_MODE_G_FIELD_PREFERENCE,
     QLDEC_TRICK_MODE_S_FIELD_PREFERENCE,
     {{0, 0}}},
#endif

    /* DecDispAttrib */
    {"DecDispAttrib-TVSystem",
     PLDEV_CNTL_IOCTL_GET_DEC_DISP_ATTRIB,
     PLDEV_CNTL_IOCTL_SET_DEC_DISP_ATTRIB,
	 (char *)&decDispAttrib,
	 FIELD_OFFSET(QPCODEC_DEC_DISP_ATTRIB, TVSystem),
     {{0, 0}}},
    {"DecDispAttrib-ForceRemote",
     PLDEV_CNTL_IOCTL_GET_DEC_DISP_ATTRIB,
     PLDEV_CNTL_IOCTL_SET_DEC_DISP_ATTRIB,
	 (char *)&decDispAttrib,
	 FIELD_OFFSET(QPCODEC_DEC_DISP_ATTRIB, ForceRemote),
     {{0, "0"},
      {1, "1"},
      {0, 0}}},
    {"DecDispAttrib-InputSizeX",
     PLDEV_CNTL_IOCTL_GET_DEC_DISP_ATTRIB,
     PLDEV_CNTL_IOCTL_SET_DEC_DISP_ATTRIB,
	 (char *)&decDispAttrib,
	 FIELD_OFFSET(QPCODEC_DEC_DISP_ATTRIB, inputSize.Read),
     {{0, 0}}},
    {"DecDispAttrib-InputSizeY",
     PLDEV_CNTL_IOCTL_GET_DEC_DISP_ATTRIB,
     PLDEV_CNTL_IOCTL_SET_DEC_DISP_ATTRIB,
	 (char *)&decDispAttrib,
	 FIELD_OFFSET(QPCODEC_DEC_DISP_ATTRIB, inputSize.Read),
     {{0, 0}}},
    {"DecDispAttrib-OutputSizeX",
     PLDEV_CNTL_IOCTL_GET_DEC_DISP_ATTRIB,
     PLDEV_CNTL_IOCTL_SET_DEC_DISP_ATTRIB,
	 (char *)&decDispAttrib,
	 FIELD_OFFSET(QPCODEC_DEC_DISP_ATTRIB, outputSize.Read),
     {{0, 0}}},
    {"DecDispAttrib-OutputSizeY",
     PLDEV_CNTL_IOCTL_GET_DEC_DISP_ATTRIB,
     PLDEV_CNTL_IOCTL_SET_DEC_DISP_ATTRIB,
	 (char *)&decDispAttrib,
	 FIELD_OFFSET(QPCODEC_DEC_DISP_ATTRIB, outputSize.Read),
     {{0, 0}}},
    {"DecDispAttrib-InputOriginX",
     PLDEV_CNTL_IOCTL_GET_DEC_DISP_ATTRIB,
     PLDEV_CNTL_IOCTL_SET_DEC_DISP_ATTRIB,
	 (char *)&decDispAttrib,
	 FIELD_OFFSET(QPCODEC_DEC_DISP_ATTRIB, inputOrigin.Read),
     {{0, 0}}},
    {"DecDispAttrib-InputOriginY",
     PLDEV_CNTL_IOCTL_GET_DEC_DISP_ATTRIB,
     PLDEV_CNTL_IOCTL_SET_DEC_DISP_ATTRIB,
	 (char *)&decDispAttrib,
	 FIELD_OFFSET(QPCODEC_DEC_DISP_ATTRIB, inputOrigin.Read),
     {{0, 0}}},
    {"DecDispAttrib-OutputOriginX",
     PLDEV_CNTL_IOCTL_GET_DEC_DISP_ATTRIB,
     PLDEV_CNTL_IOCTL_SET_DEC_DISP_ATTRIB,
	 (char *)&decDispAttrib,
	 FIELD_OFFSET(QPCODEC_DEC_DISP_ATTRIB, outputOrigin.Read),
     {{0, 0}}},
    {"DecDispAttrib-OutputOriginY",
     PLDEV_CNTL_IOCTL_GET_DEC_DISP_ATTRIB,
     PLDEV_CNTL_IOCTL_SET_DEC_DISP_ATTRIB,
	 (char *)&decDispAttrib,
	 FIELD_OFFSET(QPCODEC_DEC_DISP_ATTRIB, outputOrigin.Read),
     {{0, 0}}},

    /* DecDispMode */
    {"DecDispMode-Operation",
     PLDEV_CNTL_IOCTL_GET_DEC_DISP_MODE,
     PLDEV_CNTL_IOCTL_SET_DEC_DISP_MODE,
	 (char *)&decDispMode,
	 FIELD_OFFSET(QPCODEC_DEC_DISP_MODE, operation),
     {{0, 0}}},
    {"DecDispMode-DisplayMode",
     PLDEV_CNTL_IOCTL_GET_DEC_DISP_MODE,
     PLDEV_CNTL_IOCTL_SET_DEC_DISP_MODE,
	 (char *)&decDispMode,
	 FIELD_OFFSET(QPCODEC_DEC_DISP_MODE, displayMode),
     {{0, 0}}},

    /* DecAudioControl */
    {"DecAudioControl-AudioType",
     PLDEV_CNTL_IOCTL_GET_DEC_AUDIO_CONTROL,
     PLDEV_CNTL_IOCTL_SET_DEC_AUDIO_CONTROL,
      (char *)&decAudControl,
	FIELD_OFFSET(QPCODEC_DEC_AUDIO_CONTROL, audio_type),
     {{0, "lpcm"},
      {1, "g726"},
      {2, "aac"},
      {3, "mp1"},
      {4, "mp2"},
      {5, "mp3"},
      {6, "ac3"},
      {7, "g723"},
      {8, "g711"},
      {10, "g729"},
      {11, "silk"},
     {0, 0}}},
    {"DecAudioControl-ChNum",
     PLDEV_CNTL_IOCTL_GET_DEC_AUDIO_CONTROL,
     PLDEV_CNTL_IOCTL_SET_DEC_AUDIO_CONTROL,
      (char *)&decAudControl,
	FIELD_OFFSET(QPCODEC_DEC_AUDIO_CONTROL, ch_num),
     {{0, "0"},
      {1, "1"},
      {2, "2"},
     {0, 0}}},
    {"DecAudioControl-SampleRate",
     PLDEV_CNTL_IOCTL_GET_DEC_AUDIO_CONTROL,
     PLDEV_CNTL_IOCTL_SET_DEC_AUDIO_CONTROL,
      (char *)&decAudControl,
	FIELD_OFFSET(QPCODEC_DEC_AUDIO_CONTROL, sample_rate),
     {{0, "44.1k"},
      {1, "48k"},
      {2, "32k"},
      {4, "22.5k"},
      {5, "24k"},
      {6, "16k"},
      {8, "8k"},
     {0, 0}}},
    {"DecAudioControl-mode",
     PLDEV_CNTL_IOCTL_GET_DEC_AUDIO_CONTROL,
     PLDEV_CNTL_IOCTL_SET_DEC_AUDIO_CONTROL,
      (char *)&decAudControl,
	FIELD_OFFSET(QPCODEC_DEC_AUDIO_CONTROL, mode),
     {{0, "a-law"},
      {1, "u-law"},
      {0, 0}}},
    {"DecAudioControl-UpSample",
     PLDEV_CNTL_IOCTL_GET_DEC_AUDIO_CONTROL,
     PLDEV_CNTL_IOCTL_SET_DEC_AUDIO_CONTROL,
      (char *)&decAudControl,
	FIELD_OFFSET(QPCODEC_DEC_AUDIO_CONTROL, upsample),
     {{0, "0"},
      {1, "1"},
      {0, 0}}},

    /* SysLink */
    {"SysLink-VideoInput",
     PLDEV_CNTL_IOCTL_GET_SYS_LINK,
     PLDEV_CNTL_IOCTL_SET_SYS_LINK,
	 (char *)&sys_link,
	 FIELD_OFFSET(QPCODEC_SYS_LINK, video_input),
     {{0, "viu"},
      {1, "host"},
      {2, "usb"},
      {4, "null"},
      {8, "internal"},
      {0, 0}}},
    {"SysLink-VideoInCh",
     PLDEV_CNTL_IOCTL_GET_SYS_LINK,
     PLDEV_CNTL_IOCTL_SET_SYS_LINK,
	 (char *)&sys_link,
	 FIELD_OFFSET(QPCODEC_SYS_LINK, video_in_ch),
     {{0, 0}}},
    {"SysLink-VideoOutput",
     PLDEV_CNTL_IOCTL_GET_SYS_LINK,
     PLDEV_CNTL_IOCTL_SET_SYS_LINK,
	 (char *)&sys_link,
	 FIELD_OFFSET(QPCODEC_SYS_LINK, video_output),
     {{0, "vou"},
      {1, "host"},
      {2, "special"},
      {4, "null"},
      {8, "internal"},
      {0, 0}}},
    {"SysLink-VideoOutCh",
     PLDEV_CNTL_IOCTL_GET_SYS_LINK,
     PLDEV_CNTL_IOCTL_SET_SYS_LINK,
	 (char *)&sys_link,
	 FIELD_OFFSET(QPCODEC_SYS_LINK, video_out_ch),
     {{0, 0}}},
    {"SysLink-AudioInput",
     PLDEV_CNTL_IOCTL_GET_SYS_LINK,
     PLDEV_CNTL_IOCTL_SET_SYS_LINK,
	 (char *)&sys_link,
	 FIELD_OFFSET(QPCODEC_SYS_LINK, audio_input),
     {{0, "aiu"},
      {1, "host"},
      {8, "internal"},
      {0, 0}}},
    {"SysLink-AudioInCh",
     PLDEV_CNTL_IOCTL_GET_SYS_LINK,
     PLDEV_CNTL_IOCTL_SET_SYS_LINK,
	 (char *)&sys_link,
	 FIELD_OFFSET(QPCODEC_SYS_LINK, audio_in_ch),
     {{0, 0}}},
    {"SysLink-AudioOutput",
     PLDEV_CNTL_IOCTL_GET_SYS_LINK,
     PLDEV_CNTL_IOCTL_SET_SYS_LINK,
	 (char *)&sys_link,
	 FIELD_OFFSET(QPCODEC_SYS_LINK, audio_output),
     {{0, "aou"},
      {1, "host"},
      {8, "internal"},
      {0, 0}}},
    {"SysLink-AudioOutCh",
     PLDEV_CNTL_IOCTL_GET_SYS_LINK,
     PLDEV_CNTL_IOCTL_SET_SYS_LINK,
	 (char *)&sys_link,
	 FIELD_OFFSET(QPCODEC_SYS_LINK, audio_out_ch),
     {{0, 0}}},

    /* SysFunction */
    {"SysFunction-Function",
     PLDEV_CNTL_IOCTL_GET_SYS_FUNCTION,
     PLDEV_CNTL_IOCTL_SET_SYS_FUNCTION,
	 (char *)&function,
	 FIELD_OFFSET(QPCODEC_SYS_FUNCTION, function),
     {{0, "encode"},
      {1, "decode"},
      {0, 0}}},
    {"SysFunction-Video",
     PLDEV_CNTL_IOCTL_GET_SYS_FUNCTION,
     PLDEV_CNTL_IOCTL_SET_SYS_FUNCTION,
	 (char *)&function,
	 FIELD_OFFSET(QPCODEC_SYS_FUNCTION, function),
     {{0, "none"},
      {1, "h264"},
      {2, "jpeg"},
      {4, "passthrough"},
      {5, "decimation"},
      {6, "motiondetect"},
      {7, "viu_osd"},
      {8, "vou_osd"},
      {0, 0}}},
    {"SysFunction-Audio",
     PLDEV_CNTL_IOCTL_GET_SYS_FUNCTION,
     PLDEV_CNTL_IOCTL_SET_SYS_FUNCTION,
	 (char *)&function,
	 FIELD_OFFSET(QPCODEC_SYS_FUNCTION, function),
     {{0, "none"},
      {1, "audio"},
      {4, "passthrough"},
      {0, 0}}},

#if 0
    /* MiscControl */
    {"MiscControl-VideoInputSystem",
     QLMISC_CONTROL_G_VIDEO_INPUT_SYSTEM,
     QLMISC_CONTROL_S_VIDEO_INPUT_SYSTEM,
     {{0x1, "ntsc"},
      {0x2, "pal"},
      {0x4, "secam"},
      {0x8, "720p"},
      {0x10, "1080i"},
      {0, 0}}},
    {"InsertKeyFrame",
     QLENC_INSERT_KEY_FRAME_G,
     QLENC_INSERT_KEY_FRAME_S,
     {{0, 0}}},
#endif

    {"AouConfig-AoEn",
     PLDEV_CNTL_IOCTL_GET_AOU_CONFIG,
     PLDEV_CNTL_IOCTL_SET_AOU_CONFIG,
	 (char *)&aouConfig,
	 FIELD_OFFSET(QPCODEC_AOU_CONFIG, ao_en),
     {
      {0, 0}}},
  {"AouConfig-AoS",
     PLDEV_CNTL_IOCTL_GET_AOU_CONFIG,
     PLDEV_CNTL_IOCTL_SET_AOU_CONFIG,
	 (char *)&aouConfig,
	 FIELD_OFFSET(QPCODEC_AOU_CONFIG, ao_s),
     {
      {0, "master"},
      {1, "slave"},
      {0, 0}}},

    {"OsdMemory-Command",
     PLDEV_CNTL_IOCTL_GET_OSD_MEMORY,
     PLDEV_CNTL_IOCTL_SET_OSD_MEMORY,
	 (char *)&osdMem,
	 FIELD_OFFSET(QPCODEC_OSD_MEM_CONFIG, command),
     {{0, "request"},
      {1, "return"},
      {2, "free"},
      {0, 0}}},

    {"OsdMemory-Size",
     PLDEV_CNTL_IOCTL_GET_OSD_MEMORY,
     PLDEV_CNTL_IOCTL_SET_OSD_MEMORY,
	 (char *)&osdMem,
	 FIELD_OFFSET(QPCODEC_OSD_MEM_CONFIG, memSize),
     {
      {1024*1024, "1MB"},
      {2*1024*1024, "2MB"},
      {0, 0}}},

    {"OsdMemory-StartAddress",
     PLDEV_CNTL_IOCTL_GET_OSD_MEMORY,
     PLDEV_CNTL_IOCTL_SET_OSD_MEMORY,
	 (char *)&osdMem,
	 FIELD_OFFSET(QPCODEC_OSD_MEM_CONFIG, memStartAddress),
     {
      {0, 0}}},

    {"Decimation-input_format",
     PLDEV_CNTL_IOCTL_GET_DECIMATION,
     PLDEV_CNTL_IOCTL_SET_DECIMATION,
	 (char *)&decimation,
	 FIELD_OFFSET(QPCODEC_DECIMATION_FACTOR, input_format),
     {{0, "qyuv"},
      {1, "yv12"},
      {0, 0}}},

    {"Decimation-output_format",
     PLDEV_CNTL_IOCTL_GET_DECIMATION,
     PLDEV_CNTL_IOCTL_SET_DECIMATION,
	 (char *)&decimation,
	 FIELD_OFFSET(QPCODEC_DECIMATION_FACTOR, output_format),
     {{0, "qyuv"},
      {1, "yuy2"},
      {0, 0}}},

    {"Decimation-scale_factor",
     PLDEV_CNTL_IOCTL_GET_DECIMATION,
     PLDEV_CNTL_IOCTL_SET_DECIMATION,
	 (char *)&decimation,
	 FIELD_OFFSET(QPCODEC_DECIMATION_FACTOR, scale_factor.read),
     {
      {0, 0}}},

    {"DeInterlace-mode",
     PLDEV_CNTL_IOCTL_GET_DEINTERLACE,
     PLDEV_CNTL_IOCTL_SET_DEINTERLACE,
	 (char *)&deinterlacemode,
	 0,
     {{0, "none"},
      {1, "topfield"},
      {2, "twofield"},
      {0, 0}}},

    {"EncCapMode-CapMode",
     PLDEV_CNTL_IOCTL_GET_ENC_CAP_MODE,
     PLDEV_CNTL_IOCTL_SET_ENC_CAP_MODE,
	 (char *)&enccapmode,
	 FIELD_OFFSET(QPCODEC_ENC_CAP_MODE, capMode),
     {{0, "continuous"},
      {1, "ondemand"},
      {0, 0}}},

    {"EncCapMode-TrigMode",
     PLDEV_CNTL_IOCTL_GET_ENC_CAP_MODE,
     PLDEV_CNTL_IOCTL_SET_ENC_CAP_MODE,
	 (char *)&enccapmode,
	 FIELD_OFFSET(QPCODEC_ENC_CAP_MODE, trigMode),
     {{1, "api"},
      {2, "gpio"},
      {0, 0}}},

    {"EncCapMode-GpioPin",
     PLDEV_CNTL_IOCTL_GET_ENC_CAP_MODE,
     PLDEV_CNTL_IOCTL_SET_ENC_CAP_MODE,
	 (char *)&enccapmode,
	 FIELD_OFFSET(QPCODEC_ENC_CAP_MODE, gpioPin),
     {
      {0, 0}}},

    {"ExternalTrigger-Enable",
     PLDEV_CNTL_IOCTL_GET_EXTERNAL_TRIGGER,
     PLDEV_CNTL_IOCTL_SET_EXTERNAL_TRIGGER,
	 (char *)&externaltriggermode,
	 FIELD_OFFSET(QPCODEC_EXTERNAL_TRIGGER_MODE, Enable),
     {
      {0, 0}}},

    {"ExternalTrigger-GpioPin",
     PLDEV_CNTL_IOCTL_GET_EXTERNAL_TRIGGER,
     PLDEV_CNTL_IOCTL_SET_EXTERNAL_TRIGGER,
	 (char *)&externaltriggermode,
	 FIELD_OFFSET(QPCODEC_EXTERNAL_TRIGGER_MODE, gpioPin),
     {
      {0, 0}}},

    {"PTSCounterReset-Enable",
     PLDEV_CNTL_IOCTL_GET_PTS_COUNTER_RESET,
     PLDEV_CNTL_IOCTL_SET_PTS_COUNTER_RESET,
	 (char *)&ptscounterreset,
	 FIELD_OFFSET(QPCODEC_PTS_COUNTER_RESET, Enable),
     {
      {0, 0}}},

    {"PTSCounterReset-GpioPin",
     PLDEV_CNTL_IOCTL_GET_PTS_COUNTER_RESET,
     PLDEV_CNTL_IOCTL_SET_PTS_COUNTER_RESET,
	 (char *)&ptscounterreset,
	 FIELD_OFFSET(QPCODEC_PTS_COUNTER_RESET, gpioPin),
     {
      {0, 0}}},

    {"PTSCounterReset-Immediate",
     PLDEV_CNTL_IOCTL_GET_PTS_COUNTER_RESET,
     PLDEV_CNTL_IOCTL_SET_PTS_COUNTER_RESET,
	 (char *)&ptscounterreset,
	 FIELD_OFFSET(QPCODEC_PTS_COUNTER_RESET, immediate),
     {
      {0, 0}}},

    {"MiscRateControl-QpUpdateInterval",
     PLDEV_CNTL_IOCTL_GET_RATE_CONTROL_EX,
     PLDEV_CNTL_IOCTL_SET_RATE_CONTROL_EX,
	 (char *)&rateControlEx,
	 FIELD_OFFSET(RATE_CONTROL_EX, Read),
     {
      {0, 0}}},

    {"MiscRateControl-LocalActivity",
     PLDEV_CNTL_IOCTL_GET_RATE_CONTROL_EX,
     PLDEV_CNTL_IOCTL_SET_RATE_CONTROL_EX,
	 (char *)&rateControlEx,
	 FIELD_OFFSET(RATE_CONTROL_EX, Read),
     {{0, "0"},
      {1, "1"},
      {0, 0}}},

	{"MiscRateControl-MDInterval",
     PLDEV_CNTL_IOCTL_GET_RATE_CONTROL_EX,
     PLDEV_CNTL_IOCTL_SET_RATE_CONTROL_EX,
	 (char *)&rateControlEx,
	 FIELD_OFFSET(RATE_CONTROL_EX, Read),
     {
      {0, 0}}},

	{"LargeCompressBuf-Enable",
     PLDEV_CNTL_IOCTL_GET_LARGE_COMPRESS_BUFFER_CONTROL,
     PLDEV_CNTL_IOCTL_SET_LARGE_COMPRESS_BUFFER_CONTROL,
	 (char *)&largecompressbuffercontrol,
	 0,
     {
      {0, 0}}},
	{"LargeCompressBuf-MaxBitrate",
     PLDEV_CNTL_IOCTL_GET_LARGE_COMPRESS_BUFFER_CONTROL,
     PLDEV_CNTL_IOCTL_SET_LARGE_COMPRESS_BUFFER_CONTROL,
	 (char *)&largecompressbuffercontrol,
	 0,
     {
      {0, 0}}},

    {"LowBitrateMode-Enable",
     PLDEV_CNTL_IOCTL_GET_ENABLE_LOW_BITRATE_MODE,
     PLDEV_CNTL_IOCTL_SET_ENABLE_LOW_BITRATE_MODE,
	 (char *)&lowbitratemode,
	 0,
     {
      {0, 0}}},

    {"AVDiscardControl-Enable",
     PLDEV_CNTL_IOCTL_GET_ENABLE_AV_DISCARD_CONTROL,
     PLDEV_CNTL_IOCTL_SET_ENABLE_AV_DISCARD_CONTROL,
	 (char *)&avdiscardcontrolmode,
	 0,
     {
      {0, 0}}},

    {"UseSWPTS-Mode",
     PLDEV_CNTL_IOCTL_GET_USE_SW_PTS,
     PLDEV_CNTL_IOCTL_SET_USE_SW_PTS,
	 (char *)&useswptsmode,
	 0,
     {
      {0, 0}}},

    {0, 0, 0, 0}
};


#if 0
void usage(int nExit)
{
    int nItemIndex, nDescriptIndex;
    int nStuffing;

    printf("\n");
    printf("  [%s version %s]\n", MODULE_NAME, MODULE_VERSION);
    printf("  -------------------------------------------------\n");
    printf("  %s <stream> <arg1> <arg2> ...\n", MODULE_NAME);
    printf("\n");
    printf("  stream selection,\n");
    printf("    enc                                => Config VIU encode.\n");
    printf("    dec                                => Config VOU decode.\n");
    printf("    hiuenc                             => Config HIU encode.\n");
    printf("    hiudec                             => Config HIU decode.\n");
    printf("    hiumjpegdec                        => Config HIU MJPEG decode.\n");
    printf("\n");
    printf("  arg for card selection,\n");
    printf("    card                               => Card number. Could be 0/1/2/3, default is 0.\n");
    printf("\n");
    printf("  arg command,\n");
    printf("    get <item_name>|all                => Get config item.\n");
    printf("    set <item_name>                    => Set config item.\n");
    printf("    setscript <script_file>            => Set config item(s) from script file.\n");
    printf("\n");
    printf("  config items,\n");

    nItemIndex = 0;
    while (gstaConfigTable[nItemIndex].pszItemName != 0) {
        printf("    %s", gstaConfigTable[nItemIndex].pszItemName);

        nStuffing = 36 - strlen(gstaConfigTable[nItemIndex].pszItemName);
        while (nStuffing--) {
            printf(" ");
        }

        nDescriptIndex = 0;
        while (gstaConfigTable[nItemIndex].staDescript[nDescriptIndex].pszDescript != 0) {
            printf("/%s", gstaConfigTable[nItemIndex].staDescript[nDescriptIndex].pszDescript);
            nDescriptIndex++;
        }
        printf("\n");

        nItemIndex++;
    }

    printf("\n");
    printf("  examples, \n", MODULE_NAME);
    printf("    %s -enc -getall\n", MODULE_NAME);
    printf("    %s -enc -get SystemControl-StreamType\n", MODULE_NAME);
    printf("    %s -card 1 -enc -setscript config.dat\n", MODULE_NAME);

    printf("\n");

    if (nExit) {
        exit(nExit);
    }
}
#endif


void do_get_all(DEVHANDLE hDev)
{
	int			lErr = 0;
	unsigned int	ulValue;
	int				nItemIndex, nDescriptIndex;
	int				nStuffing;

	printf("\n");

	nItemIndex = 0;
	while (gstaConfigTable[nItemIndex].pszItemName != 0)
	{
		printf("    %s", gstaConfigTable[nItemIndex].pszItemName);
		nStuffing = 36 - (int)strlen(gstaConfigTable[nItemIndex].pszItemName);
		while (nStuffing--)
		{
			printf(" ");
		}
		printf("=    ");
		lErr = ioctl(hDev, gstaConfigTable[nItemIndex].nControlIdGet, gstaConfigTable[nItemIndex].pConfig);
		if (!lErr)
		{
			if (!strcasecmp(gstaConfigTable[nItemIndex].pszItemName, "SystemControl-Version"))
			{
				ulValue = sysControl.Artesa.version;
			}
			else if (!strcasecmp(gstaConfigTable[nItemIndex].pszItemName, "SystemControl-XferMode"))
			{
				ulValue = sysControl.Artesa.xfer_mode;
			}
			else if (!strcasecmp(gstaConfigTable[nItemIndex].pszItemName, "DecStreamId-Type"))
			{
				ulValue = decStreamID.streamIdEx.stream_type;
			}
			else if (!strcasecmp(gstaConfigTable[nItemIndex].pszItemName, "DecStreamId-Data"))
			{
				ulValue = decStreamID.streamIdEx.stream_data;
			}
			else if (!strcasecmp(gstaConfigTable[nItemIndex].pszItemName, "DecStreamId-XferMode"))
			{
				ulValue = decStreamID.streamIdEx.xfer_mode;
			}
			else if (!strcasecmp(gstaConfigTable[nItemIndex].pszItemName, "DecStreamId-ByteSwap"))
			{
				ulValue = decStreamID.streamIdEx.bytes_swap;
			}
			else if (!strcasecmp(gstaConfigTable[nItemIndex].pszItemName, "DecDispAttrib-InputSizeX"))
			{
				ulValue = decDispAttrib.inputSize.Attrib.x;
			}
			else if (!strcasecmp(gstaConfigTable[nItemIndex].pszItemName, "DecDispAttrib-InputSizeY"))
			{
				ulValue = decDispAttrib.inputSize.Attrib.y;
			}
			else if (!strcasecmp(gstaConfigTable[nItemIndex].pszItemName, "DecDispAttrib-OutputSizeX"))
			{
				ulValue = decDispAttrib.outputSize.Attrib.x;
			}
			else if (!strcasecmp(gstaConfigTable[nItemIndex].pszItemName, "DecDispAttrib-OutputSizeY"))
			{
				ulValue = decDispAttrib.outputSize.Attrib.y;
			}
			else if (!strcasecmp(gstaConfigTable[nItemIndex].pszItemName, "DecDispAttrib-InputOriginX"))
			{
				ulValue = decDispAttrib.inputOrigin.Attrib.x;
			}
			else if (!strcasecmp(gstaConfigTable[nItemIndex].pszItemName, "DecDispAttrib-InputOriginY"))
			{
				ulValue = decDispAttrib.inputOrigin.Attrib.y;
			}
			else if (!strcasecmp(gstaConfigTable[nItemIndex].pszItemName, "DecDispAttrib-OutputOriginX"))
			{
				ulValue = decDispAttrib.outputOrigin.Attrib.x;
			}
			else if (!strcasecmp(gstaConfigTable[nItemIndex].pszItemName, "DecDispAttrib-OutputOriginY"))
			{
				ulValue = decDispAttrib.outputOrigin.Attrib.y;
			}
			else if (!strcasecmp(gstaConfigTable[nItemIndex].pszItemName, "SysFunction-Function"))
			{
				ulValue = function.functionEx.function;
			}
			else if (!strcasecmp(gstaConfigTable[nItemIndex].pszItemName, "SysFunction-Video"))
			{
				ulValue = function.functionEx.video;
			}
			else if (!strcasecmp(gstaConfigTable[nItemIndex].pszItemName, "SysFunction-Audio"))
			{
				ulValue = function.functionEx.audio;
			}
			else if (!strcasecmp(gstaConfigTable[nItemIndex].pszItemName, "MiscRateControl-QpUpdateInterval"))
			{
				ulValue = rateControlEx.Artesa.base_qp_length;
			}
			else if (!strcasecmp(gstaConfigTable[nItemIndex].pszItemName, "MiscRateControl-LocalActivity"))
			{
				ulValue = rateControlEx.Artesa.local_activity_on;
			}
			else if (!strcasecmp(gstaConfigTable[nItemIndex].pszItemName, "MiscRateControl-MDInterval"))
			{
				ulValue = rateControlEx.Artesa.motion_detect_interval;
			}
			else if (!strcasecmp(gstaConfigTable[nItemIndex].pszItemName, "LargeCompressBuf-Enable"))
			{
				if (largecompressbuffercontrol & QPCODEC_LARGE_COMPRESS_BUFFER_CONTROL_ENABLE)
					ulValue = 1;
				else
					ulValue = 0;
			}
			else if (!strcasecmp(gstaConfigTable[nItemIndex].pszItemName, "LargeCompressBuf-MaxBitrate"))
			{
				ulValue = largecompressbuffercontrol & (~QPCODEC_LARGE_COMPRESS_BUFFER_CONTROL_ENABLE);
			}
			else
			{
				memcpy(&ulValue,
					   gstaConfigTable[nItemIndex].pConfig + gstaConfigTable[nItemIndex].offset,
					   sizeof(unsigned int)
					   );
			}

			nDescriptIndex = 0;
			while (gstaConfigTable[nItemIndex].staDescript[nDescriptIndex].pszDescript != 0)
			{
				if (gstaConfigTable[nItemIndex].staDescript[nDescriptIndex].nValue == ulValue)
				{
					printf("%s\n", gstaConfigTable[nItemIndex].staDescript[nDescriptIndex].pszDescript);
					break;
				}
				nDescriptIndex++;
			}
			if (gstaConfigTable[nItemIndex].staDescript[nDescriptIndex].pszDescript == 0)
			{
				printf("%u\n", ulValue);
			}
		}
		else
		{
			printf("failed in ioctl()\n");
		}
		nItemIndex++;
    }

    printf("\n");
}


int do_get(DEVHANDLE hDev,
			char *pszItem
			)
{
	int			lErr = 1;
	unsigned int	ulValue;
	int				nItemIndex, nDescriptIndex;
	int				nStuffing;

	nItemIndex = 0;
	while (gstaConfigTable[nItemIndex].pszItemName != 0)
	{
		if (!strcasecmp(pszItem, gstaConfigTable[nItemIndex].pszItemName))
		{
			printf("      %s", gstaConfigTable[nItemIndex].pszItemName);
			nStuffing = 20 - (int)strlen(gstaConfigTable[nItemIndex].pszItemName);
			while (nStuffing--)
			{
				printf(" ");
			}
			printf("=    ");
			lErr = ioctl(hDev, gstaConfigTable[nItemIndex].nControlIdGet, gstaConfigTable[nItemIndex].pConfig);
			if (!lErr)
			{
				if (!strcasecmp(pszItem, "SystemControl-Version"))
				{
					ulValue = sysControl.Artesa.version;
				}
				else if (!strcasecmp(pszItem, "SystemControl-XferMode"))
				{
					ulValue = sysControl.Artesa.xfer_mode;
				}
				else if (!strcasecmp(pszItem, "DecStreamId-Type"))
				{
					ulValue = decStreamID.streamIdEx.stream_type;
				}
				else if (!strcasecmp(pszItem, "DecStreamId-Data"))
				{
					ulValue = decStreamID.streamIdEx.stream_data;
				}
				else if (!strcasecmp(pszItem, "DecStreamId-XferMode"))
				{
					ulValue = decStreamID.streamIdEx.xfer_mode;
				}
				else if (!strcasecmp(pszItem, "DecStreamId-ByteSwap"))
				{
					ulValue = decStreamID.streamIdEx.bytes_swap;
				}
				else if (!strcasecmp(pszItem, "DecDispAttrib-InputSizeX"))
				{
					ulValue = decDispAttrib.inputSize.Attrib.x;
				}
				else if (!strcasecmp(pszItem, "DecDispAttrib-InputSizeY"))
				{
					ulValue = decDispAttrib.inputSize.Attrib.y;
				}
				else if (!strcasecmp(pszItem, "DecDispAttrib-OutputSizeX"))
				{
					ulValue = decDispAttrib.outputSize.Attrib.x;
				}
				else if (!strcasecmp(pszItem, "DecDispAttrib-OutputSizeY"))
				{
					ulValue = decDispAttrib.outputSize.Attrib.y;
				}
				else if (!strcasecmp(pszItem, "DecDispAttrib-InputOriginX"))
				{
					ulValue = decDispAttrib.inputOrigin.Attrib.x;
				}
				else if (!strcasecmp(pszItem, "DecDispAttrib-InputOriginY"))
				{
					ulValue = decDispAttrib.inputOrigin.Attrib.y;
				}
				else if (!strcasecmp(pszItem, "DecDispAttrib-OutputOriginX"))
				{
					ulValue = decDispAttrib.outputOrigin.Attrib.x;
				}
				else if (!strcasecmp(pszItem, "DecDispAttrib-OutputOriginY"))
				{
					ulValue = decDispAttrib.outputOrigin.Attrib.y;
				}
				else if (!strcasecmp(pszItem, "SysFunction-Function"))
				{
					ulValue = function.functionEx.function;
				}
				else if (!strcasecmp(pszItem, "SysFunction-Video"))
				{
					ulValue = function.functionEx.video;
				}
				else if (!strcasecmp(pszItem, "SysFunction-Audio"))
				{
					ulValue = function.functionEx.audio;
				}
				else if (!strcasecmp(pszItem, "MiscRateControl-QpUpdateInterval"))
				{
					ulValue = rateControlEx.Artesa.base_qp_length;
				}
				else if (!strcasecmp(pszItem, "MiscRateControl-LocalActivity"))
				{
					ulValue = rateControlEx.Artesa.local_activity_on;
				}
				else if (!strcasecmp(pszItem, "MiscRateControl-MDInterval"))
				{
					ulValue = rateControlEx.Artesa.motion_detect_interval;
				}
				else if (!strcasecmp(pszItem, "LargeCompressBuf-Enable"))
				{
					if (largecompressbuffercontrol & QPCODEC_LARGE_COMPRESS_BUFFER_CONTROL_ENABLE)
						ulValue = 1;
					else
						ulValue = 0;
				}
				else if (!strcasecmp(pszItem, "LargeCompressBuf-MaxBitrate"))
				{
					ulValue = largecompressbuffercontrol & (~QPCODEC_LARGE_COMPRESS_BUFFER_CONTROL_ENABLE);
				}
				else
				{
					memcpy(&ulValue,
						   gstaConfigTable[nItemIndex].pConfig + gstaConfigTable[nItemIndex].offset,
						   sizeof(unsigned int)
						   );
				}

				nDescriptIndex = 0;
				while (gstaConfigTable[nItemIndex].staDescript[nDescriptIndex].pszDescript != 0)
				{
					if (gstaConfigTable[nItemIndex].staDescript[nDescriptIndex].nValue == ulValue)
					{
						printf("%s\n", gstaConfigTable[nItemIndex].staDescript[nDescriptIndex].pszDescript);
						break;
                    }
					nDescriptIndex++;
				}
				if (gstaConfigTable[nItemIndex].staDescript[nDescriptIndex].pszDescript == 0)
				{
					printf("%u\n", ulValue);
				}
			}
			else
			{
				printf("failed in ioctl()\n");
			}
		}
		nItemIndex++;
	}

    //printf("\n");

	return (lErr);
}



int do_set(DEVHANDLE hDev,
			char *pszItem,
			char *pszValue
			)
{
	int			lErr = 1;
	unsigned int	ulValue;
	int				nItemIndex, nDescriptIndex;
	int				nStuffing;

	//printf("\n");

	nItemIndex = 0;
	while (gstaConfigTable[nItemIndex].pszItemName != 0)
	{
		if (!strcasecmp(pszItem, gstaConfigTable[nItemIndex].pszItemName))
		{
			printf("    %s", gstaConfigTable[nItemIndex].pszItemName);
			nStuffing = 36 - (int)strlen(gstaConfigTable[nItemIndex].pszItemName);
			while (nStuffing--)
			{
				printf(" ");
			}
			printf("=    ");


			if (gstaConfigTable[nItemIndex].staDescript[0].pszDescript == 0)
			{
                ulValue = atoi(pszValue);
				if (!strcasecmp(pszItem, "SystemControl-Version"))
				{
					sysControl.Artesa.version = ulValue;
				}
				else if (!strcasecmp(pszItem, "SystemControl-XferMode"))
				{
					sysControl.Artesa.xfer_mode = ulValue;
				}
				else if (!strcasecmp(pszItem, "InputControl-InFrameRate"))
				{
					inputControl.Artesa.in_frame_rate = (ulValue | IN_FRAME_RATE_ABSOLUTE);
				}
				else if (!strcasecmp(pszItem, "DecStreamId-Type"))
				{
					decStreamID.streamIdEx.stream_type = ulValue;
				}
				else if (!strcasecmp(pszItem, "DecStreamId-Data"))
				{
					decStreamID.streamIdEx.stream_data = ulValue;
				}
				else if (!strcasecmp(pszItem, "DecStreamId-XferMode"))
				{
					decStreamID.streamIdEx.xfer_mode = ulValue;
				}
				else if (!strcasecmp(pszItem, "DecStreamId-ByteSwap"))
				{
					decStreamID.streamIdEx.bytes_swap = ulValue;
				}
				else if (!strcasecmp(pszItem, "DecDispAttrib-InputSizeX"))
				{
					decDispAttrib.inputSize.Attrib.x = ulValue;
				}
				else if (!strcasecmp(pszItem, "DecDispAttrib-InputSizeY"))
				{
					decDispAttrib.inputSize.Attrib.y = ulValue;
				}
				else if (!strcasecmp(pszItem, "DecDispAttrib-OutputSizeX"))
				{
					decDispAttrib.outputSize.Attrib.x = ulValue;
				}
				else if (!strcasecmp(pszItem, "DecDispAttrib-OutputSizeY"))
				{
					decDispAttrib.outputSize.Attrib.y = ulValue;
				}
				else if (!strcasecmp(pszItem, "DecDispAttrib-InputOriginX"))
				{
					decDispAttrib.inputOrigin.Attrib.x = ulValue;
				}
				else if (!strcasecmp(pszItem, "DecDispAttrib-InputOriginY"))
				{
					decDispAttrib.inputOrigin.Attrib.y = ulValue;
				}
				else if (!strcasecmp(pszItem, "DecDispAttrib-OutputOriginX"))
				{
					decDispAttrib.outputOrigin.Attrib.x = ulValue;
				}
				else if (!strcasecmp(pszItem, "DecDispAttrib-OutputOriginY"))
				{
					decDispAttrib.outputOrigin.Attrib.y = ulValue;
				}
				else if (!strcasecmp(pszItem, "SysFunction-Function"))
				{
					function.functionEx.function = ulValue;
				}
				else if (!strcasecmp(pszItem, "SysFunction-Video"))
				{
					function.functionEx.video = ulValue;
				}
				else if (!strcasecmp(pszItem, "SysFunction-Audio"))
				{
					function.functionEx.audio = ulValue;
				}
				else if (!strcasecmp(pszItem, "MiscRateControl-QpUpdateInterval"))
				{
					rateControlEx.Artesa.base_qp_length = ulValue;
				}
				else if (!strcasecmp(pszItem, "MiscRateControl-LocalActivity"))
				{
					rateControlEx.Artesa.local_activity_on = ulValue;
				}
				else if (!strcasecmp(pszItem, "MiscRateControl-MDInterval"))
				{
					rateControlEx.Artesa.motion_detect_interval = ulValue;
				}
				else if (!strcasecmp(pszItem, "LargeCompressBuf-Enable"))
				{
					if (ulValue==1)
						largecompressbuffercontrol = largecompressbuffercontrol | QPCODEC_LARGE_COMPRESS_BUFFER_CONTROL_ENABLE;
					else
						largecompressbuffercontrol = largecompressbuffercontrol & (~QPCODEC_LARGE_COMPRESS_BUFFER_CONTROL_ENABLE);
				}
				else if (!strcasecmp(pszItem, "LargeCompressBuf-MaxBitrate"))
				{
					if (largecompressbuffercontrol & QPCODEC_LARGE_COMPRESS_BUFFER_CONTROL_ENABLE)
						largecompressbuffercontrol = QPCODEC_LARGE_COMPRESS_BUFFER_CONTROL_ENABLE | (ulValue & (~QPCODEC_LARGE_COMPRESS_BUFFER_CONTROL_ENABLE));
					else
						largecompressbuffercontrol = (ulValue & (~QPCODEC_LARGE_COMPRESS_BUFFER_CONTROL_ENABLE));
				}
				else
				{
					memcpy(gstaConfigTable[nItemIndex].pConfig + gstaConfigTable[nItemIndex].offset,
						   &ulValue,
						   sizeof(unsigned int)
						   );
				}

                lErr = ioctl(hDev, gstaConfigTable[nItemIndex].nControlIdSet, gstaConfigTable[nItemIndex].pConfig);
				if (!lErr)
				{
					printf("%u\n", ulValue);
				}
				else
				{
					printf("failed in ioctl()\n");
				}
			}
			else
			{
				nDescriptIndex = 0;
				while (gstaConfigTable[nItemIndex].staDescript[nDescriptIndex].pszDescript != 0)
				{
					if (!strcasecmp(gstaConfigTable[nItemIndex].staDescript[nDescriptIndex].pszDescript, pszValue))
					{
						break;
					}
					nDescriptIndex++;
				}

				if (gstaConfigTable[nItemIndex].staDescript[nDescriptIndex].pszDescript != 0)
				{
					ulValue = gstaConfigTable[nItemIndex].staDescript[nDescriptIndex].nValue;
					if (!strcasecmp(pszItem, "SystemControl-Version"))
					{
						sysControl.Artesa.version = ulValue;
					}
					else if (!strcasecmp(pszItem, "SystemControl-XferMode"))
					{
						sysControl.Artesa.xfer_mode = ulValue;
					}
					else if (!strcasecmp(pszItem, "DecStreamId-Type"))
					{
						decStreamID.streamIdEx.stream_type = ulValue;
					}
					else if (!strcasecmp(pszItem, "DecStreamId-Data"))
					{
						decStreamID.streamIdEx.stream_data = ulValue;
					}
					else if (!strcasecmp(pszItem, "DecStreamId-XferMode"))
					{
						decStreamID.streamIdEx.xfer_mode = ulValue;
					}
					else if (!strcasecmp(pszItem, "DecStreamId-ByteSwap"))
					{
						decStreamID.streamIdEx.bytes_swap = ulValue;
					}
					else if (!strcasecmp(pszItem, "DecDispAttrib-InputSizeX"))
					{
						decDispAttrib.inputSize.Attrib.x = ulValue;
					}
					else if (!strcasecmp(pszItem, "DecDispAttrib-InputSizeY"))
					{
						decDispAttrib.inputSize.Attrib.y = ulValue;
					}
					else if (!strcasecmp(pszItem, "DecDispAttrib-OutputSizeX"))
					{
						decDispAttrib.outputSize.Attrib.x = ulValue;
					}
					else if (!strcasecmp(pszItem, "DecDispAttrib-OutputSizeY"))
					{
						decDispAttrib.outputSize.Attrib.y = ulValue;
					}
					else if (!strcasecmp(pszItem, "DecDispAttrib-InputOriginX"))
					{
						decDispAttrib.inputOrigin.Attrib.x = ulValue;
					}
					else if (!strcasecmp(pszItem, "DecDispAttrib-InputOriginY"))
					{
						decDispAttrib.inputOrigin.Attrib.y = ulValue;
					}
					else if (!strcasecmp(pszItem, "DecDispAttrib-OutputOriginX"))
					{
						decDispAttrib.outputOrigin.Attrib.x = ulValue;
					}
					else if (!strcasecmp(pszItem, "DecDispAttrib-OutputOriginY"))
					{
						decDispAttrib.outputOrigin.Attrib.y = ulValue;
					}
					else if (!strcasecmp(pszItem, "SysFunction-Function"))
					{
						function.functionEx.function = ulValue;
					}
					else if (!strcasecmp(pszItem, "SysFunction-Video"))
					{
						function.functionEx.video = ulValue;
					}
					else if (!strcasecmp(pszItem, "SysFunction-Audio"))
					{
						function.functionEx.audio = ulValue;
					}
					else if (!strcasecmp(pszItem, "MiscRateControl-QpUpdateInterval"))
					{
						rateControlEx.Artesa.base_qp_length = ulValue;
					}
					else if (!strcasecmp(pszItem, "MiscRateControl-LocalActivity"))
					{
						rateControlEx.Artesa.local_activity_on = ulValue;
					}
					else if (!strcasecmp(pszItem, "MiscRateControl-MDInterval"))
					{
						rateControlEx.Artesa.motion_detect_interval = ulValue;
					}
					else if (!strcasecmp(pszItem, "LargeCompressBuf-Enable"))
					{
						if (ulValue==1)
							largecompressbuffercontrol = largecompressbuffercontrol | QPCODEC_LARGE_COMPRESS_BUFFER_CONTROL_ENABLE;
						else
							largecompressbuffercontrol = largecompressbuffercontrol & (~QPCODEC_LARGE_COMPRESS_BUFFER_CONTROL_ENABLE);
					}
					else if (!strcasecmp(pszItem, "LargeCompressBuf-MaxBitrate"))
					{
						if (largecompressbuffercontrol & QPCODEC_LARGE_COMPRESS_BUFFER_CONTROL_ENABLE)
							largecompressbuffercontrol = QPCODEC_LARGE_COMPRESS_BUFFER_CONTROL_ENABLE | (ulValue & (~QPCODEC_LARGE_COMPRESS_BUFFER_CONTROL_ENABLE));
						else
							largecompressbuffercontrol = (ulValue & (~QPCODEC_LARGE_COMPRESS_BUFFER_CONTROL_ENABLE));
					}
					else
					{
						memcpy(gstaConfigTable[nItemIndex].pConfig + gstaConfigTable[nItemIndex].offset,
							   &ulValue,
							   sizeof(unsigned int)
							   );
					}

					lErr = ioctl(hDev, gstaConfigTable[nItemIndex].nControlIdSet, gstaConfigTable[nItemIndex].pConfig);
					if (!lErr)
					{
						printf("%s\n", gstaConfigTable[nItemIndex].staDescript[nDescriptIndex].pszDescript);
					}
					else
					{
						printf("failed in ioctl()\n");
                    }
				}
				else
				{
					printf("failed, item not found\n");
				}
			}
        }
        nItemIndex++;
    }
    //printf("\n");

	return (lErr);
}


char *skip_whitespace(char *Str)
{
    //printf(" SKIPWHITE Enter. Str=%s\n", Str);
    while (*Str == ' ' || *Str == '\t' || *Str == '\n' || *Str == '#' || *Str == '/') {
        if (*Str == '\n') {
            /* End of line */
            return 0;
        }
        if (*Str && ((*Str == '#') || ((*Str == '/') && (*(Str + 1) == '/')))) {
            /* found comment */
            return 0;
        }
        Str++;
    }
    //printf(" SKIPWHITE Exit.  Str=%s\n", Str);
    return (Str);
}


char *skip_to_whitespace(char *Str)
{
    while (*Str && *Str != ' ' && *Str != '\t' && *Str != '\n' && *Str != '\r') {
        if (*Str && ((*Str == '#') || ((*Str == '/') && (*(Str + 1) == '/')))) {
            /* found comment */
            return 0;
        }
        Str++;
    }
    return (Str);
}


int do_set_script(DEVHANDLE hDev,
				   char *scrfilename
				   )
{
	int		lErr = 0;
	FILE	*fscript = fopen(scrfilename, "r");
	ssize_t  currbytesread = 0;
	size_t currlinesize = 0;
    //char	tempbuff[256];
	char	*currlinebuff = NULL;
	char	*argv[4];

    if (fscript > 0)
	{
		printf("****** Executing script file %s \n", scrfilename);
		rewind(fscript);
		currlinebuff = malloc(256);
		while (1)
		{
			currbytesread = getline(&currlinebuff, &currlinesize, fscript);
			if (currbytesread > 0)
			{
				argv[0] = skip_whitespace(currlinebuff);
				if (!argv[0])
					continue;
				argv[1] = skip_to_whitespace(argv[0]);
				if (!argv[1])
					continue;
				*argv[1]++ = 0;

				argv[1] = skip_whitespace(argv[1]);
				if (!argv[1])
					continue;
				argv[2] = skip_to_whitespace(argv[1]);
				if (!argv[2])
					continue;
				*argv[2]++ = 0;

				lErr = do_set(hDev, argv[0], argv[1]);
			}
			else
			{
				break;
            }
        }
		if (currlinebuff)
			free(currlinebuff);
        fclose(fscript);
    }
	else
	{
		lErr = 1;
		printf("Unable to open file %s ", scrfilename);
	}
	return (lErr);
}



int qlconf(component_t *pComponent,
		   TQP_OBJECT_HANDLE hTaskUpStream,
		   char *scrfilename
		   )
{
    int	lErr;

	// get the current settings from the driver
	lErr = ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_SYSTEM_CONTROL, &sysControl);
	lErr = (0 != lErr) ? lErr : ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_PICTURE_RESOLUTION, &picResolution);
	lErr = (0 != lErr) ? lErr : ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_INPUT_CONTROL, &inputControl);
	lErr = (0 != lErr) ? lErr : ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_RATE_CONTROL, &rateControl);
	lErr = (0 != lErr) ? lErr : ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_VBR_BIT_RATE, &vbrControl);
	lErr = (0 != lErr) ? lErr : ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_FILTER_CONTROL, &fltrControl);
	lErr = (0 != lErr) ? lErr : ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_GOP_LOOP_FILTER, &gopLoopFltr);
	lErr = (0 != lErr) ? lErr : ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_OUT_PICTURE_RESOLUTION, &outResolution);
	lErr = (0 != lErr) ? lErr : ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM, &audControl);
	lErr = (0 != lErr) ? lErr : ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX, &audControlEx);
	lErr = (0 != lErr) ? lErr : ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_ENC_MJPEG_QUALITY, &mjpegQuality);
	lErr = (0 != lErr) ? lErr : ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_ENC_MJPEG_FRAME_BUF, &mjpegBufNo);

	lErr = (0 != lErr) ? lErr : ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_DEC_STREAM_ID, &decStreamID);
	lErr = (0 != lErr) ? lErr : ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_DEC_OUTPUT_MODE, &decOutputMode);
	lErr = (0 != lErr) ? lErr : ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_DEC_RESOLUTION, &decResolution);
	lErr = (0 != lErr) ? lErr : ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_DEC_STOP_MODE, &decStopMode);
	lErr = (0 != lErr) ? lErr : ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_DEC_DISP_ATTRIB, &decDispAttrib);
	lErr = (0 != lErr) ? lErr : ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_DEC_DISP_MODE, &decDispMode);
	lErr = (0 != lErr) ? lErr : ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_DEC_AUDIO_CONTROL, &decAudControl);

	lErr = (0 != lErr) ? lErr : ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_AOU_CONFIG, &aouConfig);
	//lErr = (0 != lErr) ? lErr : ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_OSD_MEMORY, &osdMem);
	lErr = (0 != lErr) ? lErr : ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_DECIMATION, &decimation);
	lErr = (0 != lErr) ? lErr : ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_DEINTERLACE, &deinterlacemode);
	lErr = (0 != lErr) ? lErr : ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_ENC_CAP_MODE, &enccapmode);
	lErr = (0 != lErr) ? lErr : ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_EXTERNAL_TRIGGER, &externaltriggermode);
	lErr = (0 != lErr) ? lErr : ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_PTS_COUNTER_RESET, &ptscounterreset);
	lErr = (0 != lErr) ? lErr : ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_RATE_CONTROL_EX, &rateControlEx);
	ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_LARGE_COMPRESS_BUFFER_CONTROL, &largecompressbuffercontrol); // not check error to work with old version
	ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_ENABLE_LOW_BITRATE_MODE, &lowbitratemode); // not check error to work with old version
	ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_ENABLE_AV_DISCARD_CONTROL, &avdiscardcontrolmode); // not check error to work with old version
	ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_USE_SW_PTS, &useswptsmode); // not check error to work with old version
	lErr = (0 != lErr) ? lErr : ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_SYS_FUNCTION, &function);
	lErr = (0 != lErr) ? lErr : ioctl(pComponent->hDev, PLDEV_CNTL_IOCTL_GET_SYS_LINK, &sys_link);

	sys_link.video_in_ch = (TQP_UINT32)((hTaskUpStream == TQP_NULL_HANDLE) ? 0 : hTaskUpStream);
	sys_link.audio_in_ch = (TQP_UINT32)((hTaskUpStream == TQP_NULL_HANDLE) ? 0 : hTaskUpStream);

	if (lErr)
	{
		printf("%s() ioctl failed lErr(%u) ", __FUNCTION__, lErr);
	}
	else
	{
		// set the script
		lErr = do_set_script(pComponent->hDev, scrfilename);
	}

	// copy to the caller
	memcpy(&pComponent->sysControl,		&sysControl,	sizeof(QPCODEC_SYSTEM_CONTROL));
	memcpy(&pComponent->picResolution,	&picResolution,	sizeof(QPCODEC_PICTURE_RESOLUTION));
	memcpy(&pComponent->inputControl,	&inputControl,	sizeof(QPCODEC_INPUT_CONTROL));
	memcpy(&pComponent->rateControl,	&rateControl,	sizeof(QPCODEC_RATE_CONTROL));
	memcpy(&pComponent->vbrControl,		&vbrControl,	sizeof(QPCODEC_VBR_BIT_RATE));
	memcpy(&pComponent->fltrControl,	&fltrControl,	sizeof(QPCODEC_FILTER_CONTROL));
	memcpy(&pComponent->gopLoopFltr,	&gopLoopFltr,	sizeof(QPCODEC_GOP_LOOP_FILTER));
	memcpy(&pComponent->outResolution,	&outResolution,	sizeof(QPCODEC_OUT_PICTURE_RESOLUTION));
	memcpy(&pComponent->audControl,		&audControl,	sizeof(QPCODEC_AUDIO_CONTROL_PARAM));
	memcpy(&pComponent->audControlEx,	&audControlEx,	sizeof(QPCODEC_AUDIO_CONTROL_EX));
	memcpy(&pComponent->mjpegQuality,	&mjpegQuality,	sizeof(TQP_UINT32));
	memcpy(&pComponent->mjpegBufNo,		&mjpegBufNo,	sizeof(TQP_UINT32));

	memcpy(&pComponent->decStreamID,	&decStreamID,	sizeof(QPCODEC_DEC_STREAM_ID_EX));
	memcpy(&pComponent->decOutputMode,	&decOutputMode,	sizeof(TQP_UINT32));
	memcpy(&pComponent->decResolution,	&decResolution,	sizeof(QPCODEC_PICTURE_RESOLUTION));
	memcpy(&pComponent->decStopMode,	&decStopMode,	sizeof(QPCODEC_DEC_STOP_MODE));
	memcpy(&pComponent->decDispAttrib,	&decDispAttrib,	sizeof(QPCODEC_DEC_DISP_ATTRIB));
	memcpy(&pComponent->decDispMode,	&decDispMode,	sizeof(QPCODEC_DEC_DISP_MODE));
	memcpy(&pComponent->decAudControl,	&decAudControl,	sizeof(QPCODEC_DEC_AUDIO_CONTROL));

	memcpy(&pComponent->aouConfig,		&aouConfig,		sizeof(QPCODEC_AOU_CONFIG));
	memcpy(&pComponent->osdMem,			&osdMem,		sizeof(QPCODEC_OSD_MEM_CONFIG));
	memcpy(&pComponent->decimation,		&decimation,	sizeof(QPCODEC_DECIMATION_FACTOR));
	memcpy(&pComponent->deinterlacemode,&deinterlacemode,	sizeof(TQP_UINT32));
	memcpy(&pComponent->enccapmode,		&enccapmode,	sizeof(QPCODEC_ENC_CAP_MODE));
	memcpy(&pComponent->externaltriggermode,		&externaltriggermode,	sizeof(QPCODEC_EXTERNAL_TRIGGER_MODE));
	memcpy(&pComponent->ptscounterreset, &ptscounterreset,	sizeof(QPCODEC_PTS_COUNTER_RESET));
	memcpy(&pComponent->rateControlEx,  &rateControlEx,	sizeof(RATE_CONTROL_EX));
	memcpy(&pComponent->largecompressbuffercontrol,  &largecompressbuffercontrol,	sizeof(TQP_UINT32));
	memcpy(&pComponent->lowbitratemode,  &lowbitratemode,	sizeof(TQP_UINT32));
	memcpy(&pComponent->avdiscardcontrolmode,  &avdiscardcontrolmode,	sizeof(TQP_UINT32));
	memcpy(&pComponent->useswptsmode,  &useswptsmode,	sizeof(TQP_UINT32));

	memcpy(&pComponent->function,		&function,		sizeof(QPCODEC_SYS_FUNCTION));
	memcpy(&pComponent->link,			&sys_link,		sizeof(QPCODEC_SYS_LINK));

	// setup port type and open format
	pComponent->ports[STREAM_DIR_IN].portType = MAX_COMPONENT_PORT;
	pComponent->ports[STREAM_DIR_OUT].portType = MAX_COMPONENT_PORT;

	if (CODEC_OPEN_EX_ENCODE == pComponent->function.functionEx.function)
	{
		if (CODEC_OPEN_EX_VID_NONE != pComponent->function.functionEx.video)
		{
			pComponent->ulWidth = pComponent->outResolution.out_pic_width;
			pComponent->ulHeight = pComponent->outResolution.out_pic_height;

			// input
			if (SYS_LINK_VID_IN_HOST == pComponent->link.video_input)
			{
				pComponent->ports[STREAM_DIR_IN].portType = COMPONENT_PORT_YUV_IN;

				pComponent->ports[STREAM_DIR_IN].openFormat.yuv.nWidth = pComponent->picResolution.Artesa.in_pic_width;
				pComponent->ports[STREAM_DIR_IN].openFormat.yuv.nHeight = pComponent->picResolution.Artesa.in_pic_height;
				pComponent->ports[STREAM_DIR_IN].openFormat.yuv.nBitCount = 12;
				pComponent->ports[STREAM_DIR_IN].openFormat.yuv.nDataType = QP_YUV_DATA_TYPE_YV12;
				pComponent->ports[STREAM_DIR_IN].openFormat.yuv.nFrameRate = 30;
			}
			else if (SYS_LINK_VID_IN_INTERNAL == pComponent->link.video_input)
			{
				pComponent->ports[STREAM_DIR_IN].portType = COMPONENT_PORT_VIRTUAL_IN;
				pComponent->link.video_in_ch = (TQP_UINT32) hTaskUpStream;
			}

			// output
			if (SYS_LINK_VID_OUT_HOST == pComponent->link.video_output)
			{
				if (CODEC_OPEN_EX_VID_PASSTHROUGH == pComponent->function.functionEx.video)
				{
					pComponent->ports[STREAM_DIR_OUT].portType = COMPONENT_PORT_YUV_OUT;

					pComponent->ports[STREAM_DIR_OUT].openFormat.yuv.nWidth = pComponent->outResolution.out_pic_width;
					pComponent->ports[STREAM_DIR_OUT].openFormat.yuv.nHeight = pComponent->outResolution.out_pic_height;
					pComponent->ports[STREAM_DIR_OUT].openFormat.yuv.nBitCount = 12;
					pComponent->ports[STREAM_DIR_OUT].openFormat.yuv.nDataType = (pComponent->busType == QPHCI_BUS_PCI) ? QP_YUV_DATA_TYPE_YV12 : QP_YUV_DATA_TYPE_QYUV;
					pComponent->ports[STREAM_DIR_OUT].openFormat.yuv.nFrameRate = 30;
				}
				else if (CODEC_OPEN_EX_VID_DECIMATION == pComponent->function.functionEx.video)
				{
					pComponent->ports[STREAM_DIR_OUT].portType = COMPONENT_PORT_YUV_OUT;

					if ((QPCODEC_DECIMATION_OUTPUT_YUY2 == pComponent->decimation.output_format) ||
						(QPCODEC_DECIMATION_OUTPUT_QYUV == pComponent->decimation.output_format))
					{
						pComponent->ulWidth = (pComponent->ulWidth/pComponent->decimation.scale_factor.factor.h_denominator)*pComponent->decimation.scale_factor.factor.h_numerator;
						pComponent->ulHeight = (pComponent->ulHeight/pComponent->decimation.scale_factor.factor.v_denominator)*pComponent->decimation.scale_factor.factor.v_numerator;
					}

					if (QPCODEC_DECIMATION_OUTPUT_QYUV == pComponent->decimation.output_format)
					{
						pComponent->ulWidth = (pComponent->ulWidth/16)*16;
						pComponent->ulHeight = (pComponent->ulHeight/16)*16;
					}
					else if (QPCODEC_DECIMATION_OUTPUT_YUY2 == pComponent->decimation.output_format)
					{
						pComponent->ulWidth = (pComponent->ulWidth/2)*2;
						pComponent->ulHeight = (pComponent->ulHeight/2)*2;
					}

					pComponent->ports[STREAM_DIR_OUT].openFormat.yuv.nWidth = pComponent->ulWidth;
					pComponent->ports[STREAM_DIR_OUT].openFormat.yuv.nHeight = pComponent->ulHeight;
					if (QPCODEC_DECIMATION_OUTPUT_QYUV==pComponent->decimation.output_format)
					{
						pComponent->ports[STREAM_DIR_OUT].openFormat.yuv.nBitCount = 12;
						pComponent->ports[STREAM_DIR_OUT].openFormat.yuv.nDataType = QP_YUV_DATA_TYPE_QYUV;
					}
					else
					{   /* QPCODEC_DECIMATION_OUTPUT_YUY2 */
						pComponent->ports[STREAM_DIR_OUT].openFormat.yuv.nBitCount = 16;
						pComponent->ports[STREAM_DIR_OUT].openFormat.yuv.nDataType = QP_YUV_DATA_TYPE_YUYV;
					}
					pComponent->ports[STREAM_DIR_OUT].openFormat.yuv.nFrameRate = 30;
				}
				else if (CODEC_OPEN_EX_VID_MOTIONDETECT == pComponent->function.functionEx.video)
				{
					pComponent->ports[STREAM_DIR_OUT].portType = COMPONENT_PORT_ACTIVITY_OUT;
					pComponent->ports[STREAM_DIR_OUT].openFormat.activity.nWidth = pComponent->ulWidth;
					pComponent->ports[STREAM_DIR_OUT].openFormat.activity.nHeight = pComponent->ulHeight;
					pComponent->ports[STREAM_DIR_OUT].openFormat.activity.nBitCount = 16;
				}
				else
				{
					pComponent->ports[STREAM_DIR_OUT].portType = COMPONENT_PORT_COMP_OUT;
					pComponent->ports[STREAM_DIR_OUT].openFormat.vid.nDataType = (CODEC_OPEN_EX_VID_JPEG == pComponent->function.functionEx.video) ? QP_COMP_VID_DATA_TYPE_MJPEG : QP_COMP_VID_DATA_TYPE_H264;
					if ((CODEC_OPEN_EX_VID_JPEG == pComponent->function.functionEx.video) &&
						(1 == pComponent->enccapmode.capMode)
						)
					{
						pComponent->ports[STREAM_DIR_OUT].unJPEGOnDemand = 1;
					}
				}
			}
			else if (SYS_LINK_VID_OUT_INTERNAL == pComponent->link.video_output)
			{
				pComponent->ports[STREAM_DIR_OUT].portType = COMPONENT_PORT_VIRTUAL_OUT;
			}
		}
		else
		{
			if (CODEC_OPEN_EX_AUD_NONE != pComponent->function.functionEx.audio)
			{
				// input
				if (SYS_LINK_AUD_IN_INTERNAL == pComponent->link.audio_input)
				{
					pComponent->ports[STREAM_DIR_IN].portType = COMPONENT_PORT_VIRTUAL_IN;
					pComponent->link.audio_in_ch = (TQP_UINT32) hTaskUpStream;
				}
				else if (SYS_LINK_AUD_IN_HOST == pComponent->link.audio_input)
				{
					if (pComponent->audControl.disAi == 1)	//not from AIU
						pComponent->ports[STREAM_DIR_IN].portType = COMPONENT_PORT_PCM_IN;
				}

				// output
				if (SYS_LINK_AUD_OUT_HOST == pComponent->link.audio_output)
				{
					if (CODEC_OPEN_EX_AUD_PASSTHROUGH == pComponent->function.functionEx.audio)
					{
						pComponent->ports[STREAM_DIR_OUT].portType = COMPONENT_PORT_PCM_OUT;

						switch (pComponent->audControl.sample_rate)
						{
							case SAMPLE_RATE_44_1K:
								pComponent->ports[STREAM_DIR_OUT].openFormat.pcm.nSamplesPerSec = 44100;
								break;
							case SAMPLE_RATE_32K:
								pComponent->ports[STREAM_DIR_OUT].openFormat.pcm.nSamplesPerSec = 32000;
								break;
							case SAMPLE_RATE_22_5K:
								pComponent->ports[STREAM_DIR_OUT].openFormat.pcm.nSamplesPerSec = 22050;
								break;
							case SAMPLE_RATE_24K:
								pComponent->ports[STREAM_DIR_OUT].openFormat.pcm.nSamplesPerSec = 24000;
								break;
							case SAMPLE_RATE_16K:
								pComponent->ports[STREAM_DIR_OUT].openFormat.pcm.nSamplesPerSec = 16000;
								break;
							case SAMPLE_RATE_8K:
								pComponent->ports[STREAM_DIR_OUT].openFormat.pcm.nSamplesPerSec = 8000;
								break;
							case SAMPLE_RATE_48K:
							default:
								pComponent->ports[STREAM_DIR_OUT].openFormat.pcm.nSamplesPerSec = 48000;
								break;
						}

						pComponent->ports[STREAM_DIR_OUT].openFormat.pcm.nBitsPerSample = 16;
						pComponent->ports[STREAM_DIR_OUT].openFormat.pcm.nChannels = 2;
						pComponent->ports[STREAM_DIR_OUT].openFormat.pcm.nBlockAlign = 4;
						pComponent->ports[STREAM_DIR_OUT].openFormat.pcm.nAvgBytesPerSec = pComponent->ports[STREAM_DIR_OUT].openFormat.pcm.nSamplesPerSec * 4;
					}
					else
					{
						pComponent->ports[STREAM_DIR_OUT].portType = COMPONENT_PORT_COMP_AUD_OUT;
						//pComponent->ports[STREAM_DIR_OUT].openFormat.vid.nDataType = QP_COMP_VID_DATA_TYPE_H264;
					}
				}
			}
		}
	}
	else
	{
		if (CODEC_OPEN_EX_VID_NONE != pComponent->function.functionEx.video)
		{
			pComponent->ulWidth = pComponent->decDispAttrib.outputSize.Attrib.x;
			pComponent->ulHeight = pComponent->decDispAttrib.outputSize.Attrib.y;

			// input
			if (SYS_LINK_VID_IN_HOST == pComponent->link.video_input)
			{
				pComponent->ports[STREAM_DIR_IN].portType = COMPONENT_PORT_COMP_IN;
				pComponent->ports[STREAM_DIR_IN].openFormat.vid.nDataType = (CODEC_OPEN_EX_VID_JPEG == pComponent->function.functionEx.video) ? QP_COMP_VID_DATA_TYPE_MJPEG : QP_COMP_VID_DATA_TYPE_H264;
			}
			else if (SYS_LINK_VID_IN_INTERNAL == pComponent->link.video_input)
			{
				pComponent->ports[STREAM_DIR_IN].portType = COMPONENT_PORT_VIRTUAL_IN;
				pComponent->link.video_in_ch = (TQP_UINT32) hTaskUpStream;
			}

			// output
			if (SYS_LINK_VID_OUT_HOST == pComponent->link.video_output)
			{
				pComponent->ports[STREAM_DIR_OUT].portType = COMPONENT_PORT_YUV_OUT;

				pComponent->ports[STREAM_DIR_OUT].openFormat.yuv.nWidth = pComponent->decDispAttrib.outputSize.Attrib.x;
				pComponent->ports[STREAM_DIR_OUT].openFormat.yuv.nHeight = pComponent->decDispAttrib.outputSize.Attrib.y;
				pComponent->ports[STREAM_DIR_OUT].openFormat.yuv.nBitCount = 12;
				pComponent->ports[STREAM_DIR_OUT].openFormat.yuv.nDataType = (pComponent->busType == QPHCI_BUS_PCI) ? QP_YUV_DATA_TYPE_YV12 : QP_YUV_DATA_TYPE_QYUV;
				pComponent->ports[STREAM_DIR_OUT].openFormat.yuv.nFrameRate = 30;
			}
			else if (SYS_LINK_VID_OUT_INTERNAL == pComponent->link.video_output)
			{
				pComponent->ports[STREAM_DIR_OUT].portType = COMPONENT_PORT_VIRTUAL_OUT;
			}
			else if (SYS_LINK_VID_OUT_NULL == pComponent->link.video_output)
			{
				pComponent->ports[STREAM_DIR_OUT].portType = COMPONENT_PORT_VIRTUAL_OUT;
			}
		}
		else
		{
			if (CODEC_OPEN_EX_AUD_NONE != pComponent->function.functionEx.audio)
			{
				// input
				if (SYS_LINK_AUD_IN_HOST == pComponent->link.audio_input)
				{
					if (pComponent->decAudControl.audio_type == 0)	//LPCM
						pComponent->ports[STREAM_DIR_IN].portType = COMPONENT_PORT_PCM_IN;
					else
						pComponent->ports[STREAM_DIR_IN].portType = COMPONENT_PORT_COMP_AUD_IN;
				}
				else if (SYS_LINK_AUD_IN_INTERNAL == pComponent->link.audio_input)
				{
					pComponent->ports[STREAM_DIR_IN].portType = COMPONENT_PORT_VIRTUAL_IN;
					pComponent->link.audio_in_ch = (TQP_UINT32) hTaskUpStream;
				}

				// output
				if (SYS_LINK_AUD_OUT_HOST == pComponent->link.audio_output)
				{
					pComponent->ports[STREAM_DIR_OUT].portType = COMPONENT_PORT_PCM_OUT;

					switch (pComponent->decAudControl.sample_rate)
					{
						case SAMPLE_RATE_44_1K:
							pComponent->ports[STREAM_DIR_OUT].openFormat.pcm.nSamplesPerSec = 44100;
							break;
						case SAMPLE_RATE_32K:
							pComponent->ports[STREAM_DIR_OUT].openFormat.pcm.nSamplesPerSec = 32000;
							break;
						case SAMPLE_RATE_22_5K:
							pComponent->ports[STREAM_DIR_OUT].openFormat.pcm.nSamplesPerSec = 22050;
							break;
						case SAMPLE_RATE_24K:
							pComponent->ports[STREAM_DIR_OUT].openFormat.pcm.nSamplesPerSec = 24000;
							break;
						case SAMPLE_RATE_16K:
							pComponent->ports[STREAM_DIR_OUT].openFormat.pcm.nSamplesPerSec = 16000;
							break;
						case SAMPLE_RATE_8K:
							pComponent->ports[STREAM_DIR_OUT].openFormat.pcm.nSamplesPerSec = 8000;
							break;
						case SAMPLE_RATE_48K:
						default:
							pComponent->ports[STREAM_DIR_OUT].openFormat.pcm.nSamplesPerSec = 48000;
							break;
					}

					pComponent->ports[STREAM_DIR_OUT].openFormat.pcm.nBitsPerSample = 16;
					pComponent->ports[STREAM_DIR_OUT].openFormat.pcm.nChannels = 2;
					pComponent->ports[STREAM_DIR_OUT].openFormat.pcm.nBlockAlign = 4;
					pComponent->ports[STREAM_DIR_OUT].openFormat.pcm.nAvgBytesPerSec = pComponent->ports[STREAM_DIR_OUT].openFormat.pcm.nSamplesPerSec * 4;
				}
			}
		}
	}

	return (lErr);
}

int qlconfDump(component_t *pComponent)
{
	component_t *p;
	p = pComponent;
	printf("========DUMP COMPONENT hDev=%lld=======\n", (long long) p->hDev);
	printf("busType: %d\n",p->busType);
	printf("portType[]: %d %d\n",
	  p->ports[0].portType, p->ports[1].portType);
	printf("openFormat[0].yuv %d %d %d %u %u\n",
	  p->ports[0].openFormat.yuv.nWidth,
	  p->ports[0].openFormat.yuv.nHeight,
	  p->ports[0].openFormat.yuv.nBitCount,
	  p->ports[0].openFormat.yuv.nFrameRate,
	  p->ports[0].openFormat.yuv.nDataType);
	printf("openFormat[1].yuv %d %d %d %u %u\n",
	  p->ports[1].openFormat.yuv.nWidth,
	  p->ports[1].openFormat.yuv.nHeight,
	  p->ports[1].openFormat.yuv.nBitCount,
	  p->ports[1].openFormat.yuv.nFrameRate,
	  p->ports[1].openFormat.yuv.nDataType);
	printf("======================\n");
	printf("sysControl: %u %u %u %u %u %u %u %u %d %d\n",
	  p->sysControl.Artesa.stream_type,
	  p->sysControl.Artesa.stream_data,
	  p->sysControl.Artesa.profile,
	  p->sysControl.Artesa.level,
	  p->sysControl.Artesa.ff_mode,
	  p->sysControl.Artesa.spsr_freq,
	  p->sysControl.Artesa.v_mode,
	  p->sysControl.Artesa.cabac_init_idc,
	  p->sysControl.Artesa.version,
	  p->sysControl.Artesa.xfer_mode);
	printf("picResolution: %u %u\n",
	  p->picResolution.Artesa.in_pic_width,
	  p->picResolution.Artesa.in_pic_height);
	printf("inputControl: %u %u %u %u %u %u %u %u %u %u\n",
	  p->inputControl.Artesa.scan_format,
	  p->inputControl.Artesa.src_mode,
	  p->inputControl.Artesa.sync_mode,
	  p->inputControl.Artesa.data_type,
	  p->inputControl.Artesa.in_frame_rate,
	  p->inputControl.Artesa.out_frame_rate,
	  p->inputControl.Artesa.fmt,
	  p->inputControl.Artesa.pd_en,
	  p->inputControl.Artesa.ck_edge,
	  p->inputControl.Artesa.bytes_swap);
	printf("rateControl: %u %u %u %u %u %u\n",
	  p->rateControl.Artesa.avg_bit_rate,
	  p->rateControl.Artesa.qp_update_freq,
	  p->rateControl.Artesa.mode,
	  p->rateControl.Artesa.fixed,
	  p->rateControl.Artesa.vbr,
	  p->rateControl.Artesa.intra_refresh_mode);
	printf("vbrControl: %u %u\n",
	  p->vbrControl.min_bit_rate,
	  p->vbrControl.max_bit_rate);
	printf("fltrControl: %u %u %u %u\n",
	  p->fltrControl.start_pixel,
	  p->fltrControl.fl_en,
	  p->fltrControl.filter_strength,
	  p->fltrControl.start_line);
	printf("gopLoopFltr: %u %u %u %d %d %u %u %u %u %u\n",
	  p->gopLoopFltr.Artesa.intra_period,
	  p->gopLoopFltr.Artesa.lp_filter,
	  p->gopLoopFltr.Artesa.dis_lf_idc,
	  p->gopLoopFltr.Artesa.alpha_offset,
	  p->gopLoopFltr.Artesa.beta_offset,
	  p->gopLoopFltr.Artesa.B_num,
	  p->gopLoopFltr.Artesa.ref_n,
	  p->gopLoopFltr.Artesa.t8x8,
	  p->gopLoopFltr.Artesa.ref_p,
	  p->gopLoopFltr.Artesa.IDR);
	printf("outResolution: %u %u\n",
	  p->outResolution.out_pic_width,
	  p->outResolution.out_pic_height);
	printf("deimation: %u %u %d %d %d %d\n",
	  p->decimation.input_format,
	  p->decimation.output_format,
	  p->decimation.scale_factor.factor.h_numerator,
	  p->decimation.scale_factor.factor.h_denominator,
	  p->decimation.scale_factor.factor.v_numerator,
	  p->decimation.scale_factor.factor.v_denominator
	  );
	printf("deinterlacemode: %d\n",
	  p->deinterlacemode
	  );
	printf("enccapmode: %d %d %d\n",
	  p->enccapmode.capMode,
	  p->enccapmode.trigMode,
	  p->enccapmode.gpioPin
	  );
	printf("triggermode: %d %d\n",
	  p->externaltriggermode.Enable,
	  p->externaltriggermode.gpioPin
	  );
	printf("ptscounterreset: %d %d %d\n",
	  p->ptscounterreset.Enable,
	  p->ptscounterreset.gpioPin,
	  p->ptscounterreset.immediate
	  );
	printf("mjpeg: %u %u\n",
	  p->mjpegQuality,
	  p->mjpegBufNo);
	printf("MiscRateControl: %d %d %d\n",
	  p->rateControlEx.Artesa.base_qp_length,
	  p->rateControlEx.Artesa.local_activity_on,
	  p->rateControlEx.Artesa.motion_detect_interval);
	printf("largecompressbuffercontrol: 0x%x\n",
	  p->largecompressbuffercontrol
	  );
	printf("lowbitratemode: 0x%x\n",
	  p->lowbitratemode
	  );
	printf("avdiscardcontrolmode: 0x%x\n",
	  p->avdiscardcontrolmode
	  );
	printf("useswptsmode: 0x%x\n",
	  p->useswptsmode
	  );

	printf("DEC:decStreamID: %d %d %d %d\n",
	  p->decStreamID.streamIdEx.stream_type,
	  p->decStreamID.streamIdEx.stream_data,
	  p->decStreamID.streamIdEx.xfer_mode,
	  p->decStreamID.streamIdEx.bytes_swap
	);
	printf("DEC:decResolution: %u %u\n",
	  p->decResolution.Artesa.in_pic_width,
	  p->decResolution.Artesa.in_pic_height
	);
	printf("DEC:decDispAttrib: %d %d %d %d %d %d\n",
	  p->decDispAttrib.inputSize.Attrib.x,
	  p->decDispAttrib.inputSize.Attrib.y,
	  p->decDispAttrib.outputSize.Attrib.x,
	  p->decDispAttrib.outputSize.Attrib.y,
	  p->decDispAttrib.TVSystem,
	  p->decDispAttrib.ForceRemote
	);
    printf("AUD:audControl: %u %u %u %u %u %u %u %u %u %u %u %u %u %u\n",
      p->audControl.bit_rate,
	  p->audControl.ch_sel,
	  p->audControl.sample_rate,
	  p->audControl.ai_msb,
	  p->audControl.lrclk_i,
	  p->audControl.bclk_i,
	  p->audControl.ai_i2s,
	  p->audControl.ai_rj,
	  p->audControl.ai_m,
	  p->audControl.ch_num,
	  p->audControl.pts,
	  p->audControl.disAi,
	  p->audControl.raw,
	  p->audControl.audio_type
	);
    printf("AUD:audControlEx: %u %u %u %u %u %u %u\n",
	  p->audControlEx.mode,
	  p->audControlEx.block_size,
      p->audControlEx.sample_cnt,
      p->audControlEx.frame_len,
      p->audControlEx.raw_audio_type,
      p->audControlEx.downsample,
	  p->audControlEx.lr_swap
	);
    printf("AUD:decAudControl: %u %u %u %u %u\n",
      p->decAudControl.audio_type,
	  p->decAudControl.ch_num,
	  p->decAudControl.sample_rate,
	  p->decAudControl.mode,
	  p->decAudControl.upsample
	);
    printf("AUD:aouConfig: %u\n",
      p->aouConfig.ao_en
	);
	printf("function: 0x%x %d %d %d %d\n",
	  p->function.function,
	  p->function.functionEx.video,
	  p->function.functionEx.audio,
	  p->function.functionEx.function,
	  p->function.functionEx.version);
	printf("link: %u %u %u %u %u %u %u %u\n",
	  p->link.video_input,
	  p->link.video_in_ch,
	  p->link.video_output,
	  p->link.video_out_ch,
	  p->link.audio_input,
	  p->link.audio_in_ch,
	  p->link.audio_output,
	  p->link.audio_out_ch);
	printf("========DUMP COMPONENT END==========\n");
	return 0;
}


