/*************************************************************************/
/**      **/
/**            **/
/**                **/
/**              **/
/**                                                                     **/
/**                               **/
/**                                                  **/
/*************************************************************************/

#include <stdio.h>
#include <unistd.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <time.h>
#include <mntent.h>
#include <signal.h>
#include <asm/types.h>
#include <sys/types.h>
#include <errno.h>

#include <linux/videodev2.h>

#include "qptypes.h"
#include "pldevioctl.h"

#include "qlutils.h"
#ifdef	SUPPORT_PL3XX_OSD
#include	"qlosdfunc.h"
#endif
#include "vidpropstruct.h"

#define MODULE_NAME     "pldiag"
#define MODULE_VERSION  "1.0"

enum cmd_type_e {
    CMD_NONE = 0,
    CMD_RDREG = 1,
    CMD_WRREG = 2,
    CMD_RDMEM = 3,
    CMD_WRMEM = 4,
    CMD_DMARD = 5,
    CMD_DMAWR = 6,
    CMD_RDI2C = 7,
    CMD_WRI2C = 8,
    CMD_RDI2CNB = 9,
    CMD_WRI2CNB = 10,
    CMD_SETGPIO = 11,
#ifdef	SUPPORT_VO_OSD
    CMD_OSDTEST = 12,
#endif
#ifdef	SUPPORT_VI_OSD
    CMD_VIOSD = 13,
#endif
    CMD_SETVIDPROP = 14,
    CMD_GETVIDPROP = 15,
    CMD_SETVIDFRONTEND = 16,
    CMD_GETVIDFRONTEND = 17,
    CMD_SETVIDSOURCE = 18,
    CMD_GETVIDSOURCE = 19,
    CMD_MEMTEST = 20,
    CMD_FWLOAD = 21,
    CMD_GETDRIVERVER = 22,
    CMD_GETIOCTLAPIVER = 23,
    CMD_GETDRIVERDBGLEVEL = 24,
    CMD_SETDRIVERDBGLEVEL = 25,
    CMD_DUMPMEM = 26,
    CMD_GETDRVLOG = 27,
    CMD_GETFWLOG = 28,
    CMD_GETCURFWMEMMODE = 29,
    CMD_SETCURFWMEMMODE = 30,
    CMD_CHECKCODECERROR = 40
};

static int ghDev = 0;
char szDevName[32];

void usage(int nExit)
{
    printf("\n");
    printf("  [%s version %s]\n", MODULE_NAME, MODULE_VERSION);
    printf("  -------------------------------------------------\n");
    printf("\n");

    printf("\n");
    printf("  [%s version %s]\n", MODULE_NAME, MODULE_VERSION);
    printf("  -------------------------------------------------\n");
    printf("  %s <arg1> <arg2> ...\n", MODULE_NAME);
    printf("\n");
    printf("  arg for bus selection,\n");
    printf("    bus                     => bus type. Could be 0(USB)/1(PCIe).\n");
    printf("\n");
    printf("  arg for card selection,\n");
    printf("    card                               => Card number. Could be 0/1/2/3, default is 0.\n");
    printf("\n");
    printf("  arg command,\n");
    printf("    rdreg <addr_in_hex>\n");
    printf("    wrreg <addr_in_hex> <value_in_hex>\n");
    printf("    dumpmem <addr_in_hex> <length_in_hex>\n");
    printf("    rdmem <addr_in_hex> <length_in_hex>\n");
    printf("    wrmem <addr_in_hex> <dword_cnt_in_hex> <dw1_in_hex> <dw2_in_hex> ... (20 dwords max)\n");
    printf("    dmard <addr_in_hex> <length_in_hex>\n");
    printf("    dmawr <addr_in_hex> <dword_cnt_in_hex> <dw1_in_hex> <dw2_in_hex> ... (20 dwords max)\n");
    printf("    rdi2c <devaddr_in_hex> <regaddr_in_hex>\n");
    printf("    wri2c <devaddr_in_hex> <regaddr_in_hex> <value_in_hex>\n");
    printf("    rdi2cnb <i2cbus> <devaddr_in_hex> <write_dword_cnt_in_hex> <dw1_in_hex>... (8 dwords max) <read_dword_cnt_in_hex>\n");
    printf("    wri2cnb <i2cbus> <devaddr_in_hex> <write_dword_cnt_in_hex> <dw1_in_hex>... (8 dwords max) \n");
    printf("    gpio <gpio number> <gpio value>\n");
    printf("    setvidprop <function_in_dec> <value_in_dec>\n");
    printf("    getvidprop <function_in_dec>\n");
    printf("    setvidfrontend <0:dummy|9:usbfw|10:adv7614|11:adv7441|14:it6603|15:ov5642|16:native_mi2010>\n");
    printf("    getvidfrontend\n");
    printf("    setvidsource <value_in_dec>\n");
    printf("    getvidsource\n");
    printf("    fwload\n");
    printf("    getdriverversion\n");
    printf("    getapiversion\n");
    printf("    getdrvlog\n");
    printf("    getfwlog\n");
    printf("    checkcodecerror\n");
    printf("    getdbglevel <0:v4l2|1:codec|2:usb|3:pcie|4:peripheral>\n");
    printf("    setdbglevel <0:v4l2|1:codec|2:usb|3:pcie|4:peripheral> <0:none|1:fatal|2:error|3:warn|4:trace|5:extrace>\n");
    printf("    memtest <length_in_MB> (only for pcie device)\n");
    printf("    getcurfwmemmode\n");
    printf("    setcurfwmemmode <1:dynamic, 0:static>  =>note: only take effect on next next time encode/decode\n");

#ifdef	SUPPORT_VO_OSD
    printf("    osdtest\n");
#endif
#ifdef	SUPPORT_VI_OSD
	printf("    viosd <cmd> <data>\n");
#endif
    printf("\n");
    printf("  examples, \n");
    printf("    %s rdreg 38\n", MODULE_NAME);
    printf("    %s rdmem 0 10\n", MODULE_NAME);
    printf("    %s wrmem 0 4 11111111 22222222 33333333 44444444\n", MODULE_NAME);
    printf("    %s card 1 fwload\n", MODULE_NAME);

    printf("\n");


    if (nExit) {
        exit(nExit);
    }
}


void mem_dump(unsigned int *buf, unsigned int MemSize, unsigned int StartOff)
{
    int i;
    for (i = 0;i<MemSize;i++) {
        unsigned int dwMem = buf[i];

        if(!(i % 4)) {
            printf("\n%08X: ", StartOff+i);
        }
        printf("%08X ", dwMem);
    }
    printf("\n\n");
}


int do_rdreg(unsigned int ulAddr)
{
    int lErr = 0;
    QPCODEC_DIAG_REGISTER_STRUCT qlmemrw;

	qlmemrw.address = ulAddr;
	qlmemrw.value = 0xDEADBEEF;

    lErr = ioctl(ghDev, PLDEV_DIAG_IOCTL_REG_READ, &qlmemrw);

    if(!lErr) {
        printf("\n%s()-> reg 0x%08x, Val = 0x%08x\n", __FUNCTION__, qlmemrw.address, qlmemrw.value);
    }

    printf("%s()-> return %d\n\n", __FUNCTION__, lErr);

    return lErr;
}


int do_wrreg(unsigned int ulAddr, unsigned int dwVal)
{
    int lErr = 0;
    QPCODEC_DIAG_REGISTER_STRUCT qlmemrw;

	qlmemrw.address = ulAddr;
	qlmemrw.value = dwVal;

    lErr = ioctl(ghDev, PLDEV_DIAG_IOCTL_REG_WRITE, &qlmemrw);

    if(!lErr) {
        printf("\n%s()-> reg 0x%08x, Val = 0x%08x\n", __FUNCTION__, qlmemrw.address, dwVal);
    }

    printf("%s()-> return %d\n\n", __FUNCTION__, lErr);

    return lErr;
}


int do_dumpmem(unsigned int ulAddr, unsigned int ulSize)
{
    int lErr = 0;
    QPCODEC_DIAG_MEMORY_STRUCT qlmemrw;
    int i, loop, notused;
    FILE *fp;
#define DUMPFILE "./dumpdata.dat"
    loop =  (ulSize+255)/256;

    printf("\n%s()-> mem addr=0x%x, size=0x%x dump to %s\n", __FUNCTION__, ulAddr, ulSize, DUMPFILE);

    fp = fopen(DUMPFILE,"wb");
    if (fp)
    {
        for (i=0,notused=ulSize;i<loop;i++,notused-=256)
        {
            qlmemrw.address = ulAddr + i*256;
            if ((i+1)!=loop)
                qlmemrw.num_dwords = 256 >> 2;
            else {
                qlmemrw.num_dwords = (notused+3) >> 2;
            }
            //printf("\n%s()-> (%d %d %d)read mem addr=0x%x, size=0x%x\n", __FUNCTION__, i,loop,notused,qlmemrw.address, qlmemrw.num_dwords*4);

            lErr = ioctl(ghDev, PLDEV_DIAG_IOCTL_MEM_READ, &qlmemrw);
            if(!lErr) {
                if (fp) fwrite(&qlmemrw.value[0], 1, qlmemrw.num_dwords*4, fp);
			}
            else
                break;
        }
        fclose(fp);
    }
    else {
        printf("\n%s()-> can not open output file to %s\n", __FUNCTION__, DUMPFILE);
        lErr = -1;
    }
    printf("%s()-> return %d\n\n", __FUNCTION__, lErr);

    return lErr;
}

int do_rdmem(unsigned int ulAddr, unsigned int ulSize)
{
    int lErr = 0;
    QPCODEC_DIAG_MEMORY_STRUCT qlmemrw;

	qlmemrw.address = ulAddr;
	qlmemrw.num_dwords = ulSize >> 2;

    printf("\n%s()-> mem addr=0x%x, size=0x%x\n", __FUNCTION__, qlmemrw.address, qlmemrw.num_dwords);
    lErr = ioctl(ghDev, PLDEV_DIAG_IOCTL_MEM_READ, &qlmemrw);
    if(!lErr) mem_dump(&qlmemrw.value[0], qlmemrw.num_dwords, qlmemrw.address);

    printf("%s()-> return %d\n\n", __FUNCTION__, lErr);

    return lErr;
}


int do_wrmem(unsigned int ulAddr, unsigned int ulSize, unsigned int ulaValue[])
{
    int lErr = 0;
    QPCODEC_DIAG_MEMORY_STRUCT qlmemrw;
	int	i;

	qlmemrw.address = ulAddr;
	qlmemrw.num_dwords = ulSize;
	for (i = 0; i < ulSize; i++)
	{
		qlmemrw.value[i] = ulaValue[i];
	}

    printf("\n%s()-> mem addr=0x%x, size=0x%x\n", __FUNCTION__, qlmemrw.address, qlmemrw.num_dwords);

    if(ulSize) {
        mem_dump(&qlmemrw.value[0], qlmemrw.num_dwords, qlmemrw.address);
        lErr = ioctl(ghDev, PLDEV_DIAG_IOCTL_MEM_WRITE, &qlmemrw);
    }

    printf("%s()-> return %d\n\n", __FUNCTION__, lErr);

    return lErr;
}


int do_dmard(unsigned int ulAddr, unsigned int ulSize)
{
    int lErr = 0;
    QPCODEC_DIAG_DMA_PTR_STRUCT qlmemrw;

	qlmemrw.deviceAddress = ulAddr;
	qlmemrw.size = ulSize;
	qlmemrw.pData = malloc(qlmemrw.size);

    if(qlmemrw.pData) {
        printf("\n%s()-> mem addr=0x%x, size=0x%x, buff=0x%lx\n", __FUNCTION__, qlmemrw.deviceAddress, qlmemrw.size, (unsigned long) qlmemrw.pData);
        lErr = ioctl(ghDev, PLDEV_DIAG_IOCTL_DMA_READ_PTR, &qlmemrw);
        if(!lErr) mem_dump((unsigned int *)qlmemrw.pData, qlmemrw.size >> 2, qlmemrw.deviceAddress);
        free(qlmemrw.pData);
    } else {
        printf("%s()-> Unable to allocate memory (size=%d) for reading memory \n", __FUNCTION__, qlmemrw.size);
    }

    printf("%s()-> return %d\n\n", __FUNCTION__, lErr);

    return lErr;
}


int do_dmawr(unsigned int ulAddr, unsigned int ulSize, unsigned int ulaValue[])
{
    int lErr = 0;
    QPCODEC_DIAG_DMA_PTR_STRUCT qlmemrw;

    qlmemrw.deviceAddress= ulAddr;
    qlmemrw.size= ulSize * 4;
    qlmemrw.pData= (unsigned char *)ulaValue;

    if(ulSize) {
        printf("\n%s()-> mem addr=0x%x, size=0x%x, buff=0x%lx\n", __FUNCTION__, qlmemrw.deviceAddress, qlmemrw.size, (unsigned long) qlmemrw.pData);

        mem_dump((unsigned int *)qlmemrw.pData, qlmemrw.size >> 2, qlmemrw.deviceAddress);
        lErr = ioctl(ghDev, PLDEV_DIAG_IOCTL_DMA_WRITE_PTR, &qlmemrw);
    }

    printf("%s()-> return %d\n\n", __FUNCTION__, lErr);

    return lErr;
}



int do_rdi2c(unsigned int ulAddr, unsigned int ulSubAddr)
{
    int lErr = 0;
    QPCODEC_DIAG_I2C_WRITE_THEN_READ_STRUCT qli2crw;

    memset(&qli2crw, 0, sizeof(QPCODEC_DIAG_I2C_WRITE_THEN_READ_STRUCT));

	qli2crw.chip_address = (TQP_UINT8)ulAddr;
	qli2crw.num_write_bytes = 1;
	qli2crw.write_bytes[0] = (TQP_UINT8)ulSubAddr;
	qli2crw.num_read_bytes = 1;

    lErr = ioctl(ghDev, PLDEV_DIAG_IOCTL_I2C_WRITE_THEN_READ, &qli2crw);

    printf("%s()-> i2c dev=%02x, reg=%02x, val=%02x \n", __FUNCTION__, qli2crw.chip_address, ulSubAddr, qli2crw.read_bytes[0]);
    printf("%s()-> return %d\n\n", __FUNCTION__, lErr);

    return lErr;
}


int do_wri2c(unsigned int ulAddr, unsigned int ulSubAddr, unsigned char cValue)
{
    int lErr = 0;
    QPCODEC_DIAG_I2C_STRUCT qli2crw;

    memset(&qli2crw, 0, sizeof(QPCODEC_DIAG_I2C_STRUCT));

	qli2crw.chip_address = (TQP_UINT8)ulAddr;
	qli2crw.num_bytes = 2;
	qli2crw.bytes[0] = (TQP_UINT8)ulSubAddr;
	qli2crw.bytes[1] = cValue;

    lErr = ioctl(ghDev, PLDEV_DIAG_IOCTL_I2C_WRITE, &qli2crw);

    printf("%s()-> i2c dev=%02x, reg=%02x, val=%02x \n", __FUNCTION__, qli2crw.chip_address, ulSubAddr, cValue);
    printf("%s()-> return %d\n\n", __FUNCTION__, lErr);

    return lErr;
}

int do_rdi2cnb(unsigned int ulBus, unsigned int ulAddr, unsigned int ulWrSize, unsigned int cValue[], unsigned int ulRdSize)
{
    int lErr = 0;
    int i;
    QPCODEC_DIAG_I2C_WRITE_THEN_READ_STRUCT qli2crw;

    memset(&qli2crw, 0, sizeof(QPCODEC_DIAG_I2C_WRITE_THEN_READ_STRUCT));

	qli2crw.chip_address = (TQP_UINT8)ulAddr;
	qli2crw.num_write_bytes = ulWrSize;
    for(i=0;i<ulWrSize;i++)
    {
        qli2crw.write_bytes[i] = (TQP_UINT8)cValue[i];
    }
    qli2crw.num_read_bytes = ulRdSize;

    if (ulBus==1)
	    lErr = ioctl(ghDev, PLDEV_DIAG_IOCTL_I2C_WRITE_THEN_READ_EX, &qli2crw);
	else
	    lErr = ioctl(ghDev, PLDEV_DIAG_IOCTL_I2C_WRITE_THEN_READ, &qli2crw);

    if(lErr)
        printf("%s() ioctl failed %d %s\n", __FUNCTION__, errno, strerror(errno));
    else
    {
        printf("         0x%02x", ulAddr);
	for(i=0;i<ulWrSize;i++){
        	printf(" %02x",qli2crw.write_bytes[i]);
    	}
       for(i=0;i<ulRdSize;i++)
	   	printf(", 0x%02x", qli2crw.read_bytes[i]);
        printf("\n");
    }

    return lErr;
}


int do_wri2cnb(unsigned int ulBus, unsigned int ulAddr, unsigned int ulSize, unsigned int cValue[])
{
    int lErr = 0;
    int i;
    QPCODEC_DIAG_I2C_STRUCT qli2crw;

    memset(&qli2crw, 0, sizeof(QPCODEC_DIAG_I2C_STRUCT));

	qli2crw.chip_address = (TQP_UINT8)ulAddr;
	qli2crw.num_bytes = ulSize;
	for(i=0;i<ulSize;i++)
	{
        printf("0x%02x ", cValue[i]);
        qli2crw.bytes[i] = cValue[i];
	}
	printf("\n");
    if (ulBus==1)
	    lErr = ioctl(ghDev, PLDEV_DIAG_IOCTL_I2C_WRITE_EX, &qli2crw);
	else
	    lErr = ioctl(ghDev, PLDEV_DIAG_IOCTL_I2C_WRITE, &qli2crw);

    if(lErr)
        printf("%s() ioctl failed %d %s\n", __FUNCTION__, errno, strerror(errno));
    else
    {
        printf("%s()-> return %d\n\n", __FUNCTION__, lErr);
    }
    return lErr;
}

int do_setgpio(unsigned int ulAddr, unsigned int cValue)
{
    int lErr = 0;
	GPIO_BIT_DIRECTION		qlgpiodir;
	GPIO_BIT_VALUE			qlgpioval;

	memset(&qlgpiodir, 0, sizeof(GPIO_BIT_DIRECTION));
	memset(&qlgpioval, 0, sizeof(GPIO_BIT_VALUE));

	qlgpiodir.bBitNumber = ulAddr;
	qlgpiodir.Direction = GPIO_OUTPUT;
	lErr = ioctl(ghDev, PLDEV_CNTL_IOCTL_SET_GPIO_BIT_DIRECTION, &qlgpiodir);

	qlgpioval.bBitNumber = ulAddr;
	qlgpioval.bValue = (cValue)?1:0;
	lErr = ioctl(ghDev, PLDEV_CNTL_IOCTL_SET_GPIO_BIT_VALUE, &qlgpioval);


    printf("%s()-> return %d\n\n", __FUNCTION__, lErr);

    return lErr;
}

int do_fwload()
{
    int lErr = 0;
    return lErr;
}

char *videoprop_id_to_name(int type)
{
	switch (type)
	{
		case 1: return "PowerLine Freq";
		case 2: return "Brightness";
		case 3: return "Contrast";
		case 4: return "Hue";
		case 5: return "Saturation";
		case 6: return "Sharpness";
		case 7: return "Gamma";
		case 8: return "Auto White Balance Enabled";
		case 9: return "White Balance";
		case 10: return "Low Light Boost";
		case 11: return "Auto Focus Enabled";
		case 12: return "Focus";
		case 13: return "Auto Exposure enabled";
		case 14: return "Exposure";
		case 15: return "Auto Exposure Priority";

		case 16: return "PowerLine Freq range";
		case 17: return "Brightness range";
		case 18: return "Contrast range";
		case 19: return "Saturation range";
		case 20: return "Sharpness range";
		case 21: return "Gamma range";
		case 22: return "White Balance range";
		case 23: return "Low Light Boost range";
		case 24: return "Focus range";
		case 25: return "Exposure range";
		case 26: return "Auto Exposure Priority range";
		default :
				 return "UnKnown";
	}
}

void videoprop_usage(int func)
{
    int loop,i;
    loop = (func==0)? 15: 26;
    printf("\n");
    for (i=1;i<=loop;i++)
    {
        if (func==0)
            printf("setvidprop %d <value_in_dec> : for %s\n", i, videoprop_id_to_name(i));
        else
            printf("getvidprop %d : for %s\n", i, videoprop_id_to_name(i));
    }
    printf("\n");
}

int do_setvideoprop(int iFunc, int iValue)
{
    int lErr = 0;
    QLDEV_VIDEOPROP_PARAMETER qlvideoprop;

    memset(&qlvideoprop, 0, sizeof(QLDEV_VIDEOPROP_PARAMETER));

	qlvideoprop.type = iFunc;
	qlvideoprop.value = iValue;

    lErr = ioctl(ghDev, PLDEV_DIAG_IOCTL_VIDEOPROP_SET, &qlvideoprop);

    printf("%s()-> Func=Set %d (%s), val=%d \n", __FUNCTION__, qlvideoprop.type, videoprop_id_to_name(qlvideoprop.type), qlvideoprop.value);
    printf("%s()-> return %d\n\n", __FUNCTION__, lErr);

    return lErr;
}

int do_getvideoprop(int iFunc)
{
    int lErr = 0;
    QLDEV_VIDEOPROP_PARAMETER qlvideoprop;

    memset(&qlvideoprop, 0, sizeof(QLDEV_VIDEOPROP_PARAMETER));

	qlvideoprop.type = 100 + iFunc;

    lErr = ioctl(ghDev, PLDEV_DIAG_IOCTL_VIDEOPROP_GET, &qlvideoprop);

    printf("%s()->  Func=Get %d (%s), ret val=%d min=%d max=%d \n", __FUNCTION__,
    	iFunc, videoprop_id_to_name(iFunc), qlvideoprop.value, qlvideoprop.min, qlvideoprop.max);
    printf("%s()-> return %d\n\n", __FUNCTION__, lErr);

    return lErr;
}

int do_setvideofrontend(int iValue)
{
	/* iValue, define in qpdevdiag
	**  0: dummy
	**  1: saa7115
	**  2: mt9v112
	**  3: ov7720
	**  4: adi7401
    **  5: pe1005
    **  6: sil9011
    **  7: sil9025
    **  8: mi2010
    **  9: usbfw
    ** 10: adv7614
    ** 11: adv7441
    ** 12: tvp7002
    ** 13: auto1
    ** 14: it6603
    ** 15: ov5642
    ** 16: native_mi2010
    **
    **18: ov10633
    */

    int lErr = 0;
	TQP_UINT32 value;

	value = iValue;
    lErr = ioctl(ghDev, PLDEV_DIAG_IOCTL_VIDEO_DECODER_SET, &value);

    printf("%s()-> val=%d\n", __FUNCTION__, value);
    printf("%s()-> return %d\n\n", __FUNCTION__, lErr);

    return lErr;
}

int do_getvideofrontend(void)
{
    int lErr = 0;
	TQP_UINT32 value;

    lErr = ioctl(ghDev, PLDEV_DIAG_IOCTL_VIDEO_DECODER_GET, &value);

    printf("%s()-> ret val=%d\n", __FUNCTION__, value);
    printf("%s()-> return %d\n\n", __FUNCTION__, lErr);

    return lErr;
}

int do_setvideosource(int iValue)
{
/* iValue=0:Auto or HDMI, 1:Composite, 2:S-Video, 3:Component */
    int lErr = 0;
	TQP_UINT32 value;

	value = iValue;
    lErr = ioctl(ghDev, PLDEV_DIAG_IOCTL_VIDEO_DECODER_SOURCE_SET, &value);

    printf("%s()-> val=%d \n", __FUNCTION__, value);
    printf("%s()-> return %d\n\n", __FUNCTION__, lErr);

    return lErr;
}

int do_getvideosource(void)
{
    int lErr = 0;
	unsigned int value;

    lErr = ioctl(ghDev, PLDEV_DIAG_IOCTL_VIDEO_DECODER_SOURCE_GET, &value);

    printf("%s()-> ret val=%d\n", __FUNCTION__, value);
    printf("%s()-> return %d\n\n", __FUNCTION__, lErr);

    return lErr;
}


int do_getdriverversion(void)
{
    int lErr = 0;
	TQP_UINT32 value;

    lErr = ioctl(ghDev, PLDEV_STRM_IOCTL_GET_DRIVER_VER, &value);

    printf("%s()-> ret val=0x%x\n", __FUNCTION__, value);
    printf("%s()-> return %d\n\n", __FUNCTION__, lErr);

    return lErr;
}

int do_getapiversion(void)
{
    int lErr = 0;
	TQP_UINT32 value;

    lErr = ioctl(ghDev, PLDEV_STRM_IOCTL_GET_IOCTLAPI_VER, &value);

    printf("%s()-> ret val=0x%x\n", __FUNCTION__, value);
    printf("%s()-> return %d\n\n", __FUNCTION__, lErr);

    return lErr;
}

int do_getdriverdbglevel(int iFunc)
{
    int lErr = 0;
	TQP_UINT32 value;

	value = (iFunc & 0xFFFF) << 16;
    lErr = ioctl(ghDev, PLDEV_DIAG_IOCTL_DEBUGLEVEL_GET, &value);

    printf("%s()-> ret func=%d  val=%d \n", __FUNCTION__, (value & 0xFFFF0000) >> 16, value & 0xFFFF);
    printf("%s()-> return %d\n\n", __FUNCTION__, lErr);

    return lErr;
}

int do_setdriverdbglevel(int iFunc, int iValue)
{
    int lErr = 0;
	TQP_UINT32 value;

	value = ((iFunc & 0xFFFF) << 16)  | (iValue & 0xFFFF);
    lErr = ioctl(ghDev, PLDEV_DIAG_IOCTL_DEBUGLEVEL_SET, &value);

    printf("%s()-> ret func=%d  val=%d \n", __FUNCTION__, (value & 0xFFFF0000) >> 16, value & 0xFFFF);
    printf("%s()-> return %d\n\n", __FUNCTION__, lErr);

    return lErr;
}

int do_getcurfwmemmode()
{
    int lErr = 0;
	QPCODEC_SYS_CONFIG value;

    lErr = ioctl(ghDev, PLDEV_CNTL_IOCTL_GET_SYS_CONFIG, &value);

    printf("%s()-> ret fwmode=%d \n", __FUNCTION__, value.fixedMode);
    printf("%s()-> return %d\n\n", __FUNCTION__, lErr);

    return lErr;
}

int do_setcurfwmemmode(int iValue)
{
    int lErr = 0;
	QPCODEC_SYS_CONFIG value;

    lErr = ioctl(ghDev, PLDEV_CNTL_IOCTL_GET_SYS_CONFIG, &value);
    if (!lErr) {
        if (iValue!=0)
            iValue = 1;
        value.fixedMode = iValue;
        lErr = ioctl(ghDev, PLDEV_CNTL_IOCTL_SET_SYS_CONFIG, &value);
    }
    printf("%s()-> val=%d \n", __FUNCTION__, iValue);
    printf("%s()-> return %d\n\n", __FUNCTION__, lErr);

    return lErr;
}

int do_checkcodecerror(void)
{
    int lErr = 0;
	TQP_UINT32 value;

    lErr = ioctl(ghDev, PLDEV_CNTL_IOCTL_GET_CODEC_ERROR, &value);

    printf("%s()-> ret val=%d\n", __FUNCTION__, value);
    printf("%s()-> return %d\n\n", __FUNCTION__, lErr);

    return lErr;
}

int do_getdrvlog(void)
{
    int lErr = 0;
    QPCODEC_DIAG_PROFILING_PTR_STRUCT qlprofilingrw;
    int i, loop, notused;
    FILE *fp;
#define PROFILINGDUMPFILE "./prof_drv.dat"

    printf("\n%s()-> dump to %s\n", __FUNCTION__, PROFILINGDUMPFILE);

    fp = fopen(PROFILINGDUMPFILE,"wb");
    if (fp)
    {
        qlprofilingrw.offset = 0;
        qlprofilingrw.size = sizeof(QPCODEC_DIAG_PROFILING_STRUCT);
        qlprofilingrw.max_entries = QP_MAX_PROFILING_SIZE;
        qlprofilingrw.pData = malloc(sizeof(QPCODEC_DIAG_PROFILING_STRUCT));

        //printf("%s()-> ioctl here %d\n", __FUNCTION__, PLDEV_DIAG_IOCTL_DRVPROFILING_GET);
        if (qlprofilingrw.pData)
        {
            lErr = ioctl(ghDev, PLDEV_DIAG_IOCTL_DRVPROFILING_GET, &qlprofilingrw);
            if(!lErr) {
                if (fp) fwrite(qlprofilingrw.pData, 1, sizeof(QPCODEC_DIAG_PROFILING_STRUCT), fp);
            }
            else {
                printf("%s()-> ioctl 0x%lx return %d\n\n", __FUNCTION__, PLDEV_DIAG_IOCTL_DRVPROFILING_GET, lErr);
            }
            free(qlprofilingrw.pData);
        }
        else {
            printf("%s()-> memory allocate failed \n", __FUNCTION__);
            lErr = -1;
        }
        fclose(fp);
    }
    else {
        printf("\n%s()-> can not open profiling dump file %s\n", __FUNCTION__, PROFILINGDUMPFILE);
        lErr = -1;
    }
    printf("%s()-> get driver log size=0x%lx, return %d\n\n", __FUNCTION__, (unsigned long) sizeof(QPCODEC_DIAG_PROFILING_STRUCT), lErr);

    return lErr;
}

int do_getfwlog(void)
{
    int lErr = 0;
    QPCODEC_DIAG_MEMORY_STRUCT qlmemrw;
    int i, loop, notused;
    TQP_UINT32 ulAddr, ulSize, ulUsed, ulEnable;
    FILE *fp;

#define FWPROFILING_BUF_ADDRESS        0x2f10a0
#define FWPROFILING_BUF_COUNT_ADDRESS  0x2f10a4
#define FWPROFILING_BUF_WRITE_ADDRESS  0x2f10a8
#define FWPROFILING_BUF_ENABLE_ADDRESS 0x2f10ac
#define FWPROFILINGDUMPFILE "./prof_fw.dat"

	qlmemrw.address = FWPROFILING_BUF_ADDRESS;
	qlmemrw.num_dwords = 4;

    printf("\n%s()-> get fw log addr from 0x%x, size=0x%x\n", __FUNCTION__, qlmemrw.address, qlmemrw.num_dwords);
    lErr = ioctl(ghDev, PLDEV_DIAG_IOCTL_MEM_READ, &qlmemrw);
    if(lErr) {
        printf("%s()-> return %d\n\n", __FUNCTION__, lErr);
        return lErr;
    }
    ulAddr = qlmemrw.value[0];
    ulSize = qlmemrw.value[1];
    ulUsed = qlmemrw.value[2];
    ulEnable = qlmemrw.value[3];

    /* cannot check enable, after firmware download, enable will be set to 0 */
    if ((ulUsed-ulAddr)>0x400000) {
        printf("\n%s()-> (0x%x 0x%x) no enc/dec before, size=0, dump to %s\n", __FUNCTION__, ulAddr, ulUsed, FWPROFILINGDUMPFILE);
        /* never do encoding/decoding before */
        ulSize = 0;
    }
    else {
        if (ulSize > 0x40000) {
            printf("\n%s()-> from addr=0x%x, size=0x%x (overrun) dump to %s\n", __FUNCTION__, ulAddr, ulSize, FWPROFILINGDUMPFILE);
            ulSize = ulSize & 0x3FFFF;
        }
        else {
            printf("\n%s()-> from addr=0x%x, size=0x%x dump to %s\n", __FUNCTION__, ulAddr, ulSize, FWPROFILINGDUMPFILE);
        }
    }

    ulSize = ulSize * 16;
    loop =  (ulSize+255)/256;

    fp = fopen(FWPROFILINGDUMPFILE,"wb");
    if (fp)
    {
        for (i=0,notused=ulSize;i<loop;i++,notused-=256)
        {
            qlmemrw.address = ulAddr + i*256;
            if ((i+1)!=loop)
                qlmemrw.num_dwords = 256 >> 2;
            else {
                qlmemrw.num_dwords = (notused+3) >> 2;
            }
            //printf("\n%s()-> (%d %d %d)read mem addr=0x%x, size=0x%x\n", __FUNCTION__, i,loop,notused,qlmemrw.address, qlmemrw.num_dwords*4);

            lErr = ioctl(ghDev, PLDEV_DIAG_IOCTL_MEM_READ, &qlmemrw);
            if(!lErr) {
                if (fp) fwrite(&qlmemrw.value[0], 1, qlmemrw.num_dwords*4, fp);
			}
            else
                break;
        }
        fclose(fp);
    }
    else {
        printf("\n%s()-> can not open output file to %s\n", __FUNCTION__, FWPROFILINGDUMPFILE);
        lErr = -1;
    }
    printf("%s()-> return %d\n\n", __FUNCTION__, lErr);

    return lErr;
}

int rdmem_4bytes(unsigned int ulAddr, unsigned int *pulValue)
{
    int lErr = 0;
    QPCODEC_DIAG_MEMORY_STRUCT qlmemrw;

    qlmemrw.address = ulAddr;
    qlmemrw.num_dwords = 1;

    lErr = ioctl(ghDev, PLDEV_DIAG_IOCTL_MEM_READ, &qlmemrw);

    if(!lErr) *pulValue = qlmemrw.value[0];

    //printf("\n%s()-> addr(0x%x) value(0x%x) lErr(%d)\n", __FUNCTION__, qlmemrw.address, *pulValue, lErr);

    return lErr;
}

int wrmem_4bytes(unsigned int ulAddr, unsigned int ulValue)
{
    int lErr = 0;
    QPCODEC_DIAG_MEMORY_STRUCT qlmemrw;
    int	i;

    qlmemrw.address = ulAddr;
    qlmemrw.num_dwords = 1;
    qlmemrw.value[0] = ulValue;

    lErr = ioctl(ghDev, PLDEV_DIAG_IOCTL_MEM_WRITE, &qlmemrw);

    //printf("\n%s()-> addr(0x%x) value(0x%x) lErr(%d)\n", __FUNCTION__, qlmemrw.address, ulValue, lErr);

    return lErr;
}



unsigned int memTestDataBus(unsigned int address)
{
    unsigned int pattern;
    unsigned int readback;

    /*
     * Perform a walking 1's test at the given address.
     */
    for (pattern = 1; pattern != 0; pattern <<= 1) {
        /*
         * Write the test pattern.
         */
        wrmem_4bytes(address, pattern);

        /*
         * Read it back (immediately is okay for this test).
         */
        rdmem_4bytes(address, &readback);
        if (readback != pattern) {
            printf("\n%s()--> fail(0x%08x)\n", __FUNCTION__, pattern);
            return (pattern);
        }
    }

    printf("\n%s()--> pass\n", __FUNCTION__);

    return (0);

}   /* memTestDataBus() */


unsigned int memTestAddressBus(unsigned int baseAddress, unsigned int nBytes)
{
    unsigned int addressMask = (nBytes/sizeof(unsigned int) - 1);
    unsigned int offset;
    unsigned int testOffset;

    unsigned int pattern     = 0xAAAAAAAA;
    unsigned int antipattern = 0x55555555;
    unsigned int readback;

    printf("\n");
    printf("%s()--> test range (0x%08x - 0x%08x)\n", __FUNCTION__, baseAddress, baseAddress+nBytes);

    /*
     * Write the default pattern at each of the power-of-two offsets.
     */
    for (offset = 1; (offset & addressMask) != 0; offset <<= 1) {
        printf("\r%s()--> writing 0x%08x to 0x%08lx", __FUNCTION__, pattern, (unsigned long) baseAddress+(sizeof(unsigned int)*offset));
        wrmem_4bytes(baseAddress+(sizeof(unsigned int)*offset), pattern);
    }
    printf("\n");

    /*
     * Check for address bits stuck high.
     */
    testOffset = 0;
    wrmem_4bytes(baseAddress+(testOffset*4), antipattern);

    for (offset = 1; (offset & addressMask) != 0; offset <<= 1) {
        printf("\r%s()--> reading 0x%08lx\t\t\t", __FUNCTION__, (unsigned long) baseAddress+(sizeof(unsigned int)*offset));
        rdmem_4bytes(baseAddress+(sizeof(unsigned int)*offset), &readback);
        if (readback != pattern) {
            printf("\n%s()--> fail(0x%08lx)\n", __FUNCTION__, (unsigned long) baseAddress+(sizeof(unsigned int)*offset));
            return (baseAddress+(sizeof(unsigned int)*offset));
        }
    }
    printf("\n");

    wrmem_4bytes(baseAddress+(testOffset*4), pattern);

    /*
     * Check for address bits stuck low or shorted.
     */
    for (testOffset = 1; (testOffset & addressMask) != 0; testOffset <<= 1) {
        //printf("\r%s()--> writing 0x%08x to 0x%08x", __FUNCTION__, antipattern, baseAddress+(testOffset*4));
        wrmem_4bytes(baseAddress+(testOffset*4), antipattern);

        //printf("\r%s()--> reading 0x%08x", __FUNCTION__, baseAddress);
        rdmem_4bytes(baseAddress, &readback);
        if (readback != pattern) {
            printf("\n%s()--> fail(0x%08lx)\n", __FUNCTION__, (unsigned long) baseAddress+(sizeof(unsigned int)*testOffset));
            return (baseAddress+(sizeof(unsigned int)*testOffset));
        }

        for (offset = 1; (offset & addressMask) != 0; offset <<= 1) {
            printf("\r%s()--> reading 0x%08lx", __FUNCTION__, (unsigned long) baseAddress+(sizeof(unsigned int)*offset));
            rdmem_4bytes(baseAddress+(sizeof(unsigned int)*offset), &readback);
            if ((readback != pattern) && (offset != testOffset)) {
                printf("\n%s()--> fail(0x%08lx)\n", __FUNCTION__, (unsigned long) baseAddress+(sizeof(unsigned int)*testOffset));
                return (baseAddress+(sizeof(unsigned int)*testOffset));
            }
        }

        //printf("\r%s()--> writing 0x%08x to 0x%08x", __FUNCTION__, pattern, baseAddress+(testOffset*4));
        wrmem_4bytes(baseAddress+(testOffset*4), pattern);
    }

    printf("\n%s()--> pass\n", __FUNCTION__);

    return 0;

}   /* memTestAddressBus() */


unsigned int memTestDevice(unsigned int baseAddress, unsigned int nBytes)
{
    unsigned int offset;
    unsigned int nWords = nBytes / sizeof(unsigned int);

    unsigned int pattern;
    unsigned int antipattern;
    unsigned int readback;

    printf("\n");
    printf("%s()--> test range (0x%08x - 0x%08x)\n", __FUNCTION__, baseAddress, baseAddress+nBytes);

    /*
     * Fill memory with a known pattern.
     */
    for (pattern = 1, offset = 0; offset < nWords; pattern++, offset++) {
        printf("\r%s()--> writing 0x%08x to 0x%08x", __FUNCTION__, pattern, baseAddress+(offset*4));
        wrmem_4bytes(baseAddress+(offset*4), pattern);
    }

    printf("\n");

    /*
     * Check each location and invert it for the second pass.
     */
    for (pattern = 1, offset = 0; offset < nWords; pattern++, offset++) {
        printf("\r%s()--> reading 0x%08lx", __FUNCTION__, (unsigned long) baseAddress+(sizeof(unsigned int)*offset));
        rdmem_4bytes(baseAddress+(sizeof(unsigned int)*offset), &readback);
        if (readback != pattern) {
            printf("\n%s()--> fail(0x%08lx)\n", __FUNCTION__, (unsigned long) baseAddress+(sizeof(unsigned int)*offset));
            return (baseAddress+(sizeof(unsigned int)*offset));
        }

        antipattern = ~pattern;
        //printf("\r%s()--> writing 0x%08x to 0x%08x", __FUNCTION__, antipattern, baseAddress+(offset*4));
        wrmem_4bytes(baseAddress+(offset*4), antipattern);
    }

    /*
     * Check each location for the inverted pattern and zero it.
     */
    for (pattern = 1, offset = 0; offset < nWords; pattern++, offset++) {
        antipattern = ~pattern;
        printf("\r%s()--> reading 0x%08lx", __FUNCTION__, (unsigned long) baseAddress+(sizeof(unsigned int)*offset));
        rdmem_4bytes(baseAddress+(sizeof(unsigned int)*offset), &readback);
        if (readback != antipattern) {
            printf("\n%s()--> fail(0x%08lx)\n", __FUNCTION__, (unsigned long) baseAddress+(sizeof(unsigned int)*offset));
            return (baseAddress+(sizeof(unsigned int)*offset));
        }
    }

    printf("\n%s()--> pass\n", __FUNCTION__);

    return 0;

}   /* memTestDevice() */





#define HAS_ARG_NUM(argc, argnow, need) ((nArgNow + need) < argc)

int main(int argc, char* argv[])
{
    int lErr = 0;
    int nArgNow = 0;

    unsigned int ulCmd=-1;
    unsigned int ulCardIndex = 0;
	unsigned int busType = QPHCI_BUS_TOTAL_BUSES;

    unsigned int ulAddr;
    unsigned int ulSubAddr;
    unsigned int ulSize;
    unsigned int ulSizeEx;
	unsigned int ulBus;
    unsigned int ulaValue[20];
	int iFunc;
	int iValue;
	char *viosdCmd, *viosdData;

    unsigned int ulValueIndex = 0;
    unsigned int dummy;

    while(nArgNow < argc) {
        if(!strcasecmp(argv[nArgNow],"CARD")) {
            if (!HAS_ARG_NUM(argc, nArgNow, 1)) break;
            nArgNow++;
            ulCardIndex = atoi(argv[nArgNow++]);
        } else if(!strcasecmp(argv[nArgNow],"BUS")) {
            if (!HAS_ARG_NUM(argc, nArgNow, 1)) break;
            nArgNow++;
            busType = atoi(argv[nArgNow++]);
        } else if(!strcasecmp(argv[nArgNow],"RDREG")) {
            if (!HAS_ARG_NUM(argc, nArgNow, 1)) break;
            nArgNow++;
            ulCmd = CMD_RDREG;
            ulAddr =  strtoul(argv[nArgNow++], 0, 16);
        } else if(!strcasecmp(argv[nArgNow],"WRREG")) {
            if (!HAS_ARG_NUM(argc, nArgNow, 2)) break;
            nArgNow++;
            ulCmd = CMD_WRREG;
            ulAddr =  strtoul(argv[nArgNow++], 0, 16);
            ulaValue[0] =  strtoul(argv[nArgNow++], 0, 16);
        } else if(!strcasecmp(argv[nArgNow],"RDMEM")) {
            if (!HAS_ARG_NUM(argc, nArgNow, 2)) break;
            nArgNow++;
            ulCmd = CMD_RDMEM;
            ulAddr =  strtoul(argv[nArgNow++], 0, 16);
            ulSize =  strtoul(argv[nArgNow++], 0, 16);
        } else if(!strcasecmp(argv[nArgNow],"WRMEM")) {
            if (!HAS_ARG_NUM(argc, nArgNow, 2)) break;
            nArgNow++;
            ulCmd = CMD_WRMEM;
            ulAddr =  strtoul(argv[nArgNow++], 0, 16);
            ulSize =  strtoul(argv[nArgNow], 0, 16);
            if (!HAS_ARG_NUM(argc, nArgNow, ulSize)) break;
            nArgNow++;
            while(ulValueIndex < ulSize) {
                ulaValue[ulValueIndex++] =  strtoul(argv[nArgNow++], 0, 16);
            }
        } else if(!strcasecmp(argv[nArgNow],"DMARD")) {
            if (!HAS_ARG_NUM(argc, nArgNow, 1)) break;
            nArgNow++;
            ulCmd = CMD_DMARD;
            ulAddr =  strtoul(argv[nArgNow++], 0, 16);
            ulSize =  strtoul(argv[nArgNow++], 0, 16);
        } else if(!strcasecmp(argv[nArgNow],"DMAWR")) {
            if (!HAS_ARG_NUM(argc, nArgNow, 2)) break;
            nArgNow++;
            ulCmd = CMD_DMAWR;
            ulAddr =  strtoul(argv[nArgNow++], 0, 16);
            ulSize =  strtoul(argv[nArgNow], 0, 16);
            if (!HAS_ARG_NUM(argc, nArgNow, ulSize)) break;
            nArgNow++;
            while(ulValueIndex < ulSize) {
                ulaValue[ulValueIndex++] =  strtoul(argv[nArgNow++], 0, 16);
            }
        } else if(!strcasecmp(argv[nArgNow],"RDI2C")) {
            if (!HAS_ARG_NUM(argc, nArgNow, 2)) break;
            nArgNow++;
            ulCmd = CMD_RDI2C;
            ulAddr =  strtoul(argv[nArgNow++], 0, 16);
            ulSubAddr =  strtoul(argv[nArgNow++], 0, 16);
        } else if(!strcasecmp(argv[nArgNow],"WRI2C")) {
            if (!HAS_ARG_NUM(argc, nArgNow, 3)) break;
            nArgNow++;
            ulCmd = CMD_WRI2C;
            ulAddr =  strtoul(argv[nArgNow++], 0, 16);
            ulSubAddr =  strtoul(argv[nArgNow++], 0, 16);
            ulaValue[0] =  strtoul(argv[nArgNow++], 0, 16);
        } else if(!strcasecmp(argv[nArgNow],"RDI2CNB")) {
            if (!HAS_ARG_NUM(argc, nArgNow, 4)) break;
            nArgNow++;
            ulCmd = CMD_RDI2CNB;
            ulBus =  strtoul(argv[nArgNow++], 0, 16);
            ulAddr =  strtoul(argv[nArgNow++], 0, 16);
            ulSize =  strtoul(argv[nArgNow], 0, 16);
            if (!HAS_ARG_NUM(argc, nArgNow, ulSize)) break;
            nArgNow++;
            while(ulValueIndex < ulSize) {
                ulaValue[ulValueIndex++] =  strtoul(argv[nArgNow++], 0, 16);
            }
            ulSizeEx =  strtoul(argv[nArgNow++], 0, 16);
        } else if(!strcasecmp(argv[nArgNow],"WRI2CNB")) {
            if (!HAS_ARG_NUM(argc, nArgNow, 4)) break;
            nArgNow++;
            ulCmd = CMD_WRI2CNB;
            ulBus =  strtoul(argv[nArgNow++], 0, 16);
            ulAddr =  strtoul(argv[nArgNow++], 0, 16);
            ulSize =  strtoul(argv[nArgNow], 0, 16);
            if (!HAS_ARG_NUM(argc, nArgNow, ulSize)) break;
            nArgNow++;
            while(ulValueIndex < ulSize) {
                ulaValue[ulValueIndex++] =  strtoul(argv[nArgNow++], 0, 16);
            }
        } else if(!strcasecmp(argv[nArgNow],"GPIO")) {
            if (!HAS_ARG_NUM(argc, nArgNow, 2)) break;
            nArgNow++;
            ulCmd = CMD_SETGPIO;
            ulAddr =  strtoul(argv[nArgNow++], 0, 16);
            ulaValue[0] =  strtoul(argv[nArgNow], 0, 16);
        } else if(!strcasecmp(argv[nArgNow],"MEMTEST")) {
            if (!HAS_ARG_NUM(argc, nArgNow, 1)) break;
            nArgNow++;
            ulCmd = CMD_MEMTEST;
            ulSize =  strtoul(argv[nArgNow++], 0, 10);
        } else if(!strcasecmp(argv[nArgNow],"FWLOAD")) {
            nArgNow++;
            ulCmd = CMD_FWLOAD;
#ifdef	SUPPORT_VO_OSD
        } else if(!strcasecmp(argv[nArgNow],"OSDTEST")) {
            nArgNow++;
            ulCmd = CMD_OSDTEST;
#endif
#ifdef	SUPPORT_VI_OSD
        } else if(!strcasecmp(argv[nArgNow],"VIOSD")) {
        	if (!HAS_ARG_NUM(argc, nArgNow, 1)) break;
            nArgNow++;
            ulCmd = CMD_VIOSD;
            viosdCmd = argv[nArgNow++];
            viosdData = argv[nArgNow++];
#endif
        } else if(!strcasecmp(argv[nArgNow],"SETVIDPROP")) {
            if (!HAS_ARG_NUM(argc, nArgNow, 2))
            {
                videoprop_usage(0);
                break;
            }
            nArgNow++;
            ulCmd = CMD_SETVIDPROP;
            iFunc =  atol(argv[nArgNow++]);
            iValue =  atol(argv[nArgNow++]);
        } else if(!strcasecmp(argv[nArgNow],"GETVIDPROP")) {
            if (!HAS_ARG_NUM(argc, nArgNow, 1))
            {
                videoprop_usage(1);
                break;
            }
            nArgNow++;
            ulCmd = CMD_GETVIDPROP;
            iFunc =  atol(argv[nArgNow++]);
        } else if(!strcasecmp(argv[nArgNow],"SETVIDFRONTEND")) {
            if (!HAS_ARG_NUM(argc, nArgNow, 1))
                break;
            nArgNow++;
            ulCmd = CMD_SETVIDFRONTEND;
            iValue =  atol(argv[nArgNow++]);
        } else if(!strcasecmp(argv[nArgNow],"GETVIDFRONTEND")) {
            nArgNow++;
            ulCmd = CMD_GETVIDFRONTEND;
        } else if(!strcasecmp(argv[nArgNow],"SETVIDSOURCE")) {
            if (!HAS_ARG_NUM(argc, nArgNow, 1))
                break;
            nArgNow++;
            ulCmd = CMD_SETVIDSOURCE;
            iValue =  atol(argv[nArgNow++]);
        } else if(!strcasecmp(argv[nArgNow],"GETVIDSOURCE")) {
            nArgNow++;
            ulCmd = CMD_GETVIDSOURCE;
        } else if(!strcasecmp(argv[nArgNow],"GETDRIVERVERSION")) {
            nArgNow++;
            ulCmd = CMD_GETDRIVERVER;
        } else if(!strcasecmp(argv[nArgNow],"GETAPIVERSION")) {
            nArgNow++;
            ulCmd = CMD_GETIOCTLAPIVER;
        } else if(!strcasecmp(argv[nArgNow],"GETDBGLEVEL")) {
            if (!HAS_ARG_NUM(argc, nArgNow, 1))
                break;
            nArgNow++;
            ulCmd = CMD_GETDRIVERDBGLEVEL;
            iFunc =  atol(argv[nArgNow++]);
        } else if(!strcasecmp(argv[nArgNow],"SETDBGLEVEL")) {
            if (!HAS_ARG_NUM(argc, nArgNow, 2))
                break;
            nArgNow++;
            ulCmd = CMD_SETDRIVERDBGLEVEL;
            iFunc =  atol(argv[nArgNow++]);
            iValue =  atol(argv[nArgNow++]);
        } else if(!strcasecmp(argv[nArgNow],"GETCURFWMEMMODE")) {
            nArgNow++;
            ulCmd = CMD_GETCURFWMEMMODE;
        } else if(!strcasecmp(argv[nArgNow],"SETCURFWMEMMODE")) {
            if (!HAS_ARG_NUM(argc, nArgNow, 1))
                break;
            nArgNow++;
            ulCmd = CMD_SETCURFWMEMMODE;
            iValue =  atol(argv[nArgNow++]);
        } else if(!strcasecmp(argv[nArgNow],"DUMPMEM")) {
            if (!HAS_ARG_NUM(argc, nArgNow, 2)) break;
            nArgNow++;
            ulCmd = CMD_DUMPMEM;
            ulAddr =  strtoul(argv[nArgNow++], 0, 16);
            ulSize =  strtoul(argv[nArgNow++], 0, 16);
        } else if(!strcasecmp(argv[nArgNow],"GETDRVLOG")) {
            nArgNow++;
            ulCmd = CMD_GETDRVLOG;
        } else if(!strcasecmp(argv[nArgNow],"GETFWLOG")) {
            nArgNow++;
            ulCmd = CMD_GETFWLOG;
        } else if(!strcasecmp(argv[nArgNow],"CHECKCODECERROR")) {
            nArgNow++;
            ulCmd = CMD_CHECKCODECERROR;
        } else {
            nArgNow++;
        }
    }

    if (ulCmd!=-1)
    {
        lErr = lookup_video_device_node(ulCardIndex, COMPONENT_TYPE_ENCODE, busType, szDevName);
        if (!lErr)
        {
            ghDev = open(szDevName, O_RDWR);
            if(ghDev <= 0) {
                printf("Unable to Open %s.\n", szDevName);
                lErr = 1;
                goto FUNC_END;
            }
        }
        else {
            printf("Can not find dev node for pl330.\n");
            lErr = 1;
            goto FUNC_END;
        }
    }

    switch(ulCmd) {
    case CMD_RDREG:
        lErr = do_rdreg(ulAddr);
        break;
    case CMD_WRREG:
        lErr = do_wrreg(ulAddr, ulaValue[0]);
        break;
    case CMD_RDMEM:
        lErr = do_rdmem(ulAddr, ulSize);
        break;
    case CMD_WRMEM:
        lErr = do_wrmem(ulAddr, ulSize, ulaValue);
        break;
    case CMD_DMARD:
        lErr = do_dmard(ulAddr, ulSize);
        break;
    case CMD_DMAWR:
        lErr = do_dmawr(ulAddr, ulSize, ulaValue);
        break;
    case CMD_RDI2C:
        lErr = do_rdi2c(ulAddr, ulSubAddr);
        break;
    case CMD_WRI2C:
        lErr = do_wri2c(ulAddr, ulSubAddr, ulaValue[0]);
        break;
    case CMD_RDI2CNB:
        lErr = do_rdi2cnb(ulBus, ulAddr, ulSize, ulaValue, ulSizeEx);
        break;
    case CMD_WRI2CNB:
        lErr = do_wri2cnb(ulBus, ulAddr, ulSize, ulaValue);
        break;
	case CMD_SETGPIO:
        lErr = do_setgpio(ulAddr, ulaValue[0]);
        break;
    case CMD_MEMTEST:
        if (0 == ioctl(ghDev, PLDEV_DIAG_IOCTL_RESET_ARM, &dummy)) {
            memTestDataBus(0);
            memTestAddressBus(0, ulSize*1024*1024);
            memTestDevice(0, ulSize*1024*1024);
            ioctl(ghDev, PLDEV_DIAG_IOCTL_FIRMWARE_RELOAD, &dummy);
        }
        break;
    case CMD_FWLOAD:
        lErr = do_fwload();
        break;
#ifdef	SUPPORT_VO_OSD
    case CMD_OSDTEST:
        lErr = OSD_Test(ghDev, 1, 0, 0, 1280, 720, &ulValueIndex);
        break;
#endif
#ifdef	SUPPORT_VI_OSD
    case CMD_VIOSD:
        lErr = VIOSD_SetConfig(ghDev, viosdCmd, viosdData);
        break;
#endif
    case CMD_SETVIDPROP:
        lErr = do_setvideoprop(iFunc, iValue);
        break;
    case CMD_GETVIDPROP:
        lErr = do_getvideoprop(iFunc);
		break;
    case CMD_SETVIDFRONTEND:
        lErr = do_setvideofrontend(iValue);
        break;
    case CMD_GETVIDFRONTEND:
        lErr = do_getvideofrontend();
		break;
    case CMD_SETVIDSOURCE:
        lErr = do_setvideosource(iValue);
        break;
    case CMD_GETVIDSOURCE:
        lErr = do_getvideosource();
		break;
    case CMD_GETDRIVERVER:
        lErr = do_getdriverversion();
		break;
    case CMD_GETIOCTLAPIVER:
        lErr = do_getapiversion();
		break;
    case CMD_GETDRIVERDBGLEVEL:
        lErr = do_getdriverdbglevel(iFunc);
		break;
    case CMD_SETDRIVERDBGLEVEL:
        lErr = do_setdriverdbglevel(iFunc, iValue);
		break;
    case CMD_GETCURFWMEMMODE:
        lErr = do_getcurfwmemmode();
		break;
    case CMD_SETCURFWMEMMODE:
        lErr = do_setcurfwmemmode(iValue);
		break;
    case CMD_DUMPMEM:
        lErr = do_dumpmem(ulAddr, ulSize);
        break;
    case CMD_GETDRVLOG:
        lErr = do_getdrvlog();
        break;
    case CMD_GETFWLOG:
        lErr = do_getfwlog();
        break;
    case CMD_CHECKCODECERROR:
        lErr = do_checkcodecerror();
		break;
    default:
        usage(0);
        break;
    }

FUNC_END:

    if(ghDev > 0) close(ghDev);

    return lErr;
}

