/*************************************************************************/
/**      **/
/**            **/
/**                **/
/**              **/
/**                                                                     **/
/**                               **/
/**                                                  **/
/*************************************************************************/

#include <stdio.h>
#include <unistd.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <time.h>
#include <mntent.h>
#include <signal.h>
#include <asm/types.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>

#include <linux/videodev2.h>

#include "qptypes.h"
#include "pldevioctl.h"

#include "qlutils.h"
#include "vidpropstruct.h"

#include "qpflash.h"

#define MODULE_NAME     "plflash"
#define MODULE_VERSION  "0.1"

#define QPFLASH_SECT_SIZE 0x10000

enum cmd_type_e {
    CMD_NONE = 0,
    CMD_UPDATE,
    CMD_LIST
};

static int ghDev = 0;
char szDevName[32];

unsigned int gunFlashSize = 0;
unsigned int gunFlashSectorNums = 0;

void usage(int nExit)
{
    printf("\n");
    printf("  [%s version %s]\n", MODULE_NAME, MODULE_VERSION);
    printf("  -------------------------------------------------\n");
    printf("\n");

    printf("\n");
    printf("  [%s version %s]\n", MODULE_NAME, MODULE_VERSION);
    printf("  -------------------------------------------------\n");
    printf("  %s <arg1> <arg2> ...\n", MODULE_NAME);
    printf("\n");
    printf("  arg for card selection,\n");
    printf("    card                               => card number. Could be 0/1/2/3, default is 0.\n");
    printf("\n");
    printf("  arg command,\n");
    printf("    list\n");
    printf("    update <image_file_to_update_into_flash>\n");
    printf("\n");
    printf("  examples, \n");
    printf("    %s list\n", MODULE_NAME);
    printf("    %s update qpavstrm-v3.0.3.36.flash\n", MODULE_NAME);

    printf("\n");


    if (nExit) {
        exit(nExit);
    }
}


void mem_dump(unsigned int *buf, unsigned int MemSize, unsigned int StartOff)
{
    int i;
    for (i = 0;i<MemSize;i++) {
        unsigned int dwMem = buf[i];

        if(!(i % 4)) {
            printf("\n%08X: ", StartOff+(i*4));
        }
        printf("%08X ", dwMem);
    }
    printf("\n\n");
}


int wrreg(unsigned int ulAddr, unsigned int dwVal)
{
    int lErr = 0;
    QPCODEC_DIAG_REGISTER_STRUCT qlmemrw;

    qlmemrw.address = ulAddr;
    qlmemrw.value = dwVal;

    lErr = ioctl(ghDev, PLDEV_DIAG_IOCTL_REG_WRITE, &qlmemrw);

    return lErr;
}


int rdreg(unsigned int ulAddr, unsigned int *pdwVal)
{
    int lErr = 0;
    QPCODEC_DIAG_REGISTER_STRUCT qlmemrw;

    qlmemrw.address = ulAddr;
    qlmemrw.value = *pdwVal = 0xDEADBEEF;

    lErr = ioctl(ghDev, PLDEV_DIAG_IOCTL_REG_READ, &qlmemrw);

    if (!lErr) { 
        *pdwVal = qlmemrw.value;
    }

    return lErr;
}


int wrmem(unsigned int ulAddr, unsigned int ulWords, unsigned int *punaBuf)
{
    int lErr = 0;
    QPCODEC_DIAG_MEMORY_STRUCT qlmemrw;

    qlmemrw.address = ulAddr;
    qlmemrw.num_dwords = ulWords;
    memcpy(qlmemrw.value, punaBuf, ulWords*4);

    //printf("\n%s()-> mem addr=0x%x, size=0x%x\n", __FUNCTION__, qlmemrw.address, qlmemrw.num_dwords);

    if(ulWords) {
        //mem_dump(qlmemrw.value, qlmemrw.num_dwords, qlmemrw.address);
        lErr = ioctl(ghDev, PLDEV_DIAG_IOCTL_MEM_WRITE, &qlmemrw);
    }

    //printf("%s()-> return %d\n\n", __FUNCTION__, lErr);

    return lErr;
}


int rdmem(unsigned int ulAddr, unsigned int ulWords, unsigned int *punaBuf)
{
    int lErr = 0;
    QPCODEC_DIAG_MEMORY_STRUCT qlmemrw;

    qlmemrw.address = ulAddr;
    qlmemrw.num_dwords = ulWords;

    //printf("\n%s()-> mem addr=0x%x, size=0x%x\n", __FUNCTION__, qlmemrw.address, qlmemrw.num_dwords);

    lErr = ioctl(ghDev, PLDEV_DIAG_IOCTL_MEM_READ, &qlmemrw);
    if(!lErr) {
        //mem_dump(qlmemrw.value, qlmemrw.num_dwords, qlmemrw.address);
        memcpy(punaBuf, qlmemrw.value, ulWords*4);
    }

    //printf("%s()-> return %d\n\n", __FUNCTION__, lErr);

    return lErr;
}


int flash_get_capacity()
{
    int lErr = 0;
    unsigned int unValue = 0xDEADBEEF; 

    lErr = wrreg(0x8B0, 0x9F000003);
    if (lErr) {
        printf("%s()-> wrreg(0x8B0) failed, lErr(%d)\n", __FUNCTION__, lErr);
        return lErr;
    }

    do {
        lErr = rdreg(0x8B0, &unValue);
        if (lErr) {
            printf("%s()-> rdreg(0x8B0) failed, lErr(%d)\n", __FUNCTION__, lErr);
            return lErr;
        }

        if ((unValue & 0x3) == 0) {
            break;
        }            
    } while(1);


    lErr = rdreg(0x8B8, &unValue);
    if (lErr) {
        printf("%s()-> rdreg(0x8B8) failed, lErr(%d)\n", __FUNCTION__, lErr);
        return lErr;
    }

    gunFlashSize = (1 << ((unValue >> 16) & 0xff));
    gunFlashSectorNums = gunFlashSize / QPFLASH_SECT_SIZE;

    //printf("%s()-> gunFlashSize(%d), gunFlashSectorNums(%d)\n\n", __FUNCTION__, gunFlashSize, gunFlashSectorNums);

    return lErr;
}


unsigned int flash_set_write(void)
{
    unsigned int unRet = 0x08;
    unsigned int temp;

    rdreg(0x40, &unRet);
    temp = unRet | (1<<8);
    wrreg(0x40, temp);

    // turn on the LED
    rdreg(0x614, &temp);
    temp |= (1<<2);
    wrreg(0x614, temp);

    return unRet;
}


unsigned int flash_set_map(void)
{
    unsigned int unRet = 0x08;
    unsigned int temp;

    rdreg(0x40, &unRet);
    temp = unRet | (1<<9);
    wrreg(0x40, temp);

    return unRet;
}


void flash_restore(unsigned int val)
{
    unsigned int temp;

    wrreg(0x40, val);

    // turn off the LED
    rdreg(0x614, &temp);
    temp &= ~(1<<2);
    wrreg(0x614, temp);
}


unsigned int flash_set_map_and_write (void)
{
    unsigned int unRet = 0x08;
    unsigned int temp;

    rdreg(0x40, &unRet);
    temp = unRet | (1<<8) | (1<<9);
    wrreg(0x40, temp);

    // turn on the LED
    rdreg(0x614, &temp);
    temp |= (1<<2);
    wrreg(0x614, temp);

    return unRet;
}


unsigned int flash_clear_protection (void)
{
    unsigned int unRet = 0;

    // Read the Flash Status
    wrreg(0x8B0,0x05000003);
    do {
        unsigned int temp;
        rdreg(0x8B0, &temp);
        if ((temp & 0x3) == 0) {
            break;
        }
    } while (1);
    rdreg(0x8B8, &unRet);

    // Than write 0 to it
    wrreg(0x8B8,0);
    wrreg(0x8B0,0x01000003);
    do {
        unsigned int temp;
        rdreg(0x8B0, &temp);
        if ((temp & 0x3) == 0) {
            break;
        }
    } while (1);

    return unRet;
}


unsigned int flash_restore_protection (unsigned int val)
{
    wrreg(0x8B8, val>>24);
    wrreg(0x8B0, 0x01000003);
    do {
        unsigned int temp;
        rdreg(0x8B0, &temp);
        if ((temp & 0x3) == 0) {
            break;
        }
    } while (1);
}



int erase_sector (unsigned int sectNum)
{
    // Make Flash writable
    unsigned int flashSet = flash_set_write();
    unsigned int flashStatus = flash_clear_protection();

    wrreg(0x8B4, sectNum*QPFLASH_SECT_SIZE);
    wrreg(0x8B0,0xD8000003);

    do {
        unsigned int temp;
        rdreg(0x8B0, &temp);
        if ((temp & 0x3) == 0) {
            break;
        }
    } while (1);

    flash_restore_protection(flashStatus);
    flash_restore(flashSet);

    return 0;
}


int flash_sector_read_dwords(unsigned int unSectNum, unsigned int unWordOffset, unsigned int *punaBuf, unsigned int unWords)
{
    int nRet = -1;
    unsigned int unFlashStartAddr = 0xFC000000;
    unsigned int unByteOffset = unWordOffset*4;
    unsigned int unWordsLeft = QPFLASH_SECT_SIZE/4 - unWordOffset;

    if (unSectNum >= gunFlashSectorNums) {
        goto EXIT;
    }

    if (unByteOffset < QPFLASH_SECT_SIZE) {
        unsigned int unFlashSet = flash_set_map();
        unsigned int unFlashAddr = unFlashStartAddr + unSectNum*QPFLASH_SECT_SIZE + unByteOffset;
        if (unWords > unWordsLeft) {
            unWords = unWordsLeft;
        }
        // Make Flash fully mapped into memory
        nRet = 0;
        // split the transaction in blocks of QP_MAX_MEMORY_DWORDS words or less
        while (unWords > 0) {
            unsigned int unToRead = unWords;
            if (unToRead > QP_MAX_MEMORY_DWORDS) {
                unToRead = QP_MAX_MEMORY_DWORDS;
            }
            if (rdmem(unFlashAddr, unToRead, punaBuf) == 0) {
                unWords -= unToRead;
                nRet += unToRead;
                unFlashAddr += unToRead*4;
                punaBuf += unToRead;
            } else {
                break;
            }
        }
        flash_restore(unFlashSet);
    }

EXIT:

    return (nRet);
}


#define READBACK_CHECK 0

int flash_sector_write_dwords(unsigned int unSectNum, unsigned int unWordOffset, unsigned int *punaBuf, unsigned int unWords)
{
    int nRet = -1;
    unsigned int unFlashStartAddr = 0xFC000000;
    unsigned int unByteOffset = unWordOffset*4;
    unsigned int unWordsLeft = QPFLASH_SECT_SIZE/4 - unWordOffset;

    if (unSectNum >= gunFlashSectorNums) {
        goto EXIT;
    }

    if (unByteOffset < QPFLASH_SECT_SIZE) {
        // Make Flash writeable and fully mapped into memory
        unsigned int unFlashSet = flash_set_map_and_write();
        unsigned int unFlashStatus = flash_clear_protection();
        unsigned int unFlashAddr = unFlashStartAddr + unSectNum*QPFLASH_SECT_SIZE + unByteOffset;
        if (unWords > unWordsLeft) {
            unWords = unWordsLeft;
        }
        nRet = 0;
        // split the transaction in blocks of QP_MAX_MEMORY_DWORDS words or less
        while (unWords > 0) {
            unsigned int unToWrite = unWords;
            if (unToWrite > QP_MAX_MEMORY_DWORDS) {
                unToWrite = QP_MAX_MEMORY_DWORDS;
            }
            if (wrmem(unFlashAddr, unToWrite, punaBuf) == 0) {
#ifdef READBACK_CHECK
                unsigned int unaReadBackBuf[QP_MAX_MEMORY_DWORDS];
                // read back to see if there were any errors
                if (rdmem(unFlashAddr, unToWrite, unaReadBackBuf) == 0) {
                    if (memcmp(punaBuf, unaReadBackBuf, unToWrite*4) == 0) {
#endif 
                        unWords -= unToWrite;
                        nRet += unToWrite;
                        unFlashAddr += unToWrite*4;
                        punaBuf += unToWrite;
#ifdef READBACK_CHECK
                    }else{
                        break;
                    }
                }else{
                    break;
                }
#endif 
            }else{
                break;
            }
        }
        flash_restore_protection(unFlashStatus);
        flash_restore(unFlashSet);
    }

EXIT:

    return (nRet);
}


int do_update(char *pcaFileSrc)
{
    int lErr = 0;
    FILE *fpFile = NULL;
    struct stat stState;
    int nFileSize = 0;
    int nChar = 0;

    int nFileOffset = 0;

    unsigned char *pucaBuf = NULL;
    unsigned int unSector = 0;

    int nHasboot = 0;
    int nHasplat = 0;
    int nHasldFW = 0;
    int nHasmaFW = 0;

    fpFile = fopen(pcaFileSrc, "rb");
    if (fpFile == NULL) {
        printf("%s()-> failed to open file (%s)\n", __FUNCTION__, pcaFileSrc);
        goto EXIT;
    }

    fstat(fileno(fpFile), &stState);
    nFileSize = stState.st_size;

    if (nFileSize > gunFlashSize) {
        printf("%s()-> flash size(%d) is smaller than file image size(%d)\n", __FUNCTION__, gunFlashSize, nFileSize);
        goto EXIT;
    } else {
        printf("%s()-> flash size(%d), file image size(%d)\n", __FUNCTION__, gunFlashSize, nFileSize);
    }

    pucaBuf = (unsigned char *)malloc(((nFileSize/QPFLASH_SECT_SIZE)+1)*QPFLASH_SECT_SIZE);  // allocate one more sector size

    if (!pucaBuf) {
        printf("%s()-> malloc(%d) failed\n", __FUNCTION__, nFileSize);
        goto EXIT;
    }

    if (fread((void *)pucaBuf, 1, nFileSize, fpFile) != nFileSize) {
        printf("%s()-> failed to import data from %s\n", __FUNCTION__, pcaFileSrc);
        goto EXIT;
    }

    ////////////////////////////////////////////////////////////
    // list file sectors  

    while((unSector*QPFLASH_SECT_SIZE) < nFileSize) {
        unsigned int unCheckSum = 0;
        unsigned int unDWords = 0;
        unsigned int *punaBuf = (unsigned int *)(pucaBuf + (unSector*QPFLASH_SECT_SIZE));
        int nToRead = QPFLASH_SECT_SIZE; 
        PTQPFlashHeader pSectorH = (PTQPFlashHeader)punaBuf;

        unCheckSum = pSectorH->tag;
        unCheckSum += pSectorH->words;
        unCheckSum += pSectorH->tagType;
        unCheckSum += pSectorH->dateTime;
        unCheckSum += pSectorH->nextSector;
        unCheckSum += pSectorH->loadAddress;
        unCheckSum += pSectorH->dataChecksum;

        if (unCheckSum != pSectorH->hdrChecksum) {
            //skip this sector
            printf("%s()-> file-sector(%2d): n/a\n", __FUNCTION__, unSector);
            unSector++;
            continue;
        }

        unDWords = pSectorH->words;

        unCheckSum = 0;

        while (unDWords) {
            unDWords = unDWords - 1; 
            unCheckSum = unCheckSum + punaBuf[(unDWords) + sizeof(TQPFlashHeader)/4/*skip header*/];
        }

        printf("%s()-> file-sector(%2d): type(%c%c%c%c) data-dword-len(%5d) data-checksum(0x%08x) - data-checksum %s\n", 
               __FUNCTION__, 
               unSector,
               (pSectorH->tagType & 0xff), 
               (pSectorH->tagType & 0xff00) >> 8, 
               (pSectorH->tagType & 0xff0000) >> 16, 
               (pSectorH->tagType & 0xff000000) >> 24, 
               pSectorH->words,
               pSectorH->dataChecksum, 
               (unCheckSum == pSectorH->dataChecksum) ? "match" : "mismatch");

        if (unCheckSum != pSectorH->dataChecksum) {
            goto EXIT;
        }

        if (pSectorH->tagType == 0x746f6f62) {
            nHasboot = 1;
        } else if (pSectorH->tagType == 0x74616c70) {
            nHasplat = 1;
        } else if (pSectorH->tagType == 0x5746646c) {
            nHasldFW = 1;
        } else if (pSectorH->tagType == 0x5746616d) {
            nHasmaFW = 1;
        }

        unSector++;
    }
    printf("\n");


    if (!(nHasboot & nHasplat & nHasldFW & nHasmaFW)) {
        printf("%s()-> miss some mandatory sectors, image file could be corrupted\n", __FUNCTION__);
        printf("%s()-> boot(%s) plat(%s) ldFW(%s) maFW(%s)\n", 
               __FUNCTION__, 
               nHasboot ? "yes" : "no", 
               nHasplat ? "yes" : "no", 
               nHasldFW ? "yes" : "no", 
               nHasmaFW ? "yes" : "no");
        goto EXIT;
    }


    ////////////////////////////////////////////////////////////
    // ready to start update ?
    printf("%s()-> press 'y' to start update flash - ", __FUNCTION__);

    nChar = getchar();
     
    if (nChar != 'y') {
        goto EXIT;
    }

    printf("\n");

    printf("%s()-> start update, DO NOT remove the device \n", __FUNCTION__);


    ////////////////////////////////////////////////////////////
    // erase flash sectors 

    printf("%s()-> erasing ", __FUNCTION__);
    unSector = 0;
    while(unSector < gunFlashSectorNums) {
        printf(".");
        fflush(stdout);
        erase_sector(unSector);
        unSector++;
    }
    printf("\n");


    ////////////////////////////////////////////////////////////
    // write file into flash 

    printf("%s()-> writing ", __FUNCTION__);
    unSector = 0;
    while((unSector*QPFLASH_SECT_SIZE) < nFileSize) {
        unsigned int unCheckSum = 0;
        unsigned int unDWords = 0;
        unsigned int *punaBuf = (unsigned int *)(pucaBuf + (unSector*QPFLASH_SECT_SIZE));
        int nToRead = QPFLASH_SECT_SIZE; 
        PTQPFlashHeader pSectorH = (PTQPFlashHeader)punaBuf;

        unCheckSum = pSectorH->tag;
	unCheckSum += pSectorH->words;
	unCheckSum += pSectorH->tagType;
	unCheckSum += pSectorH->dateTime;
	unCheckSum += pSectorH->nextSector;
	unCheckSum += pSectorH->loadAddress;
	unCheckSum += pSectorH->dataChecksum;

        if (unCheckSum != pSectorH->hdrChecksum) {
            //skip this sector
            unSector++;
            continue;
        }

        unDWords = pSectorH->words;

        unCheckSum = 0;

        while (unDWords) {
            unDWords = unDWords - 1; 
            unCheckSum = unCheckSum + punaBuf[(unDWords) + sizeof(TQPFlashHeader)/4/*skip header*/];
        }

        if (unCheckSum != pSectorH->dataChecksum) {
            continue;
        }

        printf(".");
        fflush(stdout);
        flash_sector_write_dwords(unSector, 0, punaBuf, QPFLASH_SECT_SIZE/4);

        unSector++;
    }
    printf("\n");


    ////////////////////////////////////////////////////////////
    // ready to start update ?
    printf("%s()-> update complete\n", __FUNCTION__);

#if 0
    flash_sector_read_dwords(0, 0, unaBuf, 64*1024/4);

    punLen = &unaBuf[1];

    printf("*punLen(%d)\n", *punLen);

    while (*punLen) {
        *punLen = *punLen - 1; 
        unCheckSum = unCheckSum + unaBuf[(*punLen) + 8];
    }

    printf("unCheckSum(0x%08x)\n", unCheckSum);
#endif

EXIT:    
    if (fpFile) {
        fclose(fpFile);
    }

    if (pucaBuf) {
        free(pucaBuf);
    }

    return lErr;
}



int do_list()
{
    int lErr = 0;

    unsigned int unSector = 0;
    unsigned int unaBuf[QPFLASH_SECT_SIZE/4];

    while(unSector < gunFlashSectorNums) {
        unsigned int unCheckSum = 0;
        unsigned int unDWords = 0;

        PTQPFlashHeader pSectorH = (PTQPFlashHeader)unaBuf;

        flash_sector_read_dwords(unSector, 0, unaBuf, sizeof(TQPFlashHeader)/4);

        unCheckSum = pSectorH->tag;
	unCheckSum += pSectorH->words;
	unCheckSum += pSectorH->tagType;
	unCheckSum += pSectorH->dateTime;
	unCheckSum += pSectorH->nextSector;
	unCheckSum += pSectorH->loadAddress;
	unCheckSum += pSectorH->dataChecksum;

        if (unCheckSum != pSectorH->hdrChecksum) {
            //skip this sector
            printf("%s()-> file-sector(%2d): n/a\n", __FUNCTION__, unSector);
            unSector++;
            continue;
        }

        printf("%s()-> file-sector(%2d): type(%c%c%c%c) data-dword-len(%5d)\n", 
               __FUNCTION__, 
               unSector,
               (pSectorH->tagType & 0xff), 
               (pSectorH->tagType & 0xff00) >> 8, 
               (pSectorH->tagType & 0xff0000) >> 16, 
               (pSectorH->tagType & 0xff000000) >> 24, 
               pSectorH->words);

        unSector++;
    }

    return lErr;
}




#define HAS_ARG_NUM(argc, argnow, need) ((nArgNow + need) < argc)

int main(int argc, char* argv[])
{
    int lErr = 0;
    int nArgNow = 0;

    unsigned int ulCmd=-1;
    unsigned int ulCardIndex = 0;
    unsigned int busType = QPHCI_BUS_TOTAL_BUSES;

    unsigned int ulAddr;
    unsigned int ulSubAddr;
    unsigned int ulSize;
    unsigned int ulSizeEx;
    unsigned int ulBus;
    unsigned int ulaValue[20];
    int iFunc;
    int iValue;

    unsigned int ulValueIndex = 0;
    unsigned int dummy;
    char *pcaFileSrc = NULL;

    printf("\n");

    while(nArgNow < argc) {
        if(!strcasecmp(argv[nArgNow],"CARD")) {
            if (!HAS_ARG_NUM(argc, nArgNow, 1)) break;
            nArgNow++;
            ulCardIndex = atoi(argv[nArgNow++]);
        } else if(!strcasecmp(argv[nArgNow],"UPDATE")) {
            if (!HAS_ARG_NUM(argc, nArgNow, 1)) break;
            nArgNow++;
            ulCmd = CMD_UPDATE;
            pcaFileSrc = argv[nArgNow++];
        } else if(!strcasecmp(argv[nArgNow],"LIST")) {
            nArgNow++;
            ulCmd = CMD_LIST;
        } else {
            nArgNow++;
        }
    }

    if (ulCmd!=-1)
    {
        lookup_video_device_node(ulCardIndex, COMPONENT_TYPE_ENCODE, busType, szDevName);

        ghDev = open(szDevName, O_RDWR);
        if(ghDev <= 0) {
            printf("Unable to Open %s.\n", szDevName);
            lErr = 1;
            goto FUNC_END;
        }
    }

    lErr = flash_get_capacity();

    switch(ulCmd) {
    case CMD_UPDATE:
        lErr = do_update(pcaFileSrc);
        break;
    case CMD_LIST:
        lErr = do_list();
        break;
    default:
        usage(0);
        break;
    }

FUNC_END:

    printf("\n");

    if(ghDev > 0) close(ghDev);

    return lErr;
}

