/*************************************************************************/
/**      **/
/**            **/
/**                **/
/**              **/
/**                                                                     **/
/**                               **/
/**                                                  **/
/*************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include "qlstrm.h"
#include "qlmd.h"

#define MD_FIFO  "./mdfifo"

extern char *skip_whitespace(char *Str);
extern char *skip_to_whitespace(char *Str);



/*************************************************************************/
int QLMD_Initial(component_t* pComponent, PTQLMDInfo* ppQLMDInfo)
{
	int 			lErr = 0;
	int				i;
	PTQLMDInfo		pQLMDInfo;
	TQP_UINT32		uiBytesCount;

	pQLMDInfo = (TQLMDInfo *)malloc(sizeof(TQLMDInfo));

	if(pQLMDInfo == NULL)
	{
		printf("[QLMD]Error allocating instance memory\n");
		return -1;
	}
	memset(pQLMDInfo, 0, sizeof(TQLMDInfo));

	pQLMDInfo->dwMBWidth = pComponent->ulWidth>>4;
	pQLMDInfo->dwMBHeight = (pComponent->ulHeight+15)>>4;

	pQLMDInfo->dwBitCount = 16;		//only support 16bits size of local activity


	uiBytesCount = (pQLMDInfo->dwBitCount+7)>>3;
	pQLMDInfo->padwLocalAct[0] = malloc(pQLMDInfo->dwMBWidth*pQLMDInfo->dwMBHeight*uiBytesCount);
	pQLMDInfo->padwLocalAct[1] = malloc(pQLMDInfo->dwMBWidth*pQLMDInfo->dwMBHeight*uiBytesCount);
	if ((pQLMDInfo->padwLocalAct[0] == NULL)||(pQLMDInfo->padwLocalAct[1] == NULL))
	{
		printf("[QLMD]Error allocating local activity memory\n");
		if(pQLMDInfo->padwLocalAct[0])
			free(pQLMDInfo->padwLocalAct[0]);
		if(pQLMDInfo->padwLocalAct[1])
			free(pQLMDInfo->padwLocalAct[1]);
		free(pQLMDInfo);
		*ppQLMDInfo = NULL;
		return -1;
	}

    pQLMDInfo->dwIdx = 0;
    pQLMDInfo->dwDiffIdx = 0;
	pQLMDInfo->bAvaliable = QP_FALSE;
	pQLMDInfo->mpMDFifo = -1;

	*ppQLMDInfo = pQLMDInfo;

	printf("[QLMD]Initial success\n");

	return 0;
}

/*************************************************************************/
int QLMD_Release(PTQLMDInfo pQLMDInfo)
{
    if(!pQLMDInfo)
    {
        return -1;
    }

    if (pQLMDInfo->padwLocalAct[0])
    {
        free(pQLMDInfo->padwLocalAct[0]);
        pQLMDInfo->padwLocalAct[0] = NULL;
    }

    if (pQLMDInfo->padwLocalAct[1])
    {
        free(pQLMDInfo->padwLocalAct[1]);
        pQLMDInfo->padwLocalAct[1] = NULL;
    }

	if(pQLMDInfo->mpMDFifo>0)
		close(pQLMDInfo->mpMDFifo);

    if (pQLMDInfo)
        free(pQLMDInfo);

    return 0;
}


/*************************************************************************/
int QLMD_Set(PTQLMDCfg pQLMDCfg, PTQP_CHAR pszItem, PTQP_CHAR pszValue1, PTQP_CHAR pszValue2, PTQP_CHAR pszValue3)
{
	int			i;
	char		index[3];

	if((!pQLMDCfg)||(!pszItem)||(!pszValue1)||(!pszValue2))
	{
		printf("[QLMD]Parameter error in %s\n", __FUNCTION__);
		return -1;
	}

	if (!strcasecmp(pszItem, "WINNUMS"))
	{
		pQLMDCfg->uiWinNumH = atoi(pszValue1);
		pQLMDCfg->uiWinNumV = atoi(pszValue2);
		if((!pQLMDCfg->uiWinNumH)||(!pQLMDCfg->uiWinNumV))
		{
			printf("[QLMD]Get wrong win num config H:%d V:%d\n", pQLMDCfg->uiWinNumH,pQLMDCfg->uiWinNumV);
			return -1;
		}
	}
	else if (!strcasecmp(pszItem, "INTERVAL"))
	{
		pQLMDCfg->uiInterval = atoi(pszValue1);
	}
	else if (!strncasecmp(pszItem, "WIN",3))
	{
		memset(index,0,sizeof(index));
		strcpy(index,pszItem+strlen("WIN"));
		i = atoi(index)-1;
		pQLMDCfg->mTrigger[i].uiThreshold = atoi(pszValue1);
		pQLMDCfg->mTrigger[i].uiPercentage = atoi(pszValue2);
		pQLMDCfg->mTrigger[i].uiEnable = atoi(pszValue3);
	}


    return 0;
}

/*************************************************************************/
int QLMD_Config(PTQLMDInfo pQLMDInfo, PTQP_CHAR pCfgFile)
{
	FILE*			fp = 0;
	TQLMDCfg		mdCfg;
	TQP_BOOL		change=QP_FALSE;
	char			*currlinebuff = NULL;
	char			*argv[5];
	int				i;
	int 			lErr = 0;
	ssize_t         currbytesread = 0;
	size_t          currlinesize = 0;

	if((!pQLMDInfo)||(!pCfgFile))
	{
		printf("[QLMD]Parameter error in %s\n", __FUNCTION__);
		return -1;
	}

	fp = fopen(pCfgFile, "rb");
	if (fp == NULL)
	{
		printf("[QLMD]Unable to open file %s\n", pCfgFile);
		return -1;
	}
	currlinebuff = malloc(256);
	if (currlinebuff == NULL)
	{
		printf("[QLMD]Unable malloc mem in %s\n", __FUNCTION__);
		return -1;
	}
	while (1)
	{
		currbytesread = getline(&currlinebuff, &currlinesize, fp);
		if (currbytesread > 0)
		{
			argv[0] = skip_whitespace(currlinebuff);
			if (!argv[0])
				continue;
			argv[1] = skip_to_whitespace(argv[0]);
			if (!argv[1])
				continue;
			*argv[1]++ = 0;

			argv[1] = skip_whitespace(argv[1]);
			if (!argv[1])
				continue;
			argv[2] = skip_to_whitespace(argv[1]);
			if (!argv[2])
				continue;
			*argv[2]++ = 0;

			argv[2] = skip_whitespace(argv[2]);
			if (!argv[2])
				continue;
			argv[3] = skip_to_whitespace(argv[2]);
			if (!argv[3])
				continue;
			*argv[3]++ = 0;

			argv[3] = skip_whitespace(argv[3]);
			if (!argv[3])
				continue;
			argv[4] = skip_to_whitespace(argv[3]);
			if (!argv[4])
				continue;
			*argv[4]++ = 0;

			lErr = QLMD_Set(&mdCfg, argv[0], argv[1], argv[2], argv[3]);
		}
		else
		{
			break;
		}
	}

	if(lErr<0)
		goto ERR;

	if((mdCfg.uiWinNumH)&&(mdCfg.uiWinNumV))
	{
		pQLMDInfo->dwWinWidth = pQLMDInfo->dwMBWidth/mdCfg.uiWinNumH;
		pQLMDInfo->dwWinHeight = pQLMDInfo->dwMBHeight/mdCfg.uiWinNumV;
		pQLMDInfo->dwWinNum = mdCfg.uiWinNumH*mdCfg.uiWinNumV;
		printf("[QLMD]Config: win num x:%d y:%d\n",mdCfg.uiWinNumH,mdCfg.uiWinNumV);
	}

	pQLMDInfo->dwInterval = mdCfg.uiInterval?mdCfg.uiInterval:1;
	printf("[QLMD]Config: interval:%d\n",pQLMDInfo->dwInterval);

	for (i=0; i<pQLMDInfo->dwWinNum; i++)
	{
		pQLMDInfo->mTrigger[i].uiPercentage = mdCfg.mTrigger[i].uiPercentage;
		pQLMDInfo->mTrigger[i].uiThreshold = mdCfg.mTrigger[i].uiThreshold;
		pQLMDInfo->mTrigger[i].uiEnable = mdCfg.mTrigger[i].uiEnable;
		printf("[QLMD]Config: WIN[%02d](%4d)(%2d)(%2d)\n",i+1,pQLMDInfo->mTrigger[i].uiThreshold, pQLMDInfo->mTrigger[i].uiPercentage, pQLMDInfo->mTrigger[i].uiEnable);
	}

	for (i=0; i<pQLMDInfo->dwWinNum; i++)
	{
		//num of pixels
		pQLMDInfo->adwWinSize[i]= pQLMDInfo->dwWinWidth*pQLMDInfo->dwWinHeight;
	}

ERR:
	free(currlinebuff);

	fclose(fp);

    return 0;
}


/*************************************************************************/
int QLMD_Calibrate(PTQLMDInfo pQLMDInfo)
{
	int				x,y,pos,winid;
	int				startx,starty;
	int				numH, numV;
	TQP_UINT32		diff;
	PTQP_UINT8		pAct1, pAct2;
	TQP_UINT32		act1, act2;

	if((!pQLMDInfo)||(!pQLMDInfo->bAvaliable))
		return -1;

	pAct1 = (PTQP_UINT8)((pQLMDInfo->padwLocalAct)[0]);
	pAct2 = (PTQP_UINT8)((pQLMDInfo->padwLocalAct)[1]);

	numH = pQLMDInfo->dwMBWidth/pQLMDInfo->dwWinWidth;
	numV = pQLMDInfo->dwMBHeight/pQLMDInfo->dwWinHeight;

	for(winid=0;winid<pQLMDInfo->dwWinNum;winid++)
	{
		startx = (winid%numH)*pQLMDInfo->dwWinWidth;
		starty = (winid/numH)*pQLMDInfo->dwWinHeight;

		pQLMDInfo->dwFilter[winid] = 0;
		diff = 0;

		for(x=startx;x<startx+pQLMDInfo->dwWinWidth;x++)
		{
			for(y=starty;y<starty+pQLMDInfo->dwWinHeight;y++)
			{
				pos = ((y*pQLMDInfo->dwMBWidth)+x)*2;

				pAct1 = (PTQP_UINT8)((pQLMDInfo->padwLocalAct)[0]);
				pAct2 = (PTQP_UINT8)((pQLMDInfo->padwLocalAct)[1]);

				act1 = (pAct1[pos]<<8)|pAct1[pos+1];
				act2 = (pAct2[pos]<<8)|pAct2[pos+1];

				diff += (act1>act2)?(act1-act2):(act2-act1);
			}
		}
		pQLMDInfo->dwFilter[winid] = diff/(pQLMDInfo->dwWinWidth*pQLMDInfo->dwWinHeight);
		printf("[QLMD]Get win[%02d] average diff:0x%04x\n",winid+1, pQLMDInfo->dwFilter[winid]);
	}


    return 0;
}


/*************************************************************************/
int QLMD_GetResult(PTQLMDInfo pQLMDInfo, PTQP_CHAR pLocalActivity, TQP_UINT32 uiLoActLen)
{
	int				x,y,pos,winid;
	int				startx,starty;
	int				numH, numV;
	TQP_UINT32		uiBytesCount;
	PTQP_UINT8		pCurAct,pPreAct;
	TQP_INT32		uiCurAct,uiPreAct;
	TQP_INT32		uiDiff,uiRealDiff;
	TQP_UINT32		uiPercent,uiAvgDiff;
	TQP_INT32		uiActFlag = 0;
	TQP_BOOL		bFlag=0;
	static int		iActCount = 0;
	int				fd = -1;
	time_t 			now;
    char			tmpbuf[12]={0};

	if((!pQLMDInfo)||(!pLocalActivity))
	{
		printf("[QLMD]Parameter error in %s\n", __FUNCTION__);
		return -1;
	}

	if(++iActCount<pQLMDInfo->dwInterval)
	{
		return 0;
	}
	iActCount = 0;

	uiBytesCount = (pQLMDInfo->dwBitCount+7)>>3;

	if(uiLoActLen>pQLMDInfo->dwMBWidth*pQLMDInfo->dwMBHeight*uiBytesCount)
	{
		printf("[QLMD]Local activity data is too long\n");
		return -1;
	}

	memcpy(pQLMDInfo->padwLocalAct[pQLMDInfo->dwIdx], pLocalActivity, uiLoActLen);
	numH = pQLMDInfo->dwMBWidth/pQLMDInfo->dwWinWidth;
	numV = pQLMDInfo->dwMBHeight/pQLMDInfo->dwWinHeight;

//	printf("[QLMD]%d  %d   %d  %d  %d %d \n", uiLoActLen, pQLMDInfo->dwMBWidth, pQLMDInfo->dwMBHeight, pQLMDInfo->dwBitCount, numH, numV);

	memset(pQLMDInfo->adwResult, 0, MD_MAX_WIN_NUM*sizeof(TQP_UINT32));
	memset(pQLMDInfo->adwTotalDiff, 0, MD_MAX_WIN_NUM*sizeof(TQP_UINT32));

	if(pQLMDInfo->bAvaliable)
	{
		if(!pQLMDInfo->dwFilter[0])
		{
			QLMD_Calibrate(pQLMDInfo);
		}

		for(winid=0;winid<pQLMDInfo->dwWinNum;winid++)
		{
			if(pQLMDInfo->mTrigger[winid].uiEnable)
			{
				startx = (winid%numH)*pQLMDInfo->dwWinWidth;
				starty = (winid/numH)*pQLMDInfo->dwWinHeight;
//				printf("[QLMD]win:%d,startx:%d,starty:%d\n",winid+1,startx,starty);
				for(x=startx;x<startx+pQLMDInfo->dwWinWidth;x++)
				{
					for(y=starty;y<starty+pQLMDInfo->dwWinHeight;y++)
					{
						pos = ((y*pQLMDInfo->dwMBWidth)+x)*2;

						pCurAct = (PTQP_UINT8)((pQLMDInfo->padwLocalAct)[pQLMDInfo->dwIdx]);
						pPreAct = (PTQP_UINT8)((pQLMDInfo->padwLocalAct)[1-pQLMDInfo->dwIdx]);

						uiCurAct = (pCurAct[pos]<<8)|pCurAct[pos+1];
						uiPreAct = (pPreAct[pos]<<8)|pPreAct[pos+1];

						uiDiff = (uiCurAct>uiPreAct)?(uiCurAct-uiPreAct):(uiPreAct-uiCurAct);
						uiRealDiff = (uiDiff>pQLMDInfo->dwFilter[winid])?(uiDiff-pQLMDInfo->dwFilter[winid]):0;
						pQLMDInfo->adwResult[winid]+=(uiRealDiff>=pQLMDInfo->mTrigger[winid].uiThreshold);
						pQLMDInfo->adwTotalDiff[winid]+=uiRealDiff;
					}
				}
			}
		}

  		time (&now);

		for(winid=0;winid<pQLMDInfo->dwWinNum;winid++)
		{
			if(pQLMDInfo->mTrigger[winid].uiEnable)
			{
				uiPercent = pQLMDInfo->adwResult[winid]*100/pQLMDInfo->adwWinSize[winid];
				uiAvgDiff = pQLMDInfo->adwTotalDiff[winid]/pQLMDInfo->adwWinSize[winid];

//				printf("[QLMD]Win[%2d]'s result: %4d size:%4d percent:%2d\n",winid+1,pQLMDInfo->adwResult[winid],pQLMDInfo->adwWinSize[winid],uiPercent);
				if(uiPercent>=pQLMDInfo->mTrigger[winid].uiPercentage)
				{
					if(!bFlag)
					{
						printf("\n[QLMD]: get motion detect at %s", asctime(localtime(&now)));
						bFlag = QP_TRUE;
					}
					uiActFlag |= (1<<winid);
					printf("[QLMD]Win[%2d] get available activity %4d, per: %3d, avg diff: %4d\n",winid+1,pQLMDInfo->adwResult[winid],uiPercent,uiAvgDiff);
				}
			}
		}

		if(pQLMDInfo->mpMDFifo<0)
		{
		 	pQLMDInfo->mpMDFifo = open(MD_FIFO, O_WRONLY|O_NONBLOCK, 0777);
			if (pQLMDInfo->mpMDFifo < 0)
			{
//				printf("[QLMD]Unable to open fifo %s, errno:%x\n", MD_FIFO, errno);
			}
		}
		else if(uiActFlag)
		{
//			printf("[QLMD]0x%x\n", uiActFlag);
//			sprintf(tmpbuf, "0x%08x\0",uiActFlag);
			write(pQLMDInfo->mpMDFifo, &uiActFlag, sizeof(int));
		}
	}
	else
	{
		pQLMDInfo->bAvaliable = QP_TRUE;
	}
	pQLMDInfo->dwIdx = 1-pQLMDInfo->dwIdx;

    return 0;
}
