/*************************************************************************/
/**      **/
/**            **/
/**                **/
/**              **/
/**                                                                     **/
/**                               **/
/**                                                  **/
/*************************************************************************/

#ifndef __QLMD_H__
#define __QLMD_H__

#include "qptypes.h"


/*! @defgroup QLMD Motion Detection Object
 *  @{
 */


/*!
 * Motion detection maximum window number.
 */

#define MD_MAX_WIN_NUM		16


typedef struct
{
	TQP_UINT16				uiThreshold;      	//MD trigger Sentivity 
	TQP_UINT8 				uiPercentage;   	//MD trigger percentage
	TQP_UINT8 				uiEnable;   		//MD trigger enable/disable 
} TQLMDTrigger, *PTQLMDTrigger;

typedef struct
{
	TQP_UINT8 				uiWinNumH; 
	TQP_UINT8 				uiWinNumV; 
	TQP_UINT8 				uiInterval; 
	TQP_UINT8 				reserverd1; 
	TQLMDTrigger			mTrigger[MD_MAX_WIN_NUM];  		//Tiggers per MB, dwWinNums specify real size
} TQLMDCfg, *PTQLMDCfg;

typedef struct
{
	TQP_BOOL 				bAvaliable;
	TQP_UINT32				dwBitCount;
	TQP_UINT32 				dwSize;              //Real size of pbyConf
	TQP_UINT32 				dwMBWidth;			//MD width
	TQP_UINT32 				dwMBHeight;			//MD height 
	TQP_UINT32 				dwWinWidth;          
	TQP_UINT32 				dwWinHeight;         
	TQP_UINT32				dwWinNum;
	TQP_UINT32				dwInterval;
	TQP_UINT32 				dwIdx;				
	TQP_UINT32 				dwDiffIdx;				
	TQP_UINT32				dwFilter[MD_MAX_WIN_NUM];
	PTQP_UINT32				padwLocalAct[2];   	//Local activity per MB
	TQP_UINT32 				adwResult[MD_MAX_WIN_NUM];
	TQP_UINT32 				adwTotalDiff[MD_MAX_WIN_NUM];
	TQLMDTrigger			mTrigger[MD_MAX_WIN_NUM];  		//Tiggers per MB, dwWinNum specify real size
	TQP_UINT32 				adwWinSize[MD_MAX_WIN_NUM];
	int						mpMDFifo;
} TQLMDInfo, *PTQLMDInfo;




/*!
 *******************************************************************************
 * \brief
 * Create a \b QLMD instance and initialize it.
 *
 **************************************************************************** */
int QLMD_Initial(component_t* pComponent, PTQLMDInfo* ppQLMDInfo);

/*!
 *******************************************************************************
 * \brief
 * Delete an instance of the \b QLMD object.
 *
 * \see QLMD_Initial
 *
 **************************************************************************** */
int QLMD_Release(PTQLMDInfo pQLMDInfo);



int QLMD_Set(PTQLMDCfg pQLMDCfg, PTQP_CHAR pszItem, PTQP_CHAR pszValue1, PTQP_CHAR pszValue2, PTQP_CHAR pszValue3);
int QLMD_Config(PTQLMDInfo pQLMDInfo, PTQP_CHAR pCfgFile);
int QLMD_Calibrate(PTQLMDInfo pQLMDInfo);
int QLMD_GetResult(PTQLMDInfo pQLMDInfo, PTQP_CHAR pLocalActivity, TQP_UINT32 uiLoActLen);


/** \example testmd.c
 * This is an example of how to use the QLMD library.
 */

/*! @} */

#endif	// __QLMD_H__

