/*************************************************************************/
/**      **/
/**            **/
/**                **/
/**              **/
/**                                                                     **/
/**                               **/
/**                                                  **/
/*************************************************************************/

/* User APIs for VO-OSD
**
*/
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <time.h>

#include "pldevioctl.h"

#ifdef	SUPPORT_PL3XX_OSD
#include "osdstruct.h"
#include "qlosd_clut.c"
#include "qlosdfunc.h"
#include "cour_16.h"
#include "atari_8x16.h"
#ifdef SUPPORT_VO_OSD
static unsigned int ghDev;
int OSD_SetDevice(unsigned int hDev)
{
	ghDev = hDev;
	return 0;
}

int OSD_SetPixelFormat(unsigned int format)
{
	int	Err = 0;
	QLDEV_OSD_PARAMETER param;

	param.type = PARAM_SET_PIXEL_FORMAT;
	param.value= format;
	Err = ioctl(ghDev, PLDEV_DIAG_IOCTL_OSD_CONFIG, &param);

	return Err;
}

//get the format with bits perpixel
int OSD_GetPixelFormat(unsigned int *bpp)
{
	int	Err = 0;
	QLDEV_OSD_PARAMETER param;

	param.type = PARAM_GET_PIXEL_FORMAT;
	Err = ioctl(ghDev, PLDEV_DIAG_IOCTL_OSD_CONFIG, &param);

	switch (param.value){
		case PIXEL_LUT_1BIT:
			*bpp = 1;
			break;
		case PIXEL_LUT_2BIT:
			*bpp = 2;
			break;
		case PIXEL_LUT_4BIT:
			*bpp = 4;
			break;
		case PIXEL_LUT_8BIT:
			*bpp = 8;
			break;
		case PIXEL_RGB_32BIT:
		case PIXEL_YUV_32BIT:
			*bpp = 32;
			break;
		default:
			*bpp = 16;
			break;
	}
	return Err;
}

int OSD_SetResolution(unsigned int res)
{
	int	Err = 0;
	QLDEV_OSD_PARAMETER param;

	param.type = PARAM_SET_RESOLUTION;
	param.value= res;

	Err = ioctl(ghDev, PLDEV_DIAG_IOCTL_OSD_CONFIG, &param);

	return Err;
}

//get resolution with with and height
int OSD_GetResolution(int *w, int *h)
{
	int	Err = 0;
	QLDEV_OSD_PARAMETER param;

	param.type = PARAM_GET_RESOLUTION;
	Err = ioctl(ghDev, PLDEV_DIAG_IOCTL_OSD_CONFIG, &param);
	switch(param.value){
		case MASK_RES_1280X720P:
			*w = 1280;
			*h = 720;
			printf("Resolution: 1280x720p\n");
			break;

		case MASK_RES_720X480P:
			*w = 720;
			*h = 480;
			printf("Resolution: 720X480P\n");
			break;

		case MASK_RES_720X576P:
			*w = 720;
			*h = 576;
			printf("Resolution: 720X576P\n");
			break;
		default:
			*w = 0;
			*h = 0;
			printf("Resolution: value = 0x%x\n", param.value);
			break;
	}
	return Err;
}

int OSD_SetCurrent(unsigned int BufferID)
{
	int Err = 0;
	QLDEV_OSD_CONTROL 	ctrl;

	ctrl.CmdType = PARAM_SET_CURRENTOSD;
	ctrl.BufferID = BufferID;
	Err = ioctl(ghDev, PLDEV_DIAG_IOCTL_OSD_CONTROL, &ctrl);

	return Err;
}

int OSD_GetCurrent(unsigned int *id)
{
	int Err = 0;
	QLDEV_OSD_CONTROL 	ctrl;

	ctrl.CmdType = PARAM_GET_CURRENTOSD;
	Err = ioctl(ghDev, PLDEV_DIAG_IOCTL_OSD_CONTROL, &ctrl);
	*id = ctrl.BufferID;

	return Err;
}

//user=0 default, else from ouside pdata,
//swap = 0 for 0xRGBA, 1 for 0xABGR
int OSD_SetCLUT(int user, unsigned int *pdata, int swap)
{
	int Err = 0;
	unsigned int bpp;
	QLDEV_OSD_CONTROL 	ctrl;
	unsigned int *ptable;

	//verify pixel format
	OSD_GetPixelFormat(&bpp);

	//select color lookup table
	switch (bpp){
		case 1:
			if(!user)
				ptable = (unsigned int *) &CLUT_DATA_1BIT[0];
			else
				ptable = pdata;
			break;
		case 2:
			if(!user)
				ptable = (unsigned int *) &CLUT_DATA_2BIT[0];
			else
				ptable = pdata;
			break;
		case 4:
			if(!user)
				ptable = (unsigned int *) &CLUT_DATA_4BIT[0];
			else
				ptable = pdata;
			break;
		case 8:
			if(!user)
				ptable = (unsigned int *) &CLUT_DATA_8BIT[0];
			else
				ptable = pdata;
			break;

		default:
			printf("No need to set CLUT !\n");
			return -1;
	}

	//set to driver
	ctrl.CmdType = PARAM_SET_CLUT;
	ctrl.pColorData = ptable;
	ctrl.swap = swap;
	Err = ioctl(ghDev, PLDEV_DIAG_IOCTL_OSD_CONTROL, &ctrl);
	return Err;
}

//read clut from regs
int OSD_GetCLUT(unsigned int *pdata, int swap)
{
	int Err = 0;
	QLDEV_OSD_CONTROL 	ctrl;

	//set to driver
	ctrl.CmdType = PARAM_GET_CLUT;
	ctrl.pColorData = pdata;
	ctrl.swap = swap;
	Err = ioctl(ghDev, PLDEV_DIAG_IOCTL_OSD_CONTROL, &ctrl);
	return Err;
}

int OSD_SetHorizontalX2(int enable)
{
	int	Err = 0;
	QLDEV_OSD_PARAMETER param;

	param.type = PARAM_SET_HORIZONTALX2;
	param.value = (enable) ? 1 : 0 ;
	Err = ioctl(ghDev, PLDEV_DIAG_IOCTL_OSD_CONFIG, &param);

	return Err;
}

int OSD_SetVerticalX2(int enable)
{
	int	Err = 0;
	QLDEV_OSD_PARAMETER param;

	param.type = PARAM_SET_VERTICALX2;
	param.value = (enable) ? 1 : 0 ;
	Err = ioctl(ghDev, PLDEV_DIAG_IOCTL_OSD_CONFIG, &param);

	return Err;
}

int OSD_SetGlobalBlending(int enable, int blending_factor )
{
	int	Err = 0;
	QLDEV_OSD_PARAMETER param;

	param.type = PARAM_SET_BLENDING_VALUE;
	param.value = blending_factor;
	Err = ioctl(ghDev, PLDEV_DIAG_IOCTL_OSD_CONFIG, &param);

	param.type = PARAM_SET_BLENDING;
	param.value = ( enable ) ? 1 : 0 ;
	Err = ioctl(ghDev, PLDEV_DIAG_IOCTL_OSD_CONFIG, &param);

	return Err;
}

int OSD_Enable(void)
{
	int	Err = 0;
	QLDEV_OSD_PARAMETER param;

	param.type = PARAM_SET_ENABLE;
	param.value = 1;
	Err = ioctl(ghDev, PLDEV_DIAG_IOCTL_OSD_CONFIG, &param);
	return Err;
}

int OSD_Disable(void)
{
	int	Err = 0;
	QLDEV_OSD_PARAMETER param;

	param.type = PARAM_SET_DISABLE;
	param.value = 1;

	Err = ioctl(ghDev, PLDEV_DIAG_IOCTL_OSD_CONFIG, &param);

	return Err;
}

//allocate an available frame buffer
 int OSD_AllocateMemory(unsigned int *id, unsigned int  ClearBuff)
{
	int Err = 0;
	QLDEV_OSD_CONTROL 	ctrl;

	ctrl.CmdType = PARAM_MEM_ALLOCATE;
	ctrl.bUpdateTransparency = ClearBuff;
	ctrl.Transparency = 0xff;
	Err = ioctl(ghDev, PLDEV_DIAG_IOCTL_OSD_CONTROL, &ctrl);

	if (ctrl.RetCode != 0){
		printf("%s error %d\n", __FUNCTION__, ctrl.RetCode);
		return -1;
	}
	*id = ctrl.BufferID;

	return Err;
}

int OSD_ReleaseMemory(unsigned int BufferID)
{
	int Err = 0;
	QLDEV_OSD_CONTROL 	ctrl;

	ctrl.CmdType = PARAM_MEM_RELEASE;
	ctrl.BufferID = BufferID;
	Err = ioctl(ghDev, PLDEV_DIAG_IOCTL_OSD_CONTROL, &ctrl);

	return Err;
}

int OSD_Copy( unsigned int oriBufferID, int x, int y, int width, int height, unsigned int *pbuf)
{
	int Err = 0;
	QLDEV_OSD_CONTROL 	ctrl;

	ctrl.CmdType = PARAM_COPY_BUFFER;
	ctrl.BufferID = oriBufferID;
	ctrl.x = x;
	ctrl.y = y;
	ctrl.Width = width;
	ctrl.Height = height;
	ctrl.pColorData = pbuf;
	ctrl.swap = 0;

	Err = ioctl(ghDev, PLDEV_DIAG_IOCTL_OSD_CONTROL, &ctrl);

	return Err;
}

int OSD_Paste( unsigned int destBufferID, int x, int y, int width, int height, unsigned int *pColorData, int swap)
{
	int Err = 0;
	QLDEV_OSD_CONTROL 	ctrl;

	ctrl.CmdType = PARAM_PASTE_BUFFER;
	ctrl.BufferID = destBufferID;
	ctrl.x = x;
	ctrl.y = y;
	ctrl.Width = width;
	ctrl.Height = height;
	ctrl.pColorData = pColorData;
	ctrl.swap = swap;

	Err = ioctl(ghDev, PLDEV_DIAG_IOCTL_OSD_CONTROL, &ctrl);

	return Err;
}

int OSD_DrawDot(unsigned int bf_id, int x, int y, COLOR_DATA color)
{
	int Err = 0;
	QLDEV_OSD_CONTROL 	ctrl;

	ctrl.CmdType = PARAM_DRAW_DOT;
	ctrl.BufferID = bf_id;
	ctrl.x = x;
	ctrl.y = y;
	ctrl.pColorData = (unsigned int *) &color;
	ctrl.FgColor = color.Read;
	Err = ioctl(ghDev, PLDEV_DIAG_IOCTL_OSD_CONTROL, &ctrl);

	return Err;
}

int OSD_DrawRectangle(unsigned int bf_id, int x, int y, int width, int height, COLOR_DATA color)
{
	int Err = 0;
	QLDEV_OSD_CONTROL 	ctrl;

	ctrl.CmdType = PARAM_DRAW_RECT;
	ctrl.BufferID = bf_id;
	ctrl.x = x;
	ctrl.y = y;
	ctrl.Width = width;
	ctrl.Height = height;
	ctrl.pColorData = (unsigned int *) &color;
	Err = ioctl(ghDev, PLDEV_DIAG_IOCTL_OSD_CONTROL, &ctrl);

	return Err;
}

int OSD_DrawText( unsigned int bf_id, char *pString, int x,	int y, unsigned int fgColor, unsigned int bgColor, OSD_FONT_INFO *pFontInfo, int bTransparentBg )
{
	int Err = 0;
	QLDEV_OSD_CONTROL 	ctrl;
	int i = 0;

	ctrl.CmdType = PARAM_DRAW_TEXT;
	ctrl.BufferID = bf_id;
	ctrl.Width = pFontInfo->Width;
	ctrl.Height = pFontInfo->Height;

	ctrl.BgColor = bgColor;
	ctrl.FgColor = fgColor;
	ctrl.ByteWidth = pFontInfo->ByteWidth;
	ctrl.bUpdateTransparency = bTransparentBg;
	ctrl.Transparency = 0;
	ctrl.y = y;
	ctrl.swap = 1;

	while ( *pString != 0 ){
		ctrl.x = x + i * pFontInfo->Width;
		ctrl.pFontData = pFontInfo->pStartAddr + (*pString) * pFontInfo->ByteWidth * pFontInfo->Height;
		Err = ioctl(ghDev, PLDEV_DIAG_IOCTL_OSD_CONTROL, &ctrl);
		i++;
		pString++;
	}

	return Err;
}

int OSD_DrawImage(unsigned int bf_id, pOSD_IMAGE_INFO pImageInfo, int x, int y, int bUpdateTransparency, int transparency, int bDMA )
{
	int Err = 0;
	QLDEV_OSD_CONTROL 	ctrl;
	int i;

	ctrl.CmdType = PARAM_DRAW_IMAGE;
	ctrl.BufferID = bf_id;
	ctrl.x = x;
	ctrl.y = y;
	ctrl.bUpdateTransparency = bUpdateTransparency;
	ctrl.Transparency = transparency;
	ctrl.Width = pImageInfo->Width;
	ctrl.Height = 1; /* height */
	//ctrl.pColorData = ( unsigned int * ) pColorData;
	ctrl.RetCode = bDMA;
	for (i = 0; i < pImageInfo->Height ; i++ )
	{
		ctrl.y = y + i;
		ctrl.pColorData = (unsigned int *) pImageInfo->pColorData[i]; //???? maybe error
		//ctrl.pColorData = pImageInfo->pColorData; // original
		Err = ioctl(ghDev, PLDEV_DIAG_IOCTL_OSD_CONTROL, &ctrl);
	}

  	return Err;
}


int OSD_LoadFont( char *pFontFile, OSD_FONT_INFO *pFontInfo)
{
	int Err = 0;
	FILE *fSrc;
	int iHeight, iWidth, iRealWidth, iTemp,iVal;
	int iByteWidth;
	unsigned char* pFont;
	unsigned char *p;
	char aLine[256];
	int i, j, k, m, n;

	fSrc = fopen(pFontFile, "r" );
	if (fSrc == NULL){
		printf("Fail to open file %s \n", pFontFile );
		return -1;
	}

	while (1)	{
		if (fgets(aLine, 255, fSrc) == NULL){
			printf("EOF!\n");
			break;
		}
		if (sscanf(aLine, "height %d", &iHeight) == 1)break;
	}

	while (1)	{
		if (fgets(aLine, 255, fSrc) == NULL){
			printf("EOF!\n");
			break;
		}
		if (sscanf(aLine, "width %d", &iWidth) == 1){
			if ( iWidth ) break;
		}
	}

	iByteWidth = (iWidth + 7) / 8;
	printf("w=%d, h=%d, byte-w=%d \n",iWidth, iHeight, iByteWidth);

	fseek(fSrc, 0, SEEK_SET );
	pFont = malloc( iByteWidth*iHeight*256 );
	printf("pFont = 0x%8p in LoadFontTable.\n", pFont);

	p = pFont;
	i=0;
	while (1)
	{
		if (fgets(aLine, 255, fSrc) == NULL) {
			printf("EOF!\n");
			break;
		}
		if (sscanf(aLine, "width %d", &iRealWidth) != 1)	continue;
		//read data from the end width n

		i++;
		if ( iRealWidth == 0 ){
			memset( p, 0, iByteWidth*iHeight );
			p += iByteWidth*iHeight;
			continue;
		}

		for(j = 0; j < iHeight ; j++){
			if (fgets(aLine, 255, fSrc) == NULL) {
				printf("EOF!\n");
				break;
			}

			k = iRealWidth;
			m = 0;
			while (m < iRealWidth ){
				iTemp = 0;
				for ( n = 8 ; n > 0 ; n-- ){
					iVal = ( aLine[m] == '0' ) ? 0 : 1;
					iTemp += iVal <<(n-1);
					m++;
					if (m == iRealWidth) break;
				}
				*p = (unsigned char) iTemp;
				p++;
			}
		}
	}

	pFontInfo->Height = iHeight;
	pFontInfo->Width = iWidth;
	pFontInfo->ByteWidth = iByteWidth;
	pFontInfo->pStartAddr = pFont;
	fclose(fSrc);
	return 0;
}

int OSD_LoadBMP( char *pImageFile, int transparency, pOSD_IMAGE_INFO pImageInfo)
{
	int ret = -1;
	unsigned char buff[4];
	FILE *fd;
	unsigned int ulCurrRead;
	unsigned int ulTotalRead;
	unsigned int ulOffset;
	unsigned int ulWidth;
	unsigned int ulHeight;
	unsigned int ulDepth;
	unsigned int ulVal;
	unsigned int ulBMPWidth;
	unsigned char *p;
	unsigned int ulQLWidth;
	int i,j,k,iTemp;
	size_t readsize;

	int iPixelPerCycle, iBMPPaddingFactor, iFetchCount, bPadding = 0;

	fd = fopen( pImageFile, "rb" );
	//fd_tmp = fopen( "temp.bin", "wb");

	  if( fd == NULL ) {
	  	printf("Unable to open file %s \n", pImageFile);
	    return ret;
	  }

	//parse header;
	fseek( fd, 10, SEEK_SET );
	readsize = fread( (unsigned char*)buff, 1, 4, fd);
	ulOffset = buff[0] + 256 * buff[1] + 256 * 256 * buff[2] + 256 * 256 * 256 * buff[3];
	fseek( fd, 18, SEEK_SET );
	readsize = fread( buff, 1, 4, fd);
	ulWidth = buff[0] + 256 * buff[1] + 256 * 256 * buff[2] + 256 * 256 * 256 * buff[3];
	fseek( fd, 22, SEEK_SET );
	readsize = fread( buff, 1, 4, fd);
	ulHeight = buff[0] + 256 * buff[1] + 256 * 256 * buff[2] + 256 * 256 * 256 * buff[3];
	fseek( fd, 28, SEEK_SET );
	readsize = fread( buff, 1, 4, fd);
	ulDepth = buff[0] + 256 * buff[1] + 256 * 256 * buff[2] + 256 * 256 * 256 * buff[3];

	// ulDepth <= 8 means it has color look up table...the color information begin at 54 (0x36)
	if (ulDepth<=8)
	{
		fseek(	fd,	54,	SEEK_SET );
		iTemp = (0x01)<<ulDepth;

		p = malloc(sizeof( unsigned int ) * iTemp);
		//pClutInfo->pStartAddr = p;

		for ( i = 0; i < iTemp ; i++ )
		{
			readsize = fread( (unsigned char *) buff, 1, 4,fd );
			if ( i == 0 )
				*p = 0xff;
			else
				*p = (unsigned char) transparency;
			p++;
			*p = buff[0]; p++;
			*p = buff[1]; p++;
			*p = buff[2];	p++;

		}
	}

	//fseek( fd, ulOffset, SEEK_SET );
	pImageInfo->Height = ulHeight;
	pImageInfo->Width = ulWidth;
	pImageInfo->Depth = ulDepth;
	pImageInfo->ColorFormat = ulDepth; //RGB format..not useful yet;
	ret = (0x01)<<ulDepth;

	switch ( ulDepth ) {
		case 1:
		case 2:
		case 4:
		case 8:
			//QL201 will read DRAM data by 8 byte. case 1: 8 (pixels/Byte) * 8 (Byte) = 64 pixels
			//									   case 2: 4 (pixels/Byte) * 8 (Byte) = 32 pixels
			//									   case 4: 2 (pixels/Byte) * 8 (Byte) = 16 pixels


			iPixelPerCycle = 8 / ulDepth * 8;
			iBMPPaddingFactor = 8 / ulDepth * 4;

			if ( ulWidth % iBMPPaddingFactor )
			{
				printf("the width should be the order of %i. Padding will be added.\n", iPixelPerCycle);
				bPadding = 1;
			}

			if ( bPadding )
			{
				ulBMPWidth = ( ulWidth/iBMPPaddingFactor + 1 ) * iBMPPaddingFactor;
			}
			else
			{
				ulBMPWidth = ulWidth;
			}

			if ( ulBMPWidth % iPixelPerCycle )
				iFetchCount = ulBMPWidth/iPixelPerCycle + 1 ;
			else
				iFetchCount = ulBMPWidth/iPixelPerCycle ;

			iTemp = ulHeight*ulBMPWidth/iBMPPaddingFactor*4;

			ulOffset += iTemp;

			ulQLWidth = (ulWidth%(64/ulDepth)) ? (ulWidth/(64/ulDepth)+1)*2 : (ulWidth/(64/ulDepth))*2;

			for( i = 0 ; i < ulHeight ; i++ )
			{
 				iTemp = ulOffset - (i+1) * ulBMPWidth/iBMPPaddingFactor*4 ;
				fseek( fd, iTemp, SEEK_SET );

				p = malloc(sizeof( unsigned int ) * ulQLWidth );
				pImageInfo->pColorData[i] = (unsigned long) p;
				//pImageInfo->pColorData+i = (unsigned int *) p;
				//endian transformation
				for ( j = 0; j < iFetchCount*2 ; j++ )
				{
					if ( feof( fd ) )
					{
						*p = 0xff; p++;
						*p = 0xff; p++;
						*p = 0xff; p++;
						*p = 0xff; p++;

					}
					else
					{
						readsize = fread( (unsigned char *) buff, 1, 4,fd );
						//endian transform for QL202
						*p = buff[3]; p++;
						*p = buff[2]; p++;
						*p = buff[1]; p++;
						*p = buff[0];	 p++;

					}
				}

			}
			break;

		case 24:
			ulQLWidth = (ulWidth & 0x00000001) ? ulWidth+1 : ulWidth;
			//in BMP file format, the byte of per line should be multiple of 4 bytes
			//therefore, the source data of BMP has unnecessary data need to be handled.
			//we should skip these data at the end of each line.
			// iBMPWidth is the real width (unit: bytes/line)
			ulBMPWidth =  ( (3*ulWidth) % 4  ) ? (3*ulWidth/4 + 1) * 4 : 3*ulWidth ;

			printf("Analysis %s offset=0x%x, w=0x%x, h=0x%x, d=0x%x, QLw=0x%x\n", pImageFile, ulOffset, ulWidth, ulHeight, ulDepth, ulQLWidth);

			ulOffset += ulHeight*ulBMPWidth;
			for( i = 0 ; i < ulHeight ; i++ )
			{
				p = malloc(sizeof( unsigned int ) * ulQLWidth ); //ywc add.20070502
				pImageInfo->pColorData[i] = (unsigned long) p;
				//pImageInfo->pColorData+i = (unsigned int *) p;
				//printf("allocate (%d) @ 0x%08x \n", i, p);

				ulVal = ulOffset - (i+1)*ulBMPWidth;
				fseek( fd, ulVal, SEEK_SET );

				for ( j = 0 ; j < ulWidth ; j++ )
				{
					//transparency - R-G-B
					*p = (unsigned char) transparency;	p++;
					readsize = fread( p, 1, 3, fd);	p+=3;
				}
				//add padding data because DRAM is 64-bit boundary.
				if ( ulWidth & 0x00000001 )	p+=4;
				//save to a bin
				//fwrite( pImageInfo->pColorData[i], sizeof( unsigned int ), ulQLWidth, fd_tmp );

				if ( i >= MAX_DISPLAY_LINE ) break;

			}
			printf("load BMP finished.\n");
			break;
		default:
			printf("Unknown format of Image file...\n");
			break;

	}
	fclose( fd );

	return ret;
}


int OSD_SetHiLight(unsigned int BufferID, int x, int y, int w, int h, int color)
{
	int Err = 0;
	QLDEV_OSD_CONTROL 	ctrl;

	ctrl.BufferID	= BufferID;
	ctrl.x = x;
	ctrl.y = y;
	ctrl.Width		= w;
	ctrl.Height	= h;
	ctrl.pColorData = &color;

	Err = ioctl(ghDev, PLDEV_DIAG_IOCTL_OSD_CONTROL, &ctrl);
	return Err;
}

int OSD_Test(unsigned int hdev, int func, int x, int y, int w, int h, unsigned int *pdata)
{
	int Err = 0;
	unsigned int bf2, bf_id = 0;
	int width, height;
	FILE *stream = NULL;
	FILE	*fclut = NULL;
	unsigned int *pbuf, *pcol;
	OSD_FONT_INFO font_inf;
	COLOR_DATA color = *((COLOR_DATA *)pdata);
	int i, need_swap = 1;
	size_t readsize;

	//step1
	Err = OSD_SetDevice(hdev);
	Err = Err? Err: OSD_Disable();
		if (Err)printf(">%s step 1 ret = %i\n", __FUNCTION__, Err);
	//step2
	Err = Err? Err: OSD_SetPixelFormat(PIXEL_LUT_8BIT);
	if (func != 4) Err = Err? Err: OSD_SetCLUT(0, NULL, 0);
		if (Err)printf(">%s step 2 ret = %i\n", __FUNCTION__, Err);
	//step3
	Err = Err? Err: OSD_SetResolution(MASK_RES_1280X720P);
	Err = Err? Err: OSD_GetResolution(&width, &height);
		if (Err)printf(">%s step 3 ret = %i\n", __FUNCTION__, Err);
	//no scaler
	Err = Err? Err: OSD_SetHorizontalX2(0);
	Err = Err? Err: OSD_SetVerticalX2(1);	//?
		if (Err)printf(">%s step 32 ret = %i\n", __FUNCTION__, Err);

	//step4
	if (OSD_AllocateMemory(&bf_id, (func == 3)? 0:1) == 0){
		Err = Err? Err: OSD_SetCurrent(bf_id);
	}else{
		Err = -1;
		printf(">%s step 4 ret = %i\n", __FUNCTION__, Err);
		return Err;
	}

	Err = Err? Err: OSD_SetGlobalBlending(1, 0x80);

	//step 5
	switch (func){
		case 0:		//rectangle
			Err = Err? Err: OSD_DrawRectangle(bf_id, x, y, w, h, color);
			//Err = Err? Err: OSD_SetHiLight(bf_id, x, y, w-100, h-50, 0x56202020);

			printf(">%s show rect ret = %i\n", __FUNCTION__, Err);
			break;

		case 1:		//show image only for 8bpp
			//load clut
			fclut = fopen("./sample.act", "r");
			if (fclut != NULL){
				pcol = malloc(4*256);
				memset(pcol, 0, 4*256);
				#if 1	//photoshop 3Bytes for RGB
					for (i = 0; i < 256; i++){
						readsize = fread(((unsigned char *)pcol + 4 * i), 1, 3, fclut);
					}
					Err = Err? Err: OSD_SetCLUT(1, pcol, 1);
				#else
					readsize = fread((unsigned char *)pcol, 1, 4*256, fclut);
					Err = Err? Err: OSD_SetCLUT(1, pcol, 0);
				#endif
				free(pcol);
				fclose(fclut);
			}

			//load image
			stream = fopen("./sample.raw", "r");
			if (stream == NULL){
				stream = fopen("./osd_sample_1280x720.bin", "r");
				need_swap = 0;
			}

			if (stream != NULL){
				pbuf = malloc(1280*720);
				readsize = fread((unsigned char *)pbuf, 1, 1280*720, stream);

				Err = Err? Err: OSD_Paste(bf_id, 0, 0, 1280, 720, pbuf, need_swap);
				free(pbuf);
				fclose(stream);
			}
			printf(">%s show image ret = %i\n", __FUNCTION__, Err);
			break;

		case 2:		//show text
			OSD_LoadFont("./courf02.fd", &font_inf);
			OSD_DrawText(bf_id, "Hello, Welcome to SDL! ", x, y, width, height, &font_inf, 0);
			//OSD_DrawText(bf_id, "QL300 text testing", x, y+50, width, height, &font_inf, 1);
			printf(">%s show text ret = %i\n", __FUNCTION__, Err);
			break;

		case 3:
			//copy buffer from fb1 to fb0 and show
			pbuf = malloc(1280*720);
			Err = Err? Err: OSD_Copy(bf_id, x, y, w, h, pbuf);
			if (OSD_AllocateMemory(&bf2, 1) == 0){
				Err = Err? Err: OSD_SetCurrent(bf2);
				Err = Err? Err: OSD_Paste(bf2, x, y, w, h, pbuf, 0);
				OSD_ReleaseMemory(bf2);
			}
			free(pbuf);
			printf(">%s show fb1 copy from bf0 ret = %i\n", __FUNCTION__, Err);
			break;

		case 4:
			fclut = fopen("/var/sample.act", "w+");
			if (fclut != NULL){
				pcol = malloc(4*256);
				Err = Err? Err: OSD_GetCLUT(pcol, 1);
				#if 1	//photoshop only 3bytes for RGB
					for (i = 0; i < 256; i++){
						fwrite(((unsigned char *)pcol + 4 * i), 1, 3, fclut);
					}
				#else
					fwrite((unsigned char *)pcol, 1, 4*256, fclut);
				#endif
				fclose(fclut);
				free(pcol);
				printf(">%s get clut and save to /var/sample.act ret = %i\n", __FUNCTION__, Err);
			}

		default:
			break;

	}
	//step 6
	Err = Err? Err: OSD_Enable();
	if (Err) printf(">%s step 6 ret = %i\n", __FUNCTION__, Err);
	//step 7
	OSD_ReleaseMemory(bf_id);
	if (Err) printf(">%s step 7 ret = %i\n", __FUNCTION__, Err);

	return Err;
}
#endif

#ifdef SUPPORT_VI_OSD
unsigned int QLVIOSD_Init(unsigned int hdev, QPCODEC_VIOSD *pQLVIOSD)
{
	TQLTime *tTime = &(pQLVIOSD->Timestamp);
	int lErr = 0;

	pQLVIOSD->CmdType = PARAM_VIOSD_ON;
	pQLVIOSD->tIdx = 1;
	lErr = ioctl(hdev, PLDEV_DIAG_IOCTL_VIOSD_CONTROL, pQLVIOSD);
	if(pQLVIOSD->tIdx == 0)
	{
		memset(pQLVIOSD, 0, sizeof(QPCODEC_VIOSD));
		pQLVIOSD->TextList.aText = calloc(1, sizeof(TQLText)*MAX_TEXT_NUM);
		pQLVIOSD->FontList.aFont = calloc(1, sizeof(TQLFont)*MAX_FONT_NUM);
		pQLVIOSD->Text.pbyText = calloc(1, sizeof(char)*32);
		tTime->cDateSep = '/';
		tTime->cTimeSep = ':';
		tTime->cMSSep = ' ';
		tTime->bYearEn = 1;
		tTime->bMonthEn = 1;
		tTime->bDayEn = 1;
		tTime->bHourEn = 1;
		tTime->bMinEn = 1;
		tTime->bSecEn = 1;
		tTime->bMSEn = 0;	// 1;
		tTime->dwYearFmt = 0; //0: YYYY, 1:'YY, 2:YY
		tTime->dwMonthFmt = 0;
		tTime->dwHourFmt = 0; //0: 00~23, 1: 00~12 with AM/PM
		return 1;
	}
	return 0;
}

unsigned int QLVIOSD_Start(unsigned int hdev, QPCODEC_VIOSD *pQLVIOSD)
{
	TQLFont tFont;
	TQLText tText;
	unsigned int temp;
	char *string;
	char tChar[32];
	TQLText *p = &(pQLVIOSD->Text);
	int lErr = 0;

	lErr = ioctl(hdev, PLDEV_CNTL_IOCTL_GET_VIOSD_TABLE_ADDR, pQLVIOSD);
	if (!lErr)
	{
		printf("VI-OSD font_addr(0x%x) txtAddr(0x%x) timeAddr(0x%x)\n", pQLVIOSD->FontTableAddr,pQLVIOSD->TextListAddr,pQLVIOSD->TimestampAddr);
		pQLVIOSD->CmdType = PARAM_VIOSD_INIT;
		lErr = ioctl(hdev, PLDEV_DIAG_IOCTL_VIOSD_CONTROL, pQLVIOSD);
		if (lErr < 0)
		{
			printf("%s() ->VI-OSD Init Error\n",__FUNCTION__);
			return 0;
		}
		lErr = QLVIOSD_Clearbuf(hdev, pQLVIOSD);
		if (lErr < 0)
		{
			printf("%s() ->Error QLVIOSD_Reset!\n",__FUNCTION__);
			return 0;
		}
		tFont.pbyFont = a2byFont_8x16;
		tFont.dwType = 1;
		tFont.dwLen = 95;
		tFont.dwAsciiOffset = 32;
		lErr = QLVIOSD_AddFont(hdev, pQLVIOSD, &tFont);
		if (lErr < 0)
		{
			printf("%s() ->Error adding 8x16 font!\n",__FUNCTION__);
			return 0;
		}

		tFont.pbyFont = a2byFont_16x16;
		tFont.dwType = 3;
		tFont.dwLen = 127;
		tFont.dwAsciiOffset = 32;
		lErr = QLVIOSD_AddFont(hdev, pQLVIOSD, &tFont);
		if (lErr < 0)
		{
			printf("%s() ->Error adding 16x16 font!\n",__FUNCTION__);
			return 0;
		}

		tText.bEn = 0;
		tText.dwFontID = 0;
		tText.dwX = 200;
		tText.dwY = 40;
		tText.pbyText = tChar;
		string = "Sigma";
		tText.dwLen = strlen(string);
		strcpy(tText.pbyText, string);
		temp = (tText.dwFontID<2) ? 3 : 4;
		tText.dwX = tText.dwX>>temp;
		temp = (tText.dwFontID&0x01) ? 4: 3;
		tText.dwY = tText.dwY>>temp;
		lErr = QLVIOSD_AddText(hdev, pQLVIOSD, &tText);
		if (lErr < 0)
		{
			printf("Error adding user text!\n");
			return 0;
		}

		lErr = QLVIOSD_GetInfo(hdev, pQLVIOSD);
		if (lErr < 0)
		{
			printf("Error QLVIOSD_GetInfo!\n");
			return 0;
		}

		lErr = QLVIOSD_AddText(hdev, pQLVIOSD, p);
		if (lErr < 0)
		{
			printf("Error adding user text!\n");
			return 0;
		}
		return 1;
	}
	else if(pQLVIOSD->GetAddrTimeout++>=20)
	{
		//we only try 20 times to avoid drop encode data
		printf("Viosd get address time out\n");
		return 0xff;
	}
	return 0;
}

void QLVIOSD_Service(unsigned int hdev, QPCODEC_VIOSD *pQLVIOSD)
{
	int lErr = 0;
	TQLText *p = &(pQLVIOSD->Text);
	lErr = QLVIOSD_GetInfo(hdev, pQLVIOSD);
	if(!lErr)
	{
		if(pQLVIOSD->Update == 1)
		{
			printf("viosdinfo.Update\n");
			pQLVIOSD->Update = 0;
			lErr = QLVIOSD_SetText(hdev, pQLVIOSD, p);
		}
	}
	lErr = QLVIOSD_TimeUpdate(hdev, pQLVIOSD);
}

void QLVIOSD_Stop(unsigned int hdev, QPCODEC_VIOSD *pQLVIOSD)
{
	QLDEV_OSD_PARAMETER param;
	int lErr = 0;

	pQLVIOSD->CmdType = PARAM_VIOSD_OFF;
	lErr = ioctl(hdev, PLDEV_DIAG_IOCTL_VIOSD_CONTROL, pQLVIOSD);
	if(pQLVIOSD->TextList.aText)
		free(pQLVIOSD->TextList.aText);
	if(pQLVIOSD->FontList.aFont)
		free(pQLVIOSD->FontList.aFont);
	if(pQLVIOSD->Text.pbyText)
		free(pQLVIOSD->Text.pbyText);
}

unsigned int QLVIOSD_Clearbuf(unsigned int hdev, QPCODEC_VIOSD *pQLVIOSD)
{
	int lErr = 0;
    pQLVIOSD->CmdType = PARAM_VIOSD_CLEAR;
    lErr = ioctl(hdev, PLDEV_DIAG_IOCTL_VIOSD_CONTROL, pQLVIOSD);
	return lErr;
}

unsigned int QLVIOSD_GetInfo(unsigned int hdev, QPCODEC_VIOSD *pQLVIOSD)
{
	int lErr = 0;
	pQLVIOSD->CmdType = PARAM_VIOSD_INFO;
    lErr = ioctl(hdev, PLDEV_DIAG_IOCTL_VIOSD_CONTROL, pQLVIOSD);
    return lErr;
}

/*************************************************************************/
unsigned int QLVIOSD_AddFont(unsigned int hdev, QPCODEC_VIOSD *pQLVIOSD, TQLFont *ptFont)
{
    unsigned int dwIdx = 0;
	int lErr = 0;
    TQLFont *ptDevFont;

    while (pQLVIOSD->FontList.aFont[dwIdx].dwLen != 0)
    {
        dwIdx++;
        if (dwIdx == MAX_FONT_NUM)
        {
            printf("Error! Font full!\n");
            return (-1);
        }
    }
    ptDevFont = &(pQLVIOSD->FontList.aFont[dwIdx]);
    ptDevFont->pbyFont = ptFont->pbyFont;
    ptDevFont->dwType = ptFont->dwType;
    ptDevFont->dwLen = ptFont->dwLen;
    ptDevFont->dwID = dwIdx;
    ptDevFont->dwAsciiOffset = ptFont->dwAsciiOffset;

    pQLVIOSD->CmdType = PARAM_VIOSD_ADDFONT;
    pQLVIOSD->tIdx = dwIdx;
    lErr = ioctl(hdev, PLDEV_DIAG_IOCTL_VIOSD_CONTROL, pQLVIOSD);
    if(lErr)
        return lErr;
    ptFont->dwID = pQLVIOSD->FontList.aFont[dwIdx].dwID;

    lErr = ioctl(hdev, PLDEV_CNTL_IOCTL_SET_ENC_OSD_SETTINGS, &pQLVIOSD->Enable);

    return lErr;
}

/*************************************************************************/
unsigned int QLVIOSD_AddText(unsigned int hdev, QPCODEC_VIOSD *pQLVIOSD, TQLText *ptText)
{
    unsigned int dwIdx = 0;
    TQLText *ptDevText;
    unsigned int dwOffset;
    int i;
    int lErr = 0;

    while (pQLVIOSD->TextList.aText[dwIdx].dwLen != 0)
    {
        dwIdx++;
        if (dwIdx == MAX_TEXT_NUM)
        {
            return (-1);
        }
    }
    dwOffset = pQLVIOSD->FontList.aFont[ptText->dwFontID].dwAsciiOffset;
    for (i=0; i<ptText->dwLen; i++)
    {
        ptText->pbyText[i] = ptText->pbyText[i] - dwOffset;
    }

    ptDevText = &(pQLVIOSD->TextList.aText[dwIdx]);
    ptDevText->dwX = ptText->dwX;
    ptDevText->dwY = ptText->dwY;
    ptDevText->pbyText = ptText->pbyText;
    ptDevText->dwFontID = ptText->dwFontID;
    ptDevText->dwLen = ptText->dwLen;
    ptDevText->bEn = ptText->bEn;
    ptDevText->dwID = dwIdx;

    pQLVIOSD->CmdType = PARAM_VIOSD_ADDTEXT;
    pQLVIOSD->tIdx = dwIdx;
    lErr = ioctl(hdev, PLDEV_DIAG_IOCTL_VIOSD_CONTROL, pQLVIOSD);
    if(lErr)
        return lErr;
    ptText->dwID = pQLVIOSD->TextList.aText[dwIdx].dwID;

    lErr = ioctl(hdev, PLDEV_CNTL_IOCTL_SET_ENC_OSD_SETTINGS, &pQLVIOSD->Enable);

    return lErr;
}


/*************************************************************************/
unsigned int QLVIOSD_SetText(unsigned int hdev, QPCODEC_VIOSD *pQLVIOSD, TQLText *ptText)
{
    TQLText *ptDevText;
    unsigned int dwOffset;
    int i;
    int lErr = 0;

    dwOffset = pQLVIOSD->FontList.aFont[ptText->dwFontID].dwAsciiOffset;
    for (i=0; i<ptText->dwLen; i++)
    {
        ptText->pbyText[i] = ptText->pbyText[i] - dwOffset;
    }
    pQLVIOSD->TextList.dwIndex = ptText->dwID;
    ptDevText = &(pQLVIOSD->TextList.aText[ptText->dwID]);
    ptDevText->pbyText = ptText->pbyText;
    ptDevText->dwX = ptText->dwX;
    ptDevText->dwY = ptText->dwY;
    ptDevText->bEn = ptText->bEn;
    ptDevText->dwFontID = ptText->dwFontID;
    ptDevText->dwLen = ptText->dwLen;
    ptDevText->dwID = ptText->dwID;

    pQLVIOSD->CmdType = PARAM_VIOSD_SETTEXT;
    pQLVIOSD->tIdx = ptText->dwID;
    lErr = ioctl(hdev, PLDEV_DIAG_IOCTL_VIOSD_CONTROL, pQLVIOSD);

    lErr = ioctl(hdev, PLDEV_CNTL_IOCTL_SET_ENC_OSD_SETTINGS, &pQLVIOSD->Enable);
    return lErr;
}

/*************************************************************************/
char a2byMon[12][3] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

unsigned int TimeToString(struct tm *ptm_local, TQLTime *ptTime, unsigned char *pbyTime, unsigned int dwOffset)
{
    unsigned int dwFmtLen = 0;
    int i;

    if (ptTime->bYearEn && (ptTime->dwYearFmt==0))
    {
        sprintf(pbyTime+dwFmtLen, "%04d", ptm_local->tm_year+1900);
        dwFmtLen += 4;
    }
    else if (ptTime->bYearEn && (ptTime->dwYearFmt==1))
    {
        sprintf(pbyTime+dwFmtLen, "'%02d", ptm_local->tm_year+1900);
        dwFmtLen += 3;
    }

    if (ptTime->bYearEn && ptTime->bMonthEn)
    {
        pbyTime[dwFmtLen] = ptTime->cDateSep;
        dwFmtLen += 1;
    }

    if (ptTime->bMonthEn && (ptTime->dwMonthFmt==0))
    {
        sprintf(pbyTime+dwFmtLen, "%02d", ptm_local->tm_mon+1);
        dwFmtLen += 2;
    }
    else if (ptTime->bMonthEn && (ptTime->dwMonthFmt==1))
    {
        sprintf(pbyTime+dwFmtLen, "%s", a2byMon[ptm_local->tm_mon]);
        dwFmtLen += 3;
    }

    if (ptTime->bMonthEn && ptTime->bDayEn)
    {
        pbyTime[dwFmtLen] = ptTime->cDateSep;
        dwFmtLen += 1;
    }

    if (ptTime->bDayEn)
    {
        sprintf(pbyTime+dwFmtLen, "%02d", ptm_local->tm_mday);
        dwFmtLen += 2;
    }

    if ((ptTime->bYearEn||ptTime->bMonthEn||ptTime->bDayEn) &&
        (ptTime->bHourEn||ptTime->bMinEn||ptTime->bSecEn))
    {
        pbyTime[dwFmtLen] = ' ';
        dwFmtLen += 1;
    }

    if (ptTime->bHourEn && (ptTime->dwHourFmt==0))
    {
        sprintf(pbyTime+dwFmtLen, "%02d", ptm_local->tm_hour);
        dwFmtLen += 2;
    }
    else if (ptTime->bHourEn && (ptTime->dwHourFmt==1))
    {
        if (ptm_local->tm_hour == 0)
        {
            sprintf(pbyTime+dwFmtLen, "AM 12");
        }
        else if (ptm_local->tm_hour < 12)
        {
            sprintf(pbyTime+dwFmtLen, "AM %02d", ptm_local->tm_hour);
        }
        else if (ptm_local->tm_hour == 12)
        {
            sprintf(pbyTime+dwFmtLen, "PM 12");
        }
        else
        {
            sprintf(pbyTime+dwFmtLen, "PM %02d", ptm_local->tm_hour - 12);
        }
        dwFmtLen += 5;
    }

    if (ptTime->bHourEn && ptTime->bMinEn)
    {
        pbyTime[dwFmtLen] = ptTime->cTimeSep;
        dwFmtLen += 1;
    }

    if (ptTime->bMinEn)
    {
        sprintf(pbyTime+dwFmtLen, "%02d", ptm_local->tm_min);
        dwFmtLen += 2;
    }

    if (ptTime->bMinEn && ptTime->bSecEn)
    {
        pbyTime[dwFmtLen] = ptTime->cTimeSep;
        dwFmtLen += 1;
    }
    //printf("TIME: %s, len=%d\n", pbyTime, dwFmtLen);
    for (i=0; i<dwFmtLen; i++)
    {
        pbyTime[i] = pbyTime[i] - dwOffset;
    }
    return dwFmtLen;
}

unsigned int QLVIOSD_TimeUpdate(unsigned int hdev, QPCODEC_VIOSD *pQLVIOSD)
{
	TQLTime *ptTime = &(pQLVIOSD->Timestamp);
	QPCODEC_VIOSD_CALTIME *ptCalTime = &(pQLVIOSD->CalTime);
    unsigned int 				dwIdx = 1;
    unsigned int 				dwLen;
    unsigned int 				dwTemp;
    TQLText 			*ptText;
    char 				abyTime[128];
    struct tm 			tm_local;
    time_t 				tmp_time;
    int lErr = 0;

viosd_time:
    memset(abyTime, 0, 128);
    if (ptCalTime->bTimeInit == 0)
    {
        //Initial current and next time
        time(&ptCalTime->curr_time);  // get sec
        gmtime_r(&ptCalTime->curr_time, &tm_local);
        tm_local.tm_sec = 0;
        ptCalTime->curr_time = mktime(&tm_local);
        localtime_r(&ptCalTime->curr_time, &tm_local);
        ptCalTime->bMSEn = ptTime->bMSEn;
        ptCalTime->bEnable = ptTime->bEn;
        ptCalTime->bTimerSec = (ptCalTime->curr_time+1)%60;
        ptCalTime->bTimerMilliSec = 0;
        ptCalTime->bSwitch = 0;
        //g_iTimerFunCount = 0;
        //gettimeofday(&old_tv, NULL);

        //Current
        ptText = &(pQLVIOSD->TextList.aText[MAX_TEXT_NUM-2]);
        ptText->dwID = MAX_TEXT_NUM-2;
        ptText->bEn = ptTime->bEn;
        ptCalTime->dwLen = ptText->dwLen = TimeToString(&tm_local, ptTime, abyTime, pQLVIOSD->FontList.aFont[ptTime->dwFontID].dwAsciiOffset);
        ptCalTime->dwX = ptText->dwX = ptTime->dwX;
        ptCalTime->dwY = ptText->dwY = ptTime->dwY;
        ptCalTime->dwFontID = ptText->dwFontID = ptTime->dwFontID;
        ptCalTime->pbyTime = ptText->pbyText = abyTime;
        ptCalTime->dwTime = ptCalTime->curr_time;
        ptCalTime->byMSSep = ptTime->cMSSep - pQLVIOSD->FontList.aFont[ptTime->dwFontID].dwAsciiOffset;

        pQLVIOSD->CmdType = PARAM_VIOSD_SETTIME;
        pQLVIOSD->tIdx = ptText->dwID;
        ptTime->dwTimeTextID = 0;
        lErr = ioctl(hdev, PLDEV_DIAG_IOCTL_VIOSD_CONTROL, pQLVIOSD);
        lErr = ioctl(hdev, PLDEV_CNTL_IOCTL_SET_ENC_OSD_SETTINGS, &pQLVIOSD->Enable);

        //Next
        //printf("pThis->next_time : %x,pThis->curr_time : %x\n",pThis->next_time,pThis->curr_time);
        ptCalTime->next_time = ptCalTime->curr_time + 60;
        localtime_r(&ptCalTime->next_time, &tm_local);
        ptText = &(pQLVIOSD->TextList.aText[MAX_TEXT_NUM-1]);
        ptText->dwID = MAX_TEXT_NUM-1;
        ptText->bEn = ptTime->bEn;
        ptCalTime->dwLen = ptText->dwLen = TimeToString(&tm_local, ptTime, abyTime, pQLVIOSD->FontList.aFont[ptTime->dwFontID].dwAsciiOffset);
        ptCalTime->dwX = ptText->dwX = ptTime->dwX;
        ptCalTime->dwY = ptText->dwY = ptTime->dwY;
        ptCalTime->dwFontID = ptText->dwFontID = ptTime->dwFontID;
        ptCalTime->pbyTime = ptText->pbyText = abyTime;
        ptCalTime->dwTime = ptCalTime->next_time;
        ptCalTime->byMSSep = ptTime->cMSSep - pQLVIOSD->FontList.aFont[ptTime->dwFontID].dwAsciiOffset;

        pQLVIOSD->CmdType = PARAM_VIOSD_SETTIME;
        pQLVIOSD->tIdx = ptText->dwID;
        ptTime->dwTimeTextID = 1;
        lErr = ioctl(hdev, PLDEV_DIAG_IOCTL_VIOSD_CONTROL, pQLVIOSD);
        ptCalTime->bTimeInit = 1;

        lErr = ioctl(hdev, PLDEV_CNTL_IOCTL_SET_ENC_OSD_SETTINGS, &pQLVIOSD->Enable);
        return lErr;
    }

    if ((ptCalTime->bEnable != ptTime->bEn) ||
		(ptCalTime->dwFontID != ptTime->dwFontID) ||
		(ptCalTime->dwX != ptTime->dwX) ||
        (ptCalTime->dwY != ptTime->dwY))
    {
        printf("qlviosd: Modify parameters ,change it\n");
        ptCalTime->bTimeInit = 0;
        goto viosd_time;
    }

    time(&tmp_time);
    dwTemp = tmp_time%60;
    if(ptCalTime->bTimerSec != dwTemp)
    {
        ptCalTime->bTimerSec = dwTemp;
        if(ptCalTime->bTimerSec == 0)
        {
            tmp_time += 60;
            gmtime_r(&tmp_time, &tm_local);
            tm_local.tm_sec = 0;
            tmp_time = mktime(&tm_local);
            ptTime->dwTimeTextID = ptCalTime->bSwitch;
            ptCalTime->bSwitch = 1 - ptCalTime->bSwitch;
            ptCalTime->bMSEn = ptTime->bMSEn;
            ptCalTime->curr_time = ptCalTime->next_time;
            ptCalTime->next_time = tmp_time;
            localtime_r(&ptCalTime->next_time, &tm_local);
            ptText = &(pQLVIOSD->TextList.aText[MAX_TEXT_NUM-1-ptCalTime->bSwitch]);
            ptText->dwID = MAX_TEXT_NUM-1-ptCalTime->bSwitch;
            ptCalTime->bEnable = ptText->bEn = ptTime->bEn;
            ptCalTime->dwLen = ptText->dwLen = TimeToString(&tm_local, ptTime, abyTime, pQLVIOSD->FontList.aFont[ptTime->dwFontID].dwAsciiOffset);
            ptCalTime->dwX = ptText->dwX = ptTime->dwX;
            ptCalTime->dwY = ptText->dwY = ptTime->dwY;
            ptCalTime->dwFontID = ptText->dwFontID = ptTime->dwFontID;
            ptCalTime->pbyTime = ptText->pbyText = abyTime;
            ptCalTime->dwTime = ptCalTime->next_time;
            ptCalTime->byMSSep = ptTime->cMSSep - pQLVIOSD->FontList.aFont[ptTime->dwFontID].dwAsciiOffset;

            pQLVIOSD->CmdType = PARAM_VIOSD_SETTIME;
            pQLVIOSD->tIdx = ptText->dwID;
            lErr = ioctl(hdev, PLDEV_DIAG_IOCTL_VIOSD_CONTROL, pQLVIOSD);
            lErr = ioctl(hdev, PLDEV_CNTL_IOCTL_SET_ENC_OSD_SETTINGS, &pQLVIOSD->Enable);
        }
        //gettimeofday(&new_tv, NULL);
        //pQLVIOSD->bTimerMilliSec += ((new_tv.tv_sec - old_tv.tv_sec) * 1000) + (new_tv.tv_usec/1000) - (old_tv.tv_usec/1000);
        //old_tv = new_tv;
        //if (pQLVIOSD->bTimerMilliSec >= 1000)
        //    pQLVIOSD->bTimerMilliSec -= 1000;
        pQLVIOSD->CmdType = PARAM_VIOSD_UPDATETIME;
        lErr = ioctl(hdev, PLDEV_DIAG_IOCTL_VIOSD_CONTROL, pQLVIOSD);
        return lErr;
    }
    return 0;
}

int VIOSD_SetConfig(unsigned int hdev, char *viCmd, char *viData)
{
	int	Err = 0;
	QLDEV_OSD_PARAMETER param;
	QLDEV_VIOSD_CONTROL control;

	param.value = strtoul(viData, 0, 16);
	if(!strcasecmp(viCmd, "STAMPEN"))
	{
		param.type = PARAM_VIOSD_STMEN;
		Err = ioctl(hdev, PLDEV_DIAG_IOCTL_OSD_CONFIG, &param);
	}
	else if(!strcasecmp(viCmd, "STAMPFT"))
	{
		param.type = PARAM_VIOSD_STMFT;
		Err = ioctl(hdev, PLDEV_DIAG_IOCTL_OSD_CONFIG, &param);
	}
	else if(!strcasecmp(viCmd, "STAMPXP"))
	{
		param.type = PARAM_VIOSD_STMXP;
		Err = ioctl(hdev, PLDEV_DIAG_IOCTL_OSD_CONFIG, &param);
	}
	else if(!strcasecmp(viCmd, "STAMPYP"))
	{
		param.type = PARAM_VIOSD_STMYP;
		Err = ioctl(hdev, PLDEV_DIAG_IOCTL_OSD_CONFIG, &param);
	}
	else if(!strcasecmp(viCmd, "STREN"))
	{
		param.type = PARAM_VIOSD_STREN;
		Err = ioctl(hdev, PLDEV_DIAG_IOCTL_OSD_CONFIG, &param);
	}
	else if(!strcasecmp(viCmd, "STRFT"))
	{
		param.type = PARAM_VIOSD_STRFT;
		Err = ioctl(hdev, PLDEV_DIAG_IOCTL_OSD_CONFIG, &param);
	}
	else if(!strcasecmp(viCmd, "STRXP"))
	{
		param.type = PARAM_VIOSD_STRXP;
		Err = ioctl(hdev, PLDEV_DIAG_IOCTL_OSD_CONFIG, &param);
	}
	else if(!strcasecmp(viCmd, "STRYP"))
	{
		param.type = PARAM_VIOSD_STRYP;
		Err = ioctl(hdev, PLDEV_DIAG_IOCTL_OSD_CONFIG, &param);
	}
	else if(!strcasecmp(viCmd, "STR"))
	{
		control.CmdType = PARAM_VIOSD_STR;
		control.pString = viData;
		Err = ioctl(hdev, PLDEV_DIAG_IOCTL_OSD_CONTROL, &control);
	}
	return Err;
}

#endif

#endif

