/*************************************************************************/
/**      **/
/**            **/
/**                **/
/**              **/
/**                                                                     **/
/**                               **/
/**                                                  **/
/*************************************************************************/

#include "qlautoconf.h"
#include <stdio.h>
#include <stdint.h>
#include <unistd.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <time.h>
#include <mntent.h>
#include <signal.h>
#include <asm/types.h>
#include <sys/types.h>
#include <errno.h>

#include <linux/videodev2.h>

#include "qptypes.h"
#include "pldevioctl.h"
#include "plv4ldevversion.h"
#include "qlprofile.h"

#ifdef ENABLE_APP_LINUX_PROFILING
static PLAPP_DIAG_PROFILING_STRUCT * g_profiling_app_log = NULL;
int g_profling_enable = 0;

//#define PROFILING_PRINT_OUT_MSG_ONLY

static TQP_UINT32 Local_getTimeMs(void) {
	struct timespec ts;
	uint64_t ret;
	clock_gettime(CLOCK_MONOTONIC,&ts);
	ret = (uint64_t)ts.tv_sec * 1000 + ts.tv_nsec/1000000;
	return (ret & 0xFFFFFFFF);
}

static TQP_UINT32 Local_getTimeMs2(void) {
	struct timeval tv;
	uint64_t ret;
	gettimeofday(&tv, NULL);
	ret = (uint64_t)tv.tv_sec * 1000 + tv.tv_usec/1000;
	return (ret & 0xFFFFFFFF);
}

#ifdef PROFILING_PRINT_OUT_MSG_ONLY
void pl330app_profiling_log_init(void)
{
	g_profling_enable = 1;
}

void pl330app_profiling_log_add(TQP_UINT32 dwEvent, TQP_UINT32 dwChannel, TQP_UINT32 dwType, TQP_UINT32 dwPram1, TQP_UINT32 dwFrameNo, TQP_UINT32 dwSize)
{
	fprintf(stderr, "PROFILING: E=0x%08x C=%04u type=%04u Param1=%04u FrameNo=%08u Size=%08u tim=%010d\n",
				  dwEvent, dwChannel, dwType, dwPram1, dwFrameNo, dwSize, Local_getTimeMs() & 0xFFFFFFFF
				 );
}

void pl330app_profiling_log_reset(void)
{
}

void pl330app_profiling_log_fini(void)
{
	g_profling_enable = 0;
}

void pl330app_profiling_log_dump(char *filename)
{
}
#else

void pl330app_profiling_log_time_sync(void)
{
	TQP_UINT32 t1, t2, t3, ta1, ta2, ta3, tok, taok;
	t1 = Local_getTimeMs();
	ta1 = Local_getTimeMs2();
	t2 = Local_getTimeMs();
	ta2 = Local_getTimeMs2();
	t3 = Local_getTimeMs();
	ta3 = Local_getTimeMs2();
	if (t1==t2) {
		tok = t1;
		taok = ta1;
    }
	else if (t2==t3) {
		tok = t2;
		taok = ta2;
	}
	else if (ta1==ta2) {
		tok = t2;
		taok = ta1;
	}
	else if (ta2==ta3) {
		tok = t3;
		taok = ta2;
	}
	else {
		tok = t1;
		taok = ta1;
	}
	pl330app_profiling_log_add(EVENT_APP_TIME_SYNC, 0, 0, tok, 0, taok);
}

void pl330app_profiling_log_init(void)
{
	if (g_profiling_app_log)
		return;
	g_profiling_app_log = malloc(sizeof(PLAPP_DIAG_PROFILING_STRUCT));
	if (g_profiling_app_log) {
		g_profling_enable = 1;
		memset(g_profiling_app_log, 0, sizeof(PLAPP_DIAG_PROFILING_STRUCT));
        memcpy(g_profiling_app_log->header,"APPPROF", 7);
		g_profiling_app_log->cur_profiling_idx = 0;
		pl330app_profiling_log_time_sync();
	}
}
void pl330app_profiling_log_add(TQP_UINT32 dwEvent, TQP_UINT32 dwChannel, TQP_UINT32 dwType, TQP_UINT32 dwPram1, TQP_UINT32 dwFrameNo, TQP_UINT32 dwSize)
{
    TQP_UINT32 index;
    if (g_profiling_app_log) {
        if (g_profiling_app_log->looped==1) {
            /* table full, just return */
            return;
        }
		index = g_profiling_app_log->cur_profiling_idx;
		g_profiling_app_log->cur_profiling_idx ++;
		if (g_profiling_app_log->cur_profiling_idx >= PL330APP_PROFILING_MAX_SIZE) {
			//g_profiling_app_log->cur_profiling_idx = 0;
			g_profiling_app_log->looped = 1;
		}

    	g_profiling_app_log->entries[index][0] = dwEvent;
	    g_profiling_app_log->entries[index][1] = ((dwChannel & 0xF) << 28) | ((dwType & 0xF) << 24) | (dwFrameNo &0x00FFFFFF);
	    g_profiling_app_log->entries[index][2] = dwSize;
	    g_profiling_app_log->entries[index][3] = dwPram1;
	    g_profiling_app_log->entries[index][4] = Local_getTimeMs();
	}
}

void pl330app_profiling_log_reset(void)
{
	if (g_profiling_app_log) {
		g_profiling_app_log->looped = 0;
		g_profiling_app_log->cur_profiling_idx = 0;
		pl330app_profiling_log_time_sync();
	}
}

void pl330app_profiling_log_fini(void)
{
	if (g_profiling_app_log) {
		free(g_profiling_app_log);
		g_profiling_app_log = NULL;
		g_profling_enable = 0;
	}
}

void pl330app_profiling_log_dump(char *filename)
{
    int lErr = 0;
	FILE *fp;
	fprintf(stderr,"======>Dump App log data to %s\n", filename);
	uint32_t curidx = 0;

    fp = fopen(filename,"wb");
    if (fp)
    {
        if (g_profiling_app_log) {
            curidx = g_profiling_app_log->cur_profiling_idx;
            fwrite(g_profiling_app_log, 1, sizeof(PLAPP_DIAG_PROFILING_STRUCT), fp);
            fclose(fp);
        }
    }
    else {
        fprintf(stderr, "======>can not open profiling dump file %s\n", filename);
        lErr = -1;
    }
    fprintf(stderr, "======>dump profiling data %d/%d entries, size= %d bytes, return %d\n", curidx, PL330APP_PROFILING_MAX_SIZE, sizeof(PLAPP_DIAG_PROFILING_STRUCT), lErr);
    return;
}

#endif

//#else
//#error "ENABLE_APP_LINUX_PROFILING not defined"
#endif

