/*************************************************************************/
/**      **/
/**            **/
/**                **/
/**              **/
/**                                                                     **/
/**                               **/
/**                                                  **/
/*************************************************************************/

#ifndef __QLSTRM_H__
#define __QLSTRM_H__

#include <stdint.h>
#include <pthread.h>

#include "qptypes.h"
#include "pldevioctl.h"
#include "plv4ldevversion.h"


typedef enum _media_type_e
{
    MEDIA_FILE_IN = 0,
    MEDIA_FILE_OUT,
    MEDIA_UDP_IN,
    MEDIA_UDP_OUT,
    MEDIA_TCP_IN,
    MEDIA_TCP_OUT,
    MEDIA_UVCRAW_IN,
    MEDIA_UVCMJPEG_IN,
    MEDIA_SDL_OUT,
    MEDIA_MP4FILE_OUT,
    MEDIA_HLS_FILE_OUT,
    MEDIA_NULL_OUT,
    MEDIA_DGRAM_IN,
    MEDIA_DGRAM_OUT,
    MEDIA_STREAM_IN,
    MEDIA_STREAM_OUT
} media_type_e;



typedef struct _stream_media_t
{
	TQP_BOOL			opened;

	COMPONENT_PORT_TYPE	portType;
	COMPONENT_PORT_OPEN_FORMAT	openFormat;
	int					memory;	// v4l2_memory

    pthread_t			threadId;
    unsigned int		ulThreadCreated;

    media_type_e		eMediaType;
    char				*pszNameOrAddr;
    media_type_e		eMediaType2;
    char				*pszNameOrAddr1; /* for client socket path */
    char				*pszNameOrAddr2;
    unsigned int		unPort;
	int					displayType;

    unsigned int		ulForceFullBuffer;

    unsigned int		unUvcFps;
    unsigned int		unUvcFrameRepeat;
    unsigned int		unUvcPreview;
    unsigned int		unUserPTS;
    unsigned int		unJPEGOnDemand;
    unsigned int		unJPEGSeparate;
    unsigned int		unFileLoop;
    unsigned int		unVIOSDOn;
    unsigned long		unUpLimit;
    unsigned int		unPackData;
	unsigned int        unSaveOriginal;
    volatile unsigned int	unCtrlCReceived;
    unsigned int		unStarted;
	unsigned int		unSentStopCmd;
	unsigned char       gViuAddressOK;
	unsigned char       gViuosdOn;
	unsigned char       gAudAgcOn;
} stream_media_t;


#define PS_BUF_SIZE 0x20000
#define AUDIO_BUF_SIZE 0x20000
#define VIDEO_BUF_SIZE 0x100000

typedef struct _ps_mux_t
{
	unsigned int temp;
	unsigned int aFrameSize[4];
	unsigned int aFrameCnt;
	unsigned int mp4State;
	unsigned int overLen;
	unsigned int nOffset;
	unsigned int nPESHeadLen;
	unsigned int nLength;
	unsigned int LenVideoFrame;
	unsigned int LenAudioFrame;
    unsigned int RdCnt;
    unsigned char *RdPtr;
    unsigned char *RdPtr0;
    unsigned int WrCnt;
    unsigned char *WrPtr;
    unsigned char *Startaddr;
    unsigned char *Endaddr;
    unsigned char *aBuff;
    unsigned char *aWrPtr;
    unsigned int aWrCnt;

    unsigned char *vBuff;
    unsigned char *vWrPtr;

    unsigned int nAudioCount;
    unsigned int nVideoCount;
    unsigned int rCnt;
	unsigned int nFirstStream;
	unsigned int pts_offset;
	unsigned int ptsa_done;
}ps_mux_t ;

typedef enum _stream_dir_e
{
	STREAM_DIR_IN = 0,
	STREAM_DIR_OUT,
	STREAM_DIR_END

} stream_dir_e;

#define MAX_STREAM_DIR	STREAM_DIR_END


typedef struct _component_t
{
    TQP_UINT32				devInstance;
	COMPONENT_TYPE			devType;

	TQP_UINT32				busType;

	stream_media_t			ports[MAX_STREAM_DIR];

    char					szDevName[32];
	DEVHANDLE				hDev;
	TQP_OBJECT_HANDLE		hTask;

	char					*pszScriptName;

	unsigned int			ulWidth;
	unsigned int			ulHeight;

	// encoder settings
	QPCODEC_SYSTEM_CONTROL	sysControl;
	QPCODEC_PICTURE_RESOLUTION	picResolution;
	QPCODEC_INPUT_CONTROL	inputControl;
	QPCODEC_RATE_CONTROL	rateControl;
	QPCODEC_VBR_BIT_RATE	vbrControl;
	QPCODEC_FILTER_CONTROL	fltrControl;
	QPCODEC_GOP_LOOP_FILTER	gopLoopFltr;
	QPCODEC_OUT_PICTURE_RESOLUTION	outResolution;
	QPCODEC_AUDIO_CONTROL_PARAM	audControl;
	QPCODEC_AUDIO_CONTROL_EX	audControlEx;
	TQP_UINT32					mjpegQuality;
	TQP_UINT32					mjpegBufNo;


	// decoder settings
	QPCODEC_DEC_STREAM_ID_EX	decStreamID;
	TQP_UINT32					decOutputMode;
	QPCODEC_PICTURE_RESOLUTION	decResolution;
	QPCODEC_DEC_STOP_MODE		decStopMode;
	QPCODEC_DEC_DISP_ATTRIB		decDispAttrib;
	QPCODEC_DEC_DISP_MODE		decDispMode;
	QPCODEC_DEC_AUDIO_CONTROL	decAudControl;

	// misc settngs
	QPCODEC_AOU_CONFIG			aouConfig;
	QPCODEC_OSD_MEM_CONFIG		osdMem;
	QPCODEC_DECIMATION_FACTOR	decimation;
	TQP_UINT32 					deinterlacemode;
	QPCODEC_ENC_CAP_MODE		enccapmode;
	QPCODEC_EXTERNAL_TRIGGER_MODE externaltriggermode;
	QPCODEC_PTS_COUNTER_RESET 	ptscounterreset;
    RATE_CONTROL_EX             rateControlEx;
	TQP_UINT32 					largecompressbuffercontrol;
	TQP_UINT32 					lowbitratemode;
	TQP_UINT32 					avdiscardcontrolmode;
	TQP_UINT32 					useswptsmode;

	// codec settings
	QPCODEC_SYS_FUNCTION	function;
	QPCODEC_SYS_LINK		link;

} component_t;



#endif //__QLSTRM_H__
