/*************************************************************************/
/**      **/
/**            **/
/**                **/
/**              **/
/**                                                                     **/
/**                               **/
/**                                                  **/
/*************************************************************************/

#include "qlautoconf.h"
#include <stdio.h>
#include <stdint.h>
#include <unistd.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <time.h>
#include <signal.h>
#ifndef __APPLE__
#include <mntent.h>
#include <asm/types.h>
#endif
#include <sys/types.h>
#include <errno.h>

#include "qptypes.h"
#include "pldevioctl.h"
#include "plv4ldevversion.h"

#ifndef CONFIG_HOST_PLATFORM_USERSPACE
#include <linux/videodev2.h>
#else
#include <uservideodev2.h>
#include "userpl330.h"
#define ioctl(dev, cmd, arg)  plv4l_ioctl(dev, cmd, arg)
#endif

#ifndef CONFIG_HOST_PLATFORM_USERSPACE
int lookup_video_device_node(TQP_UINT32 devInstance,
							 COMPONENT_TYPE devType,
							 TQP_UINT32 busType,
							 char *pszDeviceName
							)
{
	int			hDev;
	int			lErr;
	int			nCnt = 0;
	int         nCntMax = 64;
	PLDEV_INFO	devInfo;

	//printf("%s()-> Requesting %d:%d\n", __FUNCTION__, devInstance, devType);

	memset(&devInfo, 0xaa, sizeof(PLDEV_INFO));

#ifdef CONFIG_HOST_PLATFORM_ZYNQ
	nCnt = 3;  /* we already know that video0, video1, video2 are not pl330 devices */
	nCntMax = 10;
#endif

	while (nCnt < nCntMax) {
		sprintf(pszDeviceName,
#ifdef CONFIG_HOST_PLATFORM_ASPEN
				"/dev/pl330/video%d",
#else
				"/dev/video%d",
#endif
				nCnt
			   );

		hDev = open(pszDeviceName,
					O_RDWR
				   );

		if (hDev >= 0) {
			devInfo.busType = PLDRV_VERSION_CODE;   // used for api version checking
			devInfo.type = PLIOCTLAPI_VERSION_CODE; // used for api version checking

			lErr = ioctl(hDev,
						 PLDEV_STRM_IOCTL_DEVICE_INFO,
						 &devInfo
						);
			close(hDev);

			if (lErr == 0) {
				printf("%s()-> bus(%d) inst(%d) hTask(%ld) type(%d)\n",
					   __FUNCTION__,
					   devInfo.busType,
					   devInfo.devInstance,
					   devInfo.hTask,
					   devInfo.type
					  );

				if (QPHCI_BUS_TOTAL_BUSES != busType) {
					if (busType != devInfo.busType) {
						nCnt++;
						continue;
					}
				}

				if ((devInstance == devInfo.devInstance) &&
					(devType == devInfo.type )) {
                    printf("%s()-> Got %d:%d\n", __FUNCTION__, devInfo.devInstance, devInfo.type);
					// got it
					break;
				}
			} else {
				if (lErr == QPERR_CTRL_VERSION)
					printf("%s()-> %s ioctl PLDEV_STRM_IOCTL_DEVICE_INFO APP/Driver Version Not Matched (%d)\n", __FUNCTION__, pszDeviceName, lErr);
				else
					printf("%s()-> %s ioctl PLDEV_STRM_IOCTL_DEVICE_INFO failed(%d)\n", __FUNCTION__, pszDeviceName, lErr);
			}
		}

		nCnt++;
	}

	if (nCntMax == nCnt) {
		// return empty device name
		*pszDeviceName = 0;
		return (-1);
	} else {
		return (0);
	}
}
#endif /* CONFIG_HOST_PLATFORM_USERSPACE */

int pl_reg_get(DEVHANDLE dev, uint32_t address, uint32_t *value)
{
	int ret;
	QPCODEC_DIAG_REGISTER_STRUCT qlmemrw;

	if (!value) {
		return -1;
	}

	qlmemrw.address = address;
	qlmemrw.value = 0xDEADBEEF;

	ret = ioctl(dev, PLDEV_DIAG_IOCTL_REG_READ, &qlmemrw);

	if (!ret) {
        *value = qlmemrw.value;
	}

	return ret;
}

int pl_reg_set(DEVHANDLE dev, uint32_t address, uint32_t value)
{
	int ret;
	QPCODEC_DIAG_REGISTER_STRUCT qlmemrw;

	qlmemrw.address = address;
	qlmemrw.value = value;

	ret = ioctl(dev, PLDEV_DIAG_IOCTL_REG_WRITE, &qlmemrw);

	return ret;
}

int pl_mem_get(DEVHANDLE dev, uint32_t address, uint32_t dwordlen, uint32_t *value)
{
	int ret;
	QPCODEC_DIAG_MEMORY_STRUCT qlmemrw;

	if ((dwordlen > QP_MAX_MEMORY_DWORDS) || (!value)) {
		return -1;
	}

	qlmemrw.address = address;
	qlmemrw.num_dwords = dwordlen;

	ret = ioctl(dev, PLDEV_DIAG_IOCTL_MEM_READ, &qlmemrw);

	if (!ret) {
        memcpy((void *)value, (void *)&qlmemrw.value[0], qlmemrw.num_dwords << 2);
	}

	return ret;
}

int pl_mem_set(DEVHANDLE dev, uint32_t address, uint32_t dwordlen, uint32_t *value)
{
	int ret = 0;
	QPCODEC_DIAG_MEMORY_STRUCT qlmemrw;
	uint32_t i;

	if ((dwordlen > QP_MAX_MEMORY_DWORDS) || (!value)) {
		return -1;
	}

	qlmemrw.address = address;
	qlmemrw.num_dwords = dwordlen;

	for (i = 0; i < dwordlen; i++) {
		qlmemrw.value[i] = value[i];
	}

	if(dwordlen) {
		ret = ioctl(dev, PLDEV_DIAG_IOCTL_MEM_WRITE, &qlmemrw);
	}

	return ret;
}

int pl_i2c_get(DEVHANDLE dev, uint8_t address, uint8_t subaddr, uint8_t *value)
{
	int ret;
	QPCODEC_DIAG_I2C_WRITE_THEN_READ_STRUCT qli2crw;

	if (!value) {
		return -1;
	}

	memset(&qli2crw, 0, sizeof(QPCODEC_DIAG_I2C_WRITE_THEN_READ_STRUCT));

	qli2crw.chip_address = (TQP_UINT8)address;
	qli2crw.num_write_bytes = 1;
	qli2crw.write_bytes[0] = (TQP_UINT8)subaddr;
	qli2crw.num_read_bytes = 1;

	ret = ioctl(dev, PLDEV_DIAG_IOCTL_I2C_WRITE_THEN_READ, &qli2crw);

	if (!ret) {
        *value = qli2crw.read_bytes[0];
	}

	return ret;
}

int pl_i2c_set(DEVHANDLE dev, uint8_t address, uint8_t subaddr, uint8_t value)
{
	int ret;
	QPCODEC_DIAG_I2C_STRUCT qli2crw;

	memset(&qli2crw, 0, sizeof(QPCODEC_DIAG_I2C_STRUCT));

	qli2crw.chip_address = (TQP_UINT8) address;
	qli2crw.num_bytes = 2;
	qli2crw.bytes[0] = (TQP_UINT8) subaddr;
	qli2crw.bytes[1] = (TQP_UINT8) value;

	ret = ioctl(dev, PLDEV_DIAG_IOCTL_I2C_WRITE, &qli2crw);

	return ret;
}

int pl_i2cnb_get(DEVHANDLE dev, uint32_t bus, uint8_t address, uint32_t wrsize, uint8_t *wrvalue, uint32_t rdsize, uint8_t *rdvalue)
{
	int ret;
	uint32_t i;
	QPCODEC_DIAG_I2C_WRITE_THEN_READ_STRUCT qli2crw;

	if ((wrsize > QP_MAX_SUBADDRESS_BYTES) || (!wrvalue) || (!rdvalue)) {
		return -1;
	}

	memset(&qli2crw, 0, sizeof(QPCODEC_DIAG_I2C_WRITE_THEN_READ_STRUCT));

	qli2crw.chip_address = (TQP_UINT8) address;
	qli2crw.num_write_bytes = (TQP_UINT8) wrsize;

	for(i = 0; i < wrsize; i++) {
		qli2crw.write_bytes[i] = (TQP_UINT8) wrvalue[i];
	}

	qli2crw.num_read_bytes = rdsize;

	if (bus == 1) {
		ret = ioctl(dev, PLDEV_DIAG_IOCTL_I2C_WRITE_THEN_READ_EX, &qli2crw);
	} else {
		ret = ioctl(dev, PLDEV_DIAG_IOCTL_I2C_WRITE_THEN_READ, &qli2crw);
	}

	if(!ret) {
		for(i = 0; i < rdsize; i++) {
			rdvalue[i] = qli2crw.read_bytes[i];
		}
	}

	return ret;
}

int pl_i2cnb_set(DEVHANDLE dev, uint32_t bus, uint8_t address, uint32_t wrsize, uint8_t *wrvalue)
{
	int ret;
	uint32_t i;
	QPCODEC_DIAG_I2C_STRUCT qli2crw;

	if ((wrsize > QP_MAX_I2C_BYTES) || (!wrvalue)) {
		return -1;
	}

	memset(&qli2crw, 0, sizeof(QPCODEC_DIAG_I2C_STRUCT));

	qli2crw.chip_address = (TQP_UINT8) address;
	qli2crw.num_bytes = (TQP_UINT8) wrsize;

	for(i = 0; i < wrsize; i++) {
		qli2crw.bytes[i] = wrvalue[i];
	}

	if (bus == 1) {
		ret = ioctl(dev, PLDEV_DIAG_IOCTL_I2C_WRITE_EX, &qli2crw);
	} else {
		ret = ioctl(dev, PLDEV_DIAG_IOCTL_I2C_WRITE, &qli2crw);
	}

	return ret;
}

