/*************************************************************************/
/**      **/
/**            **/
/**                **/
/**              **/
/**                                                                     **/
/**                               **/
/**                                                  **/
/*************************************************************************/
#include <stdio.h>
#include <unistd.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <time.h>
#include <mntent.h>
#include <signal.h>
#include <asm/types.h>
#include <sys/types.h>
#include <errno.h>

#include "qptypes.h"
#include "pldevioctl.h"

#include "qlstrm.h"
#include "qlvagc.h"

static int QLVAGC_ReadReg(int handle, uint32_t reg, uint32_t *pvalue)
{
    QPCODEC_DIAG_REGISTER_STRUCT qlmemrw;

	if (handle > 0)
	{
		int err;

		qlmemrw.address = reg;
		qlmemrw.value = 0xDEADBEEF;

		err = ioctl(handle, PLDEV_DIAG_IOCTL_REG_READ, &qlmemrw);
		if (!err)
		{
			if (pvalue)
				*pvalue = (uint32_t) qlmemrw.value;
			return 0;
		}
	}
	return 1;
}

static int QLVAGC_WriteReg(int handle, uint32_t reg, uint32_t value)
{
    QPCODEC_DIAG_REGISTER_STRUCT qlmemrw;

	if (handle > 0)
	{
		int err;

		qlmemrw.address = reg;
		qlmemrw.value = value;

		err = ioctl(handle, PLDEV_DIAG_IOCTL_REG_WRITE, &qlmemrw);
		if (!err)
			return 0;
	}
	return 1;
}

static int QLVAGC_ReadMem(int handle, uint32_t addr, uint32_t *pvalue, int dwords)
{
    QPCODEC_DIAG_MEMORY_STRUCT qlmemrw;

	if (!pvalue || (dwords>=QP_MAX_MEMORY_DWORDS))
		return 1;

	if (handle > 0)
	{
		int err;

		qlmemrw.address = addr;
		qlmemrw.num_dwords = dwords;

		err = ioctl(handle, PLDEV_DIAG_IOCTL_MEM_READ, &qlmemrw);
		if (!err)
		{
			memcpy((void *)pvalue,(void *)&qlmemrw.value[0],dwords*4);
			return 0;
		}
	}
	return 1;
}

static int QLVAGC_WriteMem(int handle, uint32_t addr, uint32_t *pvalue, int dwords)
{
    QPCODEC_DIAG_MEMORY_STRUCT qlmemrw;
	int i;

	if (!pvalue || (dwords>=QP_MAX_MEMORY_DWORDS))
		return 1;

	if (handle > 0)
	{
		int err;

		qlmemrw.address = addr;
		qlmemrw.num_dwords = dwords;

		for (i = 0; i < dwords; i++)
		{
			qlmemrw.value[i] = pvalue[i];
		}

		err = ioctl(handle, PLDEV_DIAG_IOCTL_MEM_WRITE, &qlmemrw);
		if (!err)
			return 0;
	}
	return 1;
}

static int QLVAGC_ReadI2Cnb(int handle, int bus, uint8_t addr, uint8_t wrsize, uint8_t *pwrvalue, uint8_t rdsize, uint8_t *prdvalue)
{
    QPCODEC_DIAG_I2C_WRITE_THEN_READ_STRUCT qli2crw;

	if ((!pwrvalue) || (wrsize>=QP_MAX_SUBADDRESS_BYTES))
		return 1;


	if (handle > 0)
	{
	    int err;
		int i;

	    memset(&qli2crw, 0, sizeof(QPCODEC_DIAG_I2C_WRITE_THEN_READ_STRUCT));

		qli2crw.chip_address = (TQP_UINT8)addr;
		qli2crw.num_write_bytes = (TQP_UINT8)wrsize;
	    for(i=0;i<wrsize;i++)
	    {
	        qli2crw.write_bytes[i] = (TQP_UINT8)pwrvalue[i];
	    }
	    qli2crw.num_read_bytes = (TQP_UINT8)rdsize;

	    if (bus==1)
		    err = ioctl(handle, PLDEV_DIAG_IOCTL_I2C_WRITE_THEN_READ_EX, &qli2crw);
		else
		    err = ioctl(handle, PLDEV_DIAG_IOCTL_I2C_WRITE_THEN_READ, &qli2crw);

	    if(!err)
	    {
			if (prdvalue)
			{
				for(i=0;(i<rdsize)&&(i<QP_MAX_I2C_BYTES); i++)
				{
	    	    	prdvalue[i] = qli2crw.read_bytes[i];
	    		}
			}
			return 0;
	    }
	}
	return 1;
}

static int QLVAGC_WriteI2Cnb(int handle, int bus, uint8_t addr, uint8_t wrsize, uint8_t *pwrvalue)
{
    QPCODEC_DIAG_I2C_STRUCT qli2crw;

	if ((!pwrvalue) || (wrsize>=QP_MAX_I2C_BYTES))
		return 1;


	if (handle > 0)
	{
	    int err;
		int i;

	    memset(&qli2crw, 0, sizeof(QPCODEC_DIAG_I2C_STRUCT));

		qli2crw.chip_address = (TQP_UINT8)addr;
		qli2crw.num_bytes = (TQP_UINT8)wrsize;
		for(i=0;i<wrsize;i++)
		{
	        qli2crw.bytes[i] = (TQP_UINT8)pwrvalue[i];
		}
	    if (bus==1)
		    err = ioctl(handle, PLDEV_DIAG_IOCTL_I2C_WRITE_EX, &qli2crw);
		else
		    err = ioctl(handle, PLDEV_DIAG_IOCTL_I2C_WRITE, &qli2crw);

		if (!err)
			return 0;
	}
	return 1;
}

static int QLVAGC_Set(PTQLVAGCInfo pQLVAGCInfo, const char *pszItem, const char *pszValue)
{
	if (!strcasecmp(pszItem, "AGCCtlEn"))
		pQLVAGCInfo->AGCCtlEn = atoi(pszValue);
	else if (!strcasecmp(pszItem, "AGCDevice"))
		pQLVAGCInfo->AGCDevice = atoi(pszValue);
	else if (!strcasecmp(pszItem, "AGCSilenceLevel"))
		pQLVAGCInfo->AGCSilenceLevel = atoi(pszValue);
	else if (!strcasecmp(pszItem, "AGCMeanHigh"))
		pQLVAGCInfo->AGCMeanHigh = atoi(pszValue);
	else if (!strcasecmp(pszItem, "AGCMeanLow"))
		pQLVAGCInfo->AGCMeanLow = atoi(pszValue);
	else if (!strcasecmp(pszItem, "AGCUpMaxHigh"))
		pQLVAGCInfo->AGCUpMaxHigh = atoi(pszValue) / 2;
	else if (!strcasecmp(pszItem, "AGCDownMinHigh"))
		pQLVAGCInfo->AGCDownMinHigh = atoi(pszValue) / 2;
	else if (!strcasecmp(pszItem, "AGCHwMin"))
		pQLVAGCInfo->AGCHwMin = atoi(pszValue);
	else if (!strcasecmp(pszItem, "AGCHwMax"))
		pQLVAGCInfo->AGCHwMax = atoi(pszValue);
	else if (!strcasecmp(pszItem, "AGCHwDef"))
		pQLVAGCInfo->AGCHwDef = atoi(pszValue);
	else if (!strcasecmp(pszItem, "AGCHwStep"))
		pQLVAGCInfo->AGCHwStep = atoi(pszValue);
	else if (!strcasecmp(pszItem, "AGCCheckCnt"))
		pQLVAGCInfo->AGCCheckCnt = atoi(pszValue);
	else if (!strcasecmp(pszItem, "AGCAllowMeanOverCnt"))
		pQLVAGCInfo->AGCAllowMeanOverCnt = atoi(pszValue);
	else if (!strcasecmp(pszItem, "AGCAllowMeanUnderCnt"))
		pQLVAGCInfo->AGCAllowMeanUnderCnt = atoi(pszValue);
	else if (!strcasecmp(pszItem, "AGCAllowHighOverCnt"))
		pQLVAGCInfo->AGCAllowHighOverCnt = atoi(pszValue);
	else if (!strcasecmp(pszItem, "AGCCheckPeriod"))
		pQLVAGCInfo->AGCCheckPeriod = atoi(pszValue);
	else if (!strcasecmp(pszItem, "AGCProtectPeriod"))
		pQLVAGCInfo->AGCProtectPeriod = atoi(pszValue);
	else if (!strcasecmp(pszItem, "AGCCheckAlways"))
		pQLVAGCInfo->AGCCheckAlways = atoi(pszValue);
	else if (!strcasecmp(pszItem, "AGCAlg"))
		pQLVAGCInfo->AGCAlg = atoi(pszValue);
	else if (!strcasecmp(pszItem, "AGCPrintInfo"))
		pQLVAGCInfo->AGCPrintInfo = atoi(pszValue);
	return 0;
}

static int QLVAGC_HWFrontEndCtrl(PTQLVAGCInfo pQLVAGCInfo, int dir, int step)
{
#if 0
	uint8_t ori_reg1;
	uint8_t retdata[5];
#endif
	uint8_t target_reg1, target_reg2;
	int do_update = 0;
	int err = 0;
	int retdir = 0;
	uint8_t data[5];

	if (pQLVAGCInfo->AGCHandle)
	{
		if (pQLVAGCInfo->AGCDevice==0)
		{
		#if 0
			data[0] = 0x10;
			data[1] = 0x17;
			err = QLVAGC_ReadI2Cnb(pQLVAGCInfo->AGCHandle, 0, 0x14, 2, data, 1, retdata);
			if (!err) {
				ori_reg1 = retdata[0];
				if (pQLVAGCInfo->AGCPrintInfo)
					fprintf(stderr, "%s: Read 0x1017= 0x%x\n", __FUNCTION__, ori_reg1);
			}
			else {
				if (pQLVAGCInfo->AGCPrintInfo)
					fprintf(stderr, "%s: Error! QLVAGC_ReadI2Cnb\n", __FUNCTION__);
			}
			if (!err) 
		#endif
			{
				switch (dir)
				{
					case 0:
						pQLVAGCInfo->cachevolval = pQLVAGCInfo->AGCHwDef;
						do_update = 1;
						retdir = 0;
						if (pQLVAGCInfo->AGCPrintInfo) {
							fprintf(stderr, "%s: init volume to %d\n", __FUNCTION__, pQLVAGCInfo->cachevolval);
						}
						break;
					case 1:
						if (pQLVAGCInfo->cachevolval > pQLVAGCInfo->AGCHwMin)
						{
							pQLVAGCInfo->cachevolval = pQLVAGCInfo->cachevolval - pQLVAGCInfo->AGCHwStep;
							if (pQLVAGCInfo->cachevolval<pQLVAGCInfo->AGCHwMin)
								pQLVAGCInfo->cachevolval = pQLVAGCInfo->AGCHwMin;	
							do_update = 2;
							retdir = 1;
							if (pQLVAGCInfo->AGCPrintInfo) {
								fprintf(stderr, "%s: decrease volume to %d\n", __FUNCTION__, pQLVAGCInfo->cachevolval);
							}
						}
						else {
							if (pQLVAGCInfo->AGCPrintInfo) {
								fprintf(stderr, "%s: current volume %d already min %d \n", 
									__FUNCTION__,pQLVAGCInfo->cachevolval, pQLVAGCInfo->AGCHwMin);
							}
						}
						break;
					case 2:
						if (pQLVAGCInfo->cachevolval < pQLVAGCInfo->AGCHwMax)
						{
							pQLVAGCInfo->cachevolval = pQLVAGCInfo->cachevolval + step * pQLVAGCInfo->AGCHwStep;
							if (pQLVAGCInfo->cachevolval>pQLVAGCInfo->AGCHwMax)
								pQLVAGCInfo->cachevolval = pQLVAGCInfo->AGCHwMax;	
							do_update = 2;
							retdir = 2;
							if (pQLVAGCInfo->AGCPrintInfo) {
								fprintf(stderr, "%s: increase volume to %d\n", __FUNCTION__, pQLVAGCInfo->cachevolval);
							}
						}
						else {
							if (pQLVAGCInfo->AGCPrintInfo) {
								fprintf(stderr, "%s: current volume %d already max %d \n", 
									__FUNCTION__,pQLVAGCInfo->cachevolval, pQLVAGCInfo->AGCHwMax);
							}
						}
						break;
				}
				
				if (do_update)
				{
					#define	TARGET_VOL1_OFFSET 0x28
					#define TARGET_VOL2_OFFSET 0x4A

					if (pQLVAGCInfo->AGCAlg==1)
					{
						target_reg1 = TARGET_VOL1_OFFSET + 4;
						target_reg2 = TARGET_VOL2_OFFSET + pQLVAGCInfo->cachevolval - 24;
					}
					else {					
						target_reg1 = TARGET_VOL1_OFFSET + pQLVAGCInfo->cachevolval / 6;
						target_reg2 = TARGET_VOL2_OFFSET + pQLVAGCInfo->cachevolval % 6;
					}
					if (target_reg2>0x4F) 
						target_reg2 = 0x4F;

					if ((pQLVAGCInfo->AGCAlg==0) ||(do_update==1))
					{
						data[0] = 0x10;
						data[1] = 0x17;
						data[2] = target_reg1;
						err = QLVAGC_WriteI2Cnb(pQLVAGCInfo->AGCHandle, 0, 0x14, 3, data);
						if (err)
						{
							if (pQLVAGCInfo->AGCPrintInfo)
								fprintf(stderr, "%s: Error! QLVAGC_WriteI2Cnb 0x1017=%d\n", __FUNCTION__, target_reg1);
						}
						else {
							if (pQLVAGCInfo->AGCPrintInfo)
								fprintf(stderr, "%s: QLVAGC_WriteI2Cnb 0x1017=%d Success\n", __FUNCTION__, target_reg1);
						}
					}
					data[0] = 0x10;
					data[1] = 0x12;
					data[2] = target_reg2;
					err = QLVAGC_WriteI2Cnb(pQLVAGCInfo->AGCHandle, 0, 0x14, 3, data);
					if (err)
					{
						if (pQLVAGCInfo->AGCPrintInfo)
							fprintf(stderr, "%s: Error! QLVAGC_WriteI2Cnb 0x1012=%d\n", __FUNCTION__, target_reg2);
					}
					else {
						if (pQLVAGCInfo->AGCPrintInfo)
							fprintf(stderr, "%s: QLVAGC_WriteI2Cnb 0x1012=%d Success\n", __FUNCTION__, target_reg2);
					}
				}
			}
		}
	}
	return retdir;
}

int QLVAGC_Initial(component_t* pComponent, PTQLVAGCInfo* ppQLVAGCInfo)
{
	PTQLVAGCInfo	pQLVAGCInfo;
	
	pQLVAGCInfo = (TQLVAGCInfo *)malloc(sizeof(TQLVAGCInfo));
	
	if(pQLVAGCInfo == NULL)
	{
		printf("[QLVAGC]Error allocating instance memory\n");
		return -1;
	}
	memset(pQLVAGCInfo, 0, sizeof(TQLVAGCInfo));
	
    pQLVAGCInfo->AGCHandle =pComponent->hDev;
	*ppQLVAGCInfo = pQLVAGCInfo;
	
	printf("[QLVAGC]Initial success\n");
    
	return 0;
}



int QLVAGC_Config(PTQLVAGCInfo pQLVAGCInfo, const char * pCfgFile)
{
	extern char *skip_whitespace(char *Str);
	extern char *skip_to_whitespace(char *Str);

	int		err = 0;
	FILE	*fscript;
	int		currbytesread = 0;
	size_t  currlinesize = 0;
    //char	tempbuff[256];
	char	*currlinebuff = NULL;
	char	*argv[4];

	fscript = fopen(pCfgFile, "r");

    if (fscript > 0)
	{
		printf("[QLVAGC]****** Read AGC config form file %s \n", pCfgFile);
		currlinebuff = malloc(256);
		while (1)
		{
			currbytesread = getline(&currlinebuff, &currlinesize, fscript);
			if (currbytesread > 0)
			{
				argv[0] = skip_whitespace(currlinebuff);
				if (!argv[0])
					continue;
				argv[1] = skip_to_whitespace(argv[0]);
				if (!argv[1])
					continue;
				*argv[1]++ = 0;

				argv[1] = skip_whitespace(argv[1]);
				if (!argv[1])
					continue;
				argv[2] = skip_to_whitespace(argv[1]);
				if (!argv[2])
					continue;
				*argv[2]++ = 0;

				err = QLVAGC_Set(pQLVAGCInfo, argv[0], argv[1]);
			}
			else
			{
				break;
            }
        }
		if (currlinebuff)
			free(currlinebuff);
        fclose(fscript);
    }
	else
	{
		printf("[QLVAGC] Unable to open file %s for AGC config ", pCfgFile);
		return -1;
	}
	return 0;
}


int QLVAGC_HwOpen(PTQLVAGCInfo pQLVAGCInfo, int force) 
{
	int err;
	if (pQLVAGCInfo->AGCCtlEn || force)
	{
		uint32_t data[5];
		uint32_t addr;
		//QLVAGC_Init();
		//QLVAGC_SetDebugLevel(0);
		//QLVAGC_InitDevice(&m_Handle, 0, MODE_GENERIC_CTRL);
		if (pQLVAGCInfo->AGCHandle)
		{
			//QLVAGC_Open(m_Handle);
			addr = 0xd80f5500;
			data[0] = 0; /* @0x1f5500 */
			data[1] = pQLVAGCInfo->AGCSilenceLevel; /* @0x1f5504 */
			data[2] = 0; /* @0x1f5508 */
			data[3] = 0; /* @0x1f550C */
			data[4] = 0; /* @0x1f5510 */
			err = QLVAGC_WriteMem(pQLVAGCInfo->AGCHandle, addr, data, 5); 
			if (pQLVAGCInfo->AGCPrintInfo && err)
				fprintf(stderr,"%s: Error = %d\n", __FUNCTION__, err);
			QLVAGC_HWFrontEndCtrl(pQLVAGCInfo, 0, 1);
		}
	}
	return 0;
}

int QLVAGC_HwClose(PTQLVAGCInfo pQLVAGCInfo) 
{
	return 0;
}

int QLVAGC_AgcOnOff(PTQLVAGCInfo pQLVAGCInfo, int enable)
{
	int err;
	time_t now;
	if (pQLVAGCInfo->AGCCtlEn)
	{
		if (enable)
		{
			now = time(NULL);
			pQLVAGCInfo->lastchecktime = now;
			pQLVAGCInfo->lastagcupdatetime = now;
			pQLVAGCInfo->sampleno = 0;
			pQLVAGCInfo->largeno = 0;
			pQLVAGCInfo->smallno = 0;
			pQLVAGCInfo->maxhighno = 0;
			pQLVAGCInfo->slienceno = 0;
			//pQLVAGCInfo->maxhighinperoid = 0;
			pQLVAGCInfo->maxehighinperoid = 0;
			pQLVAGCInfo->maxelowinperoid = 0;
			pQLVAGCInfo->maxmeaninperoid = 0;
			pQLVAGCInfo->lastvolume = 0xFFFFFFFF;
		}
		if (pQLVAGCInfo->AGCHandle)
		{
			uint32_t data[5];
			uint32_t addr;

			addr = 0xd80f5500;
			if (enable)	
				data[0] = 1;
			else
				data[0] = 0;
			err = QLVAGC_WriteMem(pQLVAGCInfo->AGCHandle, addr, data, 1); 
			if (pQLVAGCInfo->AGCPrintInfo && err)
				fprintf(stderr,"%s: Error = %d\n", __FUNCTION__, err);
			return 0;
		}
		return -1;
	}
	else
		return -1;
}

/* ret <0: error or setting off, 0: setting on, hw not enable yet, 1: setting on, hw enable */
int QLVAGC_GetAgcOnOff(PTQLVAGCInfo pQLVAGCInfo) 
{
	if (pQLVAGCInfo->AGCCtlEn && pQLVAGCInfo->AGCHandle)
	{
		uint32_t data[5];
		uint32_t addr;

		addr = 0xd80f5500;
		QLVAGC_ReadMem(pQLVAGCInfo->AGCHandle, addr, data, 1);
		if (data[0]==0)
			return 0;				
		else
			return 1;
	}
	else
		return -1;
}

/* ret 0: no change, 1: volume too large, agc decreased, 2: volume too small, agc increased */
int QLVAGC_AgcUpdate(PTQLVAGCInfo pQLVAGCInfo, int DbgByteCnt) 
{
	int ret=0;
	int reinit=0;
	int lastdir=0;

	if (pQLVAGCInfo->AGCCtlEn && pQLVAGCInfo->AGCHandle)
	{
		time_t now = time(NULL);
		if ( ((pQLVAGCInfo->AGCCheckAlways) || (difftime(now, pQLVAGCInfo->lastchecktime)>pQLVAGCInfo->AGCCheckPeriod)) &&
		     (difftime(now, pQLVAGCInfo->lastagcupdatetime)>=pQLVAGCInfo->AGCProtectPeriod) ) 
		{  /* do sample */
			uint32_t data[8];
			uint32_t addr;

		    pQLVAGCInfo->lastchecktime = now;

			data[0] = 0;
			addr = 0xd80f5500;
			QLVAGC_ReadMem(pQLVAGCInfo->AGCHandle, addr, data, 7);
			if (data[0]==1) /* check enable */
			{
				/* silence check */
				if ((data[4]==0xFFFFFFFF) ||
				    ((data[2]==1) && (data[4]==pQLVAGCInfo->lastvolume)))
				{
					pQLVAGCInfo->slienceno++;				
					if (pQLVAGCInfo->AGCPrintInfo)
					{
						fprintf(stderr, "%s: SVolume val=0x%x (0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x) agc=%d @ time=%lu \n", 
						   __FUNCTION__, data[4], data[0],data[1],data[2],data[3], data[4], data[5], data[6], pQLVAGCInfo->cachevolval, now);
					}
					if (pQLVAGCInfo->slienceno >= pQLVAGCInfo->AGCCheckCnt)
					{	/* we have full check range as silence */
						if (pQLVAGCInfo->AGCPrintInfo)
						{
							fprintf(stderr, "%s: To many silence %d %d %d %d %d [%d %d %d %d] [dir=%d]\n", __FUNCTION__, 
								pQLVAGCInfo->sampleno, pQLVAGCInfo->largeno, pQLVAGCInfo->smallno, pQLVAGCInfo->maxhighno, pQLVAGCInfo->slienceno,
								pQLVAGCInfo->AGCCheckCnt, pQLVAGCInfo->AGCAllowMeanOverCnt,
								pQLVAGCInfo->AGCAllowMeanUnderCnt,pQLVAGCInfo->AGCAllowHighOverCnt, lastdir);
						}
						pQLVAGCInfo->sampleno = 0;
						pQLVAGCInfo->largeno = 0;
						pQLVAGCInfo->smallno = 0;
						pQLVAGCInfo->maxhighno = 0;
						pQLVAGCInfo->slienceno = 0;
						//pQLVAGCInfo->maxhighinperoid = 0;
						pQLVAGCInfo->maxehighinperoid = 0;
						pQLVAGCInfo->maxelowinperoid = 0;
						pQLVAGCInfo->maxmeaninperoid = 0;
					}
				}
				else  
				{
					int32_t curmax, curmin;
					if (pQLVAGCInfo->AGCPrintInfo)
					{
						fprintf(stderr, "%s: AVolume val=0x%x (0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x) agc=%d @ time=%lu \n", 
						   __FUNCTION__, data[4], data[0],data[1],data[2],data[3], data[4], data[5], data[6], pQLVAGCInfo->cachevolval, now);
					}
					pQLVAGCInfo->sampleno++;
					curmax = (int32_t) data[5];
					curmin = -1 * ((int32_t) data[6]);
					//if (data[3] > pQLVAGCInfo->maxhighinperoid) 
					//	pQLVAGCInfo->maxhighinperoid = data[3];	
					if (curmax > pQLVAGCInfo->maxehighinperoid) 
						pQLVAGCInfo->maxehighinperoid = curmax;
					if (curmin > pQLVAGCInfo->maxelowinperoid) 
						pQLVAGCInfo->maxelowinperoid = curmin;
					if (data[4] > pQLVAGCInfo->maxmeaninperoid)
						pQLVAGCInfo->maxmeaninperoid = data[4];
					if (data[4] >= (uint32_t)pQLVAGCInfo->AGCMeanHigh) 
						pQLVAGCInfo->largeno++;	
					//if ((data[4] <= (uint32_t)pQLVAGCInfo->AGCMeanLow) &&
					//    (data[3] <= (uint32_t)pQLVAGCInfo->AGCUpMaxHigh)) 
					//	pQLVAGCInfo->smallno++;	
					if ((data[4] <= (uint32_t)pQLVAGCInfo->AGCMeanLow) &&
					    (curmax <= pQLVAGCInfo->AGCUpMaxHigh) &&
					    (curmin <= pQLVAGCInfo->AGCUpMaxHigh ))
					{
						pQLVAGCInfo->smallno++;
					}
					//if (data[3]	> (uint32_t)pQLVAGCInfo->AGCDownMinHigh)
					//	pQLVAGCInfo->maxhighno++;
					if (curmax > pQLVAGCInfo->AGCDownMinHigh)
					{
						pQLVAGCInfo->maxhighno++;
					}
					else if (curmin > pQLVAGCInfo->AGCDownMinHigh)
					{
						pQLVAGCInfo->maxhighno++;
					}
					#if 0
					if (data[3] >= pQLVAGCInfo->maxhighfull) 
					{
						pQLVAGCInfo->maxhighfull = data[3];	
						pQLVAGCInfo->maxhightime = now;	
					}
					#endif
					/* checking */
					if ((pQLVAGCInfo->maxhighno >= pQLVAGCInfo->AGCAllowHighOverCnt) ||
						(pQLVAGCInfo->largeno>=pQLVAGCInfo->AGCAllowMeanOverCnt) )
					{
						/* got too many time over the high value, try to low down the volume */
						/* or volume too large */
						lastdir = 1;
						ret = QLVAGC_HWFrontEndCtrl(pQLVAGCInfo, lastdir, 1);
						reinit = 1;
					}
					//else if ((pQLVAGCInfo->smallno>=pQLVAGCInfo->AGCAllowMeanUnderCnt) &&
					//		(pQLVAGCInfo->maxhighinperoid < (uint32_t) pQLVAGCInfo->AGCUpMaxHigh))					       
					else if ((pQLVAGCInfo->smallno>=pQLVAGCInfo->AGCAllowMeanUnderCnt) &&
							(pQLVAGCInfo->maxehighinperoid < pQLVAGCInfo->AGCUpMaxHigh) && 
							(pQLVAGCInfo->maxelowinperoid < pQLVAGCInfo->AGCUpMaxHigh) )
					{   /* volume too small, and maxhigh not over Up limit */
						int step = 1;
						lastdir = 2;
						if ((pQLVAGCInfo->AGCHwStep<=3) && (pQLVAGCInfo->maxmeaninperoid<((uint32_t) (pQLVAGCInfo->AGCMeanLow)>>2)))
							step = 2;
						ret = QLVAGC_HWFrontEndCtrl(pQLVAGCInfo, lastdir, step);
						reinit = 1;
					}
					else if (pQLVAGCInfo->sampleno >= pQLVAGCInfo->AGCCheckCnt) 
					{
						reinit = 1;
					}
					if (reinit)
					{	
						if (pQLVAGCInfo->AGCPrintInfo)
						{
							fprintf(stderr, "%s: Volume check %d %d %d %d %d [%d %d %d %d] [dir=%d]\n", __FUNCTION__, 
								pQLVAGCInfo->sampleno, pQLVAGCInfo->largeno, pQLVAGCInfo->smallno, pQLVAGCInfo->maxhighno, pQLVAGCInfo->slienceno,
								pQLVAGCInfo->AGCCheckCnt, pQLVAGCInfo->AGCAllowMeanOverCnt,
								pQLVAGCInfo->AGCAllowMeanUnderCnt,pQLVAGCInfo->AGCAllowHighOverCnt, lastdir);
						}
						pQLVAGCInfo->sampleno = 0;
						pQLVAGCInfo->largeno = 0;
						pQLVAGCInfo->smallno = 0;
						pQLVAGCInfo->maxhighno = 0;
						pQLVAGCInfo->slienceno = 0;
						//pQLVAGCInfo->maxhighinperoid = 0;
						pQLVAGCInfo->maxehighinperoid = 0;
						pQLVAGCInfo->maxelowinperoid = 0;
						pQLVAGCInfo->maxmeaninperoid = 0;
					}
					if (ret)
					{
						if (pQLVAGCInfo->AGCPrintInfo)
							fprintf(stderr,"%s: chage agc value dir=(%d) @ bytecnt=%d\n", __FUNCTION__, ret, DbgByteCnt); 
						pQLVAGCInfo->lastagcupdatetime = now;
					}
				}
				pQLVAGCInfo->lastvolume = data[4];
			}				
		} 
	}
	return ret;
}

void QLVAGC_Release(PTQLVAGCInfo pQLVAGCInfo) 
{
	if (pQLVAGCInfo)
	{
	  	QLVAGC_AgcOnOff(pQLVAGCInfo, 0);
		QLVAGC_HwClose(pQLVAGCInfo);
        free(pQLVAGCInfo);
	}
}



