#!/bin/sh

#-----------------------------------------------------------------------------
# PL330 Test Suite
#-----------------------------------------------------------------------------
#set -x
#-----------------------------------------------------------------------------
# Setup test constants & variables
#-----------------------------------------------------------------------------
# standard test configuration variables
vHome=$PWD                     # Initial path for test execution
vTestName=`basename $0`        # Test script name
vRunTime=10                    # Number of seconds to run specific test functions
vRunLoop=0
vRunningScripts=""
vRunningOutput=""
vCheckDisable="Disable"
vTestAudSuite=0
vTestViuSuite=0
vTestVouSuite=0
vTestCamSuite=0
vTestRes=720
vReloadDriverScript="./load_pldev300usb.sh"
vSudo="sudo"
vOutputDir="./testout"
#configuration variables from args
vConfTestTime=10
vConfTestLoop=0
vConfNoReload=0
#-----------------------------------------------------------------------------
# Functions
#-----------------------------------------------------------------------------
# Usage menu
doTestScripts () {
    if [ "$4" = "" ]; then
        echo "============================================="
        echo "===>Error: Format not matched!"
        echo "============================================="
        return 1
    fi
    loop=1
    vRunLoop=$1
    vRunTime=$2
    vRunningScripts=$3
    vRunningOutput=$4
    if [ -e "$3" ]; then
        echo "============================================="
        echo "....... $vRunningScripts in Loop $loop......."
        echo "============================================="
    else
        echo "========================================================="
        echo "....... $vRunningScripts (Not Exist) in Loop $loop......."
        echo "========================================================="
    fi
	while [ 1 ]; do
	sh ./userinput.sh $vRunTime | $vRunningScripts
	sleep 3
    if [ "$vRunningOutput" = "$vCheckDisable" ]; then
        echo "===>Info: Skip Output File Check...."
    else
        if [ -e "$vRunningOutput" ]; then
            vTestFileName=`basename $vRunningOutput`
            cp -f $vRunningOutput ${vOutputDir}/${vTestFileName}
            s=$(ls -lah $vRunningOutput | awk '{ print $5}')
            echo "===>Info: Out File $vRunningOutput Length=$s"
            if [ "$s" = "" ]; then
                echo "===>Error: No data file!"
                return 1
            fi
            if [ "$s" = "0" ]; then
                echo "===>Error: No data output!"
                return 1
            fi
        else
            echo "===>Error: Output File $vRunningOutput not exist...."
        fi
    fi
    echo "============================================="
    echo "....... $vRunningScripts in Loop $loop..END.."
    echo "============================================="
	loop=`expr "$loop" + 1`
	if [ $loop -ge $vRunLoop ]; then
        break
    fi
    done
    return 0
}

doReloadDriver () {
    if [ $vConfNoReload -gt 0 ]; then
        echo ""
        echo "===> Skip Reload PL330 Driver"
        echo ""
    else
        echo ""
        echo "===> Start Reload PL330 Driver"
        echo "$vSudo $vReloadDriverScript"
        $vSudo $vReloadDriverScript
        echo ""
    fi

}
doTestSuites () {
    if [ $vTestAudSuite -eq 1 ]; then
        echo "==== Do audio test suites ===="
        doReloadDriver
        doTestScripts $vConfTestLoop $vConfTestTime ./aiu_enc_aac.sh ./aiu_test.aac
        doReloadDriver
        doTestScripts $vConfTestLoop $vConfTestTime ./aiu_enc_g711.sh ./aiu_test.g711
        doReloadDriver
        doTestScripts $vConfTestLoop $vConfTestTime ./aiu_enc_g729.sh ./aiu_test.g729
        doReloadDriver
        doTestScripts $vConfTestLoop $vConfTestTime ./aiu_enc_lpcm.sh ./aiu_test.pcm
        doReloadDriver
        doTestScripts $vConfTestLoop $vConfTestTime ./aiu_enc_silk.sh $vCheckDisable
        doReloadDriver
        doTestScripts $vConfTestLoop $vConfTestTime ./aou_dec_aac.sh $vCheckDisable
        doReloadDriver
        doTestScripts $vConfTestLoop $vConfTestTime ./aou_dec_g711.sh $vCheckDisable
        doReloadDriver
        doTestScripts $vConfTestLoop $vConfTestTime ./aou_dec_g729.sh ./aiu_test.g729
        doReloadDriver
        doTestScripts $vConfTestLoop $vConfTestTime ./aou_dec_lpcm.sh ./aiu_test.pcm
        doReloadDriver
        doTestScripts $vConfTestLoop $vConfTestTime ./aou_dec_silk.sh ./aiu_test.silk
        if [ -e ./test_8k_1ch.pcm ]; then
            doReloadDriver
            doTestScripts $vConfTestLoop $vConfTestTime ./hiu_enc_g711.sh ./aiu_test.g711
        fi
        if [ -e ./test_48k.pcm ]; then
            doReloadDriver
            doTestScripts $vConfTestLoop $vConfTestTime ./hiu_enc_aac.sh ./aiu_test.aac
        fi
        if [ -e ./test_16k_1ch.pcm ]; then
            doReloadDriver
            doTestScripts $vConfTestLoop $vConfTestTime ./hiu_enc_silk.sh ./aiu_test.silk
        fi
    fi

    if [ $vTestCamSuite -eq 1 ]; then
        echo "==== Do Cam test suites ===="
        doReloadDriver
        doTestScripts $vConfTestLoop $vConfTestTime ./hiumjpegdec_internal_to_enc_1280x720.sh ./output.264
        doReloadDriver
        doTestScripts $vConfTestLoop $vConfTestTime ./hiumjpegdec_internal_to_enc_1280x720_ts.sh ./output.mpg
    fi

    if [ $vTestViuSuite -eq 1 ]; then
        echo "==== Do VIU test suites for EVK===="
        if [ $vTestRes -eq 720 ]; then
            doReloadDriver
            doTestScripts $vConfTestLoop $vConfTestTime ./viu_enc_1280x720.sh ./viu_1280x720.264
            doReloadDriver
            doTestScripts $vConfTestLoop $vConfTestTime ./ts_enc_1280x720.sh ./ts_1280x720.mpg
        fi
        if [ $vTestRes -eq 1080 ]; then
            doReloadDriver
            doTestScripts $vConfTestLoop $vConfTestTime ./viu_enc_1920x1080p.sh ./viu_1920x1080.264
            doReloadDriver
            doTestScripts $vConfTestLoop $vConfTestTime ./ts_enc_1920x1080p.sh ./ts_1920x1080p.mpg
        fi
    fi

    if [ $vTestViuSuite -eq 2 ]; then
        echo "==== Do VIU test suites for IT6604/JT===="
        if [ $vTestRes -eq 720 ]; then
            doReloadDriver
            doTestScripts $vConfTestLoop $vConfTestTime ./it6604_enc_264_1280x720.sh ./viu_1280x720.264
            doReloadDriver
            doTestScripts $vConfTestLoop $vConfTestTime ./it6604_enc_ts_1280x720.sh ./ts_1280x720.mpg
        fi
        if [ $vTestRes -eq 1080 ]; then
            doReloadDriver
            doTestScripts $vConfTestLoop $vConfTestTime ./it6604_enc_264_1920x1080p.sh ./viu_1920x1080.264
            doReloadDriver
            doTestScripts $vConfTestLoop $vConfTestTime ./it6604_enc_ts_1920x1080p.sh ./ts_1920x1080p.mpg
        fi
    fi

    if [ $vTestViuSuite -eq 3 ]; then
        echo "==== Do VIU test suites for webcam===="
        doReloadDriver
        doTestScripts $vConfTestLoop $vConfTestTime ./webcam_enc_264_1280x720.sh ./webcam_output.264
        doReloadDriver
        doTestScripts $vConfTestLoop $vConfTestTime ./webcam_enc_ts_1280x720.sh ./ts_1280x720.mpg
        doReloadDriver
        doTestScripts $vConfTestLoop $vConfTestTime ./webcam_enc_264_1280x720_osd.sh ./webcam_output.264
        doReloadDriver
        doTestScripts $vConfTestLoop $vConfTestTime ./webcam_enc_264_decimation.sh ./webcam_output.264
        doReloadDriver
        doTestScripts $vConfTestLoop $vConfTestTime ./webcam_enc_264_decimation_osd.sh ./webcam_output.264
        doReloadDriver
        doTestScripts $vConfTestLoop $vConfTestTime ./webcam_enc_264_mjpeg_lpcm_1280x720.sh ./webcam_output.264
        doReloadDriver
        doTestScripts $vConfTestLoop $vConfTestTime ./webcam_enc_264_vid_passthrough_1280x720.sh ./webcam_output.264
        doReloadDriver
        doTestScripts $vConfTestLoop $vConfTestTime ./webcam_enc_mjpeg_1280x720.sh ./webcam_output.mjpg
        doReloadDriver
        doTestScripts $vConfTestLoop $vConfTestTime ./webcam_motiondetect_1280x720.sh ./md.bin
    fi

    if [ $vTestVouSuite -eq 1 ]; then
        echo "==== Do VOU test suites for EVK===="
        if [ $vTestRes -eq 720 ]; then
            doReloadDriver
            doTestScripts $vConfTestLoop $vConfTestTime ./vou_dec_1280x720.sh $vCheckDisable
        fi
        if [ $vTestRes -eq 1080 ]; then
            doReloadDriver
            doTestScripts $vConfTestLoop $vConfTestTime ./vou_dec_1920x1080p.sh $vCheckDisable
        fi
    fi
    if [ $vTestVouSuite -eq 2 ]; then
        echo "==== Do VOU test suites for IT6604/JT===="
        if [ $vTestRes -eq 720 ]; then
            doReloadDriver
            doTestScripts $vConfTestLoop $vConfTestTime ./vou_dec_1280x720.sh $vCheckDisable
        fi
        if [ $vTestRes -eq 1080 ]; then
            doReloadDriver
            doTestScripts $vConfTestLoop $vConfTestTime ./vou_dec_1920x1080p.sh $vCheckDisable
        fi
    fi
}

showUsage() {
    echo "Usage: ./$vTestName <aud/evk/it6604/jt/evk1080/it1080/jt1080/cam/webcam> <testtime> <loopcnt> <noreload>"
    exit 1
}
echo
echo "===============Test Suite Start==============="
echo
if [ -z $1 ]; then
    showUsage
fi
if [ "$2" != "" ]; then
    vConfTestTime=$2
fi
if [ "$3" != "" ]; then
    vConfTestLoop=$3
fi
if [ "$4" != "" ]; then
    vConfNoReload=$4
fi

if [ "$1" = "aud" ]; then
    vTestAudSuite=1
    vSudo="sudo"
    vReloadDriverScript="./load_pldev300usb.sh"
elif [ "$1" = "evk" ]; then
    vTestViuSuite=1
    vTestVouSuite=1
    vSudo="sudo"
    vReloadDriverScript="./load_pldev300usb.sh"
elif [ "$1" = "evk1080" ]; then
    vTestViuSuite=1
    vTestVouSuite=1
    vTestRes=1080
    vSudo="sudo"
    vReloadDriverScript="./load_pldev300usb.sh"
elif [ "$1" = "it6604" ]; then
    vTestViuSuite=2
    vSudo="sudo"
    vReloadDriverScript="./load_pldev300usb.sh"
elif [ "$1" = "it1080" ]; then
    vTestViuSuite=2
    vTestRes=1080
    vSudo="sudo"
    vReloadDriverScript="./load_pldev300usb.sh"
elif [ "$1" = "jt" ]; then
    vTestViuSuite=2
    vTestVouSuite=2
    vSudo=""
    vReloadDriverScript="./load_pldev300usb.sh"
elif [ "$1" = "jt1080" ]; then
    vTestViuSuite=2
    vTestVouSuite=2
    vTestRes=1080
    vSudo=""
    vReloadDriverScript="./load_pldev300usb.sh"
elif [ "$1" = "cam" ]; then
    vTestCamSuite=1
    vSudo="sudo"
    vReloadDriverScript="./load_pldev300usb.sh"
elif [ "$1" = "webcam" ]; then
    vTestViuSuite=3
    vSudo="sudo"
    vReloadDriverScript="./load_pldev300webcam.sh"
else
    showUsage
fi

if [ -d $vOutputDir ]; then
    echo ""
else
    echo "==> create $vOutputDir"
    mkdir $vOutputDir
fi

doTestSuites
echo
echo
echo "================Test Suite End================"
echo
echo


