/*************************************************************************/
/**      **/
/**            **/
/**                **/
/**              **/
/**                                                                     **/
/**                               **/
/**                                                  **/
/*************************************************************************/

#ifndef __OSDSTRUCT_H
#define __OSDSTRUCT_H

#define 	OSD_BASE_ADDR			0x210000	//only for test

#define	NO_ERR						0
#define	ERR_OUT_OF_MEMORY		1
#define	ERR_ALLOCATE_MEMORY		2
#define	ERR_FORMAT_SUPPORT		3

//registers
#define REG_OSD_CONFIGURATION				0x13b0
#define REG_OSD_CLUT_ADDRESS				0x13b4
#define REG_OSD_CLUT_DATA					0x13b8
#define REG_OSD_START_ADDRESS				0x13bc
#define REG_OSD_DISPLAY_WINDOW_START	0x13c0
#define REG_OSD_DISPLAY_WINDOW_SIZE		0x13c4
#define REG_OSD_DRAM_REQUEST_INTERVAL	0x13c8
#define REG_OSD_HILIGHT_CONFIGURATION	0x13cc
#define REG_OSD_HILIGHT_WINDOW_START	0x13d0
#define REG_OSD_HILIGHT_WINDOW_SIZE		0x13d4
#define REG_OSD_HORIZONTAL_PIXEL_OFFSET	0x13d8

#define REG_OSD_LAYERSELECTION			0x1388
//layer define
#define MASK_ENABLE_OSD					0x00000210 	//OSD-VS-Border
#define MASK_DISABLE_OSD					0x00000110 	//VS-VS-Border

//pixel format
typedef enum _PIXEL_FORMAT{
PIXEL_LUT_1BIT 	=0,
PIXEL_LUT_2BIT 	=1,
PIXEL_LUT_4BIT	=2,
PIXEL_LUT_8BIT	=3,
PIXEL_UYVY_16BIT	=4,
PIXEL_YUYV_16BIT	=5,
PIXEL_YUV_32BIT	=6,
PIXEL_RGB_32BIT	=7,
PIXEL_YUV_16BIT_1B	=8,
PIXEL_RGB_16BIT_1B	=9,
PIXEL_YUV_16BIT_2B	=0xa,
PIXEL_RGB_16BIT_2B	=0xb,
PIXEL_MAX_FORMAT		=PIXEL_RGB_16BIT_2B
}OSD_PIXEL_FORMAT;

//resolution
typedef enum _WIN_RESOLUTION{
MASK_RES_352X240I		=0x01600078,
MASK_RES_352X288I		=0x01600090,
MASK_RES_720X480I		=0x02D000F0,
MASK_RES_720X576I		=0x02D00120,
MASK_RES_720X480P		=0x02D001E0,
MASK_RES_720X576P		=0x02D00240,
MASK_RES_1280X720P	=0x050002D0,
MASK_RES_1280X1080I	=0x0500021C,
MASK_RES_1280X1080P	=0x05000438
}WIN_RESOLUTION;

typedef struct _qldev_osd_param{
	unsigned int	type;
	unsigned int	value;
}QLDEV_OSD_PARAMETER, *PQLDEV_OSD_PARAMETER;

typedef enum _OSDCMD_TYPE{
	PARAM_SET_PIXEL_FORMAT = 0,
	PARAM_SET_RESOLUTION,
	PARAM_SET_HORIZONTALX2,
	PARAM_SET_VERTICALX2,
	PARAM_SET_BLENDING,
	PARAM_SET_BLENDING_VALUE,
	PARAM_SET_ENABLE,
	PARAM_SET_DISABLE,
	PARAM_GET_PIXEL_FORMAT,
	PARAM_GET_RESOLUTION,
	PARAM_SET_ADDRESS,

	PARAM_SET_CLUT = 0x10,
	PARAM_GET_CLUT,
	PARAM_MEM_ALLOCATE,
	PARAM_MEM_RELEASE,
	PARAM_SET_CURRENTOSD,
	PARAM_GET_CURRENTOSD,
	PARAM_DRAW_DOT,
	PARAM_DRAW_RECT,
	PARAM_DRAW_IMAGE,
	PARAM_DRAW_TEXT,
	PARAM_PASTE_BUFFER,
	PARAM_COPY_BUFFER,
	PARAM_SET_HILIGHT,

	PARAM_VIOSD_STMEN = 0x20,
	PARAM_VIOSD_STMFT,
	PARAM_VIOSD_STMXP,
	PARAM_VIOSD_STMYP,
	PARAM_VIOSD_STREN,
	PARAM_VIOSD_STRFT,
	PARAM_VIOSD_STRXP,
	PARAM_VIOSD_STRYP,
	PARAM_VIOSD_STR,

	PARAM_VIOSD_ON = 0x30,
	PARAM_VIOSD_OFF,
	PARAM_VIOSD_INIT,
	PARAM_VIOSD_CLEAR,
	PARAM_VIOSD_ADDFONT,
	PARAM_VIOSD_ADDTEXT,
	PARAM_VIOSD_SETTEXT,
	PARAM_VIOSD_DELTEXT,
	PARAM_VIOSD_SETTIME,
	PARAM_VIOSD_UPDATETIME,
	PARAM_VIOSD_INFO
} OSDCMD_TYPE, *POSDCMD_TYPE;

typedef struct _qldev_osd_control{
	unsigned int		CmdType;
	unsigned int		BufferID;
	int		x;
	int		y;
	int		Width;
	int		Height;
	unsigned int		*pColorData;
	unsigned int 		swap;
	unsigned int		RetCode;
	unsigned int		bUpdateTransparency;
	unsigned int		Transparency;
	unsigned int		BgColor;
	unsigned int		FgColor;
	unsigned char		*pFontData;
	unsigned int		ByteWidth;
}QLDEV_OSD_CONTROL, *PQLDEV_OSD_CONTROL;


typedef union _osd_config{
	unsigned int	Read;
	struct
	{
		unsigned int	en	: 1;	// bit 0
		unsigned int	h2x_e		: 1;	// bit 1
		unsigned int	h2x_flt		: 2;	// bit 2-3
		unsigned int	v2x_e		: 1;	// bit 4
		unsigned int	v2x_flt		: 2;	// bit 5-6
		unsigned int	reserved0		: 1;	// bit 7
		unsigned int	osd_mem_burst	: 4;	// bit 8-11
		unsigned int 	osd_data_length : 4; //bit 12-15
		unsigned int 	f_flush : 1;	// bit 16
		unsigned int 	f_lwm : 2;	// bit 17-18
		unsigned int 	g_blend_en : 2; // bit 19-20
		unsigned int 	reserved1	:	3;	// bit 21-23
		unsigned int 	g_blend_factor	: 8;	//bit 24-31
	} Reg;
} OSD_CONFIG, *POSD_CONFIG;


typedef struct _OSD_SHEET_INFO{
	unsigned int		BufferID;
	unsigned int		StartAddr;
}OSD_SHEET_INFO, *POSD_SHEET_INFO;


#define MAX_OSD_BUFFER_NUM	1
typedef struct _osd_disp_info{
	unsigned int		Width;
	unsigned int		Height;
	unsigned int		PixelFormat;
	unsigned int		CurrentBufferID;
	unsigned int		UsedSheetMap;
	OSD_SHEET_INFO	mSheet[MAX_OSD_BUFFER_NUM];
}OSD_DISPLAY_INFO, *pOSD_DISPLAY_INFO;

typedef union _color_data{
	unsigned int Read;
	struct{
		unsigned int alpha:8;
		unsigned int B:8;
		unsigned int G:8;
		unsigned int R:8;
	}RGBFormat;

	struct{
		unsigned int alpha:8;
		unsigned int V:8;
		unsigned int U:8;
		unsigned int Y:8;
	}YUVFormat;
}COLOR_DATA, *pCOLOR_DATA;

#define MAX_DISPLAY_LINE	720
typedef struct _osd_image_info{
	unsigned int		Height;
	unsigned int		Width;
	unsigned int		Depth;
	unsigned int		ColorFormat;
	//unsigned int		*pColorData;
	unsigned long		pColorData[MAX_DISPLAY_LINE];
}OSD_IMAGE_INFO, *pOSD_IMAGE_INFO;

typedef struct _osd_font_info{
	unsigned int		Height;
	unsigned int		Width;
	unsigned int 		ByteWidth;
	unsigned int		TotalChar;
	unsigned int		FileLength;
	unsigned char		*pStartAddr;
}OSD_FONT_INFO, *pOSD_FONT_INFO;

typedef struct _qldev_viosd_control{
	unsigned int	CmdType;
	char	*pString;
}QLDEV_VIOSD_CONTROL, *PQLDEV_VIOSD_CONTROL;

typedef struct _QPVIOSD_CONTROL
{
	unsigned int 	CmdType;
	unsigned int	Update;
	unsigned int	TimeStampEnable;
	unsigned int	TimeStampFontType;
	unsigned int	TimeStampXPosition;
	unsigned int	TimeStampYPosition;
	unsigned int	TestStringEnable;
	unsigned int	TestStringFontType;
	unsigned int	TestStringXPosition;
	unsigned int	TestStringYPosition;
	char	TestString[32];
}QPVIOSD_CONTROL, *PQPVIOSD_CONTROL;

#endif

