///////////////////////////////////////////////////////////////////////////////
//     
//
// Pldevioctl.h
//
// Description:
//
//  PLV4LDEV ioctl interface
//
// History:
//	Jan 13, 2010	(dt) created
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#ifndef __PLDEVIOCTL_H__
#define __PLDEVIOCTL_H__

#include "qlautoconf.h"
#include "qpdevcntl.h"
#include "qpdevdiag.h"
#include "qpmpgcodec.h"

#ifndef CONFIG_HOST_PLATFORM_USERSPACE
#define DEVHANDLE   int
#else
#define DEVHANDLE   intptr_t
#endif // CONFIG_HOST_PLATFORM_USERSPACE

#ifndef CONFIG_HOST_PLATFORM_USERSPACE
#ifndef _IOR_BAD
#define _IOR_BAD(type,nr,size)		_IOR(type,nr,size)
#endif //!_IOR_BAD
#ifndef _IOW_BAD
#define _IOW_BAD(type,nr,size)		_IOW(type,nr,size)
#endif //!_IOW_BAD
#ifndef _IOWR_BAD
#define _IOWR_BAD(type,nr,size)		_IOWR(type,nr,size)
#endif //!_IOWR_BAD
#else // CONFIG_HOST_PLATFORM_USERSPACE
#include "userioctl.h"
#endif // CONFIG_HOST_PLATFORM_USERSPACE

// IOCTL API version number
#define PLIOCTLAPI_VERSION_MAJOR  1
#define PLIOCTLAPI_VERSION_MINOR  2
#define PLIOCTLAPI_VERSION_BRANCH 6

#define PLIOCTLAPI_VERSION(a,b,c) (((a) << 16) + ((b) << 8) + (c))
#define PLIOCTLAPI_VERSION_CODE  ((PLIOCTLAPI_VERSION_MAJOR << 16) + (PLIOCTLAPI_VERSION_MINOR << 8) + PLIOCTLAPI_VERSION_BRANCH)
// IOCTL API: version 1.0.0	date: 2011/07/06
// IOCTL API: version 1.0.1	date: 2011/09/6
//add  QPCODEC_INPUT_CONTROL Artesa.force_60p_in field
// IOCTL API: version 1.2.0	date: 2012/01/12
// refer to QPDEVCNTL_VER from 1.0 to 2.0
// support configurable silk frame length (20ms/40ms/60ms)
// IOCTL API: version 1.2.1	date: 2012/03/08
// add PLDEV_DIAG_IOCTL_VIDEO_DECODER_GET/PLDEV_DIAG_IOCTL_VIDEO_DECODER_SET
// add PLDEV_CNTL_IOCTL_SET_ENC_OSD_SETTINGS and OSD related control
// ioctl base
// IOCTL API: version 1.2.2	date: 2012/06/10
// add COMPONENT_PORT_ACTIVITY_OUT support
// add PLDEV_CNTL_IOCTL_SET_LARGE_COMPRESS_BUFFER_CONTROL/LDEV_CNTL_IOCTL_GET_LARGE_COMPRESS_BUFFER_CONTROL
// add PLDEV_DIAG_IOCTL_DEBUGLEVEL_SET/PLDEV_DIAG_IOCTL_DEBUGLEVEL_GET
// add PLDEV_CNTL_IOCTL_GET_CODEC_ERROR
// IOCTL API: version 1.2.3	date: 2012/09/20
// add PLDEV_CNTL_IOCTL_SET_ENC_AUD_INPUT_VOL and PLDEV_CNTL_IOCTL_SET_DEC_AUD_OUTPUT_VOL
// add PLDEV_CNTL_IOCTL_GET_ENC_AUD_INPUT_VOL and PLDEV_CNTL_IOCTL_GET_DEC_AUD_OUTPUT_VOL
// IOCTL API: version 1.2.4	date: 2013/04/19
// add COMPONENT_PORT_TYPE_FORCE_FULL_BUFFER support
// add PLDEV_CNTL_IOCTL_SET_ENABLE_LOW_BITRATE_MODE and PLDEV_CNTL_IOCTL_GET_ENABLE_LOW_BITRATE_MODE
// IOCTL API: version 1.2.5	date: 2013/06/11
// add PLDEV_CNTL_IOCTL_SET_ENABLE_AV_DISCARD_CONTROL and PLDEV_CNTL_IOCTL_GET_ENABLE_AV_DISCARD_CONTROL
// add PLDEV_CNTL_IOCTL_SET_USE_SW_PTS and PLDEV_CNTL_IOCTL_GET_USE_SW_PTS
// add lr_swap field in AUDIO_CONTROL_EX
// IOCTL API: version 1.2.5	(no change) date: 2013/11/15
// add PLDEV_DIAG_IOCTL_DRVPROFILING_GET
// IOCTL API: version 1.2.6 date: 2014/02/14
// Low level qpdevcntl.h version change from 3.3 to 3.4

#define PLDEV_STRM_IOCTL_BASE		0
#define PLDEV_CNTL_IOCTL_BASE		0
#define PLDEV_DIAG_IOCTL_BASE		0
#define PLDEV_CNTL_331_IOCTL_BASE	0

#define PLDEV_STRM_IOCTL_TYPE		'S'
#define PLDEV_CNTL_IOCTL_TYPE		'C'
#define PLDEV_DIAG_IOCTL_TYPE		'D'
#define PLDEV_CNTL_331_IOCTL_TYPE	'P'




/////////////////////////////////////////////////////////////////////////////
//
//

// stream extension interface
//

typedef enum _COMPONENT_TYPE
{
	COMPONENT_TYPE_ENCODE = 0,
	COMPONENT_TYPE_DECODE,
	COMPONENT_TYPE_END

} COMPONENT_TYPE, *PCOMPONENT_TYPE;

#define MAX_COMPONENT_TYPE	COMPONENT_TYPE_END


typedef enum _COMPONENT_PORT_TYPE
{
	COMPONENT_PORT_COMP_OUT = 0,
	COMPONENT_PORT_COMP_AUD_OUT,
	COMPONENT_PORT_INDEX_OUT,
	COMPONENT_PORT_YUV_OUT,
	COMPONENT_PORT_PCM_OUT,
	COMPONENT_PORT_VIRTUAL_OUT,

	COMPONENT_PORT_YUV_IN,
	COMPONENT_PORT_COMP_IN,
	COMPONENT_PORT_COMP_AUD_IN,
	COMPONENT_PORT_PCM_IN,
	COMPONENT_PORT_VIRTUAL_IN,
	COMPONENT_PORT_ACTIVITY_OUT,
	COMPONENT_PORT_END

} COMPONENT_PORT_TYPE, *PCOMPONENT_PORT_TYPE;

#define MAX_COMPONENT_PORT		COMPONENT_PORT_END

#define COMPONENT_PORT_TYPE_FORCE_FULL_BUFFER	0x80000000

#pragma pack(4)

typedef union _COMPONENT_PORT_OPEN_FORMAT
{
	QP_YUV_DATAFORMAT				yuv;
	QP_PCM_DATAFORMAT				pcm;
	QP_COMPRESSED_VID_DATAFORMAT	vid;
	QP_ACTIVITY_DATAFORMAT			activity;
} COMPONENT_PORT_OPEN_FORMAT, *PCOMPONENT_PORT_OPEN_FORMAT;


typedef struct _COMPONENT_PORT_OPEN
{
	TQP_UINT32					type;
	COMPONENT_PORT_OPEN_FORMAT	format;
	int							memory;	// v4l2_memory
} COMPONENT_PORT_OPEN, *PCOMPONENT_PORT_OPEN;


typedef struct _PLDEV_INFO
{
	TQP_UINT32			busType;
	TQP_UINT32			devInstance;
	TQP_UINT32			type;
	TQP_OBJECT_HANDLE	hTask;
} PLDEV_INFO, *PPLDEV_INFO;

#pragma pack()

#define PLDEV_STRM_IOCTL_PORT_OPEN		_IOW_BAD(PLDEV_STRM_IOCTL_TYPE, PLDEV_STRM_IOCTL_BASE, COMPONENT_PORT_OPEN)
#define PLDEV_STRM_IOCTL_PORT_CLOSE		_IOW_BAD(PLDEV_STRM_IOCTL_TYPE, PLDEV_STRM_IOCTL_BASE + 1, COMPONENT_PORT_TYPE)
#define PLDEV_STRM_IOCTL_DEVICE_INFO	_IOWR_BAD(PLDEV_STRM_IOCTL_TYPE, PLDEV_STRM_IOCTL_BASE + 2, PLDEV_INFO)
#define PLDEV_STRM_IOCTL_ACQUIRE		_IO(PLDEV_STRM_IOCTL_TYPE, PLDEV_STRM_IOCTL_BASE + 3)

#define PLDEV_STRM_IOCTL_GET_DRIVER_VER		_IOWR_BAD(PLDEV_STRM_IOCTL_TYPE, PLDEV_STRM_IOCTL_BASE + 10, TQP_UINT32)
#define PLDEV_STRM_IOCTL_GET_IOCTLAPI_VER	_IOWR_BAD(PLDEV_STRM_IOCTL_TYPE, PLDEV_STRM_IOCTL_BASE + 11, TQP_UINT32)


/////////////////////////////////////////////////////////////////////////////
//
//

// device control interface
//

#define PLDEV_CNTL_IOCTL_SET		PLDEV_CNTL_IOCTL_BASE
#define PLDEV_CNTL_IOCTL_GET		(PLDEV_CNTL_IOCTL_BASE + 100)


// encoder
//
#define PLDEV_CNTL_IOCTL_SET_SYSTEM_CONTROL		_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_SYSTEM_CONTROL, QPCODEC_SYSTEM_CONTROL)
#define PLDEV_CNTL_IOCTL_SET_PICTURE_RESOLUTION	_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_PICTURE_RESOLUTION, QPCODEC_PICTURE_RESOLUTION)
#define PLDEV_CNTL_IOCTL_SET_INPUT_CONTROL		_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_INPUT_CONTROL, QPCODEC_INPUT_CONTROL)
#define PLDEV_CNTL_IOCTL_SET_RATE_CONTROL		_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_RATE_CONTROL, QPCODEC_RATE_CONTROL)
#define PLDEV_CNTL_IOCTL_SET_VBR_BIT_RATE		_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_VBR_BIT_RATE, QPCODEC_VBR_BIT_RATE)
#define PLDEV_CNTL_IOCTL_SET_GOP_LOOP_FILTER	_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_GOP_LOOP_FILTER, QPCODEC_GOP_LOOP_FILTER)
#define PLDEV_CNTL_IOCTL_SET_ET_CONTROL			_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_ET_CONTROL, QPCODEC_ET_CONTROL)
#define PLDEV_CNTL_IOCTL_SET_FILTER_CONTROL		_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_FILTER_CONTROL, QPCODEC_FILTER_CONTROL)
#define PLDEV_CNTL_IOCTL_SET_OUT_PICTURE_RESOLUTION	_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_OUT_PICTURE_RESOLUTION, QPCODEC_OUT_PICTURE_RESOLUTION)
#define PLDEV_CNTL_IOCTL_SET_BLOCK_SIZE			_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_BLOCK_SIZE, QPCODEC_BLOCK_SIZE)
#define PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM	_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_AUDIO_CONTROL_PARAM, QPCODEC_AUDIO_CONTROL_PARAM)
#define PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_EX	_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_AUDIO_CONTROL_EX, QPCODEC_AUDIO_CONTROL_EX)
#define PLDEV_CNTL_IOCTL_SET_REL_CUR_VID_BUF	_IO(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_REL_CUR_VID_BUF)
#define PLDEV_CNTL_IOCTL_SET_INSERT_I_FRAME		_IO(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_INSERT_I_FRAME)
#define PLDEV_CNTL_IOCTL_SET_INSERT_USER_DATA	_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_INSERT_USER_DATA, QPCODEC_INSERT_USER_DATA)
#define PLDEV_CNTL_IOCTL_SET_MP4_VID_NUM_BLOCK	_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_MP4_VID_NUM_BLOCK, QPCODEC_MP4_VID_NUM_BLOCK)
#define PLDEV_CNTL_IOCTL_SET_AUDIO_ENHANCEMENT	_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_AUDIO_ENHANCEMENT, QPCODEC_AUDIO_ENHANCEMENT)
#define PLDEV_CNTL_IOCTL_SET_ENC_STOP_MODE		_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_ENC_STOP_MODE, QPCODEC_ENC_STOP_MODE)
#define PLDEV_CNTL_IOCTL_SET_INDEX_CAP_FREQ		_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_INDEX_CAP_FREQ, QPCODEC_INDEX_CAP_FREQ)
#define PLDEV_CNTL_IOCTL_SET_ENC_VID_PADDING	_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_ENC_VID_PADDING, QPCODEC_ENC_VID_PADDING)
#define PLDEV_CNTL_IOCTL_SET_ENC_FREEZE_VIDEO	_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_ENC_FREEZE_VIDEO, QPCODEC_ENC_FREEZE_VIDEO)
#define PLDEV_CNTL_IOCTL_SET_ENC_STILL_VIDEO_INPUT	_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_ENC_STILL_VIDEO_INPUT, QPCODEC_ENC_STILL_VIDEO_INPUT)
#define PLDEV_CNTL_IOCTL_SET_ENC_CAP_MODE		_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_ENC_CAP_MODE, QPCODEC_ENC_CAP_MODE)
#define PLDEV_CNTL_IOCTL_SET_ENC_GET_A_FRAME	_IO(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_ENC_GET_A_FRAME)
#define PLDEV_CNTL_IOCTL_SET_ENC_MJPEG_QUALITY	_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_ENC_MJPEG_QUALITY, TQP_UINT32)
#define PLDEV_CNTL_IOCTL_SET_ENC_MJPEG_FRAME_BUF	_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_ENC_MJPEG_FRAME_BUF, TQP_UINT32)
#define PLDEV_CNTL_IOCTL_SET_ENC_ALL_SETTINGS	_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_ENC_ALL_SETTINGS, QPCODEC_ENC_ALL_SETTINGS)
#define PLDEV_CNTL_IOCTL_SET_ENC_OSD_SETTINGS	_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_ENC_OSD_SETTINGS, TQP_UINT32)
#define PLDEV_CNTL_IOCTL_SET_ENC_AUD_INPUT_VOL	_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_AUDIO_INPUT_VOLUME, TQP_UINT32)

#define PLDEV_CNTL_IOCTL_GET_SYSTEM_CONTROL		_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_SYSTEM_CONTROL, QPCODEC_SYSTEM_CONTROL)
#define PLDEV_CNTL_IOCTL_GET_PICTURE_RESOLUTION	_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_PICTURE_RESOLUTION, QPCODEC_PICTURE_RESOLUTION)
#define PLDEV_CNTL_IOCTL_GET_INPUT_CONTROL		_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_INPUT_CONTROL, QPCODEC_INPUT_CONTROL)
#define PLDEV_CNTL_IOCTL_GET_RATE_CONTROL		_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_RATE_CONTROL, QPCODEC_RATE_CONTROL)
#define PLDEV_CNTL_IOCTL_GET_VBR_BIT_RATE		_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_VBR_BIT_RATE, QPCODEC_VBR_BIT_RATE)
#define PLDEV_CNTL_IOCTL_GET_GOP_LOOP_FILTER	_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_GOP_LOOP_FILTER, QPCODEC_GOP_LOOP_FILTER)
#define PLDEV_CNTL_IOCTL_GET_ET_CONTROL			_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_ET_CONTROL, QPCODEC_ET_CONTROL)
#define PLDEV_CNTL_IOCTL_GET_FILTER_CONTROL		_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_FILTER_CONTROL, QPCODEC_FILTER_CONTROL)
#define PLDEV_CNTL_IOCTL_GET_OUT_PICTURE_RESOLUTION	_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_OUT_PICTURE_RESOLUTION, QPCODEC_OUT_PICTURE_RESOLUTION)
#define PLDEV_CNTL_IOCTL_GET_BLOCK_SIZE			_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_BLOCK_SIZE, QPCODEC_BLOCK_SIZE)
#define PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM	_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_AUDIO_CONTROL_PARAM, QPCODEC_AUDIO_CONTROL_PARAM)
#define PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX	_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_AUDIO_CONTROL_EX, QPCODEC_AUDIO_CONTROL_EX)
#define PLDEV_CNTL_IOCTL_GET_VIOSD_TABLE_ADDR	_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_VIOSD_TABLE_ADDR, QPCODEC_VIOSD)
#define PLDEV_CNTL_IOCTL_GET_GET_CUR_VID_BUF_INFO	_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_GET_CUR_VID_BUF_INFO, QPCODEC_GET_CUR_VID_BUF_INFO)
#define PLDEV_CNTL_IOCTL_GET_MP4_VID_NUM_BLOCK	_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_MP4_VID_NUM_BLOCK, QPCODEC_MP4_VID_NUM_BLOCK)
#define PLDEV_CNTL_IOCTL_GET_AUDIO_ENHANCEMENT	_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_AUDIO_ENHANCEMENT, QPCODEC_AUDIO_ENHANCEMENT)
#define PLDEV_CNTL_IOCTL_GET_ENC_STOP_MODE		_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_ENC_STOP_MODE, QPCODEC_ENC_STOP_MODE)
#define PLDEV_CNTL_IOCTL_GET_INDEX_CAP_FREQ		_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_INDEX_CAP_FREQ, QPCODEC_INDEX_CAP_FREQ)
#define PLDEV_CNTL_IOCTL_GET_ENC_VID_PADDING	_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_ENC_VID_PADDING, QPCODEC_ENC_VID_PADDING)
#define PLDEV_CNTL_IOCTL_GET_ENC_FREEZE_VIDEO	_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_ENC_FREEZE_VIDEO, QPCODEC_ENC_FREEZE_VIDEO)
#define PLDEV_CNTL_IOCTL_GET_ENC_STILL_VIDEO_INPUT	_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_ENC_STILL_VIDEO_INPUT, QPCODEC_ENC_STILL_VIDEO_INPUT)
#define PLDEV_CNTL_IOCTL_GET_ENC_CAP_MODE		_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_ENC_CAP_MODE, QPCODEC_ENC_CAP_MODE)
#define PLDEV_CNTL_IOCTL_GET_ENC_MJPEG_QUALITY	_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_ENC_MJPEG_QUALITY, TQP_UINT32)
#define PLDEV_CNTL_IOCTL_GET_ENC_MJPEG_FRAME_BUF	_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_ENC_MJPEG_FRAME_BUF, TQP_UINT32)
#define PLDEV_CNTL_IOCTL_GET_ENC_ALL_SETTINGS	_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_ENC_ALL_SETTINGS, QPCODEC_ENC_ALL_SETTINGS)
#define PLDEV_CNTL_IOCTL_GET_ENC_AUD_INPUT_VOL	_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_AUDIO_INPUT_VOLUME, TQP_UINT32)

// decoder
//
#define PLDEV_CNTL_IOCTL_SET_DEC_STREAM_ID		_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_DEC_STREAM_ID, QPCODEC_DEC_STREAM_ID_EX)
#define PLDEV_CNTL_IOCTL_SET_DEC_OUTPUT_MODE	_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_DEC_OUTPUT_MODE, TQP_UINT32)
#define PLDEV_CNTL_IOCTL_SET_DEC_RESOLUTION		_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_DEC_RESOLUTION, QPCODEC_PICTURE_RESOLUTION)
#define PLDEV_CNTL_IOCTL_SET_DEC_STOP_MODE		_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_DEC_STOP_MODE, QPCODEC_DEC_STOP_MODE)
#define PLDEV_CNTL_IOCTL_SET_DEC_TRICK_MODE		_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_DEC_TRICK_MODE, QPCODEC_DEC_TRICK_MODE)
#define PLDEV_CNTL_IOCTL_SET_DEC_DISP_ATTRIB	_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_DEC_DISP_ATTRIB, QPCODEC_DEC_DISP_ATTRIB)
#define PLDEV_CNTL_IOCTL_SET_DEC_DISP_MODE		_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_DEC_DISP_MODE, QPCODEC_DEC_DISP_MODE)
#define PLDEV_CNTL_IOCTL_SET_DEC_AUDIO_CONTROL	_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_DEC_AUDIO_CONTROL, QPCODEC_DEC_AUDIO_CONTROL)
#define PLDEV_CNTL_IOCTL_SET_DEC_DATA_XFER_CONTROL	_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_DEC_DATA_XFER_CONTROL, QPCODEC_DEC_DATA_XFER_CONTROL)
#define PLDEV_CNTL_IOCTL_SET_DEC_ALL_SETTINGS	_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_DEC_ALL_SETTINGS, QPCODEC_DEC_ALL_SETTINGS)
#define PLDEV_CNTL_IOCTL_SET_DEC_AUD_OUTPUT_VOL	_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_AUDIO_OUTPUT_VOLUME, TQP_UINT32)

#define PLDEV_CNTL_IOCTL_GET_DEC_STREAM_ID		_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_DEC_STREAM_ID, QPCODEC_DEC_STREAM_ID_EX)
#define PLDEV_CNTL_IOCTL_GET_DEC_OUTPUT_MODE	_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_DEC_OUTPUT_MODE, TQP_UINT32)
#define PLDEV_CNTL_IOCTL_GET_DEC_RESOLUTION		_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_DEC_RESOLUTION, QPCODEC_PICTURE_RESOLUTION)
#define PLDEV_CNTL_IOCTL_GET_DEC_STOP_MODE		_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_DEC_STOP_MODE, QPCODEC_DEC_STOP_MODE)
#define PLDEV_CNTL_IOCTL_GET_DEC_TRICK_MODE		_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_DEC_TRICK_MODE, QPCODEC_DEC_TRICK_MODE)
#define PLDEV_CNTL_IOCTL_GET_DEC_DISP_ATTRIB	_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_DEC_DISP_ATTRIB, QPCODEC_DEC_DISP_ATTRIB)
#define PLDEV_CNTL_IOCTL_GET_DEC_DISP_MODE		_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_DEC_DISP_MODE, QPCODEC_DEC_DISP_MODE)
#define PLDEV_CNTL_IOCTL_GET_DEC_GET_PLAY_INFO	_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_DEC_GET_PLAY_INFO, QPCODEC_DEC_GET_PLAY_INFO)
#define PLDEV_CNTL_IOCTL_GET_DEC_AUDIO_CONTROL	_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_DEC_AUDIO_CONTROL, QPCODEC_DEC_AUDIO_CONTROL)
#define PLDEV_CNTL_IOCTL_GET_DEC_DATA_XFER_CONTROL	_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_DEC_DATA_XFER_CONTROL, QPCODEC_DEC_DATA_XFER_CONTROL)
#define PLDEV_CNTL_IOCTL_GET_DEC_ALL_SETTINGS	_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_DEC_ALL_SETTINGS, QPCODEC_DEC_ALL_SETTINGS)
#define PLDEV_CNTL_IOCTL_GET_DEC_AUD_OUTPUT_VOL	_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_AUDIO_OUTPUT_VOLUME, TQP_UINT32)

// misc
//
#define PLDEV_CNTL_IOCTL_SET_BEGIN_CHANNEL_CHANGE	_IO(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_BEGIN_CHANNEL_CHANGE)
#define PLDEV_CNTL_IOCTL_SET_END_CHANNEL_CHANGE	_IO(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_END_CHANNEL_CHANGE)
#define PLDEV_CNTL_IOCTL_SET_656_OUT_CNTL		_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_656_OUT_CNTL, QPCODEC_OUT_CNTL)
#define PLDEV_CNTL_IOCTL_SET_I2S_OUT_CNTL		_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_I2S_OUT_CNTL, QPCODEC_OUT_CNTL)
#define PLDEV_CNTL_IOCTL_SET_VOU_CONFIG			_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_VOU_CONFIG, QPCODEC_VOU_CONFIG)
#define PLDEV_CNTL_IOCTL_SET_AOU_CONFIG			_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_AOU_CONFIG, QPCODEC_AOU_CONFIG)
#define PLDEV_CNTL_IOCTL_SET_POWER_STATE		_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_POWER_STATE, QPCODEC_POWER_STATE)
#define PLDEV_CNTL_IOCTL_SET_ERROR_RECOVERY		_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_ERROR_RECOVERY, QPCODEC_ERROR_RECOVERY)
#define PLDEV_CNTL_IOCTL_SET_OSD_MEMORY			_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_OSD_MEMORY, QPCODEC_OSD_MEM_CONFIG)
#define PLDEV_CNTL_IOCTL_SET_DECIMATION			_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_DECIMATION, QPCODEC_DECIMATION_FACTOR)
#define PLDEV_CNTL_IOCTL_SET_DEINTERLACE		_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_DEINTERLACE, TQP_UINT32)
#define PLDEV_CNTL_IOCTL_SET_EXTERNAL_TRIGGER	_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_EXTERNAL_TRIGGER_TO_SYNC, QPCODEC_EXTERNAL_TRIGGER_MODE)
#define PLDEV_CNTL_IOCTL_SET_PTS_COUNTER_RESET	_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_PTS_COUNTER_RESET, QPCODEC_PTS_COUNTER_RESET)
#define PLDEV_CNTL_IOCTL_SET_RATE_CONTROL_EX	_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_RATE_CONTROL_EX, RATE_CONTROL_EX)
#define PLDEV_CNTL_IOCTL_SET_LARGE_COMPRESS_BUFFER_CONTROL	_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_LARGE_COMPRESS_BUFFER_CONTROL, TQP_UINT32)
#define PLDEV_CNTL_IOCTL_SET_ENABLE_LOW_BITRATE_MODE		_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_ENABLE_LOW_BITRATE_MODE, TQP_UINT32)
#define PLDEV_CNTL_IOCTL_SET_ENABLE_AV_DISCARD_CONTROL		_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_ENABLE_AV_DISCARD_CONTROL, TQP_UINT32)
#define PLDEV_CNTL_IOCTL_SET_USE_SW_PTS			_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_USE_SW_PTS, TQP_UINT32)

#define PLDEV_CNTL_IOCTL_GET_ENCODER_STATUS		_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_ENCODER_STATUS, QPCODEC_CODEC_STATUS)
#define PLDEV_CNTL_IOCTL_GET_DECODER_STATUS		_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_DECODER_STATUS, QPCODEC_CODEC_STATUS)
#define PLDEV_CNTL_IOCTL_GET_656_OUT_CNTL		_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_656_OUT_CNTL, QPCODEC_OUT_CNTL)
#define PLDEV_CNTL_IOCTL_GET_I2S_OUT_CNTL		_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_I2S_OUT_CNTL, QPCODEC_OUT_CNTL)
#define PLDEV_CNTL_IOCTL_GET_VOU_CONFIG			_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_VOU_CONFIG, QPCODEC_VOU_CONFIG)
#define PLDEV_CNTL_IOCTL_GET_AOU_CONFIG			_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_AOU_CONFIG, QPCODEC_AOU_CONFIG)
#define PLDEV_CNTL_IOCTL_GET_POWER_STATE		_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_POWER_STATE, QPCODEC_POWER_STATE)
#define PLDEV_CNTL_IOCTL_GET_ERROR_RECOVERY		_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_ERROR_RECOVERY, QPCODEC_ERROR_RECOVERY)
#define PLDEV_CNTL_IOCTL_GET_OSD_MEMORY			_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_OSD_MEMORY, QPCODEC_OSD_MEM_CONFIG)
#define PLDEV_CNTL_IOCTL_GET_DECIMATION			_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_DECIMATION, QPCODEC_DECIMATION_FACTOR)
#define PLDEV_CNTL_IOCTL_GET_DEINTERLACE		_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_DEINTERLACE, TQP_UINT32)
#define PLDEV_CNTL_IOCTL_GET_EXTERNAL_TRIGGER	_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_EXTERNAL_TRIGGER_TO_SYNC, QPCODEC_EXTERNAL_TRIGGER_MODE)
#define PLDEV_CNTL_IOCTL_GET_PTS_COUNTER_RESET	_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_PTS_COUNTER_RESET, QPCODEC_PTS_COUNTER_RESET)
#define PLDEV_CNTL_IOCTL_GET_RATE_CONTROL_EX	_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_RATE_CONTROL_EX, PRATE_CONTROL_EX)
#define PLDEV_CNTL_IOCTL_GET_LARGE_COMPRESS_BUFFER_CONTROL	_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_LARGE_COMPRESS_BUFFER_CONTROL, TQP_UINT32)
#define PLDEV_CNTL_IOCTL_GET_ENABLE_LOW_BITRATE_MODE		_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_ENABLE_LOW_BITRATE_MODE, TQP_UINT32)
#define PLDEV_CNTL_IOCTL_GET_ENABLE_AV_DISCARD_CONTROL		_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_ENABLE_AV_DISCARD_CONTROL, TQP_UINT32)
#define PLDEV_CNTL_IOCTL_GET_USE_SW_PTS			_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_USE_SW_PTS, TQP_UINT32)

// gpio
//
#define PLDEV_CNTL_IOCTL_SET_GPIO_BIT_DIRECTION	_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_GPIO_BIT_DIRECTION, GPIO_BIT_DIRECTION)
#define PLDEV_CNTL_IOCTL_SET_GPIO_BIT_VALUE		_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_GPIO_BIT_VALUE, GPIO_BIT_VALUE)

#define PLDEV_CNTL_IOCTL_GET_GPIO_BIT_DIRECTION	_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_GPIO_BIT_DIRECTION, GPIO_BIT_DIRECTION)
#define PLDEV_CNTL_IOCTL_GET_GPIO_BIT_VALUE		_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_GPIO_BIT_VALUE, GPIO_BIT_VALUE)


// codec
//
#define PLDEV_CNTL_IOCTL_SET_SYS_FUNCTION		_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_SYS_FUNCTION, QPCODEC_SYS_FUNCTION)
#define PLDEV_CNTL_IOCTL_SET_SYS_LINK			_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_SYS_LINK, QPCODEC_SYS_LINK)
#define PLDEV_CNTL_IOCTL_SET_SYS_CONFIG			_IOW_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_SET + QPCODEC_PROP_SYS_CONFIG, QPCODEC_SYS_CONFIG)

#define PLDEV_CNTL_IOCTL_GET_SYS_FUNCTION		_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_SYS_FUNCTION, QPCODEC_SYS_FUNCTION)
#define PLDEV_CNTL_IOCTL_GET_SYS_LINK			_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_SYS_LINK, QPCODEC_SYS_LINK)
#define PLDEV_CNTL_IOCTL_GET_SYS_CONFIG			_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_SYS_CONFIG, QPCODEC_SYS_CONFIG)

#define PLDEV_CNTL_IOCTL_GET_CODEC_ERROR		_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_IS_CODEC_ERROR, TQP_UINT32)
// Low level version
#define PLDEV_CNTL_IOCTL_GET_CTRL_VERSION		_IOWR_BAD(PLDEV_CNTL_IOCTL_TYPE, PLDEV_CNTL_IOCTL_GET + QPCODEC_PROP_CTRL_VERSION, QPCODEC_CTRL_VERSION)


/////////////////////////////////////////////////////////////////////////////
//
//

// diagnostic interface
//

#define PLDEV_DIAG_IOCTL_I2C_WRITE				_IOW_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_DIAG_I2C_WRITE, QPCODEC_DIAG_I2C_STRUCT)
#define PLDEV_DIAG_IOCTL_I2C_READ				_IOWR_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_DIAG_I2C_READ, QPCODEC_DIAG_I2C_STRUCT)
#define PLDEV_DIAG_IOCTL_I2C_WRITE_THEN_READ	_IOWR_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_DIAG_I2C_WRITE_THEN_READ, QPCODEC_DIAG_I2C_WRITE_THEN_READ_STRUCT)
#define PLDEV_DIAG_IOCTL_MEM_READ				_IOWR_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_DIAG_MEM_READ, QPCODEC_DIAG_MEMORY_STRUCT)
#define PLDEV_DIAG_IOCTL_MEM_WRITE				_IOW_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_DIAG_MEM_WRITE, QPCODEC_DIAG_MEMORY_STRUCT)
#define PLDEV_DIAG_IOCTL_REG_READ				_IOWR_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_DIAG_REG_READ, QPCODEC_DIAG_REGISTER_STRUCT)
#define PLDEV_DIAG_IOCTL_REG_WRITE				_IOW_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_DIAG_REG_WRITE, QPCODEC_DIAG_REGISTER_STRUCT)
#define PLDEV_DIAG_IOCTL_HCI_REG_READ			_IOWR_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_DIAG_HCI_REG_READ, QPCODEC_DIAG_REGISTER_STRUCT)
#define PLDEV_DIAG_IOCTL_HCI_REG_WRITE			_IOW_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_DIAG_HCI_REG_WRITE, QPCODEC_DIAG_REGISTER_STRUCT)
#define PLDEV_DIAG_IOCTL_FIRMWARE_COMMAND		_IOWR_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_DIAG_FIRMWARE_COMMAND, QPCODEC_DIAG_FW_COMMAND)
#define PLDEV_DIAG_IOCTL_FIRMWARE_DOWNLOAD_PTR	_IOW_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_DIAG_FIRMWARE_DOWNLOAD_PTR, QPCODEC_DIAG_FW_DOWNLOAD_PTR)
#define PLDEV_DIAG_IOCTL_DMA_READ_PTR			_IOWR_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_DIAG_DMA_READ_PTR, QPCODEC_DIAG_DMA_PTR_STRUCT)
#define PLDEV_DIAG_IOCTL_DMA_WRITE_PTR			_IOW_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_DIAG_DMA_WRITE_PTR, QPCODEC_DIAG_DMA_PTR_STRUCT)
#define PLDEV_DIAG_IOCTL_ENABLE_DRIVER_I2C		_IO(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_DIAG_ENABLE_DRIVER_I2C)
#define PLDEV_DIAG_IOCTL_DISABLE_DRIVER_I2C		_IO(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_DIAG_DISABLE_DRIVER_I2C)

#ifndef CONFIG_HOST_PLATFORM_USERSPACE
#define	SUPPORT_PL3XX_OSD
#endif // CONFIG_HOST_PLATFORM_USERSPACE

#ifdef	SUPPORT_PL3XX_OSD
#define	SUPPORT_VO_OSD
#define	SUPPORT_VI_OSD
#define PLDEV_DIAG_IOCTL_OSD_CONFIG				_IOWR_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_DIAG_OSD_CONFIG, QLDEV_OSD_PARAMETER)
#define PLDEV_DIAG_IOCTL_OSD_CONTROL			_IOWR_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_DIAG_OSD_CONTROL, QLDEV_OSD_CONTROL)
#define PLDEV_DIAG_IOCTL_VIOSD_CONTROL			_IOWR_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_DIAG_VIOSD_CONTROL, QPCODEC_VIOSD)
#endif

#define PLDEV_DIAG_IOCTL_VIDEOPROP_SET		_IOW_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_DIAG_VIDEOPROP_SET, QLDEV_VIDEOPROP_PARAMETER)
#define PLDEV_DIAG_IOCTL_VIDEOPROP_GET		_IOWR_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_DIAG_VIDEOPROP_GET, QLDEV_VIDEOPROP_PARAMETER)

#define PLDEV_DIAG_IOCTL_VIDEO_DECODER_SOURCE_SET		_IOW_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_VIDEO_DECODER_CHANNEL, TQP_UINT32)
#define PLDEV_DIAG_IOCTL_VIDEO_DECODER_SOURCE_GET		_IOWR_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_VIDEO_DECODER_CHANNEL_GET, TQP_UINT32)

#define PLDEV_DIAG_IOCTL_VIDEO_DECODER_SET		_IOW_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_VIDEO_DECODER, TQP_UINT32)
#define PLDEV_DIAG_IOCTL_VIDEO_DECODER_GET		_IOWR_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_VIDEO_DECODER_GET, TQP_UINT32)

#define PLDEV_DIAG_IOCTL_I2C_WRITE_EX			_IOW_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_DIAG_I2C_WRITE_EX, QPCODEC_DIAG_I2C_STRUCT)
#define PLDEV_DIAG_IOCTL_I2C_READ_EX			_IOWR_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_DIAG_I2C_READ_EX, QPCODEC_DIAG_I2C_STRUCT)
#define PLDEV_DIAG_IOCTL_I2C_WRITE_THEN_READ_EX	_IOWR_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_DIAG_I2C_WRITE_THEN_READ_EX, QPCODEC_DIAG_I2C_WRITE_THEN_READ_STRUCT)

#define PLDEV_DIAG_IOCTL_FIRMWARE_RELOAD		_IOW_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_DIAG_FIRMWARE_RELOAD, TQP_UINT32)
#define PLDEV_DIAG_IOCTL_RESET_ARM			_IOW_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_DIAG_RESET_ARM, TQP_UINT32)

#define PLDEV_DIAG_IOCTL_DEBUGLEVEL_SET		_IOW_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_DIAG_DEBUGLEVEL_SET, TQP_UINT32)
#define PLDEV_DIAG_IOCTL_DEBUGLEVEL_GET		_IOWR_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_DIAG_DEBUGLEVEL_GET, TQP_UINT32)

#define PLDEV_DIAG_IOCTL_FIRMWARE_SWITCH_MODE		_IOW_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_DIAG_FIRMWARE_SWITCH_MODE, TQP_UINT32)

#define PLDEV_DIAG_IOCTL_DRVPROFILING_GET	_IOWR_BAD(PLDEV_DIAG_IOCTL_TYPE, PLDEV_DIAG_IOCTL_BASE + QPCODEC_DIAG_DRVPROFILING_GET, QPCODEC_DIAG_PROFILING_PTR_STRUCT)

#endif //__PLDEVIOCTL_H__





