///////////////////////////////////////////////////////////////////////////////
//     
//
// Ccomponent.c
//
// Description:
//
//  component implementation
//
// History:
//	Jan 14, 2010	(dt) created
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#include "qlautoconf.h"

#ifndef CONFIG_HOST_PLATFORM_USERSPACE
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/delay.h>
#include <linux/version.h>
#include <linux/mm.h>
#include <asm/uaccess.h>

#include <linux/vmalloc.h>
#include <linux/slab.h>

#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
#include <mach/pl330xp.h>
#endif
#else // CONFIG_HOST_PLATFORM_USERSPACE
//#include <sys/user.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <inttypes.h>
#include "pldevioctl.h"
#include "uservideodev2.h"
#include "userwrap.h"
#endif // CONFIG_HOST_PLATFORM_USERSPACE

#include "qpdbgmgr.h"
#include "cobjectmgr.h"
#include "cdevice.h"
#include "ccomponent.h"
#ifndef CONFIG_HOST_PLATFORM_USERSPACE
#include "qplinuxkermap.h"
#endif

#ifdef SUPPORT_PL3XX_OSD
#include "qlosd.h"
#endif
#include "qlvideoprop.h"
#include "plprofiling.h"
#include "plv4ldevversion.h"

extern TQP_DbgMCompID	g_PLV4LDevDBGCompID;
extern TQP_DbgMCompID	g_QPCodecDBGCompID;
extern TQP_DbgMCompID	g_QPUsbDBGCompID;
extern TQP_DbgMCompID	g_QPPCIeDBGCompID;
extern TQP_DbgMCompID	g_QPPeripheralDBGCompID;
//extern TQP_DbgMCompID	PaulDBGCompID;


extern struct vm_operations_struct	plv4l_mmap_ops;


/////////////////////////////////////////////////////////////////////////////
//
//
static PORT_DIRECTION CComponent_PortGetDirection(COMPONENT_PORT_TYPE PortType)
{
    switch (PortType)
    {
        case COMPONENT_PORT_COMP_OUT:
        case COMPONENT_PORT_COMP_AUD_OUT:
        case COMPONENT_PORT_INDEX_OUT:
        case COMPONENT_PORT_YUV_OUT:
        case COMPONENT_PORT_PCM_OUT:
        case COMPONENT_PORT_VIRTUAL_OUT:
        case COMPONENT_PORT_ACTIVITY_OUT:
            return PORT_DIR_READ;
        case COMPONENT_PORT_YUV_IN:
        case COMPONENT_PORT_COMP_IN:
        case COMPONENT_PORT_COMP_AUD_IN:
        case COMPONENT_PORT_PCM_IN:
        case COMPONENT_PORT_VIRTUAL_IN:
        default:
            return PORT_DIR_WRITE;
    }
}

static EQP_Errors CComponent_PortOpen(CComponent *This,
									  PCOMPONENT_PORT_OPEN pOpenParam
									  )
{
	EQP_Errors					qpStatus;
	CPort						*pPort;
	PORT_DIRECTION				dir;
	TQP_UINT32					dwOpenType;
	PCOMPONENT_PORT_OPEN_FORMAT	pOpenFormat = QP_NULL;
	TQP_UINT32					frame_nbs;
	TQP_UINT32					frame_size;
	TQP_BOOL					bBufferPartialFill;
	TQP_BOOL					bBufferFrameAligned;
	TQP_BOOL					bSupportUserPtr;

	TQP_BOOL					bForceFullBuffer = QP_FALSE;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
				 "%s hTask[%d] Type[%d] [\n",
				 __FUNCTION__,
				 This->m_hTask,
				 pOpenParam ? pOpenParam->type : -1
				 ));

	PROFILING_LOG(EVENT_COMP_PORTOPEN, (TQP_UINT32) This->m_hTask, (TQP_UINT32) pOpenParam ? pOpenParam->type : -1, 0, 0, (TQP_UINT32) pOpenParam ? pOpenParam->type : -1);

 	if ((pOpenParam) && (pOpenParam->type & COMPONENT_PORT_TYPE_FORCE_FULL_BUFFER))
    {
		bForceFullBuffer = QP_TRUE;
		pOpenParam->type &= ~COMPONENT_PORT_TYPE_FORCE_FULL_BUFFER;
	}

	// validate parameters
	//
	if (!pOpenParam ||
		(pOpenParam->type >= MAX_COMPONENT_PORT)
		)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
					 "%s pOpenParam(0x%x) type(%d)\n",
					 __FUNCTION__,
					 pOpenParam,
					 pOpenParam ? pOpenParam->type : -1
					 ));
		return (QPERR_PARMS);
	}

	ENTER_CRITICAL(&This->m_Object)

	pPort = &This->m_Ports[CComponent_PortGetDirection(pOpenParam->type)];

	if (pPort->m_valid)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
					 "%s port(%d) opened already\n",
					 __FUNCTION__,
					 pOpenParam->type
					 ));
		LEAVE_CRITICAL(&This->m_Object)
		return (QPERR_INVALID);
	}

	// only allow user pointer for PCIe bus
	//
#if LINUX_VERSION_CODE < KERNEL_VERSION(3,7,0)
	bSupportUserPtr = (PLV4LDEV_TYPE_QL300_PCIE_EVK == This->m_pDevice->m_type);
//	bSupportUserPtr = QP_FALSE;
#else
	bSupportUserPtr = QP_FALSE;
#endif /* LINUX_VERSION_CODE < KERNEL_VERSION(3,7,0) */

	// setup port specific parameters
	//
	switch (pOpenParam->type)
	{
		case COMPONENT_PORT_YUV_IN:
			dir = PORT_DIR_WRITE;
			dwOpenType = QPMPGCODEC_ENC_YUV_IN;
			pOpenFormat = &pOpenParam->format;
			frame_nbs = 4;
			frame_size = (pOpenParam->format.yuv.nWidth *
						 pOpenParam->format.yuv.nHeight *
						 pOpenParam->format.yuv.nBitCount) / 8
						 ;
			frame_size = (frame_size + PAGE_SIZE - 1) & PAGE_MASK;
			bBufferPartialFill = QP_FALSE;
			bBufferFrameAligned = QP_TRUE;
			break;

		case COMPONENT_PORT_COMP_IN:
			dir = PORT_DIR_WRITE;
			dwOpenType = QPMPGCODEC_DEC_MPEG_IN;
			pOpenFormat = &pOpenParam->format;
#ifdef CONFIG_HOST_PLATFORM_ASPEN
			frame_nbs = 2;
#else
			frame_nbs = QPCODECLIB_MAX_DATA_ENTRIES / 16;
#endif
			frame_size = 1024 * 1024;
			bBufferFrameAligned = QP_FALSE;
			bBufferPartialFill = QP_TRUE;
			// disable user pointer support for this port
			// because it may need to access user data
			// from the driver
			//
			bSupportUserPtr = QP_FALSE;
			break;

		case COMPONENT_PORT_COMP_AUD_IN:
			dir = PORT_DIR_WRITE;
			dwOpenType = QPMPGCODEC_DEC_AES_IN;
			frame_nbs = QPCODECLIB_MAX_DATA_ENTRIES / 4;
			frame_size = 8 * 1024;
			if (frame_size < PAGE_SIZE)
				frame_size = PAGE_SIZE;
			bBufferFrameAligned = QP_FALSE;
			bBufferPartialFill = QP_TRUE;
			// disable user pointer support for this port
			// because it may need to access user data
			// from the driver
			//
			bSupportUserPtr = QP_FALSE;
			break;

		case COMPONENT_PORT_COMP_OUT:
			dir = PORT_DIR_READ;
			dwOpenType = QPMPGCODEC_ENC_MPEG_OUT;
			pOpenFormat = &pOpenParam->format;
			if (QP_COMP_VID_DATA_TYPE_MJPEG == pOpenParam->format.vid.nDataType)
			{
#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
				frame_nbs = 1;
#else
				frame_nbs = 8;
#endif
				frame_size = 1024 * 1024;
				bBufferFrameAligned = QP_TRUE;
			}
			else
			{
				frame_nbs = QPCODECLIB_MAX_DATA_ENTRIES / 16;
#if defined (_QP_ANDROID)
				frame_size = 16 * 1024;
#else // _QP_ANDROID
#ifdef CONFIG_HOST_PLATFORM_ASPEN
				frame_size = 256 * 1024;
#else
				frame_size = 64 * 1024;
#endif
#endif // _QP_ANDROID
				bBufferFrameAligned = QP_FALSE;
			}

			if (bForceFullBuffer) {
				bBufferPartialFill = QP_FALSE;
			} else {
				bBufferPartialFill = QP_TRUE;
			}
			break;

		case COMPONENT_PORT_COMP_AUD_OUT:
			dir = PORT_DIR_READ;
			dwOpenType = QPMPGCODEC_ENC_AES_OUT;
			frame_nbs = QPCODECLIB_MAX_DATA_ENTRIES / 4;
			frame_size = 8 * 1024;
			if (frame_size < PAGE_SIZE)
				frame_size = PAGE_SIZE;
			bBufferFrameAligned = QP_TRUE;
			bBufferPartialFill = QP_TRUE;
			break;

		case COMPONENT_PORT_INDEX_OUT:
			dir = PORT_DIR_READ;
			dwOpenType = QPMPGCODEC_ENC_INDEX_OUT;
			frame_nbs = 128;
			frame_size = 30 * 20;
			bBufferFrameAligned = QP_FALSE;
			bBufferPartialFill = QP_TRUE;
			break;

		case COMPONENT_PORT_YUV_OUT:
			dir = PORT_DIR_READ;
			dwOpenType = (COMPONENT_TYPE_ENCODE == This->m_type) ? QPMPGCODEC_ENC_YUV_OUT : QPMPGCODEC_DEC_YUV_OUT;
			pOpenFormat = &pOpenParam->format;
			frame_nbs = 4;
			frame_size = (pOpenParam->format.yuv.nWidth *
						 pOpenParam->format.yuv.nHeight *
						 pOpenParam->format.yuv.nBitCount) / 8
						 ;
			frame_size = (frame_size + PAGE_SIZE - 1) & PAGE_MASK;
			bBufferPartialFill = QP_FALSE;
			bBufferFrameAligned = QP_TRUE;
			break;

		case COMPONENT_PORT_PCM_OUT:
		{
			dir = PORT_DIR_READ;
			dwOpenType = (COMPONENT_TYPE_ENCODE == This->m_type) ? QPMPGCODEC_ENC_PCM_OUT : QPMPGCODEC_DEC_PCM_OUT;
			pOpenFormat = &pOpenParam->format;
			frame_nbs = 64;

			if (COMPONENT_TYPE_ENCODE == This->m_type)
			{
				IMpegCodec					*pMpegCodec = CDevice_getMpegCodec(This->m_pDevice);
				QPCODEC_AUDIO_CONTROL_PARAM	audCntl;
				QPCODEC_AUDIO_CONTROL_EX	audCntlEx;
				TQP_UINT32					dwSize;

				pMpegCodec->Get(pMpegCodec,
								(TQP_REFGUID)&PROPSETID_QPCODEC_CONTROL,
								This->m_hTask,
								QPCODEC_PROP_AUDIO_CONTROL_PARAM,
								QP_NULL,
								&audCntl,
								&dwSize
								);

				pMpegCodec->Get(pMpegCodec,
								(TQP_REFGUID)&PROPSETID_QPCODEC_CONTROL,
								This->m_hTask,
								QPCODEC_PROP_AUDIO_CONTROL_EX,
								QP_NULL,
								&audCntlEx,
								&dwSize
								);

				frame_size = audCntlEx.sample_cnt * audCntl.ch_num * 2;
				frame_size = (frame_size + PAGE_SIZE - 1) & PAGE_MASK;
				bBufferPartialFill = QP_TRUE;
			}
			else
			{
				frame_size = PAGE_SIZE;
				bBufferPartialFill = QP_FALSE;
			}
			bBufferFrameAligned = QP_FALSE;
			break;
		}

		case COMPONENT_PORT_ACTIVITY_OUT:
			dir = PORT_DIR_READ;
			dwOpenType = QPMPGCODEC_ENC_ACTIVITY_OUT;
			pOpenFormat = &pOpenParam->format;
			frame_nbs = 4;
			frame_size = ((pOpenParam->format.activity.nWidth/16) *
						 (pOpenParam->format.activity.nHeight/16) *
						 pOpenParam->format.activity.nBitCount) / 8
						 ;
			frame_size = (frame_size + PAGE_SIZE - 1) & PAGE_MASK;

			bBufferFrameAligned = QP_FALSE;
			bBufferPartialFill = QP_TRUE;
			break;

		case COMPONENT_PORT_PCM_IN:
			dir = PORT_DIR_WRITE;
			dwOpenType = QPMPGCODEC_ENC_PCM_IN;
#if defined(CONFIG_HOST_PLATFORM_SMP8644_REF) || defined(CONFIG_HOST_PLATFORM_SMP8654_REF)
			frame_nbs = QPCODECLIB_MAX_DATA_ENTRIES / 8;
			frame_size = 16 * 1024;
#else
			frame_nbs = QPCODECLIB_MAX_DATA_ENTRIES / 4;
			frame_size = 4 * 1024;
#endif
			if (frame_size < PAGE_SIZE)
				frame_size = PAGE_SIZE;
			bBufferFrameAligned = QP_FALSE;
			bBufferPartialFill = QP_FALSE;
			break;

		case COMPONENT_PORT_VIRTUAL_IN:
		case COMPONENT_PORT_VIRTUAL_OUT:
			dir = PORT_DIR_NONE;
			dwOpenType = QPMPGCODEC_CODEC_VIRTUAL;
			frame_nbs = 0;
			frame_size = 0;
			bBufferFrameAligned = QP_FALSE;
			bBufferPartialFill = QP_TRUE;
			break;

		default:
			LEAVE_CRITICAL(&This->m_Object)
			return (QPERR_PARMS);
	}

	// instantiate the port
	//
	if (!CPort_Constructor(pPort,
						   This,
						   OBJECT_CRITICAL_HEAVY,
						   pOpenParam->type,
						   dir,
						   dwOpenType,
						   pOpenFormat,
						   pOpenParam->memory,
						   (V4L2_MEMORY_MMAP == pOpenParam->memory) ? frame_nbs : QPCODECLIB_MAX_DATA_ENTRIES,
						   frame_size,
						   bBufferPartialFill,
						   bBufferFrameAligned,
						   bSupportUserPtr
						   ))
    {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
					 "%s CPort_Constructor() failed\n",
					 __FUNCTION__
					 ));
		LEAVE_CRITICAL(&This->m_Object)
		return (QPERR_FAIL);
	}

	qpStatus = CPort_Create(pPort);
    if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
					 "%s CPort_Create() failed status(%d)\n",
					 __FUNCTION__,
					 qpStatus
					 ));
		CPort_Destructor(pPort);
	}

	LEAVE_CRITICAL(&This->m_Object)

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
				 "%s status(%d) ]\n",
				 __FUNCTION__,
				 qpStatus
				 ));

	return (qpStatus);
}



static EQP_Errors CComponent_PortClose(CComponent *This,
									   COMPONENT_PORT_TYPE type
									   )
{
	CPort	*pPort;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
				 "%s hTask[%d] Type[%d] [\n",
				 __FUNCTION__,
				 This->m_hTask,
				 type
				 ));

	if ((type >= MAX_COMPONENT_PORT) ||
		(type < 0)
		)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
					 "%s invalid type(%d)\n",
					 __FUNCTION__,
					 type
					 ));
		return (QPERR_PARMS);
	}

	ENTER_CRITICAL(&This->m_Object)

	pPort = &This->m_Ports[CComponent_PortGetDirection(type)];

	if (pPort->m_valid)
	{
		CPort_Close(pPort);
		CPort_Destructor(pPort);
	}

	LEAVE_CRITICAL(&This->m_Object)
	return (QPERR_SUCCESS);
}



static EQP_Errors CComponent_Start(CComponent *This)
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	CPort		*pPort;
	int			i;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
				 "%s hTask[%d] [\n",
				 __FUNCTION__,
				 This->m_hTask
				 ));

	PROFILING_LOG(EVENT_COMP_START, (TQP_UINT32) This->m_hTask, This->m_type, 0, 0, 0);

	ENTER_CRITICAL(&This->m_Object)

	// from stop to acquire
	//
	for (i = 0; i < MAX_PORT_DIRECTION/*MAX_COMPONENT_PORT*/; i++)
	{
		pPort = &This->m_Ports[i];
		if (pPort->m_valid)
		{
			if (PORT_STATE_STOP == pPort->m_State)
			{
				qpStatus = CPort_SetState(pPort,
										  PORT_STATE_ACQUIRE,
										  PORT_STATE_STOP
										  );
				if (!QP_SUCCESS(qpStatus))
				{
					goto CComponent_Start_Exit;
				}
			}
		}
	}

	// from acquire to pause
	//
	for (i = 0; i < MAX_PORT_DIRECTION/*MAX_COMPONENT_PORT*/; i++)
	{
		pPort = &This->m_Ports[i];
		if (pPort->m_valid)
		{
			if (PORT_STATE_ACQUIRE == pPort->m_State)
			{
				qpStatus = CPort_SetState(pPort,
										  PORT_STATE_PAUSE,
										  PORT_STATE_ACQUIRE
										  );
				if (!QP_SUCCESS(qpStatus))
				{
					goto CComponent_Start_Exit;
				}
			}
		}
	}

	// from pause to run
	//
	for (i = 0; i < MAX_PORT_DIRECTION/*MAX_COMPONENT_PORT*/; i++)
	{
		pPort = &This->m_Ports[i];
		if (pPort->m_valid)
		{
			qpStatus = CPort_SetState(pPort,
									  PORT_STATE_RUN,
									  pPort->m_State
									  );
			if (!QP_SUCCESS(qpStatus))
			{
				goto CComponent_Start_Exit;
			}
		}
	}

CComponent_Start_Exit:

	if (!QP_SUCCESS(qpStatus))
	{
		for (i = 0; i < MAX_PORT_DIRECTION/*MAX_COMPONENT_PORT*/; i++)
		{
			pPort = &This->m_Ports[i];
			if (pPort->m_valid)
			{
				CPort_SetState(pPort,
		 					   PORT_STATE_STOP,
		 					   pPort->m_State
		 					   );
			}
		}
	}

	LEAVE_CRITICAL(&This->m_Object)

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
				 "%s status(%d) ]\n",
				 __FUNCTION__,
				 qpStatus
				 ));

	return (qpStatus);
}



static EQP_Errors CComponent_Stop(CComponent *This)
{
	CPort	*pPort;
	int		i;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
				 "%s hTask[%d] [\n",
				 __FUNCTION__,
				 This->m_hTask
				 ));

	PROFILING_LOG(EVENT_COMP_STOP,(TQP_UINT32) This->m_hTask, This->m_type, 0, 0, 0);

	ENTER_CRITICAL(&This->m_Object)

	// stop all ports
	//
	for (i = 0; i < MAX_PORT_DIRECTION/*MAX_COMPONENT_PORT*/; i++)
	{
		pPort = &This->m_Ports[i];
		if (pPort->m_valid)
		{
			CPort_SetState(pPort,
		 				   PORT_STATE_STOP,
		 				   pPort->m_State
		 				   );
		}
	}

	LEAVE_CRITICAL(&This->m_Object)

	// always succeed
	return (QPERR_SUCCESS);
}



static EQP_Errors CComponent_Acquire(CComponent *This)
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	CPort		*pPort;
	int			i;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
				 "%s hTask[%d] [\n",
				 __FUNCTION__,
				 This->m_hTask
				 ));

	PROFILING_LOG(EVENT_COMP_ACQUIRE, (TQP_UINT32) This->m_hTask, 0, 0, 0, 0);

	ENTER_CRITICAL(&This->m_Object)

	// from stop to acquire
	//
	for (i = 0; i < MAX_PORT_DIRECTION/*MAX_COMPONENT_PORT*/; i++)
	{
		pPort = &This->m_Ports[i];
		if (pPort->m_valid)
		{
			if (PORT_STATE_STOP == pPort->m_State)
			{
				qpStatus = CPort_SetState(pPort,
										  PORT_STATE_ACQUIRE,
										  PORT_STATE_STOP
										  );
				if (!QP_SUCCESS(qpStatus))
				{
					goto CComponent_Acquire_Exit;
				}
			}
		}
	}

	// from acquire to pause
	//
	for (i = 0; i < MAX_PORT_DIRECTION/*MAX_COMPONENT_PORT*/; i++)
	{
		pPort = &This->m_Ports[i];
		if (pPort->m_valid)
		{
			if (PORT_STATE_ACQUIRE == pPort->m_State)
			{
				qpStatus = CPort_SetState(pPort,
										  PORT_STATE_PAUSE,
										  PORT_STATE_ACQUIRE
										  );
				if (!QP_SUCCESS(qpStatus))
				{
					goto CComponent_Acquire_Exit;
				}
			}
		}
	}

CComponent_Acquire_Exit:

	if (!QP_SUCCESS(qpStatus))
	{
		for (i = 0; i < MAX_PORT_DIRECTION/*MAX_COMPONENT_PORT*/; i++)
		{
			pPort = &This->m_Ports[i];
			if (pPort->m_valid)
			{
				CPort_SetState(pPort,
		 					   PORT_STATE_STOP,
		 					   pPort->m_State
		 					   );
			}
		}
	}

	LEAVE_CRITICAL(&This->m_Object)

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
				 "%s status(%d) ]\n",
				 __FUNCTION__,
				 qpStatus
				 ));

	return (qpStatus);
}



/////////////////////////////////////////////////////////////////////////////
//
//


// constructor
//
CComponent * CComponent_Constructor(CComponent *pComponent,
#ifndef CONFIG_HOST_PLATFORM_USERSPACE
									struct video_device *pVideoDev,
#else
									struct user_video_device *pVideoDev,
#endif // CONFIG_HOST_PLATFORM_USERSPACE
									CDevice *pDevice,
									COMPONENT_TYPE type
									)
{
	if (pComponent)
	{
		int	i;

		// CObject
		//
		CObject_Constructor(&pComponent->m_Object,
							QP_NULL,
		   					OBJECT_CRITICAL_LIGHT
		   					);

		// initialize members
		//
		pComponent->m_pVideoDev = pVideoDev;
		pComponent->m_pDevice = pDevice;
		pComponent->m_hTask = TQP_NULL_HANDLE;
		pComponent->m_hObject = TQP_NULL_HANDLE;
		pComponent->m_type = type;

		for (i = 0; i < MAX_PORT_DIRECTION/*MAX_COMPONENT_PORT*/; i++)
		{
			memset(&pComponent->m_Ports[i],
				   0,
				   sizeof(CPort)
				   );
			pComponent->m_Ports[i].m_valid = QP_FALSE;
		}
#ifndef CONFIG_HOST_PLATFORM_USERSPACE
		init_waitqueue_head(&pComponent->wait_frame);
#endif // CONFIG_HOST_PLATFORM_USERSPACE
	}

	return (pComponent);
}



// destructor
//
void CComponent_Destructor(CComponent *This)
{
 	CObject_Destructor(&This->m_Object);
}



EQP_Errors CComponent_Open(CComponent *This)
{
	IMpegCodec	*pMpegCodec = CDevice_getMpegCodec(This->m_pDevice);
	EQP_Errors	qpStatus;

	// allocate task
	//
	if (COMPONENT_TYPE_ENCODE == This->m_type)
	{
		qpStatus = pMpegCodec->AllocEncodeTask(pMpegCodec,
											   &This->m_hTask
											   );
	}
	else
	{
		qpStatus = pMpegCodec->AllocDecodeTask(pMpegCodec,
											   &This->m_hTask
											   );
	}

	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
					 "%s AllocTask(%d) failed status(%d)\n",
					 __FUNCTION__,
					 This->m_type,
					 qpStatus
					 ));

		PROFILING_LOG(EVENT_COMP_OPEN_FAILED, (TQP_UINT32) This->m_hTask, This->m_type, 0, 0, qpStatus);

	}
	else
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_WARN),
					 "%s AllocTask(%d) hTask(%d)\n",
					 __FUNCTION__,
					 This->m_type,
					 This->m_hTask
					 ));

		PROFILING_LOG(EVENT_COMP_OPEN, (TQP_UINT32) This->m_hTask, This->m_type, 0, 0, 0);

		// add to the device's component manager
		//
		This->m_hObject = CObjectMgr_AddObject(This->m_pDevice->m_pComponentMgr,
											   This
											   );
	}

	return (qpStatus);

}



EQP_Errors CComponent_Close(CComponent *This)
{
	IMpegCodec	*pMpegCodec = CDevice_getMpegCodec(This->m_pDevice);
	int			i;

    PROFILING_LOG(EVENT_COMP_CLOSE, (TQP_UINT32) This->m_hTask, This->m_type, 0, 0, 0);

	// make sure close all the opened ports
	//
	for (i = 0; i < MAX_COMPONENT_PORT; i++)
	{
		CComponent_PortClose(This,
							 (COMPONENT_PORT_TYPE)i
							 );
	}

	// release the allocated task
	//
	if (TQP_NULL_HANDLE != This->m_hTask)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_WARN),
					 "%s ReleaseTask(%d) hTask(%d)\n",
					 __FUNCTION__,
					 This->m_type,
					 This->m_hTask
					 ));

		pMpegCodec->ReleaseTask(pMpegCodec,
								This->m_hTask
								);
		This->m_hTask = TQP_NULL_HANDLE;
	}

	// remove ourself from the device's component manager
	//
	if (TQP_NULL_HANDLE != This->m_hObject)
	{
		CObjectMgr_RemoveObject(This->m_pDevice->m_pComponentMgr,
								This->m_hObject
								);
		This->m_hObject = TQP_NULL_HANDLE;
	}

	return (QPERR_SUCCESS);
}



#ifndef CONFIG_HOST_PLATFORM_USERSPACE
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
int CComponent_Ioctl(struct file *filp,
					 unsigned int cmd,
					 void *arg
					 )
#else // LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)
int CComponent_Ioctl(struct inode *inode,
					 struct file *filp,
					 unsigned int cmd,
					 void *arg
					 )
#endif //LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
#else // CONFIG_HOST_PLATFORM_USERSPACE
int CComponent_Ioctl( intptr_t dev,
					 unsigned int cmd,
					 void *arg
					 )
#endif // CONFIG_HOST_PLATFORM_USERSPACE
{
#ifndef CONFIG_HOST_PLATFORM_USERSPACE
    CComponent			*pComponent = (CComponent *)filp->private_data;
#else
    CComponent *pComponent = (CComponent *) dev;
#endif // CONFIG_HOST_PLATFORM_USERSPACE
	unsigned int		nSize = _IOC_SIZE(cmd);
	//unsigned int		nDir = _IOC_DIR(cmd);
	unsigned int		nType = _IOC_TYPE(cmd);
	unsigned int		nr = _IOC_NR(cmd);
	EQP_Errors			qpStatus = QPERR_PARMS;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_EXTRACE),
				 "%s() cmd(0x%08x) type(%c) nr(%d) size(%d)\n",
				 __FUNCTION__,
				 cmd,
				 nType,
				 nr,
				 nSize
				 ));

	switch (cmd)
	{
/////////////////////////////////////////////////////////////////////////////
//
//
		// v4l ioctl's

		case VIDIOC_STREAMON:
			qpStatus = CComponent_Start(pComponent);
			break;

		case VIDIOC_STREAMOFF:
			qpStatus = CComponent_Stop(pComponent);
			break;

		case VIDIOC_QBUF:
		{
			CPort				*pPort;
			struct v4l2_buffer	*pv4lbuf;

			pv4lbuf = (struct v4l2_buffer *)arg;
			if (!pv4lbuf ||
				(pv4lbuf->sequence >= MAX_COMPONENT_PORT)
				)
			{
                if (pv4lbuf)
                {
                    QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
							 "%s() VIDIOC_QBUF unknown sequence(%d)!!\n",
							 __FUNCTION__,
							 pv4lbuf->sequence
							 ));
                }
                else
                {
                    QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
                                  "%s() VIDIOC_QBUF null pointer!!\n",
                                  __FUNCTION__
                                  ));
                }
				qpStatus = QPERR_PARMS;
			}
			else
			{
				pPort = &pComponent->m_Ports[CComponent_PortGetDirection(pv4lbuf->sequence)];
				qpStatus = CPort_QBuf(pPort,
									  pv4lbuf
									  );
			}
			break;
		}

		case VIDIOC_DQBUF:
		{
			CPort				*pPort;
			struct v4l2_buffer	*pv4lbuf;

			pv4lbuf = (struct v4l2_buffer *)arg;
			if (!pv4lbuf ||
				(pv4lbuf->sequence >= MAX_COMPONENT_PORT)
				)
			{
                if (pv4lbuf)
                {
                    QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
                                  "%s() VIDIOC_DQBUF unknown sequence(%d)!!\n",
                                  __FUNCTION__,
                                  pv4lbuf->sequence
                                  ));
                }
                else
                {
                    QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
                                  "%s() VIDIOC_DQBUF null pointer!!\n",
                                  __FUNCTION__
                                  ));
                }
				qpStatus = QPERR_PARMS;
			}
			else
			{
				pPort = &pComponent->m_Ports[CComponent_PortGetDirection(pv4lbuf->sequence)];
				qpStatus = CPort_DQBuf(pPort,
									   pv4lbuf
									   );
			}
			break;
		}

		case VIDIOC_REQBUFS:
		{
			CPort						*pPort;
			struct v4l2_requestbuffers	*reqbuf;

			reqbuf = (struct v4l2_requestbuffers *)arg;
			if (!reqbuf ||
				(reqbuf->count >= MAX_COMPONENT_PORT)
				)
			{
                if (reqbuf)
                {
                    QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
                                  "%s() VIDIOC_REQBUFS unknown sequence(%d)!!\n",
                                  __FUNCTION__,
                                  reqbuf->count
                                  ));
                }
                else
                {
                    QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
                                  "%s() VIDIOC_REQBUFS null pointer!!\n",
                                  __FUNCTION__
                                  ));
                }
				qpStatus = QPERR_PARMS;
			}
			else
			{
				pPort = &pComponent->m_Ports[CComponent_PortGetDirection(reqbuf->count)];
				qpStatus = CPort_ReqBuf(pPort,
										reqbuf
										);
			}
			break;
		}

		case VIDIOC_QUERYBUF:
		{
			CPort				*pPort;
			struct v4l2_buffer	*pv4lbuf;

			pv4lbuf = (struct v4l2_buffer *)arg;
			if (!pv4lbuf ||
				(pv4lbuf->sequence >= MAX_COMPONENT_PORT)
				)
			{
                if (pv4lbuf)
                {
                    QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
                                  "%s() VIDIOC_QUERYBUF unknown sequence(%d)!!\n",
                                  __FUNCTION__,
                                  pv4lbuf->sequence
                                  ));
                }
                else
                {
                    QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
                                  "%s() VIDIOC_QUERYBUF null pointer!!\n",
                                  __FUNCTION__
                                  ));
                }
				qpStatus = QPERR_PARMS;
			}
			else
			{
				pPort = &pComponent->m_Ports[CComponent_PortGetDirection(pv4lbuf->sequence)];
				qpStatus = CPort_QueryBuf(pPort,
										  pv4lbuf
										  );
			}
			break;
		}


/////////////////////////////////////////////////////////////////////////////
//
//

		// stream extension interface
		//
		case PLDEV_STRM_IOCTL_PORT_OPEN:
		{
			PCOMPONENT_PORT_OPEN	pOpen = (PCOMPONENT_PORT_OPEN)arg;

			qpStatus = CComponent_PortOpen(pComponent,
										   pOpen
										   );
			break;
		}

		case PLDEV_STRM_IOCTL_PORT_CLOSE:
		{
			PCOMPONENT_PORT_TYPE	pType = (PCOMPONENT_PORT_TYPE)arg;

            PROFILING_LOG(EVENT_COMP_PORTCLOSE, (TQP_UINT32) pComponent->m_hTask, pComponent->m_type, 0, 0, 0);

			qpStatus = CComponent_PortClose(pComponent,
											*pType
											);
			break;
		}

		case PLDEV_STRM_IOCTL_DEVICE_INFO:
		{
			PPLDEV_INFO	pInfo = (PPLDEV_INFO)arg;

			/* special checking for driver and api version */
			if ((pInfo->busType!=PLDRV_VERSION_CODE)||(pInfo->type!=PLIOCTLAPI_VERSION_CODE))
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
					 		 "%s WARNING: API Version not match! App (DRV:0x%x API:0x%x), Driver (DRV:0x%x API:0x%x)!!\n",
					 	   	 __FUNCTION__,
							 pInfo->busType, pInfo->type, PLDRV_VERSION_CODE, PLIOCTLAPI_VERSION_CODE
							 ));
			}
			if ( ((pInfo->busType & 0xFFFFFF00) != (PLDRV_VERSION_CODE & 0xFFFFFF00)) ||
				 ((pInfo->type & 0XFFFFFF00) != (PLIOCTLAPI_VERSION_CODE & 0xFFFFFF00)))
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
					 		 "%s ERROR: Found but PLDEV APP and Driver version not match! App (DRV:0x%x API:0x%x), Driver (DRV:0x%x API:0x%x)!!\n",
							 __FUNCTION__,
							 pInfo->busType, pInfo->type, PLDRV_VERSION_CODE, PLIOCTLAPI_VERSION_CODE
							 ));
				return QPERR_CTRL_VERSION;
			}
			switch (pComponent->m_pDevice->m_type)
			{
				case PLV4LDEV_TYPE_QL300_FX2:
				case PLV4LDEV_TYPE_QL300_USB_EVB:
				case PLV4LDEV_TYPE_QL300_USB_EVK:
				case PLV4LDEV_TYPE_QL300_USB_CAM:
				case PLV4LDEV_TYPE_QL300_USB_CUS1:
				case PLV4LDEV_TYPE_QL300_USB_CUS2:
				case PLV4LDEV_TYPE_QL300_USB_CUS3:
				case PLV4LDEV_TYPE_QL300_USB_CUS4:
				case PLV4LDEV_TYPE_QL300_USB_FC3:
					pInfo->busType = QPHCI_BUS_USB;
					break;

				case PLV4LDEV_TYPE_QL300_STR_EVK:
					pInfo->busType = QPHCI_BUS_STR8133_REF;
					break;

				case PLV4LDEV_TYPE_QL300_PCIE_EVK:
					pInfo->busType = QPHCI_BUS_PCI;
					break;

				case PLV4LDEV_TYPE_QL300_MASTER:
					pInfo->busType = QPHCI_BUS_INTERNAL;
					break;

				default:
					pInfo->busType = -1;
					break;
			}
			pInfo->devInstance = pComponent->m_pDevice->m_id;
			pInfo->type = pComponent->m_type;
			pInfo->hTask = pComponent->m_hTask;
			qpStatus = QPERR_SUCCESS;
			break;
		}

		case PLDEV_STRM_IOCTL_ACQUIRE:
		{
			qpStatus = CComponent_Acquire(pComponent);
			break;
		}

		case PLDEV_STRM_IOCTL_GET_DRIVER_VER:
		{
			PTQP_UINT32	pVersion = (PTQP_UINT32)arg;

			*pVersion = PLDRV_VERSION_CODE;
			qpStatus = QPERR_SUCCESS;
			break;
		}

		case PLDEV_STRM_IOCTL_GET_IOCTLAPI_VER:
		{
			PTQP_UINT32	pVersion = (PTQP_UINT32)arg;

			*pVersion = PLIOCTLAPI_VERSION_CODE;
			qpStatus = QPERR_SUCCESS;
			break;
		}

/////////////////////////////////////////////////////////////////////////////
//
//

		// device control interface
		//
		case PLDEV_CNTL_IOCTL_SET_SYSTEM_CONTROL:
		case PLDEV_CNTL_IOCTL_SET_PICTURE_RESOLUTION:
		case PLDEV_CNTL_IOCTL_SET_INPUT_CONTROL:
		case PLDEV_CNTL_IOCTL_SET_RATE_CONTROL:
		case PLDEV_CNTL_IOCTL_SET_VBR_BIT_RATE:
		case PLDEV_CNTL_IOCTL_SET_GOP_LOOP_FILTER:
		case PLDEV_CNTL_IOCTL_SET_ET_CONTROL:
		case PLDEV_CNTL_IOCTL_SET_FILTER_CONTROL:
		case PLDEV_CNTL_IOCTL_SET_OUT_PICTURE_RESOLUTION:
		case PLDEV_CNTL_IOCTL_SET_BLOCK_SIZE:
		case PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM:
		case PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_EX:
		case PLDEV_CNTL_IOCTL_SET_REL_CUR_VID_BUF:
		case PLDEV_CNTL_IOCTL_SET_INSERT_I_FRAME:
		case PLDEV_CNTL_IOCTL_SET_INSERT_USER_DATA:
		case PLDEV_CNTL_IOCTL_SET_MP4_VID_NUM_BLOCK:
		case PLDEV_CNTL_IOCTL_SET_AUDIO_ENHANCEMENT:
		case PLDEV_CNTL_IOCTL_SET_ENC_STOP_MODE:
		case PLDEV_CNTL_IOCTL_SET_INDEX_CAP_FREQ:
		case PLDEV_CNTL_IOCTL_SET_ENC_VID_PADDING:
		case PLDEV_CNTL_IOCTL_SET_ENC_FREEZE_VIDEO:
		case PLDEV_CNTL_IOCTL_SET_ENC_STILL_VIDEO_INPUT:
		case PLDEV_CNTL_IOCTL_SET_ENC_CAP_MODE:
		case PLDEV_CNTL_IOCTL_SET_ENC_GET_A_FRAME:
		case PLDEV_CNTL_IOCTL_SET_ENC_MJPEG_QUALITY:
		case PLDEV_CNTL_IOCTL_SET_ENC_MJPEG_FRAME_BUF:
		case PLDEV_CNTL_IOCTL_SET_ENC_ALL_SETTINGS:
		case PLDEV_CNTL_IOCTL_SET_ENC_AUD_INPUT_VOL:

		case PLDEV_CNTL_IOCTL_SET_DEC_STREAM_ID:
		case PLDEV_CNTL_IOCTL_SET_DEC_OUTPUT_MODE:
		case PLDEV_CNTL_IOCTL_SET_DEC_RESOLUTION:
		case PLDEV_CNTL_IOCTL_SET_DEC_STOP_MODE:
		case PLDEV_CNTL_IOCTL_SET_DEC_TRICK_MODE:
		case PLDEV_CNTL_IOCTL_SET_DEC_DISP_ATTRIB:
		case PLDEV_CNTL_IOCTL_SET_DEC_DISP_MODE:
		case PLDEV_CNTL_IOCTL_SET_DEC_AUDIO_CONTROL:
		case PLDEV_CNTL_IOCTL_SET_DEC_DATA_XFER_CONTROL:
		case PLDEV_CNTL_IOCTL_SET_DEC_ALL_SETTINGS:
		case PLDEV_CNTL_IOCTL_SET_DEC_AUD_OUTPUT_VOL:

		case PLDEV_CNTL_IOCTL_SET_BEGIN_CHANNEL_CHANGE:
		case PLDEV_CNTL_IOCTL_SET_END_CHANNEL_CHANGE:
		case PLDEV_CNTL_IOCTL_SET_656_OUT_CNTL:
		case PLDEV_CNTL_IOCTL_SET_I2S_OUT_CNTL:
		case PLDEV_CNTL_IOCTL_SET_VOU_CONFIG:
		case PLDEV_CNTL_IOCTL_SET_AOU_CONFIG:
		case PLDEV_CNTL_IOCTL_SET_POWER_STATE:
		case PLDEV_CNTL_IOCTL_SET_ERROR_RECOVERY:
		case PLDEV_CNTL_IOCTL_SET_OSD_MEMORY:
#ifdef	SUPPORT_VI_OSD
		case PLDEV_CNTL_IOCTL_SET_ENC_OSD_SETTINGS:
#endif
		case PLDEV_CNTL_IOCTL_SET_DECIMATION:
		case PLDEV_CNTL_IOCTL_SET_DEINTERLACE:
		case PLDEV_CNTL_IOCTL_SET_EXTERNAL_TRIGGER:
		case PLDEV_CNTL_IOCTL_SET_PTS_COUNTER_RESET:
		case PLDEV_CNTL_IOCTL_SET_RATE_CONTROL_EX:
		case PLDEV_CNTL_IOCTL_SET_LARGE_COMPRESS_BUFFER_CONTROL:
		case PLDEV_CNTL_IOCTL_SET_ENABLE_LOW_BITRATE_MODE:
		case PLDEV_CNTL_IOCTL_SET_ENABLE_AV_DISCARD_CONTROL:
		case PLDEV_CNTL_IOCTL_SET_USE_SW_PTS:

		case PLDEV_CNTL_IOCTL_SET_GPIO_BIT_DIRECTION:
		case PLDEV_CNTL_IOCTL_SET_GPIO_BIT_VALUE:

		case PLDEV_CNTL_IOCTL_SET_SYS_FUNCTION:
		case PLDEV_CNTL_IOCTL_SET_SYS_LINK:
		case PLDEV_CNTL_IOCTL_SET_SYS_CONFIG:
		{
			IMpegCodec	*pMpegCodec = CDevice_getMpegCodec(pComponent->m_pDevice);

			qpStatus = pMpegCodec->Set(pMpegCodec,
									   (TQP_REFGUID)&PROPSETID_QPCODEC_CONTROL,
									   pComponent->m_hTask,
									   nr - PLDEV_CNTL_IOCTL_SET,
									   (TQP_PVOID)arg,
									   QP_NULL,
									   nSize
									   );
			break;
		}

		case PLDEV_CNTL_IOCTL_GET_SYSTEM_CONTROL:
		case PLDEV_CNTL_IOCTL_GET_PICTURE_RESOLUTION:
		case PLDEV_CNTL_IOCTL_GET_INPUT_CONTROL:
		case PLDEV_CNTL_IOCTL_GET_RATE_CONTROL:
		case PLDEV_CNTL_IOCTL_GET_VBR_BIT_RATE:
		case PLDEV_CNTL_IOCTL_GET_GOP_LOOP_FILTER:
		case PLDEV_CNTL_IOCTL_GET_ET_CONTROL:
		case PLDEV_CNTL_IOCTL_GET_FILTER_CONTROL:
		case PLDEV_CNTL_IOCTL_GET_OUT_PICTURE_RESOLUTION:
		case PLDEV_CNTL_IOCTL_GET_BLOCK_SIZE:
		case PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM:
		case PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX:
		case PLDEV_CNTL_IOCTL_GET_VIOSD_TABLE_ADDR:
		case PLDEV_CNTL_IOCTL_GET_GET_CUR_VID_BUF_INFO:
		case PLDEV_CNTL_IOCTL_GET_MP4_VID_NUM_BLOCK:
		case PLDEV_CNTL_IOCTL_GET_AUDIO_ENHANCEMENT:
		case PLDEV_CNTL_IOCTL_GET_ENC_STOP_MODE:
		case PLDEV_CNTL_IOCTL_GET_INDEX_CAP_FREQ:
		case PLDEV_CNTL_IOCTL_GET_ENC_VID_PADDING:
		case PLDEV_CNTL_IOCTL_GET_ENC_FREEZE_VIDEO:
		case PLDEV_CNTL_IOCTL_GET_ENC_STILL_VIDEO_INPUT:
		case PLDEV_CNTL_IOCTL_GET_ENC_CAP_MODE:
		case PLDEV_CNTL_IOCTL_GET_ENC_MJPEG_QUALITY:
		case PLDEV_CNTL_IOCTL_GET_ENC_MJPEG_FRAME_BUF:
		case PLDEV_CNTL_IOCTL_GET_ENC_ALL_SETTINGS:
		case PLDEV_CNTL_IOCTL_GET_ENC_AUD_INPUT_VOL:

		case PLDEV_CNTL_IOCTL_GET_DEC_STREAM_ID:
		case PLDEV_CNTL_IOCTL_GET_DEC_OUTPUT_MODE:
		case PLDEV_CNTL_IOCTL_GET_DEC_RESOLUTION:
		case PLDEV_CNTL_IOCTL_GET_DEC_STOP_MODE:
		case PLDEV_CNTL_IOCTL_GET_DEC_TRICK_MODE:
		case PLDEV_CNTL_IOCTL_GET_DEC_DISP_ATTRIB:
		case PLDEV_CNTL_IOCTL_GET_DEC_DISP_MODE:
		case PLDEV_CNTL_IOCTL_GET_DEC_GET_PLAY_INFO:
		case PLDEV_CNTL_IOCTL_GET_DEC_AUDIO_CONTROL:
		case PLDEV_CNTL_IOCTL_GET_DEC_DATA_XFER_CONTROL:
		case PLDEV_CNTL_IOCTL_GET_DEC_ALL_SETTINGS:
		case PLDEV_CNTL_IOCTL_GET_DEC_AUD_OUTPUT_VOL:

		case PLDEV_CNTL_IOCTL_GET_ENCODER_STATUS:
		case PLDEV_CNTL_IOCTL_GET_DECODER_STATUS:
		case PLDEV_CNTL_IOCTL_GET_656_OUT_CNTL:
		case PLDEV_CNTL_IOCTL_GET_I2S_OUT_CNTL:
		case PLDEV_CNTL_IOCTL_GET_VOU_CONFIG:
		case PLDEV_CNTL_IOCTL_GET_AOU_CONFIG:
		case PLDEV_CNTL_IOCTL_GET_POWER_STATE:
		case PLDEV_CNTL_IOCTL_GET_ERROR_RECOVERY:
		case PLDEV_CNTL_IOCTL_GET_OSD_MEMORY:
		case PLDEV_CNTL_IOCTL_GET_DECIMATION:
		case PLDEV_CNTL_IOCTL_GET_DEINTERLACE:
		case PLDEV_CNTL_IOCTL_GET_EXTERNAL_TRIGGER:
		case PLDEV_CNTL_IOCTL_GET_PTS_COUNTER_RESET:
		case PLDEV_CNTL_IOCTL_GET_RATE_CONTROL_EX:
		case PLDEV_CNTL_IOCTL_GET_LARGE_COMPRESS_BUFFER_CONTROL:
		case PLDEV_CNTL_IOCTL_GET_ENABLE_LOW_BITRATE_MODE:
		case PLDEV_CNTL_IOCTL_GET_ENABLE_AV_DISCARD_CONTROL:
		case PLDEV_CNTL_IOCTL_GET_USE_SW_PTS:

		case PLDEV_CNTL_IOCTL_GET_GPIO_BIT_DIRECTION:
		case PLDEV_CNTL_IOCTL_GET_GPIO_BIT_VALUE:

		case PLDEV_CNTL_IOCTL_GET_SYS_FUNCTION:
		case PLDEV_CNTL_IOCTL_GET_SYS_LINK:
		case PLDEV_CNTL_IOCTL_GET_SYS_CONFIG:
        case PLDEV_CNTL_IOCTL_GET_CODEC_ERROR:
		{
			IMpegCodec	*pMpegCodec = CDevice_getMpegCodec(pComponent->m_pDevice);
			TQP_UINT32	dwGot = 0;

			qpStatus = pMpegCodec->Get(pMpegCodec,
									   (TQP_REFGUID)&PROPSETID_QPCODEC_CONTROL,
									   pComponent->m_hTask,
									   nr - PLDEV_CNTL_IOCTL_GET,
									   (TQP_PVOID)arg,
									   (TQP_PVOID)arg,
									   &dwGot
									   );
			break;
		}


/////////////////////////////////////////////////////////////////////////////
//
//
		// diagnostic interface
		//
		case PLDEV_DIAG_IOCTL_FIRMWARE_DOWNLOAD_PTR:
		{
			PQPCODEC_DIAG_FW_DOWNLOAD_PTR	pFW = (PQPCODEC_DIAG_FW_DOWNLOAD_PTR)arg;
			QPCODEC_DIAG_FW_DOWNLOAD_PTR	FW;
			ICodecLib						*pCodecLib = CDevice_getCodecLib(pComponent->m_pDevice);

			memcpy(&FW,
				   pFW,
				   sizeof(QPCODEC_DIAG_FW_DOWNLOAD_PTR)
				   );

#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
			FW.pFW = kmalloc(FW.FWSize, GFP_KERNEL);
#else
			FW.pFW = vmalloc_32(FW.FWSize);
#endif
			if (!FW.pFW)
			{
				qpStatus = QPERR_NOMEM;
			}
			else
			{
				if (copy_from_user(FW.pFW, pFW->pFW, pFW->FWSize))
				{
					qpStatus = QPERR_NOMEM;
				}
				else
				{
					qpStatus = pCodecLib->Set(pCodecLib,
											  (TQP_REFGUID)&PROPSETID_QPCODEC_DIAG,
											  pComponent->m_hTask,
											  nr - PLDEV_DIAG_IOCTL_BASE,
											  (TQP_PVOID)&FW,
											  QP_NULL,
											  nSize
											  );
				}
#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
				kfree(FW.pFW);
#else
				vfree(FW.pFW);
#endif
			}
			break;
		}

		case PLDEV_DIAG_IOCTL_DMA_WRITE_PTR:
		{
			PQPCODEC_DIAG_DMA_PTR_STRUCT	pDMA = (PQPCODEC_DIAG_DMA_PTR_STRUCT)arg;
			QPCODEC_DIAG_DMA_PTR_STRUCT		DMA;
			ICodecLib						*pCodecLib = CDevice_getCodecLib(pComponent->m_pDevice);

			memcpy(&DMA,
				   pDMA,
				   sizeof(QPCODEC_DIAG_DMA_PTR_STRUCT)
				   );

#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
			DMA.pData = kmalloc(DMA.size, GFP_KERNEL);
#else
			DMA.pData = vmalloc_32(DMA.size);
#endif
			if (!DMA.pData)
			{
				qpStatus = QPERR_NOMEM;
			}
			else
			{
				if (copy_from_user(DMA.pData,
								   pDMA->pData,
								   pDMA->size
								   ))
				{
					qpStatus = QPERR_NOMEM;
				}
				else
				{
					qpStatus = pCodecLib->Set(pCodecLib,
											  (TQP_REFGUID)&PROPSETID_QPCODEC_DIAG,
											  pComponent->m_hTask,
											  nr - PLDEV_DIAG_IOCTL_BASE,
											  (TQP_PVOID)&DMA,
											  QP_NULL,
											  nSize
											  );
				}
#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
				kfree(DMA.pData);
#else
				vfree(DMA.pData);
#endif
			}
			break;
		}

		case PLDEV_DIAG_IOCTL_I2C_WRITE:
        case PLDEV_DIAG_IOCTL_I2C_WRITE_EX:
		case PLDEV_DIAG_IOCTL_MEM_WRITE:
		case PLDEV_DIAG_IOCTL_REG_WRITE:
		case PLDEV_DIAG_IOCTL_HCI_REG_WRITE:
		case PLDEV_DIAG_IOCTL_ENABLE_DRIVER_I2C:
		case PLDEV_DIAG_IOCTL_DISABLE_DRIVER_I2C:
		case PLDEV_DIAG_IOCTL_VIDEO_DECODER_SET:
		{
			ICodecLib	*pCodecLib = CDevice_getCodecLib(pComponent->m_pDevice);

			qpStatus = pCodecLib->Set(pCodecLib,
									  (TQP_REFGUID)&PROPSETID_QPCODEC_DIAG,
									  pComponent->m_hTask,
									  nr - PLDEV_DIAG_IOCTL_BASE,
									  (TQP_PVOID)arg,
									  QP_NULL,
									  nSize
									  );
			break;
		}

		case PLDEV_DIAG_IOCTL_DMA_READ_PTR:
		{
			PQPCODEC_DIAG_DMA_PTR_STRUCT	pDMA = (PQPCODEC_DIAG_DMA_PTR_STRUCT)arg;
			QPCODEC_DIAG_DMA_PTR_STRUCT		DMA;
			ICodecLib						*pCodecLib = CDevice_getCodecLib(pComponent->m_pDevice);
			TQP_UINT32						dwGot = 0;

			memcpy(&DMA,
				   pDMA,
				   sizeof(QPCODEC_DIAG_DMA_PTR_STRUCT)
				   );

#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
			DMA.pData = kmalloc(DMA.size, GFP_KERNEL);
#else
			DMA.pData = vmalloc_32(DMA.size);
#endif
			if (!DMA.pData)
			{
				qpStatus = QPERR_NOMEM;
			}
			else
			{
				qpStatus = pCodecLib->Get(pCodecLib,
										  (TQP_REFGUID)&PROPSETID_QPCODEC_DIAG,
										  pComponent->m_hTask,
										  nr - PLDEV_DIAG_IOCTL_BASE,
										  (TQP_PVOID)&DMA,
										  (TQP_PVOID)&DMA,
										  &dwGot
										  );
				if (QP_SUCCESS(qpStatus))
				{
					if (copy_to_user(pDMA->pData,
									 DMA.pData,
									 pDMA->size
									 ))
					{
						qpStatus = QPERR_NOMEM;
					}
				}
#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
				kfree(DMA.pData);
#else
				vfree(DMA.pData);
#endif
			}
			break;
		}

		case PLDEV_DIAG_IOCTL_I2C_READ:
		case PLDEV_DIAG_IOCTL_I2C_READ_EX:
		case PLDEV_DIAG_IOCTL_I2C_WRITE_THEN_READ:
		case PLDEV_DIAG_IOCTL_I2C_WRITE_THEN_READ_EX:
		case PLDEV_DIAG_IOCTL_MEM_READ:
		case PLDEV_DIAG_IOCTL_REG_READ:
		case PLDEV_DIAG_IOCTL_HCI_REG_READ:
		case PLDEV_DIAG_IOCTL_FIRMWARE_COMMAND:
		case PLDEV_DIAG_IOCTL_VIDEO_DECODER_GET:
		{
			ICodecLib	*pCodecLib = CDevice_getCodecLib(pComponent->m_pDevice);
			TQP_UINT32	dwGot = 0;

			qpStatus = pCodecLib->Get(pCodecLib,
									  (TQP_REFGUID)&PROPSETID_QPCODEC_DIAG,
									  pComponent->m_hTask,
									  nr - PLDEV_DIAG_IOCTL_BASE,
									  (TQP_PVOID)arg,
									  (TQP_PVOID)arg,
									  &dwGot
									  );

#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
#if ENABLE_CODEC_MEM_LOG
			if (cmd == PLDEV_DIAG_IOCTL_REG_READ) {
				codec_mem_log_dump();
			}
#endif
#endif

			break;
		}

	#ifdef	SUPPORT_PL3XX_OSD
		case PLDEV_DIAG_IOCTL_OSD_CONFIG:
			qpStatus = qldiag_osd_param(pComponent->m_pDevice, arg);
			break;

		case PLDEV_DIAG_IOCTL_OSD_CONTROL:
			qpStatus = qldiag_osd_control(pComponent->m_pDevice, arg);
			break;

	#ifdef	SUPPORT_VI_OSD
		case PLDEV_DIAG_IOCTL_VIOSD_CONTROL:
			qpStatus = qldiag_viosd_control(pComponent->m_pDevice, arg);
			break;
	#endif
	#endif


		case PLDEV_DIAG_IOCTL_VIDEOPROP_SET:
			qpStatus = (EQP_Errors) qldiag_videoprop_set(pComponent->m_pDevice, arg);
			break;
		case PLDEV_DIAG_IOCTL_VIDEOPROP_GET:
			qpStatus = (EQP_Errors) qldiag_videoprop_get(pComponent->m_pDevice, arg);
			break;
		case PLDEV_DIAG_IOCTL_VIDEO_DECODER_SOURCE_SET:
		{
			ICodecLib	*pCodecLib = CDevice_getCodecLib(pComponent->m_pDevice);
			//TQP_UINT32	dwGot = 0;
			TQP_UINT32	*p = (TQP_UINT32 *)arg;
			VIDEO_SOURCE newarg;
			/* map dwValue to VIDEO_SOURCE */
			switch (*p)
			{
				case 0: /* AUTO/HDMI */
					newarg = VIDEO_SOURCE_DIGITAL1;
					break;
				case 1:	/* Composite */
					newarg = VIDEO_SOURCE_CVIDEO1;
					break;
				case 2:	/* S-Video */
					newarg = VIDEO_SOURCE_SVIDEO1;
					break;
				case 3:	/* Component */
					newarg = VIDEO_SOURCE_NVIDEO1;
					break;
				default:
					qpStatus = QPERR_PARMS;
					return (QpToLinuxKerStatus(qpStatus));
			}
			qpStatus = pCodecLib->Set(pCodecLib,
									  (TQP_REFGUID)&PROPSETID_QPCODEC_DIAG,
									  pComponent->m_hTask,
									  nr - PLDEV_DIAG_IOCTL_BASE,
									  (TQP_PVOID)&newarg,
									  QP_NULL,
									  nSize
									  );
			break;
		}
		case PLDEV_DIAG_IOCTL_VIDEO_DECODER_SOURCE_GET:
		{
			ICodecLib	*pCodecLib = CDevice_getCodecLib(pComponent->m_pDevice);
			TQP_UINT32	dwGot = 0;
            TQP_UINT32 *p = (TQP_UINT32 *)arg;
			VIDEO_SOURCE newarg;

			qpStatus = pCodecLib->Get(pCodecLib,
									  (TQP_REFGUID)&PROPSETID_QPCODEC_DIAG,
									  pComponent->m_hTask,
									  nr - PLDEV_DIAG_IOCTL_BASE,
									  QP_NULL,
									  (TQP_PVOID)&newarg,
									  &dwGot
									  );
			/* map VIDEO_SOURCE to dwValue*/
			switch (newarg)
			{
				case VIDEO_SOURCE_DIGITAL1: /* AUTO/HDMI */
					*p = 0;
					break;
				case VIDEO_SOURCE_CVIDEO1:	/* Composite */
					*p = 1;
					break;
				case VIDEO_SOURCE_SVIDEO1:	/* S-Video */
					*p = 2;
					break;
				case VIDEO_SOURCE_NVIDEO1:	/* Component */
					*p = 3;
					break;
				default:
					*p = 0;
					qpStatus = QPERR_PARMS;
					return (QpToLinuxKerStatus(qpStatus));
			}
			break;
		}
		case PLDEV_DIAG_IOCTL_FIRMWARE_RELOAD:
		case PLDEV_DIAG_IOCTL_RESET_ARM:
		{
			ICodecLib *pCodecLib = CDevice_getCodecLib(pComponent->m_pDevice);
			qpStatus = pCodecLib->Set(pCodecLib,
						  (TQP_REFGUID)&PROPSETID_QPCODEC_DIAG,
						  pComponent->m_hTask,
						  nr - PLDEV_DIAG_IOCTL_BASE,
						  QP_NULL,
						  QP_NULL,
						  nSize
						  );
			break;
		}

		case PLDEV_DIAG_IOCTL_DEBUGLEVEL_SET:
		{
            TQP_UINT32 *p = (TQP_UINT32 *)arg;
			TQP_UINT32 func, value;

			func = ((*p) & 0xFFFF0000) >> 16;
			value = (*p) & 0xFFFF;

			switch (func)
			{
				case 0:
					qpStatus = QPDbgMSetCompLevel(g_PLV4LDevDBGCompID, value);
					break;
				case 1:
					qpStatus = QPDbgMSetCompLevel(g_QPCodecDBGCompID, value);
					break;
				case 2:
					qpStatus = QPDbgMSetCompLevel(g_QPUsbDBGCompID, value);
					break;
				case 3:
					qpStatus = QPDbgMSetCompLevel(g_QPPCIeDBGCompID, value);
					break;
				case 4:
					qpStatus = QPDbgMSetCompLevel(g_QPPeripheralDBGCompID, value);
					break;
			#ifdef QP_DBG_TO_BUFFER
				case 10:
					qpStatus = QPDbgMSetBufCompLevel(g_PLV4LDevDBGCompID, value);
					break;
				case 11:
					qpStatus = QPDbgMSetBufCompLevel(g_QPCodecDBGCompID, value);
					break;
				case 12:
					qpStatus = QPDbgMSetBufCompLevel(g_QPUsbDBGCompID, value);
					break;
				case 13:
					qpStatus = QPDbgMSetBufCompLevel(g_QPPCIeDBGCompID, value);
					break;
				case 14:
					qpStatus = QPDbgMSetBufCompLevel(g_QPPeripheralDBGCompID, value);
					break;
				//case 15:
				//	qpStatus = QPDbgMSetBufCompLevel(PaulDBGCompID, value);
				//	break;
				case 20:
					QPDbgMTimeStampPrinting(1);
					qpStatus = QPDbgMSetBufInit();
					break;
				case 21:
					qpStatus = QPDbgMSetBufDump("/tmp/kernellog.txt");
					break;
			#endif
				default:
					qpStatus = QPERR_PARMS;
					break;
			}
			break;
		}
		case PLDEV_DIAG_IOCTL_DEBUGLEVEL_GET:
		{
            TQP_UINT32 *p = (TQP_UINT32 *)arg;
			TQP_UINT32 func;
			unsigned long value = 0;
			func = ((*p) & 0xFFFF0000) >> 16;
			switch (func)
			{
				case 0:
					qpStatus = QPDbgMGetCompLevel(g_PLV4LDevDBGCompID, &value);
					break;
				case 1:
					qpStatus = QPDbgMGetCompLevel(g_QPCodecDBGCompID, &value);
					break;
				case 2:
					qpStatus = QPDbgMGetCompLevel(g_QPUsbDBGCompID, &value);
					break;
				case 3:
					qpStatus = QPDbgMGetCompLevel(g_QPPCIeDBGCompID, &value);
					break;
				case 4:
					qpStatus = QPDbgMGetCompLevel(g_QPPeripheralDBGCompID, &value);
					break;
			#ifdef QP_DBG_TO_BUFFER
				case 10:
					qpStatus = QPDbgMGetBufCompLevel(g_PLV4LDevDBGCompID, &value);
					break;
				case 11:
					qpStatus = QPDbgMGetBufCompLevel(g_QPCodecDBGCompID, &value);
					break;
				case 12:
					qpStatus = QPDbgMGetBufCompLevel(g_QPUsbDBGCompID, &value);
					break;
				case 13:
					qpStatus = QPDbgMGetBufCompLevel(g_QPPCIeDBGCompID, &value);
					break;
				case 14:
					qpStatus = QPDbgMGetBufCompLevel(g_QPPeripheralDBGCompID, &value);
					break;
				//case 15:
				//	qpStatus = QPDbgMGetBufCompLevel(PaulDBGCompID, &value);
				//	break;
			#endif
				default:
					qpStatus = QPERR_PARMS;
					break;
			}
			if (qpStatus==QPERR_SUCCESS)
				*p = (func << 16) | (value & 0xFFFF);
		 	break;
		}
		case PLDEV_DIAG_IOCTL_DRVPROFILING_GET:
		{
			PQPCODEC_DIAG_PROFILING_PTR_STRUCT pProfiling = (PQPCODEC_DIAG_PROFILING_PTR_STRUCT)arg;
			if (pProfiling!=NULL) {
                if (pProfiling->max_entries != QP_MAX_PROFILING_SIZE)
                {
                    QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
                        		 "%s PLDEV_DIAG_IOCTL_DRVPROFILING_GET buf size not match %d vs %d!!\n",
                        		 __FUNCTION__,
                        		 pProfiling->max_entries,
                        		 QP_MAX_PROFILING_SIZE
                        		 ));
					qpStatus = QPERR_PARMS;
                }
                else
                {
                    PTQP_UINT8 pFromProfiling = (PTQP_UINT8) PORFILING_LOG_GET_ADDRESS();
                    if (pFromProfiling!=0)
                    {
                        int ret1;
                        ret1=copy_to_user((void * )pProfiling->pData,
                                         (void * )pFromProfiling + pProfiling->offset,
                                         pProfiling->size);

                        if (ret1!=0)
                        {
                            QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
                                		 "%s PLDEV_DIAG_IOCTL_DRVPROFILING_GET copy failed!! %p %p %d %d ret=%d\n",
                                		 __FUNCTION__,
                                		 pProfiling->pData,
                                		 pFromProfiling,
                                		 pProfiling->size, sizeof(QPCODEC_DIAG_PROFILING_STRUCT), ret1
                            			 ));
                            qpStatus = QPERR_PARMS;
                        }
                        else {
                            qpStatus=QPERR_SUCCESS;
                        }
                    }
                    else {
                        QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
                            		 "%s PLDEV_DIAG_IOCTL_DRVPROFILING_GET no driver buf!!\n",
                            		 __FUNCTION__
                        			 ));
                        qpStatus = QPERR_NOMEM;
                    }
                }
			}
			else {
                    QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
                        		 "%s PLDEV_DIAG_IOCTL_DRVPROFILING_GET no return buf!!\n",
                        		 __FUNCTION__
                        		 ));
					qpStatus = QPERR_PARMS;
			}
            break;
		}
		default:
			qpStatus = QPERR_NOTSUPP;
			break;
	}

	return (QpToLinuxKerStatus(qpStatus));
}



#ifndef CONFIG_HOST_PLATFORM_USERSPACE
int CComponent_Mmap(CComponent *This,
					struct vm_area_struct *vma
					)
{
	CPort				*pPort;
	COMPONENT_PORT_TYPE	type;
	struct v4l2_buffer	*pV4l2Buf;
	long				lErr = 0;
	unsigned long		offset = vma->vm_pgoff << PAGE_SHIFT;
    int					nIndex;
    unsigned long		ulStart = 0;
    unsigned long		ulAddr = 0;
    unsigned long		ulSize = 0;

	// find the port this buffer belongs to
	//
	type = offset >> PORT_MMAP_BUF_TYPE_SHIFT;
	if (type >= MAX_COMPONENT_PORT)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
					 "%s vma->vm_pgoff(0x%x) not found!!!\n",
					 __FUNCTION__,
					 vma->vm_pgoff
					 ));
		return (-EINVAL);
	}

	pPort = &This->m_Ports[CComponent_PortGetDirection(type)];
	if (V4L2_MEMORY_MMAP != pPort->m_memoryType)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
					 "%s type(%d) invalid memory type(%d)!!!\n",
					 __FUNCTION__,
					 type,
					 pPort->m_memoryType
					 ));
		return (-EINVAL);
	}

	offset &= ~PORT_MMAP_BUF_TYPE_MASK;
	offset = offset >> PAGE_SHIFT;

	for (nIndex = 0; nIndex < pPort->m_frame_nbs; nIndex++)
	{
		pV4l2Buf = &pPort->m_v4l2Buffers[nIndex];
		if ((pV4l2Buf->m.offset >> PAGE_SHIFT) == offset)
		{
			break;
		}
	}

	if (nIndex >= pPort->m_frame_nbs)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
					 "%s vma->vm_pgoff(0x%x) not found!!!\n",
					 __FUNCTION__,
					 vma->vm_pgoff
					 ));
		return (-EINVAL);
	}

	// VM_IO marks the area as being an mmaped region for I/O to a
	// device. It also prevents the region from being core dumped.
	//
	vma->vm_flags |= VM_IO;
#if LINUX_VERSION_CODE < KERNEL_VERSION(3,7,0)
	vma->vm_flags |= VM_RESERVED;
#else
	vma->vm_flags |= VM_DONTEXPAND | VM_DONTDUMP;
#endif /* LINUX_VERSION_CODE < KERNEL_VERSION(3,7,0) */

#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
	vma->vm_page_prot = pgprot_noncached(vma->vm_page_prot);
#endif

	ulStart = vma->vm_start;
	ulAddr = (unsigned long)(pPort->m_pBufMem + pV4l2Buf->m.offset);
	ulSize = vma->vm_end - vma->vm_start;

	while (ulSize > 0)
	{
#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
		if ((lErr = remap_pfn_range(vma,
									ulStart,
									__pa((void *)ulAddr) >> PAGE_SHIFT,
									PAGE_SIZE,
									PAGE_SHARED
									)) < 0
									)
#else
		if ((lErr = vm_insert_page(vma,
								   ulStart,
								   vmalloc_to_page((void *)ulAddr)
								   )) < 0
								   )
#endif
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
						 "%s vm_insert_page() start(%d) addr(%d) failed(%d)!!!\n",
						 __FUNCTION__,
						 ulStart,
						 ulAddr,
						 lErr
						 ));
			return (lErr);
		}

		ulStart += PAGE_SIZE;
		ulAddr += PAGE_SIZE;
		ulSize -= PAGE_SIZE;
    }

	vma->vm_ops = &plv4l_mmap_ops;
	vma->vm_private_data = pV4l2Buf;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
				 "%s vma->vm_pgoff(0x%x) found buf(%d) lErr(%d)\n",
				 __FUNCTION__,
				 vma->vm_pgoff,
				 nIndex,
				 lErr
				 ));

	return (lErr);
}
#else
void* CComponent_Mmap(intptr_t dev, size_t port, size_t offset)
{
    CComponent *pComponent = (CComponent *) dev;
	offset &= ~PORT_MMAP_BUF_TYPE_MASK;
	//printf("%s() ==>port %d offset 0x%x   (%p)\n", __FUNCTION__, port, offset, pComponent->m_Ports[CComponent_PortGetDirection(port)].m_pBufMem);
	return (void *)(pComponent->m_Ports[CComponent_PortGetDirection(port)].m_pBufMem + offset);
}
#endif // CONFIG_HOST_PLATFORM_USERSPACE

#ifndef CONFIG_HOST_PLATFORM_USERSPACE
int CComponent_Poll(CComponent *This,
					struct file *filp, poll_table *wait
					)
{
	CPort				*pPort;
	//COMPONENT_PORT_TYPE	type;
	EQP_Errors	qpStatus;
	int i, mask=0, IsEmpty;

	if (TQP_NULL_HANDLE == This->m_hObject)
		return POLLERR;

	poll_wait(filp, &(This->wait_frame), wait);

	for (i = 0; i < MAX_PORT_DIRECTION/*MAX_COMPONENT_PORT*/; i++)
	{
		pPort = &This->m_Ports[i];
		if (pPort->m_valid)
		{
				qpStatus = CPort_CheckBuf(pPort, &IsEmpty);
				if (qpStatus==QPERR_SUCCESS)
				{
					if (IsEmpty==0)
					{
						//printk("===> [Port %d dir %d ] is not empty\n", i, pPort->m_dir);
						switch (pPort->m_dir)
						{
							case PORT_DIR_READ:
								mask |= POLLIN|POLLRDNORM;
								break;

							case PORT_DIR_WRITE:
								mask |= POLLOUT|POLLWRNORM;
								break;

							case PORT_DIR_NONE:
							default:
								mask |= 0;
						}
					}
					else
					{
						//printk("===> [Port %d dir %d ] is empty or error\n", i, pPort->m_dir);
					}
				}
				else {
					//printk("===> [Port %d dir %d ] is empty or error\n", i, pPort->m_dir);
				}
		}
		else {
			continue;
		}
	}
	return (mask);
}
#endif // CONFIG_HOST_PLATFORM_USERSPACE


