///////////////////////////////////////////////////////////////////////////////
//     
//
// Ccomponent.h
//
// Description:
//
//  component declaration
//
// History:
//	Dec 18, 2009	(dt) created
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __CCOMPONENT_H__
#define __CCOMPONENT_H__

#include "qlautoconf.h"
#ifndef CONFIG_HOST_PLATFORM_USERSPACE
#include <linux/types.h>
#include <linux/file.h>
#include <linux/fs.h>
#include <linux/poll.h>
#include "pldevioctl.h"
#endif

#include "qptypes.h"
#include "cobject.h"
#include "cport.h"


/////////////////////////////////////////////////////////////////////////////
//
//


#ifndef __CDEVICE_FWD_DEFINED__
#define __CDEVICE_FWD_DEFINED__
typedef struct CDevice CDevice;
#endif //!__CDEVICE_FWD_DEFINED__


#ifndef __CCOMPONENT_FWD_DEFINED__
#define __CCOMPONENT_FWD_DEFINED__
typedef struct CComponent CComponent;
#endif //!__CCOMPONENT_FWD_DEFINED__


struct CComponent
{
	// public CObject
	//
	CObject				m_Object;

	// CComponent
	//
#ifndef CONFIG_HOST_PLATFORM_USERSPACE
    struct video_device *m_pVideoDev;
	wait_queue_head_t 	wait_frame;
#else
    struct user_video_device *m_pVideoDev;
	//wait_queue_head_t 	wait_frame;
#endif // CONFIG_HOST_PLATFORM_USERSPACE
	CDevice				*m_pDevice;
	COMPONENT_TYPE		m_type;
	TQP_OBJECT_HANDLE	m_hTask;	// codec lib task handle
	TQP_OBJECT_HANDLE	m_hObject;	// handle acquired from object manager
	CPort				m_Ports[MAX_PORT_DIRECTION/*MAX_COMPONENT_PORT*/];
};


CComponent * CComponent_Constructor(CComponent *pComponent,
#ifndef CONFIG_HOST_PLATFORM_USERSPACE
									struct video_device *pVideoDev,
#else
									struct user_video_device *pVideoDev,
#endif
									CDevice *pDevice,
									COMPONENT_TYPE type
									);
void CComponent_Destructor(CComponent *This);
EQP_Errors CComponent_Open(CComponent *This);
EQP_Errors CComponent_Close(CComponent *This);
#ifndef CONFIG_HOST_PLATFORM_USERSPACE
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
int CComponent_Ioctl(struct file *filp,
					 unsigned int cmd,
					 void *arg
					 );
#else // LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)
int CComponent_Ioctl(struct inode *inode,
					 struct file *filp,
					 unsigned int cmd,
					 void *arg
					 );
#endif //LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
int CComponent_Mmap(CComponent *This,
					struct vm_area_struct *vma
					);
int CComponent_Poll(CComponent *This,
					struct file *filp, poll_table *wait
					);
#else // CONFIG_HOST_PLATFORM_USERSPACE
int CComponent_Ioctl( intptr_t dev,
					unsigned int cmd,
					void *arg
					);
void* CComponent_Mmap(intptr_t dev, size_t port, size_t offset);
#endif // CONFIG_HOST_PLATFORM_USERSPACE
// access functions
//
static inline TQP_OBJECT_HANDLE CComponent_GetTaskHandle(CComponent *This) {return (This->m_hTask);}

#endif //__CCOMPONENT_H__




