///////////////////////////////////////////////////////////////////////////////
//      
//
// Cdevice.c
//
// Description:
//
//  cdevice implementation
//
// History:
//	Jan 18, 2010	(dt) created
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#include "qlautoconf.h"

#ifndef CONFIG_HOST_PLATFORM_USERSPACE
#include <linux/version.h>
#ifdef __LINUX24__
#include <linux/module.h>
#include <linux/videodev.h>
#include "videodev2.h"
#else //!__LINUX24__
#include <linux/module.h>
#include <linux/videodev2.h>
#endif //__LINUX24__
#include <linux/init.h>
#include <linux/delay.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/interrupt.h>
#include <linux/spinlock.h>
//#include <linux/video_decoder.h>
//#include <linux/video_encoder.h>
#include <linux/i2c.h>
#include <linux/i2c-algo-bit.h>
#include <linux/list.h>
#include <linux/unistd.h>
//#include <linux/byteorder/swab.h>
#include <linux/pagemap.h>
#include <linux/ctype.h>
#include <linux/slab.h>
#include <linux/poll.h>
#include <linux/page-flags.h>
#include <asm/uaccess.h>
#if LINUX_VERSION_CODE > KERNEL_VERSION(3,3,0)
#include <asm/switch_to.h>
#else
#include <asm/system.h>
#endif
#include <asm/io.h>

#include <linux/vmalloc.h>

#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
#include <mach/pl330xp.h>
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,26)
#include <linux/semaphore.h>
#else //LINUX_VERSION_CODE < KERNEL_VERSION(2,6,26)
#include <asm/semaphore.h>
#endif //LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,26)
#else // CONFIG_HOST_PLATFORM_USERSPACE
#include <stdint.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include "userwrap.h"
#endif // CONFIG_HOST_PLATFORM_USERSPACE

#include "qpdbgmgr.h"
#include "plprofiling.h"
#include "pldevioctl.h"
#include "ccomponent.h"
#include "cdevice.h"
#include "qlfops.h"
#ifndef CONFIG_HOST_PLATFORM_USERSPACE
#include "qplinuxkermap.h"

MODULE_LICENSE("GPL");
#endif



extern TQP_DbgMCompID	g_PLV4LDevDBGCompID;

#ifndef CONFIG_HOST_PLATFORM_USERSPACE
extern struct semaphore g_module_lock;
#else
#include <semaphore.h>
extern sem_t *g_module_lock;
#endif


#ifndef CONFIG_HOST_PLATFORM_USERSPACE
/*
#define PL300_USB_EVK_VID_FW_FILENAME	"/mnt/usb_qcam.bin"
#define PL300_USB_EVB_VID_FW_FILENAME	"/mnt/codeconly.bin"
#define PL300_PCIE_EVK_VID_FW_FILENAME	"/mnt/codeconly.bin"
#define PL300_USB_CAM_VID_FW_FILENAME	"/mnt/usb_webcam.bin"
#define PL300_USB_FC3_VID_FW_FILENAME	"/mnt/fc3_usbwebcam.bin"
#define PL300_STR_EVK_VID_FW_FILENAME	"/mnt/codeconly.bin"
#define PL300_AUD_FW_FILENAME			"/mnt/arc_codec.bin"
*/

#ifdef CONFIG_HOST_PLATFORM_ASPEN
#define PL300_USB_EVK_VID_FW_FILENAME	"/tmp/bin/qpvidfwusb.bin"
#define PL300_USB_EVB_VID_FW_FILENAME	"/tmp/bin/qpvidfwpcie.bin"
#define PL300_PCIE_EVK_VID_FW_FILENAME	"/tmp/bin/qpvidfwpcie.bin"
#define PL300_USB_CAM_VID_FW_FILENAME	"/tmp/bin/qpvidfwusbcam.bin"
#define PL300_USB_CUS1_VID_FW_FILENAME	"/tmp/bin/qpvidfwcustcam1.bin"
#define PL300_USB_CUS2_VID_FW_FILENAME	"/tmp/bin/qpvidfwcustcam2.bin"
#define PL300_USB_CUS3_VID_FW_FILENAME	"/tmp/bin/qpvidfwcustcam3.bin"
#define PL300_USB_CUS4_VID_FW_FILENAME	"/tmp/bin/qpvidfwcustcam4.bin"
#define PL300_USB_FC3_VID_FW_FILENAME	"/tmp/bin/fc3_usbwebcam.bin"
#define PL300_STR_EVK_VID_FW_FILENAME	"/tmp/bin/qpvidfwpcie.bin"
#define PL300_AUD_FW_FILENAME			"/tmp/bin/qpaudfw.bin"
#else
#define PL300_USB_EVK_VID_FW_FILENAME	"/mnt/qpvidfwusb.bin"
#define PL300_USB_EVB_VID_FW_FILENAME	"/mnt/qpvidfwpcie.bin"
#define PL300_PCIE_EVK_VID_FW_FILENAME	"/mnt/qpvidfwpcie.bin"
#define PL300_USB_CAM_VID_FW_FILENAME	"/mnt/qpvidfwusbcam.bin"
#define PL300_USB_CUS1_VID_FW_FILENAME	"/mnt/qpvidfwcustcam1.bin"
#define PL300_USB_CUS2_VID_FW_FILENAME	"/mnt/qpvidfwcustcam2.bin"
#define PL300_USB_CUS3_VID_FW_FILENAME	"/mnt/qpvidfwcustcam3.bin"
#define PL300_USB_CUS4_VID_FW_FILENAME	"/mnt/qpvidfwcustcam4.bin"
#define PL300_USB_FC3_VID_FW_FILENAME	"/mnt/fc3_usbwebcam.bin"
#define PL300_STR_EVK_VID_FW_FILENAME	"/mnt/qpvidfwpcie.bin"
#define PL300_AUD_FW_FILENAME			"/mnt/qpaudfw.bin"
#endif
#else
#define PL300_USB_EVK_VID_FW_FILENAME	"./qpvidfwusb.bin"
#define PL300_USB_EVB_VID_FW_FILENAME	"./qpvidfwpcie.bin"
#define PL300_PCIE_EVK_VID_FW_FILENAME	"./qpvidfwpcie.bin"
#define PL300_USB_CAM_VID_FW_FILENAME	"./qpvidfwusbcam.bin"
#define PL300_USB_CUS1_VID_FW_FILENAME	"./qpvidfwcustcam1.bin"
#define PL300_USB_CUS2_VID_FW_FILENAME	"./qpvidfwcustcam2.bin"
#define PL300_USB_CUS3_VID_FW_FILENAME	"./qpvidfwcustcam3.bin"
#define PL300_USB_CUS4_VID_FW_FILENAME	"./qpvidfwcustcam4.bin"
#define PL300_USB_FC3_VID_FW_FILENAME	"./fc3_usbwebcam.bin"
#define PL300_STR_EVK_VID_FW_FILENAME	"./qpvidfwpcie.bin"
#define PL300_AUD_FW_FILENAME			"./qpaudfw.bin"
#endif
/////////////////////////////////////////////////////////////////////////////
//
//

CDevice	*g_pPlV4lDevices[PLV4LDEV_MAX_DEVICES] =
{
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0
};

/////////////////////////////////////////////////////////////////////////////
//
//

// V4L interfaces
//
#ifndef CONFIG_HOST_PLATFORM_USERSPACE

#if 0
static ssize_t plv4l_read(struct file *filp,
						  char *buf,
						  size_t count,
						  loff_t *pos
						  )
{
	return (-EPERM);
}



static ssize_t plv4l_write(struct file *filp,
						   const char *buf,
						   size_t count,
						   loff_t *pos
						   )
{
	return (-EPERM);
}
#endif


#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
static int plv4l_open(struct file *filp)
#else //LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)
static int plv4l_open(struct inode *inode,
					  struct file *filp
					  )
#endif //LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
{
	int		lErr = 0;
    int			i, j;
	CDevice		*pDevice;
	CComponent	*pComponent;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
    int			minor = video_devdata(filp)->minor;;
#else //LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)
	int			minor = MINOR(inode->i_rdev);
#endif //LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)

	down(&g_module_lock);

	for (i = 0; i < PLV4LDEV_MAX_DEVICES; i++)
	{
        pDevice = g_pPlV4lDevices[i];
        if (pDevice)
		{
			for (j = 0; j < MAX_COMPONENT_TYPE; j++)
			{
                if (pDevice->m_pVideoDevices[j])
				{
                    if (pDevice->m_pVideoDevices[j]->minor == minor)
					{
						// got it
						goto _do_open;
                    }
                }
            }
        }
    }

	up(&g_module_lock);


	// not found
	//
	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
				 "%s() minor(%d) not found\n",
				 __FUNCTION__,
				 minor
				 ));
	return (-ENODEV);

_do_open:

	// create and open the component
	//
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)) && !defined(CONFIG_HOST_PLATFORM_PL330SOC)
	pComponent = kmalloc(sizeof(CComponent),
						 GFP_KERNEL
						 );
#else
	pComponent = vmalloc(sizeof(CComponent));
#endif
	if (pComponent)
	{
		if (!CComponent_Constructor(pComponent,
									pDevice->m_pVideoDevices[j],
									pDevice,
									j
									))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
						 "%s() minor(%d) CComponent_Constructor failed!!!\n",
						 __FUNCTION__,
						 minor
						 ));
		#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)) && !defined(CONFIG_HOST_PLATFORM_PL330SOC)
			kfree((void *)pComponent);
		#else
			vfree((void *)pComponent);
		#endif
			lErr = -ENOMEM;
		}
		else
		{
			// component open will add the component to the object manager
			//
			CComponent_Open(pComponent);

			// save to file private data
			//
            filp->private_data = pComponent;
		}
	}
	else
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
					 "%s() minor(%d) can not allocate pComponent\n",
					 __FUNCTION__,
					 minor
					 ));
		lErr = -ENOMEM;
	}

	up(&g_module_lock);

	return (lErr);
}



#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
static long plv4l_ioctl(struct file *filp,
					   unsigned int cmd,
					   unsigned long arg
					   )
#else //LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)
static long plv4l_ioctl(struct inode *inode,
					   struct file *filp,
					   unsigned int cmd,
					   unsigned long arg
					   )
#endif //LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
{
    int			i, j;
	CDevice		*pDevice;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
    int			minor = video_devdata(filp)->minor;;
#else //LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)
	int			minor = MINOR(inode->i_rdev);
#endif //LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)

	down(&g_module_lock);

	for (i = 0; i < PLV4LDEV_MAX_DEVICES; i++)
	{
        pDevice = g_pPlV4lDevices[i];
        if (pDevice)
		{
			for (j = 0; j < MAX_COMPONENT_TYPE; j++)
			{
                if (pDevice->m_pVideoDevices[j])
				{
                    if (pDevice->m_pVideoDevices[j]->minor == minor)
					{
						// got it
                        up(&g_module_lock);
						goto _do_ioctl;
                    }
                }
            }
        }
    }

	up(&g_module_lock);


	// not found
	//
	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_WARN),
				 "%s() minor(%d) not found\n",
				 __FUNCTION__,
				 minor
				 ));
	return (-ENODEV);

_do_ioctl:

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
	return (video_usercopy(filp,
						   cmd,
						   arg,
						   (v4l2_kioctl) CComponent_Ioctl
						   ));
#else //LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)
	return (video_usercopy(inode,
						   filp,
						   cmd,
						   arg,
						   CComponent_Ioctl
						   ));
#endif //LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
}



#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
static int plv4l_close(struct file *filp)
#else //LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)
static int plv4l_close(struct inode *inode,
					   struct file *filp
					   )
#endif //LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
{
	long		lErr = -EPERM;
    int			i, j;
    CDevice		*pDevice;
    CComponent	*pComponent = (CComponent *)filp->private_data;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
    int			minor = video_devdata(filp)->minor;;
#else //LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)
    int			minor = MINOR(inode->i_rdev);
#endif //LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)

	down(&g_module_lock);

    for (i = 0; i < PLV4LDEV_MAX_DEVICES; i++)
    {
        pDevice = g_pPlV4lDevices[i];
        if (pDevice)
        {
            for (j = 0; j < MAX_COMPONENT_TYPE; j++)
            {
                if (pDevice->m_pVideoDevices[j])
                {
                    if (pDevice->m_pVideoDevices[j]->minor == minor)
                    {
                         // got it
                         goto _do_close;
                    }
                }
            }
        }
    }

	up(&g_module_lock);

    // not found
    //
    QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
				 "%s() minor(%d) not found\n",
				 __FUNCTION__,
				 minor
				 ));
    return (-ENODEV);

_do_close:
	if (pComponent)
	{
		CComponent_Close(pComponent);
		CComponent_Destructor(pComponent);
	#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)) && !defined(CONFIG_HOST_PLATFORM_PL330SOC)
		kfree((void *)pComponent);
	#else
		vfree((void *)pComponent);
	#endif
		lErr = 0;
	}

	up(&g_module_lock);
    return lErr;
}


static unsigned int plv4l_poll(struct file *filp,
							   poll_table *wait
							   )
{
	int		lErr = 0;
    CComponent	*pComponent = (CComponent *)filp->private_data;

	//printk("plv4l_poll called\n");

	if (pComponent)
	{
		lErr = CComponent_Poll(pComponent,
								filp, wait
								);
	}

	return (lErr);/*POLLIN|POLLRDNORM*/
	//return (0);
}



static void plv4l_mmap_open(struct vm_area_struct *vma)
{
    return;
}


static void plv4l_mmap_close(struct vm_area_struct *vma)
{
    return;
}



struct vm_operations_struct plv4l_mmap_ops =
{
	open:	plv4l_mmap_open,
	close:	plv4l_mmap_close,
};



static int plv4l_mmap(struct file *filp,
					  struct vm_area_struct *vma
					  )
{
	int	lErr = -EPERM;
    CComponent	*pComponent = (CComponent *)filp->private_data;

	vma->vm_ops = &plv4l_mmap_ops;

	if (pComponent)
	{
		return (CComponent_Mmap(pComponent,
								vma
								));
	}

    return lErr;
}



#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
static struct v4l2_file_operations plv4l_fops =
#else //LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)
static struct file_operations plv4l_fops =
#endif //LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,29)
{
	owner:   THIS_MODULE,
	open:    plv4l_open,
	ioctl:   plv4l_ioctl,
	release: plv4l_close,
	poll:    plv4l_poll,
	mmap:    plv4l_mmap,
};



static struct video_device plv4l_videodevice =
{
	.name	= "SDL H.264 Codec (V4L)",
	.fops	= &plv4l_fops,
	.minor	= -1,
};



/////////////////////////////////////////////////////////////////////////////
//
//


#ifdef __LINUX24__
inline struct video_device *video_device_alloc(void)
{
    struct video_device *vfd;

    vfd = kmalloc(sizeof(*vfd),GFP_KERNEL);
    if (NULL == vfd) {
        return NULL;
    }
    memset(vfd,0,sizeof(*vfd));
    return vfd;
}
#endif //__LINUX24__



inline void video_device_release(struct video_device *vfd)
{
	if (vfd) kfree(vfd);
}

#else // CONFIG_HOST_PLATFORM_USERSPACE

static int plv4l_dev_exist(intptr_t dev)
{
	int ret = 1;
    UserDev *pUserdev;
	pUserdev = (UserDev *)dev;

	sem_wait(g_module_lock);
	if ((dev==0)|| (pUserdev->devidx >= PLV4LDEV_MAX_DEVICES) || (pUserdev->devidx<0))
	{
        ret = 0;
	}
	else
    {
        if ((g_pPlV4lDevices[pUserdev->devidx]==0) ||
            (g_pPlV4lDevices[pUserdev->devidx] != (CDevice *)(pUserdev->devid)))
        {
            ret = 0;
        }
    }
	sem_post(g_module_lock);
	return ret;
}

int plv4l_dev_existed(int devid, int minor)
{
    int			j;
	CDevice		*pDevice;
    UserDev     *pUserdev;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
				 "%s() devid(%d) minor(%d) enter\n",
				 __FUNCTION__,
				 devid, minor
				 ));


	if ((devid < PLV4LDEV_MAX_DEVICES) && (devid>=0))
	{
        //down(&g_module_lock);
        sem_wait(g_module_lock);
        pDevice = g_pPlV4lDevices[devid];
        if (pDevice)
		{
			for (j = 0; j < MAX_COMPONENT_TYPE; j++)
			{
                if (pDevice->m_pVideoDevices[j])
				{
                    if (pDevice->m_pVideoDevices[j]->minor == minor)
					{
				        //up(&g_module_lock);
				        sem_post(g_module_lock);
						return (1);
                    }
                }
            }
        }
        //up(&g_module_lock);
        sem_post(g_module_lock);
    }

	// not found
	//
	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
				 "%s() devid(%d) minor(%d) not found\n",
				 __FUNCTION__,
				 devid, minor
				 ));
	return (0);	
}

intptr_t plv4l_open(int devid, int minor)
{
    int			j;
	CDevice		*pDevice;
	CComponent	*pComponent;
    UserDev     *pUserdev;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
				 "%s() devid(%d) minor(%d) enter\n",
				 __FUNCTION__,
				 devid, minor
				 ));


	if ((devid < PLV4LDEV_MAX_DEVICES) && (devid>=0))
	{
        //down(&g_module_lock);
        sem_wait(g_module_lock);
        pDevice = g_pPlV4lDevices[devid];
        if (pDevice)
		{
			for (j = 0; j < MAX_COMPONENT_TYPE; j++)
			{
                if (pDevice->m_pVideoDevices[j])
				{
                    if (pDevice->m_pVideoDevices[j]->minor == minor)
					{
						// got it
						goto _do_open;
                    }
                }
            }
        }
        //up(&g_module_lock);
        sem_post(g_module_lock);
    }

	// not found
	//
	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
				 "%s() devid(%d) minor(%d) not found\n",
				 __FUNCTION__,
				 devid, minor
				 ));
	return (0);

_do_open:
    
	// create and open the component
	//
    pUserdev = (UserDev *)malloc(sizeof(UserDev));
	pComponent = (CComponent *)malloc(sizeof(CComponent));

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
			 	"%s() allocate pUserdev= %p pComponent = %p j=%d\n",
			 	__FUNCTION__,
				pUserdev, pComponent, j
			 	));
	if ((pComponent) && (pUserdev))
	{
		if (!CComponent_Constructor(pComponent,
									pDevice->m_pVideoDevices[j],
									pDevice,
									j
									))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
						 "%s() minor(%d) CComponent_Constructor failed!!!\n",
						 __FUNCTION__,
						 minor
						 ));
			free((void *)pComponent);
			free((void *)pUserdev);
		}
		else
		{
			// component open will add the component to the object manager
			//
			CComponent_Open(pComponent);
            //up(&g_module_lock);
            sem_post(g_module_lock);

            pUserdev->devidx = devid;
            pUserdev->devid = (intptr_t)(void*)pDevice;
            pUserdev->compid = (intptr_t)(void*)pComponent;
			return ((intptr_t)pUserdev);
		}
	}
	else
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
					 "%s() minor(%d) can not allocate pComponent\n",
					 __FUNCTION__,
					 minor
					 ));
		if (pComponent)
			free((void *)pComponent);
		if (pUserdev)
			free((void *)pUserdev);
	}

    //up(&g_module_lock);
    sem_post(g_module_lock);

	return (0);

}


int plv4l_ioctl(intptr_t dev,
				unsigned int cmd,
				void *arg
                )
{
    if (plv4l_dev_exist(dev)) 
    {
        UserDev *pUserdev;

        pUserdev = (UserDev *)dev;
        return CComponent_Ioctl(pUserdev->compid, cmd, arg);
    }
    else {
        return -EPERM;
    }
}



int plv4l_close(intptr_t dev)
{
	int		ret = -EPERM;
    CComponent	*pComponent;
    UserDev *pUserdev;

    pUserdev = (UserDev *)dev;
    pComponent = (CComponent *)(pUserdev->compid);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
				 "%s() pUserdev=%p enter\n",
				 __FUNCTION__,
				 dev
				 ));

    
    if (plv4l_dev_exist(dev)) 
    {
        if (pComponent)
        {
            CComponent_Close(pComponent);
            CComponent_Destructor(pComponent);
            free((void *)pComponent);
            ret = 0;
        }
    }
    if (dev)
    {
        free((void *)dev);
    }
    return ret;
}

void *plv4l_mmap(intptr_t dev, size_t port, size_t offset)
{
    if (plv4l_dev_exist(dev)) 
    {
	    UserDev *pUserdev;

		pUserdev = (UserDev *)dev;
        return CComponent_Mmap(pUserdev->compid, port, offset);
    }
    else {
        return 0;
    }
}

#endif // CONFIG_HOST_PLATFORM_USERSPACE

static EQP_Errors CDevice_Callback(TQP_PVOID Context,
								   TQP_UINT32 dwCode,
								   TQP_PVOID pParam
								   )
{
	return (QPERR_SUCCESS);
}


/////////////////////////////////////////////////////////////////////////////
//
//


// constructor
//
CDevice * CDevice_Constructor(CDevice *pDevice,
							  TQP_UINT32 id,
							  PLV4LDEV_TYPE type,
							  PPLV4LDEV_PARENT_DEV pDev,
#ifndef CONFIG_HOST_PLATFORM_USERSPACE
							  struct device *pGenericDevice,
#else
							  void *pGenericDevice,
#endif
							  TQP_PVOID RegisterBase,
							  TQP_PVOID MemoryBase,
							  TQP_UINT32 PageSize,
							  TQP_UINT32 InterruptNumber,
							  TQP_UINT32 DefaultFWMemMode
							  )
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
				"%s()-> Entry\n",
				__FUNCTION__
				));
	if (pDevice)
	{
		char	*pvidfilename = 0;
		char	*paudfilename = 0;
		int		i;

		// CObject
		//
		CObject_Constructor(&pDevice->m_Object,
							QP_NULL,
		   					OBJECT_CRITICAL_LIGHT
		   					);

		// initialize members
		//
		pDevice->m_id = id;
		if ((type > MAX_PLV4LDEV_TYPE) ||
			(type <= PLV4LDEV_TYPE_NONE)
			)
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
						 "%s()-> Wrong type %d, MAX=%d, NONE=%d\n",
						 __FUNCTION__,
						 type,
						 MAX_PLV4LDEV_TYPE,
						 PLV4LDEV_TYPE_NONE
						 ));
			return (QP_NULL);
		}
		pDevice->m_type = type;

		for (i = 0; i < MAX_COMPONENT_TYPE; i++)
		{
			pDevice->m_pVideoDevices[i] = QP_NULL;
		}

		pDevice->m_device.read = pDev->read;
		pDevice->m_pGenericDevice = pGenericDevice;

		switch (type)
		{
			case PLV4LDEV_TYPE_QL300_USB_EVB:
				pDevice->m_initData.bDownloadFW                         = QP_TRUE;

				pDevice->m_initData.mpgCodecInitData.bDontInitHW        = QP_FALSE;
				pDevice->m_initData.mpgCodecInitData.bUseExtFW          = QP_TRUE;
				pvidfilename                                            = PL300_USB_EVB_VID_FW_FILENAME;
				paudfilename                                            = PL300_AUD_FW_FILENAME;
				pDevice->m_initData.mpgCodecInitData.AccessMode         = QPHCI_MODE_INDIRECT;
				pDevice->m_initData.mpgCodecInitData.BusType            = QPHCI_BUS_USB;
				pDevice->m_initData.mpgCodecInitData.MemType            = QPCODEC_MEM_TYPE_mDDR;
				pDevice->m_initData.mpgCodecInitData.MemSize            = 512;
				pDevice->m_initData.mpgCodecInitData.ChipType           = QPCODEC_CHIP_ARTESA;

				pDevice->m_usbBusData.dwPipeCmdWrite                    = 1;
				pDevice->m_usbBusData.dwPipeCmdRead                     = 3;
				pDevice->m_usbBusData.dwPipeDataWrite                   = 0;
				pDevice->m_usbBusData.dwPipeDataRead                    = 2;

				pDevice->m_initData.mpgCodecInitData.BusData            = &pDevice->m_usbBusData;
				pDevice->m_initData.mpgCodecInitData.BusDataSize        = sizeof(QPCODEC_USB_BUS_DATA);

				pDevice->m_initData.mpgCodecInitData.ErrorRecovery      = QPCODEC_ERROR_RECOVERY_ALWAYS;

				pDevice->m_initData.vidDecoderInitData.type             = VIDDECODER_DUMMY;
				pDevice->m_initData.vidDecoderInitData.channel          = VIDEO_SOURCE_DIGITAL1;
				pDevice->m_initData.vidDecoderInitData.multiType        = MULTIPLE_VIDDECODER_NONE;
				pDevice->m_initData.vidEncoderInitData.type             = VIDENCODER_DUMMY;
				pDevice->m_initData.vidEncoderInitData.standard         = VIDEO_STANDARD_NTSC;
				pDevice->m_initData.tunerInitData.type                  = TUNER_DUMMY;
				pDevice->m_initData.tvAudioInitData.type                = TVAUDIO_DUMMY;
				pDevice->m_initData.audCodecInitData.type               = AUDCODEC_DUMMY;
				pDevice->m_initData.i2cInitData.type					= QPI2C_TYPE_USE_QLCODEC;
				pDevice->m_initData.i2cInitData.clk						= 14;
				pDevice->m_initData.i2cInitData.data					= 15;
				pDevice->m_initData.i2cInitDataEx.type					= QPI2C_TYPE_DUMMY;
				pDevice->m_initData.i2cInitDataEx.clk					= 14;
				pDevice->m_initData.i2cInitDataEx.data					= 15;
        		break;

    		case PLV4LDEV_TYPE_QL300_USB_EVK:
				pDevice->m_initData.bDownloadFW                         = QP_TRUE;

				pDevice->m_initData.mpgCodecInitData.bDontInitHW        = QP_FALSE;
#ifdef CONFIG_FW_INCLUDED_IN_DRIVER
				pDevice->m_initData.mpgCodecInitData.bUseExtFW          = QP_FALSE;
#else
				pDevice->m_initData.mpgCodecInitData.bUseExtFW          = QP_TRUE;
#endif
				pvidfilename                                            = PL300_USB_EVK_VID_FW_FILENAME;
				paudfilename                                            = PL300_AUD_FW_FILENAME;
        		pDevice->m_initData.mpgCodecInitData.AccessMode         = QPHCI_MODE_DIRECT;
        		pDevice->m_initData.mpgCodecInitData.BusType            = QPHCI_BUS_USB;
        		pDevice->m_initData.mpgCodecInitData.MemType            = QPCODEC_MEM_TYPE_mDDR;
        		pDevice->m_initData.mpgCodecInitData.MemSize            = 512;
        		pDevice->m_initData.mpgCodecInitData.ChipType           = QPCODEC_CHIP_ARTESA;

        		pDevice->m_usbBusData.dwPipeCmdWrite                    = 1;
        		pDevice->m_usbBusData.dwPipeCmdRead                     = 3;
        		pDevice->m_usbBusData.dwPipeDataWrite                   = 0;
        		pDevice->m_usbBusData.dwPipeDataRead                    = 2;

        		pDevice->m_initData.mpgCodecInitData.BusData            = &pDevice->m_usbBusData;
        		pDevice->m_initData.mpgCodecInitData.BusDataSize        = sizeof(QPCODEC_USB_BUS_DATA);

				pDevice->m_initData.mpgCodecInitData.ErrorRecovery      = QPCODEC_ERROR_RECOVERY_ALWAYS;

#ifdef CONFIG_HOST_PLATFORM_ZYNQ
        		pDevice->m_initData.vidDecoderInitData.type             = VIDDECODER_DUMMY;
#else
        		pDevice->m_initData.vidDecoderInitData.type             = VIDDECODER_ADI7441;
#endif
				pDevice->m_initData.vidDecoderInitData.channel          = VIDEO_SOURCE_DIGITAL1;
				pDevice->m_initData.vidDecoderInitData.multiType        = MULTIPLE_VIDDECODER_NONE;
#if defined(CONFIG_HOST_PLATFORM_ZYNQ) || defined(CONFIG_HOST_PLATFORM_ASPEN)
        		pDevice->m_initData.vidEncoderInitData.type             = VIDENCODER_DUMMY;
#else
        		pDevice->m_initData.vidEncoderInitData.type             = VIDENCODER_ADI9889;
#endif
        		pDevice->m_initData.vidEncoderInitData.standard         = VIDEO_STANDARD_720P;
        		pDevice->m_initData.tunerInitData.type                  = TUNER_DUMMY;
        		pDevice->m_initData.tvAudioInitData.type                = TVAUDIO_DUMMY;
        		pDevice->m_initData.audCodecInitData.type               = AUDCODEC_DUMMY;
				pDevice->m_initData.i2cInitData.type					= QPI2C_TYPE_USE_USB;
				pDevice->m_initData.i2cInitData.clk						= 14;
				pDevice->m_initData.i2cInitData.data					= 15;
#ifdef CONFIG_USE_USB_SW_I2C
				pDevice->m_initData.i2cInitDataEx.type					= QPI2C_TYPE_USE_USB_SWI2C;
#else
				pDevice->m_initData.i2cInitDataEx.type					= QPI2C_TYPE_DUMMY;
#endif
                pDevice->m_initData.mpgCodecInitData.GPIODirections     = 0;
				pDevice->m_initData.mpgCodecInitData.GPIOValues         = 0;
				pDevice->m_initData.i2cInitDataEx.clk					= 14;
				pDevice->m_initData.i2cInitDataEx.data					= 15;
        		break;

    		case PLV4LDEV_TYPE_QL300_USB_CAM:
				pDevice->m_initData.bDownloadFW                         = QP_TRUE;

				pDevice->m_initData.mpgCodecInitData.bDontInitHW        = QP_FALSE;
#ifdef CONFIG_FW_INCLUDED_IN_DRIVER
				pDevice->m_initData.mpgCodecInitData.bUseExtFW          = QP_FALSE;
#else
				pDevice->m_initData.mpgCodecInitData.bUseExtFW          = QP_TRUE;
#endif
				pvidfilename                                            = PL300_USB_CAM_VID_FW_FILENAME;
				paudfilename                                            = PL300_AUD_FW_FILENAME;
        		pDevice->m_initData.mpgCodecInitData.AccessMode         = QPHCI_MODE_DIRECT;
        		pDevice->m_initData.mpgCodecInitData.BusType            = QPHCI_BUS_USB;
        		pDevice->m_initData.mpgCodecInitData.MemType            = QPCODEC_MEM_TYPE_mDDR;
        		pDevice->m_initData.mpgCodecInitData.MemSize            = 512;
        		pDevice->m_initData.mpgCodecInitData.ChipType           = QPCODEC_CHIP_ARTESA;

        		pDevice->m_usbBusData.dwPipeCmdWrite                    = 1;
        		pDevice->m_usbBusData.dwPipeCmdRead                     = 3;
        		pDevice->m_usbBusData.dwPipeDataWrite                   = 0;
        		pDevice->m_usbBusData.dwPipeDataRead                    = 2;

        		pDevice->m_initData.mpgCodecInitData.BusData            = &pDevice->m_usbBusData;
        		pDevice->m_initData.mpgCodecInitData.BusDataSize        = sizeof(QPCODEC_USB_BUS_DATA);

				pDevice->m_initData.mpgCodecInitData.ErrorRecovery      = QPCODEC_ERROR_RECOVERY_ALWAYS;

        		pDevice->m_initData.vidDecoderInitData.type             = VIDDECODER_USBFW;
				pDevice->m_initData.vidDecoderInitData.channel          = VIDEO_SOURCE_DIGITAL1;
				pDevice->m_initData.vidDecoderInitData.multiType        = MULTIPLE_VIDDECODER_NONE;
        		pDevice->m_initData.vidEncoderInitData.type             = VIDENCODER_DUMMY;
        		pDevice->m_initData.vidEncoderInitData.standard         = VIDEO_STANDARD_720P;
        		pDevice->m_initData.tunerInitData.type                  = TUNER_DUMMY;
        		pDevice->m_initData.tvAudioInitData.type                = TVAUDIO_DUMMY;
#ifdef CONFIG_NO_FM31
        		pDevice->m_initData.audCodecInitData.type               = AUDCODEC_DUMMY;
#else
        		pDevice->m_initData.audCodecInitData.type               = AUDCODEC_FM31_BUS2;
#endif
				pDevice->m_initData.i2cInitData.type					= QPI2C_TYPE_USE_USB;
				pDevice->m_initData.i2cInitData.clk						= 14;
				pDevice->m_initData.i2cInitData.data					= 15;
#ifdef CONFIG_NO_FM31
				pDevice->m_initData.i2cInitDataEx.type					= QPI2C_TYPE_DUMMY;
				pDevice->m_initData.i2cInitDataEx.clk					= 14;
				pDevice->m_initData.i2cInitDataEx.data					= 15;
#else
				pDevice->m_initData.i2cInitDataEx.type					= QPI2C_TYPE_USE_USB_SWI2C;
				pDevice->m_initData.i2cInitDataEx.clk					= 14;
				pDevice->m_initData.i2cInitDataEx.data					= 15;
#endif
                pDevice->m_initData.mpgCodecInitData.GPIODirections     = 0x17;
                pDevice->m_initData.mpgCodecInitData.GPIOValues         = 0x13;
        		break;

    		case PLV4LDEV_TYPE_QL300_USB_CUS1:
				pDevice->m_initData.bDownloadFW                         = QP_TRUE;

				pDevice->m_initData.mpgCodecInitData.bDontInitHW        = QP_FALSE;
#ifdef CONFIG_FW_INCLUDED_IN_DRIVER
				pDevice->m_initData.mpgCodecInitData.bUseExtFW          = QP_FALSE;
#else
				pDevice->m_initData.mpgCodecInitData.bUseExtFW          = QP_TRUE;
#endif
				pvidfilename                                            = PL300_USB_CUS1_VID_FW_FILENAME;
				paudfilename                                            = PL300_AUD_FW_FILENAME;
        		pDevice->m_initData.mpgCodecInitData.AccessMode         = QPHCI_MODE_DIRECT;
        		pDevice->m_initData.mpgCodecInitData.BusType            = QPHCI_BUS_USB;
        		pDevice->m_initData.mpgCodecInitData.MemType            = QPCODEC_MEM_TYPE_mDDR;
        		pDevice->m_initData.mpgCodecInitData.MemSize            = 512;
        		pDevice->m_initData.mpgCodecInitData.ChipType           = QPCODEC_CHIP_ARTESA;

        		pDevice->m_usbBusData.dwPipeCmdWrite                    = 1;
        		pDevice->m_usbBusData.dwPipeCmdRead                     = 3;
        		pDevice->m_usbBusData.dwPipeDataWrite                   = 0;
        		pDevice->m_usbBusData.dwPipeDataRead                    = 2;

        		pDevice->m_initData.mpgCodecInitData.BusData            = &pDevice->m_usbBusData;
        		pDevice->m_initData.mpgCodecInitData.BusDataSize        = sizeof(QPCODEC_USB_BUS_DATA);

				pDevice->m_initData.mpgCodecInitData.ErrorRecovery      = QPCODEC_ERROR_RECOVERY_ALWAYS;

        		pDevice->m_initData.vidDecoderInitData.type             = VIDDECODER_USBFW;
				pDevice->m_initData.vidDecoderInitData.channel          = VIDEO_SOURCE_DIGITAL1;
				pDevice->m_initData.vidDecoderInitData.multiType        = MULTIPLE_VIDDECODER_NONE;
        		pDevice->m_initData.vidEncoderInitData.type             = VIDENCODER_DUMMY;
        		pDevice->m_initData.vidEncoderInitData.standard         = VIDEO_STANDARD_720P;
        		pDevice->m_initData.tunerInitData.type                  = TUNER_DUMMY;
        		pDevice->m_initData.tvAudioInitData.type                = TVAUDIO_DUMMY;
        		pDevice->m_initData.audCodecInitData.type               = AUDCODEC_DUMMY;	/* 2011/11/02 TBD: should change to support CX20708 */
				pDevice->m_initData.i2cInitData.type					= QPI2C_TYPE_USE_USB;
				pDevice->m_initData.i2cInitData.clk						= 14;
				pDevice->m_initData.i2cInitData.data					= 15;
#ifdef CONFIG_USE_USB_SW_I2C
				pDevice->m_initData.i2cInitDataEx.type					= QPI2C_TYPE_USE_USB_SWI2C;
#else
				pDevice->m_initData.i2cInitDataEx.type					= QPI2C_TYPE_DUMMY;
#endif
                pDevice->m_initData.mpgCodecInitData.GPIODirections     = 0;
				pDevice->m_initData.mpgCodecInitData.GPIOValues         = 0;
				pDevice->m_initData.i2cInitDataEx.clk					= 14;
				pDevice->m_initData.i2cInitDataEx.data					= 15;
        		break;

    		case PLV4LDEV_TYPE_QL300_USB_CUS2:
				pDevice->m_initData.bDownloadFW                         = QP_TRUE;

				pDevice->m_initData.mpgCodecInitData.bDontInitHW        = QP_FALSE;
#ifdef CONFIG_FW_INCLUDED_IN_DRIVER
				pDevice->m_initData.mpgCodecInitData.bUseExtFW          = QP_FALSE;
#else
				pDevice->m_initData.mpgCodecInitData.bUseExtFW          = QP_TRUE;
#endif
				pvidfilename                                            = PL300_USB_CUS2_VID_FW_FILENAME;
				paudfilename                                            = PL300_AUD_FW_FILENAME;
        		pDevice->m_initData.mpgCodecInitData.AccessMode         = QPHCI_MODE_DIRECT;
        		pDevice->m_initData.mpgCodecInitData.BusType            = QPHCI_BUS_USB;
        		pDevice->m_initData.mpgCodecInitData.MemType            = QPCODEC_MEM_TYPE_mDDR;
        		pDevice->m_initData.mpgCodecInitData.MemSize            = 512;
        		pDevice->m_initData.mpgCodecInitData.ChipType           = QPCODEC_CHIP_ARTESA;

        		pDevice->m_usbBusData.dwPipeCmdWrite                    = 1;
        		pDevice->m_usbBusData.dwPipeCmdRead                     = 3;
        		pDevice->m_usbBusData.dwPipeDataWrite                   = 0;
        		pDevice->m_usbBusData.dwPipeDataRead                    = 2;

        		pDevice->m_initData.mpgCodecInitData.BusData            = &pDevice->m_usbBusData;
        		pDevice->m_initData.mpgCodecInitData.BusDataSize        = sizeof(QPCODEC_USB_BUS_DATA);

				pDevice->m_initData.mpgCodecInitData.ErrorRecovery      = QPCODEC_ERROR_RECOVERY_ALWAYS;

        		pDevice->m_initData.vidDecoderInitData.type             = VIDDECODER_USBFW;
				pDevice->m_initData.vidDecoderInitData.channel          = VIDEO_SOURCE_DIGITAL1;
				pDevice->m_initData.vidDecoderInitData.multiType        = MULTIPLE_VIDDECODER_NONE;
        		pDevice->m_initData.vidEncoderInitData.type             = VIDENCODER_DUMMY;
        		pDevice->m_initData.vidEncoderInitData.standard         = VIDEO_STANDARD_720P;
        		pDevice->m_initData.tunerInitData.type                  = TUNER_DUMMY;
        		pDevice->m_initData.tvAudioInitData.type                = TVAUDIO_DUMMY;
        		pDevice->m_initData.audCodecInitData.type               = AUDCODEC_DUMMY; /* 2011/11/02 TBD: should change to support CX20708 */
				pDevice->m_initData.i2cInitData.type					= QPI2C_TYPE_USE_USB;
				pDevice->m_initData.i2cInitData.clk						= 14;
				pDevice->m_initData.i2cInitData.data					= 15;
#ifdef CONFIG_USE_USB_SW_I2C
				pDevice->m_initData.i2cInitDataEx.type					= QPI2C_TYPE_USE_USB_SWI2C;
#else
				pDevice->m_initData.i2cInitDataEx.type					= QPI2C_TYPE_DUMMY;
#endif
				pDevice->m_initData.i2cInitDataEx.clk					= 14;
				pDevice->m_initData.i2cInitDataEx.data					= 15;
                pDevice->m_initData.mpgCodecInitData.GPIODirections     = 0;
				pDevice->m_initData.mpgCodecInitData.GPIOValues         = 0;
        		break;

#ifndef CONFIG_HOST_PLATFORM_USERSPACE
    		case PLV4LDEV_TYPE_QL300_USB_CUS3:
				pDevice->m_initData.bDownloadFW                         = QP_TRUE;

				pDevice->m_initData.mpgCodecInitData.bDontInitHW        = QP_FALSE;
#ifdef CONFIG_FW_INCLUDED_IN_DRIVER
				pDevice->m_initData.mpgCodecInitData.bUseExtFW          = QP_FALSE;
#else
				pDevice->m_initData.mpgCodecInitData.bUseExtFW          = QP_TRUE;
#endif
				pvidfilename                                            = PL300_USB_CUS3_VID_FW_FILENAME;
				paudfilename                                            = PL300_AUD_FW_FILENAME;
        		pDevice->m_initData.mpgCodecInitData.AccessMode         = QPHCI_MODE_DIRECT;
        		pDevice->m_initData.mpgCodecInitData.BusType            = QPHCI_BUS_USB;
        		pDevice->m_initData.mpgCodecInitData.MemType            = QPCODEC_MEM_TYPE_mDDR;
        		pDevice->m_initData.mpgCodecInitData.MemSize            = 512;
        		pDevice->m_initData.mpgCodecInitData.ChipType           = QPCODEC_CHIP_ARTESA;

        		pDevice->m_usbBusData.dwPipeCmdWrite                    = 1;
        		pDevice->m_usbBusData.dwPipeCmdRead                     = 3;
        		pDevice->m_usbBusData.dwPipeDataWrite                   = 0;
        		pDevice->m_usbBusData.dwPipeDataRead                    = 2;

        		pDevice->m_initData.mpgCodecInitData.BusData            = &pDevice->m_usbBusData;
        		pDevice->m_initData.mpgCodecInitData.BusDataSize        = sizeof(QPCODEC_USB_BUS_DATA);

				pDevice->m_initData.mpgCodecInitData.ErrorRecovery      = QPCODEC_ERROR_RECOVERY_ALWAYS;

        		pDevice->m_initData.vidDecoderInitData.type             = VIDDECODER_USBFW;
				pDevice->m_initData.vidDecoderInitData.channel          = VIDEO_SOURCE_DIGITAL1;
				pDevice->m_initData.vidDecoderInitData.multiType        = MULTIPLE_VIDDECODER_NONE;
        		pDevice->m_initData.vidEncoderInitData.type             = VIDENCODER_DUMMY;
        		pDevice->m_initData.vidEncoderInitData.standard         = VIDEO_STANDARD_720P;
        		pDevice->m_initData.tunerInitData.type                  = TUNER_DUMMY;
        		pDevice->m_initData.tvAudioInitData.type                = TVAUDIO_DUMMY;
        		pDevice->m_initData.audCodecInitData.type               = AUDCODEC_DUMMY; /* 2011/11/02 TBD: should change to support CX20708 */
				pDevice->m_initData.i2cInitData.type					= QPI2C_TYPE_USE_USB;
				pDevice->m_initData.i2cInitData.clk						= 14;
				pDevice->m_initData.i2cInitData.data					= 15;
#ifdef CONFIG_USE_USB_SW_I2C
				pDevice->m_initData.i2cInitDataEx.type					= QPI2C_TYPE_USE_USB_SWI2C;
#else
				pDevice->m_initData.i2cInitDataEx.type					= QPI2C_TYPE_DUMMY;
#endif
				pDevice->m_initData.i2cInitDataEx.clk					= 14;
				pDevice->m_initData.i2cInitDataEx.data					= 15;
                pDevice->m_initData.mpgCodecInitData.GPIODirections     = 0;
				pDevice->m_initData.mpgCodecInitData.GPIOValues         = 0;
        		break;

    		case PLV4LDEV_TYPE_QL300_USB_CUS4:
				pDevice->m_initData.bDownloadFW                         = QP_TRUE;

				pDevice->m_initData.mpgCodecInitData.bDontInitHW        = QP_FALSE;
#ifdef CONFIG_FW_INCLUDED_IN_DRIVER
				pDevice->m_initData.mpgCodecInitData.bUseExtFW          = QP_FALSE;
#else
				pDevice->m_initData.mpgCodecInitData.bUseExtFW          = QP_TRUE;
#endif
				pvidfilename                                            = PL300_USB_CUS4_VID_FW_FILENAME;
				paudfilename                                            = PL300_AUD_FW_FILENAME;
        		pDevice->m_initData.mpgCodecInitData.AccessMode         = QPHCI_MODE_DIRECT;
        		pDevice->m_initData.mpgCodecInitData.BusType            = QPHCI_BUS_USB;
        		pDevice->m_initData.mpgCodecInitData.MemType            = QPCODEC_MEM_TYPE_mDDR;
        		pDevice->m_initData.mpgCodecInitData.MemSize            = 512;
        		pDevice->m_initData.mpgCodecInitData.ChipType           = QPCODEC_CHIP_ARTESA;

        		pDevice->m_usbBusData.dwPipeCmdWrite                    = 1;
        		pDevice->m_usbBusData.dwPipeCmdRead                     = 3;
        		pDevice->m_usbBusData.dwPipeDataWrite                   = 0;
        		pDevice->m_usbBusData.dwPipeDataRead                    = 2;

        		pDevice->m_initData.mpgCodecInitData.BusData            = &pDevice->m_usbBusData;
        		pDevice->m_initData.mpgCodecInitData.BusDataSize        = sizeof(QPCODEC_USB_BUS_DATA);

				pDevice->m_initData.mpgCodecInitData.ErrorRecovery      = QPCODEC_ERROR_RECOVERY_ALWAYS;

        		pDevice->m_initData.vidDecoderInitData.type             = VIDDECODER_USBFW;
				pDevice->m_initData.vidDecoderInitData.channel          = VIDEO_SOURCE_DIGITAL1;
				pDevice->m_initData.vidDecoderInitData.multiType        = MULTIPLE_VIDDECODER_NONE;
        		pDevice->m_initData.vidEncoderInitData.type             = VIDENCODER_DUMMY;
        		pDevice->m_initData.vidEncoderInitData.standard         = VIDEO_STANDARD_720P;
        		pDevice->m_initData.tunerInitData.type                  = TUNER_DUMMY;
        		pDevice->m_initData.tvAudioInitData.type                = TVAUDIO_DUMMY;
        		pDevice->m_initData.audCodecInitData.type               = AUDCODEC_DUMMY; /* 2011/11/02 TBD: should change to support CX20708 */
				pDevice->m_initData.i2cInitData.type					= QPI2C_TYPE_USE_USB;
				pDevice->m_initData.i2cInitData.clk						= 14;
				pDevice->m_initData.i2cInitData.data					= 15;
#ifdef CONFIG_USE_USB_SW_I2C
				pDevice->m_initData.i2cInitDataEx.type					= QPI2C_TYPE_USE_USB_SWI2C;
#else
				pDevice->m_initData.i2cInitDataEx.type					= QPI2C_TYPE_DUMMY;
#endif
				pDevice->m_initData.i2cInitDataEx.clk					= 14;
				pDevice->m_initData.i2cInitDataEx.data					= 15;
                pDevice->m_initData.mpgCodecInitData.GPIODirections     = 0x17;
				pDevice->m_initData.mpgCodecInitData.GPIOValues         = 0x13;
        		break;

    		case PLV4LDEV_TYPE_QL300_USB_FC3:
				pDevice->m_initData.bDownloadFW                         = QP_TRUE;

				pDevice->m_initData.mpgCodecInitData.bDontInitHW        = QP_FALSE;
#ifdef CONFIG_FW_INCLUDED_IN_DRIVER
				pDevice->m_initData.mpgCodecInitData.bUseExtFW          = QP_FALSE;
#else
				pDevice->m_initData.mpgCodecInitData.bUseExtFW          = QP_TRUE;
#endif
				pvidfilename                                            = PL300_USB_FC3_VID_FW_FILENAME;
				paudfilename                                            = PL300_AUD_FW_FILENAME;
        		pDevice->m_initData.mpgCodecInitData.AccessMode         = QPHCI_MODE_DIRECT;
        		pDevice->m_initData.mpgCodecInitData.BusType            = QPHCI_BUS_USB;
        		pDevice->m_initData.mpgCodecInitData.MemType            = QPCODEC_MEM_TYPE_mDDR;
        		pDevice->m_initData.mpgCodecInitData.MemSize            = 512;
        		pDevice->m_initData.mpgCodecInitData.ChipType           = QPCODEC_CHIP_ARTESA;

        		pDevice->m_usbBusData.dwPipeCmdWrite                    = 1;
        		pDevice->m_usbBusData.dwPipeCmdRead                     = 3;
        		pDevice->m_usbBusData.dwPipeDataWrite                   = 0;
        		pDevice->m_usbBusData.dwPipeDataRead                    = 2;

        		pDevice->m_initData.mpgCodecInitData.BusData            = &pDevice->m_usbBusData;
        		pDevice->m_initData.mpgCodecInitData.BusDataSize        = sizeof(QPCODEC_USB_BUS_DATA);

				pDevice->m_initData.mpgCodecInitData.ErrorRecovery      = QPCODEC_ERROR_RECOVERY_ALWAYS;

        		pDevice->m_initData.vidDecoderInitData.type             = VIDDECODER_USBFW;
				pDevice->m_initData.vidDecoderInitData.channel          = VIDEO_SOURCE_DIGITAL1;
				pDevice->m_initData.vidDecoderInitData.multiType        = MULTIPLE_VIDDECODER_NONE;
        		pDevice->m_initData.vidEncoderInitData.type             = VIDENCODER_DUMMY;
        		pDevice->m_initData.vidEncoderInitData.standard         = VIDEO_STANDARD_720P;
        		pDevice->m_initData.tunerInitData.type                  = TUNER_DUMMY;
        		pDevice->m_initData.tvAudioInitData.type                = TVAUDIO_DUMMY;
        		pDevice->m_initData.audCodecInitData.type               = AUDCODEC_DUMMY;
				pDevice->m_initData.i2cInitData.type					= QPI2C_TYPE_USE_USB;
				pDevice->m_initData.i2cInitData.clk						= 14;
				pDevice->m_initData.i2cInitData.data					= 15;
				pDevice->m_initData.i2cInitDataEx.type					= QPI2C_TYPE_DUMMY;
				pDevice->m_initData.i2cInitDataEx.clk					= 14;
				pDevice->m_initData.i2cInitDataEx.data					= 15;
                pDevice->m_initData.mpgCodecInitData.GPIODirections     = 0x17;
                pDevice->m_initData.mpgCodecInitData.GPIOValues         = 0x13;
        		break;

    		case PLV4LDEV_TYPE_QL300_PCIE_EVK:
				pDevice->m_initData.bDownloadFW                         = QP_TRUE;

				pDevice->m_initData.mpgCodecInitData.bDontInitHW        = QP_FALSE;
#ifdef CONFIG_FW_INCLUDED_IN_DRIVER
				pDevice->m_initData.mpgCodecInitData.bUseExtFW          = QP_FALSE;
#else
				pDevice->m_initData.mpgCodecInitData.bUseExtFW          = QP_TRUE;
#endif
				pvidfilename                                            = PL300_PCIE_EVK_VID_FW_FILENAME;
				paudfilename                                            = PL300_AUD_FW_FILENAME;
        		pDevice->m_initData.mpgCodecInitData.AccessMode         = QPHCI_MODE_DIRECT;
        		pDevice->m_initData.mpgCodecInitData.BusType            = QPHCI_BUS_PCI;
        		pDevice->m_initData.mpgCodecInitData.MemType            = QPCODEC_MEM_TYPE_mDDR;
        		pDevice->m_initData.mpgCodecInitData.MemSize            = 512;
        		pDevice->m_initData.mpgCodecInitData.ChipType           = QPCODEC_CHIP_ARTESA;

#ifndef CONFIG_HOST_PLATFORM_USERSPACE
        		pDevice->m_initData.mpgCodecInitData.BusData            = pDevice->m_device.p_pciedev;
        		pDevice->m_initData.mpgCodecInitData.BusDataSize        = sizeof(struct pci_dev);
#endif

				pDevice->m_initData.mpgCodecInitData.ErrorRecovery      = QPCODEC_ERROR_RECOVERY_ALWAYS;

        		pDevice->m_initData.vidDecoderInitData.type             = VIDDECODER_DUMMY;
				pDevice->m_initData.vidDecoderInitData.channel          = VIDEO_SOURCE_DIGITAL1;
				pDevice->m_initData.vidDecoderInitData.multiType        = MULTIPLE_VIDDECODER_NONE;
        		pDevice->m_initData.vidEncoderInitData.type             = VIDENCODER_DUMMY;
        		pDevice->m_initData.vidEncoderInitData.standard         = VIDEO_STANDARD_NTSC;
        		pDevice->m_initData.tunerInitData.type                  = TUNER_DUMMY;
        		pDevice->m_initData.tvAudioInitData.type                = TVAUDIO_DUMMY;
        		pDevice->m_initData.audCodecInitData.type               = AUDCODEC_DUMMY;
				pDevice->m_initData.i2cInitData.type					= QPI2C_TYPE_USE_QLCODEC;
				pDevice->m_initData.i2cInitData.clk						= 14;
				pDevice->m_initData.i2cInitData.data					= 15;
				//pDevice->m_initData.i2cInitDataEx.type					= QPI2C_TYPE_DUMMY;
				//peter???
				pDevice->m_initData.i2cInitDataEx.type					= QPI2C_TYPE_USE_QLCODEC;
				pDevice->m_initData.i2cInitDataEx.clk					= 14;
				pDevice->m_initData.i2cInitDataEx.data					= 15;
        		break;

    		case PLV4LDEV_TYPE_QL300_STR_EVK:
				pDevice->m_initData.bDownloadFW                         = QP_TRUE;

				pDevice->m_initData.mpgCodecInitData.bDontInitHW        = QP_FALSE;
				pDevice->m_initData.mpgCodecInitData.bUseExtFW          = QP_TRUE;
				pvidfilename                                            = PL300_STR_EVK_VID_FW_FILENAME;
				paudfilename                                            = PL300_AUD_FW_FILENAME;
        		pDevice->m_initData.mpgCodecInitData.AccessMode         = QPHCI_MODE_DIRECT;
        		pDevice->m_initData.mpgCodecInitData.BusType            = QPHCI_BUS_CUSTOM_1;
        		pDevice->m_initData.mpgCodecInitData.MemType            = QPCODEC_MEM_TYPE_mDDR;
        		pDevice->m_initData.mpgCodecInitData.MemSize            = 512;
        		pDevice->m_initData.mpgCodecInitData.ChipType           = QPCODEC_CHIP_ARTESA;

#ifndef CONFIG_HOST_PLATFORM_USERSPACE
        		pDevice->m_initData.mpgCodecInitData.BusData            = pDevice->m_device.p_dev;
        		pDevice->m_initData.mpgCodecInitData.BusDataSize        = sizeof(struct device);
#endif

				pDevice->m_initData.mpgCodecInitData.ErrorRecovery      = QPCODEC_ERROR_RECOVERY_ALWAYS;

        		pDevice->m_initData.vidDecoderInitData.type             = VIDDECODER_ADI7401;
				pDevice->m_initData.vidDecoderInitData.channel          = VIDEO_SOURCE_DIGITAL1;
				pDevice->m_initData.vidDecoderInitData.multiType        = MULTIPLE_VIDDECODER_NONE;
        		pDevice->m_initData.vidEncoderInitData.type             = VIDENCODER_ADI7393;
        		pDevice->m_initData.vidEncoderInitData.standard         = VIDEO_STANDARD_720P;
        		pDevice->m_initData.tunerInitData.type                  = TUNER_DUMMY;
        		pDevice->m_initData.tvAudioInitData.type                = TVAUDIO_DUMMY;
        		pDevice->m_initData.audCodecInitData.type               = AUDCODEC_DUMMY;
				pDevice->m_initData.i2cInitData.type					= QPI2C_TYPE_USE_QLCODEC;
				pDevice->m_initData.i2cInitData.clk						= 14;
				pDevice->m_initData.i2cInitData.data					= 15;
        		break;

    		case PLV4LDEV_TYPE_QL300_MASTER:
				pDevice->m_initData.bDownloadFW                         = QP_FALSE;

				pDevice->m_initData.mpgCodecInitData.bDontInitHW        = QP_TRUE;
				pDevice->m_initData.mpgCodecInitData.bUseExtFW          = QP_FALSE;
        		pDevice->m_initData.mpgCodecInitData.AccessMode         = QPHCI_MODE_MASTER;
        		pDevice->m_initData.mpgCodecInitData.BusType            = QPHCI_BUS_INTERNAL;
        		pDevice->m_initData.mpgCodecInitData.MemType            = QPCODEC_MEM_TYPE_mDDR;
        		pDevice->m_initData.mpgCodecInitData.MemSize            = 512;
        		pDevice->m_initData.mpgCodecInitData.ChipType           = QPCODEC_CHIP_ARTESA;

#ifndef CONFIG_HOST_PLATFORM_USERSPACE
        		pDevice->m_initData.mpgCodecInitData.BusData            = pDevice->m_device.p_dev;
        		pDevice->m_initData.mpgCodecInitData.BusDataSize        = sizeof(struct device);
#endif

				pDevice->m_initData.mpgCodecInitData.ErrorRecovery      = QPCODEC_ERROR_RECOVERY_MANUAL;

        		pDevice->m_initData.vidDecoderInitData.type             = VIDDECODER_DUMMY;
				pDevice->m_initData.vidDecoderInitData.channel          = VIDEO_SOURCE_DIGITAL1;
				pDevice->m_initData.vidDecoderInitData.multiType        = MULTIPLE_VIDDECODER_NONE;
        		pDevice->m_initData.vidEncoderInitData.type             = VIDENCODER_DUMMY;
        		pDevice->m_initData.vidEncoderInitData.standard         = VIDEO_STANDARD_720P;
        		pDevice->m_initData.tunerInitData.type                  = TUNER_DUMMY;
        		pDevice->m_initData.tvAudioInitData.type                = TVAUDIO_DUMMY;
        		pDevice->m_initData.audCodecInitData.type               = AUDCODEC_DUMMY;
				pDevice->m_initData.i2cInitData.type					= QPI2C_TYPE_USE_QLCODEC;
				pDevice->m_initData.i2cInitData.clk						= 14;
				pDevice->m_initData.i2cInitData.data					= 15;
				pDevice->m_initData.i2cInitDataEx.type					= QPI2C_TYPE_SOC_INTERNAL_GPIO;
				pDevice->m_initData.i2cInitDataEx.clk					= 14;
				pDevice->m_initData.i2cInitDataEx.data					= 15;
        		break;
#endif  // CONFIG_HOST_PLATFORM_USERSPACE
			default:
				return (QP_NULL);
		}

#if defined(CONFIG_QL300_AVIU_SIL9025)
    pDevice->m_initData.vidDecoderInitData.type             = VIDDECODER_SIL9025;
#elif defined(CONFIG_QL300_AVIU_ADV7614)
    pDevice->m_initData.vidDecoderInitData.type             = VIDDECODER_ADV7614;
#elif defined(CONFIG_QL300_AVIU_ADI7401)
    pDevice->m_initData.vidDecoderInitData.type             = VIDDECODER_ADI7401;
#elif defined(CONFIG_QL300_AVIU_PE1005)
    pDevice->m_initData.vidDecoderInitData.type             = VIDDECODER_PE1005;
#elif defined(CONFIG_QL300_AVIU_ADI7441)
    pDevice->m_initData.vidDecoderInitData.type             = VIDDECODER_ADI7441;
#elif defined(CONFIG_QL300_AVIU_IT6603)
    pDevice->m_initData.vidDecoderInitData.type             = VIDDECODER_IT6603;
#elif defined(CONFIG_QL300_AVIU_OV5642)
    pDevice->m_initData.vidDecoderInitData.type             = VIDDECODER_OV5642;
    pDevice->m_initData.mpgCodecInitData.GPIODirections		= pDevice->m_initData.mpgCodecInitData.GPIODirections | (1<<0)|(1<<1);		//gpio0 and gpio1 output high
    pDevice->m_initData.mpgCodecInitData.GPIOValues 		= pDevice->m_initData.mpgCodecInitData.GPIOValues | (1<<0)|(1<<1);
#elif defined(CONFIG_QL300_AVIU_OV10633)
    pDevice->m_initData.vidDecoderInitData.type             = VIDDECODER_OV10633;
#elif defined(CONFIG_QL300_AVIU_NATIVE_MI2010)
    pDevice->m_initData.vidDecoderInitData.type             = VIDDECODER_NATIVE_MI2010;
#endif

#if defined(CONFIG_QL300_AVOU_ADI7393)
    pDevice->m_initData.vidEncoderInitData.type             = VIDENCODER_ADI7393;
#elif defined(CONFIG_QL300_AVOU_SIL9034)
    pDevice->m_initData.vidEncoderInitData.type             = VIDENCODER_SIL9034;
#elif defined(CONFIG_QL300_AVOU_ADI9889)
    pDevice->m_initData.vidEncoderInitData.type             = VIDENCODER_ADI9889;
#endif

#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
    pDevice->m_initData.vidDecoderInitData.type             = pl330_frondend_type;
#endif

        pDevice->m_initData.mpgCodecInitData.RegisterBase       = RegisterBase;
        pDevice->m_initData.mpgCodecInitData.MemoryBase         = MemoryBase;
        pDevice->m_initData.mpgCodecInitData.PageSize           = PageSize;
        pDevice->m_initData.mpgCodecInitData.InterruptNumber    = InterruptNumber;

		pDevice->m_initData.mpgCodecInitData.VerFwAPI           = 1;
		pDevice->m_initData.mpgCodecInitData.FwIntMode          = 0;
#if defined(CONFIG_FW_STATIC_MEM_USAGE)
		QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
					 "%s()-> config to use Static Memory Allocation for FW\n",
                     __FUNCTION__
					 ));
		pDevice->m_initData.mpgCodecInitData.FwFixedMode        = 0;	// static mode
#else
		if (DefaultFWMemMode) {
            QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
						 "%s()-> config to use Dynamic Memory Allocation for FW, module set to dynamic\n",
						 __FUNCTION__
						 ));
            pDevice->m_initData.mpgCodecInitData.FwFixedMode        = 1;	// dynamic mode
        }
        else {
            QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
						 "%s()-> config to use Dynamic Memory Allocation for FW, module set to fix\n",
						 __FUNCTION__
						 ));
            pDevice->m_initData.mpgCodecInitData.FwFixedMode        = 0;	// static
        }
#endif

		pDevice->m_initData.mpgCodecInitData.VIUMode            = 2;
		pDevice->m_initData.mpgCodecInitData.VIUFormat          = 1;
		pDevice->m_initData.mpgCodecInitData.VIUStartPixel      = 0;
		pDevice->m_initData.mpgCodecInitData.VIUStartLine       = 0;
		pDevice->m_initData.mpgCodecInitData.ClkEdge            = 0;
		pDevice->m_initData.mpgCodecInitData.VIUSyncCode1       = 0xf1f1f1da;
		pDevice->m_initData.mpgCodecInitData.VIUSyncCode2       = 0xb6f1f1b6;

		pDevice->m_initData.mpgCodecInitData.AIControls_ai_msb  = 1;
		pDevice->m_initData.mpgCodecInitData.AIControls_lrclk_i = 0;
		pDevice->m_initData.mpgCodecInitData.AIControls_bclk_i  = 0;
		pDevice->m_initData.mpgCodecInitData.AIControls_ai_i2s  = 1;
		pDevice->m_initData.mpgCodecInitData.AIControls_ai_rj   = 0;
#ifdef 	CONFIG_PL_AI_FIX_IN_MASTER
		pDevice->m_initData.mpgCodecInitData.AIControls_ai_m   = 1;
#else
		pDevice->m_initData.mpgCodecInitData.AIControls_ai_m   = 0;
#endif
		pDevice->m_initData.mpgCodecInitData.VOEnable           = QP_TRUE;
		pDevice->m_initData.mpgCodecInitData.VOUMode            = 0;
		pDevice->m_initData.mpgCodecInitData.VOUStartPixel      = 1;
		pDevice->m_initData.mpgCodecInitData.VOUStartLine       = 0;

		pDevice->m_initData.mpgCodecInitData.AOEnable           = QP_TRUE;
		pDevice->m_initData.mpgCodecInitData.AOControls_ao_msb  = 1;
		pDevice->m_initData.mpgCodecInitData.AOControls_lrclk_i = 1;
		pDevice->m_initData.mpgCodecInitData.AOControls_bclk_i  = 0;
		pDevice->m_initData.mpgCodecInitData.AOControls_ao_i2s  = 1;
		pDevice->m_initData.mpgCodecInitData.AOControls_ao_rj   = 1;
#ifdef CONFIG_PL_AO_FIX_IN_SLAVE
		pDevice->m_initData.mpgCodecInitData.AOControls_ao_s    = 1;
#else
		pDevice->m_initData.mpgCodecInitData.AOControls_ao_s    = 0;
#endif
		pDevice->m_initData.mpgCodecInitData.bRestoreUSBMode    = QP_FALSE;

		if (pDevice->m_initData.mpgCodecInitData.bUseExtFW)
		{
			long	lErr;

			lErr = qlf_load(pvidfilename,
							&pDevice->m_initData.mpgCodecInitData.pVideoFW,
							&pDevice->m_initData.mpgCodecInitData.VidFWSize
							);
			if (!lErr)
			{
				lErr = qlf_load(paudfilename,
								&pDevice->m_initData.mpgCodecInitData.pAudioFW,
								&pDevice->m_initData.mpgCodecInitData.AudFWSize
								);
			}

			if (lErr)
			{
				pDevice->m_initData.mpgCodecInitData.bUseExtFW = QP_FALSE;

				if (pDevice->m_initData.mpgCodecInitData.pVideoFW)
				{
#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
					kfree(pDevice->m_initData.mpgCodecInitData.pVideoFW);
#else
					vfree(pDevice->m_initData.mpgCodecInitData.pVideoFW);
#endif
					pDevice->m_initData.mpgCodecInitData.pVideoFW = QP_NULL;
				}
				if (pDevice->m_initData.mpgCodecInitData.pAudioFW)
				{
#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
					kfree(pDevice->m_initData.mpgCodecInitData.pAudioFW);
#else
					vfree(pDevice->m_initData.mpgCodecInitData.pAudioFW);
#endif
					pDevice->m_initData.mpgCodecInitData.pAudioFW = QP_NULL;
				}
			}
		}

		pDevice->m_PowerState = PLV4LDEV_PWR_STATE_D0;

		pDevice->m_pComponentMgr = QP_NULL;

		pDevice->m_pCodecLib = QP_NULL;
		pDevice->m_pMpegCodec = QP_NULL;
		pDevice->m_pVidDecoder = QP_NULL;
		pDevice->m_pVidEncoder = QP_NULL;
		pDevice->m_pTuner = QP_NULL;
		pDevice->m_pTVAudio = QP_NULL;
		pDevice->m_pAudCodec = QP_NULL;

#ifdef SUPPORT_PL3XX_OSD
		memset(&pDevice->m_OSDInfo, 0, sizeof(struct OSDInfo));
#endif
	}

	return (pDevice);
}



void CDevice_Destructor(CDevice *This)
{
	int	i;

	if (This->m_initData.mpgCodecInitData.pVideoFW)
	{
#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
		kfree(This->m_initData.mpgCodecInitData.pVideoFW);
#else
		vfree(This->m_initData.mpgCodecInitData.pVideoFW);
#endif
		This->m_initData.mpgCodecInitData.pVideoFW = QP_NULL;
	}

	if (This->m_initData.mpgCodecInitData.pAudioFW)
	{
#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
		kfree(This->m_initData.mpgCodecInitData.pAudioFW);
#else
		vfree(This->m_initData.mpgCodecInitData.pAudioFW);
#endif
		This->m_initData.mpgCodecInitData.pAudioFW = QP_NULL;
	}

	if (This->m_pComponentMgr)
	{
		CObjectMgr_Destructor(This->m_pComponentMgr);
		kfree((void *)This->m_pComponentMgr);
		This->m_pComponentMgr = QP_NULL;
	}

	for (i = 0; i < MAX_COMPONENT_TYPE; i++)
	{
		if (This->m_pVideoDevices[i])
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_WARN),
						 "%s()-> video_unregister_device type(%d) minor(%d)\n",
						 __FUNCTION__,
						 i,
						 This->m_pVideoDevices[i]->minor
						 ));
#ifndef CONFIG_HOST_PLATFORM_USERSPACE
			video_unregister_device(This->m_pVideoDevices[i]);
#else
			free(This->m_pVideoDevices[i]);
#endif // CONFIG_HOST_PLATFORM_USERSPACE
			This->m_pVideoDevices[i] = QP_NULL;
		}
	}

	CObject_Destructor(&This->m_Object);
}



EQP_Errors CDevice_Create(CDevice *This)
{
	EQP_Errors	qpStatus;
	int			i;

	PROFILING_LOG_INIT();

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
				 "%s This->m_device.read=%p\n",
				 __FUNCTION__, This->m_device.read
				 ));

#ifndef CONFIG_HOST_PLATFORM_USERSPACE

	// allocate and register V4L device
	//
	for (i = 0; i < MAX_COMPONENT_TYPE; i++)
	{
#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,11,0)
        struct v4l2_device *v4l2_dev = &This->m_v4l2Devices[i];

        snprintf(v4l2_dev->name, sizeof(v4l2_dev->name), "pl330_%d", i);
        if (v4l2_device_register(This->m_pGenericDevice, v4l2_dev) < 0) {
			QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
							 "%s()-> (%d) Could not register v4l2_device\n",
							 __FUNCTION__, i));
            return -1;
        }

#endif
		This->m_pVideoDevices[i] = video_device_alloc();
		if (This->m_pVideoDevices[i])
		{
			memcpy(This->m_pVideoDevices[i],
				   &plv4l_videodevice,
				   sizeof(struct video_device)
				   );
#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,11,0)
            This->m_pVideoDevices[i]->v4l2_dev = v4l2_dev;
#endif
#ifndef __LINUX24__
			video_set_drvdata(This->m_pVideoDevices[i],
#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,11,0)
                              This->m_pGenericDevice
#else
                              NULL
#endif
							  );
#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,11,0)
            This->m_pVideoDevices[i]->dev_parent = This->m_pGenericDevice;
#else
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,27)
            This->m_pVideoDevices[i]->parent = This->m_pGenericDevice;
#else //LINUX_VERSION_CODE < KERNEL_VERSION(2,6,27)
            This->m_pVideoDevices[i]->dev = This->m_pGenericDevice;
#endif //LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,27)
#endif
            This->m_pVideoDevices[i]->release = video_device_release;
#endif //!__LINUX24__

			if (video_register_device(This->m_pVideoDevices[i],
									  VFL_TYPE_GRABBER,
									  -1
									  ))
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
							 "%s()-> Unable to register v4l type(%d)\n",
							 __FUNCTION__,
							 i
							 ));
                video_device_release(This->m_pVideoDevices[i]);
                This->m_pVideoDevices[i] = QP_NULL;
            }
			else
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_WARN),
							 "%s()-> Registered v4l type(%d) minor(%d)\n",
							 __FUNCTION__,
							 i,
							 This->m_pVideoDevices[i]->minor
							 ));
            }
		}
        else {
				QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
							 "%s()-> %d can not allocated mem in video_device_alloc()\n",
							 __FUNCTION__,
							 i
							 ));
        }
	}
#else // CONFIG_HOST_PLATFORM_USERSPACE
	// allocate and register V4L device
	//
	for (i = 0; i < MAX_COMPONENT_TYPE; i++)
	{
		This->m_pVideoDevices[i] = malloc(sizeof(struct user_video_device));
		if (This->m_pVideoDevices[i])
		{
            This->m_pVideoDevices[i]->minor = i;
            QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_WARN),
						 "%s()-> Registered v4l type(%d) minor(%d)\n",
						 __FUNCTION__,
						 i,
						 This->m_pVideoDevices[i]->minor
						 ));
		}
	}
#endif // CONFIG_HOST_PLATFORM_USERSPACE

	// create component manager
	//
	This->m_pComponentMgr = (CObjectMgr *)kmalloc(sizeof(CObjectMgr),
												  GFP_KERNEL
												  );
	if (!CObjectMgr_Constructor(This->m_pComponentMgr,
								&This->m_Object,
								OBJECT_CRITICAL_HEAVY,
								QP_NULL
								))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
					 "%s() CObjectMgr_Constructor Failed!!!!\n",
					 __FUNCTION__
					 ));
		CDevice_Close(This);
		return (QPERR_NOMEM);
	}


	// initialize codec library
	//
	qpStatus = QPCodecInitLibrary(This->m_device.read,
								  QP_NULL,
								  &This->m_initData,
								  &This->m_pCodecLib
								  );

	// get pointers to library interfaces
	//
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPCodecLib_GetMpegCodec(This->m_pCodecLib, &This->m_pMpegCodec);
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPCodecLib_GetVideoDecoder(This->m_pCodecLib, &This->m_pVidDecoder);
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPCodecLib_GetVideoEncoder(This->m_pCodecLib, &This->m_pVidEncoder);
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPCodecLib_GetTuner(This->m_pCodecLib, &This->m_pTuner);
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPCodecLib_GetTVAudio(This->m_pCodecLib, &This->m_pTVAudio);
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPCodecLib_GetAudioCodec(This->m_pCodecLib, &This->m_pAudCodec);

	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
					 "%s QPCodecInitLibrary() Failed() Status(%d)\n",
					 __FUNCTION__,
					 qpStatus
					 ));
		CDevice_Close(This);
		return (qpStatus);
	}

	qpStatus = QPGenericDevice_InitDevice(This->m_pCodecLib,
										  CDevice_Callback,
										  This,
										  QP_NULL
										  );

	if (This->m_initData.mpgCodecInitData.pVideoFW)
	{
#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
		kfree(This->m_initData.mpgCodecInitData.pVideoFW);
#else
		vfree(This->m_initData.mpgCodecInitData.pVideoFW);
#endif
		This->m_initData.mpgCodecInitData.pVideoFW = QP_NULL;
	}

	if (This->m_initData.mpgCodecInitData.pAudioFW)
	{
#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
		kfree(This->m_initData.mpgCodecInitData.pAudioFW);
#else
		vfree(This->m_initData.mpgCodecInitData.pAudioFW);
#endif
		This->m_initData.mpgCodecInitData.pAudioFW = QP_NULL;
	}

	if (!QP_SUCCESS(qpStatus))
	{
		CDevice_Close(This);
		return (qpStatus);
	}

	CObject_Init(&This->m_Object);
	return (qpStatus);
}



EQP_Errors CDevice_Close(CDevice *This)
{
	int	i;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
				 "%s\n",
				 __FUNCTION__
				 ));

	// close all the remaining components
	//
	while (0 != CObjectMgr_GetNumberOfObjects(This->m_pComponentMgr))
	{
		CComponent	*pComponent = CObjectMgr_GetObjectByIndex(This->m_pComponentMgr,
															  0
															  );
		if (pComponent)
		{
			CComponent_Close(pComponent);
			CComponent_Destructor(pComponent);
#ifndef CONFIG_HOST_PLATFORM_USERSPACE
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)) && !defined(CONFIG_HOST_PLATFORM_PL330SOC)
			kfree((void *)pComponent);
#else
			vfree((void *)pComponent);
#endif
#else
			free((void *)pComponent);
#endif
		}
	}

	// unregister video device
	//
	for (i = 0; i < MAX_COMPONENT_TYPE; i++)
	{
		if (This->m_pVideoDevices[i])
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_WARN),
						 "%s()-> video_unregister_device type(%d) minor(%d)\n",
						 __FUNCTION__,
						 i,
						 This->m_pVideoDevices[i]->minor
						 ));
#ifndef CONFIG_HOST_PLATFORM_USERSPACE
			video_unregister_device(This->m_pVideoDevices[i]);
#else
			free((void *)This->m_pVideoDevices[i]);
#endif
			This->m_pVideoDevices[i] = QP_NULL;
		}
	}

	// release all interfaces we have acquired
	//
	if (This->m_pAudCodec)
	{
		QPGenericDevice_Release(This->m_pAudCodec);
		This->m_pAudCodec = QP_NULL;
	}

	if (This->m_pTVAudio)
	{
		QPGenericDevice_Release(This->m_pTVAudio);
		This->m_pTVAudio = QP_NULL;
	}

	if (This->m_pTuner)
	{
		QPGenericDevice_Release(This->m_pTuner);
		This->m_pTuner = QP_NULL;
	}

	if (This->m_pVidEncoder)
	{
		QPGenericDevice_Release(This->m_pVidEncoder);
		This->m_pVidEncoder = QP_NULL;
	}

	if (This->m_pVidDecoder)
	{
		QPCodecLib_GetVideoDecoder(This->m_pCodecLib, &This->m_pVidDecoder);

		QPGenericDevice_Release(This->m_pVidDecoder);
		This->m_pVidDecoder = QP_NULL;
	}

	if (This->m_pMpegCodec)
	{
		QPGenericDevice_Release(This->m_pMpegCodec);
		This->m_pMpegCodec = QP_NULL;
	}

	if (This->m_pCodecLib)
	{
		QPGenericDevice_Release(This->m_pCodecLib);
		QPCodecDoneLibrary(This->m_pCodecLib);
		This->m_pCodecLib = QP_NULL;
	}

	CObject_Done(&This->m_Object);

	PORFILING_LOG_FINT();

	return (QPERR_SUCCESS);
}



EQP_Errors CDevice_PowerUp(CDevice *This)
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;

	if (QP_NULL != This->m_pCodecLib)
	{
		qpStatus = This->m_pCodecLib->Enable(This->m_pCodecLib);
	}
	This->m_PowerState = PLV4LDEV_PWR_STATE_D0;
	return (qpStatus);
}



EQP_Errors CDevice_PowerDown(CDevice *This)
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;

	if (QP_NULL != This->m_pCodecLib)
	{
		qpStatus = This->m_pCodecLib->Disable(This->m_pCodecLib);
	}
	This->m_PowerState = PLV4LDEV_PWR_STATE_D3;
	return (qpStatus);
}



/////////////////////////////////////////////////////////////////////////////
//
//

// low level hardware access functions
//

int CDevice_rdmem(CDevice *This,
				   unsigned int ulAddr,
				   unsigned int ulSize,
				   unsigned char *pucaBuf,
				   int isusrbuf
				   )
{
	ICodecLib					*pCodecLib = This->m_pCodecLib;
	EQP_Errors					qpStatus = QPERR_SUCCESS;
	QPCODEC_DIAG_MEMORY_STRUCT	mem;
	TQP_UINT32					dwGot = 0;
	int							nDwordCount = ulSize / 4;
	int							nXfer;
	unsigned int				ulOffset = 0;

    while (nDwordCount && QP_SUCCESS(qpStatus))
	{
		nXfer = min(nDwordCount, QP_MAX_MEMORY_DWORDS);

		mem.address = ulAddr + ulOffset;
		mem.num_dwords = nXfer;

		qpStatus = pCodecLib->Get(pCodecLib,
								  (TQP_REFGUID)&PROPSETID_QPCODEC_DIAG,
								  TQP_NULL_HANDLE,
								  QPCODEC_DIAG_MEM_READ,
								  (TQP_PVOID)&mem,
								  (TQP_PVOID)&mem,
								  &dwGot
								  );
		if (QP_SUCCESS(qpStatus))
		{
			if (isusrbuf)
			{
				copy_to_user(pucaBuf + ulOffset,
							 (unsigned char *)&mem.value[0],
							 nXfer * 4
							 );
			}
			else
			{
				memcpy(pucaBuf + ulOffset,
					   (unsigned char *)&mem.value[0],
					   nXfer * 4
					   );
			}
		}

		nDwordCount -= nXfer;
		ulOffset += (nXfer * 4);
    }

	return (QpToLinuxKerStatus(qpStatus));
}



int CDevice_wrmem(CDevice *This,
				   unsigned int ulAddr,
				   unsigned int ulSize,
				   unsigned char *pucaBuf,
				   int isusrbuf
				   )
{
	ICodecLib					*pCodecLib = This->m_pCodecLib;
	EQP_Errors					qpStatus = QPERR_SUCCESS;
	QPCODEC_DIAG_MEMORY_STRUCT	mem;
	int							nDwordCount = ulSize / 4;
	int							nXfer;
	unsigned int				ulOffset = 0;

    while (nDwordCount && QP_SUCCESS(qpStatus))
	{
		nXfer = min(nDwordCount, QP_MAX_MEMORY_DWORDS);

		mem.address = ulAddr + ulOffset;
		mem.num_dwords = nXfer;

		if (isusrbuf)
		{
			copy_from_user((unsigned char *)&mem.value[0],
						   pucaBuf + ulOffset,
						   nXfer * 4
						   );
		}
		else
		{
			memcpy((unsigned char *)&mem.value[0],
				   pucaBuf + ulOffset,
				   nXfer * 4
				   );
		}

		qpStatus = pCodecLib->Set(pCodecLib,
								  (TQP_REFGUID)&PROPSETID_QPCODEC_DIAG,
								  TQP_NULL_HANDLE,
								  QPCODEC_DIAG_MEM_WRITE,
								  (TQP_PVOID)&mem,
								  QP_NULL,
								  sizeof(QPCODEC_DIAG_MEMORY_STRUCT)
								  );

		nDwordCount -= nXfer;
		ulOffset += (nXfer * 4);
    }

	return (QpToLinuxKerStatus(qpStatus));
}



int CDevice_dmard(CDevice *This,
				   unsigned int ulAddr,
				   unsigned int ulSize,
				   unsigned char *pucaBuf,
				   int isusrbuf
				   )
{
	ICodecLib					*pCodecLib = This->m_pCodecLib;
	EQP_Errors					qpStatus;
	QPCODEC_DIAG_DMA_PTR_STRUCT	dma;
	TQP_UINT32					dwGot = 0;

	if (isusrbuf)
	{
#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
		dma.pData = kmalloc(ulSize, GFP_KERNEL);
#else
		dma.pData = vmalloc_32(ulSize);
#endif
	}
	else
	{
		dma.pData = pucaBuf;
	}

	if (!dma.pData)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
					 "%s() vmalloc %d bytes failed\n",
					 __FUNCTION__,
					 ulSize
					 ));
		return (-ENOMEM);
	}

	dma.deviceAddress = ulAddr;
	dma.size = ulSize;

	qpStatus = pCodecLib->Get(pCodecLib,
							  (TQP_REFGUID)&PROPSETID_QPCODEC_DIAG,
							  TQP_NULL_HANDLE,
							  QPCODEC_DIAG_DMA_READ_PTR,
							  (TQP_PVOID)&dma,
							  (TQP_PVOID)&dma,
							  &dwGot
							  );
	if (isusrbuf)
	{
		if (QP_SUCCESS(qpStatus))
		{
			copy_to_user(pucaBuf,
						 dma.pData,
						 ulSize
						 );
		}
#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
		kfree(dma.pData);
#else
		vfree(dma.pData);
#endif
	}

	return (QpToLinuxKerStatus(qpStatus));
}



int CDevice_dmawr(CDevice *This,
				   unsigned int ulAddr,
				   unsigned int ulSize,
				   unsigned char *pucaBuf,
				   int isusrbuf
				   )
{
	ICodecLib					*pCodecLib = This->m_pCodecLib;
	EQP_Errors					qpStatus;
	QPCODEC_DIAG_DMA_PTR_STRUCT	dma;

	if (isusrbuf)
	{
#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
		dma.pData = kmalloc(ulSize, GFP_KERNEL);
#else
		dma.pData = vmalloc_32(ulSize);
#endif
		if (dma.pData)
		{
			copy_from_user(dma.pData,
						   pucaBuf,
						   ulSize
						   );
		}
	}
	else
	{
		dma.pData = pucaBuf;
	}

	if (!dma.pData)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
					 "%s() vmalloc %d bytes failed\n",
					 __FUNCTION__,
					 ulSize
					 ));
		return (-ENOMEM);
	}

	dma.deviceAddress = ulAddr;
	dma.size = ulSize;

	qpStatus = pCodecLib->Set(pCodecLib,
							  (TQP_REFGUID)&PROPSETID_QPCODEC_DIAG,
							  TQP_NULL_HANDLE,
							  QPCODEC_DIAG_DMA_WRITE_PTR,
							  (TQP_PVOID)&dma,
							  QP_NULL,
							  sizeof(QPCODEC_DIAG_DMA_PTR_STRUCT)
							  );

	if (isusrbuf)
	{
#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
		kfree(dma.pData);
#else
		vfree(dma.pData);
#endif
	}

	return (QpToLinuxKerStatus(qpStatus));
}



int CDevice_rdreg(CDevice *This,
				   unsigned int ulAddr,
				   unsigned char *pucaBuf,
				   int isusrbuf
				   )
{
	ICodecLib						*pCodecLib = This->m_pCodecLib;
	EQP_Errors						qpStatus;
	QPCODEC_DIAG_REGISTER_STRUCT	reg;
	TQP_UINT32						dwGot = 0;

	reg.address = ulAddr;

	qpStatus = pCodecLib->Get(pCodecLib,
							  (TQP_REFGUID)&PROPSETID_QPCODEC_DIAG,
							  TQP_NULL_HANDLE,
							  QPCODEC_DIAG_REG_READ,
							  (TQP_PVOID)&reg,
							  (TQP_PVOID)&reg,
							  &dwGot
							  );
	if (QP_SUCCESS(qpStatus))
	{
		if (isusrbuf)
		{
			copy_to_user(pucaBuf,
						 (unsigned char *)&reg.value,
						 4
						 );
		}
		else
		{
			memcpy(pucaBuf,
				   (unsigned char *)&reg.value,
				   4
				   );
		}
	}

	return (QpToLinuxKerStatus(qpStatus));
}



int CDevice_wrreg(CDevice *This,
				   unsigned int ulAddr,
				   unsigned char *pucaBuf,
				   int isusrbuf
				   )
{
	ICodecLib						*pCodecLib = This->m_pCodecLib;
	EQP_Errors						qpStatus;
	QPCODEC_DIAG_REGISTER_STRUCT	reg;

	if (isusrbuf)
	{
		copy_from_user((unsigned char *)&reg.value,
					   pucaBuf,
					   4
					   );
	}
	else
	{
		memcpy((unsigned char *)&reg.value,
			   pucaBuf,
			   4
			   );
	}

	reg.address = ulAddr;

	qpStatus = pCodecLib->Set(pCodecLib,
							  (TQP_REFGUID)&PROPSETID_QPCODEC_DIAG,
							  TQP_NULL_HANDLE,
							  QPCODEC_DIAG_REG_WRITE,
							  (TQP_PVOID)&reg,
							  QP_NULL,
							  sizeof(QPCODEC_DIAG_REGISTER_STRUCT)
							  );

	return (QpToLinuxKerStatus(qpStatus));
}



int CDevice_rdhci(CDevice *This,
				   unsigned int ulAddr,
				   unsigned char *pucaBuf,
				   int isusrbuf
				   )
{
	ICodecLib						*pCodecLib = This->m_pCodecLib;
	EQP_Errors						qpStatus;
	QPCODEC_DIAG_REGISTER_STRUCT	reg;
	TQP_UINT32						dwGot = 0;

	reg.address = ulAddr;
	reg.value = 0;

	qpStatus = pCodecLib->Get(pCodecLib,
							  (TQP_REFGUID)&PROPSETID_QPCODEC_DIAG,
							  TQP_NULL_HANDLE,
							  QPCODEC_DIAG_HCI_REG_READ,
							  (TQP_PVOID)&reg,
							  (TQP_PVOID)&reg,
							  &dwGot
							  );
	if (QP_SUCCESS(qpStatus))
	{
		if (isusrbuf)
		{
			copy_to_user(pucaBuf,
						 (unsigned char *)&reg.value,
						 4
						 );
		}
		else
		{
			memcpy(pucaBuf,
				   (unsigned char *)&reg.value,
				   4
				   );
		}
	}

	return (QpToLinuxKerStatus(qpStatus));
}



int CDevice_wrhci(CDevice *This,
				   unsigned int ulAddr,
				   unsigned char *pucaBuf,
				   int isusrbuf
				   )
{
	ICodecLib						*pCodecLib = This->m_pCodecLib;
	EQP_Errors						qpStatus;
	QPCODEC_DIAG_REGISTER_STRUCT	reg;

	if (isusrbuf)
	{
		copy_from_user((unsigned char *)&reg.value,
					   pucaBuf,
					   4
					   );
	}
	else
	{
		memcpy((unsigned char *)&reg.value,
			   pucaBuf,
			   4
			   );
	}

	reg.address = ulAddr;

	qpStatus = pCodecLib->Set(pCodecLib,
							  (TQP_REFGUID)&PROPSETID_QPCODEC_DIAG,
							  TQP_NULL_HANDLE,
							  QPCODEC_DIAG_HCI_REG_WRITE,
							  (TQP_PVOID)&reg,
							  QP_NULL,
							  sizeof(QPCODEC_DIAG_REGISTER_STRUCT)
							  );

	return (QpToLinuxKerStatus(qpStatus));
}



int CDevice_rdi2c(CDevice *This,
				   unsigned char ucDevAddr,
				   unsigned int  ulRegSize,
				   unsigned char *pucRegAddr,
				   unsigned int  ulSize,
				   unsigned char *pucaBuf,
				   int isusrbuf
				   )
{
	ICodecLib						*pCodecLib = This->m_pCodecLib;
	EQP_Errors						qpStatus = QPERR_SUCCESS;
	QPCODEC_DIAG_I2C_WRITE_THEN_READ_STRUCT	i2c;
	TQP_UINT32						dwGot = 0;

	i2c.chip_address = ucDevAddr;
	i2c.num_write_bytes = ulRegSize;
	if (ulRegSize)
	{
		memcpy((unsigned char *)&i2c.write_bytes[0],
			   pucRegAddr,
			   ulRegSize
			   );
	}
	i2c.num_read_bytes = ulSize;

	qpStatus = pCodecLib->Get(pCodecLib,
							  (TQP_REFGUID)&PROPSETID_QPCODEC_DIAG,
							  TQP_NULL_HANDLE,
							  QPCODEC_DIAG_I2C_WRITE_THEN_READ,
							  (TQP_PVOID)&i2c,
							  (TQP_PVOID)&i2c,
							  &dwGot
							  );
	if (QP_SUCCESS(qpStatus))
	{
		if (isusrbuf)
		{
			copy_to_user(pucaBuf,
						 (unsigned char *)&i2c.read_bytes[0],
						 ulSize
						 );
		}
		else
		{
			memcpy(pucaBuf,
				   (unsigned char *)&i2c.read_bytes[0],
				   ulSize
				   );
		}
	}

	return (QpToLinuxKerStatus(qpStatus));
}



int CDevice_wri2c(CDevice *This,
				   unsigned char ucDevAddr,
				   unsigned int  ulSize,
				   unsigned char *pucaBuf,
				   int isusrbuf
				   )
{
	ICodecLib				*pCodecLib = This->m_pCodecLib;
	EQP_Errors				qpStatus = QPERR_SUCCESS;
	QPCODEC_DIAG_I2C_STRUCT	i2c;

	if (isusrbuf)
	{
		copy_from_user((unsigned char *)&i2c.bytes[0],
					   pucaBuf,
					   ulSize
					   );
	}
	else
	{
		memcpy((unsigned char *)&i2c.bytes[0],
			   pucaBuf,
			   ulSize
			   );
	}

	qpStatus = pCodecLib->Set(pCodecLib,
							  (TQP_REFGUID)&PROPSETID_QPCODEC_DIAG,
							  TQP_NULL_HANDLE,
							  QPCODEC_DIAG_I2C_WRITE,
							  (TQP_PVOID)&i2c,
							  QP_NULL,
							  sizeof(QPCODEC_DIAG_I2C_STRUCT)
							  );

	return (QpToLinuxKerStatus(qpStatus));
}
