///////////////////////////////////////////////////////////////////////////////
//     
//
// Cdevice.h
//
// Description:
//
//  Device class
//
// History:
//	Dec 17, 2009	(dt) created
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __CDEVICE_H__
#define __CDEVICE_H__


#include "qlautoconf.h"
#ifndef CONFIG_HOST_PLATFORM_USERSPACE
#include <linux/types.h>
#include <linux/file.h>
#include <linux/fs.h>
#include <linux/timer.h>
#include <linux/list.h>
#include <linux/pci.h>


#include <linux/version.h>
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,18)
#include <media/v4l2-dev.h>
#endif
#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,11,0)
#include <media/v4l2-device.h>
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,27)
#include <media/v4l2-ioctl.h>
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,26)
#include <linux/semaphore.h>
#else
#include <asm/semaphore.h>
#endif


#ifdef __LINUX24__
#include "videodev2.h"
#else
#include <linux/videodev2.h>
#endif

#else // CONFIG_HOST_PLATFORM_USERSPACE
#include <stdint.h>
#include <sys/types.h>
//#ifdef CONFIG_HOST_PLATFORM_ANDROID
//#include <linux/time.h>
//#endif // CONFIG_HOST_PLATFORM_ANDROID
#ifdef CONFIG_HOST_PLATFORM_OSX
#include <CoreFoundation/CoreFoundation.h>
#include <IOKit/IOKitLib.h>
#include <IOKit/IOCFPlugIn.h>
#include <IOKit/usb/IOUSBLib.h>
#include <IOKit/usb/USBSpec.h>
#include <IOKit/IOMessage.h>
#else
#include <libusb.h>
#endif
#include <uservideodev2.h>

struct UserDev {
    int devidx;
    intptr_t devid;
    intptr_t compid;
};
typedef struct UserDev  UserDev;
#endif // CONFIG_HOST_PLATFORM_USERSPACE


#include "qptypes.h"
#include "qpcodeclib.h"
#include "qpmpgcodec.h"
#include "cobject.h"
#include "cobjectmgr.h"
#include "ccomponent.h"

#ifdef SUPPORT_PL3XX_OSD
#include "qlosd.h"

struct OSDInfo {
	OSD_DISPLAY_INFO mDisplay;
	unsigned int unFrameBufferAddr;
	#ifdef SUPPORT_VI_OSD
	QPVIOSD_CONTROL mViosdDisplay;
	unsigned int mViosdON;
	#endif
};
#endif

/////////////////////////////////////////////////////////////////////////////
//
//


typedef enum _PLV4LDEV_PowerStates
{
    PLV4LDEV_PWR_STATE_D0	= 0,	// on
    PLV4LDEV_PWR_STATE_D1	,		// stand by
    PLV4LDEV_PWR_STATE_D2	,		// hibernate
    PLV4LDEV_PWR_STATE_D3			// off
} PLV4LDEV_PowerStates, *PPLV4LDEV_PowerStates;


typedef enum _PLV4LDEV_TYPE
{
	PLV4LDEV_TYPE_NONE		= 0,
	PLV4LDEV_TYPE_QL201_FX2,
	PLV4LDEV_TYPE_QL300_FX2,
	PLV4LDEV_TYPE_QL201_USB_EVK,
	PLV4LDEV_TYPE_QL300_USB_EVB,
	PLV4LDEV_TYPE_QL300_USB_EVK,
	PLV4LDEV_TYPE_QL300_USB_CAM,
	PLV4LDEV_TYPE_QL300_USB_CUS1,
	PLV4LDEV_TYPE_QL300_USB_CUS2,
	PLV4LDEV_TYPE_QL300_USB_CUS3,
	PLV4LDEV_TYPE_QL300_USB_CUS4,
	PLV4LDEV_TYPE_QL300_USB_FC3,
	PLV4LDEV_TYPE_QL300_STR_EVK,
	PLV4LDEV_TYPE_QL300_PCIE_EVK,
	PLV4LDEV_TYPE_QL300_MASTER,
	PLV4LDEV_TYPE_END

} PLV4LDEV_TYPE, *PPLV4LDEV_TYPE;

#define MAX_PLV4LDEV_TYPE	PLV4LDEV_TYPE_END


#ifndef CONFIG_HOST_PLATFORM_USERSPACE
typedef union _PLV4LDEV_PARENT_DEV
{
#ifndef __LINUX24__
	struct usb_interface	*p_usbintf;
#else //__LINUX24__
	struct usb_device		*p_usbdev;
#endif //__LINUX24__
	struct pci_dev			*p_pciedev;
	struct device			*p_dev;
	void					*read;

} PLV4LDEV_PARENT_DEV, *PPLV4LDEV_PARENT_DEV;
#else // CONFIG_HOST_PLATFORM_USERSPACE
typedef union _PLV4LDEV_PARENT_DEV
{
#ifdef CONFIG_HOST_PLATFORM_OSX
    IOUSBInterfaceInterface182 **p_usbdev;
#else
    libusb_device           *p_usbdev;
#endif
	void					*read;

} PLV4LDEV_PARENT_DEV, *PPLV4LDEV_PARENT_DEV;

struct user_video_device {
	int minor;
};

#endif // CONFIG_HOST_PLATFORM_USERSPACE


/////////////////////////////////////////////////////////////////////////////
//
//


#ifndef __CDEVICE_FWD_DEFINED__
#define __CDEVICE_FWD_DEFINED__
typedef struct CDevice CDevice;
#endif //!__CDEVICE_FWD_DEFINED__


struct CDevice
{
	// public CObject
	//
	CObject					m_Object;

	// CDevice
	//
	TQP_UINT32				m_id;
	PLV4LDEV_TYPE			m_type;
#ifndef CONFIG_HOST_PLATFORM_USERSPACE
	PLV4LDEV_PARENT_DEV		m_device;
	struct device			*m_pGenericDevice;

	// V4L video devices
	//
    struct video_device		*m_pVideoDevices[MAX_COMPONENT_TYPE];
#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,11,0)
    struct v4l2_device       m_v4l2Devices[MAX_COMPONENT_TYPE];
#endif
#else  // CONFIG_HOST_PLATFORM_USERSPACE
#ifdef CONFIG_HOST_PLATFORM_OSX
    io_object_t             usbNotification;
#endif
	PLV4LDEV_PARENT_DEV		m_device;
	void *m_pGenericDevice;

	// V4L video devices
	//
    struct user_video_device	*m_pVideoDevices[MAX_COMPONENT_TYPE];
#endif

	// init data
	//
	QPCODECLIB_INITDATA		m_initData;

	// saved resource list
	//
	QPCODEC_USB_BUS_DATA	m_usbBusData;

	// device power state
	//
	PLV4LDEV_PowerStates	m_PowerState;

	// component manager
	//
	CObjectMgr				*m_pComponentMgr;

	// codec interface objects goes here
	//
	ICodecLib				*m_pCodecLib;
	IMpegCodec				*m_pMpegCodec;
	IVideoDecoder			*m_pVidDecoder;
	IVideoEncoder			*m_pVidEncoder;
	ITuner					*m_pTuner;
	ITVAudio				*m_pTVAudio;
	IAudioCodec				*m_pAudCodec;

#ifdef SUPPORT_PL3XX_OSD
	struct OSDInfo	m_OSDInfo;
#endif

};



CDevice * CDevice_Constructor(CDevice *pDevice,
							  TQP_UINT32 id,
							  PLV4LDEV_TYPE type,
							  PPLV4LDEV_PARENT_DEV pDev,
#ifndef CONFIG_HOST_PLATFORM_USERSPACE
							  struct device *pGenericDevice,
#else
							  void *pGenericDevice,
#endif
							  TQP_PVOID RegisterBase,
							  TQP_PVOID MemoryBase,
							  TQP_UINT32 PageSize,
							  TQP_UINT32 InterruptNumber,
							  TQP_UINT32 DefaultFWMemMode
							  );
void CDevice_Destructor(CDevice *This);
EQP_Errors CDevice_Create(CDevice *This);
EQP_Errors CDevice_Close(CDevice *This);
EQP_Errors CDevice_PowerUp(CDevice *This);
EQP_Errors CDevice_PowerDown(CDevice *This);


// access functions
//
static inline ICodecLib * CDevice_getCodecLib(CDevice *This) {return (This->m_pCodecLib);}
static inline IMpegCodec * CDevice_getMpegCodec(CDevice *This) {return (This->m_pMpegCodec);}
static inline IVideoDecoder * CDevice_getVideoDecoder(CDevice *This) {return (This->m_pVidDecoder);}
static inline IVideoEncoder * CDevice_getVideoEncoder(CDevice *This) {return (This->m_pVidEncoder);}
static inline ITuner * CDevice_getTuner(CDevice *This) {return (This->m_pTuner);}
static inline ITVAudio * CDevice_getTVAudio(CDevice *This) {return (This->m_pTVAudio);}
static inline IAudioCodec * CDevice_getAudioCodec(CDevice *This) {return (This->m_pAudCodec);}


#define PLV4LDEV_MAX_DEVICES	24
extern CDevice *g_pPlV4lDevices[PLV4LDEV_MAX_DEVICES];
extern int g_PlV4l_Default_FW_Mem_Mode;


/////////////////////////////////////////////////////////////////////////////
//
//

// low level hardware access functions
//
int CDevice_rdmem(CDevice *This,
				   unsigned int ulAddr,
				   unsigned int ulSize,
				   unsigned char *pucaBuf,
				   int isusrbuf
				   );
int CDevice_wrmem(CDevice *This,
				   unsigned int ulAddr,
				   unsigned int ulSize,
				   unsigned char *pucaBuf,
				   int isusrbuf
				   );
int CDevice_dmard(CDevice *This,
				   unsigned int ulAddr,
				   unsigned int ulSize,
				   unsigned char *pucaBuf,
				   int isusrbuf
				   );
int CDevice_dmawr(CDevice *This,
				   unsigned int ulAddr,
				   unsigned int ulSize,
				   unsigned char *pucaBuf,
				   int isusrbuf
				   );
int CDevice_rdreg(CDevice *This,
				   unsigned int ulAddr,
				   unsigned char *pucaBuf,
				   int isusrbuf
				   );
int CDevice_wrreg(CDevice *This,
				   unsigned int ulAddr,
				   unsigned char *pucaBuf,
				   int isusrbuf
				   );
int CDevice_rdhci(CDevice *This,
				   unsigned int ulAddr,
				   unsigned char *pucaBuf,
				   int isusrbuf
				   );
int CDevice_wrhci(CDevice *This,
				   unsigned int ulAddr,
				   unsigned char *pucaBuf,
				   int isusrbuf
				   );
int CDevice_rdi2c(CDevice *This,
				   unsigned char ucDevAddr,
				   unsigned int ulRegSize,
				   unsigned char *pucRegAddr,
				   unsigned int ulSize,
				   unsigned char *pucaBuf,
				   int isusrbuf
				   );
int CDevice_wri2c(CDevice *This,
				   unsigned char ucDevAddr,
				   unsigned int ulSize,
				   unsigned char *pucaBuf,
				   int isusrbuf
				   );

#endif //__CDEVICE_H__



