///////////////////////////////////////////////////////////////////////////////
//     
//
// Cport.c
//
// Description:
//
//  base port implementation
//
// History:
//	Jan 05, 2010	(dt) created
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////

#include "qlautoconf.h"

#ifndef CONFIG_HOST_PLATFORM_USERSPACE
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/delay.h>
#include <linux/version.h>
#include <linux/vmalloc.h>
#include <linux/slab.h>

#if defined(CONFIG_HOST_PLATFORM_ASPEN) || defined(CONFIG_HOST_PLATFORM_PL330SOC) || defined(CONFIG_HOST_PLATFORM_SMP8644_REF) || defined(CONFIG_HOST_PLATFORM_SMP8654_REF)
#include <asm/cacheflush.h>
#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
#include <mach/pl330xp.h>
#endif
#endif
#else // CONFIG_HOST_PLATFORM_USERSPACE
#include <string.h>
#include <stdlib.h>
#include "userwrap.h"
#endif // CONFIG_HOST_PLATFORM_USERSPACE

#include "plprofiling.h"

#include "qpdbgmgr.h"
#include "cdevice.h"
#include "cport.h"
#include "ccomponent.h"

extern TQP_DbgMCompID	g_PLV4LDevDBGCompID;


/////////////////////////////////////////////////////////////////////////////
//
//


static QUEUE_ENTRY* CQueue_GetEntryByV4L2Buf(CQueue *This,
											 struct v4l2_buffer *pV4L2Buf
											 )
{
    QUEUE_ENTRY		*pthis_queue = This->m_Queue.pHead;
    QUEUE_ENTRY		*pprev_queue = QP_NULL;
	PPORT_DATA_REQ	pDataReq;

	ENTER_CRITICAL(&This->m_Object)

    while (pthis_queue)
    {
		pDataReq = (PPORT_DATA_REQ)pthis_queue->Data;

	    if (pDataReq->pV4L2Buf->m.offset == pV4L2Buf->m.offset)
		{
		    if (!pprev_queue)
		    {
			    This->m_Queue.pHead = pthis_queue->pNext;
			    if (!This->m_Queue.pHead)
				    This->m_Queue.pTail = This->m_Queue.pHead;
		    }
		    else
		    {
			    pprev_queue->pNext = pthis_queue->pNext;
			    if (!pthis_queue->pNext)
				    This->m_Queue.pTail = pprev_queue;
		    }
		    This->m_dwNbInQueue--;
		    break;
	    }
	    pprev_queue = pthis_queue;
	    pthis_queue = pthis_queue->pNext;
    }

	LEAVE_CRITICAL(&This->m_Object)

    return (pthis_queue);
}



static QUEUE_ENTRY* CQueue_PeekEntryByV4L2Buf(CQueue *This,
											  struct v4l2_buffer *pV4L2Buf
											  )
{
    QUEUE_ENTRY		*pthis_queue = This->m_Queue.pHead;
	PPORT_DATA_REQ	pDataReq;

	ENTER_CRITICAL(&This->m_Object)

    while (pthis_queue)
    {
		pDataReq = (PPORT_DATA_REQ)pthis_queue->Data;

	    if (pDataReq->pV4L2Buf->m.offset == pV4L2Buf->m.offset)
		    break;

	    pthis_queue = pthis_queue->pNext;
    }

	LEAVE_CRITICAL(&This->m_Object)

    return (pthis_queue);
}



static QUEUE_ENTRY* CQueue_GetEntryByBufDesc(CQueue *This,
											 PQP_BUFFER_DESCRIPTOR pBufDesc
											 )
{
    QUEUE_ENTRY		*pthis_queue = This->m_Queue.pHead;
    QUEUE_ENTRY		*pprev_queue = QP_NULL;
	PPORT_DATA_REQ	pDataReq;

	ENTER_CRITICAL(&This->m_Object)

    while (pthis_queue)
    {
		pDataReq = (PPORT_DATA_REQ)pthis_queue->Data;

	    if (pDataReq->pBufDesc == pBufDesc)
		{
		    if (!pprev_queue)
		    {
			    This->m_Queue.pHead = pthis_queue->pNext;
			    if (!This->m_Queue.pHead)
				    This->m_Queue.pTail = This->m_Queue.pHead;
		    }
		    else
		    {
			    pprev_queue->pNext = pthis_queue->pNext;
			    if (!pthis_queue->pNext)
				    This->m_Queue.pTail = pprev_queue;
		    }
		    This->m_dwNbInQueue--;
		    break;
	    }
	    pprev_queue = pthis_queue;
	    pthis_queue = pthis_queue->pNext;
    }

	LEAVE_CRITICAL(&This->m_Object)

    return (pthis_queue);
}



static QUEUE_ENTRY* CQueue_PeekEntryByBufDesc(CQueue *This,
											  PQP_BUFFER_DESCRIPTOR pBufDesc
											  )
{
    QUEUE_ENTRY		*pthis_queue = This->m_Queue.pHead;
	PPORT_DATA_REQ	pDataReq;

	ENTER_CRITICAL(&This->m_Object)

    while (pthis_queue)
    {
		pDataReq = (PPORT_DATA_REQ)pthis_queue->Data;

	    if (pDataReq->pBufDesc == pBufDesc)
		    break;

	    pthis_queue = pthis_queue->pNext;
    }

	LEAVE_CRITICAL(&This->m_Object)

    return (pthis_queue);
}


static void CPort_TimeStamp(CPort *This,
							PQP_BUFFER_DESCRIPTOR pBufDesc,
							struct v4l2_buffer *pV4L2Buf
							)
{
	if (PORT_DIR_READ == This->m_dir)
	{
		if (pBufDesc->ulFlags & QP_BUFDESC_FLAG_PTS)
		{
			// construct time stamp
			//
			pV4L2Buf->flags = ((pBufDesc->ulPTS) >> 2) & 0x7FFFFFFF;
			pV4L2Buf->flags |= 0x80000000;
		}
		else
		{
			pV4L2Buf->flags = 0;
		}
	}

    if (This->m_discontinuity)
	{
		This->m_discontinuity = QP_FALSE;
	}
}



static void CPort_FillFrameInfo(CPort *This,
								struct v4l2_buffer *pV4L2Buf,
								TQP_UINT32 DataUsed,
								TQP_UINT32 OptionsFlags
								)
{
	pV4L2Buf->bytesused = DataUsed;
	pV4L2Buf->reserved = OptionsFlags;
	This->m_picture_num++;
}



static void CPort_BuildBufferDescriptor(CPort *This,
										PQP_BUFFER_DESCRIPTOR pBufDesc,
										struct v4l2_buffer *pV4L2Buf
										)
{
	if (pV4L2Buf->flags & QP_BUFDESC_FLAG_PTS)
	{
		pBufDesc->ulFlags |= QP_BUFDESC_FLAG_PTS;
		pBufDesc->ulPTS = ((0x7FFFFFFF & pV4L2Buf->reserved)) << 2;
	}
	else
	{
		pBufDesc->ulFlags = 0;
		pBufDesc->ulPTS = 0;
	}
	pBufDesc->DataBufferArray->Duration = 0;
	pBufDesc->DataBufferArray->FrameExtent = pV4L2Buf->length;
	pBufDesc->DataBufferArray->DataUsed = pV4L2Buf->bytesused;
	if (V4L2_MEMORY_MMAP == This->m_memoryType)
	{
		pBufDesc->DataBufferArray->Data = This->m_pBufMem + pV4L2Buf->m.offset;
	}
	else
	{
		pBufDesc->DataBufferArray->Data = (TQP_PVOID)pV4L2Buf->m.userptr;
	}
	if (V4L2_MEMORY_MMAP == pV4L2Buf->memory)
	{
		pBufDesc->ulFlags |= QP_BUFDESC_FLAG_BUFFER_DRIVER;
	}
	else
	{
		pBufDesc->ulFlags |= QP_BUFDESC_FLAG_BUFFER_USER;
	}
	pBufDesc->DataBufferArray->OptionsFlags = 0;
	pBufDesc->NumberOfBuffers = 1;
	pBufDesc->ulBufferIndex = 0;
	pBufDesc->ulBufferOffset = 0;
	pBufDesc->ulBufferSize = pV4L2Buf->length;
	pBufDesc->ulDMABufferIndex = 0;
	pBufDesc->ulDMABufferOffset = 0;
	pBufDesc->ulTotalUsed = 0;
	if (This->m_bBufferPartialFill)
	{
		pBufDesc->ulFlags |= QP_BUFDESC_FLAG_PARTIAL_FILL_OK;
	}
	if (This->m_bBufferFrameAligned)
	{
		pBufDesc->ulFlags |= QP_BUFDESC_FLAG_FRAME_ALIGNED;
	}
	pBufDesc->Status = QPERR_PENDING;
	pBufDesc->pBuffer = QP_NULL;
	pBufDesc->unAlignedNumberBytes = 0;
}



static void CPort_BuildScatterGatherList(CPort *This,
										 PQP_BUFFER_DESCRIPTOR pBufDesc,
										 struct v4l2_buffer *pV4L2Buf
										 )
{
	pBufDesc->NumberOfScatterGatherElements = 0;
}



static void CPort_CleanUpDataRequest(CPort *This,
									 PPORT_DATA_REQ pDataReq
									 )
{
	PQP_BUFFER_DESCRIPTOR	pBufDesc = pDataReq->pBufDesc;

	// delete temporary buffer if any
	if (pBufDesc->pBuffer)
	{
		PLvfree(pBufDesc->pBuffer);
		pBufDesc->pBuffer = QP_NULL;
	}
	if (pBufDesc->ScatterGatherBuffer)
	{
		vfree(pBufDesc->ScatterGatherBuffer);
		pBufDesc->ScatterGatherBuffer = QP_NULL;
	}
}



static void CPort_SubmitBuffers(CPort *This)
{
	CQueue		*pQueue;
	QUEUE_ENTRY	*pEntry;
	IMpegCodec	*pMpegCodec;

    // Submit as many buffers as we can to the library
	//

	// Only give the buffers to the firmware if we are not stopped
	//
	if ((V4L2_MEMORY_MMAP != This->m_memoryType) ||
		(TQP_NULL_HANDLE == This->m_hStreamLib) ||
		(PORT_STATE_STOP == This->m_State) ||
		(PORT_DIR_NONE == This->m_dir)
		)
	{
		return;
	}

	// get the codec chip interface
	//
	pMpegCodec = CDevice_getMpegCodec(This->m_pComponent->m_pDevice);

	// find the right queue based on the port direction
	//
	pQueue = (PORT_DIR_READ == This->m_dir) ? This->m_pQueueFree : This->m_pQueueData;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, /*QPDBGM_LEVEL_ERROR*/QPDBGM_LEVEL_EXTRACE),
				 "%s(%d) - hStreamLib(%d) dir(%d) pQueue(0x%x) free(0x%x) data(0x%x)\n",
				 __FUNCTION__,
				 This->m_id,
				 This->m_hStreamLib,
				 This->m_dir,
				 pQueue,
				 This->m_pQueueFree,
				 This->m_pQueueData
				 ));

	// get a free queue entry
	//
	pEntry = CQueue_GetOneEntry(pQueue);

	// save it to the data request queue
	//
	while (pEntry &&
		   !This->m_EOS
		   )
	{
		PPORT_DATA_REQ	pDataReq = (PPORT_DATA_REQ)pEntry->Data;
		EQP_Errors		qpStatus;

		CPort_BuildBufferDescriptor(This,
									pDataReq->pBufDesc,
									pDataReq->pV4L2Buf
									);
		CPort_BuildScatterGatherList(This,
									 pDataReq->pBufDesc,
									 pDataReq->pV4L2Buf
									 );

		// save this request to the lib queue
		//
		CQueue_AddEntry(This->m_pQueueLib,
						pEntry
						);

		// add the buffer to the codec library
		//
		qpStatus = pMpegCodec->AddBuffer(pMpegCodec,
										 This->m_hStreamLib,
										 pDataReq->pBufDesc
										 );
		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
						 "%s(%d) - hStreamLib(%d) AddBuffer id(%d) (0x%X) FAILED!!!!\n",
						 __FUNCTION__,
						 This->m_id,
						 This->m_hStreamLib,
						 pDataReq->nId,
						 pDataReq->pBufDesc
						 ));

			if (pDataReq->pBufDesc->ScatterGatherBuffer)
			{
				vfree(pDataReq->pBufDesc->ScatterGatherBuffer);
				pDataReq->pBufDesc->ScatterGatherBuffer = QP_NULL;
			}
			CQueue_RemoveEntry(This->m_pQueueLib,
							   pEntry
							   );
			CQueue_AddEntry(This->m_pQueueFree,
							pEntry
							);
			break;
		}
		else
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, /*QPDBGM_LEVEL_TRACE*/QPDBGM_LEVEL_EXTRACE),
						 "%s(%d) - hStreamLib(%d) AddBuffer id(%d) (0x%X)\n",
						 __FUNCTION__,
						 This->m_id,
						 This->m_hStreamLib,
						 pDataReq->nId,
						 pDataReq->pBufDesc
						 ));
		}

		pEntry = CQueue_GetOneEntry(pQueue);
	}
}



static void CPort_ProcessCompletedBuffer(CPort *This,
										 struct v4l2_buffer *pV4L2Buf
										 )
{
}



static EQP_Errors CPort_OnBufferComplete(CPort *This,
										 QUEUE_ENTRY *pEntry
										 )
{
	PPORT_DATA_REQ			pDataReq = (PPORT_DATA_REQ)pEntry->Data;
	PQP_BUFFER_DESCRIPTOR	pBufDesc = pDataReq->pBufDesc;
	struct v4l2_buffer		*pV4L2Buf = pDataReq->pV4L2Buf;
	CQueue					*pQueue;

    CPort_TimeStamp(This,
					pBufDesc,
					pV4L2Buf
					);
	CPort_FillFrameInfo(This,
						pV4L2Buf,
						pBufDesc->DataBufferArray->DataUsed,
						pBufDesc->DataBufferArray->OptionsFlags
						);
	CPort_ProcessCompletedBuffer(This,
								 pV4L2Buf
								 );
	CPort_CleanUpDataRequest(This,
							 pDataReq
							 );

	// find the right queue based on the port direction
	if (V4L2_MEMORY_MMAP == This->m_memoryType)
	{
		pQueue = (PORT_DIR_READ == This->m_dir) ? This->m_pQueueData : This->m_pQueueFree;
	}
	else
	{
		pQueue = This->m_pQueueData;
	}

	// move entry to the queue
	CQueue_AddEntry(pQueue,
					pEntry
					);

	// push data through
	CPort_SubmitBuffers(This);

#ifndef CONFIG_HOST_PLATFORM_USERSPACE
	wake_up_interruptible(&(This->m_pComponent->wait_frame));
#endif // CONFIG_HOST_PLATFORM_USERSPACE

	return (QPERR_SUCCESS);
}



static EQP_Errors CPort_StreamCallback(TQP_PVOID Context,
									   TQP_UINT32 dwCode,
									   TQP_PVOID pParam
									   )
{
	CPort		*pPort = (CPort *)Context;
	EQP_Errors	qpStatus = QPERR_SUCCESS;

	switch (dwCode)
	{
		case QPMPGCODEC_CMD_BUFFER_OVERFLOW:
			break;

		case QPMPGCODEC_CMD_STOP_COMPLETED:
			break;

		case QPMPGCODEC_CMD_DONE_DATA:
		{
			PQP_BUFFER_DESCRIPTOR	pBufDesc = (PQP_BUFFER_DESCRIPTOR)pParam;
			QUEUE_ENTRY				*pEntry;

			if (QP_NULL != (pEntry = CQueue_GetEntryByBufDesc(pPort->m_pQueueLib, pBufDesc)))
			{
				PPORT_DATA_REQ	pDataReq = (PPORT_DATA_REQ)pEntry->Data;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
							 "%s(%d) : MPGCODEC_CMD_DONE_DATA id(%d) pBufDesc(0x%X) status(%d)\n",
							 __FUNCTION__,
							 pPort->m_hStreamLib,
							 pDataReq->nId,
							 pBufDesc,
							 pBufDesc->Status
							 ));

				qpStatus = CPort_OnBufferComplete(pPort,
												  pEntry
												  );
			}
			break;
		}

		case QPMPGCODEC_CMD_END_OF_STREAM:
			break;

		case QPMPGCODEC_CMD_GET_STRM_TIME:
			break;

		case QPMPGCODEC_CMD_START_TIMEOUT:
			break;

		case QPMPGCODEC_CMD_DEC_COMPLETE:
			QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_WARN),
						 "%s QPMPGCODEC_CMD_DEC_COMPLETE\n",
						 __FUNCTION__
						 ));
			break;

		default:
			qpStatus = QPERR_NOTIMPL;
			break;
	}

	return (qpStatus);
}



static EQP_Errors CPort_CodecOpen(CPort *This,
								  IMpegCodec *pMpegCodec
								  )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;

	if (!This->m_bDisabled &&
		(TQP_NULL_HANDLE == This->m_hStreamLib)
		)
	{
		qpStatus = pMpegCodec->Open(pMpegCodec,
									This->m_pComponent->m_hTask,
									This->m_dwOpenType,
									&This->m_openFormat,
									&This->m_hStreamLib,
									CPort_StreamCallback,
									(TQP_PVOID)This
									);

		if (!QP_SUCCESS(qpStatus))
		{
			This->m_hStreamLib = TQP_NULL_HANDLE;

			QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_WARN),
						 "%s pMpegCodec->Open failed(%d)\n",
						 __FUNCTION__,
						 qpStatus
						 ));
		}
		else
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
						 "%s pMpegCodec->Open task(%d) codec handle(%d)\n",
						 __FUNCTION__,
						 This->m_pComponent->m_hTask,
						 This->m_hStreamLib
						 ));
		}
	}

	return (qpStatus);
}



static void CPort_ResetCounters(CPort *This)
{
	This->m_picture_num = 0;
	This->m_picture_in_num = 0;
	This->m_discontinuity = QP_FALSE;
	This->m_EOS = QP_FALSE;
}



/////////////////////////////////////////////////////////////////////////////
//
//


// constructor
//
CPort * CPort_Constructor(CPort *pPort,
						  CComponent *pComponent,
						  TQP_UINT32 dwAttributes,
						  COMPONENT_PORT_TYPE id,
						  PORT_DIRECTION dir,
						  TQP_UINT32 dwOpenType,
						  PCOMPONENT_PORT_OPEN_FORMAT pOpenFormat,
						  int memoryType,
						  TQP_UINT32 frame_nbs,
						  TQP_UINT32 frame_size,
						  TQP_BOOL bBufferPartialFill,
						  TQP_BOOL bBufferFrameAligned,
						  TQP_BOOL bSupportUserPtr
						  )
{
	if (pPort)
	{
		// CObject
		//
		CObject_Constructor(&pPort->m_Object,
							&pComponent->m_Object,
		  					dwAttributes
		  					);

		// initialize members
		//
		pPort->m_valid = QP_TRUE;
		pPort->m_pComponent = pComponent;
		pPort->m_id = id;
		pPort->m_hStreamLib = TQP_NULL_HANDLE;
		pPort->m_dir = dir;
		pPort->m_dwOpenType = dwOpenType;
		if (pOpenFormat)
		{
			memcpy(&pPort->m_openFormat,
				   pOpenFormat,
				   sizeof(COMPONENT_PORT_OPEN_FORMAT)
				   );
		}
		else
		{
			memset(&pPort->m_openFormat,
				   0,
				   sizeof(COMPONENT_PORT_OPEN_FORMAT)
				   );
		}
		QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
					 "%s() id(type)=%d frame_nbs=%d frame_size=%d\n",
					 __FUNCTION__, id, frame_nbs, frame_size
					 ));

		pPort->m_memoryType = memoryType;
		pPort->m_frame_nbs = frame_nbs;
		pPort->m_frame_size = frame_size;
		pPort->m_bBufferPartialFill = bBufferPartialFill;
		pPort->m_bBufferFrameAligned = bBufferFrameAligned;
		pPort->m_bSupportUserPtr = bSupportUserPtr;
		pPort->m_State = PORT_STATE_STOP;
		pPort->m_EOS = QP_FALSE;
		pPort->m_picture_num = 0;
		pPort->m_picture_in_num = 0;
		pPort->m_discontinuity = QP_FALSE;
		pPort->m_bDisabled = QP_FALSE;
		pPort->m_pBufMem = QP_NULL;
		pPort->m_pQueueFree = QP_NULL;
		pPort->m_pQueueData = QP_NULL;
		pPort->m_pQueueLib = QP_NULL;
		pPort->m_pQueueUser = QP_NULL;
	}

	return (pPort);
}



// destructor
//
void CPort_Destructor(CPort *This)
{
	if (This->m_pQueueUser)
	{
		CQueue_Destructor(This->m_pQueueUser);
		vfree((void *)This->m_pQueueUser);
		This->m_pQueueUser = QP_NULL;
	}

	if (This->m_pQueueLib)
	{
		CQueue_Destructor(This->m_pQueueLib);
		vfree((void *)This->m_pQueueLib);
		This->m_pQueueLib = QP_NULL;
	}

	if (This->m_pQueueData)
	{
		CQueue_Destructor(This->m_pQueueData);
		vfree((void *)This->m_pQueueData);
		This->m_pQueueData = QP_NULL;
	}

	if (This->m_pQueueFree)
	{
		CQueue_Destructor(This->m_pQueueFree);
		vfree((void *)This->m_pQueueFree);
		This->m_pQueueFree = QP_NULL;
	}

	if (This->m_pBufMem)
	{
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,15)
		unsigned long	ulAddr = (unsigned long)This->m_pBufMem;
		size_t			ulSize = This->m_frame_nbs * This->m_frame_size;

		while (ulSize > 0)
		{
			ClearPageReserved(vmalloc_to_page((void *)ulAddr));
			ulAddr += PAGE_SIZE;
			ulSize -= PAGE_SIZE;
		}
#endif //LINUX_VERSION_CODE < KERNEL_VERSION(2,6,15)

#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
		kfree((void *)This->m_pBufMem);
#else
		vfree((void *)This->m_pBufMem);
#endif
		This->m_pBufMem = QP_NULL;
	}

	This->m_valid = QP_FALSE;

	CObject_Destructor(&This->m_Object);
}



EQP_Errors CPort_Create(CPort *This)
{
	CQueue		*pQueue;
	EQP_Errors	qpStatus;
	IMpegCodec	*pMpegCodec = CDevice_getMpegCodec(This->m_pComponent->m_pDevice);
	int			i;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
				 "%s\n",
				 __FUNCTION__
				 ));

	if (PORT_DIR_NONE != This->m_dir)
	{
		// allocate buffer
		//
		if (V4L2_MEMORY_MMAP == This->m_memoryType)
		{
#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
			This->m_pBufMem = kmalloc(This->m_frame_nbs * This->m_frame_size, GFP_KERNEL | GFP_ATOMIC);
#else
			This->m_pBufMem = (PTQP_UINT8) vmalloc_32(This->m_frame_nbs * This->m_frame_size);
#endif
			if (!This->m_pBufMem)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
							 "%s Unable to allocate buffer(%d x %d)!\n",
							 __FUNCTION__,
							 This->m_frame_nbs,
							 This->m_frame_size
							 ));
				return (QPERR_NOMEM);
			}
		}
		else
		{
			if (!This->m_bSupportUserPtr)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
							 "%s port(%d) does not support user pointer!\n",
							 __FUNCTION__,
							 This->m_id
							 ));
				return (QPERR_NOTSUPP);
			}
		}

		// allocate data queues
		//

		// free queue
		pQueue = (CQueue *)vmalloc(sizeof(CQueue));
		if (!pQueue)
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
						 "%s Unable to create CQueue!\n",
						 __FUNCTION__
						 ));
			return (QPERR_NOMEM);
		}

		This->m_pQueueFree = CQueue_Constructor(pQueue,
												&This->m_Object,
												OBJECT_CRITICAL_HEAVY
												);
		if (!This->m_pQueueFree)
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
						 "%s Unable to create m_pQueueFree!\n",
						 __FUNCTION__
						 ));
			vfree((void *)pQueue);
			return (QPERR_FAIL);
		}

		// data queue
		pQueue = (CQueue *)vmalloc(sizeof(CQueue));
		if (!pQueue)
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
						 "%s Unable to create CQueue!\n",
						 __FUNCTION__
						 ));
			return (QPERR_NOMEM);
		}

		This->m_pQueueData = CQueue_Constructor(pQueue,
												&This->m_Object,
												OBJECT_CRITICAL_HEAVY
												);
		if (!This->m_pQueueData)
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
						 "%s Unable to create m_pQueueData!\n",
						 __FUNCTION__
						 ));
			vfree((void *)pQueue);
			return (QPERR_FAIL);
		}

		// lib queue
		pQueue = (CQueue *)vmalloc(sizeof(CQueue));
		if (!pQueue)
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
						 "%s Unable to create CQueue!\n",
						 __FUNCTION__
						 ));
			return (QPERR_NOMEM);
		}

		This->m_pQueueLib = CQueue_Constructor(pQueue,
											   &This->m_Object,
											   OBJECT_CRITICAL_HEAVY
											   );
		if (!This->m_pQueueLib)
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
						 "%s Unable to create m_pQueueLib!\n",
						 __FUNCTION__
						 ));
			vfree((void *)pQueue);
			return (QPERR_FAIL);
		}

		// user queue
		pQueue = (CQueue *)vmalloc(sizeof(CQueue));
		if (!pQueue)
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
						 "%s Unable to create CQueue!\n",
						 __FUNCTION__
						 ));
			return (QPERR_NOMEM);
		}

		This->m_pQueueUser = CQueue_Constructor(pQueue,
												&This->m_Object,
												OBJECT_CRITICAL_HEAVY
												);
		if (!This->m_pQueueUser)
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
						 "%s Unable to create m_pQueueUser!\n",
						 __FUNCTION__
						 ));
			vfree((void *)pQueue);
			return (QPERR_FAIL);
		}

		// initialize free queue
		//
		for (i = 0; i < This->m_frame_nbs; i++)
		{
			memset(&This->m_BufDesces[i],
				   0,
				   sizeof(QP_BUFFER_DESCRIPTOR)
				   );
			This->m_BufDesces[i].DataBufferArray = &This->m_DataBufferArray[i];

			This->m_v4l2Buffers[i].index = i;
			This->m_v4l2Buffers[i].type = V4L2_BUF_TYPE_VIDEO_CAPTURE | V4L2_BUF_TYPE_VIDEO_OUTPUT;
			This->m_v4l2Buffers[i].bytesused = 0;
			This->m_v4l2Buffers[i].flags = 0;
			This->m_v4l2Buffers[i].field = V4L2_FIELD_INTERLACED;
			This->m_v4l2Buffers[i].sequence = This->m_id;
			This->m_v4l2Buffers[i].memory = This->m_memoryType;
			if (V4L2_MEMORY_MMAP == This->m_memoryType)
			{
				This->m_v4l2Buffers[i].m.offset = i * This->m_frame_size;
				This->m_v4l2Buffers[i].length = This->m_frame_size;
			}
			else
			{
				This->m_v4l2Buffers[i].m.userptr = 0;
				This->m_v4l2Buffers[i].length = 0;
			}

			This->m_PortReqs[i].pBufDesc = &This->m_BufDesces[i];
			This->m_PortReqs[i].pV4L2Buf = &This->m_v4l2Buffers[i];
			This->m_PortReqs[i].nId = i;

			This->m_Entries[i].pNext = QP_NULL;
			This->m_Entries[i].Data = &This->m_PortReqs[i];
			CQueue_AddEntry(This->m_pQueueFree,
							&This->m_Entries[i]
							);
		}
	}

	This->m_bDisabled = QP_FALSE;

	qpStatus = CPort_CodecOpen(This,
							   pMpegCodec
							   );

	return (qpStatus);
}



EQP_Errors CPort_Close(CPort *This)
{
	int	i;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
				 "%s\n",
				 __FUNCTION__
				 ));

	// make sure we are stopped
	//
	if (PORT_STATE_STOP != This->m_State)
	{
		CPort_SetState(This,
					   PORT_STATE_STOP,
					   This->m_State
					   );
	}

	// close codec lib handle
	//
	if (TQP_NULL_HANDLE != This->m_hStreamLib)
	{
		IMpegCodec	*pMpegCodec = CDevice_getMpegCodec(This->m_pComponent->m_pDevice);
		pMpegCodec->Close(pMpegCodec,
						  This->m_hStreamLib
						  );
		This->m_hStreamLib = TQP_NULL_HANDLE;
	}

	// flush queues
	//
	for (i = 0; i < This->m_frame_nbs; i++)
	{
		if (This->m_BufDesces[i].ScatterGatherBuffer)
		{
			vfree(This->m_BufDesces[i].ScatterGatherBuffer);
			This->m_BufDesces[i].ScatterGatherBuffer = QP_NULL;
		}
	}

	if (This->m_pQueueUser)
	{
		CQueue_Destructor(This->m_pQueueUser);
		vfree((void *)This->m_pQueueUser);
		This->m_pQueueUser = QP_NULL;
	}

	if (This->m_pQueueLib)
	{
		CQueue_Destructor(This->m_pQueueLib);
		vfree((void *)This->m_pQueueLib);
		This->m_pQueueLib = QP_NULL;
	}

	if (This->m_pQueueData)
	{
		CQueue_Destructor(This->m_pQueueData);
		vfree((void *)This->m_pQueueData);
		This->m_pQueueData = QP_NULL;
	}

	if (This->m_pQueueFree)
	{
		CQueue_Destructor(This->m_pQueueFree);
		vfree((void *)This->m_pQueueFree);
		This->m_pQueueFree = QP_NULL;
	}

	if (This->m_pBufMem)
	{
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,15)
		unsigned long	ulAddr = (unsigned long)This->m_pBufMem;
		size_t			ulSize = This->m_frame_nbs * This->m_frame_size;

		while (ulSize > 0)
		{
			ClearPageReserved(vmalloc_to_page((void *)ulAddr));
			ulAddr += PAGE_SIZE;
			ulSize -= PAGE_SIZE;
		}
#endif //LINUX_VERSION_CODE < KERNEL_VERSION(2,6,15)

#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
		kfree((void *)This->m_pBufMem);
#else
		vfree((void *)This->m_pBufMem);
#endif
		This->m_pBufMem = QP_NULL;
	}

	This->m_bDisabled = QP_FALSE;

	return (QPERR_SUCCESS);
}



EQP_Errors CPort_Enable(CPort *This)
{
	ENTER_CRITICAL(&This->m_Object)

	if (This->m_bDisabled)
	{
		This->m_bDisabled = QP_FALSE;
	}

	LEAVE_CRITICAL(&This->m_Object)

	return (QPERR_SUCCESS);
}



EQP_Errors CPort_Disable(CPort *This)
{
	ENTER_CRITICAL(&This->m_Object)

	This->m_hStreamLib = TQP_NULL_HANDLE;
	This->m_bDisabled = QP_TRUE;

	LEAVE_CRITICAL(&This->m_Object)

	return (QPERR_SUCCESS);
}



EQP_Errors CPort_SetState(CPort *This,
						  PORT_STATE to_state,
						  PORT_STATE from_state
						  )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	IMpegCodec	*pMpegCodec = CDevice_getMpegCodec(This->m_pComponent->m_pDevice);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_WARN),
				 "%s port_id(%d) hStreamLib(%d) to_state(%d) [\n",
				 __FUNCTION__,
				 This->m_id,
				 This->m_hStreamLib,
				 to_state
				 ));

	if (This->m_State == to_state)
	{
		return (QPERR_SUCCESS);
	}

	switch (to_state)
	{
		case PORT_STATE_STOP:

			QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
						 "%s PORT_STATE_STOP codec handle(%d)\n",
						 __FUNCTION__,
						 This->m_hStreamLib
						 ));

			if (TQP_NULL_HANDLE != This->m_hStreamLib)
			{
				QUEUE_ENTRY	*pEntry;

				qpStatus = pMpegCodec->Stop(pMpegCodec,
											This->m_hStreamLib
											);

				// flush all the queues
				//
				if (This->m_pQueueLib)
				{
					while ((pEntry = CQueue_GetOneEntry(This->m_pQueueLib)))
					{
						CPort_CleanUpDataRequest(This,
												 (PPORT_DATA_REQ)pEntry->Data
												 );
						CQueue_AddEntry(This->m_pQueueFree,
										pEntry
										);
					}
				}

				if (This->m_pQueueData)
				{
					while ((pEntry = CQueue_GetOneEntry(This->m_pQueueData)))
					{
						CPort_CleanUpDataRequest(This,
												 (PPORT_DATA_REQ)pEntry->Data
												 );
						CQueue_AddEntry(This->m_pQueueFree,
										pEntry
										);
					}
				}

				if (This->m_pQueueUser)
				{
					while ((pEntry = CQueue_GetOneEntry(This->m_pQueueUser)))
					{
						CPort_CleanUpDataRequest(This,
												 (PPORT_DATA_REQ)pEntry->Data
												 );
						CQueue_AddEntry(This->m_pQueueFree,
										pEntry
										);
					}
				}
			}

			if (!QP_SUCCESS(qpStatus))
			{
				qpStatus = QPERR_SUCCESS;
			}

			break;

		case PORT_STATE_ACQUIRE:

			if (PORT_STATE_STOP == from_state)
			{
				qpStatus = CPort_CodecOpen(This,
										   pMpegCodec
										   );

				CPort_ResetCounters(This);
			}

			break;

		case PORT_STATE_PAUSE:

			if (!This->m_bDisabled)
			{
				qpStatus = CPort_CodecOpen(This,
										   pMpegCodec
										   );

				if (TQP_NULL_HANDLE != This->m_hStreamLib)
				{
					if (PORT_STATE_RUN == from_state)
					{
						qpStatus = pMpegCodec->Stop(pMpegCodec,
													This->m_hStreamLib
													);
					}
					else
					{
						qpStatus = pMpegCodec->Acquire(pMpegCodec,
													   This->m_hStreamLib
													   );
					}
				}
			}
			else
			{
				qpStatus = QPERR_INVALID;
			}

			QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
						 "%s PORT_STATE_PAUSE handle(%d) qp_status(%d)\n",
						 __FUNCTION__,
						 This->m_hStreamLib,
						 qpStatus
						 ));
			if (!QP_SUCCESS(qpStatus) &&
				(PORT_STATE_RUN == from_state)
				)
			{
				qpStatus = QPERR_SUCCESS;
			}

			break;

		case PORT_STATE_RUN:

			This->m_discontinuity = QP_TRUE;

			qpStatus = CPort_CodecOpen(This,
									   pMpegCodec
									   );

			if (TQP_NULL_HANDLE != This->m_hStreamLib)
			{
				qpStatus = pMpegCodec->Start(pMpegCodec,
											 This->m_hStreamLib
											 );
			}
			else
			{
				qpStatus = QPERR_INVALID;
			}

			QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
						 "%s PORT_STATE_RUN handle(%d) qp_status(%d)\n",
						 __FUNCTION__,
						 This->m_hStreamLib,
						 qpStatus
						 ));

			// pass buffers to the firmware
			//
			if (QP_SUCCESS(qpStatus))
			{
				CPort_SubmitBuffers(This);
			}
			break;

		default:
			qpStatus = QPERR_NOTIMPL;
			break;
	}

	if (QP_SUCCESS(qpStatus))
	{
		This->m_State = to_state;
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_WARN),
				 "%s task(%d) handle(%d) from_state(%d) to_state(%d) qp_status(%d)]\n",
				 __FUNCTION__,
				 This->m_pComponent->m_hTask,
				 This->m_hStreamLib,
				 from_state,
				 to_state,
				 qpStatus
				 ));

	return (qpStatus);
}



EQP_Errors CPort_ReqBuf(CPort *This,
						struct v4l2_requestbuffers *reqbuf
						)
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;

	if (V4L2_MEMORY_MMAP == reqbuf->memory)
	{
        reqbuf->count = This->m_frame_nbs;

		QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
					 "%s(%d)-> count(%d)\n",
					 __FUNCTION__,
					 This->m_id,
					 reqbuf->count
					 ));
    }
	else
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_WARN),
					 "%s(%d)-> wrong memory type(%d)\n",
					 __FUNCTION__,
					 This->m_id,
					 reqbuf->memory
					 ));
		qpStatus = QPERR_INVALID;
    }

	return (qpStatus);
}



EQP_Errors CPort_QueryBuf(CPort *This,
						  struct v4l2_buffer *buf
						  )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
    int			index = buf->index;
    int			type = buf->type;

	if (V4L2_MEMORY_MMAP != This->m_memoryType)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_WARN),
					 "%s(%d)-> wrong memory type(%d)\n",
					 __FUNCTION__,
					 This->m_id,
					 This->m_memoryType
					 ));
	}

	if (index < This->m_frame_nbs)
	{
		memset(buf,
			   0,
			   sizeof(*buf)
			   );
		buf->index = index;
		// keep the port type in 4 most significant bits
		buf->m.offset = This->m_v4l2Buffers[index].m.offset | (This->m_id << PORT_MMAP_BUF_TYPE_SHIFT);
		buf->length = This->m_v4l2Buffers[index].length;
		buf->type = type;

		QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
					 "%s(%d)-> idx=%d, addr=%x, size=%x type=%d type=%d \n",
					 __FUNCTION__,
					 This->m_id,
					 buf->index,
					 buf->m.offset,
					 buf->length,
					 buf->type,
					 type
					 ));
	}
	else
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_WARN),
					 "%s(%d)-> idx=%d >= frame_nb(%d)\n",
					 __FUNCTION__,
					 This->m_id,
					 buf->index,
					 This->m_frame_nbs
					 ));
		qpStatus = QPERR_PARMS;
    }

	return (qpStatus);
}



EQP_Errors CPort_DQBuf(CPort *This,
					   struct v4l2_buffer *pv4lbufapp
					   )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	CQueue		*pQueue;
	QUEUE_ENTRY	*pEntry;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_EXTRACE),
				 "%s(%d)-> dir(%d) hTask(%d) [\n",
				 __FUNCTION__,
				 This->m_id,
				 This->m_dir,
				 This->m_pComponent->m_hTask
				 ));

	if (V4L2_MEMORY_MMAP == This->m_memoryType)
	{
		if (This->m_State != PORT_STATE_RUN)
		{
			if (This->m_dir==PORT_DIR_READ) {
				PROFILING_LOG(EVENT_COMP_D_OUT_FAILED, (TQP_UINT32) This->m_pComponent->m_hTask, This->m_id, 0, 0, 0);
			}

			return (QPERR_AGAIN);
		}

		switch (This->m_dir)
		{
			case PORT_DIR_READ:
				pQueue = This->m_pQueueData;
				break;

			case PORT_DIR_WRITE:
				pQueue = This->m_pQueueFree;
				break;

			case PORT_DIR_NONE:
			default:
				return (QPERR_NOTSUPP);
		}

#ifdef DQBUF_BLOCKING_MODE
RETRY:
#endif
		// get one entry from the queue
		if (pQueue)
            pEntry = CQueue_GetOneEntry(pQueue);
        else {
            pEntry = QP_NULL;
        }

		if (pEntry)
		{
			PPORT_DATA_REQ		pDataReq = (PPORT_DATA_REQ)pEntry->Data;
			struct v4l2_buffer	*pV4L2Buf = pDataReq->pV4L2Buf;

#if defined(CONFIG_HOST_PLATFORM_SMP8644_REF) || defined(CONFIG_HOST_PLATFORM_SMP8654_REF)
			unsigned long addr;
			unsigned long start_addr = (unsigned long)(This->m_pBufMem + pV4L2Buf->m.offset);
			unsigned long len = pV4L2Buf->bytesused;
			for (addr = start_addr; addr < (start_addr + len); addr += PAGE_SIZE) {
				flush_data_cache_page(addr);
			}
#endif

#if defined(CONFIG_HOST_PLATFORM_ASPEN) || defined(CONFIG_HOST_PLATFORM_PL330SOC)
			__cpuc_coherent_kern_range(This->m_pBufMem + pV4L2Buf->m.offset, This->m_pBufMem + pV4L2Buf->m.offset + pV4L2Buf->bytesused);
#endif

			#if 0
			{
				unsigned char *p = (PTQP_UINT8) (This->m_pBufMem + pV4L2Buf->m.offset);

				QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, /*QPDBGM_LEVEL_WARN*/QPDBGM_LEVEL_TRACE),
					 	"CPort_DQBuf() pointer %p data %02x %02x %02x %02x\n",
					 	p, p[0], p[1], p[2], p[3]
					 	));
			}
			#endif
			pv4lbufapp->reserved = pV4L2Buf->reserved;
			pv4lbufapp->bytesused = pV4L2Buf->bytesused;
			pv4lbufapp->m.offset = pV4L2Buf->m.offset;
			pv4lbufapp->index = pV4L2Buf->index;
			pv4lbufapp->length = pV4L2Buf->length;
			pv4lbufapp->flags = pV4L2Buf->flags;

#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
#if ENABLE_CODEC_MEM_LOG
			codec_mem_log_add_ex(CODEC_MEM_LOG_DQBUF, 0, pV4L2Buf->bytesused);
#endif
#endif

			if (This->m_dir==PORT_DIR_READ) {
				PROFILING_LOG(EVENT_COMP_D_OUT, (TQP_UINT32) This->m_pComponent->m_hTask, This->m_id, CQueue_GetQueueLevel(This->m_pQueueData), This->m_picture_num, pV4L2Buf->bytesused);
				if (pV4L2Buf->flags & QP_KSSTREAM_HEADER_OPTIONSF_PREROLL ) {
                    PROFILING_LOG(EVENT_COMP_D_OUT_INCOMPLETE_FRAME, (TQP_UINT32) This->m_pComponent->m_hTask, This->m_id, 0, This->m_picture_num, pV4L2Buf->bytesused);
				}
			}

			// move entry to the user queue
			CQueue_AddEntry(This->m_pQueueUser,
							pEntry
							);

		}
		else
		{
#ifndef DQBUF_BLOCKING_MODE
			pv4lbufapp->reserved = 0;
			pv4lbufapp->bytesused = 0;
			pv4lbufapp->m.offset = 0;
			pv4lbufapp->length = 0;
			qpStatus = QPERR_AGAIN;
			//if (This->m_dir==PORT_DIR_READ)
			//	PROFILING_LOG(EVENT_COMP_D_OUT_NODATA, (TQP_UINT32) This->m_pComponent->m_hTask, This->m_id, 0, 0, 0);
#else
			if (This->m_State == PORT_STATE_RUN)
			{
				DelayMilliSeconds(1);
				goto RETRY;
			}
			else
			{
				pv4lbufapp->reserved = 0;
				pv4lbufapp->bytesused = 0;
				pv4lbufapp->m.offset = 0;
				pv4lbufapp->length = 0;
				qpStatus = QPERR_AGAIN;
			}
#endif
		}
	}
	else
	{
		// get one entry from the data queue
		pEntry = CQueue_GetOneEntry(This->m_pQueueData);

		if (pEntry)
		{
			PPORT_DATA_REQ		pDataReq = (PPORT_DATA_REQ)pEntry->Data;
			struct v4l2_buffer	*pV4L2Buf = pDataReq->pV4L2Buf;

			pv4lbufapp->reserved = pV4L2Buf->reserved;
			pv4lbufapp->bytesused = pV4L2Buf->bytesused;
			pv4lbufapp->m.userptr = pV4L2Buf->m.userptr;
			pv4lbufapp->index = pV4L2Buf->index;
			pv4lbufapp->length = pV4L2Buf->length;
			pv4lbufapp->flags = pV4L2Buf->flags;

			// move entry back to the free queue
			CQueue_AddEntry(This->m_pQueueFree,
							pEntry
							);
		}
		else
		{
			pv4lbufapp->reserved = 0;
			pv4lbufapp->bytesused = 0;
			pv4lbufapp->m.userptr = 0;
			pv4lbufapp->length = 0;
			pv4lbufapp->flags = 0;
			qpStatus = QPERR_AGAIN;
		}
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_EXTRACE),
				 "%s(%d)<- status(%d) buf index(%d) addr(0x%x) len(%d) used(%d) ]\n",
				 __FUNCTION__,
				 This->m_id,
				 qpStatus,
				 pv4lbufapp->index,
				 pv4lbufapp->m.offset,
				 pv4lbufapp->length,
				 pv4lbufapp->bytesused
				 ));

	return (qpStatus);
}



EQP_Errors CPort_QBuf(CPort *This,
					  struct v4l2_buffer *pv4lbufapp
					  )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	QUEUE_ENTRY	*pEntry;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_EXTRACE),
				 "%s(%d)-> dir(%d) hTask(%d) [\n",
				 __FUNCTION__,
				 This->m_id,
				 This->m_dir,
				 This->m_pComponent->m_hTask
				 ));

	if (PORT_DIR_NONE == This->m_dir)
	{
		return (QPERR_NOTSUPP);
	}

	if (V4L2_MEMORY_MMAP == This->m_memoryType)
	{
		pEntry = CQueue_GetEntryByV4L2Buf(This->m_pQueueUser,
										  pv4lbufapp
										  );
		if (pEntry)
		{
			PPORT_DATA_REQ	pDataReq = (PPORT_DATA_REQ)pEntry->Data;

			if (PORT_DIR_READ == This->m_dir)
			{
				pDataReq->pV4L2Buf->bytesused = 0;
				pDataReq->pV4L2Buf->reserved = 0;
				pDataReq->pV4L2Buf->flags = 0;
				CQueue_AddEntry(This->m_pQueueFree,
								pEntry
								);
			}
			else
			{
#if defined(CONFIG_HOST_PLATFORM_ASPEN) || defined(CONFIG_HOST_PLATFORM_PL330SOC)
#if LINUX_VERSION_CODE < KERNEL_VERSION(3,7,0)
				__cpuc_flush_user_range(This->m_pBufMem + pDataReq->pV4L2Buf->m.offset,
							This->m_pBufMem + pDataReq->pV4L2Buf->m.offset + pDataReq->pV4L2Buf->bytesused,
							VM_IO | VM_RESERVED
							);
#else
				__cpuc_flush_user_range(This->m_pBufMem + pDataReq->pV4L2Buf->m.offset,
							This->m_pBufMem + pDataReq->pV4L2Buf->m.offset + pDataReq->pV4L2Buf->bytesused,
							VM_IO | VM_DONTEXPAND | VM_DONTDUMP
							);
#endif /* LINUX_VERSION_CODE < KERNEL_VERSION(3,7,0) */
#endif

                This->m_picture_in_num++;

				pDataReq->pV4L2Buf->bytesused = pv4lbufapp->bytesused;
				pDataReq->pV4L2Buf->reserved = pv4lbufapp->reserved;
				pDataReq->pV4L2Buf->flags = pv4lbufapp->flags;

				PROFILING_LOG(EVENT_COMP_D_IN, (TQP_UINT32) This->m_pComponent->m_hTask, This->m_id, 1+CQueue_GetQueueLevel(This->m_pQueueData), This->m_picture_in_num, pv4lbufapp->bytesused);

				CQueue_AddEntry(This->m_pQueueData,
								pEntry
								);
			}

			// push data through
			CPort_SubmitBuffers(This);
		}
		else
		{

			if (PORT_DIR_WRITE == This->m_dir) {
				PROFILING_LOG(EVENT_COMP_D_IN_FAILED, (TQP_UINT32) This->m_pComponent->m_hTask, This->m_id, 0, 0, 0);
			}

			qpStatus = QPERR_FAIL;
		}
	}
	else
	{
		if ((TQP_NULL_HANDLE == This->m_hStreamLib) ||
			(PORT_STATE_STOP == This->m_State) ||
			(PORT_DIR_NONE == This->m_dir)
			)
		{
			return (QPERR_AGAIN);
		}

		// get a free queue entry
		//
		pEntry = CQueue_GetOneEntry(This->m_pQueueFree);

		if (pEntry)
		{
			PPORT_DATA_REQ	pDataReq = (PPORT_DATA_REQ)pEntry->Data;
			IMpegCodec		*pMpegCodec = CDevice_getMpegCodec(This->m_pComponent->m_pDevice);

			// copy user v4l buffer structure
			//
			memcpy(pDataReq->pV4L2Buf,
				   pv4lbufapp,
				   sizeof(struct v4l2_buffer)
				   );

			// build the buffer descriptor
			//
			CPort_BuildBufferDescriptor(This,
										pDataReq->pBufDesc,
										pDataReq->pV4L2Buf
										);
			CPort_BuildScatterGatherList(This,
										 pDataReq->pBufDesc,
										 pDataReq->pV4L2Buf
										 );
			// save this request to the lib queue
			//
			CQueue_AddEntry(This->m_pQueueLib,
							pEntry
							);

			// add the buffer to the codec library
			//
			qpStatus = pMpegCodec->AddBuffer(pMpegCodec,
											 This->m_hStreamLib,
											 pDataReq->pBufDesc
											 );
			if (!QP_SUCCESS(qpStatus))
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
							 "%s(%d) - hStreamLib(%d) AddBuffer id(%d) (0x%X) FAILED!!!!\n",
							 __FUNCTION__,
							 This->m_id,
							 This->m_hStreamLib,
							 pDataReq->nId,
							 pDataReq->pBufDesc
							 ));

				if (pDataReq->pBufDesc->ScatterGatherBuffer)
				{
					vfree(pDataReq->pBufDesc->ScatterGatherBuffer);
					pDataReq->pBufDesc->ScatterGatherBuffer = QP_NULL;
				}
				CQueue_RemoveEntry(This->m_pQueueLib,
								   pEntry
								   );
				CQueue_AddEntry(This->m_pQueueFree,
								pEntry
								);
			}
		}
		else
		{
			qpStatus = QPERR_NOMEM;
		}
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_EXTRACE),
				 "%s(%d)-> buf index(%d) addr(0x%x) used(%d) status(%d)\n",
				 __FUNCTION__,
				 This->m_id,
				 pv4lbufapp->index,
				 pv4lbufapp->m.offset,
				 pv4lbufapp->bytesused,
				 qpStatus
				 ));
	return (qpStatus);
}


EQP_Errors CPort_CheckBuf(CPort *This, TQP_BOOL *IsEmpty)
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	CQueue		*pQueue;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
				 "%s(%d)-> [\n",
				 __FUNCTION__,
				 This->m_id
				 ));

	if (V4L2_MEMORY_MMAP == This->m_memoryType)
	{
		if (This->m_State != PORT_STATE_RUN)
		{
			return (QPERR_AGAIN);
		}

		switch (This->m_dir)
		{
			case PORT_DIR_READ:
				pQueue = This->m_pQueueData;
				break;

			case PORT_DIR_WRITE:
				pQueue = This->m_pQueueFree;
				break;

			case PORT_DIR_NONE:
			default:
				return (QPERR_NOTSUPP);
		}

		if (pQueue==QP_NULL)
			*IsEmpty = 1;
		else
			*IsEmpty = CQueue_IsEmpty(pQueue);
	}
	else
	{
		if (This->m_pQueueData==QP_NULL)
			*IsEmpty = 1;
		else
			*IsEmpty = CQueue_IsEmpty(This->m_pQueueData);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_EXTRACE),
				 "%s(%d)<- status(%d) IsEmpty=%d ]\n",
				 __FUNCTION__,
				 This->m_id,
				 qpStatus,
				 *IsEmpty
				 ));

	return (qpStatus);
}







