///////////////////////////////////////////////////////////////////////////////
//        
//
// Cport.h
//
// Description: 
//
//  base port declaration
// 
// History:
//	Jan 05, 2010	(dt) created
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __CPORT_H__
#define __CPORT_H__


#include "pldevioctl.h"

#include "qptypes.h"
#include "cobject.h"
#include "cqueue.h"
#include "qpcodeclib.h"
#include "util.h"
#ifdef CONFIG_HOST_PLATFORM_USERSPACE
#include "uservideodev2.h"
#endif


typedef enum _PORT_STATE
{
    PORT_STATE_STOP,
    PORT_STATE_ACQUIRE,
    PORT_STATE_PAUSE,
    PORT_STATE_RUN
} PORT_STATE, *PPORT_STATE;



typedef enum _PORT_DIRECTION
{
	PORT_DIR_READ = 0,
	PORT_DIR_WRITE,
	PORT_DIR_NONE

} PORT_DIRECTION, *PPORT_DIRECTION;

#define MAX_PORT_DIRECTION	PORT_DIR_NONE


#define PORT_MMAP_BUF_TYPE_SHIFT	28
#define PORT_MMAP_BUF_TYPE_MASK		0xF0000000

/////////////////////////////////////////////////////////////////////////////
//
//

// data request structure
//
typedef struct _PORT_DATA_REQ
{
	struct v4l2_buffer		*pV4L2Buf;
	PQP_BUFFER_DESCRIPTOR	pBufDesc;
	int						nId;
} PORT_DATA_REQ, *PPORT_DATA_REQ;


/////////////////////////////////////////////////////////////////////////////
//
//

#ifndef __CCOMPONENT_FWD_DEFINED__
#define __CCOMPONENT_FWD_DEFINED__
typedef struct CComponent CComponent;
#endif //!__CCOMPONENT_FWD_DEFINED__


#ifndef __CPORT_FWD_DEFINED__
#define __CPORT_FWD_DEFINED__
typedef struct CPort CPort;
#endif //!__CPORT_FWD_DEFINED__


struct CPort
{
	// public CObject
	//
	CObject						m_Object;

	// CPort
	//
	TQP_BOOL					m_valid;

	CComponent					*m_pComponent;
	COMPONENT_PORT_TYPE			m_id;

	TQP_OBJECT_HANDLE			m_hStreamLib;	// codec lib stream handle

	PORT_DIRECTION				m_dir;
	TQP_UINT32					m_dwOpenType;
	COMPONENT_PORT_OPEN_FORMAT	m_openFormat;
	int							m_memoryType;
	TQP_UINT32					m_frame_nbs;
	TQP_UINT32					m_frame_size;
	TQP_BOOL					m_bBufferPartialFill;
	TQP_BOOL					m_bBufferFrameAligned;
	TQP_BOOL					m_bSupportUserPtr;

	volatile PORT_STATE			m_State;
	volatile TQP_BOOL			m_EOS;
	TQP_UINT64					m_picture_num;
	TQP_UINT64					m_picture_in_num;
    TQP_BOOL					m_discontinuity;

	TQP_BOOL					m_bDisabled;

    PTQP_UINT8					m_pBufMem;
	QUEUE_ENTRY					m_Entries[QPCODECLIB_MAX_DATA_ENTRIES];
	PORT_DATA_REQ				m_PortReqs[QPCODECLIB_MAX_DATA_ENTRIES];
	QP_BUFFER_DESCRIPTOR		m_BufDesces[QPCODECLIB_MAX_DATA_ENTRIES];
	QP_KSSTREAM_HEADER			m_DataBufferArray[QPCODECLIB_MAX_DATA_ENTRIES];
	struct v4l2_buffer			m_v4l2Buffers[QPCODECLIB_MAX_DATA_ENTRIES];
	CQueue						*m_pQueueFree;
	CQueue						*m_pQueueData;
	CQueue						*m_pQueueLib;
	CQueue						*m_pQueueUser;
};



// CPort interfaces
//
CPort * CPort_Constructor(CPort *pPort,
						  CComponent *pComponent,
						  TQP_UINT32 dwAttributes,
						  COMPONENT_PORT_TYPE id,
						  PORT_DIRECTION dir,
						  TQP_UINT32 dwOpenType,
						  PCOMPONENT_PORT_OPEN_FORMAT pOpenFormat,
						  int memoryType,
						  TQP_UINT32 frame_nbs,
						  TQP_UINT32 frame_size,
						  TQP_BOOL bBufferPartialFill,
						  TQP_BOOL bBufferFrameAligned,
						  TQP_BOOL bSupportUserPtr
						  );
void CPort_Destructor(CPort *This);
EQP_Errors CPort_Create(CPort *This);
EQP_Errors CPort_Close(CPort *This);
EQP_Errors CPort_Enable(CPort *This);
EQP_Errors CPort_Disable(CPort *This);
EQP_Errors CPort_SetState(CPort *This,
						  PORT_STATE to_state, 
						  PORT_STATE from_state
						  );
EQP_Errors CPort_ReqBuf(CPort *This, 
						struct v4l2_requestbuffers *reqbuf
						);
EQP_Errors CPort_QueryBuf(CPort *This, 
						  struct v4l2_buffer *buf
						  );
EQP_Errors CPort_DQBuf(CPort *This, 
					   struct v4l2_buffer *pv4lbufapp
					   );
EQP_Errors CPort_QBuf(CPort *This, 
					  struct v4l2_buffer *pv4lbufapp
					  );
EQP_Errors CPort_CheckBuf(CPort *This, TQP_BOOL *IsEmpty);
#endif //__CPORT_H__



