#include "qlautoconf.h"
#include "qpdbgmgr.h"
#include "pldevioctl.h"
#include "plprofiling.h"

#ifdef ENABLE_LINUX_PROFILING

#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/delay.h>
#include <linux/version.h>
#include <linux/mm.h>
#include <asm/uaccess.h>

#include <linux/vmalloc.h>
#include <linux/slab.h>
#include <linux/semaphore.h>


QPCODEC_DIAG_PROFILING_STRUCT * g_profiling_log = NULL;
unsigned int g_profiling_mode = 1;
struct semaphore g_profiling_lock;
extern TQP_DbgMCompID	g_PLV4LDevDBGCompID;

//#define PROFILING_PRINT_OUT_MSG_ONLY
#ifdef PROFILING_PRINT_OUT_MSG_ONLY
void pl330_profiling_log_init(void)
{
}
void pl330_profiling_log_add(TQP_UINT32 dwEvent, TQP_UINT32 dwChannel, TQP_UINT32 dwType, TQP_UINT32 dwPram1, TQP_UINT32 dwFrameNo, TQP_UINT32 dwSize)
{
	unsigned long jiffies_s;
	jiffies_s = jiffies - INITIAL_JIFFIES;
	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
				 "PROFILING: E=0x%08x C=%04u type=%04u param1=%04u FrameNo=%08u Size=%08u jif=%010d jifftomsec=%010d\n",
				  dwEvent, dwChannel, dwType, dwPram1, dwFrameNo, dwSize, jiffies_s, jiffies_to_msecs(jiffies_s) & 0xFFFFFFFF
				 ));
}

void pl330_profiling_log_reset(void)
{
}

void pl330_profiling_log_fini(void)
{
}


void* pl330_profiling_get_log_address(void)
{
	return 0;
}
#else
void pl330_profiling_log_init(void)
{
	if (g_profiling_log)
		return;
	g_profiling_log = vmalloc_32(sizeof(QPCODEC_DIAG_PROFILING_STRUCT));
	if (g_profiling_log) {
		memset(g_profiling_log, 0, sizeof(QPCODEC_DIAG_PROFILING_STRUCT));
        memcpy(g_profiling_log->header,"DRVPROF", 7);
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,37)
		sema_init(&g_profiling_lock, 1);
#else
		init_MUTEX(&g_profiling_lock);
#endif
	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
				 "PROFILING: Allocate buffer @ %p\n",
				 g_profiling_log
				 ));
	}
}
void pl330_profiling_log_add(TQP_UINT32 dwEvent, TQP_UINT32 dwChannel, TQP_UINT32 dwType, TQP_UINT32 dwPram1, TQP_UINT32 dwFrameNo, TQP_UINT32 dwSize)
{
    TQP_UINT32 index;
	unsigned long jiffies_s;
	jiffies_s = jiffies - INITIAL_JIFFIES;
    if (g_profiling_log) {
        if (g_profiling_log->looped==1) {
            /* table full, just return */
            return;
        }
		down(&g_profiling_lock);
		index = g_profiling_log->cur_profiling_idx;
		g_profiling_log->cur_profiling_idx ++;
		if (g_profiling_log->cur_profiling_idx >= QP_MAX_PROFILING_SIZE) {
			//g_profiling_log->cur_profiling_idx = 0;
			g_profiling_log->looped = 1;
		}

    	g_profiling_log->entries[index][0] = dwEvent;
	    g_profiling_log->entries[index][1] = ((dwChannel & 0xF) << 28) | ((dwType & 0xF) << 24) | (dwFrameNo &0x00FFFFFF);
	    g_profiling_log->entries[index][2] = dwSize;
	    g_profiling_log->entries[index][3] = dwPram1;
	    g_profiling_log->entries[index][4] = jiffies_to_msecs(jiffies_s) & 0xFFFFFFFF;

        #if 0
        QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
				 	"PROFILING: [%08x][%08x][%08x][%08x][%08x][%08x]\n",
				 	g_profiling_log->entries[index][0],
				 	g_profiling_log->entries[index][1],
				 	g_profiling_log->entries[index][2],
				 	g_profiling_log->entries[index][3],
				 	g_profiling_log->entries[index][4]
				 	));
        #endif
		up(&g_profiling_lock);
	}
}

void pl330_profiling_log_reset(void)
{
	if (g_profiling_log) {
		down(&g_profiling_lock);
		g_profiling_log->looped = 0;
		g_profiling_log->cur_profiling_idx = 0;
		up(&g_profiling_lock);
	}
}

void pl330_profiling_log_fini(void)
{
	if (g_profiling_log) {
		vfree(g_profiling_log);
		g_profiling_log = NULL;
	}
}

void* pl330_profiling_get_log_address(void)
{
	if (g_profiling_log) {
		return ((void *)g_profiling_log);
	}
	return 0;
}

#endif

//#else
//#error "ENABLE_LINUX_PROFILING not defined"
#endif
