/*************************************************************************/
/**      **/
/**            **/
/**                **/
/**              **/
/**                                                                     **/
/**    **/
/**                 **/
/**                                                                     **/
/**                               **/
/**                                                  **/
/*************************************************************************/

#include "qlautoconf.h"

#ifndef CONFIG_HOST_PLATFORM_USERSPACE
#include <linux/version.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/mm.h>
#include <linux/module.h>
#include <linux/poll.h>
#include <linux/slab.h>
#include <linux/vmalloc.h>
#include <asm/io.h>
#include <linux/string.h>

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/fs.h>
#include <asm/uaccess.h>
#include <asm/fcntl.h>


int qlf_open(unsigned long *pdwHandle, unsigned char *pFName, unsigned int dwReadOnly)
{
    int Err= 0;
    if(dwReadOnly) {
        *pdwHandle = (unsigned long)filp_open(pFName, 0, 0);
    } else {
        *pdwHandle = (unsigned long)filp_open(pFName, O_CREAT | O_RDWR | O_TRUNC, 0);
    }
    if(IS_ERR((void*)*pdwHandle)) {
        Err= *pdwHandle;
        *pdwHandle= 0;
    }
    printk("qlusb: open fx2fw %s. Err=%d \n", pFName, Err);
    return Err;
}

int qlf_close(unsigned long dwHandle)
{
    if((NULL != (struct file*)dwHandle) && (!(IS_ERR((struct file*)dwHandle))))
        return filp_close((struct file*)dwHandle, 0);
    else
        return 0;
}

int qlf_write(unsigned long dwHandle, unsigned char *pBuff, unsigned int dwLen)
{
    mm_segment_t    orgfs;
    int Err = 0;

    orgfs = get_fs();
    set_fs(get_ds());

    if((NULL != (struct file*)dwHandle) && (!(IS_ERR((struct file*)dwHandle))))
#ifndef __LINUX24__
        Err = vfs_write((void*)dwHandle, pBuff, dwLen, &((struct file*)dwHandle)->f_pos);
#else
        Err = ((struct file*)dwHandle)->f_op->write((void*)dwHandle, pBuff, dwLen, &((struct file*)dwHandle)->f_pos);
#endif
    else
        Err = -EINVAL;

    set_fs(orgfs);
    return Err;
}

int qlf_read(unsigned long dwHandle, unsigned char *pBuff, unsigned int dwLen)
{
    mm_segment_t    orgfs;
    int Err = 0;

    if ((struct file*)dwHandle == NULL)
        return -ENOENT;
    if (((struct file*)dwHandle)->f_op->read == NULL)
        return -ENOSYS;

    orgfs = get_fs();
    set_fs(get_ds());

    if((NULL != (struct file*)dwHandle) && (!(IS_ERR((struct file*)dwHandle))))
#ifndef __LINUX24__
        Err = vfs_read((struct file*)dwHandle, pBuff, dwLen, &((struct file*)dwHandle)->f_pos);
#else
        Err = ((struct file*)dwHandle)->f_op->read((void*)dwHandle, pBuff, dwLen, &((struct file*)dwHandle)->f_pos);
#endif
    else
        Err = -EINVAL;

    set_fs(orgfs);
    return Err;
}



int qlf_load(char *pszFileName,
			  unsigned char **ppucaBuff,
			  unsigned int *pulBuffLen
			  )
{
	struct file		*filp = 0;
	int				l=0, curRead;
    int				lErr = 0;
    unsigned char	*pucaBuff = NULL;

	*ppucaBuff = NULL;
	*pulBuffLen = 0;

	lErr = qlf_open((unsigned long *)&filp,
					pszFileName,
					1
					);

    if (!lErr &&
		filp
		)
	{
#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,18,0)
		l = filp->f_path.dentry->d_inode->i_size;
#else
		l = filp->f_dentry->d_inode->i_size;
#endif

		// allocate buffer
#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
		pucaBuff = (unsigned char *)kmalloc(l, GFP_KERNEL);
#else
		pucaBuff = (unsigned char *)vmalloc(l);
#endif

		if (pucaBuff)
		{
			curRead = qlf_read((unsigned long)filp, pucaBuff, l);

			if ((curRead < 0) ||
				(curRead != l)
				)
			{
				printk("Failed to read '%s' expected(%d) read(%d).\n", pszFileName, l, curRead);
#ifdef USE_KMALLOC_INSTEAD_OF_VMALLOC
				kfree(pucaBuff);
#else
				vfree(pucaBuff);
#endif
				lErr = -EINVAL;
			}
		}
		else
		{
			printk("Failed to allocate (%d) bytes.\n", l);
			lErr = -ENOMEM;
		}
		qlf_close((unsigned long)filp);
	}

	if (!lErr)
	{
		*ppucaBuff = pucaBuff;
		*pulBuffLen = l;
	}

	printk("%s()-> file %s, size %d\n", __FUNCTION__, pszFileName, *pulBuffLen);

	return (lErr);
}
#else /* CONFIG_HOST_PLATFORM_USERSPACE */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

int qlf_load(char *pszFileName,
              unsigned char **ppucaBuff,
              unsigned int *pulBuffLen
              )
{
    long			l=0;
    int             lErr = 0;
    FILE            *fp;
    unsigned char	*pucaBuff = NULL;
    size_t          curRead;

    *ppucaBuff = NULL;
    *pulBuffLen = 0;

    fp = fopen(pszFileName, "rb");

    if (fp!=NULL)
    {
        fseek(fp, 0, SEEK_END);
        l=ftell(fp);
        fseek(fp, 0, SEEK_SET);

        if (l>0)
        {
            pucaBuff = (unsigned char *)malloc(l);
            if (pucaBuff)
            {
                curRead = fread(pucaBuff, 1, l, fp);

                if (curRead != l)
                {
                    fprintf(stderr, "%s()-> Failed to read '%s' expected(%ld) read(%zu).\n", __FUNCTION__, pszFileName, l, curRead);
                    free(pucaBuff);
                    lErr = -EINVAL;
                }
            }
            else
            {
                fprintf(stderr, "%s()-> Failed to allocate (%ld) bytes.\n", __FUNCTION__, l);
                lErr = -ENOMEM;
            }
        }
        else {
            fprintf(stderr, "%s()-> Failed to seed '%s' for length (%ld).\n", __FUNCTION__, pszFileName, l);
            lErr = -EINVAL;
        }
        fclose(fp);
    }
    else {
        lErr = -ENOENT;
    }

    if (!lErr)
    {
        *ppucaBuff = pucaBuff;
        *pulBuffLen = (unsigned int) l;
    }


    fprintf(stderr, "%s()-> file %s, size %d Err=%d\n", __FUNCTION__, pszFileName, *pulBuffLen, lErr);

    return (lErr);
}
#endif /* CONFIG_HOST_PLATFORM_USERSPACE */





