/*************************************************************************/
/**      **/
/**            **/
/**                **/
/**              **/
/**                                                                     **/
/**    **/
/**                  **/
/**                                                                     **/
/**                               **/
/**                                                  **/
/**  Changed by Frank.Wei  for QL3xx, base on QL2XX         			**/
/*************************************************************************/
#include "qlautoconf.h"

#include "qptypes.h"
#include "pldevioctl.h"

#ifdef SUPPORT_PL3XX_OSD
#include <linux/sched.h>
#include <linux/delay.h>
#include <linux/version.h>
#include <linux/vmalloc.h>
#include <linux/slab.h>

#include "cdevice.h"

int qldiag_osd_param(CDevice *pDevice, void *p)
{
	int lErr = 0;
	QLDEV_OSD_PARAMETER *param = (QLDEV_OSD_PARAMETER *)p;

	switch(param->type){
#ifdef SUPPORT_VO_OSD
		case PARAM_SET_PIXEL_FORMAT:
			lErr = qldiag_osd_SetPixelFormat(pDevice, &param->value);
			break;
		case PARAM_SET_RESOLUTION:
			lErr = qldiag_osd_SetResolution(pDevice, &param->value);
			break;
		case PARAM_SET_HORIZONTALX2:
			lErr = qldiag_osd_SetHorizontalX2(pDevice, &param->value);
			break;
		case PARAM_SET_VERTICALX2:
			lErr = qldiag_osd_SetVerticalX2(pDevice, &param->value);
			break;
		case PARAM_SET_BLENDING:
			lErr = qldiag_osd_EnableBlending(pDevice, &param->value);
			break;
		case PARAM_SET_BLENDING_VALUE:
			lErr = qldiag_osd_SetBlendingValue(pDevice, &param->value);
			break;
		case PARAM_SET_ENABLE:
			lErr = qldiag_osd_EnableOSD(pDevice, &param->value);
			break;
		case PARAM_SET_DISABLE:
			lErr = qldiag_osd_DisableOSD(pDevice, &param->value);
			break;
		case PARAM_GET_PIXEL_FORMAT:
			lErr = qldiag_osd_GetPixelFormat(pDevice, &param->value);
			break;
		case PARAM_GET_RESOLUTION:
			lErr = qldiag_osd_GetResolution(pDevice, &param->value);
			break;
		case PARAM_SET_ADDRESS:
			lErr = qldiag_osd_SetStartAddress(pDevice, &param->value);
			break;
#endif

#ifdef SUPPORT_VI_OSD
		case PARAM_VIOSD_STMEN:
			lErr = qldiag_viosd_TimeStampEnable(pDevice, &param->value);
			break;
		case PARAM_VIOSD_STMFT:
			lErr = qldiag_viosd_TimeStampFontType(pDevice, &param->value);
			break;
		case PARAM_VIOSD_STMXP:
			lErr = qldiag_viosd_TimeStampXPosition(pDevice, &param->value);
			break;
		case PARAM_VIOSD_STMYP:
			lErr = qldiag_viosd_TimeStampYPosition(pDevice, &param->value);
			break;
		case PARAM_VIOSD_STREN:
			lErr = qldiag_viosd_TimeStringEnable(pDevice, &param->value);
			break;
		case PARAM_VIOSD_STRFT:
			lErr = qldiag_viosd_TimeStringFontType(pDevice, &param->value);
			break;
		case PARAM_VIOSD_STRXP:
			lErr = qldiag_viosd_TimeStringXPosition(pDevice, &param->value);
			break;
		case PARAM_VIOSD_STRYP:
			lErr = qldiag_viosd_TimeStringYPosition(pDevice, &param->value);
			break;
#endif
	}
	return lErr;
}

int qldiag_osd_control(CDevice *pDevice, void *p)
{
	int lErr = 0;
	PQLDEV_OSD_CONTROL pOsd = (PQLDEV_OSD_CONTROL)p;

#ifdef SUPPORT_VI_OSD
	if(pOsd->CmdType == PARAM_VIOSD_STR)
	{
		PQLDEV_VIOSD_CONTROL pViOsd = (PQLDEV_VIOSD_CONTROL )p;
		lErr = qldiag_viosd_TimeString(pDevice, pViOsd);
	}
#endif

#ifdef SUPPORT_VO_OSD
	switch(pOsd->CmdType)
	{
		case PARAM_SET_CLUT:
			lErr = qldiag_osd_SetCLUT(pDevice, pOsd);
			break;
		case PARAM_GET_CLUT:
			lErr = qldiag_osd_GetCLUT(pDevice, pOsd);
			break;
		case PARAM_MEM_ALLOCATE:
			lErr = qldiag_osd_AllocateMemory(pDevice, pOsd);
			break;
		case PARAM_MEM_RELEASE:
			lErr = qldiag_osd_FreeMemory(pDevice, pOsd);
			break;
		case PARAM_SET_CURRENTOSD:
			lErr = qldiag_osd_SetCurrentOSD(pDevice, pOsd);
			break;
		case PARAM_GET_CURRENTOSD:
			lErr = qldiag_osd_GetCurrentOSD(pDevice, pOsd);
			break;
		case PARAM_DRAW_DOT:
			lErr = qldiag_osd_DrawDot(pDevice, pOsd);
			break;
		case PARAM_DRAW_RECT:
			lErr = qldiag_osd_DrawRect(pDevice, pOsd);
			break;
		case PARAM_DRAW_IMAGE:
			lErr = qldiag_osd_DrawImage(pDevice, pOsd);
			break;
		case PARAM_DRAW_TEXT:
			lErr = qldiag_osd_DrawText(pDevice, pOsd);
			break;
		case PARAM_PASTE_BUFFER:
			lErr = qldiag_osd_PasteBuffer(pDevice, pOsd);
			break;
		case PARAM_COPY_BUFFER:
			lErr = qldiag_osd_CopyBuffer(pDevice, pOsd);
			break;
		case PARAM_SET_HILIGHT:
			lErr = qldiag_osd_SetHiLight(pDevice, pOsd);
			break;
	}
#endif
	return lErr;
}

#ifdef SUPPORT_VO_OSD
int qldiag_osd_SetStartAddress(CDevice *pDevice, void *p)
{
	int lErr = 0;
	unsigned int value = *((unsigned int *)p);

	pDevice->m_OSDInfo.unFrameBufferAddr = value;
	return lErr;
}

int qldiag_osd_SetPixelFormat(CDevice *pDevice, void *p)
{
	int lErr = 0;
	OSD_CONFIG reg;
	unsigned int reg_val;
	unsigned int value = *((unsigned int *)p);

	lErr= CDevice_rdreg(pDevice, REG_OSD_CONFIGURATION, (unsigned char*)&reg_val, 0);
	reg.Read = reg_val;
	reg.Reg.osd_mem_burst = 0xf;
	if (value <= PIXEL_MAX_FORMAT){
		pDevice->m_OSDInfo.mDisplay.PixelFormat = value;
		reg.Reg.osd_data_length = value;
		printk("%s: Set pixel format value:0x%x \n", __FUNCTION__, value);
	}else{
		printk("%s: Set unknown value:0x%x \n", __FUNCTION__, value);
	}
	reg_val = reg.Read;
       lErr= CDevice_wrreg(pDevice, REG_OSD_CONFIGURATION, (unsigned char*)&reg_val, 0);
	return lErr;
}

int qldiag_osd_GetPixelFormat(CDevice *pDevice, void *p)
{
	int lErr = 0;
	OSD_CONFIG reg;
	unsigned int reg_val;
	unsigned int *pval = (unsigned int *)p;

       lErr = CDevice_rdreg(pDevice, REG_OSD_CONFIGURATION, (unsigned char*)&reg_val, 0);

	reg.Read = reg_val;
	reg.Reg.osd_data_length &= 0x0f;

	if (reg.Reg.osd_data_length <= PIXEL_MAX_FORMAT){
		pDevice->m_OSDInfo.mDisplay.PixelFormat = *pval = reg.Reg.osd_data_length;
		//printk("%s: Get pixel format value:0x%x \n", __FUNCTION__, reg.Reg.osd_data_length);
	}else{
		printk("%s: Get unknown pixel format value:0x%x \n", __FUNCTION__, reg.Reg.osd_data_length);
		lErr = - EINVAL;
	}

	return lErr;
}

int qldiag_osd_SetHorizontalPixelOffset(CDevice *pDevice, void *p)
{
	int lErr = 0;
	unsigned int reg_val;
	unsigned int value = *((unsigned int *)p);

	switch (value){
		case MASK_RES_720X480I:
		case MASK_RES_720X480P:
		case MASK_RES_720X576I:
		case MASK_RES_720X576P:
			reg_val =  0x37;
			break;

		case MASK_RES_1280X1080I:
		case MASK_RES_1280X1080P:
		case MASK_RES_1280X720P:
		default:
			reg_val =  0x47;
			break;
	}
	lErr= CDevice_wrreg(pDevice, REG_OSD_HORIZONTAL_PIXEL_OFFSET, (unsigned char*)&reg_val, 0);
	return lErr;
}

int qldiag_osd_SetResolution(CDevice *pDevice, void *p)
{
	int lErr = 0;
	unsigned int reg_start = 0;
	unsigned int reg_size = 0;
	WIN_RESOLUTION value = *((WIN_RESOLUTION *)p);

	//reset hight light window here?
	lErr= CDevice_wrreg(pDevice, REG_OSD_HILIGHT_WINDOW_START, (unsigned char*)&reg_start, 0);
	lErr= CDevice_wrreg(pDevice, REG_OSD_HILIGHT_WINDOW_SIZE, (unsigned char*)&reg_size, 0);

	switch (value){
		case MASK_RES_352X240I:
			reg_size = value;
			reg_start = 0x00600018;
			pDevice->m_OSDInfo.mDisplay.Height = 240;
			pDevice->m_OSDInfo.mDisplay.Width = 352;
			break;

		case MASK_RES_352X288I:
			reg_size = value;
			reg_start = 0x00600020;
			pDevice->m_OSDInfo.mDisplay.Height = 288;
			pDevice->m_OSDInfo.mDisplay.Width = 352;
			break;

		case MASK_RES_720X480I:
		case MASK_RES_720X480P:
			reg_size = value;
			reg_start = 0x00600018;
			pDevice->m_OSDInfo.mDisplay.Height = 480;
			pDevice->m_OSDInfo.mDisplay.Width = 720;
			break;

		case MASK_RES_720X576I:
		case MASK_RES_720X576P:
			reg_size = value;
			reg_start = 0x00600030;
			pDevice->m_OSDInfo.mDisplay.Height = 576;
			pDevice->m_OSDInfo.mDisplay.Width = 720;
			break;

		case MASK_RES_1280X1080I:
		case MASK_RES_1280X1080P:
			reg_size = value;
			reg_start = 0x0172001a;
			pDevice->m_OSDInfo.mDisplay.Height = 1080;
			pDevice->m_OSDInfo.mDisplay.Width = 1280;
			break;

		case MASK_RES_1280X720P:
			reg_size = value;
			reg_start = 0x0172001a;
			pDevice->m_OSDInfo.mDisplay.Height = 720;
			pDevice->m_OSDInfo.mDisplay.Width = 1280;
			break;

		default:		//close the osd window
			reg_size = 0;
			reg_start = 0;
			pDevice->m_OSDInfo.mDisplay.Height = 0;
			pDevice->m_OSDInfo.mDisplay.Width = 0;
			break;
	}

	qldiag_osd_SetHorizontalPixelOffset(pDevice, &reg_size);

	lErr= CDevice_wrreg(pDevice, REG_OSD_DISPLAY_WINDOW_START, (unsigned char*)&reg_start, 0);
	lErr= CDevice_wrreg(pDevice, REG_OSD_DISPLAY_WINDOW_SIZE, (unsigned char*)&reg_size, 0);
	printk("%s: Set Resolution value:0x%x \n", __FUNCTION__, value);
	return lErr;
}

int qldiag_osd_GetResolution(CDevice *pDevice, void *p)
{
	int lErr = 0;
	unsigned int reg_size = 0;
	unsigned int *pval = (unsigned int *)p;

	lErr= CDevice_rdreg(pDevice, REG_OSD_DISPLAY_WINDOW_SIZE, (unsigned char*)&reg_size, 0);
	*pval = reg_size;
	//printk("%s: Get Resolution value:0x%x \n", __FUNCTION__, reg_size);

	return lErr;
}


int qldiag_osd_SetHorizontalX2(CDevice *pDevice, void *p)
{
	int lErr = 0;
	OSD_CONFIG reg;
	unsigned int reg_val;
	unsigned int value = *((unsigned int *)p);

	lErr= CDevice_rdreg(pDevice, REG_OSD_CONFIGURATION, (unsigned char*)&reg_val, 0);
	reg.Read = reg_val;
	reg.Reg.h2x_e = (value)? 1 : 0 ;
	reg_val = reg.Read;
	lErr= CDevice_wrreg(pDevice, REG_OSD_CONFIGURATION, (unsigned char*)&reg_val, 0);
	return lErr;
}

int qldiag_osd_SetVerticalX2(CDevice *pDevice, void *p)
{
	int lErr = 0;
	OSD_CONFIG reg;
	unsigned int reg_val;
	unsigned int value = *((unsigned int *)p);

	lErr= CDevice_rdreg(pDevice, REG_OSD_CONFIGURATION, (unsigned char*)&reg_val, 0);
	reg.Read = reg_val;
	reg.Reg.v2x_e = (value)? 1 : 0 ;
	reg_val = reg.Read;
	lErr= CDevice_wrreg(pDevice, REG_OSD_CONFIGURATION, (unsigned char*)&reg_val, 0);
	return lErr;
}

int qldiag_osd_SetFilter(CDevice *pDevice, void *p)
{
	int lErr = 0;
	OSD_CONFIG reg;
	unsigned int reg_val;
	unsigned int value = *((unsigned int *)p);

	lErr= CDevice_rdreg(pDevice, REG_OSD_CONFIGURATION, (unsigned char*)&reg_val, 0);
	reg.Read = reg_val;
	if (value <4)
	reg.Reg.h2x_flt = reg.Reg.v2x_flt = value;
	reg_val = reg.Read;
	lErr= CDevice_wrreg(pDevice, REG_OSD_CONFIGURATION, (unsigned char*)&reg_val, 0);
	return lErr;
}

int qldiag_osd_SetRequestInterval(CDevice *pDevice, void *p)
{
	int lErr = 0;
	unsigned int reg_val = 16;
	unsigned int value = *((unsigned int *)p);

	if (value < 0x80) reg_val = value;
	lErr= CDevice_wrreg(pDevice, REG_OSD_CONFIGURATION, (unsigned char*)&reg_val, 0);
	return lErr;
}

int qldiag_osd_EnableBlending(CDevice *pDevice, void *p)
{
	int lErr = 0;
	OSD_CONFIG reg;
	unsigned int reg_val;
	unsigned int value = *((unsigned int *)p);

	lErr= CDevice_rdreg(pDevice, REG_OSD_CONFIGURATION, (unsigned char*)&reg_val, 0);
	reg.Read = reg_val;
	reg.Reg.g_blend_en = (value)? 1 : 0 ;
	reg_val = reg.Read;
	lErr= CDevice_wrreg(pDevice, REG_OSD_CONFIGURATION, (unsigned char*)&reg_val, 0);
	return lErr;
}

int qldiag_osd_SetBlendingValue(CDevice *pDevice, void *p)
{
	int lErr = 0;
	OSD_CONFIG reg;
	unsigned int reg_val;
	unsigned int value = *((unsigned int *)p);

	lErr= CDevice_rdreg(pDevice, REG_OSD_CONFIGURATION, (unsigned char*)&reg_val, 0);
	reg.Read = reg_val;
	reg.Reg.g_blend_factor = value & 0xff;	// 0 = 100%OSD 0xff=transparence
	reg_val = reg.Read;
	lErr= CDevice_wrreg(pDevice, REG_OSD_CONFIGURATION, (unsigned char*)&reg_val, 0);
	return lErr;
}

//R-G-B-Alf inside
int	qldiag_osd_SetCLUT(CDevice *pDevice, void *p)
{
	int lErr = 0;
	unsigned int reg_val;
	int i;
	PQLDEV_OSD_CONTROL pQLOSD = (PQLDEV_OSD_CONTROL)p;

	reg_val = 0x00000700;
	lErr= CDevice_wrreg(pDevice, REG_OSD_CLUT_ADDRESS, (unsigned char*)&reg_val, 0);
	switch (pDevice->m_OSDInfo.mDisplay.PixelFormat){
		case PIXEL_LUT_1BIT:
			if (pQLOSD->swap == 1) *((unsigned int*)pQLOSD->pColorData) = ulBigToLittleEndian(*((unsigned int*)pQLOSD->pColorData));
			lErr = lErr ? lErr : CDevice_wrreg(pDevice, REG_OSD_CLUT_DATA, (unsigned char*)pQLOSD->pColorData, 0);
			if (pQLOSD->swap == 1) *((unsigned int*)(pQLOSD->pColorData + 1)) = ulBigToLittleEndian(*((unsigned int*)(pQLOSD->pColorData + 1)));
			lErr = lErr ? lErr : CDevice_wrreg(pDevice, REG_OSD_CLUT_DATA, (unsigned char*)(pQLOSD->pColorData+1), 0);
			break;

		case PIXEL_LUT_2BIT:
			for (i = 0; i < 4;  i++){
				if (pQLOSD->swap == 1)
					*((unsigned int*)(pQLOSD->pColorData + i)) = ulBigToLittleEndian(*((unsigned int*)(pQLOSD->pColorData + i)));
				lErr = lErr ? lErr : CDevice_wrreg(pDevice, REG_OSD_CLUT_DATA, (unsigned char*)(pQLOSD->pColorData + i), 0);
			}
			break;

		case PIXEL_LUT_4BIT:
			for (i = 0;  i < 16;  i++){
				if (pQLOSD->swap == 1)
					*((unsigned int*)(pQLOSD->pColorData + i)) = ulBigToLittleEndian(*((unsigned int*)(pQLOSD->pColorData + i)));
				lErr = lErr ? lErr : CDevice_wrreg(pDevice, REG_OSD_CLUT_DATA, (unsigned char*)(pQLOSD->pColorData + i), 0);
			}
			break;

		case PIXEL_LUT_8BIT:
			for (i = 0; i <256; i++){
				if (pQLOSD->swap == 1)
					*((unsigned int*)(pQLOSD->pColorData + i)) = ulBigToLittleEndian(*((unsigned int*)(pQLOSD->pColorData + i)));
				lErr = lErr ? lErr : CDevice_wrreg(pDevice, REG_OSD_CLUT_DATA, (unsigned char*)(pQLOSD->pColorData + i), 1);
			}
			break;

		default:
			printk("%s: Not support pixel format:0x%x \n", __FUNCTION__, pDevice->m_OSDInfo.mDisplay.PixelFormat);
			lErr = ERR_FORMAT_SUPPORT;
			break;
	}

	reg_val = 0x00000400;
	lErr = lErr ? lErr : CDevice_wrreg(pDevice, REG_OSD_CLUT_ADDRESS, (unsigned char*)&reg_val, 0);
	//printk("%s: Set CLUT pixel format = %x\n", __FUNCTION__, pDevice->m_OSDInfo.mDisplay.PixelFormat);
	return lErr;
}

int	qldiag_osd_GetCLUT(CDevice *pDevice, void *p)
{
	int lErr = 0;
	unsigned int reg_val;
	int i;
	PQLDEV_OSD_CONTROL pQLOSD = (PQLDEV_OSD_CONTROL)p;

	reg_val = 0x00000700;
	lErr= CDevice_wrreg(pDevice, REG_OSD_CLUT_ADDRESS, (unsigned char*)&reg_val, 0);
	switch (pDevice->m_OSDInfo.mDisplay.PixelFormat){
		case PIXEL_LUT_1BIT:
			lErr = lErr ? lErr : CDevice_rdreg(pDevice, REG_OSD_CLUT_DATA, (unsigned char*)pQLOSD->pColorData, 1);
			lErr = lErr ? lErr : CDevice_rdreg(pDevice, REG_OSD_CLUT_DATA, (unsigned char*)(pQLOSD->pColorData+1), 1);
			break;

		case PIXEL_LUT_2BIT:
			for (i = 0; i < 4;  i++)
				lErr = lErr ? lErr : CDevice_rdreg(pDevice, REG_OSD_CLUT_DATA, (unsigned char*)(pQLOSD->pColorData + i), 1);
			break;

		case PIXEL_LUT_4BIT:
			for (i = 0;  i < 16;  i++)
				lErr = lErr ? lErr : CDevice_rdreg(pDevice, REG_OSD_CLUT_DATA, (unsigned char*)(pQLOSD->pColorData + i), 1);
			break;

		case PIXEL_LUT_8BIT:
			for (i = 0; i <256; i++){
				lErr = lErr ? lErr : CDevice_rdreg(pDevice, REG_OSD_CLUT_DATA, (unsigned char*)(pQLOSD->pColorData + i), 1);
				if (pQLOSD->swap == 1)
					*((unsigned int*)(pQLOSD->pColorData + i)) = ulBigToLittleEndian(*((unsigned int*)(pQLOSD->pColorData + i)));
			}
			break;

		default:
			printk("%s: Not support pixel format:0x%x \n", __FUNCTION__, pDevice->m_OSDInfo.mDisplay.PixelFormat);
			lErr = ERR_FORMAT_SUPPORT;
			break;
	}

	reg_val = 0x00000400;
	lErr = lErr ? lErr : CDevice_wrreg(pDevice, REG_OSD_CLUT_ADDRESS, (unsigned char*)&reg_val, 0);
	return lErr;
}

int qldiag_osd_EnableOSD(CDevice *pDevice, void *p)
{
	int lErr = 0;
	OSD_CONFIG reg;
	unsigned int reg_val;
	//unsigned int value = *((unsigned int *)p);

	lErr= CDevice_rdreg(pDevice, REG_OSD_CONFIGURATION, (unsigned char*)&reg_val, 0);
	reg.Read = reg_val;
	reg.Reg.en = 1;
	reg.Reg.f_flush = 0 ;
	reg.Reg.f_lwm = 1;
	reg_val = reg.Read;
	lErr= CDevice_wrreg(pDevice, REG_OSD_CONFIGURATION, (unsigned char*)&reg_val, 0);

	//Layer Selection
	reg_val = MASK_ENABLE_OSD;
	lErr= CDevice_wrreg(pDevice, REG_OSD_LAYERSELECTION, (unsigned char*)&reg_val, 0);

	return lErr;
}

int qldiag_osd_DisableOSD(CDevice *pDevice, void *p)
{
	int lErr = 0;
	OSD_CONFIG reg;
	unsigned int reg_val;
	//unsigned int value = *((unsigned int *)p);

	//Layer Selection
	reg_val = MASK_DISABLE_OSD;
	lErr= CDevice_wrreg(pDevice, REG_OSD_LAYERSELECTION, (unsigned char*)&reg_val, 0);

	lErr= CDevice_rdreg(pDevice, REG_OSD_CONFIGURATION, (unsigned char*)&reg_val, 0);
	reg.Read = reg_val;
	reg.Reg.en = 0;
	reg.Reg.f_flush = 1;
	reg.Reg.f_lwm = 1;
	reg_val = reg.Read;
	lErr= CDevice_wrreg(pDevice, REG_OSD_CONFIGURATION, (unsigned char*)&reg_val, 0);

	return lErr;
}

int qldiag_osd_SetHiLight(CDevice *pDevice, void *p)
{
	int lErr = 0;
	unsigned int reg_start;
	unsigned int reg_size;
       PQLDEV_OSD_CONTROL pQLOSD = (PQLDEV_OSD_CONTROL)p;

	lErr= CDevice_rdreg(pDevice, REG_OSD_DISPLAY_WINDOW_START, (unsigned char*)&reg_start, 0);
       reg_start += pQLOSD->x << 16;
       reg_start += pQLOSD->y;
	lErr= CDevice_wrreg(pDevice, REG_OSD_HILIGHT_WINDOW_START, (unsigned char*)&reg_start, 0);

	//bit31-24 clut, bit23-0 yuv?
	reg_size = *(pQLOSD->pColorData);
	lErr=  lErr ? lErr : CDevice_wrreg(pDevice, REG_OSD_HILIGHT_CONFIGURATION, (unsigned char*)&reg_size, 0);
       reg_size =  (pQLOSD->Width << 16 ) | pQLOSD->Height;
	lErr=  lErr ? lErr : CDevice_wrreg(pDevice, REG_OSD_HILIGHT_WINDOW_SIZE, (unsigned char*)&reg_size, 0);

	return lErr;
}

int qldiag_osd_AllocateMemory(CDevice *pDevice, void *p)
{
	unsigned int ulMapVal;
	unsigned int ulTemp;
	int	i;
	int lErr = 0;
	COLOR_DATA bgcolor;
	int iEmptySpace = -1;
	PQLDEV_OSD_CONTROL pQLOSD = (PQLDEV_OSD_CONTROL)p;
	unsigned int fbuf_size;
	//unsigned int *pfbuf_start;
	//unsigned int memType;

	//find a free memory ID
	ulMapVal = pDevice->m_OSDInfo.mDisplay.UsedSheetMap;
	for ( i = 0; i < MAX_OSD_BUFFER_NUM; i++ ){
		 ulTemp = ulMapVal & (0x00000001<<i);
		 if (ulTemp > 0 ) {
		 	 continue;
		 }else{
		 	iEmptySpace = i;
		 	break;
		 }
	}

	if ( i >= 0 && i < MAX_OSD_BUFFER_NUM) {
		pQLOSD->BufferID = i;
	}else{
		pQLOSD->RetCode = ERR_OUT_OF_MEMORY;
		printk("%s: Waring::No more frame buffer available\n", __FUNCTION__);
		return ERR_OUT_OF_MEMORY;
	}

	switch (pDevice->m_OSDInfo.mDisplay.PixelFormat){
		case PIXEL_LUT_8BIT:
			fbuf_size = pDevice->m_OSDInfo.mDisplay.Width * pDevice->m_OSDInfo.mDisplay.Height;
		#if 1
			if ((pDevice->m_OSDInfo.unFrameBufferAddr == 0) || (pDevice->m_OSDInfo.unFrameBufferAddr == 0xffffffff)){
				pQLOSD->RetCode = ERR_OUT_OF_MEMORY;
				printk("%s: Waring::No frame buffer available, VO on now?\n", __FUNCTION__);
				return ERR_OUT_OF_MEMORY;
			}
			pDevice->m_OSDInfo.mDisplay.mSheet[i].StartAddr = pDevice->m_OSDInfo.unFrameBufferAddr + i * fbuf_size;
		#else
			pDevice->m_OSDInfo.mDisplay.mSheet[i].StartAddr = OSD_BASE_ADDR + i * fbuf_size;
		#endif

			//marked this memory
			pDevice->m_OSDInfo.mDisplay.UsedSheetMap |= (0x00000001<<i);
			pDevice->m_OSDInfo.mDisplay.mSheet[i].BufferID = pDevice->m_OSDInfo.mDisplay.CurrentBufferID = i;
			bgcolor.Read =0x0;
			printk("%s map=%d, bufferID=%d, addr:0x%08x \n", __FUNCTION__, pDevice->m_OSDInfo.mDisplay.UsedSheetMap, i, pDevice->m_OSDInfo.mDisplay.mSheet[i].StartAddr);
			break;

		default:
			printk("%s: Not support pixel format:0x%x \n", __FUNCTION__, pDevice->m_OSDInfo.mDisplay.PixelFormat);
			lErr = ERR_FORMAT_SUPPORT;
			return lErr;
	}

	if (pQLOSD->bUpdateTransparency){
		pQLOSD->pColorData = (unsigned int*)&bgcolor;
		pQLOSD->x = 0;
		pQLOSD->y = 0;
		pQLOSD->Width = pDevice->m_OSDInfo.mDisplay.Width;
		pQLOSD->Height = pDevice->m_OSDInfo.mDisplay.Height;

		//clear the buffer with the defined color
		lErr = qldiag_osd_DrawRect(pDevice, pQLOSD);
	}
	pQLOSD->RetCode = lErr;

	return lErr;
}

int qldiag_osd_FreeMemory(CDevice *pDevice, void *p )
{
	int lErr = 0;
	unsigned int	ulMapVal;
	unsigned int BufferID;
	PQLDEV_OSD_CONTROL pQLOSD = (PQLDEV_OSD_CONTROL)p;

	ulMapVal = pDevice->m_OSDInfo.mDisplay.UsedSheetMap;
	BufferID = pQLOSD->BufferID;

	lErr = (ulMapVal & (0x00000001<<BufferID)) ? NO_ERR : ERR_ALLOCATE_MEMORY;

	if ( !lErr ){
		pDevice->m_OSDInfo.mDisplay.UsedSheetMap &= ~(0x00000001<<BufferID);
	}
	pQLOSD->RetCode = lErr;

	return lErr;
}

int	qldiag_osd_SetCurrentOSD(CDevice *pDevice,  void *p )
{
	int lErr = 0;
	unsigned int dramAddr;
	PQLDEV_OSD_CONTROL pQLOSD =(PQLDEV_OSD_CONTROL)p;

	pDevice->m_OSDInfo.mDisplay.CurrentBufferID = pQLOSD->BufferID;
	dramAddr = pDevice->m_OSDInfo.mDisplay.mSheet[pQLOSD->BufferID].StartAddr;
	printk("%s: addr =  %x\n", __FUNCTION__, dramAddr);
	lErr= CDevice_wrreg(pDevice, REG_OSD_START_ADDRESS, (unsigned char*)&dramAddr, 0);

	return lErr;
}

int	qldiag_osd_GetCurrentOSD(CDevice *pDevice,  void *p )
{
	PQLDEV_OSD_CONTROL pQLOSD = (PQLDEV_OSD_CONTROL)p;

	pQLOSD->BufferID = pDevice->m_OSDInfo.mDisplay.CurrentBufferID;
	return pQLOSD->BufferID;
}

#define UPDATE_USER_WINDOW	//copy from (0,y) of whole window
int	qldiag_osd_PasteBuffer(CDevice *pDevice,  void *p)
{
	int Err = 0;
	int i, res;
	int offset, memAddr;
	unsigned char *pMem;
	unsigned int temp1, temp2 = 0;
	PQLDEV_OSD_CONTROL pQLOSD  = (PQLDEV_OSD_CONTROL)p;
	unsigned int reg630;
	unsigned int update_width;

#ifdef CONFIG_QL300_USB_EVK		//none DMA endian setting
	unsigned char *buff = NULL;
#endif

	switch (pDevice->m_OSDInfo.mDisplay.PixelFormat){
		case PIXEL_LUT_8BIT:
		#ifdef UPDATE_USER_WINDOW	//line update
			memAddr = pDevice->m_OSDInfo.mDisplay.mSheet[pQLOSD->BufferID].StartAddr + pQLOSD->y * pDevice->m_OSDInfo.mDisplay.Width;
			pMem = (unsigned char *) pQLOSD->pColorData + pQLOSD->y * pDevice->m_OSDInfo.mDisplay.Width;
			update_width = pDevice->m_OSDInfo.mDisplay.Width;
		#else	//copy from (0, 0) of the update window
			memAddr = pDevice->m_OSDInfo.mDisplay.mSheet[pQLOSD->BufferID].StartAddr + pQLOSD->x + pQLOSD->y * pDevice->m_OSDInfo.mDisplay.Width;
			pMem = (unsigned char *) pQLOSD->pColorData;
			update_width = pQLOSD->Width;
		#endif

			//if Big endian, DMA set big
			//TBD: need protect to make sure the DMA only use here
			if (pQLOSD->swap == 1){
				CDevice_rdreg(pDevice, 0x630, (unsigned char *)&reg630, 0);
				reg630 |= 0x100;
				CDevice_wrreg(pDevice, 0x630, (unsigned char *)&reg630, 0);
			}

			for (i = 0; i < pQLOSD->Height ; i++){
				offset = i * pDevice->m_OSDInfo.mDisplay.Width;
				res = (memAddr + offset) % 4;

				if (res == 0){	//DWORD
			#ifdef CONFIG_QL300_USB_EVK
                    buff  = kmalloc(update_width, GFP_KERNEL);
                    if (buff) {
                        if (pQLOSD->swap == 1)
                            BytesSwap(pMem, buff, update_width);
                        else
                            memcpy(buff, pMem, update_width);

                        Err = CDevice_dmawr(pDevice, memAddr + offset, update_width, buff, 0);
                        kfree(buff);
                    }
                    else
                        Err = -ENOMEM;
			#else
					Err = CDevice_dmawr(pDevice, memAddr + offset, update_width, pMem , 1);
			#endif
				}else{//default Little Endian
					temp2 = 0xffffffff >> (4-res);
					Err = CDevice_rdmem(pDevice, memAddr + offset - res, 4, (unsigned char *)&temp1, 0);
					temp1 &= temp2;
					memcpy(&temp2, pMem, 4 -res);
					temp2 = temp2 << res;
					temp1 = temp1 + temp2;

					Err = CDevice_wrmem(pDevice, memAddr + offset - res, 4, (unsigned char *)&temp1 , 0);
					Err = CDevice_dmawr(pDevice, memAddr + offset + 4 , pQLOSD->Width -res, pMem+res , 1);
				}
				pMem += update_width;
			}

			if (pQLOSD->swap == 1){
				reg630 &= ~0x100;
				CDevice_wrreg(pDevice, 0x630, (unsigned char *)&reg630, 0);
			}
			break;

		default:
			printk("Not support yet\n");
			break;
	}
	return Err;
}

int	qldiag_osd_CopyBuffer(CDevice *pDevice, void *p)
{
	int Err = 0;
	unsigned int *pbuf;
	unsigned int offset, memAddr;
	unsigned int temp1 = 0;
	int i, res;
	PQLDEV_OSD_CONTROL pQLOSD  = (PQLDEV_OSD_CONTROL)p;
	unsigned int reg630;

	switch (pDevice->m_OSDInfo.mDisplay.PixelFormat){
		case PIXEL_LUT_8BIT:
			memAddr = pDevice->m_OSDInfo.mDisplay.mSheet[pQLOSD->BufferID].StartAddr + pQLOSD->x + pQLOSD->y * pDevice->m_OSDInfo.mDisplay.Width;

			pbuf = (unsigned int *)pQLOSD->pColorData;
			//if Big endian, DMA set big
			//TBD: need protect to make sure the DMA only use here
			if (pQLOSD->swap == 1){
				CDevice_rdreg(pDevice, 0x630, (unsigned char *)&reg630, 0);
				reg630 |= 0x1000;
				CDevice_wrreg(pDevice, 0x630, (unsigned char *)&reg630, 0);
			}

			for ( i = 0; i < pQLOSD->Height ; i++ ){
				offset = i * pDevice->m_OSDInfo.mDisplay.Width;
				res = (memAddr + offset) % 4;

				if (res == 0){	//DWORD
					Err = CDevice_dmard(pDevice, memAddr + offset, pQLOSD->Width, (unsigned char *)pbuf, 1);
				}else{//default Little Endian
					Err = CDevice_rdmem(pDevice, memAddr + offset - res, 4, (unsigned char *)&temp1, 0);
					temp1 >>= res;
					memcpy(pbuf, &temp1, 4 -res);
					Err = CDevice_dmard(pDevice, memAddr + offset + (4 -res), pQLOSD->Width -(4-res), (unsigned char *)(pbuf+(4 - res)), 1);
				}
				pbuf += pQLOSD->Width;
			}
			if (pQLOSD->swap == 1){
				reg630 &= ~0x1000;
				CDevice_wrreg(pDevice, 0x630, (unsigned char *)&reg630, 0);
			}
			break;

		default:
			printk("Not support yet\n");
			break;
	}

	return Err;
}


int	qldiag_osd_DrawRect(CDevice *pDevice, void *p)
{
	int lErr = 0;
	unsigned int memAddr;
	unsigned int	offset;
	int res, i = 0;
	PQLDEV_OSD_CONTROL pQLOSD  = (PQLDEV_OSD_CONTROL)p;
	int bDMA = 1;
	unsigned char *pMem;

	switch (pDevice->m_OSDInfo.mDisplay.PixelFormat){
		case PIXEL_LUT_8BIT:
			memAddr = pDevice->m_OSDInfo.mDisplay.mSheet[pDevice->m_OSDInfo.mDisplay.CurrentBufferID].StartAddr + pQLOSD->x + pQLOSD->y * pDevice->m_OSDInfo.mDisplay.Width;
			pMem  = kmalloc(pDevice->m_OSDInfo.mDisplay.Width, GFP_KERNEL);
			memset(pMem, *(pQLOSD->pColorData) & 0xff, pDevice->m_OSDInfo.mDisplay.Width);
			printk("%s: x=%d y=%d w=%d h=%d, color =%d\n", __FUNCTION__, pQLOSD->x, pQLOSD->y, pQLOSD->Width, pQLOSD->Height, *pMem);

			//address should be DWord, checked
			res = memAddr % 4;
			if (res)printk("%s: Warnig:: memory address changed to DWORD!\n", __FUNCTION__);

			for (i = 0; i < pQLOSD->Height ; i++)	{
				offset = i * pDevice->m_OSDInfo.mDisplay.Width ;
				if (bDMA){
					lErr = CDevice_dmawr(pDevice, memAddr+offset - res, pQLOSD->Width, pMem, 0);
				}else{
					lErr = CDevice_wrmem(pDevice, memAddr+offset -res, pQLOSD->Width, pMem , 0);
				}
			}
			kfree(pMem);
			break;

		default:
			printk("%s: Not support pixel format:0x%x \n", __FUNCTION__, pDevice->m_OSDInfo.mDisplay.PixelFormat);
			lErr = ERR_FORMAT_SUPPORT;
			break;
	}

	return lErr;
}

int	qldiag_osd_DrawImage(CDevice *pDevice, void *p)
{
	int lErr = 0;
	int i, res;
	int offset, memAddr;
	PQLDEV_OSD_CONTROL pQLOSD  = (PQLDEV_OSD_CONTROL)p;

	unsigned char *pMem;

	switch (pDevice->m_OSDInfo.mDisplay.PixelFormat){
		case PIXEL_LUT_8BIT:
			memAddr = pDevice->m_OSDInfo.mDisplay.mSheet[pQLOSD->BufferID].StartAddr + pQLOSD->x + pQLOSD->y * pDevice->m_OSDInfo.mDisplay.Width;
			pMem = (unsigned char *) pQLOSD->pColorData;

			res = memAddr % 4;
			if (res)printk("%s: Warnig:: memory address should be DWORD!\n", __FUNCTION__);

			for (i = 0; i < pQLOSD->Height; i++){
				offset = i * pDevice->m_OSDInfo.mDisplay.Width;

				if (1/*pQLOSD->RetCode > 0*/){
					lErr = CDevice_dmawr(pDevice, memAddr+offset, pQLOSD->Width, pMem, 1);
				}else{
					lErr = CDevice_wrmem(pDevice, memAddr+offset, pQLOSD->Width, pMem , 1);
				}
				pMem += pQLOSD->Width;
			}
			break;

		default:
			printk("%s: Not support pixel format:0x%x \n", __FUNCTION__, pDevice->m_OSDInfo.mDisplay.PixelFormat);
			lErr = ERR_FORMAT_SUPPORT;
			break;
	}
	return lErr;
}

int	qldiag_osd_DrawDot(CDevice *pDevice, void *p)
{
	int	lErr = 0;
	unsigned int memAddr;
	int res;
	unsigned int	pixelColor, pxdata;
	PQLDEV_OSD_CONTROL pQLOSD  = (PQLDEV_OSD_CONTROL)p;

	switch (pDevice->m_OSDInfo.mDisplay.PixelFormat){
		case PIXEL_LUT_8BIT:
			memAddr = pDevice->m_OSDInfo.mDisplay.mSheet[pQLOSD->BufferID].StartAddr + (pQLOSD->x + pDevice->m_OSDInfo.mDisplay.Width*pQLOSD->y);

			res = memAddr%4;
			lErr =CDevice_rdmem(pDevice, memAddr -res, 4, (unsigned char*)&pixelColor , 0);

			//
			pxdata = pQLOSD->FgColor & 0xff;
			if (pQLOSD->swap == 1)
				pixelColor = (pxdata << (3-res)) + (pixelColor & (~(0xff << (3-res))));
			else
				pixelColor = (pxdata << res) + (pixelColor & (~(0xff << res)));

			lErr = CDevice_wrmem(pDevice, memAddr -res, 4,(unsigned char*) &pixelColor , 0);
			break;

		default:
			printk("%s: Not support pixel format:0x%x \n", __FUNCTION__, pDevice->m_OSDInfo.mDisplay.PixelFormat);
			lErr = ERR_FORMAT_SUPPORT;
			break;
	}
	return lErr;
}

int	qldiag_osd_DrawText(CDevice *pDevice, void *p)
{
	int Err = 0;
	int i,j, res;
	int offset, memAddr;
	int curWidth;

	unsigned char *pMem;
	unsigned char cVal;
	unsigned int	pixelColor, pxdata;
	COLOR_DATA oriColor;
	PQLDEV_OSD_CONTROL pQLOSD  = (PQLDEV_OSD_CONTROL)p;

	switch (pDevice->m_OSDInfo.mDisplay.PixelFormat){
		case PIXEL_LUT_8BIT:
			memAddr = pDevice->m_OSDInfo.mDisplay.mSheet[pQLOSD->BufferID].StartAddr + pQLOSD->x + pQLOSD->y * pDevice->m_OSDInfo.mDisplay.Width;
			pMem = pQLOSD->pFontData;

			for (i = 0; i < pQLOSD->Height ; i++){
				offset = i * pDevice->m_OSDInfo.mDisplay.Width;
				curWidth = 0;
				while(curWidth < pQLOSD->Width ){
					cVal = *pMem;
					for(j = 8; j > 0; j--){
						res = (memAddr + offset)%4;
						if (pQLOSD->bUpdateTransparency > 0 ){
							//TBD: not real color!
							Err =CDevice_rdmem(pDevice, memAddr + offset - res, 4, (unsigned char*)&pixelColor , 0);
							oriColor.Read = pixelColor;
							pixelColor = (cVal & (0x00000001<<(j-1))) ? pQLOSD->FgColor : oriColor.Read;
						}else{
							pixelColor = (cVal & (0x00000001<<(j-1))) ? pQLOSD->FgColor : pQLOSD->BgColor;
						}

						pxdata = pixelColor & 0xff;
						Err =CDevice_rdmem(pDevice, memAddr + offset -res, 4, (unsigned char*)&pixelColor , 0);

						if (pQLOSD->swap == 1)
							pixelColor = (pxdata << (3-res)) + (pixelColor & (~(0xff << (3-res))));
						else
							pixelColor = (pxdata << res) + (pixelColor & (~(0xff << res)));

						Err = CDevice_wrmem(pDevice, memAddr + offset -res, 4, (unsigned char*) &pixelColor , 0);
						offset ++;
						curWidth++;

						if (curWidth == pQLOSD->Width)break;
					}
					pMem ++;
				}
			}
			break;

		default:
			printk("%s: Not support pixel format:0x%x \n", __FUNCTION__, pDevice->m_OSDInfo.mDisplay.PixelFormat);
			Err = ERR_FORMAT_SUPPORT;
			break;
	}

	return Err;
}
#endif

#ifdef SUPPORT_VI_OSD
int qldiag_viosd_TimeStampEnable(CDevice *pDevice, void *p)
{
	int lErr = 0;
	unsigned int value = *((unsigned int *)p);

	pDevice->m_OSDInfo.mViosdDisplay.TimeStampEnable = value;
	pDevice->m_OSDInfo.mViosdDisplay.Update = 1;
	return lErr;
}

int qldiag_viosd_TimeStampFontType(CDevice *pDevice, void *p)
{
	int lErr = 0;
	unsigned int value = *((unsigned int *)p);

	pDevice->m_OSDInfo.mViosdDisplay.Update = 1;
	if(value == 0)
	{
		pDevice->m_OSDInfo.mViosdDisplay.TimeStampFontType = 1;
	}
	else if(value == 1)
	{
		pDevice->m_OSDInfo.mViosdDisplay.TimeStampFontType = 3;
	}
	else
	{
		pDevice->m_OSDInfo.mViosdDisplay.Update = 0;
	}
	return lErr;
}

int qldiag_viosd_TimeStampXPosition(CDevice *pDevice, void *p)
{
	int lErr = 0;
	unsigned int value = *((unsigned int *)p);

	pDevice->m_OSDInfo.mViosdDisplay.TimeStampXPosition = value;
	return lErr;
}

int qldiag_viosd_TimeStampYPosition(CDevice *pDevice, void *p)
{
	int lErr = 0;
	unsigned int value = *((unsigned int *)p);

	pDevice->m_OSDInfo.mViosdDisplay.TimeStampYPosition = value;
	pDevice->m_OSDInfo.mViosdDisplay.Update = 1;
	return lErr;
}

int qldiag_viosd_TimeStringEnable(CDevice *pDevice, void *p)
{
	int lErr = 0;
	unsigned int value = *((unsigned int *)p);

	pDevice->m_OSDInfo.mViosdDisplay.TestStringEnable = value;
	pDevice->m_OSDInfo.mViosdDisplay.Update = 1;
	return lErr;
}

int qldiag_viosd_TimeStringFontType(CDevice *pDevice, void *p)
{
	int lErr = 0;
	unsigned int value = *((unsigned int *)p);

	pDevice->m_OSDInfo.mViosdDisplay.Update = 1;
	if(value == 0)
	{
		pDevice->m_OSDInfo.mViosdDisplay.TestStringFontType = 1;
	}
	else if(value == 1)
	{
		pDevice->m_OSDInfo.mViosdDisplay.TestStringFontType = 3;
	}
	else
	{
		pDevice->m_OSDInfo.mViosdDisplay.Update = 0;
	}
	return lErr;
}

int qldiag_viosd_TimeStringXPosition(CDevice *pDevice, void *p)
{
	int lErr = 0;
	unsigned int value = *((unsigned int *)p);

	pDevice->m_OSDInfo.mViosdDisplay.TestStringXPosition = value;
	pDevice->m_OSDInfo.mViosdDisplay.Update = 1;
	return lErr;
}

int qldiag_viosd_TimeStringYPosition(CDevice *pDevice, void *p)
{
	int lErr = 0;
	unsigned int value = *((unsigned int *)p);

	pDevice->m_OSDInfo.mViosdDisplay.TestStringYPosition = value;
	pDevice->m_OSDInfo.mViosdDisplay.Update = 1;
	return lErr;
}

int qldiag_viosd_TimeString(CDevice *pDevice, void *p)
{
	int lErr = 0;
	//unsigned int temp;
	PQLDEV_VIOSD_CONTROL pViOsd = (PQLDEV_VIOSD_CONTROL )p;

	strncpy(pDevice->m_OSDInfo.mViosdDisplay.TestString, pViOsd->pString, 32);


	pDevice->m_OSDInfo.mViosdDisplay.Update = 1;
	return lErr;
}

#define	MAX_OSD_LEN 128
int qldiag_viosd_control(CDevice *pDevice, void *p)
{
	PQPCODEC_VIOSD param = (PQPCODEC_VIOSD )p;
	int lErr = 0;

	switch(param->CmdType)
	{
		case PARAM_VIOSD_ON:
			lErr = qldiag_viosd_On(pDevice, param);
			break;
		case PARAM_VIOSD_OFF:
			lErr = qldiag_viosd_Off(pDevice, param);
			break;
		case PARAM_VIOSD_INIT:
			lErr = qldiag_viosd_Init(pDevice, param);
			break;
		case PARAM_VIOSD_CLEAR:
			lErr = qldiag_viosd_clearbuf(pDevice, param);
			break;
		case PARAM_VIOSD_ADDFONT:
			lErr = qldiag_viosd_addfont(pDevice, param);
			break;
		case PARAM_VIOSD_ADDTEXT:
			lErr = qldiag_viosd_addtext(pDevice, param);
			break;
		case PARAM_VIOSD_SETTEXT:
			lErr = qldiag_viosd_settext(pDevice, param);
			break;
		case PARAM_VIOSD_DELTEXT:
			lErr = qldiag_viosd_deltext(pDevice, param);
			break;
		case PARAM_VIOSD_SETTIME:
			lErr = qldiag_viosd_settime(pDevice, param);
			break;
		case PARAM_VIOSD_UPDATETIME:
			lErr = qldiag_viosd_updatetime(pDevice, param);
			break;
		case PARAM_VIOSD_INFO:
			lErr = qldiag_viosd_GetInfo(pDevice, param);
			break;
	}
	return lErr;
}

int qldiag_viosd_On(CDevice *pDevice, void *p)
{
	PQPCODEC_VIOSD param = (PQPCODEC_VIOSD)p;
	if(pDevice->m_OSDInfo.mViosdON == 0)
	{
		pDevice->m_OSDInfo.mViosdON = 1;
		param->tIdx = 0;
	}
	return 0;
}

int qldiag_viosd_Off(CDevice *pDevice, void *p)
{
	pDevice->m_OSDInfo.mViosdON = 0;
	//printk("qldiag_viosd_Off\n");
	return 0;
}

int	qldiag_viosd_Init(CDevice *pDevice, void *p)
{
	pDevice->m_OSDInfo.mViosdDisplay.TimeStampEnable = 0;
	pDevice->m_OSDInfo.mViosdDisplay.TimeStampFontType = 1;
	pDevice->m_OSDInfo.mViosdDisplay.TimeStampXPosition = 40;
	pDevice->m_OSDInfo.mViosdDisplay.TimeStampYPosition = 70;
	pDevice->m_OSDInfo.mViosdDisplay.TestStringEnable = 0;
	pDevice->m_OSDInfo.mViosdDisplay.TestStringFontType = 1;
	pDevice->m_OSDInfo.mViosdDisplay.TestStringXPosition = 40;
	pDevice->m_OSDInfo.mViosdDisplay.TestStringYPosition = 40;
	pDevice->m_OSDInfo.mViosdDisplay.TestString[0] = 'S';
	pDevice->m_OSDInfo.mViosdDisplay.TestString[1] = 'i';
	pDevice->m_OSDInfo.mViosdDisplay.TestString[2] = 'g';
	pDevice->m_OSDInfo.mViosdDisplay.TestString[3] = 'm';
	pDevice->m_OSDInfo.mViosdDisplay.TestString[4] = 'a';
	pDevice->m_OSDInfo.mViosdDisplay.TestString[5] = 'D';
	pDevice->m_OSDInfo.mViosdDisplay.TestString[6] = 'e';
	pDevice->m_OSDInfo.mViosdDisplay.TestString[7] = 's';
	pDevice->m_OSDInfo.mViosdDisplay.TestString[8] = 'i';
	pDevice->m_OSDInfo.mViosdDisplay.TestString[9] = 'g';
	pDevice->m_OSDInfo.mViosdDisplay.TestString[10] = 'n';
	pDevice->m_OSDInfo.mViosdDisplay.TestString[11] = 's';
	pDevice->m_OSDInfo.mViosdDisplay.TestString[12] = 0;
	//printk("%s: qldiag_viosd_Init\n", __FUNCTION__);
	return 0;
}

int qldiag_viosd_clearbuf(CDevice *pDevice, void *p)
{
    PQPCODEC_VIOSD param = (PQPCODEC_VIOSD)p;
    unsigned int dwFontSize = MAX_FONT_NUM*((128*16*16*3>>1)+2*4)>>2;
    unsigned int dwTextSize = MAX_TEXT_NUM*(2*4+128)>>2;
    unsigned int DummyBufferSize = 1024;
    unsigned int ClearBufferSize = 0,dwBufferOffset = 0;
    int Err = 0;

    char *fontbuff= 0;

    param->Timestamp.dwTimeTextID = MAX_TEXT_NUM;

    if (param->TextList.aText != NULL)
    {
        memset(param->TextList.aText, 0, sizeof(TQLText)*MAX_TEXT_NUM);
    }
    if (param->FontList.aFont != NULL)
    {
        memset(param->FontList.aFont, 0, sizeof(TQLFont)*MAX_FONT_NUM);
    }
    param->FontList.dwFontOffset = MAX_FONT_NUM*2*4;

	fontbuff= kmalloc(DummyBufferSize, GFP_KERNEL);
	memset(fontbuff,0,DummyBufferSize);

	dwFontSize <<= 2;
	dwTextSize <<= 2;
	dwBufferOffset = 0;
	while(dwFontSize > 0)
	{
		if (dwFontSize >= DummyBufferSize)
			ClearBufferSize = DummyBufferSize;
		else
			ClearBufferSize = dwFontSize;

		Err = CDevice_dmawr(pDevice, param->FontTableAddr + dwBufferOffset, ClearBufferSize, fontbuff, 0);

		dwFontSize -= ClearBufferSize;
		dwBufferOffset += ClearBufferSize;
	}

	dwBufferOffset = 0;
	while(dwTextSize > 0)
	{
		if (dwTextSize >= DummyBufferSize)
			ClearBufferSize = DummyBufferSize;
		else
			ClearBufferSize = dwTextSize;

		Err = CDevice_dmawr(pDevice, param->TextListAddr + dwBufferOffset, ClearBufferSize, fontbuff, 0);

		dwTextSize -= ClearBufferSize;
		dwBufferOffset += ClearBufferSize;
	}

	kfree(fontbuff);
	fontbuff= 0;
    return 0;
}

int qldiag_viosd_addfont(CDevice *pDevice, void *p)
{
    PQPCODEC_VIOSD param = (PQPCODEC_VIOSD)p;
    TQLFont *ptFont;
    unsigned int pdwFontPpt = param->FontTableAddr;
    unsigned int dwIdx = 0;
    unsigned int dwWidth, dwHeight, dwSize,dwTemp, dwWrPos,dwLen;
    unsigned char *buf;
    int Err = 0;


    ptFont = &(param->FontList.aFont[param->tIdx]);
    dwIdx = ptFont->dwID;

    dwTemp = param->FontList.dwFontOffset;
    Err = CDevice_wrmem(pDevice, (unsigned int) (pdwFontPpt + 4*(dwIdx*2+1)), 4, (unsigned char *)&dwTemp, 0);

    dwWidth = (ptFont->dwType&0x02) ? 16 : 8;
    dwHeight = (ptFont->dwType&0x01) ? 16 : 8;
    dwSize = dwWidth*dwHeight*ptFont->dwLen*3>>1;

	buf = kmalloc(PAGE_SIZE, GFP_KERNEL);
	if(!buf) {
		return -1;
	}
	for(dwWrPos=0;dwWrPos<dwSize;dwWrPos+=PAGE_SIZE/4)
	{
		dwLen = ((dwSize-dwWrPos)>=PAGE_SIZE)?PAGE_SIZE:(dwSize-dwWrPos);
		memcpy(buf,ptFont->pbyFont+dwWrPos,dwLen);
    	Err = CDevice_dmawr(pDevice, (param->FontTableAddr + param->FontList.dwFontOffset)+dwWrPos, dwLen, buf, 0);
    }

    param->FontList.dwFontOffset = param->FontList.dwFontOffset + dwSize;

	dwTemp = 	(ptFont->dwID<<28) |
            	(ptFont->dwLen<<16) |
            	(ptFont->dwAsciiOffset<<8) |
            	0x80000000 |
            	ptFont->dwType;

	//printk("addfont: write %x to 0x%08x\n", dwTemp,  (pdwFontPpt + (dwIdx*2)));
    Err = CDevice_wrmem(pDevice, (unsigned int) (pdwFontPpt + 4*(dwIdx*2)), 4, (unsigned char *)&dwTemp, 0);

	kfree(buf);

    return Err;
}

int qldiag_viosd_addtext(CDevice *pDevice, void *p)
{
    PQPCODEC_VIOSD param = (PQPCODEC_VIOSD)p;
    TQLText *ptText;
    unsigned int pdwTextPpt = param->TextListAddr;
    unsigned int dwIdx = 0,dwTemp = 0;
    unsigned int i;
    unsigned char *drvbuff = 0;
    int Err;

    ptText = &(param->TextList.aText[param->tIdx]);
    dwIdx = ptText->dwID;
    dwTemp = (ptText->dwY<<16) | (ptText->dwX);
	Err = CDevice_wrmem(pDevice, (unsigned int) (pdwTextPpt + 4*(34*dwIdx+1)), 4, (unsigned char *)&dwTemp, 0);

	drvbuff= kmalloc( MAX_OSD_LEN, GFP_KERNEL);

    copy_from_user(drvbuff,ptText->pbyText,MAX_OSD_LEN);

    //Set text string
    for (i=ptText->dwLen; i<(((ptText->dwLen+3)>>2)<<2); i++)
    {
        //Append to DWORD boundary
        drvbuff[i] = 0;
    }
    for (i=0; i<((ptText->dwLen+3)>>2); i++)
    {
    		dwTemp = ((unsigned int *)drvbuff)[i];
    		Err = CDevice_wrmem(pDevice, (unsigned int) (pdwTextPpt + 4*(34*dwIdx+2+i)), 4, (unsigned char *)&dwTemp, 0);
    }

    kfree(drvbuff);

    //Set text properties
    dwTemp = (ptText->bEn<<31) |
             (ptText->dwFontID<<28) |
             (ptText->dwLen<<16) |
             (param->FontList.aFont[ptText->dwFontID].dwType);

    Err = CDevice_wrmem(pDevice, (unsigned int) (pdwTextPpt + 4*(34*dwIdx)), 4, (unsigned char *)&dwTemp, 0);
    return Err;
}

int qldiag_viosd_settext(CDevice *pDevice, void *p)
{
    PQPCODEC_VIOSD param = (PQPCODEC_VIOSD)p;
    TQLText *ptText;
    unsigned int pdwTextPpt = param->TextListAddr;
    unsigned int dwIdx,dwTemp = 0;
    unsigned int i;
    unsigned char *drvbuff = 0;
    int Err;

    ptText = &(param->TextList.aText[param->tIdx]);

    dwIdx = ptText->dwID;
    //Disable text first
    Err = CDevice_dmard(pDevice, (unsigned int)(pdwTextPpt + 4*(34*dwIdx)), 4, (unsigned char *)&dwTemp, 0);
    dwTemp = 0x7FFFFFFF & dwTemp;
    Err = CDevice_wrmem(pDevice, (unsigned int)(pdwTextPpt + 4*(34*dwIdx)), 4, (unsigned char *)&dwTemp, 0);

	dwTemp = (ptText->dwY<<16) | (ptText->dwX);
	Err = CDevice_wrmem(pDevice, (unsigned int) (pdwTextPpt + 4*(34*dwIdx+1)), 4, (unsigned char *)&dwTemp, 0);

	drvbuff= kmalloc( MAX_OSD_LEN, GFP_KERNEL);

    copy_from_user(drvbuff,ptText->pbyText,MAX_OSD_LEN);

    //Set text string
    for (i=ptText->dwLen; i<(((ptText->dwLen+3)>>2)<<2); i++)
    {
        //Append to DWORD boundary
        drvbuff[i] = 0;
    }
    for (i=0; i<((ptText->dwLen+3)>>2); i++)
    {
        dwTemp = ((unsigned int *)(drvbuff))[i];
		Err = CDevice_wrmem(pDevice, (unsigned int) (pdwTextPpt + 4*(34*dwIdx+2+i)), 4, (unsigned char *)&dwTemp, 0);
    }

	kfree(drvbuff);

    //Set text properties
    dwTemp = (ptText->bEn<<31) |
             (ptText->dwFontID<<28) |
             (ptText->dwLen<<16) |
             (param->FontList.aFont[ptText->dwFontID].dwType);

    Err = CDevice_wrmem(pDevice, (unsigned int) (pdwTextPpt + 4*(34*dwIdx)), 4, (unsigned char *)&dwTemp, 0);
    return Err;
}

int qldiag_viosd_deltext(CDevice *pDevice, void *p)
{
    PQPCODEC_VIOSD param = (PQPCODEC_VIOSD)p;
    unsigned int dwIdx = param->tIdx;
    unsigned int pdwTextPpt = param->TextListAddr;
    unsigned int dwTemp = 0;
    int Err;

//    qprintk(4,"delete osd text\n");

    if (param->TextList.aText[dwIdx].dwLen != 0)
    {
        param->TextList.aText[dwIdx].dwLen = 0;
    }
    else
    {
        printk("Text len is zero\n");
        return -1;
    }
    param->TextList.dwIndex = dwIdx;
    dwTemp = 0;
    Err = CDevice_wrmem(pDevice, (unsigned int) (pdwTextPpt + 4*(34*dwIdx)), 4, (unsigned char *)&dwTemp, 0);
    return Err;
}

int qldiag_viosd_settime(CDevice *pDevice, void *p)
{
    PQPCODEC_VIOSD param = (PQPCODEC_VIOSD)p;
    QPCODEC_VIOSD_CALTIME *ptTime;
    unsigned int pdwTextPpt;
    unsigned int i,dwTemp = 0;
    unsigned char *drvbuff = 0;
    int Err;

    ptTime = &(param->CalTime);

    pdwTextPpt = param->TextListAddr + 4*(34*(MAX_TEXT_NUM-2+param->Timestamp.dwTimeTextID));

    //Disable text first
    Err = CDevice_dmard(pDevice, (unsigned int) (pdwTextPpt), 4, (unsigned char *)&dwTemp, 0);
    dwTemp = 0x7FFFFFFF & dwTemp;
    Err = CDevice_wrmem(pDevice, (unsigned int) (pdwTextPpt), 4, (unsigned char *)&dwTemp, 0);

    //Set text position
    dwTemp = (ptTime->dwY<<16) | (ptTime->dwX);
    Err = CDevice_wrmem(pDevice, (unsigned int) (pdwTextPpt + (4*1)), 4, (unsigned char *)&dwTemp, 0);

    //Set timestamp
    dwTemp = ptTime->dwTime;
    Err = CDevice_wrmem(pDevice, (unsigned int) (pdwTextPpt + (4*2)), 4, (unsigned char *)&dwTemp, 0);

    drvbuff= kmalloc( MAX_OSD_LEN, GFP_KERNEL);

    copy_from_user(drvbuff,ptTime->pbyTime,MAX_OSD_LEN);

    //Set text string
    for (i=ptTime->dwLen; i<(((ptTime->dwLen+3)>>2)<<2); i++)
    {
        //Append to DWORD boundary
        drvbuff[i] = 0;
    }
    for (i=0; i<((ptTime->dwLen+3)>>2); i++)
    {
        dwTemp = ((unsigned int *)drvbuff)[i];
        Err = CDevice_wrmem(pDevice, (unsigned int)(pdwTextPpt + 4*(3+i)), 4, (unsigned char *)&dwTemp, 0);
    }

    kfree(drvbuff);

    //Set text properties
    dwTemp = (ptTime->bEnable<<31) |
	            (ptTime->bMSEn<<24) |
	            (ptTime->dwFontID<<28) |
	            (ptTime->dwLen<<16) |
	            (ptTime->byMSSep<<8) |
	            (param->FontList.aFont[ptTime->dwFontID].dwType);

    Err = CDevice_wrmem(pDevice, (unsigned int)(pdwTextPpt), 4, (unsigned char *)&dwTemp, 0);
    return Err;
}

int qldiag_viosd_updatetime(CDevice *pDevice, void *p)
{
    PQPCODEC_VIOSD param = (PQPCODEC_VIOSD)p;
    QPCODEC_VIOSD_CALTIME *ptCalTime = &(param->CalTime);
    unsigned int pdwTimePpt = param->TimestampAddr;
    unsigned int dwTemp = 0;
    int Err;

    dwTemp = (ptCalTime->bSwitch<<24 | ptCalTime->bTimerSec<<16 | ptCalTime->bTimerMilliSec);
    Err = CDevice_wrmem(pDevice, (unsigned int)(pdwTimePpt), 4, (unsigned char *)&dwTemp, 0);
    return Err;
}

int qldiag_viosd_GetInfo(CDevice *pDevice, void *p)
{
	int lErr = 0;
	unsigned int dwIdx;
	int iXSft, iYSft;
	PQPCODEC_VIOSD pQLVIOSD = (PQPCODEC_VIOSD)p;
	TQLTime *ptTime = &(pQLVIOSD->Timestamp);
	TQLText *ptText = &(pQLVIOSD->Text);
	TQLText *ptDevText;

	pQLVIOSD->Update = pDevice->m_OSDInfo.mViosdDisplay.Update;
	ptTime->bEn = pDevice->m_OSDInfo.mViosdDisplay.TimeStampEnable;
	if(pDevice->m_OSDInfo.mViosdDisplay.TimeStampFontType == 1)
		ptTime->dwFontID = 0;
	else
	if(pDevice->m_OSDInfo.mViosdDisplay.TimeStampFontType == 3)
		ptTime->dwFontID = 1;
	iXSft = (pDevice->m_OSDInfo.mViosdDisplay.TimeStampFontType<2) ? 3 : 4;
	iYSft = (pDevice->m_OSDInfo.mViosdDisplay.TimeStampFontType&0x01) ? 4: 3;
	ptTime->dwX = pDevice->m_OSDInfo.mViosdDisplay.TimeStampXPosition>>iXSft;
	ptTime->dwY = pDevice->m_OSDInfo.mViosdDisplay.TimeStampYPosition>>iYSft;

	ptText->bEn = pDevice->m_OSDInfo.mViosdDisplay.TestStringEnable;
	if(pDevice->m_OSDInfo.mViosdDisplay.TestStringFontType == 1)
		ptText->dwFontID = 0;
	else
	if(pDevice->m_OSDInfo.mViosdDisplay.TestStringFontType == 3)
		ptText->dwFontID = 1;
	iXSft = (pDevice->m_OSDInfo.mViosdDisplay.TestStringFontType<2) ? 3 : 4;
	iYSft = (pDevice->m_OSDInfo.mViosdDisplay.TestStringFontType&0x01) ? 4: 3;
	ptText->dwX = pDevice->m_OSDInfo.mViosdDisplay.TestStringXPosition>>iXSft;
	ptText->dwY = pDevice->m_OSDInfo.mViosdDisplay.TestStringYPosition>>iYSft;
	for(dwIdx = 0; dwIdx<32; dwIdx++)
	{
		ptText->pbyText[dwIdx] = pDevice->m_OSDInfo.mViosdDisplay.TestString[dwIdx];
	}
	ptText->pbyText[31] = '\n';
	ptText->dwLen = strlen(pDevice->m_OSDInfo.mViosdDisplay.TestString);
	pQLVIOSD->Enable = ptText->bEn|ptTime->bEn;

	if(pQLVIOSD->Enable == 0)
	{
		for(dwIdx = 0; dwIdx<MAX_TEXT_NUM-2; dwIdx++)
		{
			ptDevText = &(pQLVIOSD->TextList.aText[dwIdx]);
			if(ptDevText->bEn)
			{
				pQLVIOSD->Enable = 1;
				break;
			}
		}
	}
	//printk("%s: qldiag_viosd_GetInfo\n", __FUNCTION__);
	pDevice->m_OSDInfo.mViosdDisplay.Update = 0;
	return lErr;
}

#endif

#endif

