/*************************************************************************/
/**      **/
/**            **/
/**                **/
/**              **/
/**                                                                     **/
/**    **/
/**                  **/
/**                                                                     **/
/**                               **/
/**                                                  **/
/**  Changed by Frank.Wei for QL3xx, base on QL2XX          			**/
/*************************************************************************/
#include "qlautoconf.h"

#ifndef CONFIG_HOST_PLATFORM_USERSPACE
#include <linux/sched.h>
#include <linux/delay.h>
#include <linux/version.h>
#include <linux/vmalloc.h>
#else
#include <stdlib.h>
#endif

#include "qptypes.h"
#include "pldevioctl.h"

#include "cdevice.h"

#include "qlvideoprop.h"

long qldiag_videoprop_set(CDevice *pDevice, void *p)
{
	long lErr = 0;
	QLDEV_VIDEOPROP_PARAMETER *param = (QLDEV_VIDEOPROP_PARAMETER *)p;
	IVideoDecoder *pVideoDecoder = CDevice_getVideoDecoder(pDevice);
	if (param==NULL)
		return -1;
	if (pVideoDecoder)
	{
		switch ( param->type )
		{
			case PROP_SET_POWERLINEFREQ:
				lErr = pVideoDecoder->SetPowerLineFreq(pVideoDecoder, param->value);
				break;
			case PROP_SET_BRIGHTNESS:
				lErr = pVideoDecoder->SetBrightness(pVideoDecoder, param->value);
				break;
			case PROP_SET_CONTRAST:
				lErr = pVideoDecoder->SetContrast(pVideoDecoder, param->value);
				break;
			case PROP_SET_HUE:
				lErr = pVideoDecoder->SetHue(pVideoDecoder, param->value);
				break;
			case PROP_SET_SATURATION:
				lErr = pVideoDecoder->SetSaturation(pVideoDecoder, param->value);
				break;
			case PROP_SET_SHARPNESS:
				lErr = pVideoDecoder->SetSharpness(pVideoDecoder, param->value);
				break;
			case PROP_SET_GAMMA:
				lErr = pVideoDecoder->SetGamma(pVideoDecoder, param->value);
				break;
			case PROP_SET_AUTOWHITEBALANCEENABLED:
				lErr = pVideoDecoder->SetAutoWhiteBalanceEnabled(pVideoDecoder, param->value);
				break;
			case PROP_SET_WHITEBALANCE:
				lErr = pVideoDecoder->SetWhiteBalance(pVideoDecoder, param->value);
				break;
			case PROP_SET_LOWLIGHTBOOST:
				lErr = pVideoDecoder->SetLowLightBoost(pVideoDecoder, param->value);
				break;
			case PROP_SET_AUTOFOCUSENABLED:
				lErr = pVideoDecoder->SetAutoFocusEnabled(pVideoDecoder, param->value);
				break;
			case PROP_SET_FOCUS:
				lErr = pVideoDecoder->SetFocus(pVideoDecoder, param->value);
				break;
			case PROP_SET_AUTOEXPOSUREENABLED:
				lErr = pVideoDecoder->SetAutoExposureEnabled(pVideoDecoder, param->value);
				break;
			case PROP_SET_EXPOSURE:
				lErr = pVideoDecoder->SetExposure(pVideoDecoder, param->value);
				break;
			case PROP_SET_AUTOEXPOSUREPRIORITY:
				lErr = pVideoDecoder->SetAutoExposurePriority(pVideoDecoder, param->value);
				break;
			default:
				lErr = -1;
		}
	}
	return lErr;
}

long qldiag_videoprop_get(CDevice *pDevice, void *p)
{
	long lErr = 0;
	TQP_UINT32 nEnable=0xFFFF;
	int nDefault=0, nMin=0, nMax=0;

	QLDEV_VIDEOPROP_PARAMETER *param = (QLDEV_VIDEOPROP_PARAMETER *)p;
	IVideoDecoder *pVideoDecoder = CDevice_getVideoDecoder(pDevice);
	if (param==NULL)
		return -1;
	if (pVideoDecoder)
	{
		switch ( param->type )
		{
			case PROP_GET_POWERLINEFREQ:
				lErr = pVideoDecoder->GetPowerLineFreq(pVideoDecoder, &nDefault);
				break;
			case PROP_GET_BRIGHTNESS:
				lErr = pVideoDecoder->GetBrightness(pVideoDecoder, &nDefault);
				break;
			case PROP_GET_CONTRAST:
				lErr = pVideoDecoder->GetContrast(pVideoDecoder, &nDefault);
				break;
			case PROP_GET_HUE:
				lErr = pVideoDecoder->GetHue(pVideoDecoder, &nDefault);
				break;
			case PROP_GET_SATURATION:
				lErr = pVideoDecoder->GetSaturation(pVideoDecoder, &nDefault);
				break;
			case PROP_GET_SHARPNESS:
				lErr = pVideoDecoder->GetSharpness(pVideoDecoder, &nDefault);
				break;
			case PROP_GET_GAMMA:
				lErr = pVideoDecoder->GetGamma(pVideoDecoder, &nDefault);
				break;
			case PROP_GET_AUTOWHITEBALANCEENABLED:
				lErr = pVideoDecoder->GetAutoWhiteBalanceEnabled(pVideoDecoder, &nEnable);
				break;
			case PROP_GET_WHITEBALANCE:
				lErr = pVideoDecoder->GetWhiteBalance(pVideoDecoder, &nDefault);
				break;
			case PROP_GET_LOWLIGHTBOOST:
				lErr = pVideoDecoder->GetLowLightBoost(pVideoDecoder, &nDefault);
				break;
			case PROP_GET_AUTOFOCUSENABLED:
				lErr = pVideoDecoder->GetAutoFocusEnabled(pVideoDecoder, &nEnable);
				break;
			case PROP_GET_FOCUS:
				lErr = pVideoDecoder->GetFocus(pVideoDecoder, &nDefault);
				break;
			case PROP_GET_AUTOEXPOSUREENABLED:
				lErr = pVideoDecoder->GetAutoExposureEnabled(pVideoDecoder, &nEnable);
				break;
			case PROP_GET_EXPOSURE:
				lErr = pVideoDecoder->GetExposure(pVideoDecoder, &nDefault);
				break;
			case PROP_GET_AUTOEXPOSUREPRIORITY:
				lErr = pVideoDecoder->GetAutoExposurePriority(pVideoDecoder, &nDefault);
				break;

			case PROP_GET_BRIGHTNESSRANGE:
				lErr = pVideoDecoder->GetBrightnessRange(pVideoDecoder, &nDefault, &nMin, &nMax);
				break;
			case PROP_GET_CONTRASTRANGE:
				lErr = pVideoDecoder->GetContrastRange(pVideoDecoder, &nDefault, &nMin, &nMax);
				break;
			case PROP_GET_SATURATIONRANGE:
				lErr = pVideoDecoder->GetSaturationRange(pVideoDecoder, &nDefault, &nMin, &nMax);
				break;
			case PROP_GET_SHARPNESSRANGE:
				lErr = pVideoDecoder->GetSharpnessRange(pVideoDecoder, &nDefault, &nMin, &nMax);
				break;
			case PROP_GET_GETGAMMARANGE:
				lErr = pVideoDecoder->GetGammaRange(pVideoDecoder, &nDefault, &nMin, &nMax);
				break;
			case PROP_GET_WHITEBALANCERANGE:
				lErr = pVideoDecoder->GetWhiteBalanceRange(pVideoDecoder, &nDefault, &nMin, &nMax);
				break;
			case PROP_GET_LOWLIGHTBOOSTRANGE:
				lErr = pVideoDecoder->GetLowLightBoostRange(pVideoDecoder, &nDefault, &nMin, &nMax);
				break;
			case PROP_GET_POWERLINEFREQRANGE:
				lErr = pVideoDecoder->GetPowerLineFreqRange(pVideoDecoder, &nDefault, &nMin, &nMax);
				break;
			case PROP_GET_FOCUSRANGE:
				lErr = pVideoDecoder->GetFocusRange(pVideoDecoder, &nDefault, &nMin, &nMax);
				break;
			case PROP_GET_EXPOSURERANGE:
				lErr = pVideoDecoder->GetExposureRange(pVideoDecoder, &nDefault, &nMin, &nMax);
				break;
			case PROP_GET_AUTOEXPOSUREPRIORITYRANGE:
				lErr = pVideoDecoder->GetAutoExposurePriorityRange(pVideoDecoder, &nDefault, &nMin, &nMax);
				break;

			default:
				lErr = -1;
		}
		if (nEnable!=0xFFFF)
			param->value = nEnable;
		else
			param->value = nDefault;
		param->min = nMin;
		param->max = nMax;
	}
	return lErr;
}

