///////////////////////////////////////////////////////////////////////////////
//        
//
// Qplinuxkermap.c
//
// Description: 
//
//  This is the mapping between QP status and Linux kernel status
// 
// History:
//	Jan 25, 2010	(dt) created
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#include "qplinuxkermap.h"

long QpToLinuxKerStatus(EQP_Errors qpStatus)
{
	long	lErr;

	switch (qpStatus)
	{
		case QPERR_SUCCESS:
			lErr = 0;
			break;
		case QPERR_PENDING:
			lErr = 0;
			break;
		case QPERR_FAIL:
			lErr = -EIO;
			break;
		case QPERR_NOMEM:
			lErr = -ENOMEM;
			break;
		case QPERR_PARMS:
			lErr = -EINVAL;
			break;
		case QPERR_NOTIMPL:
			lErr = -EPERM;
			break;
		case QPERR_NOTSUPP:
			lErr = -EPERM;
			break;
		case QPERR_TIMEOUT:
			lErr = -EBUSY;
			break;
		case QPERR_INVALID:
			lErr = -EPERM;
			break;
		case QPERR_CANCELLED:
			lErr = -EBUSY;
			break;
		case QPERR_NOTFOUND:
			lErr = -ENODEV;
			break;
		case QPERR_AGAIN:
			lErr = -EAGAIN;
			break;
		default:
			lErr = -EIO;
			break;
	}

	return (lErr);
}


EQP_Errors LinuxKerToQpStatus(long lErr)
{
	EQP_Errors	qpStatus;

	switch (lErr)
	{
		case 0:
			qpStatus = QPERR_SUCCESS;
			break;
		case -EIO:
			qpStatus = QPERR_FAIL;
			break;
		case -ENOMEM:
			qpStatus = QPERR_NOMEM;
			break;
		case -EINVAL:
			qpStatus = QPERR_PARMS;
			break;
		case -EPERM:
			qpStatus = QPERR_NOTIMPL;
			break;
		case -EBUSY:
			qpStatus = QPERR_TIMEOUT;
			break;
		case -ENODEV:
			qpStatus = QPERR_NOTFOUND;
			break;
		default:
			qpStatus = QPERR_FAIL;
			break;
	}

	return (qpStatus);
}




