///////////////////////////////////////////////////////////////////////////////
//    
//
// drventry.c
//
// Description:
//     pl330 soc driver entry point
//
// History:
//     Aug 3, 2011 created
//
// Authors:
//     (jkglee) Jack Lee
//
///////////////////////////////////////////////////////////////////////////////


#include "qlautoconf.h"

#include <linux/errno.h>
#include <linux/init.h>
#include <linux/mm.h>
#include <linux/module.h>
#include <linux/poll.h>
#include <asm/io.h>
#include <linux/string.h>
#include <linux/fs.h>
#include <linux/slab.h>

#include "qptypes.h"
#include "qpdbgmgr.h"
#include "qpmemmgr.h"
#include "qposmgr.h"
#include "qptimemgr.h"
#include "qpmgrspriv.h"

#include "plv4ldevversion.h"
#include "cdevice.h"

#include "cqlcodeclib.h"
#include "cqlcodec.h"
#include "hci.h"

#include <linux/interrupt.h>
#include <mach/irqs.h>

#include <asm/system.h>

#include <mach/pl330xp.h>

TQP_DbgMCompID		g_PLV4LDevDBGCompID = 0;
extern TQP_DbgMCompID	g_QPCodecDBGCompID;
extern TQP_DbgMCompID	g_QPUsbDBGCompID;
extern TQP_DbgMCompID	g_QPPeripheralDBGCompID;

struct semaphore	g_module_lock;
spinlock_t lock;

#define MODULE_NAME "SDL H.264 codec"

static int fwmemmode = 1;   /* =0: let FW default to use static memory mode, >0 let FW default to use dynamically memory mode */
module_param(fwmemmode, int, S_IRUGO);

static irqreturn_t pl330soc_m2m_irqhandler(int irq, void *dev)
{
	irqreturn_t	ret = IRQ_HANDLED;
 	CDevice		*pDevice = (CDevice *)dev;
	ICodecLib	*pCodecLib = pDevice->m_pCodecLib;
	CQLCodecLib	*pQLCodecLib = GET_INHERITED_OBJECT(CQLCodecLib, m_iCodecLib, pDevice->m_pCodecLib);

	IHCIAPI 	*pHCI = &pQLCodecLib->m_pMpgCodec->m_hci;


	TQP_UINT32	dwGot = 0;
	QPCODEC_DIAG_REGISTER_STRUCT	regIn;
	QPCODEC_DIAG_REGISTER_STRUCT	regOut;

	memset((void *)&regIn, 0, sizeof(QPCODEC_DIAG_REGISTER_STRUCT));
	memset((void *)&regOut, 0, sizeof(QPCODEC_DIAG_REGISTER_STRUCT));

	regIn.address = 0x730;

	pCodecLib->Get(
		pCodecLib,
		(TQP_REFGUID)&PROPSETID_QPCODEC_DIAG,
		0,
		QPCODEC_DIAG_REG_READ,
		(TQP_PVOID)&regIn,
		(TQP_PVOID)&regOut,
		&dwGot
		);

	regIn.address = 0x730;
	regIn.value = regOut.value;

	pCodecLib->Set(
		pCodecLib,
		(TQP_REFGUID)&PROPSETID_QPCODEC_DIAG,
		0,
		QPCODEC_DIAG_REG_WRITE,
		(TQP_PVOID)&regIn,
		NULL,
		sizeof(QPCODEC_DIAG_REGISTER_STRUCT)
		);

	if (regOut.value & 0x20000) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
					 "%s() m2m error\n",
					 __FUNCTION__
					 ));
	} else {
		if (pHCI->DMAXferDone) {
			pHCI->DMAXferDone(pHCI);
		}
	}

#if ENABLE_CODEC_MEM_LOG
	codec_mem_log_add(CODEC_MEM_LOG_M2M_IRQ);
#endif

	return ret;
}

#if defined(USE_SWIRQ_MODE)
static irqreturn_t pl330soc_swirq0_irqhandler(int irq, void *dev)
{
	irqreturn_t	ret = IRQ_HANDLED;
 	CDevice		*pDevice = (CDevice *)dev;
	ICodecLib	*pCodecLib = pDevice->m_pCodecLib;
	CQLCodecLib	*pQLCodecLib = GET_INHERITED_OBJECT(CQLCodecLib, m_iCodecLib, pDevice->m_pCodecLib);

	IHCIAPI 	*pHCI = &pQLCodecLib->m_pMpgCodec->m_hci;

	TQP_UINT32	dwGot = 0;
	QPCODEC_DIAG_REGISTER_STRUCT	regIn;
	QPCODEC_DIAG_REGISTER_STRUCT	regOut;

	memset((void *)&regIn, 0, sizeof(QPCODEC_DIAG_REGISTER_STRUCT));
	memset((void *)&regOut, 0, sizeof(QPCODEC_DIAG_REGISTER_STRUCT));

	regIn.address = 0x24;

	pCodecLib->Get(
		pCodecLib,
		(TQP_REFGUID)&PROPSETID_QPCODEC_DIAG,
		0,
		QPCODEC_DIAG_REG_READ,
		(TQP_PVOID)&regIn,
		(TQP_PVOID)&regOut,
		&dwGot
		);

	// make sure interrupt is assert, and then clear it
	if (regOut.value & (1 << 24)) {
		regIn.address = 0x24;
		regIn.value = 1 << 24;
		pCodecLib->Set(
			pCodecLib,
			(TQP_REFGUID)&PROPSETID_QPCODEC_DIAG,
			0,
			QPCODEC_DIAG_REG_WRITE,
			(TQP_PVOID)&regIn,
			NULL,
			sizeof(QPCODEC_DIAG_REGISTER_STRUCT)
			);
	}

	QPOSMSetEvtgrp(pQLCodecLib->m_pMpgCodec->m_EvtTask, QLCODEC_EVENT_ARM_INT);

#if ENABLE_CODEC_MEM_LOG
	codec_mem_log_add(CODEC_MEM_LOG_SW_IRQ);
#endif

	return ret;
}
#endif


static long pl330soc_probe()
{
	long			lErr = 0;
	PLV4LDEV_PARENT_DEV	dummy_parent_dev;
	PLV4LDEV_TYPE		devType = PLV4LDEV_TYPE_QL300_MASTER;
 	CDevice			*pDevice = QP_NULL;
        EQP_Errors		qpStatus;

	QPDbgMPrintf((
		QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
		"%s() [\n",
		__FUNCTION__
		));

	QPDbgMRegComp(&g_PLV4LDevDBGCompID, "QP V4LDev");
	QPDbgMRegComp(&g_QPCodecDBGCompID, "QP CodecLib");
	QPDbgMRegComp(&g_QPPeripheralDBGCompID, "QP Peripheral");

	QPDbgMSetCompLevel(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR);
	QPDbgMSetCompLevel(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR);
	QPDbgMSetCompLevel(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR);

#ifdef QP_DBG_TO_BUFFER
	QPDbgMSetBufCompLevel(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR);
	QPDbgMSetBufCompLevel(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR);
	QPDbgMSetBufCompLevel(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR);
#endif

	down(&g_module_lock);

	// find an empty slot for this device
	//
	if (QP_NULL == g_pPlV4lDevices[0]) {
		// create and int the device
		//
		pDevice = (CDevice *)kmalloc(sizeof(CDevice), GFP_KERNEL);
		if (pDevice) {
			memset(pDevice, 0, sizeof(CDevice));
			if (!CDevice_Constructor(
				pDevice,
				0,
				devType,
				&dummy_parent_dev,
				NULL,
				(TQP_PVOID)0xF0000000,
				(TQP_PVOID)0,
				0x100000,
				0,
				fwmemmode
				))
			{
				kfree((void *)pDevice);
				pDevice = QP_NULL;
				lErr = -ENOMEM;
				goto ERROR_EXIT;
			} else {
				qpStatus = CDevice_Create(pDevice);
				if (QP_SUCCESS(qpStatus)) {
					g_pPlV4lDevices[0] = pDevice;
					lErr = request_irq(
						QL300_IRQ_M2M1,
						pl330soc_m2m_irqhandler,
						IRQF_DISABLED,
						"PL330 M2M ISR",
						pDevice
						);

#if defined(USE_SWIRQ_MODE)
					lErr = (lErr) ? lErr : request_irq(
						QL300_IRQ_SW0,
						pl330soc_swirq0_irqhandler,
						IRQF_DISABLED,
						"PL330 SWIRQ0 ISR",
						pDevice
						);
#endif
				} else {
					CDevice_Destructor(pDevice);
					kfree((void *)pDevice);
					pDevice = QP_NULL;
					lErr = -ENOMEM;
					goto ERROR_EXIT;
				}
			}
		} else {
			lErr = -ENOMEM;
			goto ERROR_EXIT;
		}
	} else {
		lErr = -ENOMEM;
		goto ERROR_EXIT;
	}

	goto FUNC_EXIT;


ERROR_EXIT:
	if (pDevice) {
		kfree((void *)pDevice);
		pDevice = QP_NULL;
	}

FUNC_EXIT:
	up(&g_module_lock);

	QPDbgMPrintf((
		QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
		"%s() lErr(%ld) ]\n",
		__FUNCTION__,
		lErr
		));


	return lErr;
}


static void pl330soc_remove()
{
	CDevice *pDevice = g_pPlV4lDevices[0];

	QPDbgMPrintf((
		QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
		"%s() [\n",
		__FUNCTION__
		));

	down(&g_module_lock);

	if (pDevice) {
		free_irq(QL300_IRQ_M2M1, pDevice);
#if defined(USE_SWIRQ_MODE)
		free_irq(QL300_IRQ_SW0, pDevice);
#endif
		g_pPlV4lDevices[0] = QP_NULL;
		CDevice_Close(pDevice);
		CDevice_Destructor(pDevice);
		kfree((void *)pDevice);
	}

	up(&g_module_lock);

	QPDbgMPrintf((
		QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
		"%s() ]\n",
		__FUNCTION__
		));
}


static int __init pl330soc_init_module(void)
{
	long	lErr;

	QPDbgMPrintf((
		QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
		"%s() [\n",
		__FUNCTION__
		));

	QPMgrsInit();

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,37)
	sema_init(&g_module_lock, 1);
#else
	init_MUTEX(&g_module_lock);
#endif

	lErr = pl330soc_probe();

#if 1
	{
		ICodecLib			*pCodecLib = CDevice_getCodecLib(g_pPlV4lDevices[0]);
		TQP_UINT32			dwGot = 0;
		QPCODEC_DIAG_REGISTER_STRUCT	regIn;
		QPCODEC_DIAG_REGISTER_STRUCT	regOut;

		memset((void *)&regIn, 0, sizeof(QPCODEC_DIAG_REGISTER_STRUCT));
		memset((void *)&regOut, 0, sizeof(QPCODEC_DIAG_REGISTER_STRUCT));

		regIn.address = 0x38;

		pCodecLib->Get(
			pCodecLib,
			(TQP_REFGUID)&PROPSETID_QPCODEC_DIAG,
			0,
			QPCODEC_DIAG_REG_READ,
			(TQP_PVOID)&regIn,
			(TQP_PVOID)&regOut,
			&dwGot
			);

		QPDbgMPrintf((
			QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
			"%s() dwGot(%d) regOut.value(0x%08x)\n",
			__FUNCTION__,
			dwGot,
			regOut.value
			));
	}
#endif

	QPDbgMPrintf((
		QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
		"%s() ]\n",
		__FUNCTION__
		));

	return (lErr);
}


static void __exit pl330soc_fini_module(void)
{
	QPDbgMPrintf((
		QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
		"%s() [\n",
		__FUNCTION__
		));

	pl330soc_remove();

	// terminate OS manager
	//
	QPMgrsTerm();

	QPDbgMPrintf((
		QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
		"%s() ]\n",
		__FUNCTION__
		));

	return;
}


module_init(pl330soc_init_module);
module_exit(pl330soc_fini_module);

