///////////////////////////////////////////////////////////////////////////////
//     
//
// Drventry.c
//
// Description:
//
//  star 8133 driver entry point
//
// History:
//	Jan 27, 2010	(dt) created
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#include "qlautoconf.h"

#include <linux/errno.h>
#include <linux/init.h>
#include <linux/mm.h>
#include <linux/module.h>
#include <linux/poll.h>
#include <asm/io.h>
#include <linux/usb.h>
#include <linux/string.h>
#include <linux/fs.h>

#include "qptypes.h"
#include "qpdbgmgr.h"
#include "qpmemmgr.h"
#include "qposmgr.h"
#include "qptimemgr.h"
#include "qpmgrspriv.h"


#include "plv4ldevversion.h"
#include "cdevice.h"


TQP_DbgMCompID			g_PLV4LDevDBGCompID = 0;
extern TQP_DbgMCompID	g_QPCodecDBGCompID;
extern TQP_DbgMCompID	g_QPPeripheralDBGCompID;
extern TQP_DbgMCompID	g_QPUsbDBGCompID;


#define QL201B_PBI_MEM_BASE		SYSPA_FLASH_SRAM_BANK1_BASE_ADDR
//#define QL201B_PBI_MEM_BASE	SYSPA_FLASH_SRAM_BANK2_BASE_ADDR
#define STAR_813X_GPIOA1		0x00000002

struct semaphore	g_module_lock;
unsigned int		unVirtBaseAddr = 0;

static int fwmemmode = 1;
module_param(fwmemmode, int, S_IRUGO);

/////////////////////////////////////////////////////////////////////////////
//
//

static void STAR_localbus_init(void)
{

	volatile unsigned long *ulSMCBank1Register = SMC_MEM_BANK1_CONFIG_REG_ADDR;
	volatile unsigned long *ulSMCBank1Timing   = SMC_MEM_BANK1_TIMING_REG_ADDR;

	volatile unsigned long *ulMISC24 = SYSVA_MISC_BASE_ADDR + 0x24;

/*
    volatile unsigned long *ulSMCBank1Register = SMC_MEM_BANK2_CONFIG_REG_ADDR;
    volatile unsigned long *ulSMCBank1Timing   = SMC_MEM_BANK2_TIMING_REG_ADDR;

    volatile unsigned long *ulMISC24 = SYSVA_MISC_BASE_ADDR + 0x24;
*/
    /*
     * bit4: bank write protected on/off
     * bit3: burst access enable on/off
     * bit2: reserved
     * bit1: disable/enable
     * bit0: memory bus width, 0 for 8, 1 for 16
     */
	*ulSMCBank1Register = 0x7; /* wait-enable, enalbe, 16 bits */
    /*
    	*ulSMCBank1Timing = 0x330f0f0f;
    	*ulSMCBank1Timing = 0x3338180f;
    	*ulSMCBank1Timing = 0x36121812;	//0x33080808;
    	*/
	*ulSMCBank1Timing = 0x661C1C1C;

	printk("*ulSMCBank1Register(0x%08x)=0x%08x\n", ulSMCBank1Register, *ulSMCBank1Register);
	printk("*ulSMCBank1Timing(0x%08x)=0x%08x\n", ulSMCBank1Timing, *ulSMCBank1Timing);

	/* Set MISC reg0x24 bit11/bit14 to enable SMC Bank1 pins */
	*ulMISC24 |= ((0x1 << 11) | (0x1 << 14));
	//*ulMISC24 |= ((0x1 << 11) | (0x1 << 12) | (0x1 << 14) | (0x1 << 15));

	printk("*ulMISC24(0x%08x)=0x%08x\n", ulMISC24, *ulMISC24);

	// Set the VO Ouput Path from QL300
	// Enable switch to our audio output module,gpioa1 to high
	HAL_GPIOA_SET_DIRECTION_OUTPUT(STAR_813X_GPIOA1);
	HAL_GPIOA_SET_DATA_OUT_HIGH(STAR_813X_GPIOA1);

    return;
}


/////////////////////////////////////////////////////////////////////////////
//
//


static int __init plstar8133_init_module(void)
{
	long	lErr = 0;
    	CDevice				*pDevice = QP_NULL;
	EQP_Errors			qpStatus;

	PLV4LDEV_PARENT_DEV	parent_dev;

	QPMgrsInit();

	// register the component for debug output
	//
	QPDbgMRegComp(&g_PLV4LDevDBGCompID,
				  "QP V4LDev"
				  );
	QPDbgMRegComp(&g_QPCodecDBGCompID,
				  "QP CodecLib"
				  );
	QPDbgMRegComp(&g_QPPeripheralDBGCompID,
				  "QP Peripheral"
				  );

	QPDbgMSetCompLevel(g_PLV4LDevDBGCompID,
					   QPDBGM_LEVEL_ERROR
					   );
	QPDbgMSetCompLevel(g_QPCodecDBGCompID,
					   QPDBGM_LEVEL_ERROR
					   );
	QPDbgMSetCompLevel(g_QPUsbDBGCompID,
					   QPDBGM_LEVEL_ERROR
					   );
	QPDbgMSetCompLevel(g_QPPeripheralDBGCompID,
					   QPDBGM_LEVEL_ERROR
					   );

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
				 "%s() driver v%d.%d.%d %s\n",
				 __FUNCTION__,
				 PLDRV_VERSION_MAJOR,
				 PLDRV_VERSION_MINOR,
				 PLDRV_VERSION_BRANCH,
				 PLDRV_VERSION_EXPERIMENTAL
				 ));

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,37)
	sema_init(&g_module_lock, 1);
#else
	init_MUTEX(&g_module_lock);
#endif

    STAR_localbus_init();

	unVirtBaseAddr = (unsigned int)ioremap_nocache(QL201B_PBI_MEM_BASE,
												   0x800000
												   );
	if (!unVirtBaseAddr)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
					 "%s() unable to map QL201B memory space!\n",
					 __FUNCTION__
					 ));
        return (-ENOMEM);
    }

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_WARN),
				 "%s() ioremap of %x to %x \n",
				 __FUNCTION__,
				 QL201B_PBI_MEM_BASE,
				 unVirtBaseAddr
				 ));

	down(&g_module_lock);

	parent_dev.p_dev = QP_NULL;

	// only support one device
	//
	if (QP_NULL == g_pPlV4lDevices[0])
	{
		// create and int the device
		//
		pDevice = (CDevice *)kmalloc(sizeof(CDevice),
									 GFP_KERNEL
									 );
		if (pDevice)
		{
			memset(pDevice,
				   0,
				   sizeof(CDevice)
				   );

			if (!CDevice_Constructor(pDevice,
									 0,
									 PLV4LDEV_TYPE_QL300_STR_EVK,
									 &parent_dev,
									 QP_NULL,
									 (TQP_PVOID)unVirtBaseAddr,
									 (TQP_PVOID)unVirtBaseAddr,
									 0x8000,
									 0,
									 fwmemmode
									 ))
			{
				kfree((void *)pDevice);
				pDevice = QP_NULL;
				lErr = -ENOMEM;
			}
			else
			{
				qpStatus = CDevice_Create(pDevice);
				if (QP_SUCCESS(qpStatus))
				{
					g_pPlV4lDevices[0] = pDevice;
				}
				else
				{
					CDevice_Destructor(pDevice);
					kfree((void *)pDevice);
					pDevice = QP_NULL;
					lErr = -ENOMEM;
				}
			}
		}
		else
		{
			lErr = -ENOMEM;
		}
	}
	else
	{
		lErr = -ENOMEM;
	}

	up(&g_module_lock);

    return (lErr);
}


static void __exit plstar8133_fini_module(void)
{
    CDevice	*pDevice = QP_NULL;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
				 "%s() \n",
				 __FUNCTION__
				 ));

	down(&g_module_lock);
	pDevice = g_pPlV4lDevices[0];
	g_pPlV4lDevices[0] = QP_NULL;
	up(&g_module_lock);

	if (pDevice)
	{
		CDevice_Close(pDevice);
		CDevice_Destructor(pDevice);
		kfree((void *)pDevice);
	}

	if (unVirtBaseAddr)
	{
		iounmap(unVirtBaseAddr);
		unVirtBaseAddr= 0;
	}

	QPDbgMUnregComp(g_PLV4LDevDBGCompID);
	QPDbgMUnregComp(g_QPCodecDBGCompID);
	QPDbgMUnregComp(g_QPPeripheralDBGCompID);

	// terminate OS manager
	//
	QPMgrsTerm();
}


module_init(plstar8133_init_module);
module_exit(plstar8133_fini_module);
