///////////////////////////////////////////////////////////////////////////////
//     
//
// Drventry.c
//
// Description:
//
//  usb driver entry point
//
// History:
//	Jan 19, 2010	(dt) created
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#include "qlautoconf.h"

#include <linux/errno.h>
#include <linux/init.h>
#include <linux/mm.h>
#include <linux/module.h>
#include <linux/poll.h>
#include <asm/io.h>
#include <linux/usb.h>
#include <linux/string.h>
#include <linux/fs.h>
#include <linux/slab.h>

#include "qptypes.h"
#include "qpdbgmgr.h"
#include "qpmemmgr.h"
#include "qposmgr.h"
#include "qptimemgr.h"
#include "qpmgrspriv.h"


#include "plv4ldevversion.h"
#include "cdevice.h"


TQP_DbgMCompID			g_PLV4LDevDBGCompID = 0;
extern TQP_DbgMCompID	g_QPCodecDBGCompID;
extern TQP_DbgMCompID	g_QPUsbDBGCompID;
extern TQP_DbgMCompID	g_QPPeripheralDBGCompID;


/////////////////////////////////////////////////////////////////////////////
//
//


#define FX2_VENDOR_ID			0x04B4
#define FX2_DEVICE_ID			0x8613

#define PZLOGIC_OLD_VENDOR_ID	0x0547
#define PZLOGIC_VENDOR_ID		0x2331
#define SDL_VENDOR_ID			0x0658
#define FVT_VENDOR_ID			0x0408

#define QL201_FX2_ID			0x1000
#define QL201_EVK_ID			0x1001

#define PL300_FX2_ID			0x1000
#define PL300_EVB_ID			0x1001
#define PL300_EVK_ID			0x2001
#define PL300_EVK_ID_2			0x3001
#define PL300_EVK_ID_3  		0x3003


#define SDL_PL300_FX2_ID		0x1000
#define SDL_PL300_EVB_ID		0x1002

#define SDL_PL300_EVK_ID		0x1002
#define SDL_PL300_EVK_ID_2		0x1100
#define SDL_PL300_EVK_ID_3  	0x1102

#define SDL_PL300_CUS_ID_1  	0x1200
#define SDL_PL300_CUS_ID_2  	0x1202
#define SDL_PL300_CUS_ID_3  	0x1204
#define SDL_PL300_CUS_ID_4  	0x1206

#define FVT_FC3_ID				0x2051
#define FVT_FC3D_ID				0x2053
#define FVT_FC3Q_ID				0x2055

/////////////////////////////////////////////////////////////////////////////
//
//


struct semaphore g_module_lock;

static struct usb_device_id plusb_device_table[] =
{
	{USB_DEVICE(FX2_VENDOR_ID, FX2_DEVICE_ID)},
	{USB_DEVICE(PZLOGIC_OLD_VENDOR_ID, PL300_FX2_ID)},
	{USB_DEVICE(PZLOGIC_OLD_VENDOR_ID, PL300_EVB_ID)},
	{USB_DEVICE(PZLOGIC_OLD_VENDOR_ID, PL300_EVK_ID)},
	{USB_DEVICE(PZLOGIC_OLD_VENDOR_ID, PL300_EVK_ID_2)},
	{USB_DEVICE(PZLOGIC_OLD_VENDOR_ID, PL300_EVK_ID_3)},
	{USB_DEVICE(PZLOGIC_VENDOR_ID, PL300_FX2_ID)},
	{USB_DEVICE(PZLOGIC_VENDOR_ID, PL300_EVB_ID)},
	{USB_DEVICE(PZLOGIC_VENDOR_ID, PL300_EVK_ID)},
	{USB_DEVICE(PZLOGIC_VENDOR_ID, PL300_EVK_ID_2)},
	{USB_DEVICE(PZLOGIC_VENDOR_ID, PL300_EVK_ID_3)},
	{USB_DEVICE(SDL_VENDOR_ID, SDL_PL300_FX2_ID)},
	{USB_DEVICE(SDL_VENDOR_ID, SDL_PL300_EVB_ID)},
	{USB_DEVICE(SDL_VENDOR_ID, SDL_PL300_EVK_ID)},
	{USB_DEVICE(SDL_VENDOR_ID, SDL_PL300_EVK_ID_2)},
	{USB_DEVICE(SDL_VENDOR_ID, SDL_PL300_EVK_ID_3)},
	{USB_DEVICE(SDL_VENDOR_ID, SDL_PL300_CUS_ID_1)},
	{USB_DEVICE(SDL_VENDOR_ID, SDL_PL300_CUS_ID_2)},
	{USB_DEVICE(SDL_VENDOR_ID, SDL_PL300_CUS_ID_3)},
	{USB_DEVICE(SDL_VENDOR_ID, SDL_PL300_CUS_ID_4)},
	{USB_DEVICE(FVT_VENDOR_ID, FVT_FC3_ID)},
	{USB_DEVICE(FVT_VENDOR_ID, FVT_FC3D_ID)},
	{USB_DEVICE(FVT_VENDOR_ID, FVT_FC3Q_ID)},
	{}
};


MODULE_DEVICE_TABLE(usb, plusb_device_table);

#define MODULE_NAME "SDL USB H.264 codec"

static int fwmemmode = 1;   /* =0: let FW default to use static memory mode, >0 let FW default to use dynamically memory mode */
module_param(fwmemmode, int, S_IRUGO);

extern TQP_INT32 fx2Setup(struct usb_device *usbdev, unsigned long ulDevType);


PLV4LDEV_TYPE plusb_is_know_dev(struct usb_device *usbdev)
{
    if ((usbdev->descriptor.idVendor == FX2_VENDOR_ID) && (usbdev->descriptor.idProduct == FX2_DEVICE_ID))
	{
		return (PLV4LDEV_TYPE_QL300_FX2);
    }
	else if ((usbdev->descriptor.idVendor == PZLOGIC_OLD_VENDOR_ID) && (usbdev->descriptor.idProduct == PL300_FX2_ID))
	{
        return (PLV4LDEV_TYPE_QL300_FX2);
    }
	else if ((usbdev->descriptor.idVendor == PZLOGIC_OLD_VENDOR_ID) && (usbdev->descriptor.idProduct == PL300_EVB_ID))
	{
		return (PLV4LDEV_TYPE_QL300_USB_EVB);
    }
	else if ((usbdev->descriptor.idVendor == PZLOGIC_OLD_VENDOR_ID) && (usbdev->descriptor.idProduct == PL300_EVK_ID))
	{
        return (PLV4LDEV_TYPE_QL300_USB_EVK);
    }
	else if ((usbdev->descriptor.idVendor == PZLOGIC_OLD_VENDOR_ID) && (usbdev->descriptor.idProduct == PL300_EVK_ID_2))
	{
        return (PLV4LDEV_TYPE_QL300_USB_EVK);
    }
	else if ((usbdev->descriptor.idVendor == PZLOGIC_OLD_VENDOR_ID) && (usbdev->descriptor.idProduct == PL300_EVK_ID_3))
	{
        return (PLV4LDEV_TYPE_QL300_USB_CAM);
    }
	else if ((usbdev->descriptor.idVendor == PZLOGIC_VENDOR_ID) && (usbdev->descriptor.idProduct == PL300_FX2_ID))
	{
        return (PLV4LDEV_TYPE_QL300_FX2);
    }
	else if ((usbdev->descriptor.idVendor == PZLOGIC_VENDOR_ID) && (usbdev->descriptor.idProduct == PL300_EVB_ID))
	{
		return (PLV4LDEV_TYPE_QL300_USB_EVB);
    }
	else if ((usbdev->descriptor.idVendor == PZLOGIC_VENDOR_ID) && (usbdev->descriptor.idProduct == PL300_EVK_ID))
	{
        return (PLV4LDEV_TYPE_QL300_USB_EVK);
    }
	else if ((usbdev->descriptor.idVendor == PZLOGIC_VENDOR_ID) && (usbdev->descriptor.idProduct == PL300_EVK_ID_2))
	{
        return (PLV4LDEV_TYPE_QL300_USB_EVK);
    }
	else if ((usbdev->descriptor.idVendor == PZLOGIC_VENDOR_ID) && (usbdev->descriptor.idProduct == PL300_EVK_ID_3))
	{
        return (PLV4LDEV_TYPE_QL300_USB_CAM);
    }
	else if ((usbdev->descriptor.idVendor == SDL_VENDOR_ID) && (usbdev->descriptor.idProduct == SDL_PL300_FX2_ID))
	{
        return (PLV4LDEV_TYPE_QL300_FX2);
    }
	else if ((usbdev->descriptor.idVendor == SDL_VENDOR_ID) && (usbdev->descriptor.idProduct == SDL_PL300_EVB_ID))
	{
		return (PLV4LDEV_TYPE_QL300_USB_EVB);
    }
	else if ((usbdev->descriptor.idVendor == SDL_VENDOR_ID) && (usbdev->descriptor.idProduct == SDL_PL300_EVK_ID))
	{
        return (PLV4LDEV_TYPE_QL300_USB_EVK);
    }
	else if ((usbdev->descriptor.idVendor == SDL_VENDOR_ID) && (usbdev->descriptor.idProduct == SDL_PL300_EVK_ID_2))
	{
        return (PLV4LDEV_TYPE_QL300_USB_EVK);
    }
	else if ((usbdev->descriptor.idVendor == SDL_VENDOR_ID) && (usbdev->descriptor.idProduct == SDL_PL300_EVK_ID_3))
	{
        return (PLV4LDEV_TYPE_QL300_USB_CAM);
    }
	else if ((usbdev->descriptor.idVendor == SDL_VENDOR_ID) && (usbdev->descriptor.idProduct == SDL_PL300_CUS_ID_1))
	{
        return (PLV4LDEV_TYPE_QL300_USB_CUS1);
    }
	else if ((usbdev->descriptor.idVendor == SDL_VENDOR_ID) && (usbdev->descriptor.idProduct == SDL_PL300_CUS_ID_2))
	{
        return (PLV4LDEV_TYPE_QL300_USB_CUS2);
    }
	else if ((usbdev->descriptor.idVendor == SDL_VENDOR_ID) && (usbdev->descriptor.idProduct == SDL_PL300_CUS_ID_3))
	{
        return (PLV4LDEV_TYPE_QL300_USB_CUS3);
    }
	else if ((usbdev->descriptor.idVendor == SDL_VENDOR_ID) && (usbdev->descriptor.idProduct == SDL_PL300_CUS_ID_4))
	{
        return (PLV4LDEV_TYPE_QL300_USB_CUS4);
    }
	else if ((usbdev->descriptor.idVendor == FVT_VENDOR_ID) && (usbdev->descriptor.idProduct == FVT_FC3_ID))
	{
        return (PLV4LDEV_TYPE_QL300_USB_FC3);
    }
	else if ((usbdev->descriptor.idVendor == FVT_VENDOR_ID) && (usbdev->descriptor.idProduct == FVT_FC3D_ID))
	{
        return (PLV4LDEV_TYPE_QL300_USB_FC3);
    }
	else if ((usbdev->descriptor.idVendor == FVT_VENDOR_ID) && (usbdev->descriptor.idProduct == FVT_FC3Q_ID))
	{
        return (PLV4LDEV_TYPE_QL300_USB_FC3);
    }
	else
	{
		return (PLV4LDEV_TYPE_NONE);
    }
}


/////////////////////////////////////////////////////////////////////////////
//
//


#ifndef __LINUX24__
static int plusb_probe(struct usb_interface *intf,
					   const struct usb_device_id *id
					   )
#else //__LINUX24__
static void *plusb_probe(struct usb_device *dev,
						 unsigned int interface,
						 const struct usb_device_id *id
						 )
#endif //!__LINUX24__
{
	long				lErr = 0;
	PLV4LDEV_TYPE		devType;
    CDevice				*pDevice = QP_NULL;
#ifndef __LINUX24__
	struct usb_device	*usbdev = interface_to_usbdev(intf);
#else //__LINUX24__
	struct usb_device	*usbdev = dev;
#endif //!__LINUX24__
	PLV4LDEV_PARENT_DEV	parent_dev;
	EQP_Errors			qpStatus;
	TQP_UINT32			i = 0;

	// register the component for debug output
	//
	QPDbgMRegComp(&g_PLV4LDevDBGCompID,
				  "QP V4LDev"
				  );
	QPDbgMRegComp(&g_QPCodecDBGCompID,
				  "QP CodecLib"
				  );
	QPDbgMRegComp(&g_QPUsbDBGCompID,
				  "QP Usb"
				  );
	QPDbgMRegComp(&g_QPPeripheralDBGCompID,
				  "QP Peripheral"
				  );

	QPDbgMSetCompLevel(g_PLV4LDevDBGCompID,
					   QPDBGM_LEVEL_ERROR
					   );
	QPDbgMSetCompLevel(g_QPCodecDBGCompID,
					   QPDBGM_LEVEL_ERROR
					   );
	QPDbgMSetCompLevel(g_QPUsbDBGCompID,
					   QPDBGM_LEVEL_ERROR
					   );
	QPDbgMSetCompLevel(g_QPPeripheralDBGCompID,
					   QPDBGM_LEVEL_ERROR
					   );

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
				 "%s() VID=%04x, PID=%04x, (usbdev=%08x)\n",
				 __FUNCTION__,
				 usbdev->descriptor.idVendor,
				 usbdev->descriptor.idProduct,
				 usbdev
				 ));

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
				 "%s() driver v%d.%d.%d %s\n",
				 __FUNCTION__,
				 PLDRV_VERSION_MAJOR,
				 PLDRV_VERSION_MINOR,
				 PLDRV_VERSION_BRANCH,
				 PLDRV_VERSION_EXPERIMENTAL
				 ));
#ifdef CONFIG_PL_AO_FIX_IN_SLAVE
	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
				 "%s() driver special build for fix PL330 AO always in Slave mode\n",
				 __FUNCTION__
				 ));
#endif
#ifdef CONFIG_PL_AI_FIX_IN_MASTER
	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
				 "%s() driver special build for fix PL330 AI always in Master mode\n",
				 __FUNCTION__
				 ));
#endif
	{
		int i;
		struct usb_host_endpoint *endpoint;
		struct usb_endpoint_descriptor endpointdesc;

		endpoint = intf->cur_altsetting->endpoint;
		for (i = 0; i < intf->cur_altsetting->desc.bNumEndpoints; i++) {
		        endpointdesc = endpoint->desc;
                QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
                             "%s()-> endpointdesc.bmAttributes(0x%08x)\n",
                             __FUNCTION__, endpointdesc.bmAttributes
							 ));
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,18)
		        if (USB_ENDPOINT_XFER_INT == endpointdesc.bmAttributes & USB_ENDPOINT_XFERTYPE_MASK) {
                            usb_driver_set_configuration(usbdev, 1);
                            QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
                            			 "%s()reinit to bulk mode\n",
										 __FUNCTION__
										 ));
                            return -ENODEV;
		        }
#endif //LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,18)
		        endpoint++;
		}
	}

#ifndef __LINUX24__
	parent_dev.p_usbintf = intf;
#else //__LINUX24__
	parent_dev.p_usbdev = dev;
#endif //!__LINUX24__

    devType = plusb_is_know_dev(usbdev);

    switch (devType)
	{
		case PLV4LDEV_TYPE_QL201_FX2:
		case PLV4LDEV_TYPE_QL300_FX2:

			lErr = (int)fx2Setup(usbdev, devType);
			break;

		case PLV4LDEV_TYPE_QL300_USB_EVB:
		case PLV4LDEV_TYPE_QL300_USB_EVK:
		case PLV4LDEV_TYPE_QL300_USB_CAM:
		case PLV4LDEV_TYPE_QL300_USB_FC3:
		case PLV4LDEV_TYPE_QL300_USB_CUS1:
		case PLV4LDEV_TYPE_QL300_USB_CUS2:
		case PLV4LDEV_TYPE_QL300_USB_CUS3:
		case PLV4LDEV_TYPE_QL300_USB_CUS4:

			down(&g_module_lock);

			// find an empty slot for this device
			//
			for (i = 0; i < PLV4LDEV_MAX_DEVICES; i++)
			{
				if (QP_NULL == g_pPlV4lDevices[i])
				{
					// create and int the device
					//
					pDevice = (CDevice *)kmalloc(sizeof(CDevice),
												 GFP_KERNEL
												 );
					if (pDevice)
					{
						memset(pDevice,
							   0,
							   sizeof(CDevice)
							   );

						if (!CDevice_Constructor(pDevice,
												 i,
												 devType,
												 &parent_dev,
												 &usbdev->dev,
												 (TQP_PVOID)0x100000,
												 (TQP_PVOID)0,
												 0x100000,
												 0,
												 fwmemmode
												 ))
						{
							kfree((void *)pDevice);
							pDevice = QP_NULL;
							lErr = -ENOMEM;
						}
						else
						{
							qpStatus = CDevice_Create(pDevice);
							if (QP_SUCCESS(qpStatus))
							{
								g_pPlV4lDevices[i] = pDevice;
#ifndef __LINUX24__
								usb_set_intfdata(intf,
												 (void *)pDevice
												 );
#endif //!__LINUX24__
							}
							else
							{
								CDevice_Destructor(pDevice);
								kfree((void *)pDevice);
								pDevice = QP_NULL;
								lErr = -ENOMEM;
							}
						}
					}
					else
					{
						lErr = -ENOMEM;
					}
					break;
				}
			}

			if (PLV4LDEV_MAX_DEVICES == i)
			{
				// no empty slot available
				//
				lErr = -ENOMEM;
			}

			up(&g_module_lock);
			break;

		default:
			lErr = -ENODEV;
			break;
    }

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
				 "%s() probe returns %d pDevice(0x%x), id(%d)\n",
				 __FUNCTION__,
				 lErr,
				 pDevice,
				 i
				 ));

#ifndef __LINUX24__
	return (lErr);
#else //__LINUX24__
	// 0 for failure, non-zero device pointer otherwise
	//
	if (lErr)
	{
		return (0);
	}
	else
	{
        return ((void*)pDevice);
	}
#endif //!__LINUX24__
}


#ifndef __LINUX24__
static void plusb_disconnect(struct usb_interface *intf)
#else //__LINUX24__
static void plusb_disconnect(struct usb_device *dev,
							 void *pvoid
							 )
#endif //!__LINUX24__
{
	PLV4LDEV_TYPE		devType;
    CDevice				*pDevice = QP_NULL;
#ifndef __LINUX24__
	struct usb_device	*usbdev = interface_to_usbdev(intf);
#else //__LINUX24__
	struct usb_device	*usbdev = dev;
#endif //!__LINUX24__

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_TRACE),
				 "%s() VID=%04x, PID=%04x, (usbdev=%08x)\n",
				 __FUNCTION__,
				 usbdev->descriptor.idVendor,
				 usbdev->descriptor.idProduct,
				 usbdev
				 ));

    devType = plusb_is_know_dev(usbdev);

	switch (devType)
	{
		case PLV4LDEV_TYPE_QL201_FX2:
		case PLV4LDEV_TYPE_QL300_FX2:
			break;

		case PLV4LDEV_TYPE_QL300_USB_EVB:
		case PLV4LDEV_TYPE_QL300_USB_EVK:
		case PLV4LDEV_TYPE_QL300_USB_CAM:
		case PLV4LDEV_TYPE_QL300_USB_FC3:
		case PLV4LDEV_TYPE_QL300_USB_CUS1:
		case PLV4LDEV_TYPE_QL300_USB_CUS2:
		case PLV4LDEV_TYPE_QL300_USB_CUS3:
		case PLV4LDEV_TYPE_QL300_USB_CUS4:
#ifndef __LINUX24__
			pDevice = (CDevice *)usb_get_intfdata(intf);
#else //__LINUX24__
			pDevice = (CDevice *)pvoid;;
#endif //!__LINUX24__
			if (pDevice)
			{
				down(&g_module_lock);
				g_pPlV4lDevices[pDevice->m_id] = QP_NULL;
				up(&g_module_lock);

				CDevice_Close(pDevice);
				CDevice_Destructor(pDevice);
				kfree((void *)pDevice);
			}
#ifndef __LINUX24__
			usb_set_intfdata(intf,
							 0
							 );
#endif //!__LINUX24__

			QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_ERROR),
						 "%s() disconnect done!\n",
						 __FUNCTION__
						 ));
			break;

		default:
			break;
	}

    return;
}



#ifndef __LINUX24__
int plusb_suspend(struct usb_interface *intf,
				  pm_message_t message
				  )
{
    CDevice	*pDevice = (CDevice *)usb_get_intfdata(intf);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_WARN),
				 "%s() \n",
				 __FUNCTION__
				 ));
	if (pDevice)
	{
		CDevice_PowerDown(pDevice);
	}
	return (0);
}



int plusb_resume(struct usb_interface *intf)
{
    CDevice	*pDevice = (CDevice *)usb_get_intfdata(intf);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_WARN),
				 "%s() \n",
				 __FUNCTION__
				 ));
	if (pDevice)
	{
		CDevice_PowerUp(pDevice);
	}
	return (0);
}



int plusb_reset_resume(struct usb_interface *intf)
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_PLV4LDevDBGCompID, QPDBGM_LEVEL_WARN),
				 "%s() \n",
				 __FUNCTION__
				 ));
	return (plusb_resume(intf));
}
#endif //!__LINUX24__



#ifndef __LINUX24__
static struct usb_driver plusb_driver =
{
    .name        = MODULE_NAME,
    .id_table    = plusb_device_table,
    .probe       = plusb_probe,
    .disconnect  = plusb_disconnect,
    .suspend     = plusb_suspend,
    .resume      = plusb_resume,
    .resume      = plusb_reset_resume,
};
#else //__LINUX24__
static struct usb_driver plusb_driver =
{
    .name        = MODULE_NAME,
    .id_table    = plusb_device_table,
    .probe       = plusb_probe,
    .disconnect  = plusb_disconnect,
    .driver_list = LIST_HEAD_INIT(plusb_driver.driver_list),
};
#endif //!__LINUX24__



static int __init plusb_init_module(void)
{
	long	lErr;

	QPMgrsInit();

	printk("%s() \n",
		   __FUNCTION__
		   );

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,37)
	sema_init(&g_module_lock, 1);
#else
	init_MUTEX(&g_module_lock);
#endif

    lErr = usb_register(&plusb_driver);

    return (lErr);
}


static void __exit plusb_fini_module(void)
{
	usb_deregister(&plusb_driver);

	printk("%s() \n",
		   __FUNCTION__
		   );

	// terminate OS manager
	//
	QPMgrsTerm();

	return;
}


module_init(plusb_init_module);
module_exit(plusb_fini_module);

