///////////////////////////////////////////////////////////////////////////////
//        
//
// Cpciecntl.h
//
// Description: 
//
//  This is the implementation of the PCIe interface
// 
// History:
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#ifndef __CPCIECNTL_H__
#define __CPCIECNTL_H__

#include "qptypes.h"
#include "cobject.h"
#include "qpgenericdevice.h"
#include "qpbusintf.h"


/////////////////////////////////////////////////////////////////////////////
//
//

// DMA transfer mode
//
#define DMA_TRANSFER_MODE_LINEAR			0
#define DMA_TRANSFER_MODE_MB_RAS_LUMA		1
#define DMA_TRANSFER_MODE_MB_RAS_CHROMA		2
#define DMA_TRANSFER_MODE_8U8V_RAS_CHROMA	3

#define DMA_TRANSFER_MODE_MASK				0xFF

#define DMA_BUFFER_MODE_KERNEL				0x00000000
#define DMA_BUFFER_MODE_USERPTR				0x01000000
#define DMA_BUFFER_MODE_COMMON				0x02000000

#define DMA_BUFFER_MODE_MASK				0xFF000000



// DMA swap type
//
#define	DMA_DATA_SWAP_BYTE					0x1
#define	DMA_DATA_SWAP_WORD					0x2
#define	DMA_DATA_SWAP_DWORD					0x4


/////////////////////////////////////////////////////////////////////////////
//
//


#ifdef __cplusplus
extern "C" {
#endif


#ifndef __CPCIECNTL_FWD_DEFINED__
#define __CPCIECNTL_FWD_DEFINED__
typedef struct CPCIeCntl CPCIeCntl;
#endif //__CPCIECNTL_FWD_DEFINED__

struct CPCIeCntl
{
	// public CObject
	//
	CObject				m_Object;

	// IGenericDevice
	//
	DECLARE_IGENERICDEVICE(CPCIeCntl)

	// private data
	//
	TQP_PVOID			m_pDeviceExtension;

	// bus callback
	//
	QP_BUSINTF_CALLBACK	m_pBusCallbackFunc;
	TQP_PVOID			m_pBusCallbackContext;
};


EQP_Errors CPCIeCntl_ReadPCIeRegister(CPCIeCntl *This,
									  TQP_UINT32 dwAddr,
									  PTQP_UINT32 pData
									  );
EQP_Errors CPCIeCntl_WritePCIeRegister(CPCIeCntl *This,
									   TQP_UINT32 dwAddr,
									   TQP_UINT32 dwData
									   );
EQP_Errors CPCIeCntl_RegisterRead(CPCIeCntl *This,
								  TQP_UINT16 wAddr,
								  PTQP_UINT32 pData
								  );
EQP_Errors CPCIeCntl_RegisterWrite(CPCIeCntl *This,
								   TQP_UINT16 wAddr,
								   TQP_UINT32 dwData 
								   );
EQP_Errors CPCIeCntl_RegisterReadEx(CPCIeCntl *This,
									TQP_UINT16 wAddr,
									PTQP_UINT32 pData,
									TQP_UINT32 numReg
									);
EQP_Errors CPCIeCntl_MemoryRead(CPCIeCntl *This,
								TQP_UINT32 dwAddr,
								PTQP_UINT32 pData 
								);
EQP_Errors CPCIeCntl_MemoryWrite(CPCIeCntl *This,
								 TQP_UINT32 dwAddr,
								 TQP_UINT32 dwData 
								 );
EQP_Errors CPCIeCntl_StartDMAWrite(CPCIeCntl *This,
								   TQP_UINT32 ulAddr,	// in WORD(32 bit)
								   TQP_UINT32 ulLength,	// in WORD(32 bit)
								   PTQP_UINT8 pHostAddr,
								   TQP_UINT32 ulXferMode,
								   TQP_UINT32 ulPicWidthMB,
								   TQP_UINT32 ulPicHeightMB,
								   TQP_UINT32 ulSwap,
								   TQP_BOOL IsFrameMode
								   );
EQP_Errors CPCIeCntl_StartDMARead(CPCIeCntl *This,
								  TQP_UINT32 ulAddr,	// in WORD(32 bit)
								  TQP_UINT32 ulLength,	// in WORD(32 bit)
								  PTQP_UINT8 pHostAddr,
								  TQP_UINT32 ulXferMode,
								  TQP_UINT32 ulPicWidthMB,
								  TQP_UINT32 ulPicHeightMB,
								  TQP_UINT32 ulSwap,
								  TQP_BOOL IsFrameMode
								  );
EQP_Errors CPCIeCntl_EnableInterrupts(CPCIeCntl *This);
EQP_Errors CPCIeCntl_DisableInterrupt(CPCIeCntl *This);
EQP_Errors CPCIeCntl_CopyFromCommonBuffer(CPCIeCntl *This, 
										  PTQP_UINT8 pHostAddr, 
										  TQP_UINT32 ulLength 
										  );
EQP_Errors CPCIeCntl_DMAReadDone(CPCIeCntl *This);
EQP_Errors CPCIeCntl_DMAWriteDone(CPCIeCntl *This);
EQP_Errors CPCIeCntl_RegisterISR(CPCIeCntl *This);
EQP_Errors CPCIeCntl_UnregisterISR(CPCIeCntl *This);
TQP_UINT32 CPCIeCntl_GetMaxDMASize(CPCIeCntl *This);
CPCIeCntl * CPCIeCntl_Constructor(CPCIeCntl *pPCIeCntl, 
								  CObject *pParent,
								  TQP_UINT32 dwAttributes,
								  TQP_PVOID pPhysicalDeviceObject,
								  TQP_PVOID pFunctionalDeviceObject,
								  QP_BUSINTF_CALLBACK pBusCallbackFunc,
								  TQP_PVOID pBusCallbackContext
								  );
void CPCIeCntl_Destructor(CPCIeCntl *This);


#ifdef __cplusplus
}
#endif

#endif //__CPCIECNTL_H__



