/*************************************************************************/
/**      **/
/**            **/
/**                **/
/**              **/
/**                                                                     **/
/**    **/
/**                  **/
/**                                                                     **/
/**                               **/
/**                                                  **/
/*************************************************************************/

#ifndef QLPCI_DEVICE_INC
#define QLPCI_DEVICE_INC 1234

#define PLPCI_NUM_DESCR			1026
#define PLPCI_DESCR_MAX_LEN		((PLPCI_NUM_DESCR - 2) * PAGE_SIZE)

//#define PLPCIBUFF_SIZE          262144
//peter
#define PLPCIBUFF_SIZE          1920*1200


#define PED_IRQ_DMA_COMPLETE(x) (0x0001 << (x))
#define PED_IRQ_DMA_ENABLE(x)   (0x0100 << (x))
#define PED_IRQ_ARM             (0x1000)
#define PED_IRQ_UNUSED          (~0x1F0F)

#define PED_DMA_IS_BUSY(Status)	(Status & PED_STAT_CHAIN_RUNNING)


// Maximum size of transfer that we support on this device.
#define CARD_MAX_TRANSFER			0x100000    // 1 MB

// The watchdog interval.  Any request pending for this number of seconds is
// cancelled.  This is a one second granularity timer, so the timeout must be
// at least one second more than the minimum allowed.
#define CARD_WATCHDOG_INTERVAL		5

// Device State definitions
//
// State values are used to track the PnP state of the device.
//
// NOTE: The code makes assumptions about the order of these state,
// and the fact that their values are in the high word of a dword.
//
// We also use a set of dummy states to determine the actions we
// take on receipt and completion of an IRP.

// In STATE_REMOVED, we immediately fail any received I/O requests
#define STATE_REMOVED				0x00000000

// In STATE_SURPRISE_REMOVED, we immediately fail all incoming requests
#define STATE_SURPRISE_REMOVED		0x00010000

// In STATE_NEVER_STARTED, we also immediately fail all incoming requests
#define STATE_NEVER_STARTED			0x00020000

// Dummy State -- When the state is < this value, no H/W access is allowed
#define STATE_ALL_BELOW_NO_HW		0x00030000

// In STATE_REMOVE_PENDING, we also immediately fail all incoming requests
#define STATE_REMOVE_PENDING		0x00100000

// Dummy state -- When an IRP arrives at the driver, if the current
// device state is below this value, it is immediately failed
#define STATE_ALL_BELOW_FAIL		0x00FF0000

// In STATE_STARTED, requests are processed and removed from the
// queues normally
#define STATE_STARTED				0X01000000

// Dummy state -- When an IRP arrives at the driver, if the current
// device state is above this value, it is queued, not initiated on
// the device (even if the device is free)
#define STATE_ALL_ABOVE_QUEUE		0x0FFF0000

// Dummy State -- When an IRP is COMPLETED on the device, if
// the device state is below this value, we'll start another
// IRP in an attempt to drain the queue of pending requests.
#define STATE_ALL_BELOW_DRAIN		STATE_ALL_ABOVE_QUEUE

// In STATE_STOP_PENDING, we queue incoming requests
#define STATE_STOP_PENDING			0x10000000

// In STATE_STOPPED, we queue incoming requests
#define STATE_STOPPED				0x10010000

#define PED_MAX_DMA_BYTE_COUNT		(1 << 28)
#define PED_MAX_DMA_TRANSFER		(64 * 1024 * 1024)

#define PED_DMA_ALIGN_LEN			32
//#define PED_DMA_ALIGN_VIRT_ADDR(x)	((PPED_DMA_DESCRIPTOR)((((ULONG_PTR)(x)) + (PED_DMA_ALIGN_LEN - 1)) & ~(PED_DMA_ALIGN_LEN - 1)))
#define PED_DMA_ALIGN_VIRT_ADDR(x)	((PPED_DMA_DESCRIPTOR)((((x)) + (PED_DMA_ALIGN_LEN - 1)) & ~(PED_DMA_ALIGN_LEN - 1)))
#define PED_DMA_ALIGN_PHYS_ADDR(x)	(((x) + (PED_DMA_ALIGN_LEN - 1)) & ~(PED_DMA_ALIGN_LEN - 1))


// Maximum length of the buffer that we can send to IoAllocateMdl()
// according to the MSDN
#define PCI_MAX_MDL_LENGTH			((PAGE_SIZE * (0xFFFF - sizeof(MDL))) / sizeof(ULONG_PTR))

#define CHECK_QLPCI_PRESENCE(qlpcidev) if(!(((struct qlpci_device *)qlpcidev)->devconnected)) return -ENODEV;

typedef struct _LARGE_INTEGER
{
    union
    {
        struct
        {
            ULONG LowPart;
            ULONG HighPart;
        };
        ULONGLONG QuadPart;
    };
} LARGE_INTEGER, *PLARGE_INTEGER;

typedef LARGE_INTEGER PHYSICAL_ADDRESS;
//typedef ULONG PHYSICAL_ADDRESS;

typedef struct PED_DMA_ENGINE
{
#define PED_CAP_ENGINE_PRESENT		(1 << 0)
#define PED_CAP_ENGINE_TYPE(x)		(((x) >> 1) & 0x01)
#define PED_CAP_ENGINE_TYPE_C2S		1
#define PED_CAP_ENGINE_TYPE_S2C		0
#define PED_CAP_ENGINE_NUMBER(x)	(((x) >> 8) & 0xFF)
#define PED_CAP_CARD_RAM_SIZE(x)	(1UL << (((x) >> 16) & 0xFF))

    volatile ULONG				Capabilities;

#define PED_CTRL_IRQ_ENABLE			(1 << 0)
#define PED_CTRL_IRQ_ACTIVE			(1 << 1)
#define PED_CTRL_CHAIN_START		(1 << 8)
#define PED_CTRL_CHAIN_STOP			(1 << 9)
#define PED_STAT_CHAIN_RUNNING		(1 << 10)
#define PED_STAT_CHAIN_COMPLETE		(1 << 11)
#define PED_STAT_CHAIN_SHORT_ERR	(1 << 12)
#define PED_STAT_CHAIN_SHORT_SW		(1 << 13)
#define PED_STAT_CHAIN_SHORT_HW		(1 << 14)

    volatile ULONG				ControlStatus;
    volatile PHYSICAL_ADDRESS	Descriptor;
    volatile ULONG				HardwareTime;
    volatile ULONG				ChainCmplByteCount;
    volatile ULONG				Reserved[(0x0100-0x0018) / sizeof(ULONG)];
} PED_DMA_ENGINE, *PPED_DMA_ENGINE;



typedef struct PED_DMA_COMMON
{
#define PED_CARD_IRQ_ENABLE			(1 << 0)
#define PED_CARD_IRQ_ACTIVE			(1 << 1)
#define PED_CARD_IRQ_PENDING		(1 << 2)
#define PED_CARD_IRQ_MSI			(1 << 3)
#define PED_CARD_MAX_C2S_SIZE(x)	(1UL << ((((x) >> 8) & 0xFF)+7))
#define PED_CARD_MAX_S2C_SIZE(x)	(1UL << ((((x) >> 12) & 0xFF)+7))

    volatile ULONG	ControlStatus;
    volatile ULONG	FpgaVersion;
} PED_DMA_COMMON, *PPED_DMA_COMMON;


// BAR0 Register Layout
typedef struct PED_BAR0_REGISTERS
{
	// DMA Engines
	PED_DMA_ENGINE	DMA[PCI_MAX_DMAS];

	// DMA Common DMA Register Block
	PED_DMA_COMMON	Common;
} PED_BAR0_REGISTERS, *PPED_BAR0_REGISTERS;


// extended BAR0 Registers
typedef struct PED_BAR0_REGISTERS_EX
{
	volatile ULONG	VerCore;			//0x8000
	volatile ULONG	VerBE;				//0x8004
	volatile ULONG	CorePCredits;		//0x8008
	volatile ULONG	CoreNCredits;		//0x800C
	volatile ULONG	CoreCCredits;		//0x8010
	volatile ULONG	RemPCredits;		//0x8014
	volatile ULONG	RemNCredits;		//0x8018
	volatile ULONG	RemCCredits;		//0x801C
	volatile ULONG	H2CScratchPad0;		//0x8020
	volatile ULONG	H2CScratchPad1;		//0x8024
	volatile ULONG	H2CScratchPad2;		//0x8028
	volatile ULONG	H2CScratchPad3;		//0x802C

#define PED_H2C_SCRATCH_PAD_VALID	(1 << 31)
#define PED_C2H_SCRATCH_PAD_VALID	(1 << 30)
	
	volatile ULONG	UserIntEnableStatus;//0x8030
	volatile ULONG	UserDataFlowControl;//0x8034
	volatile ULONG	UserMisc;			//0x8038
	volatile ULONG	UserErrorStatistics;//0x803C
} PED_BAR0_REGISTERS_EX, *PPED_BAR0_REGISTERS_EX;



typedef struct PED_DMA_DESCRIPTOR
{
#define PED_DMA_DESCR_CTRL_IRQ_ON_COMPLETION	(1 << 0)
#define PED_DMA_DESCR_CTRL_IRQ_ON_SHORT_ERR		(1 << 1)
#define PED_DMA_DESCR_CTRL_IRQ_ON_SHORT_SW		(1 << 2)
#define PED_DMA_DESCR_CTRL_IRQ_ON_SHORT_HW		(1 << 3)
#define PED_DMA_DESCR_CTRL_SEQUENCE				(1 << 8)
#define PED_DMA_DESCR_CTRL_CONTINUE				(1 << 9)

#define PED_DMA_DESCR_CADDR_DUMMY				((ULONGLONG)1 << 59)

#define PED_DMA_DESCR_CADDR_SWAP(x)				((ULONGLONG)(x) << 56)

#define PED_DMA_DESCR_CADDR_X_LEN(x)			((ULONGLONG)(x) << 30)
#define PED_DMA_DESCR_CADDR_Y_LEN(x)			((ULONGLONG)(x) << 40)
#define PED_DMA_DESCR_CADDR_Z_LEN(x)			((ULONGLONG)(x) << 48)

#define PED_DMA_DESCR_CTRL_X_DELTA_1(x)			((x) << 12)
#define PED_DMA_DESCR_CTRL_X_DELTA_2(x)			((x) << 17)
#define PED_DMA_DESCR_CTRL_Y_DELTA(x)			((x) << 22)
#define PED_DMA_DESCR_CTRL_Z_DELTA(x)			((x) << 27)

    ULONG		Control;
    ULONG		ByteCount;
    ULONGLONG	SystemAddress;
    ULONGLONG	CardAddress;
    ULONGLONG	NextDescriptor;
} PED_DMA_DESCRIPTOR, *PPED_DMA_DESCRIPTOR;

//#pragma pack(pop)

// DMA engine direction
typedef enum
{
    DmaDir_C2S = 0,	// Card to System
    DmaDir_S2C = 1	// System to Card
} tDmaDir;


/* @doc INTERNAL pci_dma pci_dma_h _DEVICE_TRANSFER
=============================================================================

@struct DEVICE_TRANSFER |

    This structure contains the data associated with a DMA transfer.

*/
typedef struct DEVICE_TRANSFER
{
    //PDMA_ADAPTER	DmaAdapter;

    tDmaDir			DmaDirection;
    ULONG			DmaChannel;
    ULONG			DmaIndex;

	struct
	{
		PPED_DMA_DESCRIPTOR	VirtAddress;
		PHYSICAL_ADDRESS	PhysAddress;
    } DmaDescriptor;

    ULONG			MapRegsAvailable;
    ULONG			NumDescriptors;
    ULONG			UsedDescriptors;

    ULONG			Timeout;

    PVOID			hDmaRequest; //DWG_FIXUP
	BOOLEAN			bStarted;				
} DEVICE_TRANSFER, *PDEVICE_TRANSFER;



/* @doc INTERNAL pci_dma pci_dma_h PCI_MEMORY_RANGE
=============================================================================

@struct PCI_MEMORY_RANGE |

    This structure contains the data associated with a PCI Memory region.

*/
typedef struct PCI_MEMORY_RANGE
{
	ULONG				Length;
	// The number of bytes of memory the card has in this region.
	// Use the <t CARD_MAX_MEMORY_REGIONS> defininition to set the
	// maximum number of expected memory regions

	PHYSICAL_ADDRESS	PhysicalAddress;
	// System physical address assigned to the card's on board memory.

	PVOID				VirtualAddress;
	// Virtual adress of card's memory area.

} PCI_MEMORY_RANGE, *PPCI_MEMORY_RANGE;



typedef struct _PLDMABUF 
{
	// for userland buffer
	int					offset;
	struct page			**pages;

	// for kernel buffer
	void				*vmalloc;

	// for common buffer
	dma_addr_t			commonbuf;	

	// common
	struct scatterlist	*sglist;
	int					sglen;
	int					nr_pages;
	int					direction;

	// user passed in pointer and size
	unsigned long		size;
	void				*data;
} PLDMABUF, *PPLDMABUF;



typedef struct qlpci_device 
{
	// PCIe module object
	CPCIeCntl				*pPCIeCntl;

	struct pci_dev			*pcidev;
	spinlock_t				irq_lock;
	PLDMABUF				dmaBufRd;
	PLDMABUF				dmaBufWr;
	unsigned char			*dmard_buffer;
	unsigned char			*dmawr_buffer;
	dma_addr_t				dmard_dmaddr;
	dma_addr_t				dmawr_dmaddr;

	unsigned long			devconnected;

#define DMA_IDLE 0x00000000
#define DMA_BUSY 0x00000001
#define DMA_COPY 0x00000002

	unsigned long			InterruptStatus;
	unsigned long			InterruptVector;

	void __iomem			*ioaddr[3];      // re-mapped address of BAR0, BAR1, and BAR2

    //PED_DMA_DESCRIPTOR      pDmaDescr;
	PPED_BAR0_REGISTERS		pRegisters;
	PPED_BAR0_REGISTERS_EX	pRegistersEx;

	unsigned char			*DescrVAddrRd;
	unsigned char			*DescrVAddrWr;
	dma_addr_t				DescrAddrRd;
	dma_addr_t				DescrAddrWr;

	LARGE_INTEGER           DmaStartTime;
	LARGE_INTEGER           IrpStartTime;

	// bus callback
	QP_BUSINTF_CALLBACK		pBusCallbackFunc;
	TQP_PVOID				pBusCallbackContext;
} QLPCI_DEVICE, *PQLPCI_DEVICE;


EQP_Errors PedReadReg(PQLPCI_DEVICE pPciDevInfo,     // Address of our DEVICE_EXTENSION.
                      PPCI_RW_RAM_STRUCT pRegStruct  // Address of PCI_RW_RAM_STRUCT describing read operation.
                     );
EQP_Errors PedWriteReg(PQLPCI_DEVICE pPciDevInfo,    // Address of our DEVICE_EXTENSION.
                      PPCI_RW_RAM_STRUCT pRegStruct  // Address of PCI_RW_RAM_STRUCT describing write operation.
                     );
ULONG PedReadMem(PQLPCI_DEVICE pPciDevInfo,    // Address of our DEVICE_EXTENSION.
                 PPCI_RW_RAM_STRUCT pReadInfo, // Address of PCI_RW_RAM_STRUCT describing read operation.
                 PVOID Buffer,                 // Address of memory where read data is to be copied to.
                 ULONG NumBytes                // Number of bytes to be read.
                );
ULONG PedWriteMem(PQLPCI_DEVICE pPciDevInfo,     // Address of our DEVICE_EXTENSION.
                  PPCI_RW_RAM_STRUCT pWriteInfo, // Address of PCI_RW_RAM_STRUCT describing write operation.
                  PVOID Buffer,                  // Address of memory where write data is to be copied from.
                  ULONG NumBytes                 // Number of bytes to be written.
                 );

EQP_Errors PciRemoveDevice(CPCIeCntl *This);

ULONG PedReadMem(PQLPCI_DEVICE pPciDevInfo,     // Address of our qlpci_device.
                 PPCI_RW_RAM_STRUCT pReadInfo,  // Address of PCI_RW_RAM_STRUCT describing read operation.
                 PVOID Buffer,                  // Address of memory where read data is to be copied to.
                 ULONG NumBytes                 // Number of bytes to be read.
                );
 
ULONG PedWriteMem(PQLPCI_DEVICE pPciDevInfo,      // Address of our qlpci_device.
                  PPCI_RW_RAM_STRUCT pWriteInfo,  // Address of PCI_RW_RAM_STRUCT describing write operation.
                  PVOID Buffer,                   // Address of memory where write data is to be copied from.
                  ULONG NumBytes                  // Number of bytes to be written.
                 );

EQP_Errors PedReadReg(PQLPCI_DEVICE pPciDevInfo,    // Address of qlpci_device
                      PPCI_RW_RAM_STRUCT pRegStruct // Address of PCI_RW_RAM_STRUCT describing read operation.
                     );

EQP_Errors PedWriteReg(PQLPCI_DEVICE pPciDevInfo,   // Address of qlpci_device
                       PPCI_RW_RAM_STRUCT pRegStruct // Address of PCI_RW_RAM_STRUCT describing read operation.
                      );
long PedDMARead(PQLPCI_DEVICE pPciDevInfo,	// Address of our qlpci_device.
				TQP_UINT32 ulAddr,			// in WORD(32 bit)
				TQP_UINT32 ulLength,		// in WORD(32 bit)
				PTQP_UINT8 pHostAddr,
				TQP_UINT32 ulXferMode,
				TQP_UINT32 ulPicWidthMB,
				TQP_UINT32 ulPicHeightMB,
				TQP_UINT32 ulSwap,
				TQP_BOOL IsFrameMode
				);
long PedDMAWrite(PQLPCI_DEVICE pPciDevInfo,	// Address of our qlpci_device.
				 TQP_UINT32 ulAddr,			// in WORD(32 bit)
				 TQP_UINT32 ulLength,		// in WORD(32 bit)
				 PTQP_UINT8 pHostAddr,
				 TQP_UINT32 ulXferMode,
				 TQP_UINT32 ulPicWidthMB,
				 TQP_UINT32 ulPicHeightMB,
				 TQP_UINT32 ulSwap,
				 TQP_BOOL IsFrameMode
				 );
void PedDMAReadComplete(PQLPCI_DEVICE pPciDevInfo);
void PedDMAWriteComplete(PQLPCI_DEVICE pPciDevInfo);
EQP_Errors PciStartDevice(PQLPCI_DEVICE pPciDevInfo);
EQP_Errors PciStopDevice(PQLPCI_DEVICE pPciDevInfo);

EQP_Errors PciAddDevice(CPCIeCntl *This,
                        TQP_PVOID FunctionalDeviceObject,
                        TQP_PVOID PhysicalDeviceObject
                       );

void PciInitializeDeviceState(PQLPCI_DEVICE pPciDevInfo);

#endif

