///////////////////////////////////////////////////////////////////////////////
//        
//
// pci_config.h
//
// Description: 
//
//  This is the PCIe device configuration defination
// 
// History:
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#ifndef __PCI_CONFIG_H__
#define __PCI_CONFIG_H__

//peter
#include "../../LINUXV4L2.h" 

#define _DWG_CONFIG_CHECK
#define _DWG_HW_GEN3


// Bar Types
typedef enum _tPciBarType
{
    RegisterMemBarType  = 0, // Memory Bar with Registers
    RamMemBarType       = 1, // Memory Bar with RAM
    RegisterIoBarType   = 2, // IO Bar with Registers
    DisabledBarType     = 3  // Disabled Bar
} tPciBarType, *ptPciBarType;

//
// Transfer Size Definitions
typedef USHORT tPciTransferSize, *ptPciTransferSize;

#define transSize8Bit           0x01    // 8 Bit transfer
#define transSize16Bit          0x02    // 16 Bit transfer
#define transSize32Bit          0x04    // 32 Bit transfer
#define transSize64Bit          0x08    // 64 Bit transfer (not currently supported)
#define transSize32BitDma       0x10    // 32 Bit transfer
#define transSize64BitDma       0x20    // 64 Bit transfer

//
// DMA Direction Definitions
typedef USHORT tDmaDirection, *ptDmaDirection;

#define dmaDirRead              0x01    // Read from card (C2S)
#define dmaDirWrite             0x02    // Write to card (S2C)

#define PCI_TRANSFER_TYPES      (transSize8Bit | transSize16Bit | transSize32Bit)

#define PCI_MAX_BARS            5+1     // One extra for dummy DMA bar
#define PCI_MAX_DMAS            64
//#define PCI_NUM_DESCR           256
//#define PCI_NUM_DESCR           1024 
#define PCI_NUM_DESCR           4098
#define PCI_DESCR_MAX_LEN       ((PCI_NUM_DESCR - 2) * PAGE_SIZE)
#define PCI_DMA_CACHE_ENABLED   FALSE


#pragma pack(push,1)

//
//  Write/Read RAM Definitions
typedef struct PCI_RW_RAM_STRUCT
{
    ULONG                   bar;
    tPciTransferSize        transferSize;   // size of atomic operations
    USHORT                  repeatCount;    // number of times to repeat the transfer
    ULONGLONG               offset;         // offset into the Ram Block
    ULONGLONG               data;           // for single transfers
    ULONGLONG               transferTime;   // Number of nano-seconds for transfer
    ULONGLONG               driverTime;     // Number of nano-seconds for driver (includes transfer)

	// extra information for MB <-> RAS conversion
	// 
	ULONG					TransferMode;
	ULONG					PicWidthMB;
	ULONG					PicHeightMB;
	ULONG					DataSwap;
	BOOLEAN					IsFrameMode;
} PCI_RW_RAM_STRUCT, *PPCI_RW_RAM_STRUCT;

typedef PVOID               PCI_DMA_BUFFER_HANDLE;
typedef PVOID               PCI_DMA_HANDLE;

typedef ULONG               PCI_DMA_XFER_SIZE;
typedef ULONG               PCI_DMA_XFER_DIRECTION;

typedef struct PCI_PERFORMANCE_STRUCT
{
    ULONGLONG               hardwareTime;   // Number of nano-seconds for hardware
    ULONGLONG               driverTime;     // Number of nano-seconds for driver
} PCI_PERFORMANCE_STRUCT, *PPCI_PERFORMANCE_STRUCT;

typedef struct PCI_DMA_BUFFER_STRUCT
{
    PVOID                   BufferAddress;
    ULONG                   BufferLength;
    BOOLEAN                 IsWrite;
} PCI_DMA_BUFFER_STRUCT, *PPCI_DMA_BUFFER_STRUCT;

typedef struct PCI_DMA_START_STRUCT
{
    PCI_DMA_BUFFER_HANDLE   DmaBufferHandle;
    ULONG                   Bar;
    ULONGLONG               CardAddress;
    ULONG                   BufferOffset;
    ULONG                   BufferLength;
    PCI_DMA_XFER_SIZE       TransferSize;
    ULONG                   DmaChannel;
    PCI_PERFORMANCE_STRUCT *Performance;

	// extra information for MB <-> RAS conversion
	// 
	ULONG					TransferMode;
	ULONG					PicWidthMB;
	ULONG					PicHeightMB;
	ULONG					DataSwap;
	BOOLEAN					IsFrameMode;

} PCI_DMA_START_STRUCT, *PPCI_DMA_START_STRUCT;

typedef struct CARD_CONFIG_DMA_INFO
{
    tPciTransferSize		TransferSize;
    tDmaDirection			Direction;
    ULONG					MaxDmaSize;
} CARD_CONFIG_DMA_INFO, *PCARD_CONFIG_DMA_INFO;

//
// Card Configuration Information
typedef struct CARD_CONFIG_INFO
{
    USHORT					CardConfigId;
    USHORT					VendorID;
    USHORT					DeviceID;
    USHORT					InterruptAvailable;
    ULONG					DriverVersion;          // (MAJOR << 24) + (MINOR << 16) + (SUBMINOR << 8) + BUILD
    ULONG					FpgaVersion;
    ULONG					NumDmaChannels;
    CARD_CONFIG_DMA_INFO	DmaChannel[1];
} CARD_CONFIG_INFO, *PCARD_CONFIG_INFO;

// CardConfigId values
static const USHORT CardConfigId_StandardPci = 1;
static const USHORT CardConfigId_PciExpress  = 2;

// Vendor and Device IDs
static const USHORT QP_PCI_VendorID		= 0x1AF2;
static const USHORT QP_PCI_DeviceID		= 0xA001;


static const ULONG  PCI_BAR_TYPE_MASK       = 0x1;
static const ULONG  PCI_BAR_TYPE_MEM        = 0x0;
static const ULONG  PCI_BAR_TYPE_IO         = 0x1;
static const ULONG  PCI_BAR_MEM_ADDR_MASK   = 0x6;
static const ULONG  PCI_BAR_MEM_ADDR_32     = 0x0;
static const ULONG  PCI_BAR_MEM_ADDR_1M     = 0x2;
static const ULONG  PCI_BAR_MEM_ADDR_64     = 0x4;
static const ULONG  PCI_BAR_MEM_CACHE_MASK  = 0x8;
static const ULONG  PCI_BAR_MEM_CACHABLE    = 0x8;

#define PCI_BAR_MEM_MASK        (~0x0F)
#define PCI_BAR_IO_MASK         (~0x03)
#define IS_IO_BAR(x)            (((x) & PCI_BAR_TYPE_MASK) == PCI_BAR_TYPE_IO)
#define IS_MEMORY_BAR(x)        (((x) & PCI_BAR_TYPE_MASK) == PCI_BAR_TYPE_MEM)
#define IS_MEMORY_32BIT(x)      (((x) & PCI_BAR_MEM_ADDR_MASK) == PCI_BAR_MEM_ADDR_32)
#define IS_MEMORY_64BIT(x)      (((x) & PCI_BAR_MEM_ADDR_MASK) == PCI_BAR_MEM_ADDR_64)
#define IS_MEMORY_BELOW1M(x)    (((x) & PCI_BAR_MEM_ADDR_MASK) == PCI_BAR_MEM_ADDR_1M)
#define IS_MEMORY_CACHABLE(x)   (((x) & PCI_BAR_MEM_CACHE_MASK) == PCI_BAR_MEM_CACHABLE)


//
// PCI Configuration Header
typedef struct _PCI_CONFIG_HEADER
{
    // The following fields contain the first 64 bytes of PCI configuration space.
    USHORT					VendorID;
    USHORT					DeviceID;
    USHORT					Command;
    USHORT					Status;
    UCHAR					RevisionID;
    UCHAR					ProgIf;
    UCHAR					SubClass;
    UCHAR					BaseClass;
    UCHAR					CacheLineSize;
    UCHAR					LatencyTimer;
    UCHAR					HeaderType;
    UCHAR					BIST;
#define PCI_TYPE0_ADDRESS_CNT 6         // TYPE-0 configuration header values.
    ULONG					RawBaseAddresses[PCI_TYPE0_ADDRESS_CNT];
    ULONG					CardBusCISPtr;
    USHORT					SubsystemVendorID;
    USHORT					SubsystemID;
    ULONG					ROMBaseAddress;
    UCHAR					CapabilitiesPtr;
    UCHAR					Reserved1[3];
    ULONG					Reserved2[1];
    UCHAR					InterruptLine;
    UCHAR					InterruptPin;
    UCHAR					MinimumGrant;
    UCHAR					MaximumLatency;
#define PCI_DEVICE_SPECIFIC_SIZE 192    // Device specific bytes [64..256].
    UCHAR					DeviceSpecific[PCI_DEVICE_SPECIFIC_SIZE];

    // The following fields are decoded values from the PCI configuration space.
    // Make sure these start on an 8-byte boundary.
    ULONGLONG				BaseAddresses[PCI_MAX_BARS];
    ULONGLONG				BaseSizes[PCI_MAX_BARS];
    tPciBarType				BarType[PCI_MAX_BARS];
    USHORT					BarTransferTypes[PCI_MAX_BARS];
    ULONG					AddressSize;
} PCI_CONFIG_HEADER, *PPCI_CONFIG_HEADER;

#pragma pack(pop)

#endif //__PCI_CONFIG_H__


