///////////////////////////////////////////////////////////////////////////////
//        
//
// ped_dma.h
//
// Description: 
//
//  This is the implementation of the PCIe interface
// 
// History:
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


/*
=============================================================================

    (C) Copyright 2005-2006 Northwest Logic -- http://www.nwlogic.com
         

  Portions of this software are:

    (C) Copyright 2004 TriplePoint, Inc. -- http://www.TriplePoint.com
        This copyright notice must be maintained in all derivative works.

=============================================================================

@doc EXTERNAL ped_dma ped_dma_h

@module ped_dma.h |

    This module defines all the PciExpress interfaces supported by the driver.

@head3 Contents |
@index class,mfunc,func,msg,mdata,struct,enum | ped_dma_h

@end
=============================================================================
*/

#ifndef _PED_DMA_H_
#define _PED_DMA_H_

#pragma pack(push,1)

#define PED_IRQ_DMA_COMPLETE(x)	(0x0001 << (x))
#define PED_IRQ_DMA_ENABLE(x)	(0x0100 << (x))
#define PED_IRQ_UNUSED			(~0x0F0F)

#define PED_DMA_IS_BUSY(Status)	(Status & PED_STAT_CHAIN_RUNNING)

#pragma pack(pop)

VOID PedCopyPciConfig(OUT PPCI_CONFIG_HEADER PciHeader,			// Address of driver PCI information.
					  IN PPCI_COMMON_CONFIG  PciConfig			// Address of PCI configuration read from bus driver.
					  );
NTSTATUS PedReadReg(PPCIE_DEVICE_EXTENSION DeviceExtension,		// Address of our DEVICE_EXTENSION.
					PPCI_RW_RAM_STRUCT pRegStruct				// Address of PCI_RW_RAM_STRUCT describing read operation.
					);
NTSTATUS PedWriteReg(PPCIE_DEVICE_EXTENSION DeviceExtension,	// Address of our DEVICE_EXTENSION.
					 PPCI_RW_RAM_STRUCT pRegStruct				// Address of PCI_RW_RAM_STRUCT describing write operation.
					 );
ULONG PedReadMem(PPCIE_DEVICE_EXTENSION DeviceExtension,		// Address of our DEVICE_EXTENSION.
				 PPCI_RW_RAM_STRUCT pReadInfo,					// Address of PCI_RW_RAM_STRUCT describing read operation.
				 IN PVOID Buffer,								// Address of memory where read data is to be copied to.
				 IN ULONG NumBytes								// Number of bytes to be read.
				 );
ULONG PedWriteMem(PPCIE_DEVICE_EXTENSION DeviceExtension,		// Address of our DEVICE_EXTENSION.
				  PPCI_RW_RAM_STRUCT pWriteInfo,				// Address of PCI_RW_RAM_STRUCT describing write operation.
				  IN PVOID Buffer,								// Address of memory where write data is to be copied from.
				  IN ULONG NumBytes								// Number of bytes to be written.
				  );
NTSTATUS PedDmaInit(PPCIE_DEVICE_EXTENSION DeviceExtension);	// Address of our PCIE_DEVICE_EXTENSION
VOID PedDmaComplete(PPCIE_DEVICE_EXTENSION DeviceExtension,		// Address of our PCIE_DEVICE_EXTENSION.
					ULONG DmaIndex,								// Index of DMA block (0 0r 1).
					BOOLEAN bWasError							// TRUE if there was a DMA error.
					);
VOID PedDmaDpcHandler(PPCIE_DEVICE_EXTENSION DeviceExtension);	// Address of our PCIE_DEVICE_EXTENSION.
VOID PedDmaTimeout(PPCIE_DEVICE_EXTENSION DeviceExtension);		// Address of our PCIE_DEVICE_EXTENSION.
NTSTATUS PedDmaBufferAlloc(PPCIE_DEVICE_EXTENSION DeviceExtension,	// Address of our PCIE_DEVICE_EXTENSION.
						   PCI_DMA_BUFFER_STRUCT *pDmaInfo,		// DMA buffer information.
						   PCI_DMA_BUFFER **ppDmaBuffer,		// Address of return DMA buffer.
						   MDL *pMdlAddressCaller				// Address of user provided MDL
						   );
VOID PedDmaBufferFree(PCI_DMA_BUFFER *pDmaBuffer);				// Address of DMA buffer to free
NTSTATUS PedDmaBufferLock(PCI_DMA_BUFFER *pDmaBuffer,			// Address of return DMA buffer.
						  PCI_DMA_GET_SGLIST_CB pGetSgListCB,	// Pointer to the scatter gather list callback function.
						  PVOID pGetSgListUD					// Pointer to user data passed to the scatter gather list
																// callback function.
						  );
VOID PedDmaBufferUnlock(PCI_DMA_BUFFER *pDmaBuffer);			// Address of DMA buffer to free
NTSTATUS PedDmaRequestAlloc(PCI_DMA_BUFFER *pDmaBuffer,			// Caller-supplied pointer to a DMA buffer structure.
							ULONG bufferLength,					// Length of the DMA request.
							ULONG numDescriptors,				// Number of DMA descriptors needed for the DMA request.
							PCI_PERFORMANCE_STRUCT *pPerformance,	// Pointer to optional performance parameters.
							PCI_DMA_REQUEST_CB pRequestCB,		// Pointer to the request callback function.
							PCI_DMA_REQUEST **ppDmaRequest		// Address of the return DMA request.
							);
VOID PedDmaRequestFree(PCI_DMA_REQUEST *pDmaRequest);			// Address of DMA request to free
NTSTATUS PedDmaQueueBuffers(PCI_DMA_BUFFER *pDmaBuffer,			// Caller-supplied pointer to a DMA buffer structure.
							PCI_DMA_START_STRUCT *pDmaStart,	// Caller-supplied pointer to a DMA start structure.
							PCI_DMA_REQUEST_CB pRequestCB		// Pointer to optional request callback function.
							);
NTSTATUS PedDmaRelease(PPCIE_DEVICE_EXTENSION DeviceExtension);	// Address of our PCIE_DEVICE_EXTENSION.
VOID PedDmaRequestComplete(PCI_DMA_REQUEST *pDmaRequest);		// Address of DMA request.
VOID PedDmaStartLockBuffersComplete(PCI_DMA_BUFFER *pDmaBuffer,	// Address of DMA buffer descriptor.
									NTSTATUS Status				// Status of the get scatter gather function.
									);

#endif /* _PED_DMA_H_ */

