///////////////////////////////////////////////////////////////////////////////
//     
//
// Cusbcntl.c
//
// Description:
//
//  This is the implementation of the USB controller(FX2) interface
//
// History:
//	Aug 25, 2006 (dt) created
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#include "cusbcntl.h"
#include "cqlcodec.h"
#include "cqlcodeclib.h"
#include "iqlusbfw.h"
#include "qpdbgmgr.h"
#include "qpmemmgr.h"
#include "util.h"
#ifdef QP_LINXKER
#include "qlautoconf.h"
#include <linux/string.h>
#else
#include <string.h>
#endif
#include <linux/slab.h>

extern TQP_DbgMCompID	g_QPUsbDBGCompID;

/////////////////////////////////////////////////////////////////////////////
//
//

#define USB_PIPE_0			0
#define USB_PIPE_1			1
#define USB_PIPE_2			2
#define USB_PIPE_3			3

#define USB_PIPE_INVALID	0x0F


/////////////////////////////////////////////////////////////////////////////
//
//

EQP_Errors CUsbCntl_GenericCmd(CUsbCntl *This,
							   PTQP_UINT8 pOutBuf,
							   TQP_UINT32 dwOutSize,
							   PTQP_UINT8 pInBuf,
							   TQP_UINT32 dwInSize,
							   TQP_BOOL bCritical
							   )
{
#if defined(QP_LINXKER) && !defined(CONFIG_QL300_USB_EVK)
        return (QPERR_NOTIMPL);
#else
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	TQP_UINT32	dwXfered = 0;
	TQP_UINT32	i;

	if (bCritical)
	{
		ENTER_CRITICAL(&This->m_Object)
	}

	if (dwOutSize)
	{
		qpStatus = QPUsbInterface_UsbSendCmd(&This->m_pUsb->m_iUsbInterface,
											 This->m_dwPipeCmdWrite,
											 pOutBuf,
											 dwOutSize,
											 &dwXfered
											 );

		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, /*QPDBGM_LEVEL_ERROR*/QPDBGM_LEVEL_EXTRACE),
						 "CUsbCntl_GenericCmd() write pipe(%d) Failed Status(%d) xfered(%d)\n",
						 This->m_dwPipeCmdWrite,
						 qpStatus,
						 dwXfered
						 ));
		}
	}

	if (QP_SUCCESS(qpStatus) &&
		dwInSize
		)
	{
		qpStatus = QPUsbInterface_UsbSendCmd(&This->m_pUsb->m_iUsbInterface,
											 This->m_dwPipeCmdRead,
											 pInBuf,
											 dwInSize,
											 &dwXfered
											 );
		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, /*QPDBGM_LEVEL_ERROR*/QPDBGM_LEVEL_EXTRACE),
						 "CUsbCntl_GenericCmd() read pipe(%d) Failed Status(%d) xfered(%d)\n",
						 This->m_dwPipeCmdRead,
						 qpStatus,
						 dwXfered
						 ));
		}

		This->m_dwCmdInNextToggle ^= 1;
	}

	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, /*QPDBGM_LEVEL_ERROR*/QPDBGM_LEVEL_EXTRACE),
					 "CUsbCntl_GenericCmd() write len (%d) read(%d) [\n",
					 dwOutSize,
					 dwInSize
					 ));

		for (i = 0; i < dwOutSize; i++)
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, /*QPDBGM_LEVEL_ERROR*/QPDBGM_LEVEL_EXTRACE),
						  "(0x%x)",
						  pOutBuf[i]
						  ));
		}

		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, /*QPDBGM_LEVEL_ERROR*/QPDBGM_LEVEL_EXTRACE),
					 "CUsbCntl_GenericCmd() ]\n"
					 ));
	}

	if (bCritical)
	{
		LEAVE_CRITICAL(&This->m_Object)
	}

	return (qpStatus);
#endif
}



EQP_Errors CUsbCntl_ReadHciRegister(CUsbCntl *This,
									TQP_UINT8 cAddr,
									PTQP_UINT8 pData // can not use directly
									)
{
#if defined(QP_LINXKER) && !defined(CONFIG_QL300_USB_EVK)
        return (QPERR_NOTIMPL);
#else
	EQP_Errors		qpStatus;
//	USBFW_COMMAND	usbcmd;

	//peterlin
	PUSBFW_COMMAND	p_cmd = kmalloc( sizeof(USBFW_COMMAND), GFP_KERNEL );

	if( !p_cmd )
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_ReadHciRegister() no p_cmd\n"));
	}
	else
	{
		memset( p_cmd, 0x00, sizeof(USBFW_COMMAND) );
	}

	PTQP_UINT8 pData_temp = kmalloc( sizeof(TQP_UINT8), GFP_KERNEL );
	if( !pData_temp )
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_ReadHciRegister() no pData_temp\n"));
	}
	else
	{
		memcpy( pData_temp, pData, sizeof(TQP_UINT8) );
	}

	if (!pData)
	{
		return (QPERR_PARMS);
	}

	p_cmd->cmd_type = QLUSBFW_HIU_REGISTER;
	p_cmd->cmd_rw = 0;
	p_cmd->cmd_data_len = 1;
	p_cmd->pad.addr32 = cAddr;

	qpStatus = CUsbCntl_GenericCmd(This,
								   (PTQP_UINT8)p_cmd,
								   sizeof(USBFW_COMMAND),
								   pData_temp,
								   1,
								   QP_TRUE
								   );
	memcpy( pData, pData_temp, sizeof(TQP_UINT8) );

	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, /*QPDBGM_LEVEL_ERROR*/QPDBGM_LEVEL_EXTRACE),
					 "CUsbCntl_ReadHciRegister() GenericCmd Failed Status(%d)\n",
					 qpStatus
					 ));
	}

	if( pData_temp )
	{
		kfree(pData_temp);
		pData_temp = NULL;
	}

	if(p_cmd)
	{
		kfree(p_cmd);
		p_cmd = NULL;
	}
	return (qpStatus);
#endif
}



EQP_Errors CUsbCntl_WriteHciRegister(CUsbCntl *This,
									 TQP_UINT8 cAddr,
									 TQP_UINT8 ucData
									 )
{
#if defined(QP_LINXKER) && !defined(CONFIG_QL300_USB_EVK)
        return (QPERR_NOTIMPL);
#else
	EQP_Errors		qpStatus;
	
	//TQP_UINT8		usb_cmd[sizeof(USBFW_COMMAND) + sizeof(TQP_UINT8)];
	//peterlin
	TQP_UINT8* usb_cmd = kmalloc( sizeof(USBFW_COMMAND) + sizeof(TQP_UINT8), GFP_KERNEL );
	if( !usb_cmd )
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_WriteHciRegister() no usb_cmd\n"));
	}
	else
	{
		memset( usb_cmd, 0x00, sizeof(USBFW_COMMAND) + sizeof(TQP_UINT8) );
	}
	
	PUSBFW_COMMAND	p_cmd = (USBFW_COMMAND*)usb_cmd;

	p_cmd->cmd_type = QLUSBFW_HIU_REGISTER;
	p_cmd->cmd_rw = 1;
	p_cmd->cmd_data_len = sizeof(TQP_UINT8);
	p_cmd->pad.addr32 = cAddr;

	usb_cmd[sizeof(USBFW_COMMAND)] = ucData;

	qpStatus = CUsbCntl_GenericCmd(This,
								   (PTQP_UINT8)p_cmd,
								   //sizeof(usb_cmd),
								   sizeof(USBFW_COMMAND) + sizeof(TQP_UINT8),
								   QP_NULL,
								   0,
								   QP_TRUE
								   );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_WriteHciRegister() GenericCmd Failed Status(%d)\n",
					 qpStatus
					 ));
	}
	if(usb_cmd)
	{
		kfree(usb_cmd);
		usb_cmd = NULL;
	}


	return (qpStatus);
#endif
}



EQP_Errors CUsbCntl_ReadHciDWORD(CUsbCntl *This,
								 TQP_UINT8 cAddr,
								 PTQP_UINT32 pData,//can not use directly
								 TQP_BOOL bCritical
								 )
{
#if defined(QP_LINXKER) && !defined(CONFIG_QL300_USB_EVK)
        return (QPERR_NOTIMPL);
#else
	EQP_Errors		qpStatus;
	//USBFW_COMMAND	usbcmd;

	//peterlin
	PUSBFW_COMMAND	p_cmd = kmalloc( sizeof(USBFW_COMMAND), GFP_KERNEL );
	
	if( !p_cmd )
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_ReadHciDWORD() no p_cmd\n"));
	}
	else
	{
		memset( p_cmd, 0x00, sizeof(USBFW_COMMAND) );
	}

	PTQP_UINT32 pData_temp = kmalloc( sizeof(TQP_UINT32), GFP_KERNEL );
	if( !pData_temp )
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_ReadHciDWORD() no pData_temp\n"));
	}
	else
	{
		memcpy( pData_temp, pData, sizeof(TQP_UINT32) );
	}

	if (!pData)
	{
		return (QPERR_PARMS);
	}

	p_cmd->cmd_type = QLUSBFW_HIU_REGISTER;
	p_cmd->cmd_rw = 0;
	p_cmd->cmd_data_len = sizeof(TQP_UINT32);
	p_cmd->pad.addr32 = cAddr;

	qpStatus = CUsbCntl_GenericCmd(This,
								   (PTQP_UINT8)p_cmd,
								   sizeof(USBFW_COMMAND),
								   (PTQP_UINT8)pData_temp,
								   sizeof(TQP_UINT32),
								   bCritical
								   );
	memcpy( pData, pData_temp, sizeof(TQP_UINT32) );

	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_ReadHciDWORD() GenericCmd Failed Status(%d)\n",
					 qpStatus
					 ));
	}
	if( pData_temp )
	{
		kfree(pData_temp);	
		pData_temp = NULL;
	}
	if(p_cmd)
	{
		kfree(p_cmd);	
		p_cmd = NULL;
	}

	return (qpStatus);
#endif
}



EQP_Errors CUsbCntl_WriteHciDWORD(CUsbCntl *This,
								  TQP_UINT8 cAddr,
								  TQP_UINT32 dwData,
								  TQP_BOOL bCritical
								  )
{
#if defined(QP_LINXKER) && !defined(CONFIG_QL300_USB_EVK)
        return (QPERR_NOTIMPL);
#else
	EQP_Errors		qpStatus;
	//TQP_UINT8		usb_cmd[sizeof(USBFW_COMMAND) + sizeof(TQP_UINT32)];
	//peterlin
	TQP_UINT8* usb_cmd = kmalloc( sizeof(USBFW_COMMAND) + sizeof(TQP_UINT32), GFP_KERNEL );
	if( !usb_cmd )
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_WriteHciDWORD() no usb_cmd\n"));
	}
	else
	{
		memset( usb_cmd, 0x00, sizeof(USBFW_COMMAND) + sizeof(TQP_UINT32) );
	}

	PUSBFW_COMMAND	p_cmd = (USBFW_COMMAND*)usb_cmd;

	p_cmd->cmd_type = QLUSBFW_HIU_REGISTER;
	p_cmd->cmd_rw = 1;
	p_cmd->cmd_data_len = sizeof(TQP_UINT32);
	p_cmd->pad.addr32 = cAddr;

	*(PTQP_UINT32)&usb_cmd[sizeof(USBFW_COMMAND)] = dwData;

	qpStatus = CUsbCntl_GenericCmd(This,
								   (PTQP_UINT8)p_cmd,
								   //sizeof(usb_cmd),
								   sizeof(USBFW_COMMAND) + sizeof(TQP_UINT32),
								   QP_NULL,
								   0,
								   bCritical
								   );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_WriteHciDWORD() GenericCmd Failed Status(%d)\n",
					 qpStatus
					 ));
	}
	if(usb_cmd)
	{
		kfree(usb_cmd);	
		usb_cmd = NULL;
	}

	return (qpStatus);
#endif
}




EQP_Errors CUsbCntl_RegisterRead(CUsbCntl *This,
								 TQP_UINT16 wAddr,
								 PTQP_UINT32 pData //can not use directly
								 )
{
#if defined(QP_LINXKER) && !defined(CONFIG_QL300_USB_EVK)
        return (QPERR_NOTIMPL);
#else
	EQP_Errors		qpStatus;
//peterlin ????
	//USBFW_COMMAND	usbcmd;
	PUSBFW_COMMAND	p_cmd = kmalloc( sizeof(USBFW_COMMAND), GFP_KERNEL );
	if( !p_cmd )
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_RegisterRead() no p_cmd\n"));
	}
	else
	{
		memset( p_cmd, 0x00, sizeof(USBFW_COMMAND) );
	}

	PTQP_UINT32 pData_temp = kmalloc( sizeof(TQP_UINT32), GFP_KERNEL );
	if( !pData_temp )
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_RegisterRead() no pData_temp\n"));
	}
	else
	{
		memset( pData_temp, 0x00, sizeof(TQP_UINT32) );
	}

	memcpy( pData_temp, pData, sizeof(TQP_UINT32) );

	if (!pData)
	{
		return (QPERR_PARMS);
	}

	p_cmd->cmd_type = QLUSBFW_ARM_REGISTER;
	p_cmd->cmd_rw = 0;
	p_cmd->cmd_data_len = 1; //sizeof(TQP_UINT32);
	p_cmd->pad.addr32 = (QPHCI_MODE_INDIRECT == This->m_accessMode) ? (wAddr & 0x7FFF) : wAddr;	// make sure we bit is cleared

	qpStatus = CUsbCntl_GenericCmd(This,
								   (PTQP_UINT8)p_cmd,
								   sizeof(USBFW_COMMAND),
								   (PTQP_UINT8)pData_temp,
								   sizeof(TQP_UINT32),
								   QP_TRUE
								   );

	memcpy( pData, pData_temp, sizeof(TQP_UINT32) );

	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_RegisterRead() GenericCmd Failed Status(%d)\n",
					 qpStatus
					 ));
	}
	else
	{
		//QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
		//			 "CUsbCntl_RegisterRead() GenericCmd Success Status(%d) wAddr(0x%x) *pData(0x%x)\n",
		//			 qpStatus, wAddr, *pData
		//			 ));
	}

	if(pData_temp)
	{
		kfree(pData_temp);
		pData_temp = NULL;
	}

	if(p_cmd)
	{
		kfree(p_cmd);
		p_cmd = NULL;
	}

	return (qpStatus);
#endif
}



EQP_Errors CUsbCntl_RegisterWrite(CUsbCntl *This,
								  TQP_UINT16 wAddr,
								  TQP_UINT32 dwData
								  )
{
#if defined(QP_LINXKER) && !defined(CONFIG_QL300_USB_EVK)
        return (QPERR_NOTIMPL);
#else
	EQP_Errors		qpStatus;
//peterlin ??????????
//	TQP_UINT8		usb_cmd[sizeof(USBFW_COMMAND) + sizeof(TQP_UINT32)];

	TQP_UINT8* usb_cmd = kmalloc( sizeof(USBFW_COMMAND) + sizeof(TQP_UINT32), GFP_KERNEL );
	if( !usb_cmd )
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_RegisterWrite() no usb_cmd\n"));
	}
	else
	{
		memset( usb_cmd, 0x00, sizeof(USBFW_COMMAND) + sizeof(TQP_UINT32) );
	}

	PUSBFW_COMMAND	p_cmd = (USBFW_COMMAND*)usb_cmd;

	//QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
	//			 "CUsbCntl_RegisterWrite() (llx%llx) sizeof(USBFW_COMMAND) + sizeof(TQP_UINT32)(%d)\n",
	//			 p_cmd,
	//			 sizeof(USBFW_COMMAND) + sizeof(TQP_UINT32)
	//			 ));

	p_cmd->cmd_type = QLUSBFW_ARM_REGISTER;
	p_cmd->cmd_rw = 1;
	p_cmd->cmd_data_len = 1; //sizeof(TQP_UINT32);
	p_cmd->pad.addr32 = (QPHCI_MODE_INDIRECT == This->m_accessMode) ? (wAddr | 0x8000) : wAddr;	// set we bit

	*((PTQP_UINT32)&usb_cmd[sizeof(USBFW_COMMAND)]) = dwData;

	qpStatus = CUsbCntl_GenericCmd(This,
								   (PTQP_UINT8)p_cmd,
								   //sizeof(usb_cmd),
									sizeof(USBFW_COMMAND) + sizeof(TQP_UINT32),
									QP_NULL,
								   0,
								   QP_TRUE
								   );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_RegisterWrite() GenericCmd Failed Status(%d) wAddr(0x%x) dwData(0x%x)\n",
					 qpStatus, wAddr, dwData
					 ));
	}
	else
	{
		//QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
		//			 "CUsbCntl_RegisterWrite() GenericCmd Success Status(%d) wAddr(0x%x) dwData(0x%x)\n",
		//			 qpStatus, wAddr, dwData
		//			 ));
	}

	//peterlin
	if(usb_cmd)
	{
		kfree(usb_cmd);
		usb_cmd = NULL;
	}
	return (qpStatus);
#endif
}


#define MAX_NUM_FOR_REGISTER_WRITE_EX	32		// we are using 13 so far.

EQP_Errors CUsbCntl_RegisterWriteEx(CUsbCntl *This,
								  TQP_UINT16 wAddr,
								  PTQP_UINT32 pData,
								  TQP_UINT32 numReg
								  )
{
#if defined(QP_LINXKER) && !defined(CONFIG_QL300_USB_EVK)
        return (QPERR_NOTIMPL);
#else
	EQP_Errors		qpStatus;
	//TQP_UINT8		usb_cmd[sizeof(USBFW_COMMAND) + (MAX_NUM_FOR_REGISTER_WRITE_EX*sizeof(TQP_UINT32))];
	//peterlin
	TQP_UINT8* usb_cmd = kmalloc( sizeof(USBFW_COMMAND) + (MAX_NUM_FOR_REGISTER_WRITE_EX*sizeof(TQP_UINT32)), GFP_KERNEL );
	if( !usb_cmd )
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_RegisterWriteEx() no usb_cmd\n"));
	}
	else
	{
		memset( usb_cmd, 0x00, sizeof(USBFW_COMMAND) + (MAX_NUM_FOR_REGISTER_WRITE_EX*sizeof(TQP_UINT32)) );
	}

	PUSBFW_COMMAND	p_cmd = (USBFW_COMMAND*)usb_cmd;

	if (numReg > MAX_NUM_FOR_REGISTER_WRITE_EX)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_RegisterWriteEx() numReg(%d) should no more than %d\n",
					 numReg,
					 MAX_NUM_FOR_REGISTER_WRITE_EX
					 ));
		return (QPERR_PARMS);
	}

	p_cmd->cmd_type = QLUSBFW_HCI_CMD;
	p_cmd->cmd_rw = 1;
	p_cmd->cmd_data_len = (TQP_UINT16)numReg;
	p_cmd->pad.addr32 = (QPHCI_MODE_INDIRECT == This->m_accessMode) ? (wAddr | 0x8000) : wAddr;	// set we bit

	memcpy(&usb_cmd[sizeof(USBFW_COMMAND)], (PTQP_UINT8)pData, numReg*sizeof(TQP_UINT32));

	qpStatus = CUsbCntl_GenericCmd(This,
								   (PTQP_UINT8)p_cmd,
								   sizeof(USBFW_COMMAND) + (numReg*sizeof(TQP_UINT32)),
								   QP_NULL,
								   0,
								   QP_TRUE
								   );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_RegisterWriteEx() GenericCmd Failed Status(%d)\n",
					 qpStatus
					 ));
	}
	if(usb_cmd)
	{
		kfree(usb_cmd);	
		usb_cmd = NULL;
	}


	return (qpStatus);
#endif
}



EQP_Errors CUsbCntl_RegisterReadEx(CUsbCntl *This,
								   TQP_UINT16 wAddr,
								   PTQP_UINT32 pData, //can not use directly
								   TQP_UINT32 numReg
								   )
{
#if defined(QP_LINXKER) && !defined(CONFIG_QL300_USB_EVK)
        return (QPERR_NOTIMPL);
#else
	EQP_Errors		qpStatus;
	//USBFW_COMMAND	usbcmd;
	//peterlin
	PUSBFW_COMMAND	p_cmd = kmalloc( sizeof(USBFW_COMMAND), GFP_KERNEL );
	
	if( !p_cmd )
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_RegisterReadEx() no p_cmd\n"));
	}
	else
	{
		memset( p_cmd, 0x00, sizeof(USBFW_COMMAND) );
	}

	PTQP_UINT32 pData_temp = kmalloc( sizeof(TQP_UINT32) * numReg, GFP_KERNEL );
	if( !pData_temp )
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_RegisterReadEx() no pData_temp\n"));
	}
	else
	{
		memcpy( pData_temp, pData, sizeof(TQP_UINT32) * numReg );
	}

	if (!pData)
	{
		return (QPERR_PARMS);
	}

#ifdef REDUCE_HCI_MESSAGES
	p_cmd->cmd_type = QLUSBFW_HCI_CMD;
#else
	p_cmd->cmd_type = QLUSBFW_ARM_REGISTER;
#endif
	p_cmd->cmd_rw = 0;
	p_cmd->cmd_data_len = (TQP_UINT16)numReg;
	p_cmd->pad.addr32 = (QPHCI_MODE_INDIRECT == This->m_accessMode) ? (wAddr & 0x7FFF) : wAddr;	// make sure we bit is cleared

	qpStatus = CUsbCntl_GenericCmd(This,
								   (PTQP_UINT8)p_cmd,
								   sizeof(USBFW_COMMAND),
								   (PTQP_UINT8)pData_temp,
								   numReg * sizeof(TQP_UINT32),
								   QP_TRUE
								   );
	memcpy( pData, pData_temp, sizeof(TQP_UINT32) * numReg );

	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_RegisterReadEx() GenericCmd Failed Status(%d)\n",
					 qpStatus
					 ));
	}
	if( pData_temp )
	{
		kfree(pData_temp);	
		pData_temp = NULL;
	}
	if(p_cmd)
	{
		kfree(p_cmd);	
		p_cmd = NULL;
	}

	return (qpStatus);
#endif
}



EQP_Errors CUsbCntl_MemoryRead(CUsbCntl *This,
							   TQP_UINT32 dwAddr,
							   PTQP_UINT32 pData //can not use directly
							   )
{
#if defined(QP_LINXKER) && !defined(CONFIG_QL300_USB_EVK)
        return (QPERR_NOTIMPL);
#else
#if 0
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	TQP_UINT32	armAddr = dwAddr >> 2;

	if (!pData)
	{
		return (QPERR_PARMS);
	}

	ENTER_CRITICAL(&This->m_Object)

	qpStatus = CUsbCntl_WriteHciDWORD(This,
									  14,
									  armAddr & 0x7FFFFFFF,
									  QP_FALSE
									  );

	if (QP_SUCCESS(qpStatus))
	{
		qpStatus = CUsbCntl_ReadHciDWORD(This,
										 18,
										 pData,
										 QP_FALSE
										 );
	}

	LEAVE_CRITICAL(&This->m_Object)

	return (qpStatus);
#else
	EQP_Errors		qpStatus;
	//TQP_UINT8		usb_cmd[sizeof(USBFW_COMMAND) + sizeof(TQP_UINT32)];
	//peterlin
	TQP_UINT8* usb_cmd = kmalloc( sizeof(USBFW_COMMAND) + sizeof(TQP_UINT32), GFP_KERNEL );
	
	if( !usb_cmd )
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_MemoryRead() no usb_cmd\n"));
	}
	else
	{
		memset( usb_cmd, 0x00, sizeof(USBFW_COMMAND) + sizeof(TQP_UINT32) );
	}

	PTQP_UINT32 pData_temp = kmalloc( sizeof(TQP_UINT32), GFP_KERNEL );

	if( !pData_temp )
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_MemoryRead() no pData_temp\n"));
	}
	else
	{
		memcpy( pData_temp, pData, sizeof(TQP_UINT32) );
	}

	
	PUSBFW_COMMAND	p_cmd = (USBFW_COMMAND*)usb_cmd;
	TQP_UINT32		armAddr = dwAddr >> 2;

	if (!pData)
	{
		return (QPERR_PARMS);
	}

	p_cmd->cmd_type = QLUSBFW_ARM_MEMORY;
	p_cmd->cmd_rw = 0;
	p_cmd->cmd_data_len = sizeof(TQP_UINT32);
	p_cmd->pad.addr32 = (QPHCI_MODE_INDIRECT == This->m_accessMode) ? (armAddr & 0x7FFFFFFF) : armAddr;	// make sure we bit is cleared

	*((PTQP_UINT32)&usb_cmd[sizeof(USBFW_COMMAND)]) = p_cmd->pad.addr32;

	qpStatus = CUsbCntl_GenericCmd(This,
								   (PTQP_UINT8)p_cmd,
								   //sizeof(usb_cmd),
								   sizeof(USBFW_COMMAND) + sizeof(TQP_UINT32),
								   (PTQP_UINT8)pData_temp,
								   sizeof(TQP_UINT32),
								   QP_TRUE
								   );
	memcpy( pData, pData_temp, sizeof(TQP_UINT32) );

	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_MemoryRead() GenericCmd Failed Status(%d)\n",
					 qpStatus
					 ));
	}

	if(pData_temp)
	{
		kfree(pData_temp);	
		pData_temp = NULL;
	}

	if(usb_cmd)
	{
		kfree(usb_cmd);	
		usb_cmd = NULL;
	}

	return (qpStatus);
#endif
#endif
}


EQP_Errors CUsbCntl_MemoryReadEx(CUsbCntl *This,
								 TQP_UINT32 ulDWordAddr,
								 TQP_UINT32 ulDWordLength,
								 PTQP_UINT32 pInBuf
								 )
{
#if defined(QP_LINXKER) && !defined(CONFIG_QL300_USB_EVK)
        return (QPERR_NOTIMPL);
#else
	EQP_Errors		qpStatus = QPERR_SUCCESS;
	//TQP_UINT8		usb_cmd[sizeof(USBFW_COMMAND) + 2 * sizeof(TQP_UINT32)];
	//peterlin
	TQP_UINT8* usb_cmd = kmalloc( sizeof(USBFW_COMMAND) + 2 * sizeof(TQP_UINT32), GFP_KERNEL );
	
	if( !usb_cmd )
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_MemoryReadEx() no usb_cmd\n"));
	}
	else
	{
		memset( usb_cmd, 0x00, sizeof(USBFW_COMMAND) + 2 * sizeof(TQP_UINT32) );
	}
	
	PUSBFW_COMMAND	p_cmd = (USBFW_COMMAND*)usb_cmd;
#ifdef QLUSBFW_DMA_RETURN_ONE_BYTE
	//TQP_UINT8		cRet;
	TQP_UINT8*		p_cRet = kmalloc( sizeof(TQP_UINT8), GFP_KERNEL );
	if( !p_cRet )
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_MemoryReadEx() no p_cRet\n"));
	}
#endif //QLUSBFW_DMA_RETURN_ONE_BYTE
	TQP_UINT32		dwXfered = 0;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CUsbCntl_MemoryReadEx() dw-addr(0x%x) dw-len(%d)\n",
				 ulDWordAddr,
				 ulDWordLength
				 ));

	p_cmd->cmd_type = QLUSBFW_DMA;
	p_cmd->cmd_rw = 0;
	p_cmd->cmd_data_len = 2 * sizeof(TQP_UINT32);
	p_cmd->pad.addr32 = 0;

	*((PTQP_UINT32)&usb_cmd[sizeof(USBFW_COMMAND)]) = ulDWordAddr;
	*((PTQP_UINT32)&usb_cmd[sizeof(USBFW_COMMAND) + 4]) = ulDWordLength;

	qpStatus = CUsbCntl_GenericCmd(This,
								   (PTQP_UINT8)p_cmd,
								   sizeof(USBFW_COMMAND) + 2 * sizeof(TQP_UINT32),
#ifdef QLUSBFW_DMA_RETURN_ONE_BYTE
								   //&cRet,
								   p_cRet,
								   1,
#else //!QLUSBFW_DMA_RETURN_ONE_BYTE
								   QP_NULL,
								   0,
#endif //QLUSBFW_DMA_RETURN_ONE_BYTE
								   QP_TRUE
								   );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_MemoryReadEx() GenericCmd Failed Status(%d)\n",
					 qpStatus
					 ));
	}
	else
	{
		ENTER_CRITICAL(&This->m_Object)

		qpStatus = QPUsbInterface_UsbSendCmd(&This->m_pUsb->m_iUsbInterface,
											 This->m_dwPipeDataRead,
											 pInBuf,
											 ulDWordLength << 2,
											 &dwXfered
											 );

		LEAVE_CRITICAL(&This->m_Object)

		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CUsbCntl_MemoryReadEx() read pipe(%d) Failed Status(%d) xfered(%d)\n",
						 This->m_dwPipeDataRead,
						 qpStatus,
						 dwXfered
						 ));
		}
		else
		{
			int nPacketNumber = 0;

			nPacketNumber = ((ulDWordLength << 2) + QP_BULK_DATA_PACKET_SIZE - 1) / QP_BULK_DATA_PACKET_SIZE;

			if ((nPacketNumber % 2) != 0) {
				This->m_dwDataInNextToggle ^= 1;
			}
		}
	}
	if( p_cRet )
	{
		kfree(p_cRet);	
		p_cRet = NULL;
	}
	if(usb_cmd)
	{
		kfree(usb_cmd);	
		usb_cmd = NULL;
	}

	return (qpStatus);
#endif
}



EQP_Errors CUsbCntl_MemoryWrite(CUsbCntl *This,
								TQP_UINT32 dwAddr,
								TQP_UINT32 dwData
								)
{
#if defined(QP_LINXKER) && !defined(CONFIG_QL300_USB_EVK)
        return (QPERR_NOTIMPL);
#else
#if 0
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	TQP_UINT32	armAddr = dwAddr >> 2;

	ENTER_CRITICAL(&This->m_Object)

	qpStatus = CUsbCntl_WriteHciDWORD(This,
									  18,
									  dwData,
									  QP_FALSE
									  );
	if (QP_SUCCESS(qpStatus))
	{
		qpStatus = CUsbCntl_WriteHciDWORD(This,
										  14,
										  armAddr | 0x80000000,
										  QP_FALSE
										  );
	}

	LEAVE_CRITICAL(&This->m_Object)

	return (qpStatus);
#else
	EQP_Errors		qpStatus;
	//TQP_UINT8		usb_cmd[sizeof(USBFW_COMMAND) + 2 * sizeof(TQP_UINT32)];
	//peter
	TQP_UINT8* usb_cmd = kmalloc( sizeof(USBFW_COMMAND) + 2 * sizeof(TQP_UINT32), GFP_KERNEL );
	
	if( !usb_cmd )
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_MemoryWrite() no usb_cmd\n"));
	}
	else
	{
		memset( usb_cmd, 0x00, sizeof(USBFW_COMMAND) + 2 * sizeof(TQP_UINT32) );
	}

	PUSBFW_COMMAND	p_cmd = (USBFW_COMMAND*)usb_cmd;
	TQP_UINT32		armAddr = dwAddr >> 2;

	p_cmd->cmd_type = QLUSBFW_ARM_MEMORY;
	p_cmd->cmd_rw = 1;
	p_cmd->cmd_data_len = sizeof(TQP_UINT32);
	p_cmd->pad.addr32 = (QPHCI_MODE_INDIRECT == This->m_accessMode) ? (armAddr | 0x80000000) : armAddr;	// set we bit

	*((PTQP_UINT32)&usb_cmd[sizeof(USBFW_COMMAND)]) = p_cmd->pad.addr32;
	*((PTQP_UINT32)&usb_cmd[sizeof(USBFW_COMMAND) + 4]) = dwData;

	qpStatus = CUsbCntl_GenericCmd(This,
								   (PTQP_UINT8)p_cmd,
								   //sizeof(usb_cmd),
								   sizeof(USBFW_COMMAND) + 2 * sizeof(TQP_UINT32),
								   QP_NULL,
								   0,
								   QP_TRUE
								   );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_MemoryWrite() GenericCmd Failed Status(%d)\n",
					 qpStatus
					 ));
	}
	if(usb_cmd)
	{
		kfree(usb_cmd);	
		usb_cmd = NULL;
	}

	return (qpStatus);
#endif
#endif
}



static void QPUsbDmaComplete(TQP_PVOID UserContext,
							 TQP_PVOID pUserContext2
							 )
{
#if defined(QP_LINXKER) && !defined(CONFIG_QL300_USB_EVK)
        return;
#else
	CUsbCntl	*This = (CUsbCntl *)UserContext;
	//CQLCodecLib	*pQLCodecLib = GET_INHERITED_OBJECT(CQLCodecLib, m_Object, This->m_Object.m_pParent);

	PQPUSB_IO_CONTEXT pIoContext = (PQPUSB_IO_CONTEXT)pUserContext2;

	if (QPUsbInterface_UsbIsDataPipeInput(&This->m_pUsb->m_iUsbInterface,
										  This->m_ioContext.Pipe
										  ))
	{
	    int nPacketNumber = 0;
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_EXTRACE),
					 "QPUsbDmaComplete() IN\n"
					 ));

        nPacketNumber = (pIoContext->dwXferSize + QP_BULK_DATA_PACKET_SIZE - 1) / QP_BULK_DATA_PACKET_SIZE;
        if ((nPacketNumber % 2) != 0) {
		    This->m_dwDataInNextToggle ^= 1;
        }


		// M2P done
		//
		This->m_pBusCallbackFunc(This->m_pBusCallbackContext,
								 QPBUSINTF_DMA_READ_DONE,
								 QP_NULL
								 );
	}
	else
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_EXTRACE),
					 "QPUsbDmaComplete() OUT\n"
					 ));
		// P2M done
		//
		This->m_pBusCallbackFunc(This->m_pBusCallbackContext,
								 QPBUSINTF_DMA_WRITE_DONE,
								 QP_NULL
								 );
	}
#endif
}



EQP_Errors CUsbCntl_StartDMAWrite(CUsbCntl *This,
								  TQP_UINT32 ulAddr,	// in WORD(32 bit)
								  TQP_UINT32 ulLength,	// in WORD(32 bit)
								  PTQP_UINT8 pHostAddr,
								  TQP_BOOL bSwap
								  )
{
#if defined(QP_LINXKER) && !defined(CONFIG_QL300_USB_EVK)
        return (QPERR_NOTIMPL);
#else
	EQP_Errors		qpStatus;
	//TQP_UINT8		usb_cmd[sizeof(USBFW_COMMAND) + 2 * sizeof(TQP_UINT32)];
	//peterlin
	TQP_UINT8* usb_cmd = kmalloc( sizeof(USBFW_COMMAND) + 2 * sizeof(TQP_UINT32), GFP_KERNEL );
	
	if( !usb_cmd )
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_StartDMAWrite() no usb_cmd\n"));
	}
	else
	{
		memset( usb_cmd, 0x00, sizeof(USBFW_COMMAND) + 2 * sizeof(TQP_UINT32) );
	}


	PUSBFW_COMMAND	p_cmd = (USBFW_COMMAND*)usb_cmd;
#ifdef QLUSBFW_DMA_RETURN_ONE_BYTE
	//TQP_UINT8		cRet;
	TQP_UINT8* p_cRet = kmalloc( sizeof(TQP_UINT8), GFP_KERNEL );
	if( !p_cRet )
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_StartDMAWrite() no p_cRet\n"));
	}
#endif //QLUSBFW_DMA_RETURN_ONE_BYTE

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_EXTRACE),
				 "CUsbCntl_StartDMAWrite() addr(0x%x) len(%d) pHostAddr(llx%llx)\n",
				 ulAddr,
				 ulLength,
				 pHostAddr
				 ));

	p_cmd->cmd_type = QLUSBFW_DMA;
	p_cmd->cmd_rw = 1;
	p_cmd->cmd_data_len = 2 * sizeof(TQP_UINT32);
	p_cmd->pad.addr32 = 0;

	*((PTQP_UINT32)&usb_cmd[sizeof(USBFW_COMMAND)]) = ulAddr;
	*((PTQP_UINT32)&usb_cmd[sizeof(USBFW_COMMAND) + 4]) = ulLength;

	qpStatus = CUsbCntl_GenericCmd(This,
								   (PTQP_UINT8)p_cmd,
								   sizeof(USBFW_COMMAND) + 2 * sizeof(TQP_UINT32),
#ifdef QLUSBFW_DMA_RETURN_ONE_BYTE
								   //&cRet,
									p_cRet,
								   1,
#else //!QLUSBFW_DMA_RETURN_ONE_BYTE
								   QP_NULL,
								   0,
#endif //QLUSBFW_DMA_RETURN_ONE_BYTE
								   QP_TRUE
								   );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_StartDMAWrite() GenericCmd Failed Status(%d)\n",
					 qpStatus
					 ));
	}
	else
	{
		// Send buffer through USB interface
		//
		qpStatus = QPUsbInterface_UsbAsyncIo(&This->m_pUsb->m_iUsbInterface,
											 This->m_dwPipeDataWrite,
											 pHostAddr,
											 ulLength << 2,
											 &This->m_ioContext
											 );
		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CUsbCntl_StartDMAWrite() QPUsbInterface_UsbAsyncIo() Failed status(%d)!!\n",
						 qpStatus
						 ));
		}
	}
	if(p_cRet)
	{
		kfree(p_cRet);	
		p_cRet = NULL;
	}

	if(usb_cmd)
	{
		kfree(usb_cmd);	
		usb_cmd = NULL;
	}

	return (qpStatus);
#endif
}



EQP_Errors CUsbCntl_StartDMARead(CUsbCntl *This,
								 TQP_UINT32 ulAddr,	// in WORD(32 bit)
								 TQP_UINT32 ulLength,	// in WORD(32 bit)
								 PTQP_UINT8 pHostAddr,
								 TQP_BOOL bSwap
								 )
{
#if defined(QP_LINXKER) && !defined(CONFIG_QL300_USB_EVK)
        return (QPERR_NOTIMPL);
#else
	EQP_Errors		qpStatus;
	//TQP_UINT8		usb_cmd[sizeof(USBFW_COMMAND) + 2 * sizeof(TQP_UINT32)];
	//peterlin
	TQP_UINT8* usb_cmd = kmalloc( sizeof(USBFW_COMMAND) + 2 * sizeof(TQP_UINT32), GFP_KERNEL );
	
	if( !usb_cmd )
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_StartDMARead() no usb_cmd\n"));
	}
	else
	{
		memset( usb_cmd, 0x00, sizeof(USBFW_COMMAND) + 2 * sizeof(TQP_UINT32) );
	}


	PUSBFW_COMMAND	p_cmd = (USBFW_COMMAND*)usb_cmd;
#ifdef QLUSBFW_DMA_RETURN_ONE_BYTE
	//TQP_UINT8		cRet;
	TQP_UINT8* p_cRet = kmalloc( sizeof(TQP_UINT8), GFP_KERNEL );
	if( !p_cRet )
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_StartDMARead() no p_cRet\n"));
	}

#endif //QLUSBFW_DMA_RETURN_ONE_BYTE

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_EXTRACE),
				 "CUsbCntl_StartDMARead() addr(0x%x) len(%d)\n",
				 ulAddr,
				 ulLength
				 ));

	p_cmd->cmd_type = QLUSBFW_DMA;
	p_cmd->cmd_rw = 0;
	p_cmd->cmd_data_len = 2 * sizeof(TQP_UINT32);
	p_cmd->pad.addr32 = 0;


	*((PTQP_UINT32)&usb_cmd[sizeof(USBFW_COMMAND)]) = ulAddr;
	*((PTQP_UINT32)&usb_cmd[sizeof(USBFW_COMMAND) + 4]) = ulLength;

	qpStatus = CUsbCntl_GenericCmd(This,
								   (PTQP_UINT8)p_cmd,
								   sizeof(USBFW_COMMAND) + 2 * sizeof(TQP_UINT32),
#ifdef QLUSBFW_DMA_RETURN_ONE_BYTE
								   //&cRet,
								   p_cRet,
								   1,
#else //!QLUSBFW_DMA_RETURN_ONE_BYTE
								   QP_NULL,
								   0,
#endif //QLUSBFW_DMA_RETURN_ONE_BYTE
								   QP_TRUE
								   );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_StartDMARead() GenericCmd Failed Status(%d)\n",
					 qpStatus
					 ));
	}
	else
	{
		// Send buffer through USB interface
		//
		qpStatus = QPUsbInterface_UsbAsyncIo(&This->m_pUsb->m_iUsbInterface,
											 This->m_dwPipeDataRead,
											 pHostAddr,
											 ulLength << 2,
											 &This->m_ioContext
											 );
		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CUsbCntl_StartDMARead() QPUsbInterface_UsbAsyncIo() Failed status(%d)!!\n",
						 qpStatus
						 ));
		}
	}
	if(p_cRet)
	{
		kfree(p_cRet);	
		p_cRet = NULL;
	}
	if(usb_cmd)
	{
		kfree(usb_cmd);	
		usb_cmd = NULL;
	}

	return (qpStatus);
#endif
}



EQP_Errors CUsbCntl_ResetArm(CUsbCntl *This,
							 TQP_BOOL fRun
							 )
{
#if defined(QP_LINXKER) && !defined(CONFIG_QL300_USB_EVK)
        return (QPERR_NOTIMPL);
#else
	EQP_Errors		qpStatus;
	//USBFW_COMMAND	usbcmd;

	//peterlin
	PUSBFW_COMMAND	p_cmd = kmalloc( sizeof(USBFW_COMMAND), GFP_KERNEL );
	if( !p_cmd )
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_ResetArm() no p_cmd\n"));
	}


	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CUsbCntl_ResetArm() run(%d)\n",
				 fRun
				 ));
	
	p_cmd->cmd_type = QLUSBFW_RESET_ARM;
	p_cmd->cmd_rw = fRun ? 1 : 0;	// 0 : hold Arm reset, 1 : running
	p_cmd->cmd_data_len = 0;
	p_cmd->pad.addr32 = 0;

	qpStatus = CUsbCntl_GenericCmd(This,
								   (PTQP_UINT8)p_cmd,
								   sizeof(USBFW_COMMAND),
								   QP_NULL,
								   0,
								   QP_TRUE
								   );
	

	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_ResetArm() GenericCmd Failed Status(%d)\n",
					 qpStatus
					 ));
	}
	if(p_cmd)
	{
		kfree(p_cmd);	
		p_cmd = NULL;
	}


	return (qpStatus);
#endif
}



EQP_Errors CUsbCntl_I2CRead(CUsbCntl *This,
							TQP_UINT8 cDeviceAddr,
							int nSize,
							PTQP_UINT8 pBuf
							)
{
#if defined(QP_LINXKER) && !defined(CONFIG_QL300_USB_EVK)
        return (QPERR_NOTIMPL);
#else
	//USBFW_COMMAND	usbcmd;
	//peterlin
	PUSBFW_COMMAND	p_cmd = kmalloc( sizeof(USBFW_COMMAND), GFP_KERNEL );
	if( !p_cmd )
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_I2CRead() no p_cmd\n"));
	}


	EQP_Errors		qpStatus;
	PTQP_UINT8		pRead;

	if (!pBuf)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_WARN),
					 "CUsbCntl_I2CRead() return buffer is NULL\n"
					 ));
		return (QPERR_PARMS);
	}

	pRead = (PTQP_UINT8)QPMMMalloc2Ex(nSize + 1);
	if (QP_NULL == pRead)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_WARN),
					 "CUsbCntl_I2CRead() unable to allocate read buffer\n"
					 ));
		return (QPERR_NOMEM);
	}

	p_cmd->cmd_type = QLUSBFW_I2C_READ;
	p_cmd->cmd_rw = 0;
	p_cmd->cmd_data_len = (unsigned short)nSize;
	p_cmd->pad.addr32 = cDeviceAddr;

	qpStatus = CUsbCntl_GenericCmd(This,
								   (PTQP_UINT8)p_cmd,
								   sizeof(USBFW_COMMAND),
								   pRead,
								   nSize + 1,
								   QP_TRUE
								   );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_I2CRead() GenericCmd Failed Status(%d)\n",
					 qpStatus
					 ));
	}
	else
	{
		memcpy(pBuf,
			   pRead,
			   nSize
			   );

		if (QLUSBFW_I2C_OK != pRead[nSize])
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CUsbCntl_I2CRead() QLUSBFW_I2C_READ Failed I2C status(%d)",
						 pRead[nSize]
						 ));
		}
	}

	QPMMFree2Ex(pRead);
	if(p_cmd)
	{
		kfree(p_cmd);	
		p_cmd = NULL;
	}


	return (qpStatus);
#endif
}



EQP_Errors CUsbCntl_I2CWrite(CUsbCntl *This,
							 TQP_UINT8 cDeviceAddr,
							 int nSize,
							 PTQP_UINT8 pBuf
							 )
{
#if defined(QP_LINXKER) && !defined(CONFIG_QL300_USB_EVK)
        return (QPERR_NOTIMPL);
#else
	EQP_Errors		qpStatus;
	PTQP_UINT8 		pusb_cmd;
	PUSBFW_COMMAND	p_cmd;
	TQP_UINT8		i2cStatus;

	if (!pBuf)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_WARN),
					 "CUsbCntl_I2CWrite() input buffer is NULL\n"
					 ));
		return (QPERR_PARMS);
	}

	pusb_cmd = (PTQP_UINT8)QPMMMalloc2Ex(sizeof(USBFW_COMMAND) + nSize);
	if (QP_NULL == pusb_cmd)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_WARN),
					 "CUsbCntl_I2CWrite() unable to allocate buffer\n"
					 ));
		return (QPERR_NOMEM);
	}

	p_cmd = (USBFW_COMMAND*)pusb_cmd;

	p_cmd->cmd_type = QLUSBFW_I2C_WRITE;
	p_cmd->cmd_rw = 1;
	p_cmd->cmd_data_len = (unsigned short)nSize;
	p_cmd->pad.addr32 = cDeviceAddr;

	// copy I2C write data to the command buffer
	//
	memcpy(pusb_cmd + sizeof(USBFW_COMMAND),
		   pBuf,
		   nSize
		   );

	qpStatus = CUsbCntl_GenericCmd(This,
								   pusb_cmd,
								   sizeof(USBFW_COMMAND) + nSize,
								   &i2cStatus,
								   1,
								   QP_TRUE
								   );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_I2CWrite() GenericCmd Failed Status(%d)\n",
					 qpStatus
					 ));
	}
	else
	{
		if (QLUSBFW_I2C_OK != i2cStatus)
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CUsbCntl_I2CWrite() Failed I2C status(%d)",
						 i2cStatus
						 ));
		}
	}

	QPMMFree2Ex(pusb_cmd);

	return (qpStatus);
#endif
}



EQP_Errors CUsbCntl_I2CWriteThenRead(CUsbCntl *This,
									 TQP_UINT8 cDeviceAddr,
									 int nWriteSize,
									 const PTQP_UINT8 pWriteBuf,
									 int nReadSize,
									 PTQP_UINT8 pReadBuf
									 )
{
#if defined(QP_LINXKER) && !defined(CONFIG_QL300_USB_EVK)
        return (QPERR_NOTIMPL);
#else
	EQP_Errors		qpStatus;
	PTQP_UINT8		pusb_cmd;
	PUSBFW_COMMAND	p_cmd;
	PTQP_UINT8		pRead;

	if (!pWriteBuf ||
		!pReadBuf
		)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_WARN),
					 "CUsbCntl_I2CWriteThenRead() buffers are NULL\n"
					 ));
		return (QPERR_PARMS);
	}

	pusb_cmd = (PTQP_UINT8)QPMMMalloc2Ex(sizeof(USBFW_COMMAND) + nWriteSize);
	if (QP_NULL == pusb_cmd)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_WARN),
					 "CUsbCntl_I2CWriteThenRead() unable to allocate pusb_cmd\n"
					 ));
		return (QPERR_NOMEM);
	}
	p_cmd = (USBFW_COMMAND*)pusb_cmd;

	pRead =  (PTQP_UINT8)QPMMMalloc2Ex(nReadSize + 1);
	if (QP_NULL == pRead)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_WARN),
					 "CUsbCntl_I2CWriteThenRead() unable to allocate pRead\n"
					 ));
		QPMMFree2Ex(pusb_cmd);
		return (QPERR_NOMEM);
	}

	p_cmd->cmd_type = QLUSBFW_I2C_READ;
	p_cmd->cmd_rw = (unsigned char)nWriteSize;
	p_cmd->cmd_data_len = (unsigned short)nReadSize;
	p_cmd->pad.addr32 = cDeviceAddr;

	memcpy(pusb_cmd + sizeof(USBFW_COMMAND),
		   pWriteBuf,
		   nWriteSize
		   );
	qpStatus = CUsbCntl_GenericCmd(This,
								   pusb_cmd,
								   sizeof(USBFW_COMMAND) + nWriteSize,
								   pRead,
								   nReadSize + 1,
								   QP_TRUE
								   );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_I2CWriteThenRead() GenericCmd Failed Status(%d)\n",
					 qpStatus
					 ));
	}
	else
	{
		memcpy(pReadBuf,
			   pRead,
			   nReadSize
			   );

		if (QLUSBFW_I2C_OK != pRead[nReadSize])
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CUsbCntl_I2CWriteThenRead() Failed I2C status(%d)\n",
						 pRead[nReadSize]
						 ));
		}
	}

	QPMMFree2Ex(pusb_cmd);
	QPMMFree2Ex(pRead);

	return (qpStatus);
#endif
}

EQP_Errors CUsbCntl_SWI2CRead(CUsbCntl *This,
							TQP_UINT8 cDeviceAddr,
							int nSize,
							PTQP_UINT8 pBuf
							)
{
#if defined(QP_LINXKER) && !defined(CONFIG_QL300_USB_EVK)
        return (QPERR_NOTIMPL);
#else
	//USBFW_COMMAND	usbcmd;
	//peterlin
	PUSBFW_COMMAND	p_cmd = kmalloc( sizeof(USBFW_COMMAND), GFP_KERNEL );
	if( !p_cmd )
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_SWI2CRead() no p_cmd\n"));
	}


	EQP_Errors		qpStatus;
	PTQP_UINT8		pRead;

	if (!pBuf)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_WARN),
					 "CUsbCntl_SWI2CRead() return buffer is NULL\n"
					 ));
		return (QPERR_PARMS);
	}

	pRead = (PTQP_UINT8)QPMMMalloc2Ex(nSize + 1);
	if (QP_NULL == pRead)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_WARN),
					 "CUsbCntl_SWI2CRead() unable to allocate read buffer\n"
					 ));
		return (QPERR_NOMEM);
	}

	p_cmd->cmd_type = QLUSBFW_SW_I2C_READ;
	p_cmd->cmd_rw = 0;
	p_cmd->cmd_data_len = (unsigned short)nSize;
	p_cmd->pad.addr32 = cDeviceAddr;

	qpStatus = CUsbCntl_GenericCmd(This,
								   (PTQP_UINT8)p_cmd,
								   sizeof(USBFW_COMMAND),
								   pRead,
								   nSize + 1,
								   QP_TRUE
								   );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_SWI2CRead() GenericCmd Failed Status(%d)\n",
					 qpStatus
					 ));
	}
	else
	{
		memcpy(pBuf,
			   pRead,
			   nSize
			   );

		if (QLUSBFW_I2C_OK != pRead[nSize])
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CUsbCntl_SWI2CRead() QLUSBFW_SW_I2C_READ Failed I2C status(%d)",
						 pRead[nSize]
						 ));
		}
	}

	QPMMFree2Ex(pRead);
	if(p_cmd)
	{
		kfree(p_cmd);	
		p_cmd = NULL;
	}


	return (qpStatus);
#endif
}



EQP_Errors CUsbCntl_SWI2CWrite(CUsbCntl *This,
							 TQP_UINT8 cDeviceAddr,
							 int nSize,
							 PTQP_UINT8 pBuf
							 )
{
#if defined(QP_LINXKER) && !defined(CONFIG_QL300_USB_EVK)
        return (QPERR_NOTIMPL);
#else
	EQP_Errors		qpStatus;
	PTQP_UINT8 		pusb_cmd;
	PUSBFW_COMMAND	p_cmd;
	TQP_UINT8		i2cStatus;

	if (!pBuf)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_WARN),
					 "CUsbCntl_SWI2CWrite() input buffer is NULL\n"
					 ));
		return (QPERR_PARMS);
	}

	pusb_cmd =  (PTQP_UINT8)QPMMMalloc2Ex(sizeof(USBFW_COMMAND) + nSize);
	if (QP_NULL == pusb_cmd)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_WARN),
					 "CUsbCntl_SWI2CWrite() unable to allocate buffer\n"
					 ));
		return (QPERR_NOMEM);
	}

	p_cmd = (USBFW_COMMAND*)pusb_cmd;

	p_cmd->cmd_type = QLUSBFW_SW_I2C_WRITE;
	p_cmd->cmd_rw = 1;
	p_cmd->cmd_data_len = (unsigned short)nSize;
	p_cmd->pad.addr32 = cDeviceAddr;

	// copy I2C write data to the command buffer
	//
	memcpy(pusb_cmd + sizeof(USBFW_COMMAND),
		   pBuf,
		   nSize
		   );

	qpStatus = CUsbCntl_GenericCmd(This,
								   pusb_cmd,
								   sizeof(USBFW_COMMAND) + nSize,
								   &i2cStatus,
								   1,
								   QP_TRUE
								   );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_SWI2CWrite() GenericCmd Failed Status(%d)\n",
					 qpStatus
					 ));
	}
	else
	{
		if (QLUSBFW_I2C_OK != i2cStatus)
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CUsbCntl_SWI2CWrite() Failed I2C status(%d)\n",
						 i2cStatus
						 ));
            qpStatus = QPERR_FAIL;
		}
	}

	QPMMFree2Ex(pusb_cmd);
	return (qpStatus);
#endif
}



EQP_Errors CUsbCntl_SWI2CWriteThenRead(CUsbCntl *This,
									 TQP_UINT8 cDeviceAddr,
									 int nWriteSize,
									 const PTQP_UINT8 pWriteBuf,
									 int nReadSize,
									 PTQP_UINT8 pReadBuf
									 )
{
#if defined(QP_LINXKER) && !defined(CONFIG_QL300_USB_EVK)
        return (QPERR_NOTIMPL);
#else
	EQP_Errors		qpStatus;
	PTQP_UINT8		pusb_cmd;
	PUSBFW_COMMAND	p_cmd;
	PTQP_UINT8		pRead;

	if (!pWriteBuf ||
		!pReadBuf
		)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_WARN),
					 "CUsbCntl_SWI2CWriteThenRead() buffers are NULL\n"
					 ));
		return (QPERR_PARMS);
	}

	pusb_cmd =  (PTQP_UINT8)QPMMMalloc2Ex(sizeof(USBFW_COMMAND) + nWriteSize);
	if (QP_NULL == pusb_cmd)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_WARN),
					 "CUsbCntl_SWI2CWriteThenRead() unable to allocate pusb_cmd\n"
					 ));
		return (QPERR_NOMEM);
	}
	p_cmd = (USBFW_COMMAND*)pusb_cmd;

	pRead =  (PTQP_UINT8)QPMMMalloc2Ex(nReadSize + 1);
	if (QP_NULL == pRead)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_WARN),
					 "CUsbCntl_SWI2CWriteThenRead() unable to allocate pRead\n"
					 ));
		QPMMFree2Ex(pusb_cmd);
		return (QPERR_NOMEM);
	}

	p_cmd->cmd_type = QLUSBFW_SW_I2C_READ;
	p_cmd->cmd_rw = (unsigned char)nWriteSize;
	p_cmd->cmd_data_len = (unsigned short)nReadSize;
	p_cmd->pad.addr32 = cDeviceAddr;

	memcpy(pusb_cmd + sizeof(USBFW_COMMAND),
		   pWriteBuf,
		   nWriteSize
		   );
	qpStatus = CUsbCntl_GenericCmd(This,
								   pusb_cmd,
								   sizeof(USBFW_COMMAND) + nWriteSize,
								   pRead,
								   nReadSize + 1,
								   QP_TRUE
								   );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_SWI2CWriteThenRead() GenericCmd Failed Status(%d)\n",
					 qpStatus
					 ));
	}
	else
	{
		memcpy(pReadBuf,
			   pRead,
			   nReadSize
			   );

		if (QLUSBFW_I2C_OK != pRead[nReadSize])
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CUsbCntl_SWI2CWriteThenRead() Failed I2C status(%d)\n",
						 pRead[nReadSize]
						 ));
		}
	}

	QPMMFree2Ex(pusb_cmd);
	QPMMFree2Ex(pRead);

	return (qpStatus);
#endif
}


EQP_Errors CUsbCntl_GetUSBSpeed(CUsbCntl *This,
								PTQP_UINT8 pData
								)
{
#if defined(QP_LINXKER) && !defined(CONFIG_QL300_USB_EVK)
        return (QPERR_NOTIMPL);
#else
	EQP_Errors		qpStatus;
	//USBFW_COMMAND	usbcmd;
	//peterlin
	PUSBFW_COMMAND	p_cmd = kmalloc( sizeof(USBFW_COMMAND), GFP_KERNEL );
	if( !p_cmd )
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_GetUSBSpeed() no p_cmd\n"));
	}



	if (!pData)
	{
		return (QPERR_PARMS);
	}

	p_cmd->cmd_type = QLUSBFW_GET_USB_SPEED;
	p_cmd->cmd_rw = 0;
	p_cmd->cmd_data_len = 1;
	p_cmd->pad.addr32 = 0;

	qpStatus = CUsbCntl_GenericCmd(This,
								   (PTQP_UINT8)p_cmd,
								   sizeof(USBFW_COMMAND),
								   (PTQP_UINT8)pData,
								   sizeof(TQP_UINT8),
								   QP_TRUE
								   );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CUsbCntl_GetUSBSpeed() GenericCmd Failed Status(%d)\n",
					 qpStatus
					 ));
	}
	if(p_cmd)
	{
		kfree(p_cmd);	
		p_cmd = NULL;
	}

	return (qpStatus);
#endif
}



CUsbCntl * CUsbCntl_Constructor(CUsbCntl *pUsbCntl,
								CObject *pParent,
								TQP_UINT32 dwAttributes,
								CUsbInterface *pUsbInterface,
								QP_BUSINTF_CALLBACK pBusCallbackFunc,
								TQP_PVOID pBusCallbackContext,
								TQP_UINT32 dwPipeCmdWrite,
								TQP_UINT32 dwPipeCmdRead,
								TQP_UINT32 dwPipeDataWrite,
								TQP_UINT32 dwPipeDataRead,
								QPHCI_MODE accessMode
								)
{
#if defined(QP_LINXKER) && !defined(CONFIG_QL300_USB_EVK)
        return NULL;
#else
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_WARN),
				 "CUsbCntl_Constructor() cmd_wr(%d) cmd_rd(%d) dma_wr(%d) dma_rd(%d) accessMode(%d)\n",
				 dwPipeCmdWrite,
				 dwPipeCmdRead,
				 dwPipeDataWrite,
				 dwPipeDataRead,
				 accessMode
				 ));

	if (pUsbCntl)
	{
		// CObject
		//
		CObject_Constructor(&pUsbCntl->m_Object,
							pParent,
							dwAttributes
		  					);

		pUsbCntl->m_pBusCallbackFunc = pBusCallbackFunc;
		pUsbCntl->m_pBusCallbackContext = pBusCallbackContext;

		pUsbCntl->m_pUsb = pUsbInterface;

		// Artesa pipes
		//
		// pUsbCntl->m_dwPipeCmdWrite = USB_PIPE_1;
		// pUsbCntl->m_dwPipeCmdRead = USB_PIPE_0;
		// pUsbCntl->m_dwPipeDataWrite = USB_PIPE_3;
		// pUsbCntl->m_dwPipeDataRead = USB_PIPE_2;
		//
		// FX2 pipes
		//
		//	pUsbCntl->m_dwPipeCmdWrite = USB_PIPE_1;
		//	pUsbCntl->m_dwPipeCmdRead = USB_PIPE_3;
		//	pUsbCntl->m_dwPipeDataWrite = USB_PIPE_0;
		//	pUsbCntl->m_dwPipeDataRead = USB_PIPE_2;
		//

		pUsbCntl->m_dwPipeCmdWrite = dwPipeCmdWrite;
		pUsbCntl->m_dwPipeCmdRead = dwPipeCmdRead;
		pUsbCntl->m_dwPipeDataWrite = dwPipeDataWrite;
		pUsbCntl->m_dwPipeDataRead = dwPipeDataRead;

		QPUsbInterface_UsbResetPipe(&pUsbInterface->m_iUsbInterface,
									pUsbCntl->m_dwPipeCmdWrite
									);
		QPUsbInterface_UsbResetPipe(&pUsbInterface->m_iUsbInterface,
									pUsbCntl->m_dwPipeCmdRead
									);
		QPUsbInterface_UsbResetPipe(&pUsbInterface->m_iUsbInterface,
									pUsbCntl->m_dwPipeDataWrite
									);
		QPUsbInterface_UsbResetPipe(&pUsbInterface->m_iUsbInterface,
									pUsbCntl->m_dwPipeDataRead
									);

		pUsbCntl->m_accessMode = accessMode;

		// clear async io context
		//
		memset((void *)&pUsbCntl->m_ioContext,
			   0,
			   sizeof(QPUSB_IO_CONTEXT)
			   );

		pUsbCntl->m_ioContext.pCompletion = QPUsbDmaComplete;
		pUsbCntl->m_ioContext.UserContext = (TQP_PVOID)pUsbCntl;
	}

	return (pUsbCntl);
#endif
}



void CUsbCntl_Destructor(CUsbCntl *This)
{
#if defined(QP_LINXKER) && !defined(CONFIG_QL300_USB_EVK)
        return;
#else
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPUsbDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CUsbCntl_Destructor()\n"
				 ));

	// CObject
	//
	CObject_Destructor(&This->m_Object);
#endif
}



