///////////////////////////////////////////////////////////////////////////////
//        
//
// Cusbcntl.h
//
// Description: 
//
//  This is the implementation of the USB controller interface
// 
// History:
//	Aug 25, 2006 (dt) created
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#ifndef __CUSBCNTL_H__
#define __CUSBCNTL_H__


#include "cqpusbintf.h"
#include "qpbusintf.h"
#include "qpmpgcodec.h"

#ifdef __cplusplus
extern "C" {
#endif

#define QP_BULK_DATA_PACKET_SIZE 512


/////////////////////////////////////////////////////////////////////////////
//
//

#pragma pack(push, 1)

// USB command header
//
typedef struct _USBFW_COMMAND
{
	unsigned char	cmd_type;
	unsigned char	cmd_rw;			// 0: read; 1: write
	unsigned short	cmd_data_len;	// write: number of bytes following this header 
									// read: number of bytes to read

	union _pad
	{
		unsigned int	addr32;
		unsigned short	addr16;
		unsigned char	addr8;
	} pad;

} USBFW_COMMAND, *PUSBFW_COMMAND;


// UVC command header
//
typedef struct _USBFW_COMMAND_UVC_HEADER
{
	unsigned char	type;
	unsigned char	req;
	struct _wselector
	{
		unsigned char	selector;
		unsigned char	zero;
	} wselector;
	struct _wunit
	{
		unsigned char	unit;
		unsigned char	zero;
	} wunit;
	unsigned short	length;			// number of bytes following this header
} USBFW_COMMAND_UVC_HEADER, *PUSBFW_COMMAND_UVC_HEADER;


// start video command
//
typedef struct _USBFW_COMMAND_START_VIDEO
{
	unsigned char	type;
	unsigned char	fps;
	unsigned short	videoType;
	unsigned short	width;
	unsigned short	height;
} USBFW_COMMAND_START_VIDEO, *PUSBFW_COMMAND_START_VIDEO;


#pragma pack(pop)


/////////////////////////////////////////////////////////////////////////////
//
//

typedef struct CUsbCntl
{
	// public CObject
	//
	CObject				m_Object;

	// private data
	//
	CUsbInterface		*m_pUsb;
	TQP_UINT32			m_dwPipeCmdWrite;
	TQP_UINT32			m_dwPipeCmdRead;
	TQP_UINT32			m_dwPipeDataWrite;
	TQP_UINT32			m_dwPipeDataRead;
	QPHCI_MODE			m_accessMode;

	QPUSB_IO_CONTEXT	m_ioContext;		// io context for async io
	

	// bus callback
	//
	QP_BUSINTF_CALLBACK	m_pBusCallbackFunc;
	TQP_PVOID			m_pBusCallbackContext;
	
	TQP_UINT32          m_dwCmdInNextToggle;
	TQP_UINT32          m_dwDataInNextToggle;

} CUsbCntl;


EQP_Errors CUsbCntl_GenericCmd(CUsbCntl *This,
							   PTQP_UINT8 pOutBuf,
							   TQP_UINT32 dwOutSize,
							   PTQP_UINT8 pInBuf,
							   TQP_UINT32 dwInSize,
							   TQP_BOOL bCritical
							   );
EQP_Errors CUsbCntl_ReadHciRegister(CUsbCntl *This,
									TQP_UINT8 cAddr,
									PTQP_UINT8 pData
									);
EQP_Errors CUsbCntl_WriteHciRegister(CUsbCntl *This,
									 TQP_UINT8 cAddr,
									 TQP_UINT8 ucData
									 );
EQP_Errors CUsbCntl_RegisterRead(CUsbCntl *This,
								 TQP_UINT16 wAddr,
								 PTQP_UINT32 pData
								 );
EQP_Errors CUsbCntl_RegisterWrite(CUsbCntl *This,
								  TQP_UINT16 wAddr,
								  TQP_UINT32 dwData 
								  );
EQP_Errors CUsbCntl_RegisterReadEx(CUsbCntl *This,
								   TQP_UINT16 wAddr,
								   PTQP_UINT32 pData,
								   TQP_UINT32 numReg
								   );
EQP_Errors CUsbCntl_RegisterWriteEx(CUsbCntl *This,
								   TQP_UINT16 wAddr,
								   PTQP_UINT32 pData,
								   TQP_UINT32 numReg
								   );
EQP_Errors CUsbCntl_MemoryRead(CUsbCntl *This,
							   TQP_UINT32 dwAddr,
							   PTQP_UINT32 pData 
							   );
EQP_Errors CUsbCntl_MemoryWrite(CUsbCntl *This,
								TQP_UINT32 dwAddr,
								TQP_UINT32 dwData 
								);
EQP_Errors CUsbCntl_MemoryReadEx(CUsbCntl *This,
								 TQP_UINT32 ulDWordAddr,		// in D-WORD(32 bit)
								 TQP_UINT32 ulDWordLength,		// in D-WORD(32 bit)
								 PTQP_UINT32 pInBuf 
								 );
EQP_Errors CUsbCntl_StartDMAWrite(CUsbCntl *This,
								  TQP_UINT32 ulAddr,	// in WORD(32 bit)
								  TQP_UINT32 ulLength,// in WORD(32 bit)
								  PTQP_UINT8 pHostAddr,
								  TQP_BOOL bSwap
								  );
EQP_Errors CUsbCntl_StartDMARead(CUsbCntl *This,
								 TQP_UINT32 ulAddr,	// in WORD(32 bit)
								 TQP_UINT32 ulLength,	// in WORD(32 bit)
								 PTQP_UINT8 pHostAddr,
								 TQP_BOOL bSwap
								 );
EQP_Errors CUsbCntl_ResetArm(CUsbCntl *This,
							 TQP_BOOL fRun
							 );
EQP_Errors CUsbCntl_I2CRead(CUsbCntl *This,
							TQP_UINT8 cDeviceAddr, 
							int nSize, 
							PTQP_UINT8 pBuf
							);
EQP_Errors CUsbCntl_I2CWrite(CUsbCntl *This,
							 TQP_UINT8 cDeviceAddr, 
							 int nSize, 
							 PTQP_UINT8 pBuf
							 );
EQP_Errors CUsbCntl_I2CWriteThenRead(CUsbCntl *This,
									 TQP_UINT8 cDeviceAddr,
									 int nWriteSize, 
									 const PTQP_UINT8 pWriteBuf,
									 int nReadSize,
									 PTQP_UINT8 pReadBuf
									 );
EQP_Errors CUsbCntl_SWI2CRead(CUsbCntl *This,
							TQP_UINT8 cDeviceAddr, 
							int nSize, 
							PTQP_UINT8 pBuf
							);
EQP_Errors CUsbCntl_SWI2CWrite(CUsbCntl *This,
							 TQP_UINT8 cDeviceAddr, 
							 int nSize, 
							 PTQP_UINT8 pBuf
							 );
EQP_Errors CUsbCntl_SWI2CWriteThenRead(CUsbCntl *This,
									 TQP_UINT8 cDeviceAddr,
									 int nWriteSize, 
									 const PTQP_UINT8 pWriteBuf,
									 int nReadSize,
									 PTQP_UINT8 pReadBuf
									 );
EQP_Errors CUsbCntl_GetUSBSpeed(CUsbCntl *This,
								PTQP_UINT8 pData
								);
CUsbCntl * CUsbCntl_Constructor(CUsbCntl *pUsbCntl, 
								CObject *pParent,
								TQP_UINT32 dwAttributes,
								CUsbInterface *pUsbInterface,
								QP_BUSINTF_CALLBACK pBusCallbackFunc,
								TQP_PVOID pBusCallbackContext,
								TQP_UINT32 dwPipeCmdWrite,
								TQP_UINT32 dwPipeCmdRead,
								TQP_UINT32 dwPipeDataWrite,
								TQP_UINT32 dwPipeDataRead,
								QPHCI_MODE accessMode
								);

void CUsbCntl_Destructor(CUsbCntl *This);


#ifdef __cplusplus
}
#endif

#endif //__CUSBCNTL_H__



