///////////////////////////////////////////////////////////////////////////////
//        
//
// Iqlusbfw.h
//
// Description: 
//
//  This is the USB firmware API
// 
// History:
//	Taken from the EVK release
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#ifndef __IQLUSBFW_H__
#define __IQLUSBFW_H__

#define QLUSBFW_HIU_REGISTER					0
#define QLUSBFW_ARM_REGISTER					1
#define QLUSBFW_ARM_MEMORY						2
#define QLUSBFW_HCI_CMD							3
#define QLUSBFW_I2C_WRITE						5
#define QLUSBFW_GPIF_INIT						6
#define QLUSBFW_RESET_ARM						7
#define QLUSBFW_I2C_READ						8
#define QLUSBFW_DMA								9
#define QLUSBFW_WAIT_ARM_MSG					10
#define QLUSBFW_SW_I2C_WRITE					11
#define QLUSBFW_SW_I2C_READ						12
#define QLUSBFW_I2C_WRITE_MULTIPLE				15

#define QLUSBFW_UVC_CONTROL						16
#define QLUSBFW_UVC_FC3_STARTVIDEO				17
#define QLUSBFW_UVC_FC3_STOPVIDEO				18

#define QLUSBFW_GET_USB_SPEED					20


// Unit define
//
#define UVC_UNIT_CT								0x0001	// UVC camera terminal
#define UVC_UNIT_PU								0x0002	// UVC processing unit
#define UVC_UNIT_XU1							0x0003	// UVC extension unit


#ifndef _QCAM_FIRMWARE

// QLUSBFW_UVC_CONTROL

// Request define
//
#define SET_CUR									0x01
#define GET_CUR									0x81
#define GET_MIN									0x82
#define GET_MAX									0x83
#define GET_RES									0x84
#define GET_LEN									0x85
#define GET_INFO								0x86
#define GET_DEF									0x87

// Selector defines for CT
//
#define CT_CONTROL_UNDEFINED                    0x00	
#define CT_SCANNING_MODE_CONTROL                0x01	// wLength == 1, 0: Interlaced; 1: Progressive
#define CT_AE_MODE_CONTROL                      0x02	// wLength == 1
#define CT_AE_PRIORITY_CONTROL                  0x03	// wLength == 1
#define CT_EXPOSURE_TIME_ABSOLUTE_CONTROL       0x04	// wLength == 4, in uint of 100 us(1/10000 sec)
#define CT_EXPOSURE_TIME_RELATIVE_CONTROL       0x05	// wLength == 1 
#define CT_FOCUS_ABSOLUTE_CONTROL               0x06	// wLength == 2
#define CT_FOCUS_RELATIVE_CONTROL               0x07	// wLength == 1
#define CT_FOCUS_AUTO_CONTROL                   0x08	// wLength == 1, 0: disable; 1: enable
#define CT_IRIS_ABSOLUTE_CONTROL                0x09	// wLength == 2
#define CT_IRIS_RELATIVE_CONTROL                0x0A	// wLength == 1
#define CT_ZOOM_ABSOLUTE_CONTROL                0x0B	// wLength == 2
#define CT_ZOOM_RELATIVE_CONTROL                0x0C	// wLength == 3
#define CT_PANTILT_ABSOLUTE_CONTROL             0x0D	// wLength == 8
#define CT_PANTILT_RELATIVE_CONTROL             0x0E	// wLength == 4
#define CT_ROLL_ABSOLUTE_CONTROL                0x0F	// wLength == 2
#define CT_ROLL_RELATIVE_CONTROL                0x10	// wLength == 2
#define CT_PRIVACY_CONTROL                      0x11	// wLength == 1

// Selector defines for PU
//
#define PU_CONTROL_UNDEFINED                    0x00
#define PU_BACKLIGHT_COMPENSATION_CONTROL       0x01	// wLength == 2
#define PU_BRIGHTNESS_CONTROL                   0x02	// wLength == 2
#define PU_CONTRAST_CONTROL                     0x03	// wLength == 2
#define PU_GAIN_CONTROL                         0x04	// wLength == 2
#define PU_POWER_LINE_FREQUENCY_CONTROL         0x05	// wLength == 1
#define PU_HUE_CONTROL                          0x06	// wLength == 2
#define PU_SATURATION_CONTROL                   0x07	// wLength == 2
#define PU_SHARPNESS_CONTROL                    0x08	// wLength == 2
#define PU_GAMMA_CONTROL                        0x09	// wLength == 2
#define PU_WHITE_BALANCE_TEMPERATURE_CONTROL    0x0A	// wLength == 2
#define PU_WHITE_BALANCE_TEMPERATURE_AUTO_CONTROL 0x0B	// wLength == 1, 0: disable; 1: enable
#define PU_WHITE_BALANCE_COMPONENT_CONTROL      0x0C	// wLength == 4
#define PU_WHITE_BALANCE_COMPONENT_AUTO_CONTROL 0x0D	// wLength == 1, 0: disable; 1: enable
#define PU_DIGITAL_MULTIPLIER_CONTROL           0x0E	// wLength == 2
#define PU_DIGITAL_MULTIPLIER_LIMIT_CONTROL     0x0F	// wLength == 2
#define PU_HUE_AUTO_CONTROL                     0x10	// wLength == 1
#define PU_ANALOG_VIDEO_STANDARD_CONTROL        0x11	// wLength == 1
#define PU_ANALOG_LOCK_STATUS_CONTROL           0x12	// wLength == 1

// Selector defines for extension unit
//
#define XU_I2C                                  0x01
#define XU_H264_MODE                            0x02

// QLUSBFW_UVC_FC3_STARTVIDEO
//
// videoType:
#define QLUSBFW_UVC_FC3_START_VIDEO_QYUV		64
#define QLUSBFW_UVC_FC3_START_VIDEO_MJPEG		32
#define QLUSBFW_UVC_FC3_START_VIDEO_H264		16

#endif //!_QCAM_FIRMWARE


// control USB firmware return 
#define QLUSBFW_DMA_RETURN_ONE_BYTE

// firmware return code
#define QLUSBFW_I2C_BERROR						6	// I2C Status: I2C error; Bit Error
#define QLUSBFW_I2C_NACK						7	// I2C Status: I2C error; No Acknowledge
#define QLUSBFW_I2C_OK							8	// I2C positive return code

#endif //__IQLUSBFW_H__




