/*************************************************************************/
/**      **/
/**            **/
/**                **/
/**              **/
/**                                                                     **/
/**    **/
/**                  **/
/**                                                                     **/
/**                               **/
/**                                                  **/
/*************************************************************************/

#include "qlautoconf.h"

#include <linux/errno.h>
#include <linux/init.h>
#include <linux/mm.h>
#include <linux/module.h>
#include <linux/poll.h>
#include <linux/slab.h>
#include <linux/vmalloc.h>
#include <asm/io.h>
#include <linux/usb.h>
#include <linux/string.h>

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/fs.h>
#include <asm/uaccess.h>
#include <asm/fcntl.h>

#include "cdevice.h"
#include "qlfops.h"

// Vendor specific request code for Anchor Upload/Download
// This one is implemented in the core
#define ANCHOR_LOAD_INTERNAL	0xA0

// This command is not implemented in the core.  Requires firmware
#define ANCHOR_LOAD_EXTERNAL	0xA3

// EZ-USB Control and Status Register.  Bit 0 controls 8051 reset
#define CPUCS_REG_EZUSB			0x7F92
#define CPUCS_REG_FX2			0xE600

#define FX2_XFER_CHUNK_SIZE 64

static long fx2Reset(struct usb_device *qlusbdev, unsigned char reset) 
{
    int Err= 0;
    unsigned long ulCurrXferred= 0;

    ulCurrXferred = usb_control_msg(
        qlusbdev, usb_sndctrlpipe(qlusbdev, 0), ANCHOR_LOAD_INTERNAL, 
        0x40, CPUCS_REG_EZUSB, 0, &reset, 1, 3*HZ);

    if(ulCurrXferred!=1) {
        Err= -ENODEV;
    } else {
        ulCurrXferred = usb_control_msg(
            qlusbdev, usb_sndctrlpipe(qlusbdev, 0), ANCHOR_LOAD_INTERNAL, 
            0x40, CPUCS_REG_FX2,   0, &reset, 1, 3*HZ);

        if(ulCurrXferred!=1) {
            Err= -ENODEV;
        }
    }

    printk("qlusb: fx2reset %d, Err=%x(%d)\n", reset, Err, Err);
    return Err;
}

#define QL201_FX2_FW_FILENAME            "/mnt/ql201fx2.bin"
#define QL300_FX2_FW_FILENAME            "/mnt/ql300fx2.bin"

long fx2Download(struct usb_device *qlusbdev, unsigned long ulDevType) 
{
    int Err= 0;
    unsigned long ulXferChunkSize= 0, ulCurrXferred= 0, ulfx2FwSize= 0, fp= 0;
    unsigned char buff[FX2_XFER_CHUNK_SIZE]= { 0 };
    char *pfilename= 0;

    switch(ulDevType) {
    case PLV4LDEV_TYPE_QL201_FX2:
        pfilename = QL201_FX2_FW_FILENAME;
        break;
    case PLV4LDEV_TYPE_QL300_FX2:
        pfilename = QL300_FX2_FW_FILENAME;
        break;
    }

    Err= qlf_open(&fp, pfilename, 1);
    if(fp && !Err) {

        while(1) {
            ulXferChunkSize= qlf_read(fp, buff, FX2_XFER_CHUNK_SIZE);

            if(ulXferChunkSize>0) {

                ulCurrXferred = usb_control_msg(
                    qlusbdev, usb_sndctrlpipe(qlusbdev, 0), ANCHOR_LOAD_INTERNAL, 
                    0x40, ulfx2FwSize,   0, buff, ulXferChunkSize, 3*HZ);

                if(ulXferChunkSize==ulCurrXferred) {
                    ulfx2FwSize+= ulXferChunkSize;
                } else {
                    printk("qlusb: Error while writing FX2 firmware \n");
                    break;
                }

            } else {
                printk("qlusb: No more FX2 FW data to xfer (%d bytes) \n", ulfx2FwSize);
                break;
            }
        }

    } else {
        printk("qlusb: Unable to open FX2 FW file %s  (fp=%x,Err=%d)\n", pfilename, fp, Err);
    }
    if(fp) {
        qlf_close(fp);
        fp= 0;
    }
    return Err;
}


long fx2Setup(struct usb_device *qlusbdev, unsigned long ulDevType) 
{
    int Err= 0;

    Err= Err ? Err : fx2Reset(qlusbdev, 1);

    Err= Err ? Err : fx2Download(qlusbdev, ulDevType);

    Err= Err ? Err : fx2Reset(qlusbdev, 0);

    printk("qlusb: FX2 setup returns %x(%d) \n", Err, Err);
    return Err;
}



/* EOF : fx2.c */

