///////////////////////////////////////////////////////////////////////////////
//        
//
// Qpusbintf.h
//
// Description: 
//
//  This is the USB bus interface
// 
// History:
//	Aug 22, 2006 (dt) created
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __QPUSBINTF_H__
#define __QPUSBINTF_H__

#include "qptypes.h"

#ifdef __cplusplus
extern "C" {
#endif

/////////////////////////////////////////////////////////////////////////////
//
//


#ifndef __IUSBINTF_FWD_DEFINED__
#define __IUSBINTF_FWD_DEFINED__
typedef struct IUsbInterface IUsbInterface;
#endif //__IUSBINTF_FWD_DEFINED__


// Callback function for asynchronous read/writes
//
typedef void (*QPUSBCOMPLETION)(TQP_PVOID UserContext, TQP_PVOID pUserContext2);


// Context information for asynchronous read/writes
//
typedef struct _QPUSB_IO_CONTEXT
{
	IUsbInterface			*pUsb;			// Ptr to the USB context used for the operation.
	TQP_UINT32				Pipe;			// Pipe index.
	QPUSBCOMPLETION			pCompletion;	// Pointer to the caller's completion routine.
	TQP_PVOID				UserContext;	// Caller's completion context.
	TQP_UINT64				ullStartTime;	// for profiling
	TQP_UINT32				dwXferSize;		// Amount of data to be transfer
	EQP_Errors				qpStatus;		// io status
	TQP_UINT32				transferSize;	// actual size transfered
	PTQP_UINT8				pCallerBuffer;	// caller buffer
} QPUSB_IO_CONTEXT, *PQPUSB_IO_CONTEXT;



/////////////////////////////////////////////////////////////////////////////
//
//


    struct IUsbInterface
    {
		// IUsbInterface
		EQP_Errors (*UsbCallBusDriver)(IUsbInterface *This,
									   TQP_PVOID pUrb
									   );
		EQP_Errors (*UsbSendCmd)(IUsbInterface *This,
								 TQP_UINT32 dwPipeNum, 
								 TQP_PVOID pCmd, 
								 TQP_UINT32 dwSize,
								 TQP_UINT32* pdwXfered
								 );
		EQP_Errors (*UsbAsyncIo)(IUsbInterface *This,
								 TQP_UINT32 dwPipeNum, 
								 PTQP_UINT8 pBuff, 
								 TQP_UINT32 dwSize, 
								 PQPUSB_IO_CONTEXT pIoContext
								 );
		EQP_Errors (*UsbResetPipe)(IUsbInterface *This,
								   TQP_UINT32 dwPipeNum
								   );
		EQP_Errors (*UsbAbortPipe)(IUsbInterface *This,
								   TQP_UINT32 dwPipeNum
								   );
		TQP_BOOL (*UsbIsDataPipeInput)(IUsbInterface *This,
									   TQP_UINT32 dwPipeNum
									   );
    };


/////////////////////////////////////////////////////////////////////////////
//
//

#define QPUsbInterface_UsbCall(This, pUrb) \
    (This)->UsbCallBusDriver(This, pUrb)

#define QPUsbInterface_UsbSendCmd(This, dwPipeNum, pCmd, dwSize, pdwXfered) \
    (This)->UsbSendCmd(This, dwPipeNum, pCmd, dwSize, pdwXfered)

#define QPUsbInterface_UsbAsyncIo(This, dwPipeNum, pBuff, dwSize, pIoContext) \
    (This)->UsbAsyncIo(This, dwPipeNum, pBuff, dwSize, pIoContext)

#define QPUsbInterface_UsbResetPipe(This, dwPipeNum) \
    (This)->UsbResetPipe(This, dwPipeNum)

#define QPUsbInterface_UsbAbortPipe(This, dwPipeNum) \
    (This)->UsbAbortPipe(This, dwPipeNum)

#define QPUsbInterface_UsbIsDataPipeInput(This, dwPipeNum) \
    (This)->UsbIsDataPipeInput(This, dwPipeNum)

#ifdef __cplusplus
}
#endif

#endif //__QPUSBINTF_H__




