///////////////////////////////////////////////////////////////////////////////
//        
//
// Cchannel.h
//
// Description: 
//
//	channels represent the data streaming on codec chip
// 
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#ifndef __CCHANNEL_H__
#define __CCHANNEL_H__


#include "qptypes.h"
#include "cobject.h"
#include "cqueue.h"
#ifdef _QPCODEC_CALLBACK_IN_THREAD
#include "cfifo.h"
#endif //_QPCODEC_CALLBACK_IN_THREAD
#include "qpcodeclib.h"
#include "qpmpgcodec.h"

#ifdef __cplusplus
extern "C" {
#endif

/////////////////////////////////////////////////////////////////////////////
//
//

typedef enum _CHANNEL_DIRECTION
{
	CHANNEL_DIR_READ = 0,
	CHANNEL_DIR_WRITE,
	CHANNEL_DIR_NONE

} CHANNEL_DIRECTION, *PCHANNEL_DIRECTION;

#define MAX_CHANNEL_DIRECTION	CHANNEL_DIR_NONE


#define PTS_NTSC_INCREMENT			(3003)
#define PTS_PAL_INCREMENT			(3600)
#define KS_NTSC_INCREMENT			(333667)
#define KS_PAL_INCREMENT			(400000)

#define PTS_NTSC_INCREMENT_1000X	(3003003)
#define PTS_PAL_INCREMENT_1000X		(3600000)


#define PTS_TO_KSPTS(ulPts)	((TQP_INT64)(ulPts) * 10000 / 90)


#ifdef _QPCODEC_CALLBACK_IN_THREAD

#pragma pack(push, 4)

typedef struct _CALLBACK_ENTRY
{
	QP_DEVICE_CALLBACK		pDeviceCallback;	// codec client call back function
	TQP_PVOID				callbackContext;	// codec client call back function context
 	TQP_UINT32				dwCode;
 	TQP_PVOID				pParam;
} CALLBACK_ENTRY, *PCALLBACK_ENTRY;

#pragma pack(pop)

#endif //_QPCODEC_CALLBACK_IN_THREAD

/////////////////////////////////////////////////////////////////////////////
//
//

#ifndef __CTASK_FWD_DEFINED__
#define __CTASK_FWD_DEFINED__
typedef struct CTask CTask;
#endif //!__CTASK_FWD_DEFINED__


#ifndef __CCHANNEL_FWD_DEFINED__
#define __CCHANNEL_FWD_DEFINED__
typedef struct CChannel CChannel;
#endif //!__CCHANNEL_FWD_DEFINED__


struct CChannel
{
	// public CObject
	//
	CObject	m_Object;

	// CChannel
	//
	EQP_Errors (*Open)(CChannel *This,
					   TQP_OBJECT_HANDLE hChannel,
					   TQP_UINT32 dwFlags,
					   TQP_PVOID pDataFormat,
					   QP_DEVICE_CALLBACK pFuncCallback,
					   TQP_PVOID context
					   );
	EQP_Errors (*Close)(CChannel *This);
	EQP_Errors (*Start)(CChannel *This);
	EQP_Errors (*Stop)(CChannel *This);
	EQP_Errors (*Acquire)(CChannel *This);
	EQP_Errors (*Pause)(CChannel *This);
	EQP_Errors (*Step)(CChannel *This,
					   TQP_UINT32 nb
					   );
	EQP_Errors (*SetRate)(CChannel *This,
						  TQP_INT32 lNewRate
						  );
	EQP_Errors (*GetRate)(CChannel *This,
						  PTQP_INT32 plRate
						  );
	EQP_Errors (*BeginFlush)(CChannel *This);
	EQP_Errors (*Flush)(CChannel *This);
	EQP_Errors (*EndFlush)(CChannel *This);
	EQP_Errors (*AddBuffer)(CChannel *This,
							PQP_BUFFER_DESCRIPTOR pBufDesc
							);
	EQP_Errors (*CancelBuffer)(CChannel *This,
							   PQP_BUFFER_DESCRIPTOR pBufDesc, 
							   TQP_BOOL bTimeOut
							   );
	EQP_Errors (*TimeoutBuffer)(CChannel *This,
								PQP_BUFFER_DESCRIPTOR pBufDesc
								);
	EQP_Errors (*XferData)(CChannel *This,
						   PQP_BUFFER_DESCRIPTOR pBufDesc
						   );
	TQP_BOOL (*GetBuffer)(CChannel *This,
						  PQP_BUFFER_DESCRIPTOR *ppBufDesc,
						  PTQP_UINT8 *ppBuffer,
						  PTQP_UINT32 pSize
						  );
	TQP_BOOL (*GetBufferYUV)(CChannel *This,
							 PQP_BUFFER_DESCRIPTOR *ppBufDesc,
							 PTQP_UINT8 *ppYBuffer,
							 PTQP_UINT32 pYSize, 
							 PTQP_UINT8 *ppUVBuffer, 
							 PTQP_UINT32 pUVSize
							 );
	TQP_BOOL (*GetBufferYUVRAS)(CChannel *This,
								PQP_BUFFER_DESCRIPTOR *ppBufDesc,
								PTQP_UINT8 *ppYBuffer,
								PTQP_UINT32 pYSize, 
								PTQP_UINT8 *ppUBuffer, 
								PTQP_UINT32 pUSize,
								PTQP_UINT8 *ppVBuffer, 
								PTQP_UINT32 pVSize
								);
	void (*CompleteBuffer)(CChannel *This,
						   PQP_BUFFER_DESCRIPTOR pBufDesc
						   );
	EQP_Errors (*BeginChannelChange)(CChannel *This);
	EQP_Errors (*EndChannelChange)(CChannel *This);
	EQP_Errors (*GetResolution)(CChannel *This,
								PTQP_UINT32 pWidth,
								PTQP_UINT32 pHeight
								);
	EQP_Errors (*GetYUVFormat)(CChannel *This,
							   PTQP_UINT32 pYUVFormat
							   );

	TQP_UINT32				m_dwOpenFlags;

	QP_DEVICE_CALLBACK		m_pDeviceCallback;				// codec client call back function
	TQP_PVOID				m_callbackContext;				// codec client call back function context

	TQP_OBJECT_HANDLE		m_hTask;						// task handle this channel is attached
	TQP_OBJECT_HANDLE		m_hChannel;						// handle to this channel
	TQP_INT32				m_dataType;						// task data type of this channel
	TQP_INT32				m_FWDataType;					// firmware data type of this channel
	CHANNEL_DIRECTION		m_ChannelDirection;				// channel direction
	QPMPGCODEC_OPEN_TYPE	m_ChannelType;					// channel open type
	CTask					*m_pTask;

	TQP_BOOL				m_bOpened;
	QPSTATE					m_State;						// channel state
	TQP_BOOL				m_bPaused;

	// data request management stuffs
	//
	QUEUE_ENTRY				m_Entries[QPCODECLIB_MAX_DATA_ENTRIES];
	CQueue					*m_pFreeQueue;
	CQueue					*m_pDataRequestQueue;
	CQueue					*m_pDataPendingQueue;

#ifdef _QPCODEC_CALLBACK_IN_THREAD
	CFifo					*m_pPendingCallbackFifo;
#endif //_QPCODEC_CALLBACK_IN_THREAD

	
	TQP_UINT64				m_llStartTime;

	// in flush flag
	//
	TQP_BOOL				m_bFlushing;

	// need byte swap
	//
	TQP_BOOL				m_bByteSwap;

	TQP_UINT64				m_ullCntBytes;

	// handle non-dword aligned buffer
	//
	TQP_UINT8				m_savedDword[4];
	TQP_UINT32				m_savedNumberBytes;

	// timestamp
	//
	TQP_INT64				m_llBasePts;
	TQP_INT64				m_llPrevPts;
	TQP_INT64				m_llAddPts;
};



// constructor
//
CChannel * CChannel_Constructor(CChannel *pChannel,
								CObject *pParent,
								TQP_UINT32 dwAttributes,
								TQP_OBJECT_HANDLE hTask,
								TQP_INT32 dataType,
								TQP_INT32 FWDataType,
								CHANNEL_DIRECTION ChannelDirection,
								QPMPGCODEC_OPEN_TYPE ChannelType,
								CTask *pTask
								);
// destructor
//
void CChannel_Destructor(CChannel *This);



// devcie callback
//
EQP_Errors CChannel_DeviceCallback(CChannel *This,
								   TQP_UINT32 dwCode,
						   		   TQP_PVOID pParam
								   );
#ifdef _QPCODEC_CALLBACK_IN_THREAD
TQP_BOOL CChannel_GetDeviceCallback(CChannel *This);
TQP_BOOL CChannel_CheckDeviceCallback(CChannel *This);
#endif //_QPCODEC_CALLBACK_IN_THREAD

// helper
//
QPSTATE CChannel_GetState(CChannel *This);
TQP_BOOL CChannel_isOpened(CChannel *This);
TQP_BOOL CChannel_needByteSwapping(CChannel *This);




/////////////////////////////////////////////////////////////////////////////
//
//

typedef struct CNullChannel
{
	// public CChannel
	//
	CChannel	m_Channel;
} CNullChannel;


// constructor
//
CNullChannel * CNullChannel_Constructor(CNullChannel *pNullChannel, 
										CObject *pParent,
										TQP_OBJECT_HANDLE hTask,
										CTask *pTask
										);

// destructor
//
void CNullChannel_Destructor(CNullChannel *This);


/////////////////////////////////////////////////////////////////////////////
//
//


typedef struct CMPEGOutChannel 
{
	// public CChannel
	//
	CChannel	m_Channel;
} CMPEGOutChannel;


// constructor
//
CMPEGOutChannel * CMPEGOutChannel_Constructor(CMPEGOutChannel *pMpgOutChannel, 
											  CObject *pParent,
											  TQP_UINT32 dwAttributes,
											  TQP_OBJECT_HANDLE hTask,
											  CTask *pTask
											  );

// destructor
//
void CMPEGOutChannel_Destructor(CMPEGOutChannel *This);



/////////////////////////////////////////////////////////////////////////////
//
//


typedef struct CAESOutChannel 
{
	// public CChannel
	//
	CChannel	m_Channel;
} CAESOutChannel;


// constructor
//
CAESOutChannel * CAESOutChannel_Constructor(CAESOutChannel *pAesOutChannel, 
											CObject *pParent,
											TQP_UINT32 dwAttributes,
											TQP_OBJECT_HANDLE hTask,
											CTask *pTask
											);

// destructor
//
void CAESOutChannel_Destructor(CAESOutChannel *This);



/////////////////////////////////////////////////////////////////////////////
//
//


typedef struct CYUVOutChannel 
{
	// public CChannel
	//
	CChannel	m_Channel;

	// CYUVOutChannel 
	//
	TQP_UINT32	m_dwFrameSize;

	TQP_UINT32	m_ulAvgTimePerFrame;
	TQP_UINT32	m_ulAvgPTSPerFrame;
	TQP_UINT32	m_ulAvgPTSPerFrame1000X;

	TQP_UINT32	m_ulVideoFramesCount;
	TQP_UINT32	m_ulDroppedFramesCount;

	TQP_INT32	m_lWidth;
	TQP_INT32	m_lHeight;
	TQP_INT32	m_nBitCount;
	TQP_INT32	m_nDataType;

} CYUVOutChannel;


// constructor
//
CYUVOutChannel * CYUVOutChannel_Constructor(CYUVOutChannel *pYUVOutChannel, 
											CObject *pParent,
											TQP_UINT32 dwAttributes,
											TQP_OBJECT_HANDLE hTask,
											QPMPGCODEC_OPEN_TYPE ChannelType,
											CTask *pTask
											);

// destructor
//
void CYUVOutChannel_Destructor(CYUVOutChannel *This);



/////////////////////////////////////////////////////////////////////////////
//
//


typedef struct CPCMOutChannel 
{
	// public CChannel
	//
	CChannel	m_Channel;

} CPCMOutChannel;


// constructor
//
CPCMOutChannel * CPCMOutChannel_Constructor(CPCMOutChannel *pPCMOutChannel, 
											CObject *pParent,
											TQP_UINT32 dwAttributes,
											TQP_OBJECT_HANDLE hTask,
											QPMPGCODEC_OPEN_TYPE ChannelType,
											CTask *pTask
											);

// destructor
//
void CPCMOutChannel_Destructor(CPCMOutChannel *This);




/////////////////////////////////////////////////////////////////////////////
//
//


typedef struct CVBIOutChannel 
{
	// public CChannel
	//
	CChannel	m_Channel;

	// CVBIOutChannel
	//
    TQP_INT32	m_Top_StartLine;		// inclusive
    TQP_INT32	m_Top_EndLine;			// inclusive
  	TQP_UINT32	m_Top_StartPixel;		// starting from 0
    TQP_INT32	m_Top_SamplesPerLine;	// maximum (1440 - StartPixel)

    TQP_INT32	m_Bottom_StartLine;		// inclusive
    TQP_INT32	m_Bottom_EndLine;		// inclusive
  	TQP_UINT32	m_Bottom_StartPixel;	// starting from 0
    TQP_INT32	m_Bottom_SamplesPerLine;// maximum (1440 - StartPixel)

    TQP_UINT32	m_BufferSize;		// Bytes
	TQP_INT32	m_nVBIDataCaptureSize;
	TQP_INT32	m_nVBILines;

	TQP_UINT32	m_ulAvgTimePerFrame;
	TQP_UINT32	m_ulAvgPTSPerFrame;
	TQP_UINT32	m_ulAvgPTSPerFrame1000X;
	TQP_UINT32	m_ulAvgTimePerField;
	TQP_UINT32	m_ulAvgPTSPerField;

	TQP_UINT64	m_ullVBIFramesCount;
	TQP_UINT32	m_ulDroppedFramesCount;

} CVBIOutChannel;


// constructor
//
CVBIOutChannel * CVBIOutChannel_Constructor(CVBIOutChannel *pVBIOutChannel, 
											CObject *pParent,
											TQP_UINT32 dwAttributes,
											TQP_OBJECT_HANDLE hTask,
											CTask *pTask
											);

// destructor
//
void CVBIOutChannel_Destructor(CVBIOutChannel *This);


// helper
//
TQP_BOOL CVBIOutChannel_GetVBIFormat(CVBIOutChannel *This,
									 PQP_VBI_DATAFORMAT pVBIFormat
									 );



/////////////////////////////////////////////////////////////////////////////
//
//


typedef struct CIndexOutChannel 
{
	// public CChannel
	//
	CChannel	m_Channel;
} CIndexOutChannel;


// constructor
//
CIndexOutChannel * CIndexOutChannel_Constructor(CIndexOutChannel *pIndexOutChannel, 
												CObject *pParent,
												TQP_UINT32 dwAttributes,
												TQP_OBJECT_HANDLE hTask,
												CTask *pTask
												);

// destructor
//
void CIndexOutChannel_Destructor(CIndexOutChannel *This);




/////////////////////////////////////////////////////////////////////////////
//
//


typedef struct CActivityOutChannel 
{
	// public CChannel
	//
	CChannel	m_Channel;
} CActivityOutChannel;


// constructor
//
CActivityOutChannel * CActivityOutChannel_Constructor(CActivityOutChannel *pActivityOutChannel, 
											  CObject *pParent,
											  TQP_UINT32 dwAttributes,
											  TQP_OBJECT_HANDLE hTask,
											  CTask *pTask
											  );

// destructor
//
void CActivityOutChannel_Destructor(CActivityOutChannel *This);




/////////////////////////////////////////////////////////////////////////////
//
//


typedef struct CYUVInChannel 
{
	// public CChannel
	//
	CChannel	m_Channel;

	// CYUVInChannel 
	//
	TQP_UINT32	m_dwFrameSize;
	TQP_UINT32	m_ulVideoFramesCount;

	TQP_INT32	m_lWidth;
	TQP_INT32	m_lHeight;
	TQP_INT32	m_nBitCount;
	TQP_INT32	m_nDataType;

} CYUVInChannel;


// constructor
//
CYUVInChannel * CYUVInChannel_Constructor(CYUVInChannel *pYUVInChannel, 
										  CObject *pParent,
										  TQP_UINT32 dwAttributes,
										  TQP_OBJECT_HANDLE hTask,
										  CTask *pTask
										  );

// destructor
//
void CYUVInChannel_Destructor(CYUVInChannel *This);



/////////////////////////////////////////////////////////////////////////////
//
//


typedef struct CPCMInChannel 
{
	// public CChannel
	//
	CChannel	m_Channel;
} CPCMInChannel;


// constructor
//
CPCMInChannel * CPCMInChannel_Constructor(CPCMInChannel *pPCMInChannel, 
										  CObject *pParent,
										  TQP_UINT32 dwAttributes,
										  TQP_OBJECT_HANDLE hTask,
										  CTask *pTask
										  );

// destructor
//
void CPCMInChannel_Destructor(CPCMInChannel *This);




/////////////////////////////////////////////////////////////////////////////
//
//

typedef struct CMPEGInChannel 
{
	// public CChannel
	//
	CChannel	m_Channel;

	// CMPEGInChannel 
	//
	TQP_INT32	m_nDataType;

} CMPEGInChannel;


// constructor
//
CMPEGInChannel * CMPEGInChannel_Constructor(CMPEGInChannel *pMpgInChannel, 
											CObject *pParent,
											TQP_UINT32 dwAttributes,
											TQP_OBJECT_HANDLE hTask,
											CTask *pTask
											);
// destructor
//
void CMPEGInChannel_Destructor(CMPEGInChannel *This);



/////////////////////////////////////////////////////////////////////////////
//
//

typedef struct CAESInChannel 
{
	// public CChannel
	//
	CChannel	m_Channel;
} CAESInChannel;


// constructor
//
CAESInChannel * CAESInChannel_Constructor(CAESInChannel *pAesInChannel, 
										  CObject *pParent,
										  TQP_UINT32 dwAttributes,
										  TQP_OBJECT_HANDLE hTask,
										  CTask *pTask
										  );
// destructor
//
void CAESInChannel_Destructor(CAESInChannel *This);



/////////////////////////////////////////////////////////////////////////////
//
//

typedef struct CVBIInChannel 
{
	// public CChannel
	//
	CChannel	m_Channel;

	// CVBIInChannel
	//
    TQP_INT32	m_Top_StartLine;		// inclusive
    TQP_INT32	m_Top_EndLine;			// inclusive
  	TQP_UINT32	m_Top_StartPixel;		// starting from 0
    TQP_INT32	m_Top_SamplesPerLine;	// maximum (1440 - StartPixel)

    TQP_INT32	m_Bottom_StartLine;		// inclusive
    TQP_INT32	m_Bottom_EndLine;		// inclusive
  	TQP_UINT32	m_Bottom_StartPixel;	// starting from 0
    TQP_INT32	m_Bottom_SamplesPerLine;// maximum (1440 - StartPixel)

    TQP_UINT32	m_BufferSize;			// Bytes

	TQP_UINT64	m_ullVBIFramesCount;

} CVBIInChannel;


// constructor
//
CVBIInChannel * CVBIInChannel_Constructor(CVBIInChannel *pVBIInChannel, 
										  CObject *pParent,
										  TQP_UINT32 dwAttributes,
										  TQP_OBJECT_HANDLE hTask,
										  CTask *pTask
										  );
// destructor
//
void CVBIInChannel_Destructor(CVBIInChannel *This);


#ifdef __cplusplus
}
#endif

#endif //__CCHANNEL_H__



