///////////////////////////////////////////////////////////////////////////////
//     
//
// Cdecodertask.c
//
// Description:
//
//	decoder task
//
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////

#ifdef QP_LINXKER
#include "qlautoconf.h"
#include "plprofiling.h"
#endif
#include "ctask.h"
#include "qpdbgmgr.h"
#include "qposmgr.h"
#include "fwapi.h"
#include "cqlcodec.h"
#include "util.h"
#ifndef QP_LINXKER
#include <string.h>
#endif

extern TQP_DbgMCompID	g_QPCodecDBGCompID;



/////////////////////////////////////////////////////////////////////////////
//
//

// task interfaces
//
EQP_Errors CDecoderTask_Start(CTask *This,
 							  TQP_OBJECT_HANDLE hTask
 							  )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CDecoderTask_Start() hTask(%d)\n",
				 hTask
				 ));

	qpStatus = CQLCodec_UpdateDecoderConfig(This->m_pMpegCodec,
											hTask
											);

	This->m_TaskData[hTask].bDone = QP_FALSE;

	// start the decoder
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWDECAPI_StartDecoder(This->m_pMpegCodec,
                                                                        (TQP_UINT32) hTask,
																		  This->m_TaskData[hTask].m_streamID.Read,
																		  This->m_TaskData[hTask].m_outputMode,
																		  This->m_pMpegCodec->m_VOUMode,
																		  This->m_pMpegCodec->m_VOUStartPixel,
																		  This->m_pMpegCodec->m_VOUStartLine,
																		  This->m_pMpegCodec->m_AOControls.Read,
																		  This->m_pMpegCodec->m_AudControls.Read
																		  );

	if (QP_SUCCESS(qpStatus))
	{
		This->m_TaskData[hTask].m_State = TASK_STATE_RUN;
	}

	return (qpStatus);
}



EQP_Errors CDecoderTask_Stop(CTask *This,
 							 TQP_OBJECT_HANDLE hTask
 							 )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CDecoderTask_Stop() hTask(%d)\n",
				 hTask
				 ));

	if (TASK_STATE_RUN == This->m_TaskData[hTask].m_State)
	{
		TQP_UINT32	ulEvents;
		TQP_INT32	lWaitStopCompleteTimeOut = 2000;
		TQP_INT64	llTimeTo;
		TQP_BOOL	bWaitForReadComplete = QP_FALSE;
		TQP_BOOL	bWaitForWriteComplete = QP_FALSE;

		QPOSMClearEvtgrp(This->m_TaskData[hTask].m_EvtReply,
						 QLCODEC_EVENT_STOP_DONE
						 );

		if (QPDEC_DEC_STOP_IMMEDIATELY == This->m_TaskData[hTask].m_decStopMode)
			lWaitStopCompleteTimeOut = 2000;
		else
			lWaitStopCompleteTimeOut = 15000;

		// send FW stop decoder command
		//
		qpStatus = QPFWDECAPI_StopDecoder(This->m_pMpegCodec,
										  (TQP_UINT32) hTask,
										  This->m_TaskData[hTask].m_decStopMode,
										  This->m_TaskData[hTask].m_decStopDispMode,
										  0,
										  0
										  );

		// change task state to stop
		//
		This->m_TaskData[hTask].m_State = TASK_STATE_STOP;

		// wait for stop completion
		//
		if (QP_SUCCESS(qpStatus))
		{
			qpStatus = QPOSMWaitEvtgrp(This->m_TaskData[hTask].m_EvtReply,
									   QLCODEC_EVENT_STOP_DONE,
									   &ulEvents,
									   lWaitStopCompleteTimeOut
									   );
			if (!QP_SUCCESS(qpStatus))
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CDecoderTask_Stop() hTask(%d) QPOSMWaitEvtgrp failed(%d) m_bDecDone(%d)!!!\n",
							 hTask,
							 qpStatus,
							 This->m_TaskData[hTask].bDone
							 ));
				qpStatus = This->m_TaskData[hTask].bDone ? QPERR_SUCCESS : qpStatus;
			}
		}
		else
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CDecoderTask_Stop() hTask(%d) QPFWDECAPI_StopDecoder failed(%d)!!!\n",
						 hTask,
						 qpStatus
						 ));
		}

		// reset task state
		This->m_TaskData[hTask].m_State = TASK_STATE_IDLE;
		This->m_TaskData[hTask].bDone = QP_FALSE;

		if (CQLCodec_IsCodecIdle(This->m_pMpegCodec))
		{
			// wait for the outstanding buffer to complete
			//
			ENTER_CRITICAL_IO_PENDING(This)
			bWaitForReadComplete = (This->m_pPending[CHANNEL_DIR_READ] && This->m_pPending[CHANNEL_DIR_READ]->pTaskData && (This->m_pPending[CHANNEL_DIR_READ]->pTaskData->id == hTask));
			bWaitForWriteComplete = (This->m_pPending[CHANNEL_DIR_WRITE] && This->m_pPending[CHANNEL_DIR_WRITE]->pTaskData && (This->m_pPending[CHANNEL_DIR_WRITE]->pTaskData->id == hTask));
			LEAVE_CRITICAL_IO_PENDING(This)

			if (bWaitForReadComplete)
			{
				llTimeTo = 1000;

				BeginTimer(&llTimeTo);

				while (This->m_pPending[CHANNEL_DIR_READ])
				{
					if (Timeout(llTimeTo,
								5
								))
					{
						QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
									 "CDecoderTask_Stop() wait for read DMA compelte timeout!!!\n"
									 ));
						break;
					}
				}
			}

			if (bWaitForWriteComplete)
			{
				llTimeTo = 1000;

				BeginTimer(&llTimeTo);

				while (This->m_pPending[CHANNEL_DIR_WRITE])
				{
					if (Timeout(llTimeTo,
								5
								))
					{
						QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
									 "CDecoderTask_Stop() wait for write DMA compelte timeout!!!\n"
									 ));
						break;
					}
				}
			}
		}
	}

	return (qpStatus);
}



EQP_Errors CDecoderTask_Pause(CTask *This,
 							  TQP_OBJECT_HANDLE hTask
 							  )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CDecoderTask_Pause() hTask(%d)\n",
				 hTask
				 ));

	qpStatus = QPFWDECAPI_PauseDisplay(This->m_pMpegCodec,
									   (TQP_UINT32) hTask,
									   This->m_TaskData[hTask].m_decPauseDispMode
									   );

	return (qpStatus);
}



EQP_Errors CDecoderTask_Resume(CTask *This,
 							   TQP_OBJECT_HANDLE hTask
 							   )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CDecoderTask_Resume() hTask(%d)\n",
				 hTask
				 ));

	qpStatus = QPFWDECAPI_ResumeDisplay(This->m_pMpegCodec,
										(TQP_UINT32) hTask
										);

	return (qpStatus);
}



void CDecoderTask_ProcessArmMessage(CTask *This,
									PHOST_MESSAGE pHostMsg,
									PHOST_MESSAGE_STATUS pHostMsg_status,
									TQP_UINT32 param1,
									TQP_UINT32 param2,
									TQP_UINT32 param3,
									TQP_UINT32 param4,	// QL201B
									TQP_UINT32 param5	// QL201B
									)
{
	TQP_OBJECT_HANDLE taskId;
	TASK_DATA_TYPE	dataType;

	// handle the message
	//
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CDecoderTask_ProcessArmMessage() cmd(0x%x) channel(%d)\n",
				 pHostMsg->Reg.command,
				 pHostMsg->RegArtesa.channel_number
				 ));

	switch (pHostMsg->Reg.command)
	{
		case DecStaMsgReply:

			if (QPCODEC_CHIP_ARTESA_ALL & This->m_pMpegCodec->m_ChipType)
				taskId = pHostMsg->RegArtesa.channel_number;
			else
				taskId = pHostMsg_status->Reg.msg_id;

			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
						 "CDecoderTask_ProcessArmMessage() DecStaMsgReply param1 == (%d) taskId(%d)!!\n",
						 param1,
						 taskId
						 ));

			if (StopDecompress == param1)
			{
				if (QPCODEC_CHIP_BERINGER == This->m_pMpegCodec->m_ChipType)
				{
					if (!CTask_FindTask(This,
										TASK_TYPE_DEC,
										TASK_DATA_TYPE_COMP_VID,
										&taskId
										))
					{
						CTask_FindTask(This,
									   TASK_TYPE_DEC,
									   TASK_DATA_TYPE_COMP_AUD,
									   &taskId
									   );
					}
				}

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CDecoderTask_ProcessArmMessage() DecStaMsgReply StopDecompress taskId(%d)\n",
							 taskId
							 ));
				QPOSMSetEvtgrp(This->m_TaskData[taskId].m_EvtReply,
							   QLCODEC_EVENT_STOP_DONE
							   );
			}
			else if (DecGetPlayInfo == param1)
			{
				TQP_BOOL	bSignalCaller = QP_TRUE;

				switch (param2)
				{
					case QPDEC_INFO_TYPE_MASTER_CLK:
						QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
									 "CDecoderTask_ProcessArmMessage() DecGetPlayInfo QPDEC_INFO_TYPE_MASTER_CLK\n"
									 ));
						break;
					case QPDEC_INFO_TYPE_CUR_DISP_PTS:
						QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
									 "CDecoderTask_ProcessArmMessage() DecGetPlayInfo QPDEC_INFO_TYPE_CUR_DISP_PTS\n"
									 ));
						break;
					case QPDEC_INFO_TYPE_DISPLAYED_CNT:
						QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
									 "CDecoderTask_ProcessArmMessage() DecGetPlayInfo QPDEC_INFO_TYPE_DISPLAYED_CNT\n"
									 ));
						break;
					case QPDEC_INFO_TYPE_DECODED_CNT:
						QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
									 "CDecoderTask_ProcessArmMessage() DecGetPlayInfo QPDEC_INFO_TYPE_DECODED_CNT\n"
									 ));
						break;
					case QPDEC_INFO_TYPE_GET_SHARE_MEM:
						QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
									 "CDecoderTask_ProcessArmMessage() DecGetPlayInfo QPDEC_INFO_TYPE_GET_SHARE_MEM wr_addr(0x%x) rd_addr(0x%x)\n",
									 param3,
									 param4
									 ));
						This->m_TaskData[taskId].m_decFWSharedMemWr = param3;
						This->m_TaskData[taskId].m_decFWSharedMemRd = param4;
						break;
					default:
						QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
									 "CDecoderTask_ProcessArmMessage() DecGetPlayInfo unknown(%d)\n",
									 param2
									 ));
						bSignalCaller = QP_FALSE;
						break;
				}

				if (bSignalCaller)
				{
					QPOSMSetEvtgrp(This->m_TaskData[taskId].m_EvtReply,
								   QLCODEC_EVENT_DEC_GOT_PLAY_INFO
								   );
				}
			}
			else if (GetOsdMemAddr == param1)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
									 "CDecoderTask_ProcessArmMessage() GetOsdMemAddr start_addr(0x%x)\n",
									 param2
									 ));
				This->m_TaskData[taskId].m_decFWSharedMemWr = param2;

				QPOSMSetEvtgrp(This->m_TaskData[taskId].m_EvtReply,
								   QLCODEC_EVENT_DEC_GOT_PLAY_INFO
								   );
			}

			QPFWAPI_AckARMMessage(This->m_pMpegCodec,
								  pHostMsg,
								  pHostMsg_status,
								  QP_TRUE
								  );
			break;

		case DecDataXferReq:
		{
			ARM_BUFFER_TYPE		type;
			TQP_BOOL			bTaskDataFound;
			CChannel			*pChannel = QP_NULL;
			CHANNEL_DIRECTION	dir = CHANNEL_DIR_NONE;
			TQP_UINT32			ulWidth = 1280, ulHeight = 720;

			if (QPCODEC_CHIP_ARTESA_ALL & This->m_pMpegCodec->m_ChipType)
			{
				taskId = pHostMsg->RegArtesa.channel_number;
				bTaskDataFound = CTask_FindDataType(This,
													taskId,
													(FIRMWARE_DATA_TYPE)(param1 & 0xFFFF),
													&dataType
													);
			}
			else
			{
				bTaskDataFound = CTask_FindTaskDataType(This,
														TASK_TYPE_DEC,
														(FIRMWARE_DATA_TYPE)(param1 & 0xFFFF),
														&taskId,
														&dataType
														);
			}

			if (!bTaskDataFound)
			{
#if 1
				QPFWAPI_AckARMMessage(This->m_pMpegCodec,
									  pHostMsg,
									  pHostMsg_status,
									  QP_TRUE
									  );
#else
				CTask_ReturnArmBuffer(This,
									  pHostMsg,
									  pHostMsg_status,
									  param1,
									  QP_TRUE
									  );
#endif
				return;
			}

			// get the channel and direction associated with this message
			//
			pChannel = This->m_TaskData[taskId].pChannel[dataType];
			dir = This->m_TaskData[taskId].direction[dataType];

			switch (param1 & 0xFFFF)
			{
				case FWDataTypeCmprSysVidInput:
				case FWDataTypeCmprAudInput:
				case FWDataTypeIndexInput:
				case FWDataTypeRawAudOutput:
				case FWDataTypeRawAudInput:
					type = ARM_BUF_OTHERS;
					break;

				case FWDataTypeRawVidOutput:
				{
					TQP_UINT32	ulFormat = QP_YUV_DATA_TYPE_QYUV;

					pChannel->GetYUVFormat(pChannel,
										   &ulFormat
										   );
					pChannel->GetResolution(pChannel,
											&ulWidth,
											&ulHeight
											);

					if (QP_YUV_DATA_TYPE_QYUV == ulFormat)
					{
						type = ARM_BUF_YUV;
					}
					else
					{
						type = ARM_BUF_YUVMB2RAS;
					}

					break;
				}

				default:
					CTask_ReturnArmBuffer(This,
										  pHostMsg,
										  pHostMsg_status,
										  param1,
										  QP_TRUE
										  );
					return;
			}

			This->m_TaskData[taskId].ArmRequestNumber[dataType]++;

			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN/*QPDBGM_LEVEL_TRACE*/),
						 "CDecoderTask_ProcessArmMessage() DecDataXferReq task(%d) dataType(%d):#(%d) msg_id(%d) param1(0x%x) param2(0x%x) param3(0x%x) param4(0x%x) param5(%d)\n",
						 taskId,
						 dataType,
						 This->m_TaskData[taskId].ArmRequestNumber[dataType],
						 pHostMsg_status->RegArtesa.channel_number,
						 param1,
						 param2,
						 param3,
						 param4,
						 param5
						 ));

#ifdef QP_LINXKER
			PROFILING_LOG(EVENT_FW_D_OUT_REQ2, taskId, This->m_TaskData[taskId].direction[dataType]/*dataType*/, 0, This->m_TaskData[taskId].ArmRequestNumber[dataType], param4*4);
#endif

			ENTER_CRITICAL(&This->m_TaskData[taskId].m_Object)

			// remember the message
			//
			if (ARM_BUF_YUV == type)
			{
				TASK_SAVE_YUV_MESSAGE(&This->m_TaskData[taskId],
									  dataType,
									  pHostMsg->Read,
									  pHostMsg_status->Read,
									  param1,
									  param2,
									  param3,
									  param4,
									  param5 & 0x7FFFFFFF,
									  (CHANNEL_DIR_READ == dir) && (0 != (param5 & 0x80000000)),/*usePTS*/
									  QP_FALSE,
									  ulWidth,
									  ulHeight
									  );
			}
			else if (ARM_BUF_YUVMB2RAS == type)
			{
				TASK_SAVE_YUVMB2RAS_MESSAGE(&This->m_TaskData[taskId],
											dataType,
											pHostMsg->Read,
											pHostMsg_status->Read,
											param1,
											param2,
											param3,
											param4,
											param5 & 0x7FFFFFFF,
											(CHANNEL_DIR_READ == dir) && (0 != (param5 & 0x80000000)),
											QP_FALSE,
											ulWidth,
											ulHeight
											);
			}
			else
			{
				TASK_SAVE_OTHERS_MESSAGE(&This->m_TaskData[taskId],
										 dataType,
										 pHostMsg->Read,
										 pHostMsg_status->Read,
										 param1,
										 param2,
										 param3,
										 param4,
										 param5 & 0x7FFFFFFF,
										 (CHANNEL_DIR_READ == dir) && (0 != (param5 & 0x80000000)),
										 QP_FALSE
										 );
			}

			LEAVE_CRITICAL(&This->m_TaskData[taskId].m_Object)

			QPFWAPI_AckARMMessage(This->m_pMpegCodec,
								  pHostMsg,
								  pHostMsg_status,
								  QP_TRUE
								  );


			if (TASK_STATE_IDLE == This->m_TaskData[taskId].m_State)
			{
				ENTER_CRITICAL(&This->m_TaskData[taskId].m_Object)

				do {
					// flush the active request
					This->CompleteArm(This,
									  &This->m_TaskData[taskId],
									  dataType
									  );
				} while (CFifo_GetFifo(This->m_TaskData[taskId].pArmMsgFifo[dataType],
									   &This->m_TaskData[taskId].ArmRequest[dataType]
									   ));

				LEAVE_CRITICAL(&This->m_TaskData[taskId].m_Object)
			}
			else
			{
				// kick off streaming
				//
				This->ProcessDataStreaming(This);
			}

			break;
		}

		case DecStatusReport:
		{
			CChannel	*pVidChannel = QP_NULL;
			CChannel	*pAudChannel = QP_NULL;
			TQP_UINT32	dwCode = 0;

			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN/*QPDBGM_LEVEL_TRACE*/),
						 "CDecoderTask_ProcessArmMessage() DecStatusReport param1 == (%d) param2 == (%d)!!\n",
						 param1,
						 param2
						 ));

			QPFWAPI_AckARMMessage(This->m_pMpegCodec,
								  pHostMsg,
								  pHostMsg_status,
								  QP_TRUE
								  );

			// figure out if we need to notify the status
			//
			if (QPDEC_STATUS_TYPE_ERR == param1)
			{
				switch (param2)
				{
					case QPDEC_STATUS_SEQUENCE_PARAM_CHANGED:
						if (QPCODEC_CHIP_ARTESA_ALL & This->m_pMpegCodec->m_ChipType)
						{
							taskId = pHostMsg->RegArtesa.channel_number;
							pVidChannel = This->m_TaskData[taskId].pChannel[TASK_DATA_TYPE_COMP_VID];
						}
						else
						{
							if (CTask_FindTask(This,
											   TASK_TYPE_DEC,
											   TASK_DATA_TYPE_COMP_VID,
											   &taskId
											   ))
							{
								pVidChannel = This->m_TaskData[taskId].pChannel[TASK_DATA_TYPE_COMP_VID];
							}
						}
						dwCode = QPMPGCODEC_CMD_DEC_SEQ_PARAM_CHANGED;
						break;

					case QPDEC_STATUS_BITSTREAM_ERROR:
						if (QPCODEC_CHIP_ARTESA_ALL & This->m_pMpegCodec->m_ChipType)
						{
							taskId = pHostMsg->RegArtesa.channel_number;
							pVidChannel = This->m_TaskData[taskId].pChannel[TASK_DATA_TYPE_COMP_VID];
						}
						else
						{
							if (CTask_FindTask(This,
											   TASK_TYPE_DEC,
											   TASK_DATA_TYPE_COMP_VID,
											   &taskId
											   ))
							{
								pVidChannel = This->m_TaskData[taskId].pChannel[TASK_DATA_TYPE_COMP_VID];
							}
						}
						dwCode = QPMPGCODEC_CMD_DEC_VID_ERR;
						break;

					default:
						break;
				}
			}
			else if (QPDEC_STATUS_TYPE_DISP_READY == param1)
			{
				if (QPCODEC_CHIP_ARTESA_ALL & This->m_pMpegCodec->m_ChipType)
				{
					taskId = pHostMsg->RegArtesa.channel_number;
					pVidChannel = This->m_TaskData[taskId].pChannel[TASK_DATA_TYPE_COMP_VID];
				}
				else
				{
					if (CTask_FindTask(This,
									   TASK_TYPE_DEC,
									   TASK_DATA_TYPE_COMP_VID,
									   &taskId
									   ))
					{
						pVidChannel = This->m_TaskData[taskId].pChannel[TASK_DATA_TYPE_COMP_VID];
					}
				}
				dwCode = QPMPGCODEC_CMD_DEC_RDY_TO_DISP;
			}
			else if (QPDEC_STATUS_TYPE_DEC_DONE == param1)
			{
				if (QPCODEC_CHIP_ARTESA_ALL & This->m_pMpegCodec->m_ChipType)
				{
					taskId = pHostMsg->RegArtesa.channel_number;
					pVidChannel = This->m_TaskData[taskId].pChannel[TASK_DATA_TYPE_COMP_VID];
					pAudChannel = This->m_TaskData[taskId].pChannel[TASK_DATA_TYPE_COMP_AUD];
				}
				else
				{
					if (CTask_FindTask(This,
									   TASK_TYPE_DEC,
									   TASK_DATA_TYPE_COMP_VID,
									   &taskId
									   ))
					{
						pVidChannel = This->m_TaskData[taskId].pChannel[TASK_DATA_TYPE_COMP_VID];
					}
					if (CTask_FindTask(This,
									   TASK_TYPE_DEC,
									   TASK_DATA_TYPE_COMP_AUD,
									   &taskId
									   ))
					{
						pAudChannel = This->m_TaskData[taskId].pChannel[TASK_DATA_TYPE_COMP_AUD];
					}
				}
				dwCode = QPMPGCODEC_CMD_DEC_COMPLETE;
				This->m_TaskData[taskId].bDone = QP_TRUE;
			}

			if (pVidChannel)
			{
				CChannel_DeviceCallback(pVidChannel,
										dwCode,
										QP_NULL
										);
			}
			if (pAudChannel)
			{
				CChannel_DeviceCallback(pAudChannel,
										dwCode,
										QP_NULL
										);
			}
			break;
		}

		default:
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
						 "CDecoderTask_ProcessArmMessage() Unknown Command == (0x%x)!!\n",
						 pHostMsg->Reg.command
						 ));
			QPFWAPI_AckARMMessage(This->m_pMpegCodec,
								  pHostMsg,
								  pHostMsg_status,
								  QP_TRUE
								  );
			break;
	}
}


#define _DEC_MINIMUM_DATA_RET		0x1000

void CDecoderTask_ProcessIoComplete(CTask *This,
									PTASK_DATA pTaskData,
									TASK_DATA_TYPE dataType,
									TQP_BOOL bStreaming
									)
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, /*QPDBGM_LEVEL_WARN*/QPDBGM_LEVEL_TRACE),
				 "CDecoderTask_ProcessIoComplete() taskId(%d) dataType(%d)\n",
				 pTaskData->id,
				 dataType
				 ));

	if (TASK_DATA_TYPE_YUV == dataType)
	{
		if (QP_NULL != pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.ALL.valid)
		{
			if (((ARM_BUF_YUV == pTaskData->ArmRequest[dataType].ArmBuffer.type) &&
				(pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulYOffset == pTaskData->UserBuffer[dataType].BUFFER.YUV.ulYLength) &&
				(pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulUVOffset == pTaskData->UserBuffer[dataType].BUFFER.YUV.ulUVLength)
				) ||
				((ARM_BUF_YUVMB2RAS == pTaskData->ArmRequest[dataType].ArmBuffer.type) &&
				(pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulYOffset == pTaskData->UserBuffer[dataType].BUFFER.YUVRAS.ulYLength) &&
				(pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulUOffset == pTaskData->UserBuffer[dataType].BUFFER.YUVRAS.ulULength) &&
				(pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulVOffset == pTaskData->UserBuffer[dataType].BUFFER.YUVRAS.ulVLength)
				) ||
				((ARM_BUF_YUVRAS == pTaskData->ArmRequest[dataType].ArmBuffer.type) &&
				(pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulYOffset == pTaskData->UserBuffer[dataType].BUFFER.YUVRAS.ulYLength) &&
				(pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulUOffset == pTaskData->UserBuffer[dataType].BUFFER.YUVRAS.ulULength) &&
				(pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulVOffset == pTaskData->UserBuffer[dataType].BUFFER.YUVRAS.ulVLength)
				) ||
				((ARM_BUF_OTHERS == pTaskData->ArmRequest[dataType].ArmBuffer.type) &&
				(pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulOffset == (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulSize << 2))
				))
			{
				// complete arm buffer
				//
				This->CompleteArm(This,
								  pTaskData,
								  dataType
								  );
				// complete user buffer
				//
				This->CompleteUser(This,
								   pTaskData,
								   dataType
								   );
			}
		}
		else
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
						 "CDecoderTask_ProcessIoComplete() ARM (YUV)buffer released already!\n"
						 ));
			This->CompleteUser(This,
							   pTaskData,
							   dataType
							   );
		}
	}
	else
	{
		TQP_BOOL	bReleaseUser = QP_FALSE;

		// check arm buffer
		//
		if (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.ALL.valid)
		{
			if (pTaskData->UserBuffer[dataType].pBufDesc)
			{
				if (pTaskData->UserBuffer[dataType].pBufDesc->ulBufferOffset == pTaskData->UserBuffer[dataType].BUFFER.OTHERS.ulLength)
				{
					if (pTaskData->UserBuffer[dataType].pBufDesc->ulFlags & QP_BUFDESC_FLAG_EOS)
					{
						pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulLast = QP_TRUE;
					}

					pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulFrameFlags = 0;
					if (pTaskData->UserBuffer[dataType].pBufDesc->ulFlags & QP_BUFDESC_FLAG_FRAME_START)
					{
						pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulFrameFlags |= 0x02;
					}
					if (pTaskData->UserBuffer[dataType].pBufDesc->ulFlags & QP_BUFDESC_FLAG_FRAME_END)
					{
						pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulFrameFlags |= 0x01;
					}

					// write unaligned bytes to the end of the arm buffer
					// this only works for MJPEG decoder frame aligned buffer
					//
					if (pTaskData->UserBuffer[dataType].pBufDesc->unAlignedNumberBytes > 0)
					{
						TQP_UINT32	ulArmAddress = (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulAddr << 2) + pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulOffset;
						TQP_UINT32	ulData = 0;

						QPHCI_MemoryWrite(&This->m_pMpegCodec->m_hci,
										  ulArmAddress,
										  pTaskData->UserBuffer[dataType].pBufDesc->unAlignedData
										  );
						QPHCI_MemoryRead(&This->m_pMpegCodec->m_hci,
										 ulArmAddress,
										 &ulData
										 );
						if (ulData != pTaskData->UserBuffer[dataType].pBufDesc->unAlignedData)
						{
							QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
										 "CDecoderTask_ProcessIoComplete() PCI_READ_ERROR read(0x%x)!=write(0x%x)\n",
										 ulData,
										 pTaskData->UserBuffer[dataType].pBufDesc->unAlignedData
										 ));
						}
						pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulOffset += 4;
						pTaskData->UserBuffer[dataType].pBufDesc->unAlignedNumberBytes = 0;
					}

					bReleaseUser = QP_TRUE;
				}
			}

			if (QPDEC_DEC_XFER_MODE_PARTIAL == pTaskData->m_decXferMode)
			{
				if (((0 != pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulOffset) &&
					bReleaseUser &&
					(pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulOffset >= pTaskData->m_decXferMinimumSize)) ||
					(0 == pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulSize) ||
					pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulLast ||
					(pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulOffset == (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulSize << 2))
					)
				{
					This->CompleteArm(This,
									  pTaskData,
									  dataType
									  );
				}
			}
			else
			{
				if ((pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulOffset == (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulSize << 2)) ||
					pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulLast
					)
				{
					This->CompleteArm(This,
									  pTaskData,
									  dataType
									  );
				}
			}

			if (bReleaseUser)
			{
				This->CompleteUser(This,
								   pTaskData,
								   dataType
								   );
			}
		}
	}

	if (bStreaming)
	{
#if 1
		This->ProcessDataStreaming(This);
#else
		QPOSMSetEvtgrp(This->m_Thread.m_EvtWait,
					   QLCODEC_EVENT_NEW_BUFFER
					   );
#endif
	}
}



