///////////////////////////////////////////////////////////////////////////////
//     
//
// Cencodertask.c
//
// Description:
//
//	encoder task
//
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#ifdef QP_LINXKER

#include "qlautoconf.h"

#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
#include <mach/pl330xp.h>
#endif
#include "plprofiling.h"
#endif

#include "ctask.h"
#include "qpdbgmgr.h"
#include "qposmgr.h"
#include "fwapi.h"
#include "cqlcodec.h"
#include "util.h"
#ifndef QP_LINXKER
#include <string.h>
#endif

extern TQP_DbgMCompID	g_QPCodecDBGCompID;




/////////////////////////////////////////////////////////////////////////////
//
//

// task interfaces
//
EQP_Errors CEncoderTask_Start(CTask *This,
 							  TQP_OBJECT_HANDLE hTask
 							  )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "CEncoderTask_Start hTask(%d)\n",
				 hTask
				 ));

	qpStatus = CQLCodec_UpdateMiscConfig(This->m_pMpegCodec,
										 hTask
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CEncoderTask_Start() CQLCodec_UpdateMiscConfig() failed(%d)\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	qpStatus = QPFWAPI_MailboxReady(This->m_pMpegCodec,
									500
									);
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CEncoderTask_Start() QPFWAPI_MailboxReady() failed(%d)\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	qpStatus = CQLCodec_UpdateEncoderConfig(This->m_pMpegCodec,
											hTask,
			     							QP_FALSE
			     							);
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_StartEncoder(This->m_pMpegCodec,
																		  (TQP_UINT32) hTask
																		  );
	QPFWAPI_MailboxDone(This->m_pMpegCodec);

	if (QP_SUCCESS(qpStatus))
	{
		This->m_TaskData[hTask].m_State = TASK_STATE_RUN;
	}

	return (qpStatus);
}



EQP_Errors CEncoderTask_Stop(CTask *This,
 							 TQP_OBJECT_HANDLE hTask
 							 )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CEncoderTask_Stop() hTask(%d)\n",
				 hTask
				 ));

	if (TASK_STATE_RUN == This->m_TaskData[hTask].m_State)
	{
		TQP_BOOL	bStopAtGOP;
		TQP_INT32	lWaitStopCompleteTimeOut;
		TQP_UINT32	ulEvents;
		TQP_INT64	llTimeTo;
		TQP_BOOL	bWaitForReadComplete = QP_FALSE;
		TQP_BOOL	bWaitForWriteComplete = QP_FALSE;

		QPOSMClearEvtgrp(This->m_TaskData[hTask].m_EvtReply,
						 QLCODEC_EVENT_STOP_DONE
						 );
		if (QP_NULL == This->m_TaskData[hTask].pChannel[TASK_DATA_TYPE_COMP_VID])
		{
			bStopAtGOP = QP_FALSE;
			lWaitStopCompleteTimeOut = 500;
		}
		else
		{
			if (QP_ENC_STOP_IMMEDIATELY == This->m_TaskData[hTask].m_EncStopMode)
			{
				bStopAtGOP = QP_FALSE;
				lWaitStopCompleteTimeOut = 1000;
			}
			else
			{
				bStopAtGOP = QP_TRUE;
				lWaitStopCompleteTimeOut = 2500;
			}
		}

		// send FW stop encoder command
		//
		qpStatus = QPFWENCAPI_StopEncoder(This->m_pMpegCodec,
										  (TQP_UINT32) hTask,
										  bStopAtGOP,
										  This->m_TaskData[hTask].m_etControl.Reg.data_channel_type
										  );

		// move encoder state to stop and wait for it
		// to be idle
		//
		This->m_TaskData[hTask].m_State = TASK_STATE_STOP;

		// wait for stop completion
		//
		if (QP_SUCCESS(qpStatus))
		{
#ifdef QP_EVENT_TIME_DEBUG
            TQP_UINT64 startT, endT;            
            startT = QPTMGetSysTimeMilliS();
#endif
			qpStatus = QPOSMWaitEvtgrp(This->m_TaskData[hTask].m_EvtReply,
									   QLCODEC_EVENT_STOP_DONE,
									   &ulEvents,
									   lWaitStopCompleteTimeOut
									   );
#ifdef QP_EVENT_TIME_DEBUG
            endT = QPTMGetSysTimeMilliS();
#endif
			if (!QP_SUCCESS(qpStatus))
			{
#ifndef QP_EVENT_TIME_DEBUG
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CEncoderTask_Stop() QPOSMWaitEvtgrp failed(%d)!!!\n",
							 qpStatus
							 ));
#else
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CEncoderTask_Stop() QPOSMWaitEvtgrp failed(%d) (%d)(0x%llx 0x%llx 0x%llx)!!!\n",
							 qpStatus, lWaitStopCompleteTimeOut, startT, endT, (endT-startT)
							 ));
#endif
			}
#ifdef QP_EVENT_TIME_DEBUG
			else
            {
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CEncoderTask_Stop() QPOSMWaitEvtgrp success(%d)!!!\n",
							 qpStatus
							 ));
            }
#endif
		}
		else
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CEncoderTask_Stop() QPFWENCAPI_StopEncoder failed(%d)!!!\n",
						 qpStatus
						 ));
		}

		// reset task state
		This->m_TaskData[hTask].m_State = TASK_STATE_IDLE;

		if (CQLCodec_IsCodecIdle(This->m_pMpegCodec))
		{
			// wait for the outstanding buffer to complete
			//
			ENTER_CRITICAL_IO_PENDING(This)
			bWaitForReadComplete = (This->m_pPending[CHANNEL_DIR_READ] && This->m_pPending[CHANNEL_DIR_READ]->pTaskData && (This->m_pPending[CHANNEL_DIR_READ]->pTaskData->id == hTask));
			bWaitForWriteComplete = (This->m_pPending[CHANNEL_DIR_WRITE] && This->m_pPending[CHANNEL_DIR_WRITE]->pTaskData && (This->m_pPending[CHANNEL_DIR_WRITE]->pTaskData->id == hTask));
			LEAVE_CRITICAL_IO_PENDING(This)

			if (bWaitForReadComplete)
			{
				llTimeTo = 1000;

				BeginTimer(&llTimeTo);

				while (This->m_pPending[CHANNEL_DIR_READ])
				{
					if (Timeout(llTimeTo,
								5
								))
					{
						QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
									 "CEncoderTask_Stop() wait for read DMA compelte timeout!!!\n"
									 ));
						break;
					}
				}
			}

			if (bWaitForWriteComplete)
			{
				llTimeTo = 1000;

				BeginTimer(&llTimeTo);

				while (This->m_pPending[CHANNEL_DIR_WRITE])
				{
					if (Timeout(llTimeTo,
								5
								))
					{
						QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
									 "CEncoderTask_Stop() wait for write DMA compelte timeout!!!\n"
									 ));
						break;
					}
				}
			}
		}
	}

	return (qpStatus);
}



EQP_Errors CEncoderTask_Pause(CTask *This,
 							  TQP_OBJECT_HANDLE hTask
 							  )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "CEncoderTask_Pause() hTask(%d)\n",
				 hTask
				 ));

	qpStatus = QPFWENCAPI_PauseEncoder(This->m_pMpegCodec,
									   (TQP_UINT32) hTask
									   );
	return (qpStatus);
}



EQP_Errors CEncoderTask_Resume(CTask *This,
 							   TQP_OBJECT_HANDLE hTask
 							   )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "CEncoderTask_Resume() hTask(%d)\n",
				 hTask
				 ));

	qpStatus = QPFWENCAPI_ResumeEncoder(This->m_pMpegCodec,
										(TQP_UINT32)hTask
										);
	return (qpStatus);
}



void CEncoderTask_ProcessArmMessage(CTask *This,
									PHOST_MESSAGE pHostMsg,
									PHOST_MESSAGE_STATUS pHostMsg_status,
									TQP_UINT32 param1,
									TQP_UINT32 param2,
									TQP_UINT32 param3,
									TQP_UINT32 param4,	// QL201B
									TQP_UINT32 param5	// QL201B
									)
{
	TQP_OBJECT_HANDLE   taskId;
	TASK_DATA_TYPE	dataType;

	// handle the message
	//
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CEncoderTask_ProcessArmMessage() cmd(0x%x) channel(%d) p1(0x%x) p2(0x%x) p3(0x%x) p4(0x%x) p5(0x%x)\n",
				 pHostMsg->Reg.command,
				 pHostMsg->RegArtesa.channel_number,
				 param1,
				 param2,
				 param3,
				 param4,
				 param5
				 ));

	switch (pHostMsg->Reg.command)
	{
		case EncStaMsgReply:
		{
			TQP_BOOL	bExtraParam = (0 != (param1 & 0x80000000));

			if (QPCODEC_CHIP_ARTESA_ALL & This->m_pMpegCodec->m_ChipType)
				taskId = pHostMsg->RegArtesa.channel_number;
			else
				taskId = pHostMsg_status->Reg.msg_id;

			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
						 "CEncoderTask_ProcessArmMessage() EncStaMsgReply param1 == (0x%x) taskId(%d)\n",
						 param1,
						 taskId
						 ));

			switch (param1 & 0x7FFFFFFF)
			{
				case StopCompress:

					if (QPCODEC_CHIP_BERINGER == This->m_pMpegCodec->m_ChipType)
					{
						if (DATA_CHANNEL_TYPE_MPEG_2_HIU == param2)
						{
							if (!CTask_FindTask(This,
												TASK_TYPE_ENC,
												TASK_DATA_TYPE_COMP_VID,
												(TQP_OBJECT_HANDLE *)&taskId
												))
							{
								CTask_FindTask(This,
											   TASK_TYPE_ENC,
											   TASK_DATA_TYPE_COMP_AUD,
											   (TQP_OBJECT_HANDLE *)&taskId
											   );
							}
						}
						else if (DATA_CHANNEL_TYPE_YUV_2_HIU == param2)
						{
							CTask_FindTask(This,
										   TASK_TYPE_ENC,
										   TASK_DATA_TYPE_YUV,
										   (TQP_OBJECT_HANDLE *)&taskId
										   );
						}
					}

					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
								 "CEncoderTask_ProcessArmMessage() EncStaMsgReply StopCompress taskId(%d) param2(0x%d)\n",
								 taskId,
								 param2
								 ));
					QPOSMSetEvtgrp(This->m_TaskData[taskId].m_EvtReply,
								   QLCODEC_EVENT_STOP_DONE
								   );
					break;

				case EncMiscMessage:
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
								 "CEncoderTask_ProcessArmMessage() EncStaMsgReply EncMiscMessage param2(%d)\n",
								 param2
								 ));
					if (EncMiscFreezeVideo == param2)
					{
						QPOSMSetEvtgrp(This->m_TaskData[taskId].m_EvtReply,
									   QLCODEC_EVENT_ENC_VIDEO_FROZEN
									   );
					}
					break;

				case GetViosdTableAddr:
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
								 "CEncoderTask_ProcessArmMessage() EncStaMsgReply GetViosdTableAddr(0x%x, 0x%x)\n",
								 param2,
								 param3
								 ));
					This->m_TaskData[taskId].m_encFontTableAddr = (param3<<2);
					This->m_TaskData[taskId].m_encTextListAddr = (param2<<2);
					This->m_TaskData[taskId].m_encSyncTimeAddr = (param4<<2);

					QPOSMSetEvtgrp(This->m_TaskData[taskId].m_EvtReply,
								   QLCODEC_EVENT_ENC_GOT_VIOSD_TABLE
								   );
					break;

				case GetCurVidBuf:
				{
					TQP_UINT32	nParamAddr = param2;
					TQP_UINT32	nParamNum = param3;

					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
								 "CEncoderTask_ProcessArmMessage() EncStaMsgReply GetCurVidBuf (addr(0x%x), size(%d))\n",
								 nParamAddr,
								 nParamNum
								 ));
					if (bExtraParam)
					{
						QPHCI_MemoryRead(&This->m_pMpegCodec->m_hci,
										 nParamAddr << 2,
										 &This->m_TaskData[taskId].m_encVidBufLumaAddr
										 );
						QPHCI_MemoryRead(&This->m_pMpegCodec->m_hci,
										 (nParamAddr + 1) << 2,
										 &This->m_TaskData[taskId].m_encVidBufChromaAddr
										 );
						QPHCI_MemoryRead(&This->m_pMpegCodec->m_hci,
										 (nParamAddr + 2) << 2,
										 &This->m_TaskData[taskId].m_encFrameLABufAddr
										 );
						QPHCI_MemoryRead(&This->m_pMpegCodec->m_hci,
										 (nParamAddr + 3) << 2,
										 &This->m_TaskData[taskId].m_encTopLABufAddr
										 );
						QPHCI_MemoryRead(&This->m_pMpegCodec->m_hci,
										 (nParamAddr + 4) << 2,
										 &This->m_TaskData[taskId].m_encBottomLABufAddr
										 );
						QPHCI_MemoryRead(&This->m_pMpegCodec->m_hci,
										 (nParamAddr + 5) << 2,
										 &This->m_TaskData[taskId].m_encVidBufPTS
										 );
					}
					else
					{
						This->m_TaskData[taskId].m_encVidBufLumaAddr = (TQP_UINT32)-1;
					}

					QPOSMSetEvtgrp(This->m_TaskData[taskId].m_EvtReply,
								   QLCODEC_EVENT_ENC_GOT_CUR_VID_BUF
								   );
					break;
				}

				default:
					break;
			}

			QPFWAPI_AckARMMessage(This->m_pMpegCodec,
								  pHostMsg,
								  pHostMsg_status,
								  QP_TRUE
								  );
			break;
		}

		case EncDataOutReq:
		case EncDataOutLastReq:
		{
			ARM_BUFFER_TYPE		type;
			TQP_BOOL			bTaskDataFound;
			CChannel			*pChannel = QP_NULL;
			CHANNEL_DIRECTION	dir = CHANNEL_DIR_NONE;
			TQP_UINT32			ulWidth=0, ulHeight=0;

			if (QPCODEC_CHIP_ARTESA_ALL & This->m_pMpegCodec->m_ChipType)
			{
				taskId = pHostMsg->RegArtesa.channel_number;
				bTaskDataFound = CTask_FindDataType(This,
													taskId,
													(FIRMWARE_DATA_TYPE)(param1 & 0xFFFF),
													&dataType
													);
			}
			else
			{
				bTaskDataFound = CTask_FindTaskDataType(This,
														TASK_TYPE_ENC,
														(FIRMWARE_DATA_TYPE)(param1 & 0xFFFF),
														(TQP_OBJECT_HANDLE *)&taskId,
														&dataType
														);
			}

			if (!bTaskDataFound)
			{
#if 1
				QPFWAPI_AckARMMessage(This->m_pMpegCodec,
									  pHostMsg,
									  pHostMsg_status,
									  QP_TRUE
									  );
#else
				CTask_ReturnArmBuffer(This,
									  pHostMsg,
									  pHostMsg_status,
									  param1,
									  QP_TRUE
									  );
#endif
				return;
			}

			// get the channel and direction associated with this message
			//
			pChannel = This->m_TaskData[taskId].pChannel[dataType];
			dir = This->m_TaskData[taskId].direction[dataType];

			switch (param1 & 0xFFFF)
			{
				case FWDataTypeRawVidInput:
					type = ARM_BUF_YUVRAS;
					break;

				case FWDataTypeCmprSysVidOutput:
#ifdef QP_DEBUG
					if (param4 == This->m_TaskData[taskId].ArmBufferAddr)
					{
						QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
									 ">>>> SAME SIZE REQUEST (addr(0x%x), size(%d)) <<<<<\n",
									 param2,
									 param4
									 ));
					}
					This->m_TaskData[taskId].ArmBufferAddr = param4;
#endif //QP_DEBUG
					// fall through
				case FWDataTypeRawAudInput:
				case FWDataTypeRawAudOutput:
				case FWDataTypeCmprAudOutput:
				case FWDataTypeIndexOutput:
				case FWDataTypeVBIOutput:
				case FWDataTypeRawYUYVInput:
				case FWDataTypeRawYUYVOutput:
					type = ARM_BUF_OTHERS;
					break;

				case FWDataTypeRawVidOutput:
				{
					TQP_UINT32	ulFormat = QP_YUV_DATA_TYPE_QYUV;

					// drop this frame if we have reach our threshhold
					//
					if (((QPHCI_BUS_USB == This->m_pMpegCodec->m_hci.m_bus_type) ||
						(QPHCI_BUS_INTERNAL == This->m_pMpegCodec->m_hci.m_bus_type)) &&
						(CFifo_GetFifoLevel(This->m_TaskData[taskId].pArmMsgFifo[dataType]) >= 2)
						)
					{
						CTask_ReturnArmBuffer(This,
											  pHostMsg,
											  pHostMsg_status,
											  param1,
											  QP_TRUE
											  );
						return;
					}

					// get YUV format and resolution
					//
					pChannel->GetYUVFormat(pChannel,
										   &ulFormat
										   );
					pChannel->GetResolution(pChannel,
											&ulWidth,
											&ulHeight
											);

					if (QP_YUV_DATA_TYPE_QYUV == ulFormat)
					{
						type = ARM_BUF_YUV;
					}
					else
					{
						type = ARM_BUF_YUVMB2RAS;
					}
					break;
				}

				case FWDataTypeCmprSysVidInput:
				case FWDataTypeCmprAudInput:
				case FWDataTypeIndexInput:
				default:
					CTask_ReturnArmBuffer(This,
										  pHostMsg,
										  pHostMsg_status,
										  param1,
										  QP_TRUE
										  );
					return;
			}

			if (param1 & 0xFF0000)
			{
			switch ((param1 & 0xFF0000)>>16)
			{
				case RawAudioAC3:
				case RawAudioAAC:
				case RawAudioAAC2:
				case RawAudioDTS:
					This->m_TaskData[taskId].m_audioControlExLPCM.Reg.raw_type = (param1 & 0xFF0000)>>16;
					break;

				default:
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CEncoderTask_ProcessArmMessage() unknown compressed audio type(p1)=0x%x\n", param1 & 0xFF0000));
					break;
			}
			}

			// increment number of arm request
			//
			This->m_TaskData[taskId].ArmRequestNumber[dataType]++;

			if (EncDataOutReq == pHostMsg->Reg.command)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN/*QPDBGM_LEVEL_TRACE*/),
							 "CEncoderTask_ProcessArmMessage() EncDataOutReq task(%d) dataType(%d):#(%d) msg_id(%d) p1(0x%x) p2(0x%x) p3(0x%x) p4(0x%x) p5(0x%x)\n",
							 taskId,
							 dataType,
							 This->m_TaskData[taskId].ArmRequestNumber[dataType],
							 pHostMsg_status->RegArtesa.channel_number,
							 param1,
							 param2,
							 param3,
							 param4,
							 param5
							 ));

#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
#if ENABLE_CODEC_MEM_LOG
				codec_mem_log_add_ex(CODEC_MEM_LOG_PROCESS_ARM_MESSAGE, This->m_TaskData[taskId].ArmRequestNumber[dataType], param4*4);
#endif
#endif
#ifdef QP_LINXKER
				PROFILING_LOG(EVENT_FW_D_OUT_REQ, taskId, dataType, 0, This->m_TaskData[taskId].ArmRequestNumber[dataType], param4*4);
#endif

			}
			else
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CEncoderTask_ProcessArmMessage() EncDataOutLastReq task(%d) dataType(%d):#(%d) msg_id(%d) p1(0x%x) p2(0x%x) p3(0x%x) p4(0x%x) p5(%d)\n",
							 taskId,
							 dataType,
							 This->m_TaskData[taskId].ArmRequestNumber[dataType],
							 pHostMsg_status->RegArtesa.channel_number,
							 param1,
							 param2,
							 param3,
							 param4,
							 param5
							 ));
#ifdef QP_LINXKER
				PROFILING_LOG(EVENT_FW_D_OUT_LAST_REQ, taskId, dataType, 0, This->m_TaskData[taskId].ArmRequestNumber[dataType], param4*4);
#endif

				if ((0 == param2) &&
					(0 == param4)
					)
				{
					// set a bogus address so the logic to determine
					// valid ARM request won't break
					//
					param2 = ARM_BUFFER_BOGUS_ADDRESS;
				}
			}

			ENTER_CRITICAL(&This->m_TaskData[taskId].m_Object)

			// remember the message
			//
			if (ARM_BUF_YUV == type)
			{
				TASK_SAVE_YUV_MESSAGE(&This->m_TaskData[taskId],
									  dataType,
									  pHostMsg->Read,
									  pHostMsg_status->Read,
									  param1,
									  param2,
									  param3,
									  param4,
									  param5 & 0x7FFFFFFF,
									  (CHANNEL_DIR_READ == dir) && (0 != (param5 & 0x80000000)),
									  (EncDataOutLastReq == pHostMsg->Reg.command),
									  ulWidth,
									  ulHeight
									  );
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "%s(): ARM_BUF_YUV msg_level(%d) task(%d) dataType(%d):#(%d) msg_id(%d)\n",
							 __FUNCTION__,
							 CFifo_GetFifoLevel(This->m_TaskData[taskId].pArmMsgFifo[dataType]),
							 taskId,
							 dataType,
							 This->m_TaskData[taskId].ArmRequestNumber[dataType],
							 pHostMsg_status->RegArtesa.channel_number
							 ));
			}
			else if (ARM_BUF_YUVMB2RAS == type)
			{
				TASK_SAVE_YUVMB2RAS_MESSAGE(&This->m_TaskData[taskId],
											dataType,
											pHostMsg->Read,
											pHostMsg_status->Read,
											param1,
											param2,
											param3,
											param4,
											param5 & 0x7FFFFFFF,
											(CHANNEL_DIR_READ == dir) && (0 != (param5 & 0x80000000)),
											(EncDataOutLastReq == pHostMsg->Reg.command),
											ulWidth,
											ulHeight
											);
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "%s(): ARM_BUF_YUVMB2RAS msg_level(%d) task(%d) dataType(%d):#(%d) msg_id(%d)\n",
							 __FUNCTION__,
							 CFifo_GetFifoLevel(This->m_TaskData[taskId].pArmMsgFifo[dataType]),
							 taskId,
							 dataType,
							 This->m_TaskData[taskId].ArmRequestNumber[dataType],
							 pHostMsg_status->RegArtesa.channel_number
							 ));
			}
			else if (ARM_BUF_YUVRAS == type)
			{
				TASK_SAVE_YUVRAS_MESSAGE(&This->m_TaskData[taskId],
										 dataType,
										 pHostMsg->Read,
										 pHostMsg_status->Read,
										 param1,
										 param2,
										 param3,
										 param4,
										 param5 & 0x7FFFFFFF,
										 (CHANNEL_DIR_READ == dir) && (0 != (param5 & 0x80000000)),
										 (EncDataOutLastReq == pHostMsg->Reg.command)
										 );
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "%s(): ARM_BUF_YUVRAS msg_level(%d) task(%d) dataType(%d):#(%d) msg_id(%d)\n",
							 __FUNCTION__,
							 CFifo_GetFifoLevel(This->m_TaskData[taskId].pArmMsgFifo[dataType]),
							 taskId,
							 dataType,
							 This->m_TaskData[taskId].ArmRequestNumber[dataType],
							 pHostMsg_status->RegArtesa.channel_number
							 ));
			}
			else
			{
				TASK_SAVE_OTHERS_MESSAGE(&This->m_TaskData[taskId],
										 dataType,
										 pHostMsg->Read,
										 pHostMsg_status->Read,
										 param1,
										 param2,
										 param3,
										 param4,
										 param5 & 0x7FFFFFFF,
										 (CHANNEL_DIR_READ == dir) && (0 != (param5 & 0x80000000)),
										 (EncDataOutLastReq == pHostMsg->Reg.command)
										 );
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "%s(): ARM_BUF_OTHERS msg_level(%d) task(%d) dataType(%d):#(%d) msg_id(%d)\n",
							 __FUNCTION__,
							 CFifo_GetFifoLevel(This->m_TaskData[taskId].pArmMsgFifo[dataType]),
							 taskId,
							 dataType,
							 This->m_TaskData[taskId].ArmRequestNumber[dataType],
							 pHostMsg_status->RegArtesa.channel_number
							 ));
			}

			LEAVE_CRITICAL(&This->m_TaskData[taskId].m_Object)

			QPFWAPI_AckARMMessage(This->m_pMpegCodec,
								  pHostMsg,
								  pHostMsg_status,
								  QP_TRUE
								  );

			if (TASK_STATE_IDLE == This->m_TaskData[taskId].m_State)
			{
				ENTER_CRITICAL(&This->m_TaskData[taskId].m_Object)

				do {
					// flush the active request
					This->CompleteArm(This,
									  &This->m_TaskData[taskId],
									  dataType
									  );
				} while (CFifo_GetFifo(This->m_TaskData[taskId].pArmMsgFifo[dataType],
									   &This->m_TaskData[taskId].ArmRequest[dataType]
									   ));

				LEAVE_CRITICAL(&This->m_TaskData[taskId].m_Object)
			}
			else
			{
				// kick off streaming even for zero size buffer
				//
				This->ProcessDataStreaming(This);
			}
			break;
		}

		case EncErrorReport:
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
						 "CEncoderTask_ProcessArmMessage() EncErrorReport param1 == (%d)!!\n",
						 param1
						 ));

			QPFWAPI_AckARMMessage(This->m_pMpegCodec,
								  pHostMsg,
								  pHostMsg_status,
								  QP_TRUE
								  );
			break;

		default:
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
						 "CEncoderTask_ProcessArmMessage() Unknown Command == (0x%x)!!\n",
						 pHostMsg->Reg.command
						 ));

			QPFWAPI_AckARMMessage(This->m_pMpegCodec,
								  pHostMsg,
								  pHostMsg_status,
								  QP_TRUE
								  );
			break;
	}
}



void CEncoderTask_ProcessIoComplete(CTask *This,
									PTASK_DATA pTaskData,
									TASK_DATA_TYPE dataType,
									TQP_BOOL bStreaming
									)
{
	TQP_BOOL	bMoreProcessingRequired = QP_FALSE;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, /*QPDBGM_LEVEL_WARN*/QPDBGM_LEVEL_TRACE),
				 "CEncoderTask_ProcessIoComplete() Task_id(%d) arm_buf(lx%lx) usr_buf(lx%lx) usr_offset(%d) usr_size(%d)\n",
				 pTaskData->id,
				 pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.ALL.valid,
				 pTaskData->UserBuffer[dataType].pBufDesc,
				 pTaskData->UserBuffer[dataType].pBufDesc->ulBufferOffset,
				 pTaskData->UserBuffer[dataType].pBufDesc->ulBufferSize
				 ));

	if (TASK_DATA_TYPE_YUV == dataType)
	{
		if (QP_NULL != pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.ALL.valid)
		{
			if (((ARM_BUF_YUV == pTaskData->ArmRequest[dataType].ArmBuffer.type) &&
				(pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulYOffset == pTaskData->UserBuffer[dataType].BUFFER.YUV.ulYLength) &&
				(pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulUVOffset == pTaskData->UserBuffer[dataType].BUFFER.YUV.ulUVLength)
				) ||
				((ARM_BUF_YUVMB2RAS == pTaskData->ArmRequest[dataType].ArmBuffer.type) &&
				(pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulYOffset == pTaskData->UserBuffer[dataType].BUFFER.YUVRAS.ulYLength) &&
				(pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulUOffset == pTaskData->UserBuffer[dataType].BUFFER.YUVRAS.ulULength) &&
				(pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulVOffset == pTaskData->UserBuffer[dataType].BUFFER.YUVRAS.ulVLength)
				) ||
				((ARM_BUF_YUVRAS == pTaskData->ArmRequest[dataType].ArmBuffer.type) &&
				(pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulYOffset == pTaskData->UserBuffer[dataType].BUFFER.YUVRAS.ulYLength) &&
				(pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulUOffset == pTaskData->UserBuffer[dataType].BUFFER.YUVRAS.ulULength) &&
				(pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulVOffset == pTaskData->UserBuffer[dataType].BUFFER.YUVRAS.ulVLength)
				) ||
				((ARM_BUF_OTHERS == pTaskData->ArmRequest[dataType].ArmBuffer.type) &&
				(pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulOffset == (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulSize << 2))
				))
			{
				// complete arm buffer
				//
				This->CompleteArm(This,
								  pTaskData,
								  dataType
								  );

				// complete user buffer
				//
				This->CompleteUser(This,
								   pTaskData,
								   dataType
								   );
			}
			else
			{
				bMoreProcessingRequired = QP_TRUE;
			}
		}
		else
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID,  QPDBGM_LEVEL_WARN),
						 "CEncoderTask_ProcessIoComplete() ARM (YUV)buffer released already!"
						 ));
			This->CompleteUser(This,
							   pTaskData,
							   dataType
							   );
		}
	}
#if 0 // Prevent a error case in virtual audio device while audio source is HDMI
      // In such case, filter connect virtual audio device in 44.1K sampling rate while HDMI provide 48K sampling rate data
	else if (TASK_DATA_TYPE_PCM == dataType)
	{
		TQP_BOOL    bPCMFinish = QP_FALSE;
		// check arm buffer
		//
		if (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.ALL.valid)
		{
			if (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulOffset <= (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulSize << 2))
			{
                if (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulOffset < (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulSize << 2))
                {
                    bPCMFinish = QP_TRUE;
                }

				if (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulLast)
				{
					pTaskData->UserBuffer[dataType].pBufDesc->ulFlags |= QP_BUFDESC_FLAG_EOS;
					This->CompleteUser(This,
									   pTaskData,
									   dataType
									   );
				}
				else
				{
					if (pTaskData->UserBuffer[dataType].pBufDesc->ulFlags & QP_BUFDESC_FLAG_PARTIAL_FILL_OK)
					{
						if ((pTaskData->UserBuffer[dataType].pBufDesc->ulFlags & QP_BUFDESC_FLAG_FRAME_ALIGNED) &&
							(pTaskData->UserBuffer[dataType].pBufDesc->ulFlags & QP_BUFDESC_FLAG_INCOMPLETE_FRAME) &&
							(pTaskData->UserBuffer[dataType].pBufDesc->ulBufferOffset < pTaskData->UserBuffer[dataType].pBufDesc->ulBufferSize)
							)
						{
							pTaskData->UserBuffer[dataType].pBufDesc->ulFlags &= ~QP_BUFDESC_FLAG_INCOMPLETE_FRAME;
						}
						else
						{
							This->CompleteUser(This,
											   pTaskData,
											   dataType
											   );
						}
					}
				}
				This->CompleteArm(This,
								  pTaskData,
								  dataType
								  );
			}
			else
			{
				bMoreProcessingRequired = QP_TRUE;
			}

			if (pTaskData->UserBuffer[dataType].pBufDesc)
			{
				if ((pTaskData->UserBuffer[dataType].pBufDesc->ulBufferOffset == pTaskData->UserBuffer[dataType].pBufDesc->ulBufferSize) || (bPCMFinish == QP_TRUE))
				{
					if (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.ALL.valid)
					{
						pTaskData->UserBuffer[dataType].pBufDesc->ulFlags |= QP_BUFDESC_FLAG_INCOMPLETE_FRAME;
					}
					This->CompleteUser(This,
									   pTaskData,
									   dataType
									   );
				}
			}
		}
	}
#endif
	else
	{
		// check arm buffer
		//
		if (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.ALL.valid)
		{
			if (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulOffset == (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulSize << 2))
			{
#if 1
				switch (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulCompressAudioType) {
				    case RawAudioAC3:
				        pTaskData->UserBuffer[dataType].pBufDesc->ulFlags |= QP_BUFDESC_FLAG_RAW_AUDIO_AC3;
				        break;
				    case RawAudioAAC:
				        pTaskData->UserBuffer[dataType].pBufDesc->ulFlags |= QP_BUFDESC_FLAG_RAW_AUDIO_AAC;
				        break;
				    case RawAudioAAC2:
				        pTaskData->UserBuffer[dataType].pBufDesc->ulFlags |= QP_BUFDESC_FLAG_RAW_AUDIO_AAC2;
				        break;
				    case RawAudioDTS:
				        pTaskData->UserBuffer[dataType].pBufDesc->ulFlags |= QP_BUFDESC_FLAG_RAW_AUDIO_DTS;
					    break;
				}
			    QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
						     "%s() compressed audio type (0x%x) ulFlags(0x%08x)\n",
						     __FUNCTION__,
						     pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulCompressAudioType,
						     pTaskData->UserBuffer[dataType].pBufDesc->ulFlags));
#endif

				if (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulLast)
				{
					pTaskData->UserBuffer[dataType].pBufDesc->ulFlags |= QP_BUFDESC_FLAG_EOS;

					This->CompleteUser(This,
									   pTaskData,
									   dataType
									   );
				}
				else
				{
					if (pTaskData->UserBuffer[dataType].pBufDesc->ulFlags & QP_BUFDESC_FLAG_PARTIAL_FILL_OK)
					{
						if ((pTaskData->UserBuffer[dataType].pBufDesc->ulFlags & QP_BUFDESC_FLAG_FRAME_ALIGNED) &&
							(pTaskData->UserBuffer[dataType].pBufDesc->ulFlags & QP_BUFDESC_FLAG_INCOMPLETE_FRAME) &&
							(pTaskData->UserBuffer[dataType].pBufDesc->ulBufferOffset < pTaskData->UserBuffer[dataType].pBufDesc->ulBufferSize)
							)
						{
							pTaskData->UserBuffer[dataType].pBufDesc->ulFlags &= ~QP_BUFDESC_FLAG_INCOMPLETE_FRAME;
						}
						else
						{
							This->CompleteUser(This,
											   pTaskData,
											   dataType
											   );
						}
					}
				}

				This->CompleteArm(This,
								  pTaskData,
								  dataType
								  );
			}
			else
			{
				bMoreProcessingRequired = QP_TRUE;
			}

			if (pTaskData->UserBuffer[dataType].pBufDesc)
			{
				if (pTaskData->UserBuffer[dataType].pBufDesc->ulBufferOffset == pTaskData->UserBuffer[dataType].pBufDesc->ulBufferSize)
				{
					if (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.ALL.valid)
					{
						pTaskData->UserBuffer[dataType].pBufDesc->ulFlags |= QP_BUFDESC_FLAG_INCOMPLETE_FRAME;
					}

					This->CompleteUser(This,
									   pTaskData,
									   dataType
									   );
				}
			}
		}
	}

	if (bStreaming)
	{
#if 1
		This->ProcessDataStreaming(This);
#else
		QPOSMSetEvtgrp(This->m_Thread.m_EvtWait,
					   QLCODEC_EVENT_NEW_BUFFER
					   );
#endif
	}
}



