///////////////////////////////////////////////////////////////////////////////
//        
//
// Cmpgcodec.c
//
// Description: 
//
//	implementation of the IMpegCodec interface of the codec chip
// 
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#include "qpmpgcodec.h"
#include "cchannel.h"
#include "cqlcodec.h"
#include "ctask.h"
#include "qpdbgmgr.h"
#include "util.h"


extern TQP_DbgMCompID	g_QPCodecDBGCompID;


// IMpegCodec
//
EQP_Errors CQLCodec_AllocEncodeTask(IMpegCodec *This_p, 
									PTQP_OBJECT_HANDLE phTask
									)
{
	EQP_Errors	qpStatus;
	CQLCodec	*This = GET_INHERITED_OBJECT(CQLCodec, m_iMpegCodec, This_p);

	if (QP_NULL == phTask)
	{
		return (QPERR_PARMS);
	}

	qpStatus = This->m_pTask->Alloc(This->m_pTask, 
									TASK_TYPE_ENC, 
									phTask
									);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_AllocEncodeTask() qpStatus(%d) phTask(%d)\n",
				 qpStatus,
				 *phTask
				 ));
	return (qpStatus);
}



EQP_Errors CQLCodec_AllocDecodeTask(IMpegCodec *This_p, 
									PTQP_OBJECT_HANDLE phTask
									)
{
	EQP_Errors	qpStatus;
	CQLCodec	*This = GET_INHERITED_OBJECT(CQLCodec, m_iMpegCodec, This_p);

	if (QP_NULL == phTask)
	{
		return (QPERR_PARMS);
	}

	qpStatus = This->m_pTask->Alloc(This->m_pTask, 
									TASK_TYPE_DEC, 
									phTask
									);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_AllocDecodeTask() qpStatus(%d) phTask(%d)\n",
				 qpStatus,
				 *phTask
				 ));
	return (qpStatus);
}



EQP_Errors CQLCodec_ReleaseTask(IMpegCodec *This_p, 
								TQP_OBJECT_HANDLE hTask
								)
{
	EQP_Errors	qpStatus;
	CQLCodec	*This = GET_INHERITED_OBJECT(CQLCodec, m_iMpegCodec, This_p);

	qpStatus = This->m_pTask->Release(This->m_pTask, 
									  hTask
									  );

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_ReleaseTask() qpStatus(%d) hTask(%d)\n",
				 qpStatus,
				 hTask
				 ));
	return (qpStatus);

}


EQP_Errors CQLCodec_Close(IMpegCodec *This_p, 
						  TQP_OBJECT_HANDLE hStream
						  );


EQP_Errors CQLCodec_Open(IMpegCodec *This_p, 
						 TQP_OBJECT_HANDLE hTask,
						 TQP_UINT32 dwFlags,
						 TQP_PVOID pDataFormat,
						 PTQP_OBJECT_HANDLE phStream,
						 QP_DEVICE_CALLBACK pFuncCallback,
						 TQP_PVOID context
						 )
{
	TQP_OBJECT_HANDLE	hChannel = TQP_NULL_HANDLE;
	CChannel			*pChannel = QP_NULL;
	EQP_Errors			qpStatus;
	CQLCodec			*This = GET_INHERITED_OBJECT(CQLCodec, m_iMpegCodec, This_p);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_Open() hTask(%d) dwFlags(%d)\n",
				 hTask,
				 dwFlags
				 ));

	if (QP_NULL == phStream)
	{
		return (QPERR_PARMS);
	}

	if ((hTask >= MAX_TASKS) ||
		!This->m_pTask->m_TaskData[hTask].valid
		)
	{
		return (QPERR_PARMS);
	}

	// allocate channel
	//
	switch (dwFlags)
	{
		case QPMPGCODEC_ENC_MPEG_OUT:
		{
			CMPEGOutChannel	*pMpgOutChannel;

			// create mpeg out channels
			//
			pMpgOutChannel = (CMPEGOutChannel *)QPMMMalloc2Ex(sizeof(CMPEGOutChannel));

			if (!CMPEGOutChannel_Constructor(pMpgOutChannel, 
											 &This->m_Object,
											 OBJECT_CRITICAL_LIGHT,
											 hTask,
											 This->m_pTask
											 ))
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodec_Open() Failed can not create pMpgOutChannel!!!!\n"
							 ));
				if (pMpgOutChannel)
				{
					QPMMFree2Ex((void *)pMpgOutChannel);
				}
				return (QPERR_NOMEM);
			}

			pChannel = &pMpgOutChannel->m_Channel;
			break;
		}

		case QPMPGCODEC_ENC_AES_OUT:
		{
			CAESOutChannel	*pAesOutChannel;

			// create aes out channels
			//
			pAesOutChannel = (CAESOutChannel *)QPMMMalloc2Ex(sizeof(CAESOutChannel));

			if (!CAESOutChannel_Constructor(pAesOutChannel, 
											&This->m_Object,
											OBJECT_CRITICAL_LIGHT,
											hTask,
											This->m_pTask
											))
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodec_Open() Failed can not create pAesOutChannel!!!!\n"
							 ));
				if (pAesOutChannel)
				{
					QPMMFree2Ex((void *)pAesOutChannel);
				}
				return (QPERR_NOMEM);
			}

			pChannel = &pAesOutChannel->m_Channel;
			break;
		}

		case QPMPGCODEC_ENC_VBI_OUT:
		{
			CVBIOutChannel	*pVbiOutChannel;

			// create vbi out channels
			//
			pVbiOutChannel = (CVBIOutChannel *)QPMMMalloc2Ex(sizeof(CVBIOutChannel));

			if (!CVBIOutChannel_Constructor(pVbiOutChannel, 
											&This->m_Object,
											OBJECT_CRITICAL_LIGHT,
											hTask,
											This->m_pTask
											))
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodec_Open() Failed can not create pVbiOutChannel!!!!\n"
							 ));
				if (pVbiOutChannel)
				{
					QPMMFree2Ex((void *)pVbiOutChannel);
				}
				return (QPERR_NOMEM);
			}

			pChannel = &pVbiOutChannel->m_Channel;
			break;
		}

		case QPMPGCODEC_ENC_INDEX_OUT:
		{
			CIndexOutChannel	*pIndexOutChannel;

			// create index out channels
			//
			pIndexOutChannel = (CIndexOutChannel *)QPMMMalloc2Ex(sizeof(CIndexOutChannel));

			if (!CIndexOutChannel_Constructor(pIndexOutChannel, 
											  &This->m_Object,
											  OBJECT_CRITICAL_LIGHT,
											  hTask,
											  This->m_pTask
											  ))
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodec_Open() Failed can not create pIndexOutChannel!!!!\n"
							 ));
				if (pIndexOutChannel)
				{
					QPMMFree2Ex((void *)pIndexOutChannel);
				}
				return (QPERR_NOMEM);
			}

			pChannel = &pIndexOutChannel->m_Channel;
			break;
		}

		case QPMPGCODEC_ENC_YUV_IN:
		{
			CYUVInChannel	*pYuvInChannel;

			// create yuv in channels
			//
			pYuvInChannel = (CYUVInChannel *)QPMMMalloc2Ex(sizeof(CYUVInChannel));

			if (!CYUVInChannel_Constructor(pYuvInChannel, 
										   &This->m_Object,
										   OBJECT_CRITICAL_LIGHT,
										   hTask,
										   This->m_pTask
										   ))
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodec_Open() Failed can not create pYuvInChannel!!!!\n"
							 ));
				if (pYuvInChannel)
				{
					QPMMFree2Ex((void *)pYuvInChannel);
				}
				return (QPERR_NOMEM);
			}

			pChannel = &pYuvInChannel->m_Channel;
			break;
		}

		case QPMPGCODEC_ENC_PCM_IN:
		{
			CPCMInChannel	*pPcmInChannel;

			// create pcm in channels
			//
			pPcmInChannel = (CPCMInChannel *)QPMMMalloc2Ex(sizeof(CPCMInChannel));

			if (!CPCMInChannel_Constructor(pPcmInChannel, 
										   &This->m_Object,
										   OBJECT_CRITICAL_LIGHT,
										   hTask,
										   This->m_pTask
										   ))
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodec_Open() Failed can not create pPcmInChannel!!!!\n"
							 ));
				if (pPcmInChannel)
				{
					QPMMFree2Ex((void *)pPcmInChannel);
				}
				return (QPERR_NOMEM);
			}

			pChannel = &pPcmInChannel->m_Channel;
			break;
		}

		case QPMPGCODEC_ENC_PCM_OUT:
		{
			CPCMOutChannel	*pPcmOutChannel;

			// create pcm out channels
			//
			pPcmOutChannel = (CPCMOutChannel *)QPMMMalloc2Ex(sizeof(CPCMOutChannel));

			if (!CPCMOutChannel_Constructor(pPcmOutChannel, 
											&This->m_Object,
											OBJECT_CRITICAL_LIGHT,
											hTask,
											QPMPGCODEC_ENC_PCM_OUT,
											This->m_pTask
											))
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodec_Open() Failed can not create pPcmOutChannel!!!!\n"
							 ));
				if (pPcmOutChannel)
				{
					QPMMFree2Ex((void *)pPcmOutChannel);
				}
				return (QPERR_NOMEM);
			}

			pChannel = &pPcmOutChannel->m_Channel;
			break;
		}

		case QPMPGCODEC_ENC_YUV_OUT:
		{
			CYUVOutChannel	*pYuvOutChannel;

			// create yuv out channels
			//
			pYuvOutChannel = (CYUVOutChannel *)QPMMMalloc2Ex(sizeof(CYUVOutChannel));

			if (!CYUVOutChannel_Constructor(pYuvOutChannel, 
											 &This->m_Object,
											 OBJECT_CRITICAL_LIGHT,
											 hTask,
											 QPMPGCODEC_ENC_YUV_OUT,
											 This->m_pTask
											 ))
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodec_Open() Failed can not create pYuvOutChannel!!!!\n"
							 ));
				if (pYuvOutChannel)
				{
					QPMMFree2Ex((void *)pYuvOutChannel);
				}
				return (QPERR_NOMEM);
			}

			pChannel = &pYuvOutChannel->m_Channel;
			break;
		}

		case QPMPGCODEC_ENC_ACTIVITY_OUT:
		{
			CActivityOutChannel	*pActivityOutChannel;

			// create acitivity out channels
			//
			pActivityOutChannel = (CActivityOutChannel *)QPMMMalloc2Ex(sizeof(CActivityOutChannel));

			if (!CActivityOutChannel_Constructor(pActivityOutChannel, 
											 &This->m_Object,
											 OBJECT_CRITICAL_LIGHT,
											 hTask,
											 This->m_pTask
											 ))
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodec_Open() Failed can not create pActivityOutChannel!!!!\n"
							 ));
				if (pActivityOutChannel)
				{
					QPMMFree2Ex((void *)pActivityOutChannel);
				}
				return (QPERR_NOMEM);
			}

			pChannel = &pActivityOutChannel->m_Channel;
			break;
		}

		case QPMPGCODEC_DEC_MPEG_IN:
		{
			CMPEGInChannel	*pMpgInChannel;

			// create mpeg in channel
			//
			pMpgInChannel = (CMPEGInChannel *)QPMMMalloc2Ex(sizeof(CMPEGInChannel));

			if (!CMPEGInChannel_Constructor(pMpgInChannel, 
											&This->m_Object,
											OBJECT_CRITICAL_LIGHT,
											hTask,
											This->m_pTask
											))
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodec_Open() Failed can not create pMpgInChannel!!!!\n"
							 ));
				if (pMpgInChannel)
				{
					QPMMFree2Ex((void *)pMpgInChannel);
				}
				return (QPERR_NOMEM);
			}

			pChannel = &pMpgInChannel->m_Channel;
			break;
		}

		case QPMPGCODEC_DEC_AES_IN:
		{
			CAESInChannel	*pAesInChannel;

			// create aes in channels
			//
			pAesInChannel = (CAESInChannel *)QPMMMalloc2Ex(sizeof(CAESInChannel));

			if (!CAESInChannel_Constructor(pAesInChannel, 
										   &This->m_Object,
										   OBJECT_CRITICAL_LIGHT,
										   hTask,
										   This->m_pTask
										   ))
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodec_Open() Failed can not create pAesInChannel!!!!\n"
							 ));
				if (pAesInChannel)
				{
					QPMMFree2Ex((void *)pAesInChannel);
				}
				return (QPERR_NOMEM);
			}

			pChannel = &pAesInChannel->m_Channel;
			break;
		}

		case QPMPGCODEC_DEC_YUV_OUT:
		{
			CYUVOutChannel	*pYuvDecOutChannel;

			// create mpeg out channels
			//
			pYuvDecOutChannel = (CYUVOutChannel *)QPMMMalloc2Ex(sizeof(CYUVOutChannel));

			if (!CYUVOutChannel_Constructor(pYuvDecOutChannel, 
											&This->m_Object,
											OBJECT_CRITICAL_LIGHT,
											hTask,
											QPMPGCODEC_DEC_YUV_OUT,
											This->m_pTask
											))
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodec_Open() Failed can not create pYuvDecOutChannel!!!!\n"
							 ));
				if (pYuvDecOutChannel)
				{
					QPMMFree2Ex((void *)pYuvDecOutChannel);
				}
				return (QPERR_NOMEM);
			}

			pChannel = &pYuvDecOutChannel->m_Channel;
			break;
		}

		case QPMPGCODEC_DEC_PCM_OUT:
		{
			CPCMOutChannel	*pPcmDecOutChannel;

			// create pcm out channels
			//
			pPcmDecOutChannel = (CPCMOutChannel *)QPMMMalloc2Ex(sizeof(CPCMOutChannel));

			if (!CPCMOutChannel_Constructor(pPcmDecOutChannel, 
											&This->m_Object,
											OBJECT_CRITICAL_LIGHT,
											hTask,
											QPMPGCODEC_DEC_PCM_OUT,
											This->m_pTask
											))
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodec_Open() Failed can not create pPcmDecOutChannel!!!!\n"
							 ));
				if (pPcmDecOutChannel)
				{
					QPMMFree2Ex((void *)pPcmDecOutChannel);
				}
				return (QPERR_NOMEM);
			}

			pChannel = &pPcmDecOutChannel->m_Channel;
			break;
		}

		case QPMPGCODEC_DEC_VBI_IN:
		{
			CVBIInChannel	*pVbiInChannel;

			// create vbi in channel
			//
			pVbiInChannel = (CVBIInChannel *)QPMMMalloc2Ex(sizeof(CVBIInChannel));

			if (!CVBIInChannel_Constructor(pVbiInChannel, 
										   &This->m_Object,
										   OBJECT_CRITICAL_LIGHT,
										   hTask,
										   This->m_pTask
										   ))
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodec_Open() Failed can not create pVbiInChannel!!!!\n"
							 ));
				if (pVbiInChannel)
				{
					QPMMFree2Ex((void *)pVbiInChannel);
				}
				return (QPERR_NOMEM);
			}

			pChannel = &pVbiInChannel->m_Channel;
			break;
		}

		case QPMPGCODEC_CODEC_VIRTUAL:
		{
			CNullChannel	*pNullChannel;

			// create mpeg out channels
			//
			pNullChannel = (CNullChannel *)QPMMMalloc2Ex(sizeof(CNullChannel));

			if (!CNullChannel_Constructor(pNullChannel, 
										  &This->m_Object,
										  hTask,
										  This->m_pTask
										  ))
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodec_Open() Failed can not create pNullChannel!!!!\n"
							 ));
				if (pNullChannel)
				{
					QPMMFree2Ex((void *)pNullChannel);
				}
				return (QPERR_NOMEM);
			}

			pChannel = &pNullChannel->m_Channel;
			break;
		}

		case QPMPGCODEC_DEC_INDEX_IN:
		default:
			return (QPERR_PARMS);
	}

	// add the channel to the object manager
	//
	hChannel = CObjectMgr_AddObject(This->m_pChannelMgr, 
									pChannel
									);

	// actually open the channel
	//
	qpStatus = pChannel->Open(pChannel, 
							  hChannel,
							  dwFlags,
							  pDataFormat,
							  pFuncCallback,
							  context
							  );
	if (QP_SUCCESS(qpStatus))
	{
		// return handle
		//
		*phStream = hChannel;
	}
	else
	{
		CQLCodec_Close(This_p, 
					   hChannel
					   );
	}

	return (qpStatus);
}



EQP_Errors CQLCodec_Close(IMpegCodec *This_p, 
						  TQP_OBJECT_HANDLE hStream
						  )
{
	CQLCodec	*This = GET_INHERITED_OBJECT(CQLCodec, m_iMpegCodec, This_p);
	CChannel	*pChannel = QP_NULL;
	EQP_Errors	qpStatus = QPERR_PARMS;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_Close() hStream(%d)\n",
				 hStream
				 ));

	// find and remove the channel from the channel manager
	//
	pChannel = (CChannel *)CObjectMgr_RemoveObject(This->m_pChannelMgr, 
												   hStream
												   );

	if (pChannel)
	{
		// close the channel
		//
		qpStatus = pChannel->Close(pChannel);

		// call the proper destructor and free the memory
		//
		switch (pChannel->m_dwOpenFlags)
		{
			case QPMPGCODEC_ENC_MPEG_OUT:
			{
				CMPEGOutChannel	*pMpgOutChannel = GET_INHERITED_OBJECT(CMPEGOutChannel, m_Channel, pChannel);
				CMPEGOutChannel_Destructor(pMpgOutChannel);
				QPMMFree2Ex((void *)pMpgOutChannel);
				break;
			}

			case QPMPGCODEC_ENC_AES_OUT:
			{
				CAESOutChannel	*pAesOutChannel = GET_INHERITED_OBJECT(CAESOutChannel, m_Channel, pChannel);
				CAESOutChannel_Destructor(pAesOutChannel);
				QPMMFree2Ex((void *)pAesOutChannel);
				break;
			}

			case QPMPGCODEC_ENC_VBI_OUT:
			{
				CVBIOutChannel	*pVbiOutChannel = GET_INHERITED_OBJECT(CVBIOutChannel, m_Channel, pChannel);
				CVBIOutChannel_Destructor(pVbiOutChannel);
				QPMMFree2Ex((void *)pVbiOutChannel);
				break;
			}

			case QPMPGCODEC_ENC_INDEX_OUT:
			{
				CIndexOutChannel	*pIndexOutChannel = GET_INHERITED_OBJECT(CIndexOutChannel, m_Channel, pChannel);
				CIndexOutChannel_Destructor(pIndexOutChannel);
				QPMMFree2Ex((void *)pIndexOutChannel);
				break;
			}

			case QPMPGCODEC_ENC_YUV_IN:
			{
				CYUVInChannel	*pYuvInChannel = GET_INHERITED_OBJECT(CYUVInChannel, m_Channel, pChannel);
				CYUVInChannel_Destructor(pYuvInChannel);
				QPMMFree2Ex((void *)pYuvInChannel);
				break;
			}

			case QPMPGCODEC_ENC_PCM_IN:
			{
				CPCMInChannel	*pPcmInChannel = GET_INHERITED_OBJECT(CPCMInChannel, m_Channel, pChannel);
				CPCMInChannel_Destructor(pPcmInChannel);
				QPMMFree2Ex((void *)pPcmInChannel);
				break;
			}

			case QPMPGCODEC_ENC_PCM_OUT:
			{
				CPCMOutChannel	*pPcmOutChannel = GET_INHERITED_OBJECT(CPCMOutChannel, m_Channel, pChannel);
				CPCMOutChannel_Destructor(pPcmOutChannel);
				QPMMFree2Ex((void *)pPcmOutChannel);
				break;
			}

			case QPMPGCODEC_ENC_YUV_OUT:
			{
				CYUVOutChannel	*pYuvOutChannel = GET_INHERITED_OBJECT(CYUVOutChannel, m_Channel, pChannel);
				CYUVOutChannel_Destructor(pYuvOutChannel);
				QPMMFree2Ex((void *)pYuvOutChannel);
				break;
			}

			case QPMPGCODEC_ENC_ACTIVITY_OUT:
			{
				CActivityOutChannel	*pActivityOutChannel = GET_INHERITED_OBJECT(CActivityOutChannel, m_Channel, pChannel);
				CActivityOutChannel_Destructor(pActivityOutChannel);
				QPMMFree2Ex((void *)pActivityOutChannel);
				break;
			}
						
			case QPMPGCODEC_DEC_MPEG_IN:
			{
				CMPEGInChannel	*pMpgInChannel = GET_INHERITED_OBJECT(CMPEGInChannel, m_Channel, pChannel);
				CMPEGInChannel_Destructor(pMpgInChannel);
				QPMMFree2Ex((void *)pMpgInChannel);
				break;
			}

			case QPMPGCODEC_DEC_AES_IN:
			{
				CAESInChannel	*pAesInChannel = GET_INHERITED_OBJECT(CAESInChannel, m_Channel, pChannel);
				CAESInChannel_Destructor(pAesInChannel);
				QPMMFree2Ex((void *)pAesInChannel);
				break;
			}

			case QPMPGCODEC_DEC_YUV_OUT:
			{
				CYUVOutChannel	*pYuvDecOutChannel = GET_INHERITED_OBJECT(CYUVOutChannel, m_Channel, pChannel);
				CYUVOutChannel_Destructor(pYuvDecOutChannel);
				QPMMFree2Ex((void *)pYuvDecOutChannel);
				break;
			}

			case QPMPGCODEC_DEC_PCM_OUT:
			{
				CPCMOutChannel	*pPcmDecOutChannel = GET_INHERITED_OBJECT(CPCMOutChannel, m_Channel, pChannel);
				CPCMOutChannel_Destructor(pPcmDecOutChannel);
				QPMMFree2Ex((void *)pPcmDecOutChannel);
				break;
			}

			case QPMPGCODEC_DEC_VBI_IN:
			{
				CVBIInChannel	*pVbiInChannel = GET_INHERITED_OBJECT(CVBIInChannel, m_Channel, pChannel);
				CVBIInChannel_Destructor(pVbiInChannel);
				QPMMFree2Ex((void *)pVbiInChannel);
				break;
			}

			case QPMPGCODEC_DEC_INDEX_IN:
			default:
				QPMMFree2Ex((void *)pChannel);
				break;
		}
	}

	return (qpStatus);
}



EQP_Errors CQLCodec_Start(IMpegCodec *This_p, 
						  TQP_OBJECT_HANDLE hStream
						  )
{
	CQLCodec	*This = GET_INHERITED_OBJECT(CQLCodec, m_iMpegCodec, This_p);
	CChannel	*pChannel = QP_NULL;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_Start() hStream(%d)\n",
				 hStream
				 ));

	// find the channel from the channel manager
	//
	pChannel = (CChannel *)CObjectMgr_GetObjectByHandle(This->m_pChannelMgr, 
														hStream
														);

	if (pChannel)
	{
		return (pChannel->Start(pChannel));
	}

	return (QPERR_PARMS);
}



EQP_Errors CQLCodec_Stop(IMpegCodec *This_p, 
						 TQP_OBJECT_HANDLE hStream
						 )
{
	CQLCodec	*This = GET_INHERITED_OBJECT(CQLCodec, m_iMpegCodec, This_p);
	CChannel	*pChannel = QP_NULL;
	EQP_Errors	qpStatus;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_Stop() hStream(%d)\n",
				 hStream
				 ));
	// find the channel from the channel manager
	//
	pChannel = (CChannel *)CObjectMgr_GetObjectByHandle(This->m_pChannelMgr, 
														hStream
														);

	if (pChannel)
	{
		qpStatus = pChannel->Stop(pChannel);

#ifdef _QPCODEC_CALLBACK_IN_THREAD
		{
			TQP_INT64	llTimeTo;

			QPOSMSetEvtgrp(This->m_EvtCallbackThread,
						   QLCODEC_EVENT_CALLBACK_THREAD_DATA
						   );

			// wait for the pending callback to complete
			//
			llTimeTo = 1000;

			BeginTimer(&llTimeTo);

			while (CQLCodec_IsCallbackPending(This))
			{
				if (Timeout(llTimeTo, 
							5
							))
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
								 "CQLCodec_Stop() wait for pending callback compelte timeout!!!\n" 
								 ));
					break;
				}
			}
		}
#endif //_QPCODEC_CALLBACK_IN_THREAD

		return (qpStatus);
	}

	return (QPERR_PARMS);
}



EQP_Errors CQLCodec_Acquire(IMpegCodec *This_p, 
							TQP_OBJECT_HANDLE hStream
							)
{
	CQLCodec	*This = GET_INHERITED_OBJECT(CQLCodec, m_iMpegCodec, This_p);
	CChannel	*pChannel = QP_NULL;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_Acquire() hStream(%d)\n",
				 hStream
				 ));

	// find the channel from the channel manager
	//
	pChannel = (CChannel *)CObjectMgr_GetObjectByHandle(This->m_pChannelMgr, 
														hStream
														);

	if (pChannel)
	{
		return (pChannel->Acquire(pChannel));
	}

	return (QPERR_PARMS);
}



EQP_Errors CQLCodec_Pause(IMpegCodec *This_p, 
						  TQP_OBJECT_HANDLE hStream
						  )
{
	CQLCodec	*This = GET_INHERITED_OBJECT(CQLCodec, m_iMpegCodec, This_p);
	CChannel	*pChannel = QP_NULL;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_Pause() hStream(%d)\n",
				 hStream
				 ));

	// find the channel from the channel manager
	//
	pChannel = (CChannel *)CObjectMgr_GetObjectByHandle(This->m_pChannelMgr, 
														hStream
														);

	if (pChannel)
	{
		return (pChannel->Pause(pChannel));
	}

	return (QPERR_PARMS);
}



EQP_Errors CQLCodec_Step(IMpegCodec *This_p, 
						 TQP_OBJECT_HANDLE hStream,
						 TQP_UINT32 nb
						 )
{
	CQLCodec	*This = GET_INHERITED_OBJECT(CQLCodec, m_iMpegCodec, This_p);
	CChannel	*pChannel = QP_NULL;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_Step hStream(%d) nb(%d)\n", 
				 hStream,
				 nb
				 ));

	// find the channel from the channel manager
	//
	pChannel = (CChannel *)CObjectMgr_GetObjectByHandle(This->m_pChannelMgr, 
														hStream
														);

	if (pChannel)
	{
		return (pChannel->Step(pChannel, 
							   nb
							   ));
	}

	return (QPERR_PARMS);
}



EQP_Errors CQLCodec_SetRate(IMpegCodec *This_p, 
							TQP_OBJECT_HANDLE hStream,
							TQP_INT32 lNewRate
							)
{
	CQLCodec	*This = GET_INHERITED_OBJECT(CQLCodec, m_iMpegCodec, This_p);
	CChannel	*pChannel = QP_NULL;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_SetRate() hStream(%d)\n",
				 hStream
				 ));

	// find the channel from the channel manager
	//
	pChannel = (CChannel *)CObjectMgr_GetObjectByHandle(This->m_pChannelMgr, 
														hStream
														);

	if (pChannel)
	{
		return (pChannel->SetRate(pChannel, 
								  lNewRate
								  ));
	}

	return (QPERR_PARMS);
}



EQP_Errors CQLCodec_GetRate(IMpegCodec *This_p, 
							TQP_OBJECT_HANDLE hStream,
							PTQP_INT32 plRate
							)
{
	CQLCodec	*This = GET_INHERITED_OBJECT(CQLCodec, m_iMpegCodec, This_p);
	CChannel	*pChannel = QP_NULL;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_GetRate() hStream(%d)\n",
				 hStream
				 ));

	// find the channel from the channel manager
	//
	pChannel = (CChannel *)CObjectMgr_GetObjectByHandle(This->m_pChannelMgr, 
														hStream
														);

	if (pChannel)
	{
		return (pChannel->GetRate(pChannel, 
								  plRate
								  ));
	}

	return (QPERR_PARMS);
}



EQP_Errors CQLCodec_BeginFlush(IMpegCodec *This_p, 
							   TQP_OBJECT_HANDLE hStream
							   )
{
	CQLCodec	*This = GET_INHERITED_OBJECT(CQLCodec, m_iMpegCodec, This_p);
	CChannel	*pChannel = QP_NULL;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_BeginFlush() hStream(%d)\n",
				 hStream
				 ));

	// find the channel from the channel manager
	//
	pChannel = (CChannel *)CObjectMgr_GetObjectByHandle(This->m_pChannelMgr, 
														hStream
														);

	if (pChannel)
	{
		return (pChannel->BeginFlush(pChannel));
	}

	return (QPERR_PARMS);
}



EQP_Errors CQLCodec_Flush(IMpegCodec *This_p, 
						  TQP_OBJECT_HANDLE hStream
						  )
{
	CQLCodec	*This = GET_INHERITED_OBJECT(CQLCodec, m_iMpegCodec, This_p);
	CChannel	*pChannel = QP_NULL;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_Flush() hStream(%d)\n",
				 hStream
				 ));

	// find the channel from the channel manager
	//
	pChannel = (CChannel *)CObjectMgr_GetObjectByHandle(This->m_pChannelMgr, 
														hStream
														);

	if (pChannel)
	{
		return (pChannel->Flush(pChannel));
	}

	return (QPERR_PARMS);
}



EQP_Errors CQLCodec_EndFlush(IMpegCodec *This_p, 
							 TQP_OBJECT_HANDLE hStream
							 )
{
	CQLCodec	*This = GET_INHERITED_OBJECT(CQLCodec, m_iMpegCodec, This_p);
	CChannel	*pChannel = QP_NULL;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_EndFlush() hStream(%d)\n",
				 hStream
				 ));

	// find the channel from the channel manager
	//
	pChannel = (CChannel *)CObjectMgr_GetObjectByHandle(This->m_pChannelMgr, 
														hStream
														);

	if (pChannel)
	{
		return (pChannel->EndFlush(pChannel));
	}

	return (QPERR_PARMS);
}



EQP_Errors CQLCodec_AddBuffer(IMpegCodec *This_p, 
							  TQP_OBJECT_HANDLE hStream,
							  PQP_BUFFER_DESCRIPTOR pBufDesc										 
							  )
{
	CQLCodec	*This = GET_INHERITED_OBJECT(CQLCodec, m_iMpegCodec, This_p);
	CChannel	*pChannel = QP_NULL;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_EXTRACE),
				 "CQLCodec_AddBuffer() hStream(%d)\n",
				 hStream
				 ));

	// find the channel from the channel manager
	//
	pChannel = (CChannel *)CObjectMgr_GetObjectByHandle(This->m_pChannelMgr, 
														hStream
														);

	if (pChannel)
	{
		return (pChannel->AddBuffer(pChannel, 
									pBufDesc
									));
	}

	return (QPERR_PARMS);
}



EQP_Errors CQLCodec_CancelBuffer(IMpegCodec *This_p, 
								 TQP_OBJECT_HANDLE hStream,
								 PQP_BUFFER_DESCRIPTOR pBufDesc
								 )
{
	CQLCodec	*This = GET_INHERITED_OBJECT(CQLCodec, m_iMpegCodec, This_p);
	CChannel	*pChannel = QP_NULL;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_EXTRACE),
				 "CQLCodec_CancelBuffer() hStream(%d)\n",
				 hStream
				 ));

	// find the channel from the channel manager
	//
	pChannel = (CChannel *)CObjectMgr_GetObjectByHandle(This->m_pChannelMgr, 
														hStream
														);

	if (pChannel)
	{
		return (pChannel->CancelBuffer(pChannel, 
									   pBufDesc,
									   QP_FALSE
									   ));
	}

	return (QPERR_NOTFOUND);
}



EQP_Errors CQLCodec_TimeoutBuffer(IMpegCodec *This_p, 
								  TQP_OBJECT_HANDLE hStream,
								  PQP_BUFFER_DESCRIPTOR pBufDesc
								  )
{
	CQLCodec	*This = GET_INHERITED_OBJECT(CQLCodec, m_iMpegCodec, This_p);
	CChannel	*pChannel = QP_NULL;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_EXTRACE),
				 "CQLCodec_TimeoutBuffer() hStream(%d)\n",
				 hStream
				 ));

	// find the channel from the channel manager
	//
	pChannel = (CChannel *)CObjectMgr_GetObjectByHandle(This->m_pChannelMgr, 
														hStream
														);

	if (pChannel)
	{
		return (pChannel->TimeoutBuffer(pChannel, 
										pBufDesc
										));
	}

	return (QPERR_NOTFOUND);
}



EQP_Errors CQLCodec_GetTime(IMpegCodec *This_p, 
							TQP_OBJECT_HANDLE hStream,
		  					PTQP_UINT64 pTime
		  					)
{
	return (QPERR_NOTIMPL);
}



EQP_Errors CQLCodec_XferData(IMpegCodec *This_p, 
							 TQP_OBJECT_HANDLE hStream, 
							 PQP_BUFFER_DESCRIPTOR pBufDesc
							 )
{
	CQLCodec	*This = GET_INHERITED_OBJECT(CQLCodec, m_iMpegCodec, This_p);
	CChannel	*pChannel = QP_NULL;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_EXTRACE),
				 "CQLCodec_XferData() hStream(%d)\n",
				 hStream
				 ));

	// find the channel from the channel manager
	//
	pChannel = (CChannel *)CObjectMgr_GetObjectByHandle(This->m_pChannelMgr, 
														hStream
														);

	if (pChannel)
	{
		return (pChannel->XferData(pChannel, 
								   pBufDesc
								   ));
	}

	return (QPERR_PARMS);
}





