///////////////////////////////////////////////////////////////////////////////
//        
//
// Cqlcodec.h
//
// Description: 
//
//  This is the definations of the codec chip
// 
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __CQLCODEC_H__
#define __CQLCODEC_H__


#include "qptypes.h"
#include "hci.h"
#include "fwapi.h"
#include "qpdevcntl.h"
#include "qposmgr.h"
#include "qpmpgcodec.h"
#include "cobject.h"
#include "cobjectmgr.h"
#include "cchannel.h"
#include "ctask.h"


#ifdef __cplusplus
extern "C" {
#endif


/////////////////////////////////////////////////////////////////////////////
//
//

#define QLCODEC_REG_CHIP_VERSION		0x0038


#pragma pack(push, 4)

typedef union _QLCODEC_CHIP_VERSION
{
	TQP_UINT32	Read; 
	struct
	{
		TQP_UINT32	sub_version		: 4;	// bit 0 - 3
		TQP_UINT32	version			: 4;	// bit 4 - 7
		TQP_UINT32	reserved		: 8;	// bit 8 - 15
		TQP_UINT32	product			: 4;	// bit 16 - 19
		TQP_UINT32	reserved1		: 12;	// bit 20 - 31
	} Reg;
} QLCODEC_CHIP_VERSION, *PQLCODEC_CHIP_VERSION;

#pragma pack(pop)


// product
#define QLCODEC_VER_PRODUCT_QL330_A0	0x0
#define QLCODEC_VER_PRODUCT_QL330_A1	0x0
#define QLCODEC_VER_PRODUCT_QL330_B0	0x1
#define QLCODEC_VER_PRODUCT_QL331_A0	0x2

// version
#define QLCODEC_VER_VERSION_QL330_A0	0x1
#define QLCODEC_VER_VERSION_QL330_A1	0x1
#define QLCODEC_VER_VERSION_QL330_B0	0x2
#define QLCODEC_VER_VERSION_QL331_A0	0x1

// sub version
#define QLCODEC_VER_SUBVERSION_QL330_A0	0x0
#define QLCODEC_VER_SUBVERSION_QL330_A1	0x0
#define QLCODEC_VER_SUBVERSION_QL330_B0	0x0
#define QLCODEC_VER_SUBVERSION_QL331_A0	0x0


#define QLCODEC_VER_QL330_A0			((QLCODEC_VER_PRODUCT_QL330_A0 << 16) | (QLCODEC_VER_VERSION_QL330_A0 << 4) | QLCODEC_VER_SUBVERSION_QL330_A0)
#define QLCODEC_VER_QL330_A1			((QLCODEC_VER_PRODUCT_QL330_A1 << 16) | (QLCODEC_VER_VERSION_QL330_A1 << 4) | QLCODEC_VER_SUBVERSION_QL330_A1)
#define QLCODEC_VER_QL330_B0			((QLCODEC_VER_PRODUCT_QL330_B0 << 16) | (QLCODEC_VER_VERSION_QL330_B0 << 4) | QLCODEC_VER_SUBVERSION_QL330_B0)
#define QLCODEC_VER_QL331_A0			((QLCODEC_VER_PRODUCT_QL331_A0 << 16) | (QLCODEC_VER_VERSION_QL331_A0 << 4) | QLCODEC_VER_SUBVERSION_QL331_A0)


/////////////////////////////////////////////////////////////////////////////
//
//

#ifndef __CQLCODECLIB_FWD_DEFINED__
#define __CQLCODECLIB_FWD_DEFINED__
typedef struct CQLCodecLib CQLCodecLib;
#endif //__CQLCODECLIB_FWD_DEFINED__


#ifndef __CQLCODEC_FWD_DEFINED__
#define __CQLCODEC_FWD_DEFINED__
typedef struct CQLCodec CQLCodec;
#endif //__CQLCODEC_FWD_DEFINED__

struct CQLCodec
{
	// public CObject
	//
	CObject				m_Object;

#ifdef _QPCODEC_CALLBACK_IN_THREAD

	// public CThread
	//
	CThread				m_Thread;

#endif //_QPCODEC_CALLBACK_IN_THREAD

	// public IMpegCodec
	//
	IMpegCodec			m_iMpegCodec;

	// HCI
	//
	IHCIAPI				m_hci;
	TQP_BOOL			m_bHCIInited;

	// CQLCodec
	//
	CUsbCntl			*m_pUsbCntl;
	CPCIeCntl			*m_pPCIeCntl;

	QP_DEVICE_CALLBACK	m_pDeviceCallback;				// codec client call back function
	TQP_PVOID			m_callbackContext;				// codec client call back function context

	CTask				*m_pTask;						// streaming task

	PTQP_UINT8			m_pVideoFW;
	PTQP_UINT8			m_pAudioFW;
	TQP_UINT32			m_QL201FWSize;
	TQP_UINT32			m_QL201AudFWSize;
	TQP_BOOL			m_bVideoFWUpdated;
	TQP_BOOL			m_bAudioFWUpdated;

	// encoder start parameter register map
	//
	TQP_UINT16			m_ENC_REG_MESSAGE;
	TQP_UINT16			m_ENC_REG_SYSTEM_CONTROL;
	TQP_UINT16			m_ENC_REG_PICTURE_RESOLUTION;
	TQP_UINT16			m_ENC_REG_INPUT_CONTROL;
	TQP_UINT16			m_ENC_REG_RATE_CONTROL;
	TQP_UINT16			m_ENC_REG_BIT_RATE;
	TQP_UINT16			m_ENC_REG_FILTER_CONTROL;
	TQP_UINT16			m_ENC_REG_GOP_LOOP_FILTER;
	TQP_UINT16			m_ENC_REG_ET_CONTROL;
	TQP_UINT16			m_ENC_REG_BLOCK_SIZE;
	TQP_UINT16			m_ENC_REG_OUT_PIC_RESOLUTION;
	TQP_UINT16			m_ENC_REG_AUDIO_CONTROL_PARAM;
	TQP_UINT16			m_ENC_REG_AUDIO_CONTROL_EX;

	// channel manager
	//
	CObjectMgr			*m_pChannelMgr;	

	// vou configuration
	//
	TQP_BOOL			m_VOEnable;
	TQP_UINT32			m_VOUMode;
	TQP_UINT32			m_VOUStartPixel;
	TQP_UINT32			m_VOUStartLine;

	// ao configuration
	//
	TQP_BOOL			m_AOEnable;
	TQP_UINT32			m_AOControls_ao_msb;
	TQP_UINT32			m_AOControls_lrclk_i;
	TQP_UINT32			m_AOControls_bclk_i;
	TQP_UINT32			m_AOControls_ao_i2s;
	TQP_UINT32			m_AOControls_ao_rj;
	TQP_UINT32			m_AOControls_ao_s;

	DEC_AO_CONTROL_PARAM	m_AOControls;
	DEC_AUDIO_CONTROL_PARAM	m_AudControls;


	// viu settings
	//
	TQP_UINT32			m_VIUMode;
	TQP_UINT32			m_VIUFormat;
	TQP_UINT32			m_VIUStartPixel;
	TQP_UINT32			m_VIUStartLine;
	TQP_UINT32			m_ClkEdge;
	TQP_UINT32			m_ulViuSyncCode1;
	TQP_UINT32			m_ulViuSyncCode2;

	// ai configuration
	//
	TQP_UINT32			m_AIControls_ai_msb;
	TQP_UINT32			m_AIControls_lrclk_i;
	TQP_UINT32			m_AIControls_bclk_i;
	TQP_UINT32			m_AIControls_ai_i2s;
	TQP_UINT32			m_AIControls_ai_rj;
	TQP_UINT32			m_AIControls_ai_m;

	// firmware version
	//
	TQP_UINT32			m_ulEncFirmwareVer;
	TQP_UINT32			m_ulDecFirmwareVer;
	TQP_UINT32			m_ulSysFirmwareVer;
	TQP_UINT32			m_ulAudFirmwareVer;

	// interrupts
	//
	TQP_BOOL			m_bInterruptAttached;
	TQP_UINT32			m_interruptNumber;
	TQP_OSMIntSRID		m_isrID;

	// memory settings
	//
	QPCODEC_MEM_TYPE	m_MemType;
	TQP_UINT32			m_MemSize;	// in units of Mb

	// chip type
	//
	QPCODEC_CHIP_TYPE	m_ChipType;
	QLCODEC_CHIP_VERSION	m_ChipVersion;

	// api version
	//
	TQP_UINT32			m_VerFwAPI;

	// firmware configuration
	//
	TQP_UINT32			m_FwIntMode;
	TQP_UINT32			m_FwFixedMode;

	// GPIO settings
	//
	TQP_UINT32			m_GPIODirections;
	TQP_UINT32			m_GPIOValues;

	// Mutexs and other synchronization objects
	//
	TQP_OSMSemID		m_SemaphoreFWAPI;	// mutex for FWAPI
	TQP_OSMEvtgrpID		m_EvtTask;			// Isr/Task synchronization events
	TQP_OSMEvtgrpID		m_EvtTaskReply;		// Isr/Task synchronization events
	TQP_OSMEvtgrpID		m_EvtSyncDma;		// Isr/Dma(sync) synchronization events
#ifdef _QPCODEC_CALLBACK_IN_THREAD
	TQP_OSMEvtgrpID		m_EvtCallbackThread;// events for callback thread
	TQP_OSMEvtgrpID		m_EvtCallbackThreadReply;// events for callback thread
#endif //_QPCODEC_CALLBACK_IN_THREAD

	// codec state
	//
	TQP_UINT32			m_State;

	// power state
	//
	TQP_UINT32			m_PowerState;

	// error recovery
	//
	TQP_BOOL			m_ErrorRecovery;

	// usb default mode
	//
	TQP_UINT32			m_USBDefaultMode;
	
	// pll4 and pll5
	TQP_UINT32			m_Pll4;
	TQP_UINT32			m_Pll5;
	
	TQP_UINT32			m_UseArtesaExt340;

	// ai and ao volume
	TQP_UINT32			m_AIVolume;
	TQP_UINT32			m_AOVolume;
	
	// filter out HDCP content
	TQP_UINT32			m_FilterOutHDCPContent;
	TQP_UINT32			m_HDCPState;
};


enum CQLCodec_PowerStates
{   
    QLCODEC_PWR_STATE_D0	= 0,	// on
    QLCODEC_PWR_STATE_D1	,		// stand by
    QLCODEC_PWR_STATE_D2	,		// hibernate
    QLCODEC_PWR_STATE_D3			// off
};



enum CQLCodec_TaskEvents
{   
    QLCODEC_EVENT_ARM_INT				= BIT(THREAD_EVENTS_WAIT_END + 1),
    QLCODEC_EVENT_DMA_READ_DONE			= BIT(THREAD_EVENTS_WAIT_END + 2),
    QLCODEC_EVENT_DMA_WRITE_DONE		= BIT(THREAD_EVENTS_WAIT_END + 3),
    QLCODEC_EVENT_NEW_BUFFER			= BIT(THREAD_EVENTS_WAIT_END + 4),
    QLCODEC_EVENT_FLUSH					= BIT(THREAD_EVENTS_WAIT_END + 5),
    QLCODEC_EVENT_CANCEL_BUFFER			= BIT(THREAD_EVENTS_WAIT_END + 6),
    QLCODEC_EVENT_FLUSH_ARM				= BIT(THREAD_EVENTS_WAIT_END + 7),

    QLCODEC_EVENT_MASK					= THREAD_EVENT_MASK |
										  QLCODEC_EVENT_ARM_INT | 
										  QLCODEC_EVENT_DMA_READ_DONE | 
										  QLCODEC_EVENT_DMA_WRITE_DONE |
										  QLCODEC_EVENT_NEW_BUFFER |
										  QLCODEC_EVENT_FLUSH |
										  QLCODEC_EVENT_CANCEL_BUFFER |
										  QLCODEC_EVENT_FLUSH_ARM
};



enum CQLCodec_TaskReplyEvents
{   
    QLCODEC_EVENT_FLUSH_DONE			= BIT(THREAD_EVENTS_REPLY_END + 1),
    QLCODEC_EVENT_CANCEL_BUFFER_DONE	= BIT(THREAD_EVENTS_REPLY_END + 2),
    QLCODEC_EVENT_FLUSH_ARM_DONE		= BIT(THREAD_EVENTS_REPLY_END + 3),
    QLCODEC_EVENT_STOP_DONE				= BIT(THREAD_EVENTS_REPLY_END + 4),
    QLCODEC_EVENT_DEC_GOT_PLAY_INFO		= BIT(THREAD_EVENTS_REPLY_END + 5),
    QLCODEC_EVENT_ENC_VIDEO_FROZEN		= BIT(THREAD_EVENTS_REPLY_END + 6),
    QLCODEC_EVENT_ENC_GOT_VIOSD_TABLE	= BIT(THREAD_EVENTS_REPLY_END + 7),
    QLCODEC_EVENT_ENC_GOT_CUR_VID_BUF	= BIT(THREAD_EVENTS_REPLY_END + 8),


    QLCODEC_EVENT_REPLY_MASK			= THREAD_EVENT_REPLY_MASK |
										  QLCODEC_EVENT_FLUSH_DONE |
										  QLCODEC_EVENT_CANCEL_BUFFER_DONE |
										  QLCODEC_EVENT_FLUSH_ARM_DONE |
										  QLCODEC_EVENT_STOP_DONE |
										  QLCODEC_EVENT_DEC_GOT_PLAY_INFO |
										  QLCODEC_EVENT_ENC_VIDEO_FROZEN |
										  QLCODEC_EVENT_ENC_GOT_VIOSD_TABLE |
										  QLCODEC_EVENT_ENC_GOT_CUR_VID_BUF
};



enum CQLCodec_SyncEvents
{   
    QLCODEC_EVENT_DMA_SYNC_READ_DONE	= BIT(0),
    QLCODEC_EVENT_DMA_SYNC_WRITE_DONE	= BIT(1),

    QLCODEC_EVENT_DMA_SYNC_MASK			= QLCODEC_EVENT_DMA_SYNC_READ_DONE | 
										  QLCODEC_EVENT_DMA_SYNC_WRITE_DONE
};



#ifdef _QPCODEC_CALLBACK_IN_THREAD

enum CQLCodec_CallbackThreadEvents
{   
    QLCODEC_EVENT_CALLBACK_THREAD_DATA	= BIT(THREAD_EVENTS_WAIT_END + 1),

    QLCODEC_EVENT_CALLBACK_THREAD_MASK	= THREAD_EVENT_MASK |
										  QLCODEC_EVENT_CALLBACK_THREAD_DATA
};

#endif //_QPCODEC_CALLBACK_IN_THREAD


// constructor
//
CQLCodec * CQLCodec_Constructor(CQLCodec *pQLCodec, 
								CObject *pParent,
								TQP_UINT32 dwAttributes,
								CUsbCntl *pUsbCntl,
								CPCIeCntl *pPCIeCntl
								);

// destructor
//
void CQLCodec_Destructor(CQLCodec *This);

EQP_Errors CQLCodec_UpdateMiscConfig(CQLCodec *This, 
									 TQP_OBJECT_HANDLE hTask
									 );
EQP_Errors CQLCodec_UpdateEncoderConfig(CQLCodec *This,
										TQP_OBJECT_HANDLE hTask,
										TQP_BOOL bGrabMailbox
										);
EQP_Errors CQLCodec_UpdateDecoderConfig(CQLCodec *This, 
										TQP_OBJECT_HANDLE hTask
										);

EQP_Errors CQLCodec_InitializeMemory(CQLCodec *This);
EQP_Errors CQLCodec_AOSwitch(CQLCodec *This,
							 TQP_BOOL bOn
							 );
EQP_Errors CQLCodec_VOSwitch(CQLCodec *This,
							 TQP_BOOL bOn
							 );
EQP_Errors CQLCodec_SetGPIODefaults(CQLCodec *This);
EQP_Errors CQLCodec_PowerUp(CQLCodec *This);
EQP_Errors CQLCodec_PowerDown(CQLCodec *This);


/////////////////////////////////////////////////////////////////////////////
//
//

// Firmware download
//
EQP_Errors CQLCodec_FWDownload(CQLCodec *This,
							   PTQP_UINT8 pFwBuffer,
							   TQP_UINT32 ulFWSize,
							   TQP_BOOL doReset, 
							   TQP_UINT32 ulStartAddr,
							   TQP_BOOL bCheckState,
							   TQP_BOOL bVerify
							   );
EQP_Errors CQLCodec_FWDownloadAll(CQLCodec *This,
								  TQP_BOOL bCheckState,
								  TQP_BOOL bVerify
								  );
EQP_Errors CQLCodec_FWUpdate(CQLCodec *This,
							 PTQP_UINT8 pFwBuffer,
							 TQP_UINT32 ulFWSize,
							 TQP_UINT32 ulStartAddr
							 );
EQP_Errors CQLCodec_FWSwitchMode(CQLCodec *This,
								 TQP_UINT32 ulMode,
								 TQP_BOOL bCheckState
								 );


// DMA
//
EQP_Errors CQLCodec_StartDMAWrite(CQLCodec *This, 
								  TQP_UINT32 ulAtmAddr,	// in WORD 
								  PTQP_UINT8 pHostAddr,
								  TQP_UINT32 ulLength,	// in byte
								  TQP_BOOL bSwap,
								  TQP_BOOL bSync,
								  TQP_UINT32 ulXferMode,
								  TQP_UINT32 ulPicWidth,
								  TQP_UINT32 ulPicHeight,
								  TQP_BOOL IsFrameMode
								  );
EQP_Errors CQLCodec_StartDMARead(CQLCodec *This, 
								 TQP_UINT32 ulAtmAddr,	// in WORD
								 PTQP_UINT8 pHostAddr,
								 TQP_UINT32 ulLength,	// in byte
								 TQP_BOOL bSwap,
								 TQP_BOOL bSync,
								 TQP_UINT32 ulXferMode,
								 TQP_UINT32 ulPicWidth,
								 TQP_UINT32 ulPicHeight,
								 TQP_BOOL IsFrameMode
								 );

// GPIO
//
EQP_Errors CQLCodec_SetGPIOBitDirection(CQLCodec *This, 
										PGPIO_BIT_DIRECTION pBitDirection
										);
EQP_Errors CQLCodec_GetGPIOBitDirection(CQLCodec *This, 
										PGPIO_BIT_DIRECTION pBitDirection
										);
EQP_Errors CQLCodec_SetGPIOBitValue(CQLCodec *This, 
									PGPIO_BIT_VALUE pBitValue
									);
EQP_Errors CQLCodec_GetGPIOBitValue(CQLCodec *This, 
									PGPIO_BIT_VALUE pBitValue
									);

// helper
//
TQP_BOOL CQLCodec_IsCodecIdle(CQLCodec *This);
TQP_BOOL CQLCodec_IsCodecIdleWithoutDMAPending(CQLCodec *This);
TQP_BOOL CQLCodec_IsCodecOpen(CQLCodec *This);
void CQLCodec_ClrCodecError(CQLCodec *This);
TQP_BOOL CQLCodec_IsCodecError(CQLCodec *This);
#ifdef _QPCODEC_CALLBACK_IN_THREAD
TQP_BOOL CQLCodec_IsCallbackPending(CQLCodec *This);
#endif //_QPCODEC_CALLBACK_IN_THREAD

#ifdef __cplusplus
}
#endif

#endif //__CQLCODEC_H__




