///////////////////////////////////////////////////////////////////////////////
//        
//
// CQLCodecLib.c
//
// Description: 
//
//	QLCodec library.
// 
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#include "cqlcodeclib.h"
#include "util.h"
#include "qpdbgmgr.h"
#include "qpmemmgr.h"
#include "qpgenericdevice.h"
#include "qpdevdiag.h"
#include "hci.h"
#include "fwapi.h"
#ifndef QP_LINXKER
#include <string.h>
#endif
#include "codeclib.h"
#include "usblib.h"

extern TQP_DbgMCompID	g_QPCodecDBGCompID;


/////////////////////////////////////////////////////////////////////////////
//
//

#define I_MPGCODEC(This)	(&This->m_pMpgCodec->m_iMpegCodec)
#define I_I2C(This)			(&This->m_pI2CProvider->m_ii2c)
#define I_I2CEX(This)		(&This->m_pI2CProviderEx->m_ii2c)
#define I_VIDDECODER(This)	(&This->m_pVidDecoder->m_iVideoDecoder)
#define I_VIDENCODER(This)	(&This->m_pVidEncoder->m_iVideoEncoder)
#define I_TUNER(This)		(&This->m_pTuner->m_iTuner)
#define I_TVAUDIO(This)		(&This->m_pTVAudio->m_iTvAudio)
#define I_AUDCODEC(This)	(&This->m_pAudCodec->m_iAudCodec)


/////////////////////////////////////////////////////////////////////////////
//
//

// define _QPCODEC_PURE in your platform.h to build the codec library 
// without all the peripheral support 
//

TQP_BOOL CQLCodecLib_CreateInitUSB(CQLCodecLib *This,
								   TQP_UINT32 dwPipeCmdWrite,
								   TQP_UINT32 dwPipeCmdRead,
								   TQP_UINT32 dwPipeDataWrite,
								   TQP_UINT32 dwPipeDataRead,
								   QPHCI_MODE accessMode
								   );
void CQLCodecLib_DeleteDoneUSB(CQLCodecLib *This);
TQP_BOOL CQLCodecLib_CreatePCIe(CQLCodecLib *This);
void CQLCodecLib_DeletePCIe(CQLCodecLib *This);
EQP_Errors CQLCodecLib_InitPCIe(CQLCodecLib *This);
void CQLCodecLib_DonePCIe(CQLCodecLib *This);
TQP_BOOL CQLCodecLib_CreatePeripherals(CQLCodecLib *This); 
void CQLCodecLib_DeletePeripherals(CQLCodecLib *This);
EQP_Errors CQLCodecLib_InitPeripherals(CQLCodecLib *This);
void CQLCodecLib_DonePeripherals(CQLCodecLib *This);

TQP_BOOL CQLCodecLib_CreatePeripheralsVidDecoder(CQLCodecLib *This);   // Andy 2008.10.14
void CQLCodecLib_DeletePeripheralsVidDecoder(CQLCodecLib *This);   // Andy 2008.10.14
EQP_Errors CQLCodecLib_InitPeripheralsVidDecoder(CQLCodecLib *This);   // Andy 2008.10.14
void CQLCodecLib_DonePeripheralsVidDecoder(CQLCodecLib *This);   // Andy 2008.10.14


/////////////////////////////////////////////////////////////////////////////
//
//


// CObject
//
TQP_BOOL CQLCodecLib_Init(CQLCodecLib *This)
{
	CQLCodec		*pQLCodec;
	QPI2C_TYPE		i2cType;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE), 
				 "CQLCodecLib_Init()\n"
				 ));

	ENTER_CRITICAL(&This->m_Object)

	if (QPHCI_BUS_USB == This->m_InitData.mpgCodecInitData.BusType)
	{
		PQPCODEC_USB_BUS_DATA	pBusData = (PQPCODEC_USB_BUS_DATA)This->m_InitData.mpgCodecInitData.BusData;

		if (!pBusData ||
			(sizeof(QPCODEC_USB_BUS_DATA) != This->m_InitData.mpgCodecInitData.BusDataSize)
			)
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR), 
						 "CQLCodecLib_Init() invalide USB bus data(0x%x) size(%d)\n",
						 pBusData,
						 This->m_InitData.mpgCodecInitData.BusDataSize
						 ));
			goto CQLCodecLib_Init_Err;
		}

		if (!CQLCodecLib_CreateInitUSB(This, 
									   pBusData->dwPipeCmdWrite,
									   pBusData->dwPipeCmdRead,
									   pBusData->dwPipeDataWrite,
									   pBusData->dwPipeDataRead,
									   This->m_InitData.mpgCodecInitData.AccessMode
									   ))
		{
			goto CQLCodecLib_Init_Err;
		}

		// seletc I2C type
		//
		i2cType = QPI2C_TYPE_USE_USB;
	}
	else if (QPHCI_BUS_201_EMULATION == This->m_InitData.mpgCodecInitData.BusType)
	{
		// use QL201 emulation for I2C access
		//
		i2cType = QPI2C_TYPE_USE_201_EMULATION;
	}
	else if (QPHCI_BUS_PCI == This->m_InitData.mpgCodecInitData.BusType)
	{
		EQP_Errors	qpStatus;

		if (!CQLCodecLib_CreatePCIe(This))
		{
			goto CQLCodecLib_Init_Err;
		}

		qpStatus = CQLCodecLib_InitPCIe(This);

		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CQLCodecLib_InitDevice() CQLCodecLib_InitPCIe() Failed status(%d)!!!\n",
						 qpStatus
						 ));
			goto CQLCodecLib_Init_Err;
		}

		// seletc I2C type
		//
		i2cType = QPI2C_TYPE_USE_QLCODEC;
	}
	else if (QPHCI_BUS_INTERNAL == This->m_InitData.mpgCodecInitData.BusType)
	{
		i2cType = QPI2C_TYPE_SOC_INTERNAL_HW;
	}
	else
	{
		// default use chip I2C access
		//
		i2cType = QPI2C_TYPE_USE_QLCODEC;
	}

	if ((This->m_InitData.i2cInitData.type < QPI2C_TYPE_FIRST) ||
		(This->m_InitData.i2cInitData.type > QPI2C_TYPE_LAST)
		)
	{
		This->m_InitData.i2cInitData.type = i2cType;
	}

	if ((This->m_InitData.i2cInitDataEx.type < QPI2C_TYPE_FIRST) ||
		(This->m_InitData.i2cInitDataEx.type > QPI2C_TYPE_LAST)
		)
	{
		This->m_InitData.i2cInitDataEx.type = QPI2C_TYPE_DUMMY;
	}

	// create components
	//

	// create mpeg codec chip
	//
	pQLCodec = (CQLCodec *)QPMMMalloc2Ex(sizeof(CQLCodec));

	if (!pQLCodec)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CQLCodecLib_Init() pQLCodec == NULL!!!\n"
					 ));
		goto CQLCodecLib_Init_Err;
	}

	// zero init the allocated memory
	//
	memset((void *)pQLCodec,
		   0,
		   sizeof(CQLCodec)
		   );

	This->m_pMpgCodec = CQLCodec_Constructor(pQLCodec,
											 &This->m_Object,
											 OBJECT_CRITICAL_LIGHT,
											 This->m_pUsbCntl,
											 This->m_pPCIeCntl
											 );
	if (QP_NULL == This->m_pMpgCodec)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CQLCodecLib_Init() CQLCodec_Constructor() failed!!!\n" 
					 ));
		QPMMFree2Ex((void *)pQLCodec);
		goto CQLCodecLib_Init_Err;
	}

	// create all peripherals
	//
	if (!CQLCodecLib_CreatePeripherals(This))
	{
		goto CQLCodecLib_Init_Err;
	}

	LEAVE_CRITICAL(&This->m_Object)
	return (CObject_Init(&This->m_Object));

CQLCodecLib_Init_Err:
	LEAVE_CRITICAL(&This->m_Object)
	return (QP_FALSE);
}




TQP_BOOL CQLCodecLib_Done(CQLCodecLib *This)
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodecLib_Done()\n"
				 ));

	ENTER_CRITICAL(&This->m_Object)

	// delete all the objects we created during Init()
	//
	CQLCodecLib_DeletePeripherals(This);

	if (This->m_pMpgCodec)
	{
		CQLCodec_Destructor(This->m_pMpgCodec);
		QPMMFree2Ex((void *)This->m_pMpgCodec);
		This->m_pMpgCodec = QP_NULL;
	}

	if (QPHCI_BUS_USB == This->m_InitData.mpgCodecInitData.BusType)
	{
		CQLCodecLib_DeleteDoneUSB(This);
	}
	else if (QPHCI_BUS_PCI == This->m_InitData.mpgCodecInitData.BusType)
	{
		CQLCodecLib_DonePCIe(This);
		CQLCodecLib_DeletePCIe(This);
	}

	LEAVE_CRITICAL(&This->m_Object)
	return (CObject_Done(&This->m_Object));
}



// IGenericDevice
//
static EQP_Errors CQLCodecLib_InitDevice(ICodecLib *This_p, 
										 QP_DEVICE_CALLBACK pFuncCallback,
		  								 TQP_PVOID context,
		  								 TQP_PVOID pInitData
		  								 )
{
	EQP_Errors	qpStatus;
	CQLCodecLib	*This =	GET_INHERITED_OBJECT(CQLCodecLib, m_iCodecLib, This_p);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodecLib_InitDevice() initdata(0x%x)\n",
				 pInitData
				 ));

	ENTER_CRITICAL(&This->m_Object)

	if (!CObject_IsInitialized(&This->m_Object))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR), 
					 "CQLCodecLib_InitDevice() Error Library Not Initialized\n" 
					 ));
		qpStatus = QPERR_FAIL;
		goto CQLCodecLib_InitDevice_Exit;
	}

	DelayMilliSeconds(1);


	// Clear error condition
	//
	This->m_dwDeviceError = 0;


	// save callback address and context
	//
	This->m_pDeviceCallback = pFuncCallback;
	This->m_callbackContext = context;

	DelayMilliSeconds(1);

	// initialize the codec chip
	//
	qpStatus = QPGenericDevice_InitDevice(I_MPGCODEC(This), 
										  pFuncCallback,
										  context,
										  (TQP_PVOID)&This->m_InitData.mpgCodecInitData
										  );

	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CQLCodecLib_InitDevice() m_pMpgCodec->InitDevice Failed status(0x%x)!!!\n",
					 qpStatus
					 ));
		goto CQLCodecLib_InitDevice_Exit;
	}

	// initialize peripherals
	//
	qpStatus = CQLCodecLib_InitPeripherals(This);
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CQLCodecLib_InitDevice() CQLCodecLib_InitPeripherals() Failed status(%d)!!!\n",
					 qpStatus
					 ));
		goto CQLCodecLib_InitDevice_Exit;
	}

	if (This->m_InitData.bDownloadFW)
	{
		// downloading microcode
		//
		qpStatus = CQLCodec_FWDownloadAll(This->m_pMpgCodec, 
										  QP_FALSE,
										  QP_TRUE
										  );

		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR), 
						 "CQLCodecLib_InitDevice() m_pMpgCodec->FWDownload Failed status(0x%x)!!!\n",
						 qpStatus
						 ));
////			return (qpStatus);
			qpStatus = QPERR_SUCCESS;
		}
	}
	else
	{
		if (!This->m_InitData.mpgCodecInitData.bDontInitHW)
		{
			// reset the codec chip
			//
			qpStatus = QPHCI_ResetArm(&This->m_pMpgCodec->m_hci,
									  QP_FALSE
									  );
			DelayMilliSeconds(1);

			qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_ReInit(&This->m_pMpgCodec->m_hci);
			qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : CQLCodec_InitializeMemory(This->m_pMpgCodec);

			DelayMilliSeconds(15);

			// init dummy ARM loop
			//
			qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_InitArmLoop(&This->m_pMpgCodec->m_hci);

			qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : CQLCodec_AOSwitch(This->m_pMpgCodec, 
																			This->m_pMpgCodec->m_AOEnable
																			);
			qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : CQLCodec_VOSwitch(This->m_pMpgCodec,
																			This->m_pMpgCodec->m_VOEnable
																			);
			qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : CQLCodec_SetGPIODefaults(This->m_pMpgCodec);
		 
			if ((QPHCI_BUS_INTERNAL != This->m_pMpgCodec->m_hci.m_bus_type) ||
				(QPHCI_MODE_MASTER != This->m_pMpgCodec->m_hci.m_access_mode)
				)
			{
#ifndef _QP_QPSOS
				TQP_UINT32	USBLibSharedMemory;
				TQP_UINT32	QPSOSVersionBlk[2];

				QPHCI_MemoryRead(&This->m_pMpgCodec->m_hci, 
								 0x100, 
								 &QPSOSVersionBlk[0]
								 );
				QPHCI_MemoryRead(&This->m_pMpgCodec->m_hci, 
								 0x104, 
								 &QPSOSVersionBlk[1]
								 );

				if (QPSOSGetOSMajorVersion((PTQP_UINT16)&QPSOSVersionBlk[0]) >= 3)
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
								 "CQLCodecLib_InitDevice() QPSOS3 or greater\n"
								 ));
					USBLibSharedMemory = PHY_USB_AREA;
				}
				else
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
								 "CQLCodecLib_InitDevice() QPSOS2\n"
								 ));
					USBLibSharedMemory = FLAT_USBAREA;
				}

				// init firmware configuration
				//
				QPHCI_MemoryWrite(&This->m_pMpgCodec->m_hci, 
								  CONFIG_SIGN_FIXED_MODE_PHY_ADDR, 
								  This->m_pMpgCodec->m_FwFixedMode
								  );
				QPHCI_MemoryWrite(&This->m_pMpgCodec->m_hci, 
								  CONFIG_SIGN_INT_MODE_PHY_ADDR, 
								  This->m_pMpgCodec->m_FwIntMode
								  );

				// init USB firmware configuration
				//
				QPHCI_MemoryWrite(&This->m_pMpgCodec->m_hci, 
								  USBLibSharedMemory + FIELD_OFFSET(struct usb_system_configs, useMode), 
								  QPPF_USBLIBGADGETMODE_FX2CLONE
								  );
#endif //!_QP_QPSOS
			}

			// start ARM running
			//
			qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_ResetArm(&This->m_pMpgCodec->m_hci,
																		 QP_TRUE
																		 );
		}
	}

	if (QP_SUCCESS(qpStatus))
	{
		This->m_pMpgCodec->m_PowerState = QLCODEC_PWR_STATE_D0;

		if ((QPHCI_BUS_USB == This->m_InitData.mpgCodecInitData.BusType) &&
			(QPHCI_MODE_DIRECT == This->m_InitData.mpgCodecInitData.AccessMode)
			)
		{
#ifndef _QP_QPSOS
			TQP_UINT32	USBLibSharedMemory;
			TQP_UINT32	QPSOSVersionBlk[2];

			QPHCI_MemoryRead(&This->m_pMpgCodec->m_hci, 
							 0x100, 
							 &QPSOSVersionBlk[0]
							 );
			QPHCI_MemoryRead(&This->m_pMpgCodec->m_hci, 
							 0x104, 
							 &QPSOSVersionBlk[1]
							 );

			if (QPSOSGetOSMajorVersion((PTQP_UINT16)&QPSOSVersionBlk[0]) >= 3)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodecLib_InitDevice() QPSOS3 or greater\n"
							 ));
				USBLibSharedMemory = PHY_USB_AREA;
			}
			else
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodecLib_InitDevice() QPSOS2\n"
							 ));
				USBLibSharedMemory = FLAT_USBAREA;
			}

			// read default USB mode
			//
			QPHCI_MemoryRead(&This->m_pMpgCodec->m_hci,
							 USBLibSharedMemory + FIELD_OFFSET(struct usb_system_configs, useMode_in_flash),
							 &This->m_pMpgCodec->m_USBDefaultMode
							 );
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
						 "CQLCodecLib_InitDevice() m_USBDefaultMode(%d)\n",
						 This->m_pMpgCodec->m_USBDefaultMode
						 ));
#endif //!_QP_QPSOS
		}
	}

CQLCodecLib_InitDevice_Exit:
	LEAVE_CRITICAL(&This->m_Object)
	return (qpStatus);
}




static EQP_Errors CQLCodecLib_Release(ICodecLib *This_p)
{
	CQLCodecLib	*This =	GET_INHERITED_OBJECT(CQLCodecLib, m_iCodecLib, This_p);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodecLib_Release()\n"
				 ));

	ENTER_CRITICAL(&This->m_Object)

	CQLCodecLib_DonePeripherals(This);

	if (This->m_pMpgCodec)
		QPGenericDevice_Release(I_MPGCODEC(This));

	LEAVE_CRITICAL(&This->m_Object)
	return (QPERR_SUCCESS);
}



static EQP_Errors CQLCodecLib_Reset(ICodecLib *This_p)
{
	return (QPERR_SUCCESS);
}



static EQP_Errors CQLCodecLib_Set(ICodecLib *This_p, 
								  TQP_REFGUID guid,
								  TQP_OBJECT_HANDLE hIndex,
								  TQP_UINT32 dwCode,
								  TQP_PVOID pInput,
								  TQP_PVOID pOutput,
								  TQP_UINT32 dwSize
								  )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	CQLCodecLib	*This =	GET_INHERITED_OBJECT(CQLCodecLib, m_iCodecLib, This_p);

	//peter???
	/*
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodecLib_Set()\n"
				 ));
	*/
	if (IsEqualTQPGUID((TQP_REFGUID)&PROPSETID_QPCODEC_DIAG, 
					   guid
					   ))
	{
		switch (dwCode)
		{
			case QPCODEC_DIAG_I2C_WRITE:
			{
				PQPCODEC_DIAG_I2C_STRUCT	pI2C = (PQPCODEC_DIAG_I2C_STRUCT)pInput;

				qpStatus = II2C_Write(&This->m_pI2CProvider->m_ii2c,
									  pI2C->chip_address,
									  pI2C->num_bytes,
									  pI2C->bytes
									  );
				break;
			}

			case QPCODEC_DIAG_I2C_WRITE_EX:
			{
				PQPCODEC_DIAG_I2C_STRUCT	pI2C = (PQPCODEC_DIAG_I2C_STRUCT)pInput;

				qpStatus = II2C_Write(&This->m_pI2CProviderEx->m_ii2c,
									  pI2C->chip_address,
									  pI2C->num_bytes,
									  pI2C->bytes
									  );
				break;
			}

			case QPCODEC_DIAG_MEM_WRITE:
			{
				PQPCODEC_DIAG_MEMORY_STRUCT	pMem = (PQPCODEC_DIAG_MEMORY_STRUCT)pInput;
				TQP_UINT32				i;

				if (QLCODEC_PWR_STATE_D0 != This->m_pMpgCodec->m_PowerState)
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
								 "QPCODEC_DIAG_MEM_WRITE wrong power state(%d)\n",
								 This->m_pMpgCodec->m_PowerState
								 ));
					qpStatus = QPERR_INVALID;
				}
				else
				{
					for (i = 0; i < pMem->num_dwords; i++)
					{
						qpStatus = QPHCI_MemoryWrite(&This->m_pMpgCodec->m_hci,
													 pMem->address + (i * 4), 
													 pMem->value[i]
													 );
						if (!QP_SUCCESS(qpStatus))
							break;
					}
				}

				break;
			}

			case QPCODEC_DIAG_REG_WRITE:
			{
				PQPCODEC_DIAG_REGISTER_STRUCT	pReg = (PQPCODEC_DIAG_REGISTER_STRUCT)pInput;

				qpStatus = QPHCI_RegisterWrite(&This->m_pMpgCodec->m_hci,
											   (TQP_UINT16)pReg->address, 
											   pReg->value
											   );
				break;
			}

			case QPCODEC_DIAG_HCI_REG_WRITE:
			{
				PQPCODEC_DIAG_REGISTER_STRUCT	pReg = (PQPCODEC_DIAG_REGISTER_STRUCT)pInput;

				qpStatus = QPHCI_WriteHciRegister(&This->m_pMpgCodec->m_hci,
												  (TQP_UINT8)pReg->address, 
												  (TQP_UINT8)pReg->value
												  );
				break;
			}

			case QPCODEC_DIAG_FIRMWARE_COMMAND:
			{
				PQPCODEC_DIAG_FW_COMMAND	pCmd = (PQPCODEC_DIAG_FW_COMMAND)pInput;
				ARM_MESSAGE					msg;
				TQP_UINT32					i;

				if (QLCODEC_PWR_STATE_D0 != This->m_pMpgCodec->m_PowerState)
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
								 "QPCODEC_DIAG_FIRMWARE_COMMAND wrong power state(%d)\n",
								 This->m_pMpgCodec->m_PowerState
								 ));
					qpStatus = QPERR_INVALID;
				}
				else
				{
					qpStatus = QPFWAPI_MailboxReady(This->m_pMpgCodec, 
													500
													);

					if (QP_SUCCESS(qpStatus))
					{
						for (i = 0; i < pCmd->num_param; i++)
						{
							qpStatus = QPHCI_RegisterWrite(&This->m_pMpgCodec->m_hci,
														   (TQP_UINT16)(REG_TO_ARM_1 - (i * 4)),
														   pCmd->parameters[i]
														   );
							if (!QP_SUCCESS(qpStatus))
							{
								break;
							}
						}

						if (QP_SUCCESS(qpStatus))
						{
							msg.Read = 0;
							msg.Reg.command = pCmd->command_code;

							qpStatus = QPFWAPI_SendMessageToARM(This->m_pMpgCodec,
																(TQP_UINT32) hIndex,
																&msg, 
																QP_FALSE,
																QP_NULL,
																0
																);
						}

						QPFWAPI_MailboxDone(This->m_pMpgCodec);
					}
				}
				break;
			}

			case QPCODEC_DIAG_FIRMWARE_DOWNLOAD:
			{
				PQPCODEC_DIAG_FW_DOWNLOAD	pFW = (PQPCODEC_DIAG_FW_DOWNLOAD)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodecLib_Set() QPCODEC_DIAG_FIRMWARE_DOWNLOAD size(%d) addr(0x%x) reset(%d)\n",
							 pFW->FWSize,
							 pFW->startAddress,
							 pFW->bArmReset
							 ));

				if (QLCODEC_PWR_STATE_D0 != This->m_pMpgCodec->m_PowerState)
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
								 "CQLCodecLib_Set() QPCODEC_DIAG_FIRMWARE_DOWNLOAD wrong power state(%d)\n",
								 This->m_pMpgCodec->m_PowerState
								 ));
					qpStatus = QPERR_INVALID;
				}
				else
				{
					if (CQLCodec_IsCodecIdleWithoutDMAPending(This->m_pMpgCodec))
					{
						qpStatus = CQLCodec_FWUpdate(This->m_pMpgCodec,
													 pFW->FW, 
													 pFW->FWSize, 
													 pFW->startAddress
													 );
						if (QP_SUCCESS(qpStatus))
						{
							qpStatus = CQLCodec_FWDownload(This->m_pMpgCodec,
														   pFW->FW,
														   pFW->FWSize,
														   pFW->bArmReset,
														   pFW->startAddress,
														   QP_TRUE,
														   QP_TRUE
														   );
						}
					}
					else
					{
						QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
									 "CQLCodecLib_Set() QPCODEC_DIAG_FIRMWARE_DOWNLOAD codec is opened\n"
									 ));
						qpStatus = QPERR_INVALID;
					}
				}
				break;
			}

			case QPCODEC_DIAG_FIRMWARE_DOWNLOAD_PTR:
			{
				PQPCODEC_DIAG_FW_DOWNLOAD_PTR	pFW = (PQPCODEC_DIAG_FW_DOWNLOAD_PTR)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodecLib_Set() QPCODEC_DIAG_FIRMWARE_DOWNLOAD_PTR buf(0x%x) size(%d) addr(0x%x) reset(%d)\n",
							 pFW->pFW,
							 pFW->FWSize,
							 pFW->startAddress,
							 pFW->bArmReset
							 ));

				if (QLCODEC_PWR_STATE_D0 != This->m_pMpgCodec->m_PowerState)
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
								 "CQLCodecLib_Set() QPCODEC_DIAG_FIRMWARE_DOWNLOAD_PTR wrong power state(%d)\n",
								 This->m_pMpgCodec->m_PowerState
								 ));
					qpStatus = QPERR_INVALID;
				}
				else
				{
					if (CQLCodec_IsCodecIdleWithoutDMAPending(This->m_pMpgCodec))
					{
						qpStatus = CQLCodec_FWUpdate(This->m_pMpgCodec,
													 pFW->pFW,
													 pFW->FWSize, 
													 pFW->startAddress
													 );
						if (QP_SUCCESS(qpStatus))
						{
							qpStatus = CQLCodec_FWDownload(This->m_pMpgCodec,
														   pFW->pFW,
														   pFW->FWSize,
														   pFW->bArmReset,
														   pFW->startAddress,
														   QP_TRUE,
														   QP_TRUE
														   );
						}
					}
					else
					{
						QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
									 "CQLCodecLib_Set() QPCODEC_DIAG_FIRMWARE_DOWNLOAD_PTR codec is opened\n"
									 ));
						qpStatus = QPERR_INVALID;
					}
				}
				break;
			}

			case QPCODEC_DIAG_DMA_WRITE:
			{
				PQPCODEC_DIAG_DMA_STRUCT	pDMA = (PQPCODEC_DIAG_DMA_STRUCT)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodecLib_Set() QPCODEC_DIAG_DMA_WRITE addr(0x%x) size(%d)\n",
							 pDMA->deviceAddress,
							 pDMA->size
							 ));

				if (QLCODEC_PWR_STATE_D0 != This->m_pMpgCodec->m_PowerState)
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
								 "CQLCodecLib_Set() QPCODEC_DIAG_DMA_WRITE wrong power state(%d)\n",
								 This->m_pMpgCodec->m_PowerState
								 ));
					qpStatus = QPERR_INVALID;
				}
				else
				{
					ENTER_CRITICAL(&This->m_Object)

					if (CQLCodec_IsCodecIdleWithoutDMAPending(This->m_pMpgCodec))
					{
						qpStatus = CQLCodec_StartDMAWrite(This->m_pMpgCodec,
														  pDMA->deviceAddress >> 2,
														  pDMA->data,
														  pDMA->size,
														  QP_FALSE,
														  QP_TRUE,
														  DMA_TRANSFER_MODE_LINEAR | DMA_BUFFER_MODE_COMMON,
														  0,
														  0,
														  QP_TRUE
														  );
					}
					else
					{
						qpStatus = This->m_pMpgCodec->m_pTask->DMARequest(This->m_pMpgCodec->m_pTask, 
																		  pDMA->deviceAddress >> 2,
																		  pDMA->data,
																		  pDMA->size,
																		  CHANNEL_DIR_WRITE,
																		  QP_FALSE
																		  );
					}

					LEAVE_CRITICAL(&This->m_Object)
				}

				break;
			}

			case QPCODEC_DIAG_DMA_WRITE_PTR:
			{
				PQPCODEC_DIAG_DMA_PTR_STRUCT	pDMA = (PQPCODEC_DIAG_DMA_PTR_STRUCT)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodecLib_Set() QPCODEC_DIAG_DMA_WRITE_PTR addr(0x%x) size(%d) buf(0x%x)\n",
							 pDMA->deviceAddress,
							 pDMA->size,
							 pDMA->pData
							 ));

				if (QLCODEC_PWR_STATE_D0 != This->m_pMpgCodec->m_PowerState)
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
								 "CQLCodecLib_Set() QPCODEC_DIAG_DMA_WRITE_PTR wrong power state(%d)\n",
								 This->m_pMpgCodec->m_PowerState
								 ));
					qpStatus = QPERR_INVALID;
				}
				else
				{
					ENTER_CRITICAL(&This->m_Object)

					if (CQLCodec_IsCodecIdleWithoutDMAPending(This->m_pMpgCodec))
					{
						qpStatus = CQLCodec_StartDMAWrite(This->m_pMpgCodec,
														  pDMA->deviceAddress >> 2,
														  pDMA->pData,
														  pDMA->size,
														  QP_FALSE,
														  QP_TRUE,
														  DMA_TRANSFER_MODE_LINEAR | DMA_BUFFER_MODE_USERPTR,
														  0,
														  0,
														  QP_TRUE
														  );
					}
					else
					{
						qpStatus = This->m_pMpgCodec->m_pTask->DMARequest(This->m_pMpgCodec->m_pTask, 
																		  pDMA->deviceAddress >> 2,
																		  pDMA->pData,
																		  pDMA->size,
																		  CHANNEL_DIR_WRITE,
																		  QP_FALSE
																		  );
					}

					LEAVE_CRITICAL(&This->m_Object)
				}

				break;
			}

			case QPCODEC_DIAG_HW_RESET:
			{
				PQPCODEC_DIAG_HW_RESET_STRUCT	pReset = (PQPCODEC_DIAG_HW_RESET_STRUCT)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodecLib_Set() QPCODEC_DIAG_HW_RESET run(%d)\n",
							 pReset->bArmRunning
							 ));

				if (!CQLCodec_IsCodecOpen(This->m_pMpgCodec))
				{
					qpStatus = QPHCI_ResetArm(&This->m_pMpgCodec->m_hci,
											  pReset->bArmRunning
											  );
					if (!pReset->bArmRunning)
					{
						qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_ReInit(&This->m_pMpgCodec->m_hci);
						qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : CQLCodec_InitializeMemory(This->m_pMpgCodec);
						qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : CQLCodec_AOSwitch(This->m_pMpgCodec, 
																						This->m_pMpgCodec->m_AOEnable
																						);
						qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : CQLCodec_VOSwitch(This->m_pMpgCodec,
																						This->m_pMpgCodec->m_VOEnable
																						);
						qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : CQLCodec_SetGPIODefaults(This->m_pMpgCodec);
					}
				}
				else
				{
					qpStatus = QPERR_INVALID;
				}

				break;
			}


			case QPCODEC_DIAG_DMA_WRITE_PTR_EX:
			{
				PQPCODEC_DIAG_DMA_PTR_STRUCT_EX	pDMA = (PQPCODEC_DIAG_DMA_PTR_STRUCT_EX)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodecLib_Set() QPCODEC_DIAG_DMA_WRITE_PTR_EX addr(0x%x) size(%d) buf(0x%x) swap(%d) xferMode(%d) w(%d) h(%d) isFrame(%d)\n",
							 pDMA->deviceAddress,
							 pDMA->size,
							 pDMA->pData,
							 pDMA->bSwap,
							 pDMA->ulXferMode,
							 pDMA->ulPicWidth,
							 pDMA->ulPicHeight,
							 pDMA->IsFrameMode
							 ));

				if (QLCODEC_PWR_STATE_D0 != This->m_pMpgCodec->m_PowerState)
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
								 "CQLCodecLib_Set() QPCODEC_DIAG_DMA_WRITE_PTR_EX wrong power state(%d)\n",
								 This->m_pMpgCodec->m_PowerState
								 ));
					qpStatus = QPERR_INVALID;
				}
				else
				{
					if (CQLCodec_IsCodecIdleWithoutDMAPending(This->m_pMpgCodec))
					{
						ENTER_CRITICAL(&This->m_Object)

						qpStatus = CQLCodec_StartDMAWrite(This->m_pMpgCodec,
														  pDMA->deviceAddress >> 2,
														  pDMA->pData,
														  pDMA->size,
														  pDMA->bSwap,
														  QP_TRUE,
														  pDMA->ulXferMode,
														  pDMA->ulPicWidth,
														  pDMA->ulPicHeight,
														  pDMA->IsFrameMode
														  );

						LEAVE_CRITICAL(&This->m_Object)
					}
					else
					{
						qpStatus = QPERR_INVALID;
					}
				}

				break;
			}

			case QPCODEC_DIAG_EMULATION_ON:
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodecLib_Set() QPCODEC_DIAG_EMULATION_ON\n"
							 ));
				This->m_pMpgCodec->m_hci.m_bEmulationMode = QP_TRUE;
				break;

			case QPCODEC_DIAG_EMULATION_OFF:
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodecLib_Set() QPCODEC_DIAG_EMULATION_OFF\n"
							 ));
				This->m_pMpgCodec->m_hci.m_bEmulationMode = QP_FALSE;
				break;

			case QPCODEC_VIDEO_DECODER:   // Andy 2008.10.14: select video decoder
			{
				PQPVIDDECODER_TYPE	pVidType = (PQPVIDDECODER_TYPE)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodecLib_Set() QPCODEC_VIDEO_DECODER new (0x%d) old(0x%d)\n",
							 *pVidType,
							 This->m_InitData.vidDecoderInitData.type
							 ));

				if (*pVidType != This->m_InitData.vidDecoderInitData.type)
				{
					ENTER_CRITICAL(&This->m_Object)

					CQLCodecLib_DonePeripheralsVidDecoder(This);
					CQLCodecLib_DeletePeripheralsVidDecoder(This);
					This->m_InitData.vidDecoderInitData.type = *pVidType;
					CQLCodecLib_CreatePeripheralsVidDecoder(This);
					CQLCodecLib_InitPeripheralsVidDecoder(This);

					LEAVE_CRITICAL(&This->m_Object)
				}

				break;
			}
			
			case QPCODEC_VIDEO_DECODER_ZOOM_POS:   // 2009.0213 by jkglee, set camera zoom position
			{
				PTQP_INT32	pnZoomPos = (PTQP_INT32)pInput;
				IVideoDecoder	*pVidDecoder = &This->m_pVidDecoder->m_iVideoDecoder;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodecLib_Set() QPCODEC_VIDEO_DECODER_ZOOM_POS to (0x%d)\n",
							 *pnZoomPos
							 ));
							 
				if (pVidDecoder->SetLensRoomPos) {
				    pVidDecoder->SetLensRoomPos(pVidDecoder, (TQP_INT16)*pnZoomPos);
				}

				break;
			}			
			
			case QPCODEC_VIDEO_DECODER_CHANNEL:
			{
				VIDEO_SOURCE	*pSource = (VIDEO_SOURCE *)pInput;

				This->m_InitData.vidDecoderInitData.channel = *pSource;
				
				This->m_pVidDecoder->m_iVideoDecoder.SelectVideoSource(&This->m_pVidDecoder->m_iVideoDecoder,
																	   This->m_InitData.vidDecoderInitData.channel);
				
				break;
			}			

			case QPCODEC_DIAG_FIRMWARE_SWITCH_MODE:
			{
				PTQP_UINT32	pMode = (PTQP_UINT32)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodecLib_Set() QPCODEC_DIAG_FIRMWARE_SWITCH_MODE mode(%d)\n",
							 *pMode
							 ));

				qpStatus = CQLCodec_FWSwitchMode(This->m_pMpgCodec,
												 *pMode, 
												 QP_TRUE
												 );
				break;
			}
			
			case QPCODEC_DIAG_FIRMWARE_RELOAD:
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodecLib_Set() QPCODEC_DIAG_FIRMWARE_RELOAD\n"
							 ));
							 
				CQLCodec_FWDownloadAll(This->m_pMpgCodec, QP_TRUE, QP_TRUE);

				break;
			}

			case QPCODEC_DIAG_RESET_ARM:
			{	
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodecLib_Set() QPCODEC_DIAG_RESET_ARM\n"
							 ));
				qpStatus = QPHCI_ResetArm(&This->m_pMpgCodec->m_hci, QP_FALSE);
				break;
			}
			
			default:
				break;
		}
	}
	else
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "CQLCodecLib_Set() unknown property\n" 
					 ));
		qpStatus = QPERR_NOTIMPL;
	}


	return (qpStatus);
}



static EQP_Errors CQLCodecLib_Get(ICodecLib *This_p, 
								  TQP_REFGUID guid,
								  TQP_OBJECT_HANDLE hIndex,
								  TQP_UINT32 dwCode,
								  TQP_PVOID pInput,
								  TQP_PVOID pOutput,
								  PTQP_UINT32 pdwSizeGot
								  )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	CQLCodecLib	*This =	GET_INHERITED_OBJECT(CQLCodecLib, m_iCodecLib, This_p);

	//peter??
	/*
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodecLib_Get() code(%d)\n", 
				 dwCode
				 ));
	*/
	if (IsEqualTQPGUID((TQP_REFGUID)&PROPSETID_QPCODEC_DIAG, 
					   guid
					   ))
	{
		switch (dwCode)
		{
			case QPCODEC_DIAG_I2C_READ:
			{
				PQPCODEC_DIAG_I2C_STRUCT	pI2CIn = (PQPCODEC_DIAG_I2C_STRUCT)pInput;
				PQPCODEC_DIAG_I2C_STRUCT	pI2COut = (PQPCODEC_DIAG_I2C_STRUCT)pOutput;

				//peter
				//???
				/*
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodecLib_Get() QPCODEC_DIAG_I2C_READ (0x%x), read nb(%d)\n",
							 pI2CIn->chip_address,
							 pI2CIn->num_bytes
							 ));
				*/
				qpStatus = II2C_Read(&This->m_pI2CProvider->m_ii2c,
									 pI2CIn->chip_address, 
									 pI2CIn->num_bytes, 
									 pI2COut->bytes
									 );
				pI2COut->num_bytes = pI2CIn->num_bytes;
				*pdwSizeGot = sizeof(QPCODEC_DIAG_I2C_STRUCT);
				break;
			}

			case QPCODEC_DIAG_I2C_WRITE_THEN_READ:
			{
				PQPCODEC_DIAG_I2C_WRITE_THEN_READ_STRUCT	pI2CIn = (PQPCODEC_DIAG_I2C_WRITE_THEN_READ_STRUCT)pInput;
				PQPCODEC_DIAG_I2C_WRITE_THEN_READ_STRUCT	pI2COut = (PQPCODEC_DIAG_I2C_WRITE_THEN_READ_STRUCT)pOutput;

				//peter??
				/*
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodecLib_Get() QPCODEC_DIAG_I2C_WRITE_THEN_READ(0x%x), write nb(%d) read nb(%d)\n",
							 pI2CIn->chip_address,
							 pI2CIn->num_write_bytes,
							 pI2CIn->num_read_bytes
							 ));
				*/
				qpStatus = II2C_WriteThenRead(&This->m_pI2CProvider->m_ii2c,
											  pI2CIn->chip_address,
											  pI2CIn->num_write_bytes,
											  pI2CIn->write_bytes,
											  pI2CIn->num_read_bytes,
											  pI2COut->read_bytes
											  );
				pI2COut->num_read_bytes = pI2CIn->num_read_bytes;
				*pdwSizeGot = sizeof(QPCODEC_DIAG_I2C_WRITE_THEN_READ_STRUCT);
				break;
			}

			case QPCODEC_DIAG_I2C_READ_EX:
			{
				PQPCODEC_DIAG_I2C_STRUCT	pI2CIn = (PQPCODEC_DIAG_I2C_STRUCT)pInput;
				PQPCODEC_DIAG_I2C_STRUCT	pI2COut = (PQPCODEC_DIAG_I2C_STRUCT)pOutput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodecLib_Get() QPCODEC_DIAG_I2C_READ (0x%x), read nb(%d)\n",
							 pI2CIn->chip_address,
							 pI2CIn->num_bytes
							 ));

				qpStatus = II2C_Read(&This->m_pI2CProviderEx->m_ii2c,
									 pI2CIn->chip_address,
									 pI2CIn->num_bytes,
									 pI2COut->bytes
									 );
				pI2COut->num_bytes = pI2CIn->num_bytes;
				*pdwSizeGot = sizeof(QPCODEC_DIAG_I2C_STRUCT);
				break;
			}

			case QPCODEC_DIAG_I2C_WRITE_THEN_READ_EX:
			{
				PQPCODEC_DIAG_I2C_WRITE_THEN_READ_STRUCT	pI2CIn = (PQPCODEC_DIAG_I2C_WRITE_THEN_READ_STRUCT)pInput;
				PQPCODEC_DIAG_I2C_WRITE_THEN_READ_STRUCT	pI2COut = (PQPCODEC_DIAG_I2C_WRITE_THEN_READ_STRUCT)pOutput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodecLib_Get() QPCODEC_DIAG_I2C_WRITE_THEN_READ(0x%x), write nb(%d) read nb(%d)\n",
							 pI2CIn->chip_address,
							 pI2CIn->num_write_bytes,
							 pI2CIn->num_read_bytes
							 ));

				qpStatus = II2C_WriteThenRead(&This->m_pI2CProviderEx->m_ii2c,
											  pI2CIn->chip_address,
											  pI2CIn->num_write_bytes,
											  pI2CIn->write_bytes,
											  pI2CIn->num_read_bytes,
											  pI2COut->read_bytes
											  );
				pI2COut->num_read_bytes = pI2CIn->num_read_bytes;
				*pdwSizeGot = sizeof(QPCODEC_DIAG_I2C_WRITE_THEN_READ_STRUCT);
				break;
			}

			case QPCODEC_DIAG_MEM_READ:
			{
				PQPCODEC_DIAG_MEMORY_STRUCT	pMemIn = (PQPCODEC_DIAG_MEMORY_STRUCT)pInput;
				PQPCODEC_DIAG_MEMORY_STRUCT	pMemOut = (PQPCODEC_DIAG_MEMORY_STRUCT)pOutput;
				TQP_UINT32				i;

				if (QLCODEC_PWR_STATE_D0 != This->m_pMpgCodec->m_PowerState)
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
								 "QPCODEC_DIAG_MEM_READ wrong power state(%d)\n",
								 This->m_pMpgCodec->m_PowerState
								 ));
					qpStatus = QPERR_INVALID;
				}
				else
				{
					for (i = 0; i < pMemIn->num_dwords; i++)
					{
						qpStatus = QPHCI_MemoryRead(&This->m_pMpgCodec->m_hci,
													pMemIn->address + (i * 4), 
													&pMemOut->value[i]
													);
						if (!QP_SUCCESS(qpStatus))
							break;
					}
				}
				pMemOut->num_dwords = pMemIn->num_dwords;
				pMemOut->address = pMemIn->address;
				*pdwSizeGot = sizeof(QPCODEC_DIAG_MEMORY_STRUCT);
				break;
			}

			case QPCODEC_DIAG_REG_READ:
			{
				PQPCODEC_DIAG_REGISTER_STRUCT	pRegIn = (PQPCODEC_DIAG_REGISTER_STRUCT)pInput;
				PQPCODEC_DIAG_REGISTER_STRUCT	pRegOut = (PQPCODEC_DIAG_REGISTER_STRUCT)pOutput;

				qpStatus = QPHCI_RegisterRead(&This->m_pMpgCodec->m_hci,
											  (TQP_UINT16)pRegIn->address, 
											  &pRegOut->value
											  );
				pRegOut->address = pRegIn->address;
				*pdwSizeGot = sizeof(QPCODEC_DIAG_REGISTER_STRUCT);
				break;
			}

			case QPCODEC_DIAG_REG_READ_EX:
			{
				PQPCODEC_DIAG_REGISTER_STRUCT_EX	pRegIn = (PQPCODEC_DIAG_REGISTER_STRUCT_EX)pInput;
				PQPCODEC_DIAG_REGISTER_STRUCT_EX	pRegOut = (PQPCODEC_DIAG_REGISTER_STRUCT_EX)pOutput;

				qpStatus = QPHCI_RegisterReadEx(&This->m_pMpgCodec->m_hci,
												(TQP_UINT16)pRegIn->address, 
												&pRegOut->value[0],
												pRegIn->num_reg
												);
				pRegOut->address = pRegIn->address;
				pRegOut->num_reg = pRegIn->num_reg;
				*pdwSizeGot = sizeof(QPCODEC_DIAG_REGISTER_STRUCT_EX);
				break;
			}

			case QPCODEC_DIAG_HCI_REG_READ:
			{
				PQPCODEC_DIAG_REGISTER_STRUCT	pRegIn = (PQPCODEC_DIAG_REGISTER_STRUCT)pInput;
				PQPCODEC_DIAG_REGISTER_STRUCT	pRegOut = (PQPCODEC_DIAG_REGISTER_STRUCT)pOutput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodecLib_Get() QPCODEC_DIAG_HCI_REG_READ (%d)\n",
							 pRegIn->address
							 ));

				qpStatus = QPHCI_ReadHciRegister(&This->m_pMpgCodec->m_hci,
												 (TQP_UINT8)pRegIn->address, 
												 (PTQP_UINT8)&pRegOut->value
												 );
				pRegOut->address = pRegIn->address;
				*pdwSizeGot = sizeof(QPCODEC_DIAG_REGISTER_STRUCT);
				break;
			}

			case QPCODEC_DIAG_DMA_READ:
			{
				PQPCODEC_DIAG_DMA_STRUCT	pDMA = (PQPCODEC_DIAG_DMA_STRUCT)pOutput;
				PQPCODEC_DIAG_DMA_STRUCT	pCmd = (PQPCODEC_DIAG_DMA_STRUCT)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodecLib_Get() QPCODEC_DIAG_DMA_READ addr(0x%x) size(%d)\n",
							 pCmd->deviceAddress,
							 pCmd->size
							 ));

				if (QLCODEC_PWR_STATE_D0 != This->m_pMpgCodec->m_PowerState)
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
								 "CQLCodecLib_Get() QPCODEC_DIAG_DMA_READ wrong power state(%d)\n",
								 This->m_pMpgCodec->m_PowerState
								 ));
					qpStatus = QPERR_INVALID;
				}
				else
				{
					ENTER_CRITICAL(&This->m_Object)

					if (CQLCodec_IsCodecIdleWithoutDMAPending(This->m_pMpgCodec))
					{
						qpStatus = CQLCodec_StartDMARead(This->m_pMpgCodec,
														 pCmd->deviceAddress >> 2,
														 pDMA->data,
														 pCmd->size,
														 QP_FALSE,
														 QP_TRUE,
														 DMA_TRANSFER_MODE_LINEAR | DMA_BUFFER_MODE_COMMON,
														 0,
														 0,
														 QP_TRUE
														 );
					}
					else
					{
						qpStatus = This->m_pMpgCodec->m_pTask->DMARequest(This->m_pMpgCodec->m_pTask, 
																		  pCmd->deviceAddress >> 2,
																		  pDMA->data,
																		  pCmd->size,
																		  CHANNEL_DIR_READ,
																		  QP_FALSE
																		  );
					}

					LEAVE_CRITICAL(&This->m_Object)
				}

				*pdwSizeGot = sizeof(QPCODEC_DIAG_DMA_STRUCT);
				break;
			}

			case QPCODEC_DIAG_DMA_READ_PTR:
			{
				PQPCODEC_DIAG_DMA_PTR_STRUCT	pDMA = (PQPCODEC_DIAG_DMA_PTR_STRUCT)pOutput;
				PQPCODEC_DIAG_DMA_PTR_STRUCT	pCmd = (PQPCODEC_DIAG_DMA_PTR_STRUCT)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodecLib_Get() QPCODEC_DIAG_DMA_READ_PTR addr(0x%x) size(%d) buf(0x%x)\n",
							 pCmd->deviceAddress,
							 pCmd->size,
							 pDMA->pData
							 ));

				if (QLCODEC_PWR_STATE_D0 != This->m_pMpgCodec->m_PowerState)
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
								 "CQLCodecLib_Get() QPCODEC_DIAG_DMA_READ_PTR wrong power state(%d)\n",
								 This->m_pMpgCodec->m_PowerState
								 ));
					qpStatus = QPERR_INVALID;
				}
				else
				{
					ENTER_CRITICAL(&This->m_Object)

					if (CQLCodec_IsCodecIdleWithoutDMAPending(This->m_pMpgCodec))
					{
						qpStatus = CQLCodec_StartDMARead(This->m_pMpgCodec,
														 pCmd->deviceAddress >> 2,
														 pDMA->pData,
														 pCmd->size,
														 QP_FALSE,
														 QP_TRUE,
														 DMA_TRANSFER_MODE_LINEAR | DMA_BUFFER_MODE_USERPTR,
														 0,
														 0,
														 QP_TRUE
														 );
					}
					else
					{
						qpStatus = This->m_pMpgCodec->m_pTask->DMARequest(This->m_pMpgCodec->m_pTask, 
																		  pCmd->deviceAddress >> 2,
																		  pDMA->pData,
																		  pCmd->size,
																		  CHANNEL_DIR_READ,
																		  QP_FALSE
																		  );
					}

					LEAVE_CRITICAL(&This->m_Object)
				}

				*pdwSizeGot = sizeof(QPCODEC_DIAG_DMA_PTR_STRUCT);
				break;
			}


			case QPCODEC_DIAG_DMA_READ_PTR_EX:
			{
				PQPCODEC_DIAG_DMA_PTR_STRUCT_EX	pDMA = (PQPCODEC_DIAG_DMA_PTR_STRUCT_EX)pOutput;
				PQPCODEC_DIAG_DMA_PTR_STRUCT_EX	pCmd = (PQPCODEC_DIAG_DMA_PTR_STRUCT_EX)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodecLib_Get() QPCODEC_DIAG_DMA_READ_PTR_EX addr(0x%x) size(%d) buf(0x%x) swap(%d) xferMode(%d) w(%d) h(%d) isFrame(%d)\n",
							 pCmd->deviceAddress,
							 pCmd->size,
							 pDMA->pData,
							 pCmd->bSwap,
							 pCmd->ulXferMode,
							 pCmd->ulPicWidth,
							 pCmd->ulPicHeight,
							 pCmd->IsFrameMode
							 ));

				if (QLCODEC_PWR_STATE_D0 != This->m_pMpgCodec->m_PowerState)
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
								 "CQLCodecLib_Get() QPCODEC_DIAG_DMA_READ_PTR_EX wrong power state(%d)\n",
								 This->m_pMpgCodec->m_PowerState
								 ));
					qpStatus = QPERR_INVALID;
				}
				else
				{
					if (CQLCodec_IsCodecIdleWithoutDMAPending(This->m_pMpgCodec))
					{
						ENTER_CRITICAL(&This->m_Object)

						qpStatus = CQLCodec_StartDMARead(This->m_pMpgCodec,
														 pCmd->deviceAddress >> 2,
														 pDMA->pData,
														 pCmd->size,
														 pCmd->bSwap,
														 QP_TRUE,
														 pCmd->ulXferMode,
														 pCmd->ulPicWidth,
														 pCmd->ulPicHeight,
														 pCmd->IsFrameMode
														 );

						LEAVE_CRITICAL(&This->m_Object)
					}
					else
					{
						qpStatus = QPERR_INVALID;
					}
				}

				*pdwSizeGot = sizeof(QPCODEC_DIAG_DMA_PTR_STRUCT);
				break;
			}

			case QPCODEC_VIDEO_DECODER:   // Andy 2008.10.14: get video decoder
			case QPCODEC_VIDEO_DECODER_GET:
			{
				PQPVIDDECODER_TYPE	pVidType = (PQPVIDDECODER_TYPE)pOutput;

				*pVidType = This->m_InitData.vidDecoderInitData.type;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodecLib_Get() QPCODEC_VIDEO_DECODER (0x%x)\n",
							 *pVidType
							 ));

				break;
			}
			
			case QPCODEC_VIDEO_DECODER_CHANNEL:
			case QPCODEC_VIDEO_DECODER_CHANNEL_GET:
			{
				VIDEO_SOURCE	*pSource = (VIDEO_SOURCE *)pOutput;

				*pSource = This->m_InitData.vidDecoderInitData.channel;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_Get() QPCODEC_VIDEO_DECODER_CHANNEL (%d)\n",
							 *pSource
							 ));
				break;
			}			

			case QPCODEC_DIAG_GET_USB_SPEED:
			{
				PTQP_UINT32	pSpeed = (PTQP_UINT32)pOutput;
				TQP_UINT8	speed = -1;

				if ((QPHCI_BUS_USB == This->m_InitData.mpgCodecInitData.BusType) &&
					(QPHCI_MODE_DIRECT == This->m_InitData.mpgCodecInitData.AccessMode) &&
					This->m_pUsbCntl
					)
				{
					qpStatus = CUsbCntl_GetUSBSpeed(This->m_pUsbCntl, 
													&speed
													);
				}
				else
				{
					qpStatus = QPERR_NOTSUPP;
					*pdwSizeGot = 0;
				}

				if (QP_SUCCESS(qpStatus))
				{
					*pSpeed = (TQP_UINT32)speed;
					*pdwSizeGot = sizeof(TQP_UINT32);
				}
				else
				{
					*pdwSizeGot = 0;
				}
				break;
			}

			default:
				break;
		}
	}
	else
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN), 
					 "CQLCodecLib_Get() unknown property\n" 
					 ));
		qpStatus = QPERR_NOTIMPL;
	}

	return (qpStatus);
}



// ICodecLib
//
static EQP_Errors CQLCodecLib_GetMpegCodec(ICodecLib *This_p, 
										   IMpegCodec** ppMpegCodec
										   )
{
	CQLCodecLib	*This =	GET_INHERITED_OBJECT(CQLCodecLib, m_iCodecLib, This_p);

	*ppMpegCodec = This->m_pMpgCodec ? I_MPGCODEC(This) : QP_NULL;
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodecLib_GetMpegCodec() *ppMpegCodec(0x%X)\n", 
				 *ppMpegCodec
				 ));
	return (QPERR_SUCCESS);
}



static EQP_Errors CQLCodecLib_GetVideoDecoder(ICodecLib *This_p, 
											  IVideoDecoder** ppVideoDecoder
											  )
{
	CQLCodecLib	*This =	GET_INHERITED_OBJECT(CQLCodecLib, m_iCodecLib, This_p);

	*ppVideoDecoder = This->m_pVidDecoder ? I_VIDDECODER(This) : QP_NULL;
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodecLib_GetVideoDecoder() *ppVideoDecoder(0x%X)\n", 
				 *ppVideoDecoder
				 ));
	return (*ppVideoDecoder ? QPERR_SUCCESS : QPERR_NOTSUPP);
}



static EQP_Errors CQLCodecLib_GetVideoEncoder(ICodecLib *This_p, 
											  IVideoEncoder** ppVideoEncoder
											  )
{
	CQLCodecLib	*This =	GET_INHERITED_OBJECT(CQLCodecLib, m_iCodecLib, This_p);

	*ppVideoEncoder = This->m_pVidEncoder ? I_VIDENCODER(This) : QP_NULL;
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodecLib_GetVideoEncoder() *ppVideoEncoder(0x%X)\n", 
				 *ppVideoEncoder
				 ));
	return (*ppVideoEncoder ? QPERR_SUCCESS : QPERR_NOTSUPP);
}



static EQP_Errors CQLCodecLib_GetAudioCodec(ICodecLib *This_p, 
											IAudioCodec** ppAudioCodec
											)
{
	CQLCodecLib	*This =	GET_INHERITED_OBJECT(CQLCodecLib, m_iCodecLib, This_p);

	*ppAudioCodec = This->m_pAudCodec ? I_AUDCODEC(This) : QP_NULL;
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodecLib_GetAudioCodec() *ppAudioCodec(0x%X)\n", 
				 *ppAudioCodec
				 ));
	return (*ppAudioCodec ? QPERR_SUCCESS : QPERR_NOTSUPP);
}



static EQP_Errors CQLCodecLib_GetTuner(ICodecLib *This_p, 
									   ITuner** ppTuner
									   )
{
	CQLCodecLib	*This =	GET_INHERITED_OBJECT(CQLCodecLib, m_iCodecLib, This_p);

	*ppTuner = This->m_pTuner ? I_TUNER(This) : QP_NULL;
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodecLib_GetTuner() *ppTuner(0x%X)\n", 
				 *ppTuner
				 ));
	return (*ppTuner ? QPERR_SUCCESS : QPERR_NOTSUPP);
}



static EQP_Errors CQLCodecLib_GetTVAudio(ICodecLib *This_p, 
										 ITVAudio** ppTVAudio
										 )
{
	CQLCodecLib	*This =	GET_INHERITED_OBJECT(CQLCodecLib, m_iCodecLib, This_p);

	*ppTVAudio = This->m_pTVAudio ? I_TVAUDIO(This) : QP_NULL;
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodecLib_GetMpegCodec() *ppTVAudio(0x%X)\n", 
				 *ppTVAudio
				 ));
	return (*ppTVAudio ? QPERR_SUCCESS : QPERR_NOTSUPP);
}



static EQP_Errors CQLCodecLib_Disable(ICodecLib *This_p)
{
	EQP_Errors	qpStatus;
	CQLCodecLib	*This =	GET_INHERITED_OBJECT(CQLCodecLib, m_iCodecLib, This_p);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodecLib_Disable()\n"
				 ));

	// need to program something to turn off the power
	//

	qpStatus = QPGenericDevice_Release(This_p);

	if (QP_SUCCESS(qpStatus) &&
		(QPHCI_BUS_PCI == This->m_InitData.mpgCodecInitData.BusType)
		)
	{
		CQLCodecLib_DonePCIe(This);
	}

	return (qpStatus);
}



static EQP_Errors CQLCodecLib_Enable(ICodecLib *This_p)
{
	EQP_Errors	qpStatus;
	CQLCodecLib	*This =	GET_INHERITED_OBJECT(CQLCodecLib, m_iCodecLib, This_p);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodecLib_Enable()\n" 
				 ));

	if (QPHCI_BUS_PCI == This->m_InitData.mpgCodecInitData.BusType)
	{
		qpStatus = CQLCodecLib_InitPCIe(This);
		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CQLCodecLib_Enable() CQLCodecLib_InitPCIe() Failed status(%d)!!!\n",
						 qpStatus
						 ));
			return (qpStatus);
		}
	}

	qpStatus = QPGenericDevice_InitDevice(This_p, 
										  This->m_pDeviceCallback, 
										  This->m_callbackContext, 
										  QP_NULL
										  );

	// restore to defualt USB mode if required
	//
	if (This->m_InitData.mpgCodecInitData.bRestoreUSBMode)
	{
		if (QP_SUCCESS(qpStatus) &&
			(QPHCI_BUS_USB == This->m_InitData.mpgCodecInitData.BusType) &&
			(QPPF_USBLIBGADGETMODE_UVC == This->m_pMpgCodec->m_USBDefaultMode)
			)
		{
			CQLCodec_FWSwitchMode(This->m_pMpgCodec, 
								  QXU_FIRMWARE_UVC, 
								  QP_TRUE
								  );
		}
	}

	return (qpStatus);
}




// Error
//
void CQLCodecLib_setError(CQLCodecLib *This, 
						  TQP_UINT32 dwError
						  ) 
{
	This->m_dwDeviceError |= dwError;
}



void CQLCodecLib_clrError(CQLCodecLib *This) 
{
	This->m_dwDeviceError = 0;
}



TQP_UINT32 CQLCodecLib_getError(CQLCodecLib *This) 
{
	return (This->m_dwDeviceError);
}


TQP_BOOL CQLCodecLib_isError(CQLCodecLib *This) 
{
	return (0 != This->m_dwDeviceError);
}



// constructor
//
CQLCodecLib * CQLCodecLib_Constructor(CQLCodecLib *pQLCodecLib,
									  TQP_PVOID pPDO,
									  TQP_PVOID pPDOLayered,
									  PQPCODECLIB_INITDATA pInitData
									  )
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodecLib_Constructor()\n"
				 ));

	if (pQLCodecLib)
	{
		CObject_Constructor(&pQLCodecLib->m_Object, 
							QP_NULL,
							OBJECT_CRITICAL_LIGHT
							////OBJECT_DEFAULT
		  					);

		// fill in the function table
		//

		// IGenericDevice
		//
		pQLCodecLib->m_iCodecLib.InitDevice = CQLCodecLib_InitDevice;
		pQLCodecLib->m_iCodecLib.Release = CQLCodecLib_Release;
		pQLCodecLib->m_iCodecLib.Reset = CQLCodecLib_Reset;
		pQLCodecLib->m_iCodecLib.Set = CQLCodecLib_Set;
		pQLCodecLib->m_iCodecLib.Get = CQLCodecLib_Get;

		// ICodecLib
		//
		pQLCodecLib->m_iCodecLib.GetMpegCodec = CQLCodecLib_GetMpegCodec;
		pQLCodecLib->m_iCodecLib.GetVideoDecoder = CQLCodecLib_GetVideoDecoder;
		pQLCodecLib->m_iCodecLib.GetVideoEncoder = CQLCodecLib_GetVideoEncoder;
		pQLCodecLib->m_iCodecLib.GetAudioCodec = CQLCodecLib_GetAudioCodec;
		pQLCodecLib->m_iCodecLib.GetTuner = CQLCodecLib_GetTuner;
		pQLCodecLib->m_iCodecLib.GetTVAudio = CQLCodecLib_GetTVAudio;
		pQLCodecLib->m_iCodecLib.Disable = CQLCodecLib_Disable;
		pQLCodecLib->m_iCodecLib.Enable = CQLCodecLib_Enable;

		pQLCodecLib->m_pDO = pPDO;
		pQLCodecLib->m_PDOLayered = pPDOLayered;

		// copy init data
		//
		memcpy(&pQLCodecLib->m_InitData,
			   pInitData,
			   sizeof(QPCODECLIB_INITDATA)
			   );

		pQLCodecLib->m_pDeviceCallback = QP_NULL;
		pQLCodecLib->m_callbackContext = QP_NULL;

		// USB bus interface
		//
		pQLCodecLib->m_pUsbInterface = QP_NULL;

		// Components on the board
		//
		pQLCodecLib->m_pUsbCntl = QP_NULL;
		pQLCodecLib->m_pPCIeCntl = QP_NULL;
		pQLCodecLib->m_pMpgCodec = QP_NULL;
		pQLCodecLib->m_pI2CProvider = QP_NULL;
		pQLCodecLib->m_pI2CProviderEx = QP_NULL;
		pQLCodecLib->m_pVidDecoder = QP_NULL;
		pQLCodecLib->m_pVidEncoder = QP_NULL;
		pQLCodecLib->m_pAudCodec = QP_NULL;
		pQLCodecLib->m_pTuner = QP_NULL;
		pQLCodecLib->m_pTVAudio = QP_NULL;

		// Error
		//
		pQLCodecLib->m_dwDeviceError = 0;
	}

	return (pQLCodecLib);
}



// destructor
//
void CQLCodecLib_Destructor(CQLCodecLib *This)
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodecLib_Destructor()\n"
				 ));
	// CObject
	//
	CObject_Destructor(&This->m_Object);
}





