///////////////////////////////////////////////////////////////////////////////
//        
//
// Cqlcodeclib.h
//
// Description: 
//
//  Header file for codec library.
// 
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __CQLCODECLIB_H__
#define __CQLCODECLIB_H__


#include "qptypes.h"
#include "cobject.h"
#include "qpcodeclib.h"
#include "cqlcodec.h"
#include "ci2c.h"
#include "ctuner.h"
#include "ctvaudio.h"
#include "cviddecoder.h"
#include "cvidencoder.h"
#include "caudcodec.h"

#ifdef __cplusplus
extern "C" {
#endif



#ifndef __CQLCODECLIB_FWD_DEFINED__
#define __CQLCODECLIB_FWD_DEFINED__
typedef struct CQLCodecLib CQLCodecLib;
#endif //__CQLCODECLIB_FWD_DEFINED__


struct CQLCodecLib 
{
	// public CObject
	//
	CObject					m_Object;

	// public ICodecLib
	//
	ICodecLib				m_iCodecLib;

	// CQLCodecLib 
	//
	TQP_PVOID				m_pDO;				// device PDO
	TQP_PVOID				m_PDOLayered;		// PDO
	QPCODECLIB_INITDATA		m_InitData;

	QP_DEVICE_CALLBACK		m_pDeviceCallback;	// codec client call back function
	TQP_PVOID				m_callbackContext;	// codec client call back function context

	// USB bus interface
	//
	CUsbInterface			*m_pUsbInterface;

	// PCI bus interface
	//
	CPCIeCntl				*m_pPCIeCntl;

	// Components on the board
	//
	CUsbCntl				*m_pUsbCntl;		// USB controller
	CQLCodec				*m_pMpgCodec;		// H.264 codec chip
	CI2C					*m_pI2CProvider;	// I2C provider
	CI2C					*m_pI2CProviderEx;	// I2C provider Extend     
	CVideoDecoder			*m_pVidDecoder;		// video decoder chip
	CVideoEncoder			*m_pVidEncoder;		// video encoder chip
	CAudioCodec				*m_pAudCodec;		// audio codec chip
	CTuner					*m_pTuner;			// tv tuner
	CTVAudio				*m_pTVAudio;		// tv audio

	// Error code
	//
	TQP_UINT32				m_dwDeviceError;	// Holds the internal Error Code
};


// constructor
//
CQLCodecLib * CQLCodecLib_Constructor(CQLCodecLib *pQLCodecLib,
									  TQP_PVOID pPDO,
									  TQP_PVOID pPDOLayered,
									  PQPCODECLIB_INITDATA pInitData
									  );

// destructor
//
void CQLCodecLib_Destructor(CQLCodecLib *This);

// CObject
//
TQP_BOOL CQLCodecLib_Init(CQLCodecLib *This);
TQP_BOOL CQLCodecLib_Done(CQLCodecLib *This);


// CQLCodecLib
//

// Error
//
void CQLCodecLib_setError(CQLCodecLib *This, 
						  TQP_UINT32 dwError
						  );
void CQLCodecLib_clrError(CQLCodecLib *This);
TQP_UINT32 CQLCodecLib_getError(CQLCodecLib *This);
TQP_BOOL CQLCodecLib_isError(CQLCodecLib *This);

#ifdef __cplusplus
}
#endif


#endif //__CQLCODECLIB_H__




