///////////////////////////////////////////////////////////////////////////////
//     
//
// cqueue.c
//
// Description:
//
//	implementation of the base task that running the streaming
//
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////

#include "ctask.h"

#ifdef QP_LINXKER
#include "qlautoconf.h"
#include <linux/version.h>
#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
#include <mach/pl330xp.h>
#endif
#endif

#ifndef QP_LINXKER
#include <string.h>
#endif
#include "qpdbgmgr.h"
#include "qposmgr.h"
#include "qpmemmgr.h"
#include "hci.h"
#include "cqlcodec.h"
#include "cqlcodeclib.h"
#include "util.h"
#ifdef QP_LINXKER
#include "plprofiling.h"
#include "pldevioctl.h"
#endif

/////////////////////////////////////////////////////////////////////////////
//
//

// encoder task
//
extern EQP_Errors CEncoderTask_Start(CTask *This,
 									 TQP_OBJECT_HANDLE hTask
 									 );
extern EQP_Errors CEncoderTask_Stop(CTask *This,
 									TQP_OBJECT_HANDLE hTask
 									);
extern EQP_Errors CEncoderTask_Pause(CTask *This,
 									 TQP_OBJECT_HANDLE hTask
 									 );
extern EQP_Errors CEncoderTask_Resume(CTask *This,
 									  TQP_OBJECT_HANDLE hTask
 									  );
extern void CEncoderTask_ProcessArmMessage(CTask *This,
										   PHOST_MESSAGE pHostMsg,
										   PHOST_MESSAGE_STATUS pHostMsg_status,
										   TQP_UINT32 param1,
										   TQP_UINT32 param2,
										   TQP_UINT32 param3,
										   TQP_UINT32 param4,	// QL201B
										   TQP_UINT32 param5	// QL201B
										   );
extern void CEncoderTask_ProcessIoComplete(CTask *This,
										   PTASK_DATA pTaskData,
										   TASK_DATA_TYPE dataType,
										   TQP_BOOL bStreaming
										   );


/////////////////////////////////////////////////////////////////////////////
//
//

// decoder task
//
extern EQP_Errors CDecoderTask_Start(CTask *This,
 									 TQP_OBJECT_HANDLE hTask
 									 );
extern EQP_Errors CDecoderTask_Stop(CTask *This,
 									TQP_OBJECT_HANDLE hTask
 									);
extern EQP_Errors CDecoderTask_Pause(CTask *This,
 									 TQP_OBJECT_HANDLE hTask
 									 );
extern EQP_Errors CDecoderTask_Resume(CTask *This,
 									  TQP_OBJECT_HANDLE hTask
 									  );
extern void CDecoderTask_ProcessArmMessage(CTask *This,
										   PHOST_MESSAGE pHostMsg,
										   PHOST_MESSAGE_STATUS pHostMsg_status,
										   TQP_UINT32 param1,
										   TQP_UINT32 param2,
										   TQP_UINT32 param3,
										   TQP_UINT32 param4,	// QL201B
										   TQP_UINT32 param5	// QL201B
										   );
extern void CDecoderTask_ProcessIoComplete(CTask *This,
										   PTASK_DATA pTaskData,
										   TASK_DATA_TYPE dataType,
										   TQP_BOOL bStreaming
										   );



/////////////////////////////////////////////////////////////////////////////
//
//


extern TQP_DbgMCompID	g_QPCodecDBGCompID;
extern TQP_UINT32		g_syncModeTable[19];


/////////////////////////////////////////////////////////////////////////////
//
//


static TQP_BOOL CTask_InitTasks(CTask *This)
{
	int			i;
	EQP_Errors	qpStatus;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTask_InitTasks()\n"
				 ));

	qpStatus = QPOSMCreateEvtgrp(&This->m_EvtDamReqComplete,
								 0
								 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CTask_InitTasks() QPOSMCreateEvtgrp() failed!\n"
					 ));
		return (QP_FALSE);
	}

	for (i = 0; i < MAX_TASKS; i++)
	{
		memset((void *)&This->m_TaskData[i],
			   0,
			   sizeof(TASK_DATA)
			   );

		This->m_TaskData[i].id = i;
		This->m_TaskData[i].valid = QP_FALSE;
		This->m_TaskData[i].m_State = TASK_STATE_IDLE;
		This->m_TaskData[i].m_Error = QPERR_SUCCESS;

		CObject_Constructor(&This->m_TaskData[i].m_Object,
							&This->m_Object,
							OBJECT_CRITICAL_LIGHT
							);
	}

	for (i = 0; i < MAX_CHANNEL_DIRECTION; i++)
	{
		memset((void *)&This->m_ioPending[i],
			   0,
			   sizeof(TASK_IO_PENDING)
			   );
		This->m_pPending[i] = QP_NULL;
	}

	This->m_dwMaxDMASize = (TQP_UINT32)-1;
	This->m_StartID = 0;
	This->m_taskIdPCMOut = TQP_NULL_HANDLE;
	This->m_taskIdAESOut = TQP_NULL_HANDLE;

	memset((void *)&This->m_dmaRequest,
		   0,
		   sizeof(TASK_DMA_REQUEST)
		   );
	This->m_pDmaRequest = QP_NULL;

	return (QP_TRUE);
}



static void CTask_DoneTasks(CTask *This)
{
	int	i;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTask_DoneTasks()\n"
				 ));

	for (i = 0; i < MAX_TASKS; i++)
	{
		This->Release(This,
					  i
					  );
		CObject_Destructor(&This->m_TaskData[i].m_Object);
	}

	if (This->m_EvtDamReqComplete)
	{
		QPOSMDeleteEvtgrp(This->m_EvtDamReqComplete);
		This->m_EvtDamReqComplete = QP_NULL;
	}
}



static void CTask_ResetState(CTask *This,
							 PTASK_DATA pTaskData
							 )
{
	int	i;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTask_ResetState() (%d)\n",
				 pTaskData->id
				 ));

	for (i = 0; i < MAX_TASK_DATA_TYPE; i++)
	{
		if (QP_NULL != pTaskData->pArmMsgFifo[i])
		{
			// flush all the arm requests
			//
			do {
				This->CompleteArm(This,
								  pTaskData,
								  (TASK_DATA_TYPE)i
								  );
			} while (CFifo_GetFifo(pTaskData->pArmMsgFifo[i],
								   &pTaskData->ArmRequest[i]
								   ));
		}

		memset((void *)&pTaskData->UserBuffer[i],
			   0,
			   sizeof(TASK_USER_BUFFER)
			   );
		memset((void *)&pTaskData->ArmRequest[i],
			   0,
			   sizeof(TASK_ARM_REQUEST)
			   );
		pTaskData->ArmRequestNumber[i] = 0;
#ifdef QP_DEBUG
		pTaskData->ArmBufferAddr = 0;
#endif //QP_DEBUG
		pTaskData->pBufDescToCancel[i] = QP_NULL;
		pTaskData->bFlushing[i] = QP_FALSE;
	}

	pTaskData->m_dwStarted = 0;
	pTaskData->m_dwPaused = 0;
	pTaskData->m_bAcquired = QP_FALSE;
	pTaskData->m_State = TASK_STATE_IDLE;
	pTaskData->m_Error = QPERR_SUCCESS;
	pTaskData->m_StartID = TASK_DATA_TYPE_COMP_VID;
	pTaskData->bDone = QP_FALSE;
}



static void CTask_LoadDefaultSettings(CTask *This,
									  PTASK_DATA pTaskData
									  )
{
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_pMpegCodec->m_ChipType)
	{
		// encoder settings
		pTaskData->m_systemControl.Read = 0;
		pTaskData->m_systemControl.RegArtesa.stream_type = ARTESA_STREAM_TYPE_ELEMENTARY;
		pTaskData->m_systemControl.RegArtesa.stream_data = ARTESA_STREAM_DATA_VIDEO;
		pTaskData->m_systemControl.RegArtesa.profile = PROFILE_EXTENDED;
		pTaskData->m_systemControl.RegArtesa.level = LEVEL_4;
#ifndef CONFIG_HOST_PLATFORM_STR8133_REF
		pTaskData->m_systemControl.RegArtesa.spsr_freq = 0;
#else
		pTaskData->m_systemControl.RegArtesa.spsr_freq = 1;
#endif //!CONFIG_HOST_PLATFORM_STR8133_REF
		pTaskData->m_systemControl.RegArtesa.ff_mod = FF_MODE_FRAME_ONLY;
		pTaskData->m_systemControl.RegArtesa.v_mode = V_MODE_CAVLC;
		pTaskData->m_systemControl.RegArtesa.cabac_init_idc = 0;
		pTaskData->m_systemControl.RegArtesa.version = 1;
		if (1 == This->m_pMpegCodec->m_VerFwAPI)
		{
			pTaskData->m_systemControl.RegArtesa.xfer_mode = XFER_MODE_FRAME;
		}

		pTaskData->m_pictureResolution.Read = 0;
		pTaskData->m_pictureResolution.RegArtesa.in_pic_width = 1280;
		pTaskData->m_pictureResolution.RegArtesa.in_pic_height = 720;

		pTaskData->m_syncMode = SYNC_MODE_1280x720p;

		pTaskData->m_inputControl.Read = 0;
		pTaskData->m_inputControl.RegArtesa.scan_format = SCAN_FORMAT_PROG;
		pTaskData->m_inputControl.RegArtesa.src_mode = SRC_MODE_HIU;
		pTaskData->m_inputControl.RegArtesa.sync_mode = g_syncModeTable[pTaskData->m_syncMode];
		pTaskData->m_inputControl.RegArtesa.data_type = DATA_TYPE_RAW;
		pTaskData->m_inputControl.RegArtesa.in_frame_rate = IN_FRAME_RATE_ABSOLUTE | 30;
		pTaskData->m_inputControl.RegArtesa.out_frame_rate = 30;
		pTaskData->m_inputControl.RegArtesa.force_60p_in = 0;
		pTaskData->m_inputControl.RegArtesa.pd_en = 0;
		pTaskData->m_inputControl.RegArtesa.ck_edge = This->m_pMpegCodec->m_ClkEdge;

		pTaskData->m_rateControl.Read = 0;
		pTaskData->m_rateControl.RegArtesa.avg_bit_rate = 4000;
		pTaskData->m_rateControl.RegArtesa.qp_update_freq = 80;
		pTaskData->m_rateControl.RegArtesa.mode = RATE_CONTROL_MODE_HIU;
		pTaskData->m_rateControl.RegArtesa.fixed = RATE_CONTROL_NORMAL;
		pTaskData->m_rateControl.RegArtesa.vbr = 1;
		pTaskData->m_rateControl.RegArtesa.intra_refresh_mode = RATE_CONTROL_INTRA_REFRESH_MODE_OFF;

		pTaskData->m_rateControlEx.Artesa.base_qp_length = 80;
		pTaskData->m_rateControlEx.Artesa.local_activity_on = 0;
		pTaskData->m_rateControlEx.Artesa.motion_detect_interval = 10;

		pTaskData->m_bitRate.Read = 0;
		pTaskData->m_bitRate.Reg.min_bit_rate = 2000;
		pTaskData->m_bitRate.Reg.max_bit_rate = 8000;

		pTaskData->m_filterControl.Read = 0;
		pTaskData->m_filterControl.Reg.start_pixel = This->m_pMpegCodec->m_VIUStartPixel;
		pTaskData->m_filterControl.Reg.start_line = This->m_pMpegCodec->m_VIUStartLine;
		pTaskData->m_filterControl.Reg.fl_en = 0;
		pTaskData->m_filterControl.Reg.filter_strength = 2;

		pTaskData->m_gopLoopFilter.Read = 0;
		pTaskData->m_gopLoopFilter.RegArtesa.intra_period = 30;
		pTaskData->m_gopLoopFilter.RegArtesa.lp_filter = 1;
		pTaskData->m_gopLoopFilter.RegArtesa.dis_lf_idc = 0;
		pTaskData->m_gopLoopFilter.RegArtesa.alpha_offset = 0;
		pTaskData->m_gopLoopFilter.RegArtesa.beta_offset = 0;
		pTaskData->m_gopLoopFilter.RegArtesa.B_num = 1;
		pTaskData->m_gopLoopFilter.RegArtesa.ref_n = GOP_LOOP_ALL_REF;
		pTaskData->m_gopLoopFilter.RegArtesa.t8x8 = 1;
		pTaskData->m_gopLoopFilter.RegArtesa.ref_p = GOP_LOOP_PRE_SELECT_ONE_REF_ON;
		pTaskData->m_gopLoopFilter.RegArtesa.IDR = IDR_OPEN_GOP;

		pTaskData->m_blkXferSize.Read = 0;
		pTaskData->m_blkXferSize.RegArtesa.block_size = 16;

		pTaskData->m_outPictureResolution.Read = 0;
		pTaskData->m_outPictureResolution.RegArtesa.out_pic_width = 1280;
		pTaskData->m_outPictureResolution.RegArtesa.out_pic_height = 720;

		pTaskData->m_audioControlParam.Read = 0;
		pTaskData->m_audioControlParam.Reg.bit_rate = 128;	// in 1000 bps unit
		pTaskData->m_audioControlParam.Reg.ch_sel = CH_SEL_MUX;
		pTaskData->m_audioControlParam.Reg.sample_rate = SAMPLE_RATE_48K;
		pTaskData->m_audioControlParam.Reg.ai_msb = This->m_pMpegCodec->m_AIControls_ai_msb;
		pTaskData->m_audioControlParam.Reg.lrclk_i = This->m_pMpegCodec->m_AIControls_lrclk_i;
		pTaskData->m_audioControlParam.Reg.bclk_i = This->m_pMpegCodec->m_AIControls_bclk_i;	// 0
		pTaskData->m_audioControlParam.Reg.ai_i2s = This->m_pMpegCodec->m_AIControls_ai_i2s;
		pTaskData->m_audioControlParam.Reg.ai_rj = This->m_pMpegCodec->m_AIControls_ai_rj;
		pTaskData->m_audioControlParam.Reg.ai_m = This->m_pMpegCodec->m_AIControls_ai_m;
		pTaskData->m_audioControlParam.Reg.ch_num = 2;
		pTaskData->m_audioControlParam.Reg.pts = 0;
		pTaskData->m_audioControlParam.Reg.disAi = 0;
		pTaskData->m_audioControlParam.Reg.raw = 0;
		pTaskData->m_audioControlParam.Reg.audio_type = AUDIO_TYPE_AAC;

		pTaskData->m_audioControlExAAC.Read = 0;
		pTaskData->m_audioControlExAAC.Reg.aac_ver = AAC_VER_MPEG_2;
		pTaskData->m_audioControlExAAC.Reg.h_type = H_TYPE_ADTS;
		pTaskData->m_audioControlExAAC.Reg.TNS = 1;
		pTaskData->m_audioControlExAAC.Reg.IS = 1;
		pTaskData->m_audioControlExAAC.Reg.PNS = 1;
		pTaskData->m_audioControlExAAC.Reg.MS = 1;
		pTaskData->m_audioControlExAAC.Reg.cutoff_freq = 18000;	// 18k
		pTaskData->m_audioControlExAAC.Reg.downsample = 0;
		pTaskData->m_audioControlExAAC.Reg.lr_swap = 0;

		pTaskData->m_audioControlExG711.Read = 0;
		pTaskData->m_audioControlExG711.Reg.mode = MODE_A_LAW;
		pTaskData->m_audioControlExG711.Reg.downsample = 0;
		pTaskData->m_audioControlExG711.Reg.lr_swap = 0;
#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
		pTaskData->m_audioControlExG711.Reg.block_size = BLOCK_SIZE_160;
#else
		pTaskData->m_audioControlExG711.Reg.block_size = BLOCK_SIZE_80;
#endif

		pTaskData->m_audioControlExLPCM.Read = 0;
		pTaskData->m_audioControlExLPCM.Reg.sample_cnt = 512;
		pTaskData->m_audioControlExLPCM.Reg.downsample = 0;
		pTaskData->m_audioControlExLPCM.Reg.lr_swap = 0;

		pTaskData->m_audioControlExSILK.Read = 0;
		pTaskData->m_audioControlExSILK.Reg.frame_len = FRAME_SIZE_60;
		pTaskData->m_audioControlExSILK.Reg.downsample = 0;
		pTaskData->m_audioControlExSILK.Reg.lr_swap = 0;

		pTaskData->m_EncIndexCapFreq = 32;
		pTaskData->m_MP4VideoBlockNumber = 5;

		pTaskData->m_audioEnhancement.gain1 = 0x2666;	// 0.3
		pTaskData->m_audioEnhancement.gain2 = 0x0;		// 0
		pTaskData->m_audioEnhancement.add = 0x2666;		// 0.3
		pTaskData->m_audioEnhancement.sub = 0x7FFF;		// 1.0
		pTaskData->m_audioEnhancement.att1 = 0x7FFF;		// 1.0
		pTaskData->m_audioEnhancement.att2 = 0x0000;		// 0
		pTaskData->m_audioEnhancement.lgain = 0x0;
		pTaskData->m_audioEnhancement.rgain = 0x0;

		pTaskData->m_EncStopMode = QP_ENC_STOP_IMMEDIATELY;

		pTaskData->m_bEncEnableVidPadding = QP_TRUE;
		pTaskData->m_bEncVidFrozen = QP_FALSE;
		pTaskData->m_bEncVidStillInput = QP_FALSE;

		pTaskData->m_EncCapMode.capMode = QPCODEC_ENC_CAP_MODE_CONTINUOUS;
		pTaskData->m_EncCapMode.trigMode = 0;
		pTaskData->m_EncCapMode.gpioPin = 0;

		pTaskData->m_EncMjpegQuality = 0;		// firmware internal
		pTaskData->m_EncMjpegFrameBuffer = 0;	// firmware internal

		pTaskData->m_ExternalTriggerToSync.Enable = 0;
		pTaskData->m_ExternalTriggerToSync.gpioPin = 0;
		pTaskData->m_PTSCounterReset.Enable = 0;
		pTaskData->m_PTSCounterReset.gpioPin = 0;
		pTaskData->m_PTSCounterReset.immediate = 0;

		pTaskData->m_RawVideoDecimationFactor.input_format = QPCODEC_DECIMATION_INPUT_QYUV;
		pTaskData->m_RawVideoDecimationFactor.output_format = QPCODEC_DECIMATION_OUTPUT_YUY2;
		pTaskData->m_RawVideoDecimationFactor.scale_factor.factor.h_numerator = 1;
		pTaskData->m_RawVideoDecimationFactor.scale_factor.factor.h_denominator = 4;
		pTaskData->m_RawVideoDecimationFactor.scale_factor.factor.v_numerator = 1;
		pTaskData->m_RawVideoDecimationFactor.scale_factor.factor.v_denominator = 4;

		pTaskData->m_DeinterlaceMode = QPCODEC_DEINTERLACE_TOP_FIELD_MODE;

		pTaskData->m_LargeCompressBufferControl = 12000 /*| QPCODEC_LARGE_COMPRESS_BUFFER_CONTROL_ENABLE*/;

		pTaskData->m_EnableLowBitrateMode = 0;

		pTaskData->m_EnableAVDiscardControl = 0;
		pTaskData->m_UseSWPTS = 0;

		pTaskData->m_encFontTableAddr = (TQP_UINT32)-1;
		pTaskData->m_encTextListAddr = (TQP_UINT32)-1;

		pTaskData->m_encVidBufLumaAddr = (TQP_UINT32)-1;
		pTaskData->m_encVidBufChromaAddr = (TQP_UINT32)-1;
		pTaskData->m_encFrameLABufAddr = (TQP_UINT32)-1;
		pTaskData->m_encTopLABufAddr = (TQP_UINT32)-1;
		pTaskData->m_encBottomLABufAddr = (TQP_UINT32)-1;
		pTaskData->m_encVidBufPTS = 0;

		// decoder settings
		if (1 == This->m_pMpegCodec->m_VerFwAPI)
		{
			pTaskData->m_streamID.Read = 0;
			pTaskData->m_streamID.RegEx.stream_type = ARTESA_STREAM_TYPE_ELEMENTARY;
			pTaskData->m_streamID.RegEx.stream_data = ARTESA_STREAM_DATA_VIDEO;
			pTaskData->m_streamID.RegEx.xfer_mode = XFER_MODE_BLOCK;
		}
		else
		{
			pTaskData->m_streamID.Reg.stream_type = STREAM_TYPE_VID_STREAM;
		}
		pTaskData->m_outputMode = QPDEC_DEC_START_HIU |
								  QPDEC_DEC_START_VOU_AUTO
								  ;
		pTaskData->m_decodeResolution.Read = 0;
		pTaskData->m_decodeResolution.Reg.pic_width = 1280;
		pTaskData->m_decodeResolution.Reg.pic_height = 720;

		pTaskData->m_decStopMode = QPDEC_DEC_STOP_IMMEDIATELY;
  		pTaskData->m_decStopDispMode = QPDEC_DISP_LAST_FRAME;

  		pTaskData->m_decPauseDispMode = QPDEC_DISP_LAST_FRAME;
		pTaskData->m_decTrickPlaySpeed = QPDEC_TRICK_SPD_NORMAL;
		pTaskData->m_decTrickPlayDirection = QPDEC_TRICK_DIR_FORWARD;
		pTaskData->m_decTrickFieldPreference = QPDEC_TRICK_DISP_DEFAULT;
  		pTaskData->m_decDispMode = QPDEC_DISP_MODE_PROGRESSIVE_FRAME;
  		pTaskData->m_decDispTVSystem = QPDEC_DISP_ATTRIB_720p;
  		pTaskData->m_decDispForceRemote = 0;
  		pTaskData->m_decDispInputSize.Read = 0;
  		pTaskData->m_decDispInputSize.Attrib.x = 1280;
  		pTaskData->m_decDispInputSize.Attrib.y = 720;
  		pTaskData->m_decDispOutputSize.Read = 0;
  		pTaskData->m_decDispOutputSize.Attrib.x = 1280;
  		pTaskData->m_decDispOutputSize.Attrib.y = 720;
  		pTaskData->m_decDispInputOrigin.Read = 0;
  		pTaskData->m_decDispInputOrigin.Attrib.x = 0;
  		pTaskData->m_decDispInputOrigin.Attrib.y = 0;
  		pTaskData->m_decDispOutputOrigin.Read = 0;
  		pTaskData->m_decDispOutputOrigin.Attrib.x = 0;
  		pTaskData->m_decDispOutputOrigin.Attrib.y = 0;
		pTaskData->m_decXferMode = QPDEC_DEC_XFER_MODE_PARTIAL;
		pTaskData->m_decXferMinimumSize = 0;


		// decoder playback information
		//
		pTaskData->m_decFWSharedMemWr = (TQP_UINT32)-1;	// byte address
		pTaskData->m_decFWSharedMemRd = (TQP_UINT32)-1;	// byte address

		// codec settings
		//
		if (0 == This->m_pMpegCodec->m_VerFwAPI)
		{
			pTaskData->m_codec_function.function = (TASK_TYPE_ENC == pTaskData->type) ? CODEC_OPEN_VID_AUD_ENCODER : CODEC_OPEN_VID_AUD_DECODER;
		}
		else
		{
			pTaskData->m_codec_function.function = 0;
			pTaskData->m_codec_function.functionEx.video = CODEC_OPEN_EX_VID_H264;
			pTaskData->m_codec_function.functionEx.audio = CODEC_OPEN_EX_AUD;
			pTaskData->m_codec_function.functionEx.function = (TASK_TYPE_ENC == pTaskData->type) ? CODEC_OPEN_EX_ENCODE : CODEC_OPEN_EX_DECODE;
			pTaskData->m_codec_function.functionEx.version = 1;
		}
		pTaskData->m_video_input =  SYS_LINK_VID_IN_HOST;
		pTaskData->m_video_in_ch = 0;
		pTaskData->m_video_output = SYS_LINK_VID_OUT_HOST;
		pTaskData->m_video_out_ch = 0;
		pTaskData->m_audio_input = (TASK_TYPE_ENC == pTaskData->type) ? SYS_LINK_AUD_IN_AIU : SYS_LINK_AUD_IN_HOST;
		pTaskData->m_audio_in_ch = 0;
		pTaskData->m_audio_output = (TASK_TYPE_ENC == pTaskData->type) ? SYS_LINK_AUD_OUT_HOST : SYS_LINK_AUD_OUT_AOU;
		pTaskData->m_audio_out_ch = 0;
	}
	else
	{
		// encoder settings
		pTaskData->m_systemControl.Read = 0;
#ifdef __TEST_HIU_ENCODE
		pTaskData->m_systemControl.Reg.stream_type = STREAM_TYPE_VID_STREAM;
#else //!__TEST_HIU_ENCODE
		pTaskData->m_systemControl.Reg.stream_type = STREAM_TYPE_PROGRAM_STREAM;
#endif //__TEST_HIU_ENCODE
		pTaskData->m_systemControl.Reg.profile = PROFILE_MAIN;
		pTaskData->m_systemControl.Reg.level = LEVEL_3;
		pTaskData->m_systemControl.Reg.cap_mod = CAP_MODE_HIU;
		pTaskData->m_systemControl.Reg.ff_mod = FF_MODE_FRAME_ONLY;
		pTaskData->m_systemControl.Reg.spsr_freq = 0;
		pTaskData->m_systemControl.Reg.v_mode = V_MODE_CABAC;
		pTaskData->m_systemControl.Reg.cabac_init_idc = 0;
		pTaskData->m_systemControl.Reg.version = 1;

		pTaskData->m_pictureResolution.Read = 0;
#ifdef __TEST_HIU_ENCODE
		pTaskData->m_pictureResolution.Reg.pic_width = 640;
#else //!__TEST_HIU_ENCODE
		pTaskData->m_pictureResolution.Reg.pic_width = 720;
#endif //__TEST_HIU_ENCODE
		pTaskData->m_pictureResolution.Reg.v_scale = FILTER_MODE_NONE;	// QL201B
		pTaskData->m_pictureResolution.Reg.pic_height = 480;
		pTaskData->m_pictureResolution.Reg.h_scale = FILTER_MODE_NONE;	// QL201B

		pTaskData->m_inputControl.Read = 0;
#ifdef __TEST_HIU_ENCODE
		pTaskData->m_inputControl.Reg.mode = INPUT_MODE_PROG;
		pTaskData->m_inputControl.Reg.fmt = FMT_PROGRESSIVE;
#else //!__TEST_HIU_ENCODE
		pTaskData->m_inputControl.Reg.mode = This->m_pMpegCodec->m_VIUMode;
		pTaskData->m_inputControl.Reg.fmt = This->m_pMpegCodec->m_VIUFormat;
#endif //__TEST_HIU_ENCODE
		pTaskData->m_inputControl.Reg.ck_edge = This->m_pMpegCodec->m_ClkEdge;
#ifdef __TEST_HIU_ENCODE
		pTaskData->m_inputControl.Reg.src = INPUT_CONTROL_HIU_RAW;
#else //!__TEST_HIU_ENCODE
		pTaskData->m_inputControl.Reg.src = INPUT_CONTROL_VIU;
#endif //__TEST_HIU_ENCODE
		pTaskData->m_inputControl.Reg.dst = INPUT_CONTROL_HIU_COMPRESSED;
		pTaskData->m_inputControl.Reg.bl_e = 0;
		pTaskData->m_inputControl.Reg.black_line_num = 0;
		pTaskData->m_inputControl.Reg.in_frame_rate = IN_FRAME_RATE_ABSOLUTE | 30;
		pTaskData->m_inputControl.Reg.out_frame_rate = 30;
		pTaskData->m_inputControl.Reg.pd_en = 0;

		pTaskData->m_rateControl.Read = 0;
		pTaskData->m_rateControl.Reg.avg_bit_rate = 1500;	// in 1000 bps unit
#ifdef __TEST_HIU_ENCODE
		pTaskData->m_rateControl.Reg.qp_update_freq = 40;
#else //!__TEST_HIU_ENCODE
		pTaskData->m_rateControl.Reg.qp_update_freq = 45;
#endif //__TEST_HIU_ENCODE
		pTaskData->m_rateControl.Reg.vbr = 1;

		pTaskData->m_bitRate.Read = 0;
		pTaskData->m_bitRate.Reg.min_bit_rate = 750;
		pTaskData->m_bitRate.Reg.max_bit_rate = 2250;

		pTaskData->m_filterControl.Read = 0;
#ifdef __TEST_HIU_ENCODE
		pTaskData->m_filterControl.Reg.start_pixel = 0;
		pTaskData->m_filterControl.Reg.start_line = 0;
#else //!__TEST_HIU_ENCODE
		pTaskData->m_filterControl.Reg.start_pixel = This->m_pMpegCodec->m_VIUStartPixel;
		pTaskData->m_filterControl.Reg.start_line = This->m_pMpegCodec->m_VIUStartLine;
#endif //__TEST_HIU_ENCODE
		pTaskData->m_filterControl.Reg.fl_en = 1;
		pTaskData->m_filterControl.Reg.filter_strength = 2;

		pTaskData->m_gopLoopFilter.Read = 0;
		pTaskData->m_gopLoopFilter.Reg.intra_period = 30;
		pTaskData->m_gopLoopFilter.Reg.lp_filter = 1;
		pTaskData->m_gopLoopFilter.Reg.dis_lf_idc = 0;
		pTaskData->m_gopLoopFilter.Reg.alpha_offset = 0;
		pTaskData->m_gopLoopFilter.Reg.beta_offset = 0;
		pTaskData->m_gopLoopFilter.Reg.B_num = 1;
		pTaskData->m_gopLoopFilter.Reg.IDR = IDR_OPEN_GOP;

		pTaskData->m_etControl.Read = 0;
		pTaskData->m_etControl.Reg.et_threshold_offset = 40;
		pTaskData->m_etControl.Reg.et_threshold_max = 30;
		pTaskData->m_etControl.Reg.et = 1;
		pTaskData->m_etControl.Reg.et_sad_threshold = 16;
#ifdef __TEST_YUV_OUT
		pTaskData->m_etControl.Reg.data_channel_type = DATA_CHANNEL_TYPE_YUV_2_HIU;
#else //!__TEST_YUV_OUT
		pTaskData->m_etControl.Reg.data_channel_type = DATA_CHANNEL_TYPE_MPEG_2_HIU;
#endif //__TEST_YUV_OUT
		pTaskData->m_etControl.Reg.ref_num = REF_NUM_3_FRAMES;

		pTaskData->m_audioControlParam.Read = 0;
		pTaskData->m_audioControlParam.Reg.bit_rate = 128;	// in 1000 bps unit
		pTaskData->m_audioControlParam.Reg.ch_sel = CH_SEL_MUX;
		pTaskData->m_audioControlParam.Reg.sample_rate = SAMPLE_RATE_48K;
		pTaskData->m_audioControlParam.Reg.ai_msb = This->m_pMpegCodec->m_AIControls_ai_msb;
		pTaskData->m_audioControlParam.Reg.lrclk_i = This->m_pMpegCodec->m_AIControls_lrclk_i;
		pTaskData->m_audioControlParam.Reg.bclk_i = This->m_pMpegCodec->m_AIControls_bclk_i;	// 0
		pTaskData->m_audioControlParam.Reg.ai_i2s = This->m_pMpegCodec->m_AIControls_ai_i2s;
		pTaskData->m_audioControlParam.Reg.ai_rj = This->m_pMpegCodec->m_AIControls_ai_rj;
		pTaskData->m_audioControlParam.Reg.ai_m = This->m_pMpegCodec->m_AIControls_ai_m;
		pTaskData->m_audioControlParam.Reg.ch_num = 2;
		pTaskData->m_audioControlParam.Reg.pts = 0;
		pTaskData->m_audioControlParam.Reg.disAi = 0;
		pTaskData->m_audioControlParam.Reg.raw = 0;
		pTaskData->m_audioControlParam.Reg.audio_type = AUDIO_TYPE_AAC;

		pTaskData->m_audioControlExAAC.Read = 0;
		pTaskData->m_audioControlExAAC.Reg.aac_ver = AAC_VER_MPEG_2;
		pTaskData->m_audioControlExAAC.Reg.h_type = H_TYPE_ADTS;
		pTaskData->m_audioControlExAAC.Reg.TNS = 1;
		pTaskData->m_audioControlExAAC.Reg.IS = 1;
		pTaskData->m_audioControlExAAC.Reg.PNS = 1;
		pTaskData->m_audioControlExAAC.Reg.MS = 1;
		pTaskData->m_audioControlExAAC.Reg.cutoff_freq = 18000;	// 18k

		pTaskData->m_audioControlExG711.Read = 0;
		pTaskData->m_audioControlExG711.Reg.mode = MODE_A_LAW;
		pTaskData->m_audioControlExG711.Reg.block_size = BLOCK_SIZE_80;

		pTaskData->m_audioControlExLPCM.Read = 0;
		pTaskData->m_audioControlExLPCM.Reg.sample_cnt = 512;

		pTaskData->m_audioControlExSILK.Read = 0;
		pTaskData->m_audioControlExSILK.Reg.frame_len = FRAME_SIZE_60;

		pTaskData->m_EncIndexCapFreq = 32;
		pTaskData->m_MP4VideoBlockNumber = 5;

		pTaskData->m_audioEnhancement.gain1 = 0x2666;	// 0.3
		pTaskData->m_audioEnhancement.gain2 = 0x0;		// 0
		pTaskData->m_audioEnhancement.add = 0x2666;		// 0.3
		pTaskData->m_audioEnhancement.sub = 0x7FFF;		// 1.0
		pTaskData->m_audioEnhancement.att1 = 0x7FFF;		// 1.0
		pTaskData->m_audioEnhancement.att2 = 0x0000;		// 0
		pTaskData->m_audioEnhancement.lgain = 0x0;
		pTaskData->m_audioEnhancement.rgain = 0x0;

		pTaskData->m_EncStopMode = QP_ENC_STOP_IMMEDIATELY;

		pTaskData->m_bEncEnableVidPadding = QP_TRUE;
		pTaskData->m_bEncVidFrozen = QP_FALSE;
		pTaskData->m_bEncVidStillInput = QP_FALSE;

		pTaskData->m_EncCapMode.capMode = QPCODEC_ENC_CAP_MODE_CONTINUOUS;
		pTaskData->m_EncCapMode.trigMode = 0;
		pTaskData->m_EncCapMode.gpioPin = 0;

		pTaskData->m_encFontTableAddr = (TQP_UINT32)-1;
		pTaskData->m_encTextListAddr = (TQP_UINT32)-1;
		pTaskData->m_encSyncTimeAddr = (TQP_UINT32)-1;

		pTaskData->m_encVidBufLumaAddr = (TQP_UINT32)-1;
		pTaskData->m_encVidBufChromaAddr = (TQP_UINT32)-1;
		pTaskData->m_encFrameLABufAddr = (TQP_UINT32)-1;
		pTaskData->m_encTopLABufAddr = (TQP_UINT32)-1;
		pTaskData->m_encBottomLABufAddr = (TQP_UINT32)-1;
		pTaskData->m_encVidBufPTS = 0;

		// decoder settings
#ifdef __TEST_640_ES_DECODE
		pTaskData->m_streamID.Reg.stream_type = STREAM_TYPE_VID_STREAM;
#else //!__TEST_640_ES_DECODE
		pTaskData->m_streamID.Reg.stream_type = STREAM_TYPE_PROGRAM_STREAM;
#endif //__TEST_640_ES_DECODE
		pTaskData->m_outputMode = QPDEC_DEC_START_VOU |
								  QPDEC_DEC_START_VOU_AUTO
								  ;
		pTaskData->m_decodeResolution.Read = 0;
#ifdef __TEST_640_ES_DECODE
		pTaskData->m_decodeResolution.Reg.pic_width = 640;
#else //!__TEST_640_ES_DECODE
		pTaskData->m_decodeResolution.Reg.pic_width = 720;
#endif //__TEST_640_ES_DECODE
		pTaskData->m_decodeResolution.Reg.pic_height = 480;

		pTaskData->m_decStopMode = QPDEC_DEC_STOP_IMMEDIATELY;
  		pTaskData->m_decStopDispMode = QPDEC_DISP_LAST_FRAME;

  		pTaskData->m_decPauseDispMode = QPDEC_DISP_LAST_FRAME;
		pTaskData->m_decTrickPlaySpeed = QPDEC_TRICK_SPD_NORMAL;
		pTaskData->m_decTrickPlayDirection = QPDEC_TRICK_DIR_FORWARD;
		pTaskData->m_decTrickFieldPreference = QPDEC_TRICK_DISP_DEFAULT;
  		pTaskData->m_decDispMode = QPDEC_DISP_MODE_NORMAL;
  		pTaskData->m_decDispTVSystem = QPDEC_DISP_ATTRIB_NTSC;
  		pTaskData->m_decDispForceRemote = 0;
  		pTaskData->m_decDispInputSize.Read = 0;
#ifdef __TEST_640_ES_DECODE
  		pTaskData->m_decDispInputSize.Attrib.x = 640;
#else //!__TEST_640_ES_DECODE
  		pTaskData->m_decDispInputSize.Attrib.x = 720;
#endif //__TEST_640_ES_DECODE
  		pTaskData->m_decDispInputSize.Attrib.y = 480;
  		pTaskData->m_decDispOutputSize.Read = 0;
#ifdef __TEST_640_ES_DECODE
  		pTaskData->m_decDispOutputSize.Attrib.x = 640;
#else //!__TEST_640_ES_DECODE
  		pTaskData->m_decDispOutputSize.Attrib.x = 720;
#endif //__TEST_640_ES_DECODE
  		pTaskData->m_decDispOutputSize.Attrib.y = 480;
  		pTaskData->m_decDispInputOrigin.Read = 0;
  		pTaskData->m_decDispInputOrigin.Attrib.x = 0;
  		pTaskData->m_decDispInputOrigin.Attrib.y = 0;
  		pTaskData->m_decDispOutputOrigin.Read = 0;
  		pTaskData->m_decDispOutputOrigin.Attrib.x = 0;
  		pTaskData->m_decDispOutputOrigin.Attrib.y = 0;
		pTaskData->m_decXferMode = QPDEC_DEC_XFER_MODE_ALL;
		pTaskData->m_decXferMinimumSize = 0;

		// decoder playback information
		//
		pTaskData->m_decFWSharedMemWr = (TQP_UINT32)-1;	// byte address
		pTaskData->m_decFWSharedMemRd = (TQP_UINT32)-1;	// byte address
	}
}



static void CTask_BuildIoBlock(CTask *This,
							   PTASK_DATA pTaskData,
							   TASK_DATA_TYPE dataType
							   )
{
	CHANNEL_DIRECTION	dir = pTaskData->direction[dataType];

	ENTER_CRITICAL_IO_PENDING(This)

	if (CTask_isIOReady(This,
						dir
						))
	{
		TQP_UINT32	ulRemain = min(pTaskData->UserBuffer[dataType].BUFFER.OTHERS.ulLength - pTaskData->UserBuffer[dataType].pBufDesc->ulBufferOffset,
								   (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulSize << 2) - pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulOffset
								   );

		if (ulRemain)
		{
			ulRemain = min(This->m_dwMaxDMASize, ulRemain);

			This->m_ioPending[dir].pHostAddress = pTaskData->UserBuffer[dataType].BUFFER.OTHERS.pBuffer + pTaskData->UserBuffer[dataType].pBufDesc->ulBufferOffset;
			This->m_ioPending[dir].ulArmAddress = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulAddr + (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulOffset >> 2);
			This->m_ioPending[dir].ulLength = min(This->m_dwMaxDMASize, ulRemain);
			This->m_ioPending[dir].ulXferMode = DMA_TRANSFER_MODE_LINEAR;
			if (pTaskData->UserBuffer[dataType].pBufDesc->ulFlags & QP_BUFDESC_FLAG_BUFFER_DRIVER)
			{
				This->m_ioPending[dir].ulXferMode |= DMA_BUFFER_MODE_KERNEL;
			}
			else if (pTaskData->UserBuffer[dataType].pBufDesc->ulFlags & QP_BUFDESC_FLAG_BUFFER_USER)
			{
				This->m_ioPending[dir].ulXferMode |= DMA_BUFFER_MODE_USERPTR;
			}
			else
			{
				This->m_ioPending[dir].ulXferMode |= DMA_BUFFER_MODE_COMMON;
			}
			This->m_ioPending[dir].ulPicWidth = 0;
			This->m_ioPending[dir].ulPicHeight = 0;

			if (This->m_ioPending[dir].ulLength & 0x3)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CTask_BuildIoBlock() size(0x%x) not dword aligned\n",
							 This->m_ioPending[dir].ulLength
							 ));
			}
			This->m_ioPending[dir].pTaskData = pTaskData;
			This->m_ioPending[dir].dataType = dataType;
			This->m_ioPending[dir].status = QPERR_PENDING;

			This->m_pPending[dir] = &This->m_ioPending[dir];
		}
		else
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "\nCTask_BuildIoBlock() user buf size(%d) offset(%d) arm buf size(%d) offset(%d)\n",
						 pTaskData->UserBuffer[dataType].BUFFER.OTHERS.ulLength,
						 pTaskData->UserBuffer[dataType].pBufDesc->ulBufferOffset,
						 pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulSize << 2,
						 pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulOffset
						 ));

			if (TASK_TYPE_ENC == pTaskData->type)
			{
				CEncoderTask_ProcessIoComplete(This,
											   pTaskData,
											   dataType,
											   QP_FALSE
											   );
			}
			else
			{
				CDecoderTask_ProcessIoComplete(This,
											   pTaskData,
											   dataType,
											   QP_FALSE
											   );
			}
		}
	}

	LEAVE_CRITICAL_IO_PENDING(This)
}



static void CTask_BuildIoBlockYUV(CTask *This,
								  PTASK_DATA pTaskData,
								  TASK_DATA_TYPE dataType
								  )
{
	CHANNEL_DIRECTION	dir = pTaskData->direction[dataType];

	ENTER_CRITICAL_IO_PENDING(This)

	if (CTask_isIOReady(This,
						dir
						))
	{
		TQP_UINT32	ulRemainY = pTaskData->UserBuffer[dataType].BUFFER.YUV.ulYLength - pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulYOffset;
		TQP_UINT32	ulRemainUV = pTaskData->UserBuffer[dataType].BUFFER.YUV.ulUVLength - pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulUVOffset;
		PTQP_UINT8	pYBuffer = pTaskData->UserBuffer[dataType].BUFFER.YUV.pYBuffer + pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulYOffset;
		PTQP_UINT8	pUVBuffer = pTaskData->UserBuffer[dataType].BUFFER.YUV.pUVBuffer + pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulUVOffset;
		TQP_UINT32	ulArmYAddress = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulYAddr + (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulYOffset >> 2);
		TQP_UINT32	ulArmUVAddress = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulUVAddr + (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulUVOffset >> 2);

		if (ulRemainY)
		{
			ulRemainY = min(This->m_dwMaxDMASize, ulRemainY);

			This->m_ioPending[dir].pHostAddress = pYBuffer;
			This->m_ioPending[dir].ulArmAddress = ulArmYAddress;
			This->m_ioPending[dir].ulLength = ulRemainY;
			This->m_ioPending[dir].ulXferMode = DMA_TRANSFER_MODE_LINEAR;//DMA_TRANSFER_MODE_MB_RAS_LUMA;
			if (pTaskData->UserBuffer[dataType].pBufDesc->ulFlags & QP_BUFDESC_FLAG_BUFFER_DRIVER)
			{
				This->m_ioPending[dir].ulXferMode |= DMA_BUFFER_MODE_KERNEL;
			}
			else if (pTaskData->UserBuffer[dataType].pBufDesc->ulFlags & QP_BUFDESC_FLAG_BUFFER_USER)
			{
				This->m_ioPending[dir].ulXferMode |= DMA_BUFFER_MODE_USERPTR;
			}
			else
			{
				This->m_ioPending[dir].ulXferMode |= DMA_BUFFER_MODE_COMMON;
			}
			This->m_ioPending[dir].ulPicWidth = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulWidth;
			This->m_ioPending[dir].ulPicHeight = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulHeight;
			This->m_ioPending[dir].pTaskData = pTaskData;
			This->m_ioPending[dir].dataType = dataType;
			This->m_ioPending[dir].status = QPERR_PENDING;

			This->m_pPending[dir] = &This->m_ioPending[dir];

			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
						 "CTask_BuildIoBlockYUV(%d) Y HostAddr(0x%x) ulArmAddress(0x%x) len(%d) w(%d) h(%d)\n",
						 dir,
						 This->m_ioPending[dir].pHostAddress,
						 This->m_ioPending[dir].ulArmAddress,
						 This->m_ioPending[dir].ulLength,
						 This->m_ioPending[dir].ulPicWidth,
						 This->m_ioPending[dir].ulPicHeight
						 ));
		}
		else if (ulRemainUV)
		{
			ulRemainUV = min(This->m_dwMaxDMASize, ulRemainUV);

			This->m_ioPending[dir].pHostAddress = pUVBuffer;
			This->m_ioPending[dir].ulArmAddress = ulArmUVAddress;
			This->m_ioPending[dir].ulLength = ulRemainUV;
			This->m_ioPending[dir].ulXferMode = DMA_TRANSFER_MODE_LINEAR;//DMA_TRANSFER_MODE_MB_RAS_CHROMA;
			if (pTaskData->UserBuffer[dataType].pBufDesc->ulFlags & QP_BUFDESC_FLAG_BUFFER_DRIVER)
			{
				This->m_ioPending[dir].ulXferMode |= DMA_BUFFER_MODE_KERNEL;
			}
			else if (pTaskData->UserBuffer[dataType].pBufDesc->ulFlags & QP_BUFDESC_FLAG_BUFFER_USER)
			{
				This->m_ioPending[dir].ulXferMode |= DMA_BUFFER_MODE_USERPTR;
			}
			else
			{
				This->m_ioPending[dir].ulXferMode |= DMA_BUFFER_MODE_COMMON;
			}
			This->m_ioPending[dir].ulPicWidth = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulWidth;
			This->m_ioPending[dir].ulPicHeight = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulHeight;
			This->m_ioPending[dir].pTaskData = pTaskData;
			This->m_ioPending[dir].dataType = dataType;
			This->m_ioPending[dir].status = QPERR_PENDING;

			This->m_pPending[dir] = &This->m_ioPending[dir];

			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
						 "CTask_BuildIoBlockYUV(%d) UV HostAddr(0x%x) ulArmAddress(0x%x) len(%d) w(%d) h(%d)\n",
						 dir,
						 This->m_ioPending[dir].pHostAddress,
						 This->m_ioPending[dir].ulArmAddress,
						 This->m_ioPending[dir].ulLength,
						 This->m_ioPending[dir].ulPicWidth,
						 This->m_ioPending[dir].ulPicHeight
						 ));
		}
		else
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CTask_BuildIoBlockYUV() How do we get here???\n"
						 ));

			if (TASK_TYPE_ENC == pTaskData->type)
			{
				CEncoderTask_ProcessIoComplete(This,
											   pTaskData,
											   dataType,
											   QP_FALSE
											   );
			}
			else
			{
				CDecoderTask_ProcessIoComplete(This,
											   pTaskData,
											   dataType,
											   QP_FALSE
											   );
			}
		}
	}

	LEAVE_CRITICAL_IO_PENDING(This)
}



static void CTask_BuildIoBlockYUVMB2RAS(CTask *This,
										PTASK_DATA pTaskData,
										TASK_DATA_TYPE dataType
										)
{
	CHANNEL_DIRECTION	dir = pTaskData->direction[dataType];

	ENTER_CRITICAL_IO_PENDING(This)

	if (CTask_isIOReady(This,
						dir
						))
	{
		TQP_UINT32	ulRemainY = pTaskData->UserBuffer[dataType].BUFFER.YUVRAS.ulYLength - pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulYOffset;
		TQP_UINT32	ulRemainU = pTaskData->UserBuffer[dataType].BUFFER.YUVRAS.ulULength - pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulUOffset;
		TQP_UINT32	ulRemainV = pTaskData->UserBuffer[dataType].BUFFER.YUVRAS.ulVLength - pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulVOffset;
		PTQP_UINT8	pYBuffer = pTaskData->UserBuffer[dataType].BUFFER.YUVRAS.pYBuffer + pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulYOffset;
		PTQP_UINT8	pUBuffer = pTaskData->UserBuffer[dataType].BUFFER.YUVRAS.pUBuffer + pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulUOffset;
		PTQP_UINT8	pVBuffer = pTaskData->UserBuffer[dataType].BUFFER.YUVRAS.pVBuffer + pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulVOffset;
		TQP_UINT32	ulArmYAddress = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulYAddr + (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulYOffset >> 2);
		TQP_UINT32	ulArmUAddress = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulUVAddr + (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulUOffset >> 1);
		TQP_UINT32	ulArmVAddress = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulUVAddr + (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulVOffset >> 1) + 16;
		TQP_UINT32	ulSizePerMBRow;
		TQP_UINT32	ulMaxRows;
		TQP_UINT32	ulRowsRemain;

		if (ulRemainY)
		{
			ulSizePerMBRow = 16 * pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulWidth;
			ulMaxRows = This->m_dwMaxDMASize / ulSizePerMBRow;
			ulRowsRemain = ulRemainY / ulSizePerMBRow;

			ulRowsRemain = min(ulMaxRows, ulRowsRemain);

			if (ulMaxRows > ulRowsRemain)
			{
				This->m_ioPending[dir].ulLength = ulRemainY;
				This->m_ioPending[dir].ulPicHeight = ulRemainY / pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulWidth;
			}
			else
			{
				This->m_ioPending[dir].ulLength = ulRowsRemain * ulSizePerMBRow;
				This->m_ioPending[dir].ulPicHeight = ulRowsRemain * 16;
			}

			This->m_ioPending[dir].pHostAddress = pYBuffer;
			This->m_ioPending[dir].ulArmAddress = ulArmYAddress;
			This->m_ioPending[dir].ulXferMode = DMA_TRANSFER_MODE_MB_RAS_LUMA;
			if (pTaskData->UserBuffer[dataType].pBufDesc->ulFlags & QP_BUFDESC_FLAG_BUFFER_DRIVER)
			{
				This->m_ioPending[dir].ulXferMode |= DMA_BUFFER_MODE_KERNEL;
			}
			else if (pTaskData->UserBuffer[dataType].pBufDesc->ulFlags & QP_BUFDESC_FLAG_BUFFER_USER)
			{
				This->m_ioPending[dir].ulXferMode |= DMA_BUFFER_MODE_USERPTR;
			}
			else
			{
				This->m_ioPending[dir].ulXferMode |= DMA_BUFFER_MODE_COMMON;
			}
			This->m_ioPending[dir].ulPicWidth = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulWidth;
			This->m_ioPending[dir].pTaskData = pTaskData;
			This->m_ioPending[dir].dataType = dataType;
			This->m_ioPending[dir].status = QPERR_PENDING;

			This->m_pPending[dir] = &This->m_ioPending[dir];

			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
						 "CTask_BuildIoBlockYUVMB2RAS(%d) Y HostAddr(0x%x) ulArmAddress(0x%x) len(%d) w(%d) h(%d)\n",
						 dir,
						 This->m_ioPending[dir].pHostAddress,
						 This->m_ioPending[dir].ulArmAddress,
						 This->m_ioPending[dir].ulLength,
						 This->m_ioPending[dir].ulPicWidth,
						 This->m_ioPending[dir].ulPicHeight
						 ));
		}
		else if (ulRemainU)
		{
			ulSizePerMBRow = 8 * (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulWidth >> 1);
			ulMaxRows = This->m_dwMaxDMASize / ulSizePerMBRow;
			ulRowsRemain = ulRemainU / ulSizePerMBRow;

			ulRowsRemain = min(ulMaxRows, ulRowsRemain);

			if (ulMaxRows > ulRowsRemain)
			{
				This->m_ioPending[dir].ulLength = ulRemainU;
				This->m_ioPending[dir].ulPicHeight = ulRemainU / (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulWidth >> 2);
			}
			else
			{
				This->m_ioPending[dir].ulLength = ulRowsRemain * ulSizePerMBRow;
				This->m_ioPending[dir].ulPicHeight = ulRowsRemain * 16;
			}
			This->m_ioPending[dir].pHostAddress = pUBuffer;
			This->m_ioPending[dir].ulArmAddress = ulArmUAddress;
			This->m_ioPending[dir].ulXferMode = DMA_TRANSFER_MODE_MB_RAS_CHROMA;
			if (pTaskData->UserBuffer[dataType].pBufDesc->ulFlags & QP_BUFDESC_FLAG_BUFFER_DRIVER)
			{
				This->m_ioPending[dir].ulXferMode |= DMA_BUFFER_MODE_KERNEL;
			}
			else if (pTaskData->UserBuffer[dataType].pBufDesc->ulFlags & QP_BUFDESC_FLAG_BUFFER_USER)
			{
				This->m_ioPending[dir].ulXferMode |= DMA_BUFFER_MODE_USERPTR;
			}
			else
			{
				This->m_ioPending[dir].ulXferMode |= DMA_BUFFER_MODE_COMMON;
			}
			This->m_ioPending[dir].ulPicWidth = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulWidth;
			This->m_ioPending[dir].pTaskData = pTaskData;
			This->m_ioPending[dir].dataType = dataType;
			This->m_ioPending[dir].status = QPERR_PENDING;

			This->m_pPending[dir] = &This->m_ioPending[dir];

			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
						 "CTask_BuildIoBlockYUVMB2RAS(%d) U HostAddr(0x%x) ulArmAddress(0x%x) len(%d) w(%d) h(%d)\n",
						 dir,
						 This->m_ioPending[dir].pHostAddress,
						 This->m_ioPending[dir].ulArmAddress,
						 This->m_ioPending[dir].ulLength,
						 This->m_ioPending[dir].ulPicWidth,
						 This->m_ioPending[dir].ulPicHeight
						 ));

		}
		else if (ulRemainV)
		{
			ulSizePerMBRow = 8 * (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulWidth >> 1);
			ulMaxRows = This->m_dwMaxDMASize / ulSizePerMBRow;
			ulRowsRemain = ulRemainV / ulSizePerMBRow;

			ulRowsRemain = min(ulMaxRows, ulRowsRemain);

			if (ulMaxRows > ulRowsRemain)
			{
				This->m_ioPending[dir].ulLength = ulRemainV;
				This->m_ioPending[dir].ulPicHeight = ulRemainV / (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulWidth >> 2);
			}
			else
			{
				This->m_ioPending[dir].ulLength = ulRowsRemain * ulSizePerMBRow;
				This->m_ioPending[dir].ulPicHeight = ulRowsRemain * 16;
			}

			This->m_ioPending[dir].pHostAddress = pVBuffer;
			This->m_ioPending[dir].ulArmAddress = ulArmVAddress;
			This->m_ioPending[dir].ulXferMode = DMA_TRANSFER_MODE_MB_RAS_CHROMA;
			if (pTaskData->UserBuffer[dataType].pBufDesc->ulFlags & QP_BUFDESC_FLAG_BUFFER_DRIVER)
			{
				This->m_ioPending[dir].ulXferMode |= DMA_BUFFER_MODE_KERNEL;
			}
			else if (pTaskData->UserBuffer[dataType].pBufDesc->ulFlags & QP_BUFDESC_FLAG_BUFFER_USER)
			{
				This->m_ioPending[dir].ulXferMode |= DMA_BUFFER_MODE_USERPTR;
			}
			else
			{
				This->m_ioPending[dir].ulXferMode |= DMA_BUFFER_MODE_COMMON;
			}
			This->m_ioPending[dir].ulPicWidth = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulWidth;
			This->m_ioPending[dir].pTaskData = pTaskData;
			This->m_ioPending[dir].dataType = dataType;
			This->m_ioPending[dir].status = QPERR_PENDING;

			This->m_pPending[dir] = &This->m_ioPending[dir];

			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
						 "CTask_BuildIoBlockYUVMB2RAS(%d) V HostAddr(0x%x) ulArmAddress(0x%x) len(%d) w(%d) h(%d)\n",
						 dir,
						 This->m_ioPending[dir].pHostAddress,
						 This->m_ioPending[dir].ulArmAddress,
						 This->m_ioPending[dir].ulLength,
						 This->m_ioPending[dir].ulPicWidth,
						 This->m_ioPending[dir].ulPicHeight
						 ));
		}
		else
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CTask_BuildIoBlockYUVMB2RAS() How do we get here???\n"
						 ));

			if (TASK_TYPE_ENC == pTaskData->type)
			{
				CEncoderTask_ProcessIoComplete(This,
											   pTaskData,
											   dataType,
											   QP_FALSE
											   );
			}
			else
			{
				CDecoderTask_ProcessIoComplete(This,
											   pTaskData,
											   dataType,
											   QP_FALSE
											   );
			}
		}
	}

	LEAVE_CRITICAL_IO_PENDING(This)
}



static void CTask_BuildIoBlockYUVRAS(CTask *This,
									 PTASK_DATA pTaskData,
									 TASK_DATA_TYPE dataType
									 )
{
	CHANNEL_DIRECTION	dir = pTaskData->direction[dataType];

	ENTER_CRITICAL_IO_PENDING(This)

	if (CTask_isIOReady(This,
						dir
						))
	{
		TQP_UINT32	ulRemainY = pTaskData->UserBuffer[dataType].BUFFER.YUVRAS.ulYLength - pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulYOffset;
		TQP_UINT32	ulRemainU = pTaskData->UserBuffer[dataType].BUFFER.YUVRAS.ulULength - pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulUOffset;
		TQP_UINT32	ulRemainV = pTaskData->UserBuffer[dataType].BUFFER.YUVRAS.ulVLength - pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulVOffset;
		PTQP_UINT8	pYBuffer = pTaskData->UserBuffer[dataType].BUFFER.YUVRAS.pYBuffer + pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulYOffset;
		PTQP_UINT8	pUBuffer = pTaskData->UserBuffer[dataType].BUFFER.YUVRAS.pUBuffer + pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulUOffset;
		PTQP_UINT8	pVBuffer = pTaskData->UserBuffer[dataType].BUFFER.YUVRAS.pVBuffer + pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulVOffset;
		TQP_UINT32	ulArmYAddress = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulYAddr + (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulYOffset >> 2);
		TQP_UINT32	ulArmUAddress = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulUAddr + (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulUOffset >> 2);
		TQP_UINT32	ulArmVAddress = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulVAddr + (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulVOffset >> 2);

		if (ulRemainY)
		{
			ulRemainY = min(This->m_dwMaxDMASize, ulRemainY);

			This->m_ioPending[dir].pHostAddress = pYBuffer;
			This->m_ioPending[dir].ulArmAddress = ulArmYAddress;
			This->m_ioPending[dir].ulLength = ulRemainY;
			This->m_ioPending[dir].ulXferMode = DMA_TRANSFER_MODE_LINEAR;
			if (pTaskData->UserBuffer[dataType].pBufDesc->ulFlags & QP_BUFDESC_FLAG_BUFFER_DRIVER)
			{
				This->m_ioPending[dir].ulXferMode |= DMA_BUFFER_MODE_KERNEL;
			}
			else if (pTaskData->UserBuffer[dataType].pBufDesc->ulFlags & QP_BUFDESC_FLAG_BUFFER_USER)
			{
				This->m_ioPending[dir].ulXferMode |= DMA_BUFFER_MODE_USERPTR;
			}
			else
			{
				This->m_ioPending[dir].ulXferMode |= DMA_BUFFER_MODE_COMMON;
			}
			This->m_ioPending[dir].ulPicWidth = 0;
			This->m_ioPending[dir].ulPicHeight = 0;
			This->m_ioPending[dir].pTaskData = pTaskData;
			This->m_ioPending[dir].dataType = dataType;
			This->m_ioPending[dir].status = QPERR_PENDING;

			This->m_pPending[dir] = &This->m_ioPending[dir];

			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
						 "CTask_BuildIoBlockYUVRAS(%d) Y HostAddr(0x%x) ulArmAddress(0x%x) len(%d) w(%d) h(%d)\n",
						 dir,
						 This->m_ioPending[dir].pHostAddress,
						 This->m_ioPending[dir].ulArmAddress,
						 This->m_ioPending[dir].ulLength,
						 This->m_ioPending[dir].ulPicWidth,
						 This->m_ioPending[dir].ulPicHeight
						 ));
		}
		else if (ulRemainU)
		{
			ulRemainU = min(This->m_dwMaxDMASize, ulRemainU);

			This->m_ioPending[dir].pHostAddress = pUBuffer;
			This->m_ioPending[dir].ulArmAddress = ulArmUAddress;
			This->m_ioPending[dir].ulLength = ulRemainU;
			This->m_ioPending[dir].ulXferMode = DMA_TRANSFER_MODE_LINEAR;
			if (pTaskData->UserBuffer[dataType].pBufDesc->ulFlags & QP_BUFDESC_FLAG_BUFFER_DRIVER)
			{
				This->m_ioPending[dir].ulXferMode |= DMA_BUFFER_MODE_KERNEL;
			}
			else if (pTaskData->UserBuffer[dataType].pBufDesc->ulFlags & QP_BUFDESC_FLAG_BUFFER_USER)
			{
				This->m_ioPending[dir].ulXferMode |= DMA_BUFFER_MODE_USERPTR;
			}
			else
			{
				This->m_ioPending[dir].ulXferMode |= DMA_BUFFER_MODE_COMMON;
			}
			This->m_ioPending[dir].ulPicWidth = 0;
			This->m_ioPending[dir].ulPicHeight = 0;
			This->m_ioPending[dir].pTaskData = pTaskData;
			This->m_ioPending[dir].dataType = dataType;
			This->m_ioPending[dir].status = QPERR_PENDING;

			This->m_pPending[dir] = &This->m_ioPending[dir];

			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
						 "CTask_BuildIoBlockYUVRAS(%d) U HostAddr(0x%x) ulArmAddress(0x%x) len(%d) w(%d) h(%d)\n",
						 dir,
						 This->m_ioPending[dir].pHostAddress,
						 This->m_ioPending[dir].ulArmAddress,
						 This->m_ioPending[dir].ulLength,
						 This->m_ioPending[dir].ulPicWidth,
						 This->m_ioPending[dir].ulPicHeight
						 ));
		}
		else if (ulRemainV)
		{
			ulRemainV = min(This->m_dwMaxDMASize, ulRemainV);

			This->m_ioPending[dir].pHostAddress = pVBuffer;
			This->m_ioPending[dir].ulArmAddress = ulArmVAddress;
			This->m_ioPending[dir].ulLength = ulRemainV;
			This->m_ioPending[dir].ulXferMode = DMA_TRANSFER_MODE_LINEAR;
			if (pTaskData->UserBuffer[dataType].pBufDesc->ulFlags & QP_BUFDESC_FLAG_BUFFER_DRIVER)
			{
				This->m_ioPending[dir].ulXferMode |= DMA_BUFFER_MODE_KERNEL;
			}
			else if (pTaskData->UserBuffer[dataType].pBufDesc->ulFlags & QP_BUFDESC_FLAG_BUFFER_USER)
			{
				This->m_ioPending[dir].ulXferMode |= DMA_BUFFER_MODE_USERPTR;
			}
			else
			{
				This->m_ioPending[dir].ulXferMode |= DMA_BUFFER_MODE_COMMON;
			}
			This->m_ioPending[dir].ulPicWidth = 0;
			This->m_ioPending[dir].ulPicHeight = 0;
			This->m_ioPending[dir].pTaskData = pTaskData;
			This->m_ioPending[dir].dataType = dataType;
			This->m_ioPending[dir].status = QPERR_PENDING;

			This->m_pPending[dir] = &This->m_ioPending[dir];

			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
						 "CTask_BuildIoBlockYUVRAS(%d) V HostAddr(0x%x) ulArmAddress(0x%x) len(%d) w(%d) h(%d)\n",
						 dir,
						 This->m_ioPending[dir].pHostAddress,
						 This->m_ioPending[dir].ulArmAddress,
						 This->m_ioPending[dir].ulLength,
						 This->m_ioPending[dir].ulPicWidth,
						 This->m_ioPending[dir].ulPicHeight
						 ));
		}
		else
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CTask_BuildIoBlockYUVRAS() How do we get here???\n"
						 ));

			if (TASK_TYPE_ENC == pTaskData->type)
			{
				CEncoderTask_ProcessIoComplete(This,
											   pTaskData,
											   dataType,
											   QP_FALSE
											   );
			}
			else
			{
				CDecoderTask_ProcessIoComplete(This,
											   pTaskData,
											   dataType,
											   QP_FALSE
											   );
			}
		}
	}

	LEAVE_CRITICAL_IO_PENDING(This)
}



static TQP_BOOL CTask_BuildIoBlockSideBandDMA(CTask *This,
											  PTASK_DMA_REQUEST pDMARequest
											  )
{
	CHANNEL_DIRECTION	dir = pDMARequest->dir;
	TQP_BOOL			bRet = QP_FALSE;

	ENTER_CRITICAL_IO_PENDING(This)

	if (CTask_isIOReady(This,
						dir
						))
	{
		TQP_UINT32	ulRemain = pDMARequest->ulLength - pDMARequest->ulOffset;

		if (ulRemain)
		{
			ulRemain = min(This->m_dwMaxDMASize, ulRemain);

			This->m_ioPending[dir].pHostAddress = pDMARequest->pBuffer + pDMARequest->ulOffset;
			This->m_ioPending[dir].ulArmAddress = pDMARequest->ulArmAddr + (pDMARequest->ulOffset >> 2);
			This->m_ioPending[dir].ulLength = min(This->m_dwMaxDMASize, ulRemain);
			This->m_ioPending[dir].ulXferMode = DMA_TRANSFER_MODE_LINEAR | DMA_BUFFER_MODE_KERNEL;
			This->m_ioPending[dir].ulPicWidth = 0;
			This->m_ioPending[dir].ulPicHeight = 0;

			if (This->m_ioPending[dir].ulLength & 0x3)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CTask_BuildIoBlockSideBandDMA() size(0x%x) not dword aligned\n",
							 This->m_ioPending[dir].ulLength
							 ));
			}
			This->m_ioPending[dir].pTaskData = QP_NULL;
			This->m_ioPending[dir].dataType = TASK_DATA_TYPE_INVALID;
			This->m_ioPending[dir].status = QPERR_PENDING;

			This->m_pPending[dir] = &This->m_ioPending[dir];
			bRet = QP_TRUE;
		}
	}

	LEAVE_CRITICAL_IO_PENDING(This)
	return (bRet);
}



static void CTask_SendIoBlock(CTask *This,
							  PTASK_DATA pTaskData,
							  TASK_DATA_TYPE dataType
							  )
{
	CHANNEL_DIRECTION	dir = pTaskData->direction[dataType];

	if (This->m_pPending[dir])
	{
		TQP_BOOL	bSwap = CChannel_needByteSwapping(pTaskData->pChannel[dataType]);

		// advance the task id and dataType so the scheduler will start next time
		// in a round robin fashion
		//
		if ((TASK_DATA_TYPE_COMP_AUD != dataType) ||
			(This->m_taskIdAESOut != pTaskData->id)
			)
		{
		if ((TASK_DATA_TYPE_PCM != dataType) ||
			(This->m_taskIdPCMOut != pTaskData->id)
			)
		{
			pTaskData->m_StartID = (dataType + 1) % MAX_TASK_DATA_TYPE;
			This->m_StartID = (pTaskData->id + 1) % MAX_TASKS;
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, /*QPDBGM_LEVEL_TRACE*/QPDBGM_LEVEL_WARN),
						 "CTask_SendIoBlock() (%d:%d) next(%d:%d)\n",
						 pTaskData->id,
						 dataType,
						 This->m_StartID,
						 pTaskData->m_StartID
						 ));
		}
		}
		if (CHANNEL_DIR_READ == dir)
		{
			if (This->m_pPending[dir]->ulLength < sizeof(TQP_UINT32))
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CTask_SendIoBlock() CHANNEL_DIR_READ size(%d)\n",
							 This->m_pPending[dir]->ulLength
							 ));
				QPOSMSetEvtgrp(This->m_Thread.m_EvtWait,
							   QLCODEC_EVENT_DMA_READ_DONE
							   );
			}
			else
			{
				CQLCodec_StartDMARead(This->m_pMpegCodec,
									  This->m_pPending[dir]->ulArmAddress,
									  This->m_pPending[dir]->pHostAddress,
									  This->m_pPending[dir]->ulLength & 0xFFFFFFFC,
									  bSwap,
									  QP_FALSE,
									  This->m_pPending[dir]->ulXferMode,
									  This->m_pPending[dir]->ulPicWidth,
									  This->m_pPending[dir]->ulPicHeight,
									  QP_TRUE
									  );
			}
		}
		else if (CHANNEL_DIR_WRITE == dir)
		{
			if (This->m_pPending[dir]->ulLength < sizeof(TQP_UINT32))
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CTask_SendIoBlock() CHANNEL_DIR_WRITE size(%d)\n",
							 This->m_pPending[dir]->ulLength
							 ));
				QPOSMSetEvtgrp(This->m_Thread.m_EvtWait,
							   QLCODEC_EVENT_DMA_WRITE_DONE
							   );
			}
			else
			{
				CQLCodec_StartDMAWrite(This->m_pMpegCodec,
									   This->m_pPending[dir]->ulArmAddress,
									   This->m_pPending[dir]->pHostAddress,
									   This->m_pPending[dir]->ulLength & 0xFFFFFFFC,
									   bSwap,
									   QP_FALSE,
									   This->m_pPending[dir]->ulXferMode,
									   This->m_pPending[dir]->ulPicWidth,
									   This->m_pPending[dir]->ulPicHeight,
									   QP_TRUE
									   );
			}
		}
		else
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CTask_SendIoBlock() Direction (%d) How do we get here???\n",
						 dir
						 ));

  			//ENTER_CRITICAL_IO_PENDING(This)
			//This->m_pPending[dir] = QP_NULL;
  			//LEAVE_CRITICAL_IO_PENDING(This)
		}
	}
}



static void CTask_SendIoBlockSideBandDMA(CTask *This,
										 PTASK_DMA_REQUEST pDMARequest
										 )
{
	CHANNEL_DIRECTION	dir = pDMARequest->dir;

	if (This->m_pPending[dir])
	{
		TQP_BOOL	bSwap = pDMARequest->bSwap;

		if (CHANNEL_DIR_READ == dir)
		{
			if (This->m_pPending[dir]->ulLength < sizeof(TQP_UINT32))
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CTask_SendIoBlockSideBandDMA() CHANNEL_DIR_READ size(%d)\n",
							 This->m_pPending[dir]->ulLength
							 ));
				This->m_pDmaRequest = QP_NULL;
				QPOSMSetEvtgrp(This->m_EvtDamReqComplete,
							   TASK_EVENT_DMA_REQ_DONE
							   );
			}
			else
			{
				CQLCodec_StartDMARead(This->m_pMpegCodec,
									  This->m_pPending[dir]->ulArmAddress,
									  This->m_pPending[dir]->pHostAddress,
									  This->m_pPending[dir]->ulLength & 0xFFFFFFFC,
									  bSwap,
									  QP_FALSE,
									  This->m_pPending[dir]->ulXferMode,
									  This->m_pPending[dir]->ulPicWidth,
									  This->m_pPending[dir]->ulPicHeight,
									  QP_TRUE
									  );
			}
		}
		else
		{
			if (This->m_pPending[dir]->ulLength < sizeof(TQP_UINT32))
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CTask_SendIoBlockSideBandDMA() CHANNEL_DIR_WRITE size(%d)\n",
							 This->m_pPending[dir]->ulLength
							 ));
				This->m_pDmaRequest = QP_NULL;
				QPOSMSetEvtgrp(This->m_EvtDamReqComplete,
							   TASK_EVENT_DMA_REQ_DONE
							   );
			}
			else
			{
				CQLCodec_StartDMAWrite(This->m_pMpegCodec,
									   This->m_pPending[dir]->ulArmAddress,
									   This->m_pPending[dir]->pHostAddress,
									   This->m_pPending[dir]->ulLength & 0xFFFFFFFC,
									   bSwap,
									   QP_FALSE,
									   This->m_pPending[dir]->ulXferMode,
									   This->m_pPending[dir]->ulPicWidth,
									   This->m_pPending[dir]->ulPicHeight,
									   QP_TRUE
									   );
			}
		}
	}
}



// CTask
//
static EQP_Errors CTask_Alloc(CTask *This,
							  TASK_TYPE type,
							  TQP_OBJECT_HANDLE *phTask
							  )
{
	int			i, j;
	PTASK_DATA	pTaskData = QP_NULL;

	// init maximum dma transfer size if not set
	//
	if ((TQP_UINT32)-1 == This->m_dwMaxDMASize)
	{
		This->m_dwMaxDMASize = QPHCI_GetMaxDMASize(&This->m_pMpegCodec->m_hci);
	}

	*phTask = TQP_NULL_HANDLE;

	ENTER_CRITICAL(&This->m_Object)

	for (i = 0; i < MAX_TASKS; i++)
	{
		if (!This->m_TaskData[i].valid)
		{
			EQP_Errors	qpStatus;

			// create firmware reply event
			//
			qpStatus = QPOSMCreateEvtgrp(&This->m_TaskData[i].m_EvtReply,
										 0
										 );
			if (!QP_SUCCESS(qpStatus))
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CTask_Alloc() QPOSMCreateEvtgrp() failed!\n"
							 ));
				break;
			}

			// get the task data
			//
			pTaskData = &This->m_TaskData[i];

			// save task type(encode, decode)
			//
			pTaskData->type = type;

			// load default settings
			//
			CTask_LoadDefaultSettings(This,
									  pTaskData
									  );

			// clear all the session information
			//
			pTaskData->m_dwSession = 0;
			for (j = 0; j < MAX_TASK_DATA_TYPE; j++)
			{
				pTaskData->pChannel[j] = QP_NULL;
			}

			// restore to default state
			//
			CTask_ResetState(This,
							 pTaskData
							 );

			// save the handle
			*phTask = i;
			pTaskData->valid = QP_TRUE;
			break;
		}
	}

	LEAVE_CRITICAL(&This->m_Object)

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTask_Alloc() handle(%d)\n",
				 *phTask
				 ));
	return (pTaskData ? QPERR_SUCCESS : QPERR_NOMEM);
}



static EQP_Errors CTask_Release(CTask *This,
								TQP_OBJECT_HANDLE hTask
								)
{
	int			i;
	PTASK_DATA	pTaskData = &This->m_TaskData[hTask];

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTask_Release() (%d)\n",
				 hTask
				 ));

	ENTER_CRITICAL(&This->m_Object)

	if (pTaskData->valid)
	{
		for (i = 0; i < MAX_TASK_DATA_TYPE; i++)
		{
			This->Close(This,
						hTask,
						(TASK_DATA_TYPE)i,
						QP_FALSE
						);
		}

		if (pTaskData->m_EvtReply)
		{
			QPOSMDeleteEvtgrp(pTaskData->m_EvtReply);
			pTaskData->m_EvtReply = QP_NULL;
		}
		pTaskData->valid = QP_FALSE;
	}

	if (hTask == This->m_taskIdPCMOut)
	{
		This->m_taskIdPCMOut = TQP_NULL_HANDLE;
	}
	else if (hTask == This->m_taskIdAESOut)
	{
		This->m_taskIdAESOut = TQP_NULL_HANDLE;
	}


	LEAVE_CRITICAL(&This->m_Object)

	return (QPERR_SUCCESS);
}



static EQP_Errors CTask_Open(CTask *This,
							 TQP_OBJECT_HANDLE hTask,
							 TASK_DATA_TYPE dataType,
							 CHANNEL_DIRECTION direction,
							 FIRMWARE_DATA_TYPE FWDataType,
							 CChannel *pChannel
							 )
{
	PTASK_DATA	pTaskData;
	CFifo		*pFifo;
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	int			i, j;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTask_Open() hTask(%d) dataType(%d) dir(%d)\n",
				 hTask,
				 dataType,
				 direction
				 ));

	if ((hTask > MAX_TASKS) ||
		(dataType > MAX_TASK_DATA_TYPE)
		)
	{
		return (QPERR_PARMS);
	}

	// get the pointer to the task data
	//
	pTaskData = &This->m_TaskData[hTask];

	ENTER_CRITICAL(&pTaskData->m_Object)

	if (TASK_DATA_TYPE_VIRTUAL != dataType)
	{
		if (!pTaskData->valid ||
			(QP_NULL != pTaskData->pChannel[dataType])
			)
		{
			LEAVE_CRITICAL(&pTaskData->m_Object)
			return (QPERR_INVALID);
		}

		// create arm request fifo
		//
		pFifo = (CFifo *)QPMMMalloc2Ex(sizeof(CFifo));
		if (pFifo)
		{
			pTaskData->pArmMsgFifo[dataType] = CFifo_Constructor(pFifo,
																 &This->m_Object,
																 OBJECT_CRITICAL_LIGHT,
																 ARM_REQ_FIFO_DEPTH + 1,
																 sizeof(TASK_ARM_REQUEST)
																 );
			if (!pTaskData->pArmMsgFifo[dataType])
			{
				LEAVE_CRITICAL(&pTaskData->m_Object)
				QPMMFree2Ex((void *)pFifo);
				return (QPERR_NOMEM);
			}
		}
		else
		{
			LEAVE_CRITICAL(&pTaskData->m_Object)
			return (QPERR_NOMEM);
		}
	}

	// reset user and arm buffer
	//
	memset((void *)&pTaskData->UserBuffer[dataType],
		   0,
		   sizeof(TASK_USER_BUFFER)
		   );
	memset((void *)&pTaskData->ArmRequest[dataType],
		   0,
		   sizeof(TASK_ARM_REQUEST)
		   );
	pTaskData->ArmRequestNumber[dataType] = 0;
	pTaskData->pBufDescToCancel[dataType] = QP_NULL;
	pTaskData->bFlushing[dataType] = QP_FALSE;
#ifdef QP_DEBUG
	if ((TASK_DATA_TYPE_COMP_VID == dataType) &&
		(TASK_TYPE_ENC == pTaskData->type)
		)
	{
		pTaskData->ArmBufferAddr = 0;
	}
#endif //QP_DEBUG

	// save the channel and direction
	//
	pTaskData->pChannel[dataType] = pChannel;
	pTaskData->direction[dataType] = direction;
	pTaskData->FWDataType[dataType] = FWDataType;

	if (0 == pTaskData->m_dwSession)
	{
		pTaskData->m_bAcquired = QP_FALSE;

		// create firmware channel if this the first session of this task
		//
		qpStatus = QPFWCODECAPI_SystemOpen(This->m_pMpegCodec,
										   (TQP_UINT32) hTask,
										   This->m_TaskData[hTask].m_codec_function.function
										   );

		// check if we are the first task open
		//
		if (QP_SUCCESS(qpStatus))
		{
			ENTER_CRITICAL(&This->m_Object)

			for (i = 0; i < MAX_TASKS; i++)
			{
				if (This->m_TaskData[i].valid &&
					0 != This->m_TaskData[i].m_dwSession
					)
				{
					break;
				}
			}

			// enable interrupt and create streaming thread
			//
			if (MAX_TASKS == i)
			{
				// clear io pending
				//
				for (j = 0; j < MAX_CHANNEL_DIRECTION; j++)
				{
					memset((void *)&This->m_ioPending[j],
						   0,
						   sizeof(TASK_IO_PENDING)
						   );
					This->m_pPending[j] = QP_NULL;
				}

				// clear task events
				//
				QPOSMClearEvtgrp(This->m_pMpegCodec->m_EvtTask,
								 QLCODEC_EVENT_MASK
								 );
				// create the streaming thread
				//
				if (!CThread_ThreadInit(&This->m_Thread))
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
								 "CTask_Open() CThread_ThreadInit() failed!\n"
								 ));
					qpStatus = QPERR_FAIL;
				}
				else
				{
					// enable interrupt
					//
					QPHCI_EnableInterrupt(&This->m_pMpegCodec->m_hci,
										  HCI_INTERRUPT_ARM | HCI_INTERRUPT_P2M | HCI_INTERRUPT_M2P
										  );
				}
			}

			LEAVE_CRITICAL(&This->m_Object)
		}
		else
		{
			if (QP_NULL != pTaskData->pArmMsgFifo[dataType])
			{
				CFifo_Destructor(pTaskData->pArmMsgFifo[dataType]);
				QPMMFree2Ex((void *)pTaskData->pArmMsgFifo[dataType]);
				pTaskData->pArmMsgFifo[dataType] = QP_NULL;
			}
		}
	}

	if (QP_SUCCESS(qpStatus))
	{
		// update session bitmap
		//
		pTaskData->m_dwSession |= (1 << dataType);
	}

	LEAVE_CRITICAL(&pTaskData->m_Object)
	return (qpStatus);
}



static EQP_Errors CTask_Close(CTask *This,
							  TQP_OBJECT_HANDLE hTask,
							  TASK_DATA_TYPE dataType,
							  TQP_BOOL bTaskCritical
							  )
{
	PTASK_DATA	pTaskData;
	int			i, j;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTask_Close() hTask(%d) dataType(%d)\n",
				 hTask,
				 dataType
				 ));

	if ((hTask > MAX_TASKS) ||
		(dataType > MAX_TASK_DATA_TYPE)
		)
	{
		return (QPERR_PARMS);
	}

	// get the pointer to the task data
	//
	pTaskData = &This->m_TaskData[hTask];

	if (!pTaskData->valid ||
		(!pTaskData->pChannel[dataType])
		)
	{
		return ((TASK_DATA_TYPE_VIRTUAL == dataType) ? QPERR_SUCCESS : QPERR_INVALID);
	}

	ENTER_CRITICAL(&pTaskData->m_Object)

	pTaskData->m_dwSession &= ~(1 << dataType);

	// complete current arm request
	//
	This->CompleteArm(This,
					  pTaskData,
					  dataType
					  );

	// complete current user request
	//
	This->CompleteUser(This,
					   pTaskData,
					   dataType
					   );

	pTaskData->pChannel[dataType] = QP_NULL;

	// flush then delete the arm request fifo
	//
	if (QP_NULL != pTaskData->pArmMsgFifo[dataType])
	{
		// flush all the arm requests
		//
		do {
			This->CompleteArm(This,
							  pTaskData,
							  dataType
							  );
		} while (CFifo_GetFifo(pTaskData->pArmMsgFifo[dataType],
							   &pTaskData->ArmRequest[dataType]
							   ));

		CFifo_Destructor(pTaskData->pArmMsgFifo[dataType]);
		QPMMFree2Ex((void *)pTaskData->pArmMsgFifo[dataType]);
		pTaskData->pArmMsgFifo[dataType] = QP_NULL;
	}

	if (0 == pTaskData->m_dwSession)
	{
		// close the firmware channel if this is the last session
		//
		QPFWCODECAPI_SystemClose(This->m_pMpegCodec,
								 (TQP_UINT32) hTask
								 );

		// check if this the last task opened
		//
		if (bTaskCritical)
		{
			ENTER_CRITICAL(&This->m_Object)
		}

		for (i = 0; i < MAX_TASKS; i++)
		{
			if (This->m_TaskData[i].valid &&
				(0 != This->m_TaskData[i].m_dwSession)
				)
			{
				break;
			}
		}

		// disable interrupt and delete streaming thread
		//
		if (MAX_TASKS == i)
		{
			// disable interrupt
			//
			QPHCI_DisableInterrupt(&This->m_pMpegCodec->m_hci,
								   HCI_INTERRUPT_ARM | HCI_INTERRUPT_P2M | HCI_INTERRUPT_M2P
								   );

			// destroy the streaming thread
			//
			CThread_ThreadDone(&This->m_Thread);

			// clear io pending
			//
			for (j = 0; j < MAX_CHANNEL_DIRECTION; j++)
			{
				memset((void *)&This->m_ioPending[j],
					   0,
					   sizeof(TASK_IO_PENDING)
					   );
				This->m_pPending[j] = QP_NULL;
			}

			// clear outstanding side band DMA request
			//
			if (This->m_pDmaRequest)
			{
				This->m_pDmaRequest = QP_NULL;
				QPOSMSetEvtgrp(This->m_EvtDamReqComplete,
							   TASK_EVENT_DMA_REQ_DONE
							   );
			}

			if ((QPCODEC_ERROR_RECOVERY_ALWAYS == This->m_pMpegCodec->m_ErrorRecovery) ||
				(CQLCodec_IsCodecError(This->m_pMpegCodec) &&
				(QPCODEC_ERROR_RECOVERY_ON_ERROR == This->m_pMpegCodec->m_ErrorRecovery))
				)
			{
				EQP_Errors	qpStatus;
#ifdef QP_EVENT_TIME_DEBUG
                TQP_UINT64 startT, endT;
                startT = QPTMGetSysTimeMilliS();
#endif
                // wait a little bit for firmware to finish previous job
                //
                DelayMilliSeconds(50);
#ifdef QP_EVENT_TIME_DEBUG
                endT = QPTMGetSysTimeMilliS();

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
                              "CTask_Close() delay time check (0x%llx 0x%llx 0x%llx)!!!\n",
                              startT, endT, (endT-startT)
                              ));
#endif

				// downloading microcode
				//
				qpStatus = CQLCodec_FWDownloadAll(This->m_pMpegCodec,
												  QP_FALSE,
												  QP_TRUE
												  );

				if (!QP_SUCCESS(qpStatus))
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
								 "CTask_Close() m_pMpgCodec->FWDownload Failed status(0x%x)!!!\n",
								 qpStatus
								 ));
				}
				else
				{
					CQLCodec_ClrCodecError(This->m_pMpegCodec);
				}

			}
		}

		if (bTaskCritical)
		{
			LEAVE_CRITICAL(&This->m_Object)
		}
	}

	LEAVE_CRITICAL(&pTaskData->m_Object)

	return (QPERR_SUCCESS);
}



static EQP_Errors CTask_Start(CTask *This,
							  TQP_OBJECT_HANDLE hTask,
							  TASK_DATA_TYPE dataType
							  )
{
	PTASK_DATA	pTaskData;
	EQP_Errors	qpStatus = QPERR_SUCCESS;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTask_Start() hTask(%d) dataType(%d)\n",
				 hTask,
				 dataType
				 ));

	if ((hTask > MAX_TASKS) ||
		(dataType > MAX_TASK_DATA_TYPE)
		)
	{
		return (QPERR_PARMS);
	}

	// get the pointer to the task data
	//
	pTaskData = &This->m_TaskData[hTask];

	if (!pTaskData->valid ||
		(!pTaskData->pChannel[dataType])
		)
	{
		return (QPERR_INVALID);
	}

	if (CTask_UseVideoInputDevice(This, pTaskData) &&
		!CTask_NeedVideoInputDevice(This)
		)
	{
		CQLCodecLib	*pQLCodecLib = GET_INHERITED_OBJECT(CQLCodecLib, m_Object, This->m_pMpegCodec->m_Object.m_pParent);
		TQP_UINT32	fps;
		TQP_UINT32	vdwidth, vdheight;

		if (pTaskData->m_inputControl.RegArtesa.in_frame_rate & IN_FRAME_RATE_ABSOLUTE)
		{
			fps = pTaskData->m_inputControl.RegArtesa.in_frame_rate & ~IN_FRAME_RATE_ABSOLUTE;
		}
		else
		{
			switch (pTaskData->m_inputControl.RegArtesa.in_frame_rate)
			{
				case IN_FRAME_RATE_PAL:
				case IN_FRAME_RATE_SECAM:
					fps = 25;
					break;
				case IN_FRAME_RATE_NTSC:
				default:
					fps = 30;
					break;
			}
		}


		// start video input device
		//
		if ((CODEC_OPEN_EX_DECODE == pTaskData->m_codec_function.functionEx.function)
			&&(SYS_LINK_VID_IN_VIU == pTaskData->m_video_input))
		{
			//for case: sensor out mjpeg stream to viu, panxing, 2011-4-14 16:25:12
			vdwidth = pTaskData->m_pictureResolution.Reg.pic_width;
			vdheight = pTaskData->m_pictureResolution.Reg.pic_height;
			if(CODEC_OPEN_EX_VID_JPEG == pTaskData->m_codec_function.functionEx.video)
			{
				if (pQLCodecLib->m_pVidDecoder->m_iVideoDecoder.SetVideoDataType)
				{
					QPVideoDecoder_SetVideoDataType(&pQLCodecLib->m_pVidDecoder->m_iVideoDecoder,
                                                    (VIDEO_DATATYPE)DATA_TYPE_JPEG
								  					);
				}
				else
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 				 "CTask_Start() The configured video decoder doesn't implement SetVideoDataType()\n"
				 				 ));
				}
			}
		}
		else
		{
			if (pQLCodecLib->m_pVidDecoder->m_iVideoDecoder.SetVideoDataType)
			{
				QPVideoDecoder_SetVideoDataType(&pQLCodecLib->m_pVidDecoder->m_iVideoDecoder,
												(VIDEO_DATATYPE)DATA_TYPE_RAW
								  				);
			}
			else
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
			 				 "CTask_Start() The configured video decoder doesn't implement SetVideoDataType()\n"
			 				 ));
			}
			vdwidth = pTaskData->m_pictureResolution.Reg.pic_width;
			vdheight = pTaskData->m_pictureResolution.Reg.pic_height;
		}

		QPVideoDecoder_SetVideoRes(&pQLCodecLib->m_pVidDecoder->m_iVideoDecoder,
								   vdwidth,
								   vdheight,
								   pTaskData->m_inputControl.RegArtesa.scan_format,
								   fps
								   );

		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
			 		 "%s(): &m_iVideoDecoder(0x%08x)\n", __FUNCTION__, &pQLCodecLib->m_pVidDecoder->m_iVideoDecoder));

		QPVideoDecoder_StartVideo(&pQLCodecLib->m_pVidDecoder->m_iVideoDecoder,
								  fps
								  );
	}

	ENTER_CRITICAL(&pTaskData->m_Object)

	if (0 == pTaskData->m_dwStarted)
	{
		// call the appropriate start function
		//
		if (TASK_TYPE_ENC == pTaskData->type)
		{
			qpStatus = CEncoderTask_Start(This,
										  hTask
										  );
		}
		else
		{
			qpStatus = CDecoderTask_Start(This,
										  hTask
										  );
		}

		pTaskData->m_Error = qpStatus;
	}

	if (QP_SUCCESS(qpStatus))
	{
		pTaskData->m_dwStarted |= (1 << dataType);
		if ((TASK_DATA_TYPE_COMP_AUD == dataType) &&
			(CHANNEL_DIR_READ == pTaskData->direction[dataType])
			)
		{
			This->m_taskIdAESOut = hTask;
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTask_Start() assign m_taskIdAESOut to hTask(%d) dataType(%d)\n",
				 hTask,
				 dataType
				 ));
		}
		else
		if ((TASK_DATA_TYPE_PCM == dataType) &&
			(CHANNEL_DIR_READ == pTaskData->direction[dataType])
			)
		{
			This->m_taskIdPCMOut = hTask;
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTask_Start() assign m_taskIdPCMOut to hTask(%d) dataType(%d)\n",
				 hTask,
				 dataType
				 ));
		}
	}

	LEAVE_CRITICAL(&pTaskData->m_Object)

	return (qpStatus);
}



static EQP_Errors CTask_Stop(CTask *This,
							 TQP_OBJECT_HANDLE hTask,
							 TASK_DATA_TYPE dataType
							 )
{
	PTASK_DATA	pTaskData;
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	TQP_BOOL	bStop = QP_FALSE;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTask_Stop() hTask(%d) dataType(%d)\n",
				 hTask,
				 dataType
				 ));

	if ((hTask > MAX_TASKS) ||
		(dataType > MAX_TASK_DATA_TYPE)
		)
	{
		return (QPERR_PARMS);
	}

	// get the pointer to the task data
	//
	pTaskData = &This->m_TaskData[hTask];

	if (!pTaskData->valid ||
		(!pTaskData->pChannel[dataType])
		)
	{
		return (QPERR_INVALID);
	}

	ENTER_CRITICAL(&pTaskData->m_Object)

	if ((0 != pTaskData->m_dwStarted) &&
		(TASK_STATE_IDLE != pTaskData->m_State)
		)
	{
		bStop = QP_TRUE;
	}

	if ((hTask == This->m_taskIdPCMOut) &&
		(TASK_DATA_TYPE_PCM == dataType)
		)
	{
		This->m_taskIdPCMOut = TQP_NULL_HANDLE;
	}
	else if ((hTask == This->m_taskIdAESOut) &&
		(TASK_DATA_TYPE_COMP_AUD == dataType)
		)
	{
		This->m_taskIdAESOut = TQP_NULL_HANDLE;
	}

	// update the bitmap
	//
	pTaskData->m_dwStarted &= ~(1 << dataType);
	LEAVE_CRITICAL(&pTaskData->m_Object)

	if (bStop)
	{
		// call the appropriate stop function
		//
		if (TASK_TYPE_ENC == pTaskData->type)
		{
			qpStatus = CEncoderTask_Stop(This,
										 hTask
										 );
		}
		else
		{
			qpStatus = CDecoderTask_Stop(This,
										 hTask
										 );
		}

		if (QPERR_SUCCESS == pTaskData->m_Error)
		{
			pTaskData->m_Error = qpStatus;
		}
	}

	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "CTask_Stop(%d) Failed! (%d) error(%d)\n",
					 hTask,
					 qpStatus,
					 pTaskData->m_Error
					 ));
	}

	if (CTask_UseVideoInputDevice(This, pTaskData) &&
		!CTask_NeedVideoInputDevice(This)
		)
	{
		CQLCodecLib	*pQLCodecLib = GET_INHERITED_OBJECT(CQLCodecLib, m_Object, This->m_pMpegCodec->m_Object.m_pParent);

		// stop video input device
		//
		QPVideoDecoder_StopVideo(&pQLCodecLib->m_pVidDecoder->m_iVideoDecoder);
	}

	return (qpStatus);
}



static EQP_Errors CTask_Acquire(CTask *This,
								TQP_OBJECT_HANDLE hTask,
								TASK_DATA_TYPE dataType
								)
{
	PTASK_DATA	pTaskData;
	EQP_Errors	qpStatus = QPERR_SUCCESS;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTask_Acquire() hTask(%d) dataType(%d)\n",
				 hTask,
				 dataType
				 ));

	if ((hTask > MAX_TASKS) ||
		(dataType > MAX_TASK_DATA_TYPE)
		)
	{
		return (QPERR_PARMS);
	}

	// get the pointer to the task data
	//
	pTaskData = &This->m_TaskData[hTask];

	if (!pTaskData->valid ||
		(!pTaskData->pChannel[dataType])
		)
	{
		return (QPERR_INVALID);
	}

	ENTER_CRITICAL(&pTaskData->m_Object)

	if (!pTaskData->m_bAcquired)
	{
		qpStatus = QPFWCODECAPI_SystemLink(This->m_pMpegCodec,
										   (TQP_UINT32)hTask,
										   pTaskData->m_video_input,
										   pTaskData->m_video_in_ch,
										   pTaskData->m_video_output,
										   pTaskData->m_video_out_ch,
										   pTaskData->m_audio_input,
										   pTaskData->m_audio_in_ch,
										   pTaskData->m_audio_output,
										   pTaskData->m_audio_out_ch
										   );

		if (QP_SUCCESS(qpStatus))
		{
			pTaskData->m_bAcquired = QP_TRUE;

		}
	}

	LEAVE_CRITICAL(&pTaskData->m_Object)

	return (qpStatus);
}



static EQP_Errors CTask_Pause(CTask *This,
							  TQP_OBJECT_HANDLE hTask,
							  TASK_DATA_TYPE dataType
							  )
{
	PTASK_DATA	pTaskData;
	EQP_Errors	qpStatus = QPERR_SUCCESS;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTask_Pause() hTask(%d) dataType(%d)\n",
				 hTask,
				 dataType
				 ));

	if ((hTask > MAX_TASKS) ||
		(dataType > MAX_TASK_DATA_TYPE)
		)
	{
		return (QPERR_PARMS);
	}

	// get the pointer to the task data
	//
	pTaskData = &This->m_TaskData[hTask];

	if (!pTaskData->valid ||
		(!pTaskData->pChannel[dataType])
		)
	{
		return (QPERR_INVALID);
	}

	ENTER_CRITICAL(&pTaskData->m_Object)

	// call the appropriate pause function
	//
	if (0 == pTaskData->m_dwPaused)
	{
		if (TASK_TYPE_ENC == pTaskData->type)
		{
			qpStatus = CEncoderTask_Pause(This,
										  hTask
										  );
		}
		else
		{
			qpStatus = CDecoderTask_Pause(This,
										  hTask
										  );
		}
	}

	if (QP_SUCCESS(qpStatus))
	{
		pTaskData->m_dwPaused |= (1 << dataType);
	}

	LEAVE_CRITICAL(&pTaskData->m_Object)

	return (qpStatus);
}



static EQP_Errors CTask_Resume(CTask *This,
							   TQP_OBJECT_HANDLE hTask,
							   TASK_DATA_TYPE dataType
							   )
{
	PTASK_DATA	pTaskData;
	EQP_Errors	qpStatus = QPERR_SUCCESS;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTask_Resume() hTask(%d) dataType(%d)\n",
				 hTask,
				 dataType
				 ));

	if ((hTask > MAX_TASKS) ||
		(dataType > MAX_TASK_DATA_TYPE)
		)
	{
		return (QPERR_PARMS);
	}

	// get the pointer to the task data
	//
	pTaskData = &This->m_TaskData[hTask];

	if (!pTaskData->valid ||
		(!pTaskData->pChannel[dataType])
		)
	{
		return (QPERR_INVALID);
	}

	ENTER_CRITICAL(&pTaskData->m_Object)

	pTaskData->m_dwPaused &= ~(1 << dataType);

	if (0 == pTaskData->m_dwPaused)
	{
		// call the appropriate resume function
		//
		if (TASK_TYPE_ENC == pTaskData->type)
		{
			qpStatus = CEncoderTask_Resume(This,
										   hTask
										   );
		}
		else
		{
			qpStatus = CDecoderTask_Resume(This,
										   hTask
										   );
		}
	}

	LEAVE_CRITICAL(&pTaskData->m_Object)

	return (qpStatus);
}



static TQP_BOOL CTask_CancelBuffer(CTask *This,
								   TQP_OBJECT_HANDLE hTask,
								   TASK_DATA_TYPE dataType,
								   PQP_BUFFER_DESCRIPTOR pBufDesc
								   )
{
	EQP_Errors	qpStatus;
	TQP_UINT32	ulEvents;
	PTASK_DATA	pTaskData;
	TQP_BOOL	bRet;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "[ CTask_CancelBuffer() hTask(%d) dataType(%d) bufDesc(0x%x)\n",
				 hTask,
				 dataType,
				 pBufDesc
				 ));

	if ((hTask > MAX_TASKS) ||
		(dataType > MAX_TASK_DATA_TYPE)
		)
	{
		return (QP_FALSE);
	}

	// get the pointer to the task data
	//
	pTaskData = &This->m_TaskData[hTask];

	if (!pTaskData->valid ||
		(!pTaskData->pChannel[dataType])
		)
	{
		return (QP_FALSE);
	}

	if (pTaskData->pBufDescToCancel[dataType])
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "CTask_CancelBuffer() (0x%x) One at a time please! ]\n",
					 pTaskData->pBufDescToCancel[dataType]
					 ));
		return (QP_FALSE);
	}

	if (!IsSafeToCall())
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "CTask_CancelBuffer() Not safe to call!\n"
					 ));
		return (QP_FALSE);
	}

	////ENTER_CRITICAL(&pTaskData->m_Object)

	// remember buffer to cancel
	//
	pTaskData->pBufDescToCancel[dataType] = pBufDesc;

	// signal thread to cancel this buffer
	//
	QPOSMClearEvtgrp(This->m_Thread.m_EvtReply,
					 QLCODEC_EVENT_CANCEL_BUFFER_DONE
					 );
	QPOSMSetEvtgrp(This->m_Thread.m_EvtWait,
				   QLCODEC_EVENT_CANCEL_BUFFER
				   );
	qpStatus = QPOSMWaitEvtgrp(This->m_Thread.m_EvtReply,
							   QLCODEC_EVENT_CANCEL_BUFFER_DONE,
							   &ulEvents,
							   2000
							   );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "  CTask_CancelBuffer() failed (0x%x) qpStatus(%d) ]\n",
					 pTaskData->pBufDescToCancel[dataType],
					 qpStatus
					 ));
	}
	else
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
					 "  CTask_CancelBuffer() (0x%x) ]\n",
					 pTaskData->pBufDescToCancel[dataType]
					 ));
	}

	////LEAVE_CRITICAL(&pTaskData->m_Object)

	bRet = (QP_NULL == pTaskData->pBufDescToCancel);

	if (pTaskData->pBufDescToCancel[dataType])
	{
		pTaskData->pBufDescToCancel[dataType] = QP_NULL;
	}

	return (bRet);
}



static TQP_BOOL CTask_NewBuffer(CTask *This,
								TQP_OBJECT_HANDLE hTask
								)
{
	if ((hTask > MAX_TASKS) ||
		!This->m_TaskData[hTask].valid
		)
	{
		return (QP_FALSE);
	}

	QPOSMSetEvtgrp(This->m_Thread.m_EvtWait,
				   QLCODEC_EVENT_NEW_BUFFER
				   );
	return (QP_TRUE);
}



static TQP_BOOL CTask_Flush(CTask *This,
							TQP_OBJECT_HANDLE hTask,
							TASK_DATA_TYPE dataType
							)
{
	EQP_Errors			qpStatus;
	TQP_UINT32			ulEvents;
	PTASK_DATA			pTaskData;
	TQP_BOOL			bRet;
	TQP_INT64			llTimeTo;
	TQP_BOOL			bWaitForIoComplete = QP_FALSE;
	CHANNEL_DIRECTION	dir;


	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTask_Flush() hTask(%d) dataType(%d)[\n",
				 hTask,
				 dataType
				 ));

	if ((hTask > MAX_TASKS) ||
		(dataType > MAX_TASK_DATA_TYPE)
		)
	{
		return (QP_FALSE);
	}

	// get the pointer to the task data
	//
	pTaskData = &This->m_TaskData[hTask];

	if (!pTaskData->valid ||
		(!pTaskData->pChannel[dataType])
		)
	{
		return (QP_FALSE);
	}

	////ENTER_CRITICAL(&pTaskData->m_Object)

	// clear the wait event and set the flush flag
	//
	QPOSMClearEvtgrp(pTaskData->m_EvtReply,
					 QLCODEC_EVENT_FLUSH_DONE
					 );
	pTaskData->bFlushing[dataType] = QP_TRUE;

	// wait for the outstanding buffer to complete
	//
	dir = pTaskData->direction[dataType];
	ENTER_CRITICAL_IO_PENDING(This)
	bWaitForIoComplete = This->m_pPending[dir] && (This->m_pPending[dir]->pTaskData == pTaskData);
	LEAVE_CRITICAL_IO_PENDING(This)

	if (bWaitForIoComplete)
	{
		llTimeTo = 1000;

		BeginTimer(&llTimeTo);

		while (This->m_pPending[dir])
		{
			if (Timeout(llTimeTo,
						5
						))
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CTask_Flush() wait for DMA compelte timeout!!!\n"
							 ));
				pTaskData->bFlushing[dataType] = QP_FALSE;
				return (QP_FALSE);
			}
		}
	}

	// signal the streaming task to complete both ARM and USER requests
	//
	QPOSMSetEvtgrp(This->m_Thread.m_EvtWait,
				   QLCODEC_EVENT_FLUSH
				   );
	qpStatus = QPOSMWaitEvtgrp(pTaskData->m_EvtReply,
							   QLCODEC_EVENT_FLUSH_DONE,
							   &ulEvents,
							   2000
							   );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "  CTask_Flush() failed! flushing(%d) qpStatus(%d) ]\n",
					 pTaskData->bFlushing[dataType],
					 qpStatus
					 ));
	}
	else
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
					 "  CTask_Flush() flushing(%d) ]\n",
					 pTaskData->bFlushing[dataType]
					 ));
	}

	////LEAVE_CRITICAL(&pTaskData->m_Object)

	bRet = !pTaskData->bFlushing[dataType];

	if (pTaskData->bFlushing[dataType])
		pTaskData->bFlushing[dataType] = QP_FALSE;

	return (bRet);
}



static TQP_BOOL CTask_FlushArm(CTask *This)
{
	EQP_Errors	qpStatus;
	TQP_UINT32	ulEvents;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTask_FlushArm()\n"
				 ));

	ENTER_CRITICAL(&This->m_Object)

	QPOSMClearEvtgrp(This->m_Thread.m_EvtReply,
					 QLCODEC_EVENT_FLUSH_ARM_DONE
					 );
	QPOSMSetEvtgrp(This->m_Thread.m_EvtWait,
				   QLCODEC_EVENT_FLUSH_ARM
				   );
	qpStatus = QPOSMWaitEvtgrp(This->m_Thread.m_EvtReply,
							   QLCODEC_EVENT_FLUSH_ARM_DONE,
							   &ulEvents,
							   2000
							   );
	LEAVE_CRITICAL(&This->m_Object)

	return (QP_SUCCESS(qpStatus));
}



// side band DMA request
//
static EQP_Errors CTask_DMARequest(CTask *This,
								   TQP_UINT32 ulArmAddr,	// in WORD
								   PTQP_UINT8 pHostAddr,
								   TQP_UINT32 ulLength,		// in byte
								   CHANNEL_DIRECTION dir,
								   TQP_BOOL bSwap
								   )
{
	EQP_Errors	qpStatus;
	TQP_UINT32	ulEvents;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "CTask_DMARequest() ulArmAddr(0x%x), pHostAddr(0x%x) ulLength(%d) dir(%d) bSwap(%d)[\n",
				 ulArmAddr,
				 pHostAddr,
				 ulLength,
				 dir,
				 bSwap
				 ));

	if (0 == ulLength)
	{
		return (QPERR_SUCCESS);
	}

	////ENTER_CRITICAL(&This->m_Object)

	if (This->m_pDmaRequest)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CTask_DMARequest() arm(0x%x) dir(%d) len(0x%d) one at a time please!\n",
					 This->m_pDmaRequest->ulArmAddr,
					 This->m_pDmaRequest->dir,
					 This->m_pDmaRequest->ulLength
					 ));
		////LEAVE_CRITICAL(&This->m_Object)
		return (QPERR_INVALID);
	}

	QPOSMClearEvtgrp(This->m_EvtDamReqComplete,
					 TASK_EVENT_DMA_REQ_DONE
					 );

	This->m_dmaRequest.ulArmAddr = ulArmAddr;
	This->m_dmaRequest.pBuffer = pHostAddr;
	This->m_dmaRequest.ulLength = ulLength;
	This->m_dmaRequest.dir = dir;
	This->m_dmaRequest.bSwap = bSwap;
	This->m_dmaRequest.ulOffset = 0;

	This->m_pDmaRequest = &This->m_dmaRequest;

	QPOSMSetEvtgrp(This->m_Thread.m_EvtWait,
				   QLCODEC_EVENT_NEW_BUFFER
				   );

	qpStatus = QPOSMWaitEvtgrp(This->m_EvtDamReqComplete,
							   TASK_EVENT_DMA_REQ_DONE,
							   &ulEvents,
							   2000
							   );
	if (QP_SUCCESS(qpStatus))
	{
		This->m_pDmaRequest = QP_NULL;
	}
	else
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CTask_DMARequest() failed! arm(0x%x) dir(%d) len(0x%d) qpStatus(%d)\n",
					 This->m_pDmaRequest->ulArmAddr,
					 This->m_pDmaRequest->dir,
					 This->m_pDmaRequest->ulLength,
					 qpStatus
					 ));
	}
	////LEAVE_CRITICAL(&This->m_Object)

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "CTask_DMARequest()]\n"
				 ));
	return (qpStatus);
}



// This function is called after host DMA complete
//
static void CTask_CompleteData(CTask *This,
							   CHANNEL_DIRECTION dir
							   )
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, /*QPDBGM_LEVEL_WARN*/QPDBGM_LEVEL_TRACE),
				 "CTask_CompleteData() dir(%d) pending(0x%x)\n",
				 dir,
				 This->m_pPending[dir]
				 ));

	if (This->m_pPending[dir])
	{
		TQP_UINT32			transferSize = This->m_pPending[dir]->ulLength;
		TASK_DATA_TYPE		dataType = This->m_pPending[dir]->dataType;
		PTASK_DATA			pTaskData = This->m_pPending[dir]->pTaskData;
		PTASK_DMA_REQUEST	pDmaRequest = This->m_pDmaRequest;

		if ((QP_NULL == pTaskData) &&
			(QP_NULL == pDmaRequest)
			)
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 ">>>>  CTask_CompleteData() dir(%d) pending(0x%x) pTaskData(0x%x) m_pDmaRequest(0x%x) dataType(%d) <<<<\n",
						 dir,
						 This->m_pPending[dir],
						 pTaskData,
						 pDmaRequest,
						 dataType
						 ));
			return;
		}

		if (transferSize >= sizeof(TQP_UINT32))
		{
			if (CHANNEL_DIR_READ == dir)
			{
				QPHCI_DMAReadDone(&This->m_pMpegCodec->m_hci);
				QPHCI_CopyFromCommonBuffer(&This->m_pMpegCodec->m_hci,
										   This->m_pPending[dir]->pHostAddress,
										   This->m_pPending[dir]->ulLength
										   );
			}
			else
			{
				QPHCI_DMAWriteDone(&This->m_pMpegCodec->m_hci);
			}
		}

		if (pTaskData)
		{
			// data streaming DMA completion
			//
			if (ARM_BUF_OTHERS == pTaskData->ArmRequest[dataType].ArmBuffer.type)
			{
				pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulOffset += transferSize;
				pTaskData->UserBuffer[dataType].pBufDesc->ulBufferOffset += transferSize;
				pTaskData->UserBuffer[dataType].pBufDesc->ulTotalUsed += transferSize;
			}
			else if (ARM_BUF_YUV == pTaskData->ArmRequest[dataType].ArmBuffer.type)
			{
				if (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulYOffset < pTaskData->UserBuffer[dataType].BUFFER.YUV.ulYLength)
					pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulYOffset += transferSize;
				else
					pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulUVOffset += transferSize;
				pTaskData->UserBuffer[dataType].pBufDesc->ulBufferOffset += transferSize;
				pTaskData->UserBuffer[dataType].pBufDesc->ulTotalUsed += transferSize;
			}
			else if (ARM_BUF_YUVMB2RAS == pTaskData->ArmRequest[dataType].ArmBuffer.type)
			{
				if (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulYOffset < pTaskData->UserBuffer[dataType].BUFFER.YUVRAS.ulYLength)
					pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulYOffset += transferSize;
				else if (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulUOffset < pTaskData->UserBuffer[dataType].BUFFER.YUVRAS.ulULength)
					pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulUOffset += transferSize;
				else
					pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulVOffset += transferSize;
				pTaskData->UserBuffer[dataType].pBufDesc->ulBufferOffset += transferSize;
				pTaskData->UserBuffer[dataType].pBufDesc->ulTotalUsed += transferSize;
			}
			else if (ARM_BUF_YUVRAS == pTaskData->ArmRequest[dataType].ArmBuffer.type)
			{
				if (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulYOffset < pTaskData->UserBuffer[dataType].BUFFER.YUVRAS.ulYLength)
					pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulYOffset += transferSize;
				else if (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulUOffset < pTaskData->UserBuffer[dataType].BUFFER.YUVRAS.ulULength)
					pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulUOffset += transferSize;
				else
					pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulVOffset += transferSize;
				pTaskData->UserBuffer[dataType].pBufDesc->ulBufferOffset += transferSize;
				pTaskData->UserBuffer[dataType].pBufDesc->ulTotalUsed += transferSize;
			}
			else
			{
				// how do we get here?
				//
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CTask_CompleteData() Unknow Arm buffer type(%d)\n",
							 pTaskData->ArmRequest[dataType].ArmBuffer.type
							 ));
			}

		}
		else if (pDmaRequest)
		{
			// side band DMA completion
			//
			pDmaRequest->ulOffset += transferSize;
		}

		This->ProcessIoComplete(This,
								dir
								);

	}
}



static TQP_BOOL CTask_CompleteUser(CTask *This,
								   PTASK_DATA pTaskData,
								   TASK_DATA_TYPE dataType
								   )
{
	PQP_BUFFER_DESCRIPTOR	pBufDesc = pTaskData->UserBuffer[dataType].pBufDesc;
	CChannel				*pChannel = pTaskData->pChannel[dataType];

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTask_CompleteUser() (0x%x)\n",
				 pBufDesc
				 ));

	if (pBufDesc)
	{
		// check if this buffer is io pending
		//
		if (This->m_pPending[pTaskData->direction[dataType]] &&
			(This->m_pPending[pTaskData->direction[dataType]]->pTaskData == pTaskData)
			)
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
						 "CTask_CompleteUser() IOPending\n"
						 ));
			return (QP_FALSE);
		}
		else
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_EXTRACE),
						 "CTask_CompleteUser() ulBufferOffset(%d) ulBufferSize(%d)\n",
						 pBufDesc->ulBufferOffset,
						 pBufDesc->ulBufferSize
						 ));
		}

		// clear QP_BUFDESC_FLAG_PTS
		if ((CHANNEL_DIR_WRITE == pTaskData->direction[dataType]) &&
			(pTaskData->UserBuffer[dataType].pBufDesc->ulFlags & QP_BUFDESC_FLAG_PTS)
			)
		{
			pTaskData->UserBuffer[dataType].pBufDesc->ulFlags &= ~QP_BUFDESC_FLAG_PTS;
		}

		// remember return status
		//
		////pBufDesc->Status = pTaskData->UserBuffer[dataType].status;
		pBufDesc->Status = QPERR_SUCCESS;

		// clear the user portion of the task data
		//
		memset((void *)&pTaskData->UserBuffer[dataType],
			   0,
			   sizeof(TASK_USER_BUFFER)
			   );

		// return this buffer
		//
		pChannel->CompleteBuffer(pChannel,
								 pBufDesc
								 );
		return (QP_TRUE);
	}

	return (QP_FALSE);
}



static void CTask_CompleteArm(CTask *This,
							  PTASK_DATA pTaskData,
							  TASK_DATA_TYPE dataType
							  )
{
	EQP_Errors	qpStatus;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTask_CompleteArm() cmd(0x%x) status(0x%x) reqid(%d)\n",
				 pTaskData->ArmRequest[dataType].HostMsg.Read,
				 pTaskData->ArmRequest[dataType].HostMsg_status.Read,
				 pTaskData->ArmRequest[dataType].reqId
				 ));

	if (QP_NULL != pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.ALL.valid)
	{
		HOST_MESSAGE		HostMsg;
		HOST_MESSAGE_STATUS	HostMsg_status;
		TQP_UINT32			ulDataType;
		TQP_UINT32			ulArmOffset;
		TQP_UINT32			ulPTS;
		TQP_UINT32			ulPTSValid;
		TQP_UINT32			ulLast;
		TQP_UINT32			ulFrameFlags = 0;
		TQP_UINT32			reqId;
		TQP_UINT32			ulFwData;

		if (ARM_BUF_OTHERS == pTaskData->ArmRequest[dataType].ArmBuffer.type)
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE/*QPDBGM_LEVEL_WARN*/),
						 "CTask_CompleteArm() type(0x%x) addr(0x%x) size(%d) offset(%d) PTS(%d) valid(%d) last(%d) frameFlags(0x%x)\n",
						 pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulDataType,
						 pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulAddr,
						 pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulSize << 2,
						 pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulOffset,
						 pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulPTS,
						 pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulPTSValid,
						 pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulLast,
						 pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulFrameFlags
						 ));

			ulDataType = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulDataType;
			ulArmOffset = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulOffset >> 2;
			ulPTS = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulPTS;
			ulPTSValid = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulPTSValid;
			ulLast = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulLast;
			ulFrameFlags = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulFrameFlags;
			ulFwData = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.OTHERS.ulFwData;
		}
		else if (ARM_BUF_YUV == pTaskData->ArmRequest[dataType].ArmBuffer.type)
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE/*QPDBGM_LEVEL_WARN*/),
						 "CTask_CompleteArm() type(0x%x) Y Addr(0x%x) UV Addr(0x%x) PTS(%d) valid(%d) last(%d)\n",
						 pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulDataType,
						 pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulYAddr,
						 pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulUVAddr,
						 pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulPTS,
						 pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulPTSValid,
						 pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulLast
						 ));

			ulDataType = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulDataType;
			ulArmOffset = (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulYOffset +
						  pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulUVOffset) >> 2
						  ;
			ulPTS = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulPTS;
			ulPTSValid = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulPTSValid;
			ulLast = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulLast;
			ulFwData = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUV.ulFwData;
		}
		else if (ARM_BUF_YUVMB2RAS == pTaskData->ArmRequest[dataType].ArmBuffer.type)
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE/*QPDBGM_LEVEL_WARN*/),
						 "CTask_CompleteArm() type(0x%x) Y Addr(0x%x) UV Addr(0x%x) PTS(%d) valid(%d) last(%d)\n",
						 pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulDataType,
						 pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulYAddr,
						 pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulUVAddr,
						 pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulPTS,
						 pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulPTSValid,
						 pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulLast
						 ));

			ulDataType = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulDataType;
			ulArmOffset = (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulYOffset +
						  pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulUOffset +
						  pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulVOffset
						  ) >> 2
						  ;
			ulPTS = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulPTS;
			ulPTSValid = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulPTSValid;
			ulLast = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulLast;
			ulFwData = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVMB2RAS.ulFwData;
		}
		else
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE/*QPDBGM_LEVEL_WARN*/),
						 "CTask_CompleteArm() type(0x%x) Y Addr(0x%x) U Addr(0x%x) V Addr(0x%x) PTS(%d) valid(%d) last(%d)\n",
						 pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulDataType,
						 pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulYAddr,
						 pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulUAddr,
						 pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulVAddr,
						 pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulPTS,
						 pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulPTSValid,
						 pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulLast
						 ));

			ulDataType = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulDataType;
			ulArmOffset = (pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulYOffset +
						  pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulUOffset +
						  pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulVOffset
						  ) >> 2
						  ;
			ulPTS = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulPTS;
			ulPTSValid = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulPTSValid;
			ulLast = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulLast;
			ulFwData = pTaskData->ArmRequest[dataType].ArmBuffer.BUFFER.YUVRAS.ulFwData;
		}

		// remember the arm message for later use in this function
		//
		HostMsg.Read = pTaskData->ArmRequest[dataType].HostMsg.Read;
		HostMsg_status.Read = pTaskData->ArmRequest[dataType].HostMsg_status.Read;
		reqId = pTaskData->ArmRequest[dataType].reqId;

		// clear the arm portion of the task data
		//
		memset((void *)&pTaskData->ArmRequest[dataType],
			   0,
			   sizeof(TASK_ARM_REQUEST)
			   );

		// wait for the mailbox
		//
		qpStatus = QPFWAPI_MailboxReady(This->m_pMpegCodec,
				     					1000
				     					);

		if (QP_SUCCESS(qpStatus))
		{
			ARM_MESSAGE	msg;
#ifdef REDUCE_HCI_MESSAGES
			ARM_MESSAGE_STATUS	status;
			TQP_UINT32      	reg[13];
#endif

			msg.Read = 0;
			msg.RegArtesa.channel_number = HostMsg.RegArtesa.channel_number;

#ifdef REDUCE_HCI_MESSAGES
			status.Read = 0;
			status.RegArtesa.channel_number = HostMsg_status.RegArtesa.channel_number;
			status.Reg.rply = 0;
			status.Reg.status = 1;
#endif

			switch (HostMsg.Reg.command)
			{
				case EncDataOutReq:
				case EncDataOutLastReq:

#ifdef QP_LINXKER
	  				PROFILING_LOG(EVENT_FW_D_OUT_REQ_ACK, HostMsg.RegArtesa.channel_number,  pTaskData->direction[dataType]/*dataType*/,
	  				       0, pTaskData->ArmRequestNumber[dataType] /*HostMsg.Reg.command*/, ulArmOffset*4);
#endif
					msg.Reg.command = EncDataXferDone;
#ifndef REDUCE_HCI_MESSAGES
					QPHCI_RegisterWrite(&This->m_pMpegCodec->m_hci,
										REG_TO_ARM_1,
										ulDataType
										);
					QPHCI_RegisterWrite(&This->m_pMpegCodec->m_hci,
										REG_TO_ARM_2,
										ulArmOffset
										);
					QPHCI_RegisterWrite(&This->m_pMpegCodec->m_hci,
										REG_TO_ARM_3,
										ulPTS
										);
					QPHCI_RegisterWrite(&This->m_pMpegCodec->m_hci,
										REG_TO_ARM_4,
										ulPTSValid
										);
					QPHCI_RegisterWrite(&This->m_pMpegCodec->m_hci,
										REG_TO_ARM_6,
										ulFwData
										);
#else
					reg[12] = msg.Read;
					reg[11] = ulDataType;
					reg[10] = ulArmOffset;
					reg[9] = ulPTS;
					reg[8] = ulPTSValid;
					reg[7] = 0;
					reg[6] = ulFwData;
					reg[5] = 0;
					reg[4] = 0;
					reg[3] = 0;
					reg[2] = 0;
					reg[1] = 0;
					reg[0] = status.Read;

					QPHCI_RegisterWriteEx(&This->m_pMpegCodec->m_hci,
										REG_TO_ARM_MESSAGE_STATUS,
										reg,
										13
										);
#endif
					break;

				case DecDataXferReq:
#ifdef QP_LINXKER
	  				PROFILING_LOG(EVENT_FW_D_OUT_REQ_ACK, HostMsg.RegArtesa.channel_number, pTaskData->direction[dataType]/*dataType*/,
                                0, pTaskData->ArmRequestNumber[dataType] /*HostMsg.Reg.command*/, ulArmOffset*4);
#endif
					msg.Reg.command = ulLast ? DecLastDataXferDone : DecDataXferDone;
#ifndef REDUCE_HCI_MESSAGES
					QPHCI_RegisterWrite(&This->m_pMpegCodec->m_hci,
										DEC_REG_PARAM_1,
										ulDataType
										);
					QPHCI_RegisterWrite(&This->m_pMpegCodec->m_hci,
										DEC_REG_PARAM_2,
										ulArmOffset
										);
					QPHCI_RegisterWrite(&This->m_pMpegCodec->m_hci,
										DEC_REG_PARAM_3,
										ulPTS
										);
					QPHCI_RegisterWrite(&This->m_pMpegCodec->m_hci,
										DEC_REG_PARAM_4,
										ulPTSValid
										);
					QPHCI_RegisterWrite(&This->m_pMpegCodec->m_hci,
										DEC_REG_PARAM_5,
										ulFrameFlags
										);
					QPHCI_RegisterWrite(&This->m_pMpegCodec->m_hci,
										REG_TO_ARM_6,
										ulFwData
										);
#else
					reg[12] = msg.Read;
					reg[11] = ulDataType;
					reg[10] = ulArmOffset;
					reg[9] = ulPTS;
					reg[8] = ulPTSValid;
					reg[7] = ulFrameFlags;
					reg[6] = ulFwData;
					reg[5] = 0;
					reg[4] = 0;
					reg[3] = 0;
					reg[2] = 0;
					reg[1] = 0;
					reg[0] = status.Read;

					QPHCI_RegisterWriteEx(&This->m_pMpegCodec->m_hci,
										REG_TO_ARM_MESSAGE_STATUS,
										reg,
										13
										);
#endif
					break;

				default:
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
								 "CTask_CompleteArm() Unknown toHostCmd(0x%x) type(0x%x):#(%d) size(0x%x)!!!\n",
								 HostMsg.Reg.command,
								 ulDataType,
								 reqId,
								 ulArmOffset
								 ));

					QPFWAPI_MailboxDone(This->m_pMpegCodec);
					return;
			}

			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, /*QPDBGM_LEVEL_TRACE*/QPDBGM_LEVEL_WARN),
						 "CTask_CompleteArm() toHostCmd(0x%x) msg_id(%d) toArmCmd(0x%x) channel(%d) type(0x%x):#(%d) size(0x%x) ulFwData(%d)\n",
						 HostMsg.Reg.command,
						 HostMsg_status.RegArtesa.channel_number,
						 msg.Reg.command,
						 msg.RegArtesa.channel_number,
						 ulDataType,
						 reqId,
						 ulArmOffset,
						 ulFwData
						 ));

#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
#if ENABLE_CODEC_MEM_LOG
			codec_mem_log_add_ex(CODEC_MEM_LOG_COMPLETE_ARM, reqId, ulArmOffset*4);
#endif
#endif

#ifndef REDUCE_HCI_MESSAGES
			QPFWAPI_SendMessageToARM(This->m_pMpegCodec,
									 pTaskData->id,
									 &msg,
									 QP_FALSE,
									 &HostMsg_status,
									 0
									 );
#endif

			QPFWAPI_MailboxDone(This->m_pMpegCodec);
		}
		else
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CTask_CompleteArm() Mailbox BUSY!!!!\n"
						 ));
		}
	}
}



static void CTask_ProcessArmMessage(CTask *This)
{
	HOST_MESSAGE		HostMsg;
	HOST_MESSAGE_STATUS	HostMsg_status;
	TQP_UINT32			param1, param2, param3, param4, param5;
	EQP_Errors			qpStatus;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTask_ProcessArmMessage()\n"
				 ));

	qpStatus = QPFWAPI_GetARMMessage(This->m_pMpegCodec,
									 &HostMsg,
									 &HostMsg_status,
									 &param1,
									 &param2,
									 &param3,
									 &param4,
									 &param5
									 );

	// handle the message
	//
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID,  QPDBGM_LEVEL_WARN),
					 "CTask_ProcessArmMessage() WaitForARMMessage() Failed status(%d)!!\n",
					 qpStatus
					 ));
	}
	else
	{

#ifdef QP_LINXKER
		PROFILING_LOG(EVENT_FW_FROM_ARM_MSG, HostMsg.RegArtesa.channel_number, 0, 0, 0, HostMsg.Reg.command);
#endif

		if (HostMsg.Reg.command <= EncMsgEnd)
		{
			CEncoderTask_ProcessArmMessage(This,
										   &HostMsg,
										   &HostMsg_status,
										   param1,
										   param2,
										   param3,
										   param4,	// QL201B
										   param5	// QL201B
										   );
		}
		else
		{
			CDecoderTask_ProcessArmMessage(This,
										   &HostMsg,
										   &HostMsg_status,
										   param1,
										   param2,
										   param3,
										   param4,	// QL201B
										   param5	// QL201B
										   );
		}
	}

}



static void CTask_ProcessIoComplete(CTask *This,
									CHANNEL_DIRECTION dir
									)
{
	PTASK_DATA		pTaskData = QP_NULL;
	TASK_DATA_TYPE	dataType = (TASK_DATA_TYPE)0;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTask_ProcessIoComplete() %s\n",
				 (CHANNEL_DIR_READ == dir) ? "read" : "write"
				 ));

	ENTER_CRITICAL_IO_PENDING(This)

	if (This->m_pPending[dir])
	{
		pTaskData = This->m_pPending[dir]->pTaskData;
		dataType = This->m_pPending[dir]->dataType;

		This->m_pPending[dir] = QP_NULL;
		memset((void *)&This->m_ioPending[dir],
			   0,
			   sizeof(TASK_IO_PENDING)
			   );
	}

	LEAVE_CRITICAL_IO_PENDING(This)

	if (pTaskData)
	{
		if (TASK_TYPE_ENC == pTaskData->type)
		{
			CEncoderTask_ProcessIoComplete(This,
										   pTaskData,
										   dataType,
										   QP_TRUE
										   );
		}
		else
		{
			CDecoderTask_ProcessIoComplete(This,
										   pTaskData,
										   dataType,
										   QP_TRUE
										   );
		}
	}
	else if (This->m_pDmaRequest)
	{
		if (This->m_pDmaRequest->ulOffset == This->m_pDmaRequest->ulLength)
		{
			This->m_pDmaRequest = QP_NULL;
			QPOSMSetEvtgrp(This->m_EvtDamReqComplete,
						   TASK_EVENT_DMA_REQ_DONE
						   );
		}

		This->ProcessDataStreaming(This);
	}
}



static void CTask_ProcessDataStreaming(CTask *This)
{
	int			i, j;
	int			curTaskId;
	PTASK_DATA	pTaskData;
	int			curDataId;
	CChannel	*pChannel;
	TQP_INT32	StartId, DataId;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN/*QPDBGM_LEVEL_TRACE*/),
				 "CTask_ProcessDataStreaming() rd_ready(%d) wr_ready(%d) start(%d)[\n",
				 CTask_isIOReady(This, CHANNEL_DIR_READ),
				 CTask_isIOReady(This, CHANNEL_DIR_WRITE),
				 This->m_StartID
				 ));

	// high priority arm requests
	//
	if (TQP_NULL_HANDLE != This->m_taskIdAESOut)
	{
		StartId = (TQP_INT32) This->m_taskIdAESOut;
		pTaskData = &This->m_TaskData[StartId];
		if (pTaskData->valid)
		{
			ENTER_CRITICAL(&pTaskData->m_Object)

			if (TASK_STATE_IDLE != pTaskData->m_State)
			{
				curDataId =  TASK_DATA_TYPE_COMP_AUD;
				pChannel = pTaskData->pChannel[curDataId];
				if (pChannel &&
					pTaskData->pArmMsgFifo[curDataId] &&
					CTask_isIOReady(This, pTaskData->direction[curDataId])
					)
				{
					if (QP_NULL == pTaskData->ArmRequest[curDataId].ArmBuffer.BUFFER.ALL.valid)
					{
						CFifo_GetFifo(pTaskData->pArmMsgFifo[curDataId],
									  &pTaskData->ArmRequest[curDataId]
									  );
					}

					if ((QP_NULL != pTaskData->ArmRequest[curDataId].ArmBuffer.BUFFER.ALL.valid) &&
						!pTaskData->bFlushing[curDataId]
						)
					{
						if (QP_NULL == pTaskData->UserBuffer[curDataId].pBufDesc)
						{
							if (!pChannel->GetBuffer(pChannel,
													 &pTaskData->UserBuffer[curDataId].pBufDesc,
													 &pTaskData->UserBuffer[curDataId].BUFFER.OTHERS.pBuffer,
													 &pTaskData->UserBuffer[curDataId].BUFFER.OTHERS.ulLength
													 ))
							{
								QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
											 "CTask_ProcessDataStreaming() no buffer available on task(%d) data(%d)\n",
											 StartId,
											 curDataId
											 ));
							}
						}

						if (pTaskData->UserBuffer[curDataId].pBufDesc)
						{
							// if the arm buffer carry PTS, pass it on
							//
							if ((CHANNEL_DIR_READ == pTaskData->direction[curDataId]) &&
								(pTaskData->ArmRequest[curDataId].ArmBuffer.BUFFER.ALL.ulPTSValid) &&
								(!(pTaskData->UserBuffer[curDataId].pBufDesc->ulFlags & QP_BUFDESC_FLAG_PTS))
								)
							{
									pTaskData->ArmRequest[curDataId].ArmBuffer.BUFFER.ALL.ulPTSValid = QP_TRUE;
									pTaskData->ArmRequest[curDataId].ArmBuffer.BUFFER.ALL.ulPTS = (TQP_UINT32)(pTaskData->UserBuffer[curDataId].pBufDesc->ulPTS >> 2);
									// Sometimes the transmission will be seperated into multiple parts,
                                                                        // so move the clear behavior to CompleteUser().
                                                                        //
									//pTaskData->UserBuffer[curDataId].pBufDesc->ulFlags &= ~QP_BUFDESC_FLAG_PTS;
							}

							// if the arm buffer carry PTS, pass it on
							//
							if ((CHANNEL_DIR_READ == pTaskData->direction[curDataId]) &&
								(pTaskData->ArmRequest[curDataId].ArmBuffer.BUFFER.ALL.ulPTSValid) &&
								(!(pTaskData->UserBuffer[curDataId].pBufDesc->ulFlags & QP_BUFDESC_FLAG_PTS))
								)
							{
								pTaskData->UserBuffer[curDataId].pBufDesc->ulPTS = pTaskData->ArmRequest[curDataId].ArmBuffer.BUFFER.ALL.ulPTS << 2;
								pTaskData->UserBuffer[curDataId].pBufDesc->ulFlags |= QP_BUFDESC_FLAG_PTS;
								pTaskData->ArmRequest[curDataId].ArmBuffer.BUFFER.ALL.ulPTSValid = QP_FALSE;
							}
							if (pTaskData->ArmRequest[curDataId].ArmBuffer.BUFFER.OTHERS.ulWrapAround)
							{
								pTaskData->UserBuffer[curDataId].pBufDesc->ulFlags |= QP_BUFDESC_FLAG_INCOMPLETE_FRAME;
#ifdef QP_LINXKER
                                PROFILING_LOG(EVENT_FW_D_OUT_INCOMPLETE_FRAME, pTaskData->id, pTaskData->direction[curDataId]/*dataType*/,
                                    0, pTaskData->ArmRequestNumber[curDataId] /*HostMsg.Reg.command*/, 0);
#endif
							}
							CTask_BuildIoBlock(This,
											   pTaskData,
											   (TASK_DATA_TYPE)curDataId
											   );
							CTask_SendIoBlock(This,
									  pTaskData,
									  (TASK_DATA_TYPE)curDataId
									  );
							if (!CTask_isIOReady(This, CHANNEL_DIR_READ) &&
								!CTask_isIOReady(This, CHANNEL_DIR_WRITE)
								)
							{
								LEAVE_CRITICAL(&pTaskData->m_Object)

								QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN/*QPDBGM_LEVEL_TRACE*/),
											 "CTask_ProcessDataStreaming ]()\n"
											 ));
								return;
							}
						}
					}
				}
			}

			LEAVE_CRITICAL(&pTaskData->m_Object)
		}
	}
	else if (TQP_NULL_HANDLE != This->m_taskIdPCMOut)
	{
		StartId = (TQP_INT32) This->m_taskIdPCMOut;
		pTaskData = &This->m_TaskData[StartId];
		if (pTaskData->valid)
		{
			ENTER_CRITICAL(&pTaskData->m_Object)

			if (TASK_STATE_IDLE != pTaskData->m_State)
			{
				pChannel = pTaskData->pChannel[TASK_DATA_TYPE_PCM];
				if (pChannel &&
					pTaskData->pArmMsgFifo[TASK_DATA_TYPE_PCM] &&
					CTask_isIOReady(This, pTaskData->direction[TASK_DATA_TYPE_PCM])
					)
				{
					if (QP_NULL == pTaskData->ArmRequest[TASK_DATA_TYPE_PCM].ArmBuffer.BUFFER.ALL.valid)
					{
						CFifo_GetFifo(pTaskData->pArmMsgFifo[TASK_DATA_TYPE_PCM],
									  &pTaskData->ArmRequest[TASK_DATA_TYPE_PCM]
									  );
					}

					if ((QP_NULL != pTaskData->ArmRequest[TASK_DATA_TYPE_PCM].ArmBuffer.BUFFER.ALL.valid) &&
						!pTaskData->bFlushing[TASK_DATA_TYPE_PCM]
						)
					{
						if (QP_NULL == pTaskData->UserBuffer[TASK_DATA_TYPE_PCM].pBufDesc)
						{
							if (!pChannel->GetBuffer(pChannel,
													 &pTaskData->UserBuffer[TASK_DATA_TYPE_PCM].pBufDesc,
													 &pTaskData->UserBuffer[TASK_DATA_TYPE_PCM].BUFFER.OTHERS.pBuffer,
													 &pTaskData->UserBuffer[TASK_DATA_TYPE_PCM].BUFFER.OTHERS.ulLength
													 ))
							{
								QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
											 "CTask_ProcessDataStreaming() no buffer available on task(%d) data(%d)\n",
											 StartId,
											 TASK_DATA_TYPE_PCM
											 ));
							}
						}

						if (pTaskData->UserBuffer[TASK_DATA_TYPE_PCM].pBufDesc)
						{
							// if the arm buffer carry PTS, pass it on
							//
							if ((CHANNEL_DIR_READ == pTaskData->direction[TASK_DATA_TYPE_PCM]) &&
								(pTaskData->ArmRequest[TASK_DATA_TYPE_PCM].ArmBuffer.BUFFER.ALL.ulPTSValid) &&
								(!(pTaskData->UserBuffer[TASK_DATA_TYPE_PCM].pBufDesc->ulFlags & QP_BUFDESC_FLAG_PTS))
								)
							{
								pTaskData->UserBuffer[TASK_DATA_TYPE_PCM].pBufDesc->ulPTS = pTaskData->ArmRequest[TASK_DATA_TYPE_PCM].ArmBuffer.BUFFER.ALL.ulPTS << 2;
								pTaskData->UserBuffer[TASK_DATA_TYPE_PCM].pBufDesc->ulFlags |= QP_BUFDESC_FLAG_PTS;
								pTaskData->ArmRequest[TASK_DATA_TYPE_PCM].ArmBuffer.BUFFER.ALL.ulPTSValid = QP_FALSE;
							}

							CTask_BuildIoBlock(This,
											   pTaskData,
											   TASK_DATA_TYPE_PCM
											   );
							CTask_SendIoBlock(This,
											  pTaskData,
											  TASK_DATA_TYPE_PCM
											  );
							if (!CTask_isIOReady(This, CHANNEL_DIR_READ) &&
								!CTask_isIOReady(This, CHANNEL_DIR_WRITE)
								)
							{
								LEAVE_CRITICAL(&pTaskData->m_Object)

								QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN/*QPDBGM_LEVEL_TRACE*/),
											 "CTask_ProcessDataStreaming ]()\n"
											 ));
								return;
							}
						}
					}
				}
			}

			LEAVE_CRITICAL(&pTaskData->m_Object)
		}
	}
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN/*QPDBGM_LEVEL_TRACE*/), "CTask_ProcessDataStreaming ]() start side band DMA request\n"));

	// side band DMA request
	//
	if (This->m_pDmaRequest)
	{
		if (CTask_BuildIoBlockSideBandDMA(This,
										  This->m_pDmaRequest
										  ))
		{
			CTask_SendIoBlockSideBandDMA(This,
										 This->m_pDmaRequest
										 );
		}

		if (!CTask_isIOReady(This, CHANNEL_DIR_READ) &&
			!CTask_isIOReady(This, CHANNEL_DIR_WRITE)
			)
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN/*QPDBGM_LEVEL_TRACE*/),
						 "CTask_ProcessDataStreaming ]()\n"
						 ));
			return;
		}
	}
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN/*QPDBGM_LEVEL_TRACE*/), "CTask_ProcessDataStreaming ]() start reqular schedular\n"));

	// reqular schedular
	//
	StartId = This->m_StartID;

	for (i = 0; i < MAX_TASKS; i++)
	{
		curTaskId = (StartId + i) % MAX_TASKS;
		pTaskData = &This->m_TaskData[curTaskId];

		if (pTaskData->valid)
		{
			ENTER_CRITICAL(&pTaskData->m_Object)

			if (TASK_STATE_IDLE != pTaskData->m_State)
			{
				DataId = pTaskData->m_StartID;

				for (j = 0; j < MAX_TASK_DATA_TYPE; j++)
				{
					curDataId = (DataId + j) % MAX_TASK_DATA_TYPE;
					pChannel = pTaskData->pChannel[curDataId];

					if (pChannel &&
						pTaskData->pArmMsgFifo[curDataId] &&
						CTask_isIOReady(This, pTaskData->direction[curDataId])
						)
					{
						if (QP_NULL == pTaskData->ArmRequest[curDataId].ArmBuffer.BUFFER.ALL.valid)
						{
							CFifo_GetFifo(pTaskData->pArmMsgFifo[curDataId],
										  &pTaskData->ArmRequest[curDataId]
										  );
						}

						if ((QP_NULL != pTaskData->ArmRequest[curDataId].ArmBuffer.BUFFER.ALL.valid) &&
							!pTaskData->bFlushing[curDataId]
							)
						{
							if (QP_NULL == pTaskData->UserBuffer[curDataId].pBufDesc)
							{
								if (ARM_BUF_YUV == pTaskData->ArmRequest[curDataId].ArmBuffer.type)
								{
									if (!pChannel->GetBufferYUV(pChannel,
																&pTaskData->UserBuffer[curDataId].pBufDesc,
																&pTaskData->UserBuffer[curDataId].BUFFER.YUV.pYBuffer,
																&pTaskData->UserBuffer[curDataId].BUFFER.YUV.ulYLength,
																&pTaskData->UserBuffer[curDataId].BUFFER.YUV.pUVBuffer,
																&pTaskData->UserBuffer[curDataId].BUFFER.YUV.ulUVLength
																))
									{
										QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
													 "CTask_ProcessDataStreaming() no buffer available on task(%d) data(%d) pending(%d) >>>\n",
													 curTaskId,
													 curDataId,
													 CFifo_GetFifoLevel(pTaskData->pArmMsgFifo[curDataId])
													 ));
#if 0
										// drop this frame if we are running out of YUV buffer for display
										//
										if ((TASK_TYPE_DEC == pTaskData->type) &&
											(CHANNEL_DIR_READ == pTaskData->direction[curDataId]) &&
											(CFifo_GetFifoLevel(pTaskData->pArmMsgFifo[curDataId]) >= 2)
											)
										{
											pTaskData->ArmRequest[curDataId].ArmBuffer.BUFFER.YUV.ulYOffset = pTaskData->ArmRequest[curDataId].ArmBuffer.BUFFER.YUV.ulSize << 2;
											pTaskData->ArmRequest[curDataId].ArmBuffer.BUFFER.YUV.ulUVOffset = pTaskData->ArmRequest[curDataId].ArmBuffer.BUFFER.YUV.ulSize << 1;

											This->CompleteArm(This,
															  pTaskData,
															  curDataId
															  );
										}
#endif
										continue;
									}
								}
								else if (ARM_BUF_YUVMB2RAS == pTaskData->ArmRequest[curDataId].ArmBuffer.type)
								{
									if (!pChannel->GetBufferYUVRAS(pChannel,
																   &pTaskData->UserBuffer[curDataId].pBufDesc,
																   &pTaskData->UserBuffer[curDataId].BUFFER.YUVRAS.pYBuffer,
																   &pTaskData->UserBuffer[curDataId].BUFFER.YUVRAS.ulYLength,
																   &pTaskData->UserBuffer[curDataId].BUFFER.YUVRAS.pUBuffer,
																   &pTaskData->UserBuffer[curDataId].BUFFER.YUVRAS.ulULength,
																   &pTaskData->UserBuffer[curDataId].BUFFER.YUVRAS.pVBuffer,
																   &pTaskData->UserBuffer[curDataId].BUFFER.YUVRAS.ulVLength
																   ))
									{
										QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
													 "CTask_ProcessDataStreaming() no buffer available on task(%d) data(%d)\n",
													 curTaskId,
													 curDataId
													 ));
										continue;
									}
								}
								else if (ARM_BUF_YUVRAS == pTaskData->ArmRequest[curDataId].ArmBuffer.type)
								{
									if (!pChannel->GetBufferYUVRAS(pChannel,
																   &pTaskData->UserBuffer[curDataId].pBufDesc,
																   &pTaskData->UserBuffer[curDataId].BUFFER.YUVRAS.pYBuffer,
																   &pTaskData->UserBuffer[curDataId].BUFFER.YUVRAS.ulYLength,
																   &pTaskData->UserBuffer[curDataId].BUFFER.YUVRAS.pUBuffer,
																   &pTaskData->UserBuffer[curDataId].BUFFER.YUVRAS.ulULength,
																   &pTaskData->UserBuffer[curDataId].BUFFER.YUVRAS.pVBuffer,
																   &pTaskData->UserBuffer[curDataId].BUFFER.YUVRAS.ulVLength
																   ))
									{
										QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
													 "CTask_ProcessDataStreaming() no buffer available on task(%d) data(%d)\n",
													 curTaskId,
													 curDataId
													 ));
										continue;
									}
								}
								else
								{
									if (!pChannel->GetBuffer(pChannel,
															 &pTaskData->UserBuffer[curDataId].pBufDesc,
															 &pTaskData->UserBuffer[curDataId].BUFFER.OTHERS.pBuffer,
															 &pTaskData->UserBuffer[curDataId].BUFFER.OTHERS.ulLength
															 ))
									{
										QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
													 "CTask_ProcessDataStreaming() no buffer available on task(%d) data(%d)\n",
													 curTaskId,
													 curDataId
													 ));
										continue;
									}
								}
							}

							if (pTaskData->UserBuffer[curDataId].pBufDesc)
							{

								// if the user buffer carry PTS, pass it on
								//
								if ((CHANNEL_DIR_WRITE == pTaskData->direction[curDataId]) &&
									(pTaskData->UserBuffer[curDataId].pBufDesc->ulFlags & QP_BUFDESC_FLAG_PTS) &&
									(!pTaskData->ArmRequest[curDataId].ArmBuffer.BUFFER.ALL.ulPTSValid)
									)
								{
									pTaskData->ArmRequest[curDataId].ArmBuffer.BUFFER.ALL.ulPTSValid = QP_TRUE;
									pTaskData->ArmRequest[curDataId].ArmBuffer.BUFFER.ALL.ulPTS = (TQP_UINT32)(pTaskData->UserBuffer[curDataId].pBufDesc->ulPTS >> 2);
									// Sometimes the transmission will be seperated into multiple parts,
                                                                        // so move the clear behavior to CompleteUser().
                                                                        //
									//pTaskData->UserBuffer[curDataId].pBufDesc->ulFlags &= ~QP_BUFDESC_FLAG_PTS;
								}

								// if the arm buffer carry PTS, pass it on
								//
								if ((CHANNEL_DIR_READ == pTaskData->direction[curDataId]) &&
									(pTaskData->ArmRequest[curDataId].ArmBuffer.BUFFER.ALL.ulPTSValid) &&
									(!(pTaskData->UserBuffer[curDataId].pBufDesc->ulFlags & QP_BUFDESC_FLAG_PTS))
									)
								{
									pTaskData->UserBuffer[curDataId].pBufDesc->ulPTS = pTaskData->ArmRequest[curDataId].ArmBuffer.BUFFER.ALL.ulPTS << 2;
									pTaskData->UserBuffer[curDataId].pBufDesc->ulFlags |= QP_BUFDESC_FLAG_PTS;
									pTaskData->ArmRequest[curDataId].ArmBuffer.BUFFER.ALL.ulPTSValid = QP_FALSE;
								}

								if (ARM_BUF_YUV == pTaskData->ArmRequest[curDataId].ArmBuffer.type)
								{
									if ((TASK_STATE_RUN != pTaskData->m_State) &&
										(CHANNEL_DIR_READ == pTaskData->direction[curDataId])
										)
									{
										continue;
									}

									CTask_BuildIoBlockYUV(This,
														  pTaskData,
														  (TASK_DATA_TYPE)curDataId
														  );
								}
								else if (ARM_BUF_YUVMB2RAS == pTaskData->ArmRequest[curDataId].ArmBuffer.type)
								{
									if ((TASK_STATE_RUN != pTaskData->m_State) &&
										(CHANNEL_DIR_READ == pTaskData->direction[curDataId])
										)
									{
										continue;
									}

									CTask_BuildIoBlockYUVMB2RAS(This,
																pTaskData,
																(TASK_DATA_TYPE)curDataId
																);
								}
								else if (ARM_BUF_YUVRAS == pTaskData->ArmRequest[curDataId].ArmBuffer.type)
								{
									if ((TASK_STATE_RUN != pTaskData->m_State) &&
										(CHANNEL_DIR_READ == pTaskData->direction[curDataId])
										)
									{
										continue;
									}

									CTask_BuildIoBlockYUVRAS(This,
															 pTaskData,
															 (TASK_DATA_TYPE)curDataId
															 );
								}
								else
								{
									if (pTaskData->ArmRequest[curDataId].ArmBuffer.BUFFER.OTHERS.ulWrapAround)
									{
										pTaskData->UserBuffer[curDataId].pBufDesc->ulFlags |= QP_BUFDESC_FLAG_INCOMPLETE_FRAME;
#ifdef QP_LINXKER
                            			PROFILING_LOG(EVENT_FW_D_OUT_INCOMPLETE_FRAME, pTaskData->id, pTaskData->direction[curDataId]/*dataType*/,
                                    		0, pTaskData->ArmRequestNumber[curDataId] /*HostMsg.Reg.command*/, 0);
#endif
									}
									CTask_BuildIoBlock(This,
													   pTaskData,
													   (TASK_DATA_TYPE)curDataId
													   );
								}
								CTask_SendIoBlock(This,
												  pTaskData,
												  (TASK_DATA_TYPE)curDataId
												  );
								if (!CTask_isIOReady(This, CHANNEL_DIR_READ) &&
									!CTask_isIOReady(This, CHANNEL_DIR_WRITE)
									)
								{
									LEAVE_CRITICAL(&pTaskData->m_Object)

									QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN/*QPDBGM_LEVEL_TRACE*/),
												 "CTask_ProcessDataStreaming ]()\n"
												 ));
									return;
								}
							}
						}
					}
				}
			}
			else
			{
				for (j = 0; j < MAX_TASK_DATA_TYPE; j++)
				{
					if (pTaskData->pChannel[j] &&
						pTaskData->pArmMsgFifo[j]
						)
					{
						do {
							This->CompleteArm(This,
											  pTaskData,
											  (TASK_DATA_TYPE)j
											  );
						} while (CFifo_GetFifo(pTaskData->pArmMsgFifo[j],
											   &pTaskData->ArmRequest[j]
											   ));
					}
				}
			}

			LEAVE_CRITICAL(&pTaskData->m_Object)
		}
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN/*QPDBGM_LEVEL_TRACE*/),
				 "CTask_ProcessDataStreaming() ]\n"
				 ));
}




// CThread
//

// thread proc
//
static int CTask_ThreadProc(CThread *This_p)
{
	EQP_Errors	qpStatus;
	TQP_UINT32	ulFlaggedEvents = 0;
	CTask		*This = (CTask *)This_p->m_context;

#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
	struct sched_param sp = { .sched_priority = 20 };
#endif

	CQLCodecLib	*pQLCodecLib = GET_INHERITED_OBJECT(CQLCodecLib, m_Object, This->m_pMpegCodec->m_Object.m_pParent);
	TQPTM_Time prevTime = 0;
	TQPTM_Time currTime = 0;

	This->m_pMpegCodec->m_HDCPState = HDCP_STATE_FREE;

#if 0
#ifdef QP_LINXKER
#ifdef __LINUX24__
	sprintf(current->comm, "%s", __FUNCTION__);
	daemonize();
#else
	char szThreadName[32];
	sprintf(szThreadName, "%s", __FUNCTION__);
	daemonize(szThreadName);
	current->flags |= PF_NOFREEZE;
#endif
	set_user_nice(current, -10);
#endif
#endif

#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
	sched_setscheduler(current, SCHED_FIFO, &sp);
#endif

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTask_ThreadProc() ENTER (0x%x) (0x%x) m_FilterOutHDCPContent(%d)\n",
				 This_p,
				 This,
				 This->m_pMpegCodec->m_FilterOutHDCPContent
				 ));

	while (QP_TRUE)
	{
#ifdef INTEGRATE_USB_EMU_INT_THREAD_TO_TASK_THREAD
		QPOSMISREntry(This->m_pMpegCodec->m_interruptNumber);
#endif

		// wait on the event
		//
		qpStatus = QPOSMWaitEvtgrp(This_p->m_EvtWait,
								   QLCODEC_EVENT_MASK,
								   &ulFlaggedEvents,
#ifdef INTEGRATE_USB_EMU_INT_THREAD_TO_TASK_THREAD
								   0
#else
#if 0
								   This->m_pMpegCodec->m_FilterOutHDCPContent ? 500 /*ms*/ : -1 /*always_wait*/
#else
									500 /* ms */
#endif
#endif
								   );


#ifdef INTEGRATE_USB_EMU_INT_THREAD_TO_TASK_THREAD
		if (QPERR_TIMEOUT == qpStatus)
		{
			DelayMilliSeconds(1);
			continue;
		}
#endif

		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_EXTRACE),
					 "CTask_ThreadProc() thread woke up events(0x%x) status(%d)\n",
					 ulFlaggedEvents,
					 qpStatus
					 ));

		if (!QP_SUCCESS(qpStatus) && (QPERR_TIMEOUT != qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CTask_ThreadProc() QPOSMWaitEvtgrp (%d)failed!!!\n",
						 qpStatus
						 ));
			break;
		}

		if (ulFlaggedEvents & QLCODEC_EVENT_ARM_INT)
		{
			This->ProcessArmMessage(This);
		}

		if (ulFlaggedEvents & QLCODEC_EVENT_DMA_READ_DONE)
		{
			This->CompleteData(This,
							   CHANNEL_DIR_READ
							   );
		}

		if (ulFlaggedEvents & QLCODEC_EVENT_DMA_WRITE_DONE)
		{
			This->CompleteData(This,
							   CHANNEL_DIR_WRITE
							   );
		}

		if (ulFlaggedEvents & QLCODEC_EVENT_NEW_BUFFER)
		{
			This->ProcessDataStreaming(This);
		}

		if (ulFlaggedEvents & QLCODEC_EVENT_FLUSH)
		{
			PTASK_DATA	pTaskData;
			int			i, j;

			for (i = 0; i < MAX_TASKS; i++)
			{
				pTaskData = &This->m_TaskData[i];

				if (pTaskData->valid)
				{
					for (j = 0; j < MAX_TASK_DATA_TYPE; j++)
					{
						if (pTaskData->bFlushing[j])
						{
							This->ProcessFlush(This,
											   pTaskData,
											   (TASK_DATA_TYPE)j
											   );
							QPOSMSetEvtgrp(pTaskData->m_EvtReply,
										   QLCODEC_EVENT_FLUSH_DONE
										   );
						}
					}
				}
			}
		}

		if (ulFlaggedEvents & QLCODEC_EVENT_CANCEL_BUFFER)
		{
			This->ProcessCancelBuffer(This);
			QPOSMSetEvtgrp(This_p->m_EvtReply,
						   QLCODEC_EVENT_CANCEL_BUFFER_DONE
						   );
		}

		if (ulFlaggedEvents & QLCODEC_EVENT_FLUSH_ARM)
		{
			This->ProcessFlushArm(This);
			QPOSMSetEvtgrp(This_p->m_EvtReply,
						   QLCODEC_EVENT_FLUSH_ARM_DONE
						   );
		}

		if (ulFlaggedEvents & THREAD_EVENT_EXIT)
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN/*QPDBGM_LEVEL_TRACE*/),
						 "CTask_ThreadProc() THREAD_EVENT_EXIT\n"
						 ));
			QPOSMSetEvtgrp(This_p->m_EvtReply,
						   THREAD_EVENT_EXIT_DONE
						   );
			break;
		}

		/* check HDCP status */
		if (This->m_pMpegCodec->m_FilterOutHDCPContent)
		{
			TQP_UINT32 ulToCheckDHCPStatus = 0;

#ifndef INTEGRATE_USB_EMU_INT_THREAD_TO_TASK_THREAD
			if (QPERR_TIMEOUT == qpStatus)
			{
				prevTime = QPTMGetSysTimeMilliS();
				ulToCheckDHCPStatus = 1;
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CTask_ThreadProc(): QPERR_TIMEOUT in QPOSMWaitEvtgrp(QLCODEC_EVENT_MASK)\n"));
			}
			else
			{
#endif
				currTime = QPTMGetSysTimeMilliS();
				if ((currTime - prevTime) >= 500) {
					prevTime = currTime;
					ulToCheckDHCPStatus = 1;
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CTask_ThreadProc(): TIMEOUT in (currTime - prevTime)\n"));
				}
#ifndef INTEGRATE_USB_EMU_INT_THREAD_TO_TASK_THREAD
			}
#endif

			if (ulToCheckDHCPStatus)
			{
				TQP_UINT32 ulIsHDCPContent = 0;
				TQP_UINT32 ulIsVideoStarted = 0;

				QPVideoDecoder_IsVideoStarted(&pQLCodecLib->m_pVidDecoder->m_iVideoDecoder, &ulIsVideoStarted);

				QPVideoDecoder_IsHDCPContent(&pQLCodecLib->m_pVidDecoder->m_iVideoDecoder, &ulIsHDCPContent);

				if (ulIsHDCPContent && (HDCP_STATE_FREE == This->m_pMpegCodec->m_HDCPState) && ulIsVideoStarted)
				{
					QPVideoDecoder_SetTriState(&pQLCodecLib->m_pVidDecoder->m_iVideoDecoder, 1);
					This->m_pMpegCodec->m_HDCPState = HDCP_STATE_BLOCKING;
				}

				if (!ulIsHDCPContent && (HDCP_STATE_BLOCKING == This->m_pMpegCodec->m_HDCPState))
				{
					QPVideoDecoder_SetTriState(&pQLCodecLib->m_pVidDecoder->m_iVideoDecoder, 0);
					This->m_pMpegCodec->m_HDCPState = HDCP_STATE_FREE;
				}
			}
		}
	}

	This->m_pMpegCodec->m_HDCPState = HDCP_STATE_FREE;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN/*QPDBGM_LEVEL_TRACE*/),
				 "CTask_ThreadProc() See Ya!\n"
				 ));

	return (0);
}



static void CTask_ProcessCancelBuffer(CTask *This)
{
	PTASK_DATA	pTaskData;
	int			i, j;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTask_ProcessCancelBuffer()\n"
				 ));

	for (i = 0; i < MAX_TASKS; i++)
	{
		pTaskData = &This->m_TaskData[i];

		if (pTaskData->valid)
		{
			for (j = 0; j < MAX_TASK_DATA_TYPE; j++)
			{
				if ((pTaskData->pBufDescToCancel[j]) &&
					(pTaskData->pBufDescToCancel[j] == pTaskData->UserBuffer[j].pBufDesc)
					)
				{
					CHANNEL_DIRECTION	dir = pTaskData->direction[j];

					if ((QP_NULL == This->m_pPending[dir]) ||
						(This->m_pPending[dir]->pTaskData != pTaskData)
						)
					{
						This->CompleteUser(This,
										   pTaskData,
										   (TASK_DATA_TYPE)j
										   );
						pTaskData->pBufDescToCancel[j] = QP_NULL;
					}
				}
			}
		}
	}
}



static void CTask_ProcessFlush(CTask *This,
							   PTASK_DATA pTaskData,
							   TASK_DATA_TYPE dataType
							   )
{
	CHANNEL_DIRECTION	dir;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTask_ProcessFlush() taskId(%d) dataType(%d)\n",
				 pTaskData->id,
				 dataType
				 ));

	ENTER_CRITICAL(&pTaskData->m_Object)

	if (TASK_TYPE_DEC == pTaskData->type)
	{
		QPFWDECAPI_Flush(This->m_pMpegCodec,
						 pTaskData->id,
						 pTaskData->m_decFlushBufSelect
						 );
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "CTask_ProcessFlush() QPFWDECAPI_Flush()\n"
					 ));
	}

	dir = pTaskData->direction[dataType];

	if ((QP_NULL == This->m_pPending[dir]) ||
		(This->m_pPending[dir]->pTaskData != pTaskData)
		)
	{
		do {
			// flush the active request
			This->CompleteArm(This,
							  pTaskData,
							  dataType
							  );
		} while (CFifo_GetFifo(pTaskData->pArmMsgFifo[dataType],
							   &pTaskData->ArmRequest[dataType]
							   ));
		This->CompleteUser(This,
						   pTaskData,
						   dataType
						   );
		pTaskData->bFlushing[dataType] = QP_FALSE;
	}
	else
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CTask_ProcessFlush() pending(0x%x)!!!! pending task(0x%x) task(0x%x)\n",
					 This->m_pPending[dir],
					 This->m_pPending[dir] ? This->m_pPending[dir]->pTaskData : 0,
					 pTaskData
					 ));
	}

	LEAVE_CRITICAL(&pTaskData->m_Object)
}



static void CTask_ProcessFlushArm(CTask *This)
{
	int	i, j;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTask_ProcessFlushArm()\n"
				 ));

	for (i = 0; i < MAX_TASKS; i++)
	{
		for (j = 0; j < MAX_TASK_DATA_TYPE; j++)
		{
			if (QP_NULL != This->m_TaskData[i].pArmMsgFifo[j])
			{
				do {
					// flush the active request
					This->CompleteArm(This,
									  &This->m_TaskData[i],
									  (TASK_DATA_TYPE)j
									  );
				} while (CFifo_GetFifo(This->m_TaskData[i].pArmMsgFifo[j],
									   &This->m_TaskData[i].ArmRequest[j]
									   ));
			}
		}
	}
}



// helpers
//
TASK_STATE CTask_GetTaskState(CTask *This,
							  TQP_OBJECT_HANDLE hTask
							  )
{
	return (This->m_TaskData[hTask].valid ? This->m_TaskData[hTask].m_State : TASK_STATE_IDLE);
}


TQP_BOOL CTask_isOpened(CTask *This,
						TQP_OBJECT_HANDLE hTask
						)
{
	return (This->m_TaskData[hTask].valid ? (0 != This->m_TaskData[hTask].m_dwSession) : QP_FALSE);
}



TQP_BOOL CTask_isStarted(CTask *This,
						 TQP_OBJECT_HANDLE hTask
						 )
{
	return (CTask_isOpened(This, hTask) ? (0 != This->m_TaskData[hTask].m_dwStarted) : QP_FALSE);
}



TQP_BOOL CTask_isIOReady(CTask *This,
						 CHANNEL_DIRECTION dir
						 )
{
	if (QPHCI_BUS_PCI == This->m_pMpegCodec->m_hci.m_bus_type)
	{
		// full-duplex
		//
		return (!This->m_pPending[dir]);
	}
	else
	{
		// half-duplex
		//
		return (!This->m_pPending[CHANNEL_DIR_READ] &&
				!This->m_pPending[CHANNEL_DIR_WRITE]
				);
	}
}



TQP_BOOL CTask_isEncoderStarted(CTask *This)
{
	int	i;

	ENTER_CRITICAL(&This->m_Object)

	for (i = 0; i < MAX_TASKS; i++)
	{
		ENTER_CRITICAL(&This->m_TaskData[i].m_Object)

		if (This->m_TaskData[i].valid &&
			(TASK_TYPE_ENC == This->m_TaskData[i].type)
			)
		{
			if (0 != This->m_TaskData[i].m_dwStarted)
			{
				LEAVE_CRITICAL(&This->m_TaskData[i].m_Object)
				LEAVE_CRITICAL(&This->m_Object)
				return (QP_TRUE);
			}
		}

		LEAVE_CRITICAL(&This->m_TaskData[i].m_Object)
	}

	LEAVE_CRITICAL(&This->m_Object)
	return (QP_FALSE);
}



TQP_BOOL CTask_isDecoderStarted(CTask *This)
{
	int	i;

	ENTER_CRITICAL(&This->m_Object)

	for (i = 0; i < MAX_TASKS; i++)
	{
		ENTER_CRITICAL(&This->m_TaskData[i].m_Object)

		if (This->m_TaskData[i].valid &&
			(TASK_TYPE_DEC == This->m_TaskData[i].type)
			)
		{
			if (0 != This->m_TaskData[i].m_dwStarted)
			{
				LEAVE_CRITICAL(&This->m_TaskData[i].m_Object)
				LEAVE_CRITICAL(&This->m_Object)
				return (QP_TRUE);
			}
		}
		LEAVE_CRITICAL(&This->m_TaskData[i].m_Object)
	}

	LEAVE_CRITICAL(&This->m_Object)
	return (QP_FALSE);
}



TQP_BOOL CTask_UseVideoInputDevice(CTask *This,
								   PTASK_DATA pTaskData
								   )
{
	if (0 == This->m_pMpegCodec->m_VerFwAPI)
	{
		if ((CODEC_OPEN_VID_AUD_DECODER == pTaskData->m_codec_function.function) ||
			(CODEC_OPEN_JPEG_DECODER == pTaskData->m_codec_function.function)
			)
		{
			return (QP_FALSE);
		}
	}
	else
	{
		//change "&&" to "||" for enable sensor output mjpeg to viu, panxing, 2011-4-14 11:09:35
		if ((CODEC_OPEN_EX_ENCODE != pTaskData->m_codec_function.functionEx.function) &&
			(CODEC_OPEN_EX_VID_NONE == pTaskData->m_codec_function.functionEx.video)
			)
		{
			return (QP_FALSE);
		}
	}

	return (pTaskData->valid &&
			((SYS_LINK_VID_IN_VIU == pTaskData->m_video_input) ||
			(SYS_LINK_VID_IN_HDMI == pTaskData->m_video_input) ||
			(SYS_LINK_VID_IN_HDMI_DBLCLK == pTaskData->m_video_input))
			//(SYS_LINK_VID_IN_INTERNAL == pTaskData->m_video_input))
			);
}



TQP_BOOL CTask_UseVideoOutputDevice(CTask *This,
									PTASK_DATA pTaskData
									)
{
	if (0 == This->m_pMpegCodec->m_VerFwAPI)
	{
		if ((CODEC_OPEN_VID_AUD_ENCODER == pTaskData->m_codec_function.function) ||
			(CODEC_OPEN_JPEG_ENCODER == pTaskData->m_codec_function.function)
			)
		{
			return (QP_FALSE);
		}
	}
	else
	{
		if (((CODEC_OPEN_EX_DECODE == pTaskData->m_codec_function.functionEx.function) &&
			(CODEC_OPEN_EX_VID_NONE == pTaskData->m_codec_function.functionEx.video)) ||
			((CODEC_OPEN_EX_ENCODE == pTaskData->m_codec_function.functionEx.function) &&
			(CODEC_OPEN_EX_VID_PASSTHROUGH != pTaskData->m_codec_function.functionEx.video))
			)
		{
			return (QP_FALSE);
		}
	}

	return (pTaskData->valid &&
			(SYS_LINK_VID_OUT_VOU == pTaskData->m_video_output)
			);
}



TQP_BOOL CTask_UseAudioInputDevice(CTask *This,
								   PTASK_DATA pTaskData
								   )
{
	if (0 == This->m_pMpegCodec->m_VerFwAPI)
	{
		if ((CODEC_OPEN_VID_AUD_DECODER == pTaskData->m_codec_function.function) ||
			(CODEC_OPEN_JPEG_DECODER == pTaskData->m_codec_function.function) ||
			(CODEC_OPEN_VID_PASSTHROUGH == pTaskData->m_codec_function.function)
			)
		{
			return (QP_FALSE);
		}
	}
	else
	{
		if ((CODEC_OPEN_EX_ENCODE != pTaskData->m_codec_function.functionEx.function) ||
			(CODEC_OPEN_EX_AUD_NONE == pTaskData->m_codec_function.functionEx.audio)
			)
		{
			return (QP_FALSE);
		}
	}

	return (pTaskData->valid &&
			(SYS_LINK_AUD_IN_AIU == pTaskData->m_audio_input)
			);
}



TQP_BOOL CTask_UseAudioOutputDevice(CTask *This,
									PTASK_DATA pTaskData
									)
{
	if (0 == This->m_pMpegCodec->m_VerFwAPI)
	{
		if (CODEC_OPEN_VID_AUD_DECODER != pTaskData->m_codec_function.function)
		{
			return (QP_FALSE);
		}
	}
	else
	{
		if (((CODEC_OPEN_EX_DECODE == pTaskData->m_codec_function.functionEx.function) &&
			(CODEC_OPEN_EX_AUD_NONE == pTaskData->m_codec_function.functionEx.audio)) ||
			((CODEC_OPEN_EX_ENCODE == pTaskData->m_codec_function.functionEx.function) &&
			(CODEC_OPEN_EX_AUD_PASSTHROUGH != pTaskData->m_codec_function.functionEx.audio))
			)
		{
			return (QP_FALSE);
		}
	}

	return (pTaskData->valid &&
			(SYS_LINK_AUD_OUT_AOU == pTaskData->m_audio_output)
			);
}



TQP_BOOL CTask_NeedVideoInputDevice(CTask *This)
{
	int			i;
	PTASK_DATA	pTaskData;

	ENTER_CRITICAL(&This->m_Object)

	for (i = 0; i < MAX_TASKS; i++)
	{
		pTaskData = &This->m_TaskData[i];

		ENTER_CRITICAL(&pTaskData->m_Object)

		if (pTaskData->valid)
		{
			if ((0 != pTaskData->m_dwStarted) &&
				CTask_UseVideoInputDevice(This, pTaskData)
				)
			{
				LEAVE_CRITICAL(&pTaskData->m_Object)
				LEAVE_CRITICAL(&This->m_Object)
				return (QP_TRUE);
			}
		}

		LEAVE_CRITICAL(&pTaskData->m_Object)
	}

	LEAVE_CRITICAL(&This->m_Object)
	return (QP_FALSE);
}



TQP_BOOL CTask_NeedVideoOutputDevice(CTask *This)
{
	int			i;
	PTASK_DATA	pTaskData;

	ENTER_CRITICAL(&This->m_Object)

	for (i = 0; i < MAX_TASKS; i++)
	{
		pTaskData = &This->m_TaskData[i];

		ENTER_CRITICAL(&pTaskData->m_Object)

		if (pTaskData->valid)
		{
			if ((0 != pTaskData->m_dwStarted) &&
				CTask_UseVideoOutputDevice(This, pTaskData)
				)
			{
				LEAVE_CRITICAL(&pTaskData->m_Object)
				LEAVE_CRITICAL(&This->m_Object)
				return (QP_TRUE);
			}
		}

		LEAVE_CRITICAL(&pTaskData->m_Object)
	}

	LEAVE_CRITICAL(&This->m_Object)
	return (QP_FALSE);
}



TQP_BOOL CTask_NeedAudioInputDevice(CTask *This)
{
	int			i;
	PTASK_DATA	pTaskData;

	ENTER_CRITICAL(&This->m_Object)

	for (i = 0; i < MAX_TASKS; i++)
	{
		pTaskData = &This->m_TaskData[i];

		ENTER_CRITICAL(&pTaskData->m_Object)

		if (pTaskData->valid)
		{
			if ((0 != pTaskData->m_dwStarted) &&
				CTask_UseAudioInputDevice(This, pTaskData)
				)
			{
				LEAVE_CRITICAL(&pTaskData->m_Object)
				LEAVE_CRITICAL(&This->m_Object)
				return (QP_TRUE);
			}
		}

		LEAVE_CRITICAL(&pTaskData->m_Object)
	}

	LEAVE_CRITICAL(&This->m_Object)
	return (QP_FALSE);
}



TQP_BOOL CTask_NeedAudioOutputDevice(CTask *This)
{
	int			i;
	PTASK_DATA	pTaskData;

	ENTER_CRITICAL(&This->m_Object)

	for (i = 0; i < MAX_TASKS; i++)
	{
		pTaskData = &This->m_TaskData[i];

		ENTER_CRITICAL(&pTaskData->m_Object)

		if (pTaskData->valid)
		{
			if ((0 != pTaskData->m_dwStarted) &&
				CTask_UseAudioOutputDevice(This, pTaskData)
				)
			{
				LEAVE_CRITICAL(&pTaskData->m_Object)
				LEAVE_CRITICAL(&This->m_Object)
				return (QP_TRUE);
			}
		}

		LEAVE_CRITICAL(&pTaskData->m_Object)
	}

	LEAVE_CRITICAL(&This->m_Object)
	return (QP_FALSE);
}



TQP_BOOL CTask_FindTaskDataType(CTask *This,
								TASK_TYPE taskType,
								FIRMWARE_DATA_TYPE fwDataType,
								TQP_OBJECT_HANDLE *phTask,
								TASK_DATA_TYPE *pdataType
								)
{
	int	i, j;

	for (i = 0; i < MAX_TASKS; i++)
	{
		if (This->m_TaskData[i].valid &&
			(taskType == This->m_TaskData[i].type)
			)
		{
			for (j = 0; j < MAX_TASK_DATA_TYPE; j++)
			{
				if (This->m_TaskData[i].pChannel[j] &&
					(fwDataType == This->m_TaskData[i].FWDataType[j])
					)
				{
					*phTask = i;
					*pdataType = (TASK_DATA_TYPE)j;
					return (QP_TRUE);
				}
			}
		}
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "CTask_FindTaskDataType() %s: fwDataType(0x%x) not found!!\n",
				 (TASK_TYPE_ENC == taskType) ? "ENCODER" : "DECODER",
				 fwDataType
				 ));
	return (QP_FALSE);
}



TQP_BOOL CTask_FindTask(CTask *This,
						TASK_TYPE taskType,
						TASK_DATA_TYPE dataType,
						TQP_OBJECT_HANDLE *phTask
						)
{
	int	i;

	for (i = 0; i < MAX_TASKS; i++)
	{
		if (This->m_TaskData[i].valid &&
			(taskType == This->m_TaskData[i].type)
			)
		{
			if (This->m_TaskData[i].pChannel[dataType])
			{
				*phTask = i;
				return (QP_TRUE);
			}
		}
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "CTask_FindTask() %s: dataType(%d) not found!!\n",
				 (TASK_TYPE_ENC == taskType) ? "ENCODER" : "DECODER",
				 dataType
				 ));
	return (QP_FALSE);
}



TQP_BOOL CTask_FindDataType(CTask *This,
							TQP_OBJECT_HANDLE hTask,
							FIRMWARE_DATA_TYPE fwDataType,
							TASK_DATA_TYPE *pdataType
							)
{
	int	j;

	for (j = 0; j < MAX_TASK_DATA_TYPE; j++)
	{
		if (This->m_TaskData[hTask].pChannel[j] &&
			(fwDataType == This->m_TaskData[hTask].FWDataType[j])
			)
		{
			*pdataType = (TASK_DATA_TYPE)j;
			return (QP_TRUE);
		}
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "CTask_FindDataType() hTask(%d): fwDataType(0x%x) not found!!\n",
				 hTask,
				 fwDataType
				 ));
	return (QP_FALSE);
}



void CTask_ReturnArmBuffer(CTask *This,
						   PHOST_MESSAGE pMsg,
						   PHOST_MESSAGE_STATUS pStatus,
						   TQP_UINT32 param1,
						   TQP_BOOL bAck
						   )
{
	EQP_Errors	qpStatus;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, /*QPDBGM_LEVEL_TRACE*/QPDBGM_LEVEL_WARN),
				 "CTask_ReturnArmBuffer() cmd(0x%x) channel(%d) type(0x%x) fwDara(%d)\n",
				 pMsg->Reg.command,
				 pMsg->RegArtesa.channel_number,
				 param1 & 0xFFFF,
				 (param1 >> 24) & 0xFF
				 ));

	if (bAck)
	{
		QPFWAPI_AckARMMessage(This->m_pMpegCodec,
							  pMsg,
							  pStatus,
							  QP_TRUE
							  );
	}

	// wait for the mailbox
	//
	qpStatus = QPFWAPI_MailboxReady(This->m_pMpegCodec,
			     					1000
			     					);

	if (QP_SUCCESS(qpStatus))
	{
		ARM_MESSAGE	msg;

		msg.Read = 0;
		msg.RegArtesa.channel_number = pMsg->RegArtesa.channel_number;

		switch (pMsg->Reg.command)
		{
			case EncDataOutReq:
			case EncDataOutLastReq:
				msg.Reg.command = EncDataXferDone;
				break;
			case DecDataXferReq:
				msg.Reg.command = DecLastDataXferDone/*DecDataXferDone*/;
				break;
			default:
				QPFWAPI_MailboxDone(This->m_pMpegCodec);
				return;
		}

		QPHCI_RegisterWrite(&This->m_pMpegCodec->m_hci,
							REG_TO_ARM_1,
							(param1 & 0xFFFF)
							);
		QPHCI_RegisterWrite(&This->m_pMpegCodec->m_hci,
							REG_TO_ARM_2,
							0
							);
		QPHCI_RegisterWrite(&This->m_pMpegCodec->m_hci,
							REG_TO_ARM_3,
							0
							);
		QPHCI_RegisterWrite(&This->m_pMpegCodec->m_hci,
							REG_TO_ARM_4,
							0
							);
		QPHCI_RegisterWrite(&This->m_pMpegCodec->m_hci,
							REG_TO_ARM_5,
							0
							);
		QPHCI_RegisterWrite(&This->m_pMpegCodec->m_hci,
							REG_TO_ARM_6,
							(param1 >> 24) & 0xFF
							);

		QPFWAPI_SendMessageToARM(This->m_pMpegCodec,
								 pMsg->RegArtesa.channel_number,
								 &msg,
								 QP_FALSE,
								 pStatus,
								 0
								 );

		QPFWAPI_MailboxDone(This->m_pMpegCodec);
	}
}



// constructor
//
CTask * CTask_Constructor(CTask *pTask,
						  CObject *pParent,
						  TQP_UINT32 dwAttributes,
						  TQP_UINT32 dwPriority,
						  CQLCodec *pMpegCodec,
						  TQP_OSMEvtgrpID evtWait,
						  TQP_OSMEvtgrpID evtReply
						  )
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTask_Constructor()\n"
				 ));

	if (pTask)
	{
		CThread	*pThread;


		CObject_Constructor(&pTask->m_Object,
							pParent,
		  					dwAttributes
		  					);

		pThread = CThread_Constructor(&pTask->m_Thread,
									  "CTask_ThreadProc",
									  (TQP_PVOID)pTask,
									  dwPriority,
									  evtWait,
									  evtReply,
#if defined(QP_LINXKER) && defined(CONFIG_QL300_PCIE_EVK) && defined(CONFIG_ENABLE_TESTAPPS_USERBUF)
#if LINUX_VERSION_CODE < KERNEL_VERSION(3,7,0)
									  QP_FALSE	// do not remove user space mapping
#else
									  QP_TRUE	// remove user space mapping
#endif /* LINUX_VERSION_CODE < KERNEL_VERSION(3,7,0) */
#else
									  QP_TRUE	// remove user space mapping
#endif
									  );
		if (!pThread)
		{
			pTask = QP_NULL;
		}
		else
		{
			// fill in the functions table
			//

			// CThread
			pTask->m_Thread.m_ThreadProc = CTask_ThreadProc;

			// CTask
			pTask->Alloc = CTask_Alloc;
			pTask->Release = CTask_Release;
			pTask->Open = CTask_Open;
			pTask->Close = CTask_Close;
			pTask->Start = CTask_Start;
			pTask->Stop = CTask_Stop;
			pTask->Acquire = CTask_Acquire;
			pTask->Pause = CTask_Pause;
			pTask->Resume = CTask_Resume;
			pTask->CancelBuffer = CTask_CancelBuffer;
			pTask->NewBuffer = CTask_NewBuffer;
			pTask->Flush = CTask_Flush;
			pTask->FlushArm = CTask_FlushArm;
			pTask->DMARequest = CTask_DMARequest;

			pTask->CompleteData = CTask_CompleteData;
			pTask->CompleteUser = CTask_CompleteUser;
			pTask->CompleteArm = CTask_CompleteArm;
			pTask->ProcessArmMessage = CTask_ProcessArmMessage;
			pTask->ProcessIoComplete = CTask_ProcessIoComplete;
			pTask->ProcessDataStreaming = CTask_ProcessDataStreaming;
			pTask->ProcessCancelBuffer = CTask_ProcessCancelBuffer;
			pTask->ProcessFlush = CTask_ProcessFlush;
			pTask->ProcessFlushArm = CTask_ProcessFlushArm;

			// init member variables
			//
			pTask->m_pMpegCodec = pMpegCodec;

			CObject_Constructor(&pTask->m_CritSectionIOPending,
								&pTask->m_Object,
								OBJECT_CRITICAL_HEAVY
								);

			if (!CTask_InitTasks(pTask))
			{
				CTask_Destructor(pTask);
				return (QP_NULL);
			}
		}
	}

	return (pTask);
}



// destructor
//
void CTask_Destructor(CTask *This)
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTask_Destructor()\n"
				 ));

	CTask_DoneTasks(This);

	// make sure we destroy the thread before killing the task
	//
	CThread_ThreadDone(&This->m_Thread);

	CThread_Destructor(&This->m_Thread);

	// delete the io pending critical section
	//
	CObject_Destructor(&This->m_CritSectionIOPending);

	// CObject
	//
	CObject_Destructor(&This->m_Object);
}

