///////////////////////////////////////////////////////////////////////////////
//     
//
// CTask.h
//
// Description:
//
//	Header file which defines the task that running the streaming
//
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __CTASK_H__
#define __CTASK_H__


#include "qptypes.h"
#include "cthread.h"
#include "cchannel.h"
#include "cfifo.h"
#include "fwapi.h"
#include "qpdevcntl.h"

#ifdef __cplusplus
extern "C" {
#endif


/////////////////////////////////////////////////////////////////////////////
//
//

#pragma pack(push, 4)

typedef enum _ARM_BUFFER_TYPE
{
	ARM_BUF_INVALID = 0,
	ARM_BUF_YUV,		// 420 MB format
	ARM_BUF_YUVMB2RAS,	// 420 MB to Raster format
	ARM_BUF_YUVRAS,		// 420 Raster format
	ARM_BUF_OTHERS
} ARM_BUFFER_TYPE, *PARM_BUFFER_TYPE;


typedef struct _TASK_ARM_BUFFER
{
	union _BUFFER
	{
		struct
		{
			TQP_UINT32	valid;
			TQP_UINT32	reserved1;
			TQP_UINT32	reserved2;
			TQP_UINT32	reserved3;
			TQP_UINT32	ulPTS;
			TQP_UINT32	reserved4;
			TQP_UINT32	reserved5;
			TQP_UINT32	reserved6;
			TQP_UINT32	ulPTSValid;
			TQP_UINT32	ulDataType;
			TQP_UINT32	reserved7;
			TQP_UINT32	reserved8;
			TQP_UINT32	reserved9;
			TQP_UINT32	ulFwData;
		} ALL;

		struct
		{
			TQP_UINT32	ulYAddr;
			TQP_UINT32	ulUVAddr;
			TQP_UINT32	reserved1;
			TQP_UINT32	ulSize;	// Luma size
			TQP_UINT32	ulPTS;
			TQP_UINT32	ulYOffset;
			TQP_UINT32	ulUVOffset;
			TQP_UINT32	reserved2;
			TQP_UINT32	ulPTSValid;
			TQP_UINT32	ulDataType;
			TQP_UINT32	ulLast;
			TQP_UINT32	ulWidth;
			TQP_UINT32	ulHeight;
			TQP_UINT32	ulFwData;
		} YUV;

		struct
		{
			TQP_UINT32	ulYAddr;
			TQP_UINT32	ulUVAddr;
			TQP_UINT32	reserved1;
			TQP_UINT32	ulSize;	// Luma size
			TQP_UINT32	ulPTS;
			TQP_UINT32	ulYOffset;
			TQP_UINT32	ulUOffset;
			TQP_UINT32	ulVOffset;
			TQP_UINT32	ulPTSValid;
			TQP_UINT32	ulDataType;
			TQP_UINT32	ulLast;
			TQP_UINT32	ulWidth;
			TQP_UINT32	ulHeight;
			TQP_UINT32	ulFwData;
		} YUVMB2RAS;

		struct
		{
			TQP_UINT32	ulYAddr;
			TQP_UINT32	ulUAddr;
			TQP_UINT32	ulVAddr;
			TQP_UINT32	ulSize;	// Luma size
			TQP_UINT32	ulPTS;
			TQP_UINT32	ulYOffset;
			TQP_UINT32	ulUOffset;
			TQP_UINT32	ulVOffset;
			TQP_UINT32	ulPTSValid;
			TQP_UINT32	ulDataType;
			TQP_UINT32	ulLast;
			TQP_UINT32	reserved1;
			TQP_UINT32	reserved2;
			TQP_UINT32	ulFwData;
		} YUVRAS;

		struct
		{
			TQP_UINT32	ulAddr;
			TQP_UINT32	reserved1;
			TQP_UINT32	reserved2;
			TQP_UINT32	ulSize;
			TQP_UINT32	ulPTS;
			TQP_UINT32	ulOffset;
			TQP_UINT32	reserved3;
			TQP_UINT32	ulCompressAudioType; //reserved4; by jkglee: 2012_0605
			TQP_UINT32	ulPTSValid;
			TQP_UINT32	ulDataType;
			TQP_UINT32	ulLast;
			TQP_UINT32	ulFrameFlags;
			TQP_UINT32	ulWrapAround;
			TQP_UINT32	ulFwData;
		} OTHERS;

	} BUFFER;

	ARM_BUFFER_TYPE	type;	// buffer type

} TASK_ARM_BUFFER, *PTASK_ARM_BUFFER;


#define ARM_BUFFER_BOGUS_ADDRESS	0xBEEFCAFE

typedef struct _TASK_USER_BUFFER
{
	union
	{
		struct
		{
			PTQP_UINT8		pYBuffer;
			TQP_UINT32		ulYLength;
			PTQP_UINT8		pUVBuffer;
			TQP_UINT32		ulUVLength;
			PTQP_UINT8		reserved1;
			TQP_UINT32		reserved2;
		} YUV;

		struct
		{
			PTQP_UINT8		pYBuffer;
			TQP_UINT32		ulYLength;
			PTQP_UINT8		pUBuffer;
			TQP_UINT32		ulULength;
			PTQP_UINT8		pVBuffer;
			TQP_UINT32		ulVLength;
		} YUVRAS;

		struct
		{
			PTQP_UINT8		pBuffer;
			TQP_UINT32		ulLength;
			TQP_PVOID		reserved1;
			TQP_UINT32		reserved2;
			TQP_PVOID		reserved3;
			TQP_UINT32		reserved4;
		} OTHERS;

	} BUFFER;

	// status of the user buffer
	//
	EQP_Errors	status;

	// buffer descriptor for this user buffer
	//
	PQP_BUFFER_DESCRIPTOR	pBufDesc;

} TASK_USER_BUFFER, *PTASK_USER_BUFFER;



typedef struct _TASK_ARM_REQUEST
{
	TASK_ARM_BUFFER		ArmBuffer;
	HOST_MESSAGE		HostMsg;
	HOST_MESSAGE_STATUS	HostMsg_status;
	TQP_UINT32			reqId;
} TASK_ARM_REQUEST, *PTASK_ARM_REQUEST;


typedef enum _TASK_TYPE
{
	TASK_TYPE_ENC = 0,
	TASK_TYPE_DEC,
	TASK_TYPE_END
} TASK_TYPE, *PTASK_TYPE;


typedef enum _TASK_DATA_TYPE
{
	TASK_DATA_TYPE_COMP_VID = 0,
	TASK_DATA_TYPE_COMP_AUD,
	TASK_DATA_TYPE_INDEX,
	TASK_DATA_TYPE_VBI,
	TASK_DATA_TYPE_YUV,
	TASK_DATA_TYPE_PCM,
	TASK_DATA_TYPE_VIRTUAL,
	TASK_DATA_TYPE_END,
	TASK_DATA_TYPE_INVALID = -1

} TASK_DATA_TYPE, *PTASK_DATA_TYPE;


#define MAX_TASK_DATA_TYPE	TASK_DATA_TYPE_END


// Task state
//
typedef enum _TASK_STATE
{
	TASK_STATE_IDLE = 0,
	TASK_STATE_RUN,
	TASK_STATE_STOP
} TASK_STATE, *PTASK_STATE;


#define ARM_REQ_FIFO_DEPTH	512

typedef struct _TASK_DATA
{
	// public CObject
	//
	CObject				m_Object;


	TQP_UINT32			id;
	TQP_BOOL			valid;
	TASK_TYPE			type;

	// task state management
	//
	TQP_UINT32			m_dwSession;
	TQP_UINT32			m_dwStarted;
	TQP_UINT32			m_dwPaused;
	TQP_BOOL			m_bAcquired;
	TASK_STATE			m_State;
	EQP_Errors			m_Error;

	// scheduler, simple round robin algorithm
	//
	TQP_INT32			m_StartID;

	// current pending user and arm buffer
	//
	TASK_USER_BUFFER	UserBuffer[MAX_TASK_DATA_TYPE];
	TASK_ARM_REQUEST	ArmRequest[MAX_TASK_DATA_TYPE];
	TQP_INT32			ArmRequestNumber[MAX_TASK_DATA_TYPE];
#ifdef QP_DEBUG
	TQP_UINT32			ArmBufferAddr;
#endif //QP_DEBUG


	// current buffer to cancel
	//
	volatile PQP_BUFFER_DESCRIPTOR	pBufDescToCancel[MAX_TASK_DATA_TYPE];

	// flush flag
	//
	volatile TQP_BOOL	bFlushing[MAX_TASK_DATA_TYPE];

	// channel associated with this task
	//
	CChannel			*pChannel[MAX_TASK_DATA_TYPE];
	CHANNEL_DIRECTION	direction[MAX_TASK_DATA_TYPE];
	FIRMWARE_DATA_TYPE	FWDataType[MAX_TASK_DATA_TYPE];

	// arm message fifo
	//
	CFifo				*pArmMsgFifo[MAX_TASK_DATA_TYPE];

	// encode or decode complete flag
	//
	TQP_BOOL			bDone;

	// encoder settings
	//
	SYSTEM_CONTROL		m_systemControl;
	PICTURE_RESOLUTION	m_pictureResolution;
	INPUT_CONTROL		m_inputControl;
	TQP_UINT32			m_syncMode;
	RATE_CONTROL		m_rateControl;
	RATE_CONTROL_EX		m_rateControlEx;
	BIT_RATE			m_bitRate;
	FILTER_CONTROL		m_filterControl;
	GOP_LOOP_FILTER		m_gopLoopFilter;
	ET_CONTROL			m_etControl;
	BLOCK_XFER_SIZE		m_blkXferSize;
	OUT_PICTURE_RESOLUTION	m_outPictureResolution;
	AUDIO_CONTROL_PARAM	m_audioControlParam;
	AUDIO_CONTROL_EX_AAC	m_audioControlExAAC;
	AUDIO_CONTROL_EX_G711	m_audioControlExG711;
	AUDIO_CONTROL_EX_LPCM	m_audioControlExLPCM;
	AUDIO_CONTROL_EX_SILK	m_audioControlExSILK;
	TQP_UINT32			m_EncIndexCapFreq;
	TQP_UINT32			m_MP4VideoBlockNumber;
	QPCODEC_AUDIO_ENHANCEMENT	m_audioEnhancement;
	TQP_UINT32			m_EncStopMode;
	TQP_BOOL			m_bEncEnableVidPadding;
	TQP_BOOL			m_bEncVidFrozen;
	TQP_BOOL			m_bEncVidStillInput;
	QPCODEC_ENC_CAP_MODE	m_EncCapMode;
	TQP_UINT32			m_EncMjpegQuality;
	TQP_UINT32			m_EncMjpegFrameBuffer;
	QPCODEC_EXTERNAL_TRIGGER_MODE m_ExternalTriggerToSync;
	QPCODEC_PTS_COUNTER_RESET m_PTSCounterReset;
	QPCODEC_DECIMATION_FACTOR m_RawVideoDecimationFactor;
	TQP_UINT32			m_DeinterlaceMode;
	TQP_UINT32			m_LargeCompressBufferControl;
	TQP_UINT32			m_EnableLowBitrateMode;
	TQP_UINT32			m_EnableAVDiscardControl;
	TQP_UINT32			m_UseSWPTS;

	// encoder infomation
	//
	TQP_UINT32			m_encFontTableAddr;		// dword address
	TQP_UINT32			m_encTextListAddr;		// dword address
	TQP_UINT32			m_encSyncTimeAddr;

	TQP_UINT32			m_encVidBufLumaAddr;	// dword address
	TQP_UINT32			m_encVidBufChromaAddr;	// dword address
	TQP_UINT32			m_encFrameLABufAddr;	// dword address
	TQP_UINT32			m_encTopLABufAddr;		// dword address
	TQP_UINT32			m_encBottomLABufAddr;	// dword address
	TQP_UINT32			m_encVidBufPTS;			// right shift 2 bits

	// decoder settings
	//
	DEC_STREAM_TYPE		m_streamID;
	TQP_UINT32			m_outputMode;
	PICTURE_RESOLUTION	m_decodeResolution;
	TQP_UINT32			m_decStopMode;
  	TQP_UINT32			m_decStopDispMode;
  	TQP_UINT32			m_decPauseDispMode;
	TQP_UINT32			m_decTrickPlaySpeed;
	TQP_UINT32			m_decTrickPlayDirection;
	TQP_UINT32			m_decTrickFieldPreference;
  	TQP_UINT32			m_decDispMode;
  	TQP_UINT32			m_decDispTVSystem;
  	TQP_UINT32			m_decDispForceRemote;
  	QPDEC_DISP_ATTRIB_XY	m_decDispInputSize;
  	QPDEC_DISP_ATTRIB_XY	m_decDispOutputSize;
  	QPDEC_DISP_ATTRIB_XY	m_decDispInputOrigin;
  	QPDEC_DISP_ATTRIB_XY	m_decDispOutputOrigin;
	TQP_UINT32			m_decFlushBufSelect;
	TQP_UINT32			m_decXferMode;
	TQP_UINT32			m_decXferMinimumSize;

	// decoder playback information
	//
	TQP_UINT32			m_decFWSharedMemWr;	// byte address
	TQP_UINT32			m_decFWSharedMemRd;	// byte address

	// codec settings
	//
	QPCODEC_SYS_FUNCTION	m_codec_function;
	TQP_UINT32			m_video_input;
	TQP_UINT32			m_video_in_ch;
	TQP_UINT32			m_video_output;
	TQP_UINT32			m_video_out_ch;
	TQP_UINT32			m_audio_input;
	TQP_UINT32			m_audio_in_ch;
	TQP_UINT32			m_audio_output;
	TQP_UINT32			m_audio_out_ch;

	// firmware reply event
	//
	TQP_OSMEvtgrpID		m_EvtReply;

} TASK_DATA, *PTASK_DATA;


#define MAX_TASKS	8


// side band DMA request
//
typedef struct _TASK_DMA_REQUEST
{
	TQP_UINT32			ulArmAddr;
	PTQP_UINT8			pBuffer;
	TQP_UINT32			ulLength;
	TQP_UINT32			ulOffset;
	CHANNEL_DIRECTION	dir;
	TQP_BOOL			bSwap;
} TASK_DMA_REQUEST, *PTASK_DMA_REQUEST;


enum CTask_DmaRequestEvents
{
    TASK_EVENT_DMA_REQ_DONE	= BIT(0),

    TASK_EVENT_DMA_REQ_MASK = TASK_EVENT_DMA_REQ_DONE
};


// DMA pending
//
typedef struct _TASK_IO_PENDING
{
	// DMA
	TQP_UINT32		ulArmAddress;
	PTQP_UINT8		pHostAddress;
	TQP_UINT32		ulLength;
	TQP_UINT32		ulXferMode;
	TQP_UINT32		ulPicWidth;
	TQP_UINT32		ulPicHeight;

	// status
	EQP_Errors		status;			// io status

	// task data
	//
	PTASK_DATA		pTaskData;
	TASK_DATA_TYPE	dataType;

} TASK_IO_PENDING, *PTASK_IO_PENDING;



#pragma pack(pop)



/////////////////////////////////////////////////////////////////////////////
//
//


#ifndef __CTASK_FWD_DEFINED__
#define __CTASK_FWD_DEFINED__
typedef struct CTask CTask;
#endif //!__CTASK_FWD_DEFINED__



struct CTask
{
	// public CObject
	//
	CObject		m_Object;

	// public CThread
	//
	CThread		m_Thread;

	// CTask
	//
	EQP_Errors (*Alloc)(CTask *This,
						TASK_TYPE type,
						TQP_OBJECT_HANDLE *phTask
						);
	EQP_Errors (*Release)(CTask *This,
						  TQP_OBJECT_HANDLE hTask
						  );
	EQP_Errors (*Open)(CTask *This,
					   TQP_OBJECT_HANDLE hTask,
					   TASK_DATA_TYPE dataType,
					   CHANNEL_DIRECTION direction,
					   FIRMWARE_DATA_TYPE FWDataType,
					   CChannel *pChannel
					   );
	EQP_Errors (*Close)(CTask *This,
						TQP_OBJECT_HANDLE hTask,
						TASK_DATA_TYPE dataType,
						TQP_BOOL bTaskCritical
						);
	EQP_Errors (*Start)(CTask *This,
						TQP_OBJECT_HANDLE hTask,
						TASK_DATA_TYPE dataType
						);
	EQP_Errors (*Stop)(CTask *This,
					   TQP_OBJECT_HANDLE hTask,
					   TASK_DATA_TYPE dataType
					   );
	EQP_Errors (*Acquire)(CTask *This,
						  TQP_OBJECT_HANDLE hTask,
						  TASK_DATA_TYPE dataType
						  );
	EQP_Errors (*Pause)(CTask *This,
						TQP_OBJECT_HANDLE hTask,
						TASK_DATA_TYPE dataType
						);
	EQP_Errors (*Resume)(CTask *This,
						 TQP_OBJECT_HANDLE hTask,
						 TASK_DATA_TYPE dataType
						 );
	TQP_BOOL (*CancelBuffer)(CTask *This,
							 TQP_OBJECT_HANDLE hTask,
							 TASK_DATA_TYPE dataType,
							 PQP_BUFFER_DESCRIPTOR pBufDesc
							 );
	TQP_BOOL (*NewBuffer)(CTask *This,
						  TQP_OBJECT_HANDLE hTask
						  );
	TQP_BOOL (*Flush)(CTask *This,
					  TQP_OBJECT_HANDLE hTask,
					  TASK_DATA_TYPE dataType
					  );
	TQP_BOOL (*FlushArm)(CTask *This);
	EQP_Errors (*DMARequest)(CTask *This,
							 TQP_UINT32 ulArmAddr,	// in WORD
							 PTQP_UINT8 pHostAddr,
							 TQP_UINT32 ulLength,	// in byte
							 CHANNEL_DIRECTION dir,
							 TQP_BOOL bSwap
							 );
	void (*CompleteData)(CTask *This,
						 CHANNEL_DIRECTION dir
						 );
	TQP_BOOL (*CompleteUser)(CTask *This,
							 PTASK_DATA pTaskData,
							 TASK_DATA_TYPE dataType
							 );
	void (*CompleteArm)(CTask *This,
						PTASK_DATA pTaskData,
						TASK_DATA_TYPE dataType
						);
	void (*ProcessArmMessage)(CTask *This);
	void (*ProcessIoComplete)(CTask *This,
							  CHANNEL_DIRECTION dir
							  );
	void (*ProcessDataStreaming)(CTask *This);
	void (*ProcessCancelBuffer)(CTask *This);
	void (*ProcessFlush)(CTask *This,
						 PTASK_DATA pTaskData,
						 TASK_DATA_TYPE dataType
						 );
	void (*ProcessFlushArm)(CTask *This);

	CQLCodec			*m_pMpegCodec;

	// task data holder
	//
	TASK_DATA			m_TaskData[MAX_TASKS];

	// side band DMA request
	//
	TASK_DMA_REQUEST	m_dmaRequest;
	volatile PTASK_DMA_REQUEST	m_pDmaRequest;
	TQP_OSMEvtgrpID		m_EvtDamReqComplete;

	// current IO pending
	//
	TASK_IO_PENDING		m_ioPending[MAX_CHANNEL_DIRECTION];
	volatile PTASK_IO_PENDING	m_pPending[MAX_CHANNEL_DIRECTION];
	CObject				m_CritSectionIOPending;

	// maximum DMA size
	//
	TQP_UINT32			m_dwMaxDMASize;

	// scheduler, simple round robin algorithm
	//
	TQP_INT32			m_StartID;

	// raw PCM out task
	//
	TQP_OBJECT_HANDLE	m_taskIdPCMOut;
	// raw AES out task
	//
	TQP_OBJECT_HANDLE	m_taskIdAESOut;
};



// helpers
//
TASK_STATE CTask_GetTaskState(CTask *This,
							  TQP_OBJECT_HANDLE hTask
							  );
TQP_BOOL CTask_isOpened(CTask *This,
						TQP_OBJECT_HANDLE hTask
						);
TQP_BOOL CTask_isStarted(CTask *This,
						 TQP_OBJECT_HANDLE hTask
						 );
TQP_BOOL CTask_isIOReady(CTask *This,
						 CHANNEL_DIRECTION dir
						 );
TQP_BOOL CTask_isEncoderStarted(CTask *This);
TQP_BOOL CTask_isDecoderStarted(CTask *This);
TQP_BOOL CTask_UseVideoInputDevice(CTask *This,
								   PTASK_DATA pTaskData
								   );
TQP_BOOL CTask_UseVideoOutputDevice(CTask *This,
									PTASK_DATA pTaskData
									);
TQP_BOOL CTask_UseAudioInputDevice(CTask *This,
								   PTASK_DATA pTaskData
								   );
TQP_BOOL CTask_UseAudioOutputDevice(CTask *This,
									PTASK_DATA pTaskData
									);
TQP_BOOL CTask_NeedVideoInputDevice(CTask *This);
TQP_BOOL CTask_NeedVideoOutputDevice(CTask *This);
TQP_BOOL CTask_NeedAudioInputDevice(CTask *This);
TQP_BOOL CTask_NeedAudioOutputDevice(CTask *This);
TQP_BOOL CTask_FindTaskDataType(CTask *This,
								TASK_TYPE taskType,
								FIRMWARE_DATA_TYPE fwDataType,
								TQP_OBJECT_HANDLE *phTask,
								TASK_DATA_TYPE *pdataType
								);
TQP_BOOL CTask_FindTask(CTask *This,
						TASK_TYPE taskType,
						TASK_DATA_TYPE dataType,
						TQP_OBJECT_HANDLE *phTask
						);
TQP_BOOL CTask_FindDataType(CTask *This,
							TQP_OBJECT_HANDLE hTask,
							FIRMWARE_DATA_TYPE fwDataType,
							TASK_DATA_TYPE *pdataType
							);
void CTask_ReturnArmBuffer(CTask *This,
						   PHOST_MESSAGE pMsg,
						   PHOST_MESSAGE_STATUS pStatus,
						   TQP_UINT32 param1,
						   TQP_BOOL bAck
						   );

#define ENTER_CRITICAL_IO_PENDING(This) \
	CObject_enterCritical(&((This)->m_CritSectionIOPending));

#define LEAVE_CRITICAL_IO_PENDING(This) \
	CObject_leaveCritical(&((This)->m_CritSectionIOPending));



// constructor
//
CTask * CTask_Constructor(CTask *pTask,
						  CObject *pParent,
						  TQP_UINT32 dwAttributes,
						  TQP_UINT32 dwPriority,
						  CQLCodec *pMpegCodec,
						  TQP_OSMEvtgrpID evtWait,
						  TQP_OSMEvtgrpID evtReply
						  );

// destructor
//
void CTask_Destructor(CTask *This);


// handy macro for saving Arm message
//
#define TASK_SAVE_YUV_MESSAGE(pTaskData, dataType, msg, status, param1, param2, param3, param4, param5, bpts, last, w, h)\
	{\
	TASK_ARM_REQUEST	armReq;\
	armReq.HostMsg.Read = msg;\
	armReq.HostMsg_status.Read = status;\
	armReq.ArmBuffer.type = ARM_BUF_YUV;\
	armReq.ArmBuffer.BUFFER.YUV.ulDataType = (param1 & 0xFFFF);\
	armReq.ArmBuffer.BUFFER.YUV.ulYAddr = param2;\
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_pMpegCodec->m_ChipType)\
	{\
		armReq.ArmBuffer.BUFFER.YUV.ulUVAddr = param3;\
	}\
	else \
	{\
		armReq.ArmBuffer.BUFFER.YUV.ulUVAddr = (param2 + (((param3 >> 16) & 0xFFFF) * 64));\
	}\
	armReq.ArmBuffer.BUFFER.YUV.ulSize = param4;\
	armReq.ArmBuffer.BUFFER.YUV.ulYOffset = 0;\
	armReq.ArmBuffer.BUFFER.YUV.ulUVOffset = 0;\
	armReq.ArmBuffer.BUFFER.YUV.ulPTS = param5;\
	armReq.ArmBuffer.BUFFER.YUV.ulPTSValid = bpts;\
	armReq.ArmBuffer.BUFFER.YUV.ulLast = last;\
	armReq.ArmBuffer.BUFFER.YUV.ulWidth = w;\
	armReq.ArmBuffer.BUFFER.YUV.ulHeight = h;\
	armReq.ArmBuffer.BUFFER.YUV.ulFwData = ((param1 >> 24) & 0xFF);\
	armReq.reqId = (pTaskData)->ArmRequestNumber[dataType];\
	CFifo_SetFifo((pTaskData)->pArmMsgFifo[dataType], &armReq);\
	}

#define TASK_SAVE_YUVMB2RAS_MESSAGE(pTaskData, dataType, msg, status, param1, param2, param3, param4, param5, bpts, last, w, h)\
	{\
	TASK_ARM_REQUEST	armReq;\
	armReq.HostMsg.Read = msg;\
	armReq.HostMsg_status.Read = status;\
	armReq.ArmBuffer.type = ARM_BUF_YUVMB2RAS;\
	armReq.ArmBuffer.BUFFER.YUVMB2RAS.ulDataType = (param1 & 0xFFFF);\
	armReq.ArmBuffer.BUFFER.YUVMB2RAS.ulYAddr = param2;\
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_pMpegCodec->m_ChipType)\
	{\
		armReq.ArmBuffer.BUFFER.YUVMB2RAS.ulUVAddr = param3;\
	}\
	else \
	{\
		armReq.ArmBuffer.BUFFER.YUVMB2RAS.ulUVAddr = (param2 + (((param3 >> 16) & 0xFFFF) * 64));\
	}\
	armReq.ArmBuffer.BUFFER.YUVMB2RAS.ulSize = param4;\
	armReq.ArmBuffer.BUFFER.YUVMB2RAS.ulYOffset = 0;\
	armReq.ArmBuffer.BUFFER.YUVMB2RAS.ulUOffset = 0;\
	armReq.ArmBuffer.BUFFER.YUVMB2RAS.ulVOffset = 0;\
	armReq.ArmBuffer.BUFFER.YUVMB2RAS.ulPTS = param5;\
	armReq.ArmBuffer.BUFFER.YUVMB2RAS.ulPTSValid = bpts;\
	armReq.ArmBuffer.BUFFER.YUVMB2RAS.ulLast = last;\
	armReq.ArmBuffer.BUFFER.YUVMB2RAS.ulWidth = w;\
	armReq.ArmBuffer.BUFFER.YUVMB2RAS.ulHeight = h;\
	armReq.ArmBuffer.BUFFER.YUVMB2RAS.ulFwData = ((param1 >> 24) & 0xFF);\
	armReq.reqId = (pTaskData)->ArmRequestNumber[dataType];\
	CFifo_SetFifo((pTaskData)->pArmMsgFifo[dataType], &armReq);\
	}

#if 1
#define TASK_SAVE_YUVRAS_MESSAGE(pTaskData, dataType, msg, status, param1, param2, param3, param4, param5, bpts, last)\
	{\
	TASK_ARM_REQUEST	armReq;\
	armReq.HostMsg.Read = msg;\
	armReq.HostMsg_status.Read = status;\
	armReq.ArmBuffer.type = ARM_BUF_YUVRAS;\
	armReq.ArmBuffer.BUFFER.YUVRAS.ulDataType = (param1 & 0xFFFF);\
	armReq.ArmBuffer.BUFFER.YUVRAS.ulYAddr = param2;\
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_pMpegCodec->m_ChipType)\
	{\
		armReq.ArmBuffer.BUFFER.YUVRAS.ulUAddr = param3;\
		armReq.ArmBuffer.BUFFER.YUVRAS.ulVAddr = param3 + (((param4 + 1023) >> 10) << 8);\
	}\
	else \
	{\
		armReq.ArmBuffer.BUFFER.YUVRAS.ulUAddr = (param2 + ((param3 >> 15) & 0x1FFFF));\
		armReq.ArmBuffer.BUFFER.YUVRAS.ulVAddr = (armReq.ArmBuffer.BUFFER.YUVRAS.ulUAddr + (param3 & 0x7FFF));\
	}\
	armReq.ArmBuffer.BUFFER.YUVRAS.ulSize = param4;\
	armReq.ArmBuffer.BUFFER.YUVRAS.ulYOffset = 0;\
	armReq.ArmBuffer.BUFFER.YUVRAS.ulUOffset = 0;\
	armReq.ArmBuffer.BUFFER.YUVRAS.ulVOffset = 0;\
	armReq.ArmBuffer.BUFFER.YUVRAS.ulPTS = param5;\
	armReq.ArmBuffer.BUFFER.YUVRAS.ulPTSValid = bpts;\
	armReq.ArmBuffer.BUFFER.YUVRAS.ulLast = last;\
	armReq.ArmBuffer.BUFFER.YUVRAS.ulFwData = ((param1 >> 24) & 0xFF);\
	armReq.reqId = (pTaskData)->ArmRequestNumber[dataType];\
	CFifo_SetFifo((pTaskData)->pArmMsgFifo[dataType], &armReq);\
	}
#else
#define TASK_SAVE_YUVRAS_MESSAGE(pTaskData, dataType, msg, status, param1, param2, param3, param4, param5, bpts, last)\
	{\
	TASK_ARM_REQUEST	armReq;\
	armReq.HostMsg.Read = msg;\
	armReq.HostMsg_status.Read = status;\
	armReq.ArmBuffer.type = ARM_BUF_YUVRAS;\
	armReq.ArmBuffer.BUFFER.YUVRAS.ulDataType = (param1 & 0xFFFF);\
	armReq.ArmBuffer.BUFFER.YUVRAS.ulYAddr = param2;\
	armReq.ArmBuffer.BUFFER.YUVRAS.ulUAddr = (param2 + (((param3 >> 16) & 0xFFFF) * 64));\
	armReq.ArmBuffer.BUFFER.YUVRAS.ulVAddr = (armReq.ArmBuffer.BUFFER.YUVRAS.ulUAddr + ((param3 & 0xFFFF) * 64));\
	armReq.ArmBuffer.BUFFER.YUVRAS.ulSize = param4;\
	armReq.ArmBuffer.BUFFER.YUVRAS.ulYOffset = 0;\
	armReq.ArmBuffer.BUFFER.YUVRAS.ulUOffset = 0;\
	armReq.ArmBuffer.BUFFER.YUVRAS.ulVOffset = 0;\
	armReq.ArmBuffer.BUFFER.YUVRAS.ulPTS = param5;\
	armReq.ArmBuffer.BUFFER.YUVRAS.ulPTSValid = bpts;\
	armReq.ArmBuffer.BUFFER.YUVRAS.ulLast = last;\
	armReq.ArmBuffer.BUFFER.YUVRAS.ulFwData = ((param1 >> 24) & 0xFF);\
	armReq.reqId = (pTaskData)->ArmRequestNumber[dataType];\
	CFifo_SetFifo((pTaskData)->pArmMsgFifo[dataType], &armReq);\
	}
#endif

#define TASK_SAVE_OTHERS_MESSAGE(pTaskData, dataType, msg, status, param1, param2, param3, param4, param5, bpts, last)\
	{\
	TASK_ARM_REQUEST	armReq;\
	armReq.HostMsg.Read = msg;\
	armReq.HostMsg_status.Read = status;\
	armReq.ArmBuffer.type = ARM_BUF_OTHERS;\
	armReq.ArmBuffer.BUFFER.OTHERS.ulCompressAudioType = ((param1 >> 16) & 0xFF);\
	armReq.ArmBuffer.BUFFER.OTHERS.ulDataType = (param1 & 0xFFFF);\
	armReq.ArmBuffer.BUFFER.OTHERS.ulAddr = param2;\
	armReq.ArmBuffer.BUFFER.OTHERS.ulSize = param4;\
	armReq.ArmBuffer.BUFFER.OTHERS.ulOffset = 0;\
	armReq.ArmBuffer.BUFFER.OTHERS.ulPTS = param5;\
	armReq.ArmBuffer.BUFFER.OTHERS.ulPTSValid = bpts;\
	armReq.ArmBuffer.BUFFER.OTHERS.ulLast = last;\
	armReq.ArmBuffer.BUFFER.OTHERS.ulWrapAround = (param3 & 0x1);\
	armReq.ArmBuffer.BUFFER.OTHERS.ulFrameFlags = 0;\
	armReq.ArmBuffer.BUFFER.OTHERS.ulFwData = ((param1 >> 24) & 0xFF);\
	armReq.reqId = (pTaskData)->ArmRequestNumber[dataType];\
	CFifo_SetFifo((pTaskData)->pArmMsgFifo[dataType], &armReq);\
	}

#ifdef __cplusplus
}
#endif

#endif //__CTASK_H__



