///////////////////////////////////////////////////////////////////////////////
//     
//
// Fwapi.c
//
// Description:
//
//	Firmware API
//
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////
#ifdef QP_LINXKER
#include "qlautoconf.h"
#include "plprofiling.h"
#endif

#include "hci.h"
#include "cqlcodec.h"
#include "fwapi.h"
#include "qpdbgmgr.h"
#include "qposmgr.h"
#include "util.h"

extern TQP_DbgMCompID	g_QPCodecDBGCompID;


///////////////////////////////////////////////////////////////////////////////
//

#define QPFWAPI_ENTER_CRITICAL(sID) \
   	QPOSMWaitSem(sID, -1);

#define QPFWAPI_LEAVE_CRITICAL(sID) \
	QPOSMSignalSem(sID);


///////////////////////////////////////////////////////////////////////////////
//

EQP_Errors QPFWAPI_SendMessageToARM(CQLCodec* This,
									TQP_UINT32 taskId,
									PARM_MESSAGE pMsg,
									TQP_BOOL bNeedReply,
									PHOST_MESSAGE_STATUS pHostMsg_status,
									TQP_UINT32 ulTimeOut
									)
{
	ARM_MESSAGE_STATUS	status;
	EQP_Errors			qpStatus;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWAPI_SendMessageToARM wrong power state(%d)\n",
					 This->m_PowerState
					 ));
#ifdef QP_LINXKER
        PROFILING_LOG(EVENT_FW_TO_ARM_MSG_FAILED, taskId, 0, 0, 0, pMsg->Read);
#endif

		return (QPERR_INVALID);
	}

	// wait for firmware ready
	//
	if (ulTimeOut > 0)
	{
		qpStatus = QPFWAPI_MailboxReady(This,
										ulTimeOut
										);
		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "QPFWAPI_SendMessageToARM QPFWAPI_MailboxReady() failed status(%d)\n",
						 qpStatus
						 ));
#ifdef QP_LINXKER
			PROFILING_LOG(EVENT_FW_TO_ARM_MSG_FAILED, taskId, 0, 0, 1, pMsg->Read);
#endif
			return (qpStatus);
		}
	}

	// set the busy bit
	//
	status.Read = 0;

	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		if (pHostMsg_status)
		{
			status.RegArtesa.channel_number = pHostMsg_status->RegArtesa.channel_number;
		}
		else
		{
			status.RegArtesa.channel_number = pMsg->RegArtesa.channel_number;
		}
	}
	else
	{
		status.Reg.msg_id = taskId;
	}

#ifdef QP_LINXKER
	PROFILING_LOG(EVENT_FW_TO_ARM_MSG, taskId, 0, 0, 0, pMsg->Read);
#endif

	status.Reg.rply = bNeedReply ? 1 : 0;
	status.Reg.status = 1;

	qpStatus = QPHCI_RegisterWrite(&This->m_hci,
			     				   REG_TO_ARM_MESSAGE_STATUS,
			     				   status.Read
			     				   );

	// write the command
	//
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  REG_TO_ARM_MESSAGE,
																	  pMsg->Read
																	  );

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE/*QPDBGM_LEVEL_WARN*/),
				 "QPFWAPI_SendMessageToARM() taskId(%d) REG_TO_ARM_MESSAGE_STATUS(0x%x) REG_TO_ARM_MESSAGE(0x%x) status(%d)\n",
				 taskId,
				 status.Read,
				 pMsg->Read,
				 qpStatus
				 ));

	// interrupt ARM, only effactive in master mode
	//
	QPHCI_SetInterrupt(&This->m_hci,
					   HCI_INTERRUPT_ARM
					   );

	if (ulTimeOut > 0)
	{
		QPFWAPI_MailboxDone(This);
	}

	return (qpStatus);
}



EQP_Errors QPFWAPI_WaitForARMMessage(CQLCodec* This,
									 PHOST_MESSAGE pMsg,
									 PHOST_MESSAGE_STATUS pStatus,
									 TQP_UINT32 ulTimeout,
									 PTQP_UINT32 pParm1,
									 PTQP_UINT32 pParm2,
									 PTQP_UINT32 pParm3,
									 PTQP_UINT32 pParm4,// QL201B
									 PTQP_UINT32 pParm5	// QL201B
									 )
{
	EQP_Errors	qpStatus;
	TQP_UINT32	mask = 0;


	if (QPHCI_BUS_USB != This->m_hci.m_bus_type)
	{
		// check interrupt status
		//
		qpStatus = QPHCI_GetInterruptStatus(&This->m_hci,
											&mask
											);
		if (QP_SUCCESS(qpStatus) &&
			(mask & HCI_INTERRUPT_ARM)
			)
		{
			qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterRead(&This->m_hci,
																			 REG_TO_HOST_MESSAGE,
																			 &pMsg->Read
																			 );
			qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterRead(&This->m_hci,
																			 REG_TO_HOST_MESSAGE_STATUS,
																			 &pStatus->Read
																			 );
			qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterRead(&This->m_hci,
																			 REG_TO_HOST_1,
																			 pParm1
																			 );
			qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterRead(&This->m_hci,
																			 REG_TO_HOST_2,
																			 pParm2
																			 );
			qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterRead(&This->m_hci,
																			 REG_TO_HOST_3,
																			 pParm3
																			 );
			qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterRead(&This->m_hci,
																			 REG_TO_HOST_4,
																			 pParm4
																			 );
			qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterRead(&This->m_hci,
																			 REG_TO_HOST_5,
																			 pParm5
																			 );


			// clear ARM interrupt
			//
			qpStatus = QPHCI_ClearInterrupt(&This->m_hci,
											HCI_INTERRUPT_ARM
											);
		}
		else
		{
			qpStatus = QPERR_FAIL;
		}
	}
	else
	{
		qpStatus = QPERR_FAIL;
	}

	return (qpStatus);
}




EQP_Errors QPFWAPI_GetARMMessage(CQLCodec* This,
								 PHOST_MESSAGE pMsg,
								 PHOST_MESSAGE_STATUS pStatus,
								 PTQP_UINT32 pParm1,
								 PTQP_UINT32 pParm2,
								 PTQP_UINT32 pParm3,
								 PTQP_UINT32 pParm4,// QL201B
								 PTQP_UINT32 pParm5	// QL201B
								 )
{
	EQP_Errors	qpStatus;

	if (QPHCI_BUS_CUSTOM_1 == This->m_hci.m_bus_type)
	{
		qpStatus = QPHCI_RegisterRead(&This->m_hci,
									  REG_TO_HOST_MESSAGE,
									  &pMsg->Read
									  );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterRead(&This->m_hci,
																		 REG_TO_HOST_MESSAGE_STATUS,
																		 &pStatus->Read
																		 );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterRead(&This->m_hci,
																		 REG_TO_HOST_1,
																		 pParm1
																		 );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterRead(&This->m_hci,
																		 REG_TO_HOST_2,
																		 pParm2
																		 );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterRead(&This->m_hci,
																		 REG_TO_HOST_3,
																		 pParm3
																		 );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterRead(&This->m_hci,
																		 REG_TO_HOST_4,
																		 pParm4
																		 );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterRead(&This->m_hci,
																		 REG_TO_HOST_5,
																		 pParm5
																		 );
	}
  	else
	{
		TQP_UINT32	reg[8];
		TQP_UINT32	index;
		TQP_UINT32	num_reg;

		num_reg = 7;

		num_reg = ((REG_TO_HOST_MESSAGE_STATUS - REG_TO_HOST_MESSAGE) / sizeof(TQP_UINT32)) + 1;

		qpStatus = QPHCI_RegisterReadEx(&This->m_hci,
										REG_TO_HOST_MESSAGE,
										reg,
										num_reg
										);

		if (QP_SUCCESS(qpStatus))
		{
			index = 0;
			pMsg->Read = reg[index++];
			*pParm1 = reg[index++];
			*pParm2 = reg[index++];
			*pParm3 = reg[index++];
			*pParm4 = reg[index++];
			*pParm5 = reg[index++];
			pStatus->Read = reg[index++];
		}
	}

	return (qpStatus);
}



EQP_Errors QPFWAPI_AckARMMessage(CQLCodec* This,
								 PHOST_MESSAGE pMsg,
								 PHOST_MESSAGE_STATUS pStatus,
								 TQP_BOOL fCritical
								 )
{
	EQP_Errors	qpStatus=QPERR_SUCCESS;

	if (1 == pStatus->Reg.rply)
	{
		ARM_MESSAGE	msg;
		TQP_UINT32	taskId;

		if (QPCODEC_CHIP_BERINGER == This->m_ChipType)
		{
			msg.Reg.command = (pMsg->Reg.command < 0x80) ? EncMsgReply : DecMsgReply;
			msg.Reg.reserved = 0;
			taskId = pStatus->Reg.msg_id;
		}
		else
		{
			msg.RegArtesa.command = (pMsg->RegArtesa.command < 0x80) ? EncMsgReply : DecMsgReply;
			msg.RegArtesa.channel_number = pMsg->RegArtesa.channel_number;
			taskId = pMsg->RegArtesa.channel_number;
		}

		qpStatus = QPFWAPI_SendMessageToARM(This,
											taskId,
											&msg,
											QP_FALSE,
											pStatus,
											fCritical ? 1000 : 0
											);
		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "QPFWAPI_AckARMMessage() failed status(0x%x)\n",
						 qpStatus
						 ));
			return (qpStatus);
		}
	}

	////QPFWAPI_ENTER_CRITICAL(This->m_SemaphoreFWAPI)

#ifndef REDUCE_HCI_MESSAGES
	// clear bit 0 of the host message status
	//
	pStatus->Reg.status = 0;

	qpStatus = QPHCI_RegisterWrite(&This->m_hci,
								   REG_TO_HOST_MESSAGE_STATUS,
								   pStatus->Read
								   );
#endif

	////QPFWAPI_LEAVE_CRITICAL(This->m_SemaphoreFWAPI)

	return (qpStatus);
}


////#define QPCODEC_DONT_FAIL_MAILBOX_READY

EQP_Errors QPFWAPI_MailboxReady(CQLCodec* This,
								TQP_UINT32 ulTimeOut
								)
{
	ARM_MESSAGE_STATUS	status;
	EQP_Errors			qpStatus;
	TQP_INT64			llTimeTo = (TQP_INT64)ulTimeOut;

	// wait for firmware ready
	//
	BeginTimer(&llTimeTo);


	do
	{

		// enter critical section for the mailbox
		//
		QPFWAPI_ENTER_CRITICAL(This->m_SemaphoreFWAPI)


		qpStatus = QPHCI_RegisterRead(&This->m_hci,
									  REG_TO_ARM_MESSAGE_STATUS,
									  &status.Read
									  );
		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "QPFWAPI_MailboxReady QPHCI_RegisterRead() failed status(%d)\n",
						 qpStatus
						 ));
			QPFWAPI_LEAVE_CRITICAL(This->m_SemaphoreFWAPI)
			return (qpStatus);
		}

		if (0 != status.Reg.status)
		{
			QPFWAPI_LEAVE_CRITICAL(This->m_SemaphoreFWAPI)

			if (Timeout(llTimeTo,
						1
						))
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "QPFWAPI_MailboxReady REG_TO_ARM_MESSAGE_STATUS status timeout\n"
							 ));
#ifndef QPCODEC_DONT_FAIL_MAILBOX_READY
				return (QPERR_FAIL);
#else //QPCODEC_DONT_FAIL_MAILBOX_READY
				// don't fail mailbox ready so JTAG users won't timeout
				// while break into ICE
				//
				break;
#endif //!QPCODEC_DONT_FAIL_MAILBOX_READY
			}
		}

	} while (0 != status.Reg.status);

	return (QPERR_SUCCESS);
}



EQP_Errors QPFWAPI_MailboxDone(CQLCodec* This)
{
	QPFWAPI_LEAVE_CRITICAL(This->m_SemaphoreFWAPI)
	return (QPERR_SUCCESS);
}







