///////////////////////////////////////////////////////////////////////////////
//     
//
// Fwapi.h
//
// Description:
//
//  Firmware API
//
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __FWAPI_H__
#define __FWAPI_H__

#include "qptypes.h"

#ifdef __cplusplus
extern "C" {
#endif

// Host registers
//
#define REG_TO_ARM_MESSAGE				0x06FC
#define REG_TO_ARM_1					0x06F8
#define REG_TO_ARM_2					0x06F4
#define REG_TO_ARM_3					0x06F0
#define REG_TO_ARM_4					0x06EC
#define REG_TO_ARM_5					0x06E8
#define REG_TO_ARM_6					0x06E4
#define REG_TO_ARM_7					0x06E0
#define REG_TO_ARM_8					0x06DC
#define REG_TO_ARM_9					0x06D8
#define REG_TO_ARM_10					0x06D4
#define REG_TO_ARM_11					0x06D0
#define REG_TO_ARM_MESSAGE_STATUS		0x06CC
#define REG_TO_HOST_MESSAGE_STATUS		0x06C8
#define REG_TO_HOST_5					0x06C4
#define REG_TO_HOST_4					0x06C0
#define REG_TO_HOST_3					0x06BC
#define REG_TO_HOST_2					0x06B8
#define REG_TO_HOST_1					0x06B4
#define REG_TO_HOST_MESSAGE				0x06B0


#pragma pack(push, 4)

typedef union _ARM_MESSAGE
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	command			: 16;	// bit 0 - 15
		TQP_UINT32	reserved		: 16;	// bit 15 - 31
	} Reg;
	struct
	{
		TQP_UINT32	command			: 16;	// bit 0 - 15
		TQP_UINT32	channel_number	: 16;	// bit 15 - 31
	} RegArtesa;
} ARM_MESSAGE, *PARM_MESSAGE;


typedef union _ARM_MESSAGE_STATUS
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	status			: 1;	// bit 0
		TQP_UINT32	reserved1		: 7;	// bit 1 - 7
		TQP_UINT32	rply 			: 1;	// bit 8
		TQP_UINT32	block 			: 1;	// bit 9
		TQP_UINT32	reserved2		: 6;	// bit 10 - 15
		TQP_UINT32	msg_id			: 16;	// bit 16 - 31
	} Reg;
	struct
	{
		TQP_UINT32	status			: 1;	// bit 0
		TQP_UINT32	reserved1		: 7;	// bit 1 - 7
		TQP_UINT32	rply 			: 1;	// bit 8
		TQP_UINT32	block 			: 1;	// bit 9
		TQP_UINT32	reserved2		: 6;	// bit 10 - 15
		TQP_UINT32	channel_number	: 16;	// bit 16 - 31
	} RegArtesa;
} ARM_MESSAGE_STATUS, *PARM_MESSAGE_STATUS;


typedef union _HOST_MESSAGE
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	command			: 16;	// bit 0 - 15
		TQP_UINT32	reserved		: 16;	// bit 16 - 31
	} Reg;
	struct
	{
		TQP_UINT32	command			: 16;	// bit 0 - 15
		TQP_UINT32	channel_number	: 16;	// bit 16 - 31
	} RegArtesa;
} HOST_MESSAGE, *PHOST_MESSAGE;


typedef union _HOST_MESSAGE_STATUS
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	status			: 1;	// bit 0
		TQP_UINT32	reserved1		: 7;	// bit 1 - 7
		TQP_UINT32	rply 			: 1;	// bit 8
		TQP_UINT32	reserved2		: 7;	// bit 9 - 15
		TQP_UINT32	msg_id			: 16;	// bit 16 - 31
	} Reg;
	struct
	{
		TQP_UINT32	status			: 1;	// bit 0
		TQP_UINT32	reserved1		: 7;	// bit 1 - 7
		TQP_UINT32	rply 			: 1;	// bit 8
		TQP_UINT32	reserved2		: 7;	// bit 9 - 15
		TQP_UINT32	channel_number	: 16;	// bit 16 - 31
	} RegArtesa;
} HOST_MESSAGE_STATUS, *PHOST_MESSAGE_STATUS;

#pragma pack(pop)


#define WAIT_INFINITE	0x7FFFFFFF


#ifndef __CQLCODEC_FWD_DEFINED__
#define __CQLCODEC_FWD_DEFINED__
typedef struct CQLCodec CQLCodec;
#endif //__CQLCODEC_FWD_DEFINED__

// Firmware API
//

EQP_Errors QPFWAPI_SendMessageToARM(CQLCodec* This,
									TQP_UINT32 taskId,
									PARM_MESSAGE pMsg,
									TQP_BOOL bNeedReply,
									PHOST_MESSAGE_STATUS pHostMsg_status,
									TQP_UINT32 ulTimeOut
									);
EQP_Errors QPFWAPI_WaitForARMMessage(CQLCodec* This,
									 PHOST_MESSAGE pMsg,
									 PHOST_MESSAGE_STATUS pStatus,
									 TQP_UINT32 ulTimeout,
									 PTQP_UINT32 pParm1,
									 PTQP_UINT32 pParm2,
									 PTQP_UINT32 pParm3,
									 PTQP_UINT32 pParm4,
									 PTQP_UINT32 pParm5
									 );
EQP_Errors QPFWAPI_GetARMMessage(CQLCodec* This,
								 PHOST_MESSAGE pMsg,
								 PHOST_MESSAGE_STATUS pStatus,
								 PTQP_UINT32 pParm1,
								 PTQP_UINT32 pParm2,
								 PTQP_UINT32 pParm3,
								 PTQP_UINT32 pParm4,
								 PTQP_UINT32 pParm5
								 );
EQP_Errors QPFWAPI_AckARMMessage(CQLCodec* This,
								 PHOST_MESSAGE pMsg,
								 PHOST_MESSAGE_STATUS pStatus,
								 TQP_BOOL fCritical
								 );
EQP_Errors QPFWAPI_MailboxReady(CQLCodec* This,
								TQP_UINT32 ulTimeOut
								);
EQP_Errors QPFWAPI_MailboxDone(CQLCodec* This);


// Firmware data type
//
typedef enum _FIRMWARE_DATA_TYPE
{
	FWDataTypeRawVidInput		= 0x01,
	FWDataTypeRawAudInput		= 0x02,
	FWDataTypeCmprSysVidInput	= 0x03,
	FWDataTypeCmprAudInput		= 0x04,
	FWDataTypeIndexInput		= 0x05,
	FWDataTypeVBIInput			= 0x06,
	FWDataTypeRawYUYVInput		= 0x07,
	FWDataTypeRawVidOutput		= 0x81,
	FWDataTypeRawAudOutput		= 0x82,
	FWDataTypeCmprSysVidOutput	= 0x83,
	FWDataTypeCmprAudOutput		= 0x84,
	FWDataTypeIndexOutput		= 0x85,
	FWDataTypeVBIOutput			= 0x86,
	FWDataTypeRawYUYVOutput		= 0x87
} FIRMWARE_DATA_TYPE, *PFIRMWARE_DATA_TYPE;

// Compressed audio
//
typedef enum _COMPRESSED_AUDIO_TYPE
{
	RawAudioAC3		= 0x01,
	RawAudioAAC		= 0x02,
	RawAudioAAC2	= 0x03,
	RawAudioDTS		= 0x04,
} COMPRESSED_AUDIO_TYPE, *PCOMPRESSED_AUDIO_TYPE;

/////////////////////////////////////////////////////////////////////////////
//
//

// Encoder
//


// Encoder control command code
//
typedef enum _ENCODER_COMMAND_CODE
{
	GetEncoderVersion	= 0x00,
	StartCompress		= 0x01,
	StopCompress		= 0x02,
	PauseCompress		= 0x03,
	ResumeCompress		= 0x04,
	GetCmprDbgInfo		= 0x05,
	UpdateCmprConfig	= 0x06,
	GetViosdTableAddr	= 0x07,
	GetCurVidBuf		= 0x08,
	RelCurVidBuf		= 0x09,

	InsertIFrame		= 0x0A,		// Beringer
	InsertUserData		= 0x0B,		// Beringer
	EncMiscMessage		= 0x2F,		// Beringer

	setAudioInputVolume	= 0x0A,		// Artesa
	insertIFrame		= 0x0B,		// Artesa
	insertUserData		= 0x0C,		// Artesa
	encMiscMessage		= 0x10,		// Artesa

	SetEncMode			= 0x11,
	GetAFrame			= 0x12,

	EncDataXferDone		= 0x30,
	EncMsgReply			= 0x31,

	EncMsgEnd			= 0x7F
} ENCODER_COMMAND_CODE, *PENCODER_COMMAND_CODE;



// Encoder control status code
//
typedef enum _ENCODER_STATUS_CODE
{
	EncDataOutReq		= 0x40,
	EncDataOutLastReq	= 0x41,
	EncStaMsgReply		= 0x50,
	EncErrorReport		= 0x51
} ENCODER_STATUS_CODE, *PENCODER_STATUS_CODE;


#pragma pack(push, 4)

// StartCompress
//
typedef union _SYSTEM_CONTROL
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	stream_type	: 4;	// bit 0 - 3
		TQP_UINT32	profile		: 4;	// bit 4 - 7
		TQP_UINT32	level		: 4;	// bit 8 - 11
		TQP_UINT32	cap_mod		: 2;	// bit 12 - 13
		TQP_UINT32	ff_mod		: 2;	// bit 14 - 15
		TQP_UINT32	spsr_freq	: 8;	// bit 16 - 23
		TQP_UINT32	v_mode		: 2;	// bit 24 - 25
		TQP_UINT32	cabac_init_idc	: 2;	// bit 26 - 27
		TQP_UINT32	version		: 4;	// bit 28 - 31
	} Reg;
	struct
	{
		TQP_UINT32	stream_type	: 3;	// bit 0 - 2
		TQP_UINT32	stream_data	: 5;	// bit 3 - 7
		TQP_UINT32	profile		: 4;	// bit 8 - 11
		TQP_UINT32	level		: 4;	// bit 12 - 15
		TQP_UINT32	spsr_freq	: 8;	// bit 16 - 23
		TQP_UINT32	ff_mod		: 2;	// bit 24 - 25
		TQP_UINT32	v_mode		: 1;	// bit 26
		TQP_UINT32	cabac_init_idc	: 2;// bit 27 - 28
		TQP_UINT32	version		: 2;	// bit 29 - 30
		TQP_UINT32	xfer_mode	: 1;	// bit 31
	} RegArtesa;
} SYSTEM_CONTROL, *PSYSTEM_CONTROL;


typedef union _PICTURE_RESOLUTION
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	pic_width	: 11;	// bit 0 - 10
		TQP_UINT32	reserved1	: 3;	// bit 11 - 13
		TQP_UINT32	h_scale		: 2;	// bit 14 - 15	// QL201B
		TQP_UINT32	pic_height	: 11;	// bit 16 - 26
		TQP_UINT32	reserved2	: 3;	// bit 27 - 29
		TQP_UINT32	v_scale		: 2;	// bit 30 - 31	// QL201B
	} Reg;
	struct
	{
		TQP_UINT32	in_pic_width: 11;	// bit 0 - 10
		TQP_UINT32	reserved1	: 5;	// bit 11 - 15
		TQP_UINT32	in_pic_height: 11;	// bit 16 - 26
		TQP_UINT32	reserved2	: 5;	// bit 27 - 31
	} RegArtesa;
} PICTURE_RESOLUTION, *PPICTURE_RESOLUTION;


typedef union _INPUT_CONTROL
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	mode		: 3;	// bit 0 - 2
		TQP_UINT32	src			: 3;	// bit 3 - 5
		TQP_UINT32	dst			: 3;	// bit 6 - 8
		TQP_UINT32	bl_e		: 1;	// bit 9
		TQP_UINT32	black_line_num: 6;	// bit 10 - 15
		TQP_UINT32	in_frame_rate: 7;	// bit 16 - 22
		TQP_UINT32	out_frame_rate: 6;	// bit 23 - 28
		TQP_UINT32	fmt			: 1;	// bit 29
		TQP_UINT32	pd_en		: 1;	// bit 30
		TQP_UINT32	ck_edge		: 1;	// bit 31
	} Reg;
	struct
	{
		TQP_UINT32	scan_format	: 3;	// bit 0 - 2
		TQP_UINT32	src_mode	: 3;	// bit 3 - 5
		TQP_UINT32	bytes_swap	: 1;	// bit 6
		TQP_UINT32	sync_mode	: 2;	// bit 7 - 8
		TQP_UINT32	data_type	: 7;	// bit 9 - 15
		TQP_UINT32	in_frame_rate: 7;	// bit 16 - 22
		TQP_UINT32	out_frame_rate: 6;	// bit 23 - 28
		TQP_UINT32	force_60p_in			: 1;	// bit 29
		TQP_UINT32	pd_en		: 1;	// bit 30
		TQP_UINT32	ck_edge		: 1;	// bit 31
	} RegArtesa;
} INPUT_CONTROL, *PINPUT_CONTROL;



typedef union _RATE_CONTROL
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	avg_bit_rate: 16;	// bit 0 - 15
		TQP_UINT32	qp_update_freq: 12;	// bit 16 - 27
		TQP_UINT32	reserved	: 3;	// bit 28 - 30
		TQP_UINT32	vbr			: 1;	// bit 31
	} Reg;
	struct
	{
		TQP_UINT32	avg_bit_rate: 16;	// bit 0 - 15
		TQP_UINT32	qp_update_freq: 12;	// bit 16 - 27
		TQP_UINT32	mode		: 1;	// bit 28 // not used in firmware
		TQP_UINT32	intra_refresh_mode : 1;	// bit 29
		TQP_UINT32	fixed		: 1;	// bit 30
		TQP_UINT32	vbr			: 1;	// bit 31
	} RegArtesa;
} RATE_CONTROL, *PRATE_CONTROL;


typedef union _BIT_RATE
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	min_bit_rate: 16;	// bit 0 - 15
		TQP_UINT32	max_bit_rate: 16;	// bit 16 - 31
	} Reg;
} BIT_RATE, *PBIT_RATE;


typedef union _FILTER_CONTROL
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	start_pixel	: 11;	// bit 0 - 10
		TQP_UINT32	fl_en		: 1;	// bit 11
		TQP_UINT32	filter_strength	: 4;// bit 12 - 15
		TQP_UINT32	start_line	: 11;	// bit 16 - 26
		TQP_UINT32	reserved	: 4;	// bit 27 - 30
		TQP_UINT32	dis_skp		: 1;	// bit 31
	} Reg;
} FILTER_CONTROL, *PFILTER_CONTROL;


typedef union _GOP_LOOP_FILTER
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	intra_period: 16;	// bit 0 - 15
		TQP_UINT32	lp_filter	: 1;	// bit 16
		TQP_UINT32	dis_lf_idc	: 1;	// bit 17
		TQP_INT32	alpha_offset: 4;	// bit 18 - 21
		TQP_INT32	beta_offset	: 4;	// bit 22 - 25
		TQP_UINT32	B_num		: 2;	// bit 26 - 27
		TQP_UINT32	reserved	: 3;	// bit 28 - 30
		TQP_UINT32	IDR			: 1;	// bit 31
	} Reg;
	struct
	{
		TQP_UINT32	intra_period: 16;	// bit 0 - 15
		TQP_UINT32	lp_filter	: 1;	// bit 16
		TQP_UINT32	dis_lf_idc	: 1;	// bit 17
		TQP_INT32	alpha_offset: 4;	// bit 18 - 21
		TQP_INT32	beta_offset	: 4;	// bit 22 - 25
		TQP_UINT32	B_num		: 2;	// bit 26 - 27
		TQP_UINT32	ref_n		: 1;	// bit 28
		TQP_UINT32	t8x8		: 1;	// bit 29
		TQP_UINT32	ref_p		: 1;	// bit 30
		TQP_UINT32	IDR			: 1;	// bit 31
	} RegArtesa;
} GOP_LOOP_FILTER, *PGOP_LOOP_FILTER;


typedef union _ET_CONTROL
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	et_threshold_offset : 6;	// bit 0 - 5
		TQP_UINT32	reserved1	: 2;	// bit 6 - 7
		TQP_UINT32	et_threshold_max : 6;// bit 8 - 13
		TQP_UINT32	reserved2	: 1;	// bit 14
		TQP_UINT32	et			: 1;	// bit 15
		TQP_UINT32	et_sad_threshold: 8;// bit 16 - 23
		TQP_UINT32	data_channel_type: 4;	// bit 24 - 27
		TQP_UINT32	ref_num		: 2;	// bit 28 - 29
		TQP_UINT32	reserved3	: 2;	// bit 30 - 31
	} Reg;
} ET_CONTROL, *PET_CONTROL;


typedef union _BLOCK_XFER_SIZE
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	block_size	: 16;	// bit 0 - 15	in unit of k bytes
		TQP_UINT32	slice_group	: 16;	// bit 16 - 31
	} RegArtesa;
	struct
	{
		TQP_UINT32	block_size	: 16;	// bit 0 - 15	in unit of k bytes
		TQP_UINT32	slice_limit	: 14;	// bit 16 - 29
		TQP_UINT32	multi_slice_mode	: 1;	// bit 30
		TQP_UINT32	low_latency_mode	: 1;	// bit 31
	} RegArtesaExt340;
} BLOCK_XFER_SIZE, *PBLOCK_XFER_SIZE;



typedef union _OUT_PICTURE_RESOLUTION
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	out_pic_width: 11;	// bit 0 - 10
		TQP_UINT32	reserved1	: 5;	// bit 11 - 15
		TQP_UINT32	out_pic_height: 11;	// bit 16 - 26
		TQP_UINT32	reserved2	: 5;	// bit 27 - 31
	} RegArtesa;
} OUT_PICTURE_RESOLUTION, *POUT_PICTURE_RESOLUTION;



typedef union _AUDIO_CONTROL_PARAM
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	bit_rate	: 10;	// bit 0 - 9
		TQP_UINT32	ch_sel		: 2;	// bit 10 - 11
		TQP_UINT32	sample_rate	: 4;	// bit 12 - 15
		TQP_UINT32	aou_pass_en		: 1;	// bit 16
		TQP_UINT32	ai_msb		: 1;	// bit 17
		TQP_UINT32	lrclk_i		: 1;	// bit 18
		TQP_UINT32	bclk_i		: 1;	// bit 19
		TQP_UINT32	ai_i2s		: 1;	// bit 20
		TQP_UINT32	ai_rj		: 1;	// bit 21
		TQP_UINT32	ai_m		: 1;	// bit 22
		TQP_UINT32	ch_num		: 2;	// bit 23 - 24
		TQP_UINT32	pts			: 1;	// bit 25
		TQP_UINT32	disAi		: 1;	// bit 26
		TQP_UINT32	raw			: 1;	// bit 27
		TQP_UINT32	audio_type	: 4;	// bit 28 - 31
	} Reg;
} AUDIO_CONTROL_PARAM, *PAUDIO_CONTROL_PARAM;



typedef union _AUDIO_CONTROL_EX_AAC
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	aac_ver		: 3;	// bit 0 - 2
		TQP_UINT32	h_type		: 1;	// bit 3
		TQP_UINT32	TNS			: 1;	// bit 4
		TQP_UINT32	IS			: 1;	// bit 5
		TQP_UINT32	PNS			: 1;	// bit 6
		TQP_UINT32	MS			: 1;	// bit 7
		TQP_UINT32	reserved	: 6;	// bit 8 - 13
		TQP_UINT32	lr_swap		: 1;	// bit 14
		TQP_UINT32  downsample	: 1;	// bit 15
		TQP_UINT32	cutoff_freq	: 16;	// bit 16 - 31
	} Reg;
} AUDIO_CONTROL_EX_AAC, *PAUDIO_CONTROL_EX_AAC;



typedef union _AUDIO_CONTROL_EX_G711
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	mode		: 1;	// bit 0
		TQP_UINT32	block_size	: 2;	// bit 1 - 2
		TQP_UINT32	reserved	: 11;	// bit 3 - 13
		TQP_UINT32	lr_swap		: 1;	// bit 14
		TQP_UINT32  downsample	: 1;	// bit 15
		TQP_UINT32	reserved1	: 16;	// bit 16 - 31
	} Reg;
} AUDIO_CONTROL_EX_G711, *PAUDIO_CONTROL_EX_G711;



typedef union _AUDIO_CONTROL_EX_LPCM
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	sample_cnt	: 11;	// bit 0 - 10
		TQP_UINT32	reserved	: 3;	// bit 11 - 13
		TQP_UINT32	lr_swap		: 1;	// bit 14
		TQP_UINT32  downsample	: 1;	// bit 15
		TQP_UINT32	reserved1	: 8;	// bit 16 - 23
		TQP_UINT32	raw_type	: 8;	// bit 24 - 31
	} Reg;
} AUDIO_CONTROL_EX_LPCM, *PAUDIO_CONTROL_EX_LPCM;

typedef union _AUDIO_CONTROL_EX_SILK
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	frame_len	: 2;	// bit 0 - 1
		TQP_UINT32	reserved	: 12;	// bit 2 - 13
		TQP_UINT32	lr_swap		: 1;	// bit 14
		TQP_UINT32  downsample	: 1;	// bit 15
		TQP_UINT32	reserved1	: 16;	// bit 16 - 31
	} Reg;
} AUDIO_CONTROL_EX_SILK, *PAUDIO_CONTROL_EX_SILK;

#pragma pack(pop)


// GetCmprDbgInfo
//
#define QPENC_INFO_TYPE_MASTER_CLK		1
#define QPENC_INFO_TYPE_CUR_CAP_VID_PTS	2
#define QPENC_INFO_TYPE_CUR_CAP_AUD_PTS	3
#define QPENC_INFO_TYPE_CAP_FRAME_CNT	4
#define QPENC_INFO_TYPE_ENC_FRAME_CNT	5



#pragma pack(push, 4)

// setAudioInputVolume
//
typedef union _SET_AUDIO_INPUT_VOLUME
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	vol	: 4;	// bit 0 - 3
		TQP_UINT32  reserved: 28; // bit 4 - 31
	} Reg;
} SET_AUDIO_INPUT_VOLUME, *PSET_AUDIO_INPUT_VOLUME;


#pragma pack(pop)


// Misc command code (EncMiscMessage/encMiscMessage)
//
typedef enum _ENCODER_MISC_CMD
{
	EncMiscSetIndex							= 1,
	EncMiscSetSyncCode						= 2,
	EncMiscSetMP4VidFrame					= 3,
	EncMiscAudioEnhancement					= 4,	// QL201B
	EncMiscEnableVidPadding					= 5,
	EncMiscEnableVUIParameters				= 6,
	EncMiscVBISettings						= 7,
	EncMiscOSDSettings						= 8,
	EncMiscFreezeVideo						= 9,
	EncMiscStillVideoInput					= 10,
	EncMiscSetMJPEGQuality					= 11,
	EncMiscSetMJPEGFrameBuffer				= 12,
	EncMiscSetExternalTriggerToSync			= 15,
	EncMiscSetPTSResetByTrigger				= 16,
	EncMiscSetRawVideoDecimation			= 17,
	EncMiscSetDeinterlaceMode				= 18,
	EncMiscSetRateControlEx					= 19,
	EncMiscSetLargeCompressBufferControl	= 20,
	EncMiscSetLowBitrateMode				= 21,
	EncMiscSetAVDiscardControl				= 22,
	EncMiscSetUseSWPTS						= 23
} ENCODER_MISC_CMD, *PENCODER_MISC_CMD;


// Encoder API
//
EQP_Errors QPFWENCAPI_StartEncoder(CQLCodec* This,
								   TQP_UINT32 taskId
								   );
EQP_Errors QPFWENCAPI_StopEncoder(CQLCodec* This,
								  TQP_UINT32 taskId,
								  TQP_BOOL bStopAtGOP,
								  TQP_UINT32 channel_dataType
								  );
EQP_Errors QPFWENCAPI_PauseEncoder(CQLCodec* This,
								   TQP_UINT32 taskId
								   );
EQP_Errors QPFWENCAPI_ResumeEncoder(CQLCodec* This,
									TQP_UINT32 taskId
									);
EQP_Errors QPFWENCAPI_GetEncoderVersion(CQLCodec* This,
										TQP_UINT32 taskId,
										PTQP_UINT32 pVersion
										);
EQP_Errors QPFWENCAPI_GetEncoderDbgInfo(CQLCodec* This,
										TQP_UINT32 taskId,
										TQP_UINT32 type
										);
EQP_Errors QPFWENCAPI_SetSystemControl(CQLCodec* This,
									   TQP_UINT32 value
									   );
EQP_Errors QPFWENCAPI_SetPictureResolution(CQLCodec* This,
										   TQP_UINT32 value
  		  								   );
EQP_Errors QPFWENCAPI_SetInputControl(CQLCodec* This,
									  TQP_UINT32 value
									  );
EQP_Errors QPFWENCAPI_SetRateControl(CQLCodec* This,
  		  							 TQP_UINT32 value
  		  							 );
EQP_Errors QPFWENCAPI_SetVBRBitRate(CQLCodec* This,
  		  							TQP_UINT32 value
  		  							);
EQP_Errors QPFWENCAPI_SetFilterControl(CQLCodec* This,
									   TQP_UINT32 value
  		  							   );
EQP_Errors QPFWENCAPI_SetGOPLoopFilter(CQLCodec* This,
  		  							   TQP_UINT32 value
  		  							   );
// beringer only
EQP_Errors QPFWENCAPI_SetETControl(CQLCodec* This,
  		  						   TQP_UINT32 value
  		  						   );
// artesa new defination
EQP_Errors QPFWENCAPI_SetBlockSize(CQLCodec* This,
								   TQP_UINT32 value
  		  						   );
// artesa only
EQP_Errors QPFWENCAPI_SetOutPictureResolution(CQLCodec* This,
											  TQP_UINT32 value
  		  									  );
EQP_Errors QPFWENCAPI_SetAudioControlParameters(CQLCodec* This,
  		  										TQP_UINT32 value
  		  										);
EQP_Errors QPFWENCAPI_SetAudioControlExtension(CQLCodec* This,
  		  									   TQP_UINT32 value
  		  									   );
EQP_Errors QPFWENCAPI_UpdateConfig(CQLCodec* This,
								   TQP_UINT32 taskId
								   );
EQP_Errors QPFWENCAPI_SetVIOSD(CQLCodec* This,
										   TQP_UINT32 taskId,
										   TQP_UINT32 pbVIOSDen
										   );

EQP_Errors QPFWENCAPI_GetViosdTableaddr(CQLCodec* This,
										   TQP_UINT32 taskId,
										   PTQP_UINT32 pFontTableAddr,	// in dwords
										   PTQP_UINT32 pTextListAddr,	// in dwords
										   PTQP_UINT32 pSyncTimeAddr	// in dwords
										   );
EQP_Errors QPFWENCAPI_GetCurVidBufInfo(CQLCodec* This,
									   TQP_UINT32 taskId,
									   TQP_UINT32 holdBuffer,
									   PTQP_UINT32 pVidBufLumaAddr,		// in dwords
									   PTQP_UINT32 pVidBufChromaAddr,	// in dwords
									   PTQP_UINT32 pFrameLABufAddr,		// in dwords
									   PTQP_UINT32 pTopLABufAddr,		// in dwords
									   PTQP_UINT32 pBottomLABufAddr,	// in dwords
									   PTQP_UINT32 pVidBufPTS			// right shift 2 bits
									   );
EQP_Errors QPFWENCAPI_ReleaseCurBuf(CQLCodec* This,
									TQP_UINT32 taskId
									);
EQP_Errors QPFWENCAPI_SetAudioInputVolume(CQLCodec* This,
										  TQP_UINT32 taskId,
										  TQP_UINT32 vol
										  );
EQP_Errors QPFWENCAPI_InsertIFrame(CQLCodec* This,
								   TQP_UINT32 taskId
								   );
EQP_Errors QPFWENCAPI_InsertUserData(CQLCodec* This,
									 TQP_UINT32 taskId,
									 TQP_UINT32 numUserData,
									 PTQP_UINT8 pUserData
									 );
EQP_Errors QPFWENCAPI_SetIndexCapture(CQLCodec* This,
									  TQP_UINT32 taskId,
									  TQP_UINT32 freq
									  );
EQP_Errors QPFWENCAPI_SetViuSyncCode(CQLCodec* This,
									 TQP_UINT32 taskId,
									 TQP_UINT32 code1,
  		  							 TQP_UINT32 code2
  		  							 );
EQP_Errors QPFWENCAPI_SetMP4VideoBlockNumber(CQLCodec* This,
											 TQP_UINT32 taskId,
											 TQP_UINT32 num
											 );
EQP_Errors QPFWENCAPI_SetAudioEnhancement(CQLCodec* This,
										  TQP_UINT32 taskId,
										  TQP_UINT32 gain1,
										  TQP_UINT32 gain2,
										  TQP_UINT32 add,
										  TQP_UINT32 sub,
										  TQP_UINT32 att1,
										  TQP_UINT32 att2,
										  TQP_UINT32 lgain,
										  TQP_UINT32 rgain
										  );
EQP_Errors QPFWENCAPI_EnableVidPadding(CQLCodec* This,
									   TQP_UINT32 taskId,
									   TQP_UINT32 enable
  		  							   );
EQP_Errors QPFWENCAPI_EnableVUIParam(CQLCodec* This,
									 TQP_UINT32 taskId,
									 TQP_UINT32 enable,
									 TQP_UINT32 aspect_ratio
  		  							 );
EQP_Errors QPFWENCAPI_FreezeVideo(CQLCodec* This,
								  TQP_UINT32 taskId,
								  TQP_UINT32 enable
  		  						  );
EQP_Errors QPFWENCAPI_SetVBIInfo(CQLCodec* This,
								 TQP_UINT32 taskId,
								 TQP_UINT32 enable,
  		  						 TQP_UINT32 top_start_line,
  		  						 TQP_UINT32 top_num_lines,
  		  						 TQP_UINT32 top_start_pixel,
  		  						 TQP_UINT32 top_num_samples,
  		  						 TQP_UINT32 bottom_start_line,
  		  						 TQP_UINT32 bottom_num_lines,
  		  						 TQP_UINT32 bottom_start_pixel,
  		  						 TQP_UINT32 bottom_num_samples
  		  						 );
EQP_Errors QPFWENCAPI_StillVideoInput(CQLCodec* This,
									  TQP_UINT32 taskId,
									  TQP_UINT32 enable
  		  							  );
EQP_Errors QPFWENCAPI_SetEncMode(CQLCodec* This,
								 TQP_UINT32 taskId,
								 TQP_UINT32 capMode,
								 TQP_UINT32 trigMode,
								 TQP_UINT32 gpioPin
								 );
EQP_Errors QPFWENCAPI_GetAFrame(CQLCodec* This,
								TQP_UINT32 taskId
								);
EQP_Errors QPFWENCAPI_SetMJPEGQuality(CQLCodec* This,
									  TQP_UINT32 taskId,
									  TQP_UINT32 quality
  		  							  );
EQP_Errors QPFWENCAPI_SetMJPEGFrameBuffer(CQLCodec* This,
										  TQP_UINT32 taskId,
										  TQP_UINT32 nb
  		  								  );
EQP_Errors QPFWENCAPI_SetExternalTriggerToSync(CQLCodec* This,
											   TQP_UINT32 taskId,
											   TQP_UINT32 enable,
											   TQP_UINT32 gpioPin
  		  								 	   );
EQP_Errors QPFWENCAPI_SetPTSResetByTrigger(CQLCodec* This,
										   TQP_UINT32 taskId,
										   TQP_UINT32 enable,
										   TQP_UINT32 gpioPin,
										   TQP_UINT32 immediate
  		  								   );
EQP_Errors QPFWENCAPI_SetRawVideoDecimation(CQLCodec* This,
										    TQP_UINT32 taskId,
										    TQP_UINT32 input_format,
										    TQP_UINT32 output_format,
										    TQP_UINT32 scale_factor
  		  								    );
EQP_Errors QPFWENCAPI_SetDeinterlaceMode(CQLCodec* This,
										 TQP_UINT32 taskId,
										 TQP_UINT32 mode
  		  								 );
EQP_Errors QPFWENCAPI_SetRateControlEx(CQLCodec* This,
									   TQP_UINT32 taskId,
									   TQP_UINT32 base_qp_length,
									   TQP_UINT32 local_activity_on,
									   TQP_UINT32 motion_detect_interval
  		  							   );
EQP_Errors QPFWENCAPI_SetLargeCompressBufferControl(CQLCodec* This,
												    TQP_UINT32 taskId,
												    TQP_UINT32 large_compress_buffer_control
  		  										    );
 EQP_Errors QPFWENCAPI_SetLowBitrateMode(CQLCodec* This,
    									 TQP_UINT32 taskId,
    									 TQP_UINT32 enable_low_bitrate_mode
										 );
 EQP_Errors QPFWENCAPI_SetAVDiscardControl(CQLCodec* This,
    									   TQP_UINT32 taskId,
    									   TQP_UINT32 enable_av_discard_control
										   );
 EQP_Errors QPFWENCAPI_SetUseSWPTS(CQLCodec* This,
    							   TQP_UINT32 taskId,
    							   TQP_UINT32 use_sw_pts
								   );


/////////////////////////////////////////////////////////////////////////////
//
//

// Decoder
//

// Decoder register defination
//
#define DEC_REG_MESSAGE					REG_TO_ARM_MESSAGE
#define DEC_REG_PARAM_1					REG_TO_ARM_1
#define DEC_REG_PARAM_2					REG_TO_ARM_2
#define DEC_REG_PARAM_3					REG_TO_ARM_3
#define DEC_REG_PARAM_4					REG_TO_ARM_4
#define DEC_REG_PARAM_5					REG_TO_ARM_5
#define DEC_REG_PARAM_6					REG_TO_ARM_6
#define DEC_REG_PARAM_7					REG_TO_ARM_7
#define DEC_REG_PARAM_8					REG_TO_ARM_8
#define DEC_REG_PARAM_9					REG_TO_ARM_9
#define DEC_REG_PARAM_10				REG_TO_ARM_10
#define DEC_REG_PARAM_11				REG_TO_ARM_11



// Decoder Control Command Code
//
typedef enum _DECODER_COMMAND_CODE
{
	GetDecoderVersion	= 0x80,
	StartDecompress		= 0x81,
	StopDecompress		= 0x82,
	PauseDecDisplay		= 0x83,
	ResumeDecDisplay	= 0x84,
	SetDecTrickMode		= 0x85,
	DecFlush			= 0x86,
	DecGetPlayInfo		= 0x87,

	ResumeNormal		= 0x88,
	StepSlowPlay		= 0x89,

	setAudioOutputVolume = 0x88,	// Artesa
	resumeNormal		= 0x89,		// Artesa
	stepSlowPlay		= 0x8A,		// Artesa
	decMiscMessage		= 0x90,		// Artesa

	DecDataXferDone		= 0xA0,
	DecLastDataXferDone	= 0xA1,
	DecMsgReply			= 0xA2,

	GetOsdMemAddr	= 0xB0,

	SetDecDisplayAttrib	= 0xE0,
	SetDecDisplayMode	= 0xE1,
	SetDecDisplayScale	= 0xE2

} DECODER_COMMAND_CODE, *PDECODER_COMMAND_CODE;

// Decoder control status code
//
typedef enum _DECODER_STATUS_CODE
{
	DecDataXferReq		= 0xC0,
	DecStaMsgReply		= 0xD0,
	DecStatusReport		= 0xD1
} DECODER_STATUS_CODE, *PDECODER_STATUS_CODE;


#pragma pack(push, 4)

// StartDecompress
//

// parameter 1
typedef union _DEC_STREAM_TYPE
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	stream_type;
	} Reg;
	struct
	{
		TQP_UINT32	stream_type	: 4;	// bit 0 - 3
		TQP_UINT32	stream_data	: 4;	// bit 4 - 7
		TQP_UINT32	auto_detect	: 1;	// bit 8
		TQP_UINT32	reserved	: 19;	// bit 9 - 27
		TQP_UINT32	bytes_swap	: 1;	// bit 28
		TQP_UINT32	version		: 2;	// bit 29 - 30
		TQP_UINT32	xfer_mode	: 1;	// bit 31
	} RegEx;
} DEC_STREAM_TYPE, *PDEC_STREAM_TYPE;


// parameter 4
typedef union _DEC_START_VO_OFFSET
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	startPixel	: 16;	// bit 0 - 15
		TQP_UINT32	startLine	: 16;	// bit 16 - 31
	} Reg;
} DEC_START_VO_OFFSET, *PDEC_START_VO_OFFSET;


// parameter 5
typedef union _DEC_AO_CONTROL_PARAM
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	ao_en		: 1;	// bit 0
		TQP_UINT32	ao_msb		: 1;	// bit 1
		TQP_UINT32	lrclk_i		: 1;	// bit 2
		TQP_UINT32	bclk_i		: 1;	// bit 3
		TQP_UINT32	ao_i2s		: 1;	// bit 4
		TQP_UINT32	ao_rj		: 1;	// bit 5
		TQP_UINT32	ao_s		: 1;	// bit 6
		TQP_UINT32	reserved	: 25;	// bit 7 - 31
	} Reg;
} DEC_AO_CONTROL_PARAM, *PDEC_AO_CONTROL_PARAM;


// parameter 6
typedef union _DEC_AUDIO_CONTROL_PARAM
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	audio_type	: 4;	// bit 0 - 3
		TQP_UINT32	reserved1	: 4;	// bit 4 - 7
		TQP_UINT32	ch_num		: 2;	// bit 8 - 9
		TQP_UINT32	reserved2	: 2;	// bit 10 - 11
		TQP_UINT32	sample_rate	: 4;	// bit 12 - 15
		TQP_UINT32	reserved3	: 4;	// bit 16 - 19
		TQP_UINT32	mode		: 1;	// bit 20
		TQP_UINT32	upsample	: 1;	// bit 21
		TQP_UINT32	reserved4	: 10;	// bit 22 - 31
	} Reg;
} DEC_AUDIO_CONTROL_PARAM, *PDEC_AUDIO_CONTROL_PARAM;



// SetDecTrickMode
//

// parameter 1
typedef union _DEC_TRICK_MODE_SKIP_REPEAT_CNT
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	repeat_cnt	: 16;	// bit 0 - 15
		TQP_UINT32	skip_cnt	: 16;	// bit 16 - 31
	} Reg;
} DEC_TRICK_MODE_SKIP_REPEAT_CNT, *PDEC_TRICK_MODE_SKIP_REPEAT_CNT;



// DecStatusReport
//

// param1
#define QPDEC_STATUS_TYPE_ERR			0
#define QPDEC_STATUS_TYPE_DISP_READY	1
#define QPDEC_STATUS_TYPE_DEC_DONE		2


// param2 for QPDEC_STATUS_TYPE_ERR
#define QPDEC_STATUS_SEQUENCE_PARAM_CHANGED	1
#define QPDEC_STATUS_BITSTREAM_ERROR	2


#pragma pack(pop)



// Decoder API
//
EQP_Errors QPFWDECAPI_GetDecoderVersion(CQLCodec* This,
										TQP_UINT32 taskId,
										PTQP_UINT32 pVersion
										);
EQP_Errors QPFWDECAPI_StartDecoder(CQLCodec* This,
								   TQP_UINT32 taskId,
								   TQP_UINT32 streamID,
  		  						   TQP_UINT32 outMode,
								   TQP_UINT32 VOUMode,
								   TQP_UINT32 VOUStartPixel,
								   TQP_UINT32 VOUStartLine,
								   TQP_UINT32 AOControls,
								   TQP_UINT32 AudControls
  		  						   );
EQP_Errors QPFWDECAPI_StopDecoder(CQLCodec* This,
								  TQP_UINT32 taskId,
								  TQP_UINT32 stopMode,
  		  						  TQP_UINT32 displayMode,
								  TQP_UINT32 pts1,
								  TQP_UINT32 pts2
								  );
EQP_Errors QPFWDECAPI_PauseDisplay(CQLCodec* This,
								   TQP_UINT32 taskId,
								   TQP_UINT32 displayMode
								   );
EQP_Errors QPFWDECAPI_ResumeDisplay(CQLCodec* This,
									TQP_UINT32 taskId
									);
EQP_Errors QPFWDECAPI_SetTrickMode(CQLCodec* This,
								   TQP_UINT32 taskId,
								   TQP_UINT32 playSpeed,
								   TQP_UINT32 playDirection,
								   TQP_UINT32 step,
								   TQP_UINT32 PTS,
								   TQP_UINT32 fieldPreference
								   );
EQP_Errors QPFWDECAPI_Flush(CQLCodec* This,
							TQP_UINT32 taskId,
						    TQP_UINT32 buffer_sel
							);
EQP_Errors QPFWDECAPI_GetPlayInfo(CQLCodec* This,
								  TQP_UINT32 taskId,
								  TQP_UINT32 type
								  );
EQP_Errors QPFWDECAPI_SetDisplayAttrib(CQLCodec* This,
									   TQP_UINT32 taskId,
									   TQP_UINT32 TVSystem,
  		  							   TQP_UINT32 input_size,
  		  							   TQP_UINT32 output_size,
  		  							   TQP_UINT32 input_origin,
  		  							   TQP_UINT32 output_origin
  		  							   );
EQP_Errors QPFWDECAPI_SetDisplayMode(CQLCodec* This,
									 TQP_UINT32 taskId,
  		  							 TQP_UINT32 operation,
  		  							 TQP_UINT32 displayMode
  		  							 );


/////////////////////////////////////////////////////////////////////////////
//
//

// Codec Config
//


// Codec config command code
//
typedef enum _CODEC_COMMAND_CODE
{
	CodecSystemConfig	= 0xF0,
	CodecSystemOpen		= 0xF1,
	CodecSystemLink		= 0xF2,
	CodecSystemClose	= 0xF3

} CODEC_COMMAND_CODE, *PCODEC_COMMAND_CODE;

// CodecSystemConfig
//

// CodecSystemOpen
//

// CodecSystemLink
//

// parameter 1
typedef union _CODEC_SYSTEM_LINK
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	video_input	: 4;	// bit 0 - 3
		TQP_UINT32	video_in_ch	: 4;	// bit 4 - 7
		TQP_UINT32	video_output: 4;	// bit 8 - 11
		TQP_UINT32	video_out_ch: 4;	// bit 12 - 15
		TQP_UINT32	audio_input	: 4;	// bit 16 - 19
		TQP_UINT32	audio_in_ch	: 4;	// bit 20 - 23
		TQP_UINT32	audio_output: 4;	// bit 24 - 27
		TQP_UINT32	audio_out_ch: 4;	// bit 28 - 31
	} Reg;
} CODEC_SYSTEM_LINK, *PCODEC_SYSTEM_LINK;


// CodecSystemClose
//


// Codec API
//
EQP_Errors QPFWCODECAPI_SystemConfig(CQLCodec* This,
									 TQP_UINT32 taskId
									 );
EQP_Errors QPFWCODECAPI_SystemOpen(CQLCodec* This,
								   TQP_UINT32 taskId,
								   TQP_UINT32 function
								   );
EQP_Errors QPFWCODECAPI_SystemLink(CQLCodec* This,
								   TQP_UINT32 taskId,
								   TQP_UINT32 video_input,
								   TQP_UINT32 video_in_ch,
								   TQP_UINT32 video_output,
								   TQP_UINT32 video_out_ch,
								   TQP_UINT32 audio_input,
								   TQP_UINT32 audio_in_ch,
								   TQP_UINT32 audio_output,
								   TQP_UINT32 audio_out_ch
								   );
EQP_Errors QPFWCODECAPI_SystemClose(CQLCodec* This,
								   TQP_UINT32 taskId
								   );

EQP_Errors QPFWDECAPI_GetVouOsdMem(CQLCodec* This,
								   TQP_UINT32 taskId,
								   TQP_UINT32 command,
								   TQP_UINT32 memsize
								   );
EQP_Errors QPFWDECAPI_SetAudioOutputVolume(CQLCodec* This,
									 TQP_UINT32 taskId,
  		  							 TQP_UINT32 vol
  		  							 );
#ifdef __cplusplus
}
#endif

#endif //__FWAPI_H__

