///////////////////////////////////////////////////////////////////////////////
//     
//
// Fwcodecapi.c
//
// Description:
//
//	Firmware codec API
//
// History:
//	April 23, 2008	(dt) created
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////
#ifdef QP_LINXKER
#include "qlautoconf.h"
#include "plprofiling.h"
#endif

#include "hci.h"
#include "cqlcodec.h"
#include "fwapi.h"
#include "qpdbgmgr.h"

extern TQP_DbgMCompID	g_QPCodecDBGCompID;



EQP_Errors QPFWCODECAPI_SystemConfig(CQLCodec* This,
									 TQP_UINT32 taskId
									 )
{
	EQP_Errors	qpStatus;
	ARM_MESSAGE	msg;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWCODECAPI_SystemConfig() wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	if (!(QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType))
	{
		return (QPERR_SUCCESS);
	}


	qpStatus = QPFWAPI_MailboxReady(This,
									500
									);
	if (!QP_SUCCESS(qpStatus))
		return (qpStatus);

	msg.Read = 0;
	msg.Reg.command = CodecSystemConfig;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.channel_number = taskId;
	}

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																		   taskId,
																		   &msg,
																		   QP_FALSE,
																		   QP_NULL,
																		   0
																		   );
	QPFWAPI_MailboxDone(This);
	return (qpStatus);
}



EQP_Errors QPFWCODECAPI_SystemOpen(CQLCodec* This,
								   TQP_UINT32 taskId,
								   TQP_UINT32 function
								   )
{
	EQP_Errors	qpStatus;
	ARM_MESSAGE	msg;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWCODECAPI_SystemOpen() wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	if (!(QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType))
	{
		return (QPERR_SUCCESS);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "QPFWCODECAPI_SystemOpen() taskId(%d) function(0x%x)\n",
				 taskId,
  		  		 function
				 ));

	qpStatus = QPFWAPI_MailboxReady(This,
									500
									);
	if (!QP_SUCCESS(qpStatus))
		return (qpStatus);

#ifdef QP_LINXKER
	PROFILING_LOG(EVENT_FW_SYSTEM_OPEN, taskId, 0, 0, 0, 0);
#endif

	qpStatus = QPHCI_RegisterWrite(&This->m_hci,
								   DEC_REG_PARAM_1,
								   function
								   );

	msg.Read = 0;
	msg.Reg.command = CodecSystemOpen;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.channel_number = taskId;
	}

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																		   taskId,
																		   &msg,
																		   QP_FALSE,
																		   QP_NULL,
																		   0
																		   );
	QPFWAPI_MailboxDone(This);
	return (qpStatus);
}



EQP_Errors QPFWCODECAPI_SystemLink(CQLCodec* This,
								   TQP_UINT32 taskId,
								   TQP_UINT32 video_input,
								   TQP_UINT32 video_in_ch,
								   TQP_UINT32 video_output,
								   TQP_UINT32 video_out_ch,
								   TQP_UINT32 audio_input,
								   TQP_UINT32 audio_in_ch,
								   TQP_UINT32 audio_output,
								   TQP_UINT32 audio_out_ch
								   )
{
	EQP_Errors			qpStatus;
	CODEC_SYSTEM_LINK	sysLink;
	ARM_MESSAGE			msg;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWCODECAPI_SystemLink() wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	if (!(QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType))
	{
		return (QPERR_SUCCESS);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "QPFWCODECAPI_SystemLink() taskId(%d) video_input(%d) video_in_ch(%d) video_output(%d) video_out_ch(%d) audio_input(%d) audio_in_ch(%d) audio_output(%d) audio_out_ch(%d)\n",
				 taskId,
				 video_input,
				 video_in_ch,
				 video_output,
				 video_out_ch,
				 audio_input,
				 audio_in_ch,
				 audio_output,
				 audio_out_ch
				 ));

	qpStatus = QPFWAPI_MailboxReady(This,
									500
									);
	if (!QP_SUCCESS(qpStatus))
		return (qpStatus);

	sysLink.Reg.video_input = video_input;
	sysLink.Reg.video_in_ch = video_in_ch;
	sysLink.Reg.video_output = video_output;
	sysLink.Reg.video_out_ch = video_out_ch;
	sysLink.Reg.audio_input = audio_input;
	sysLink.Reg.audio_in_ch = audio_in_ch;
	sysLink.Reg.audio_output = audio_output;
	sysLink.Reg.audio_out_ch = audio_out_ch;

	qpStatus = QPHCI_RegisterWrite(&This->m_hci,
								   DEC_REG_PARAM_1,
								   sysLink.Read
								   );

#ifdef QP_LINXKER
	PROFILING_LOG(EVENT_FW_SYSTEM_LINK, taskId, 0, 0, 0, video_in_ch);
#endif

	msg.Read = 0;
	msg.Reg.command = CodecSystemLink;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.channel_number = taskId;
	}

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																		   taskId,
																		   &msg,
																		   QP_FALSE,
																		   QP_NULL,
																		   0
																		   );
	QPFWAPI_MailboxDone(This);
	return (qpStatus);
}



EQP_Errors QPFWCODECAPI_SystemClose(CQLCodec* This,
								   TQP_UINT32 taskId
								   )
{
	ARM_MESSAGE	msg;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWCODECAPI_SystemClose() wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	if (!(QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType))
	{
		return (QPERR_SUCCESS);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "QPFWCODECAPI_SystemClose() taskId(%d)\n",
				 taskId
				 ));

#ifdef QP_LINXKER
	PROFILING_LOG(EVENT_FW_SYSTEM_CLOSE, taskId, 0, 0, 0, 0);
#endif

	msg.Read = 0;
	msg.Reg.command = CodecSystemClose;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.channel_number = taskId;
	}

	return (QPFWAPI_SendMessageToARM(This,
									 taskId,
									 &msg,
									 QP_FALSE,
									 QP_NULL,
									 500
									 ));
}
