///////////////////////////////////////////////////////////////////////////////
//     
//
// Fwdecapi.c
//
// Description:
//
//	Firmware decoder API
//
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#include "hci.h"
#include "cqlcodec.h"
#include "fwapi.h"
#include "qpdbgmgr.h"
#include "qposmgr.h"

extern TQP_DbgMCompID	g_QPCodecDBGCompID;


EQP_Errors QPFWDECAPI_GetDecoderVersion(CQLCodec* This,
										TQP_UINT32 taskId,
										PTQP_UINT32 pVersion
										)
{
	ARM_MESSAGE	msg;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWDECAPI_GetDecoderVersion wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	msg.Read = 0;
	msg.Reg.command = GetDecoderVersion;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.channel_number = taskId;
	}

	return (QPFWAPI_SendMessageToARM(This,
									 taskId,
									 &msg,
									 QP_FALSE,
									 QP_NULL,
									 500
									 ));
}


EQP_Errors QPFWDECAPI_StartDecoder(CQLCodec* This,
								   TQP_UINT32 taskId,
								   TQP_UINT32 streamID,
  		  						   TQP_UINT32 outMode,
								   TQP_UINT32 VOUMode,
								   TQP_UINT32 VOUStartPixel,
								   TQP_UINT32 VOUStartLine,
								   TQP_UINT32 AOControls,
								   TQP_UINT32 AudControls
  		  						   )
{
	EQP_Errors			qpStatus;
	ARM_MESSAGE			msg;
	DEC_START_VO_OFFSET	offset;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWDECAPI_StartDecoder wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "QPFWDECAPI_StartDecoder() taskId(%d) strmID(0x%x) outMode(0x%x) VOUMode(%d) start_pixel(%d) start_line(%d) AOControl(0x%x) AudControl(0x%x)\n",
				 taskId,
				 streamID,
  		  		 outMode,
				 VOUMode,
				 VOUStartPixel,
				 VOUStartLine,
				 AOControls,
				 AudControls
				 ));

	qpStatus = QPFWAPI_MailboxReady(This,
									500
									);
	if (!QP_SUCCESS(qpStatus))
		return (qpStatus);

	qpStatus = QPHCI_RegisterWrite(&This->m_hci,
								   DEC_REG_PARAM_1,
								   streamID
								   );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  DEC_REG_PARAM_2,
																	  outMode
																	  );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  DEC_REG_PARAM_3,
																	  VOUMode
																	  );
	offset.Reg.startPixel = VOUStartPixel;
	offset.Reg.startLine = VOUStartLine;

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  DEC_REG_PARAM_4,
																	  offset.Read
																	  );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  DEC_REG_PARAM_5,
																	  AOControls
																	  );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  DEC_REG_PARAM_6,
																	  AudControls
																	  );

	msg.Read = 0;
	msg.Reg.command = StartDecompress;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.channel_number = taskId;
	}

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																		   taskId,
																		   &msg,
																		   QP_FALSE,
																		   QP_NULL,
																		   0
																		   );

	if (QP_SUCCESS(qpStatus))
	{
		This->m_pTask->m_TaskData[taskId].m_decTrickPlaySpeed = QPDEC_TRICK_SPD_NORMAL;
		This->m_pTask->m_TaskData[taskId].m_decTrickPlayDirection = QPDEC_TRICK_DIR_FORWARD;
		This->m_pTask->m_TaskData[taskId].m_decTrickFieldPreference = QPDEC_TRICK_DISP_DEFAULT;
	}

	QPFWAPI_MailboxDone(This);
	return (qpStatus);
}



EQP_Errors QPFWDECAPI_StopDecoder(CQLCodec* This,
								  TQP_UINT32 taskId,
								  TQP_UINT32 stopMode,
  		  						  TQP_UINT32 displayMode,
								  TQP_UINT32 pts1,
								  TQP_UINT32 pts2
								  )
{
	EQP_Errors	qpStatus;
	ARM_MESSAGE	msg;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWDECAPI_StopDecoder wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "QPFWDECAPI_StopDecoder() stopMode(%d) display_mode(%d)\n",
				 stopMode,
  		  		 displayMode
				 ));

	qpStatus = QPFWAPI_MailboxReady(This,
									500
									);
	if (!QP_SUCCESS(qpStatus))
		return (qpStatus);

	qpStatus = QPHCI_RegisterWrite(&This->m_hci,
								   DEC_REG_PARAM_1,
								   stopMode
								   );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  DEC_REG_PARAM_2,
																	  displayMode
																	  );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  DEC_REG_PARAM_3,
																	  pts1
																	  );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  DEC_REG_PARAM_4,
																	  pts2
																	  );

	msg.Read = 0;
	msg.Reg.command = StopDecompress;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.channel_number = taskId;
	}

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																		   taskId,
																		   &msg,
																		   QP_FALSE,
																		   QP_NULL,
																		   0
																		   );
	if (QP_SUCCESS(qpStatus))
	{
		This->m_pTask->m_TaskData[taskId].m_decTrickPlaySpeed = QPDEC_TRICK_SPD_NORMAL;
		This->m_pTask->m_TaskData[taskId].m_decTrickPlayDirection = QPDEC_TRICK_DIR_FORWARD;
		This->m_pTask->m_TaskData[taskId].m_decTrickFieldPreference = QPDEC_TRICK_DISP_DEFAULT;
	}

	QPFWAPI_MailboxDone(This);
	return (qpStatus);
}


EQP_Errors QPFWDECAPI_PauseDisplay(CQLCodec* This,
								   TQP_UINT32 taskId,
								   TQP_UINT32 displayMode
								   )
{
	EQP_Errors	qpStatus;
	ARM_MESSAGE	msg;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWDECAPI_PauseDisplay wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	qpStatus = QPFWAPI_MailboxReady(This,
									500
									);
	if (!QP_SUCCESS(qpStatus))
		return (qpStatus);

	qpStatus = QPHCI_RegisterWrite(&This->m_hci,
								   DEC_REG_PARAM_1,
								   displayMode
								   );
	msg.Read = 0;
	msg.Reg.command = PauseDecDisplay;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.channel_number = taskId;
	}

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																		   taskId,
																		   &msg,
																		   QP_FALSE,
																		   QP_NULL,
																		   0
																		   );
	if (QP_SUCCESS(qpStatus))
	{
		This->m_pTask->m_TaskData[taskId].m_decTrickPlaySpeed = QPDEC_TRICK_SPD_NORMAL;
		This->m_pTask->m_TaskData[taskId].m_decTrickPlayDirection = QPDEC_TRICK_DIR_FORWARD;
		This->m_pTask->m_TaskData[taskId].m_decTrickFieldPreference = QPDEC_TRICK_DISP_DEFAULT;
	}

	QPFWAPI_MailboxDone(This);
	return (qpStatus);
}



EQP_Errors QPFWDECAPI_ResumeDisplay(CQLCodec* This,
									TQP_UINT32 taskId
									)
{
	EQP_Errors	qpStatus;
	ARM_MESSAGE	msg;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWDECAPI_ResumeDisplay wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	msg.Read = 0;
	msg.Reg.command = ResumeDecDisplay;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.channel_number = taskId;
	}

	qpStatus = QPFWAPI_SendMessageToARM(This,
										taskId,
										&msg,
										QP_FALSE,
										QP_NULL,
										500
										);
	if (QP_SUCCESS(qpStatus))
	{
		This->m_pTask->m_TaskData[taskId].m_decTrickPlaySpeed = QPDEC_TRICK_SPD_NORMAL;
		This->m_pTask->m_TaskData[taskId].m_decTrickPlayDirection = QPDEC_TRICK_DIR_FORWARD;
		This->m_pTask->m_TaskData[taskId].m_decTrickFieldPreference = QPDEC_TRICK_DISP_DEFAULT;
	}

	return (qpStatus);
}



EQP_Errors QPFWDECAPI_SetTrickMode(CQLCodec* This,
								   TQP_UINT32 taskId,
								   TQP_UINT32 playSpeed,
								   TQP_UINT32 playDirection,
								   TQP_UINT32 step,
								   TQP_UINT32 PTS,
								   TQP_UINT32 fieldPreference
								   )
{
	EQP_Errors	qpStatus;
	ARM_MESSAGE	msg;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWDECAPI_SetTrickMode wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	qpStatus = QPFWAPI_MailboxReady(This,
									500
									);
	if (!QP_SUCCESS(qpStatus))
		return (qpStatus);

	qpStatus = QPHCI_RegisterWrite(&This->m_hci,
								   DEC_REG_PARAM_1,
								   playSpeed
								   );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  DEC_REG_PARAM_2,
																	  playDirection
																	  );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  DEC_REG_PARAM_3,
																	  step
																	  );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  DEC_REG_PARAM_4,
																	  PTS
																	  );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  DEC_REG_PARAM_5,
																	  fieldPreference
																	  );

	msg.Read = 0;
	msg.Reg.command = SetDecTrickMode;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.channel_number = taskId;
	}

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																		   taskId,
																		   &msg,
																		   QP_FALSE,
																		   QP_NULL,
																		   0
																		   );
	QPFWAPI_MailboxDone(This);
	return (qpStatus);
}



EQP_Errors QPFWDECAPI_Flush(CQLCodec* This,
							TQP_UINT32 taskId,
						    TQP_UINT32 buffer_sel
							)
{
	EQP_Errors	qpStatus;
	ARM_MESSAGE	msg;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWDECAPI_Flush wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "QPFWDECAPI_Flush() buffer_sel(%d)\n",
				 buffer_sel
				 ));

	qpStatus = QPFWAPI_MailboxReady(This,
									500
									);
	if (!QP_SUCCESS(qpStatus))
		return (qpStatus);

	qpStatus = QPHCI_RegisterWrite(&This->m_hci,
			     				   DEC_REG_PARAM_1,
			     				   buffer_sel
			     				   );
	msg.Read = 0;
	msg.Reg.command = DecFlush;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.channel_number = taskId;
	}

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																		   taskId,
																		   &msg,
																		   QP_FALSE,
																		   QP_NULL,
																		   0
																		   );
	QPFWAPI_MailboxDone(This);
	return (qpStatus);
}



EQP_Errors QPFWDECAPI_GetPlayInfo(CQLCodec* This,
								  TQP_UINT32 taskId,
								  TQP_UINT32 type
								  )
{
	EQP_Errors	qpStatus;
	ARM_MESSAGE	msg;
	TQP_UINT32	mask = 0;
	TQP_UINT32	ulEvents;
	TQP_BOOL	bRestore = QP_FALSE;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWDECAPI_GetPlayInfo wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	// get the current interrupt mask
	//
	qpStatus = QPHCI_GetInterruptMask(&This->m_hci,
									  &mask
									  );
	if (!QP_SUCCESS(qpStatus))
		return (qpStatus);

	// enable ARM interrupt if it is not enabled
	//
	if (0 == (mask & HCI_INTERRUPT_ARM))
	{
		bRestore = QP_TRUE;

		// clear any outstanding ARM interrupt
		//
		QPHCI_ClearInterrupt(&This->m_hci,
							 HCI_INTERRUPT_ARM
							 );

		// enable ARM interrupt
		//
		QPHCI_EnableInterrupt(&This->m_hci,
							  HCI_INTERRUPT_ARM
							  );
	}

	// clear the event
	//
	QPOSMClearEvtgrp(This->m_pTask->m_TaskData[taskId].m_EvtReply,
					 QLCODEC_EVENT_DEC_GOT_PLAY_INFO
					 );

	qpStatus = QPFWAPI_MailboxReady(This,
									500
									);
	if (QP_SUCCESS(qpStatus))
	{
		// send the message
		//
		qpStatus = QPHCI_RegisterWrite(&This->m_hci,
									   DEC_REG_PARAM_1,
									   type
									   );
		msg.Read = 0;
		msg.Reg.command = DecGetPlayInfo;
		if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
		{
			msg.RegArtesa.channel_number = taskId;
		}

		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																			   taskId,
																			   &msg,
																			   QP_FALSE,
																			   QP_NULL,
																			   0
																			   );
		QPFWAPI_MailboxDone(This);
	}

	if (QP_SUCCESS(qpStatus))
	{
		// wait for reply
		//
		qpStatus = QPOSMWaitEvtgrp(This->m_pTask->m_TaskData[taskId].m_EvtReply,
								   QLCODEC_EVENT_DEC_GOT_PLAY_INFO,
								   &ulEvents,
								   2000
								   );

		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "QPFWDECAPI_GetPlayInfo() QPOSMWaitEvtgrp() Failed status(%d)\n",
						 qpStatus
						 ));
		}
	}

	// clear Got Shared Memory done
	//
	QPOSMClearEvtgrp(This->m_pTask->m_TaskData[taskId].m_EvtReply,
					 QLCODEC_EVENT_DEC_GOT_PLAY_INFO
					 );

	if (bRestore)
	{
		// restore ARM interrupt
		//
		QPHCI_SetInterruptMask(&This->m_hci,
							   mask
							   );
	}
	return (qpStatus);
}



EQP_Errors QPFWDECAPI_SetDisplayAttrib(CQLCodec* This,
									   TQP_UINT32 taskId,
									   TQP_UINT32 TVSystem,
  		  							   TQP_UINT32 input_size,
  		  							   TQP_UINT32 output_size,
  		  							   TQP_UINT32 input_origin,
  		  							   TQP_UINT32 output_origin
  		  							   )
{
	EQP_Errors	qpStatus;
	ARM_MESSAGE	msg;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWDECAPI_SetDisplayAttrib wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	qpStatus = QPFWAPI_MailboxReady(This,
									500
									);
	if (!QP_SUCCESS(qpStatus))
		return (qpStatus);

	qpStatus = QPHCI_RegisterWrite(&This->m_hci,
			     				   DEC_REG_PARAM_1,
			     				   TVSystem
			     				   );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  DEC_REG_PARAM_2,
																	  input_size
																	  );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  DEC_REG_PARAM_3,
																	  output_size
																	  );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  DEC_REG_PARAM_4,
																	  input_origin
																	  );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  DEC_REG_PARAM_5,
																	  output_origin
																	  );
	msg.Read = 0;
	msg.Reg.command = SetDecDisplayAttrib;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.channel_number = taskId;
	}

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																		   taskId,
																		   &msg,
																		   QP_FALSE,
																		   QP_NULL,
																		   0
																		   );
	QPFWAPI_MailboxDone(This);
	return (qpStatus);
}



EQP_Errors QPFWDECAPI_SetDisplayMode(CQLCodec* This,
									 TQP_UINT32 taskId,
  		  							 TQP_UINT32 operation,
  		  							 TQP_UINT32 displayMode
  		  							 )
{
	EQP_Errors	qpStatus;
	ARM_MESSAGE	msg;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWDECAPI_SetDisplayMode wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	qpStatus = QPFWAPI_MailboxReady(This,
									500
									);
	if (!QP_SUCCESS(qpStatus))
		return (qpStatus);

	qpStatus = QPHCI_RegisterWrite(&This->m_hci,
								   DEC_REG_PARAM_1,
								   operation
								   );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  DEC_REG_PARAM_2,
																	  displayMode
																	  );
	msg.Read = 0;
	msg.Reg.command = SetDecDisplayMode;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.channel_number = taskId;
	}

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																		   taskId,
																		   &msg,
																		   QP_FALSE,
																		   QP_NULL,
																		   0
																		   );
	QPFWAPI_MailboxDone(This);
	return (qpStatus);
}



EQP_Errors QPFWDECAPI_GetVouOsdMem(CQLCodec* This,
								   TQP_UINT32 taskId,
								   TQP_UINT32 command,
								   TQP_UINT32 memsize
								   )
{
	EQP_Errors	qpStatus;
	ARM_MESSAGE	msg;
	TQP_UINT32	ulEvents;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWDECAPI_GetVouOsdMem wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	qpStatus = QPFWAPI_MailboxReady(This,
									500
									);
	if (!QP_SUCCESS(qpStatus))
		return (qpStatus);

	qpStatus = QPHCI_RegisterWrite(&This->m_hci,
								   DEC_REG_PARAM_1,
								   command
								   );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  DEC_REG_PARAM_2,
																	  memsize
																	  );

	msg.Read = 0;
	msg.Reg.command = GetOsdMemAddr;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.channel_number = taskId;
	}

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																		   taskId,
																		   &msg,
																		   QP_FALSE,
																		   QP_NULL,
																		   0
																		   );
	QPFWAPI_MailboxDone(This);

	if (QP_SUCCESS(qpStatus))
	{
		// wait for reply
		//
		qpStatus = QPOSMWaitEvtgrp(This->m_pTask->m_TaskData[taskId].m_EvtReply,
								   QLCODEC_EVENT_DEC_GOT_PLAY_INFO,
								   &ulEvents,
								   2000
								   );

		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "QPFWDECAPI_GetVouOsdMem() QPOSMWaitEvtgrp() Failed status(%d)\n",
						 qpStatus
						 ));
		}
	}

	// clear Got Shared Memory done
	//
	QPOSMClearEvtgrp(This->m_pTask->m_TaskData[taskId].m_EvtReply,
					 QLCODEC_EVENT_DEC_GOT_PLAY_INFO
					 );

	return (qpStatus);
}

EQP_Errors QPFWDECAPI_SetAudioOutputVolume(CQLCodec* This,
									 TQP_UINT32 taskId,
  		  							 TQP_UINT32 vol
  		  							 )
{
	EQP_Errors	qpStatus;
	ARM_MESSAGE	msg;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWDECAPI_SetAudioOutputVolume wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "QPFWDECAPI_SetAudioOutputVolume() vol(%d)\n",
				 vol
				 ));

	qpStatus = QPFWAPI_MailboxReady(This,
									500
									);
	if (!QP_SUCCESS(qpStatus))
		return (qpStatus);

	if (vol > 15)
		vol = 15;

	qpStatus = QPHCI_RegisterWrite(&This->m_hci,
								   DEC_REG_PARAM_1,
								   vol
								   );

	msg.Read = 0;
	msg.Reg.command = setAudioOutputVolume;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.channel_number = taskId;
	}

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																		   taskId,
																		   &msg,
																		   QP_FALSE,
																		   QP_NULL,
																		   0
																		   );
	QPFWAPI_MailboxDone(This);
	return (qpStatus);
}



