///////////////////////////////////////////////////////////////////////////////
//     
//
// Fwencapi.c
//
// Description:
//
//	Firmware encoder API
//
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#include "hci.h"
#include "cqlcodec.h"
#include "fwapi.h"
#include "qpdbgmgr.h"
#include "qposmgr.h"
#include "util.h"
#ifndef QP_LINXKER
#include <string.h>
#endif


extern TQP_DbgMCompID	g_QPCodecDBGCompID;


// HDMI: {VSYNC, HSYNC}, SD: {frame_valid, line_valid}
// (0: positive, 1: negative)
TQP_UINT32	g_syncModeTable[19] =
{
	0,	// 1920x1080i @59.94/60Hz (Format 5)
	0,	// 1920x1080p @59.94/60Hz (Format 16), @29.97/30Hz (Format 34)
	0,	// 1440x1088i @59.94/60Hz
	1,	// 1440x576p @50Hz (Format 29 & 30)
	3,	// 1440x480p @59.94/60Hz (Format 14 & 15)
	0,	// 1280x720p @59.94/60Hz (Format 4)
	3,	// 720(1440)x480i @59.94/60Hz (Format 6 & 7)
	3,	// 720(1440)x576i @59.94/60Hz (Format 21 & 22)
	3,	// 720x480p @59.94/60Hz (Format 2 & 3)
	3,	// 704x576p @50Hz
	3,	// 720x480i @59.94/60Hz
	3,	// 704x576i @50Hz
	3,	// 640x480p @59.94/60Hz (Format 1)
	3,	// 640x480i @60Hz
	0,	// 176x128i @59.94/60Hz (Format CUSTOM_176x128i)
	2,	// HDMI (VSYNC: neg; HSYNC: pos), SD (frame_valid: neg; line_valid: pos)
	3,	// HDMI (VSYNC: neg; HSYNC: neg), SD (frame_valid: neg; line_valid: neg)
	0,	// HDMI (VSYNC: pos; HSYNC: pos), SD (frame_valid: pos; line_valid: pos)
	1	// HDMI (VSYNC: pos; HSYNC: neg), SD (frame_valid: pos; line_valid: neg)
};


/////////////////////////////////////////////////////////////////////////////
//
//


EQP_Errors QPFWENCAPI_StartEncoder(CQLCodec* This,
								   TQP_UINT32 taskId
								   )
{
	ARM_MESSAGE	msg;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_StartEncoder wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "QPFWENCAPI_StartEncoder() taskId(%d)\n",
				 taskId
				 ));

	msg.Read = 0;
	msg.Reg.command = StartCompress;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.channel_number = taskId;
	}

	return (QPFWAPI_SendMessageToARM(This,
									 taskId,
									 &msg,
									 QP_FALSE,
									 QP_NULL,
									 0
									 ));
}



EQP_Errors QPFWENCAPI_StopEncoder(CQLCodec* This,
								  TQP_UINT32 taskId,
								  TQP_BOOL bStopAtGOP,
								  TQP_UINT32 channel_dataType
								  )
{
	EQP_Errors	qpStatus;
	ARM_MESSAGE	msg;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_StopEncoder wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "QPFWENCAPI_StopEncoder() taskId(%d) bStopAtGOP(%d) channel_dataType(%d)\n",
				 taskId,
				 bStopAtGOP,
				 channel_dataType
				 ));

	qpStatus = QPFWAPI_MailboxReady(This,
									500
									);
	if (!QP_SUCCESS(qpStatus))
		return (qpStatus);

	qpStatus = QPHCI_RegisterWrite(&This->m_hci,
								   REG_TO_ARM_1,
								   bStopAtGOP ? 1 : 0
								   );

	if (QPCODEC_CHIP_BERINGER == This->m_ChipType)
	{
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  REG_TO_ARM_2,
																		  channel_dataType
																		  );
	}

	msg.Read = 0;
	msg.Reg.command = StopCompress;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.channel_number = taskId;
	}

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																		   taskId,
																		   &msg,
																		   QP_FALSE,
																		   QP_NULL,
																		   0
																		   );
	QPFWAPI_MailboxDone(This);
	return (qpStatus);
}



EQP_Errors QPFWENCAPI_PauseEncoder(CQLCodec* This,
								   TQP_UINT32 taskId
								   )
{
	ARM_MESSAGE	msg;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_PauseEncoder wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "QPFWENCAPI_PauseEncoder()\n"
				 ));

	msg.Read = 0;
	msg.Reg.command = PauseCompress;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.channel_number = taskId;
	}

	return (QPFWAPI_SendMessageToARM(This,
									 taskId,
									 &msg,
									 QP_FALSE,
									 QP_NULL,
									 500
									 ));
}



EQP_Errors QPFWENCAPI_ResumeEncoder(CQLCodec* This,
									TQP_UINT32 taskId
									)
{
	ARM_MESSAGE	msg;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_ResumeEncoder wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "QPFWENCAPI_ResumeEncoder()\n"
				 ));

	msg.Read = 0;
	msg.Reg.command = ResumeCompress;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.channel_number = taskId;
	}

	return (QPFWAPI_SendMessageToARM(This,
									 taskId,
									 &msg,
									 QP_FALSE,
									 QP_NULL,
									 500
									 ));
}



EQP_Errors QPFWENCAPI_GetEncoderVersion(CQLCodec* This,
										TQP_UINT32 taskId,
										PTQP_UINT32 pVersion
										)
{
	return (QPERR_SUCCESS);
}



EQP_Errors QPFWENCAPI_GetEncoderDbgInfo(CQLCodec* This,
										TQP_UINT32 taskId,
										TQP_UINT32 type
										)
{
	EQP_Errors	qpStatus;
	ARM_MESSAGE	msg;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_GetEncoderDbgInfo wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "QPFWENCAPI_GetEncoderDbgInfo() type(%d)\n",
				 type
				 ));

	qpStatus = QPFWAPI_MailboxReady(This,
									500
									);
	if (!QP_SUCCESS(qpStatus))
		return (qpStatus);

	qpStatus = QPHCI_RegisterWrite(&This->m_hci,
								   REG_TO_ARM_1,
								   type
								   );

	msg.Read = 0;
	msg.Reg.command = GetCmprDbgInfo;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.channel_number = taskId;
	}

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																		   taskId,
																		   &msg,
																		   QP_FALSE,
																		   QP_NULL,
																		   0
																		   );
	QPFWAPI_MailboxDone(This);
	return (qpStatus);
}



EQP_Errors QPFWENCAPI_SetSystemControl(CQLCodec* This,
									   TQP_UINT32 value
									   )
{
	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetSystemControl wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "QPFWENCAPI_SetSystemControl     reg (0x%04x) : (0x%08x)\n",
				 This->m_ENC_REG_SYSTEM_CONTROL,
				 value
				 ));

	return (QPHCI_RegisterWrite(&This->m_hci,
								This->m_ENC_REG_SYSTEM_CONTROL,
								value
								));
}



EQP_Errors QPFWENCAPI_SetPictureResolution(CQLCodec* This,
										   TQP_UINT32 value
  		  								   )
{
	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetPictureResolution wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "QPFWENCAPI_SetPictureResolution reg (0x%04x) : (0x%08x)\n",
				 This->m_ENC_REG_PICTURE_RESOLUTION,
				 value
				 ));

	return (QPHCI_RegisterWrite(&This->m_hci,
								This->m_ENC_REG_PICTURE_RESOLUTION,
								value
								));
}



EQP_Errors QPFWENCAPI_SetInputControl(CQLCodec* This,
									  TQP_UINT32 value
									  )
{
	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetInputControl wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "QPFWENCAPI_SetInputControl      reg (0x%04x) : (0x%08x)\n",
				 This->m_ENC_REG_INPUT_CONTROL,
				 value
				 ));

	return (QPHCI_RegisterWrite(&This->m_hci,
								This->m_ENC_REG_INPUT_CONTROL,
								value
								));
}



EQP_Errors QPFWENCAPI_SetRateControl(CQLCodec* This,
									 TQP_UINT32 value
  		  							 )
{
	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetRateControl wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "QPFWENCAPI_SetRateControl       reg (0x%04x) : (0x%08x)\n",
				 This->m_ENC_REG_RATE_CONTROL,
				 value
				 ));

	return (QPHCI_RegisterWrite(&This->m_hci,
								This->m_ENC_REG_RATE_CONTROL,
								value
								));
}



EQP_Errors QPFWENCAPI_SetVBRBitRate(CQLCodec* This,
  		  							TQP_UINT32 value
  		  							)
{
	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetVBRBitRate wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "QPFWENCAPI_SetVBRBitRate        reg (0x%04x) : (0x%08x)\n",
				 This->m_ENC_REG_BIT_RATE,
				 value
				 ));

	return (QPHCI_RegisterWrite(&This->m_hci,
								This->m_ENC_REG_BIT_RATE,
								value
								));
}



EQP_Errors QPFWENCAPI_SetFilterControl(CQLCodec* This,
									   TQP_UINT32 value
  		  							   )
{
	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetFilterControl wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "QPFWENCAPI_SetFilterControl     reg (0x%04x) : (0x%08x)\n",
				 This->m_ENC_REG_FILTER_CONTROL,
				 value
				 ));

	return (QPHCI_RegisterWrite(&This->m_hci,
								This->m_ENC_REG_FILTER_CONTROL,
								value
								));
}



EQP_Errors QPFWENCAPI_SetGOPLoopFilter(CQLCodec* This,
  		  							   TQP_UINT32 value
  		  							   )
{
	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetGOPLoopFilter wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "QPFWENCAPI_SetGOPLoopFilter     reg (0x%04x) : (0x%08x)\n",
				 This->m_ENC_REG_GOP_LOOP_FILTER,
				 value
				 ));

	return (QPHCI_RegisterWrite(&This->m_hci,
								This->m_ENC_REG_GOP_LOOP_FILTER,
								value
								));
}



EQP_Errors QPFWENCAPI_SetETControl(CQLCodec* This,
  		  						   TQP_UINT32 value
  		  						   )
{
	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetETControl wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		return (QPERR_SUCCESS);
	}
	else
	{

		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetETControl         reg (0x%04x) : (0x%08x)\n",
					 This->m_ENC_REG_ET_CONTROL,
					 value
					 ));

		return (QPHCI_RegisterWrite(&This->m_hci,
									This->m_ENC_REG_ET_CONTROL,
									value
									));
	}
}



EQP_Errors QPFWENCAPI_SetBlockSize(CQLCodec* This,
								   TQP_UINT32 value
  		  						   )
{
	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetBlockSize wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetBlockSize  reg (0x%04x) : (0x%08x)\n",
					 This->m_ENC_REG_BLOCK_SIZE,
					 value
					 ));

		return (QPHCI_RegisterWrite(&This->m_hci,
									This->m_ENC_REG_BLOCK_SIZE,
									value
									));
	}
	else
	{
		return (QPERR_SUCCESS);
	}
}



EQP_Errors QPFWENCAPI_SetOutPictureResolution(CQLCodec* This,
											  TQP_UINT32 value
  		  									  )
{
	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetOutPictureResolution wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetOutPicResolution  reg (0x%04x) : (0x%08x)\n",
					 This->m_ENC_REG_OUT_PIC_RESOLUTION,
					 value
					 ));

		return (QPHCI_RegisterWrite(&This->m_hci,
									This->m_ENC_REG_OUT_PIC_RESOLUTION,
									value
									));
	}
	else
	{
		return (QPERR_SUCCESS);
	}
}



EQP_Errors QPFWENCAPI_SetAudioControlParameters(CQLCodec* This,
  		  										TQP_UINT32 value
  		  										)
{
	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetAudioControlParameters wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "QPFWENCAPI_SetAudioControl      reg (0x%04x) : (0x%08x)\n",
				 This->m_ENC_REG_AUDIO_CONTROL_PARAM,
				 value
				 ));

	return (QPHCI_RegisterWrite(&This->m_hci,
								This->m_ENC_REG_AUDIO_CONTROL_PARAM,
								value
								));
}



EQP_Errors QPFWENCAPI_SetAudioControlExtension(CQLCodec* This,
  		  									   TQP_UINT32 value
  		  									   )
{
	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetAudioControlExtension wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "QPFWENCAPI_SetAudioControlEx    reg (0x%04x) : (0x%08x)\n",
				 This->m_ENC_REG_AUDIO_CONTROL_EX,
				 value
				 ));

	return (QPHCI_RegisterWrite(&This->m_hci,
								This->m_ENC_REG_AUDIO_CONTROL_EX,
								value
								));
}



// This function is always called with mailbox semaphore
// grabbed already
//
EQP_Errors QPFWENCAPI_UpdateConfig(CQLCodec* This,
								   TQP_UINT32 taskId
								   )
{
	ARM_MESSAGE	msg;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_UpdateConfig wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	msg.Read = 0;
	msg.Reg.command = UpdateCmprConfig;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.channel_number = taskId;
	}

	return (QPFWAPI_SendMessageToARM(This,
									 taskId,
									 &msg,
									 QP_FALSE,
									 QP_NULL,
									 0
									 ));
}

EQP_Errors QPFWENCAPI_SetVIOSD(CQLCodec* This,
										   TQP_UINT32 taskId,
										   TQP_UINT32 pbVIOSDen
										   )
{
	EQP_Errors	qpStatus;
	ARM_MESSAGE	msg;
	TQP_UINT32	mask = 0;
	//TQP_UINT32	ulEvents;
	TQP_BOOL	bRestore = QP_FALSE;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "%s() -> wrong power state(%d)\n",__FUNCTION__,
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	// get the current interrupt mask
	//
	qpStatus = QPHCI_GetInterruptMask(&This->m_hci,
									  &mask
									  );
	if (!QP_SUCCESS(qpStatus))
		return (qpStatus);

	// enable ARM interrupt if it is not enabled
	//
	if (0 == (mask & HCI_INTERRUPT_ARM))
	{
		bRestore = QP_TRUE;

		// clear any outstanding ARM interrupt
		//
		QPHCI_ClearInterrupt(&This->m_hci,
							 HCI_INTERRUPT_ARM
							 );

		// enable ARM interrupt
		//
		QPHCI_EnableInterrupt(&This->m_hci,
							  HCI_INTERRUPT_ARM
							  );
	}

	qpStatus = QPFWAPI_MailboxReady(This,
									500
									);
	if (!QP_SUCCESS(qpStatus))
		return (qpStatus);

		// send the message
		//
	qpStatus = QPHCI_RegisterWrite(&This->m_hci,
								   REG_TO_ARM_1,
								   EncMiscOSDSettings
								   );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  REG_TO_ARM_2,
																		  pbVIOSDen
																		  );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  REG_TO_ARM_3,
																		  1
																		  );

	msg.Read = 0;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.command = encMiscMessage;
		msg.RegArtesa.channel_number = taskId;
	}
	else
	{
		msg.Reg.command = EncMiscMessage;
	}


	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																		   taskId,
																		   &msg,
																		   QP_FALSE,
																		   QP_NULL,
																		   0
																		   );
	QPFWAPI_MailboxDone(This);

	if (bRestore)
	{
		// restore ARM interrupt
		//
		QPHCI_SetInterruptMask(&This->m_hci,
							   mask
							   );
	}

	return (qpStatus);
}
EQP_Errors QPFWENCAPI_GetViosdTableaddr(CQLCodec* This,
										   TQP_UINT32 taskId,
										   PTQP_UINT32 pFontTableAddr,	// in dwords
										   PTQP_UINT32 pTextListAddr,	// in dwords
										   PTQP_UINT32 pSyncTimeAddr	// in dwords
										   )
{
	EQP_Errors	qpStatus;
	ARM_MESSAGE	msg;
	TQP_UINT32	mask = 0;
	TQP_UINT32	ulEvents;
	TQP_BOOL	bRestore = QP_FALSE;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_GetViosdTableaddr wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	// get the current interrupt mask
	//
	qpStatus = QPHCI_GetInterruptMask(&This->m_hci,
									  &mask
									  );
	if (!QP_SUCCESS(qpStatus))
		return (qpStatus);

	// enable ARM interrupt if it is not enabled
	//
	if (0 == (mask & HCI_INTERRUPT_ARM))
	{
		bRestore = QP_TRUE;

		// clear any outstanding ARM interrupt
		//
		QPHCI_ClearInterrupt(&This->m_hci,
							 HCI_INTERRUPT_ARM
							 );

		// enable ARM interrupt
		//
		QPHCI_EnableInterrupt(&This->m_hci,
							  HCI_INTERRUPT_ARM
							  );
	}

	// clear the event
	//
	QPOSMClearEvtgrp(This->m_pTask->m_TaskData[taskId].m_EvtReply,
					 QLCODEC_EVENT_ENC_GOT_VIOSD_TABLE
					 );

	qpStatus = QPFWAPI_MailboxReady(This,
									500
									);
	if (QP_SUCCESS(qpStatus))
	{
		// send the message
		//
		qpStatus = QPHCI_RegisterWrite(&This->m_hci,
									   REG_TO_ARM_1,
									   0
									   );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  REG_TO_ARM_2,
																		  0
																		  );

		msg.Read = 0;
		msg.Reg.command = GetViosdTableAddr;
		if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
		{
			msg.RegArtesa.channel_number = taskId;
		}

		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																			   taskId,
																			   &msg,
																			   QP_FALSE,
																			   QP_NULL,
																			   0
																			   );
		QPFWAPI_MailboxDone(This);
	}

	if (QP_SUCCESS(qpStatus))
	{
		// wait for reply
		//
		qpStatus = QPOSMWaitEvtgrp(This->m_pTask->m_TaskData[taskId].m_EvtReply,
								   QLCODEC_EVENT_ENC_GOT_VIOSD_TABLE,
								   &ulEvents,
								   100
								   );

		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "QPFWENCAPI_GetViosdTableaddr() QPOSMWaitEvtgrp() Failed status(%d)\n",
						 qpStatus
						 ));
		}
		else
		{
			*pFontTableAddr = This->m_pTask->m_TaskData[taskId].m_encFontTableAddr;
			*pTextListAddr = This->m_pTask->m_TaskData[taskId].m_encTextListAddr;
			*pSyncTimeAddr = This->m_pTask->m_TaskData[taskId].m_encSyncTimeAddr;
		}
	}

	// clear got_font_text_table
	//
	QPOSMClearEvtgrp(This->m_pTask->m_TaskData[taskId].m_EvtReply,
					 QLCODEC_EVENT_ENC_GOT_VIOSD_TABLE
					 );

	if (bRestore)
	{
		// restore ARM interrupt
		//
		QPHCI_SetInterruptMask(&This->m_hci,
							   mask
							   );
	}
	return (qpStatus);
}



EQP_Errors QPFWENCAPI_GetCurVidBufInfo(CQLCodec* This,
									   TQP_UINT32 taskId,
									   TQP_UINT32 holdBuffer,
									   PTQP_UINT32 pVidBufLumaAddr,		// in dwords
									   PTQP_UINT32 pVidBufChromaAddr,	// in dwords
									   PTQP_UINT32 pFrameLABufAddr,		// in dwords
									   PTQP_UINT32 pTopLABufAddr,		// in dwords
									   PTQP_UINT32 pBottomLABufAddr,	// in dwords
									   PTQP_UINT32 pVidBufPTS			// right shift 2 bits
									   )
{
	EQP_Errors	qpStatus;
	ARM_MESSAGE	msg;
	TQP_UINT32	mask = 0;
	TQP_UINT32	ulEvents;
	TQP_BOOL	bRestore = QP_FALSE;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_GetCurVidBufInfo wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	// get the current interrupt mask
	//
	qpStatus = QPHCI_GetInterruptMask(&This->m_hci,
									  &mask
									  );
	if (!QP_SUCCESS(qpStatus))
		return (qpStatus);

	// enable ARM interrupt if it is not enabled
	//
	if (0 == (mask & HCI_INTERRUPT_ARM))
	{
		bRestore = QP_TRUE;

		// clear any outstanding ARM interrupt
		//
		QPHCI_ClearInterrupt(&This->m_hci,
							 HCI_INTERRUPT_ARM
							 );

		// enable ARM interrupt
		//
		QPHCI_EnableInterrupt(&This->m_hci,
							  HCI_INTERRUPT_ARM
							  );
	}

	// clear the event
	//
	QPOSMClearEvtgrp(This->m_pTask->m_TaskData[taskId].m_EvtReply,
					 QLCODEC_EVENT_ENC_GOT_CUR_VID_BUF
					 );

	qpStatus = QPFWAPI_MailboxReady(This,
									500
									);
	if (QP_SUCCESS(qpStatus))
	{
		// send the message
		//
		qpStatus = QPHCI_RegisterWrite(&This->m_hci,
									   REG_TO_ARM_1,
									   holdBuffer
									   );

		msg.Read = 0;
		msg.Reg.command = GetCurVidBuf;
		if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
		{
			msg.RegArtesa.channel_number = taskId;
		}

		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																			   taskId,
																			   &msg,
																			   QP_FALSE,
																			   QP_NULL,
																			   0
																			   );
		QPFWAPI_MailboxDone(This);
	}

	if (QP_SUCCESS(qpStatus))
	{
		// wait for reply
		//
		qpStatus = QPOSMWaitEvtgrp(This->m_pTask->m_TaskData[taskId].m_EvtReply,
								   QLCODEC_EVENT_ENC_GOT_CUR_VID_BUF,
								   &ulEvents,
								   2000
								   );

		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "QPFWENCAPI_GetCurVidBufInfo() QPOSMWaitEvtgrp() Failed status(%d)\n",
						 qpStatus
						 ));
		}
		else
		{
			if (-1 == This->m_pTask->m_TaskData[taskId].m_encVidBufLumaAddr)
			{
				qpStatus = QPERR_FAIL;
			}
			else
			{
				*pVidBufLumaAddr = This->m_pTask->m_TaskData[taskId].m_encVidBufLumaAddr;
				*pVidBufChromaAddr = This->m_pTask->m_TaskData[taskId].m_encVidBufChromaAddr;
				*pFrameLABufAddr = This->m_pTask->m_TaskData[taskId].m_encFrameLABufAddr;
				*pTopLABufAddr = This->m_pTask->m_TaskData[taskId].m_encTopLABufAddr;
				*pBottomLABufAddr = This->m_pTask->m_TaskData[taskId].m_encBottomLABufAddr;
				*pVidBufPTS = This->m_pTask->m_TaskData[taskId].m_encVidBufPTS;
			}
		}
	}

	// clear got_cur_vid_buf
	//
	QPOSMClearEvtgrp(This->m_pTask->m_TaskData[taskId].m_EvtReply,
					 QLCODEC_EVENT_ENC_GOT_CUR_VID_BUF
					 );

	if (bRestore)
	{
		// restore ARM interrupt
		//
		QPHCI_SetInterruptMask(&This->m_hci,
							   mask
							   );
	}
	return (qpStatus);
}



EQP_Errors QPFWENCAPI_ReleaseCurBuf(CQLCodec* This,
									TQP_UINT32 taskId
									)
{
	ARM_MESSAGE	msg;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_ReleaseCurBuf wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	msg.Read = 0;
	msg.Reg.command = RelCurVidBuf;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.channel_number = taskId;
	}

	return (QPFWAPI_SendMessageToARM(This,
									 taskId,
									 &msg,
									 QP_FALSE,
									 QP_NULL,
									 500
									 ));
}



EQP_Errors QPFWENCAPI_SetAudioInputVolume(CQLCodec* This,
										  TQP_UINT32 taskId,
										  TQP_UINT32 vol
										  )
{
	EQP_Errors	qpStatus;
	ARM_MESSAGE	msg;

	if (QPCODEC_CHIP_BERINGER == This->m_ChipType)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetAudioInputVolume unsupported in QL2xx\n"
					 ));
		return (QPERR_NOTSUPP);
	}

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetAudioInputVolume wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "QPFWENCAPI_SetAudioInputVolume() vol(%d)\n",
				 vol
				 ));

	if (vol>15)
		vol = 15;

	qpStatus = QPFWAPI_MailboxReady(This,
									500
									);
	if (!QP_SUCCESS(qpStatus))
		return (qpStatus);

	qpStatus = QPHCI_RegisterWrite(&This->m_hci,
								   REG_TO_ARM_1,
								   vol
								   );

	msg.Read = 0;
	msg.Reg.command = setAudioInputVolume;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.channel_number = taskId;
	}

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																		   taskId,
																		   &msg,
																		   QP_FALSE,
																		   QP_NULL,
																		   0
																		   );
	QPFWAPI_MailboxDone(This);
	return (qpStatus);
}



EQP_Errors QPFWENCAPI_InsertIFrame(CQLCodec* This,
								   TQP_UINT32 taskId
								   )
{
	ARM_MESSAGE	msg;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_InsertIFrame wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	msg.Read = 0;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.command = insertIFrame;
		msg.RegArtesa.channel_number = taskId;
	}
	else
	{
		msg.Reg.command = InsertIFrame;
	}


	return (QPFWAPI_SendMessageToARM(This,
									 taskId,
									 &msg,
									 QP_FALSE,
									 QP_NULL,
									 500
									 ));
}



EQP_Errors QPFWENCAPI_InsertUserData(CQLCodec* This,
									 TQP_UINT32 taskId,
									 TQP_UINT32 numUserData,
									 PTQP_UINT8 pUserData
									 )
{
	EQP_Errors	qpStatus;
	ARM_MESSAGE	msg;
	TQP_UINT32	buf[8];
	TQP_UINT32	i, numDwords;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_InsertUserData wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "QPFWENCAPI_InsertUserData() numUserData(%d)\n",
				 numUserData
				 ));
	if (numUserData > 32)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_InsertUserData wrong size(%d)\n",
					 numUserData
					 ));
		return (QPERR_PARMS);
	}

	numDwords = (numUserData + 3) >> 2;

	memcpy(&buf[0],
		   pUserData,
		   numUserData
		   );

	qpStatus = QPFWAPI_MailboxReady(This,
									500
									);
	if (!QP_SUCCESS(qpStatus))
		return (qpStatus);

	qpStatus = QPHCI_RegisterWrite(&This->m_hci,
								   REG_TO_ARM_1,
								   numUserData
								   );

	for (i = 0; i < numDwords; i++)
	{
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  (TQP_UINT16)(REG_TO_ARM_2 + (i * 4)),
																		  buf[i]
																		  );
	}

	msg.Read = 0;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.command = insertUserData;
		msg.RegArtesa.channel_number = taskId;
	}
	else
	{
		msg.Reg.command = InsertUserData;
	}

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																		   taskId,
																		   &msg,
																		   QP_FALSE,
																		   QP_NULL,
																		   0
																		   );
	QPFWAPI_MailboxDone(This);
	return (qpStatus);
}



EQP_Errors QPFWENCAPI_SetIndexCapture(CQLCodec* This,
									  TQP_UINT32 taskId,
									  TQP_UINT32 freq
									  )
{
	EQP_Errors	qpStatus;
	ARM_MESSAGE	msg;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetIndexCapture wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "QPFWENCAPI_SetIndexCapture() freq(%d)\n",
				 freq
				 ));

	qpStatus = QPFWAPI_MailboxReady(This,
									500
									);
	if (!QP_SUCCESS(qpStatus))
		return (qpStatus);

	qpStatus = QPHCI_RegisterWrite(&This->m_hci,
								   REG_TO_ARM_1,
								   EncMiscSetIndex
								   );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  REG_TO_ARM_2,
																	  freq	//0 -- disable,
																			// any other number - power of 2 up to 32, ----- 1, 2, 4, 8, 16, 32
																	  );

	msg.Read = 0;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.command = encMiscMessage;
		msg.RegArtesa.channel_number = taskId;
	}
	else
	{
		msg.Reg.command = EncMiscMessage;
	}

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																		   taskId,
																		   &msg,
																		   QP_FALSE,
																		   QP_NULL,
																		   0
																		   );
	QPFWAPI_MailboxDone(This);
	return (qpStatus);
}



EQP_Errors QPFWENCAPI_SetViuSyncCode(CQLCodec* This,
									 TQP_UINT32 taskId,
									 TQP_UINT32 code1,
  		  							 TQP_UINT32 code2
  		  							 )
{
	EQP_Errors	qpStatus;
	ARM_MESSAGE	msg;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetViuSyncCode wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "QPFWENCAPI_SetViuSyncCode() (0x%x) (0x%x)\n",
				 code1,
				 code2
				 ));

	qpStatus = QPFWAPI_MailboxReady(This,
									500
									);
	if (!QP_SUCCESS(qpStatus))
		return (qpStatus);

	qpStatus = QPHCI_RegisterWrite(&This->m_hci,
								   REG_TO_ARM_1,
								   EncMiscSetSyncCode	// set sync code
								   );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  REG_TO_ARM_2,
																	  code1
																	  );

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  REG_TO_ARM_3,
																	  code2
																	  );

	msg.Read = 0;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.command = encMiscMessage;
		msg.RegArtesa.channel_number = taskId;
	}
	else
	{
		msg.Reg.command = EncMiscMessage;
	}

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																		   taskId,
																		   &msg,
																		   QP_FALSE,
																		   QP_NULL,
																		   0
																		   );
	QPFWAPI_MailboxDone(This);
	return (qpStatus);
}



EQP_Errors QPFWENCAPI_SetMP4VideoBlockNumber(CQLCodec* This,
											 TQP_UINT32 taskId,
											 TQP_UINT32 num
											 )
{
	EQP_Errors	qpStatus;
	ARM_MESSAGE	msg;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetMP4VideoBlockNumber wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "QPFWENCAPI_SetMP4VideoBlockNumber() num(%d)\n",
				 num
				 ));

	qpStatus = QPFWAPI_MailboxReady(This,
									500
									);
	if (!QP_SUCCESS(qpStatus))
		return (qpStatus);

	qpStatus = QPHCI_RegisterWrite(&This->m_hci,
								   REG_TO_ARM_1,
								   EncMiscSetMP4VidFrame	// set video block number
								   );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  REG_TO_ARM_2,
																	  num	// 1 - 10
																	  );

	msg.Read = 0;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.command = encMiscMessage;
		msg.RegArtesa.channel_number = taskId;
	}
	else
	{
		msg.Reg.command = EncMiscMessage;
	}

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																		   taskId,
																		   &msg,
																		   QP_FALSE,
																		   QP_NULL,
																		   0
																		   );
	QPFWAPI_MailboxDone(This);
	return (qpStatus);
}



EQP_Errors QPFWENCAPI_SetAudioEnhancement(CQLCodec* This,
										  TQP_UINT32 taskId,
										  TQP_UINT32 gain1,
										  TQP_UINT32 gain2,
										  TQP_UINT32 add,
										  TQP_UINT32 sub,
										  TQP_UINT32 att1,
										  TQP_UINT32 att2,
										  TQP_UINT32 lgain,
										  TQP_UINT32 rgain
										  )
{
	EQP_Errors	qpStatus;
	ARM_MESSAGE	msg;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetAudioEnhancement wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "QPFWENCAPI_SetAudioEnhancement() gain1(%d) gain2(%d) add(%d) sub(%d) att1(%d) att2(%d) lgain(%d) rgain(%d)\n",
				 gain1,
				 gain2,
				 add,
				 sub,
				 att1,
				 att2,
				 lgain,
				 rgain
				 ));

	qpStatus = QPFWAPI_MailboxReady(This,
									500
									);
	if (!QP_SUCCESS(qpStatus))
		return (qpStatus);

	qpStatus = QPHCI_RegisterWrite(&This->m_hci,
								   REG_TO_ARM_1,
								   EncMiscAudioEnhancement	// set audio enhancement
								   );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  REG_TO_ARM_2,
																	  gain1
																	  );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  REG_TO_ARM_3,
																	  gain2
																	  );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  REG_TO_ARM_4,
																	  add
																	  );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  REG_TO_ARM_5,
																	  sub
																	  );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  REG_TO_ARM_6,
																	  att1
																	  );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  REG_TO_ARM_7,
																	  att2
																	  );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  REG_TO_ARM_8,
																	  lgain
																	  );

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  REG_TO_ARM_9,
																	  rgain
																	  );


	msg.Read = 0;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.command = encMiscMessage;
		msg.RegArtesa.channel_number = taskId;
	}
	else
	{
		msg.Reg.command = EncMiscMessage;
	}

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																		   taskId,
																		   &msg,
																		   QP_FALSE,
																		   QP_NULL,
																		   0
																		   );
	QPFWAPI_MailboxDone(This);
	return (qpStatus);
}



EQP_Errors QPFWENCAPI_EnableVidPadding(CQLCodec* This,
									   TQP_UINT32 taskId,
									   TQP_UINT32 enable
  		  							   )
{
	EQP_Errors	qpStatus;
	ARM_MESSAGE	msg;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_EnableVidPadding wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "QPFWENCAPI_EnableVidPadding() (%d)\n",
				 enable
				 ));

	qpStatus = QPFWAPI_MailboxReady(This,
									500
									);
	if (!QP_SUCCESS(qpStatus))
		return (qpStatus);

	qpStatus = QPHCI_RegisterWrite(&This->m_hci,
								   REG_TO_ARM_1,
								   EncMiscEnableVidPadding	// enable padding
								   );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  REG_TO_ARM_2,
																	  enable
																	  );

	msg.Read = 0;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.command = encMiscMessage;
		msg.RegArtesa.channel_number = taskId;
	}
	else
	{
		msg.Reg.command = EncMiscMessage;
	}

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																		   taskId,
																		   &msg,
																		   QP_FALSE,
																		   QP_NULL,
																		   0
																		   );
	QPFWAPI_MailboxDone(This);
	return (qpStatus);
}



EQP_Errors QPFWENCAPI_EnableVUIParam(CQLCodec* This,
									 TQP_UINT32 taskId,
									 TQP_UINT32 enable,
									 TQP_UINT32 aspect_ratio
  		  							 )
{
	EQP_Errors	qpStatus;
	ARM_MESSAGE	msg;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_EnableVUIParam wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "QPFWENCAPI_EnableVUIParam() enable(%d) aspect_ratio(%d)\n",
				 enable,
				 aspect_ratio
				 ));

	qpStatus = QPFWAPI_MailboxReady(This,
									500
									);
	if (!QP_SUCCESS(qpStatus))
		return (qpStatus);

	qpStatus = QPHCI_RegisterWrite(&This->m_hci,
								   REG_TO_ARM_1,
								   EncMiscEnableVUIParameters	// enable VUI parameter
								   );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  REG_TO_ARM_2,
																	  enable
																	  );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  REG_TO_ARM_3,
																	  aspect_ratio
																	  );


	msg.Read = 0;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.command = encMiscMessage;
		msg.RegArtesa.channel_number = taskId;
	}
	else
	{
		msg.Reg.command = EncMiscMessage;
	}

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																		   taskId,
																		   &msg,
																		   QP_FALSE,
																		   QP_NULL,
																		   0
																		   );
	QPFWAPI_MailboxDone(This);
	return (qpStatus);
}



EQP_Errors QPFWENCAPI_FreezeVideo(CQLCodec* This,
								  TQP_UINT32 taskId,
								  TQP_UINT32 enable
  		  						  )
{
	EQP_Errors	qpStatus;
	ARM_MESSAGE	msg;
	TQP_UINT32	ulEvents;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_FreezeVideo wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	if (TASK_STATE_RUN != CTask_GetTaskState(This->m_pTask,
											 taskId
											 ))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_FreezeVideo wrong task state(%d)\n",
					 This->m_pTask->m_TaskData[taskId].m_State,
					 CTask_GetTaskState(This->m_pTask, taskId)
					 ));
		return (QPERR_INVALID);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "QPFWENCAPI_FreezeVideo() (%d)\n",
				 enable
				 ));

	// clear the event
	//
	QPOSMClearEvtgrp(This->m_pTask->m_TaskData[taskId].m_EvtReply,
					 QLCODEC_EVENT_ENC_VIDEO_FROZEN
					 );

	qpStatus = QPFWAPI_MailboxReady(This,
									500
									);
	if (!QP_SUCCESS(qpStatus))
		return (qpStatus);

	qpStatus = QPHCI_RegisterWrite(&This->m_hci,
								   REG_TO_ARM_1,
								   EncMiscFreezeVideo	// freeze video
								   );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  REG_TO_ARM_2,
																	  enable
																	  );

	msg.Read = 0;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.command = encMiscMessage;
		msg.RegArtesa.channel_number = taskId;
	}
	else
	{
		msg.Reg.command = EncMiscMessage;
	}

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																		   taskId,
																		   &msg,
																		   QP_TRUE,
																		   QP_NULL,
																		   0
																		   );
	QPFWAPI_MailboxDone(This);

	if (QP_SUCCESS(qpStatus))
	{
		// wait for reply
		//
		qpStatus = QPOSMWaitEvtgrp(This->m_pTask->m_TaskData[taskId].m_EvtReply,
								   QLCODEC_EVENT_ENC_VIDEO_FROZEN,
								   &ulEvents,
								   2000
								   );

		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "QPFWENCAPI_FreezeVideo() QPOSMWaitEvtgrp() Failed status(%d)\n",
						 qpStatus
						 ));
		}
	}

	// clear the event
	//
	QPOSMClearEvtgrp(This->m_pTask->m_TaskData[taskId].m_EvtReply,
					 QLCODEC_EVENT_ENC_VIDEO_FROZEN
					 );

	return (qpStatus);
}





EQP_Errors QPFWENCAPI_SetVBIInfo(CQLCodec* This,
								 TQP_UINT32 taskId,
								 TQP_UINT32 enable,
  		  						 TQP_UINT32 top_start_line,
  		  						 TQP_UINT32 top_num_lines,
  		  						 TQP_UINT32 top_start_pixel,
  		  						 TQP_UINT32 top_num_samples,
  		  						 TQP_UINT32 bottom_start_line,
  		  						 TQP_UINT32 bottom_num_lines,
  		  						 TQP_UINT32 bottom_start_pixel,
  		  						 TQP_UINT32 bottom_num_samples
  		  						 )
{
	EQP_Errors	qpStatus;
	ARM_MESSAGE	msg;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetVBIInfo() wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "QPFWENCAPI_SetVBIInfo() enable(%d)\n top(start line(%d) lines(%d) start pixel(%d) samples(%d))\n bottom(start line(%d) lines(%d) start pixel(%d) samples(%d))\n",
				 enable,
  		  		 top_start_line,
  		  		 top_num_lines,
  		  		 top_start_pixel,
  		  		 top_num_samples,
  		  		 bottom_start_line,
  		  		 bottom_num_lines,
  		  		 bottom_start_pixel,
  		  		 bottom_num_samples
				 ));

	qpStatus = QPFWAPI_MailboxReady(This,
									500
									);
	if (!QP_SUCCESS(qpStatus))
		return (qpStatus);

	qpStatus = QPHCI_RegisterWrite(&This->m_hci,
								   REG_TO_ARM_1,
								   EncMiscVBISettings	// VBI settings
								   );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  REG_TO_ARM_2,
																	  enable
																	  );

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  REG_TO_ARM_3,
																	  ((top_start_line << 16) & 0xFFFF0000) | (top_num_lines & 0xFFFF)
																	  );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  REG_TO_ARM_4,
																	  ((top_start_pixel << 16) & 0xFFFF0000) | (top_num_samples & 0xFFFF)
																	  );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  REG_TO_ARM_5,
																	  ((bottom_start_line << 16) & 0xFFFF0000) | (bottom_num_lines & 0xFFFF)
																	  );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  REG_TO_ARM_6,
																	  ((bottom_start_pixel << 16) & 0xFFFF0000) | (bottom_num_samples & 0xFFFF)
																	  );


	msg.Read = 0;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.command = encMiscMessage;
		msg.RegArtesa.channel_number = taskId;
	}
	else
	{
		msg.Reg.command = EncMiscMessage;
	}

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																		   taskId,
																		   &msg,
																		   QP_FALSE,
																		   QP_NULL,
																		   0
																		   );
	QPFWAPI_MailboxDone(This);
	return (qpStatus);
}



EQP_Errors QPFWENCAPI_StillVideoInput(CQLCodec* This,
									  TQP_UINT32 taskId,
									  TQP_UINT32 enable
  		  							  )
{
	EQP_Errors	qpStatus;
	ARM_MESSAGE	msg;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_StillVideoInput wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "QPFWENCAPI_StillVideoInput() (%d)\n",
				 enable
				 ));

	qpStatus = QPFWAPI_MailboxReady(This,
									500
									);
	if (!QP_SUCCESS(qpStatus))
		return (qpStatus);

	qpStatus = QPHCI_RegisterWrite(&This->m_hci,
								   REG_TO_ARM_1,
								   EncMiscStillVideoInput	// still video input
								   );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  REG_TO_ARM_2,
																	  enable
																	  );

	msg.Read = 0;
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		msg.RegArtesa.command = encMiscMessage;
		msg.RegArtesa.channel_number = taskId;
	}
	else
	{
		msg.Reg.command = EncMiscMessage;
	}

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																		   taskId,
																		   &msg,
																		   QP_FALSE,
																		   QP_NULL,
																		   0
																		   );
	QPFWAPI_MailboxDone(This);
	return (qpStatus);
}



EQP_Errors QPFWENCAPI_SetEncMode(CQLCodec* This,
								 TQP_UINT32 taskId,
								 TQP_UINT32 capMode,
								 TQP_UINT32 trigMode,
								 TQP_UINT32 gpioPin
								 )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	ARM_MESSAGE	msg;
	PTASK_DATA	pTaskData;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetEncMode wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	pTaskData = &This->m_pTask->m_TaskData[taskId];

	if (((0 == This->m_VerFwAPI) &&
		(CODEC_OPEN_JPEG_ENCODER == pTaskData->m_codec_function.function)) ||
		((0 != This->m_VerFwAPI) &&
		(CODEC_OPEN_EX_VID_JPEG == pTaskData->m_codec_function.functionEx.video))
		)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetEncMode(%d) capMode(%d) trigMode(0x%x) gpio_pin(%d)\n",
					 taskId,
					 capMode,
					 trigMode,
					 gpioPin
					 ));

		qpStatus = QPFWAPI_MailboxReady(This,
										500
										);
		if (!QP_SUCCESS(qpStatus))
			return (qpStatus);

		qpStatus = QPHCI_RegisterWrite(&This->m_hci,
									   REG_TO_ARM_1,
									   capMode
									   );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  REG_TO_ARM_2,
																		  trigMode
																		  );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  REG_TO_ARM_3,
																		  gpioPin
																		  );

		msg.Read = 0;
		if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
		{
			msg.RegArtesa.command = SetEncMode;
			msg.RegArtesa.channel_number = taskId;
		}
		else
		{
			msg.Reg.command = SetEncMode;
		}

		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																			   taskId,
																			   &msg,
																			   QP_FALSE,
																			   QP_NULL,
																			   0
																			   );
		QPFWAPI_MailboxDone(This);
	}
	return (qpStatus);
}



EQP_Errors QPFWENCAPI_GetAFrame(CQLCodec* This,
								TQP_UINT32 taskId
								)
{
	ARM_MESSAGE	msg;
	PTASK_DATA	pTaskData;


	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_GetAFrame wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	pTaskData = &This->m_pTask->m_TaskData[taskId];

	if (((0 == This->m_VerFwAPI) &&
		(CODEC_OPEN_JPEG_ENCODER == pTaskData->m_codec_function.function)) ||
		((0 != This->m_VerFwAPI) &&
		(CODEC_OPEN_EX_VID_JPEG == pTaskData->m_codec_function.functionEx.video))
		)
	{

		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_GetAFrame()\n"
					 ));

		msg.Read = 0;
		if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
		{
			msg.RegArtesa.command = GetAFrame;
			msg.RegArtesa.channel_number = taskId;
		}
		else
		{
			msg.Reg.command = GetAFrame;
		}

		return (QPFWAPI_SendMessageToARM(This,
										 taskId,
										 &msg,
										 QP_FALSE,
										 QP_NULL,
										 500
										 ));
	}
	else
	{
		return (QPERR_SUCCESS);
	}
}



EQP_Errors QPFWENCAPI_SetMJPEGQuality(CQLCodec* This,
									  TQP_UINT32 taskId,
									  TQP_UINT32 quality
  		  							  )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	ARM_MESSAGE	msg;
	PTASK_DATA	pTaskData;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetMJPEGQuality wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	if (quality > 15)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetMJPEGQuality invalid quality(%d)\n",
					 quality
					 ));
		return (QPERR_PARMS);
	}

	pTaskData = &This->m_pTask->m_TaskData[taskId];

	if (((0 == This->m_VerFwAPI) &&
		(CODEC_OPEN_JPEG_ENCODER == pTaskData->m_codec_function.function)) ||
		((0 != This->m_VerFwAPI) &&
		(CODEC_OPEN_EX_VID_JPEG == pTaskData->m_codec_function.functionEx.video))
		)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetMJPEGQuality() (%d)\n",
					 quality
					 ));

		qpStatus = QPFWAPI_MailboxReady(This,
										500
										);
		if (!QP_SUCCESS(qpStatus))
			return (qpStatus);

		qpStatus = QPHCI_RegisterWrite(&This->m_hci,
									   REG_TO_ARM_1,
									   EncMiscSetMJPEGQuality
									   );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  REG_TO_ARM_2,
																		  quality
																		  );

		msg.Read = 0;
		if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
		{
			msg.RegArtesa.command = encMiscMessage;
			msg.RegArtesa.channel_number = taskId;
		}
		else
		{
			msg.Reg.command = EncMiscMessage;
		}

		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																			   taskId,
																			   &msg,
																			   QP_FALSE,
																			   QP_NULL,
																			   0
																			   );
		QPFWAPI_MailboxDone(This);
	}
	return (qpStatus);
}



EQP_Errors QPFWENCAPI_SetMJPEGFrameBuffer(CQLCodec* This,
										  TQP_UINT32 taskId,
										  TQP_UINT32 nb
  		  								  )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	ARM_MESSAGE	msg;
	PTASK_DATA	pTaskData;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetMJPEGFrameBuffer wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	pTaskData = &This->m_pTask->m_TaskData[taskId];

	if (((0 == This->m_VerFwAPI) &&
		(CODEC_OPEN_JPEG_ENCODER == pTaskData->m_codec_function.function)) ||
		((0 != This->m_VerFwAPI) &&
		(CODEC_OPEN_EX_VID_JPEG == pTaskData->m_codec_function.functionEx.video))
		)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetMJPEGFrameBuffer() (%d)\n",
					 nb
					 ));

		qpStatus = QPFWAPI_MailboxReady(This,
										500
										);
		if (!QP_SUCCESS(qpStatus))
			return (qpStatus);

		qpStatus = QPHCI_RegisterWrite(&This->m_hci,
									   REG_TO_ARM_1,
									   EncMiscSetMJPEGFrameBuffer
									   );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  REG_TO_ARM_2,
																		  nb
																		  );

		msg.Read = 0;
		if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
		{
			msg.RegArtesa.command = encMiscMessage;
			msg.RegArtesa.channel_number = taskId;
		}
		else
		{
			msg.Reg.command = EncMiscMessage;
		}

		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																			   taskId,
																			   &msg,
																			   QP_FALSE,
																			   QP_NULL,
																			   0
																			   );
		QPFWAPI_MailboxDone(This);
	}
	return (qpStatus);
}

EQP_Errors QPFWENCAPI_SetExternalTriggerToSync(CQLCodec* This,
										 	   TQP_UINT32 taskId,
										 	   TQP_UINT32 enable,
											   TQP_UINT32 gpioPin
											   )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	ARM_MESSAGE	msg;
	PTASK_DATA	pTaskData;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetExternalTrigger wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	pTaskData = &This->m_pTask->m_TaskData[taskId];

	if (1)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetExternalTriggerToSync(%d) enable(%d) gpioPin(0x%x)\n",
					 taskId,
					 enable,
					 gpioPin
					 ));

		qpStatus = QPFWAPI_MailboxReady(This,
										500
										);
		if (!QP_SUCCESS(qpStatus))
			return (qpStatus);

		qpStatus = QPHCI_RegisterWrite(&This->m_hci,
									   REG_TO_ARM_1,
									   EncMiscSetExternalTriggerToSync
									   );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  REG_TO_ARM_2,
																		  enable
																		  );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  REG_TO_ARM_3,
																		  gpioPin
																		  );
		msg.Read = 0;
		if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
		{
			msg.RegArtesa.command = encMiscMessage;
			msg.RegArtesa.channel_number = taskId;
		}
		else
		{
			msg.Reg.command = EncMiscMessage;
		}

		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																			   taskId,
																			   &msg,
																			   QP_FALSE,
																			   QP_NULL,
																			   0
																			   );
		QPFWAPI_MailboxDone(This);
	}
	return (qpStatus);
}

EQP_Errors QPFWENCAPI_SetPTSResetByTrigger(CQLCodec* This,
										   TQP_UINT32 taskId,
										   TQP_UINT32 enable,
										   TQP_UINT32 gpioPin,
										   TQP_UINT32 immediate
  		  								   )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	ARM_MESSAGE	msg;
	PTASK_DATA	pTaskData;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetPTSResetByTrigger wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	pTaskData = &This->m_pTask->m_TaskData[taskId];

	if (1)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetPTSResetByTrigger() enable(%d) gpioPin(%d) immediate(%d)\n",
					 enable, gpioPin, immediate
					 ));

		qpStatus = QPFWAPI_MailboxReady(This,
										500
										);
		if (!QP_SUCCESS(qpStatus))
			return (qpStatus);

		qpStatus = QPHCI_RegisterWrite(&This->m_hci,
									   REG_TO_ARM_1,
									   EncMiscSetPTSResetByTrigger
									   );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  REG_TO_ARM_2,
																		  enable
																		  );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  REG_TO_ARM_3,
																		  gpioPin
																		  );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  REG_TO_ARM_4,
																		  immediate
																		  );

		msg.Read = 0;
		if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
		{
			msg.RegArtesa.command = encMiscMessage;
			msg.RegArtesa.channel_number = taskId;
		}
		else
		{
			msg.Reg.command = EncMiscMessage;
		}

		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																			   taskId,
																			   &msg,
																			   QP_FALSE,
																			   QP_NULL,
																			   0
																			   );
		QPFWAPI_MailboxDone(This);
	}
	return (qpStatus);
}


EQP_Errors QPFWENCAPI_SetRawVideoDecimation(CQLCodec* This,
										    TQP_UINT32 taskId,
										    TQP_UINT32 input_format,
										    TQP_UINT32 output_format,
										    TQP_UINT32 scale_factor
  		  								    )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	ARM_MESSAGE	msg;
	PTASK_DATA	pTaskData;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetRawVideoDecimation wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	pTaskData = &This->m_pTask->m_TaskData[taskId];

	if (	((0 != This->m_VerFwAPI) && (CODEC_OPEN_EX_VID_DECIMATION == pTaskData->m_codec_function.functionEx.video))
			|| ((0 == This->m_VerFwAPI) && (CODEC_OPEN_VID_DECIMATION == pTaskData->m_codec_function.functionEx.video))	)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetRawVideoDecimation() input_format(%d) output_format(%d) scale_factor(%d)\n",
					 input_format, output_format, scale_factor
					 ));

		qpStatus = QPFWAPI_MailboxReady(This,
										500
										);
		if (!QP_SUCCESS(qpStatus))
			return (qpStatus);

		qpStatus = QPHCI_RegisterWrite(&This->m_hci,
									   REG_TO_ARM_1,
									   EncMiscSetRawVideoDecimation
									   );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  REG_TO_ARM_2,
																		  input_format
																		  );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  REG_TO_ARM_3,
																		  output_format
																		  );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  REG_TO_ARM_4,
																		  scale_factor
																		  );

		msg.Read = 0;
		if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
		{
			msg.RegArtesa.command = encMiscMessage;
			msg.RegArtesa.channel_number = taskId;
		}
		else
		{
			msg.Reg.command = EncMiscMessage;
		}

		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																			   taskId,
																			   &msg,
																			   QP_FALSE,
																			   QP_NULL,
																			   0
																			   );
		QPFWAPI_MailboxDone(This);
	}
	return (qpStatus);
}

EQP_Errors QPFWENCAPI_SetDeinterlaceMode(CQLCodec* This,
										 TQP_UINT32 taskId,
										 TQP_UINT32 mode
  		  								 )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	ARM_MESSAGE	msg;
	PTASK_DATA	pTaskData;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetDeinterlaceMode() wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	pTaskData = &This->m_pTask->m_TaskData[taskId];

	if (((0 == This->m_VerFwAPI) && (CODEC_OPEN_VID_AUD_ENCODER == pTaskData->m_codec_function.function)) ||
		((0 == This->m_VerFwAPI) && (CODEC_OPEN_JPEG_ENCODER == pTaskData->m_codec_function.function)) ||
		((0 == This->m_VerFwAPI) && (CODEC_OPEN_VID_PASSTHROUGH == pTaskData->m_codec_function.function)) ||
		((0 == This->m_VerFwAPI) && (CODEC_OPEN_VID_DECIMATION == pTaskData->m_codec_function.function)) ||
		((0 != This->m_VerFwAPI) && (CODEC_OPEN_EX_ENCODE == pTaskData->m_codec_function.functionEx.function))
		)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetDeinterlaceMode() mode(%d)\n",
					 mode
					 ));

		qpStatus = QPFWAPI_MailboxReady(This,
										500
										);
		if (!QP_SUCCESS(qpStatus))
			return (qpStatus);

		qpStatus = QPHCI_RegisterWrite(&This->m_hci,
									   REG_TO_ARM_1,
									   EncMiscSetDeinterlaceMode
									   );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  REG_TO_ARM_2,
																		  mode
																		  );

		msg.Read = 0;
		if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
		{
			msg.RegArtesa.command = encMiscMessage;
			msg.RegArtesa.channel_number = taskId;
		}
		else
		{
			msg.Reg.command = EncMiscMessage;
		}

		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																			   taskId,
																			   &msg,
																			   QP_FALSE,
																			   QP_NULL,
																			   0
																			   );
		QPFWAPI_MailboxDone(This);
	}
	return (qpStatus);
}

EQP_Errors QPFWENCAPI_SetRateControlEx(CQLCodec* This,
									   TQP_UINT32 taskId,
									   TQP_UINT32 base_qp_length,
									   TQP_UINT32 local_activity_on,
									   TQP_UINT32 motion_detect_interval
  		  							   )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	ARM_MESSAGE	msg;
	PTASK_DATA	pTaskData;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetDeinterlaceMode() wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	pTaskData = &This->m_pTask->m_TaskData[taskId];

	if (((0 == This->m_VerFwAPI) && (CODEC_OPEN_VID_AUD_ENCODER == pTaskData->m_codec_function.function)) ||
		((0 == This->m_VerFwAPI) && (CODEC_OPEN_JPEG_ENCODER == pTaskData->m_codec_function.function)) ||
		((0 != This->m_VerFwAPI) && (CODEC_OPEN_EX_ENCODE == pTaskData->m_codec_function.functionEx.function))
		)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetRateControlEx() base_qp_length(%d) local_activity_on(%d) motion_detect_interval(%d)\n",
					 base_qp_length,
					 local_activity_on,
					 motion_detect_interval
					 ));

		qpStatus = QPFWAPI_MailboxReady(This,
										500
										);
		if (!QP_SUCCESS(qpStatus))
			return (qpStatus);

		qpStatus = QPHCI_RegisterWrite(&This->m_hci,
									   REG_TO_ARM_1,
									   EncMiscSetRateControlEx
									   );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  REG_TO_ARM_2,
																		  base_qp_length
																		  );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  REG_TO_ARM_3,
																		  local_activity_on
																		  );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  REG_TO_ARM_4,
																		  motion_detect_interval
																		  );

		msg.Read = 0;
		if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
		{
			msg.RegArtesa.command = encMiscMessage;
			msg.RegArtesa.channel_number = taskId;
		}
		else
		{
			msg.Reg.command = EncMiscMessage;
		}

		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																			   taskId,
																			   &msg,
																			   QP_FALSE,
																			   QP_NULL,
																			   0
																			   );
		QPFWAPI_MailboxDone(This);
	}
	return (qpStatus);
}

EQP_Errors QPFWENCAPI_SetLargeCompressBufferControl(CQLCodec* This,
												    TQP_UINT32 taskId,
												    TQP_UINT32 large_compress_buffer_control
  		  										    )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	ARM_MESSAGE	msg;
	PTASK_DATA	pTaskData;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetLargeCompressBufferControl() wrong power state(%d)\n",
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	pTaskData = &This->m_pTask->m_TaskData[taskId];

	if ((0 != This->m_VerFwAPI) && (CODEC_OPEN_EX_ENCODE == pTaskData->m_codec_function.functionEx.function))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPFWENCAPI_SetLargeCompressBufferControl() large_compress_buffer_control(0x%08x)\n",
					 large_compress_buffer_control
					 ));

		qpStatus = QPFWAPI_MailboxReady(This, 500);

		if (!QP_SUCCESS(qpStatus))
			return (qpStatus);

		qpStatus = QPHCI_RegisterWrite(&This->m_hci,
									   REG_TO_ARM_1,
									   EncMiscSetLargeCompressBufferControl
									   );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  REG_TO_ARM_2,
																		  (large_compress_buffer_control & QPCODEC_LARGE_COMPRESS_BUFFER_CONTROL_ENABLE) ? 1 : 0
																		  );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  REG_TO_ARM_3,
																		  large_compress_buffer_control & (~QPCODEC_LARGE_COMPRESS_BUFFER_CONTROL_ENABLE)
																		  );

		msg.Read = 0;
		if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
		{
			msg.RegArtesa.command = encMiscMessage;
			msg.RegArtesa.channel_number = taskId;
		}
		else
		{
			msg.Reg.command = EncMiscMessage;
		}

		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																			   taskId,
																			   &msg,
																			   QP_FALSE,
																			   QP_NULL,
																			   0
																			   );
		QPFWAPI_MailboxDone(This);
	}
	return (qpStatus);
}


 EQP_Errors QPFWENCAPI_SetLowBitrateMode(CQLCodec* This,
    									 TQP_UINT32 taskId,
    									 TQP_UINT32 enable_low_bitrate_mode
										 )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	ARM_MESSAGE	msg;
	PTASK_DATA	pTaskData;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "%s() wrong power state(%d)\n",
					 __FUNCTION__,
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	pTaskData = &This->m_pTask->m_TaskData[taskId];

	if ((0 != This->m_VerFwAPI) && (CODEC_OPEN_EX_ENCODE == pTaskData->m_codec_function.functionEx.function))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "%s() enable_low_bitrate_mode(%d)\n",
					 __FUNCTION__,
					 enable_low_bitrate_mode
					 ));

		qpStatus = QPFWAPI_MailboxReady(This, 500);

		if (!QP_SUCCESS(qpStatus))
			return (qpStatus);

		qpStatus = QPHCI_RegisterWrite(&This->m_hci,
									   REG_TO_ARM_1,
									   EncMiscSetLowBitrateMode
									   );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  REG_TO_ARM_2,
																		  enable_low_bitrate_mode
																		  );

		msg.Read = 0;
		if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
		{
			msg.RegArtesa.command = encMiscMessage;
			msg.RegArtesa.channel_number = taskId;
		}
		else
		{
			msg.Reg.command = EncMiscMessage;
		}

		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																			   taskId,
																			   &msg,
																			   QP_FALSE,
																			   QP_NULL,
																			   0
																			   );
		QPFWAPI_MailboxDone(This);
	}
	return (qpStatus);
}


 EQP_Errors QPFWENCAPI_SetAVDiscardControl(CQLCodec* This,
    									   TQP_UINT32 taskId,
    									   TQP_UINT32 enable_av_discard_control
										   )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	ARM_MESSAGE	msg;
	PTASK_DATA	pTaskData;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "%s() wrong power state(%d)\n",
					 __FUNCTION__,
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	pTaskData = &This->m_pTask->m_TaskData[taskId];

	if ((0 != This->m_VerFwAPI) && (CODEC_OPEN_EX_ENCODE == pTaskData->m_codec_function.functionEx.function))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "%s() enable_av_discard_control(%d)\n",
					 __FUNCTION__,
					 enable_av_discard_control
					 ));

		qpStatus = QPFWAPI_MailboxReady(This, 500);

		if (!QP_SUCCESS(qpStatus))
			return (qpStatus);

		qpStatus = QPHCI_RegisterWrite(&This->m_hci,
									   REG_TO_ARM_1,
									   EncMiscSetAVDiscardControl
									   );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  REG_TO_ARM_2,
																		  enable_av_discard_control ? 1 : 0
																		  );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  REG_TO_ARM_3,
																		  (enable_av_discard_control == ENC_AV_DISCARD_CONTROL_ENABLE_AND_CONCATENATE_PTS) ? 1 : 0
																		  );

		msg.Read = 0;
		if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
		{
			msg.RegArtesa.command = encMiscMessage;
			msg.RegArtesa.channel_number = taskId;
		}
		else
		{
			msg.Reg.command = EncMiscMessage;
		}

		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																			   taskId,
																			   &msg,
																			   QP_FALSE,
																			   QP_NULL,
																			   0
																			   );
		QPFWAPI_MailboxDone(This);
	}

	return (qpStatus);
}


 EQP_Errors QPFWENCAPI_SetUseSWPTS(CQLCodec* This,
    							   TQP_UINT32 taskId,
    							   TQP_UINT32 use_sw_pts
								   )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	ARM_MESSAGE	msg;
	PTASK_DATA	pTaskData;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "%s() wrong power state(%d)\n",
					 __FUNCTION__,
					 This->m_PowerState
					 ));
		return (QPERR_INVALID);
	}

	pTaskData = &This->m_pTask->m_TaskData[taskId];

	if ((0 != This->m_VerFwAPI) && (CODEC_OPEN_EX_ENCODE == pTaskData->m_codec_function.functionEx.function))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "%s() use_sw_pts(%d)\n",
					 __FUNCTION__,
					 use_sw_pts
					 ));

		qpStatus = QPFWAPI_MailboxReady(This, 500);

		if (!QP_SUCCESS(qpStatus))
			return (qpStatus);

		qpStatus = QPHCI_RegisterWrite(&This->m_hci,
									   REG_TO_ARM_1,
									   EncMiscSetUseSWPTS
									   );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  REG_TO_ARM_2,
																		  use_sw_pts
																		  );

		msg.Read = 0;
		if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
		{
			msg.RegArtesa.command = encMiscMessage;
			msg.RegArtesa.channel_number = taskId;
		}
		else
		{
			msg.Reg.command = EncMiscMessage;
		}

		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWAPI_SendMessageToARM(This,
																			   taskId,
																			   &msg,
																			   QP_FALSE,
																			   QP_NULL,
																			   0
																			   );
		QPFWAPI_MailboxDone(This);
	}

	return (qpStatus);
}

