///////////////////////////////////////////////////////////////////////////////
//     
//
// Hci.c
//
// Description:
//
//  codec chip host control interface
//
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#include "hci.h"
#include "qpdbgmgr.h"
#include "qpmemmgr.h"
#include "util.h"
#include "qldev_interface.h"
#include "cqpusbintf.h"
#include "cusbcntl.h"
#include "cqlcodec.h"

#include "../common/qptypes.h"

#include "../common/qpdbgmgr.h"
#include "../common/qpmemmgr.h"

#include "../managers/qpmemmgrint.h"

#if defined(QP_LINXKER) || defined(_QP_LINUX) || defined(_QP_MACOSX)
#include "qlautoconf.h"
#endif //QP_LINXKER




extern TQP_DbgMCompID	g_QPCodecDBGCompID;


/////////////////////////////////////////////////////////////////////////////
//
//

// Indirect Access mode
//

// Host control interface registers (indirect access mode)
//
#define REG_HCI_INTERRUPT_ENABLE			0
#define REG_HCI_INTERRUPT_STATUS			1
#define REG_HCI_SETUP_MODE					2
#define REG_HCI_CONTROL						3
#define REG_HCI_REGISTER_ADDR_0				8
#define REG_HCI_REGISTER_ADDR_1				9
#define REG_HCI_REGISTER_DATA_0				10
#define REG_HCI_REGISTER_DATA_1				11
#define REG_HCI_REGISTER_DATA_2				12
#define REG_HCI_REGISTER_DATA_3				13
#define REG_HCI_MEMORY_ADDR_0				14
#define REG_HCI_MEMORY_ADDR_1				15
#define REG_HCI_MEMORY_ADDR_2				16
#define REG_HCI_MEMORY_ADDR_3				17
#define REG_HCI_MEMORY_DATA_0				18
#define REG_HCI_MEMORY_DATA_1				19
#define REG_HCI_MEMORY_DATA_2				20
#define REG_HCI_MEMORY_DATA_3				21
#define REG_HCI_SLAVE_MODE_WR_CYCLE_CNT		22


#pragma pack(push, 1)

typedef union _INTERRUPT_ENABLE
{
	TQP_UINT8	Read;
	struct
	{
		TQP_UINT8	arm			: 1;	// bit 0
		TQP_UINT8	p2m			: 1;	// bit 1
		TQP_UINT8	m2p			: 1;	// bit 2
		TQP_UINT8	reserved	: 5;	// bit 3 - 7
	} Reg;
} INTERRUPT_ENABLE, *PINTERRUPT_ENABLE;


typedef union _INTERRUPT_STATUS
{
	TQP_UINT8	Read;
	struct
	{
		TQP_UINT8	arm			: 1;	// bit 0
		TQP_UINT8	p2m			: 1;	// bit 1
		TQP_UINT8	m2p			: 1;	// bit 2
		TQP_UINT8	reserved	: 5;	// bit 3 - 7
	} Reg;
} INTERRUPT_STATUS, *PINTERRUPT_STATUS;


typedef union _SETUP_MODE
{
	TQP_UINT8	Read;
	struct
	{
		TQP_UINT8	width		: 1;	// bit 0
		TQP_UINT8	intel		: 1;	// bit 1
		TQP_UINT8	slave		: 1;	// bit 2
		TQP_UINT8	endian		: 1;	// bit 3
		TQP_UINT8	reserved	: 4;	// bit 4 - 7
	} Reg;
} SETUP_MODE, *PSETUP_MODE;


typedef union _QL_CONTROL
{
	TQP_UINT8	Read;
	struct
	{
		TQP_UINT8	arm			: 1;	// bit 0
		TQP_UINT8	reserved	: 7;	// bit 1 - 7
	} Reg;
} QL_CONTROL, *PQL_CONTROL;


typedef union _SLAVE_MODE_WR_CYCLE_CNT
{
	TQP_UINT8	Read;
	struct
	{
		TQP_UINT8	sla_cnt		: 8;	// bit 0 - 7
	} Reg;
} SLAVE_MODE_WR_CYCLE_CNT, *PSLAVE_MODE_WR_CYCLE_CNT;

#pragma pack(pop)


/////////////////////////////////////////////////////////////////////////////
//
//

// Host DMA registers
//
#define REG_P2M_M2P_DMA_CONTROL				0x0630
#define REG_P2M_DMA_ADDR					0x0640
#define REG_P2M_DMA_LENGTH					0x0644
#define REG_M2P_DMA_ADDR					0x0648
#define REG_M2P_DMA_LENGTH					0x064C



#pragma pack(push, 4)

typedef union _P2M_M2P_DMA_CONTROL
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	p2m_size	: 2;	// bit 0 - 1
		TQP_UINT32	reserved	: 2;	// bit 2 - 3
		TQP_UINT32	m2p_size	: 2;	// bit 4 - 5
		TQP_UINT32	reserved2	: 2;	// bit 6 - 7
		TQP_UINT32	pe			: 1;	// bit 8
		TQP_UINT32	reserved3	: 3;	// bit 9 - 11
		TQP_UINT32	me			: 1;	// bit 12
		TQP_UINT32	reserved4	: 19;	// bit 13 - 31
	} Reg;
} P2M_M2P_DMA_CONTROL, *PP2M_M2P_DMA_CONTROL;

#pragma pack(pop)



/////////////////////////////////////////////////////////////////////////////
//
//

// Direct Access mode
//

// Host control interface registers (direct access mode)
//
#define REG_HOST_DIRECT_MAPPING_CNTL		0x800
#define REG_INTERNAL_REG_ADDR_CNTL			0x804
#define REG_INTERNAL_REG_DATA				0x808
#define REG_HOST_DIRECT_MODE_MISC			0x80C
#define REG_MEM_MAPPING_0_START_ADDR		0x810
#define REG_MEM_MAPPING_0_END_ADDR			0x814
#define REG_MEM_MAPPING_0_OFFSET_ADDR		0x818
#define REG_MEM_MAPPING_1_START_ADDR		0x81C
#define REG_MEM_MAPPING_1_END_ADDR			0x820
#define REG_MEM_MAPPING_1_OFFSET_ADDR		0x824
#define REG_MEM_MAPPING_2_START_ADDR		0x828
#define REG_MEM_MAPPING_2_END_ADDR			0x82C
#define REG_MEM_MAPPING_2_OFFSET_ADDR		0x830
#define REG_MEM_MAPPING_3_START_ADDR		0x834
#define REG_MEM_MAPPING_3_END_ADDR			0x838
#define REG_MEM_MAPPING_3_OFFSET_ADDR		0x83C
#define REG_HOST_DMA_CNTL_STATUS			0x840
#define REG_HOST_DMA_RD_BUFFER_START_ADDR	0x844
#define REG_HOST_DMA_RD_BUFFER_END_ADDR		0x848
#define REG_HOST_DMA_RD_BUFFER_CUR_PTR		0x84C
#define REG_HOST_DMA_RD_SIZE				0x850
#define REG_HOST_DMA_WR_BUFFER_START_ADDR	0x854
#define REG_HOST_DMA_WR_BUFFER_END_ADDR		0x858
#define REG_HOST_DMA_WR_BUFFER_CUR_PTR		0x85C
#define REG_HOST_DMA_WR_SIZE				0x860
#define REG_HOST_MEM_ACCESS_TIMEOUT_CNTL	0x864
#define REG_HOST_MEM_ACCESS_TIMEOUT_ADDR	0x868



#pragma pack(push, 4)

typedef union _HOST_DIRECT_MAPPING_CNTL
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	arm_e		: 1;	// bit 0
		TQP_UINT32	sr_e		: 1;	// bit 1
		TQP_UINT32	sw_e		: 1;	// bit 2
		TQP_UINT32	mat_e		: 1;	// bit 3
		TQP_UINT32	reserved	: 12;	// bit 4 - 15
		TQP_UINT32	arm_s		: 1;	// bit 16
		TQP_UINT32	sr_s		: 1;	// bit 17
		TQP_UINT32	sw_s		: 1;	// bit 18
		TQP_UINT32	mat_s		: 1;	// bit 19
		TQP_UINT32	reserved2	: 12;	// bit 20 - 31
	} Reg;
} HOST_DIRECT_MAPPING_CNTL, *PHOST_DIRECT_MAPPING_CNTL;


typedef union _INTERNAL_REG_ADDR_CNTL
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	addr		: 14;	// bit 0 - 13
		TQP_UINT32	reserved	: 1;	// bit 14
		TQP_UINT32	rw			: 1;	// bit 15
		TQP_UINT32	reserved2	: 16;	// bit 16 - 31
	} Reg;
} INTERNAL_REG_ADDR_CNTL, *PINTERNAL_REG_ADDR_CNTL;



typedef union _HOST_DIRECT_MODE_MISC
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	arm_rst		: 1;	// bit 0
		TQP_UINT32	reserved	: 31;	// bit 1 - 31
	} Reg;
} HOST_DIRECT_MODE_MISC, *PHOST_DIRECT_MODE_MISC;



typedef union _HOST_DMA_CNTL_STATUS
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	addr		: 23;	// bit 0 - 22
		TQP_UINT32	reserved	: 3;	// bit 23 - 25
		TQP_UINT32	dr_s		: 1;	// bit 26
		TQP_UINT32	dw_s		: 1;	// bit 27
		TQP_UINT32	dword		: 1;	// bit 28
		TQP_UINT32	dr_rst		: 1;	// bit 29
		TQP_UINT32	dw_rst		: 1;	// bit 30
		TQP_UINT32	d_en		: 1;	// bit 31
	} Reg;
} HOST_DMA_CNTL_STATUS, *PHOST_DMA_CNTL_STATUS;



typedef union _HOST_MEM_ACCESS_TIMEOUT_CNTL
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	matc		: 31;	// bit 0 - 30
		TQP_UINT32	mat_e		: 1;	// bit 31
	} Reg;
} HOST_MEM_ACCESS_TIMEOUT_CNTL, *PHOST_MEM_ACCESS_TIMEOUT_CNTL;


#pragma pack(pop)


/////////////////////////////////////////////////////////////////////////////
//
//

// Direct Mode access function
//

typedef union _QPHCI_UINT32
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	low		: 16;	// bit 0 - 15
		TQP_UINT32	high	: 16;	// bit 16 - 31
	} Uint32;
} QPHCI_UINT32, *PQPHCI_UINT32;


// address below 0x2000 reserved for registers mapping
// address 0x3124 reserved for streaming port

#define QPHCI_DM_MEMMAPPING_START_MIN	0x4000
#define QPHCI_DM_MEMMAPPING_STREAM_PORT	0x3124


static TQP_BOOL DM_FindMemoryMapping(IHCIAPI *This,
									 TQP_UINT32 dwAddr,
									 PTQP_UINT8 *linearAddr
									 )
{
	int	i;

	for (i = 0; i < QPHCI_MEM_MAPPING_MAX_PAGES; i++)
	{
		if ((dwAddr >= This->m_mem_mapping_offset[i]) &&
			(dwAddr < This->m_mem_mapping_offset[i] + This->m_PageSize)
			)
		{
			*linearAddr = This->m_pMemoryBase + (dwAddr - This->m_mem_mapping_offset[i]) + This->m_mem_mapping_start_addr[i];
			return (QP_TRUE);
		}
	}

	if (QPHCI_MEM_MAPPING_MAX_PAGES == i)
	{
		// no match, re-map to page 1
		//
		This->m_mem_mapping_offset[0] = dwAddr - (dwAddr % This->m_PageSize);
		QPHCI_RegisterWrite(This,
							(TQP_UINT16)(REG_MEM_MAPPING_1_OFFSET_ADDR),
							This->m_mem_mapping_offset[0]
							);

		*linearAddr = This->m_pMemoryBase + (dwAddr - This->m_mem_mapping_offset[0]) + This->m_mem_mapping_start_addr[0];
		return (QP_TRUE);
	}

	return (QP_FALSE);

}


static EQP_Errors DM_ReadHciRegister(IHCIAPI *This,
									 TQP_UINT8 cAddr,
									 PTQP_UINT8 pData
									 )
{
	return (QPERR_NOTSUPP);
}



static EQP_Errors DM_WriteHciRegister(IHCIAPI * This,
									  TQP_UINT8 cAddr,
									  TQP_UINT8 ucData
									  )
{
	return (QPERR_NOTSUPP);
}

#ifdef CONFIG_HOST_PLATFORM_STR8133_REF

static int DM_ThreadProc(CThread *This_p)
{
	EQP_Errors	qpStatus;
	TQP_UINT32	ulFlaggedEvents = 0;
	IHCIAPI		*This = (IHCIAPI *)This_p->m_context;

#if 0
#ifdef QP_LINXKER
#ifdef __LINUX24__
	sprintf(current->comm, "%s", __FUNCTION__);
	daemonize();
#else
	char szThreadName[32];
	sprintf(szThreadName, "%s", __FUNCTION__);
	daemonize(szThreadName);
	current->flags |= PF_NOFREEZE;
#endif
#endif
#endif

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "EMU_ThreadProc() ENTER (0x%x) (0x%x)\n",
				 This_p,
				 This
				 ));

	while (QP_TRUE)
	{
		if (QP_NULL == This->m_EvtEmuTask)
			break;

    QPTMDelayMilliS(1);
    CQLCodec_InterruptHandler((void *)This->m_pMpegCodec);

    // wait on the event
		//
		qpStatus = QPOSMWaitEvtgrp(This_p->m_EvtWait,
								   THREAD_EVENT_MASK,
								   &ulFlaggedEvents,
								   0	// polling
								   );

		if (ulFlaggedEvents & THREAD_EVENT_EXIT)
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
						 "EMU_ThreadProc() THREAD_EVENT_EXIT\n"
						 ));
			QPOSMSetEvtgrp(This_p->m_EvtReply,
						   THREAD_EVENT_EXIT_DONE
						   );
			break;
		}
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "DM_ThreadProc() See Ya!\n"
				 ));

	return (0);
}

#endif // CONFIG_HOST_PLATFORM_STR8133_REF

static EQP_Errors DM_RegisterRead(IHCIAPI * This,
								  TQP_UINT16 wAddr,
								  PTQP_UINT32 pData
								  )
{
	QPHCI_UINT32	val;

	if (0x800 == (wAddr & 0x1F00))
	{
		// read value
		//
		val.Uint32.low = *(volatile TQP_UINT16 *)(This->m_pRegisterBase + wAddr);
		val.Uint32.high = *(volatile TQP_UINT16 *)(This->m_pRegisterBase + wAddr + 2);
	}
	else
	{
		// write register
		//
		*(TQP_UINT16 *)(This->m_pRegisterBase + REG_INTERNAL_REG_ADDR_CNTL) = wAddr;

		// read value
		//
		val.Uint32.low = *(volatile TQP_UINT16 *)(This->m_pRegisterBase + REG_INTERNAL_REG_DATA);
		val.Uint32.high = *(volatile TQP_UINT16 *)(This->m_pRegisterBase + REG_INTERNAL_REG_DATA + 2);
	}

	*pData = val.Read;

	return (QPERR_SUCCESS);
}



static EQP_Errors DM_RegisterWrite(IHCIAPI * This,
								   TQP_UINT16 wAddr,
								   TQP_UINT32 dwData
								   )
{
	if (0x800 == (wAddr & 0x1F00))
	{
		*(TQP_UINT16 *)(This->m_pRegisterBase + wAddr) = (TQP_UINT16)(dwData & 0xFFFF);
		*(TQP_UINT16 *)(This->m_pRegisterBase + wAddr + 2) = (TQP_UINT16)((dwData >> 16) & 0xFFFF);
	}
	else
	{
		// write data first
		//
		*(TQP_UINT16 *)(This->m_pRegisterBase + REG_INTERNAL_REG_DATA) = (TQP_UINT16)(dwData & 0xFFFF);
		*(TQP_UINT16 *)(This->m_pRegisterBase + REG_INTERNAL_REG_DATA + 2) = (TQP_UINT16)((dwData >> 16) & 0xFFFF);

		// write register
		//
		*(TQP_UINT16 *)(This->m_pRegisterBase + REG_INTERNAL_REG_ADDR_CNTL) = wAddr | 0x8000;
	}

	return (QPERR_SUCCESS);
}



static EQP_Errors DM_MemoryRead(IHCIAPI * This,
								TQP_UINT32 dwAddr,
								PTQP_UINT32 pData
								)
{
	QPHCI_UINT32	val;
	PTQP_UINT8		linearAddr;

	if (!DM_FindMemoryMapping(This,
							  dwAddr,
							  &linearAddr
							  ))
	{
		return (QPERR_FAIL);
	}
	else
	{
		// read value
		//
		val.Uint32.low = *(volatile TQP_UINT16 *)linearAddr;
		val.Uint32.high = *(volatile TQP_UINT16 *)(linearAddr + 2);

		*pData = val.Read;

		return (QPERR_SUCCESS);
	}
}



static EQP_Errors DM_MemoryWrite(IHCIAPI * This,
								 TQP_UINT32 dwAddr,
								 TQP_UINT32 dwData
								 )
{
	PTQP_UINT8	linearAddr;

	if (!DM_FindMemoryMapping(This,
							  dwAddr,
							  &linearAddr
							  ))
	{
		return (QPERR_FAIL);
	}
	else
	{
		// write value
		//
		*(TQP_UINT16 *)(linearAddr) = (TQP_UINT16)(dwData & 0xFFFF);
		*(TQP_UINT16 *)(linearAddr + 2) = (TQP_UINT16)((dwData >> 16) & 0xFFFF);
		return (QPERR_SUCCESS);
	}
}

#ifndef CONFIG_HOST_PLATFORM_STR8133_REF

static EQP_Errors DM_StartDMAWrite(IHCIAPI * This,
								   TQP_UINT32 ulAddr,
								   TQP_UINT32 ulLength,
								   PTQP_UINT8 pHostAddr,
								   TQP_UINT32 ulXferMode,
								   TQP_UINT32 ulPicWidthMB,
								   TQP_UINT32 ulPicHeightMB,
								   TQP_UINT32 ulSwap,
								   TQP_BOOL IsFrameMode
								   )
{
	P2M_M2P_DMA_CONTROL		cntl;
	HOST_DMA_CNTL_STATUS	dmaCntl;

	// setup DMA byte swap
	//
	cntl.Read = 0;
	cntl.Reg.pe = ((0 != ulSwap) ? 1 : 0);

	QPHCI_RegisterWrite(This,
						REG_P2M_M2P_DMA_CONTROL,
						cntl.Read
						);

	// setup DMA control register
	//
	dmaCntl.Read = 0;

	dmaCntl.Reg.addr = QPHCI_DM_MEMMAPPING_STREAM_PORT;
	dmaCntl.Reg.dword = 1;
	dmaCntl.Reg.dr_rst = 1;
	dmaCntl.Reg.dw_rst = 1;
	dmaCntl.Reg.d_en = 0;

	QPHCI_RegisterWrite(This,
						REG_HOST_DMA_CNTL_STATUS,
						dmaCntl.Read
						);

	dmaCntl.Reg.dr_rst = 0;
	dmaCntl.Reg.dw_rst = 0;
	dmaCntl.Reg.d_en = 1;

	QPHCI_RegisterWrite(This,
						REG_HOST_DMA_CNTL_STATUS,
						dmaCntl.Read
						);

	// setup start address
	//
	QPHCI_RegisterWrite(This,
						REG_HOST_DMA_WR_BUFFER_START_ADDR,
						ulAddr << 2
						);

	// setup end address
	//
	QPHCI_RegisterWrite(This,
						REG_HOST_DMA_WR_BUFFER_END_ADDR,
						(ulAddr + ulLength - 1) << 2
						);

	// setup current address
	//
	QPHCI_RegisterWrite(This,
						REG_HOST_DMA_WR_BUFFER_CUR_PTR,
						ulAddr << 2
						);

	// write size
	//
	QPHCI_RegisterWrite(This,
						REG_HOST_DMA_WR_SIZE,
						ulLength << 2
						);

	return (QPERR_SUCCESS);
}



static EQP_Errors DM_StartDMARead(IHCIAPI * This,
								  TQP_UINT32 ulAddr,
								  TQP_UINT32 ulLength,
								  PTQP_UINT8 pHostAddr,
								  TQP_UINT32 ulXferMode,
								  TQP_UINT32 ulPicWidthMB,
								  TQP_UINT32 ulPicHeightMB,
								  TQP_UINT32 ulSwap,
								  TQP_BOOL IsFrameMode
								  )
{
	P2M_M2P_DMA_CONTROL		cntl;
	HOST_DMA_CNTL_STATUS	dmaCntl;

	// setup DMA byte swap
	//
	cntl.Read = 0;
	cntl.Reg.me = ((0 != ulSwap) ? 1 : 0);

	QPHCI_RegisterWrite(This,
						REG_P2M_M2P_DMA_CONTROL,
						cntl.Read
						);

	// setup DMA control register
	//
	dmaCntl.Read = 0;

	dmaCntl.Reg.addr = QPHCI_DM_MEMMAPPING_STREAM_PORT;
	dmaCntl.Reg.dword = 1;
	dmaCntl.Reg.dr_rst = 1;
	dmaCntl.Reg.dw_rst = 1;
	dmaCntl.Reg.d_en = 0;

	QPHCI_RegisterWrite(This,
						REG_HOST_DMA_CNTL_STATUS,
						dmaCntl.Read
						);

	dmaCntl.Reg.dr_rst = 0;
	dmaCntl.Reg.dw_rst = 0;
	dmaCntl.Reg.d_en = 1;

	QPHCI_RegisterWrite(This,
						REG_HOST_DMA_CNTL_STATUS,
						dmaCntl.Read
						);

	// setup start address
	//
	QPHCI_RegisterWrite(This,
						REG_HOST_DMA_RD_BUFFER_START_ADDR,
						ulAddr << 2
						);

	// setup end address
	//
	QPHCI_RegisterWrite(This,
						REG_HOST_DMA_RD_BUFFER_END_ADDR,
						(ulAddr + ulLength - 1) << 2
						);

	// setup current address
	//
	QPHCI_RegisterWrite(This,
						REG_HOST_DMA_RD_BUFFER_CUR_PTR,
						ulAddr << 2
						);

	// read size
	//
	QPHCI_RegisterWrite(This,
						REG_HOST_DMA_RD_SIZE,
						ulLength << 2
						);

	return (QPERR_SUCCESS);
}

#else

static EQP_Errors DM_StartDMAWrite(IHCIAPI * This,
								   TQP_UINT32 ulAddr,
								   TQP_UINT32 ulLength,
								   PTQP_UINT8 pHostAddr,
								   TQP_UINT32 ulXferMode,
								   TQP_UINT32 ulPicWidthMB,
								   TQP_UINT32 ulPicHeightMB,
								   TQP_UINT32 ulSwap,
								   TQP_BOOL IsFrameMode
								   )
{
	P2M_M2P_DMA_CONTROL		cntl;
	HOST_DMA_CNTL_STATUS	dmaCntl;
	TQP_INT32				nStartAddr = (ulAddr + 3) & 0xFFFFFFFC;				// always positive
	TQP_INT32				nStartExtra = nStartAddr - (TQP_INT32)ulAddr;		// always positive
	TQP_INT32				nEndAddr = (ulAddr + ulLength) & 0xFFFFFFFC;
	TQP_INT32				nEndExtra = (ulAddr + ulLength) - nEndAddr;
	TQP_UINT32				ulDMALength = (nEndAddr > nStartAddr) ? (nEndAddr - nStartAddr) : 0;
	PTQP_UINT32				ptr = (PTQP_UINT32)pHostAddr;
	TQP_PVOID				StreamPort;
	TQP_INT32					i;
	TQP_UINT32				ulTemp[8];

#ifndef INTEGRATE_USB_EMU_INT_THREAD_TO_TASK_THREAD
	// create emulated interrupt thread
	//
	if (!CThread_IsThreadInited(&This->m_Thread))
	{
        CThread_ThreadInit(&This->m_Thread);
	}
#endif

	if (ulDMALength > 0)
	{
	#if 0	//close swap TODO::
		// setup DMA byte swap
		//
		cntl.Read = 0;
		QPHCI_RegisterRead(This,
							REG_P2M_M2P_DMA_CONTROL,
							&cntl.Read
							);
		cntl.Reg.pe = ((0 != ulSwap) ? 1 : 0);
		QPHCI_RegisterWrite(This,
							REG_P2M_M2P_DMA_CONTROL,
							cntl.Read
							);
	#endif
		// setup DMA control register
		//
		dmaCntl.Read = 0;

		dmaCntl.Reg.addr = QPHCI_DM_MEMMAPPING_STREAM_PORT;
		dmaCntl.Reg.dword = 1;
		dmaCntl.Reg.dr_rst = 1;
		dmaCntl.Reg.dw_rst = 1;
		dmaCntl.Reg.d_en = 0;

		QPHCI_RegisterWrite(This,
							REG_HOST_DMA_CNTL_STATUS,
							dmaCntl.Read
							);

		dmaCntl.Reg.dr_rst = 0;
		dmaCntl.Reg.dw_rst = 0;
		dmaCntl.Reg.d_en = 1;

		QPHCI_RegisterWrite(This,
							REG_HOST_DMA_CNTL_STATUS,
							dmaCntl.Read
							);

		// setup start address
		//
		QPHCI_RegisterWrite(This,
							REG_HOST_DMA_WR_BUFFER_START_ADDR,
							(TQP_UINT32)nStartAddr << 2
							);

		// setup end address
		//
		QPHCI_RegisterWrite(This,
							REG_HOST_DMA_WR_BUFFER_END_ADDR,
							(TQP_UINT32)nEndAddr << 2
							);

		// setup current address
		//
		QPHCI_RegisterWrite(This,
							REG_HOST_DMA_WR_BUFFER_CUR_PTR,
							(TQP_UINT32)nStartAddr << 2
							);

		// write size
		//
		QPHCI_RegisterWrite(This,
							REG_HOST_DMA_WR_SIZE,
							ulDMALength << 2
							);

		QPHCI_GetStreamingPortAddress(This,
									  (TQP_PVOID *)&StreamPort
									   );

		if (0 != ulSwap)
		{
			BytesSwap((TQP_UINT8 *)ptr,
					  (TQP_UINT8 *)&ulTemp[0],
					  nStartExtra * sizeof(TQP_UINT32)
					  );

			for (i = 0; i < nStartExtra; i++)
			{
				QPHCI_MemoryWrite(This,
								  (ulAddr + i) << 2,
								  ulTemp[i]
								  );
				ptr++;
			}
		}
		else
		{
			for (i = 0; i < nStartExtra; i++)
			{
				QPHCI_MemoryWrite(This,
								  (ulAddr + i) << 2,
								  *ptr
								  );
				ptr++;
			}
		}

		for (i = 0; i < (TQP_INT32)ulDMALength; i++)
		{
			*((volatile TQP_UINT16 *)StreamPort) = (TQP_UINT16)(*ptr & 0x0000ffff);
			*((volatile TQP_UINT16 *)StreamPort) = (TQP_UINT16)(*ptr >> 16);
			ptr++;
		}

		if (0 != ulSwap)
		{
			BytesSwap((TQP_UINT8 *)ptr,
					  (TQP_UINT8 *)&ulTemp[0],
					  nEndExtra * sizeof(TQP_UINT32)
					  );

			for (i = 0; i < nEndExtra; i++)
			{
				QPHCI_MemoryWrite(This,
								  (ulAddr + nStartExtra + ulDMALength + i) << 2,
								  ulTemp[i]
								  );
			}
		}
		else
		{
			for (i = 0; i < nEndExtra; i++)
			{
				QPHCI_MemoryWrite(This,
								  (ulAddr + nStartExtra + ulDMALength + i) << 2,
								  *ptr
								  );
				ptr++;
			}
		}
	}
	else
	{
		if (0 != ulSwap)
		{
			BytesSwap((TQP_UINT8 *)ptr,
					  (TQP_UINT8 *)&ulTemp[0],
					  ulLength * sizeof(TQP_UINT32)
					  );

			for (i = 0; i < (TQP_INT32)ulLength; i++)
			{
				QPHCI_MemoryWrite(This,
								  (ulAddr + i) << 2,
								  ulTemp[i]
								  );
			}
		}
		else
		{
			for (i = 0; i < (TQP_INT32)ulLength; i++)
			{
				QPHCI_MemoryWrite(This,
								  (ulAddr + i) << 2,
								  *ptr
								  );
				ptr++;
			}
		}
	}


    QPOSMSetEvtgrp(This->m_pMpegCodec->m_EvtTask,
                   QLCODEC_EVENT_DMA_WRITE_DONE
                   );
    QPOSMSetEvtgrp(This->m_pMpegCodec->m_EvtSyncDma,
                   QLCODEC_EVENT_DMA_SYNC_WRITE_DONE
                   );
    return (QPERR_SUCCESS);
}

static EQP_Errors DM_StartDMARead(IHCIAPI * This,
								  TQP_UINT32 ulAddr,
								  TQP_UINT32 ulLength,
								  PTQP_UINT8 pHostAddr,
								  TQP_UINT32 ulXferMode,
								  TQP_UINT32 ulPicWidthMB,
								  TQP_UINT32 ulPicHeightMB,
								  TQP_UINT32 ulSwap,
								  TQP_BOOL IsFrameMode
								  )
{
	P2M_M2P_DMA_CONTROL		cntl;
	HOST_DMA_CNTL_STATUS	dmaCntl;
	TQP_UINT32				ulStartAddr = ulAddr & 0xFFFFFFFC;
	TQP_UINT32				ulEndAddr = ((ulAddr + ulLength + 0x03) & 0xFFFFFFFC);
	TQP_UINT32				ulStartExtra = ulAddr - ulStartAddr;
	TQP_UINT32				ulEndExtra = ulEndAddr - (ulAddr + ulLength);
	TQP_UINT32				ulDMALength = ulEndAddr - ulStartAddr;
	PTQP_UINT32				ptr = (PTQP_UINT32)pHostAddr;
	TQP_PVOID				StreamPort;
	TQP_UINT16				vp1, vp2;
	TQP_UINT32				i;

#ifndef INTEGRATE_USB_EMU_INT_THREAD_TO_TASK_THREAD
    // create emulated interrupt thread
    //
    if (!CThread_IsThreadInited(&This->m_Thread))
    {
        CThread_ThreadInit(&This->m_Thread);
    }
#endif

#if 0	//close swap TODO:
	// setup DMA byte swap
	//
	cntl.Read = 0;
	QPHCI_RegisterRead(This,
						REG_P2M_M2P_DMA_CONTROL,
						&cntl.Read
						);

	cntl.Reg.me = ((0 != ulSwap) ? 1 : 0);
	QPHCI_RegisterWrite(This,
						REG_P2M_M2P_DMA_CONTROL,
						cntl.Read
						);
#endif
    // setup DMA control register
    //
    dmaCntl.Read = 0;

    dmaCntl.Reg.addr = QPHCI_DM_MEMMAPPING_STREAM_PORT;
    dmaCntl.Reg.dword = 1;
    dmaCntl.Reg.dr_rst = 1;
    dmaCntl.Reg.dw_rst = 1;
    dmaCntl.Reg.d_en = 0;

    QPHCI_RegisterWrite(This,
                        REG_HOST_DMA_CNTL_STATUS,
                        dmaCntl.Read
                        );

    dmaCntl.Reg.dr_rst = 0;
    dmaCntl.Reg.dw_rst = 0;
    dmaCntl.Reg.d_en = 1;

    QPHCI_RegisterWrite(This,
                        REG_HOST_DMA_CNTL_STATUS,
                        dmaCntl.Read
                        );

    // setup start address
    //
    QPHCI_RegisterWrite(This,
                        REG_HOST_DMA_RD_BUFFER_START_ADDR,
                        ulStartAddr << 2
                        );

    // setup end address
    //
    QPHCI_RegisterWrite(This,
                        REG_HOST_DMA_RD_BUFFER_END_ADDR,
                        ulEndAddr << 2
                        );

    // setup current address
    //
    QPHCI_RegisterWrite(This,
                        REG_HOST_DMA_RD_BUFFER_CUR_PTR,
                        ulStartAddr << 2
                        );

    // read size
    //
    QPHCI_RegisterWrite(This,
						REG_HOST_DMA_RD_SIZE,
						ulDMALength << 2
						);

    // read from the streaming port
	//
    QPHCI_GetStreamingPortAddress(This, &StreamPort);

	for (i = 0; i < ulStartExtra; i++)
	{
		vp1 = *((volatile TQP_UINT16 *)StreamPort);
		vp2 = *((volatile TQP_UINT16 *)StreamPort);
	}

    for (i = 0; i < ulLength; i++)
    {
		vp1 = *((volatile TQP_UINT16 *)StreamPort);
		vp2 = *((volatile TQP_UINT16 *)StreamPort);
		*ptr++ = (vp1) | (vp2 << 16);
	}

	for (i = 0; i < ulEndExtra; i++)
	{
		vp1 = *((volatile TQP_UINT16 *)StreamPort);
		vp2 = *((volatile TQP_UINT16 *)StreamPort);
    }

    QPOSMSetEvtgrp(This->m_pMpegCodec->m_EvtTask,
                   QLCODEC_EVENT_DMA_READ_DONE
                   );
    QPOSMSetEvtgrp(This->m_pMpegCodec->m_EvtSyncDma,
                   QLCODEC_EVENT_DMA_SYNC_READ_DONE
                   );

	return (QPERR_SUCCESS);
}

#endif //!CONFIG_HOST_PLATFORM_STR8133_REF


static EQP_Errors DM_ResetArm(IHCIAPI * This,
							  TQP_BOOL fRun
							  )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	TQP_UINT32	dwReg = 0;
	TQP_UINT32	dwMask = 0xFFFFFFFF;
	TQP_INT64	llTimeTo = 3000;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "DM_ResetArm() run(%d)\n",
				 fRun
				 ));

	if (!fRun)
	{
		// disable all hw modules
		//
		QPHCI_RegisterWrite(This,
							0,
							0
							);

		// ARM reset register --> Stop ARM program
		//
		QPHCI_RegisterWrite(This,
							REG_HOST_DIRECT_MODE_MISC,
							0
							);
		// Set arm intrrupt enable, this will be cleared
		// if reset successfully
		//
		QPHCI_RegisterWrite(This,
							REG_HOST_DIRECT_MAPPING_CNTL,
							1
							);

		// Kill timer 1
		// 3.1.1.5 System Timer Control  1
		//
		QPHCI_RegisterWrite(This,
							0x10,
							0x1
							);

		// 3.1.1.8 Current Timer Count
		//
		QPHCI_RegisterRead(This,
						   0x1C,
						   &dwReg
						   );
		// Add an offset
		//
		dwReg += 0xFFFF;

		// 3.1.1.7 Timer Reset Load Count
		//
		QPHCI_RegisterWrite(This,
							0x18,
							dwReg
							);

		// Counter reset CHIP
		// 3.1.1.5 System Timer Control  1
		//
		QPHCI_RegisterWrite(This,
							0x10,
							0x108
							);

		// wait before any PIO access
		//
		DelayMilliSeconds(15);

		// wait until the interrupt bit cleared
		//
		BeginTimer(&llTimeTo);

		do
		{
			QPHCI_RegisterRead(This,
							   REG_HOST_DIRECT_MAPPING_CNTL,
							   &dwMask
							   );
			if (Timeout(llTimeTo,
						1
						))
			{
				break;
			}
		} while (dwMask != 0);

		// Disable WDT after reset
		// 3.1.1.5 System Timer Control  1
		//
		QPHCI_RegisterWrite(This,
							0x10,
							0
							);
		qpStatus = (0 == dwMask) ? QPERR_SUCCESS : QPERR_FAIL;
	}

	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "DM_ResetArm() FAILED!!!!\n"
					 ));
	}
	else
	{
		// clear the mail box
		//
		QPHCI_RegisterWrite(This,
							REG_TO_ARM_MESSAGE_STATUS,
							0
							);

		if (fRun)
		{
			// ARM reset register --> enable ARM running
			//
			QPHCI_RegisterWrite(This,
								REG_HOST_DIRECT_MODE_MISC,
								1
								);
		}
		else
		{
			// stop arm running
			//
			QPHCI_RegisterWrite(This,
								REG_HOST_DIRECT_MODE_MISC,
								0
								);
		}
	}

	return (qpStatus);
}



static EQP_Errors DM_SetInterrupt(IHCIAPI * This,
								  TQP_UINT32 mask
								  )
{
	return (QPERR_NOTSUPP);
}



static EQP_Errors DM_ClearInterrupt(IHCIAPI * This,
									TQP_UINT32 mask
									)
{
	HOST_DIRECT_MAPPING_CNTL	intrStatus;
	EQP_Errors					qpStatus;

	qpStatus = QPHCI_RegisterRead(This,
								  REG_HOST_DIRECT_MAPPING_CNTL,
								  &intrStatus.Read
								  );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "DM_ClearInterrupt() QPHCI_RegisterRead FAILED!!!! status(%d)\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	// clear interrupt, make sure we don't accidently clear
	// other interrupts
	//
	intrStatus.Reg.arm_s = (mask & HCI_INTERRUPT_ARM) != 0;
	intrStatus.Reg.sr_s = (mask & HCI_INTERRUPT_M2P) != 0;
	intrStatus.Reg.sw_s = (mask & HCI_INTERRUPT_P2M) != 0;

	if (intrStatus.Read)
	{
		qpStatus = QPHCI_RegisterWrite(This,
									   REG_HOST_DIRECT_MAPPING_CNTL,
									   intrStatus.Read
									   );
		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "DM_ClearInterrupt() QPHCI_RegisterWrite FAILED!!!! status(%d)\n",
						 qpStatus
						 ));
			return (qpStatus);
		}
	}

	return (QPERR_SUCCESS);
}



static EQP_Errors DM_EnableInterrupt(IHCIAPI * This,
									 TQP_UINT32 mask
									 )
{
	HOST_DIRECT_MAPPING_CNTL	intrStatus;
	EQP_Errors					qpStatus;

	qpStatus = QPHCI_RegisterRead(This,
								  REG_HOST_DIRECT_MAPPING_CNTL,
								  &intrStatus.Read
								  );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "DM_EnableInterrupt() QPHCI_RegisterRead FAILED!!!! status(%d)\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	// enable specified interrupts while not touching
	// others
	//
	if (mask & HCI_INTERRUPT_ARM)
		intrStatus.Reg.arm_e = 1;
	if (mask & HCI_INTERRUPT_M2P)
		intrStatus.Reg.sr_e = 1;
	if (mask & HCI_INTERRUPT_P2M)
		intrStatus.Reg.sw_e = 1;

	// make sure we don't accidently clear any interrupts
	//
	intrStatus.Reg.arm_s = 0;
	intrStatus.Reg.sr_s = 0;
	intrStatus.Reg.sw_s = 0;

	if (intrStatus.Read)
	{
		qpStatus = QPHCI_RegisterWrite(This,
									   REG_HOST_DIRECT_MAPPING_CNTL,
									   intrStatus.Read
									   );
		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "DM_EnableInterrupt() QPHCI_RegisterWrite FAILED!!!! status(%d)\n",
						 qpStatus
						 ));
			return (qpStatus);
		}
	}

	return (QPERR_SUCCESS);
}



static EQP_Errors DM_DisableInterrupt(IHCIAPI * This,
									  TQP_UINT32 mask
									  )
{
	HOST_DIRECT_MAPPING_CNTL	intrStatus;
	EQP_Errors					qpStatus;

	qpStatus = QPHCI_RegisterRead(This,
								  REG_HOST_DIRECT_MAPPING_CNTL,
								  &intrStatus.Read
								  );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "DM_DisableInterrupt() QPHCI_RegisterRead FAILED!!!! status(%d)\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	// disable specified interrupts while not touching
	// others
	//
	if (mask & HCI_INTERRUPT_ARM)
		intrStatus.Reg.arm_e = 0;
	if (mask & HCI_INTERRUPT_M2P)
		intrStatus.Reg.sr_e = 0;
	if (mask & HCI_INTERRUPT_P2M)
		intrStatus.Reg.sw_e = 0;

	// make sure we don't accidently clear any interrupts
	//
	intrStatus.Reg.arm_s = 0;
	intrStatus.Reg.sr_s = 0;
	intrStatus.Reg.sw_s = 0;

	qpStatus = QPHCI_RegisterWrite(This,
								   REG_HOST_DIRECT_MAPPING_CNTL,
								   intrStatus.Read
								   );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "DM_DisableInterrupt() QPHCI_RegisterWrite FAILED!!!! status(%d)\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	return (QPERR_SUCCESS);
}



static EQP_Errors DM_GetInterruptStatus(IHCIAPI * This,
										PTQP_UINT32 pmask
										)
{
	HOST_DIRECT_MAPPING_CNTL	intrStatus;
	EQP_Errors					qpStatus;
	TQP_UINT32					mask = 0;

	qpStatus = QPHCI_RegisterRead(This,
								  REG_HOST_DIRECT_MAPPING_CNTL,
								  &intrStatus.Read
								  );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "DM_GetInterruptStatus() QPHCI_RegisterRead FAILED!!!! status(%d)\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	// return the interrupt status
	//
	if (intrStatus.Reg.arm_s)
		mask |= HCI_INTERRUPT_ARM;
	if (intrStatus.Reg.sr_s)
		mask |= HCI_INTERRUPT_M2P;
	if (intrStatus.Reg.sw_s)
		mask |= HCI_INTERRUPT_P2M;

	*pmask = mask;

	return (QPERR_SUCCESS);
}



static EQP_Errors DM_GetInterruptMask(IHCIAPI * This,
									  PTQP_UINT32 pmask
									  )
{
	HOST_DIRECT_MAPPING_CNTL	intrStatus;
	EQP_Errors					qpStatus;
	TQP_UINT32					mask = 0;

	qpStatus = QPHCI_RegisterRead(This,
								  REG_HOST_DIRECT_MAPPING_CNTL,
								  &intrStatus.Read
								  );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "DM_GetInterruptMask() QPHCI_RegisterRead FAILED!!!! status(%d)\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	// return the interrupt status
	//
	if (intrStatus.Reg.arm_e)
		mask |= HCI_INTERRUPT_ARM;
	if (intrStatus.Reg.sr_e)
		mask |= HCI_INTERRUPT_M2P;
	if (intrStatus.Reg.sw_e)
		mask |= HCI_INTERRUPT_P2M;

	*pmask = mask;

	return (QPERR_SUCCESS);
}



static EQP_Errors DM_SetInterruptMask(IHCIAPI * This,
									  TQP_UINT32 mask
									  )
{
	HOST_DIRECT_MAPPING_CNTL	intrStatus;
	EQP_Errors					qpStatus;

	intrStatus.Read = 0;

	if (mask & HCI_INTERRUPT_ARM)
		intrStatus.Reg.arm_e = 1;
	if (mask & HCI_INTERRUPT_M2P)
		intrStatus.Reg.sr_e = 1;
	if (mask & HCI_INTERRUPT_P2M)
		intrStatus.Reg.sw_e = 1;

	qpStatus = QPHCI_RegisterWrite(This,
								   REG_HOST_DIRECT_MAPPING_CNTL,
								   intrStatus.Read
								   );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "DM_SetInterruptMask() QPHCI_RegisterWrite FAILED!!!! status(%d)\n",
					 qpStatus
					 ));
	}

	return (qpStatus);
}



/////////////////////////////////////////////////////////////////////////////
//
//


// Indirect Mode access function
//

static EQP_Errors IDM_ResetArm(IHCIAPI * This,
							   TQP_BOOL fRun
							   )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	TQP_UINT32	dwReg = 0;
	TQP_UINT8	cMask = 0xFF;
	TQP_INT64	llTimeTo = 3000;


	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "IDM_ResetArm() run(%d)\n",
				 fRun
				 ));

	if (!fRun)
	{
		// disable all hw modules
		//
		QPHCI_RegisterWrite(This,
							0,
							0
							);
		// ARM reset register --> Stop ARM program
		//
		QPHCI_WriteHciRegister(This,
							   REG_HCI_CONTROL,
							   0
							   );

		// Set arm intrrupt enable, this will be cleared
		// if reset successfully
		//
		QPHCI_WriteHciRegister(This,
							   REG_HCI_INTERRUPT_ENABLE,
							   1
							   );

		// Kill timer 1
		// 3.1.1.5 System Timer Control  1
		//
		QPHCI_RegisterWrite(This,
							0x10,
							0x1
							);

		// 3.1.1.8 Current Timer Count
		//
		QPHCI_RegisterRead(This,
						   0x1C,
						   &dwReg
						   );
		// Add an offset
		//
		dwReg += 0xFFFF;

		// 3.1.1.7 Timer Reset Load Count
		//
		QPHCI_RegisterWrite(This,
							0x18,
							dwReg
							);

		// Counter reset CHIP
		// 3.1.1.5 System Timer Control  1
		//
		QPHCI_RegisterWrite(This,
							0x10,
							0x108
							);

		// wait before any PIO access
		//
		DelayMilliSeconds(15);


		// wait until the interrupt bit cleared
		//
		BeginTimer(&llTimeTo);

		do
		{
			QPHCI_ReadHciRegister(This,
								  REG_HCI_INTERRUPT_ENABLE,
								  &cMask
								  );
			if (Timeout(llTimeTo,
						1
						))
			{
				break;
			}
		} while (cMask != 0);

		// Disable WDT after reset
		// 3.1.1.5 System Timer Control  1
		//
		QPHCI_RegisterWrite(This,
							0x10,
							0
							);
		qpStatus = (0 == cMask) ? QPERR_SUCCESS : QPERR_FAIL;
	}

	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "IDM_ResetArm() FAILED!!!!\n"
					 ));
	}
	else
	{
		// clear the mail box
		//
		QPHCI_RegisterWrite(This,
							REG_TO_ARM_MESSAGE_STATUS,
							0
							);

		if (fRun)
		{
			// ARM reset register --> enable ARM running
			//
			QPHCI_WriteHciRegister(This,
								   REG_HCI_CONTROL,
								   1
								   );
		}
		else
		{
			// stop arm running
			//
			QPHCI_WriteHciRegister(This,
								   REG_HCI_CONTROL,
								   0
								   );
		}
	}

	return (qpStatus);
}



static EQP_Errors IDM_SetInterrupt(IHCIAPI * This,
								   TQP_UINT32 mask
								   )
{
	return (QPERR_NOTSUPP);
}



static EQP_Errors IDM_ClearInterrupt(IHCIAPI * This,
									 TQP_UINT32 mask
									 )
{
	INTERRUPT_STATUS	intrStatus;
	EQP_Errors			qpStatus;

	qpStatus = QPHCI_ReadHciRegister(This,
									 REG_HCI_INTERRUPT_STATUS,
									 &intrStatus.Read
									 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "IDM_ClearInterrupt() QPHCI_ReadHciRegister FAILED!!!! status(%d)\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	// clear interrupt, make sure we don't accidently clear
	// other interrupts
	//
	intrStatus.Reg.arm = (mask & HCI_INTERRUPT_ARM) != 0;
	intrStatus.Reg.m2p = (mask & HCI_INTERRUPT_M2P) != 0;
	intrStatus.Reg.p2m = (mask & HCI_INTERRUPT_P2M) != 0;

	if (intrStatus.Read)
	{
		qpStatus = QPHCI_WriteHciRegister(This,
										  REG_HCI_INTERRUPT_STATUS,
										  intrStatus.Read
										  );
		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "DM_ClearInterrupt() QPHCI_WriteHciRegister FAILED!!!! status(%d)\n",
						 qpStatus
						 ));
			return (qpStatus);
		}
	}

	return (QPERR_SUCCESS);
}



static EQP_Errors IDM_EnableInterrupt(IHCIAPI * This,
									  TQP_UINT32 mask
									  )
{
	INTERRUPT_ENABLE	intrStatus;
	EQP_Errors			qpStatus;

	qpStatus = QPHCI_ReadHciRegister(This,
									 REG_HCI_INTERRUPT_ENABLE,
									 &intrStatus.Read
									 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "IDM_EnableInterrupt() QPHCI_ReadHciRegister FAILED!!!! status(%d)\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	// enable specified interrupts while not touching
	// others
	//
	if (mask & HCI_INTERRUPT_ARM)
		intrStatus.Reg.arm = 1;
	if (mask & HCI_INTERRUPT_M2P)
		intrStatus.Reg.m2p = 1;
	if (mask & HCI_INTERRUPT_P2M)
		intrStatus.Reg.p2m = 1;

	if (intrStatus.Read)
	{
		qpStatus = QPHCI_WriteHciRegister(This,
										  REG_HCI_INTERRUPT_ENABLE,
										  intrStatus.Read
										  );
		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "IDM_EnableInterrupt() QPHCI_WriteHciRegister FAILED!!!! status(%d)\n",
						 qpStatus
						 ));
			return (qpStatus);
		}
	}

	return (QPERR_SUCCESS);
}



static EQP_Errors IDM_DisableInterrupt(IHCIAPI * This,
									   TQP_UINT32 mask
									   )
{
	INTERRUPT_ENABLE	intrStatus;
	EQP_Errors			qpStatus;

	qpStatus = QPHCI_ReadHciRegister(This,
									 REG_HCI_INTERRUPT_ENABLE,
									 &intrStatus.Read
									 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "IDM_DisableInterrupt() QPHCI_ReadHciRegister FAILED!!!! status(%d)\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	// disable specified interrupts while not touching
	// others
	//
	if (mask & HCI_INTERRUPT_ARM)
		intrStatus.Reg.arm = 0;
	if (mask & HCI_INTERRUPT_M2P)
		intrStatus.Reg.m2p = 0;
	if (mask & HCI_INTERRUPT_P2M)
		intrStatus.Reg.p2m = 0;

	qpStatus = QPHCI_WriteHciRegister(This,
									  REG_HCI_INTERRUPT_ENABLE,
									  intrStatus.Read
									  );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "IDM_DisableInterrupt() QPHCI_WriteHciRegister FAILED!!!! status(%d)\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	return (QPERR_SUCCESS);
}



static EQP_Errors IDM_GetInterruptStatus(IHCIAPI * This,
										 PTQP_UINT32 pmask
										 )
{
	INTERRUPT_STATUS	intrStatus;
	EQP_Errors			qpStatus;
	TQP_UINT32			mask = 0;

	qpStatus = QPHCI_ReadHciRegister(This,
									 REG_HCI_INTERRUPT_STATUS,
									 &intrStatus.Read
									 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_EXTRACE),
					 "IDM_GetInterruptStatus() QPHCI_ReadHciRegister FAILED!!!! status(%d)\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	// return the interrupt status
	//
	if (intrStatus.Reg.arm)
		mask |= HCI_INTERRUPT_ARM;
	if (intrStatus.Reg.m2p)
		mask |= HCI_INTERRUPT_M2P;
	if (intrStatus.Reg.p2m)
		mask |= HCI_INTERRUPT_P2M;

	*pmask = mask;

	return (QPERR_SUCCESS);
}



static EQP_Errors IDM_GetInterruptMask(IHCIAPI * This,
									   PTQP_UINT32 pmask
									   )
{
	INTERRUPT_ENABLE	intrStatus;
	EQP_Errors			qpStatus;
	TQP_UINT32			mask = 0;

	qpStatus = QPHCI_ReadHciRegister(This,
									 REG_HCI_INTERRUPT_ENABLE,
									 &intrStatus.Read
									 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "IDM_GetInterruptMask() QPHCI_ReadHciRegister FAILED!!!! status(%d)\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	// return the interrupt mask
	//
	if (intrStatus.Reg.arm)
		mask |= HCI_INTERRUPT_ARM;
	if (intrStatus.Reg.m2p)
		mask |= HCI_INTERRUPT_M2P;
	if (intrStatus.Reg.p2m)
		mask |= HCI_INTERRUPT_P2M;

	*pmask = mask;

	return (QPERR_SUCCESS);
}



static EQP_Errors IDM_SetInterruptMask(IHCIAPI * This,
									   TQP_UINT32 mask
									   )
{
	INTERRUPT_ENABLE	intrStatus;
	EQP_Errors			qpStatus;

	intrStatus.Read = 0;

	if (mask & HCI_INTERRUPT_ARM)
		intrStatus.Reg.arm = 1;
	if (mask & HCI_INTERRUPT_M2P)
		intrStatus.Reg.m2p = 1;
	if (mask & HCI_INTERRUPT_P2M)
		intrStatus.Reg.p2m = 1;

	qpStatus = QPHCI_WriteHciRegister(This,
									  REG_HCI_INTERRUPT_ENABLE,
									  intrStatus.Read
									  );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "IDM_SetInterruptMask() QPHCI_WriteHciRegister FAILED!!!! status(%d)\n",
					 qpStatus
					 ));
	}

	return (qpStatus);
}




/////////////////////////////////////////////////////////////////////////////
//
//

#define REG_ARM_IRQ_ENABLE					0x0004
#define REG_ARM_FIQ_ENABLE					0x0008
#define REG_ARM_INTERRUPT_STATUS			0x000C
#define REG_ARM_SW_INTERRUPT_CNTL			0x0024

#define ARM_INTERRUPT_MASK_SW				(1 << 24)	// use SW0
#define ARM_INTERRUPT_MASK_SW_1				(1 << 25)	// SW1


#if 0
static EQP_Errors MM_DMAXferDone(IHCIAPI * This)
{
	if (This->DMAXferDone) {
		return This->DMAXferDone(This);
	} else {
		return (QPERR_SUCCESS);
	}
}
#endif


static EQP_Errors MM_DMAWriteDone(IHCIAPI * This)
{
	QPOSMSetEvtgrp(This->m_pMpegCodec->m_EvtTask,
				   QLCODEC_EVENT_DMA_WRITE_DONE
				   );
	QPOSMSetEvtgrp(This->m_pMpegCodec->m_EvtSyncDma,
				   QLCODEC_EVENT_DMA_SYNC_WRITE_DONE
				   );

	This->DMAXferDone = QP_NULL;

	return (QPERR_SUCCESS);
}


static EQP_Errors MM_DMAReadDone(IHCIAPI * This)
{
	QPOSMSetEvtgrp(This->m_pMpegCodec->m_EvtTask,
				   QLCODEC_EVENT_DMA_READ_DONE
				   );
	QPOSMSetEvtgrp(This->m_pMpegCodec->m_EvtSyncDma,
				   QLCODEC_EVENT_DMA_SYNC_READ_DONE
				   );

	This->DMAXferDone = QP_NULL;

	return (QPERR_SUCCESS);
}


static void MM_DMAWriteComplete(void * refCon,
								EQP_MMM2MReqStates state
								)
{
	IHCIAPI	*pHCI = (IHCIAPI *)refCon;

	QPOSMSetEvtgrp(pHCI->m_pMpegCodec->m_EvtTask,
				   QLCODEC_EVENT_DMA_WRITE_DONE
				   );
	QPOSMSetEvtgrp(pHCI->m_pMpegCodec->m_EvtSyncDma,
				   QLCODEC_EVENT_DMA_SYNC_WRITE_DONE
				   );
}



static void MM_DMAReadComplete(void * refCon,
							   EQP_MMM2MReqStates state
							   )
{
	IHCIAPI	*pHCI = (IHCIAPI *)refCon;

	QPOSMSetEvtgrp(pHCI->m_pMpegCodec->m_EvtTask,
				   QLCODEC_EVENT_DMA_READ_DONE
				   );
	QPOSMSetEvtgrp(pHCI->m_pMpegCodec->m_EvtSyncDma,
				   QLCODEC_EVENT_DMA_SYNC_READ_DONE
				   );
}



// Master Mode access function
//

static EQP_Errors MM_ReadHciRegister(IHCIAPI *This,
									 TQP_UINT8 cAddr,
									 PTQP_UINT8 pData
									 )
{
	return (QPERR_NOTSUPP);
}



static EQP_Errors MM_WriteHciRegister(IHCIAPI * This,
									  TQP_UINT8 cAddr,
									  TQP_UINT8 ucData
									  )
{
	return (QPERR_NOTSUPP);
}



static EQP_Errors MM_RegisterRead(IHCIAPI * This,
								  TQP_UINT16 wAddr,
								  PTQP_UINT32 pData
								  )
{
	*pData = *(TQP_UINT32 volatile *)(This->m_pRegisterBase + wAddr);
	return (QPERR_SUCCESS);
}



static EQP_Errors MM_RegisterWrite(IHCIAPI * This,
								   TQP_UINT16 wAddr,
								   TQP_UINT32 dwData
								   )
{
	*((TQP_UINT32 volatile *)(This->m_pRegisterBase + wAddr)) = dwData;
	return (QPERR_SUCCESS);
}



static EQP_Errors MM_MemoryRead(IHCIAPI * This,
								TQP_UINT32 dwAddr,
								PTQP_UINT32 pData
								)
{
	*pData = *(TQP_UINT32 volatile *)((PTQP_INT8)This->m_pMemoryBase + dwAddr);
	return (QPERR_SUCCESS);
}



static EQP_Errors MM_MemoryWrite(IHCIAPI * This,
								 TQP_UINT32 dwAddr,
								 TQP_UINT32 dwData
								 )
{
	*((TQP_UINT32 volatile *)((PTQP_INT8)This->m_pMemoryBase + dwAddr)) = dwData;
	return (QPERR_SUCCESS);
}



static EQP_Errors MM_StartDMAWrite(IHCIAPI *This,
								   TQP_UINT32 ulAddr,
								   TQP_UINT32 ulLength,
								   PTQP_UINT8 pHostAddr,
								   TQP_UINT32 ulXferMode,
								   TQP_UINT32 ulPicWidthMB,
								   TQP_UINT32 ulPicHeightMB,
								   TQP_UINT32 ulSwap,
								   TQP_BOOL IsFrameMode
								   )
{
#if !defined(CONFIG_HOST_PLATFORM_PL330SOC) || defined(PL330SOC_USE_MEMCPY_INSTEAD_OF_M2M_ENGINE)
#else
	TQP_MMM2MReqID		reqID;
	TQP_MMM2MReqParms	parm;
#endif
	EQP_Errors		qpStatus = QPERR_SUCCESS;

#ifdef QP_LINXKER
	//printk("%s()-> ulAddr(0x%08x) pHostAddr(0x%08x) ulLength(0x%08x)\n", __FUNCTION__, ulAddr << 2, pHostAddr, ulLength << 2);

	if (((ulXferMode & DMA_TRANSFER_MODE_LINEAR) != DMA_TRANSFER_MODE_LINEAR)/* || (ulSwap != QP_FALSE)*/)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "%s() QPERR_NOTSUPP\n",
					 __FUNCTION__
					 ));
		return (QPERR_NOTSUPP);
	}

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC) || defined(PL330SOC_USE_MEMCPY_INSTEAD_OF_M2M_ENGINE)
	if (ulSwap) {
		BytesSwap(pHostAddr, (PTQP_UINT8) (ulAddr << 2), ulLength << 2);
	} else {
		memcpy((PTQP_UINT8) (ulAddr << 2), pHostAddr, ulLength << 2);
	}

	MM_DMAWriteComplete(This, 0);
#else
	parm.srcAddr = pHostAddr;
	parm.dstAddr = ulAddr << 2;
	parm.bytes = (ulLength << 2);
	parm.byteSwap = (ulSwap != 0) ? QPMM_M2MBYTESWAP_32 : QPMM_M2MBYTESWAP_NONE;

	This->DMAXferDone = MM_DMAWriteDone;

	qpStatus = QPMMStartM2MRequest(&parm,
								   MM_DMAWriteComplete,
								   (void *)This,
								   &reqID
								   );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "MM_StartDMAWrite() QPMMStartM2MRequest FAILED!!!! status(%d)\n",
					 qpStatus
					 ));
	}
#endif
#endif //QP_LINXKER

#if defined(_QP_QPSOS)
	parm.srcAddr = (void *)QPMMVir2Phy(pHostAddr);
	parm.dstAddr = (void *)(ulAddr << 2);
	parm.bytes = (ulLength << 2);
	parm.byteSwap = (ulSwap != 0) ? QPMM_M2MBYTESWAP_32 : QPMM_M2MBYTESWAP_NONE;

	qpStatus = QPMMStartM2MRequest(&parm,
								   MM_DMAWriteComplete,
								   (void *)This,
								   &reqID
								   );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "MM_StartDMAWrite() QPMMStartM2MRequest FAILED!!!! status(%d)\n",
					 qpStatus
					 ));
	}
#endif //_QP_QPSOS

	return (qpStatus);
}



static EQP_Errors MM_StartDMARead(IHCIAPI * This,
								  TQP_UINT32 ulAddr,
								  TQP_UINT32 ulLength,
								  PTQP_UINT8 pHostAddr,
								  TQP_UINT32 ulXferMode,
								  TQP_UINT32 ulPicWidthMB,
								  TQP_UINT32 ulPicHeightMB,
								  TQP_UINT32 ulSwap,
								  TQP_BOOL IsFrameMode
								  )
{
#if !defined(CONFIG_HOST_PLATFORM_PL330SOC) || defined(PL330SOC_USE_MEMCPY_INSTEAD_OF_M2M_ENGINE)
#else
	TQP_MMM2MReqID		reqID;
	TQP_MMM2MReqParms	parm;
#endif
	EQP_Errors		qpStatus = QPERR_SUCCESS;

#ifdef QP_LINXKER
	//printk("%s()-> ulAddr(0x%08x) pHostAddr(0x%08x) ulLength(0x%08x)\n", __FUNCTION__, ulAddr << 2, pHostAddr, ulLength << 2);

	if (((ulXferMode & DMA_TRANSFER_MODE_LINEAR) != DMA_TRANSFER_MODE_LINEAR)/* || (ulSwap != QP_FALSE)*/)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "%s() QPERR_NOTSUPP\n",
					 __FUNCTION__
					 ));
		return (QPERR_NOTSUPP);
	}

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC) || defined(PL330SOC_USE_MEMCPY_INSTEAD_OF_M2M_ENGINE)
	if (ulSwap) {
		BytesSwap( (PTQP_UINT8) (ulAddr << 2), pHostAddr, ulLength << 2);
	} else {
		memcpy(pHostAddr,  (PTQP_UINT8) (ulAddr << 2), ulLength << 2);
	}

	MM_DMAReadComplete(This, 0);
#else
	parm.srcAddr = ulAddr << 2;
	parm.dstAddr = pHostAddr;
	parm.bytes = (ulLength << 2);
	parm.byteSwap = (ulSwap != 0) ? QPMM_M2MBYTESWAP_32 : QPMM_M2MBYTESWAP_NONE;

	This->DMAXferDone = MM_DMAReadDone;

	qpStatus = QPMMStartM2MRequest(&parm,
								   MM_DMAReadComplete,
								   (void *)This,
								   &reqID
								   );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "MM_StartDMARead() QPMMStartM2MRequest FAILED!!!! status(%d)\n",
					 qpStatus
					 ));
	}
#endif
#endif //QP_LINXKER


#if defined(_QP_QPSOS)
	parm.srcAddr = (void *)(ulAddr << 2);
	parm.dstAddr = (void *)QPMMVir2Phy(pHostAddr);
	parm.bytes = (ulLength << 2);
	parm.byteSwap = (ulSwap != 0) ? QPMM_M2MBYTESWAP_32 : QPMM_M2MBYTESWAP_NONE;

	qpStatus = QPMMStartM2MRequest(&parm,
								   MM_DMAReadComplete,
								   (void *)This,
								   &reqID
								   );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "MM_StartDMARead() QPMMStartM2MRequest FAILED!!!! status(%d)\n",
					 qpStatus
					 ));
	}
#endif //_QP_QPSOS

	return (qpStatus);
}



static EQP_Errors MM_ResetArm(IHCIAPI * This,
							  TQP_BOOL fRun
							  )
{
	return (QPERR_SUCCESS);
}



static EQP_Errors MM_SetInterrupt(IHCIAPI * This,
								  TQP_UINT32 mask
								  )
{
	EQP_Errors	qpStatus;
	TQP_UINT32	dwReg;

	if (0 == (mask & HCI_INTERRUPT_ARM))
		return (QPERR_NOTSUPP);

	qpStatus = QPHCI_RegisterRead(This,
								  REG_ARM_SW_INTERRUPT_CNTL,
								  &dwReg
								  );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "MM_SetInterrupt() QPHCI_RegisterRead FAILED!!!! status(%d)\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	// make sure we do not accidently clear
	// the software interrupt
	//
	if (dwReg & ARM_INTERRUPT_MASK_SW_1)
		return (QPERR_SUCCESS);

	// set the software interrupt for the firmware
	//
	qpStatus = QPHCI_RegisterWrite(This,
								   REG_ARM_SW_INTERRUPT_CNTL,
								   ARM_INTERRUPT_MASK_SW_1
								   );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "MM_SetInterrupt() QPHCI_RegisterWrite FAILED!!!! status(%d)\n",
					 qpStatus
					 ));
	}

	return (qpStatus);
}



static EQP_Errors MM_ClearInterrupt(IHCIAPI * This,
									TQP_UINT32 mask
									)
{
	EQP_Errors	qpStatus;
	TQP_UINT32	dwReg;

	if (0 == (mask & HCI_INTERRUPT_ARM))
		return (QPERR_NOTSUPP);

	qpStatus = QPHCI_RegisterRead(This,
								  REG_ARM_SW_INTERRUPT_CNTL,
								  &dwReg
								  );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "MM_ClearInterrupt() QPHCI_RegisterRead FAILED!!!! status(%d)\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	// make sure we do not accidently trigger
	// a software interrupt
	//
	if (0 == (dwReg & ARM_INTERRUPT_MASK_SW))
		return (QPERR_SUCCESS);

	// clear our software interrupt
	//
	qpStatus = QPHCI_RegisterWrite(This,
								   REG_ARM_SW_INTERRUPT_CNTL,
								   ARM_INTERRUPT_MASK_SW
								   );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "MM_ClearInterrupt() QPHCI_RegisterWrite FAILED!!!! status(%d)\n",
					 qpStatus
					 ));
	}

	return (qpStatus);
}



static EQP_Errors MM_EnableInterrupt(IHCIAPI * This,
									 TQP_UINT32 mask
									 )
{
	EQP_Errors	qpStatus;
	TQP_UINT32	dwReg;

	if (0 == (mask & HCI_INTERRUPT_ARM))
		return (QPERR_NOTSUPP);

	qpStatus = QPHCI_RegisterRead(This,
								  REG_ARM_IRQ_ENABLE,
								  &dwReg
								  );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "MM_EnableInterrupt() QPHCI_RegisterRead FAILED!!!! status(%d)\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	// enable specified interrupts while not touching
	// others
	//
	if (mask & HCI_INTERRUPT_ARM)
		dwReg |= ARM_INTERRUPT_MASK_SW;

	qpStatus = QPHCI_RegisterWrite(This,
								   REG_ARM_IRQ_ENABLE,
								   dwReg
								   );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "MM_EnableInterrupt() QPHCI_RegisterWrite FAILED!!!! status(%d)\n",
					 qpStatus
					 ));
	}

	return (qpStatus);
}



static EQP_Errors MM_DisableInterrupt(IHCIAPI * This,
									  TQP_UINT32 mask
									  )
{
	EQP_Errors	qpStatus;
	TQP_UINT32	dwReg;

	if (0 == (mask & HCI_INTERRUPT_ARM))
		return (QPERR_NOTSUPP);

	qpStatus = QPHCI_RegisterRead(This,
								  REG_ARM_IRQ_ENABLE,
								  &dwReg
								  );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "MM_DisableInterrupt() QPHCI_RegisterRead FAILED!!!! status(%d)\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	// disable specified interrupts while not touching
	// others
	//
	if (mask & HCI_INTERRUPT_ARM)
		dwReg &= ~ARM_INTERRUPT_MASK_SW;

	qpStatus = QPHCI_RegisterWrite(This,
								   REG_ARM_IRQ_ENABLE,
								   dwReg
								   );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "MM_DisableInterrupt() QPHCI_RegisterWrite FAILED!!!! status(%d)\n",
					 qpStatus
					 ));
	}

	return (qpStatus);
}



static EQP_Errors MM_GetInterruptStatus(IHCIAPI * This,
										PTQP_UINT32 pmask
										)
{
	TQP_UINT32	mask = 0;
	EQP_Errors	qpStatus;
	TQP_UINT32	dwReg;

	qpStatus = QPHCI_RegisterRead(This,
								  REG_ARM_INTERRUPT_STATUS,
								  &dwReg
								  );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "MM_GetInterruptStatus() QPHCI_RegisterRead FAILED!!!! status(%d)\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	// return the interrupt status
	//
	if (dwReg & ARM_INTERRUPT_MASK_SW)
		mask |= HCI_INTERRUPT_ARM;

	*pmask = mask;

	return (QPERR_SUCCESS);
}



static EQP_Errors MM_GetInterruptMask(IHCIAPI * This,
									  PTQP_UINT32 pmask
									  )
{
	EQP_Errors	qpStatus;
	TQP_UINT32	dwReg;
	TQP_UINT32	mask = 0;

	qpStatus = QPHCI_RegisterRead(This,
								  REG_ARM_IRQ_ENABLE,
								  &dwReg
								  );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "MM_GetInterruptMask() QPHCI_RegisterRead FAILED!!!! status(%d)\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	// return the interrupt status
	//
	if (dwReg & ARM_INTERRUPT_MASK_SW)
		mask |= HCI_INTERRUPT_ARM;

	*pmask = mask;

	return (QPERR_SUCCESS);
}



static EQP_Errors MM_SetInterruptMask(IHCIAPI * This,
									  TQP_UINT32 mask
									  )
{
	EQP_Errors	qpStatus;
	TQP_UINT32	dwReg;

	qpStatus = QPHCI_RegisterRead(This,
								  REG_ARM_IRQ_ENABLE,
								  &dwReg
								  );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "MM_SetInterruptMask() QPHCI_RegisterRead FAILED!!!! status(%d)\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	if (mask & HCI_INTERRUPT_ARM)
		dwReg |= ARM_INTERRUPT_MASK_SW;
	else
		dwReg &= ~ARM_INTERRUPT_MASK_SW;


	qpStatus = QPHCI_RegisterWrite(This,
								   REG_ARM_IRQ_ENABLE,
								   dwReg
								   );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "MM_SetInterruptMask() QPHCI_RegisterWrite FAILED!!!! status(%d)\n",
					 qpStatus
					 ));
	}

	return (qpStatus);
}



/////////////////////////////////////////////////////////////////////////////
//
//


// 201 Emulation
//

static EQP_Errors EMU_ReadHciRegister(IHCIAPI *This,
									  TQP_UINT8 cAddr,
									  PTQP_UINT8 pData
									  )
{
	TQP_BOOL	bRet;

	bRet = ReadHCIReg(cAddr,
					  pData
					  );
	return (bRet ? QPERR_SUCCESS : QPERR_FAIL);
}



static EQP_Errors EMU_WriteHciRegister(IHCIAPI * This,
									   TQP_UINT8 cAddr,
									   TQP_UINT8 ucData
									   )
{
	TQP_BOOL	bRet;

	bRet = WriteHCIReg(cAddr,
					   ucData
					   );
	return (bRet ? QPERR_SUCCESS : QPERR_FAIL);
}



static EQP_Errors EMU_RegisterRead(IHCIAPI * This,
								   TQP_UINT16 wAddr,
								   PTQP_UINT32 pData
								   )
{
	TQP_BOOL	bRet;

	bRet = ReadReg((TQP_UINT32)wAddr,
				   pData
				   );
	return (bRet ? QPERR_SUCCESS : QPERR_FAIL);
}



static EQP_Errors EMU_RegisterWrite(IHCIAPI * This,
									TQP_UINT16 wAddr,
									TQP_UINT32 dwData
									)
{
	TQP_BOOL	bRet;

	bRet = WriteReg((TQP_UINT32)wAddr,
					dwData
					);
	return (bRet ? QPERR_SUCCESS : QPERR_FAIL);
}




static EQP_Errors EMU_RegisterReadEx(IHCIAPI * This,
									 TQP_UINT16 wAddr,
									 PTQP_UINT32 pData,
									 TQP_UINT32 numReg
									 )
{
	TQP_BOOL	bRet;

	bRet = ReadRegEx((TQP_UINT32)wAddr,
					 pData,
					 numReg
					 );
	return (bRet ? QPERR_SUCCESS : QPERR_FAIL);
}



static EQP_Errors EMU_MemoryRead(IHCIAPI * This,
								 TQP_UINT32 dwAddr,
								 PTQP_UINT32 pData
								 )
{
	TQP_BOOL	bRet;

	bRet = ReadMem(dwAddr,
				   1,
				   pData
				   );
	return (bRet ? QPERR_SUCCESS : QPERR_FAIL);
}



static EQP_Errors EMU_MemoryWrite(IHCIAPI * This,
								  TQP_UINT32 dwAddr,
								  TQP_UINT32 dwData
								  )
{
	TQP_BOOL	bRet;

	bRet = WriteMem(dwAddr,
					1,
					&dwData
					);
	return (bRet ? QPERR_SUCCESS : QPERR_FAIL);
}



static EQP_Errors EMU_StartDMAWrite(IHCIAPI * This,
									TQP_UINT32 ulAddr,
									TQP_UINT32 ulLength,
									PTQP_UINT8 pHostAddr,
									TQP_UINT32 ulXferMode,
									TQP_UINT32 ulPicWidthMB,
									TQP_UINT32 ulPicHeightMB,
									TQP_UINT32 ulSwap,
									TQP_BOOL IsFrameMode
									)
{
	TQP_BOOL	bRet;
////	TQP_INT64	llStart = GetSystemTimeUtil();
////	TQP_INT64	llElapsed = 0;

	bRet = DMA_W(ulAddr << 2,
				 (unsigned long*)pHostAddr,
				 ulLength,
				 ulSwap != 0
				 );
	//Get elapsed time since operation was initiated.
////	llElapsed = GetSystemTimeUtil() - llStart;

////	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
////				 "EMU_StartDMAWrite() time(%d)\n",
////				 (TQP_INT32)(llElapsed & 0xFFFFFFFF)
////				 ));

	QPOSMSetEvtgrp(This->m_pMpegCodec->m_EvtTask,
				   QLCODEC_EVENT_DMA_WRITE_DONE
				   );
	QPOSMSetEvtgrp(This->m_pMpegCodec->m_EvtSyncDma,
				   QLCODEC_EVENT_DMA_SYNC_WRITE_DONE
				   );

	return (bRet ? QPERR_SUCCESS : QPERR_FAIL);
}



static EQP_Errors EMU_StartDMARead(IHCIAPI * This,
								   TQP_UINT32 ulAddr,
								   TQP_UINT32 ulLength,
								   PTQP_UINT8 pHostAddr,
								   TQP_UINT32 ulXferMode,
								   TQP_UINT32 ulPicWidthMB,
								   TQP_UINT32 ulPicHeightMB,
								   TQP_UINT32 ulSwap,
								   TQP_BOOL IsFrameMode
								   )
{
	TQP_BOOL	bRet;
////	TQP_INT64	llStart = GetSystemTimeUtil();
////	TQP_INT64	llElapsed = 0;

	bRet = DMA_R(ulAddr << 2,
				 (unsigned long*)pHostAddr,
				 ulLength,
				 ulSwap != 0
				 );
	//Get elapsed time since operation was initiated.
////	llElapsed = GetSystemTimeUtil() - llStart;

////	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
////				 "EMU_StartDMARead() time(%d)\n",
////				 (TQP_INT32)(llElapsed & 0xFFFFFFFF)
////				 ));
	QPOSMSetEvtgrp(This->m_pMpegCodec->m_EvtTask,
				   QLCODEC_EVENT_DMA_READ_DONE
				   );
	QPOSMSetEvtgrp(This->m_pMpegCodec->m_EvtSyncDma,
				   QLCODEC_EVENT_DMA_SYNC_READ_DONE
				   );
	return (bRet ? QPERR_SUCCESS : QPERR_FAIL);
}



static EQP_Errors EMU_ResetArm(IHCIAPI * This,
							   TQP_BOOL fRun
							   )
{
	TQP_BOOL	bRet;

	bRet = ResetArm(fRun);
	return (bRet ? QPERR_SUCCESS : QPERR_FAIL);
}



static EQP_Errors EMU_EnableInterrupt(IHCIAPI * This,
									  TQP_UINT32 mask
									  )
{
	EQP_Errors	qpStatus;

	if (This->m_bEmulationMode)
	{
		return (QPERR_SUCCESS);
	}

	if ((QPHCI_MODE_DIRECT == This->m_access_mode) || (QPHCI_MODE_MASTER == This->m_access_mode))
	{
		qpStatus = DM_EnableInterrupt(This,
									  mask
									  );
	}
	else
	{
		qpStatus = IDM_EnableInterrupt(This,
									   mask
									   );
	}

#ifndef INTEGRATE_USB_EMU_INT_THREAD_TO_TASK_THREAD
	if (QP_SUCCESS(qpStatus) &&
		mask
		)
	{
		// create emulated interrupt thread
		//
		if (!CThread_IsThreadInited(&This->m_Thread))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
						 "EMU_EnableInterrupt() mask(0x%x) CThread_ThreadInit()\n",
						 mask
						 ));
			CThread_ThreadInit(&This->m_Thread);
		}
	}
#endif

	return (qpStatus);
}


static EQP_Errors EMU_DisableInterrupt(IHCIAPI * This,
									   TQP_UINT32 mask
									   )
{
	EQP_Errors	qpStatus;
	TQP_UINT32	intEnable = 0;

	if (This->m_bEmulationMode)
	{
		return (QPERR_SUCCESS);
	}

	if ((QPHCI_MODE_DIRECT == This->m_access_mode) || (QPHCI_MODE_MASTER == This->m_access_mode))
	{
		qpStatus = DM_DisableInterrupt(This,
									   mask
									   );
	}
	else
	{
		qpStatus = IDM_DisableInterrupt(This,
										mask
										);
	}

	// read out the mask
	//
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_GetInterruptMask(This,
																		 &intEnable
																		 );


	if (QP_SUCCESS(qpStatus))
	{
		if ((0 == intEnable) ||
			((HCI_INTERRUPT_ARM | HCI_INTERRUPT_P2M | HCI_INTERRUPT_M2P) == mask)
			)
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
						 "EMU_DisableInterrupt() call CThread_ThreadDone() enable(0x%x) mask(0x%x)\n",
						 intEnable,
						 mask
						 ));
#ifndef INTEGRATE_USB_EMU_INT_THREAD_TO_TASK_THREAD
			// delete the emulated interrupt thread
			//
			if (CThread_IsThreadInited(&This->m_Thread))
			{
				CThread_ThreadDone(&This->m_Thread);
			}
#endif
		}
	}
	else
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_FATAL),
					 "EMU_DisableInterrupt() failed(%d)!!!\n",
					 qpStatus
					 ));

#ifndef INTEGRATE_USB_EMU_INT_THREAD_TO_TASK_THREAD
		if (CThread_IsThreadInited(&This->m_Thread))
		{
			// delete the emulated interrupt thread
			//
			CThread_ThreadDone(&This->m_Thread);
		}
#endif
	}

	return (qpStatus);
}



static EQP_Errors EMU_SetInterruptMask(IHCIAPI * This,
									  TQP_UINT32 mask
									  )
{
	EQP_Errors	qpStatus;

	if ((QPHCI_MODE_DIRECT == This->m_access_mode) || (QPHCI_MODE_MASTER == This->m_access_mode))
	{
		qpStatus = DM_SetInterruptMask(This,
									   mask
									   );
	}
	else
	{
		qpStatus = IDM_SetInterruptMask(This,
										mask
										);
	}

#ifndef INTEGRATE_USB_EMU_INT_THREAD_TO_TASK_THREAD
	if (mask)
	{
		// create emulated interrupt thread
		//
		if (!CThread_IsThreadInited(&This->m_Thread))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
						 "EMU_SetInterruptMask() mask(0x%x) CThread_ThreadInit()\n",
						 mask
						 ));
			CThread_ThreadInit(&This->m_Thread);
		}
	}
	else
	{
		if (CThread_IsThreadInited(&This->m_Thread))
		{
			// delete the emulated interrupt thread
			//
			CThread_ThreadDone(&This->m_Thread);
		}
	}
#endif

	return (qpStatus);
}


// 201 emulation mode interrupt thread
//

//extern void CQLCodec_InterruptHandler(void * refCon);

static int EMU_ThreadProc(CThread *This_p)
{
	EQP_Errors	qpStatus;
	TQP_UINT32	ulFlaggedEvents = 0;
	IHCIAPI		*This = (IHCIAPI *)This_p->m_context;

#if 0
#ifdef QP_LINXKER
	int		nCount = 0;
#ifdef __LINUX24__
	sprintf(current->comm, "%s", __FUNCTION__);
	daemonize();
#else
	char szThreadName[32];
	sprintf(szThreadName, "%s", __FUNCTION__);
	daemonize(szThreadName);
	current->flags |= PF_NOFREEZE;
#endif
	set_user_nice(current, -10);
#endif
#endif
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "EMU_ThreadProc() ENTER (0x%x) (0x%x)\n",
				 This_p,
				 This
				 ));

	while (QP_TRUE)
	{
		if (QP_NULL == This->m_EvtEmuTask)
			break;

		// wait on the event
		//
		qpStatus = QPOSMWaitEvtgrp(This_p->m_EvtWait,
								   THREAD_EVENT_MASK,
								   &ulFlaggedEvents,
								   0	// polling
								   );

		if ((QPERR_TIMEOUT == qpStatus) ||
			(QPERR_SUCCESS == qpStatus)
			)
		{
			//CQLCodec_InterruptHandler((void *)This->m_pMpegCodec);
			if (!This->m_bEmulationMode)
			{
				QPOSMISREntry(This->m_pMpegCodec->m_interruptNumber);
			}
#ifdef QP_LINXKER
			DelayMilliSeconds(1);
#else
			DelayMilliSeconds(1);
			//DelayMicroSeconds(250);
#endif
			//DelayMilliSeconds(30);
		}

		if (ulFlaggedEvents & THREAD_EVENT_EXIT)
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
						 "EMU_ThreadProc() THREAD_EVENT_EXIT\n"
						 ));
			QPOSMSetEvtgrp(This_p->m_EvtReply,
						   THREAD_EVENT_EXIT_DONE
						   );
			break;
		}
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "EMU_ThreadProc() See Ya!\n"
				 ));

	return (0);
}



/////////////////////////////////////////////////////////////////////////////
//
//


// USB
//

static EQP_Errors USB_ReadHciRegister(IHCIAPI *This,
									  TQP_UINT8 cAddr,
									  PTQP_UINT8 pData
									  )
{
	return (CUsbCntl_ReadHciRegister(This->m_pUsbCntl,
									 cAddr,
									 pData
									 ));
}



static EQP_Errors USB_WriteHciRegister(IHCIAPI * This,
									   TQP_UINT8 cAddr,
									   TQP_UINT8 ucData
									   )
{
	return (CUsbCntl_WriteHciRegister(This->m_pUsbCntl,
									  cAddr,
									  ucData
									  ));
}



static EQP_Errors USB_RegisterRead(IHCIAPI * This,
								   TQP_UINT16 wAddr,
								   PTQP_UINT32 pData
								   )
{
	return (CUsbCntl_RegisterRead(This->m_pUsbCntl,
								  wAddr,
								  pData
								  ));
}



static EQP_Errors USB_RegisterWrite(IHCIAPI * This,
									TQP_UINT16 wAddr,
									TQP_UINT32 dwData
									)
{
	return (CUsbCntl_RegisterWrite(This->m_pUsbCntl,
								   wAddr,
								   dwData
								   ));
}



static EQP_Errors USB_RegisterReadEx(IHCIAPI * This,
									 TQP_UINT16 wAddr,
									 PTQP_UINT32 pData,
									 TQP_UINT32 numReg
									 )
{
	return (CUsbCntl_RegisterReadEx(This->m_pUsbCntl,
									wAddr,
									pData,
									numReg
									));
}


static EQP_Errors USB_RegisterWriteEx(IHCIAPI * This,
									 TQP_UINT16 wAddr,
									 PTQP_UINT32 pData,
									 TQP_UINT32 numReg
									 )
{
	return (CUsbCntl_RegisterWriteEx(This->m_pUsbCntl,
									wAddr,
									pData,
									numReg
									));
}



static EQP_Errors USB_MemoryRead(IHCIAPI * This,
								 TQP_UINT32 dwAddr,
								 PTQP_UINT32 pData
								 )
{
	return (CUsbCntl_MemoryRead(This->m_pUsbCntl,
								dwAddr,
								pData
								));
}

static EQP_Errors USB_MemoryReadEx(IHCIAPI * This,
								 TQP_UINT32 ulDWordAddr,	// in D-WORD(32 bit)
								 TQP_UINT32 ulDWordLength,		// in D-WORD(32 bit)
								 PTQP_UINT32 pData
								 )
{
	return (CUsbCntl_MemoryReadEx(This->m_pUsbCntl,
								  ulDWordAddr,
								  ulDWordLength,
								  pData
								  ));
}


static EQP_Errors USB_MemoryWrite(IHCIAPI * This,
								  TQP_UINT32 dwAddr,
								  TQP_UINT32 dwData
								  )
{
	return (CUsbCntl_MemoryWrite(This->m_pUsbCntl,
								 dwAddr,
								 dwData
								 ));
}



static EQP_Errors USB_StartDMAWrite(IHCIAPI * This,
									TQP_UINT32 ulAddr,
									TQP_UINT32 ulLength,
									PTQP_UINT8 pHostAddr,
									TQP_UINT32 ulXferMode,
									TQP_UINT32 ulPicWidthMB,
									TQP_UINT32 ulPicHeightMB,
									TQP_UINT32 ulSwap,
									TQP_BOOL IsFrameMode
									)
{
	return (CUsbCntl_StartDMAWrite(This->m_pUsbCntl,
								   ulAddr,
								   ulLength,
								   pHostAddr,
								   ulSwap != 0
								   ));
}



static EQP_Errors USB_StartDMARead(IHCIAPI * This,
								   TQP_UINT32 ulAddr,
								   TQP_UINT32 ulLength,
								   PTQP_UINT8 pHostAddr,
								   TQP_UINT32 ulXferMode,
								   TQP_UINT32 ulPicWidthMB,
								   TQP_UINT32 ulPicHeightMB,
								   TQP_UINT32 ulSwap,
								   TQP_BOOL IsFrameMode
								   )
{
	return (CUsbCntl_StartDMARead(This->m_pUsbCntl,
								  ulAddr,
								  ulLength,
								  pHostAddr,
								  ulSwap != 0
								  ));
}



static EQP_Errors USB_ResetArm(IHCIAPI * This,
							   TQP_BOOL fRun
							   )
{
	EQP_Errors	qpStatus;

	qpStatus = CUsbCntl_ResetArm(This->m_pUsbCntl,
								 fRun
								 );

	if (QP_SUCCESS(qpStatus))
	{
		// clear the mail box
		//
		QPHCI_RegisterWrite(This,
							REG_TO_ARM_MESSAGE_STATUS,
							0
							);
	}

	return (qpStatus);
}





/////////////////////////////////////////////////////////////////////////////
//
//


// PCI
//

static EQP_Errors PCI_ReadHciRegister(IHCIAPI *This,
									  TQP_UINT8 cAddr,
									  PTQP_UINT8 pData
									  )
{
	return (QPERR_NOTSUPP);
}



static EQP_Errors PCI_WriteHciRegister(IHCIAPI * This,
									   TQP_UINT8 cAddr,
									   TQP_UINT8 ucData
									   )
{
	return (QPERR_NOTSUPP);
}



static EQP_Errors PCI_RegisterRead(IHCIAPI * This,
								   TQP_UINT16 wAddr,
								   PTQP_UINT32 pData
								   )
{
	return (CPCIeCntl_RegisterRead(This->m_pPCIeCntl,
								   wAddr,
								   pData
								   ));
}



static EQP_Errors PCI_RegisterWrite(IHCIAPI * This,
									TQP_UINT16 wAddr,
									TQP_UINT32 dwData
									)
{
	return (CPCIeCntl_RegisterWrite(This->m_pPCIeCntl,
									wAddr,
									dwData
									));
}



static EQP_Errors PCI_RegisterReadEx(IHCIAPI * This,
									 TQP_UINT16 wAddr,
									 PTQP_UINT32 pData,
									 TQP_UINT32 numReg
									 )
{
	return (CPCIeCntl_RegisterReadEx(This->m_pPCIeCntl,
									 wAddr,
									 pData,
									 numReg
									 ));
}



static EQP_Errors PCI_MemoryRead(IHCIAPI * This,
								 TQP_UINT32 dwAddr,
								 PTQP_UINT32 pData
								 )
{
	return (CPCIeCntl_MemoryRead(This->m_pPCIeCntl,
								 dwAddr,
								 pData
								 ));
}



static EQP_Errors PCI_MemoryWrite(IHCIAPI * This,
								  TQP_UINT32 dwAddr,
								  TQP_UINT32 dwData
								  )
{
	return (CPCIeCntl_MemoryWrite(This->m_pPCIeCntl,
								  dwAddr,
								  dwData
								  ));
}



static EQP_Errors PCI_StartDMAWrite(IHCIAPI * This,
									TQP_UINT32 ulAddr,
									TQP_UINT32 ulLength,
									PTQP_UINT8 pHostAddr,
									TQP_UINT32 ulXferMode,
									TQP_UINT32 ulPicWidthMB,
									TQP_UINT32 ulPicHeightMB,
									TQP_UINT32 ulSwap,
									TQP_BOOL IsFrameMode
									)
{
	return (CPCIeCntl_StartDMAWrite(This->m_pPCIeCntl,
									ulAddr,
									ulLength,
									pHostAddr,
									ulXferMode,
									ulPicWidthMB,
									ulPicHeightMB,
									ulSwap,
									IsFrameMode
									));
}



static EQP_Errors PCI_StartDMARead(IHCIAPI * This,
								   TQP_UINT32 ulAddr,
								   TQP_UINT32 ulLength,
								   PTQP_UINT8 pHostAddr,
								   TQP_UINT32 ulXferMode,
								   TQP_UINT32 ulPicWidthMB,
								   TQP_UINT32 ulPicHeightMB,
								   TQP_UINT32 ulSwap,
								   TQP_BOOL IsFrameMode
								   )
{
	return (CPCIeCntl_StartDMARead(This->m_pPCIeCntl,
								  ulAddr,
								  ulLength,
								  pHostAddr,
								  ulXferMode,
								  ulPicWidthMB,
								  ulPicHeightMB,
								  ulSwap,
								  IsFrameMode
								  ));
}



static EQP_Errors PCI_SetInterrupt(IHCIAPI * This,
								   TQP_UINT32 mask
								   )
{
	return (QPERR_NOTSUPP);
}



static EQP_Errors PCI_ClearInterrupt(IHCIAPI * This,
									 TQP_UINT32 mask
									 )
{
	return (QPERR_SUCCESS);
}



static EQP_Errors PCI_EnableInterrupt(IHCIAPI * This,
									  TQP_UINT32 mask
									  )
{
	return (CPCIeCntl_EnableInterrupts(This->m_pPCIeCntl));
}



static EQP_Errors PCI_DisableInterrupt(IHCIAPI * This,
									   TQP_UINT32 mask
									   )
{
	return (CPCIeCntl_DisableInterrupt(This->m_pPCIeCntl));
}



static EQP_Errors PCI_GetInterruptStatus(IHCIAPI * This,
										 PTQP_UINT32 pmask
										 )
{
	*pmask = 0;
	return (QPERR_SUCCESS);
}



static EQP_Errors PCI_GetInterruptMask(IHCIAPI * This,
									   PTQP_UINT32 pmask
									   )
{
	*pmask = 0;
	return (QPERR_SUCCESS);
}



static EQP_Errors PCI_SetInterruptMask(IHCIAPI * This,
									   TQP_UINT32 mask
									   )
{
	return (QPERR_SUCCESS);
}



static EQP_Errors PCI_CopyFromCommonBuffer(IHCIAPI * This,
										   PTQP_UINT8 pHostAddr,
										   TQP_UINT32 ulLength
										   )
{
	return (CPCIeCntl_CopyFromCommonBuffer(This->m_pPCIeCntl,
										   pHostAddr,
										   ulLength
										   ));
}



static EQP_Errors PCI_DMAReadDone(IHCIAPI * This)
{
	return (CPCIeCntl_DMAReadDone(This->m_pPCIeCntl));
}



static EQP_Errors PCI_DMAWriteDone(IHCIAPI * This)
{
	return (CPCIeCntl_DMAWriteDone(This->m_pPCIeCntl));
}



/////////////////////////////////////////////////////////////////////////////
//
//

// USB (Direct Mode)
//

static EQP_Errors USB_DM_MemoryRead(IHCIAPI * This,
									TQP_UINT32 dwAddr,
									PTQP_UINT32 pData
									)
{
	TQP_UINT64	linearAddr;

	if (!DM_FindMemoryMapping(This,
							  dwAddr,
							  (PTQP_UINT8 *)&linearAddr
							  ))
	{
		return (QPERR_FAIL);
	}
	else
	{
		return (CUsbCntl_MemoryRead(This->m_pUsbCntl,
									(TQP_UINT32)linearAddr,
									pData
									));
	}
}



static EQP_Errors USB_DM_MemoryWrite(IHCIAPI * This,
									 TQP_UINT32 dwAddr,
									 TQP_UINT32 dwData
									 )
{
	TQP_UINT64	linearAddr;

	if (!DM_FindMemoryMapping(This,
							  dwAddr,
							  (PTQP_UINT8 *)&linearAddr
							  ))
	{
		return (QPERR_FAIL);
	}
	else
	{
		return (CUsbCntl_MemoryWrite(This->m_pUsbCntl,
									 (TQP_UINT32)linearAddr,
									 dwData
									 ));
	}
}




/////////////////////////////////////////////////////////////////////////////
//
//

// Cache Port Access
//

#define REG_CPR_1_READ_ARRR					0x780
#define REG_CPR_1_READ_LEN					0x784
#define REG_CPR_1_READ_DATA					0x788
#define REG_CPR_1_WRITE_ARRR				0x78C
#define REG_CPR_1_WRITE_LEN					0x790
#define REG_CPR_1_WRITE_DATA				0x794



#pragma pack(push, 4)

typedef union _CPR_ADDR
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	reserved	: 2;	// bit 0 - 1
		TQP_UINT32	dword_addr	: 27;	// bit 2 - 28
		TQP_UINT32	reserved2	: 3;	// bit 29 - 31
	} Reg;
} CPR_ADDR, *PCPR_ADDR;


typedef union _CPR_RD_LEN
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	reserved	: 2;	// bit 0 - 1
		TQP_UINT32	dword_size	: 14;	// bit 2 - 15
		TQP_UINT32	srst		: 1;	// bit 16
		TQP_UINT32	flush_s		: 1;	// bit 17
		TQP_UINT32	available	: 6;	// bit 18 - 23
		TQP_UINT32	reserved2	: 2;	// bit 24 - 25
		TQP_UINT32	burst_size	: 6;	// bit 26 - 31
	} Reg;
	struct
	{
		TQP_UINT32	reserved	: 2;	// bit 0 - 1
		TQP_UINT32	dword_size	: 14;	// bit 2 - 15
		TQP_UINT32	srst		: 1;	// bit 16
		TQP_UINT32	flush_s		: 1;	// bit 17
		TQP_UINT32	available	: 8;	// bit 18 - 25
		TQP_UINT32	burst_size	: 6;	// bit 26 - 31
	} RegB0;
} CPR_RD_LEN, *PCPR_RD_LEN;


typedef union _CPR_WR_LEN
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	reserved	: 2;	// bit 0 - 1
		TQP_UINT32	dword_size	: 14;	// bit 2 - 15
		TQP_UINT32	srst		: 1;	// bit 16
		TQP_UINT32	flush_s		: 1;	// bit 17
		TQP_UINT32	empty		: 6;	// bit 18 - 23
		TQP_UINT32	reserved2	: 2;	// bit 24 - 25
		TQP_UINT32	burst_size	: 6;	// bit 26 - 31
	} Reg;
	struct
	{
		TQP_UINT32	reserved	: 2;	// bit 0 - 1
		TQP_UINT32	dword_size	: 14;	// bit 2 - 15
		TQP_UINT32	srst		: 1;	// bit 16
		TQP_UINT32	flush_s		: 1;	// bit 17
		TQP_UINT32	empty		: 8;	// bit 18 - 25
		TQP_UINT32	burst_size	: 6;	// bit 26 - 31
	} RegB0;
} CPR_WR_LEN, *PCPR_WR_LEN;


#pragma pack(pop)



static EQP_Errors CPR_MemoryRead(IHCIAPI * This,
								 TQP_UINT32 dwAddr,
								 PTQP_UINT32 pData
								 )
{
	CPR_ADDR	addr;
	CPR_RD_LEN	len;
	TQP_INT64	llTimeTo = 3000;
	TQP_UINT32	buf[4];
	TQP_UINT32	max_mem_dwrod = (This->m_pMpegCodec->m_MemSize * 0x40000);
	TQP_UINT32	addr_dword = (dwAddr >> 2);
	TQP_UINT32	index;
	TQP_INT32	i;

	ENTER_CRITICAL(&This->m_Object)

	// make sure the current available size is zero
	//
	// ...

	// write dword address
	//
	if (max_mem_dwrod >= addr_dword + 4)
	{
		index = 0;
	}
	else
	{
		addr_dword = max_mem_dwrod - 4;
		index = addr_dword + 4 - max_mem_dwrod;
	}
	addr.Read = 0;
	addr.Reg.dword_addr = addr_dword;

	QPHCI_RegisterWrite(This,
						REG_CPR_1_READ_ARRR,
						addr.Read
						);

	// write dword size
	//
	QPHCI_RegisterRead(This,
					   REG_CPR_1_READ_LEN,
					   &len.Read
					   );
	len.Reg.dword_size = 4;
	QPHCI_RegisterWrite(This,
						REG_CPR_1_READ_LEN,
						len.Read
						);

	// check data available
	//
	llTimeTo = 3000;
	BeginTimer(&llTimeTo);

	if ((QLCODEC_VER_QL330_A0 == This->m_ulChipVer) ||
		(QLCODEC_VER_QL330_A1 == This->m_ulChipVer)
		)
	{
		do
		{
			QPHCI_RegisterRead(This,
							   REG_CPR_1_READ_LEN,
							   &len.Read
							   );
			if (Timeout(llTimeTo,
						1
						))
			{
				LEAVE_CRITICAL(&This->m_Object)
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CPR_MemoryRead() dwAddr (0x%x) Timeout!\n",
							 dwAddr
							 ));
				return (QPERR_FAIL);
			}
		} while ((0x3F == len.Reg.available) || (0 == len.Reg.available)); // check for both -1 and 0
	}
	else
	{
		do
		{
			QPHCI_RegisterRead(This,
							   REG_CPR_1_READ_LEN,
							   &len.Read
							   );
			if (Timeout(llTimeTo,
						1
						))
			{
				LEAVE_CRITICAL(&This->m_Object)
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CPR_MemoryRead() dwAddr (0x%x) Timeout!\n",
							 dwAddr
							 ));
				return (QPERR_FAIL);
			}
		} while ((0xFF == len.Reg.available) || (0 == len.Reg.available)); // check for both -1 and 0
	}

	// read out the data
	//
	for (i = 0; i < 4; i ++)
	{
		QPHCI_RegisterRead(This,
						   REG_CPR_1_READ_DATA,
						   &buf[i]
						   );
	}
	*pData = buf[index];

	LEAVE_CRITICAL(&This->m_Object)

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CPR_MemoryRead() dwAddr (0x%x) Data(0x%x)\n",
				 dwAddr,
				 *pData
				 ));

	return (QPERR_SUCCESS);
}



static EQP_Errors CPR_MemoryWrite(IHCIAPI * This,
								  TQP_UINT32 dwAddr,
								  TQP_UINT32 dwData
								  )
{
	CPR_ADDR	addr;
	CPR_WR_LEN	len;
	TQP_INT64	llTimeTo = 3000;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CPR_MemoryWrite() dwAddr (0x%x) dwData(0x%x)\n",
				 dwAddr,
				 dwData
				 ));

	ENTER_CRITICAL(&This->m_Object)

	// make sure the chache port is available
	//
	// ...

	// write dword address
	//
	addr.Read = 0;
	addr.Reg.dword_addr = (dwAddr >> 2);

	QPHCI_RegisterWrite(This,
						REG_CPR_1_WRITE_ARRR,
						addr.Read
						);

	// write dword size
	//
	QPHCI_RegisterRead(This,
					   REG_CPR_1_WRITE_LEN,
					   &len.Read
					   );
	len.Reg.dword_size = 1;
	QPHCI_RegisterWrite(This,
						REG_CPR_1_WRITE_LEN,
						len.Read
						);

	// write the data
	//
	QPHCI_RegisterWrite(This,
						REG_CPR_1_WRITE_DATA,
						dwData
						);

	// wait until data is written
	//
	llTimeTo = 3000;
	BeginTimer(&llTimeTo);

	if ((QLCODEC_VER_QL330_A0 == This->m_ulChipVer) ||
		(QLCODEC_VER_QL330_A1 == This->m_ulChipVer)
		)
	{
		do
		{
			QPHCI_RegisterRead(This,
							   REG_CPR_1_WRITE_LEN,
							   &len.Read
							   );
			if (Timeout(llTimeTo,
						1
						))
			{
				LEAVE_CRITICAL(&This->m_Object)
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CPR_MemoryWrite() dwAddr (0x%x) dwData(0x%x) Timeout!\n",
							 dwAddr,
							 dwData
							 ));
				return (QPERR_FAIL);
			}
		} while (0 != len.Reg.empty);
	}
	else
	{
		do
		{
			QPHCI_RegisterRead(This,
							   REG_CPR_1_WRITE_LEN,
							   &len.Read
							   );
			if (Timeout(llTimeTo,
						1
						))
			{
				LEAVE_CRITICAL(&This->m_Object)
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CPR_MemoryWrite() dwAddr (0x%x) dwData(0x%x) Timeout!\n",
							 dwAddr,
							 dwData
							 ));
				return (QPERR_FAIL);
			}
		} while (0x42 != len.RegB0.empty);
	}

	LEAVE_CRITICAL(&This->m_Object)

	return (QPERR_SUCCESS);
}



/////////////////////////////////////////////////////////////////////////////
//
//



static EQP_Errors Generic_RegisterReadEx(IHCIAPI * This,
										 TQP_UINT16 wAddr,
										 PTQP_UINT32 pData,
										 TQP_UINT32 numReg
										 )
{
	TQP_UINT32	i;
	EQP_Errors	qpStatus = QPERR_SUCCESS;

	for (i = 0;  i < numReg; i++)
	{
		qpStatus = QPHCI_RegisterRead(This,
									  (TQP_UINT16)(wAddr + (i * sizeof(TQP_UINT32))),
									  &pData[i]
									  );
		if (!QP_SUCCESS(qpStatus))
		{
			return (qpStatus);
		}
	}

	return (qpStatus);
}




static EQP_Errors Generic_CopyFromCommonBuffer(IHCIAPI * This,
											   PTQP_UINT8 pHostAddr,
											   TQP_UINT32 ulLength
											   )
{
	return (QPERR_SUCCESS);
}



static EQP_Errors Generic_DMAReadDone(IHCIAPI * This)
{
	return (QPERR_SUCCESS);
}



static EQP_Errors Generic_DMAWriteDone(IHCIAPI * This)
{
	return (QPERR_SUCCESS);
}



/////////////////////////////////////////////////////////////////////////////
//
//



static EQP_Errors QPHCI_PowerUp(IHCIAPI *This)
{
	EQP_Errors	qpStatus;
	TQP_UINT32	dwReg = 0;

	// no power management for master mode
	//
	if ((QPHCI_BUS_INTERNAL == This->m_bus_type) &&
		(QPHCI_MODE_MASTER == This->m_access_mode)
		)
	{
		return (QPERR_SUCCESS);
	}

	// artesa bring up
	//
	if (QPCODEC_CHIP_ARTESA_FPGA & This->m_pMpegCodec->m_ChipType)
	{
		return (QPERR_SUCCESS);
	}
	else if (QPCODEC_CHIP_ARTESA == This->m_pMpegCodec->m_ChipType)
	{
//#pragma message(__FILE__ "TODO : Artesa :: QPHCI_PowerUp()")
		qpStatus = QPERR_SUCCESS;
	}
	else
	{
		// turn on pad
		//
		qpStatus = QPHCI_RegisterRead(This,
			     					  0x50,
			     					  &dwReg
			     					  );
		if (QP_SUCCESS(qpStatus))
		{
			dwReg &= ~0x100;

			qpStatus = QPHCI_RegisterWrite(This,
										   0x50,
										   dwReg
										   );
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
						 "QPHCI_PowerUp() writing(0x%x) to pad control status(%d)\n",
						 dwReg,
						 qpStatus
						 ));
		}


		// restore memory controller default
		//
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(This,
																		  0xF1C,
																		  0xF00
																		  );

		// reset the chip to bring back other default settings
		//
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_ResetArm(This,
																	 QP_FALSE
																	 );
	}

	return (qpStatus);
}



static EQP_Errors QPHCI_PowerDown(IHCIAPI *This)
{
	EQP_Errors	qpStatus;
	TQP_UINT32	dwReg = 0;


	///////////////////////////////////////////////////////////////////////////
	/* work-around to sync USB BULK packet's data toggle sequence when resume from sleep(S3) mode */
#if 1
	if ((QPHCI_BUS_USB == This->m_bus_type) && (QP_NULL != This->MemoryReadEx))
	{
		unsigned char caDummy[16];

		if (This->m_pUsbCntl->m_dwDataInNextToggle != 0) {
	    	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
		    			 "QPHCI_PowerDown() sync DataIn toggle status\n"
			    		 ));
    	    qpStatus = QPHCI_MemoryReadEx/*CUsbCntl_MemoryReadEx*/(This,
										  0x0 >> 2,	// dw-address
										  16 >> 2,	// dw-length
										  (PTQP_UINT32)caDummy);
		}

	    if (This->m_pUsbCntl->m_dwCmdInNextToggle != 0) {
    		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
		    			 "QPHCI_PowerDown() sync CmdIn toggle status\n"
			    		 ));
	    	qpStatus = QPHCI_RegisterRead(This,
									      QLCODEC_REG_CHIP_VERSION,
								          &dwReg
								          );
	    }
	}
#endif
    ///////////////////////////////////////////////////////////////////////////

	// no power management for master mode
	//
	if ((QPHCI_BUS_INTERNAL == This->m_bus_type) &&
		(QPHCI_MODE_MASTER == This->m_access_mode)
		)
	{
		return (QPERR_SUCCESS);
	}

	// artesa bring up
	//
	if (QPCODEC_CHIP_ARTESA_FPGA & This->m_pMpegCodec->m_ChipType)
	{
		return (QPERR_SUCCESS);
	}
	else if (QPCODEC_CHIP_ARTESA == This->m_pMpegCodec->m_ChipType)
	{
//#pragma message(__FILE__ "TODO : Artesa :: QPHCI_PowerDown()")
		qpStatus = QPERR_SUCCESS;
	}
	else
	{
		// stop ARM running
		//
		if (QPHCI_MODE_DIRECT == This->m_access_mode)
		{
			qpStatus = QPHCI_RegisterWrite(This,
										   REG_HOST_DIRECT_MODE_MISC,
										   0
										   );
		}
		else
		{
			qpStatus = QPHCI_WriteHciRegister(This,
											  REG_HCI_CONTROL,
											  0
											  );
		}

		// put the memory in self refresh mode
		//
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(This,
																		  0xF1C,
																		  0x4
																		  );

		// disable pad
		//
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterRead(This,
																		 0x50,
																		 &dwReg
																		 );

		if (QP_SUCCESS(qpStatus))
		{
			dwReg |= 0x106;
			qpStatus = QPHCI_RegisterWrite(This,
										   0x50,
										   dwReg
										   );
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
						 "QPHCI_PowerDown() writing(0x%x) to pad control status(%d)\n",
						 dwReg,
						 qpStatus
						 ));
		}

		// reset all modules before gating the clocks
		//
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(This,
																		  0x0,
																		  0x0
																		  );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(This,
																		  0x0,
																		  0x01FFFFFF
																		  );
#if 0
		// slow down PLL's
		//
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(This,
																		  0x2C,
																		  0x6418
																		  );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(This,
																		  0x30,
																		  0x6418
																		  );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(This,
																		  0x34,
																		  0x6418
																		  );
#endif
	}

	return (qpStatus);
}



/////////////////////////////////////////////////////////////////////////////
//
//


// Host control interface API
//
EQP_Errors QPHCI_Init(IHCIAPI * This,
					  QPHCI_MODE accessMode,
					  QPHCI_BUS busType,
					  PTQP_UINT8 pRegisterBase,
					  PTQP_UINT8 pMemoryBase,
					  TQP_UINT32 PageSize,	// depends on how many addr pins connected,
											// no bigger than 0x1000000(24 bit)
											//
					  CQLCodec *pMpegCodec,
					  CUsbCntl *pUsbCntl,
					  CPCIeCntl *pPCIeCntl
					  )
{
	int						i;
	HOST_DMA_CNTL_STATUS	dmaCntl;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "QPHCI_Init() mode(%d) bus(%d) regBase(0x%x) memBase(0x%x) page(0x%x)\n",
				 accessMode,
				 busType,
				 pRegisterBase,
				 pMemoryBase,
				 PageSize
				 ));

	if ((accessMode < QPHCI_MODE_FIRST) ||
		(accessMode > QPHCI_MODE_LAST)
		)
	{
		return (QPERR_PARMS);
	}

	if ((busType < QPHCI_BUS_FIRST) ||
		(busType > QPHCI_BUS_LAST)
		)
	{
		return (QPERR_PARMS);
	}

	CObject_Constructor(&This->m_Object,
						QP_NULL,
	  					OBJECT_CRITICAL_LIGHT
	  					);

	This->m_pMpegCodec = pMpegCodec;
	This->m_pUsbCntl = pUsbCntl;
	This->m_pPCIeCntl = pPCIeCntl;

	This->m_access_mode = accessMode;
	This->m_bus_type = busType;
	This->m_pRegisterBase = pRegisterBase;
	This->m_pMemoryBase = pMemoryBase;
	This->m_PageSize = PageSize;

	This->m_EvtEmuTask = QP_NULL;
	This->m_EvtEmuTaskReply = QP_NULL;

	This->m_bEmulationMode = QP_FALSE;

	This->m_ulChipVer = QLCODEC_VER_QL330_A1;

	if (QPHCI_BUS_201_EMULATION == This->m_bus_type)
	{
		// open the kernel driver
		//
		HCIOpenQpDriver();

		// setup function table
		//
		This->ReadHciRegister = EMU_ReadHciRegister;
		This->WriteHciRegister = EMU_WriteHciRegister;
		This->RegisterRead = EMU_RegisterRead;
		This->RegisterWrite = EMU_RegisterWrite;
		This->RegisterReadEx = EMU_RegisterReadEx;
		This->MemoryRead = EMU_MemoryRead;
		This->MemoryWrite = EMU_MemoryWrite;
		This->StartDMAWrite = EMU_StartDMAWrite;
		This->StartDMARead = EMU_StartDMARead;
		if (QPHCI_MODE_DIRECT == This->m_access_mode)
		{
			This->ResetArm = DM_ResetArm;
			This->SetInterrupt = DM_SetInterrupt;
			This->ClearInterrupt = DM_ClearInterrupt;
		}
		else
		{
			This->ResetArm = IDM_ResetArm;
			This->SetInterrupt = IDM_SetInterrupt;
			This->ClearInterrupt = IDM_ClearInterrupt;
		}
		if (0 == (QPCODEC_CHIP_ARTESA_FPGA & pMpegCodec->m_ChipType))
		{
			This->ResetArm = EMU_ResetArm;
		}
		This->EnableInterrupts = EMU_EnableInterrupt;
		This->DisableInterrupts = EMU_DisableInterrupt;
		if (QPHCI_MODE_DIRECT == This->m_access_mode)
		{
			This->GetInterruptsStatus = DM_GetInterruptStatus;
			This->GetInterruptMask = DM_GetInterruptMask;
		}
		else
		{
			This->GetInterruptsStatus = IDM_GetInterruptStatus;
			This->GetInterruptMask = IDM_GetInterruptMask;
		}
		This->SetInterruptMask = EMU_SetInterruptMask;
		This->CopyFromCommonBuffer = Generic_CopyFromCommonBuffer;
		This->DMAReadDone = Generic_DMAReadDone;
		This->DMAWriteDone = Generic_DMAWriteDone;
	}
	else if (QPHCI_BUS_USB == This->m_bus_type)
	{
		// setup function table
		//
		This->ReadHciRegister = USB_ReadHciRegister;
		This->WriteHciRegister = USB_WriteHciRegister;
		This->RegisterRead = USB_RegisterRead;
		This->RegisterWrite = USB_RegisterWrite;
		This->RegisterReadEx = USB_RegisterReadEx;
		This->RegisterWriteEx = USB_RegisterWriteEx;
		if ((QPHCI_MODE_DIRECT == This->m_access_mode) &&
			(QPCODEC_CHIP_ARTESA != This->m_pMpegCodec->m_ChipType)
			)
		{
			This->MemoryRead = USB_DM_MemoryRead;
			This->MemoryWrite = USB_DM_MemoryWrite;
		}
		else
		{
			This->MemoryRead = USB_MemoryRead;
			This->MemoryReadEx = USB_MemoryReadEx;
			This->MemoryWrite = USB_MemoryWrite;
		}
		This->StartDMAWrite = USB_StartDMAWrite;
		This->StartDMARead = USB_StartDMARead;
		This->ResetArm = USB_ResetArm;
		if (QPHCI_MODE_DIRECT == This->m_access_mode)
		{
			This->SetInterrupt = DM_SetInterrupt;
			This->ClearInterrupt = DM_ClearInterrupt;
		}
		else
		{
			This->SetInterrupt = IDM_SetInterrupt;
			This->ClearInterrupt = IDM_ClearInterrupt;
		}
		This->EnableInterrupts = EMU_EnableInterrupt;
		This->DisableInterrupts = EMU_DisableInterrupt;
		if (QPHCI_MODE_DIRECT == This->m_access_mode)
		{
			This->GetInterruptsStatus = DM_GetInterruptStatus;
			This->GetInterruptMask = DM_GetInterruptMask;
		}
		else
		{
			This->GetInterruptsStatus = IDM_GetInterruptStatus;
			This->GetInterruptMask = IDM_GetInterruptMask;
		}
		This->SetInterruptMask = EMU_SetInterruptMask;
		This->CopyFromCommonBuffer = Generic_CopyFromCommonBuffer;
		This->DMAReadDone = Generic_DMAReadDone;
		This->DMAWriteDone = Generic_DMAWriteDone;
	}
	else if (QPHCI_BUS_PCI == This->m_bus_type)
	{
		// setup function table
		//
		This->ReadHciRegister = PCI_ReadHciRegister;
		This->WriteHciRegister = PCI_WriteHciRegister;
		This->RegisterRead = PCI_RegisterRead;
		This->RegisterWrite = PCI_RegisterWrite;
		This->RegisterReadEx = PCI_RegisterReadEx;

		PCI_RegisterRead(This,
						 QLCODEC_REG_CHIP_VERSION,
						 &This->m_ulChipVer
						 );
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "QPHCI_Init() QLCODEC_REG_CHIP_VERSION(0x%08x)\n",
					 This->m_ulChipVer
					 ));

		if (QLCODEC_VER_QL330_B0 == This->m_ulChipVer)
		{
			// 330 B0 work around
			//
			This->MemoryRead = CPR_MemoryRead;
			This->MemoryWrite = CPR_MemoryWrite;
		}
		else
		{
			This->MemoryRead = PCI_MemoryRead;
			This->MemoryWrite = PCI_MemoryWrite;
		}
		This->StartDMAWrite = PCI_StartDMAWrite;
		This->StartDMARead = PCI_StartDMARead;
		This->ResetArm = DM_ResetArm;
		This->SetInterrupt = PCI_SetInterrupt;
		This->ClearInterrupt = PCI_ClearInterrupt;
		This->EnableInterrupts = PCI_EnableInterrupt;
		This->DisableInterrupts = PCI_DisableInterrupt;
		This->GetInterruptsStatus = PCI_GetInterruptStatus;
		This->GetInterruptMask = PCI_GetInterruptMask;
		This->SetInterruptMask = PCI_SetInterruptMask;
		This->CopyFromCommonBuffer = PCI_CopyFromCommonBuffer;
		This->DMAReadDone = PCI_DMAReadDone;
		This->DMAWriteDone = PCI_DMAWriteDone;
	}
	else if (QPHCI_BUS_INTERNAL == This->m_bus_type)
	{
		if (QPHCI_MODE_MASTER == This->m_access_mode)
		{
			// setup function table
			//
			This->ReadHciRegister = MM_ReadHciRegister;
			This->WriteHciRegister = MM_WriteHciRegister;
			This->RegisterRead = MM_RegisterRead;
			This->RegisterWrite = MM_RegisterWrite;
			This->RegisterReadEx = Generic_RegisterReadEx;
			This->MemoryRead = MM_MemoryRead;
			This->MemoryWrite = MM_MemoryWrite;
			This->StartDMAWrite = MM_StartDMAWrite;
			This->StartDMARead = MM_StartDMARead;
#ifdef QP_LINXKER
			This->DMAXferDone = QP_NULL;
#endif
			This->ResetArm = MM_ResetArm;
#if defined(QP_LINXKER) & !defined(USE_SWIRQ_MODE)
			This->SetInterrupt = DM_SetInterrupt;
			This->ClearInterrupt = DM_ClearInterrupt;
			This->EnableInterrupts = EMU_EnableInterrupt;
			This->DisableInterrupts = EMU_DisableInterrupt;
			This->GetInterruptsStatus = DM_GetInterruptStatus;
			This->GetInterruptMask = DM_GetInterruptMask;
			This->SetInterruptMask = EMU_SetInterruptMask;
#else
			This->SetInterrupt = MM_SetInterrupt;
			This->ClearInterrupt = MM_ClearInterrupt;
			This->EnableInterrupts = MM_EnableInterrupt;
			This->DisableInterrupts = MM_DisableInterrupt;
			This->GetInterruptsStatus = MM_GetInterruptStatus;
			This->GetInterruptMask = MM_GetInterruptMask;
			This->SetInterruptMask = MM_SetInterruptMask;

#endif
			This->CopyFromCommonBuffer = Generic_CopyFromCommonBuffer;
			This->DMAReadDone = Generic_DMAReadDone;
			This->DMAWriteDone = Generic_DMAWriteDone;
		}
		else
		{
			return (QPERR_NOTIMPL);
		}
	}
	else
	{
		if (QPHCI_MODE_DIRECT == This->m_access_mode)
		{
			// setup function table
			//
			This->ReadHciRegister = DM_ReadHciRegister;
			This->WriteHciRegister = DM_WriteHciRegister;
			This->RegisterRead = DM_RegisterRead;
			This->RegisterWrite = DM_RegisterWrite;
			This->RegisterReadEx = Generic_RegisterReadEx;
			This->MemoryRead = DM_MemoryRead;
			This->MemoryWrite = DM_MemoryWrite;
			This->StartDMAWrite = DM_StartDMAWrite;
			This->StartDMARead = DM_StartDMARead;
			This->ResetArm = DM_ResetArm;
			This->SetInterrupt = DM_SetInterrupt;
			This->ClearInterrupt = DM_ClearInterrupt;
			This->EnableInterrupts = DM_EnableInterrupt;
			This->DisableInterrupts = DM_DisableInterrupt;
			This->GetInterruptsStatus = DM_GetInterruptStatus;
			This->GetInterruptMask = DM_GetInterruptMask;
			This->SetInterruptMask = DM_SetInterruptMask;
			This->CopyFromCommonBuffer = Generic_CopyFromCommonBuffer;
			This->DMAReadDone = Generic_DMAReadDone;
			This->DMAWriteDone = Generic_DMAWriteDone;
		}
		else
		{
			return (QPERR_NOTIMPL);
		}
	}

#ifndef INTEGRATE_USB_EMU_INT_THREAD_TO_TASK_THREAD
	if ((QPHCI_BUS_USB == This->m_bus_type) ||
#if defined(QP_LINXKER) & !defined(USE_SWIRQ_MODE)
		(QPHCI_BUS_INTERNAL == This->m_bus_type) ||
#endif
		(QPHCI_BUS_201_EMULATION == This->m_bus_type)
		)
	{
		// create events for the thread
		//
		QPOSMCreateEvtgrp(&This->m_EvtEmuTask,
		 				  0
		 				  );
		QPOSMCreateEvtgrp(&This->m_EvtEmuTaskReply,
		 				  0
		 				  );
		// we need a thread to emulate 201 interrupt
		//
		CThread_Constructor(&This->m_Thread,
							"EMU_ThreadProc",
							This,
							QP_CODECTHR_PRI,
							This->m_EvtEmuTask,
							This->m_EvtEmuTaskReply,
							QP_TRUE
							);

		// CThread
		This->m_Thread.m_ThreadProc = EMU_ThreadProc;
	}
#endif

	// power up the chip
	//
	QPHCI_PowerUp(This);

	// initialize default mapping for direct mode
	//
	if (QPHCI_MODE_DIRECT == This->m_access_mode)
	{
		for (i = 0; i < QPHCI_MEM_MAPPING_MAX_PAGES; i++)
		{
			This->m_mem_mapping_offset[i] = i * PageSize;
			This->m_mem_mapping_start_addr[i] = This->m_mem_mapping_offset[i] + QPHCI_DM_MEMMAPPING_START_MIN;
			This->m_mem_mapping_end_addr[i] = This->m_mem_mapping_start_addr[i] + PageSize - 1;

			QPHCI_RegisterWrite(This,
								(TQP_UINT16)(REG_MEM_MAPPING_1_START_ADDR + (i * 12)),
								This->m_mem_mapping_start_addr[i]
								);
			QPHCI_RegisterWrite(This,
								(TQP_UINT16)(REG_MEM_MAPPING_1_END_ADDR + (i * 12)),
								This->m_mem_mapping_end_addr[i]
								);
			QPHCI_RegisterWrite(This,
								(TQP_UINT16)(REG_MEM_MAPPING_1_OFFSET_ADDR + (i * 12)),
								This->m_mem_mapping_offset[i]
								);
		}

		// setup DMA control register
		//
		dmaCntl.Read = 0;

		dmaCntl.Reg.addr = QPHCI_DM_MEMMAPPING_STREAM_PORT;
		dmaCntl.Reg.dword = 1;
		dmaCntl.Reg.dr_rst = 1;
		dmaCntl.Reg.dw_rst = 1;
		dmaCntl.Reg.d_en = 0;

		QPHCI_RegisterWrite(This,
							REG_HOST_DMA_CNTL_STATUS,
							dmaCntl.Read
							);

		dmaCntl.Reg.dr_rst = 0;
		dmaCntl.Reg.dw_rst = 0;
		dmaCntl.Reg.d_en = 1;

		QPHCI_RegisterWrite(This,
							REG_HOST_DMA_CNTL_STATUS,
							dmaCntl.Read
							);
		#ifdef CONFIG_HOST_PLATFORM_STR8133_REF
		// create events for the thread
		//
    QPOSMCreateEvtgrp(&This->m_EvtEmuTask,
                      0
                      );
    QPOSMCreateEvtgrp(&This->m_EvtEmuTaskReply,
                      0
                      );
    // we need a thread to emulate 201 interrupt
    //
    CThread_Constructor(&This->m_Thread,
                        "DM_ThreadProc",
                        This,
                        QP_CODECTHR_PRI,
                        This->m_EvtEmuTask,
                        This->m_EvtEmuTaskReply,
						QP_TRUE
                        );

    // CThread
    This->m_Thread.m_ThreadProc = DM_ThreadProc;
    #endif // CONFIG_HOST_PLATFORM_STR8133_REF
	}

	return (QPERR_SUCCESS);
}



EQP_Errors QPHCI_Done(IHCIAPI * This)
{
	HOST_DMA_CNTL_STATUS	dmaCntl;

	if (QPHCI_MODE_DIRECT == This->m_access_mode)
	{
		// disable DMA engine
		//
		dmaCntl.Read = 0;

		dmaCntl.Reg.addr = QPHCI_DM_MEMMAPPING_STREAM_PORT;
		dmaCntl.Reg.dword = 1;
		dmaCntl.Reg.d_en = 0;

		QPHCI_RegisterWrite(This,
							REG_HOST_DMA_CNTL_STATUS,
							dmaCntl.Read
							);
	}

#ifndef INTEGRATE_USB_EMU_INT_THREAD_TO_TASK_THREAD
	if ((QPHCI_BUS_USB == This->m_bus_type) ||
#if defined(QP_LINXKER) & !defined(USE_SWIRQ_MODE)
		(QPHCI_BUS_INTERNAL == This->m_bus_type) ||
#endif
		(QPHCI_BUS_201_EMULATION == This->m_bus_type)
		)
	{
		CThread_ThreadDone(&This->m_Thread);
		CThread_Destructor(&This->m_Thread);

		if (This->m_EvtEmuTask)
		{
			QPOSMDeleteEvtgrp(This->m_EvtEmuTask);
			This->m_EvtEmuTask = QP_NULL;
		}

		if (This->m_EvtEmuTaskReply)
		{
			QPOSMDeleteEvtgrp(This->m_EvtEmuTaskReply);
			This->m_EvtEmuTaskReply = QP_NULL;
		}
	}
#endif

	// power down the chip
	//
	QPHCI_PowerDown(This);


	if (QPHCI_BUS_201_EMULATION == This->m_bus_type)
	{
		HCICloseQpDriver();
	}

	This->m_bEmulationMode = QP_FALSE;

	// CObject
	//
	CObject_Destructor(&This->m_Object);

	return (QPERR_SUCCESS);
}



EQP_Errors QPHCI_ReInit(IHCIAPI * This)
{
	int						i;
	HOST_DMA_CNTL_STATUS	dmaCntl;
	EQP_Errors				qpStatus = QPERR_SUCCESS;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "QPHCI_ReInit() mode(%d) bus(%d) regBase(0x%x) memBase(0x%x) page(0x%x)\n",
				 This->m_access_mode,
				 This->m_bus_type,
				 This->m_pRegisterBase,
				 This->m_pMemoryBase,
				 This->m_PageSize
				 ));

	// initialize default mapping for direct mode
	//
	if (QPHCI_MODE_DIRECT == This->m_access_mode)
	{
		for (i = 0; i < QPHCI_MEM_MAPPING_MAX_PAGES; i++)
		{
			This->m_mem_mapping_offset[i] = i * This->m_PageSize;
			This->m_mem_mapping_start_addr[i] = This->m_mem_mapping_offset[i] + QPHCI_DM_MEMMAPPING_START_MIN;
			This->m_mem_mapping_end_addr[i] = This->m_mem_mapping_start_addr[i] + This->m_PageSize - 1;

			qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(This,
																			  (TQP_UINT16)(REG_MEM_MAPPING_1_START_ADDR + (i * 12)),
																			  This->m_mem_mapping_start_addr[i]
																			  );
			qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(This,
																			  (TQP_UINT16)(REG_MEM_MAPPING_1_END_ADDR + (i * 12)),
																			  This->m_mem_mapping_end_addr[i]
																			  );
			qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(This,
																			  (TQP_UINT16)(REG_MEM_MAPPING_1_OFFSET_ADDR + (i * 12)),
																			  This->m_mem_mapping_offset[i]
																			  );
		}

		// setup DMA control register
		//
		dmaCntl.Read = 0;

		dmaCntl.Reg.addr = QPHCI_DM_MEMMAPPING_STREAM_PORT;
		dmaCntl.Reg.dword = 1;
		dmaCntl.Reg.dr_rst = 1;
		dmaCntl.Reg.dw_rst = 1;
		dmaCntl.Reg.d_en = 0;

		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(This,
																		  REG_HOST_DMA_CNTL_STATUS,
																		  dmaCntl.Read
																		  );

		dmaCntl.Reg.dr_rst = 0;
		dmaCntl.Reg.dw_rst = 0;
		dmaCntl.Reg.d_en = 1;

		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(This,
																		  REG_HOST_DMA_CNTL_STATUS,
																		  dmaCntl.Read
																		  );
	}

	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "QPHCI_ReInit() Failed!!!! status(%d)\n",
					 qpStatus
					 ));
	}

	return (qpStatus);
}



EQP_Errors QPHCI_GetStreamingPortAddress(IHCIAPI *This,
										 TQP_PVOID *pAddr
										 )
{
	if (QPHCI_MODE_DIRECT == This->m_access_mode)
	{
		if (QP_NULL == pAddr)
		{
			return (QPERR_PARMS);
		}
		else
		{
			*pAddr = This->m_pMemoryBase + QPHCI_DM_MEMMAPPING_STREAM_PORT;
			return (QPERR_SUCCESS);
		}
	}
	else
	{
		return (QPERR_NOTSUPP);
	}
}



EQP_Errors QPHCI_InitArmLoop(IHCIAPI * This)
{
	int	i;

	// can not init arm dummy loop in master mode
	//
	if (((QPHCI_BUS_INTERNAL == This->m_bus_type) &&
		(QPHCI_MODE_MASTER == This->m_access_mode)) ||
		((QPHCI_MODE_DIRECT == This->m_access_mode) &&
		 (QPCODEC_CHIP_ARTESA == This->m_pMpegCodec->m_ChipType) &&
		 (QPHCI_BUS_USB == This->m_bus_type))
		 )
	{
		return (QPERR_SUCCESS);
	}

	// setup initial dummy arm code
	//
	if (QPHCI_MODE_DIRECT == This->m_access_mode)
	{
		QPHCI_RegisterWrite(This,
							REG_HOST_DIRECT_MODE_MISC,
							0
							);
	}
	else
	{
		QPHCI_WriteHciRegister(This,
							   REG_HCI_CONTROL,
							   0
							   );
	}

	for (i = 0; i < 8; i++)
	{
		QPHCI_MemoryWrite(This,
						  i * 4,
						  0xEAFFFFFE
						  );
	}

	return (QPERR_SUCCESS);
}



EQP_Errors QPHCI_EnableArm(IHCIAPI * This,
						   TQP_BOOL fRun
						   )
{
	EQP_Errors	qpStatus;

	// can not stop/start arm in master mode
	//
	if ((QPHCI_BUS_INTERNAL == This->m_bus_type) &&
		(QPHCI_MODE_MASTER == This->m_access_mode)
		)
	{
		return (QPERR_SUCCESS);
	}

	if (QPHCI_MODE_DIRECT == This->m_access_mode)
	{
		qpStatus = QPHCI_RegisterWrite(This,
									   REG_HOST_DIRECT_MODE_MISC,
									   fRun ? 1 : 0
									   );
	}
	else
	{
		qpStatus = QPHCI_WriteHciRegister(This,
										  REG_HCI_CONTROL,
										  (TQP_UINT8)(fRun ? 1 : 0)
										  );
	}

	return (qpStatus);
}



TQP_UINT32 QPHCI_GetMaxDMASize(IHCIAPI * This)
{
	switch(This->m_bus_type)
	{
		case QPHCI_BUS_201_EMULATION:
			return (/*0x0FFFFFFF*/QLDEV_INTF_MAX_DMA_SIZE);

		case QPHCI_BUS_USB:
#ifdef USE_BIG_USB_DMA_SIZE
			return (512*1024);
#else
			return (/*0x0FFFFFFF*/128*1024);
#endif

		case QPHCI_BUS_PCI:
			return (CPCIeCntl_GetMaxDMASize(This->m_pPCIeCntl));

		default:
			return (0x0FFFFFFF);
	}
}



TQP_BOOL QPHCI_CanSwapData(IHCIAPI * This)
{
	switch (This->m_bus_type)
	{
		case QPHCI_BUS_PCI:
#ifndef CONFIG_HOST_PLATFORM_STR8133_REF
		case QPHCI_BUS_CUSTOM_1:
#endif // !CONFIG_HOST_PLATFORM_STR8133_REF
		case QPHCI_BUS_201_EMULATION:
		case QPHCI_BUS_INTERNAL:
			return (QP_TRUE);

		case QPHCI_BUS_USB:
#ifdef CONFIG_HOST_PLATFORM_STR8133_REF
		case QPHCI_BUS_CUSTOM_1:
#endif // CONFIG_HOST_PLATFORM_STR8133_REF
		default:
			return (QP_FALSE);
	}
}








