///////////////////////////////////////////////////////////////////////////////
//        
//
// Hci.h
//
// Description: 
//
//  This is the definations of the host control interface
// 
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __HCI_H__
#define __HCI_H__


#include "qptypes.h"
#include "qpmpgcodec.h"
#include "cthread.h"
#include "cqpusbintf.h"
#include "cusbcntl.h"
#include "cpciecntl.h"

#ifdef __cplusplus
extern "C" {
#endif


/////////////////////////////////////////////////////////////////////////////
//
//

// interrupt type
//
#define HCI_INTERRUPT_ARM				0x01
#define HCI_INTERRUPT_P2M				0x02
#define HCI_INTERRUPT_M2P				0x04


// total memory mapping pages, 
// set 0 reserved for register access
//
#define QPHCI_MEM_MAPPING_MAX_PAGES		3


#ifndef __CQLCODEC_FWD_DEFINED__
#define __CQLCODEC_FWD_DEFINED__
typedef struct CQLCodec CQLCodec;
#endif //__CQLCODEC_FWD_DEFINED__


#ifndef __IHCIAPI_FWD_DEFINED__
#define __IHCIAPI_FWD_DEFINED__
typedef struct IHCIAPI IHCIAPI;
#endif //__IHCIAPI_FWD_DEFINED__

// HCI per instance data
//
struct IHCIAPI
{
	// public CObject
	//
	CObject		m_Object;

	// 201 emulation mode
	//
	CThread		m_Thread;

	// access functions
	//
	EQP_Errors (*ReadHciRegister)(IHCIAPI *This,
								  TQP_UINT8 cAddr,
								  PTQP_UINT8 pData
								  );
	EQP_Errors (*WriteHciRegister)(IHCIAPI * This,
								   TQP_UINT8 cAddr,
								   TQP_UINT8 ucData
								   );
	EQP_Errors (*RegisterRead)(IHCIAPI * This,
							   TQP_UINT16 wAddr,
							   PTQP_UINT32 pData
							   );
	EQP_Errors (*RegisterWrite)(IHCIAPI * This,
								TQP_UINT16 wAddr,
								TQP_UINT32 dwData 
								);
	EQP_Errors (*RegisterReadEx)(IHCIAPI * This,
								 TQP_UINT16 wAddr,
								 PTQP_UINT32 pData,
								 TQP_UINT32 numReg
								 );
	EQP_Errors (*RegisterWriteEx)(IHCIAPI * This,
								 TQP_UINT16 wAddr,
								 PTQP_UINT32 pData,
								 TQP_UINT32 numReg
								 );
	EQP_Errors (*MemoryRead)(IHCIAPI * This,
							 TQP_UINT32 dwAddr,
							 PTQP_UINT32 pData 
							 );
	EQP_Errors (*MemoryReadEx)(IHCIAPI * This,
							   TQP_UINT32 ulDWordAddr,		// in D-WORD(32 bit)
							   TQP_UINT32 ulDWordLength,	// in D-WORD(32 bit)
							   PTQP_UINT32 pData 
							   );							 
	EQP_Errors (*MemoryWrite)(IHCIAPI * This,
							  TQP_UINT32 dwAddr,
							  TQP_UINT32 dwData 
							  );
	EQP_Errors (*StartDMAWrite)(IHCIAPI * This,
								TQP_UINT32 ulAddr,	// in WORD(32 bit)
								TQP_UINT32 ulLength,// in WORD(32 bit)
								PTQP_UINT8 pHostAddr,
								TQP_UINT32 ulXferMode,
								TQP_UINT32 ulPicWidthMB,
								TQP_UINT32 ulPicHeightMB,
								TQP_UINT32 ulSwap,
								TQP_BOOL IsFrameMode
								);
	EQP_Errors (*StartDMARead)(IHCIAPI * This,
							   TQP_UINT32 ulAddr,	// in WORD(32 bit)
							   TQP_UINT32 ulLength,	// in WORD(32 bit)
							   PTQP_UINT8 pHostAddr,
							   TQP_UINT32 ulXferMode,
							   TQP_UINT32 ulPicWidthMB,
							   TQP_UINT32 ulPicHeightMB,
							   TQP_UINT32 ulSwap,
							   TQP_BOOL IsFrameMode
							   );
	EQP_Errors (*ResetArm)(IHCIAPI * This,
						   TQP_BOOL fRun
						   );
	EQP_Errors (*SetInterrupt)(IHCIAPI * This,
							   TQP_UINT32 mask
							   );
	EQP_Errors (*ClearInterrupt)(IHCIAPI * This,
								 TQP_UINT32 mask
								 );
	EQP_Errors (*EnableInterrupts)(IHCIAPI * This, 
								  TQP_UINT32 mask
								  );
	EQP_Errors (*DisableInterrupts)(IHCIAPI * This, 
								   TQP_UINT32 mask
								   );
	EQP_Errors (*GetInterruptsStatus)(IHCIAPI * This, 
									 PTQP_UINT32 pmask
									 );
	EQP_Errors (*GetInterruptMask)(IHCIAPI * This, 
								   PTQP_UINT32 pmask
								   );
	EQP_Errors (*SetInterruptMask)(IHCIAPI * This, 
								   TQP_UINT32 mask
								   );
	EQP_Errors (*CopyFromCommonBuffer)(IHCIAPI * This, 
									   PTQP_UINT8 pHostAddr,
									   TQP_UINT32 ulLength
									   );
	EQP_Errors (*DMAReadDone)(IHCIAPI * This);
	EQP_Errors (*DMAWriteDone)(IHCIAPI * This);
	EQP_Errors (*DMAXferDone)(IHCIAPI * This);


	CQLCodec		*m_pMpegCodec;

	// access mode and bus type
	//
	QPHCI_MODE		m_access_mode;
	QPHCI_BUS		m_bus_type;

	// base address
	//
	PTQP_UINT8		m_pRegisterBase;
	PTQP_UINT8		m_pMemoryBase;
	TQP_UINT32		m_PageSize;

	// cached direct mode mapping registers
	//
	TQP_UINT32		m_mem_mapping_start_addr[QPHCI_MEM_MAPPING_MAX_PAGES];
	TQP_UINT32		m_mem_mapping_end_addr[QPHCI_MEM_MAPPING_MAX_PAGES];
	TQP_UINT32		m_mem_mapping_offset[QPHCI_MEM_MAPPING_MAX_PAGES];

	// 201 emulation mode
	//
	TQP_OSMEvtgrpID	m_EvtEmuTask;
	TQP_OSMEvtgrpID	m_EvtEmuTaskReply;

	// USB bus type
	//
	CUsbCntl		*m_pUsbCntl;

	// PCIe bus interface
	//
	CPCIeCntl		*m_pPCIeCntl;

	// driver running in emulation mode
	//
	TQP_BOOL		m_bEmulationMode;

	TQP_UINT32		m_ulChipVer;
};


// Host control interface API
//
EQP_Errors QPHCI_Init(IHCIAPI * This,
					  QPHCI_MODE accessMode, 
					  QPHCI_BUS busType,
					  PTQP_UINT8 pRegisterBase,
					  PTQP_UINT8 pMemoryBase,
					  TQP_UINT32 PageSize,	// depends on how many addr pins connected, 
											// no bigger than 0x1000000(24 bit)
											//
					  CQLCodec *pMpegCodec,
					  CUsbCntl *pUsbCntl,
					  CPCIeCntl *pPCIeCntl
					  );
EQP_Errors QPHCI_Done(IHCIAPI * This);
EQP_Errors QPHCI_ReInit(IHCIAPI * This);
EQP_Errors QPHCI_GetStreamingPortAddress(IHCIAPI *This,
										 TQP_PVOID *pAddr
										 );
EQP_Errors QPHCI_InitArmLoop(IHCIAPI * This);
EQP_Errors QPHCI_EnableArm(IHCIAPI * This, 
						   TQP_BOOL fRun
						   );
TQP_UINT32 QPHCI_GetMaxDMASize(IHCIAPI * This);
TQP_BOOL QPHCI_CanSwapData(IHCIAPI * This);

// Macro for accessing HCI interface
//
#define QPHCI_ReadHciRegister(This, cAddr, pData) \
    (This)->ReadHciRegister(This, cAddr, pData)

#define QPHCI_WriteHciRegister(This, cAddr, ucData) \
    (This)->WriteHciRegister(This, cAddr, ucData)

#define QPHCI_RegisterRead(This, wAddr, pData) \
    (This)->RegisterRead(This, wAddr, pData)

#define QPHCI_RegisterWrite(This, wAddr, dwData) \
    (This)->RegisterWrite(This, wAddr, dwData)

#define QPHCI_RegisterReadEx(This, wAddr, pData, numReg) \
    (This)->RegisterReadEx(This, wAddr, pData, numReg)

#define QPHCI_RegisterWriteEx(This, wAddr, pData, numReg) \
    (This)->RegisterWriteEx(This, wAddr, pData, numReg)

#define QPHCI_MemoryRead(This, dwAddr, pData) \
    (This)->MemoryRead(This, dwAddr, pData)
    
#define QPHCI_MemoryReadEx(This, ulDWordAddr, ulDWordLength, pData) \
    (This)->MemoryReadEx(This, ulDWordAddr, ulDWordLength, pData)    

#define QPHCI_MemoryWrite(This, dwAddr, dwData) \
    (This)->MemoryWrite(This, dwAddr, dwData)

#define QPHCI_StartDMAWrite(This, ulAddr, ulLength, pHostAddr, ulXferMode, ulPicWidthMB, ulPicHeightMB, ulSwap, IsFrameMode) \
    (This)->StartDMAWrite(This, ulAddr, ulLength, pHostAddr, ulXferMode, ulPicWidthMB, ulPicHeightMB, ulSwap, IsFrameMode)

#define QPHCI_StartDMARead(This, ulAddr, ulLength, pHostAddr, ulXferMode, ulPicWidthMB, ulPicHeightMB, ulSwap, IsFrameMode) \
    (This)->StartDMARead(This, ulAddr, ulLength, pHostAddr, ulXferMode, ulPicWidthMB, ulPicHeightMB, ulSwap, IsFrameMode)

#define QPHCI_ResetArm(This, fRun) \
    (This)->ResetArm(This, fRun)

#define QPHCI_SetInterrupt(This, mask) \
    (This)->SetInterrupt(This, mask)

#define QPHCI_ClearInterrupt(This, mask) \
    (This)->ClearInterrupt(This, mask)

#define QPHCI_EnableInterrupt(This, mask) \
    (This)->EnableInterrupts(This, mask)

#define QPHCI_DisableInterrupt(This, mask) \
    (This)->DisableInterrupts(This, mask)

#define QPHCI_GetInterruptStatus(This, pmask) \
    (This)->GetInterruptsStatus(This, pmask)

#define QPHCI_GetInterruptMask(This, pmask) \
    (This)->GetInterruptMask(This, pmask)

#define QPHCI_SetInterruptMask(This, mask) \
    (This)->SetInterruptMask(This, mask)

#define QPHCI_CopyFromCommonBuffer(This, pHostAddr, ulLength) \
    (This)->CopyFromCommonBuffer(This, pHostAddr, ulLength)

#define QPHCI_DMAReadDone(This) \
    (This)->DMAReadDone(This)

#define QPHCI_DMAWriteDone(This) \
    (This)->DMAWriteDone(This)

#define QPHCI_DMAXferDone(This) \
    (This)->DMAXferDone(This)

#ifdef __cplusplus
}
#endif

#endif //__HCI_H__
