///////////////////////////////////////////////////////////////////////////////
//        
//
// Irq.c
//
// Description: 
//
//  codec chip interrupt handler
// 
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#include "cqlcodec.h"
#include "qpdbgmgr.h"

extern TQP_DbgMCompID	g_QPCodecDBGCompID;


void CQLCodec_InterruptHandler(void * refCon)
{
	CQLCodec	*This = (CQLCodec *)refCon;
	TQP_UINT32	intMask = 0;

	// Get interrupt status
	//
	QPHCI_GetInterruptStatus(&This->m_hci, 
							 &intMask
							 );

	// Arm Message
	//
	if (intMask & HCI_INTERRUPT_ARM)
	{
		QPHCI_ClearInterrupt(&This->m_hci, 
							 HCI_INTERRUPT_ARM
							 );
		QPOSMSetEvtgrp(This->m_EvtTask, 
					   QLCODEC_EVENT_ARM_INT
					   );
	}

#ifndef CONFIG_HOST_PLATFORM_STR8133_REF
	// M2P done
	//
	if (intMask & HCI_INTERRUPT_M2P)
	{
		QPHCI_ClearInterrupt(&This->m_hci, 
							 HCI_INTERRUPT_M2P
							 );
#ifdef _QP_REDUCE_DMA_DONE_TASK_SWITCH
		if ((QPHCI_BUS_CUSTOM_1 != This->m_hci.m_bus_type) &&
			(QPHCI_BUS_201_EMULATION != This->m_hci.m_bus_type)
			)
		{
#endif //_QP_REDUCE_DMA_DONE_TASK_SWITCH
			QPOSMSetEvtgrp(This->m_EvtTask, 
						   QLCODEC_EVENT_DMA_READ_DONE
						   );
#ifdef _QP_REDUCE_DMA_DONE_TASK_SWITCH
		}
#endif //_QP_REDUCE_DMA_DONE_TASK_SWITCH
		QPOSMSetEvtgrp(This->m_EvtSyncDma, 
					   QLCODEC_EVENT_DMA_SYNC_READ_DONE
					   );
	}

	// P2M done
	//
	if (intMask & HCI_INTERRUPT_P2M)
	{
		QPHCI_ClearInterrupt(&This->m_hci, 
							 HCI_INTERRUPT_P2M
							 );
#ifdef _QP_REDUCE_DMA_DONE_TASK_SWITCH
		if ((QPHCI_BUS_CUSTOM_1 != This->m_hci.m_bus_type) &&
			(QPHCI_BUS_201_EMULATION != This->m_hci.m_bus_type)
			)
		{
#endif //_QP_REDUCE_DMA_DONE_TASK_SWITCH
			QPOSMSetEvtgrp(This->m_EvtTask,
						   QLCODEC_EVENT_DMA_WRITE_DONE
						   );
#ifdef _QP_REDUCE_DMA_DONE_TASK_SWITCH
		}
#endif //_QP_REDUCE_DMA_DONE_TASK_SWITCH
		QPOSMSetEvtgrp(This->m_EvtSyncDma,
					   QLCODEC_EVENT_DMA_SYNC_WRITE_DONE
					   );
	}
#endif // !CONFIG_HOST_PLATFORM_STR8133_REF
}



void CQLCodec_InterruptHandlerUSB(void * refCon)
{
	CQLCodec	*This = (CQLCodec *)refCon;
	TQP_UINT32	intMask = 0;

	// Get interrupt status
	//
	QPHCI_GetInterruptStatus(&This->m_hci, 
							 &intMask
							 );
	// Arm Message
	//
	if (intMask & HCI_INTERRUPT_ARM)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_EXTRACE),
					 "CQLCodec_InterruptHandlerUSB() HCI_INTERRUPT_ARM\n"
					 ));
		QPHCI_ClearInterrupt(&This->m_hci, 
							 HCI_INTERRUPT_ARM
							 );
		QPOSMSetEvtgrp(This->m_EvtTask, 
					   QLCODEC_EVENT_ARM_INT
					   );
	}
}




