///////////////////////////////////////////////////////////////////////////////
//        
//
// Peripherals.c
//
// Description: 
//
//  Peripheral and USB init/done functions
// 
// History:
//	Sep 14, 2006 (dt) created
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////

#if defined(QP_LINXKER) || defined(_QP_LINUX) || defined(_QP_MACOSX)
#include "qlautoconf.h"
#endif
#include "cqlcodeclib.h"
#include "util.h"

#ifdef _QPCODEC_PURE

TQP_BOOL CQLCodecLib_CreateInitUSB(CQLCodecLib *This, 
								   TQP_UINT32 dwPipeCmdWrite,
								   TQP_UINT32 dwPipeCmdRead,
								   TQP_UINT32 dwPipeDataWrite,
								   TQP_UINT32 dwPipeDataRead,
								   QPHCI_MODE accessMode
								   )
{
	return (QP_TRUE);
}



void CQLCodecLib_DeleteDoneUSB(CQLCodecLib *This)
{
}



TQP_BOOL CQLCodecLib_CreatePCIe(CQLCodecLib *This)
{
	return (QP_TRUE);
}



void CQLCodecLib_DeletePCIe(CQLCodecLib *This)
{
}



EQP_Errors CQLCodecLib_InitPCIe(CQLCodecLib *This)
{
	return (QPERR_SUCCESS);
}



void CQLCodecLib_DonePCIe(CQLCodecLib *This)
{
}



TQP_BOOL CQLCodecLib_CreatePeripherals(CQLCodecLib *This)
{
	return (QP_TRUE);
}



void CQLCodecLib_DeletePeripherals(CQLCodecLib *This)
{
}



EQP_Errors CQLCodecLib_InitPeripherals(CQLCodecLib *This)
{
	return (QPERR_SUCCESS);
}



void CQLCodecLib_DonePeripherals(CQLCodecLib *This)
{
}

#else //!_QPCODEC_PURE

#include "qpdbgmgr.h"
#include "qpmemmgr.h"
#include "cit6603.h"
#include "cov10633.h"
#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
#include "csaa7115.h"
#include "cmt9v112.h"
#include "cov7720.h"
#include "cadi7401.h"
#include "cpe1005.h"
#include "csil9011.h"
#include "csil9025.h"
#include "csil9030.h"
#include "csaa7128.h"
#include "cadi7393.h"
#include "csil9034.h"
#include "cmi2010.h"
#include "cvidinputusbfw.h"
#include "cadv7614.h"
#include "cadi9889.h"
#include "cadi7441.h"
#include "ctvp7002.h"
#include "ctvp7002vh.h"
#include "cvidinputauto1.h"
#include "cfm31.h"
#include "cov5642.h"
#include "ctw9910.h"
#endif
#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
#if defined (CONFIG_QL300_AVIU_NATIVE_NT99240)
#include "cvidinputnativent99240.h"
#else
#include "cvidinputnativemi2010.h"
#endif
#endif

extern TQP_DbgMCompID	g_QPCodecDBGCompID;

/////////////////////////////////////////////////////////////////////////////
//
//

#define I_I2C(This)			(&This->m_pI2CProvider->m_ii2c)
#define I_I2CEX(This)		(&This->m_pI2CProviderEx->m_ii2c)
#define I_VIDDECODER(This)	(&This->m_pVidDecoder->m_iVideoDecoder)
#define I_VIDENCODER(This)	(&This->m_pVidEncoder->m_iVideoEncoder)
#define I_TUNER(This)		(&This->m_pTuner->m_iTuner)
#define I_TVAUDIO(This)		(&This->m_pTVAudio->m_iTvAudio)
#define I_AUDCODEC(This)	(&This->m_pAudCodec->m_iAudCodec)



static EQP_Errors CQLCodecLibBusCallback(TQP_PVOID Context, 
										 TQP_UINT32 dwCode, 
										 TQP_PVOID pParam
										 )
{
	CQLCodecLib	*pQLCodecLib = (CQLCodecLib	*)Context;

	if (pQLCodecLib &&
		pQLCodecLib->m_pMpgCodec
		) 
	{
		switch (dwCode)
		{
			case QPBUSINTF_ARM_INT:
				QPOSMSetEvtgrp(pQLCodecLib->m_pMpgCodec->m_EvtTask, 
							   QLCODEC_EVENT_ARM_INT
							   );
				break;

			case QPBUSINTF_DMA_READ_DONE:
				QPOSMSetEvtgrp(pQLCodecLib->m_pMpgCodec->m_EvtTask, 
							   QLCODEC_EVENT_DMA_READ_DONE
							   );
				QPOSMSetEvtgrp(pQLCodecLib->m_pMpgCodec->m_EvtSyncDma, 
							   QLCODEC_EVENT_DMA_SYNC_READ_DONE
							   );
				break;

			case QPBUSINTF_DMA_WRITE_DONE:
				QPOSMSetEvtgrp(pQLCodecLib->m_pMpgCodec->m_EvtTask,
							   QLCODEC_EVENT_DMA_WRITE_DONE
							   );
				QPOSMSetEvtgrp(pQLCodecLib->m_pMpgCodec->m_EvtSyncDma,
							   QLCODEC_EVENT_DMA_SYNC_WRITE_DONE
							   );
				break;

			default:
				break;
		}
	}

	return (QPERR_SUCCESS);
}



/////////////////////////////////////////////////////////////////////////////
//
//

TQP_BOOL CQLCodecLib_CreateInitUSB(CQLCodecLib *This, 
								   TQP_UINT32 dwPipeCmdWrite,
								   TQP_UINT32 dwPipeCmdRead,
								   TQP_UINT32 dwPipeDataWrite,
								   TQP_UINT32 dwPipeDataRead,
								   QPHCI_MODE accessMode
								   )
{
	CUsbInterface	*pUsbInterface;
	CUsbCntl		*pUsbCntl;


	// create USB bus interafce
	//
	pUsbInterface = (CUsbInterface *)QPMMMalloc2Ex(sizeof(CUsbInterface));
	if (!pUsbInterface)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CQLCodecLib_CreateInitUSB() pUsbInterface == NULL!!!\n"
					 ));
		return (QP_FALSE);
	}


	This->m_pUsbInterface = CUsbInterface_Constructor(pUsbInterface, 
													  &This->m_Object,
													  OBJECT_CRITICAL_LIGHT, 
													  This->m_pDO, 
													  This->m_PDOLayered
													  );
	if (!This->m_pUsbInterface)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CQLCodecLib_CreateInitUSB() CUsbInterface_Constructor() failed\n"
					 ));
		QPMMFree2Ex(pUsbInterface);
		return (QP_FALSE);
	}

	// create the USB controller 
	//
	pUsbCntl = (CUsbCntl *)QPMMMalloc2Ex(sizeof(CUsbCntl));
	if (!pUsbCntl)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CQLCodecLib_CreateInitUSB() pUsbCntl == NULL!!!\n"
					 ));
		return (QP_FALSE);
	}

	This->m_pUsbCntl = CUsbCntl_Constructor(pUsbCntl, 
											&This->m_Object, 
											OBJECT_CRITICAL_LIGHT, 
											This->m_pUsbInterface,
											CQLCodecLibBusCallback,
											This,
											dwPipeCmdWrite,
											dwPipeCmdRead,
											dwPipeDataWrite,
											dwPipeDataRead,
											accessMode
											);
	if (!This->m_pUsbCntl)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CQLCodecLib_CreateInitUSB() CUsbCntl_Constructor() failed\n"
					 ));
		QPMMFree2Ex(pUsbCntl);
		return (QP_FALSE);
	}

	return (QP_TRUE);
}



void CQLCodecLib_DeleteDoneUSB(CQLCodecLib *This)
{
	// delete USB controller
	//
	if (This->m_pUsbCntl)
	{
		CUsbCntl_Destructor(This->m_pUsbCntl);
		QPMMFree2Ex(This->m_pUsbCntl);
		This->m_pUsbCntl = QP_NULL;
	}

	// delete USB bus interface
	//
	if (This->m_pUsbInterface)
	{
		CUsbInterface_Destructor(This->m_pUsbInterface);
		QPMMFree2Ex(This->m_pUsbInterface);
		This->m_pUsbInterface = QP_NULL;
	}
}



TQP_BOOL CQLCodecLib_CreatePCIe(CQLCodecLib *This)
{
	CPCIeCntl	*pPCIeInterface;

	// create PCIe bus interafce
	//
	pPCIeInterface = (CPCIeCntl *)QPMMMalloc2Ex(sizeof(CPCIeCntl));
	if (!pPCIeInterface)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CQLCodecLib_CreateInitPCIe() pPCIeInterface == NULL!!!\n"
					 ));
		return (QP_FALSE);
	}

	This->m_pPCIeCntl = CPCIeCntl_Constructor(pPCIeInterface, 
											  &This->m_Object,
											  OBJECT_CRITICAL_LIGHT, 
											  This->m_pDO, 
											  This->m_PDOLayered,
											  CQLCodecLibBusCallback,
											  This
											  );
	if (!This->m_pPCIeCntl)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CQLCodecLib_CreateInitPCIe() CPCIeCntl_Constructor() failed\n"
					 ));
		QPMMFree2Ex(pPCIeInterface);
		return (QP_FALSE);
	}

	return (QP_TRUE);
}



void CQLCodecLib_DeletePCIe(CQLCodecLib *This)
{
	// delete PCIe bus interface
	//
	if (This->m_pPCIeCntl)
	{
		CPCIeCntl_Destructor(This->m_pPCIeCntl);
		QPMMFree2Ex(This->m_pPCIeCntl);
		This->m_pPCIeCntl = QP_NULL;
	}
}




EQP_Errors CQLCodecLib_InitPCIe(CQLCodecLib *This)
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;

	// initialize PCIe bus interface
	//
	if (This->m_pPCIeCntl)
	{
		qpStatus = QPGenericDevice_InitDevice(This->m_pPCIeCntl, 
											  QP_NULL,
											  QP_NULL,
											  (TQP_PVOID)&This->m_InitData.mpgCodecInitData
											  );

		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CQLCodecLib_InitPCIe() m_pPCIeCntl->InitDevice Failed status(0x%x)!!!\n",
						 qpStatus
						 ));
		}
	}

	return (qpStatus);
}



void CQLCodecLib_DonePCIe(CQLCodecLib *This)
{
	// delete PCIe bus interface
	//
	if (This->m_pPCIeCntl)
	{
		QPGenericDevice_Release(This->m_pPCIeCntl);
	}
}



// Andy 2008.10.14: create video decoder
TQP_BOOL CQLCodecLib_CreatePeripheralsVidDecoder(CQLCodecLib *This)
{
	CVideoDecoder	*pVidDecoder;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "%s(): type(%d) multiType(%d)\n",
				 __FUNCTION__,
				 This->m_InitData.vidDecoderInitData.type,
				 This->m_InitData.vidDecoderInitData.multiType
				 ));

	// create video decoder
	//
	switch (This->m_InitData.vidDecoderInitData.type)
	{
		case VIDDECODER_DUMMY:
			pVidDecoder = (CVideoDecoder *)QPMMMalloc2Ex(sizeof(CVideoDecoder));
			if (!pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pVidDecoder == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			This->m_pVidDecoder = (CVideoDecoder *)CVideoDecoder_Constructor(pVidDecoder, 
																			 &This->m_Object,
																			 OBJECT_DEFAULT,
																			 I_TUNER(This),
																			 I_TVAUDIO(This)
																			 );
			if (QP_NULL == This->m_pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() CVideoDecoder_Constructor() failed\n"
							 ));
				QPMMFree2Ex(pVidDecoder);
				return (QP_FALSE);
			}
			break;

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
		case VIDDECODER_SAA7115:
			pVidDecoder = (CVideoDecoder *)QPMMMalloc2Ex(sizeof(CSaa7115));
			if (!pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pVidDecoder == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			This->m_pVidDecoder = (CVideoDecoder *)CSaa7115_Constructor((CSaa7115 *)pVidDecoder, 
																		&This->m_Object,
																		OBJECT_DEFAULT,
																		I_TUNER(This),
																		I_TVAUDIO(This),
																		I_I2C(This),
																		SAA7115_DEVICE_ADDR
																		);
			if (QP_NULL == This->m_pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() CSaa7115_Constructor() failed\n"
							 ));
				QPMMFree2Ex(pVidDecoder);
				return (QP_FALSE);
			}
			break;
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
		case VIDDECODER_MT9V112:
			pVidDecoder = (CVideoDecoder *)QPMMMalloc2Ex(sizeof(CMt9V112));
			if (!pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pVidDecoder == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			This->m_pVidDecoder = (CVideoDecoder *)CMt9V112_Constructor((CMt9V112 *)pVidDecoder, 
																		&This->m_Object,
																		OBJECT_DEFAULT,
																		I_TUNER(This),
																		I_TVAUDIO(This),
																		I_I2C(This),
																		MT9V112_DEVICE_ADDR
																		);
			if (QP_NULL == This->m_pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() CMt9V112_Constructor() failed\n"
							 ));
				QPMMFree2Ex(pVidDecoder);
				return (QP_FALSE);
			}
			break;
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
		case VIDDECODER_OV7720:
			pVidDecoder = (CVideoDecoder *)QPMMMalloc2Ex(sizeof(COV7720));
			if (!pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pVidDecoder == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			This->m_pVidDecoder = (CVideoDecoder *)COV7720_Constructor((COV7720 *)pVidDecoder, 
																	   &This->m_Object,
																	   OBJECT_DEFAULT,
																	   I_TUNER(This),
																	   I_TVAUDIO(This),
																	   I_I2C(This),
																	   OV7720_DEVICE_ADDR
																	   );
			if (QP_NULL == This->m_pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() COV7720_Constructor() failed\n"
							 ));
				QPMMFree2Ex(pVidDecoder);
				return (QP_FALSE);
			}
			break;
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
		case VIDDECODER_ADI7401:   // Andy 2008.06.18
			pVidDecoder = (CVideoDecoder *)QPMMMalloc2Ex(sizeof(CADI7401));
			if (!pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pVidDecoder == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			This->m_pVidDecoder = (CVideoDecoder *)CADI7401_Constructor((CADI7401 *)pVidDecoder, 
																	   &This->m_Object,
																	   OBJECT_DEFAULT,
																	   I_TUNER(This),
																	   I_TVAUDIO(This),
																	   I_I2C(This),
																	   ADI7401_DEVICE_ADDR
																	   );
			if (QP_NULL == This->m_pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() CADI7401_Constructor() failed\n"
							 ));
				QPMMFree2Ex(pVidDecoder);
				return (QP_FALSE);
			}
			break;
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
		case VIDDECODER_PE1005:   // Andy 2008.04.30
			pVidDecoder = (CVideoDecoder *)QPMMMalloc2Ex(sizeof(CPE1005));
			if (!pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pVidDecoder == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			This->m_pVidDecoder = (CVideoDecoder *)CPE1005_Constructor((CPE1005 *)pVidDecoder, 
																	   &This->m_Object,
																	   OBJECT_DEFAULT,
																	   I_TUNER(This),
																	   I_TVAUDIO(This),
																	   I_I2CEX(This),//I_I2C(This),
																	   PE1005_DEVICE_ADDR
																	   );
			if (QP_NULL == This->m_pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() CPE1005_Constructor() failed\n"
							 ));
				QPMMFree2Ex(pVidDecoder);
				return (QP_FALSE);
			}
			break;
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
		case VIDDECODER_SIL9011:   // Andy 2008.05.12
			pVidDecoder = (CVideoDecoder *)QPMMMalloc2Ex(sizeof(CSIL9011));
			if (!pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pVidDecoder == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			This->m_pVidDecoder = (CVideoDecoder *)CSIL9011_Constructor((CSIL9011 *)pVidDecoder, 
																	   &This->m_Object,
																	   OBJECT_DEFAULT,
																	   I_TUNER(This),
																	   I_TVAUDIO(This),
																	   I_I2C(This),
																	   SIL9011_DEVICE_ADDR
																	   );
			if (QP_NULL == This->m_pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() CSIL9011_Constructor() failed\n"
							 ));
				QPMMFree2Ex(pVidDecoder);
				return (QP_FALSE);
			}
			break;
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
		case VIDDECODER_SIL9025:   // Andy 2008.05.12
			pVidDecoder = (CVideoDecoder *)QPMMMalloc2Ex(sizeof(CSIL9025));
			if (!pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pVidDecoder == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			This->m_pVidDecoder = (CVideoDecoder *)CSIL9025_Constructor((CSIL9025 *)pVidDecoder, 
																	   &This->m_Object,
																	   OBJECT_DEFAULT,
																	   I_TUNER(This),
																	   I_TVAUDIO(This),
																	   I_I2C(This),
																	   SIL9025_DEVICE_ADDR
																	   );
			if (QP_NULL == This->m_pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() CSIL9025_Constructor() failed\n"
							 ));
				QPMMFree2Ex(pVidDecoder);
				return (QP_FALSE);
			}
			break;
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
		case VIDDECODER_MI2010:
			pVidDecoder = (CVideoDecoder *)QPMMMalloc2Ex(sizeof(CMi2010));
			if (!pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pVidDecoder == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			This->m_pVidDecoder = (CVideoDecoder *)CMi2010_Constructor((CMi2010 *)pVidDecoder, 
																	   &This->m_Object,
																	   OBJECT_DEFAULT,
																	   I_TUNER(This),
																	   I_TVAUDIO(This),
																	   I_I2C(This),
																	   MI2010_DEVICE_ADDR
																	   );
			if (QP_NULL == This->m_pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() CMi2010_Constructor() failed\n"
							 ));
				QPMMFree2Ex(pVidDecoder);
				return (QP_FALSE);
			}
			break;
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
		case VIDDECODER_USBFW:
			pVidDecoder = (CVideoDecoder *)QPMMMalloc2Ex(sizeof(CVidInputUsbFW));
			if (!pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pVidDecoder == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			This->m_pVidDecoder = (CVideoDecoder *)CVidInputUsbFW_Constructor((CVidInputUsbFW *)pVidDecoder, 
																			  &This->m_Object,
																			  OBJECT_CRITICAL_LIGHT,
																			  I_TUNER(This),
																			  I_TVAUDIO(This),
																			  This->m_pUsbCntl
																			  );
			if (QP_NULL == This->m_pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() CVidInputUsbFW_Constructor() failed\n"
							 ));
				QPMMFree2Ex(pVidDecoder);
				return (QP_FALSE);
			}
			break;
#endif
			
#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
		case VIDDECODER_ADV7614:
			pVidDecoder = (CVideoDecoder *)QPMMMalloc2Ex(sizeof(CADV7614));
			if (!pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pVidDecoder == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			This->m_pVidDecoder = (CVideoDecoder *)CADV7614_Constructor((CADV7614 *)pVidDecoder, 
																	    &This->m_Object,
																	    OBJECT_DEFAULT,
																	    I_TUNER(This),
																	    I_TVAUDIO(This),
																	    I_I2C(This),
																	    ADV7614_DEVICE_ADDR_IO
																	    );
			if (QP_NULL == This->m_pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() CADV7614_Constructor() failed\n"
							 ));
				QPMMFree2Ex(pVidDecoder);
				return (QP_FALSE);
			}
			break;
#endif
			
#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
		case VIDDECODER_ADI7441:
			pVidDecoder = (CVideoDecoder *)QPMMMalloc2Ex(sizeof(CADI7441));
			if (!pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pVidDecoder == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			This->m_pVidDecoder = (CVideoDecoder *)CADI7441_Constructor((CADI7441 *)pVidDecoder, 
																	   &This->m_Object,
																	   OBJECT_DEFAULT,
																	   I_TUNER(This),
																	   I_TVAUDIO(This),
																	   I_I2C(This),
																	   ADI7441_DEVICE_ADDR,
																	   This->m_InitData.vidDecoderInitData.channel
																	   );
			if (QP_NULL == This->m_pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() CADI7441_Constructor() failed\n"
							 ));
				QPMMFree2Ex(pVidDecoder);
				return (QP_FALSE);
			}
			break;
#endif

		case VIDDECODER_IT6603:
			pVidDecoder = (CVideoDecoder *)QPMMMalloc2Ex(sizeof(CIT6603));
			if (!pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pVidDecoder == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			This->m_pVidDecoder = (CVideoDecoder *)CIT6603_Constructor((CIT6603 *)pVidDecoder, 
																	   &This->m_Object,
																	   OBJECT_DEFAULT,
																	   I_TUNER(This),
																	   I_TVAUDIO(This),
																	   I_I2CEX(This),	//I_I2C(This),
																	   IT6603_DEVICE_ADDR
																	   );
			if (QP_NULL == This->m_pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() CIT6603_Constructor() failed\n"
							 ));
				QPMMFree2Ex(pVidDecoder);
				return (QP_FALSE);
			}
			break;

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
		case VIDDECODER_TVP7002:
			pVidDecoder = (CVideoDecoder *)QPMMMalloc2Ex(sizeof(CTVP7002));
			if (!pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pVidDecoder == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			if (MULTIPLE_VIDDECODER_TYPE1 == This->m_InitData.vidDecoderInitData.multiType) 
			{
				// need to access IT6604, use S/W I2C
				This->m_pVidDecoder = (CVideoDecoder *)CTVP7002_Constructor((CTVP7002 *)pVidDecoder, 
																		   &This->m_Object,
																		   OBJECT_DEFAULT,
																		   I_TUNER(This),
																		   I_TVAUDIO(This),
																		   I_I2CEX(This),
																		   TVP7002_DEVICE_ADDR
																		   );
			} 
			else
			{
				This->m_pVidDecoder = (CVideoDecoder *)CTVP7002_Constructor((CTVP7002 *)pVidDecoder, 
																		   &This->m_Object,
																		   OBJECT_DEFAULT,
																		   I_TUNER(This),
																		   I_TVAUDIO(This),
																		   I_I2C(This),
																		   TVP7002_DEVICE_ADDR
																		   );
			}
			
			if (QP_NULL == This->m_pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() CTVP7002_Constructor() failed\n"
							 ));
				QPMMFree2Ex(pVidDecoder);
				return (QP_FALSE);
			}
			break;
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
		case VIDDECODER_TVP7002VH:
			pVidDecoder = (CVideoDecoder *)QPMMMalloc2Ex(sizeof(CTVP7002VH));
			if (!pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pVidDecoder == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			This->m_pVidDecoder = (CVideoDecoder *)CTVP7002VH_Constructor((CTVP7002VH *)pVidDecoder, 
																		  &This->m_Object,
																		  OBJECT_DEFAULT,
																		  I_TUNER(This),
																		  I_TVAUDIO(This),
																		  I_I2C(This),
																		  TVP7002VH_DEVICE_ADDR
																		  );
			
			if (QP_NULL == This->m_pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() CTVP7002VH_Constructor() failed\n"
							 ));
				QPMMFree2Ex(pVidDecoder);
				return (QP_FALSE);
			}
			break;
#endif
			
#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
		case VIDDECODER_AUTO1:
			pVidDecoder = (CVideoDecoder *)QPMMMalloc2Ex(sizeof(CVidInputUsbFW));
			if (!pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pVidDecoder == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			This->m_pVidDecoder = (CVideoDecoder *)CVidInputAuto1_Constructor((CVidInputAuto1 *)pVidDecoder, 
																			  &This->m_Object,
																			  OBJECT_DEFAULT,
																			  I_TUNER(This),
																			  I_TVAUDIO(This),
																			  I_I2C(This),
																			  AUTO1_DEVICE_ADDR /*FAKE*/
																			  );
			if (QP_NULL == This->m_pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() CVidInputAuto1_Constructor() failed\n"
							 ));
				QPMMFree2Ex(pVidDecoder);
				return (QP_FALSE);
			}

			break;			
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
		case VIDDECODER_OV5642:
			pVidDecoder = (CVideoDecoder *)QPMMMalloc2Ex(sizeof(COV5642));
			if (!pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pVidDecoder == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			This->m_pVidDecoder = (CVideoDecoder *)COV5642_Constructor((COV5642 *)pVidDecoder, 
																	   &This->m_Object,
																	   OBJECT_DEFAULT,
																	   I_TUNER(This),
																	   I_TVAUDIO(This),
																	   I_I2C(This),
																	   OV5642_DEVICE_ADDR
																	   );
			if (QP_NULL == This->m_pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() COV5642_Constructor() failed\n"
							 ));
				QPMMFree2Ex(pVidDecoder);
				return (QP_FALSE);
			}
			break;
#endif

//#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
		case VIDDECODER_OV10633:
			pVidDecoder = (CVideoDecoder *)QPMMMalloc2Ex(sizeof(COV10633));
			if (!pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pVidDecoder == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			This->m_pVidDecoder = (CVideoDecoder *)COV10633_Constructor((COV10633 *)pVidDecoder, 
																	   &This->m_Object,
																	   OBJECT_DEFAULT,
																	   I_TUNER(This),
																	   I_TVAUDIO(This),
																	   I_I2C(This),
																	   OV10633_DEVICE_ADDR
																	   );
			if (QP_NULL == This->m_pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() COV10633_Constructor() failed\n"
							 ));
				QPMMFree2Ex(pVidDecoder);
				return (QP_FALSE);
			}
			break;
//#endif

#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
#if defined (CONFIG_QL300_AVIU_NATIVE_NT99240)
		case VIDDECODER_NATIVE_NT99240:
			pVidDecoder = (CVideoDecoder *)QPMMMalloc2Ex(sizeof(CVidInputNativeNt99240));
			if (!pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pVidDecoder == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			This->m_pVidDecoder = (CVideoDecoder *)CVidInputNativeNt99240_Constructor(
				(CVidInputNativeNt99240 *)pVidDecoder, 
				&This->m_Object,
				OBJECT_DEFAULT,
				I_TUNER(This),
				I_TVAUDIO(This),
				I_I2C(This)
				);
			if (QP_NULL == This->m_pVidDecoder)
			{
				QPDbgMPrintf(
					(QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					"CQLCodecLib_CreatePeripherals() CVidInputNativeNt99240_Constructor() failed\n"
					));
				QPMMFree2Ex(pVidDecoder);
				return (QP_FALSE);
			}
			break;
#else
		case VIDDECODER_NATIVE_MI2010:
			pVidDecoder = (CVideoDecoder *)QPMMMalloc2Ex(sizeof(CVidInputNativeMi2010));
			if (!pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pVidDecoder == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			This->m_pVidDecoder = (CVideoDecoder *)CVidInputNativeMi2010_Constructor(
				(CVidInputNativeMi2010 *)pVidDecoder, 
				&This->m_Object,
				OBJECT_DEFAULT,
				I_TUNER(This),
				I_TVAUDIO(This),
				I_I2C(This)
				);
			if (QP_NULL == This->m_pVidDecoder)
			{
				QPDbgMPrintf(
					(QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					"CQLCodecLib_CreatePeripherals() CVidInputNativeMi2010_Constructor() failed\n"
					));
				QPMMFree2Ex(pVidDecoder);
				return (QP_FALSE);
			}
			break;
#endif			
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
		case VIDDECODER_TW9910:
			pVidDecoder = (CVideoDecoder *)QPMMMalloc2Ex(sizeof(CTW9910));
			if (!pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pVidDecoder == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			if (MULTIPLE_VIDDECODER_TYPE1 == This->m_InitData.vidDecoderInitData.multiType) 
			{
				// need to access IT6604, use S/W I2C
				This->m_pVidDecoder = (CVideoDecoder *)CTW9910_Constructor((CTW9910 *)pVidDecoder, 
																		   &This->m_Object,
																		   OBJECT_DEFAULT,
																		   I_TUNER(This),
																		   I_TVAUDIO(This),
																		   I_I2CEX(This),
																		   TW9910_DEVICE_ADDR
																		   );
			}
			else
			{
				This->m_pVidDecoder = (CVideoDecoder *)CTW9910_Constructor((CTW9910 *)pVidDecoder, 
																		   &This->m_Object,
																		   OBJECT_DEFAULT,
																		   I_TUNER(This),
																		   I_TVAUDIO(This),
																		   I_I2C(This),
																		   TW9910_DEVICE_ADDR
																		   );				
			}
			
			if (QP_NULL == This->m_pVidDecoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals(): CTW9910_Constructor() failed\n"
							 ));
				QPMMFree2Ex(pVidDecoder);
				return (QP_FALSE);
			}
			break;
#endif			

		default:
			break;
	}

	return (QP_TRUE);
}



TQP_BOOL CQLCodecLib_CreatePeripherals(CQLCodecLib *This) 
{
	CI2C			*pI2CProvider;
	CVideoEncoder	*pVidEncoder;
	CAudioCodec		*pAudCodec;
	CTuner			*pTuner;
	CTVAudio		*pTVAudio;


	// create I2C provider
	//
	pI2CProvider = (CI2C *)QPMMMalloc2Ex(sizeof(CI2C));

	if (!pI2CProvider)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CQLCodecLib_CreatePeripherals() pI2CProvider == NULL!!!\n"
					 ));
		return (QP_FALSE);
	}

	This->m_pI2CProvider = CI2C_Constructor(pI2CProvider, 
											&This->m_Object,
											OBJECT_DEFAULT,
											This->m_InitData.i2cInitData.type,
											This->m_InitData.i2cInitData.clk,
											This->m_InitData.i2cInitData.data,
											This->m_pUsbCntl,
											This->m_pMpgCodec
											);
	if (QP_NULL == This->m_pI2CProvider)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CQLCodecLib_CreatePeripherals() CI2C init failed\n"
					 ));
		QPMMFree2Ex(pI2CProvider);
		return (QP_FALSE);
	}

	// create extend I2C provider
	//
	pI2CProvider = (CI2C *)QPMMMalloc2Ex(sizeof(CI2C));

	if (!pI2CProvider)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CQLCodecLib_CreatePeripherals() pI2CProviderEx == NULL!!!\n"
					 ));
		return (QP_FALSE);
	}


	This->m_pI2CProviderEx = CI2C_Constructor(pI2CProvider,
											&This->m_Object,
											OBJECT_DEFAULT,
											This->m_InitData.i2cInitDataEx.type,
											This->m_InitData.i2cInitDataEx.clk,
											This->m_InitData.i2cInitDataEx.data,
											This->m_pUsbCntl,
											This->m_pMpgCodec
											);
	if (QP_NULL == This->m_pI2CProviderEx)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CQLCodecLib_CreatePeripherals() CI2CEx init failed\n"
					 ));
		QPMMFree2Ex(pI2CProvider);
		return (QP_FALSE);
	}


	// create tv tuner
	//
	switch (This->m_InitData.tunerInitData.type)
	{
		case TUNER_DUMMY:
			pTuner = (CTuner *)QPMMMalloc2Ex(sizeof(CTuner));

			if (!pTuner)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pTuner == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			This->m_pTuner = CTuner_Constructor(pTuner,
												&This->m_Object,
												OBJECT_DEFAULT
												);
			if (QP_NULL == This->m_pTuner)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() CTuner_Constructor failed!!!\n" 
							 ));
				QPMMFree2Ex((void *)pTuner);
				return (QP_FALSE);
			}
			break;

		default:
			break;
	}

	// create tv Audio
	//
	switch (This->m_InitData.tvAudioInitData.type)
	{
		case TVAUDIO_DUMMY:
			pTVAudio = (CTVAudio *)QPMMMalloc2Ex(sizeof(CTVAudio));

			if (!pTVAudio)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pTVAudio == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			This->m_pTVAudio = CTVAudio_Constructor(pTVAudio,
													&This->m_Object,
													OBJECT_DEFAULT
													);
			if (QP_NULL == This->m_pTVAudio)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() CTVAudio_Constructor() failed!!!\n" 
							 ));
				QPMMFree2Ex((void *)pTVAudio);
				return (QP_FALSE);
			}
			break;

		default:
			break;
	}

	// create video decoder
	//
	if (!CQLCodecLib_CreatePeripheralsVidDecoder(This))
	{
		return (QP_FALSE);
	}

	// create video encoder
	//
	switch (This->m_InitData.vidEncoderInitData.type)
	{
		case VIDENCODER_DUMMY:
			pVidEncoder = (CVideoEncoder *)QPMMMalloc2Ex(sizeof(CVideoEncoder));
			if (!pVidEncoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pVidEncoder == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			This->m_pVidEncoder = (CVideoEncoder *)CVideoEncoder_Constructor(pVidEncoder, 
																			 &This->m_Object,
																			 OBJECT_DEFAULT
																			 );
			if (QP_NULL == This->m_pVidEncoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() CVideoEncoder_Constructor() failed\n"
							 ));
				QPMMFree2Ex(pVidEncoder);
				return (QP_FALSE);
			}
			break;

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
		case VIDENCODER_SAA7128:
			pVidEncoder = (CVideoEncoder *)QPMMMalloc2Ex(sizeof(CSaa7128));
			if (!pVidEncoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pVidEncoder == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			This->m_pVidEncoder = (CVideoEncoder *)CSaa7128_Constructor((CSaa7128 *)pVidEncoder, 
																		&This->m_Object,
																		OBJECT_DEFAULT,
																		I_I2C(This),
																		SAA7128_DEVICE_ADDR
																		);
			if (QP_NULL == This->m_pVidEncoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() CSaa7128_Constructor() failed\n"
							 ));
				QPMMFree2Ex(pVidEncoder);
				return (QP_FALSE);
			}
			break;
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
		case VIDENCODER_ADI7393:   // Andy 2008.06.18
			pVidEncoder = (CVideoEncoder *)QPMMMalloc2Ex(sizeof(CADI7393));
			if (!pVidEncoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pVidEncoder == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			This->m_pVidEncoder = (CVideoEncoder *)CADI7393_Constructor((CADI7393 *)pVidEncoder, 
																		&This->m_Object,
																		OBJECT_DEFAULT,
																		I_I2C(This),
																		ADI7393_DEVICE_ADDR
																		);
			if (QP_NULL == This->m_pVidEncoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() CADI7393_Constructor() failed\n"
							 ));
				QPMMFree2Ex(pVidEncoder);
				return (QP_FALSE);
			}
			break;
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
		case VIDENCODER_SIL9030:   // Andy 2008.05.12
			pVidEncoder = (CVideoEncoder *)QPMMMalloc2Ex(sizeof(CSIL9030));
			if (!pVidEncoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pVidEncoder == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			This->m_pVidEncoder = (CVideoEncoder *)CSIL9030_Constructor((CSIL9030 *)pVidEncoder, 
																		&This->m_Object,
																		OBJECT_DEFAULT,
																		I_I2C(This),
																		SIL9030_DEVICE_ADDR
																		);
			if (QP_NULL == This->m_pVidEncoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() CSIL9030_Constructor() failed\n"
							 ));
				QPMMFree2Ex(pVidEncoder);
				return (QP_FALSE);
			}
			break;
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
		case VIDENCODER_SIL9034:   // Andy 2008.05.12
			pVidEncoder = (CVideoEncoder *)QPMMMalloc2Ex(sizeof(CSIL9034));
			if (!pVidEncoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pVidEncoder == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			This->m_pVidEncoder = (CVideoEncoder *)CSIL9034_Constructor((CSIL9034 *)pVidEncoder, 
																		&This->m_Object,
																		OBJECT_DEFAULT,
																		I_I2C(This),
																		SIL9034_DEVICE_ADDR
																		);
			if (QP_NULL == This->m_pVidEncoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() CSIL9034_Constructor() failed\n"
							 ));
				QPMMFree2Ex(pVidEncoder);
				return (QP_FALSE);
			}
			break;
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
		case VIDENCODER_ADI9889:
			pVidEncoder = (CVideoEncoder *)QPMMMalloc2Ex(sizeof(CADI9889));
			if (!pVidEncoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pVidEncoder == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			This->m_pVidEncoder = (CVideoEncoder *)CADI9889_Constructor((CADI9889 *)pVidEncoder, 
																		&This->m_Object,
																		OBJECT_DEFAULT,
																		I_I2C(This),
																		ADI9889_DEVICE_ADDR
																		);
			if (QP_NULL == This->m_pVidEncoder)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() CADI9889_Constructor() failed\n"
							 ));
				QPMMFree2Ex(pVidEncoder);
				return (QP_FALSE);
			}
			break;
#endif

		default:
			break;
	}

	// create audio codec
	//
	switch (This->m_InitData.audCodecInitData.type)
	{
		case AUDCODEC_DUMMY:
			pAudCodec = (CAudioCodec *)QPMMMalloc2Ex(sizeof(CAudioCodec));

			if (!pAudCodec)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pAudCodec == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			This->m_pAudCodec = CAudioCodec_Constructor(pAudCodec,
														&This->m_Object,
														OBJECT_DEFAULT,
														I_TUNER(This),
														I_TVAUDIO(This)
														);

			if (QP_NULL == This->m_pAudCodec)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() CAudioCodec_Constructor() failed\n"
							 ));
				QPMMFree2Ex(pAudCodec);
				return (QP_FALSE);
			}
			break;

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
		case AUDCODEC_FM31:
			pAudCodec = (CAudioCodec *)QPMMMalloc2Ex(sizeof(CFM31));

			if (!pAudCodec)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pAudCodec == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			This->m_pAudCodec = (CAudioCodec *) CFM31_Constructor((CFM31 *)pAudCodec,
														&This->m_Object,
														OBJECT_DEFAULT,
														I_TUNER(This),
														I_TVAUDIO(This),
														I_I2C(This),
														FM31_DEVICE_ADDR
														);

			if (QP_NULL == This->m_pAudCodec)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() CFM31_Constructor() failed\n"
							 ));
				QPMMFree2Ex(pAudCodec);
				return (QP_FALSE);
			}
			break;
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
		case AUDCODEC_FM31_BUS2:
			pAudCodec = (CAudioCodec *)QPMMMalloc2Ex(sizeof(CFM31));

			if (!pAudCodec)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() pAudCodec == NULL!!!\n"
							 ));
				return (QP_FALSE);
			}

			This->m_pAudCodec = (CAudioCodec *) CFM31_Constructor((CFM31 *)pAudCodec,
														&This->m_Object,
														OBJECT_DEFAULT,
														I_TUNER(This),
														I_TVAUDIO(This),
														I_I2CEX(This),
														FM31_DEVICE_ADDR
														);

			if (QP_NULL == This->m_pAudCodec)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodecLib_CreatePeripherals() CFM31_Constructor() failed\n"
							 ));
				QPMMFree2Ex(pAudCodec);
				return (QP_FALSE);
			}
			break;
#endif

		default:
			break;
	}

	return (QP_TRUE);
}



// Andy 2008.10.14: delete video decoder
void CQLCodecLib_DeletePeripheralsVidDecoder(CQLCodecLib *This)
{
	if (This->m_pVidDecoder)
	{
		switch (This->m_InitData.vidDecoderInitData.type)
		{
			case VIDDECODER_DUMMY:
				CVideoDecoder_Destructor(This->m_pVidDecoder);
				break;

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
			case VIDDECODER_SAA7115:
				CSaa7115_Destructor((CSaa7115 *)This->m_pVidDecoder);
				break;
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
			case VIDDECODER_MT9V112:
				CMt9V112_Destructor((CMt9V112 *)This->m_pVidDecoder);
				break;
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
			case VIDDECODER_OV7720:
				COV7720_Destructor((COV7720 *)This->m_pVidDecoder);
				break;
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
			case VIDDECODER_ADI7401:
				CADI7401_Destructor((CADI7401 *)This->m_pVidDecoder);
				break;
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
			case VIDDECODER_PE1005:
				CPE1005_Destructor((CPE1005 *)This->m_pVidDecoder);
				break;
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
			case VIDDECODER_SIL9011:
				CSIL9011_Destructor((CSIL9011 *)This->m_pVidDecoder);
				break;
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
			case VIDDECODER_SIL9025:
				CSIL9025_Destructor((CSIL9025 *)This->m_pVidDecoder);
				break;
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
			case VIDDECODER_MI2010:
				CMi2010_Destructor((CMi2010 *)This->m_pVidDecoder);
				break;
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
			case VIDDECODER_USBFW:
				CVidInputUsbFW_Destructor((CVidInputUsbFW *)This->m_pVidDecoder);
				break;
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
			case VIDDECODER_ADV7614:
				CADV7614_Destructor((CADV7614 *)This->m_pVidDecoder);
				break;
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
			case VIDDECODER_ADI7441:
				CADI7441_Destructor((CADI7441 *)This->m_pVidDecoder);
				break;
#endif
				
			case VIDDECODER_IT6603:
				CIT6603_Destructor((CIT6603 *)This->m_pVidDecoder);
				break;
				
#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
			case VIDDECODER_TVP7002:
				CTVP7002_Destructor((CTVP7002 *)This->m_pVidDecoder);
				break;				
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
			case VIDDECODER_TVP7002VH:
				CTVP7002VH_Destructor((CTVP7002VH *)This->m_pVidDecoder);
				break;				
#endif
				
#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
			case VIDDECODER_AUTO1:
				CVidInputAuto1_Destructor((CVidInputAuto1 *)This->m_pVidDecoder);
				break;				
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
			case VIDDECODER_OV5642:
				COV5642_Destructor((COV5642 *)This->m_pVidDecoder);
				break;
#endif

//#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
			case VIDDECODER_OV10633:
				COV10633_Destructor((COV10633 *)This->m_pVidDecoder);
				break;
//#endif

#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
#if defined (CONFIG_QL300_AVIU_NATIVE_NT99240)
			case VIDDECODER_NATIVE_NT99240:
				CVidInputNativeNt99240_Destructor((CVidInputNativeNt99240 *)This->m_pVidDecoder);
				break;
#else
			case VIDDECODER_NATIVE_MI2010:
				CVidInputNativeMi2010_Destructor((CVidInputNativeMi2010 *)This->m_pVidDecoder);
				break;
#endif
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
			case VIDDECODER_TW9910:
				CTW9910_Destructor((CTW9910 *)This->m_pVidDecoder);
				break;				
#endif
				
			default:
				break;
		}

		QPMMFree2Ex(This->m_pVidDecoder);
		This->m_pVidDecoder = QP_NULL;
	}
}



void CQLCodecLib_DeletePeripherals(CQLCodecLib *This)
{
	// delete all the objects we created during Init()
	//
	if (This->m_pAudCodec)
	{
		switch (This->m_InitData.audCodecInitData.type)
		{
			case AUDCODEC_DUMMY:
				CAudioCodec_Destructor(This->m_pAudCodec);
				break;

			default:
				break;
		}

		QPMMFree2Ex(This->m_pAudCodec);
		This->m_pAudCodec = QP_NULL;
	}

	if (This->m_pVidEncoder)
	{
		switch (This->m_InitData.vidEncoderInitData.type)
		{
			case VIDENCODER_DUMMY:
				CVideoEncoder_Destructor(This->m_pVidEncoder);
				break;

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
			case VIDENCODER_SAA7128:
				CSaa7128_Destructor((CSaa7128 *)This->m_pVidEncoder);
				break;
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
			case VIDENCODER_ADI7393:
				CADI7393_Destructor((CADI7393 *)This->m_pVidEncoder);
				break;
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
			case VIDENCODER_SIL9030:
				CSIL9030_Destructor((CSIL9030 *)This->m_pVidEncoder);
				break;
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
			case VIDENCODER_SIL9034:
				CSIL9034_Destructor((CSIL9034 *)This->m_pVidEncoder);
				break;
#endif

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
			case VIDENCODER_ADI9889:
				CADI9889_Destructor((CADI9889 *)This->m_pVidEncoder);
				break;
#endif

			default:
				break;
		}

		QPMMFree2Ex(This->m_pVidEncoder);
		This->m_pVidEncoder = QP_NULL;
	}

	CQLCodecLib_DeletePeripheralsVidDecoder(This);

	if (This->m_pTVAudio)
	{
		switch (This->m_InitData.tvAudioInitData.type)
		{
			case TVAUDIO_DUMMY:
				CTVAudio_Destructor(This->m_pTVAudio);
				break;

			default:
				break;
		}

		QPMMFree2Ex(This->m_pTVAudio);
		This->m_pTVAudio = QP_NULL;
	}

	if (This->m_pTuner)
	{
		switch (This->m_InitData.tunerInitData.type)
		{
			case TUNER_DUMMY:
				CTuner_Destructor(This->m_pTuner);
				break;

			default:
				break;
		}

		QPMMFree2Ex(This->m_pTuner);
		This->m_pTuner = QP_NULL;
	}

	if (This->m_pI2CProviderEx)
	{
		CI2C_Destructor(This->m_pI2CProviderEx);
		QPMMFree2Ex(This->m_pI2CProviderEx);
		This->m_pI2CProviderEx = QP_NULL;
	}

	if (This->m_pI2CProvider)
	{
		CI2C_Destructor(This->m_pI2CProvider);
		QPMMFree2Ex(This->m_pI2CProvider);
		This->m_pI2CProvider = QP_NULL;
	}
}



// Andy 2008.10.14: initialize video decoder
EQP_Errors CQLCodecLib_InitPeripheralsVidDecoder(CQLCodecLib *This)
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;

	// initialize video decoder
	//
	if (This->m_pVidDecoder)
	{
		qpStatus = QPGenericDevice_InitDevice(I_VIDDECODER(This), 
											  This->m_pDeviceCallback,
											  This->m_callbackContext,
											  (TQP_PVOID)&This->m_InitData.vidDecoderInitData
											  );

		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CQLCodecLib_InitDevice() m_pVidDecoder->InitDevice Failed status(%d)!!!\n",
						 qpStatus
						 ));
		}
	}

	return (qpStatus);
}



EQP_Errors CQLCodecLib_InitPeripherals(CQLCodecLib *This)
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;

	// initialize video decoder
	//
	qpStatus = CQLCodecLib_InitPeripheralsVidDecoder(This);
	if (!QP_SUCCESS(qpStatus))
	{
		return (qpStatus);
	}

	// initialize video encoder
	//
	if (This->m_pVidEncoder)
	{
		qpStatus = QPGenericDevice_InitDevice(I_VIDENCODER(This),
											  This->m_pDeviceCallback,
											  This->m_callbackContext,
											  (TQP_PVOID)&This->m_InitData.vidEncoderInitData
											  );

		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CQLCodecLib_InitDevice() m_pVidEncoder->InitDevice Failed status(%d)!!!\n",
						 qpStatus
						 ));
			return (qpStatus);
		}
	}

	// initialize tuner
	//
	if (This->m_pTuner)
	{
		qpStatus = QPGenericDevice_InitDevice(I_TUNER(This), 
											  This->m_pDeviceCallback,
											  This->m_callbackContext,
											  (TQP_PVOID)&This->m_InitData.tunerInitData
											  );
		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CQLCodecLib_InitDevice() m_pTuner->InitDevice Failed status(%d)!!!\n",
						 qpStatus
						 ));
			return (qpStatus);
		}
	}

	// initialize tvaudio
	//
	if (This->m_pTVAudio)
	{
		qpStatus = QPGenericDevice_InitDevice(I_TVAUDIO(This),
											  This->m_pDeviceCallback,
											  This->m_callbackContext,
											  (TQP_PVOID)&This->m_InitData.tvAudioInitData
											  );
		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CQLCodecLib_InitDevice() m_pTVAudio->InitDevice Failed status(%d)!!!\n",
						 qpStatus
						 ));
			return (qpStatus);
		}
	}

	// initialize audio codec
	//
	if (This->m_pAudCodec)
	{
		qpStatus = QPGenericDevice_InitDevice(I_AUDCODEC(This),
											  This->m_pDeviceCallback,
											  This->m_callbackContext,
											  (TQP_PVOID)&This->m_InitData.audCodecInitData
											  );
		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CQLCodecLib_InitDevice() m_pAudCodec->InitDevice Failed status(%d)!!!\n",
						 qpStatus
						 ));
			return (qpStatus);
		}
	}

	return (qpStatus);
}



// Andy 2008.10.14: release video decoder
void CQLCodecLib_DonePeripheralsVidDecoder(CQLCodecLib *This)
{
	if (This->m_pVidDecoder)
		QPGenericDevice_Release(I_VIDDECODER(This));
}



void CQLCodecLib_DonePeripherals(CQLCodecLib *This)
{
	if (This->m_pAudCodec)
		QPGenericDevice_Release(I_AUDCODEC(This));

	if (This->m_pTVAudio)
		QPGenericDevice_Release(I_TVAUDIO(This));

	if (This->m_pTuner)
		QPGenericDevice_Release(I_TUNER(This));

	if (This->m_pVidEncoder)
		QPGenericDevice_Release(I_VIDENCODER(This));

	CQLCodecLib_DonePeripheralsVidDecoder(This);
}



#endif //_QPCODEC_PURE
