///////////////////////////////////////////////////////////////////////////////
//        
//
// Qlcodeclib.c
//
// Description: 
//
//  QLCodec library Init and Done.
// 
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#include "qpcodeclib.h"
#include "cqlcodeclib.h"
#include "qpdbgmgr.h"
#include "qpmemmgr.h"
#include "util.h"
#if defined(QP_LINXKER) || defined(_QP_LINUX) || defined(_QP_MACOSX)
#include "qlautoconf.h"
#endif //QP_LINXKER

TQP_DbgMCompID	g_QPCodecDBGCompID = 0;
TQP_DbgMCompID	g_QPUsbDBGCompID = 0;
TQP_DbgMCompID	g_QPPCIeDBGCompID = 0;
TQP_DbgMCompID	g_QPPeripheralDBGCompID = 0;


EQP_Errors QPCodecInitLibrary(TQP_PVOID pContext1,	//PDEVICE_OBJECT
							  TQP_PVOID pContext2,	//PDEVICE_OBJECT
							  PQPCODECLIB_INITDATA pInitData,
							  ICodecLib** ppCodecLib
							  )
{
	CQLCodecLib		*pQLCodecLib;
	EQP_Errors		qpStatus = QPERR_SUCCESS;


	// register the component in an initialization call (only first 20 characters are saved)
	//
	if (0 == g_QPCodecDBGCompID)
		QPDbgMRegComp(&g_QPCodecDBGCompID,
					  "QP CodecLib"
					  );
	if (0 == g_QPUsbDBGCompID)
		QPDbgMRegComp(&g_QPUsbDBGCompID,
					  "QP Usb"
					  );
	if (0 == g_QPPCIeDBGCompID)
		QPDbgMRegComp(&g_QPPCIeDBGCompID,
					  "QP PCIe"
					  );
	if (0 == g_QPPeripheralDBGCompID)
		QPDbgMRegComp(&g_QPPeripheralDBGCompID,
					  "QP Peripheral"
					  );

#if 0
#if defined(_QP_WINKER)
	QPDbgMSetCompLevel(g_QPCodecDBGCompID, 
					   QPDBGM_LEVEL_TRACE
					   );
	QPDbgMSetCompLevel(g_QPUsbDBGCompID, 
					   QPDBGM_LEVEL_TRACE
					   );
	QPDbgMSetCompLevel(g_QPPCIeDBGCompID, 
					   QPDBGM_LEVEL_TRACE
					   );
	QPDbgMSetCompLevel(g_QPPeripheralDBGCompID, 
					   QPDBGM_LEVEL_TRACE
					   );
#endif //_QP_WINKER
#endif

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE), 
				 "QPCodecInitLibrary()\n" 
				 ));

	if (!pInitData || 
		!ppCodecLib
		)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_FATAL),
					 "QPCodecInitLibrary() pInitData == NULL or ppCodecLib == NULL!!!\n"
					 ));
		qpStatus = QPERR_PARMS;
		goto QPCodecInitLibrary_Err;
	}

	pQLCodecLib = (CQLCodecLib *)QPMMMalloc2Ex(sizeof(CQLCodecLib));

	if (!pQLCodecLib)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_FATAL),
					 "QPCodecInitLibrary() pQLCodecLib == NULL!!!\n"
					 ));
		qpStatus = QPERR_NOMEM;
		goto QPCodecInitLibrary_Err;
	}

	if (QP_NULL == CQLCodecLib_Constructor(pQLCodecLib, 
										   pContext1, 
										   pContext2,
										   pInitData
										   ))
	{
		QPMMFree2Ex(pQLCodecLib);
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_FATAL),
					 "QPCodecInitLibrary() FAILED - unable to create CQLCodecLib!!!\n"
					 ));
		qpStatus = QPERR_FAIL;
		goto QPCodecInitLibrary_Err;
	}

	if (!CQLCodecLib_Init(pQLCodecLib))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_FATAL),
					 "QPCodecInitLibrary() FAILED - CQLCodecLib Init Failed!!!\n"
					 ));
		CQLCodecLib_Done(pQLCodecLib);
		CQLCodecLib_Destructor(pQLCodecLib);
		QPMMFree2Ex(pQLCodecLib);
		qpStatus = QPERR_FAIL;
		goto QPCodecInitLibrary_Err;
	}

	// return only the ICodecLib interface
	//
	*ppCodecLib = &pQLCodecLib->m_iCodecLib;
	return (qpStatus);

QPCodecInitLibrary_Err:

	// unregister the component from the debug manager
	//
	QPDbgMUnregComp(g_QPCodecDBGCompID);
	g_QPCodecDBGCompID = 0;
	QPDbgMUnregComp(g_QPUsbDBGCompID);
	g_QPUsbDBGCompID = 0;
	QPDbgMUnregComp(g_QPPCIeDBGCompID);
	g_QPPCIeDBGCompID = 0;
	QPDbgMUnregComp(g_QPPeripheralDBGCompID);
	g_QPPeripheralDBGCompID = 0;
    
    if (ppCodecLib)
    {
        *ppCodecLib = QP_NULL;
    }
    else
    {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
                      "QPCodecInitLibrary() FAILED - ppCodecLib is NULL!!!\n"
                      ));
    }
	return (qpStatus);
}



EQP_Errors QPCodecDoneLibrary(ICodecLib* pCodecLib)
{
	CQLCodecLib	*pQLCodecLib;
	EQP_Errors	qpStatus;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "QPCodecDoneLibrary() pCodecLib(0x%x)\n",
				 pCodecLib
				 ));

	if (pCodecLib)
	{
		pQLCodecLib = GET_INHERITED_OBJECT(CQLCodecLib, m_iCodecLib, pCodecLib);

		if (pQLCodecLib)
		{
			CQLCodecLib_Done(pQLCodecLib);
			CQLCodecLib_Destructor(pQLCodecLib);
			QPMMFree2Ex(pQLCodecLib);
		}

		qpStatus = QPERR_SUCCESS;
	}
	else
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "QPCodecDoneLibrary() pCodecLib == NULL!!!\n"
					 ));
		qpStatus = QPERR_PARMS;
	}

#ifndef CONFIG_HOST_PLATFORM_USERSPACE
	// unregister the component from the debug manager 
	// in the termination call
	//
	if (g_QPCodecDBGCompID)
	{
		QPDbgMUnregComp(g_QPCodecDBGCompID);
		g_QPCodecDBGCompID = 0;
	}
	if (g_QPUsbDBGCompID)
	{
		QPDbgMUnregComp(g_QPUsbDBGCompID);
		g_QPUsbDBGCompID = 0;
	}
	if (g_QPPCIeDBGCompID)
	{
		QPDbgMUnregComp(g_QPPCIeDBGCompID);
		g_QPPCIeDBGCompID = 0;
	}
	if (g_QPPeripheralDBGCompID)
	{
		QPDbgMUnregComp(g_QPPeripheralDBGCompID);
		g_QPPeripheralDBGCompID = 0;
	}
#endif
	return (qpStatus);
}



