///////////////////////////////////////////////////////////////////////////////
//        
//
//	Generic ITRON 3 header file
//
//
// History:
//	Oct 20,2006	(EC) Created
//
// Authors: (EC) Enrico Cadorin
//
///////////////////////////////////////////////////////////////////////////////

#define TA_HLNG 0x01
#define TA_TFIFO 0x00
#define TA_WSGL 0x00
#define TWF_ORW 0x02
#define TWF_CLR 0x01
#define TMO_FEVR (-1)

#define E_OK 0
#define E_ID (-35)
#define E_NOMEM (-10)
#define E_NOEXS (-52)
#define E_TMOUT (-85)

#define TTS_RUN	0x01
#define TTS_RDY	0x02
#define TTS_WAI	0x04
#define TTS_SUS	0x08
#define TTS_WAS	0x0C
#define TTS_DMT	0x10

/* integer */
typedef char            B;
typedef unsigned char   UB;
typedef short           H;
typedef unsigned short  UH;
typedef long            W;
typedef unsigned long   UW;

/* typeless */
typedef long            VW;
typedef short           VH;
typedef char            VB;
typedef void            *VP;
/*typedef void          (*FP)();*/

/* types for itron objects */
typedef int             INT;
typedef unsigned int    UINT;

#ifndef WIN32
typedef char            BOOL;       /* bool type */
#endif
typedef INT             FN;         /* function number */
typedef W               ID;         /* identifire */
typedef W               BOOL_ID;    /* boolean or identifire */
typedef W               HNO;        /* handler number */
typedef W               RNO;        /* rendezvous number*/
typedef W               NODE;       /* node number */
typedef UW              ATR;        /* attribute */
typedef H               ER;         /* error code */
typedef H               PRI;        /* priority  */
typedef B               T_MSG;      /* message packet */

/* types for Timer and Alarm functions */
typedef W                   DLYTIME;    /* delay time(ms) */
typedef W                   TMO;       /* time out(ms) */


/* DLINK: Double Linked List */
typedef struct dlink_item {
	struct dlink_item           *next;  /* next reference */
	struct dlink_item           *prev;  /* prev reference */
} DLITEM;

/* DLHEADER: Header for Double Linked List */
typedef struct dlink_header {
	DLITEM           link;   /* link pointer */
} DLHEADER;



struct systime
{
    H                   millisec;   /* millisecond */
    UW                  sec;        /* second */
};

typedef struct systime  SYSTIME;    /* system time */
typedef struct systime  CYCTIME;    /* cycle time */
typedef struct systime  ALMTIME;    /* alarm time */

struct defalm
{
    VP          exinf;      /* expansion info */
    ATR         almatr;     /* alarm handler */
    void        (*almhdr)();/* alarm handler address */
    UINT        tmmode;     /* wake up time mode */
    ALMTIME     almtim;     /* handler wake up time */
};

typedef struct defalm   T_DALM;     /* alarm handler define info */

typedef struct refalm {
    VP                  exinf;      /* expansion info  */
    ALMTIME             lfttime;    /* leftover time */
} T_RALM;     /* alarm handler state info */



// Tasks

struct create_task
{
	VP		exinf;			// extended information
	ATR		tskatr;			// task attribute
	void	(*task)(INT);	// task entry point
	PRI		itskpri;		// initial task priority
	INT		stksz;			// stack size
	UW		stkaddr;		// task's stack initial value
};

typedef struct create_task      T_CTSK;

struct refer_task
{
    VP              exinf;      /* extended information */
    PRI             tskpri;     /* task priority (current) */
    UINT            tskstat;    /* task state */

    UB              dstate;     /* dispatch & interrupt state */
    UB              wstate;     /* wait state */
    INT             wupcnt;     /* wakeup count */
    ATR             tskatr;     /* task attribute */
    void            (*task)(INT); /* task entry point */
    PRI             itskpri;    /* initial priority */
};

typedef struct refer_task   T_RTSK;

ER          cre_tsk(ID, T_CTSK *);
ER          del_tsk(ID);
ER          sta_tsk(ID tid, INT stacd);
ER          dly_tsk(DLYTIME);
ER          ref_tsk(T_RTSK*, ID);

// Semaphores

typedef struct sem_create {
    VP          exinf;              // extended infomation
    ATR         sematr;             // semaphore attributes
    INT         isemcnt;            // initial semaphore count
    INT         maxsem;             // maximum semaphore count
} T_CSEM;

ER          cre_sem(ID, T_CSEM *);
ER          del_sem(ID);
ER          wai_sem(ID);
ER          twai_sem(ID, TMO);
ER          sig_sem(ID);

// Event groups

typedef struct evf_create {
    VP          exinf;              /* extended infomation */
    ATR         flgatr;             /* eventflag attribute */
    UINT        iflgptn;            /* initial eventflag pattern */
} T_CFLG;

typedef struct refer_eventflag {
    VP          exinf;              /* extended infomation */
    BOOL_ID     wtsk;               /* wait task information */
    UINT        flgptn;             /* eventflag bit pattern */
} T_RFLG;

typedef struct eventflag {
    DLHEADER        tasks;          /* wait queue */
    UINT            flgptn;         /* eventflag pattern */
    UINT            iflgptn;        /* initial eventflag pattern */
    ATR             flgatr;         /* eventflag attribute */
    VP              exinf;          /* extended information */
} EVFLAG;

ER          cre_flg(ID, T_CFLG *);
ER          del_flg(ID);
ER          set_flg(ID, UINT);
ER          clr_flg(ID, UINT);
ER          wai_flg(UINT *, ID, UINT, UINT);
ER          pol_flg(UINT *, ID, UINT, UINT);
ER          twai_flg(UINT *, ID, UINT, UINT, TMO);

/* Timer */
void        timer_entry(void);
void        timer_init(void);
ER          set_tim(SYSTIME *);
ER          get_tim(SYSTIME *);
ER          def_alm(HNO, T_DALM *);
ER          ref_alm(T_RALM *, HNO);
ER          can_alm(HNO);


// Memory pools
ER          get_blk(VP *, ID, INT);
ER          pget_blk(VP *, ID, INT);
ER          tget_blk(VP *, ID, INT, TMO);
ER          rel_blk(ID, VP);

