///////////////////////////////////////////////////////////////////////////////
//        
//
//		Constants definitions to identify a platform
//		Generic settings
//
// History:
//	Sep 26,2006	(EC) Created
//
// Authors: (EC) Enrico Cadorin
//
///////////////////////////////////////////////////////////////////////////////


#ifndef __PLATFORM_H__
#define __PLATFORM_H__

#include "qppfmgr.h"
#include "qposmgr.h"

// Enable/disable output of debug information
//#define QP_DEBUG

// Select an OS; define only one of these constants
//#define _QP_WINDOWS	// Win32
//#define _QP_WINKER	// Windows Kernel
#define _QP_ITRON		// ITRON 3

// ITRON specific definitions
#ifdef _QP_ITRON
#include "itron.h"

#define QP_MIN_TASKID		1
#define QP_NUM_TASKIDS		4
#define QP_MIN_EVTGRPID		1
#define QP_NUM_EVTGRPIDS	10
#define QP_MIN_SEMID		1
#define QP_NUM_SEMIDS		50

#define QP_MPLID			1

#endif //_QP_ITRON

// define this constant if no peripherals need to be controlled by the codec
#define _QPCODEC_PURE

// set the priority of the thread created by the coedc
#define QP_CODECTHR_PRI		QPOS_THRPRI_DEFAULT

// Access Mode
#define QPPF_DEF_ACCESSMODE		QPPF_MODE_DIRECT

// Bus type
#define QPPF_DEF_BUSTYPE		QPPF_BUS_CUSTOM_1

// Slave Interface parameters
#define QPPF_DEF_SLAVEIF_REG	0x2000000
#define QPPF_DEF_SLAVEIF_MEM	0x2000000
#define QPPF_DEF_SLAVEIF_PAGE	0x100000

// Memory parameters
#define QPPF_DEF_MEMORY_MBITS	512
#define QPPF_DEF_MEMORY_TYPE	QPPF_MEM_TYPE_MDDR

// Interrupt numbers
#define QPPF_DEF_INTNUM			1

// Video Input
#define QPPF_DEF_VIUMODE		QPPF_VINMODE_PROG
#define QPPF_DEF_VIUFORMAT		QPPF_VINFORMAT_PROGRESSIVE
#define QPPF_DEF_VIUCLKEDGE		QPPF_VINCLKEDGE_NEGATIVE
#define QPPF_DEF_VIUSTARTPEL	0
#define QPPF_DEF_VIUSTARTLINE	0
#define QPPF_DEF_VIUSYNCCODE1	0x9DB680AB
#define QPPF_DEF_VIUSYNCCODE2	0xB6F1F1B6

// Audio Input
#define QPPF_DEF_AIMSB		QPPF_AIUMSB_MSB
#define QPPF_DEF_AILRCLK	QPPF_AIULRCLK_LEFTLOW
#define QPPF_DEF_AIBCLK		QPPF_AIUBCLK_FALLING
#define QPPF_DEF_AII2S		QPPF_AIUI2S_PROGRAM
#define QPPF_DEF_AIRJ		QPPF_AIURJ_LEFT

// Video Output
#define QPPF_DEF_VOENABLE		QP_FALSE
#define QPPF_DEF_VOUMODE		QPPF_VOUMODE_PROGRESS
#define QPPF_DEF_VOUSTARTPEL	0
#define QPPF_DEF_VOUSTARTLINE	0x1D

// Audio Output
#define QPPF_DEF_AOENABLE		QP_FALSE
#define QPPF_DEF_AOMSB			QPPF_AOUMSB_MSB
#define QPPF_DEF_AOLRCLK		QPPF_AOULRCLK_LEFTLOW
#define QPPF_DEF_AOBCLK			QPPF_AOUBCLK_FALLING
#define QPPF_DEF_AOI2S			QPPF_AOUI2S_PROGRAM
#define QPPF_DEF_AORJ			QPPF_AOURJ_LEFT
#define QPPF_DEF_AOMODE			QPPF_AOUMODE_SLAVE

// Video Decoder
#define QPPF_DEF_VIDDEC_TYPE	QPPF_VIDDEC_DUMMY

// Video Encoder
#define QPPF_DEF_VIDENC_TYPE	QPPF_VIDENC_DUMMY
#define QPPF_DEF_VIDSTD			QPPF_VIDSTD_NTSC

// Tuner type
#define QPPF_DEF_TUNER_TYPE		QPPF_TUNER_DUMMY;

// TV Audio type
#define QPPF_DEF_TVAUDIO_TYPE	QPPF_TVAUDIO_DUMMY

// Audio Codec type
#define QPPF_DEF_AUDCODEC_TYPE	QPPF_AUDCODEC_DUMMY;

// Chip type
#define QPPF_DEF_CHIP_TYPE		QPPF_CHIPTYPE_BERINGER

// Use external host DMA functions QPX_Wrap_ReadDMA & QPX_Wrap_WriteDMA
//#define _QPCODEC_USE_EXTERNAL_HOST_DMA_FUNC

// Start the codec inside the thread
#define _QPCODEC_ASYNC_START 

// codec streaming callback in separate thread
//#define _QPCODEC_CALLBACK_IN_THREAD 

#endif //__PLATFORM_H__
