/**************************************************************************** 
 *  File:  MI2010.h 
 *  Description:  
 *                   
 *  Author:     Junyee Lee 
 * --------------------------------------------------------------------------
 *  Revision History: 
 *  09/16/2009  Initial revision 
 ****************************************************************************/
#ifndef MI2010_H
#define MI2010_H

#define AF_SLAVE_ID                          0x0C

#define QLUSBFW_UVC_FC3_START_VIDEO_YUY2		128
#define QLUSBFW_UVC_FC3_START_VIDEO_QYUV       64
#define QLUSBFW_UVC_FC3_START_VIDEO_MJPEG      32
#define QLUSBFW_UVC_FC3_START_VIDEO_H264       16
#define QLUSBFW_UVC_FC3_START_VIDEO_ERROR      00

#define QLUSBFW_FC3_QCAM_Mode                  1
#define QLUSBFW_FC3_UVC_Mode                   2

#define UVC_PARAM_CUR	    0   // Current
#define UVC_PARAM_DEF	    1   // Default
#define UVC_PARAM_RES   	2   // Resolution
#define UVC_PARAM_MIN	    3   // Min
#define UVC_PARAM_MAX	    4   // Mix
#define UVC_PARAM_SETTING	5   // Total


typedef long DWORD;
typedef unsigned long 	UDWORD;
typedef short	WORD;
typedef unsigned short	UWORD;
typedef unsigned char	BYTE;
typedef unsigned char	UBYTE;
#if !defined(__LINUX__)
typedef unsigned char	UCHAR;
typedef unsigned char	uint8_t;
typedef unsigned short uint16_t;
#endif

extern UDWORD VsyncAheadCnt;
extern UDWORD FC3TimerInterval;
extern UDWORD FC3_VstartOff;

extern UWORD SharpnessP[UVC_PARAM_SETTING];          // for sharpness , edge_enhance
extern UWORD GammaP[UVC_PARAM_SETTING];              // for gamma
extern UWORD SatuartionP[UVC_PARAM_SETTING];	       // for saturation
extern UWORD BrightnessP[UVC_PARAM_SETTING];	       // for brightness
extern UWORD ContrastP[UVC_PARAM_SETTING];           // for contrast
extern UWORD BackLightCpenP[UVC_PARAM_SETTING];      // for backlight compensation
extern UCHAR PowrLineFreqP[UVC_PARAM_SETTING];       // Power-Line Frequency Control (50/60Hz)
extern UWORD WBCpontP[UVC_PARAM_SETTING];            // While Balance Component Control for R/B gain
extern UCHAR AutoWBCpontP[UVC_PARAM_SETTING];        // for AWB disable/enable
extern UCHAR AutoExpoModeP[UVC_PARAM_SETTING];       // for AE mode
extern DWORD ExpoTimeAbsP[UVC_PARAM_SETTING];        // for AE exposure value, 1/10000 sec
extern UWORD WBTemperature[UVC_PARAM_SETTING];       // White Balance Temperature Control
extern UCHAR AutoWBTemperature[UVC_PARAM_SETTING];   // Auto White Balance Temperature Control
extern UCHAR FocusAuto[UVC_PARAM_SETTING];           // Auto Focus PU/CT Parameters
extern UWORD FocusP[UVC_PARAM_SETTING];              // (Relative) Focus PU/CT Parameters
extern UCHAR LowLightBoostp[UVC_PARAM_SETTING];      // Low light boost control

#if defined(__LINUX__)
void FC3LibInit(unsigned long ulI2CProvider, unsigned long ulQLCodec);
#else
void FC3LibInit(void);
#endif
void FC3_main(void);
void FC3_timer(void);


int FC3_boot_init(UCHAR iMode);
void start_video_via_codeclib(void);
void start_video(void);
void stop_video(void);

void start_still_capture(void);
void stop_still_capture(void);


void query_video_source(UCHAR *source);
// source :
//     0: sensor (Micron/Omnivision)
//     1: 861B
//     2: ITU656
void query_video_mode(UCHAR *scan_mode, UCHAR *polarity, UCHAR *frame_field_rate, UCHAR *clk_edge);
// scan_mode :
//     0: interlace
//     1: progressive
//     2: sensor(Micron)
//     3: sensor(Omnivision)
// polarity :
//     {VSYNC,HSYNC}: 861B
//     {frame_valid,line_valid}: sensor
// clk_edge :
//     0: positive
//     1: negative


void set_jpegtargetsize(UWORD iVal);
void set_bitrate(UWORD iVal);

void set_scaling_factor(UWORD iFactor);

void set_framerate(UWORD iVal);
void set_video_format(UCHAR iType);

void set_video_resolution(UWORD height, UWORD Width);
void get_video_resolution(UWORD *height, UWORD *Width);
void query_video_resolution(UWORD target_height, UWORD target_Width, UWORD *sensor_out_height, UWORD *sensor_out_Width);

void get_jpegsize(UWORD *iVal);
void get_bitrate(UWORD *iVal);
void get_framerate(UWORD *iVal);
void get_compression(UCHAR *iType);

void set_powerline_feq(BYTE *PowrLineFreqP);
void get_powerline_feq(BYTE *PowrLineFreqP);

void set_edgeenhnce(UWORD *SharpnessP);
void get_edgeenhnce(UWORD *SharpnessP);

void set_ae_enable(UCHAR *AutoExpoModeP);
void get_ae_enable(UCHAR *AutoExpoModeP);

void set_awb_enable(UCHAR *AutoWBTemperature);
void get_awb_enable(UCHAR *AutoWBTemperature);

void set_wbcp(DWORD *WBCpontP);
void get_wbcp(DWORD *WBCpontP);

void set_backlight_ctrl(UWORD *BackLightCpenP);
void get_backlight_ctrl(UWORD *BackLightCpenP);

void set_af_enable(UCHAR *FocusAuto);
void get_af_enable(UCHAR *FocusAuto);

void set_focus(UWORD *FocusP);
void get_focus(UWORD *FocusP);

void set_satuartion(UWORD *SatuartionP);
void get_satuartion(UWORD *SatuartionP);

void set_brightness(UWORD *BrightnessP);
void get_brightness(UWORD *BrightnessP);

void set_contrast(UWORD *ContrastP);
void get_contrast(UWORD *ContrastP);

void set_color_matrix(WORD *swHue, UWORD *uwSatu, BYTE *CCM);
void get_color_matrix(WORD *swHue, UWORD *uwSatu , BYTE *CCM);

void set_gamma(UWORD *GammaP);
void get_gamma(UWORD *GammaP);

void set_low_light_boost(UCHAR *LowLightBoostp);
void get_low_light_boost(UCHAR *LowLightBoostp);

void set_wb_temperature(UWORD *WBTemperature);
void get_wb_temperature(UWORD *WBTemperature);

void set_exporsure_time(DWORD  *ExpoTimeAbsP);
void get_exporsure_time(DWORD  *ExpoTimeAbsP);

void initial_camera(void);
void initial_color_parameter(void);

void set_jpeg(DWORD *Q_Table);

void cam_suspend(void);
void cam_resume(void);

void cam_vsync_process(void);

void XU1_call_back(UCHAR RequestType, UCHAR Request, UWORD Value, UWORD index, UWORD Length, UCHAR *USB_FIFO, UCHAR * bRequestErrorCode, UCHAR *FIFO_Length);
void XU2_call_back(UCHAR RequestType, UCHAR Request, UWORD Value, UWORD index, UWORD Length, UCHAR *USB_FIFO, UCHAR * bRequestErrorCode, UCHAR *FIFO_Length);


void AF_Init(int FrameSize, int MaxPos, int MinPos, int Step);
void set_AF_position(int position);
void set_AF_sensitivity(int iSensitivity);
void set_AF_forward(int step);
void set_AF_backward(int step);
void AF_Judgement(void);
void AF_Enable(int bEnable);

void get_firmware_info(UCHAR *info);
void get_firmware_rev(UCHAR *info);

void PU_call_back(UCHAR RequestType, UCHAR Request, UWORD Value, UWORD index, UWORD Length, UCHAR *USB_FIFO, UCHAR * bRequestErrorCode, UCHAR *FIFO_Length);
void CT_call_back(UCHAR RequestType, UCHAR Request, UWORD Value, UWORD index, UWORD Length, UCHAR *USB_FIFO, UCHAR * bRequestErrorCode, UCHAR *FIFO_Length);

#endif
