///////////////////////////////////////////////////////////////////////////////
//        
//
// Cobject.h
//
// Description: 
//
//	Header file which defines the base object.
// 
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#ifndef __COBJECT_H__
#define __COBJECT_H__


#ifdef _QP_WINKER
#include <wdm.h>
#endif //_QP_WINKER


#include "qptypes.h"
#include "qposmgr.h"


/////////////////////////////////////////////////////////////////////////////
//
//

// this file only defines the base objects, users of the CObject class need to 
// define their own objects.
//
#define OBJECT_BASE				0x00000000L			// objects defined in the 
													// base classes start here
#define OBJECT_GENERIC			OBJECT_BASE			// generic object
#define OBJECT_FIFO				(OBJECT_BASE + 1)	// fifo
#define OBJECT_OBJECTMGR		(OBJECT_BASE + 2)	// object manager
#define OBJECT_QUEUE			(OBJECT_BASE + 3)	// queue
#define OBJECT_STACK			(OBJECT_BASE + 4)	// stack

#define OBJECT_USER				0x0000FFFFL			// user defined objects

// the object attributes, crrently defined as either non-critical or critical.
//
#define OBJECT_DEFAULT			0x00000000L

#define OBJECT_CRITICAL_HEAVY	0x00000001L
#define OBJECT_CRITICAL_LIGHT	0x00000002L

#define OBJECT_CRITICAL			(OBJECT_CRITICAL_HEAVY | OBJECT_CRITICAL_LIGHT)


#ifdef __cplusplus
extern "C" {
#endif //__cplusplus

/////////////////////////////////////////////////////////////////////////////
//
//

// C defination, visible to both C and C++ code
//

#ifndef __COBJECT_FWD_DEFINED__
#define __COBJECT_FWD_DEFINED__
typedef struct CObject CObject;
#endif //!__COBJECT_FWD_DEFINED__

struct CObject
{
	TQP_BOOL (*Init)(CObject *This);
	TQP_BOOL (*Done)(CObject *This);

	CObject			*m_pParent;				// who is my parent?
	TQP_BOOL		m_fInitialized;			// am i initialized?
	TQP_UINT32		m_dwObjectAttributes;	// object attributes

	TQP_OSMSemID	m_semCriticalSection;
#ifdef _QP_WINKER
	KSPIN_LOCK		m_spinlock;
	KIRQL			m_irql;
#endif //_QP_WINKER
};


/////////////////////////////////////////////////////////////////////////////
//
//

#ifdef _QP_WINKER
#define INIT_CRITICAL(This) \
	(This)->m_semCriticalSection = QP_NULL;\
	if ((This)->m_dwObjectAttributes & OBJECT_CRITICAL_HEAVY) \
	{\
		KeInitializeSpinLock(&(This)->m_spinlock); \
	}\
	else if ((This)->m_dwObjectAttributes & OBJECT_CRITICAL_LIGHT) \
	{\
		QPOSMCreateSem(&(This)->m_semCriticalSection, 1, 1); \
	}\
	else \
		;

#define ENTER_CRITICAL(This) \
	if ((This)->m_dwObjectAttributes & OBJECT_CRITICAL_HEAVY) \
	{\
		KeAcquireSpinLock(&(This)->m_spinlock, &(This)->m_irql); \
	}\
	else if ((This)->m_dwObjectAttributes & OBJECT_CRITICAL_LIGHT) \
	{\
		if ((This)->m_semCriticalSection) \
		{\
			QPOSMWaitSem((This)->m_semCriticalSection, -1);\
		}\
	}\
	else \
	{\
		;\
	}

#define LEAVE_CRITICAL(This) \
	if ((This)->m_dwObjectAttributes & OBJECT_CRITICAL_HEAVY) \
	{\
		KeReleaseSpinLock(&(This)->m_spinlock, (This)->m_irql); \
	}\
	else if ((This)->m_dwObjectAttributes & OBJECT_CRITICAL_LIGHT) \
	{\
		if ((This)->m_semCriticalSection) \
		{\
			QPOSMSignalSem((This)->m_semCriticalSection);\
		}\
	}\
	else \
	{\
		;\
	}

#define DONE_CRITICAL(This) \
	if ((This)->m_dwObjectAttributes & OBJECT_CRITICAL_HEAVY) \
	{\
		(This)->m_spinlock = QP_NULL;\
	}\
	else if ((This)->m_dwObjectAttributes & OBJECT_CRITICAL_LIGHT) \
	{\
		if ((This)->m_semCriticalSection) \
		{\
			QPOSMDeleteSem((This)->m_semCriticalSection);\
			(This)->m_semCriticalSection = QP_NULL;\
		}\
	}\
	else \
	{\
		;\
	}
#else //!_QP_WINKER
#define INIT_CRITICAL(This) \
	(This)->m_semCriticalSection = QP_NULL;\
	if ((This)->m_dwObjectAttributes & OBJECT_CRITICAL_HEAVY) \
	{\
		QPOSMCreateSem(&(This)->m_semCriticalSection, 1, 1); \
	}\
	else if ((This)->m_dwObjectAttributes & OBJECT_CRITICAL_LIGHT) \
	{\
		QPOSMCreateSem(&(This)->m_semCriticalSection, 1, 1); \
	}\
	else \
	{\
		;\
	}

#define ENTER_CRITICAL(This) \
	if ((This)->m_dwObjectAttributes & OBJECT_CRITICAL_HEAVY) \
	{\
		if ((This)->m_semCriticalSection) \
		{\
			QPOSMWaitSem((This)->m_semCriticalSection, -1);\
		}\
	}\
	else if ((This)->m_dwObjectAttributes & OBJECT_CRITICAL_LIGHT) \
	{\
		if ((This)->m_semCriticalSection) \
		{\
			QPOSMWaitSem((This)->m_semCriticalSection, -1);\
		}\
	}\
	else \
	{\
		;\
	}

#define LEAVE_CRITICAL(This) \
	if ((This)->m_dwObjectAttributes & OBJECT_CRITICAL_HEAVY) \
	{\
		if ((This)->m_semCriticalSection) \
		{\
			QPOSMSignalSem((This)->m_semCriticalSection);\
		}\
	}\
	else if ((This)->m_dwObjectAttributes & OBJECT_CRITICAL_LIGHT) \
	{\
		if ((This)->m_semCriticalSection) \
		{\
			QPOSMSignalSem((This)->m_semCriticalSection);\
		}\
	}\
	else \
	{\
		;\
	}

#define DONE_CRITICAL(This) \
	if ((This)->m_dwObjectAttributes & OBJECT_CRITICAL_HEAVY) \
	{\
		if ((This)->m_semCriticalSection) \
		{\
			QPOSMDeleteSem((This)->m_semCriticalSection);\
			(This)->m_semCriticalSection = QP_NULL;\
		}\
	}\
	else if ((This)->m_dwObjectAttributes & OBJECT_CRITICAL_LIGHT) \
	{\
		if ((This)->m_semCriticalSection) \
		{\
			QPOSMDeleteSem((This)->m_semCriticalSection);\
			(This)->m_semCriticalSection = QP_NULL;\
		}\
	}\
	else \
	{\
		;\
	}
#endif //_QP_WINKER



/////////////////////////////////////////////////////////////////////////////
//
//

CObject * CObject_Constructor(CObject *pObject,
							  CObject *pParent, 
							  TQP_UINT32 dwAttributes
							  );
void CObject_Destructor(CObject *This);
#define CObject_Init(This) \
    (This)->Init(This)
#define CObject_Done(This) \
    (This)->Done(This)
CObject * CObject_GetParent(CObject *This);
TQP_BOOL CObject_IsInitialized(CObject *This);
void CObject_enterCritical(CObject *This);
void CObject_leaveCritical(CObject *This);

#ifdef __cplusplus
}
#endif //__cplusplus


/////////////////////////////////////////////////////////////////////////////
//
//

// C++ defination, visible only to C++ code
//

#ifdef __cplusplus

/////////////////////////////////////////////////////////////////////////////
//
//

#ifdef _QP_WINKER
#define INIT_CRITICAL_CPP \
	if (m_dwObjectAttributes & OBJECT_CRITICAL_HEAVY) \
	{\
		KeInitializeSpinLock(&m_spinlock); \
	}\
	else if (m_dwObjectAttributes & OBJECT_CRITICAL_LIGHT) \
	{\
		KeInitializeMutex(&m_mutex, 1); \
	}\
	else \
	{\
		;\
	}

#define ENTER_CRITICAL_CPP \
	if (m_dwObjectAttributes & OBJECT_CRITICAL_HEAVY) \
	{\
		KeAcquireSpinLock(&m_spinlock, &m_irql); \
	}\
	else if (m_dwObjectAttributes & OBJECT_CRITICAL_LIGHT) \
	{\
		KeWaitForSingleObject(&m_mutex, Executive, KernelMode, FALSE, NULL);\
	}\
	else \
	{\
		;\
	}

#define LEAVE_CRITICAL_CPP \
	if (m_dwObjectAttributes & OBJECT_CRITICAL_HEAVY) \
	{\
		KeReleaseSpinLock(&m_spinlock, m_irql); \
	}\
	else if (m_dwObjectAttributes & OBJECT_CRITICAL_LIGHT) \
	{\
		KeReleaseMutex(&m_mutex, FALSE);\
	}\
	else \
	{\
		;\
	}

#define DONE_CRITICAL_CPP \
	if (m_dwObjectAttributes & OBJECT_CRITICAL_HEAVY) \
	{\
		m_spinlock = QP_NULL; \
	}\
	else if (m_dwObjectAttributes & OBJECT_CRITICAL_LIGHT) \
	{\
	}\
	else \
	{\
		;\
	}
#else //!_QP_WINKER
#define INIT_CRITICAL_CPP \
	m_semCriticalSection = QP_NULL;\
	if (m_dwObjectAttributes & OBJECT_CRITICAL_HEAVY) \
	{\
		QPOSMCreateSem(&m_semCriticalSection, 1, 1); \
	}\
	else if (m_dwObjectAttributes & OBJECT_CRITICAL_LIGHT) \
	{\
		QPOSMCreateSem(&m_semCriticalSection, 1, 1); \
	}\
	else \
	{\
		;\
	}\

#define ENTER_CRITICAL_CPP \
	if (m_dwObjectAttributes & OBJECT_CRITICAL_HEAVY) \
	{\
		if (m_semCriticalSection) \
		{\
			QPOSMWaitSem(m_semCriticalSection, -1);\
		}\
	}\
	else if (m_dwObjectAttributes & OBJECT_CRITICAL_LIGHT) \
	{\
		if (m_semCriticalSection) \
		{\
			QPOSMWaitSem(m_semCriticalSection, -1);\
		}\
	}\
	else \
	{\
		;\
	}

#define LEAVE_CRITICAL_CPP \
	if (m_dwObjectAttributes & OBJECT_CRITICAL_HEAVY) \
	{\
		if (m_semCriticalSection) \
		{\
			QPOSMSignalSem(m_semCriticalSection);\
		}\
	}\
	else if (m_dwObjectAttributes & OBJECT_CRITICAL_LIGHT) \
	{\
		if (m_semCriticalSection) \
		{\
			QPOSMSignalSem(m_semCriticalSection);\
		}\
	}\
	else \
	{\
		;\
	}

#define DONE_CRITICAL_CPP \
	if (m_dwObjectAttributes & OBJECT_CRITICAL_HEAVY) \
	{\
		if (m_semCriticalSection) \
		{\
			QPOSMDeleteSem(m_semCriticalSection);\
			m_semCriticalSection = QP_NULL; \
		}\
	}\
	else if (m_dwObjectAttributes & OBJECT_CRITICAL_LIGHT) \
	{\
		if (m_semCriticalSection) \
		{\
			QPOSMDeleteSem(m_semCriticalSection);\
			m_semCriticalSection = QP_NULL; \
		}\
	}\
	else \
	{\
		;\
	}
#endif //!_QP_WINKER


/////////////////////////////////////////////////////////////////////////////
//
//


class CppObject
{
public:
	CppObject() 
	{
		m_pParent = QP_NULL; 
		m_fInitialized = QP_FALSE; 
		m_dwWhoAmI = OBJECT_GENERIC; 
		m_dwObjectAttributes = OBJECT_DEFAULT;
	}
	CppObject(CppObject *pParent, 
			  TQP_UINT32 dwWhoAmI,
			  TQP_UINT32 dwAttributes
			  )
	{
		m_pParent = pParent;
		m_fInitialized = QP_FALSE;
		m_dwWhoAmI = dwWhoAmI;
		m_dwObjectAttributes = dwAttributes;
		INIT_CRITICAL_CPP
	}
	virtual ~CppObject() {DONE_CRITICAL_CPP}
	virtual TQP_BOOL Init() {m_fInitialized = QP_TRUE; return (QP_TRUE);}
	virtual TQP_BOOL Done() {m_fInitialized = QP_FALSE; return (QP_TRUE);}
	CppObject * GetParent() {return (m_pParent);}
	TQP_BOOL IsInitialized() {return (m_fInitialized);}
	TQP_UINT32 WhoAmI() {return (m_dwWhoAmI);}

    void enterCritical() {ENTER_CRITICAL_CPP}
    void leaveCritical() {LEAVE_CRITICAL_CPP}

protected:

	CppObject		*m_pParent;				// who is my parent?
	TQP_BOOL		m_fInitialized;			// am i initialized?
	TQP_UINT32		m_dwWhoAmI;				// who am i?
	TQP_UINT32		m_dwObjectAttributes;	// object attributes

	TQP_OSMSemID	m_semCriticalSection;
#ifdef _QP_WINKER
	KSPIN_LOCK		m_spinlock;
	KIRQL			m_irql;
	KMUTEX			m_mutex;
#endif //_QP_WINKER
};

#endif //__cplusplus

#endif //__COBJECT_H__
