///////////////////////////////////////////////////////////////////////////////
//        
//
// Cobjectmgr.h
//
// Description: 
//
//	the object that manages multiple objects.
// 
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#ifndef __COBJECTMGR_H__
#define __COBJECTMGR_H__

#include "qptypes.h"
#include "cobject.h"


typedef EQP_Errors (*OBJECTMGR_CALLBACK)(TQP_PVOID pParam);


/////////////////////////////////////////////////////////////////////////////
//
//

// C defination, visible to both C and C++ code
//

#ifdef __cplusplus
extern "C" {
#endif //__cplusplus


#ifndef __COBJECTMGR_FWD_DEFINED__
#define __COBJECTMGR_FWD_DEFINED__
typedef struct CObjectMgr CObjectMgr;
typedef struct OBJECT_ENTRY OBJECT_ENTRY;
#endif //!__COBJECTMGR_FWD_DEFINED__


/////////////////////////////////////////////////////////////////////////////
//
//

struct OBJECT_ENTRY
{
	TQP_OBJECT_HANDLE	hObject;	// handle
	TQP_PVOID			*pObject;	// pointer to object
	OBJECT_ENTRY		*pNext;		// next object entry
};



struct CObjectMgr
{
	// CObject
	//
	CObject				m_Object;

	// CObjectMgr
	//
	TQP_OBJECT_HANDLE	m_hCurObject;
	OBJECT_ENTRY		*m_pHead;			// head of object list.
	TQP_UINT32			m_dwObjectNb;		// number of objects in the list.
	OBJECTMGR_CALLBACK	m_pFuncCallback;	// callback
};


CObjectMgr * CObjectMgr_Constructor(CObjectMgr *pObjectMgr,
									CObject *pParent, 
									TQP_UINT32 dwAttributes,
									OBJECTMGR_CALLBACK pFuncCallback
									);
void CObjectMgr_Destructor(CObjectMgr *This);
TQP_OBJECT_HANDLE CObjectMgr_AddObject(CObjectMgr *This,
									   TQP_PVOID pObject
									   );
TQP_PVOID CObjectMgr_RemoveObject(CObjectMgr *This,
								  TQP_OBJECT_HANDLE hObject
								  );
TQP_PVOID CObjectMgr_GetObjectByHandle(CObjectMgr *This,
									   TQP_OBJECT_HANDLE hObject
									   );
TQP_OBJECT_HANDLE CObjectMgr_GetHandleByObject(CObjectMgr *This,
											   TQP_PVOID pObject
											   );
TQP_PVOID CObjectMgr_GetObjectByIndex(CObjectMgr *This,
									  TQP_UINT32 dwIndex
									  );
TQP_OBJECT_HANDLE CObjectMgr_SetCurrentObject(CObjectMgr *This,
											  TQP_OBJECT_HANDLE hObject
											  );
TQP_OBJECT_HANDLE CObjectMgr_GetCurrentObject(CObjectMgr *This);
TQP_UINT32 CObjectMgr_GetNumberOfObjects(CObjectMgr *This);


#ifdef __cplusplus
}
#endif //__cplusplus



/////////////////////////////////////////////////////////////////////////////
//
//

// C++ defination, visible only to C++ code
//

#ifdef __cplusplus

/////////////////////////////////////////////////////////////////////////////
//
//

template <class T> class OBJECT_ENTRY_CPP
{
public:
	OBJECT_ENTRY_CPP() 
		: pNext(NULL)
		, hObject(0)
		, pObject(QP_NULL) 
	{
	}
	virtual ~OBJECT_ENTRY_CPP() {}

	TQP_OBJECT_HANDLE	hObject;	// handle
	T					*pObject;	// pointer to object
	OBJECT_ENTRY_CPP	*pNext;		// next object entry
};

/////////////////////////////////////////////////////////////////////////////
//
//

template <class T> class CppObjectMgr : public CppObject
{
public:
	CppObjectMgr(CppObject *pParent, 
				 OBJECTMGR_CALLBACK pFuncCallback
				 );
	CppObjectMgr(CppObject *pParent, 
				 TQP_UINT32 dwWhoAmI, 
				 TQP_UINT32 dwAttributes, 
				 OBJECTMGR_CALLBACK pFuncCallback
				 );
	virtual ~CppObjectMgr();

	TQP_OBJECT_HANDLE AddObject(T *pObject);
	T * RemoveObjectByHandle(TQP_OBJECT_HANDLE hObject);
	T * RemoveObject(T *pObject);

	T * GetObjectByHandle(TQP_OBJECT_HANDLE hObject);
	TQP_OBJECT_HANDLE GetHandleByObject(T *pObject);

	TQP_OBJECT_HANDLE SetCurrentObject(TQP_OBJECT_HANDLE hObject);
	TQP_OBJECT_HANDLE GetCurrentObject() {return (m_hCurObject);}
	TQP_UINT32 GetNumberOfObjects() {return (m_dwObjectNb);}

protected:
	void RemoveObjects();
	T * GetObjectByIndex(TQP_UINT32 dwIndex);

	TQP_OBJECT_HANDLE	m_hCurObject;
	OBJECT_ENTRY_CPP<T>	*m_pHead;			// head of object list.
	TQP_UINT32			m_dwObjectNb;		// number of objects in the list.
	OBJECTMGR_CALLBACK	m_pFuncCallback;	// callback
};


/////////////////////////////////////////////////////////////////////////////
//
//

template <class T>
CppObjectMgr<T>::CppObjectMgr(CppObject *pParent, 
							  OBJECTMGR_CALLBACK pFuncCallback
							  )
	: CppObject(pParent, 
				OBJECT_OBJECTMGR, 
				OBJECT_CRITICAL_LIGHT
				)
	, m_hCurObject(0)
	, m_pHead((OBJECT_ENTRY_CPP<T> *)QP_NULL)
	, m_dwObjectNb(0)
	, m_pFuncCallback(pFuncCallback)
{
}



template <class T>
CppObjectMgr<T>::CppObjectMgr(CppObject *pParent, 
							  TQP_UINT32 dwWhoAmI, 
							  TQP_UINT32 dwAttributes, 
							  OBJECTMGR_CALLBACK pFuncCallback
							  )
	: CppObject(pParent, 
				dwWhoAmI, 
				dwAttributes
				)
	, m_hCurObject(0)
	, m_pHead((OBJECT_ENTRY_CPP<T> *)QP_NULL)
	, m_dwObjectNb(0)
	, m_pFuncCallback(pFuncCallback)
{
}



template <class T>
CppObjectMgr<T>::~CppObjectMgr()
{
	// release all the objects
	//
	RemoveObjects();
}



template <class T>
TQP_OBJECT_HANDLE CppObjectMgr<T>::AddObject(T *pObject)
{
	TQP_OBJECT_HANDLE	hRet = 0;
	OBJECT_ENTRY_CPP<T>	*pNewEntry = new OBJECT_ENTRY_CPP<T>;

	ENTER_CRITICAL_CPP

	if (QP_NULL == m_pHead)
	{
		m_pHead = pNewEntry;
		m_pHead->hObject = 1;
		m_pHead->pObject = pObject;
		m_pHead->pNext = (OBJECT_ENTRY_CPP<T> *)QP_NULL;
		m_dwObjectNb++;
		hRet = m_pHead->hObject;
	}
	else
	{		
		OBJECT_ENTRY_CPP<T>	*ptemp_entry;
		
		if (m_pHead->hObject != 1)
		{
			ptemp_entry = pNewEntry;
			ptemp_entry->hObject = 1;
			ptemp_entry->pObject = pObject;
			ptemp_entry->pNext = m_pHead;
			m_pHead = ptemp_entry;
			m_dwObjectNb++;
			hRet = m_pHead->hObject;
		}
		else
		{
			OBJECT_ENTRY_CPP<T>	*pthis_entry = m_pHead;
			OBJECT_ENTRY_CPP<T>	*pnext_entry = m_pHead->pNext;
            
			while (pnext_entry && 
				   (pnext_entry->hObject == (pthis_entry->hObject + 1))
				   )
			{
				pthis_entry = pnext_entry;
				pnext_entry = pnext_entry->pNext;
			}
			ptemp_entry = pNewEntry;
			ptemp_entry->hObject = pthis_entry->hObject + 1;
			ptemp_entry->pObject = pObject;
			ptemp_entry->pNext = pnext_entry;
			pthis_entry->pNext = ptemp_entry;
			m_dwObjectNb++;
			hRet = ptemp_entry->hObject;
		}
	}

	LEAVE_CRITICAL_CPP

	return (hRet);
}



template <class T>
T * CppObjectMgr<T>::RemoveObjectByHandle(TQP_OBJECT_HANDLE hObject)
{
	OBJECT_ENTRY_CPP<T>	*pthis_entry;
	OBJECT_ENTRY_CPP<T>	*pprev_entry;
	OBJECT_ENTRY_CPP<T>	*pRemove = QP_NULL;
	T					*pObject = QP_NULL;

	ENTER_CRITICAL_CPP

	pthis_entry = m_pHead;
	if (pthis_entry)
	{
		if (pthis_entry->hObject == hObject)
		{
			pObject = pthis_entry->pObject;
			m_pHead = pthis_entry->pNext;
			pRemove = pthis_entry;
			m_dwObjectNb--;
		}
		else
		{
			do	{
				pprev_entry = pthis_entry;
				pthis_entry = pthis_entry->pNext;
				if (pthis_entry && (pthis_entry->hObject == hObject))
				{
					pprev_entry->pNext = pthis_entry->pNext;
					pObject = pthis_entry->pObject;
					pRemove = pthis_entry;
					m_dwObjectNb--;
					break;
				}		
			} while (pthis_entry);
		}
	}

	LEAVE_CRITICAL_CPP

	if (pRemove)
		delete pRemove;

	return (pObject);
}



template <class T>
T * CppObjectMgr<T>::RemoveObject(T *pObject)
{
	OBJECT_ENTRY_CPP<T>	*pthis_entry;
	OBJECT_ENTRY_CPP<T>	*pprev_entry;
	OBJECT_ENTRY_CPP<T>	*pRemove = QP_NULL;
	T					*pObjectRet = QP_NULL;

	ENTER_CRITICAL_CPP

	pthis_entry = m_pHead;
	if (pthis_entry)
	{
		if (pthis_entry->pObject == pObject)
		{
			pObjectRet = pthis_entry->pObject;
			m_pHead = pthis_entry->pNext;
			pRemove = pthis_entry;
			m_dwObjectNb--;
		}
		else
		{
			do	{
				pprev_entry = pthis_entry;
				pthis_entry = pthis_entry->pNext;
				if (pthis_entry && (pthis_entry->pObject == pObject))
				{
					pprev_entry->pNext = pthis_entry->pNext;
					pObjectRet = pthis_entry->pObject;
					pRemove = pthis_entry;
					m_dwObjectNb--;
					break;
				}		
			} while (pthis_entry);
		}
	}

	LEAVE_CRITICAL_CPP

	if (pRemove)
		delete pRemove;

	return (pObjectRet);
}



template <class T>
T * CppObjectMgr<T>::GetObjectByHandle(TQP_OBJECT_HANDLE hObject)
{
	OBJECT_ENTRY_CPP<T>	*pthis_entry;
	T					*pObject = QP_NULL;
	TQP_UINT32			i;

	ENTER_CRITICAL_CPP

	pthis_entry = m_pHead;	

	for (i = 0; i < m_dwObjectNb; i++)
	{
		if (pthis_entry->hObject == hObject)
		{
			pObject = pthis_entry->pObject;
		}
		pthis_entry = pthis_entry->pNext;
	}

	LEAVE_CRITICAL_CPP

	return (pObject);
}



template <class T>
TQP_OBJECT_HANDLE CppObjectMgr<T>::GetHandleByObject(T *pObject)
{
	OBJECT_ENTRY_CPP<T>	*pthis_entry;
	TQP_OBJECT_HANDLE	hObject = TQP_NULL_HANDLE;
	TQP_UINT32			i;

	ENTER_CRITICAL_CPP

	pthis_entry = m_pHead;	

	for (i = 0; i < m_dwObjectNb; i++)
	{
		if (pthis_entry->pObject == pObject)
		{
			hObject = pthis_entry->hObject;
			break;
		}
		pthis_entry = pthis_entry->pNext;
	}

	LEAVE_CRITICAL_CPP

	return (hObject);
}



template <class T>
TQP_OBJECT_HANDLE CppObjectMgr<T>::SetCurrentObject(TQP_OBJECT_HANDLE hObject)
{
	TQP_OBJECT_HANDLE	hold_object = m_hCurObject;

	m_hCurObject = hObject;
	return (hold_object);
}



template <class T>
void CppObjectMgr<T>::RemoveObjects()
{
	OBJECT_ENTRY_CPP<T>	*pthis_entry;
	OBJECT_ENTRY_CPP<T>	*ptemp_entry;

	pthis_entry = m_pHead;	

	while (pthis_entry)
	{
		ptemp_entry = pthis_entry;
		pthis_entry = pthis_entry->pNext;
		if (m_pFuncCallback && (ptemp_entry->pObject))
			m_pFuncCallback((PVOID)ptemp_entry->pObject);
		delete ptemp_entry;
	}
	m_dwObjectNb = 0;
	m_pHead = QP_NULL;
}



template <class T>
T * CppObjectMgr<T>::GetObjectByIndex(TQP_UINT32 dwIndex)
{
	OBJECT_ENTRY_CPP<T>	*pthis_entry;
	T					*pObject = NULL;
	TQP_UINT32			i;

	ENTER_CRITICAL_CPP

	pthis_entry = m_pHead;	

	for (i = 0; i < m_dwObjectNb; i++)
	{
		if (i == dwIndex)
		{
			pObject = pthis_entry->pObject;
			break;
		}
		pthis_entry = pthis_entry->pNext;
	}

	LEAVE_CRITICAL_CPP

	return (pObject);
}

#endif //__cplusplus

#endif //__CppObjectMGR_H__


