///////////////////////////////////////////////////////////////////////////////
//        
//
//		Flash Configuration API
//
// History:
//	Apr 26, 2010	(EC) Created
//
// Authors: (EC) Enrico Cadorin
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __FLASHCONFAPI__
#define __FLASHCONFAPI__

#include "qptypes.h"

#include "qldev_interface.h"

//
// Returns FALSE if the devices could not be opened
//
TQP_BOOL FlashConfAPIGetDeviceList (PL_DEV_INFO ** infoPP, int * devsNumP);

// Returns FALSE if the device is not available
TQP_BOOL FlashConfAPISelectDevice (int id);

// Returns FALSE if the devices cannot be closed
TQP_BOOL FlashConfAPICloseDevices (void);

//
// Blocks handling calls.
// A block is identified by a tag (4 characters packed ina 32-bit variable).
// A list of existing tags is obtained with FlashConfAPIGetBlocksList, the
// return value is an ID to be passed back when the tag array is not being used
// anymore (FlashConfAPIFreeBlocksList).
// A whole block can be deleted given its tag (FlashConfAPIDeleteBlock).
// A new empty block can be created with FlashConfAPICreateBlock.
//
void * FlashConfAPIGetTagsArray (PTQP_UINT32 * tagsArrayP, int * tagsNumP);
TQP_BOOL FlashConfAPIFreeTagsArray (void * listID);
TQP_BOOL FlashConfAPIDeleteBlock (TQP_UINT32 tag);
TQP_BOOL FlashConfAPICreateBlock (TQP_UINT32 tag);

//
// Key handling calls
// A list of key names within a block is obtained using FlashConfAPIGetKeysList,
// the list must be freed when not needed anymore by calling FlashConfAPIFreeKeysList
// with the ID returned by FlashConfAPIGetKeysList.
// Data for a given key is accessed by passing in the tag and the key name;
// key data must be freed when not used anymore
//
void * FlashConfAPIGetKeysList (TQP_UINT32 tag, char *** keysArrayPP, int * keysNumP);
TQP_BOOL FlashConfAPIFreeKeysList (void * listID);

void * FlashConfAPIGetKeyData (TQP_UINT32 tag, char * keyP, void ** dataPP, PTQP_UINT32 dataSizeP);
TQP_BOOL FlashConfAPIFreeKeyData (void * itemID);
TQP_BOOL FlashConfAPISetKeyData (TQP_UINT32 tag, char * keyP, void * dataP, TQP_UINT32 dataSize);
TQP_BOOL FlashConfAPIDeleteKey (TQP_UINT32 tag, char * keyP);




#endif //__FLASHCONFAPI__

