///////////////////////////////////////////////////////////////////////////////
//        
//
// PL331DevCntl.h
//
// Description: 
//
//  This is the definations of the PL331 codec chip control
// 
// History:
//	Oct 26, 2009	(dt) created
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __PL331DEVCNTL_H__
#define __PL331DEVCNTL_H__

#include "qpdevcntl.h"

// Property set for PL331Properties
//

// {B67E18BE-920A-4d0f-8886-01249BCFD165}
static const TQP_GUID PROPSETID_PL331_CONTROL = 
{ 0xb67e18be, 0x920a, 0x4d0f, { 0x88, 0x86, 0x1, 0x24, 0x9b, 0xcf, 0xd1, 0x65 } };

typedef enum _PROPERTY_PL331_CONTROL
{
	// Encoder
	//
	PL331_PROP_STREAM_CONTROL			= 0,		// Get & Set
	PL331_PROP_MODE_CONTROL				= 1,		// Get & Set
	PL331_PROP_DATA_XFER_CONTROL		= 2,		// Get & Set
	PL331_PROP_RATE_CONTROL				= 3,		// Get & Set
	PL331_PROP_GOP_LOOP_FILTER			= 6,		// Get & Set
	PL331_PROP_OUT_PICTURE_RESOLUTION	= 7,		// Get & Set
	PL331_PROP_OUT_FRAME_RATE			= 8,		// Get & Set
	PL331_PROP_AUDIO_CONTROL_PARAM		= 9,		// Get & Set
	PL331_PROP_AUDIO_CONTROL_EX			= 10,		// Get & Set
	PL331_PROP_ENC_STOP_MODE			= 11,		// Get & Set
	PL331_PROP_ENC_INPUT_OUTPUT_CONTROL	= 12,		// Get & Set
	PL331_PROP_INDEX_CAP_FREQ			= 13,		// Get & Set
	PL331_PROP_MP4_VID_NUM_BLOCK		= 14,		// Get & Set
	PL331_PROP_AUDIO_INPUT_VOLUME		= 15,		// Get & Set
	PL331_PROP_ENC_VID_PADDING			= 16,		// Get & Set
	PL331_PROP_ENC_FREEZE_VIDEO			= 17,		// Get & Set
	PL331_PROP_ENC_STILL_VIDEO_INPUT	= 18,		// Get & Set
	PL331_PROP_INSERT_I_FRAME			= 19,		// Set
	PL331_PROP_ENC_GET_A_FRAME			= 20,		// Set
	PL331_PROP_ENC_ALL_SETTINGS			= 29,		// Get & Set


	// Decoder
	//
	PL331_PROP_DEC_STREAM_TYPE			= 30,		// Get & Set
	PL331_PROP_DEC_AUDIO_CONTROL		= 31,		// Get & Set
	PL331_PROP_DEC_AO_CONTROL			= 32,		// Get & Set
	PL331_PROP_DEC_STOP_MODE			= 33,		// Get & Set
	PL331_PROP_DEC_INPUT_OUTPUT_CONTROL	= 34,		// Get & Set
	PL331_PROP_DEC_AUDIO_OUTPUT_VOLUME	= 35,		// Get & Set
	PL331_PROP_DEC_TRICK_MODE			= 36,		// Get & Set
	PL331_PROP_DEC_DATA_XFER_CONTROL	= 37,		// Get & Set
	PL331_PROP_DEC_ALL_SETTINGS			= 49,		// Get & Set


	// Misc
	//
	PL331_PROP_ENCODER_STATUS			= 50,		// Get
	PL331_PROP_DECODER_STATUS			= 51,		// Get
	PL331_PROP_656_OUT_CNTL				= 52,		// Get & Set
	PL331_PROP_I2S_OUT_CNTL				= 53,		// Get & Set
	PL331_PROP_POWER_STATE				= 54,		// Get & Set
	PL331_PROP_ERROR_RECOVERY			= 55,		// Get & Set

	// GPIO
	//
	PL331_GPIO_BIT_DIRECTION			= 60,		// Get & Set
	PL331_GPIO_BIT_VALUE				= 61,		// Get & Set

	// Codec
	//
	PL331_PROP_SYS_FUNCTION				= 70,		// Get & Set
	PL331_PROP_SYS_LINK					= 71,		// Get & Set
	PL331_PROP_SYS_CONFIG				= 72,		// Get & Set

	PL331_PROP_END						= QPCODEC_PROP_SYS_CONFIG

} PROPERTY_PL331_CONTROL;



#pragma pack(push, 1)


// Encoder
//

// PL331_PROP_STREAM_CONTROL
//
typedef struct _PL331_STREAM_CONTROL
{
	TQP_UINT32	stream_type;
	TQP_UINT32	stream_data;
	TQP_UINT32	profile;
	TQP_UINT32	level;
	TQP_UINT32	spsr_freq;
	TQP_UINT32	enc_mode;
	TQP_UINT32	v_mode;
	TQP_UINT32	cabac_init_idc;
} PL331_STREAM_CONTROL, *PPL331_STREAM_CONTROL;


// stream type
#define PL331_STREAM_TYPE_PROGRAM		0
#define PL331_STREAM_TYPE_TRANSPORT		1
#define PL331_STREAM_TYPE_MPEG4			2
#define PL331_STREAM_TYPE_ELEMENTARY	3
#define PL331_STREAM_TYPE_RAW			4
#define PL331_STREAM_TYPE_JPEG			6


// stream data - bitmap
#define PL331_STREAM_DATA_VIDEO			1
#define PL331_STREAM_DATA_AUDIO			2
#define PL331_STREAM_DATA_VBI			4
#define PL331_STREAM_DATA_USER			8

// PL331_PROP_MODE_CONTROL
//
typedef struct _PL331_MODE_CONTROL
{
	TQP_UINT32	slice_bit_limit;		// in unit of 1000 bits
	TQP_UINT32	slice_cnt_per_frame;
	TQP_UINT32	gpio_pin_no_trig;
	TQP_UINT32	capMode;
	TQP_UINT32	trigMode;
} PL331_MODE_CONTROL, *PPL331_MODE_CONTROL;


// PL331_PROP_DATA_XFER_CONTROL
// 
typedef struct _PL331_DATA_XFER_CONTROL
{
	TQP_UINT32	block_size;			// in unit of k bytes
	TQP_UINT32	slice_per_group;
	TQP_UINT32	xfer_mode;
	TQP_UINT32	ld_mode;
} PL331_DATA_XFER_CONTROL, *PPL331_DATA_XFER_CONTROL;

// xfer_mode
#define PL331_XFER_MODE_FRAME			0
#define PL331_XFER_MODE_BLOCK			1

// ld mode
#define PL331_LD_MODE_NORMAL			0
#define PL331_LD_MODE_LOW				1


// PL331_PROP_RATE_CONTROL
//
typedef struct _PL331_RATE_CONTROL
{
	// RATE_CONTROL
	//
	TQP_UINT32	avg_bit_rate;
	TQP_UINT32	qp_update_freq;
	TQP_UINT32	IDR;
	TQP_UINT32	GDR;
	TQP_UINT32	fixed;
	TQP_UINT32	vbr;

	// VBR_BIT_RATE
	//
	TQP_UINT32	min_bit_rate;
	TQP_UINT32	max_bit_rate;

	// FIXED_QP
	//
	TQP_UINT32	I_slice_qp;			// 0~51 inclusive
	TQP_UINT32	P_slice_qp;			// 0~51 inclusive
	TQP_UINT32	B_slice_qp;			// 0~51 inclusive

} PL331_RATE_CONTROL, *PPL331_RATE_CONTROL;



// PL331_PROP_GOP_LOOP_FILTER
//
typedef struct _PL331_GOP_LOOP_FILTER
{
	TQP_UINT32	intra_period;
	TQP_UINT32	lp_filter;
	TQP_UINT32	dis_lf_idc;
	TQP_INT32	alpha_offset;
	TQP_INT32	beta_offset;
	TQP_UINT32	B_num;
	TQP_UINT32	ref_n;
	TQP_UINT32	t8x8;
	TQP_UINT32	ref_p;
} PL331_GOP_LOOP_FILTER, *PPL331_GOP_LOOP_FILTER;


// PL331_PROP_OUT_PICTURE_RESOLUTION
//
typedef struct _PL331_PICTURE_RESOLUTION
{
	TQP_UINT32	pic_width;
	TQP_UINT32	pic_height;
} PL331_PICTURE_RESOLUTION, *PPL331_PICTURE_RESOLUTION;


// PL331_PROP_OUT_FRAME_RATE
//
typedef struct _PL331_OUT_FRAME_RATE
{
	TQP_UINT32	fps;
} PL331_OUT_FRAME_RATE, *PPL331_OUT_FRAME_RATE;


// PL331_PROP_AUDIO_CONTROL_PARAM
//
// QPCODEC_AUDIO_CONTROL_PARAM


// PL331_PROP_AUDIO_CONTROL_EX
//
// QPCODEC_AUDIO_CONTROL_EX


// PL331_PROP_ENC_STOP_MODE
//
// QPCODEC_ENC_STOP_MODE


// PL331_PROP_ENC_INPUT_OUTPUT_CONTROL
//
typedef struct _PL331_ENC_INPUT_OUTPUT_CONTROL
{
	TQP_UINT32	src_mode;
	TQP_UINT32	vid_src;
	TQP_UINT32	t420;				// 1: input 420 format, 0: 422 format
	TQP_UINT32	i60p;				// 1: HDMI input is 1920x1080 60p
	TQP_UINT32	pd_en;				// 1: enable 3:2 pulldown
	TQP_UINT32	pln_e;				// 1: ooutput planar format, 0: MB format
	TQP_UINT32	ste_e;				// if pln_e == 1; 1: UV in seperate plane, 0: UV mixed 
	TQP_UINT32	mix_e;				// if pln_e == 1, ste_e == 0; 1: NV12, 0: 8U 8V  
	TQP_UINT32	fl_en;				// 1: enable input filter
	TQP_UINT32	filter_strength;	// from 1 ~ 8, default 2
	TQP_UINT32	in_frame_rate;
	TQP_UINT32	ck_edge;
	TQP_UINT32	sync_mode;

	TQP_UINT32	in_pic_width;
	TQP_UINT32	in_pic_height;

	TQP_UINT32	in_start_pixel;
	TQP_UINT32	in_start_line;

	TQP_UINT32	in_cropping_width;
	TQP_UINT32	in_cropping_height;

	TQP_UINT32	outputScalerID;

} PL331_ENC_INPUT_OUTPUT_CONTROL, *PPL331_ENC_INPUT_OUTPUT_CONTROL;

// src_mode
#define PL331_SRC_MODE_INTERLACE		0
#define PL331_SRC_MODE_PROG				1
#define PL331_SRC_MODE_SPECIAL_1		2
#define PL331_SRC_MODE_SPECIAL_2		3

// vid_src
#define PL331_VID_SRC_ITU656			0
#define PL331_VID_SRC_HDMI				1
#define PL331_VID_SRC_HIU				2
#define PL331_VID_SRC_HDMI_DOUBLE_CLK	3
#define PL331_VID_SRC_ISP				4

// outputScalerID
#define PL331_OUTPUT_SCALER_0			0
#define PL331_OUTPUT_SCALER_1			1
#define PL331_OUTPUT_SCALER_X			2


// PL331_PROP_INDEX_CAP_FREQ
//
// QPCODEC_INDEX_CAP_FREQ


// PL331_PROP_MP4_VID_NUM_BLOCK
//
// QPCODEC_MP4_VID_NUM_BLOCK


// PL331_PROP_AUDIO_INPUT_VOLUME
//
typedef struct _PL331_AUDIO_VOLUME
{
	TQP_UINT32	right;
	TQP_UINT32	left;
} PL331_AUDIO_VOLUME, *PPL331_AUDIO_VOLUME;


// PL331_PROP_ENC_VID_PADDING
//
// QPCODEC_ENC_VID_PADDING


// PL331_PROP_ENC_FREEZE_VIDEO
//
// QPCODEC_ENC_FREEZE_VIDEO


// PL331_PROP_ENC_STILL_VIDEO_INPUT
//
// QPCODEC_ENC_STILL_VIDEO_INPUT


// PL331_PROP_INSERT_I_FRAME
//
typedef struct _PL331_INSERT_I_FRAME
{
	TQP_UINT32	mode;
} PL331_INSERT_I_FRAME, *PPL331_INSERT_I_FRAME;

// mode
#define INSERT_I_FRAME_START_NEW_GOP	0
#define INSERT_I_FRAME_CONTINUE_GOP		1


// PL331_PROP_ENC_GET_A_FRAME
//


// PL331_PROP_ENC_ALL_SETTINGS
//
typedef struct _PL331_ENC_ALL_SETTINGS
{
	// capture parameters
	//
	TQP_UINT32	EncStreamControl;
	TQP_UINT32	EncModeControl;
	TQP_UINT32	EncDataXferControl;
	TQP_UINT32	EncRateControl;
	TQP_UINT32	EncBitRate;
	TQP_UINT32	EncFixedQP;
	TQP_UINT32	EncGopLoopFilter;
	TQP_UINT32	EncOutputResolution;
	TQP_UINT32	EncOutputFrameRate;
	TQP_UINT32	EncAudioControlParam;
	TQP_UINT32	EncAudioControlExAAC;
	TQP_UINT32	EncAudioControlExG711;
	TQP_UINT32	EncAudioControlExLPCM;

	// config inut/output parameters 
	//
	TQP_UINT32	EncInputControl;
	TQP_UINT32	EncInputResolution;
	TQP_UINT32	EncCroppingControl;
	TQP_UINT32	EncCroppingResolution;
	TQP_UINT32	EncOutputScalerID;
	TQP_UINT32	EncSyncMode;

	// config ISP parameters
	//

	// misc.
	//
	TQP_UINT32	EncIndexCapFreq;
	TQP_UINT32	EncMP4VideoBlockNumber;
	TQP_UINT32	EncStopMode;
	TQP_UINT32	EncEnableVidPadding;
	TQP_UINT32	EncAudInputVolume;
} PL331_ENC_ALL_SETTINGS, *PPL331_ENC_ALL_SETTINGS;


// Decoder
//

// PL331_PROP_DEC_STREAM_TYPE
//
typedef struct _PL331_DEC_STREAM_TYPE
{
	TQP_UINT32	stream_type;
	TQP_UINT32	stream_data;
	TQP_UINT32	xfer_mode;
	TQP_UINT32	ld_mode;
	TQP_UINT32	auto_detect;
} PL331_DEC_STREAM_TYPE, *PPL331_DEC_STREAM_TYPE;


// PL331_PROP_DEC_AUDIO_CONTROL
//
// QPCODEC_DEC_AUDIO_CONTROL


// PL331_PROP_DEC_AO_CONTROL
//
// QPCODEC_AOU_CONFIG


// PL331_PROP_DEC_STOP_MODE
//
// QPCODEC_DEC_STOP_MODE


// PL331_PROP_DEC_INPUT_OUTPUT_CONTROL
//
typedef struct _PL331_DEC_INPUT_OUTPUT_CONTROL
{
	TQP_UINT32	tv_system;
	TQP_UINT32	display_mode;
	TQP_UINT32	field_mode;
	TQP_UINT32	scale;				// 1: use scale_factor, 0: use output size/origin

	TQP_UINT32	in_pic_width;
	TQP_UINT32	in_pic_height;

	TQP_UINT32	out_pic_width;
	TQP_UINT32	out_pic_height;

	TQP_UINT32	in_start_pixel;
	TQP_UINT32	in_start_line;

	TQP_UINT32	out_start_pixel;
	TQP_UINT32	out_start_line;

	TQP_UINT32	scale_factor;
} PL331_DEC_INPUT_OUTPUT_CONTROL, *PPL331_DEC_INPUT_OUTPUT_CONTROL;

// tv_system
#define DEC_TV_SYSTEM_NTSC				0
#define DEC_TV_SYSTEM_PAL				1

// display_mode
#define DEC_DISP_MODE_NORMAL			0
#define DEC_DISP_MODE_STOP				1
#define DEC_DISP_MODE_FREEZE			2

// field_mode
#define DEC_FIELD_MODE_PROGRESSIVE		0
#define DEC_FIELD_MODE_TOP_FIELD		1
#define DEC_FIELD_MODE_BOTTOM_FIELD		2
#define DEC_FIELD_MODE_INTERLACED_FRAME	3

// scale_factor
#define DEC_SCALE_FACTOR_NONE			0
#define DEC_SCALE_FACTOR_2X				1
#define DEC_SCALE_FACTOR_4X				2
#define DEC_SCALE_FACTOR_1_2X			3
#define DEC_SCALE_FACTOR_1_4X			4


// PL331_PROP_DEC_AUDIO_OUTPUT_VOLUME
//
// PL331_AUDIO_VOLUME


// PL331_PROP_DEC_TRICK_MODE
//
typedef struct _PL331_DEC_TRICK_MODE
{
	TQP_UINT32	repeat_cnt;		// 0..255, 0xFFFF
	TQP_UINT32	skip_cnt;		// 0..15, 0xFFFF
	TQP_UINT32	direction;
	TQP_UINT32	field_flag;
	TQP_UINT32	step;
	TQP_UINT32	PTS;
} PL331_DEC_TRICK_MODE, *PPL331_DEC_TRICK_MODE;

// repeat_cnt
#define DEC_TRICK_MODE_REPEAT_CNT_STEP	0xFFFF

// skip_cnt
#define DEC_TRICK_MODE_SKIP_CNT_TO_PTS	0xFFFF

// field_flag
// same as field_mode

// step
#define DEC_TRICK_MODE_STEP_NONE		0
#define DEC_TRICK_MODE_STEP_ONE			1


// PL331_PROP_DEC_DATA_XFER_CONTROL
//
// QPCODEC_DEC_DATA_XFER_CONTROL


// PL331_PROP_DEC_ALL_SETTINGS
//
typedef struct _PL331_DEC_ALL_SETTINGS
{
	// decode parameters
	//
	TQP_UINT32	DecStreamType;
	TQP_UINT32	DecAudioControl;
	TQP_UINT32	DecAOControl;

	// stop
	//
	TQP_UINT32	DecStopDispMode;

	// config inut/output parameters 
	//
	TQP_UINT32	DecOutputControl;
	TQP_UINT32	DecDispInputSize;
	TQP_UINT32	DecDispOutputSize;
	TQP_UINT32	DecDispInputOrigin;
	TQP_UINT32	DecDispOutputOrigin;
	TQP_UINT32	DecScalingFactor;

	// misc.
	//
	TQP_UINT32	DecAudOutVolume;
	TQP_UINT32	DecXferMode;
	TQP_UINT32	DecXferMinimumSize;
} PL331_DEC_ALL_SETTINGS, *PPL331_DEC_ALL_SETTINGS;



// Misc
//

// PL331_PROP_ENCODER_STATUS
//


// PL331_PROP_DECODER_STATUS
//


// PL331_PROP_656_OUT_CNTL
//
// QPCODEC_OUT_CNTL


// PL331_PROP_I2S_OUT_CNTL
//
// QPCODEC_OUT_CNTL


// PL331_PROP_POWER_STATE
//
// QPCODEC_POWER_STATE


// PL331_PROP_ERROR_RECOVERY
//
// QPCODEC_ERROR_RECOVERY


// GPIO
//

// PL331_GPIO_BIT_DIRECTION
//
// GPIO_BIT_DIRECTION


// PL331_GPIO_BIT_VALUE
//
// GPIO_BIT_VALUE


// Codec
//

// PL331_PROP_SYS_FUNCTION
//
typedef struct _PL331_SYS_FUNCTION
{
	TQP_UINT32	video;
	TQP_UINT32	audio;
	TQP_UINT32	function;
} PL331_SYS_FUNCTION, *PPL331_SYS_FUNCTION;

// video
#define PL331_OPEN_VID_NONE				0
#define PL331_OPEN_VID_H264				1
#define PL331_OPEN_VID_JPEG				2
#define PL331_OPEN_VID_PASSTHROUGH		4

// audio
#define PL331_OPEN_AUD_NONE				0
#define PL331_OPEN_AUD					1
#define PL331_OPEN_AUD_PASSTHROUGH		4

// function
#define PL331_OPEN_ENCODE				0
#define PL331_OPEN_DECODE				1


// PL331_PROP_SYS_LINK
//
// QPCODEC_SYS_LINK, *PQPCODEC_SYS_LINK;


// PL331_PROP_SYS_CONFIG
//
// QPCODEC_SYS_CONFIG, *PQPCODEC_SYS_CONFIG;


#pragma pack(pop)


#endif //__PL331DEVCNTL_H__



