///////////////////////////////////////////////////////////////////////////////
//
//   
//
//	ql330i2c.h
//
// Modification History:
//	Jul 31, 09	(BP) Created
//
// Authors:
//	(BP)	Brian Pin
//
///////////////////////////////////////////////////////////////////////////////
#ifndef __QL330I2C_H__
#define __QL330I2C_H__

enum ql330_hwi2c_error_code {
	HWI2C_SUCCESS = 0,
	EHWI2C_INVAL = 1,
	EHWI2C_TIMEOUT = 2,
};

/**
 * init: called by system start up thread, user would have no need to call
 */
void __ql330_hwi2c_init(void);

/**
 * i2c write: use ql330 hardware i2c function to do i2c write,
 * 	mutex protected, called in thread context.
 * @dev_addr: device address not wider than 10 bit
 * @inbuf: data buffer array not over than 7 bytes
 * @len: length of data to write
 * return: 0 when success
 */
int __ql330_hwi2c_write(unsigned dev_addr, unsigned char* inbuf, unsigned len);

/*
 * write then read: write register# in device, then read
 * 	mutex protected, called in thread context.
 * @dev_addr: i2c slave device address not wider than 10 bit
 * @wbuf: data to write to device 
 * @wlen: data length for wbuf
 * @rbuf: buffer to hold the read data
 * @rlen: data length to read
 * return: 0 when success
 */
int __ql330_hwi2c_write_then_read(unsigned dev_addr, unsigned char* wbuf,
			unsigned wlen, unsigned char* rbuf, unsigned rlen);

#endif /* __QL330I2C_H__ */
