///////////////////////////////////////////////////////////////////////////////
//        
//
// qldev_interface.h
//
// Description: 
//
//  Header file for streaming driver access interface
// 
// History:
//	taken from debugToolEx project
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __QLDEV_INTERFACE_H__
#define __QLDEV_INTERFACE_H__

#include "qptypes.h"


#ifdef __cplusplus
extern "C" {
#endif

#define QLDEV_INTF_MAX_DMA_SIZE		(32 * 1024)

#if defined(_WIN32) && !defined(_QP_WINKER)

#include <stdlib.h>

#ifdef QLDEV_EXPORTS
#define QLDEV_API __declspec(dllexport)
#else //!QLDEV_EXPORTS
#ifdef QLDEV_IMPORTS
#define QLDEV_API __declspec(dllimport)
#else //!QLDEV_IMPORTS
#define QLDEV_API
#endif //QLDEV_IMPORTS
#endif //QLDEV_EXPORTS

// XU Id
#define XU1_ID_FC3							0x02
#define XU2_ID_FC3							0x03
#define XU3_ID_FC3							0x04

#define DEFINE_GUIDSTRUCT_X(g, n) struct __declspec(uuid(g)) n
#define DEFINE_GUIDNAMED_X(n) __uuidof(struct n)

// XU1 GUID
#define STATIC_PROPSETID_VIDCAP_EXTENSION_UNIT1_FC3 0xB0E82E34, 0xAEE3, 0x4981, 0x9f, 0xd4, 0xed, 0x8c, 0xdb, 0xfb, 0x7e, 0x19
DEFINE_GUIDSTRUCT_X("B0E82E34-AEE3-4981-9FD4-ED8CDBFB7E19",PROPSETID_VIDCAP_EXTENSION_UNIT1_FC3);
#define PROPSETID_VIDCAP_EXTENSION_UNIT1_FC3 DEFINE_GUIDNAMED_X(PROPSETID_VIDCAP_EXTENSION_UNIT1_FC3)

// XU2 GUID
#define STATIC_PROPSETID_VIDCAP_EXTENSION_UNIT2_FC3 0x5B78D6A2, 0xB121, 0x4D53, 0x9A, 0xBC, 0xFF, 0x56, 0xBF, 0x64, 0x09, 0x6F
DEFINE_GUIDSTRUCT_X("5B78D6A2-B121-4D53-9ABC-FF56BF64096F",PROPSETID_VIDCAP_EXTENSION_UNIT2_FC3);
#define PROPSETID_VIDCAP_EXTENSION_UNIT2_FC3 DEFINE_GUIDNAMED_X(PROPSETID_VIDCAP_EXTENSION_UNIT2_FC3)

// XU3 GUID
#define STATIC_PROPSETID_VIDCAP_EXTENSION_UNIT3_FC3 0xB1F82F34, 0xAEE3, 0x4981, 0x9f, 0xd4, 0xed, 0x8c, 0xdb, 0xfb, 0x7e, 0x19
DEFINE_GUIDSTRUCT_X("B1F82F34-AEF3-4981-9FD4-ED8CDBFB7E19",PROPSETID_VIDCAP_EXTENSION_UNIT3_FC3);
#define PROPSETID_VIDCAP_EXTENSION_UNIT3_FC3 DEFINE_GUIDNAMED_X(PROPSETID_VIDCAP_EXTENSION_UNIT3_FC3)

// XU Control Code
#define QXU_MEMORY_READ_CONTROL				0x01
#define QXU_MEMORY_WRITE_CONTROL			0x02
#define QXU_REGISTER_READ_CONTROL			0x03
#define QXU_REGISTER_WRITE_CONTROL			0x04
#define QXU_I2C_READ_CONTROL				0x05
#define QXU_I2C_WRITE_CONTROL				0x06
#define QXU_ROM_READ_CONTROL				0x07
#define QXU_ROM_WRITE_CONTROL				0x08
#define QXU_FIRMWARE_MODE_CONTROL			0x09
#define QXU_GET_USB_SPEED					0x0A
#define QXU_GET_UVC_RUNNING_STATE           0x0B

#define QXU3_CONTROL_UNDEFINED              0x00
#define QXU3_RESOLUTION_CONTROL             0x01
#define QXU3_FRAMERATE_CONTROL              0x02
#define QXU3_AVGKBPS_CONTROL                0x03
#define QXU3_MAXMINKBPS_CONTROL             0x04
#define QXU3_GOPBNUM_CONTROL                0x05
#define QXU3_MDELAY_IN_YUV_CONTROL          0x06

#define QXU3_PROFILE_CONTROL                0x07
#define QXU3_LEVEL_CONTROL                  0x08
#define QXU3_SPSR_FREQ_CONTROL              0x09
#define QXU3_INSERT_KEY_FRAME               0x0A

#define QXU3_PROFILE_BASELINE               0
#define QXU3_PROFILE_MAIN                   1
#define QXU3_PROFILE_EXTENDED               2
//#define QXU3_PROFILE_420_10                 3
//#define QXU3_PROFILE_422_8                  4
//#define QXU3_PROFILE_422_10                 5
//#define QXU3_PROFILE_444_12                 6

#define QXU3_LEVEL_1                        0
#define QXU3_LEVEL_1_B                      1
#define QXU3_LEVEL_1_1                      2
#define QXU3_LEVEL_1_2                      3
#define QXU3_LEVEL_1_3                      4
#define QXU3_LEVEL_2                        5
#define QXU3_LEVEL_2_1                      6
#define QXU3_LEVEL_2_2                      7
#define QXU3_LEVEL_3                        8
#define QXU3_LEVEL_3_1                      9
#define QXU3_LEVEL_3_2                      10
#define QXU3_LEVEL_4                        11
#define QXU3_LEVEL_4_1                      12
#define QXU3_LEVEL_4_2                      13
#define QXU3_LEVEL_5                        14
#define QXU3_LEVEL_5_1                      15

typedef struct _PL_AVSTREAM_INFO
{
	char	*DevDesc;
	char	*HardWareID;
} PL_AVSTREAM_INFO, *PPL_AVSTREAM_INFO;


typedef struct _PL_UVC_INFO
{
	wchar_t	*DevDesc;
	wchar_t	*HardWareID;
} PL_UVC_INFO, *PPL_UVC_INFO;


typedef enum _PL_DEV_TYPE
{
	PL_DEV_TYPE_AVSTRM = 0,
	PL_DEV_TYPE_UVC
} PL_DEV_TYPE, *PPL_DEV_TYPE;


typedef struct _PL_DEV_INFO
{
	PL_DEV_TYPE				devType;
	union _info
	{
		PL_AVSTREAM_INFO	avStrm;
		PL_UVC_INFO			uvc;
	} info;
} PL_DEV_INFO, *PPL_DEV_INFO;

QLDEV_API TQP_BOOL PLFindAndOpenDevices(PL_DEV_INFO **ppInfo, 
										int *pNumDev
										);
QLDEV_API TQP_BOOL PLSetCurrentDevice(int id);
QLDEV_API TQP_BOOL PLSetUvcDevice(void *pUvcFilter, 
								  void *pUvcKsControl);
QLDEV_API TQP_BOOL PLCloseDevices(void);


QLDEV_API TQP_BOOL HCIOpenQpDriver(void);
QLDEV_API TQP_BOOL HCICloseQpDriver(void);
QLDEV_API TQP_BOOL AppOpenQpDriver(TQP_BOOL *pUVCMode);
QLDEV_API TQP_BOOL AppCloseQpDriver(void);
QLDEV_API TQP_BOOL ReadI2C(TQP_UINT8 devAddr, 
						   TQP_UINT8 numBytes, 
						   PTQP_UINT8 pBuf
						   );
QLDEV_API TQP_BOOL WriteI2C(TQP_UINT8 devAddr, 
							TQP_UINT8 numBytes, 
							PTQP_UINT8 pBuf
							);
QLDEV_API TQP_BOOL WriteThenReadI2C(TQP_UINT8 devAddr, 
									TQP_UINT8 numWrBytes, 
									PTQP_UINT8 pWrBuf, 
									TQP_UINT8 numRdBytes, 
									PTQP_UINT8 pRdBuf
									);
QLDEV_API TQP_BOOL ReadReg(TQP_UINT32 dwAddr, 
						   TQP_UINT32 *pdwVal
						   );
QLDEV_API TQP_BOOL WriteReg(TQP_UINT32 dwAddr,
							TQP_UINT32 dwVal
							);
QLDEV_API TQP_BOOL ReadRegEx(TQP_UINT32 dwAddr, 
							 TQP_UINT32 *pdwVal,
							 TQP_UINT32 numReg
							 );
QLDEV_API TQP_BOOL ReadHCIReg(TQP_UINT8 bAddr, 
							  PTQP_UINT8 pbVal
							  );
QLDEV_API TQP_BOOL WriteHCIReg(TQP_UINT8 bAddr, 
							   TQP_UINT8 bVal
							   );
QLDEV_API TQP_BOOL ReadMem(TQP_UINT32 dwAddr, 
						   TQP_UINT32 numDword, 
						   PTQP_UINT32 pbuf
						   );
QLDEV_API TQP_BOOL WriteMem(TQP_UINT32 dwAddr, 
							TQP_UINT32 numDword, 
							PTQP_UINT32 pbuf
							);
QLDEV_API TQP_BOOL DMA_R(unsigned long addr, 
						 unsigned long* data, 
						 int length1, 
						 TQP_BOOL bSwap
						 );
QLDEV_API TQP_BOOL DMA_W(unsigned long addr, 
						 unsigned long* data, 
						 int length1, 
						 TQP_BOOL bSwap
						 );
QLDEV_API TQP_BOOL ResetArm(TQP_BOOL fRun);
QLDEV_API TQP_BOOL SetFirmwareMode(void);
QLDEV_API TQP_BOOL GetUSBSpeed(TQP_UINT32 *pdwSpeed);
QLDEV_API TQP_BOOL GetUSBState(TQP_UINT32 *pdwRunning);

QLDEV_API TQP_BOOL GetExtUnitSize(GUID guid,
								  TQP_UINT32 PropertyId, 
								  TQP_UINT32 *pulSize
								  );

QLDEV_API TQP_BOOL WriteExtUnit(GUID guid,
								TQP_UINT32 PropertyId, 
							    TQP_UINT32 ulSize, 
							    TQP_UINT8 *pValue
							    );

QLDEV_API TQP_BOOL ReadExtUnit(GUID guid,
							   TQP_UINT32 PropertyId,
							   TQP_UINT32 ulSize, 
							   TQP_UINT8 *pValue
							   );

QLDEV_API TQP_BOOL ReadExtUnitDef(GUID guid,
								  TQP_UINT32 PropertyId, 
								  TQP_UINT32 ulSize, 
								  TQP_UINT8 *pValue
								  );

QLDEV_API TQP_BOOL ReadExtUnitMax(GUID guid,
								  TQP_UINT32 PropertyId, 
								  TQP_UINT32 ulSize, 
								  TQP_UINT8 *pValue
								  );

QLDEV_API TQP_BOOL ReadExtUnitMin(GUID guid,
								  TQP_UINT32 PropertyId, 
								  TQP_UINT32 ulSize, 
								  TQP_UINT8 *pValue
								  );

#else //!_WIN32 || _QP_WINKER

TQP_BOOL HCIOpenQpDriver(void)
{
	return (QP_TRUE);
}
TQP_BOOL HCICloseQpDriver(void)
{
	return (QP_TRUE);
}
TQP_BOOL AppOpenQpDriver(TQP_BOOL *pUVCMode)
{
	return (QP_TRUE);
}
TQP_BOOL AppCloseQpDriver(void)
{
	return (QP_TRUE);
}
TQP_BOOL ReadI2C(TQP_UINT8 devAddr, 
				 TQP_UINT8 numBytes, 
				 PTQP_UINT8 pBuf
				 )
{
	return (QP_TRUE);
}
TQP_BOOL WriteI2C(TQP_UINT8 devAddr, 
				  TQP_UINT8 numBytes, 
				  PTQP_UINT8 pBuf
				  )
{
	return (QP_TRUE);
}
TQP_BOOL WriteThenReadI2C(TQP_UINT8 devAddr, 
						  TQP_UINT8 numWrBytes, 
						  PTQP_UINT8 pWrBuf, 
						  TQP_UINT8 numRdBytes, 
						  PTQP_UINT8 pRdBuf
						  )
{
	return (QP_TRUE);
}
TQP_BOOL ReadReg(TQP_UINT32 dwAddr, 
				 TQP_UINT32 *pdwVal
				 )
{
	return (QP_TRUE);
}
TQP_BOOL WriteReg(TQP_UINT32 dwAddr,
				  TQP_UINT32 dwVal
				  )
{
	return (QP_TRUE);
}
TQP_BOOL ReadRegEx(TQP_UINT32 dwAddr, 
				   TQP_UINT32 *pdwVal,
				   TQP_UINT32 numReg
				   )
{
	return (QP_TRUE);
}
TQP_BOOL ReadHCIReg(TQP_UINT8 bAddr, 
					PTQP_UINT8 pbVal
					)
{
	return (QP_TRUE);
}
TQP_BOOL WriteHCIReg(TQP_UINT8 bAddr, 
					 TQP_UINT8 bVal
					 )
{
	return (QP_TRUE);
}
TQP_BOOL ReadMem(TQP_UINT32 dwAddr, 
				 TQP_UINT32 numDword, 
				 PTQP_UINT32 pbuf
				 )
{
	return (QP_TRUE);
}
TQP_BOOL WriteMem(TQP_UINT32 dwAddr, 
				  TQP_UINT32 numDword, 
				  PTQP_UINT32 pbuf
				  )
{
	return (QP_TRUE);
}
TQP_BOOL DMA_R(unsigned long addr, 
			   unsigned long* data, 
			   int length1, 
			   TQP_BOOL bSwap
			   )
{
	return (QP_TRUE);
}
TQP_BOOL DMA_W(unsigned long addr, 
			   unsigned long* data, 
			   int length1, 
			   TQP_BOOL bSwap
			   )
{
	return (QP_TRUE);
}
TQP_BOOL ResetArm(TQP_BOOL fRun)
{
	return (QP_TRUE);
}
TQP_BOOL SetFirmwareMode(void)
{
	return (QP_TRUE);
}
TQP_BOOL GetUSBSpeed(TQP_UINT32 *pdwSpeed)
{
	return (QP_TRUE);
}
TQP_BOOL GetUSBState(TQP_UINT32 *pdwRunning)
{
	return (QP_TRUE);
}

TQP_BOOL WriteExtUnit(TQP_UINT32 ExtUnitId,
					  TQP_UINT32 PropertyId,
					  TQP_UINT32 ulSize,
					  TQP_UINT8 *pValue
					  )
{
	return (QP_TRUE);
}

TQP_BOOL ReadExtUnit(TQP_UINT32 ExtUnitId,
					 TQP_UINT32 PropertyId, 
				     TQP_UINT32 ulSize, 
				     TQP_UINT8 *pValue
				     )
{
	return (QP_TRUE);
}

#endif //_WIN32 && !_QP_WINKER

#ifdef __cplusplus
}
#endif

#endif //__QLDEV_INTERFACE_H__
