///////////////////////////////////////////////////////////////////////////////
//        
//
// Qpaudcodec.h
//
// Description: 
//
//  This is the definations of the audio codec peripheral chip
// 
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __QPAUDCODEC_H__
#define __QPAUDCODEC_H__


#include "qptypes.h"
#include "qpgenericdevice.h"

#ifdef __cplusplus
extern "C" {
#endif


/////////////////////////////////////////////////////////////////////////////
//
//

typedef enum _AIN_SOURCE
{
	LINEIN1 = 1,
	LINEIN2 = 2,
	TVAUDIO = 3,
	AUXIN = 4,
	AUDINVALID = -1
} AIN_SOURCE, *PAIN_SOURCE;



/////////////////////////////////////////////////////////////////////////////
//
//

// device specific initialization data
//

typedef enum _QPAUDCODEC_TYPE
{
	AUDCODEC_DUMMY = 0,
	AUDCODEC_FM31 = 1,
	AUDCODEC_FM31_BUS2 = 2,
	AUDCODEC_INVALID = -1
} QPAUDCODEC_TYPE, *PQPAUDCODEC_TYPE;


typedef struct _QPAUDCODEC_INITDATA
{
	QPAUDCODEC_TYPE	type;
} QPAUDCODEC_INITDATA, *PQPAUDCODEC_INITDATA;



/////////////////////////////////////////////////////////////////////////////
//
//

// property supported by this interface
//



/////////////////////////////////////////////////////////////////////////////
//
//

// codec cllback command code
//



/////////////////////////////////////////////////////////////////////////////
//
//

#ifndef __IAUDCODEC_FWD_DEFINED__
#define __IAUDCODEC_FWD_DEFINED__
typedef struct IAudioCodec IAudioCodec;
#endif //__IAUDCODEC_FWD_DEFINED__

    struct IAudioCodec
    {
		// IGenericDevice
		DECLARE_IGENERICDEVICE(IAudioCodec)

		// IAudioCodec

		// Select the audio source.
		EQP_Errors (*SelectSource)(IAudioCodec *This, 
								   AIN_SOURCE src
								   );

		// Get the audio source.
		EQP_Errors (*GetSource)(IAudioCodec *This, 
								PAIN_SOURCE psrc
								);


		// Get Attenuation Range for the Device
		EQP_Errors (*GetVolumeRange)(IAudioCodec *This, 
									 int *pnDefault,
									 int *pnMin,
									 int *pnMax
									 );
		// Set Volume
		EQP_Errors (*SetVolume)(IAudioCodec *This, 
								int nLeft,
								int nRight
								);
		// Get Volume
		EQP_Errors (*GetVolume)(IAudioCodec *This, 
								int *pnLeft,
								int *pnRight
								);
		// Set Mute
		EQP_Errors (*SetMute)(IAudioCodec *This, 
							  TQP_BOOL fMuteLeft,
							  TQP_BOOL fMuteRight
							  );
		// Get Mute/Unmute state
		EQP_Errors (*GetMute)(IAudioCodec *This, 
							  TQP_BOOL *pfMuteLeft,
							  TQP_BOOL *pfMuteRight
							  );
    };


/////////////////////////////////////////////////////////////////////////////
//
//

#define QPAudioCodec_SelectSource(This, src) \
    (This)->SelectSource(This, src)

#define QPAudioCodec_GetSource(This, psrc) \
    (This)->GetSource(This, psrc)

#define QPAudioCodec_GetVolumeRange(This, pnDefault, pnMin, pnMax) \
    (This)->GetVolumeRange(This, pnDefault, pnMin, pnMax)

#define QPAudioCodec_SetVolume(This, nLeft, nRight) \
    (This)->SetVolume(This, nLeft, nRight)

#define QPAudioCodec_GetVolume(This, pnLeft, pnRight) \
    (This)->GetVolume(This, pnLeft, pnRight)

#define QPAudioCodec_SetMute(This, fMuteLeft, fMuteRight) \
    (This)->SetMute(This, fMuteLeft, fMuteRight)

#define QPAudioCodec_GetMute(This, pfMuteLeft, pfMuteRight) \
    (This)->GetMute(This, pfMuteLeft, pfMuteRight)


#ifdef __cplusplus
}
#endif

#endif //__QPAUDCODEC_H__



