///////////////////////////////////////////////////////////////////////////////
//        
//
//		Platform-independent Block Device Manager interface
//
// History:
//	Oct 18, 2007	(EC) Created
//
// Authors: (EC) Enrico Cadorin
//
///////////////////////////////////////////////////////////////////////////////


#ifndef __QPBDMGR_H__
#define __QPBDMGR_H__

#include "qptypes.h"

#ifdef __cplusplus
extern "C" {
#endif


//
// Capabilities structure
typedef struct _TQP_BDMCaps {
	TQP_BOOL		sdMem;		// true if SD memory supported
	TQP_BOOL		sdio;		// true if SDIO supported
	TQP_BOOL		ceATA;		// true if CE/ATA supported
} TQP_BDMCaps, *PTQP_BDMCaps;


//
// Returns the list of capabilities for this implementation
//
void QPBDMGetCaps (PTQP_BDMCaps capsP);


// The type of a Block Device ID
typedef void * TQP_BDMDevID;

//
// Return a list of available device IDs.
// numP contains the size of devIDs array on entry; actual number of IDs on exit.
//
EQP_Errors QPBDMGetDevIDs (TQP_BDMDevID * devIDs, PTQP_INT32 numP);


typedef struct _TQP_BDMDevInfo {
	TQP_UINT32		bytesPerBlock;		// bytes per block (usually 512)
	TQP_UINT64		blocksNum;			// number of blocks in the device
	TQP_BOOL		readOnly;			// TRUE if the device is not writable
} TQP_BDMDevInfo, *PTQP_BDMDevInfo;

//
// Return information about an available device
//
EQP_Errors QPBDMGetDevInfo (TQP_BDMDevID devID, PTQP_BDMDevInfo devInfoP);


typedef void (*TQP_BDCallback) (void * refCon);

//
// Register a calback function to be called when a device status changes
// (plugged/unplugged ...)
//
EQP_Errors QPBDMRegisterCallback (TQP_BDCallback cb, void * refCon);

//
// Read data from a device
//
EQP_Errors QPBDMRead (TQP_BDMDevID devID, TQP_UINT64 startBlock,
					PTQP_UINT32 blockNumP, void * dataP);

//
// Write data to a device
//
EQP_Errors QPBDMWrite (TQP_BDMDevID devID, TQP_UINT64 startBlock,
					PTQP_UINT32 blockNumP, void * dataP);

#ifdef __cplusplus
}
#endif

#endif //__QPBDMGR_H__
