///////////////////////////////////////////////////////////////////////////////
//        
//
// Qpcodecintf.h
//
// Description: 
//
//  This is the definations of the codec library interface
// 
// History:
//	 Nov 4, 2008	(dt) created
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __QPCODECINTF_H__
#define __QPCODECINTF_H__

#include "qpdevcntl.h"


#ifdef QPCODECINTF_EXPORTS
#define QPCODECINTF_API __declspec(dllexport)
#else //!QPCODECINTF_EXPORTS
#define QPCODECINTF_API __declspec(dllimport)
#endif //QPCODECINTF_EXPORTS


#pragma pack(push, 1)


// all the encoder parameters constants are defined in
// common\qpdevcntl.h
//
typedef struct _QPCODEC_ENCODER_PARAMS
{
	TQP_UINT32	profile;
	TQP_UINT32	level;
	TQP_UINT32	ff_mode;				// frame, field
	TQP_UINT32	v_mode;					// cabac, cavlc

	TQP_UINT32	in_pic_width;
	TQP_UINT32	in_pic_height;
	TQP_UINT32	out_pic_width;
	TQP_UINT32	out_pic_height;

	TQP_UINT32	scan_format;
	TQP_UINT32	src_mode;
	TQP_UINT32	sync_mode;
	TQP_UINT32	in_frame_rate;
	TQP_UINT32	out_frame_rate;
	TQP_UINT32	fmt;

	TQP_UINT32	vbr;
	TQP_UINT32	avg_bit_rate;			// in bps
	TQP_UINT32	min_bit_rate;			// in bps
	TQP_UINT32	max_bit_rate;			// in bps

	TQP_UINT32	max_nalu_size;			// in unit of bytes

	TQP_UINT32	intra_period;
	TQP_UINT32	B_num;
	TQP_UINT32	IDR;

} QPCODEC_ENCODER_PARAMS, *PQPCODEC_ENCODER_PARAMS;



// all the decoder parameters constants are defined in
// common\qpdevcntl.h
//
typedef struct _QPCODEC_DECODER_PARAMS
{
	TQP_UINT32	outputMode;
	TQP_UINT32	pic_width;
	TQP_UINT32	pic_height;

} QPCODEC_DECODER_PARAMS, *PQPCODEC_DECODER_PARAMS;


#define QPCODEC_MAX_NUMBER_NALU_PER_PICTURE		16

// buffer descriptor format
//
typedef struct _QPCODEC_BUFFER_DESCRIPTOR
{
	TQP_UINT32	ulFlags;			// TBD
	EQP_Errors	Status;
    TQP_UINT32	FrameExtent;		// size of the entire buffer
    TQP_UINT32	DataUsed;			// actual size used
    TQP_PVOID	Data;				// data buffer
	TQP_UINT32	DataType;			// data type

	TQP_UINT32	NumNalu;
	TQP_UINT32	NaluOffset[QPCODEC_MAX_NUMBER_NALU_PER_PICTURE];

} QPCODEC_BUFFER_DESCRIPTOR, *PQPCODEC_BUFFER_DESCRIPTOR;


#define QPCODEC_DATA_TYPE_YV12	1
#define QPCODEC_DATA_TYPE_I420	2
#define QPCODEC_DATA_TYPE_H264	3


#pragma pack(pop)


// callback fuction type
//
// arg 1 : callback function context passed to either encoder or decoder in QpInitEncoder
//         or QpInitDecoder
// arg 2 : reason for this callback, currently only QPLIBCB_DATA_DONE is defined
// arg 3 : the meaning of this parameter is based on the code in arg 2 
//         for QPLIBCB_DATA_DONE, this argument is the pointer to the QPCODEC_BUFFER_DESCRIPTOR
//         to return either input buffer or output buffer passed into encoder or decoder
// return : error code, QPERR_SUCCESS success, a negative number returned means 
//          one of the errors defined in common\qptypes.h 
// 
//
typedef EQP_Errors (__stdcall *QP_LIB_CALLBACK)(TQP_PVOID Context, TQP_UINT32 dwCode, TQP_PVOID pParam);

// callback function code
//
#define QPLIBCB_DATA_DONE	0x00000001



/////////////////////////////////////////////////////////////////////////////
//
//

// initialize the codec library, this function is the first function 
// to be called in order to use this library 
//
// arg 1 : hardware instance starting from 0 for the first codec hardware
// arg 2 : a pointer to hold the returned reference handle to the codec library 
// return : error code, QPERR_SUCCESS success, a negative number returned means 
//          one of the errors defined in common\qptypes.h 
// 
QPCODECINTF_API EQP_Errors QpInitCodecLibrary(TQP_UINT32 instance, 
											  PTQP_OBJECT_HANDLE phLib
											  );

// release the codec library
//
// arg 1 : reference handle to the codec library acquired from QpInitCodecLibrary
// return : error code, QPERR_SUCCESS success, a negative number returned means 
//          one of the errors defined in common\qptypes.h 
//
QPCODECINTF_API EQP_Errors QpDoneCodecLibrary(TQP_OBJECT_HANDLE hLib);

// initialize the encoder. before you can use the encoder you need to
// call this function to setup the encoder parameters and callback functions
// and callback context
//
// arg 1 : reference handle to the codec library acquired from QpInitCodecLibrary
// arg 2 : a pointer to hold the returned reference handle to the encoder
// arg 3 : a pointer to hold the QPCODEC_ENCODER_PARAMS for the parameter block
// arg 4 : the encoder callback function
// agr 5 : the encoder callback function context
// return : error code, QPERR_SUCCESS success, a negative number returned means 
//          one of the errors defined in common\qptypes.h 
//
QPCODECINTF_API EQP_Errors QpInitEncoder(TQP_OBJECT_HANDLE hLib, 
										 PTQP_OBJECT_HANDLE phEncoder,
										 PQPCODEC_ENCODER_PARAMS pEncParams,
										 QP_LIB_CALLBACK pFuncCallback,
										 TQP_PVOID context
										 );

// release the encoder
//
// arg 1 : reference handle to the encoder acquired from QpInitEncoder
// return : error code, QPERR_SUCCESS success, a negative number returned means 
//          one of the errors defined in common\qptypes.h 
//
QPCODECINTF_API EQP_Errors QpReleaseEncoder(TQP_OBJECT_HANDLE hEncoder);

// start the encoder
//
// arg 1 : reference handle to the encoder acquired from QpInitEncoder
// return : error code, QPERR_SUCCESS success, a negative number returned means 
//          one of the errors defined in common\qptypes.h 
//
QPCODECINTF_API EQP_Errors QpStartEncoder(TQP_OBJECT_HANDLE hEncoder); 

// stop the encoder
//
// arg 1 : reference handle to the encoder acquired from QpInitEncoder
// return : error code, QPERR_SUCCESS success, a negative number returned means 
//          one of the errors defined in common\qptypes.h 
//
QPCODECINTF_API EQP_Errors QpStopEncoder(TQP_OBJECT_HANDLE hEncoder);


// send the input buffer and output buffer to the encoder
//
// arg 1 : reference handle to the encoder acquired from QpInitEncoder
// arg 2 : pointer to the buffer descriptor that describes a YUV frame to be encoded
//         this argument can be NULL if you only want to pass output buffer(h.264) 
//         to the encoder
// arg 3 : pointer to the buffer descriptor that describe the buffer to hold returned 
//         h.264 bitstream.
//         this argument can be NULL if you only want to pass input buffer(YUV) 
//         to the encoder
// return : error code, QPERR_SUCCESS success, a negative number returned means 
//          one of the errors defined in common\qptypes.h 
//
QPCODECINTF_API EQP_Errors QpEncodeFrame(TQP_OBJECT_HANDLE hEncoder, 
										 PQPCODEC_BUFFER_DESCRIPTOR pInBuf, 
										 PQPCODEC_BUFFER_DESCRIPTOR pOutBuf
										 );

// Get the current encoder bitrate setting
// this function can be called anytime during encoding
//
// arg 1 : reference handle to the encoder acquired from QpInitEncoder
// arg 2 : a pointer to hold vbr
// arg 3 : a pointer to hold average bitrate in bps
// arg 4 : a pointer to hold minimum bitrate in bps(not applicable in cbr mode)
// arg 5 : a pointer to hold maxmum bitrate in bps(not applicable in cbr mode)
// return : error code, QPERR_SUCCESS success, a negative number returned means 
//          one of the errors defined in common\qptypes.h 
//
QPCODECINTF_API EQP_Errors QpGetEncoderBitrate(TQP_OBJECT_HANDLE hEncoder, 
											   TQP_UINT32 *p_vbr,
											   TQP_UINT32 *p_avg_bit_rate,
											   TQP_UINT32 *p_min_bit_rate,
											   TQP_UINT32 *p_max_bit_rate
											   );

// change the encoder bitrate
// this function can be called anytime during encoding
//
// arg 1 : reference handle to the encoder acquired from QpInitEncoder
// arg 2 : 0: cbr; 1 : vbr
// arg 3 : average bitrate in bps
// arg 4 : minimum bitrate in bps(not applicable in cbr mode)
// arg 5 : maxmum bitrate in bps(not applicable in cbr mode)
// return : error code, QPERR_SUCCESS success, a negative number returned means 
//          one of the errors defined in common\qptypes.h 
//
QPCODECINTF_API EQP_Errors QpSetEncoderBitrate(TQP_OBJECT_HANDLE hEncoder, 
											   TQP_UINT32 vbr,
											   TQP_UINT32 avg_bit_rate,
											   TQP_UINT32 min_bit_rate,
											   TQP_UINT32 max_bit_rate
											   );

// Get current encoder frame rate setting
// this function can be called anytime during encoding
//
// arg 1 : reference handle to the encoder acquired from QpInitEncoder
// arg 2 : a pointer to hold input frame rate in fps
// arg 3 : a pointer to hold output frame rate in fps
// return : error code, QPERR_SUCCESS success, a negative number returned means 
//          one of the errors defined in common\qptypes.h 
//
QPCODECINTF_API EQP_Errors QpGetEncoderFrameRate(TQP_OBJECT_HANDLE hEncoder, 
												 TQP_UINT32 *p_in_frame_rate,
												 TQP_UINT32 *p_out_frame_rate
												 );


// change the encoder frame rate
// this function can be called anytime during encoding
//
// arg 1 : reference handle to the encoder acquired from QpInitEncoder
// arg 2 : input frame rate in fps
// arg 3 : output frame rate in fps
// return : error code, QPERR_SUCCESS success, a negative number returned means 
//          one of the errors defined in common\qptypes.h 
//
QPCODECINTF_API EQP_Errors QpSetEncoderFrameRate(TQP_OBJECT_HANDLE hEncoder, 
												 TQP_UINT32 in_frame_rate,
												 TQP_UINT32 out_frame_rate
												 );

// Get current encoder resolution setting
// this function can not be called anytime during encoding
//
// arg 1 : reference handle to the encoder acquired from QpInitEncoder
// arg 2 : a pointer to hold input video width in pixels
// arg 3 : a pointer to hold input video height in pixels
// arg 4 : a pointer to hold output video width in pixels
// arg 5 : a pointer to hold output video height in pixels
// return : error code, QPERR_SUCCESS success, a negative number returned means 
//          one of the errors defined in common\qptypes.h 
//
QPCODECINTF_API EQP_Errors QpGetEncoderResolution(TQP_OBJECT_HANDLE hEncoder, 
												 TQP_UINT32 *p_in_video_width,
												 TQP_UINT32 *p_in_video_height,
												 TQP_UINT32 *p_out_video_width,
												 TQP_UINT32 *p_out_video_height
												 );


// change the encoder resolution
// this function can not be called anytime during encoding
//
// arg 1 : reference handle to the encoder acquired from QpInitEncoder
// arg 2 : input video width in pixels
// arg 3 : input video height in pixels
// arg 4 : output video width in pixels
// arg 5 : output video height in pixels
// return : error code, QPERR_SUCCESS success, a negative number returned means 
//          one of the errors defined in common\qptypes.h 
//
QPCODECINTF_API EQP_Errors QpSetEncoderResolution(TQP_OBJECT_HANDLE hEncoder, 
												 TQP_UINT32 in_video_width,
												 TQP_UINT32 in_video_height,
												 TQP_UINT32 out_video_width,
												 TQP_UINT32 out_video_height
												 );

// insert key(I) frame
// this function can be called anytime during encoding
//
// arg 1 : reference handle to the encoder acquired from QpInitEncoder
// return : error code, QPERR_SUCCESS success, a negative number returned means 
//          one of the errors defined in common\qptypes.h 
//
QPCODECINTF_API EQP_Errors QpInsertEncoderKeyFrame(TQP_OBJECT_HANDLE hEncoder);

// set maximum nalu size
// TBD
//
QPCODECINTF_API EQP_Errors QpSetMaxNaluSize(TQP_OBJECT_HANDLE hEncoder, 
											TQP_UINT32 max_nalu_size	// in byte unit
											);

// initialize the decoder. before you can use the decoder you need to
// call this function to setup the decoder parameters and callback functions
// and callback context
//
// arg 1 : reference handle to the codec library acquired from QpInitCodecLibrary
// arg 2 : a pointer to hold the returned reference handle to the decoder
// arg 3 : a pointer to hold the QPCODEC_DECODER_PARAMS for the parameter block
// arg 4 : the decoder callback function
// agr 5 : the decoder callback function context
// return : error code, QPERR_SUCCESS success, a negative number returned means 
//          one of the errors defined in common\qptypes.h 
//
QPCODECINTF_API EQP_Errors QpInitDecoder(TQP_OBJECT_HANDLE hLib, 
										 PTQP_OBJECT_HANDLE phDecoder,
										 PQPCODEC_DECODER_PARAMS pDecParams,
										 QP_LIB_CALLBACK pFuncCallback,
										 TQP_PVOID context
										 );

// release the decoder
//
// arg 1 : reference handle to the decoder acquired from QpInitDecoder
// return : error code, QPERR_SUCCESS success, a negative number returned means 
//          one of the errors defined in common\qptypes.h 
//
QPCODECINTF_API EQP_Errors QpReleaseDecoder(TQP_OBJECT_HANDLE hDecoder);

// start the decoder
//
// arg 1 : reference handle to the decoder acquired from QpInitDecoder
// return : error code, QPERR_SUCCESS success, a negative number returned means 
//          one of the errors defined in common\qptypes.h 
//
QPCODECINTF_API EQP_Errors QpStartDecoder(TQP_OBJECT_HANDLE hDecoder); 

// stop the decoder
//
// arg 1 : reference handle to the decoder acquired from QpInitDecoder
// return : error code, QPERR_SUCCESS success, a negative number returned means 
//          one of the errors defined in common\qptypes.h 
//
QPCODECINTF_API EQP_Errors QpStopDecoder(TQP_OBJECT_HANDLE hDecoder);

// send the input buffer and output buffer to the decoder
//
// arg 1 : reference handle to the decoder acquired from QpInitDecoder
// arg 2 : pointer to the buffer descriptor that describes a h.264 buffer to be decoded
//         this argument can be NULL if you only want to pass output buffer(YUV) 
//         to the decoder
// arg 3 : pointer to the buffer descriptor that describe the buffer to hold returned 
//         YUV frame.
//         this argument can be NULL if you only want to pass input buffer(h.264) 
//         to the decoder
// return : error code, QPERR_SUCCESS success, a negative number returned means 
//          one of the errors defined in common\qptypes.h 
//
QPCODECINTF_API EQP_Errors QpDecodeFrame(TQP_OBJECT_HANDLE hDecoder, 
										 PQPCODEC_BUFFER_DESCRIPTOR pInBuf, 
										 PQPCODEC_BUFFER_DESCRIPTOR pOutBuf
										 );

#endif //__QPCODECINTF_H__



