///////////////////////////////////////////////////////////////////////////////
//        
//
// Qpcodeclib.h
//
// Description: 
//
//  Header file for codec library interface.
// 
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __QPCODECLIB_H__
#define __QPCODECLIB_H__


/////////////////////////////////////////////////////////////////////////////
//
//

#include "qptypes.h"
#include "qpgenericdevice.h"
#include "qpviddecoder.h"
#include "qpvidencoder.h"
#include "qptuner.h"
#include "qptvaudio.h"
#include "qpaudcodec.h"
#include "qpmpgcodec.h"
#include "qpi2c.h"


#ifdef __cplusplus
extern "C" {
#endif


/////////////////////////////////////////////////////////////////////////////
//
//

// Maximum queue depth, need to be cohesive between the library
// and the client.
//

#ifdef _QPCODEC_CALLBACK_IN_THREAD
#define QPCODECLIB_MAX_DATA_ENTRIES		64
#else //!_QPCODEC_CALLBACK_IN_THREAD
#define QPCODECLIB_MAX_DATA_ENTRIES		256//128
#endif //_QPCODEC_CALLBACK_IN_THREAD




/////////////////////////////////////////////////////////////////////////////
//
//

// interface specific initialization data
//

typedef struct _QPCODECLIB_INITDATA
{
	// codeclib portion
	//
	TQP_BOOL				bDownloadFW;

	// mpgcodec portion
	//
	QPMPGCODEC_INITDATA		mpgCodecInitData;

	// video decoder portion
	//
	QPVIDDECODER_INITDATA	vidDecoderInitData;

	// video encoder portion
	//
	QPVIDENCODER_INITDATA	vidEncoderInitData;

	// tuner portion
	//
	QPTUNER_INITDATA		tunerInitData;

	// tv audio portion
	//
	QPTVAUDIO_INITDATA		tvAudioInitData;

	// audio codec portion
	//
	QPAUDCODEC_INITDATA		audCodecInitData;

	// i2c portion
	//
	QPI2C_INITDATA			i2cInitData;
	QPI2C_INITDATA			i2cInitDataEx;

} QPCODECLIB_INITDATA, *PQPCODECLIB_INITDATA;




/////////////////////////////////////////////////////////////////////////////
//
//

// property supported by this interface
//



/////////////////////////////////////////////////////////////////////////////
//
//


// codec cllback command code
//



/////////////////////////////////////////////////////////////////////////////
//
//

#ifndef __ICODECLIB_FWD_DEFINED__
#define __ICODECLIB_FWD_DEFINED__
typedef struct ICodecLib ICodecLib;
#endif //__ICODECLIB_FWD_DEFINED__

    struct ICodecLib
    {
		// IGenericDevice
		DECLARE_IGENERICDEVICE(ICodecLib)

		// ICodecLib
		EQP_Errors (*GetMpegCodec)(ICodecLib *This, 
								   IMpegCodec** ppMpegCodec
								   );
		EQP_Errors (*GetVideoDecoder)(ICodecLib *This, 
									  IVideoDecoder** ppVideoDecoder
									  );
		EQP_Errors (*GetVideoEncoder)(ICodecLib *This, 
									  IVideoEncoder** ppVideoEncoder
									  );
		EQP_Errors (*GetAudioCodec)(ICodecLib *This, 
									IAudioCodec** ppAudioCodec
								  );
		EQP_Errors (*GetTuner)(ICodecLib *This, 
							   ITuner** ppTuner
							 );
		EQP_Errors (*GetTVAudio)(ICodecLib *This, 
								 ITVAudio** ppTVAudio
								 );
		EQP_Errors (*Disable)(ICodecLib *This);
		EQP_Errors (*Enable)(ICodecLib *This);
    };



/////////////////////////////////////////////////////////////////////////////
//
//

// Codec Library entry points for Beringer and Artesa 
//
EQP_Errors QPCodecInitLibrary(TQP_PVOID pContext1,	//PDEVICE_OBJECT
							  TQP_PVOID pContext2,	//PDEVICE_OBJECT
							  PQPCODECLIB_INITDATA pInitData,
							  ICodecLib** ppCodecLib
							  );
EQP_Errors QPCodecDoneLibrary(ICodecLib* pCodecLib);


// Codec Library entry points for PL331
//
EQP_Errors PL331InitLibrary(TQP_PVOID pContext1,	//PDEVICE_OBJECT
							TQP_PVOID pContext2,	//PDEVICE_OBJECT
							PQPCODECLIB_INITDATA pInitData,
							ICodecLib** ppCodecLib
							);
EQP_Errors PL331DoneLibrary(ICodecLib* pCodecLib);


/////////////////////////////////////////////////////////////////////////////
//
//

#define QPCodecLib_GetMpegCodec(This, ppMpegCodec) \
    ((QP_NULL == This) ? QPERR_PARMS : ((This)->GetMpegCodec(This, ppMpegCodec)))

#define QPCodecLib_GetVideoDecoder(This, ppVideoDecoder) \
    ((QP_NULL == This) ? QPERR_PARMS : ((This)->GetVideoDecoder(This, ppVideoDecoder)))

#define QPCodecLib_GetVideoEncoder(This, ppVideoEncoder) \
    ((QP_NULL == This) ? QPERR_PARMS : ((This)->GetVideoEncoder(This, ppVideoEncoder)))

#define QPCodecLib_GetAudioCodec(This, ppAudioCodec) \
    ((QP_NULL == This) ? QPERR_PARMS : ((This)->GetAudioCodec(This, ppAudioCodec)))

#define QPCodecLib_GetTuner(This, ppTuner) \
    ((QP_NULL == This) ? QPERR_PARMS : ((This)->GetTuner(This, ppTuner)))

#define QPCodecLib_GetTVAudio(This, ppTVAudio) \
    ((QP_NULL == This) ? QPERR_PARMS : ((This)->GetTVAudio(This, ppTVAudio)))

#define QPCodecLib_Disable(This) \
    ((QP_NULL == This) ? QPERR_PARMS : ((This)->Disable(This)))

#define QPCodecLib_Enable(This) \
    ((QP_NULL == This) ? QPERR_PARMS : ((This)->Enable(This)))

#ifdef __cplusplus
}
#endif

#endif //__QPCODECLIB_H__





