///////////////////////////////////////////////////////////////////////////////
//        
//
//		Debug Manager interface
//
//	Platform-independent debugging interface
//
// History:
//	Aug 4, 2006	(EC) Created
//
// Authors: (EC) Enrico Cadorin
//
///////////////////////////////////////////////////////////////////////////////


#ifndef __QPDBGMGR_H__
#define __QPDBGMGR_H__

#include "qptypes.h"

#ifdef __cplusplus
extern "C" {
#endif

// A macro that contains filename:fucntionname
// There is no ANSI standard macro for this, therefore we need
// compiler specific definitions.
#ifndef __FUNCT__
#define __FUNCT__ __FILE__
#endif

#if defined (__arm__)		// ARM compilers
#undef __FUNCT__
#define __FUNCT__
#define __FILFUN__ __MODULE__ ":" __FUNCT__ ": "
#endif

enum _EQPDbgMLevels {
	QPDBGM_LEVEL_NONE = 0,	// No debugging output
	QPDBGM_LEVEL_FATAL,		// Fatal error, cannot recover
	QPDBGM_LEVEL_ERROR,		// Major error, try to recover
	QPDBGM_LEVEL_WARN,		// Warning, might not be able to recover
	QPDBGM_LEVEL_TRACE,		// Tracing information
	QPDBGM_LEVEL_EXTRACE,	// Extended tracing information
	QPDBGM_LEVEL_NUM
};


// use this macro to set first parameter of the printf function
#define QPDBGMCOMPLVL(cID,dbgLvl) (((cID & 0xFFF)<<4) + (dbgLvl & 0xF))

typedef TQP_UINT32 TQP_DbgMCompID;

//
// Capabilities structure
typedef struct _TQP_DbgMCaps {
	TQP_BOOL		isDebug;	// true if it is a debug build
	unsigned long	maxComps;	// maximum number of distinct components supported
	unsigned long	maxNameLen;	// maximum number of characters in a component name
} TQP_DbgMCaps, *PTQP_DbgMCaps;

//
// Returns the list of capabilities for this implementation
//
void QPDbgMGetCaps (PTQP_DbgMCaps capsP);



//
// Classic printf function.
//
int _QPDbgMPrintf (TQP_UINT32 compLevel, const char * format, ...);
#ifdef QP_DEBUG
#define QPDbgMPrintf(_al_) _QPDbgMPrintf _al_
#else
#define QPDbgMPrintf(_al_)
#endif

//
// Register a component by name
//	returns a component ID in cIDP; nameP is a null terminated string
//
EQP_Errors QPDbgMRegComp (TQP_DbgMCompID * cIDP, char * nameP);

//
// Unregister a component
//
EQP_Errors QPDbgMUnregComp (TQP_DbgMCompID cID);

///////////////////////////////////////////////////////////////////////////////
// Functions to be called by the environment that outputs the debug
// strings.

//
// Turn on and off the component name printing feature
//
void QPDbgMNamePrinting (TQP_BOOL enable);

//
// Turn on and off the time stamp printing feature
//
void QPDbgMTimeStampPrinting (TQP_BOOL enable);

//
// Set the debug printing mode
//
typedef enum _EQP_DbgMPrintModes {
	QPDBGM_PRINTMODE_NONE = 0,			// no debugging output generated
	QPDBGM_PRINTMODE_LOCAL,				// output goes live to local port
	QPDBGM_PRINTMODE_LOCAL_DEFERRED,	// output goes to local port, but is
										//	delayed to a lower priority thread
	QPDBGM_PRINTMODE_EXTERNAL,			// output is buffered and processed by
										//	an external	entity (e.g. qapp)

	QPDBGM_PRINTMODE_NUM
} EQP_DbgMPrintModes;

EQP_Errors QPDbgMSetPrintingMode (EQP_DbgMPrintModes mode);
EQP_DbgMPrintModes QPDbgMGetPrintingMode (void);

//
// Set a component's debug level
//
EQP_Errors QPDbgMSetCompLevel (TQP_DbgMCompID, unsigned long dbgLvl);

//
// Get a component's debug level
//
EQP_Errors QPDbgMGetCompLevel (TQP_DbgMCompID, unsigned long * dbgLvlP);

#ifdef QP_DBG_TO_BUFFER
//
// Set a component's debug level
//
EQP_Errors QPDbgMSetBufCompLevel (TQP_DbgMCompID, unsigned long dbgLvl);

//
// Get a component's debug level
//
EQP_Errors QPDbgMGetBufCompLevel (TQP_DbgMCompID, unsigned long * dbgLvlP);

EQP_Errors QPDbgMSetBufInit(void);
EQP_Errors QPDbgMSetBufDump(const char * location);
#endif

//
// Get registered components list
//
EQP_Errors QPDbgMGetCompsList (TQP_DbgMCompID *, unsigned long * numP);

//
// Get registered component's name
//
EQP_Errors QPDbgMGetCompName (TQP_DbgMCompID, char *);


#ifdef __cplusplus
}
#endif

#endif
