///////////////////////////////////////////////////////////////////////////////
//     
//
// Qpdevcntl.h
//
// Description:
//
//  This is the definations of the codec chip control
//
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __QPDEVCNTL_H__
#define __QPDEVCNTL_H__

#include "qptypes.h"

#define QPDEVCNTL_VER_MAJOR	3
#define QPDEVCNTL_VER_MINOR	4

// QPDEVCNTL_VER 1.0 -> 2.0
// - support configurable silk frame length (20ms/40ms/60ms)
// QPDEVCNTL_VER 2.0 -> 2.1
// - add mode field in QPCODEC_DEC_AUDIO_CONTROL for select g711 a-law/u-law mode
// QPDEVCNTL_VER 2.1 -> 2.2
// - change VI OSD support
// QPDEVCNTL_VER 2.2 -> 2.3
// - add new misc command LargeCompressBufferControl
// QPDEVCNTL_VER 2.3 -> 2.4
// - add multi_slice_mode and low_latency_mode for artesa extension 340
// QPDEVCNTL_VER 2.4 -> 2.5
// - add new control item QPCODEC_IS_CODEC_ERROR to report codec error status
// QPDEVCNTL_VER 2.5 -> 2.6
// - add new control item QPCODEC_PROP_AUDIO_INPUT_VOLUME and QPCODEC_PROP_AUDIO_OUTPUT_VOLUME
// - enlarge QPCODEC_AUDIO_CONTROL_EX and QPCODEC_DEC_AUDIO_CONTROL size
// QPDEVCNTL_VER 2.6 -> 3.0
// - change 2.6 to 3.0 because 2.5 -> 2.6 extend some control structure
// QPDEVCNTL_VER 3.0 -> 3.1
// - add new misc command EnableLowBitrateMode
// QPDEVCNTL_VER 3.1 -> 3.2
// - add HDCP_STATE_BLOCKING information to QPCODEC_IS_CODEC_ERROR control
// QPDEVCNTL_VER 3.2 -> 3.3
// - add lr_swap to QPCODEC_AUDIO_CONTROL_EX control
// - add new misc command EnableAVDiscardControl
// - add new misc command UseSWPTS
// QPDEVCNTL_VER 3.3 -> 3.4
// - add intra_refresh_mode to QPCODEC_RATE_CONTROL control
// - add ForceRemote to QPCODEC_DEC_DISP_ATTRIB control
// - add QPDEC_DISP_ATTRIB_1080p define for TVSystem field of QPCODEC_DEC_DISP_ATTRIB control
// - add decDispForceRemote to QPCODEC_DEC_ALL_SETTINGS control

// Property set for QldevProperties
//

// {FAE408DC-E1D4-472b-AC63-9A99A155410D}
static const TQP_GUID PROPSETID_QPCODEC_CONTROL =
{ 0xfae408dc, 0xe1d4, 0x472b, { 0xac, 0x63, 0x9a, 0x99, 0xa1, 0x55, 0x41, 0xd } };

typedef enum _QPPROPERTY_QPCODEC_CONTROL
{
	// Encoder
	//
	QPCODEC_PROP_SYSTEM_CONTROL			= 0,		// Get & Set
	QPCODEC_PROP_PICTURE_RESOLUTION		= 1,		// Get & Set
	QPCODEC_PROP_INPUT_CONTROL			= 2,		// Get & Set
	QPCODEC_PROP_RATE_CONTROL			= 3,		// Get & Set
	QPCODEC_PROP_VBR_BIT_RATE			= 4,		// Get & Set
	QPCODEC_PROP_GOP_LOOP_FILTER		= 5,		// Get & Set
	QPCODEC_PROP_ET_CONTROL				= 6,		// Get & Set
	QPCODEC_PROP_FILTER_CONTROL			= 7,		// Get & Set
	QPCODEC_PROP_OUT_PICTURE_RESOLUTION = 8,		// Get & Set (Artesa)
	QPCODEC_PROP_BLOCK_SIZE				= 9,		// Get & Set (Artesa new definaton)
	QPCODEC_PROP_ENC_OSD_SETTINGS		= 10,
	QPCODEC_PROP_AUDIO_CONTROL_PARAM	= 11,		// Get & Set
	QPCODEC_PROP_AUDIO_CONTROL_EX		= 12,		// Get & Set
	QPCODEC_PROP_VIOSD_TABLE_ADDR= 13,		// Get
	QPCODEC_PROP_GET_CUR_VID_BUF_INFO	= 14,		// Get
	QPCODEC_PROP_REL_CUR_VID_BUF		= 15,		// Set
	QPCODEC_PROP_INSERT_I_FRAME			= 16,		// Set
	QPCODEC_PROP_INSERT_USER_DATA		= 17,		// Set
	QPCODEC_PROP_MP4_VID_NUM_BLOCK		= 18,		// Get & Set
	QPCODEC_PROP_AUDIO_ENHANCEMENT		= 19,		// Get & Set
	QPCODEC_PROP_ENC_STOP_MODE			= 20,		// Get & Set
	QPCODEC_PROP_INDEX_CAP_FREQ			= 21,		// Get & Set
	QPCODEC_PROP_ENC_VID_PADDING		= 22,		// Get & Set
	QPCODEC_PROP_ENC_FREEZE_VIDEO		= 23,		// Get & Set
	QPCODEC_PROP_ENC_STILL_VIDEO_INPUT	= 24,		// Get & Set
	QPCODEC_PROP_ENC_CAP_MODE			= 25,		// Get & Set
	QPCODEC_PROP_ENC_GET_A_FRAME		= 26,		// Set
	QPCODEC_PROP_ENC_MJPEG_QUALITY		= 27,		// Get & Set
	QPCODEC_PROP_ENC_MJPEG_FRAME_BUF	= 28,		// Get & Set
	QPCODEC_PROP_ENC_ALL_SETTINGS		= 29,		// Get & Set


	// Decoder
	//
	QPCODEC_PROP_DEC_STREAM_ID			= 30,		// Get & Set
	QPCODEC_PROP_DEC_OUTPUT_MODE		= 31,		// Get & Set
	QPCODEC_PROP_DEC_RESOLUTION			= 32,		// Get & Set
	QPCODEC_PROP_DEC_STOP_MODE			= 34,		// Get & Set
	QPCODEC_PROP_DEC_TRICK_MODE			= 35,		// Get & Set
	QPCODEC_PROP_DEC_DISP_ATTRIB		= 36,		// Get & Set
	QPCODEC_PROP_DEC_DISP_MODE			= 37,		// Get & Set
	QPCODEC_PROP_DEC_GET_PLAY_INFO		= 38,		// Get
	QPCODEC_PROP_DEC_AUDIO_CONTROL		= 39,		// Get & Set
	QPCODEC_PROP_DEC_DATA_XFER_CONTROL	= 40,		// Get & Set
	QPCODEC_PROP_DEC_ALL_SETTINGS		= 49,		// Get & Set


	// Misc
	//
	QPCODEC_PROP_BEGIN_CHANNEL_CHANGE	= 50,		// Set
	QPCODEC_PROP_END_CHANNEL_CHANGE		= 51,		// Set
	QPCODEC_PROP_ENCODER_STATUS			= 52,		// Get
	QPCODEC_PROP_DECODER_STATUS			= 53,		// Get
	QPCODEC_PROP_656_OUT_CNTL			= 54,		// Get & Set
	QPCODEC_PROP_I2S_OUT_CNTL			= 55,		// Get & Set
	QPCODEC_PROP_VOU_CONFIG				= 56,		// Get & Set
	QPCODEC_PROP_AOU_CONFIG				= 57,		// Get & Set
	QPCODEC_PROP_POWER_STATE			= 58,		// Get & Set
	QPCODEC_PROP_ERROR_RECOVERY			= 59,		// Get & Set

	// GPIO
	//
	QPCODEC_GPIO_BIT_DIRECTION			= 60,		// Get & Set
	QPCODEC_GPIO_BIT_VALUE				= 61,		// Get & Set


	QPCODEC_PROP_OSD_MEMORY				= 62,		// Get & Set
	QPCODEC_PROP_DECIMATION				= 63,		// Get & Set
	QPCODEC_PROP_DEINTERLACE			= 64,		// Get & Set
	QPCODEC_PROP_EXTERNAL_TRIGGER_TO_SYNC  = 65,	// Get & Set
	QPCODEC_PROP_PTS_COUNTER_RESET		= 66,		// Get & Set
	QPCODEC_PROP_RATE_CONTROL_EX		= 67,		// Get & Set
	QPCODEC_PROP_LARGE_COMPRESS_BUFFER_CONTROL	= 68, 	// Get & Set
	QPCODEC_PROP_USE_ARTESA_EXT_340			= 69,		// Get

	// Codec
	//
	QPCODEC_PROP_SYS_FUNCTION			= 70,		// Get & Set (Artesa)
	QPCODEC_PROP_SYS_LINK				= 71,		// Get & Set (Artesa)
	QPCODEC_PROP_SYS_CONFIG				= 72,		// Get & Set (Artesa)

	QPCODEC_IS_CODEC_ERROR              = 80,       // Get

	QPCODEC_PROP_AUDIO_INPUT_VOLUME		= 86,		// Set (Artesa)
	QPCODEC_PROP_AUDIO_OUTPUT_VOLUME	= 87,		// Set (Artesa)

	//
	QPCODEC_PROP_ENABLE_LOW_BITRATE_MODE	= 90, 	// Get & Set
	QPCODEC_PROP_FILTER_OUT_HDCP_CONTENT	= 91,		// Get
	QPCODEC_PROP_ENABLE_AV_DISCARD_CONTROL	= 92, 	// Get & Set
	QPCODEC_PROP_USE_SW_PTS					= 93, 	// Get & Set

//peter	
	QPCODEC_PROP_VIU_SYN_CCODE_1				= 98, 	// Get & Set [H.P.LIN]
	QPCODEC_PROP_VIU_SYN_CCODE_2				= 99, 	// Get & Set [H.P.LIN]

	// Ctrl Version
	QPCODEC_PROP_CTRL_VERSION			= 100,		// Get

	QPCODEC_PROP_END					= QPCODEC_PROP_CTRL_VERSION

} QPPROPERTY_QPCODEC_CONTROL;



#pragma pack(push, 1)


// stream type
#define STREAM_TYPE_PROGRAM_STREAM		0
#define STREAM_TYPE_TRANSPORT_STREAM	1
#define STREAM_TYPE_MPEG4_STREAM		2
#define STREAM_TYPE_AES_VES_STREAM		3
#define STREAM_TYPE_VID_STREAM			4
#define STREAM_TYPE_AUD_STREAM			5
#define STREAM_TYPE_YUV_PCM				6
#define STREAM_TYPE_YUV					7
#define STREAM_TYPE_PCM					8
#define STREAM_TYPE_RAW_VBI				9
#define STREAM_TYPE_MOTION_DETECT_MODE	10
#define STREAM_TYPE_JPEG				11
#define STREAM_TYPE_MJPEG				12
#define STREAM_TYPE_AUTO				15	// decoder only


// Encoder
//

typedef struct _QPCODEC_SYSTEM_CONTROL
{
	struct
	{
		TQP_UINT32	stream_type;
		TQP_UINT32	profile;
		TQP_UINT32	level;
		TQP_UINT32	cap_mode;		// beringer
		TQP_UINT32	ff_mode;
		TQP_UINT32	spsr_freq;
		TQP_UINT32	v_mode;
		TQP_UINT32	cabac_init_idc;
		TQP_UINT32	version;
	} Beringer;
	struct
	{
		TQP_UINT32	stream_type;	// artesa new defination
		TQP_UINT32	stream_data;	// artesa
		TQP_UINT32	profile;
		TQP_UINT32	level;
		TQP_UINT32	ff_mode;
		TQP_UINT32	spsr_freq;
		TQP_UINT32	v_mode;
		TQP_UINT32	cabac_init_idc;
		TQP_UINT32	version		: 16;	// pack version and xfer_mode into 32 bit
		TQP_UINT32	xfer_mode	: 16;	// so the total size of the data structure is
										// compatible with old defination
		TQP_UINT32	reserved1;
		TQP_UINT32	reserved2;
	} Artesa;
} QPCODEC_SYSTEM_CONTROL, *PQPCODEC_SYSTEM_CONTROL;


// stream type (artesa defination)
#define ARTESA_STREAM_TYPE_PROGRAM		0
#define ARTESA_STREAM_TYPE_TRANSPORT	1
#define ARTESA_STREAM_TYPE_MPEG4		2
#define ARTESA_STREAM_TYPE_ELEMENTARY	3
#define ARTESA_STREAM_TYPE_RAW			4
#define ARTESA_STREAM_TYPE_SPECIAL_1	5
#define ARTESA_STREAM_TYPE_JPEG			6


// stream data - bitmap - (artesa only)
#define ARTESA_STREAM_DATA_VIDEO		1
#define ARTESA_STREAM_DATA_AUDIO		2
#define ARTESA_STREAM_DATA_VBI			4
#define ARTESA_STREAM_DATA_USER			8

// profile
#define PROFILE_BASELINE				0
#define PROFILE_MAIN					1
#define PROFILE_EXTENDED				2
#define PROFILE_420_10					3
#define PROFILE_422_8					4
#define PROFILE_422_10					5
#define PROFILE_444_12					6

// level
#define LEVEL_1							0
#define LEVEL_1_B						1
#define LEVEL_1_1						2
#define LEVEL_1_2						3
#define LEVEL_1_3						4
#define LEVEL_2							5
#define LEVEL_2_1						6
#define LEVEL_2_2						7
#define LEVEL_3							8
#define LEVEL_3_1						9
#define LEVEL_3_2						10
#define LEVEL_4							11
#define LEVEL_4_1						12
#define LEVEL_4_2						13
#define LEVEL_5							14
#define LEVEL_5_1						15

// cap mode (beringer only)
#define CAP_MODE_HIU					0
#define CAP_MODE_VIU					1
#define CAP_MODE_DEC					2

// ff mode
#define FF_MODE_AUTO					0
#define FF_MODE_FRAME_ONLY				1
#define FF_MODE_FIELD_ONLY				2

// v mod
#define V_MODE_CAVLC					0
#define V_MODE_CABAC					1

// xfer_mode
#define XFER_MODE_FRAME					0
#define XFER_MODE_BLOCK					1

// bytes_swap
#define BYTES_SWAP_BY_DRIVER			0
#define BYTES_SWAP_BY_FW				1


typedef struct _QPCODEC_PICTURE_RESOLUTION
{
	struct
	{
		TQP_UINT32	pic_width;
		TQP_UINT32	v_scale;
		TQP_UINT32	pic_height;
		TQP_UINT32	h_scale;
	} Beringer;
	struct
	{
		TQP_UINT32	in_pic_width;
		TQP_UINT32	reserved1;
		TQP_UINT32	in_pic_height;
		TQP_UINT32	reserved2;
	} Artesa;
} QPCODEC_PICTURE_RESOLUTION, *PQPCODEC_PICTURE_RESOLUTION;


// scale factor (beringer only)
#define FILTER_MODE_NONE				0
#define FILTER_MODE_1_2					1
#define FILTER_MODE_1_4					2


typedef struct _QPCODEC_INPUT_CONTROL
{
	struct
	{
		TQP_UINT32	mode;			// beringer
		TQP_UINT32	src;			// beringer
		TQP_UINT32	dst;			// beringer
		TQP_UINT32	bl_e;			// beringer
		TQP_UINT32	black_line_num;	// beringer
		TQP_UINT32	in_frame_rate;
		TQP_UINT32	out_frame_rate;
		TQP_UINT32	fmt;
		TQP_UINT32	pd_en;	// 1 : enable 3:2 pulldown
		TQP_UINT32	ck_edge;
	} Beringer;
	struct
	{
		TQP_UINT32	scan_format;	// artesa
		TQP_UINT32	src_mode;		// artesa
		TQP_UINT32	sync_mode;		// artesa
		TQP_UINT32	data_type;		// artesa
		TQP_UINT32	bytes_swap;		// fw-layer bytes swap, for encode output channel
		TQP_UINT32	in_frame_rate;
		TQP_UINT32	out_frame_rate;
		TQP_UINT32	fmt;
		TQP_UINT32	pd_en;	// 1 : enable 3:2 pulldown
		TQP_UINT32	ck_edge;
		TQP_UINT32	force_60p_in;
		TQP_UINT32	reserved1;
		TQP_UINT32	reserved2;
	} Artesa;
} QPCODEC_INPUT_CONTROL, *PQPCODEC_INPUT_CONTROL;

// mode
#define INPUT_MODE_PROG					0
#define INPUT_MODE_PROG_SRC_INTERLACE_FORMAT	1
#define INPUT_MODE_INTERLACE			2
#define INPUT_MODE_SPECIAL_1			3
#define INPUT_MODE_SPECIAL_2			4

// src, dest (Beringer)
#define INPUT_CONTROL_VIU				0
#define INPUT_CONTROL_HIU_COMPRESSED	1
#define INPUT_CONTROL_HIU_RAW			2
#define INPUT_CONTROL_VOU				3

// scan_format (Artesa)
#define SCAN_FORMAT_INTERLACE			0
#define SCAN_FORMAT_PROG				1
#define SCAN_FORMAT_SPECIAL_1			2	// Micron
#define SCAN_FORMAT_SPECIAL_2			3	// OmniVision
#define SCAN_FORMAT_SPECIAL_3			4	// AcuteLogic


// src_mode (Artesa)
#define SRC_MODE_ITU656					0
#define SRC_MODE_HDMI					1
#define SRC_MODE_HIU					2
#define SRC_MODE_HDMI_DOUBLE_CLK		3

// sync_mode (Artesa)
//
// 0  : 1920x1080i @59.94/60Hz (Format 5)
// 0  : 1920x1080p @59.94/60Hz (Format 16), @29.97/30Hz (Format 34)
// 0  : 1440x1088i @59.94/60Hz
// 1  : 1440x576p @50Hz (Format 29 & 30)
// 3  : 1440x480p @59.94/60Hz (Format 14 & 15)
// 0  : 1280x720p @59.94/60Hz (Format 4)
// 3  : 720(1440)x480i @59.94/60Hz (Format 6 & 7)
// 3  : 720(1440)x576i @59.94/60Hz (Format 21 & 22)
// 3  : 720x480p @59.94/60Hz (Format 2 & 3)
// 3  : 704x576p @50Hz
// 3  : 720x480i @59.94/60Hz
// 3  : 704x576i @50Hz
// 3  : 640x480p @59.94/60Hz (Format 1)
// 3  : 640x480i @60Hz
// 0  : 176x128i @59.94/60Hz (Format CUSTOM_176x128i)
// 2  : HDMI (VSYNC: neg; HSYNC: pos), SD (frame_valid: neg; line_valid: pos)
// 3  : HDMI (VSYNC: neg; HSYNC: neg), SD (frame_valid: neg; line_valid: neg)
// 0  : HDMI (VSYNC: pos; HSYNC: pos), SD (frame_valid: pos; line_valid: pos)
// 1  : HDMI (VSYNC: pos; HSYNC: neg), SD (frame_valid: pos; line_valid: neg)
//
#define SYNC_MODE_1920x1080i			0
#define SYNC_MODE_1920x1080p			1
#define SYNC_MODE_1440x1088i			2
#define SYNC_MODE_1440x576p				3
#define SYNC_MODE_1440x480p				4
#define SYNC_MODE_1280x720p				5
#define SYNC_MODE_720_1440x480i			6
#define SYNC_MODE_720_1440x576i			7
#define SYNC_MODE_720x480p				8
#define SYNC_MODE_704x576p				9
#define SYNC_MODE_720x480i				10
#define SYNC_MODE_704x576i				11
#define SYNC_MODE_640x480p				12
#define SYNC_MODE_640x480i				13
#define SYNC_MODE_176x128i				14
#define SYNC_MODE_NEG_POS				15
#define SYNC_MODE_NEG_NEG				16
#define SYNC_MODE_POS_POS				17
#define SYNC_MODE_POS_NEG				18



// data_type (Artesa)
#define DATA_TYPE_H264					0
#define DATA_TYPE_MPEG2					1
#define DATA_TYPE_JPEG					2
#define DATA_TYPE_RAW					3

// in_frame_rate
#define IN_FRAME_RATE_NTSC				0
#define IN_FRAME_RATE_PAL				1
#define IN_FRAME_RATE_SECAM				2
#define IN_FRAME_RATE_ABSOLUTE			0x40

// fmt
#define FMT_PROGRESSIVE					0
#define FMT_INTERLACE					1

// clk
#define CK_EDGE_POSITIVE				0
#define CK_EDGE_NEGATIVE				1


typedef struct _QPCODEC_RATE_CONTROL
{
	struct
	{
		TQP_UINT32	avg_bit_rate;
		TQP_UINT32	qp_update_freq;
		TQP_UINT32	reserved1;
		TQP_UINT32	reserved2;
		TQP_UINT32	vbr;
	} Beringer;
	struct
	{
		TQP_UINT32	avg_bit_rate;
		TQP_UINT32	qp_update_freq;
		TQP_UINT32	mode;			// artesa //20140210, firmware never use this field anymore, keep for backward compatible
		TQP_UINT32	fixed;			// artesa
		TQP_UINT32	vbr;
		TQP_UINT32	intra_refresh_mode; //20140210, bit assigned to intra_refresh_mode
		TQP_UINT32	reserved2;
	} Artesa;
} QPCODEC_RATE_CONTROL, *PQPCODEC_RATE_CONTROL;

// mode (Artesa)
#define RATE_CONTROL_MODE_VIU			0
#define RATE_CONTROL_MODE_HIU			1
//#define RATE_CONTROL_MODE_VIU_FPGA	2
//#define RATE_CONTROL_MODE_PRELOAD		3

// fixed (Artesa)
#define RATE_CONTROL_NORMAL				0
#define RATE_CONTROL_FIXED				1

// intra_refresh_mode (Artesa)
#define RATE_CONTROL_INTRA_REFRESH_MODE_OFF		0
#define RATE_CONTROL_INTRA_REFRESH_MODE_ON		1

typedef struct _QPCODEC_VBR_BIT_RATE
{
	TQP_UINT32	min_bit_rate;
	TQP_UINT32	max_bit_rate;
	TQP_UINT32	reserved1;
	TQP_UINT32	reserved2;
} QPCODEC_VBR_BIT_RATE, *PQPCODEC_VBR_BIT_RATE;



typedef struct _QPCODEC_FILTER_CONTROL
{
	TQP_UINT32	start_pixel;
	TQP_UINT32	fl_en;
	TQP_UINT32	filter_strength;
	TQP_UINT32	start_line;
	TQP_UINT32	dis_skp;
	TQP_UINT32	reserved1;
	TQP_UINT32	reserved2;
} QPCODEC_FILTER_CONTROL, *PQPCODEC_FILTER_CONTROL;



typedef struct _QPCODEC_GOP_LOOP_FILTER
{
	struct
	{
		TQP_UINT32	intra_period;
		TQP_UINT32	lp_filter;
		TQP_UINT32	dis_lf_idc;
		TQP_INT32	alpha_offset;
		TQP_INT32	beta_offset;
		TQP_UINT32	B_num;
		TQP_UINT32	reserved1;
		TQP_UINT32	reserved2;
		TQP_UINT32	IDR;
	} Beringer;
	struct
	{
		TQP_UINT32	intra_period;
		TQP_UINT32	lp_filter;
		TQP_UINT32	dis_lf_idc;
		TQP_INT32	alpha_offset;
		TQP_INT32	beta_offset;
		TQP_UINT32	B_num;
		TQP_UINT32	ref_n;
		TQP_UINT32	t8x8;
		TQP_UINT32	ref_p;
		TQP_UINT32	IDR;
		TQP_UINT32	reserved1;
		TQP_UINT32	reserved2;
	} Artesa;
} QPCODEC_GOP_LOOP_FILTER, *PQPCODEC_GOP_LOOP_FILTER;

// ref_n (Artesa)
#define GOP_LOOP_ALL_REF				0
#define GOP_LOOP_ONE_REF				1

// ref_p (Artesa)
#define GOP_LOOP_PRE_SELECT_ONE_REF_OFF	0
#define GOP_LOOP_PRE_SELECT_ONE_REF_ON	1

#define IDR_OPEN_GOP					0
#define IDR_CLOSED_GOP					1


// QPCODEC_PROP_OUT_PICTURE_RESOLUTION (artesa)
typedef struct _QPCODEC_OUT_PICTURE_RESOLUTION
{
	TQP_UINT32	out_pic_width;
	TQP_UINT32	out_pic_height;
	TQP_UINT32	reserved1;
	TQP_UINT32	reserved2;
} QPCODEC_OUT_PICTURE_RESOLUTION, *PQPCODEC_OUT_PICTURE_RESOLUTION;


// QPCODEC_PROP_BLOCK_SIZE (artesa)
typedef struct _QPCODEC_BLOCK_SIZE
{
	TQP_UINT32	block_size;
	TQP_UINT32	slice_group;
	TQP_UINT32	multi_slice_mode; // for artesa extension 340
	TQP_UINT32	low_latency_mode; // for artesa extension 340
} QPCODEC_BLOCK_SIZE, *PQPCODEC_BLOCK_SIZE;


// Beringer only
typedef struct _QPCODEC_ET_CONTROL
{
	TQP_UINT32	et_threshold_offset;
	TQP_UINT32	et_threshold_max;
	TQP_UINT32	et;
	TQP_UINT32	et_sad_threshold;
	TQP_UINT32	channel_data_type;
	TQP_UINT32	ref_num;
	TQP_UINT32	reserved1;
	TQP_UINT32	reserved2;
} QPCODEC_ET_CONTROL, *PQPCODEC_ET_CONTROL;


// channel_data_type (Beringer)
#define DATA_CHANNEL_TYPE_MPEG_2_HIU	0
#define DATA_CHANNEL_TYPE_YUV_2_HIU		1
#define DATA_CHANNEL_TYPE_YUV_2_VOU		2

// ref_num (Beringer)
#define REF_NUM_3_FRAMES				0
#define REF_NUM_1_FRAMES				1
#define REF_NUM_2_FRAMES				2


typedef struct _QPCODEC_AUDIO_CONTROL_PARAM
{
	TQP_UINT32	bit_rate;
	TQP_UINT32	ch_sel;
	TQP_UINT32	sample_rate;
	TQP_UINT32	aou_pass_en;
	TQP_UINT32	ai_msb;
	TQP_UINT32	lrclk_i;
	TQP_UINT32	bclk_i;
	TQP_UINT32	ai_i2s;
	TQP_UINT32	ai_rj;
	TQP_UINT32	ai_m;
	TQP_UINT32	ch_num;
	TQP_UINT32	pts;
	TQP_UINT32	disAi;
	TQP_UINT32	raw;
	TQP_UINT32	audio_type;
	TQP_UINT32	reserved1;
	TQP_UINT32	reserved2;
} QPCODEC_AUDIO_CONTROL_PARAM, *PQPCODEC_AUDIO_CONTROL_PARAM;


// ch_sel (when using mono input)
#define CH_SEL_MUX						0
#define CH_SEL_LEFT						1
#define CH_SEL_RIGHT					2

// sample_rate
#define SAMPLE_RATE_44_1K				0
#define SAMPLE_RATE_48K					1
#define SAMPLE_RATE_32K					2
#define SAMPLE_RATE_22_5K				4
#define SAMPLE_RATE_24K					5
#define SAMPLE_RATE_16K					6
#define SAMPLE_RATE_8K					8

#define SAMPLE_RATE_88_2K				9
#define SAMPLE_RATE_96K					10
#define SAMPLE_RATE_64K					11
#define SAMPLE_RATE_176_4K				12
#define SAMPLE_RATE_192K					13
#define SAMPLE_RATE_128K					14
#define AI_MSB_LSB						0
#define AI_MSB_MSB						1

#define LRCLK_I_LEFT_LOW				0
#define LRCLK_I_LEFT_HIGH				1

#define BCLK_I_RISING_EDGE				0
#define BCLK_I_FALLING_EDGE				1

#define AI_I2S_NORMAL					0
#define AI_I2S_PROGRAM					1

#define AI_RJ_LEFT_JUSTIFIED			0
#define AI_RJ_RIGHT_JUSTIFIED			1

#define AI_M_SLAVE						0
#define AI_M_MASTER						1

#define AUDIO_TYPE_LPCM					0
#define AUDIO_TYPE_G726					1
#define AUDIO_TYPE_AAC					2
#define AUDIO_TYPE_MP1					3
#define AUDIO_TYPE_MP2					4
#define AUDIO_TYPE_MP3					5
#define AUDIO_TYPE_AC3					6
#define AUDIO_TYPE_G723					7
#define AUDIO_TYPE_G711					8
#define AUDIO_TYPE_RAW					9
#define AUDIO_TYPE_G729					10
#define AUDIO_TYPE_SILK					11


typedef struct _QPCODEC_AUDIO_CONTROL_EX
{
	// AAC
	//
	TQP_UINT32	aac_ver;
	TQP_UINT32	h_type;
	TQP_UINT32	TNS;
	TQP_UINT32	IS;
	TQP_UINT32	PNS;
	TQP_UINT32	MS;
	TQP_UINT32	cutoff_freq;

	// G711
	//
	TQP_UINT32	mode;
	TQP_UINT32	block_size;

	// LPCM
	//
	TQP_UINT32	sample_cnt;				// maimum 512

	TQP_UINT32	frame_len;
	TQP_UINT32	raw_audio_type;
	TQP_UINT32	downsample;
	TQP_UINT32	lr_swap;
	TQP_UINT32	reserved2;
} QPCODEC_AUDIO_CONTROL_EX, *PQPCODEC_AUDIO_CONTROL_EX;

// AAC

// aac_ver
#define AAC_VER_MPEG_2					2
#define AAC_VER_MPEG_4					4

// h_type
#define H_TYPE_ADTS						0
#define H_TYPE_GA						1

// G711

// mode
#define MODE_A_LAW						0
#define MODE_U_LAW						1

// block_size
#define BLOCK_SIZE_80					0
#define BLOCK_SIZE_160					1
#define BLOCK_SIZE_240					2

// SILK
// block_size
#define FRAME_SIZE_20					0
#define FRAME_SIZE_40					1
#define FRAME_SIZE_60					2


#define MAX_FONT_NUM 8
#define MAX_TEXT_NUM 128

#pragma pack(push, 4)
typedef struct
{
    TQP_UINT8 *pbyFont; // Base address
    TQP_UINT32 dwType;   // 0: 8x8, 1: 16x16, 2:32x32
    TQP_UINT32 dwLen;    // character number
    TQP_UINT32 dwID;     // Font ID number, set by driver
    TQP_UINT32 dwAsciiOffset;
} TQLFont, *PTQLFont;

typedef struct
{
    TQLFont *aFont;
    TQP_UINT32 dwIndex;
    TQP_UINT32 dwFontOffset;
} TQLFontList, *PTQLFontList;

typedef struct
{
    TQP_UINT8 *pbyText;
    TQP_UINT32 bEn;
    TQP_UINT32 dwX;
    TQP_UINT32 dwY;
    TQP_UINT32 dwLen;
    TQP_UINT32 dwFontID;
    TQP_UINT32 dwID;
} TQLText, *PTQLText;

typedef struct
{
    TQLText *aText;
    TQP_UINT32 dwIndex;
} TQLTextList, *PTQLTextList;

/*! A data structure of \b QLVIOSD object used in function \b QLVIOSD_SetTime
 * and \b QLVIOSD_SetText to set the new text properties. */
typedef struct
{
    /*! Enable the time tag or not. */
    TQP_UINT32 bEn;

    /*! X-axis position of time tag string. */
    TQP_UINT64 dwX;

    /*! Y-axis position of time tag string. */
    TQP_UINT64 dwY;

    /*! Enable year or not. */
    TQP_UINT32 bYearEn;

    /*! Enable month or not. */
    TQP_UINT32 bMonthEn;

    /*! Enable day or not. */
    TQP_UINT32 bDayEn;

    /*! Enable hour or not. */
    TQP_UINT32 bHourEn;

    /*! Enable minute or not. */
    TQP_UINT32 bMinEn;

    /*! Enable second or not. */
    TQP_UINT32 bSecEn;

    /*! Enable millisecond or not. */
    TQP_UINT32 bMSEn;

    /*! Year format. 0: YYYY, 1:'YY */
    TQP_UINT64 dwYearFmt;

    /*! Month format. 0: MM(00~12), 1: JAN~DEC */
    TQP_UINT64 dwMonthFmt;

    /*! Hour format. 0: 24hr, 1: AM/PM 12hr*/
    TQP_UINT64 dwHourFmt;

    /*! Separater between year and month, month and day. */
    TQP_INT8 cDateSep;

    /*! Separater between hour and minute, minute and second. */
    TQP_INT8 cTimeSep;

    /*! Separater between second and millisecond. */
    TQP_INT8 cMSSep;

    /*! Use font ID for the time tag string. */
    TQP_UINT64 dwFontID;

    TQP_UINT32 dwTimeTextID;
} TQLTime;

typedef struct _QPCODEC_VIOSD_CALTIME
{
    time_t curr_time;
    time_t next_time;
    TQP_UINT32 bTimeInit;
    TQP_UINT32 bSwitch;
    TQP_UINT32 bSecUpdate;
    TQP_UINT64 bTimerSec;
    TQP_UINT64 bTimerMilliSec;
    TQP_UINT32 bEnable;
    TQP_UINT32 dwFontID;
    TQP_UINT32 dwX;
    TQP_UINT32 dwY;
    TQP_UINT32 dwLen;
    TQP_UINT32 dwTime;
    TQP_UINT32 bMSEn;
    TQP_UINT8 byMSSep;
    TQP_UINT8 *pbyTime;
} QPCODEC_VIOSD_CALTIME;

typedef struct _QPCODEC_VIOSD
{
	TQP_UINT32 CmdType;
	TQP_UINT32 tIdx;
	TQP_UINT32 FontTableAddr;
	TQP_UINT32 TextListAddr;
	TQP_UINT32 TimestampAddr;
	TQP_UINT32 Enable;
	TQP_UINT32 Update;
	TQP_UINT32 GetAddrTimeout;

	TQLText Text;
	TQLTime Timestamp;
    TQLTextList TextList;
    TQLFontList FontList;
    QPCODEC_VIOSD_CALTIME CalTime;
} QPCODEC_VIOSD, *PQPCODEC_VIOSD;
#pragma pack(pop)

// QPCODEC_PROP_GET_CUR_VID_BUF_INFO
//
typedef struct _QPCODEC_GET_CUR_VID_BUF_INFO
{
	TQP_UINT32	HoldBuffer;
	TQP_UINT32	VidBufLumaAddr;		// in dwords
	TQP_UINT32	VidBufChromaAddr;	// in dwords
	TQP_UINT32	FrameLABufAddr;		// in dwords
	TQP_UINT32	TopLABufAddr;		// in dwords
	TQP_UINT32	BottomLABufAddr;	// in dwords
	TQP_UINT32	VidBufPTS;			// right shift 2 bits

	TQP_UINT32	reserved1;
	TQP_UINT32	reserved2;
} QPCODEC_GET_CUR_VID_BUF_INFO, *PQPCODEC_GET_CUR_VID_BUF_INFO;


// QPCODEC_PROP_INSERT_USER_DATA
//
typedef struct _QPCODEC_INSERT_USER_DATA
{
	TQP_UINT32	numUserData;
	TQP_UINT8	UserData[32];
	TQP_UINT32	reserved1;
	TQP_UINT32	reserved2;
} QPCODEC_INSERT_USER_DATA, *PQPCODEC_INSERT_USER_DATA;


typedef struct _QPCODEC_MP4_VID_NUM_BLOCK
{
	TQP_UINT32	num;
} QPCODEC_MP4_VID_NUM_BLOCK, *PQPCODEC_MP4_VID_NUM_BLOCK;


typedef struct _QPCODEC_AUDIO_ENHANCEMENT
{
	TQP_UINT32	gain1;
	TQP_UINT32	gain2;
	TQP_UINT32	add;
	TQP_UINT32	sub;
	TQP_UINT32	att1;
	TQP_UINT32	att2;
	TQP_UINT32	lgain;
	TQP_UINT32	rgain;
	TQP_UINT32	reserved1;
	TQP_UINT32	reserved2;
} QPCODEC_AUDIO_ENHANCEMENT, *PQPCODEC_AUDIO_ENHANCEMENT;



typedef struct _QPCODEC_ENC_STOP_MODE
{
	TQP_UINT32	stop_mode;
} QPCODEC_ENC_STOP_MODE, *PQPCODEC_ENC_STOP_MODE;

#define QP_ENC_STOP_IMMEDIATELY	0
#define QP_ENC_STOP_GOP			1
#define QP_ENC_STOP_AUTO		2


typedef struct _QPCODEC_INDEX_CAP_FREQ
{
	TQP_UINT32	freq;	// 32 - 128
} QPCODEC_INDEX_CAP_FREQ, *PQPCODEC_INDEX_CAP_FREQ;



// QPCODEC_PROP_ENC_VID_PADDING
//
typedef struct _QPCODEC_ENC_VID_PADDING
{
	TQP_BOOL	Enable;
} QPCODEC_ENC_VID_PADDING, *PQPCODEC_ENC_VID_PADDING;



// QPCODEC_PROP_ENC_FREEZE_VIDEO
//
typedef struct _QPCODEC_ENC_FREEZE_VIDEO
{
	TQP_BOOL	Enable;
} QPCODEC_ENC_FREEZE_VIDEO, *PQPCODEC_ENC_FREEZE_VIDEO;



// QPCODEC_PROP_ENC_STILL_VIDEO_INPUT
//
typedef struct _QPCODEC_ENC_STILL_VIDEO_INPUT
{
	TQP_BOOL	Enable;
} QPCODEC_ENC_STILL_VIDEO_INPUT, *PQPCODEC_ENC_STILL_VIDEO_INPUT;


// QPCODEC_PROP_ENC_CAP_MODE
//
typedef struct _QPCODEC_ENC_CAP_MODE
{
	TQP_UINT32	capMode;
	TQP_UINT32	trigMode;
	TQP_UINT32	gpioPin;
	TQP_UINT32	reserved1;
	TQP_UINT32	reserved2;
} QPCODEC_ENC_CAP_MODE, *PQPCODEC_ENC_CAP_MODE;


// capMode
#define QPCODEC_ENC_CAP_MODE_CONTINUOUS	0
#define QPCODEC_ENC_CAP_MODE_ON_DEMAND	1

// trigMode (bitmap)
#define QPCODEC_ENC_TRIG_MODE_API		0x01
#define QPCODEC_ENC_TRIG_MODE_GPIO		0x02

// EXTERNAL TRIGGER TO SYNC
typedef struct _QPCODEC_EXTERNAL_TRIGGER_MODE
{
	TQP_BOOL	Enable;
	TQP_UINT32	gpioPin;
	TQP_UINT32	reserved1;
	TQP_UINT32	reserved2;
} QPCODEC_EXTERNAL_TRIGGER_MODE, *PQPCODEC_EXTERNAL_TRIGGER_MODE;

// PTS COUNTER RESET to 0 vis GPIO trigger
typedef struct _QPCODEC_PTS_COUNTER_RESET
{
	TQP_BOOL	Enable;
	TQP_UINT32	gpioPin;
	TQP_BOOL	immediate;
	TQP_UINT32	reserved1;
	TQP_UINT32	reserved2;
} QPCODEC_PTS_COUNTER_RESET, *PQPCODEC_PTS_COUNTER_RESET;

// DECIMATION_FACTOR
typedef struct _QPCODEC_DECIMATION_FACTOR
{
	TQP_UINT32	input_format;
	TQP_UINT32	output_format;
	union
	{
		TQP_UINT32 read;
		struct
		{
			TQP_UINT32	v_denominator	: 8;	// bit 0 - 7
			TQP_UINT32	v_numerator		: 8;	// bit 8 - 15
			TQP_UINT32	h_denominator	: 8;	// bit 16 - 23
			TQP_UINT32	h_numerator		: 8;	// bit 24 - 31
		} factor;
	} scale_factor;
	TQP_UINT32	reserved1;
	TQP_UINT32	reserved2;
} QPCODEC_DECIMATION_FACTOR, *PQPCODEC_DECIMATION_FACTOR;

#define QPCODEC_DECIMATION_INPUT_QYUV	0x00
#define QPCODEC_DECIMATION_INPUT_YV12	0x01

#define QPCODEC_DECIMATION_OUTPUT_QYUV	0x00
#define QPCODEC_DECIMATION_OUTPUT_YUY2	0x01

#define QPCODEC_DEINTERLACE_NONE			0x00
#define QPCODEC_DEINTERLACE_TOP_FIELD_MODE	0x01
#define QPCODEC_DEINTERLACE_TWO_FIELD_MODE	0x02


typedef union _RATE_CONTROL_EX
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	base_qp_length    : 12;	// bit 0 - 11
		TQP_UINT32	local_activity_on : 1;	// bit 12
		TQP_UINT32	motion_detect_interval: 16;	//bit 13-28
		TQP_UINT32	reserved		  : 3;	// bit 29 - 31
	} Artesa;
} RATE_CONTROL_EX, *PRATE_CONTROL_EX;

#define QPCODEC_LARGE_COMPRESS_BUFFER_CONTROL_ENABLE	0x80000000

// QPCODEC_PROP_ENC_ALL_SETTINGS
//
typedef struct _QPCODEC_ENC_ALL_SETTINGS
{
	TQP_UINT32	EncSystemControl;
	TQP_UINT32	EncPictureResolution;
	TQP_UINT32	EncInputControl;
	TQP_UINT32	EncSyncMode;
	TQP_UINT32	EncRateControl;
	TQP_UINT32	EncRateControlEx;
	TQP_UINT32	EncBitRate;
	TQP_UINT32	EncFilterControl;
	TQP_UINT32	EncGopLoopFilter;
	TQP_UINT32	EncEtControl;
	TQP_UINT32	EncOutPictureResolution;
	TQP_UINT32	EncBlockXferSize;
	TQP_UINT32	EncAudioControlParam;
	TQP_UINT32	EncAudioControlExAAC;
	TQP_UINT32	EncAudioControlExG711;
	TQP_UINT32	EncAudioControlExLPCM;
	TQP_UINT32	EncIndexCapFreq;
	TQP_UINT32	EncMP4VideoBlockNumber;
	TQP_UINT32	EncStopMode;
	TQP_UINT32	EncEnableVidPadding;
	TQP_UINT32	EncMjpegQuality;
	TQP_UINT32	EncMjpegFrameBuffer;
	TQP_UINT32	EncDecimationInputFormat;
	TQP_UINT32	EncDecimationOutputFormat;
	TQP_UINT32	EncDecimationScaleFactor;
	TQP_UINT32	DeinterlaceMode;
	TQP_UINT32	EncExternalTriggerToSyncEnable;
	TQP_UINT32	EncExternalTriggerToSyncGpioPin;
	TQP_UINT32	EncPTSResetByTriggerEnable;
	TQP_UINT32	EncPTSResetByTriggerGpioPin;
	TQP_UINT32	EncPTSResetByTriggerImmediate;
	TQP_UINT32	EncAudioControlExSILK;
	TQP_UINT32	LargeCompressBufferControl;
	TQP_UINT32	EnableLowBitrateMode;
	TQP_UINT32	EnableAVDiscardControl;
	TQP_UINT32	UseSWPTS;
} QPCODEC_ENC_ALL_SETTINGS, *PQPCODEC_ENC_ALL_SETTINGS;



// Decoder
//

// QPCODEC_PROP_DEC_STREAM_ID
typedef union _QPCODEC_DEC_STREAM_ID_EX
{
	TQP_UINT32	read;
	struct
	{
		TQP_UINT32	stream_type	: 4;	// bit 0 - 3
		TQP_UINT32	stream_data	: 4;	// bit 4 - 7
		TQP_UINT32	auto_detect	: 1;	// bit 8
		TQP_UINT32	reserved	: 19;	// bit 9 - 27
		TQP_UINT32	bytes_swap	: 1;	// bit 28
		TQP_UINT32	version		: 2;	// bit 29 - 30
		TQP_UINT32	xfer_mode	: 1;	// bit 31
	} streamIdEx;
} QPCODEC_DEC_STREAM_ID_EX, *PQPCODEC_DEC_STREAM_ID_EX;



// QPCODEC_PROP_DEC_OUTPUT_MODE
// bit[15:0]
//
#define QPDEC_DEC_START_VOU				0
#define QPDEC_DEC_START_HIU				1

// QPCODEC_PROP_DEC_OUTPUT_MODE
// bit[31:16]
//
#define QPDEC_DEC_START_VOU_AUTO		0x00000
#define QPDEC_DEC_START_VOU_MANUAL		0x10000


// QPCODEC_PROP_DEC_AUDIO_CONTROL
//
typedef struct _QPCODEC_DEC_AUDIO_CONTROL
{
	TQP_UINT32	audio_type;
	TQP_UINT32	ch_num;
	TQP_UINT32	sample_rate;
	TQP_UINT32	mode;
	TQP_UINT32	upsample;
	TQP_UINT32	reserved1;
	TQP_UINT32	reserved2;
} QPCODEC_DEC_AUDIO_CONTROL, *PQPCODEC_DEC_AUDIO_CONTROL;


// QPCODEC_PROP_DEC_TRICK_MODE
//
typedef struct _QPCODEC_DEC_TRICK_MODE
{
	TQP_UINT32	playSpeed;
	TQP_UINT32	playDirection;
	TQP_UINT32	step;
	TQP_UINT32	PTS;
	TQP_UINT32	fieldPreference;
	TQP_UINT32	reserved1;
	TQP_UINT32	reserved2;
} QPCODEC_DEC_TRICK_MODE, *PQPCODEC_DEC_TRICK_MODE;

#define QPDEC_TRICK_SPD_PAUSE			0x0000
#define QPDEC_TRICK_SPD_NORMAL			0x0101
#define QPDEC_TRICK_SPD_2X				0x0102
#define QPDEC_TRICK_SPD_1_2X			0x0201
#define QPDEC_TRICK_SPD_4X				0x0104
#define QPDEC_TRICK_SPD_1_4X			0x0401
#define QPDEC_TRICK_SPD_8X				0x0108
#define QPDEC_TRICK_SPD_1_8X			0x0801
#define QPDEC_TRICK_SPD_SKIP_TO			0xFFFF

#define QPDEC_TRICK_DIR_FORWARD			0
#define QPDEC_TRICK_DIR_BACKWARD		1

#define QPDEC_TRICK_STEP_NONE			0
#define QPDEC_TRICK_STEP_ONE			1

#define QPDEC_TRICK_DISP_DEFAULT		0
#define QPDEC_TRICK_DISP_TOP			1
#define QPDEC_TRICK_DISP_BOTTOM			2
#define QPDEC_TRICK_DISP_BOTH			3


// QPCODEC_PROP_DEC_DISP_MODE
//

typedef struct _QPCODEC_DEC_DISP_MODE
{
  	TQP_UINT32	operation;
  	TQP_UINT32	displayMode;
	TQP_UINT32	reserved1;
	TQP_UINT32	reserved2;
} QPCODEC_DEC_DISP_MODE, *PQPCODEC_DEC_DISP_MODE;


#define QPDEC_DISP_MODE_NORMAL				0
#define QPDEC_DISP_MODE_STOP				1
#define QPDEC_DISP_MODE_FREEZE				2

#define QPDEC_DISP_MODE_INTERLACED_FRAME	0
#define QPDEC_DISP_MODE_TOP_FIELD			1
#define QPDEC_DISP_MODE_BOTTOM_FIELD		2
#define QPDEC_DISP_MODE_PROGRESSIVE_FRAME	3

// QPCODEC_PROP_DEC_DISP_ATTRIB
//

typedef union _QPDEC_DISP_ATTRIB_XY
{
	TQP_UINT32	Read;
	struct
	{
		TQP_UINT32	x			: 13;	// bit 0 - 12
		TQP_UINT32	reserved1	: 3;	// bit 13 - 15
		TQP_UINT32	y			: 12;	// bit 16 - 27
		TQP_UINT32	reserved2	: 4;	// bit 28 - 31
	} Attrib;
} QPDEC_DISP_ATTRIB_XY, *PQPDEC_DISP_ATTRIB_XY;


typedef struct _QPCODEC_DEC_DISP_ATTRIB
{
  	TQP_UINT32				TVSystem;
  	QPDEC_DISP_ATTRIB_XY	inputSize;
  	QPDEC_DISP_ATTRIB_XY	outputSize;
  	QPDEC_DISP_ATTRIB_XY	inputOrigin;
  	QPDEC_DISP_ATTRIB_XY	outputOrigin;
	TQP_UINT32				ForceRemote;
	TQP_UINT32				reserved2;
} QPCODEC_DEC_DISP_ATTRIB, *PQPCODEC_DEC_DISP_ATTRIB;


// Beringer
#define QPDEC_DISP_ATTRIB_NTSC			0
#define QPDEC_DISP_ATTRIB_PAL			1

// Artesa
#define QPDEC_DISP_ATTRIB_576i			0
#define QPDEC_DISP_ATTRIB_480i			1
#define QPDEC_DISP_ATTRIB_480p			2
#define QPDEC_DISP_ATTRIB_720p			3
#define QPDEC_DISP_ATTRIB_1080i			4
#define QPDEC_DISP_ATTRIB_1080p			5

// QPCODEC_PROP_DEC_GET_PLAY_INFO
//
typedef struct _QPCODEC_DEC_GET_PLAY_INFO
{
	TQP_UINT32	infoType;
	union _INFO
	{
		struct
		{
			TQP_UINT32	param1;
			TQP_UINT32	param2;
		} ALL;

		struct
		{
			TQP_UINT32	wrMem;
			TQP_UINT32	rdMem;
		} GET_SHARE_MEM;
	} INFO;
	TQP_UINT32	reserved1;
	TQP_UINT32	reserved2;
} QPCODEC_DEC_GET_PLAY_INFO, *PQPCODEC_DEC_GET_PLAY_INFO;


#define QPDEC_INFO_TYPE_MASTER_CLK		1
#define QPDEC_INFO_TYPE_CUR_DISP_PTS	2
#define QPDEC_INFO_TYPE_DISPLAYED_CNT	3
#define QPDEC_INFO_TYPE_DECODED_CNT		4
#define QPDEC_INFO_TYPE_GET_SHARE_MEM	5


// QPCODEC_PROP_DEC_STOP_MODE
typedef struct _QPCODEC_DEC_STOP_MODE
{
	TQP_UINT32	stop_mode;
	TQP_UINT32	stop_disp_mode;
	TQP_UINT32	PTS1;
	TQP_UINT32	PTS2;
	TQP_UINT32	reserved1;
	TQP_UINT32	reserved2;
} QPCODEC_DEC_STOP_MODE, *PQPCODEC_DEC_STOP_MODE;


#define QPDEC_DEC_STOP_IMMEDIATELY		0
#define QPDEC_DEC_STOP_EOF				1
#define QPDEC_DEC_STOP_AT_PTS			2

#define QPDEC_DISP_LAST_FRAME			0
#define QPDEC_DISP_BLACK				1


// QPCODEC_PROP_DEC_DATA_XFER_CONTROL
//
typedef struct _QPCODEC_DEC_DATA_XFER_CONTROL
{
	TQP_UINT32	xfer_mode;
	TQP_UINT32	minimum_size;
	TQP_UINT32	reserved1;
	TQP_UINT32	reserved2;
} QPCODEC_DEC_DATA_XFER_CONTROL, *PQPCODEC_DEC_DATA_XFER_CONTROL;


#define QPDEC_DEC_XFER_MODE_ALL			0
#define QPDEC_DEC_XFER_MODE_PARTIAL		1



// QPCODEC_PROP_DEC_ALL_SETTINGS
typedef struct _QPCODEC_DEC_ALL_SETTINGS
{
	TQP_UINT32	streamID;
	TQP_UINT32	outputMode;
	TQP_UINT32	decodeResolution;
	TQP_UINT32	decStopMode;
	TQP_UINT32	decStopDispMode;
	TQP_UINT32	decDispMode;
	TQP_UINT32	decDispTVSystem;
	TQP_UINT32	decDispInputSize;
	TQP_UINT32	decDispOutputSize;
	TQP_UINT32	decDispInputOrigin;
	TQP_UINT32	decDispOutputOrigin;
	TQP_UINT32	decXferMode;
	TQP_UINT32	decXferMinimumSize;
	TQP_UINT32	decDispForceRemote;
	TQP_UINT32	reserved1;
	TQP_UINT32	reserved2;
} QPCODEC_DEC_ALL_SETTINGS, *PQPCODEC_DEC_ALL_SETTINGS;



// QPCODEC_PROP_ENCODER_STATUS, QPCODEC_PROP_DECODER_STATUS
//
typedef struct _QPCODEC_CODEC_STATUS
{
	TQP_UINT64	ullCompressByteCnt;
	TQP_UINT32	ulYUVFrameCnt;
	TQP_UINT64	ullElapsedTime;
	TQP_UINT32	ulState;
	TQP_UINT32	ulError;
	TQP_UINT32	reserved1;
	TQP_UINT32	reserved2;
} QPCODEC_CODEC_STATUS, *PQPCODEC_CODEC_STATUS;

#define QPCODEC_CODEC_STATE_IDLE		0
#define QPCODEC_CODEC_STATE_RUN			1
#define QPCODEC_CODEC_STATE_STOP		2

#define QPCODEC_CODEC_ERROR_NONE		0x00000000
#define QPCODEC_CODEC_ERROR_IOERROR		0x00000001


// QPCODEC_PROP_656_OUT_CNTL, QPCODEC_PROP_I2S_OUT_CNTL
//
typedef struct _QPCODEC_OUT_CNTL
{
	TQP_BOOL		Enable;
} QPCODEC_OUT_CNTL, *PQPCODEC_OUT_CNTL;


// QPCODEC_PROP_VOU_CONFIG
//
typedef struct _QPCODEC_VOU_CONFIG
{
	TQP_UINT32	VOUMode;
	TQP_UINT32	VOUStartPixel;
	TQP_UINT32	VOUStartLine;
	TQP_UINT32	reserved1;
	TQP_UINT32	reserved2;
} QPCODEC_VOU_CONFIG, *PQPCODEC_VOU_CONFIG;

#define QPCODEC_VOUMODE_NTSC_INTERLACED	0
#define QPCODEC_VOUMODE_PAL_INTERLACED	1
#define QPCODEC_VOUMODE_PRPGRESS		2


// QPCODEC_PROP_AOU_CONFIG
//
typedef struct _QPCODEC_AOU_CONFIG
{
	TQP_UINT32	ao_en;
	TQP_UINT32	ao_msb;
	TQP_UINT32	lrclk_i;
	TQP_UINT32	bclk_i;
	TQP_UINT32	ao_i2s;
	TQP_UINT32	ao_rj;
	TQP_UINT32	ao_s;
	TQP_UINT32	reserved1;
	TQP_UINT32	reserved2;
} QPCODEC_AOU_CONFIG, *PQPCODEC_AOU_CONFIG;

#define AO_ENABLE						1
#define AO_DISABLE						0

#define AO_MSB_LSB						0
#define AO_MSB_MSB						1

#define AO_LRCLK_I_LEFT_HIGH			0
#define AO_LRCLK_I_LEFT_LOW				1

#define AO_BCLK_I_FALLING_EDGE			0
#define AO_BCLK_I_RISING_EDGE			1

#define AO_I2S_NORMAL					0
#define AO_I2S_PROGRAM					1

#define AO_RJ_LEFT_JUSTIFIED			0
#define AO_RJ_RIGHT_JUSTIFIED			1

#define AO_MODE_MASTER					0
#define AO_MODE_SLAVE					1


// QPCODEC_PROP_POWER_STATE
//
typedef struct _QPCODEC_POWER_STATE
{
	TQP_BOOL	On;
} QPCODEC_POWER_STATE, *PQPCODEC_POWER_STATE;



// QPCODEC_PROP_ERROR_RECOVERY
//
typedef struct _QPCODEC_ERROR_RECOVERY
{
	TQP_UINT32	Level;
} QPCODEC_ERROR_RECOVERY, *PQPCODEC_ERROR_RECOVERY;


#define QPCODEC_ERROR_RECOVERY_MANUAL	0
#define QPCODEC_ERROR_RECOVERY_ON_ERROR	1
#define QPCODEC_ERROR_RECOVERY_ALWAYS	2


// GPIO
//
typedef enum _GPIO_DIRECTION
{
	GPIO_INPUT		= 0x00,
	GPIO_OUTPUT		= 0x01
} GPIO_DIRECTION;


typedef struct _GPIO_BIT_DIRECTION
{
	TQP_UINT8		bBitNumber;
	GPIO_DIRECTION	Direction;
} GPIO_BIT_DIRECTION, *PGPIO_BIT_DIRECTION;


typedef struct _GPIO_BIT_VALUE
{
	TQP_UINT8		bBitNumber;
	TQP_BOOL		bValue;		// 0 or 1
} GPIO_BIT_VALUE, *PGPIO_BIT_VALUE;


// QPCODEC_PROP_SYS_FUNCTION
//
typedef union _QPCODEC_SYS_FUNCTION
{
	TQP_UINT32		function;
	struct
	{
		TQP_UINT32	video		: 4;	// bit 0 - 3
		TQP_UINT32	audio		: 4;	// bit 4 - 7
		TQP_UINT32	function	: 1;	// bit 8
		TQP_UINT32	reserved	: 22;	// bit 9 - 30
		TQP_UINT32	version		: 1;	// bit 31
	} functionEx;
} QPCODEC_SYS_FUNCTION, *PQPCODEC_SYS_FUNCTION;


// old defination
#define CODEC_OPEN_VID_AUD_ENCODER		1
#define CODEC_OPEN_VID_AUD_DECODER		2
#define CODEC_OPEN_JPEG_ENCODER			3
#define CODEC_OPEN_JPEG_DECODER			4
#define CODEC_OPEN_VID_PASSTHROUGH		5
#define CODEC_OPEN_VID_DECIMATION		6

// new defination
#define CODEC_OPEN_EX_VID_NONE			0
#define CODEC_OPEN_EX_VID_H264			1
#define CODEC_OPEN_EX_VID_JPEG			2
#define CODEC_OPEN_EX_VID_PASSTHROUGH	4
#define CODEC_OPEN_EX_VID_DECIMATION	5
#define CODEC_OPEN_EX_VID_MOTIONDETECT	6
#define CODEC_OPEN_EX_VOU_OSD	8

#define CODEC_OPEN_EX_AUD_NONE			0
#define CODEC_OPEN_EX_AUD				1
#define CODEC_OPEN_EX_AUD_PASSTHROUGH	4

#define CODEC_OPEN_EX_ENCODE			0
#define CODEC_OPEN_EX_DECODE			1



// QPCODEC_PROP_SYS_LINK
//
typedef struct _QPCODEC_SYS_LINK
{
	TQP_UINT32	video_input;
	TQP_UINT32	video_in_ch;
	TQP_UINT32	video_output;
	TQP_UINT32	video_out_ch;
	TQP_UINT32	audio_input;
	TQP_UINT32	audio_in_ch;
	TQP_UINT32	audio_output;
	TQP_UINT32	audio_out_ch;
	TQP_UINT32	reserved1;
	TQP_UINT32	reserved2;
} QPCODEC_SYS_LINK, *PQPCODEC_SYS_LINK;


// video_input
#define SYS_LINK_VID_IN_VIU				0
#define SYS_LINK_VID_IN_HOST			1
#define SYS_LINK_VID_IN_USB				2
#define SYS_LINK_VID_IN_HDMI			4
#define SYS_LINK_VID_IN_HDMI_DBLCLK		5
#define SYS_LINK_VID_IN_DEC				6
#define SYS_LINK_VID_IN_NULL			4
#define SYS_LINK_VID_IN_INTERNAL		8

// video_output
#define SYS_LINK_VID_OUT_VOU			0
#define SYS_LINK_VID_OUT_HOST			1
#define SYS_LINK_VID_OUT_SPECIAL		2
#define SYS_LINK_VID_OUT_NULL			4
#define SYS_LINK_VID_OUT_INTERNAL		8

// audio_input
#define SYS_LINK_AUD_IN_AIU				0
#define SYS_LINK_AUD_IN_HOST			1
#define SYS_LINK_AUD_IN_INTERNAL		8

// audio_output
#define SYS_LINK_AUD_OUT_AOU			0
#define SYS_LINK_AUD_OUT_HOST			1
#define SYS_LINK_AUD_OUT_INTERNAL		8


// QPCODEC_PROP_SYS_CONFIG
//
typedef struct _QPCODEC_SYS_CONFIG
{
	TQP_UINT32	version;
	TQP_UINT32	fixedMode;
	TQP_UINT32	reserved1;
	TQP_UINT32	reserved2;
} QPCODEC_SYS_CONFIG, *PQPCODEC_SYS_CONFIG;

// QPCODEC_PROP_OSD_MEMORY
//
typedef struct _QPCODEC_OSD_MEM_CONFIG
{
	TQP_UINT32	command;
	TQP_UINT32	memSize;
	TQP_UINT32	memStartAddress;
	TQP_UINT32	status;
	TQP_UINT32	reserved1;
	TQP_UINT32	reserved2;
} QPCODEC_OSD_MEM_CONFIG, *PQPCODEC_OSD_MEM_CONFIG;

//command
#define	OSD_MEMORY_REQUEST		0
#define	OSD_MEMORY_RETURN		1
#define	OSD_MEMORY_FREE			2


typedef struct _QPCODEC_CTRL_VERSION
{
	TQP_UINT32	major;
	TQP_UINT32	minor;
	TQP_UINT32	reserved1;
	TQP_UINT32	reserved2;
} QPCODEC_CTRL_VERSION, *PQPCODEC_CTRL_VERSION;


#define HDCP_STATE_BLOCKING	0x80000000
#define HDCP_STATE_FREE		0x00000000


// EnableAVDiscardControl
#define ENC_AV_DISCARD_CONTROL_DISABLE 0
#define ENC_AV_DISCARD_CONTROL_ENABLE 1
#define ENC_AV_DISCARD_CONTROL_ENABLE_AND_CONCATENATE_PTS 2

#pragma pack(pop)


#endif //__QPDEVCNTL_H__



