///////////////////////////////////////////////////////////////////////////////
//     
//
// Qpdevdiag.h
//
// Description:
//
//  This is the definations of the diagnostic interface
//
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __QPDEVDIAG_H__
#define __QPDEVDIAG_H__


#include "qptypes.h"
#if defined (QP_LINXKER) || defined (_QP_LINUX) || defined (_QP_MACOSX)
#include "plprofiling.h"
#endif

// {01A87F1D-CE97-4acb-9B39-B1916407C36C}
static const TQP_GUID PROPSETID_QPCODEC_DIAG =
{ 0x1a87f1d, 0xce97, 0x4acb, { 0x9b, 0x39, 0xb1, 0x91, 0x64, 0x7, 0xc3, 0x6c } };


typedef enum _QPPROPERTY_QPCODEC_DIAGNOSTIC
{
    QPCODEC_DIAG_I2C_WRITE				= 0,
    QPCODEC_DIAG_I2C_READ				= 1,
    QPCODEC_DIAG_I2C_WRITE_THEN_READ	= 2,
    QPCODEC_DIAG_MEM_READ				= 3,
    QPCODEC_DIAG_MEM_WRITE				= 4,
    QPCODEC_DIAG_REG_READ				= 5,
    QPCODEC_DIAG_REG_WRITE				= 6,
    QPCODEC_DIAG_HCI_REG_READ			= 7,
    QPCODEC_DIAG_HCI_REG_WRITE			= 8,
    QPCODEC_DIAG_FIRMWARE_COMMAND		= 9,
    QPCODEC_DIAG_FIRMWARE_DOWNLOAD		= 10,
    QPCODEC_DIAG_FIRMWARE_DOWNLOAD_PTR	= 11,
	QPCODEC_DIAG_ENABLE_DRIVER_I2C		= 12,
	QPCODEC_DIAG_DISABLE_DRIVER_I2C		= 13,
	QPCODEC_DIAG_DMA_READ				= 14,
	QPCODEC_DIAG_DMA_WRITE				= 15,
	QPCODEC_DIAG_USB_CMD				= 16,
	QPCODEC_DIAG_DMA_READ_PTR			= 17,
	QPCODEC_DIAG_DMA_WRITE_PTR			= 18,
	QPCODEC_DIAG_HW_RESET				= 19,
    QPCODEC_DIAG_REG_READ_EX			= 20,
	QPCODEC_DIAG_DMA_READ_PTR_EX		= 21,
	QPCODEC_DIAG_DMA_WRITE_PTR_EX		= 22,
	QPCODEC_DIAG_EMULATION_ON			= 23,
	QPCODEC_DIAG_EMULATION_OFF			= 24,
	QPCODEC_VIDEO_DECODER				= 25,   // Andy 2008.10.14: select video decoder
    QPCODEC_DIAG_FIRMWARE_SWITCH_MODE	= 26,
	QPCODEC_DIAG_FIRMWARE_RELOAD		= 27,

	QPCODEC_DIAG_OSD_CONFIG				= 28,
	QPCODEC_DIAG_OSD_CONTROL			= 29,

	QPCODEC_DIAG_GET_USB_SPEED			= 30,

	QPCODEC_DIAG_VIDEOPROP_SET			= 31,
	QPCODEC_DIAG_VIDEOPROP_GET			= 32,
	QPCODEC_VIDEO_DECODER_CHANNEL		= 33,	// select video decoder source channel
	QPCODEC_VIDEO_DECODER_CHANNEL_GET	= 34,	// get video decoder source channel

	QPCODEC_DIAG_I2C_WRITE_EX			= 35,
	QPCODEC_DIAG_I2C_READ_EX			= 36,
	QPCODEC_DIAG_I2C_WRITE_THEN_READ_EX	= 37,
	QPCODEC_DIAG_RESET_ARM			= 38,

	QPCODEC_VIDEO_DECODER_ZOOM_POS		= 39,   // 2009.0213 by jkglee, set camera zoom pos

	QPCODEC_VIDEO_DECODER_GET			= 40,
	QPCODEC_DIAG_VIOSD_CONTROL			= 41,
	QPCODEC_DIAG_DEBUGLEVEL_SET			= 42,   // set debug level
	QPCODEC_DIAG_DEBUGLEVEL_GET			= 43,

	QPCODEC_DIAG_DRVPROFILING_GET		= 50

} QPPROPERTY_QPCODEC_DIAGNOSTIC;


#define QP_MAX_SUBADDRESS_BYTES		8
#define QP_MAX_I2C_BYTES            72     // 72 <- 44 <- 16
#define QP_MAX_MEMORY_DWORDS		(256/4)
#define QP_MAX_REGISTER_DWORDS		12

#define QP_MAX_DMA_SIZE				(32 * 1024)

#define QP_MAX_USB_CMD_SIZE			64

#pragma pack(push, 1)

typedef struct _QPCODEC_DIAG_I2C_STRUCT
{
	TQP_UINT8	chip_address;
	TQP_UINT8	num_bytes;
	TQP_UINT8	bytes[QP_MAX_I2C_BYTES];
} QPCODEC_DIAG_I2C_STRUCT, *PQPCODEC_DIAG_I2C_STRUCT;


typedef struct _QPCODEC_DIAG_I2C_WRITE_THEN_READ_STRUCT
{
    TQP_UINT8	chip_address;
    TQP_UINT8	num_write_bytes;
    TQP_UINT8	write_bytes[QP_MAX_SUBADDRESS_BYTES];
    TQP_UINT8	num_read_bytes;
    TQP_UINT8	read_bytes[QP_MAX_I2C_BYTES];
} QPCODEC_DIAG_I2C_WRITE_THEN_READ_STRUCT, *PQPCODEC_DIAG_I2C_WRITE_THEN_READ_STRUCT;


//address must be a multiple of 4
typedef struct _QPCODEC_DIAG_MEMORY_STRUCT
{
    TQP_UINT32	address;
    TQP_UINT32	num_dwords;
    TQP_UINT32	value[QP_MAX_MEMORY_DWORDS];
} QPCODEC_DIAG_MEMORY_STRUCT, *PQPCODEC_DIAG_MEMORY_STRUCT;


typedef struct _QPCODEC_DIAG_REGISTER_STRUCT
{
    TQP_UINT32	address;
    TQP_UINT32	value;
} QPCODEC_DIAG_REGISTER_STRUCT, *PQPCODEC_DIAG_REGISTER_STRUCT;


typedef struct _QPCODEC_DIAG_REGISTER_STRUCT_EX
{
    TQP_UINT32	address;
    TQP_UINT32	num_reg;
    TQP_UINT32	value[QP_MAX_REGISTER_DWORDS];
} QPCODEC_DIAG_REGISTER_STRUCT_EX, *PQPCODEC_DIAG_REGISTER_STRUCT_EX;


#define QP_NUM_FIRMWARE_PARAMETERS 16

typedef struct _QPCODEC_DIAG_FW_COMMAND
{
    TQP_UINT32	command_code;
    TQP_UINT32	return_value;
    TQP_UINT32	num_param;
    TQP_UINT32	parameters[QP_NUM_FIRMWARE_PARAMETERS];
} QPCODEC_DIAG_FW_COMMAND, *PQPCODEC_DIAG_FW_COMMAND;


#define QPCODEC_DIAG_FW_MAX_SIZE	(1024 * 1024)

typedef struct _QPCODEC_DIAG_FW_DOWNLOAD
{
	TQP_UINT8	FW[QPCODEC_DIAG_FW_MAX_SIZE];	// The firmware
	TQP_UINT32	FWSize;
	TQP_UINT32	startAddress;
	TQP_BOOL	bArmReset;
} QPCODEC_DIAG_FW_DOWNLOAD, *PQPCODEC_DIAG_FW_DOWNLOAD;


typedef struct _QPCODEC_DIAG_FW_DOWNLOAD_PTR
{
	PTQP_UINT8	pFW;			// caller has to make sure the
								// firmware buffer is locked down
								// and accessable by the codec
								// library
	TQP_UINT32	FWSize;
	TQP_UINT32	startAddress;
	TQP_BOOL	bArmReset;
} QPCODEC_DIAG_FW_DOWNLOAD_PTR, *PQPCODEC_DIAG_FW_DOWNLOAD_PTR;


typedef struct _QPCODEC_DIAG_DMA_STRUCT
{
	TQP_UINT32	deviceAddress;
	TQP_UINT32	size;
	TQP_UINT8	data[QP_MAX_DMA_SIZE + 128];
} QPCODEC_DIAG_DMA_STRUCT, *PQPCODEC_DIAG_DMA_STRUCT;


typedef struct _QPCODEC_DIAG_USB_COMMAND
{
	TQP_UINT8	outBuf[QP_MAX_USB_CMD_SIZE];
	TQP_UINT32	outSize;
	TQP_UINT8	inBuf[QP_MAX_USB_CMD_SIZE];
	TQP_UINT32	inSize;
} QPCODEC_DIAG_USB_COMMAND, *PQPCODEC_DIAG_USB_COMMAND;


typedef struct _QPCODEC_DIAG_DMA_PTR_STRUCT
{
	TQP_UINT32	deviceAddress;
	TQP_UINT32	size;
	PTQP_UINT8	pData;	// caller has to make sure the memory
						// is locked down and accessable by the
						// codec library
} QPCODEC_DIAG_DMA_PTR_STRUCT, *PQPCODEC_DIAG_DMA_PTR_STRUCT;


typedef struct _QPCODEC_DIAG_DMA_PTR_STRUCT_EX
{
	TQP_UINT32	deviceAddress;
	TQP_UINT32	size;
	PTQP_UINT8	pData;	// caller has to make sure the memory
						// is locked down and accessable by the
						// codec library
	TQP_BOOL	bSwap;
	TQP_UINT32	ulXferMode;
	TQP_UINT32	ulPicWidth;
	TQP_UINT32	ulPicHeight;
	TQP_BOOL	IsFrameMode;
} QPCODEC_DIAG_DMA_PTR_STRUCT_EX, *PQPCODEC_DIAG_DMA_PTR_STRUCT_EX;

#if defined (QP_LINXKER) || defined (_QP_LINUX) || defined (_QP_MACOSX)
typedef struct _QPCODEC_DIAG_PROFILING_STRUCT
{
	TQP_UINT8	header[8];
	TQP_UINT32	cur_profiling_idx;
	TQP_UINT32	looped;
	TQP_UINT32	entries[QP_MAX_PROFILING_SIZE][5];
} QPCODEC_DIAG_PROFILING_STRUCT, *PQPCODEC_DIAG_PROFILING_STRUCT;

typedef struct _QPCODEC_DIAG_PROFILING_PTR_STRUCT
{
	TQP_UINT32	offset;
	TQP_UINT32	size;
	TQP_UINT32	max_entries;
	PTQP_UINT8	pData;
} QPCODEC_DIAG_PROFILING_PTR_STRUCT, *PQPCODEC_DIAG_PROFILING_PTR_STRUCT;
#endif

typedef struct _QPCODEC_DIAG_HW_RESET_STRUCT
{
	TQP_BOOL	bArmRunning;
} QPCODEC_DIAG_HW_RESET_STRUCT, *PQPCODEC_DIAG_HW_RESET_STRUCT;


// QPCODEC_DIAG_FIRMWARE_SWITCH_MODE
//
#define QXU_FIRMWARE_QCAM					0x01
#define QXU_FIRMWARE_UVC					0x02


// QPCODEC_DIAG_GET_USB_SPEED
//
#define USB_SPEED_LOW						1
#define USB_SPEED_FULL						2
#define USB_SPEED_HIGH						3

#pragma pack(pop)


#endif //__QPDEVDIAG_H__

