///////////////////////////////////////////////////////////////////////////////
//        
//
//		MP4 File Writer interface
//
// History:
//	Jun 5, 2007	(EC) Created
//
// Authors: (EC) Enrico Cadorin
//
///////////////////////////////////////////////////////////////////////////////


#ifndef __QPMP4FW_H__
#define __QPMP4FW_H__

#include "qptypes.h"
#include "qpfilemgr.h"

#ifdef __cplusplus
extern "C" {
#endif

#define QPMP4FW_MAJORVERSION 0
#define QPMP4FW_MINORVERSION 1

// MP4 File Writer ID (identifies clients)
typedef void * TQP_MP4WID;

typedef struct _TQP_MP4WCaps {
	TQP_Version		version;
	TQP_BOOL		canStream;
} TQP_MP4WCaps, *PTQP_MP4WCaps;

//
// Returns the list of capabilities for this implementation
//
void QPMP4WGetCaps (PTQP_MP4WCaps capsP);

typedef enum _EQP_MP4WMode {
	QPMP4W_MODE_SHARED = 0,
	QPMP4W_MODE_SEPARATE,
	QPMP4W_MODE_STREAMING
} EQP_MP4WMode;

typedef enum _EQP_MP4WFormat {
	QPMP4W_FMT_GENERIC = 0,
	QPMP4W_FMT_PSP,
	QPMP4W_FMT_IPOD
} EQP_MP4WFormat;

typedef struct _TQP_MP4WParams {
	EQP_MP4WFormat	fmt;
	TQP_UINT32		videoWidth;
	TQP_UINT32		videoHeigth;
	TQP_UINT32		videoFrameRate;
	TQP_UINT32		videoNrBFrames;
	TQP_UINT32		videoMaxBPS;
	TQP_UINT32		videoAvgBPS;
	TQP_UINT32		videoMinBPS;
	TQP_UINT32		audioSampleRate;
	TQP_UINT32		audioBitRate;
	EQP_MP4WMode	mode;
	TQP_FMFileID	outFID;				// IN: ID of the output file
	TQP_FMVolID		tempVolID;			// IN: ID of a volume for temporary storage
	PTQP_CHAR		tempDirP;			// IN: pathname of a directory for temp storage
	TQP_UINT32		hSize;				// OUT
} TQP_MP4WParams, *PTQP_MP4WParams;

//
// Initializes an MP4 File Writer object and returns its ID
//
TQP_MP4WID QPMP4WInit (PTQP_MP4WParams parmsP);

//
// Terminates an MP4 File Writer object and releases all the resources it used
//
void QPMP4WTerm (TQP_MP4WID wID, TQP_BOOL kill);

//
// Generates the MP4 file
//
EQP_Errors QPMP4WGenFromFiles (TQP_MP4WID wID, TQP_FMFileID mediaFID, TQP_FMFileID idxFID);

//
// Processes new media data for a streaming MP4 writer
//
EQP_Errors QPMP4WNewMedia (TQP_MP4WID wID, void * bufP, PTQP_UINT32 lenP);

//
// Processes new index data for a streaming MP4 writer
//
EQP_Errors QPMP4WNewIndex (TQP_MP4WID wID, void * bufP, PTQP_UINT32 lenP);


#ifdef __cplusplus
}
#endif

#endif //__QPMP4FW_H__
