///////////////////////////////////////////////////////////////////////////////
//        
//
// Qpmpgcodec.h
//
// Description: 
//
//  This is the definations of the mpeg codec interface
// 
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __QPMPGCODEC_H__
#define __QPMPGCODEC_H__


#include "qptypes.h"
#include "qpgenericdevice.h"

#ifdef __cplusplus
extern "C" {
#endif


/////////////////////////////////////////////////////////////////////////////
//
//

typedef enum _QPMPGCODEC_OPEN_TYPE
{
	QPMPGCODEC_ENC_MPEG_OUT = 0,
	QPMPGCODEC_ENC_AES_OUT,
	QPMPGCODEC_ENC_VBI_OUT,
	QPMPGCODEC_ENC_INDEX_OUT,
	QPMPGCODEC_ENC_YUV_IN,
	QPMPGCODEC_ENC_PCM_IN,
	QPMPGCODEC_ENC_PCM_OUT,
	QPMPGCODEC_ENC_YUV_OUT,
	QPMPGCODEC_DEC_MPEG_IN,
	QPMPGCODEC_DEC_AES_IN,
	QPMPGCODEC_DEC_INDEX_IN,
	QPMPGCODEC_DEC_YUV_OUT,
	QPMPGCODEC_DEC_PCM_OUT,
	QPMPGCODEC_DEC_VBI_IN,
	QPMPGCODEC_CODEC_VIRTUAL,
	QPMPGCODEC_ENC_ACTIVITY_OUT
} QPMPGCODEC_OPEN_TYPE, *PQPMPGCODEC_OPEN_TYPE;



#pragma pack(4)

// buffer descriptor flags
//

// private, qpcodec internal use
//
#define QP_BUFDESC_FLAG_INCOMPLETE_FRAME				0x00000004
#define QP_BUFDESC_FLAG_FRAME_START						0x00000010
#define QP_BUFDESC_FLAG_FRAME_END						0x00000020

#define QP_BUFDESC_FLAG_MASK_PRIVATE					0x0000FFFF

// public
//
#define QP_BUFDESC_FLAG_PARTIAL_FILL_OK					0x00010000
#define QP_BUFDESC_FLAG_EOS								0x00020000
#define QP_BUFDESC_FLAG_PTS								0x00040000
#define QP_BUFDESC_FLAG_FRAME_ALIGNED					0x00080000
#define QP_BUFDESC_FLAG_USE_USER_DEFINED_PTS			0x00100000
#define QP_BUFDESC_FLAG_RAW_AUDIO_AC3                   0x01000000
#define QP_BUFDESC_FLAG_RAW_AUDIO_AAC                   0x02000000
#define QP_BUFDESC_FLAG_RAW_AUDIO_AAC2                  0x04000000
#define QP_BUFDESC_FLAG_RAW_AUDIO_DTS                   0x08000000
#define QP_BUFDESC_FLAG_BUFFER_DRIVER					0x10000000	// buffer pointer is driver accessable
#define QP_BUFDESC_FLAG_BUFFER_USER						0x20000000	// user pointer, may need to be mapped
#define QP_BUFDESC_FLAG_BUFFER_OVERLAY					0x40000000	// not used

#define QP_BUFDESC_FLAG_RAW_AUDIO_MASK					0x0F000000
#define QP_BUFDESC_FLAG_BUFFER_MASK						0xF0000000

#define QP_BUFDESC_FLAG_MASK_PUBLIC						0xFFFF0000




// QP_KSSTREAM_HEADER definations
//
// compatible with KSSTREAM_HEADER_OPTIONSF_* in ks.h 
// #define KSSTREAM_HEADER_OPTIONSF_SPLICEPOINT        0x00000001
// #define KSSTREAM_HEADER_OPTIONSF_PREROLL            0x00000002
// #define KSSTREAM_HEADER_OPTIONSF_DATADISCONTINUITY  0x00000004
// #define KSSTREAM_HEADER_OPTIONSF_TYPECHANGED        0x00000008
// #define KSSTREAM_HEADER_OPTIONSF_TIMEVALID          0x00000010
// #define KSSTREAM_HEADER_OPTIONSF_TIMEDISCONTINUITY  0x00000040
// #define KSSTREAM_HEADER_OPTIONSF_FLUSHONPAUSE       0x00000080
// #define KSSTREAM_HEADER_OPTIONSF_DURATIONVALID      0x00000100
// #define KSSTREAM_HEADER_OPTIONSF_ENDOFSTREAM        0x00000200
// #define KSSTREAM_HEADER_OPTIONSF_LOOPEDDATA         0x80000000

#define QP_KSSTREAM_HEADER_OPTIONSF_SPLICEPOINT			0x00000001
#define QP_KSSTREAM_HEADER_OPTIONSF_PREROLL				0x00000002
#define QP_KSSTREAM_HEADER_OPTIONSF_DATADISCONTINUITY	0x00000004
#define QP_KSSTREAM_HEADER_OPTIONSF_TYPECHANGED			0x00000008
#define QP_KSSTREAM_HEADER_OPTIONSF_TIMEVALID			0x00000010
#define QP_KSSTREAM_HEADER_OPTIONSF_TIMEDISCONTINUITY	0x00000040
#define QP_KSSTREAM_HEADER_OPTIONSF_FLUSHONPAUSE		0x00000080
#define QP_KSSTREAM_HEADER_OPTIONSF_DURATIONVALID		0x00000100
#define QP_KSSTREAM_HEADER_OPTIONSF_ENDOFSTREAM			0x00000200
#define QP_KSSTREAM_HEADER_OPTIONSF_RAW_AUDIO_AC3       0x01000000
#define QP_KSSTREAM_HEADER_OPTIONSF_RAW_AUDIO_AAC       0x02000000
#define QP_KSSTREAM_HEADER_OPTIONSF_RAW_AUDIO_AAC2      0x04000000
#define QP_KSSTREAM_HEADER_OPTIONSF_RAW_AUDIO_DTS       0x08000000
#define QP_KSSTREAM_HEADER_OPTIONSF_LOOPEDDATA			0x80000000

#define QP_KSSTREAM_HEADER_OPTIONSF_RAW_AUDIO_MASK      0x0F000000


typedef enum _QPSTATE
{
	QPSTATE_STOP,
	QPSTATE_ACQUIRE,
	QPSTATE_PAUSE,
	QPSTATE_RUN
} QPSTATE, *PQPSTATE;


typedef struct _QP_KSTIME
{
    TQP_INT64	Time;
    TQP_UINT32	Numerator;
    TQP_UINT32	Denominator;
} QP_KSTIME, *PQP_KSTIME;

typedef struct _QP_KSSTREAM_HEADER
{
    QP_KSTIME	PresentationTime;
    TQP_INT64	Duration;
    TQP_UINT32	FrameExtent;		// size of the entire buffer
    TQP_UINT32	DataUsed;			// actual size used
    TQP_PVOID	Data;				// data buffer
    TQP_UINT32	OptionsFlags;
} QP_KSSTREAM_HEADER, *PQP_KSSTREAM_HEADER;


typedef union _QP_LARGE_INTEGER 
{
	struct 
	{
		TQP_UINT32	LowPart;
		TQP_INT32	HighPart;
	} u;
	TQP_INT64	QuadPart;
} QP_LARGE_INTEGER;

typedef QP_LARGE_INTEGER *PQP_LARGE_INTEGER;



typedef union _QP_ULARGE_INTEGER 
{
    struct 
	{
        TQP_UINT32 LowPart;
        TQP_UINT32 HighPart;
    } u;
    TQP_UINT64	QuadPart;
} QP_ULARGE_INTEGER;

typedef QP_ULARGE_INTEGER *PQP_ULARGE_INTEGER;

typedef QP_LARGE_INTEGER QP_PHYSICAL_ADDRESS, *PQP_PHYSICAL_ADDRESS;


// scatter gather element
//
typedef struct _QP_SCATTER_GATHER
{
	QP_PHYSICAL_ADDRESS	PhysicalAddress;
	TQP_UINT32			ByteCount;
	TQP_UINT32			Alignment;
} QP_SCATTER_GATHER, *PQP_SCATTER_GATHER;


// dwFrameFlags
//
#define QP_VIDEO_FLAG_FRAME		0x0000L	// Frame or Field
#define QP_VIDEO_FLAG_FIELD1	0x0001L
#define QP_VIDEO_FLAG_FIELD2	0x0002L


// buffer descriptor format
//
typedef struct _QP_BUFFER_DESCRIPTOR
{
	PQP_KSSTREAM_HEADER	DataBufferArray;

	TQP_UINT32			NumberOfBuffers;
	TQP_UINT32			ulBufferIndex;
	TQP_UINT32			ulBufferOffset;
	TQP_UINT32			ulBufferSize;

	PQP_SCATTER_GATHER	ScatterGatherBuffer;
	TQP_UINT32			NumberOfScatterGatherElements;
	TQP_UINT32			ulDMABufferIndex;
	TQP_UINT32			ulDMABufferOffset;

	TQP_UINT32			ulTotalUsed;
	TQP_UINT32			ulFlags;
	EQP_Errors			Status;
	TQP_PVOID			pBuffer;	// contiguous buffer for decoder
									// if NumberOfBuffers greater than 
									// one.

	TQP_INT64			ulPTS;		// 64 bit PTS

	// additional information for raw YUV and VBI
	//
    TQP_UINT32			dwFrameFlags;
    TQP_INT64			PictureNumber;
    TQP_INT64			DropCount;

	// unaligned MJPEG data
	//
	TQP_UINT32			unAlignedData;
	TQP_UINT32			unAlignedNumberBytes;

} QP_BUFFER_DESCRIPTOR, *PQP_BUFFER_DESCRIPTOR;


// open formats
//

typedef struct _QP_YUV_DATAFORMAT
{
	TQP_INT32	nWidth;
	TQP_INT32	nHeight;
	TQP_INT32	nBitCount;
	TQP_UINT32	nFrameRate;

	TQP_UINT32	nDataType;

} QP_YUV_DATAFORMAT, *PQP_YUV_DATAFORMAT;

#define QP_YUV_DATA_TYPE_QYUV	0
#define QP_YUV_DATA_TYPE_YV12	1
#define QP_YUV_DATA_TYPE_I420	2
#define QP_YUV_DATA_TYPE_YUYV	3
#define QP_YUV_DATA_TYPE_NV12	4		// PL331
#define QP_YUV_DATA_TYPE_8U8V	5		// PL331
#define QP_YUV_DATA_TYPE_UYVY	6		// PL331


typedef struct _QP_PCM_DATAFORMAT
{
    TQP_UINT32	nChannels;
    TQP_UINT32	nSamplesPerSec;
    TQP_UINT32	nAvgBytesPerSec;
    TQP_UINT32	nBlockAlign;
    TQP_UINT32	nBitsPerSample;

} QP_PCM_DATAFORMAT, *PQP_PCM_DATAFORMAT;


typedef struct _QP_VBI_DATAFORMAT
{
    TQP_INT32	Top_StartLine;			// inclusive
    TQP_INT32	Top_EndLine;			// inclusive
  	TQP_UINT32	Top_StartPixel;			// starting from 0
    TQP_INT32	Top_SamplesPerLine;		// maximum (1440 - StartPixel)

    TQP_INT32	Bottom_StartLine;		// inclusive
    TQP_INT32	Bottom_EndLine;			// inclusive
  	TQP_UINT32	Bottom_StartPixel;		// starting from 0
    TQP_INT32	Bottom_SamplesPerLine;	// maximum (1440 - StartPixel)

    TQP_INT32	BufferSize;				// Bytes
	TQP_UINT32	nFrameRate;

} QP_VBI_DATAFORMAT, *PQP_VBI_DATAFORMAT;


typedef struct _QP_COMPRESSED_VID_DATAFORMAT
{
	TQP_UINT32	nDataType;

} QP_COMPRESSED_VID_DATAFORMAT, *PQP_COMPRESSED_VID_DATAFORMAT;

#define QP_COMP_VID_DATA_TYPE_H264	0
#define QP_COMP_VID_DATA_TYPE_MJPEG	1
#define QP_COMP_VID_DATA_TYPE_ACTIVITY	2

typedef struct _QP_ACTIVITY_DATAFORMAT
{
	TQP_INT32	nWidth;
	TQP_INT32	nHeight;
	TQP_INT32	nBitCount;

} QP_ACTIVITY_DATAFORMAT, *PQP_ACTIVITY_DATAFORMAT;

#pragma pack()


/////////////////////////////////////////////////////////////////////////////
//
//

// device specific initialization data
//

// access mode
//
typedef enum _QPHCI_MODE
{
	QPHCI_MODE_INDIRECT = 0,
	QPHCI_MODE_DIRECT,
	QPHCI_MODE_MASTER,

	QPHCI_MODE_TOTAL_MODES

} QPHCI_MODE, *PQPHCI_MODE;

#define QPHCI_MODE_FIRST	QPHCI_MODE_INDIRECT
#define QPHCI_MODE_LAST		QPHCI_MODE_MASTER


// bus type
//
typedef enum _QPHCI_BUS
{
	QPHCI_BUS_USB = 0,
	QPHCI_BUS_PCI,
	QPHCI_BUS_CUSTOM_1,
	QPHCI_BUS_201_EMULATION,
	QPHCI_BUS_INTERNAL,
	QPHCI_BUS_STR8133_REF,

	QPHCI_BUS_TOTAL_BUSES

} QPHCI_BUS, *PQPHCI_BUS;

#define QPHCI_BUS_FIRST	QPHCI_BUS_USB
#define QPHCI_BUS_LAST	QPHCI_BUS_STR8133_REF


// bus data
//
typedef struct _QPCODEC_USB_BUS_DATA
{
	TQP_UINT32	dwPipeCmdWrite;
	TQP_UINT32	dwPipeCmdRead;
	TQP_UINT32	dwPipeDataWrite;
	TQP_UINT32	dwPipeDataRead;
} QPCODEC_USB_BUS_DATA, *PQPCODEC_USB_BUS_DATA;



// memory type
//
typedef enum _QPCODEC_MEM_TYPE
{
	QPCODEC_MEM_TYPE_DDR = 0,
	QPCODEC_MEM_TYPE_mDDR,

	QPCODEC_MEM_TYPE_TOTAL_TYPES

} QPCODEC_MEM_TYPE, *PQPCODEC_MEM_TYPE;

#define QPCODEC_MEM_TYPE_FIRST	QPCODEC_MEM_TYPE_DDR
#define QPCODEC_MEM_TYPE_LAST	QPCODEC_MEM_TYPE_mDDR



// chip type
//
typedef enum _QPCODEC_CHIP_TYPE
{
	QPCODEC_CHIP_BERINGER			= 0x00000001,	// QL201B, QL202B
	QPCODEC_CHIP_ARTESA_VERTEX_4	= 0x00000002,
	QPCODEC_CHIP_ARTESA_VERTEX_5	= 0x00000004,
	QPCODEC_CHIP_ARTESA				= 0x00000008,	// A0, A1, B0

	QPCODEC_CHIP_ARTESA_FPGA		= (QPCODEC_CHIP_ARTESA_VERTEX_4 |
									  QPCODEC_CHIP_ARTESA_VERTEX_5),
	QPCODEC_CHIP_ARTESA_ALL			= (QPCODEC_CHIP_ARTESA_VERTEX_4 |
									  QPCODEC_CHIP_ARTESA_VERTEX_5 |
									  QPCODEC_CHIP_ARTESA),

	QPCODEC_CHIP_PL331_VERTEX_4		= 0x00000010,
	QPCODEC_CHIP_PL331_VERTEX_5		= 0x00000020,
	QPCODEC_CHIP_PL331				= 0x00000040,

	QPCODEC_CHIP_PL331_FPGA			= (QPCODEC_CHIP_PL331_VERTEX_4 |
									  QPCODEC_CHIP_PL331_VERTEX_5),
	QPCODEC_CHIP_PL331_ALL			= (QPCODEC_CHIP_PL331_VERTEX_4 |
									  QPCODEC_CHIP_PL331_VERTEX_5 |
									  QPCODEC_CHIP_PL331),

	QPCODEC_CHIP_FPGA_ALL			= (QPCODEC_CHIP_ARTESA_FPGA |
									   QPCODEC_CHIP_PL331_FPGA)

} QPCODEC_CHIP_TYPE, *PQPCODEC_CHIP_TYPE;



typedef struct _QPMPGCODEC_INITDATA
{
	// flag to control if we need to init the chip
	//
	TQP_BOOL			bDontInitHW;

	// access mode and bus type
	//
	QPHCI_MODE			AccessMode;
	QPHCI_BUS			BusType;

	// memory type and size
	//
	QPCODEC_MEM_TYPE	MemType;
	TQP_UINT32			MemSize;	// in units of Mb

	// chip type
	//
	QPCODEC_CHIP_TYPE	ChipType;

	// base address
	//
	TQP_PVOID			RegisterBase;
	TQP_PVOID			MemoryBase;
	TQP_UINT32			PageSize;

	// interrupt number
	//
	TQP_UINT32			InterruptNumber;

	// video input configuration
	//
	TQP_UINT32			VIUMode;
	TQP_UINT32			VIUFormat;
	TQP_UINT32			VIUStartPixel;
	TQP_UINT32			VIUStartLine;
	TQP_UINT32			ClkEdge;
	TQP_UINT32			VIUSyncCode1;
	TQP_UINT32			VIUSyncCode2;

	// audio input configuration
	//
	TQP_UINT32			AIControls_ai_msb;
	TQP_UINT32			AIControls_lrclk_i;
	TQP_UINT32			AIControls_bclk_i;
	TQP_UINT32			AIControls_ai_i2s;
	TQP_UINT32			AIControls_ai_rj;
	TQP_UINT32			AIControls_ai_m;

	// video output configuration
	//
	TQP_BOOL			VOEnable;
	TQP_UINT32			VOUMode;
	TQP_UINT32			VOUStartPixel;
	TQP_UINT32			VOUStartLine;

	// audio output configuration
	//
	TQP_BOOL			AOEnable;
	TQP_UINT32			AOControls_ao_msb;
	TQP_UINT32			AOControls_lrclk_i;
	TQP_UINT32			AOControls_bclk_i;
	TQP_UINT32			AOControls_ao_i2s;
	TQP_UINT32			AOControls_ao_rj;
	TQP_UINT32			AOControls_ao_s;

	// bus specific data
	//
	TQP_PVOID			BusData;
	TQP_UINT32			BusDataSize;

	// firmware API version
	//
	TQP_UINT32			VerFwAPI;

	// firmware configuration
	//
	TQP_UINT32			FwIntMode;
	TQP_UINT32			FwFixedMode;

	// GPIO settings
	//
	TQP_UINT32			GPIODirections;
	TQP_UINT32			GPIOValues;
	
	// error recovery
	TQP_UINT32			ErrorRecovery;

	// external firmware image
	//
	TQP_BOOL			bUseExtFW;
	PTQP_UINT8			pVideoFW;
	PTQP_UINT8			pAudioFW;
	TQP_UINT32			VidFWSize;
	TQP_UINT32			AudFWSize;

	// restore defualt USB mode upon suspense/resume
	//
	TQP_BOOL			bRestoreUSBMode;

	// pll4 and pll5	
	TQP_UINT32			Pll4;
	TQP_UINT32			Pll5;
	
	TQP_UINT32			UseArtesaExt340;
	
	TQP_UINT32			FilterOutHDCPContent;

} QPMPGCODEC_INITDATA, *PQPMPGCODEC_INITDATA;


/////////////////////////////////////////////////////////////////////////////
//
//

// property supported by this interface
//



/////////////////////////////////////////////////////////////////////////////
//
//

// codec cllback command code
//

#define QPMPGCODEC_CMD_BUFFER_OVERFLOW			0x00000100
#define QPMPGCODEC_CMD_STOP_COMPLETED			0x00000200
#define QPMPGCODEC_CMD_SCHEDULE_TIMER			0x00000400
#define QPMPGCODEC_CMD_DONE_DATA				0x00010000
#define QPMPGCODEC_CMD_END_OF_STREAM			0x00020000
#define QPMPGCODEC_CMD_GET_STRM_TIME			0x00040000
#define QPMPGCODEC_CMD_MORE_DATA				0x00080000
#define QPMPGCODEC_CMD_THREAD					0x10000000
#define QPMPGCODEC_CMD_CAP_START				0x20000000
#define QPMPGCODEC_CMD_CAP_STOP					0x40000000
#define QPMPGCODEC_CMD_START_TIMEOUT			0x00100000
#define QPMPGCODEC_CMD_DEC_VID_ERR				0x00000001
#define QPMPGCODEC_CMD_DEC_AUD_ERR				0x00000002
#define QPMPGCODEC_CMD_DEC_SEQ_PARAM_CHANGED	0x00000004
#define QPMPGCODEC_CMD_DEC_RDY_TO_DISP			0x00000008
#define QPMPGCODEC_CMD_DEC_COMPLETE				0x00000010


/////////////////////////////////////////////////////////////////////////////
//
//

// codec cllback command data
//


// MPGCODEC_CMD_CAP_START
//
#define QPMPGCODEC_CMD_CAP_START_CHECK_ISR				0x00000001
#define QPMPGCODEC_CMD_CAP_START_CHECK_COPYPROTECTION	0x00000002


/////////////////////////////////////////////////////////////////////////////
//
//

#ifndef __IMPEGCODEC_FWD_DEFINED__
#define __IMPEGCODEC_FWD_DEFINED__
typedef struct IMpegCodec IMpegCodec;
#endif //__IMPEGCODEC_FWD_DEFINED__

    struct IMpegCodec
    {
		// IGenericDevice
		DECLARE_IGENERICDEVICE(IMpegCodec)

		// IMpegCodec
		EQP_Errors (*AllocEncodeTask)(IMpegCodec *This, 
									  PTQP_OBJECT_HANDLE phTask
									  );
		EQP_Errors (*AllocDecodeTask)(IMpegCodec *This, 
									  PTQP_OBJECT_HANDLE phTask
									  );
		EQP_Errors (*ReleaseTask)(IMpegCodec *This, 
								  TQP_OBJECT_HANDLE hTask
								  );
		EQP_Errors (*Open)(IMpegCodec *This, 
						   TQP_OBJECT_HANDLE hTask,
						   TQP_UINT32 dwFlags,	// open type
						   TQP_PVOID pDataFormat,
						   PTQP_OBJECT_HANDLE phStream,
						   QP_DEVICE_CALLBACK pFuncCallback,
						   TQP_PVOID context
						   );
		EQP_Errors (*Close)(IMpegCodec *This, 
							TQP_OBJECT_HANDLE hStream
							);
		EQP_Errors (*Start)(IMpegCodec *This, 
							TQP_OBJECT_HANDLE hStream
							);
		EQP_Errors (*Stop)(IMpegCodec *This, 
							TQP_OBJECT_HANDLE hStream
							);
		EQP_Errors (*Acquire)(IMpegCodec *This, 
							  TQP_OBJECT_HANDLE hStream
							  );
		EQP_Errors (*Pause)(IMpegCodec *This, 
							TQP_OBJECT_HANDLE hStream
							);
		EQP_Errors (*Step)(IMpegCodec *This, 
						   TQP_OBJECT_HANDLE hStream,
						   TQP_UINT32 nb
						   );
		EQP_Errors (*SetRate)(IMpegCodec *This, 
							  TQP_OBJECT_HANDLE hStream,
							  TQP_INT32 lNewRate
							  );
		EQP_Errors (*GetRate)(IMpegCodec *This, 
							  TQP_OBJECT_HANDLE hStream,
							  PTQP_INT32 plRate
							  );
		EQP_Errors (*BeginFlush)(IMpegCodec *This, 
								 TQP_OBJECT_HANDLE hStream
								 );
		EQP_Errors (*Flush)(IMpegCodec *This, 
							TQP_OBJECT_HANDLE hStream
							);
		EQP_Errors (*EndFlush)(IMpegCodec *This, 
							   TQP_OBJECT_HANDLE hStream
							   );
		EQP_Errors (*AddBuffer)(IMpegCodec *This, 
								TQP_OBJECT_HANDLE hStream,
								PQP_BUFFER_DESCRIPTOR pBufDesc
								);
		EQP_Errors (*CancelBuffer)(IMpegCodec *This, 
								   TQP_OBJECT_HANDLE hStream,
								   PQP_BUFFER_DESCRIPTOR pBufDesc
								   );
		EQP_Errors (*TimeoutBuffer)(IMpegCodec *This, 
									TQP_OBJECT_HANDLE hStream,
			  						PQP_BUFFER_DESCRIPTOR pBufDesc
			  						);
		EQP_Errors (*GetTime)(IMpegCodec *This, 
							  TQP_OBJECT_HANDLE hStream,
			  				  PTQP_UINT64 pTime
			  				  );
		EQP_Errors (*XferData)(IMpegCodec *This, 
							   TQP_OBJECT_HANDLE hStream, 
					  		   PQP_BUFFER_DESCRIPTOR pBufDesc
					  		   );
    };


/////////////////////////////////////////////////////////////////////////////
//
//
#define QPMpegCodec_AllocEncodeTask(This, phTask) \
	((QP_NULL == This) ? QPERR_PARMS : ((This)->AllocEncodeTask(This, phTask)))

#define QPMpegCodec_AllocDecodeTask(This, phTask) \
	((QP_NULL == This) ? QPERR_PARMS : ((This)->AllocDecodeTask(This, phTask)))

#define QPMpegCodec_ReleaseTask(This, hTask) \
	((QP_NULL == This) ? QPERR_PARMS : ((This)->ReleaseTask(This, hTask)))

#define QPMpegCodec_Open(This, hTask, dwFlags, pDataFormat, phStream, pFuncCallback, context) \
	((QP_NULL == This) ? QPERR_PARMS : ((This)->Open(This, hTask, dwFlags, pDataFormat, phStream, pFuncCallback, context)))

#define QPMpegCodec_Close(This, hStream) \
    ((QP_NULL == This) ? QPERR_PARMS : ((This)->Close(This, hStream)))

#define QPMpegCodec_Start(This, hStream) \
    ((QP_NULL == This) ? QPERR_PARMS : ((This)->Start(This, hStream)))

#define QPMpegCodec_Stop(This, hStream) \
    ((QP_NULL == This) ? QPERR_PARMS : ((This)->Stop(This, hStream)))

#define QPMpegCodec_Acquire(This, hStream) \
    ((QP_NULL == This) ? QPERR_PARMS : ((This)->Acquire(This, hStream)))

#define QPMpegCodec_Pause(This, hStream) \
    ((QP_NULL == This) ? QPERR_PARMS : ((This)->Pause(This, hStream)))

#define QPMpegCodec_Step(This, hStream, nb) \
    ((QP_NULL == This) ? QPERR_PARMS : ((This)->Step(This, hStream, nb)))

#define QPMpegCodec_SetRate(This, hStream, lNewRate) \
    ((QP_NULL == This) ? QPERR_PARMS : ((This)->SetRate(This, hStream, lNewRate)))

#define QPMpegCodec_GetRate(This, hStream, plRate) \
    ((QP_NULL == This) ? QPERR_PARMS : ((This)->GetRate(This, hStream, plRate)))

#define QPMpegCodec_BeginFlush(This, hStream) \
    ((QP_NULL == This) ? QPERR_PARMS : ((This)->BeginFlush(This, hStream)))

#define QPMpegCodec_Flush(This, hStream) \
    ((QP_NULL == This) ? QPERR_PARMS : ((This)->Flush(This, hStream)))

#define QPMpegCodec_EndFlush(This, hStream) \
    ((QP_NULL == This) ? QPERR_PARMS : ((This)->EndFlush(This, hStream)))

#define QPMpegCodec_AddBuffer(This, hStream, pBufDesc) \
    ((QP_NULL == This) ? QPERR_PARMS : ((This)->AddBuffer(This, hStream, pBufDesc)))

#define QPMpegCodec_CancelBuffer(This, hStream, pBufDesc) \
    ((QP_NULL == This) ? QPERR_PARMS : ((This)->CancelBuffer(This, hStream, pBufDesc)))

#define QPMpegCodec_TimeoutBuffer(This, hStream, pBufDesc) \
    ((QP_NULL == This) ? QPERR_PARMS : ((This)->TimeoutBuffer(This, hStream, pBufDesc)))

#define QPMpegCodec_GetTime(This, hStream, pTime) \
    ((QP_NULL == This) ? QPERR_PARMS : ((This)->GetTime(This, hStream, pTime)))

#define QPMpegCodec_XferData(This, hStream, pBufDesc) \
    ((QP_NULL == This) ? QPERR_PARMS : ((This)->XferData(This, hStream, pBufDesc)))

#ifdef __cplusplus
}
#endif

#endif //__QPMPGCODEC_H__



