///////////////////////////////////////////////////////////////////////////////
//        
//
//		Platform Manager interface
//
//	Platform-independent OS interface
//
// History:
//	Sep 13, 2006	(EC) Created
//
// Authors: (EC) Enrico Cadorin
//
///////////////////////////////////////////////////////////////////////////////


#ifndef __QPPFMGR_H__
#define __QPPFMGR_H__

#include "qptypes.h"

#ifdef __cplusplus
extern "C" {
#endif

#define QPPFM_MAJORVERSION 0
#define QPPFM_MINORVERSION 1


//
// Capabilities structure
typedef struct _TQP_PFMCaps {
	TQP_Version		version;
} TQP_PFMCaps, *PTQP_PFMCaps;

//
// Returns the list of capabilities for this implementation
//
void QPPFMGetCaps (PTQP_PFMCaps capsP);


typedef enum _EAccessModes {
	QPPF_MODE_UNKNOWN = -1,
	QPPF_MODE_INDIRECT = 0,
	QPPF_MODE_DIRECT,
	QPPF_MODE_MASTER,

	QPFP_MODE_NUM		// keep last
} EAccessModes;

typedef struct _TAccessMode {
	EAccessModes	mode;
} TAccessMode, *PTAccessMode;


typedef enum _EBusTypes {
	QPPF_BUS_UNKNOWN = -1,
	QPPF_BUS_USB = 0,
	QPPF_BUS_PCI,
	QPPF_BUS_CUSTOM_1,
	QPPF_BUS_201_EMULATION,
	QPPF_BUS_INTERNAL,
	QPPF_BUS_STR8133_REF,

	QPFP_BUS_NUM		// keep last
} EBusTypes;

typedef struct _TBusType {
	EBusTypes		type;
} TBusType, *PTBusType;


typedef struct _TSlaveIF {
	TQP_UINT32		regBase;
	TQP_UINT32		memBase;
	TQP_UINT32		pageSize;
} TSlaveIF, *PTSlaveIF;


typedef enum _EMemTypes {
	QPPF_MEM_TYPE_UNKNOWN = -1,
	QPPF_MEM_TYPE_DDR = 0,
	QPPF_MEM_TYPE_MDDR,

	QPFP_MEM_NUM		// keep last
} EMemTypes;

typedef struct _TMemory {
	EMemTypes		type;
	TQP_UINT32		megaBits;
} TMemory, *PTMemory;


typedef struct _TInterrupt {
	TQP_UINT32		number;
} TInterrupt, *PTInterrupt;

typedef enum _EVideoInModes {
	QPPF_VINMODE_PROG = 0,
	QPPF_VINMODE_PROG_SRC_INTERLACE_FORMAT,
	QPPF_VINMODE_INTERLACE,
	QPPF_VINMODE_SPECIAL_1,
	QPPF_VINMODE_SPECIAL_2,

	QPPF_VINMMODE_NUM
} EVideoInModes;

typedef enum _EVideoInFormats {
	QPPF_VINFORMAT_PROGRESSIVE = 0,
	QPPF_VINFORMAT_INTERLACE,

	QPPF_VINFORMAT_NUM
} EVideoInFormats;

typedef enum _EVideoInClkEdges {
	QPPF_VINCLKEDGE_POSITIVE = 0,
	QPPF_VINCLKEDGE_NEGATIVE,

	QPPF_VINCLKEDGE_NUM
} EVideoInClkEdges;

typedef struct _TVideoIn {
	EVideoInModes		mode;
	EVideoInFormats		format;
	EVideoInClkEdges	edge;
	TQP_UINT32			syncCode1;
	TQP_UINT32			syncCode2;
	TQP_UINT32			startPixel;
	TQP_UINT32			startLine;
} TVideoIn, *PTVideoIn;

typedef enum _EAudioInMSBs {
	QPPF_AIUMSB_LSB = 0,
	QPPF_AIUMSB_MSB,

	QPPF_AIUMSB_NUM
} EAudioInMSBs;

typedef enum _EAudioInLRCLKs {
	QPPF_AIULRCLK_LEFTHIGH = 0,
	QPPF_AIULRCLK_LEFTLOW,

	QPPF_AIULRCLK_NUM
} EAudioInLRCLKs;

typedef enum _EAudioInBCLKs {
	QPPF_AIUBCLK_RISING = 0,
	QPPF_AIUBCLK_FALLING,

	QPPF_AIUBCLK_NUM
} EAudioInBCLKs;

typedef enum _EAudioInI2Ss {
	QPPF_AIUI2S_NORMAL = 0,
	QPPF_AIUI2S_PROGRAM,

	QPPF_AIUI2S_NUM
} EAudioInI2Ss;

typedef enum _EAudioInRJs {
	QPPF_AIURJ_LEFT = 0,
	QPPF_AIURJ_RIGHT,

	QPPF_AIURJ_NUM
} EAudioInRJs;

typedef struct _TAudioIn {
	EAudioInMSBs		msb;
	EAudioInLRCLKs		lrclk;
	EAudioInBCLKs		bclk;
	EAudioInI2Ss		i2s;
	EAudioInRJs			rj;
} TAudioIn, *PTAudioIn;

typedef enum _EVideoOutModes {
	QPPF_VOUMODE_NTSC_INTERLACED = 0,
	QPPF_VOUMODE_PAL_INTERLACED,
	QPPF_VOUMODE_PROGRESS,

	QPPF_VOUMODE_NUM
} EVideoOutModes;

typedef struct _TVideoOut {
	TQP_BOOL			enable;
	EVideoOutModes		mode;
	TQP_UINT32			startPixel;
	TQP_UINT32			startLine;
} TVideoOut, *PTVideoOut;

typedef enum _EAudioOutMSBs {
	QPPF_AOUMSB_LSB = 0,
	QPPF_AOUMSB_MSB,

	QPPF_AOUMSB_NUM
} EAudioOutMSBs;

typedef enum _EAudioOutLRCLKs {
	QPPF_AOULRCLK_LEFTHIGH = 0,
	QPPF_AOULRCLK_LEFTLOW,

	QPPF_AOULRCLK_NUM
} EAudioOutLRCLKs;

typedef enum _EAudioOutBCLKs {
	QPPF_AOUBCLK_FALLING = 0,
	QPPF_AOUBCLK_RISING,

	QPPF_AOUBCLK_NUM
} EAudioOutBCLKs;

typedef enum _EAudioOutI2Ss {
	QPPF_AOUI2S_NORMAL = 0,
	QPPF_AOUI2S_PROGRAM,

	QPPF_AOUI2S_NUM
} EAudioOutI2Ss;

typedef enum _EAudioOutRJs {
	QPPF_AOURJ_LEFT = 0,
	QPPF_AOURJ_RIGHT,

	QPPF_AOURJ_NUM
} EAudioOutRJs;

typedef enum _EAudioOutModes {
	QPPF_AOUMODE_MASTER = 0,
	QPPF_AOUMODE_SLAVE,

	QPPF_AOUMODE_NUM
} EAudioOutModes;

typedef struct _TAudioOut {
	TQP_BOOL			enable;
	EAudioOutMSBs		msb;
	EAudioOutLRCLKs		lrclk;
	EAudioOutBCLKs		bclk;
	EAudioOutI2Ss		i2s;
	EAudioOutRJs		rj;
	EAudioOutModes		mode;
} TAudioOut, *PTAudioOut;

typedef enum _EVidDecoders {
	QPPF_VIDDEC_UNKNOWN = -1,
	QPPF_VIDDEC_DUMMY = 0,
	QPPF_VIDDEC_SAA7115,
	QPPF_VIDDEC_MT9V112,
	QPPF_VIDDEC_OV7720,
	QPPF_VIDDEC_ADI7401,
	QPPF_VIDDEC_PE1005,
	QPPF_VIDDEC_SIL9011,
	QPPF_VIDDEC_SIL9025,

	QPFP_VIDDEC_NUM		// keep last
} EVidDecoders;

typedef struct _TVideoDec {
	EVidDecoders	type;
} TVideoDec, *PTVideoDec;


typedef enum _EVidEncoders {
	QPPF_VIDENC_UNKNOWN = -1,
	QPPF_VIDENC_DUMMY = 0,
	QPPF_VIDENC_SAA7128,
	QPPF_VIDENC_ADI7393,
	QPPF_VIDENC_SIL9030,
	QPPF_VIDENC_SIL9034,

	QPFP_VIDENC_NUM		// keep last
} EVidEncoders;

typedef enum _EVidStandards {
	QPPF_VIDSTD_UNKNOWN = -1,
	QPPF_VIDSTD_NTSC = 0x01,
	QPPF_VIDSTD_PAL = 0x02,
	QPPF_VIDSTD_SECAM = 0x04,
	QPPF_VIDSTD_720P = 0x08,
	QPPF_VIDSTD_1080I = 0x10
} EVidStandards;

typedef struct _TVideoEnc {
	EVidEncoders	type;
	EVidStandards	std;
} TVideoEnc, *PTVideoEnc;


typedef enum _ETuners {
	QPPF_TUNER_UNKNOWN = -1,
	QPPF_TUNER_DUMMY = 0,

	QPFP_TUNER_NUM		// keep last
} ETuners;

typedef struct _TTuner {
	ETuners	type;
} TTuner, *PTTuner;


typedef enum _ETVAudios {
	QPPF_TVAUDIO_UNKNOWN = -1,
	QPPF_TVAUDIO_DUMMY = 0,

	QPFP_TVAUDIO_NUM		// keep last
} ETVAudios;

typedef struct _TTVAudio{
	ETVAudios	type;
} TTVAudio, *PTTVAudio;


typedef enum _EAudCodecs {
	QPPF_AUDCODEC_UNKNOWN = -1,
	QPPF_AUDCODEC_DUMMY = 0,

	QPFP_AUDCODEC_NUM		// keep last
} EAudCodecs;

typedef struct _TAudCodec {
	EAudCodecs	type;
} TAudCodec, *PTAudCodec;


typedef enum _EChipTypes {
	QPPF_CHIPTYPE_UNKNOWN = -1,
	QPPF_CHIPTYPE_BERINGER = 0,		// QL201B, QL202B
	QPPF_CHIPTYPE_ARTESA,
	QPPF_CHIPTYPE_ARTESA_VIRTEX_4,	// FPGA VIRTEX-4 platform
	QPPF_CHIPTYPE_ARTESA_VIRTEX_5,	// FPGA VIRTEX-5 platform
	QPPF_CHIPTYPE_ARTESA_B0,
	QPPF_CHIPTYPE_STONEGATE,
	QPPF_CHIPTYPE_STONEGATE_VIRTEX_4,	// FPGA VIRTEX-4 platform
	QPPF_CHIPTYPE_STONEGATE_VIRTEX_5,	// FPGA VIRTEX-5 platform
	QPPF_CHIPTYPE_OAKVILLE,
	QPPF_CHIPTYPE_OAKVILLE_VIRTEX_4,	// FPGA VIRTEX-4 platform
	QPPF_CHIPTYPE_OAKVILLE_VIRTEX_5,	// FPGA VIRTEX-5 platform

	QPFP_CHIPTYPE_NUM		// keep last
} EChipTypes;

typedef struct _TChip {
	EChipTypes		type;
} TChip, *PTChip;


typedef enum _EFWAPIVersions {
	QPPF_FWAPIVERSION_UNKNOWN = -1,
	QPPF_FWAPIVERSION_0 = 0,
	QPPF_FWAPIVERSION_1 = 1,	// introduced Jun 2009

	QPFP_FWAPIVERSION_NUM		// keep last
} EFWAPIVersions;

typedef struct _TFWAPI {
	EFWAPIVersions		version;
} TFWAPI, *PTFWAPI;


typedef enum _ECodecLibIntModes {
	QPPF_CODECLIBINTMODE_UNKNOWN = -1,
	QPPF_CODECLIBINTMODE_HIU_ISR = 0,
	QPPF_CODECLIBINTMODE_SW1_ISR,

	QPFP_CODECLIBINTMODE_NUM		// keep last
} ECodecLibIntModes;

typedef enum _ECodecLibMemModes {
	QPPF_CODECLIBMEMMODE_UNKNOWN = -1,
	QPPF_CODECLIBMEMMODE_STATIC = 0,
	QPPF_CODECLIBMEMMODE_DYNAMIC,

	QPFP_CODECLIBMEMMODE_NUM		// keep last
} ECodecLibMemModes;

typedef struct _TCodecLib {
	ECodecLibMemModes		memMode;
	ECodecLibIntModes		intMode;
} TCodecLib, *PTCodecLib;


typedef enum _EI2CTypes {
	QPPF_I2CTYPE_UNKNOWN = -1,
	QPPF_I2CTYPE_USE_USB = 0,
	QPPF_I2CTYPE_USE_QLCODEC,
	QPPF_I2CTYPE_USE_201_EMULATION,
	QPPF_I2CTYPE_SOC_INTERNAL_HW,
	QPPF_I2CTYPE_SOC_INTERNAL_GPIO,

	QPFP_I2CTYPE_NUM		// keep last
} EI2CTypes;

typedef struct _TI2C {
	EI2CTypes			type;
	TQP_UINT32			clkBit;
	TQP_UINT32			dataBit;
} TI2C, *PTI2C;

typedef struct _TGPIO {
	TQP_UINT32			directions;
	TQP_UINT32			values;
} TGPIO, *PTGPIO;


typedef enum _EUsbLibGadgetModes {
	QPPF_USBLIBGADGETMODE_UNKNOWN = -1,
	QPPF_USBLIBGADGETMODE_FX2CLONE = 0,
	QPPF_USBLIBGADGETMODE_UVC,
	QPPF_USBLIBGADGETMODE_ZERO,		// For loop back testing

	QPFP_USBLIBGADGETMODE_NUM		// keep last
} EUsblibGadgetModes;

typedef struct _TUSB {
	EUsblibGadgetModes		gagMode;
} TUSB, *PTUSB;


typedef enum _EUsbBoardTypes {
	QPPF_USBBOARDTYPE_UNKNOWN = -1,
	QPPF_USBBOARDTYPE_DEFAULT = 0,
	QPPF_USBBOARDTYPE_FC3,

	QPFP_USBBOARDTYPE_NUM		// keep last
} EUsbBoardTypes;

typedef enum _EUsbSoftConnectCmdTypes {
	QPPF_USBSOFTCONNECTCMD_UNKNOWN = -1,
	QPPF_USBSOFTCONNECTCMD_SWDISCONNECT = 0,
	QPPF_USBSOFTCONNECTCMD_KEEPCONNECT,

	QPFP_USBSOFTCONNECTCMD_NUM		// keep last
} EUsbSoftConnectCmdTypes;


typedef enum _EAttrStates {
	QPPF_ATYPE_DETECTED = 0,
	QPPF_ATYPE_HARDCODED,
	QPPF_ATYPE_OVERRIDDEN,
	QPPF_ATYPE_NOTAVAILABLE,
	QPPF_ATYPE_UNKNOWN
} EAttrStates;

#define QPFP_VALIDATTR(state) ((state == QPPF_ATYPE_DETECTED) || (state == QPPF_ATYPE_HARDCODED) || (state == QPPF_ATYPE_OVERRIDDEN))

typedef enum _EAttrCodes {
	QPPF_ACODE_ACCESS,
	QPPF_ACODE_BUS,
	QPPF_ACODE_SLAVEIF,
	QPPF_ACODE_MEMORY,
	QPPF_ACODE_INTERRUPT,
	QPPF_ACODE_VIDEOIN,
	QPPF_ACODE_AUDIOIN,
	QPPF_ACODE_VIDEOOUT,
	QPPF_ACODE_AUDIOOUT,
	QPPF_ACODE_VIDEODEC,
	QPPF_ACODE_VIDEOENC,
	QPPF_ACODE_TUNER,
	QPPF_ACODE_TVAUDIO,
	QPPF_ACODE_AUDIOCODEC,
	QPPF_ACODE_CHIP,
	QPPF_ACODE_FWAPI,
	QPPF_ACODE_CODECLIB,
	QPPF_ACODE_I2C,
	QPPF_ACODE_GPIO,
	QPPF_ACODE_USB,

	QPPF_ACODE_NUM				// keep last
} EAttrCodes;

typedef union _TQP_PFMAttrStruct {
	TAccessMode			accessMode;
	TBusType			busType;
	TSlaveIF			slaveIF;
	TMemory				memory;
	TInterrupt			interrupt;
	TVideoIn			videoIn;
	TAudioIn			audioIn;
	TVideoOut			videoOut;
	TAudioOut			audioOut;
	TVideoDec			videoDec;
	TVideoEnc			videoEnc;
	TTuner				tuner;
	TTVAudio			tvAudio;
	TAudCodec			audCodec;
	TChip				chip;
	TFWAPI				fwAPI;
	TCodecLib			codecLib;
	TI2C				i2c;
	TGPIO				gpio;
	TUSB				usb;
} TQP_PFMAttrStruct, *PTQP_PFMAttrStruct;

//
// Returns an attribute's current value and status information
// If the reset flag is true, the attribute is returned to its default value
// (possibly triggering an autodetection operation)
//
EAttrStates QPPFMGetAttr (TQP_BOOL reset, EAttrCodes code, PTQP_PFMAttrStruct attrP);

//
// Sets an attribute to a new value (overriding current value)
//
EQP_Errors QPPFMSetAttr (EAttrCodes code, PTQP_PFMAttrStruct attrP);

//
// Gets the current status of the "automatic firmware download" flag
//
TQP_BOOL QPPFMGetFWDwnldFlag (void);

//
// Sets the status of the "automatic firmware download" flag
//
void QPPFMSetFWDwnldFlag (TQP_BOOL fwDF);

//
// Gets the current status of the "do not init hardware" flag
//
TQP_BOOL QPPFMGetHWInitFlag (void);

//
// Sets the status of the "do not init hardware" flag
//
void QPPFMSetHWInitFlag (TQP_BOOL hwIF);

//
// Returns pointer and size of the built-in video firmware
// If ptrP is NULL, only the size is returned, if sizeP is also NULL,
// the FW buffer is freed.
//
EQP_Errors QPPFMGetVideoFW (TQP_UINT8 ** ptrP, PTQP_UINT32 sizeP);

//
// Returns pointer and size of the built-in audio firmware
// If ptrP is NULL, only the size is returned, if sizeP is also NULL,
// the FW buffer is freed
//
EQP_Errors QPPFMGetAudioFW (TQP_UINT8 ** ptrP, PTQP_UINT32 sizeP);


#ifdef __cplusplus
}
#endif

#endif //__QPPFMGR_H__
