///////////////////////////////////////////////////////////////////////////////
//        
//
//		SDIO Manager interface
//
//	Platform-independent interface for controlling the SDIO onboard
//	chips
//
// History:
//	Aug 29, 2008	(EC) Created
//
// Authors: (EC) Enrico Cadorin
//
///////////////////////////////////////////////////////////////////////////////


#ifndef __QPSDIOMGR_H__
#define __QPSDIOMGR_H__

#include "qptypes.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef enum _ECardTypes {
	QPSDIO_CARDTYPE_NONE,
	QPSDIO_CARDTYPE_ERR,
	QPSDIO_CARDTYPE_SD,
	QPSDIO_CARDTYPE_SDIO,
	QPSDIO_CARDTYPE_MMC,
	QPSDIO_CARDTYPE_CEATA,

	QPSDIO_CARDTYPE_NUM
} EQPSDIOCardTypes;

typedef struct _TQPSDIOCardInfo {
	EQPSDIOCardTypes		type;
	TQP_BOOL		is2_0;			// TRUE for SD 2.0 memory cards
	TQP_BOOL		isHC;			// TRUE for High Capacity memory cards
	TQP_BOOL		isHighSpeed;	// TRUE when card is set in High Speed mode
	TQP_UINT32		bytesPerBlock;
	TQP_UINT64		blocksNum;
	TQP_BOOL		readOnly;		// TRUE when card write protect switch is on
	TQP_UINT8		csd[16];
	TQP_UINT8		scr[8];
	TQP_UINT8		cid[16];
} TQPSDIOCardInfo, *PTQPSDIOCardInfo;

typedef struct _TSDIOReq {
	int					slot;
	TQP_UINT64			startBlk;
	TQP_UINT32			blkNum;
	unsigned char *		dP;
	EQP_Errors			err;
} TQPSDIOReq, *PTQPSDIOReq;

//
// Capabilities structure
typedef struct _TQPSDIOCaps {
	TQP_BOOL	io;				// true if IO is supported
	TQP_BOOL	ceATA;			// true if CE/ATA is supported
	TQP_BOOL	mmc;			// true if MMC is supported
} TQPSDIOCaps, *PTQPSDIOCaps;

//
// Returns the list of capabilities for this implementation
//
void QPSDIOGetCaps (PTQPSDIOCaps capsP);

// Start/Stop the SDIO driver
TQP_BOOL SDIOStart (void);
TQP_BOOL SDIORunning (void);
TQP_BOOL SDIOStop (void);

// Slot management
extern TQP_UINT32 QPSDIOGetSlotMap (void);
extern TQP_UINT32 QPSDIOGetSlotsNum (void);
extern TQP_BOOL QPSDIOCardInfo (TQP_UINT8 slot, PTQPSDIOCardInfo cardInfoP);

// Interface to the memory card portion of the SDIO manager
typedef void (*TQPSDIOMemCallback) (void * refCon, TQP_UINT32 slotMap);

extern TQP_BOOL QPSDIOMemRegister (TQPSDIOMemCallback cb, void * refCon);
extern void QPSDIOMemUnregister (void);
extern EQP_Errors QPSDIOMemReadWrite (TQP_BOOL write, PTQPSDIOReq ioReqP);


#ifdef __cplusplus
}
#endif

#endif //__QPSDIOMGR_H__

