///////////////////////////////////////////////////////////////////////////////
//     
//
//		qpsos.h
//
// History:
//	Mar 26, 2008	(EC) Created
//
// Authors: (EC) Enrico Cadorin
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __QPSOS_H__
#define __QPSOS_H__

#include "qptypes.h"
//#include <stdint.h>


//
// Versioning interface
//

typedef enum {
	QPSOS_MOD_OS_TAG = 0x534F5351,
	QPSOS_MOD_CODECLIB_TAG = 0x4C434443,
	QPSOS_MOD_AUDIO_TAG = 0x49445541,
	QPSOS_MOD_USBLIB_TAG = 0x4C425355,
	QPSOS_MOD_USBLOLIB_TAG = 0x53554F4C,
	QPSOS_MOD_FATLIB_TAG = 0x4C544146,
	QPSOS_MOD_SDIOLIB_TAG = 0x4F494453,
	QPSOS_MOD_CODECDRV_TAG = 0x44434443,
	QPSOS_MOD_FC3LIB_TAG = 0x4C334346,
	QPSOS_MOD_USBMGR_TAG = 0x4D425355,
	QPSOS_MOD_USBTEST_TAG = 0x54425355,
	QPSOS_MOD_USBDEVICE_TAG = 0x44425355,
} EModuleTags;


#ifdef _QP_QPSOS

typedef enum {
	QPSOS_MOD_OS,
	QPSOS_MOD_CODECLIB,
	QPSOS_MOD_AUDIO,
	QPSOS_MOD_USBLIB,
	QPSOS_MOD_USBLOLIB,		// hack to support the FaceVision request
	QPSOS_MOD_FATLIB,
	QPSOS_MOD_SDIOLIB,
	QPSOS_MOD_CODECDRV,
	QPSOS_MOD_FC3LIB,
	QPSOS_MOD_USBMGR,
	QPSOS_MOD_USBTEST,
	QPSOS_MOD_USBDEVICE,

	QPSOS_MOD_NUM
} EModules;

typedef union {
	TQP_UINT8	bytes[16];
	TQP_UINT32	words[4];
	__packed struct {
		TQP_UINT32		tag;
		TQP_UINT16		minor;
		TQP_UINT16		major;
		TQP_UINT8		monthH;
		TQP_UINT8		monthL;
		TQP_UINT8		dayH;
		TQP_UINT8		dayL;
		TQP_UINT8		yearTh;
		TQP_UINT8		yearHu;
		TQP_UINT8		yearTe;
		TQP_UINT8		yearUn;
	} fields;
} TQPSOSVersionBlock, *PTQPSOSVersionBlock;

#define ModuleTag(modNum) ((modNum == QPSOS_MOD_OS) ? QPSOS_MOD_OS_TAG	\
						: ((modNum == QPSOS_MOD_CODECLIB) ? QPSOS_MOD_CODECLIB_TAG	\
						: ((modNum == QPSOS_MOD_AUDIO) ? QPSOS_MOD_AUDIO_TAG	\
						: ((modNum == QPSOS_MOD_USBLIB) ? QPSOS_MOD_USBLIB_TAG	\
						: ((modNum == QPSOS_MOD_USBLOLIB) ? QPSOS_MOD_USBLOLIB_TAG	\
						: ((modNum == QPSOS_MOD_FATLIB) ? QPSOS_MOD_FATLIB_TAG	\
						: ((modNum == QPSOS_MOD_SDIOLIB) ? QPSOS_MOD_SDIOLIB_TAG	\
						: ((modNum == QPSOS_MOD_CODECDRV) ? QPSOS_MOD_CODECDRV_TAG	\
						: ((modNum == QPSOS_MOD_FC3LIB) ? QPSOS_MOD_FC3LIB_TAG	\
						: ((modNum == QPSOS_MOD_USBMGR) ? QPSOS_MOD_USBMGR_TAG	\
						: ((modNum == QPSOS_MOD_USBTEST) ? QPSOS_MOD_USBTEST_TAG	\
						: ((modNum == QPSOS_MOD_USBDEVICE) ? QPSOS_MOD_USBDEVICE_TAG	\
						: QPSOS_MOD_NUM))))))))))))

PTQPSOSVersionBlock QPSOSGetVersion (EModules module);

#else //_QP_QPSOS

//
// Macro to extract the major version of QPSOS.
// startAddr is the address of a local memory buffer that should point to
// the QPSOS version record.  This is usually located at offset 0x100 of
// the firmware image.  Only the first two words are accessed.
//
#if defined (QP_LINXKER)
#define QPSOSGetOSMajorVersion(startAddr)	\
	((*((PTQP_UINT32) startAddr) == QPSOS_MOD_OS_TAG) ?	\
	(*((PTQP_UINT16) (((long) startAddr) + 6))) : 0)
#else
#if defined (__x86_64__)
#define QPSOSGetOSMajorVersion(startAddr)	\
	((*((PTQP_UINT64) startAddr) == QPSOS_MOD_OS_TAG) ?	\
	(*((PTQP_UINT16) (((__int64_t) startAddr) + 6))) : 0)
#else
#if defined(_M_AMD64) || defined(_M_IA64)
#define QPSOSGetOSMajorVersion(startAddr)	\
	((*((PTQP_UINT64) startAddr) == QPSOS_MOD_OS_TAG) ?	\
	(*((PTQP_UINT16) (((__int64) startAddr) + 6))) : 0)
#else
#define QPSOSGetOSMajorVersion(startAddr)	\
	((*((PTQP_UINT32) startAddr) == QPSOS_MOD_OS_TAG) ?	\
	(*((PTQP_UINT16) (((int) startAddr) + 6))) : 0)
#endif
#endif

#endif

#endif //!_QP_QPSOS


// Special shared memory area mappings.

// Old method (QPSOS 2)

// This area is no longer guaranteed to be flatly mapped by the memory manager
// since it will have to be relocated when running with Linux.
// It is guaranteed to be at 0x2F0000-2FFFFF (64 kB) in physical memory
// It is used for global storage and for debugging information.
// The first 32kB are available to be used by the QPSOS modules and they are
// statically allocated in chunks of 4kB according to the definitions below.
// The remaing 32 kB are reserved to QPSOS use (stacks, translation tables)
//
// WARNING: These addresses are PHYSICAL and therefore they might need to be
// converted to VIRTUAL before being used.
//
#define PHY_COMMONAREA 0x002F0000	// 64kB (up to 2FFFFF)

#define PHY_QPSOSAREA 0x002F0000	// 4kB (up to 2F0FFF) for QPSOS use

#define PHY_CODECAREA 0x002F1000	// 4kB (up to 2F1FFF) for the Codec's needs

#define PHY_USBAREA 0x002F2000		// 4kB (up to 2F2FFF) USB
#define FLAT_USBAREA PHY_USBAREA	// for backward compatibility

#define PHY_BOOTAREA 0x002F3000		// 4kB (up to 2F3FFF) for boot parameters
									// set by the bootloader or the host driver



// New method for QPSOS3

#ifdef _QP_QPSOS

// The Common area is located at a different address from 2F0000.
// It is still 64kB.  However, it might be moved again if a different chip
// architecture is implemented; therefore a function to get a module's space
// in the shared area is provided.
// A module's space is 4kB.

// Returns FALSE if the module does not have a shared space allocated to it.
TQP_BOOL QPSOSGetSharedAddresses (EModules	module,
								void ** 	phyAddrP,
								void **		virCachedAddrP,
								void **		virNoncachedAddrP);

#endif //_QP_QPSOS

#define PHY_SHARED_AREA		0x0F0000	// 64k at the end of the first MegaByte

#define PHY_BOOT_AREA		(PHY_SHARED_AREA + 0x0)		// boot parameters shared with loaders (4kB)
#define PHY_CODEC_AREA		(PHY_SHARED_AREA + 0x1000)	// 4kB for the CODEC library
#define PHY_USB_AREA		(PHY_SHARED_AREA + 0x2000)	// 4kB for the USB module

///////////////////////////////////////////////////////////////////////////////

#ifdef _QP_QPSOS


typedef enum {
	QPSOS_ME_INIT,
	QPSOS_ME_TERM,
	QPSOS_ME_START,
	QPSOS_ME_STOP,
} EQPSOSModEntryCodes;

typedef void * (*TQPSOSModEntryProc) (EQPSOSModEntryCodes, ...);

//
// Reboot firmware
//
typedef enum {
	QPSOS_REBOOT,			// reboot currently running firmware
	QPSOS_REBOOT_SWITCH,	// switch to loader from main or to main from anywhere else
	QPSOS_REBOOT_ROM,		// switch to ROM firmware if present
	QPSOS_REBOOT_FLASH,		// switch to Flash firmware if present
	QPSOS_REBOOT_SUSPRES,	// jump to Suspend/Resume code in ROM
} EQPSOSRebootModes;

void QPSOSReboot (EQPSOSRebootModes mode);

//
// Returns TRUE if running loader firmware, FALSE otherwise.
//
TQP_BOOL QPSOSIsLoader (void);


//
// Functions to safely access registers
//
#ifdef BUILD_TYPE_ROM

TQP_UINT32 QPSOSRegRead (TQP_UINT32 addr);
void QPSOSRegWrite (TQP_UINT32 addr, TQP_UINT32 val);
#else //BUILD_TYPE_ROM

extern TQP_UINT32 RegVirBase;

static __inline TQP_UINT32
QPSOSRegRead (TQP_UINT32 addr)
{
	return (*((volatile TQP_UINT32 *) (((TQP_UINT16) addr) + RegVirBase)));
}

static __inline void
QPSOSRegWrite (TQP_UINT32 addr, TQP_UINT32 val)
{
	*((volatile TQP_UINT32 *) (((TQP_UINT16) addr) + RegVirBase)) = (unsigned long) (val);
}

#endif

void QPSOSRegSetBits (TQP_UINT32 addr, TQP_UINT32 mask);
void QPSOSRegClearBits (TQP_UINT32 addr, TQP_UINT32 mask);
void QPSOSRegModify (TQP_UINT32 addr, TQP_UINT32 clearMask, TQP_UINT32 setMask);
int SetFIQ (int enable);
int SetIRQ (int enable);

//
// Interface to safely access USB registers (using indirect method)
//

TQP_UINT32 QPSOSUSBRegRead (TQP_UINT32 addr);
void QPSOSUSBRegWrite (TQP_UINT32 addr, TQP_UINT32 val);
void QPSOSUSBRegSetBits (TQP_UINT32 addr, TQP_UINT32 mask);
void QPSOSUSBRegClearBits (TQP_UINT32 addr, TQP_UINT32 mask);
void QPSOSUSBRegModify (TQP_UINT32 addr, TQP_UINT32 clearMask, TQP_UINT32 setMask);

//
// Interface to safely access SDIO registers (using indirect method)
//

TQP_UINT32 QPSOSSDRegRead (TQP_UINT32 addr);
void QPSOSSDRegWrite (TQP_UINT32 addr, TQP_UINT32 val);


//
// I2c interface
//

typedef enum {
	QPSOS_I2CMASTER_DEFAULT,	// the currently configured one (returned by QPPFMGetAttr)
	QPSOS_I2CMASTER_INTERNALHW,	// use the QL/PL chip's IIC module
	QPSOS_I2CMASTER_GPIO,		// use 2 GPIO pins
} EI2CMasters;

typedef struct {
	EI2CMasters		master;
	TQP_BOOL		read;		// TRUE for reading, FALSE for writing
	TQP_UINT8		addr;		// device address (7 bits)
	PTQP_UINT8		bufP;		// data buffer: must have at least max(toWrite,toRead) bytes
	unsigned int	toWrite;	// number of bytes to write	(first byte is the device address)
	unsigned int	toRead;		// number of bytes to read
} TQPSOSI2CXferBlock, *PTQPSOSI2CXferBlock;

TQP_BOOL QPSOSI2CTransfer (PTQPSOSI2CXferBlock tbP);

//
// u64toa
// returns a pointer into the caller's string where the ascii representation
// of a 64-bit unsigned integer starts (ARM compiler does not support itoa).
// base can be any value such that 0 < base <= 16.
//
#define MAX_U64_BASE10_CHARS 21
#define MAX_U64_BASE16_CHARS 17
#define MAX_U64_BASE2_CHARS 65
char * u64toa (TQP_UINT64 i64, unsigned int base,
				char * bufP, unsigned int bufSize);

//
// Return the number of leading zero bits in the argument
//
TQP_UINT32 CountLeadingZeros (TQP_UINT32 val);

//
// Return the number of trailing zero bits in the argument
//
TQP_UINT32 CountTrailingZeros (TQP_UINT32 val);

#endif //_QP_QPSOS


#endif //__QPSOS_H__

