///////////////////////////////////////////////////////////////////////////////
//
//    
//
//	qpsosdbg.h
//
// Modification History:
//	Nov 12, 07	(EC) Created
//
// Authors:
//	(EC)	Enrico Cadorin
//
///////////////////////////////////////////////////////////////////////////////

//
// External interface to the Simple OS Debug Manager

#ifndef __QPSOSDBG_H__
#define __QPSOSDBG_H__


#include "qptypes.h"


#ifdef __cplusplus
extern "C" {
#endif

#define DBG_TAG		0x62645051		// "QPdb"

#define DBGINFO_CHK	0x69645051		// "QPdi"
#define MAX_NAMELEN	32
#define EXT_PF_ENTRIES (1024)
#define MAX_COMPS	32
#define MAX_ARGS	5

typedef struct _TCompBlk {
	TQP_BOOL		registered;
	char			name[MAX_NAMELEN+1];
	TQP_UINT32		level;
} TCompBlk, *PTCompBlk;

typedef struct _TCompInfo {
	TQP_BOOL		printName;
	TQP_BOOL		printTime;
	TCompBlk		components[MAX_COMPS];
	TQP_UINT32		checkSum;	// to detect if it has changed
} TCompInfo, *PTCompInfo;

typedef struct _TPFEntry {
	TQP_UINT32	timeStamp;
	TQP_UINT32	timeStampH;
	TQP_UINT32	compLevel;
	TQP_UINT32	fmtPtr;
	TQP_UINT32	arg[MAX_ARGS];
} TPFEntry, *PTPFEntry;

//
// printf queue structure; it is a list of EXT_PF_ENTRIES printf entries
// in the device memory; the ARM (QPSOS) writes one entry per each debug
// output string and updates writeEntry accordingly; the host (qapp)
// reads available entry and updates readEntry accordingly; the queue is
// empty when readEntry is the same as writeEntry; therefore one entry must
// be left empty at all times (the ARM must check before writing); there
// isn't any race condition since each side (host and ARM) only writes its own
// entry index and the read or write operations are atomic (one word).
//

typedef struct {
	TQP_UINT32	readEntry;		// index of next entry for the host to read
	TQP_UINT32	nextEntry;		// index of next entry to be written by QPSOS
} TPFIndexes, *PTPFIndexes;

typedef struct _TPFQueue {
	TPFIndexes	indexes;
	TPFEntry	entries[EXT_PF_ENTRIES];
} TPFQueue, *PTPFQueue;


typedef struct _TDbgInfo {
	TQP_UINT32		check;
	PTCompInfo		compInfoP;
	PTPFQueue		pfQueueP;
	TQP_UINT32		sysClkKhz;
	TQP_UINT32		checkSum;	// to detect if it has changed
} TDbgInfo, *PTDbgInfo;

// This structure must be 16 bytes or less so it can fit in the low memory tag area
typedef struct _TDbgTag {
	TQP_UINT32	tags[3];
	TQP_UINT32	dbgInfoAddr;	
} TDbgTag, *PTDbgTag;

#ifdef __cplusplus
}
#endif

#endif // __QPSOSDBG_H__
