///////////////////////////////////////////////////////////////////////////////
//        
//
//		Time Manager interface
//
//	Platform-independent interface
//
// History:
//	May 29, 2007	(EC) Created
//
// Authors: (EC) Enrico Cadorin
//
///////////////////////////////////////////////////////////////////////////////


#ifndef __QPTIMEMGR_H__
#define __QPTIMEMGR_H__

#include "qptypes.h"

#ifdef __cplusplus
extern "C" {
#endif

//
typedef TQP_UINT64 TQPTM_Time;


//
// Capabilities structure
typedef struct TQPTMCaps {
 	TQP_UINT32		resolution;
	TQP_BOOL		hasRealTime;	
	TQP_BOOL		hasTimers;	
} TQP_TMCaps, *PTQP_TMCaps;

//
// Returns the list of capabilities for this implementation
//
void QPTMGetCaps (PTQP_TMCaps capsP);


//
// Getting/setting time
//
TQPTM_Time QPTMGetSysTimeMilliS (void);
TQPTM_Time QPTMGetSysTimeMicroS (void);
TQPTM_Time QPTMGetSysTimeNanoS (void);

TQPTM_Time QPTMGetSysTicks (void);


TQPTM_Time QPTMGetRealTime (void);
void QPTMSetRealTime (TQPTM_Time time);

//
// Delay thread execution.
//
EQP_Errors QPTMDelayMilliS (TQP_UINT32 milliS);
EQP_Errors QPTMDelayMicroS (TQP_UINT32 microS);


///////////////////////////////////////////////////////////////////////////////
//
// Timers
//

// timer ID
typedef void * TQP_TMTimerID;

// timer callback
typedef void (*TQP_TMCallBack) (void * refCon);

EQP_Errors QPTMStartTimer (
 					TQP_TMTimerID *	tIDP,
 					TQP_TMCallBack	cbP,
 					void *			refCon,
 					TQP_UINT32		microS);

EQP_Errors QPTMStartPeriodicTimer (
 					TQP_TMTimerID *	tIDP,
 					TQP_TMCallBack	cbP,
 					void *			refCon,
 					TQP_UINT32		microS);

EQP_Errors QPTMKillTimer (TQP_TMTimerID tID);


///////////////////////////////////////////////////////////////////////////////
//
// Helper functions
//

typedef struct TQPTMDate {
	TQP_UINT16		year;			// 1980-
 	TQP_UINT8		month;			// 1-12
	TQP_UINT8		day;			// 1-31
	TQP_UINT8		hour;			// 0-23
	TQP_UINT8		minute;			// 0-59
	TQP_UINT8		second;			// 0-59
	TQP_UINT32		microSecond;	// 0-999,999
} TQP_TMDate, *PTQP_TMDate;

void QPTMRealTimeToDate (TQPTM_Time time, PTQP_TMDate dateP);

TQPTM_Time QPTMDateToRealTime (const PTQP_TMDate dateP);

#ifdef __cplusplus
}
#endif

#endif //__QPTIMEMGR_H__
