///////////////////////////////////////////////////////////////////////////////
//        
//
// Qptuner.h
//
// Description: 
//
//  This is the definations of the tuner interface
// 
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __QPTUNER_H__
#define __QPTUNER_H__


#include "qptypes.h"
#include "qpgenericdevice.h"

#ifdef __cplusplus
extern "C" {
#endif


/////////////////////////////////////////////////////////////////////////////
//
//

typedef enum _QPPROPERTY_TUNER_MODES
{
    QPPROPERTY_TUNER_MODE_TV            = 0x0001,
    QPPROPERTY_TUNER_MODE_FM_RADIO      = 0x0002,
    QPPROPERTY_TUNER_MODE_AM_RADIO      = 0x0004,
    QPPROPERTY_TUNER_MODE_DSS           = 0x0008,
    QPPROPERTY_TUNER_MODE_ATSC          = 0x0010	// also used for DVB-T, DVB-C
} QPPROPERTY_TUNER_MODES;


typedef enum _QP_TUNER_TUNING_FLAGS
{
    QP_TUNER_TUNING_EXACT = 1,		// No fine tuning
    QP_TUNER_TUNING_FINE,			// Fine grained search
    QP_TUNER_TUNING_COARSE,			// Coarse search
} QP_TUNER_TUNING_FLAGS;


typedef enum _QP_TUNER_STRATEGY
{
    QP_TUNER_STRATEGY_PLL             = 0x01, // Tune by PLL offset
    QP_TUNER_STRATEGY_SIGNAL_STRENGTH = 0x02, // Tune by signal strength
    QP_TUNER_STRATEGY_DRIVER_TUNES    = 0x04, // Driver does fine tuning
} QP_TUNER_STRATEGY;



/////////////////////////////////////////////////////////////////////////////
//
//

// device specific initialization data
//

typedef enum _QPTUNER_TYPE
{
	TUNER_DUMMY = 0,
	TUNER_INVALID = -1
} QPTUNER_TYPE, *PQPTUNER_TYPE;


typedef struct _QPTUNER_INITDATA
{
	QPTUNER_TYPE	type;
} QPTUNER_INITDATA, *PQPTUNER_INITDATA;



/////////////////////////////////////////////////////////////////////////////
//
//

// property supported by this interface
//



/////////////////////////////////////////////////////////////////////////////
//
//

// codec cllback command code
//



/////////////////////////////////////////////////////////////////////////////
//
//


#ifndef __ITUNER_FWD_DEFINED__
#define __ITUNER_FWD_DEFINED__
typedef struct ITuner ITuner;
#endif //__ITUNER_FWD_DEFINED__

    struct ITuner
    {
		// IGenericDevice
		DECLARE_IGENERICDEVICE(ITuner)

		// ITuner
		TQP_BOOL (*SetFrequency)(ITuner *This, 
								 TQP_UINT32 ulFrequency,			// Frequency to set
								 TQP_UINT32 ulCountry				// Country code
								 );
		TQP_BOOL (*SetFMFrequency)(ITuner *This, 
								   TQP_UINT32 ulFrequency,			// Frequency to set
		  						   TQP_UINT32 ulCountry				// Country code
		  						   );
		TQP_BOOL (*GetPLLOffsetAndBusyStatus)(ITuner *This, 
											  TQP_BOOL *pbPllLocked,// Returns PLL locked flag here
		  									  PTQP_INT32 plPllOffset// Returns PLL offset flag here
		  								  );
		TQP_BOOL (*GetSignalStrength)(ITuner *This, 
									  TQP_BOOL *pbPllLocked, 
		  							  PTQP_UINT32 pulSignalStrenth
		  							  );

		void (*SetMode)(ITuner *This, 
						TQP_UINT32 mode
						);
		TQP_UINT32 (*GetMode)(ITuner *This);
		TQP_BOOL (*SetStandard)(ITuner *This, 
								TQP_UINT32 standard
								);
		TQP_UINT32 (*GetStandard)(ITuner *This);
		TQP_BOOL (*SetCountry)(ITuner *This, 
							   TQP_UINT32 country
							   );
		TQP_UINT32 (*GetCountry)(ITuner *This);

		TQP_UINT32 (*GetSupportedModes)(ITuner *This);
		TQP_UINT32 (*GetSupportedStandards)(ITuner *This, 
											TQP_UINT32 mode
											);
		TQP_UINT32 (*GetMinFrequency)(ITuner *This, 
									  TQP_UINT32 mode
									  );
		TQP_UINT32 (*GetMaxFrequency)(ITuner *This, 
									  TQP_UINT32 mode
									  );
		TQP_UINT32 (*GetNumInputs)(ITuner *This, 
								   TQP_UINT32 mode
								   );
		TQP_UINT32 (*GetTuningGranularity)(ITuner *This, 
										   TQP_UINT32 mode
										   );
		TQP_UINT32 (*GetSettlingTime)(ITuner *This, 
									  TQP_UINT32 mode
									  );
		TQP_UINT32 (*GetTuningStrategy)(ITuner *This, 
										TQP_UINT32 mode
										);
		TQP_UINT32 (*GetTuningStrategy2)(ITuner *This);

		TQP_UINT32 (*GetFrequency)(ITuner *This);
		TQP_UINT32 (*GetLastFrequency)(ITuner *This);
		TQP_UINT32 (*GetTuningFlags)(ITuner *This);
		TQP_UINT32 (*GetVideoSubChannel)(ITuner *This);
		TQP_UINT32 (*GetAudioSubChannel)(ITuner *This);
		TQP_UINT32 (*GetChannel)(ITuner *This);
		void (*SetLastFrequency)(ITuner *This, 
								 TQP_UINT32 ulLastFreq
								 );
		void (*SetTuningFlags)(ITuner *This, 
							   TQP_UINT32 ulTuningFlags
							   );
		void (*SetVideoSubChannel)(ITuner *This, 
								   TQP_UINT32 ulVidSubChannel
								   );
		void (*SetAudioSubChannel)(ITuner *This, 
								   TQP_UINT32 ulAudSubChannel
								   );
		void (*SetChannel)(ITuner *This, 
						   TQP_UINT32 ulChannel
						   );

		TQP_UINT32 (*GetInputIndex)(ITuner *This);
    };


/////////////////////////////////////////////////////////////////////////////
//
//

#define QPTuner_SetFrequency(This, ulFrequency, ulCountry) \
    (This)->SetFrequency(This, ulFrequency, ulCountry)

#define QPTuner_SetFMFrequency(This, ulFrequency, ulCountry) \
    (This)->SetFMFrequency(This, ulFrequency, ulCountry)

#define QPTuner_GetPLLOffsetAndBusyStatus(This, pbPllLocked, plPllOffset) \
    (This)->GetPLLOffsetAndBusyStatus(This, pbPllLocked, plPllOffset)

#define QPTuner_GetSignalStrength(This, pbPllLocked, pulSignalStrenth) \
    (This)->GetSignalStrength(This, pbPllLocked, pulSignalStrenth)

#define QPTuner_SetMode(This, mode) \
    (This)->SetMode(This, mode)

#define QPTuner_GetMode(This) \
    (This)->GetMode(This)

#define QPTuner_SetStandard(This, standard) \
    (This)->SetStandard(This, standard)

#define QPTuner_GetStandard(This) \
    (This)->GetStandard(This)

#define QPTuner_SetCountry(This, country) \
    (This)->SetCountry(This, country)

#define QPTuner_GetCountry(This) \
    (This)->GetCountry(This)

#define QPTuner_GetSupportedModes(This) \
    (This)->GetSupportedModes(This)

#define QPTuner_GetSupportedStandards(This, mode) \
    (This)->GetSupportedStandards(This, mode)

#define QPTuner_GetMinFrequency(This, mode) \
    (This)->GetMinFrequency(This, mode)

#define QPTuner_GetMaxFrequency(This, mode) \
    (This)->GetMaxFrequency(This, mode)

#define QPTuner_GetNumInputs(This, mode) \
    (This)->GetNumInputs(This, mode)

#define QPTuner_GetTuningGranularity(This, mode) \
    (This)->GetTuningGranularity(This, mode)

#define QPTuner_GetSettlingTime(This, mode) \
    (This)->GetSettlingTime(This, mode)

#define QPTuner_GetTuningStrategy(This, mode) \
    (This)->GetTuningStrategy(This, mode)

#define QPTuner_GetTuningStrategy2(This) \
    (This)->GetTuningStrategy2(This)

#define QPTuner_GetFrequency(This) \
    (This)->GetFrequency(This)

#define QPTuner_GetLastFrequency(This) \
    (This)->GetLastFrequency(This)

#define QPTuner_GetTuningFlags(This) \
    (This)->GetTuningFlags(This)

#define QPTuner_GetVideoSubChannel(This) \
    (This)->GetVideoSubChannel(This)

#define QPTuner_GetAudioSubChannel(This) \
    (This)->GetAudioSubChannel(This)

#define QPTuner_GetChannel(This) \
    (This)->GetChannel(This)

#define QPTuner_SetLastFrequency(This, ulLastFreq) \
    (This)->SetLastFrequency(This, ulLastFreq)

#define QPTuner_SetTuningFlags(This, ulTuningFlags) \
    (This)->SetTuningFlags(This, ulTuningFlags)

#define QPTuner_SetVideoSubChannel(This, ulVidSubChannel) \
    (This)->SetVideoSubChannel(This, ulVidSubChannel)

#define QPTuner_SetAudioSubChannel(This, ulAudSubChannel) \
    (This)->SetAudioSubChannel(This, ulAudSubChannel)

#define QPTuner_SetChannel(This, ulChannel) \
    (This)->SetChannel(This, ulChannel)

#define QPTuner_GetInputIndex(This) \
    (This)->GetInputIndex(This)

#ifdef __cplusplus
}
#endif


#endif //__QPTUNER_H__

