///////////////////////////////////////////////////////////////////////////////
//     
//
//		Common types and constants definitions
//
//
// History:
//	Aug 4, 2006	(EC) Created
//
// Authors: (EC) Enrico Cadorin
//
///////////////////////////////////////////////////////////////////////////////


#ifndef __QPTYPES_H__
#define __QPTYPES_H__

#if !defined (QP_LINXKER)
#define BIT(pos) (1 << (pos))
#else
#include "qlautoconf.h"
#if !defined(CONFIG_HOST_PLATFORM_X86_LINUX)
#define BIT(pos) (1 << (pos))
#endif
#endif

typedef int TQP_BOOL;
#define QP_TRUE	1
#define QP_FALSE	0

#define QP_NULL	0

#define QP_SWAP32(ch4)	\
		((((TQP_UINT32)(ch4) & 0xFF) << 24) |		\
		(((TQP_UINT32)(ch4) & 0xFF00) << 8) |		\
		(((TQP_UINT32)(ch4) & 0xFF0000) >> 8) |		\
		(((TQP_UINT32)(ch4) & 0xFF000000) >> 24))

#define QP_MAKEFOURCC(ch0, ch1, ch2, ch3)	\
		((TQP_UINT32)(ch0 & 0xFF) | ((TQP_UINT32)(ch1 & 0xFF) << 8) |			\
		((TQP_UINT32)(ch2 & 0xFF) << 16) | ((TQP_UINT32)(ch3 & 0xFF) << 24 ))

#define QP_MAKETWOCC(ch0, ch1)	\
		((TQP_UINT32)(ch0 & 0xFF) | ((TQP_UINT32)(ch1 & 0xFF) << 8))

typedef void * TQP_PVOID;

// These definitions are platform dependent
#if defined(WIN32)
#define longint64 __int64
#elif defined (__ghs__)		// Green Hills ARM compilers
#define longint64 long long
#elif defined (__arm__)		// ARM compilers
#define longint64 long long
#elif defined (QP_LINXKER)
#define longint64 long long
#elif defined (__LINUX__)       // LINUX
#define longint64 long long
#elif defined (__GNUC__)	// Mac OS X
#define longint64 long long
#endif
typedef longint64 TQP_INT64;
typedef TQP_INT64 *PTQP_INT64;
typedef unsigned longint64 TQP_UINT64;
typedef TQP_UINT64 *PTQP_UINT64;

// int is 32 bit on both LP64(Linux 64 bit) and LLP64(Win64)data model
//
typedef int TQP_INT32;
typedef TQP_INT32 *PTQP_INT32;
typedef unsigned int TQP_UINT32;
typedef TQP_UINT32 *PTQP_UINT32;

typedef short TQP_INT16;
typedef TQP_INT16 *PTQP_INT16;
typedef unsigned short TQP_UINT16;
typedef TQP_UINT16 *PTQP_UINT16;

typedef char TQP_INT8;
typedef TQP_INT8 *PTQP_INT8;
typedef signed char TQP_SINT8;
typedef TQP_SINT8 *PTQP_SINT8;
typedef unsigned char TQP_UINT8;
typedef TQP_UINT8 *PTQP_UINT8;

typedef TQP_INT8 TQP_CHAR, *PTQP_CHAR;
typedef TQP_UINT8 TQP_UCHAR, *PTQP_UCHAR;

typedef struct _TQP_Version {
	TQP_UINT16	major;
	TQP_UINT16	minor;
} TQP_Version, *PTQP_Version;

typedef enum _EQPErrors {
	QPERR_SUCCESS = 0,
	QPERR_PENDING = 1,			// requested operation pending
	QPERR_FAIL = -1,			// generic error
	QPERR_NOMEM = -2,			// insufficient memory
	QPERR_PARMS = -3,			// parameters error
	QPERR_NOTIMPL = -4,			// not implemented
	QPERR_NOTSUPP = -5,			// not supported
	QPERR_TIMEOUT = -6,			// timed out
	QPERR_INVALID = -7,			// invalid operation in the current state
	QPERR_CANCELLED = -8,		// requested operation cancelled
	QPERR_NOTFOUND = -9,		// requested entry not found
	QPERR_AGAIN = -10,			// try again
	QPERR_CTRL_VERSION = -11
} EQP_Errors;

#define QP_SUCCESS(Status) ((Status) >= 0)
#define QP_FAIL(Status) ((Status) < 0)


typedef struct _TQP_GUID
{
	unsigned long	Data1;
	unsigned short	Data2;
	unsigned short	Data3;
	unsigned char	Data4[8];
} TQP_GUID, *TQP_REFGUID, *PTQP_GUID;


typedef unsigned long		TQP_OBJECT_HANDLE, *PTQP_OBJECT_HANDLE;	//handle of an object
#define TQP_NULL_HANDLE		((TQP_OBJECT_HANDLE)(-1))

#include "platform.h"


#ifdef QP_LINXKER
//peter
//#define PVOID       TQP_PVOID
//#define BOOLEAN     TQP_BOOL
//#define UCHAR       TQP_UCHAR
//#define USHORT      TQP_UINT16
//#define ULONG       TQP_UINT32
#define PULONG      PTQP_UINT32
//#define ULONGLONG   TQP_UINT64
#define PULONGLONG  PTQP_UINT64
#endif

#ifndef QP_LINXKER
#ifndef _TIME_T_DEFINED
#define _TIME_T_DEFINED
#ifndef _WIN64
typedef long		time_t;
#else
typedef long long	time_t;
#endif
#endif
#endif

#endif //__QPTYPES_H__
