///////////////////////////////////////////////////////////////////////////////
//     
//
// Qpviddecoder.h
//
// Description:
//
//  This is the definations of the video decoder chip interface
//
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __QPVIDDECODER_H__
#define __QPVIDDECODER_H__


#include "qptypes.h"
#include "qpgenericdevice.h"

#ifdef __cplusplus
extern "C" {
#endif


/////////////////////////////////////////////////////////////////////////////
//
//

// video standard
//
typedef enum _VIDEO_STANDARD
{
	VIDEO_STANDARD_NTSC = 0x01,
	VIDEO_STANDARD_PAL = 0x02,
	VIDEO_STANDARD_SECAM = 0x04,

	VIDEO_STANDARD_720P = 0x08,
	VIDEO_STANDARD_1080I = 0x10,
	VIDEO_STANDARD_1080P = 0x20,

	VIDEO_STANDARD_INVALID = -1
} VIDEO_STANDARD, *PVIDEO_STANDARD;


typedef enum _VIDEO_SOURCE
{
	VIDEO_SOURCE_CVIDEO1 = 1,	//Composite
	VIDEO_SOURCE_CVIDEO2 = 2,
	VIDEO_SOURCE_CVIDEO3 = 3,
	VIDEO_SOURCE_SVIDEO1 = 10,	//S-Video
	VIDEO_SOURCE_SVIDEO2 = 11,
	VIDEO_SOURCE_SVIDEO3 = 12,
	VIDEO_SOURCE_DIGITAL1 = 20,	//Auto, HDMI
	VIDEO_SOURCE_TVTUNER = 30,
	VIDEO_SOURCE_NVIDEO1 = 40,	//Component
	VIDEO_SOURCE_INVALID = -1
} VIDEO_SOURCE, *PVIDEO_SOURCE;

typedef enum _VIDEO_DATATYPE
{
	VIDEO_DATATYPE_H264 = 0,
	VIDEO_DATATYPE_MPEG2 = 1,
	VIDEO_DATATYPE_JPEG = 2,
	VIDEO_DATATYPE_RAW = 3,
	VIDEO_DATATYPE_INVALID = -1
} VIDEO_DATATYPE, *PVIDEO_DATATYPE;


/////////////////////////////////////////////////////////////////////////////
//
//

// device specific initialization data
//

// Andy 2008.10.14
// video_decoder
#define SYS_VIDDECODER_DUMMY	0
#define SYS_VIDDECODER_SAA7115	1
#define SYS_VIDDECODER_MT9V112	2
#define SYS_VIDDECODER_OV7720	3
#define SYS_VIDDECODER_ADI7401	4
#define SYS_VIDDECODER_PE1005	5
#define SYS_VIDDECODER_SIL9011	6
#define SYS_VIDDECODER_SIL9025	7
#define SYS_VIDDECODER_MI2010	8
#define SYS_VIDDECODER_USBFW	9
#define SYS_VIDDECODER_ADV7614	10
#define SYS_VIDDECODER_ADV7441	11
#define SYS_VIDDECODER_TVP7002	12
#define SYS_VIDDECODER_AUTO1	13
#define SYS_VIDDECODER_IT6603	14
#define SYS_VIDDECODER_OV5642   15
#define SYS_VIDDECODER_NATIVE_MI2010	16
#define SYS_VIDDECODER_NATIVE_NT99240	17
#define SYS_VIDDECODER_OV10633	18
#define SYS_VIDDECODER_TW9910	19
#define SYS_VIDDECODER_TVP7002VH		20

typedef enum _QPVIDDECODER_TYPE
{
	VIDDECODER_DUMMY = 0,
	VIDDECODER_SAA7115 = 1,
	VIDDECODER_MT9V112 = 2,
	VIDDECODER_OV7720 = 3,
	VIDDECODER_ADI7401 = 4,
	VIDDECODER_PE1005 = 5,
	VIDDECODER_SIL9011 = 6,
	VIDDECODER_SIL9025 = 7,
	VIDDECODER_MI2010 = 8,
	VIDDECODER_USBFW = 9,
	VIDDECODER_ADV7614 = 10,
	VIDDECODER_ADI7441 = 11,
	VIDDECODER_TVP7002 = 12,
	VIDDECODER_AUTO1 = 13,
	VIDDECODER_IT6603 = 14,
	VIDDECODER_OV5642 = 15,
	VIDDECODER_NATIVE_MI2010 = 16,
	VIDDECODER_NATIVE_NT99240 = 17,
	VIDDECODER_OV10633 = 18,
	VIDDECODER_TW9910 = 19,
	VIDDECODER_TVP7002VH = 20,

	VIDDECODER_INVALID = -1
} QPVIDDECODER_TYPE, *PQPVIDDECODER_TYPE;

typedef enum _QP_MULTIPLE_VIDDECODER_TYPE
{
	MULTIPLE_VIDDECODER_NONE = 0,
	MULTIPLE_VIDDECODER_TYPE1 = 1
} QP_MULTIPLE_VIDDECODER_TYPE, *PQP_MULTIPLE_VIDDECODER_TYPE;


typedef struct _QPVIDDECODER_INITDATA
{
	QPVIDDECODER_TYPE	type;
	VIDEO_SOURCE		channel;
	QP_MULTIPLE_VIDDECODER_TYPE	multiType;
} QPVIDDECODER_INITDATA, *PQPVIDDECODER_INITDATA;



/////////////////////////////////////////////////////////////////////////////
//
//

// property supported by this interface
//



/////////////////////////////////////////////////////////////////////////////
//
//

// codec cllback command code
//



/////////////////////////////////////////////////////////////////////////////
//
//


#ifndef __IVIDEODECODER_FWD_DEFINED__
#define __IVIDEODECODER_FWD_DEFINED__
typedef struct IVideoDecoder IVideoDecoder;
#endif //__IVIDEODECODER_FWD_DEFINED__

    struct IVideoDecoder
    {
		// IGenericDevice
		DECLARE_IGENERICDEVICE(IVideoDecoder)

		// IVideoDecoder
		EQP_Errors (*SelectVideoSource)(IVideoDecoder *This,
										VIDEO_SOURCE source
										);
		EQP_Errors (*SetVideoSystem)(IVideoDecoder *This,
									 VIDEO_STANDARD standard
									 );
		EQP_Errors (*SetVideoDataType)(IVideoDecoder *This,
									 VIDEO_DATATYPE datatype
									 );
		EQP_Errors (*SetVcrTiming)(IVideoDecoder *This,
								   TQP_UINT32 enable
								   );
		EQP_Errors (*SetOutputEnabled)(IVideoDecoder *This,
									   TQP_UINT32 enable
									   );

		EQP_Errors (*GetVideoSource)(IVideoDecoder *This,
									 PVIDEO_SOURCE pSource
									 );
		EQP_Errors (*GetVideoSystem)(IVideoDecoder *This,
									 PVIDEO_STANDARD pStandard
									 );
		EQP_Errors (*SetVideoRes)(IVideoDecoder *This,
								  TQP_UINT32 dwWid,
			  					  TQP_UINT32 dwHgt,
			  					  TQP_UINT32 dwScanFmt,
			  					  TQP_UINT32 dwFps
			  					  );
		EQP_Errors (*GetVcrTiming)(IVideoDecoder *This,
								   TQP_UINT32 *pEnable
								   );
		EQP_Errors (*GetOutputEnabled)(IVideoDecoder *This,
									   TQP_UINT32 *pEnable
									   );

		// Brightness
		EQP_Errors (*GetBrightnessRange)(IVideoDecoder *This,
										 int *pnDefault,
					  					 int *pnMin,
					  					 int *pnMax
					  					 );
		EQP_Errors (*GetBrightness)(IVideoDecoder *This,
									int *pnParamValue
									);
		EQP_Errors (*SetBrightness)(IVideoDecoder *This,
									int nParamValue
									);
		// Contrast
		EQP_Errors (*GetContrastRange)(IVideoDecoder *This,
									   int *pnDefault,
					  				   int *pnMin,
					  				   int *pnMax
					  				   );
		EQP_Errors (*GetContrast)(IVideoDecoder *This,
								  int *pnParamValue
								  );
		EQP_Errors (*SetContrast)(IVideoDecoder *This,
								  int nParamValue
								  );
		// Saturation
		EQP_Errors (*GetSaturationRange)(IVideoDecoder *This,
										 int *pnDefault,
					  					 int *pnMin,
					  					 int *pnMax
					  					 );
		EQP_Errors (*GetSaturation)(IVideoDecoder *This,
									int *pnParamValue
									);
		EQP_Errors (*SetSaturation)(IVideoDecoder *This,
									int nParamValue
									);
		// Hue
		EQP_Errors (*GetHueRange)(IVideoDecoder *This,
								  int *pnDefault,
					  			  int *pnMin,
					  			  int *pnMax
					  			  );
		EQP_Errors (*GetHue)(IVideoDecoder *This,
							 int *pnParamValue
							 );
		EQP_Errors (*SetHue)(IVideoDecoder *This,
							 int nParamValue
							 );
		// Sharpness
		EQP_Errors (*GetSharpnessRange)(IVideoDecoder *This,
										int *pnDefault,
					  					int *pnMin,
					  					int *pnMax
					  					);
		EQP_Errors (*GetSharpness)(IVideoDecoder *This,
								   int *pnParamValue
								   );
		EQP_Errors (*SetSharpness)(IVideoDecoder *This,
								   int nParamValue
								   );

		// Macrovision detection
		EQP_Errors (*DetectCopyProtection)(IVideoDecoder *This,
										   TQP_BOOL *pCopyProtected,
										   PTQP_UINT8 pStatus // Macrovision copy control bits
										   );
		EQP_Errors (*DetectVideoStandard)(IVideoDecoder *This,
										  PVIDEO_STANDARD pStandard,
										  PTQP_UINT32 pulSignalLocked,
										  PTQP_UINT32 pulNumberOfLines
										  );
		// black frame
		//
		EQP_Errors (*MuteVideo)(IVideoDecoder *This);
		EQP_Errors (*UnmuteVideo)(IVideoDecoder *This);
		EQP_Errors (*IsVideoMuted)(IVideoDecoder *This,
								   TQP_BOOL *pbMuted
								   );

    // 2009.0213 by jkglee, get/set lens room position
		EQP_Errors (*GetLensRoomPos)(IVideoDecoder *This, PTQP_INT16 psCurrent, PTQP_INT16 psMin, PTQP_INT16 psMax);
		EQP_Errors (*SetLensRoomPos)(IVideoDecoder *This, TQP_INT16 sCurrent);

		// sensor control extension
		//

		// edge enhancement
		EQP_Errors (*GetEdgeEnhancementRange)(IVideoDecoder *This,
											  int *pnDefault,
					  						  int *pnMin,
					  						  int *pnMax
					  						  );
		EQP_Errors (*GetEdgeEnhancement)(IVideoDecoder *This,
										 int *pnParamValue
										 );
		EQP_Errors (*SetEdgeEnhancement)(IVideoDecoder *This,
										 int nParamValue
										 );
		// gamma
		EQP_Errors (*GetGammaRange)(IVideoDecoder *This,
									int *pnDefault,
					  				int *pnMin,
					  				int *pnMax
					  				);
		EQP_Errors (*GetGamma)(IVideoDecoder *This,
							   int *pnParamValue
							   );
		EQP_Errors (*SetGamma)(IVideoDecoder *This,
							   int nParamValue
							   );
		// low light boost
		EQP_Errors (*GetLowLightBoostRange)(IVideoDecoder *This,
											int *pnDefault,
					  						int *pnMin,
					  						int *pnMax
					  						);
		EQP_Errors (*GetLowLightBoost)(IVideoDecoder *This,
									   int *pnParamValue
									   );
		EQP_Errors (*SetLowLightBoost)(IVideoDecoder *This,
									   int nParamValue
									   );
		// exposure
		EQP_Errors (*GetExposureRange)(IVideoDecoder *This,
									   int *pnDefault,
					  				   int *pnMin,
					  				   int *pnMax
					  				   );
		EQP_Errors (*GetExposure)(IVideoDecoder *This,
								  int *pnParamValue
								  );
		EQP_Errors (*SetExposure)(IVideoDecoder *This,
								  int nParamValue
								  );
		EQP_Errors (*SetAutoExposureEnabled)(IVideoDecoder *This,
											 TQP_UINT32 enable
											 );
		EQP_Errors (*GetAutoExposureEnabled)(IVideoDecoder *This,
											 TQP_UINT32 *enable
											 );
		// auto exposure priority
		EQP_Errors (*GetAutoExposurePriorityRange)(IVideoDecoder *This,
												   int *pnDefault,
					  							   int *pnMin,
					  							   int *pnMax
					  							   );
		EQP_Errors (*GetAutoExposurePriority)(IVideoDecoder *This,
											  int *pnParamValue
											   );
		EQP_Errors (*SetAutoExposurePriority)(IVideoDecoder *This,
											  int nParamValue
											  );
		// white balance
		EQP_Errors (*GetWhiteBalanceRange)(IVideoDecoder *This,
										   int *pnDefault,
					  					   int *pnMin,
					  					   int *pnMax
					  					   );
		EQP_Errors (*GetWhiteBalance)(IVideoDecoder *This,
									  int *pnParamValue
									  );
		EQP_Errors (*SetWhiteBalance)(IVideoDecoder *This,
									  int nParamValue
									  );
		EQP_Errors (*SetAutoWhiteBalanceEnabled)(IVideoDecoder *This,
												 TQP_UINT32 enable
												 );
		EQP_Errors (*GetAutoWhiteBalanceEnabled)(IVideoDecoder *This,
												 TQP_UINT32 *enable
												 );
		// focus
		EQP_Errors (*GetFocusRange)(IVideoDecoder *This,
									int *pnDefault,
					  				int *pnMin,
					  				int *pnMax
					  				);
		EQP_Errors (*GetFocus)(IVideoDecoder *This,
							   int *pnParamValue
							   );
		EQP_Errors (*SetFocus)(IVideoDecoder *This,
							   int nParamValue
							   );
		EQP_Errors (*SetAutoFocusEnabled)(IVideoDecoder *This,
										  TQP_UINT32 enable
										  );
		EQP_Errors (*GetAutoFocusEnabled)(IVideoDecoder *This,
										  TQP_UINT32 *enable
										  );
		// power line frequency
		EQP_Errors (*GetPowerLineFreqRange)(IVideoDecoder *This,
											int *pnDefault,
					  						int *pnMin,
					  						int *pnMax
					  						);
		EQP_Errors (*GetPowerLineFreq)(IVideoDecoder *This,
									   int *pnParamValue
									   );
		EQP_Errors (*SetPowerLineFreq)(IVideoDecoder *This,
									   int nParamValue
									   );

		// to filter out HDCP content
		EQP_Errors (*IsHDCPContent)(IVideoDecoder *This,
									TQP_UINT32 *pnIsHDCP);
		EQP_Errors (*SetTriState)(IVideoDecoder *This,
								  TQP_UINT32 nTriState
								  );

		// start/stop video
		//
		EQP_Errors (*StartVideo)(IVideoDecoder *This,
								 TQP_UINT32 fps
								 );
		EQP_Errors (*StopVideo)(IVideoDecoder *This);
		EQP_Errors (*IsVideoStarted)(IVideoDecoder *This,
									 TQP_UINT32 *pbStarted
									 );
    };


/////////////////////////////////////////////////////////////////////////////
//
//

#define QPVideoDecoder_SelectVideoSource(This, source) \
    (This)->SelectVideoSource(This, source)

#define QPVideoDecoder_SetVideoSystem(This, standard) \
    (This)->SetVideoSystem(This, standard)

#define QPVideoDecoder_SetVideoDataType(This, datatype) \
    (This)->SetVideoDataType(This, datatype)

#define QPVideoDecoder_SetVcrTiming(This, enable) \
    (This)->SetVcrTiming(This, enable)

#define QPVideoDecoder_SetOutputEnabled(This, enable) \
    (This)->SetOutputEnabled(This, enable)

#define QPVideoDecoder_GetVideoSource(This, pSource) \
    (This)->GetVideoSource(This, pSource)

#define QPVideoDecoder_GetVideoSystem(This, pStandard) \
    (This)->GetVideoSystem(This, pStandard)

#define QPVideoDecoder_SetVideoRes(This, dwWid, dwHgt, dwScanFmt, dwFps) \
    (This)->SetVideoRes(This, dwWid, dwHgt, dwScanFmt, dwFps)

#define QPVideoDecoder_GetVcrTiming(This, pEnable) \
    (This)->GetVcrTiming(This, pEnable)

#define QPVideoDecoder_GetOutputEnabled(This, pEnable) \
    (This)->GetOutputEnabled(This, pEnable)

#define QPVideoDecoder_GetBrightnessRange(This, pnDefault, pnMin, pnMax) \
    (This)->GetBrightnessRange(This, pnDefault, pnMin, pnMax)

#define QPVideoDecoder_GetBrightness(This, pnParamValue) \
    (This)->GetBrightness(This, pnParamValue)

#define QPVideoDecoder_SetBrightness(This, pnParamValue) \
    (This)->SetBrightness(This, pnParamValue)

#define QPVideoDecoder_GetContrastRange(This, pnDefault, pnMin, pnMax) \
    (This)->GetContrastRange(This, pnDefault, pnMin, pnMax)

#define QPVideoDecoder_GetContrast(This, pnParamValue) \
    (This)->GetContrast(This, pnParamValue)

#define QPVideoDecoder_SetContrast(This, pnParamValue) \
    (This)->SetContrast(This, pnParamValue)

#define QPVideoDecoder_GetSaturationRange(This, pnDefault, pnMin, pnMax) \
    (This)->GetSaturationRange(This, pnDefault, pnMin, pnMax)

#define QPVideoDecoder_GetSaturation(This, pnParamValue) \
    (This)->GetSaturation(This, pnParamValue)

#define QPVideoDecoder_SetSaturation(This, pnParamValue) \
    (This)->SetSaturation(This, pnParamValue)

#define QPVideoDecoder_GetHueRange(This, pnDefault, pnMin, pnMax) \
    (This)->GetHueRange(This, pnDefault, pnMin, pnMax)

#define QPVideoDecoder_GetHue(This, pnParamValue) \
    (This)->GetHue(This, pnParamValue)

#define QPVideoDecoder_SetHue(This, pnParamValue) \
    (This)->SetHue(This, pnParamValue)

#define QPVideoDecoder_GetSharpnessRange(This, pnDefault, pnMin, pnMax) \
    (This)->GetSharpnessRange(This, pnDefault, pnMin, pnMax)

#define QPVideoDecoder_GetSharpness(This, pnParamValue) \
    (This)->GetSharpness(This, pnParamValue)

#define QPVideoDecoder_SetSharpness(This, pnParamValue) \
    (This)->SetSharpness(This, pnParamValue)

#define QPVideoDecoder_DetectCopyProtection(This, pCopyProtected, pStatus) \
    (This)->DetectCopyProtection(This, pCopyProtected, pStatus)

#define QPVideoDecoder_DetectVideoStandard(This, pStandard, pulSignalLocked, pulNumberOfLines) \
    (This)->DetectVideoStandard(This, pStandard, pulSignalLocked, pulNumberOfLines)

#define QPVideoDecoder_MuteVideo(This) \
    (This)->MuteVideo(This)

#define QPVideoDecoder_UnmuteVideo(This) \
    (This)->UnmuteVideo(This)

#define QPVideoDecoder_IsVideoMuted(This, pbMuted) \
    (This)->IsVideoMuted(This, pbMuted)

#define QPVideoDecoder_GetLensRoomPos(This, psCurrent, psMin, psMax) \
    (This)->GetLensRoomPos(This, pnCurrent, psMin, psMax)

#define QPVideoDecoder_SetLensRoomPos(This, sCurrent) \
    (This)->SetLensRoomPos(This, sCurrent)

#define QPVideoDecoder_GetEdgeEnhancementRange(This, pnDefault, pnMin, pnMax) \
	(This)->GetEdgeEnhancementRange(This, pnDefault, pnMin, pnMax)

#define QPVideoDecoder_GetEdgeEnhancement(This, pnParamValue) \
	(This)->GetEdgeEnhancement(This, pnParamValue)

#define QPVideoDecoder_SetEdgeEnhancement(This, nParamValue) \
	(This)->SetEdgeEnhancement(This, nParamValue)

#define QPVideoDecoder_GetGammaRange(This, pnDefault, pnMin, pnMax) \
	(This)->GetGammaRange(This, pnDefault, pnMin, pnMax)

#define QPVideoDecoder_GetGamma(This, pnParamValue) \
	(This)->GetGamma(This, pnParamValue)

#define QPVideoDecoder_SetGamma(This, nParamValue) \
	(This)->SetGamma(This, nParamValue)

#define QPVideoDecoder_GetLowLightBoostRange(This, pnDefault, pnMin, pnMax) \
	(This)->GetLowLightBoostRange(This, pnDefault, pnMin, pnMax)

#define QPVideoDecoder_GetLowLightBoost(This, pnParamValue) \
	(This)->GetLowLightBoost(This, pnParamValue)

#define QPVideoDecoder_SetLowLightBoost(This, nParamValue) \
	(This)->SetLowLightBoost(This, nParamValue)

#define QPVideoDecoder_GetExposureRange(This, pnDefault, pnMin, pnMax) \
	(This)->GetExposureRange(This, pnDefault, pnMin, pnMax)

#define QPVideoDecoder_GetExposure(This, pnParamValue) \
	(This)->GetExposure(This, pnParamValue)

#define QPVideoDecoder_SetExposure(This, nParamValue) \
	(This)->SetExposure(This, nParamValue)

#define QPVideoDecoder_SetAutoExposureEnabled(This, enable) \
	(This)->SetAutoExposureEnabled(This, enable)

#define QPVideoDecoder_GetAutoExposureEnabled(This, enable) \
	(This)->GetAutoExposureEnabled(This, enable)

#define QPVideoDecoder_GetWhiteBalanceRange(This, pnDefault, pnMin, pnMax) \
	(This)->GetWhiteBalanceRange(This, pnDefault, pnMin, pnMax)

#define QPVideoDecoder_GetWhiteBalance(This, pnParamValue) \
	(This)->GetWhiteBalance(This, pnParamValue)

#define QPVideoDecoder_SetWhiteBalance(This, nParamValue) \
	(This)->SetWhiteBalance(This, nParamValue)

#define QPVideoDecoder_SetAutoWhiteBalanceEnabled(This, enable) \
	(This)->SetAutoWhiteBalanceEnabled(This, enable)

#define QPVideoDecoder_GetAutoWhiteBalanceEnabled(This, enable) \
	(This)->GetAutoWhiteBalanceEnabled(This, enable)

#define QPVideoDecoder_GetFocusRange(This, pnDefault, pnMin, pnMax) \
	(This)->GetFocusRange(This, pnDefault, pnMin, pnMax)

#define QPVideoDecoder_GetFocus(This, pnParamValue) \
	(This)->GetFocus(This, pnParamValue)

#define QPVideoDecoder_SetFocus(This, nParamValue) \
	(This)->SetFocus(This, nParamValue)

#define QPVideoDecoder_SetAutoFocusEnabled(This, enable) \
	(This)->SetAutoFocusEnabled(This, enable)

#define QPVideoDecoder_IsHDCPContent(This, pnIsHDCP) \
	(This)->IsHDCPContent(This, pnIsHDCP)

#define QPVideoDecoder_SetTriState(This, tristate) \
	(This)->SetTriState(This, tristate)

#define QPVideoDecoder_GetAutoFocusEnabled(This, enable) \
	(This)->GetAutoFocusEnabled(This, enable)

#define QPVideoDecoder_StartVideo(This, fps) \
	(This)->StartVideo(This, fps)

#define QPVideoDecoder_StopVideo(This) \
	(This)->StopVideo(This)

#define QPVideoDecoder_IsVideoStarted(This, pbStarted) \
    (This)->IsVideoStarted(This, pbStarted)

#ifdef __cplusplus
}
#endif

#endif //__QPVIDDECODER_H__

