///////////////////////////////////////////////////////////////////////////////
//        
//
// Qpvidencoder.h
//
// Description: 
//
//  This is the definations of the video encoder chip interface
// 
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __QPVIDENCODER_H__
#define __QPVIDENCODER_H__


#include "qptypes.h"
#include "qpgenericdevice.h"
#include "qpviddecoder.h"

#ifdef __cplusplus
extern "C" {
#endif


/////////////////////////////////////////////////////////////////////////////
//
//

// device specific initialization data
//

typedef enum _QPVIDENCODER_TYPE
{
	VIDENCODER_DUMMY = 0,
	VIDENCODER_SAA7128 = 1,
	VIDENCODER_ADI7393 = 2,
	VIDENCODER_SIL9030 = 3,
	VIDENCODER_SIL9034 = 4,
	VIDENCODER_ADI9889=5,

	VIDENCODER_INVALID = -1
} QPVIDENCODER_TYPE, *PQPVIDENCODER_TYPE;


typedef struct _QPVIDENCODER_INITDATA
{
	QPVIDENCODER_TYPE	type;
	VIDEO_STANDARD		standard;
} QPVIDENCODER_INITDATA, *PQPVIDENCODER_INITDATA;



/////////////////////////////////////////////////////////////////////////////
//
//

// property supported by this interface
//


/////////////////////////////////////////////////////////////////////////////
//
//

// codec cllback command code
//



/////////////////////////////////////////////////////////////////////////////
//
//

#ifndef __IVIDEOENCODER_FWD_DEFINED__
#define __IVIDEOENCODER_FWD_DEFINED__
typedef struct IVideoEncoder IVideoEncoder;
#endif //__IVIDEOENCODER_FWD_DEFINED__

    struct IVideoEncoder
    {
		// IGenericDevice
		DECLARE_IGENERICDEVICE(IVideoEncoder)

		// IVideoEncoder

		// Enables encoder device.
		EQP_Errors (*Enable)(IVideoEncoder *This);

		// Disables encoder device.
		EQP_Errors (*Disable)(IVideoEncoder *This);

		// Get encoder status.
		EQP_Errors (*IsEnabled)(IVideoEncoder *This, 
								TQP_BOOL *pfEnabled
								);

		// Sets video system of the encoder.
		EQP_Errors (*SetVideoStandard)(IVideoEncoder *This, 
									   VIDEO_STANDARD standard
									   );

		// Get video system of the encoder.
		EQP_Errors (*GetVideoStandard)(IVideoEncoder *This, 
									   PVIDEO_STANDARD pStandard
									   );
    };


/////////////////////////////////////////////////////////////////////////////
//
//

#define IVideoEncoder_Enable(This) \
    (This)->Enable(This)

#define IVideoEncoder_Disable(This) \
    (This)->Disable(This)

#define IVideoEncoder_IsEnabled(This, pfEnabled) \
    (This)->IsEnabled(This, pfEnabled)

#define IVideoEncoder_SetVideoStandard(This, standard) \
    (This)->SetVideoStandard(This, standard)

#define IVideoEncoder_GetVideoStandard(This, pStandard) \
    (This)->GetVideoStandard(This, pStandard)

#ifdef __cplusplus
}
#endif

#endif //__QPVIDENCODER_H__

