///////////////////////////////////////////////////////////////////////////////
//        
//
// Util.h
//
// Description: 
//
//  handy utilities
// 
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __UTIL_H__
#define __UTIL_H__


#include "qptypes.h"

#ifdef __cplusplus
extern "C" {
#endif


/////////////////////////////////////////////////////////////////////////////
//
//


#define Swap4Bytes(ul) \
		((((ul) & 0xff000000) >> 24) |\
		(((ul) & 0x00ff0000) >>  8) |\
		(((ul) & 0x0000ff00) <<  8) |\
		(((ul) & 0x000000ff) << 24))

#define ulLittleToBigEndian(ul)	Swap4Bytes(ul)
#define ulBigToLittleEndian(ul)	Swap4Bytes(ul)


#define Swap2Bytes(w) \
		((((w) & 0xff00) >>  8) |\
		(((w) & 0x00ff) << 8))

#define wLittleToBigEndian(w)	Swap2Bytes(w)
#define wBigToLittleEndian(w)	Swap2Bytes(w)


#ifndef FIELD_OFFSET
#define FIELD_OFFSET(type, field)	((long)&(((type *)0)->field))
#endif //!FIELD_OFFSET

#ifndef GET_INHERITED_OBJECT
#define GET_INHERITED_OBJECT(type, filed, pObject)	((type *)((char *)pObject - FIELD_OFFSET(type, filed)))
#endif //!GET_INHERITED_OBJECT


#ifndef max
#define max(a,b)        (((a) > (b)) ? (a) : (b))
#endif

#ifndef min
#define min(a,b)        (((a) < (b)) ? (a) : (b))
#endif

#ifndef SIZEOF_ARRAY
#define SIZEOF_ARRAY(ar)	(sizeof(ar)/sizeof((ar)[0]))
#endif // !defined(SIZEOF_ARRAY)


/////////////////////////////////////////////////////////////////////////////
//
//

void DelayMilliSeconds(TQP_UINT32 Milliseconds);
void DelayMicroSeconds(TQP_UINT32 Microseconds);
TQP_INT64 GetSystemTimeUtil(void);
TQP_INT64 GetSystemTimeUtilUSec(void);
void BeginTimer(TQP_INT64 *pTimeto);
TQP_BOOL Timeout(TQP_INT64 Timeto, 
				 TQP_UINT32 delay
				 );
TQP_INT32 BytesSwap(TQP_UINT8 *pSource,	// Pointer to source data
				    TQP_UINT8 *pTarget,	// Pointer to hold the target data, NULL if inplace swapping is required
				    TQP_INT32 lSize		// Size of buffers in bytes.
				    );
TQP_BOOL IsEqualTQPGUID(TQP_REFGUID rguid1, 
						TQP_REFGUID rguid2
						);
void * QPMMMalloc2Ex(unsigned long bytes);
void QPMMFree2Ex(void * memP);
TQP_BOOL IsSafeToCall(void);

void * PLvmalloc(unsigned long bytes);
void PLvfree(void * memP);

#ifdef __cplusplus
}
#endif

#endif //__UTIL_H__



