///////////////////////////////////////////////////////////////////////////////
//        
//
//		Constants definitions to identify a platform
//
//	Win32 with debug output
//
// History:
//	Sep 26,2006	(EC) Created
//
// Authors: (EC) Enrico Cadorin
//
// modified by: Catherine Cheng
//
///////////////////////////////////////////////////////////////////////////////


#ifndef __PLATFORM_H__
#define __PLATFORM_H__

#include "qppfmgr.h"
#include "qposmgr.h"

#include <linux/types.h>
//#include <linux/string.h>
#include <linux/sched.h>


#define QP_DEBUG
#ifndef QP_LINXKER
#define QP_LINXKER
#endif //!QP_LINXKER


#define QP_CODECTHR_PRI			QPOS_THRPRI_DEFAULT

// Access Mode
#define QPPF_DEF_ACCESSMODE		QPPF_MODE_INDIRECT

// Bus type
#define QPPF_DEF_BUSTYPE		QPPF_BUS_201_EMULATION

// Slave Interface parameters
#define QPPF_DEF_SLAVEIF_REG	0x100000
#define QPPF_DEF_SLAVEIF_MEM	QP_NULL
#define QPPF_DEF_SLAVEIF_PAGE	0x100000

// Memory parameters
#define QPPF_DEF_MEMORY_MBITS	512
#define QPPF_DEF_MEMORY_TYPE	QPPF_MEM_TYPE_MDDR

// Interrupt numbers
#define QPPF_DEF_INTNUM			QPOS_ISR_CODEC

// Video Input
#define QPPF_DEF_VIUMODE		QPPF_VINMODE_INTERLACE
#define QPPF_DEF_VIUFORMAT		QPPF_VINFORMAT_INTERLACE
#define QPPF_DEF_VIUCLKEDGE		QPPF_VINCLKEDGE_POSITIVE
#define QPPF_DEF_VIUSTARTPEL	0
#define QPPF_DEF_VIUSTARTLINE	21
#define QPPF_DEF_VIUSYNCCODE1	0xF1F1F1DA
#define QPPF_DEF_VIUSYNCCODE2	0xB6F1F1B6

// Audio Input
#define QPPF_DEF_AIMSB			QPPF_AIUMSB_MSB
#define QPPF_DEF_AILRCLK		QPPF_AIULRCLK_LEFTLOW
#define QPPF_DEF_AIBCLK			QPPF_AIUBCLK_RISING
#define QPPF_DEF_AII2S			QPPF_AIUI2S_PROGRAM
#define QPPF_DEF_AIRJ			QPPF_AIURJ_LEFT

// Video Output
#define QPPF_DEF_VOENABLE		QP_TRUE
#define QPPF_DEF_VOUMODE		QPPF_VOUMODE_NTSC_INTERLACED
#define QPPF_DEF_VOUSTARTPEL	1
#define QPPF_DEF_VOUSTARTLINE	0

// Audio Output
#define QPPF_DEF_AOENABLE		QP_TRUE
#define QPPF_DEF_AOMSB			QPPF_AOUMSB_MSB
#define QPPF_DEF_AOLRCLK		QPPF_AOULRCLK_LEFTLOW
#define QPPF_DEF_AOBCLK			QPPF_AOUBCLK_FALLING
#define QPPF_DEF_AOI2S			QPPF_AOUI2S_PROGRAM
#define QPPF_DEF_AORJ			QPPF_AOURJ_LEFT
#define QPPF_DEF_AOMODE			QPPF_AOUMODE_MASTER

// Video Decoder
#define QPPF_DEF_VIDDEC_TYPE	QPPF_VIDDEC_SAA7115

// Video Encoder
#define QPPF_DEF_VIDENC_TYPE	QPPF_VIDENC_SAA7128
#define QPPF_DEF_VIDSTD			QPPF_VIDSTD_NTSC

// Tuner type
#define QPPF_DEF_TUNER_TYPE		QPPF_TUNER_DUMMY;

// TV Audio type
#define QPPF_DEF_TVAUDIO_TYPE	QPPF_TVAUDIO_DUMMY

// Audio Codec type
#define QPPF_DEF_AUDCODEC_TYPE	QPPF_AUDCODEC_DUMMY;

// Chip type
#define QPPF_DEF_CHIP_TYPE		QPPF_CHIPTYPE_BERINGER

// FW API version
#define QPPF_DEF_FWAPI_VERSION	QPPF_FWAPIVERSION_0

// CODECLIB
#define QPPF_DEF_CODECLIB_INTMODE	QPPF_CODECLIBINTMODE_HIU_ISR
#define QPPF_DEF_CODECLIB_MEMMODE	QPPF_CODECLIBMEMMODE_STATIC

// I2C
#define QPPF_DEF_I2C_TYPE		QPPF_I2CTYPE_USE_201_EMULATION
#define QPPF_DEF_I2C_CLKBIT		14
#define QPPF_DEF_I2C_DATABIT	15

// GPIO
#define QPPF_DEF_GPIO_DIRECTIONS	0x17
#define QPPF_DEF_GPIO_VALUES		0x13


// Use external host DMA functions QPX_Wrap_ReadDMA & QPX_Wrap_WriteDMA
//#define _QPCODEC_USE_EXTERNAL_HOST_DMA_FUNC

// Start the codec inside the thread
#define _QPCODEC_ASYNC_START 

// codec streaming callback in separate thread
//#define _QPCODEC_CALLBACK_IN_THREAD 

#endif //__PLATFORM_H__
