///////////////////////////////////////////////////////////////////////////////
//        
//
//		Internal File Manager interface
//
// Use compile time constants to select an OS:
//	Windows: _QP_WINDOWS
//	ITRON: _QP_ITRON
//	NuCore: _QP_NUCORE
//
// History:
//	May 31, 2007	(EC) Created
//
// Authors: (EC) Enrico Cadorin
//
///////////////////////////////////////////////////////////////////////////////



#ifndef __QPFILEMGRINT_H__
#define __QPFILEMGRINT_H__

#include "qptypes.h"
#include "qpfilemgr.h"

#define MAX_FILE_NAME_LEN	512

extern EQP_Errors FileMInit (void);
extern void FileMTerm (void);

// volumes operations
extern void FileMGetCaps (PTQP_FMCaps capsP);
extern EQP_Errors FileMGetVolsList (TQP_FMVolID *, unsigned long * numP);
extern TQP_FMVolID FileMGetVolID (const PTQP_CHAR);
extern EQP_Errors FileMGetVolInfo (TQP_FMVolID, PTQP_FMVolInfo);
extern EQP_Errors FileMGetVolName (TQP_FMVolID, PTQP_CHAR, unsigned long *);
extern EQP_Errors FileMSetVolName (TQP_FMVolID, PTQP_CHAR);
extern EQP_Errors FileMGetVolFree (TQP_FMVolID, TQP_UINT64 *);
extern TQP_BOOL FileMIsVolAvailable (TQP_FMVolID);

// file/directory operations
extern EQP_Errors FileMDirListStart (TQP_FMVolID, const PTQP_CHAR);
extern EQP_Errors FileMDirListNext (TQP_FMVolID,PTQP_FMFileInfo fileInfoP);
extern EQP_Errors FileMGetNameInfo (TQP_FMVolID, const PTQP_CHAR, PTQP_FMFileInfo);

extern EQP_Errors FileMRemove (TQP_FMVolID, const PTQP_CHAR);
extern EQP_Errors FileMRename (TQP_FMVolID, const PTQP_CHAR, const PTQP_CHAR);
extern EQP_Errors FileMMakeDir (TQP_FMVolID, const PTQP_CHAR);
extern EQP_Errors FileMOpen (TQP_FMVolID, const PTQP_CHAR, TQP_UINT32, TQP_FMFileID *);
extern EQP_Errors FileMGetFileInfo (TQP_FMVolID fID, PTQP_FMFileInfo fiP);
extern EQP_Errors FileMRead (TQP_FMFileID fID, void * bufP, TQP_UINT64 * lenP);
extern EQP_Errors FileMWrite (TQP_FMFileID fID, const void * bufP, TQP_UINT64 * lenP);
extern EQP_Errors FileMGetPos (TQP_FMFileID fID, TQP_UINT64 * posP);
extern EQP_Errors FileMSetPos (TQP_FMFileID fID, TQP_UINT64 * posP);
extern EQP_Errors FileMGetLength (TQP_FMFileID fID, TQP_UINT64 * lenP);
extern EQP_Errors FileMClose (TQP_FMFileID fID);


#endif //__QPFILEMGRINT_H__
