///////////////////////////////////////////////////////////////////////////////
//     
//
//		Memory Manager implementation for Linux kernel mode
//
//
// History:
//	April 10, 2008
//
// Authors: Catherine Cheng, Enrico Cadorin
//
///////////////////////////////////////////////////////////////////////////////


#include "qlautoconf.h"
//#include <linux/gfp.h>
#include <linux/version.h>
#include <linux/slab.h>
#include <linux/vmalloc.h>

#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
#include <asm/cacheflush.h>
#include <mach/linuxqpsos.h>
#include <mach/ql300map.h>
#include <mach/pl330xp.h>
#endif

#include "../common/qptypes.h"

#include "../common/qpdbgmgr.h"
#include "../common/qpmemmgr.h"

#include "qpmemmgrint.h"
#include "hci.h"


#define pl330_soc_virt_to_phys(vaddr) \
((((vaddr >= ARC_CODEDATA_VIR) && (vaddr < (ARC_CODEDATA_VIR + ARC_CODEDATA_LEN))) || \
  ((vaddr >= QPSOS_NONCACHEDHEAP_VIR) && (vaddr < (QPSOS_NONCACHEDHEAP_VIR + QPSOS_NONCACHEDHEAP_LEN))) \
 ) ? \
    ((vaddr >= ARC_CODEDATA_VIR) ? \
        (vaddr - ARC_CODEDATA_VIR + ARC_CODEDATA_PHY) : \
        (vaddr - QPSOS_NONCACHEDHEAP_VIR + QPSOS_NONCACHEDHEAP_PHY) \
    ) : \
    virt_to_phys(vaddr) \
)


// M2M DMA1 registers
#define M2M1_REG_OFFSET		0x0730
#define M2M1_INT_STATUS		(M2M1_REG_OFFSET + 0x00)	// 0x0730
#define M2M1_CTRL_MISC		(M2M1_REG_OFFSET + 0x04)	// 0x0734
#define M2M1_CURRENT_BC		(M2M1_REG_OFFSET + 0x08)	// 0x0738
#define M2M1_QBUFFER_SA		(M2M1_REG_OFFSET + 0x0C)	// 0x073C
#define M2M1_QBUFFER_LEN	(M2M1_REG_OFFSET + 0x10)	// 0x0740
#define M2M1_SRCCBUF_SA		(M2M1_REG_OFFSET + 0x14)	// 0x0744
#define M2M1_SRCCBUF_EA		(M2M1_REG_OFFSET + 0x18)	// 0x0748
#define M2M1_DSTCBUF_SA		(M2M1_REG_OFFSET + 0x1C)	// 0x074C
#define M2M1_DSTCBUF_EA		(M2M1_REG_OFFSET + 0x20)	// 0x0750


typedef union _TQP_MMM2MCtrlMisc {
	unsigned int d32;
	struct
	{
		unsigned reset     : 1;
		unsigned stat      : 1;
		unsigned err_code  : 6;
		unsigned burst_len : 8;
		unsigned cur_q_idx : 16;
	}b;
} TQP_MMM2MCtrlMisc, *PTQP_MMM2MCtrlMisc;

typedef union _TQP_MMM2MQueueBufCtrl
{
	TQP_UINT32 d32;
	struct
	{
		unsigned length    : 24;
		unsigned reserved  : 4;
		unsigned swap      : 2;
		unsigned load      : 1;
		unsigned i_en      : 1;
	}b;
} TQP_MMM2MQueueBufCtrl, *PTQP_MMM2MQueueBufCtrl;

typedef struct _TQP_MMM2MRequest {
	void *				srcAddr;	// source memory address
	void *				dstAddr;	// destination memory address
	TQP_UINT32			bytes;		// size in bytes
	EQP_MMM2MByteSwap	byteSwap;	// byte swapping option
	TQP_MMM2MReqCompl	complProc;	// M2M complete routine
	void *				refCon;
	struct list_head	entry;		// queue connect structure
} TQP_MMM2MRequest, *PTQP_MMM2MRequest;

typedef struct _TQP_MMM2MQueueBuf {
	TQP_UINT32 				srcAddr;				// DW 1 ~ 4
	TQP_UINT32 				dstAddr;
	TQP_MMM2MQueueBufCtrl	ctrl;
	TQP_UINT32 				errCheck;
}TQP_MMM2MQueueBuf, *PTQP_MMM2MQueueBuf;

//__attribute__((aligned(32))) static TQP_MMM2MQueueBuf queueBuf[2];		// to be an integer multiple of cache line size
#define MAX_M2M_QUEUE_NUM 512
#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
static PTQP_MMM2MQueueBuf queueBuf = NULL;
#endif

EQP_Errors MemMInit(void)
{
    EQP_Errors rv = QPERR_SUCCESS;

#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
	if (!queueBuf) {
		queueBuf = (PTQP_MMM2MQueueBuf)kmalloc(MAX_M2M_QUEUE_NUM*sizeof(TQP_MMM2MQueueBuf), GFP_KERNEL | GFP_DMA);
	}

	if (queueBuf) {
		memset((void *)&queueBuf[0], 0, MAX_M2M_QUEUE_NUM*sizeof(TQP_MMM2MQueueBuf));
	}

	WARN_ON(!queueBuf);
#endif

    return (rv);
}

EQP_Errors MemMTerm(void)
{
    EQP_Errors rv = QPERR_SUCCESS;

#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
	if (queueBuf) {
		kfree(queueBuf);
	}
#endif

    return (rv);
}

void MMGetCaps (PTQP_MMCaps capsP)
{
	capsP->clientPools = QP_FALSE;
	capsP->poolTypes = QP_TRUE;
	capsP->stdMM = QP_TRUE;
}

TQP_MMID
MMInit (void * ptr, unsigned long bytes)
{
	printk("%s()\n", __FUNCTION__);
	return (QP_NULL);
}

void
MMTerm (TQP_MMID mmID)
{
	printk("%s()\n", __FUNCTION__);
}

void *
MMMalloc (TQP_MMID mmID, unsigned long bytes)
{
	void *	rv = QP_NULL;

	//rv = kmalloc( bytes, GFP_KERNEL | __GFP_COMP );
	rv = kmalloc( bytes, GFP_KERNEL );

	return (rv);
}

void
MMFree (TQP_MMID mmID, void * ptr)
{
	kfree( ptr );
}


void *
MMMalloc2 (PTQP_UINT32 memTypeP, unsigned long bytes)
{
	void *	rv = QP_NULL;
#if !defined(CONFIG_USE_VMALLOC_FOR_MMMALLOC2) || defined(USE_KMALLOC_INSTEAD_OF_VMALLOC)
	rv = kmalloc( bytes, GFP_KERNEL );
#else
	rv = vmalloc_32( bytes);
#endif

	return (rv);
}

void
MMFree2 (void * memP)
{
#if !defined(CONFIG_USE_VMALLOC_FOR_MMMALLOC2) || defined(USE_KMALLOC_INSTEAD_OF_VMALLOC)
	kfree( memP );
#else
	vfree( memP );
#endif

/*	KIRQL	curIRQL;

	curIRQL = KeGetCurrentIrql();
	if (curIRQL <= DISPATCH_LEVEL) {
		ExFreePool(memP);
	}else{
		QPDbgMPrintf((QPDBGM_LEVEL_ERROR,"MMFree2(WINKER): running at high IRQL level %i\n"
			,curIRQL));
	}*/
}


EQP_Errors
MMStartM2MRequest(
	PTQP_MMM2MReqParms reqParmsP,
	TQP_MMM2MReqCompl complProc,
	void * refCon,
	TQP_MMM2MReqID  * reqIDP)
{
#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
	EQP_Errors	rv = QPERR_FAIL;

	TQP_MMM2MCtrlMisc ctrl;
	TQP_UINT32 regVal;

	int i = 0;
	unsigned int unBytes = 0;
	unsigned int unBytesLeft = 0;
	unsigned int unOffset = 0;
	unsigned int unSrcAddr = 0;
	unsigned int unDstAddr = 0;

	/* source address ring start */
	QPHCI_RegisterWrite((IHCIAPI *)refCon, M2M1_SRCCBUF_SA, 0);

	/* source address ring end */
	QPHCI_RegisterWrite((IHCIAPI *)refCon, M2M1_SRCCBUF_EA, (128*1024*1024-1) & 0xFFFFFFF8);

	/* dest address ring start */
	QPHCI_RegisterWrite((IHCIAPI *)refCon, M2M1_DSTCBUF_SA, 0);

	/* dest address ring end */
	QPHCI_RegisterWrite((IHCIAPI *)refCon, M2M1_DSTCBUF_EA, (128*1024*1024-1) & 0xFFFFFFF8);

	QPHCI_RegisterRead((IHCIAPI *)refCon, M2M1_CTRL_MISC, &ctrl.d32);
	ctrl.b.burst_len = 128;
	ctrl.b.err_code = 0;
	QPHCI_RegisterWrite((IHCIAPI *)refCon, M2M1_CTRL_MISC, ctrl.d32);

	unBytesLeft = reqParmsP->bytes;

	for (i=0; i<MAX_M2M_QUEUE_NUM; i++) {

		unSrcAddr = (unsigned int)reqParmsP->srcAddr + unOffset;
		unDstAddr = (unsigned int)reqParmsP->dstAddr + unOffset;

		/* bytes to page boundary
		** limitation: only one address (source and dest address) could come from vmalloc
		*/
		if (is_vmalloc_addr(unSrcAddr)) {
			unBytes = ((unSrcAddr + PAGE_SIZE) & PAGE_MASK) - unSrcAddr;
		} else  if (is_vmalloc_addr(unDstAddr)) {
			unBytes = ((unDstAddr + PAGE_SIZE) & PAGE_MASK) - unDstAddr;
		} else {
			unBytes = unBytesLeft;
		}

		if (unBytesLeft < unBytes) {
			unBytes = unBytesLeft;
		}

		unBytesLeft -= unBytes;
		unOffset += unBytes;

		__cpuc_coherent_kern_range(unSrcAddr, unSrcAddr + unBytes);

#if LINUX_VERSION_CODE < KERNEL_VERSION(3,7,0)
		__cpuc_flush_user_range(unDstAddr, unDstAddr + unBytes, VM_IO | VM_RESERVED);
#else
		__cpuc_flush_user_range(unDstAddr, unDstAddr + unBytes, VM_IO | VM_DONTEXPAND | VM_DONTDUMP);
#endif /* LINUX_VERSION_CODE < KERNEL_VERSION(3,7,0) */

		if (is_vmalloc_addr(unSrcAddr)) {
			queueBuf[i].srcAddr = page_to_phys(vmalloc_to_page((void *)(unSrcAddr & PAGE_MASK))) + (unSrcAddr & ~PAGE_MASK);
		} else {
			queueBuf[i].srcAddr = pl330_soc_virt_to_phys(unSrcAddr);
		}

		if (is_vmalloc_addr(unDstAddr)) {
			queueBuf[i].dstAddr = page_to_phys(vmalloc_to_page((void *)(unDstAddr & PAGE_MASK))) + (unDstAddr & ~PAGE_MASK);
		} else {
			queueBuf[i].dstAddr = pl330_soc_virt_to_phys(unDstAddr);
		}

		queueBuf[i].ctrl.b.length = unBytes;
		queueBuf[i].ctrl.b.swap = reqParmsP->byteSwap;
		queueBuf[i].ctrl.b.load = 1;
		if (unBytesLeft == 0) {
			/* the last queue of this request */
			queueBuf[i].ctrl.b.i_en = 1;
		} else {
			queueBuf[i].ctrl.b.i_en = 0;
		}
		queueBuf[i].errCheck = queueBuf[i].srcAddr ^ queueBuf[i].dstAddr ^ queueBuf[i].ctrl.d32;

		if (unBytesLeft == 0) {
			break;
		}
	}

	WARN_ON(unBytesLeft);

	if (unBytesLeft) {
		printk("%s: unBytesLeft=%d\n", __FUNCTION__, unBytesLeft);
	}

	__cpuc_coherent_kern_range((unsigned int)&queueBuf[0], (unsigned int)&queueBuf[0] + (MAX_M2M_QUEUE_NUM*sizeof(TQP_MMM2MQueueBuf)));

	QPHCI_RegisterWrite((IHCIAPI *)refCon, M2M1_QBUFFER_SA, pl330_soc_virt_to_phys((void *)&queueBuf[0]));

	QPHCI_RegisterWrite((IHCIAPI *)refCon, M2M1_QBUFFER_LEN, (i+1)*sizeof(TQP_MMM2MQueueBuf));

	rv = QPERR_SUCCESS;

#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
#if ENABLE_CODEC_MEM_LOG
	codec_mem_log_add(CODEC_MEM_LOG_M2M_SETUP_DONE);
#endif
#endif

	return (rv);
#else
	return (QPERR_NOTIMPL);
#endif
}

