///////////////////////////////////////////////////////////////////////////////
//        
//
//		Private interfaces to the OS managers
//
// History:
//	Aug 4, 2006	(EC) Created
//
// Authors: (EC) Enrico Cadorin
//
///////////////////////////////////////////////////////////////////////////////


#include "qptypes.h"
#include "qpmgrspriv.h"
#include "qpdbgmgrpriv.h"
#include "qpmemmgrpriv.h"
#include "qposmgrpriv.h"
#include "qppfmgrpriv.h"
#include "qpfilemgrpriv.h"
#include "qptimemgrpriv.h"

EQP_Errors QPMgrsInit (void)
{
	EQP_Errors		rv = QPERR_NOTIMPL;

	// Init the Debug Manager first
	rv = QPDbgMInitPriv();
	if (rv != QPERR_SUCCESS) {
	}

	rv = QPTMInitPriv();
	if (rv != QPERR_SUCCESS) {
	}

	// The Memory Manager must be initialized before the OS Manager
	rv = QPMemMInitPriv();
	if (rv != QPERR_SUCCESS) {
		goto initExit;
	}

	rv = QPOSMInitPriv();
	if (rv != QPERR_SUCCESS) {
		goto initExit;
	}

	rv = QPPFMInitPriv();
	if (rv != QPERR_SUCCESS) {
		goto initExit;
	}

#if defined(_QP_WINDOWS)
	rv = QPFileMInitPriv();
	if (rv != QPERR_SUCCESS) {
		goto initExit;
	}
#endif

	// more initialization here

initExit:
	return (rv);
}


EQP_Errors QPMgrsTerm (void)
{
	EQP_Errors		rv = QPERR_NOTIMPL;

#if defined(_QP_WINDOWS)
	rv = QPFileMTermPriv();
	if (rv != QPERR_SUCCESS) {
	}
#endif

	rv = QPPFMTermPriv();
	if (rv != QPERR_SUCCESS) {
	}

	rv = QPOSMTermPriv();
	if (rv != QPERR_SUCCESS) {
	}

	// more termination here

	// The Memory Manager must be terminated after the OS Manager
	rv = QPMemMTermPriv();
	if (rv != QPERR_SUCCESS) {
	}

	rv = QPTMTermPriv();
	if (rv != QPERR_SUCCESS) {
	}

	// terminate the Debug Manager last
	rv = QPDbgMTermPriv();
	if (rv != QPERR_SUCCESS) {
	}

	return (rv);
}


