///////////////////////////////////////////////////////////////////////////////
//        
//
//		OS Manager - Linux Kernel Mode specific code
//
// History:
//	April 16, 2008	
//
// Authors: Catherine Cheng, Enrico Cadorin
//
///////////////////////////////////////////////////////////////////////////////

#include "qlautoconf.h"

#include <linux/time.h>
#include <asm/param.h>
#include <linux/sched.h>
#include <linux/version.h>
#include <asm/div64.h>

#include <asm/delay.h>
#include "../common/qptypes.h"
#include "../common/qpdbgmgr.h"
#include "../common/qptimemgr.h"
#include "qptimemgrint.h"

#if LINUX_VERSION_CODE < KERNEL_VERSION(3,15,0)
#ifdef CONFIG_HOST_PLATFORM_STR8133_REF
int    gnTimeInit= 0;
#endif // CONFIG_HOST_PLATFORM_STR8133_REF
static wait_queue_head_t wq;
#else
#include <linux/delay.h>
#endif

EQP_Errors
TimeInit (void)
{
	EQP_Errors		rv = QPERR_SUCCESS;
	
#if LINUX_VERSION_CODE < KERNEL_VERSION(3,15,0)
#ifndef CONFIG_HOST_PLATFORM_STR8133_REF
        init_waitqueue_head(&wq);
#else
        if(!gnTimeInit) {
            printk("Init\n");
            init_waitqueue_head(&wq);
            gnTimeInit= 1;
        }
#endif // !CONFIG_HOST_PLATFORM_STR8133_REF
#endif

	return (rv);
}


void
TimeTerm (void)
{
}


void
TimeGetCaps (PTQP_TMCaps capsP)
{
    /* TBW */
}


TQPTM_Time
TimeGetSysTimeMilliS (void)
{
	unsigned long long ullMilliS;
	ullMilliS = TimeGetSysTimeMicroS();
	do_div(ullMilliS, 1000);
	return ullMilliS;
}

TQPTM_Time
TimeGetSysTimeMicroS (void)
{
	struct timeval tv;
	u64 t;

	do_gettimeofday(&tv);
	t = tv.tv_sec;
	t = t*1000000;
	t += tv.tv_usec;

	return t;
}

TQPTM_Time
TimeGetSysTimeNanoS (void)
{
#if 0
	struct timespec ts;
	u64  t;

	ts = current_kernel_time();
	t = *((u64 *)&(ts.tv_sec));
	t = t*1000000000;
	t += ts.tv_nsec;
#endif
	struct timeval tv;
	u64 t;

	do_gettimeofday(&tv);
	t = tv.tv_sec;
	t = t*1000000;
	t += tv.tv_usec;

	t = t*1000;

	return t;
}


TQPTM_Time
TimeGetSysTicks (void)
{
	TQPTM_Time t = (TQPTM_Time)HZ;
	return t;
}


EQP_Errors
TimeDelayMilliS (TQP_UINT32 milliS)
{
	EQP_Errors		rv = QPERR_SUCCESS;

#if LINUX_VERSION_CODE < KERNEL_VERSION(3,15,0)
    //set_current_state( TASK_INTERRUPTIBLE );
    //schedule_timeout( milliS*HZ/1000 )
#ifdef CONFIG_HOST_PLATFORM_STR8133_REF
    if(!gnTimeInit) {
        init_waitqueue_head(&wq);
        gnTimeInit= 1;
    }
#endif // CONFIG_HOST_PLATFORM_STR8133_REF

    sleep_on_timeout (&wq, (HZ * milliS + 999) / 1000);
#else
    msleep(milliS);
#endif
	return (rv);
}

EQP_Errors
TimeDelayMicroS (TQP_UINT32 microS)
{
	EQP_Errors		rv = QPERR_SUCCESS;

#if defined(CONFIG_HOST_PLATFORM_SMP8644_REF) || defined(CONFIG_HOST_PLATFORM_SMP8654_REF)
	tangox_udelay( microS );
#else
	udelay( microS );
#endif

	return (rv);
}



