///////////////////////////////////////////////////////////////////////////////
//        
//
// Cadi7393.h
//
// Description: 
//
//  ADI73931 implementation
// 
// History:
//	May 12, 2008 created
//
// Authors: Andy Yu
//
///////////////////////////////////////////////////////////////////////////////


#include "cadi7393.h"
#include "util.h"
#include "qpdbgmgr.h"

extern TQP_DbgMCompID	g_QPPeripheralDBGCompID;


/////////////////////////////////////////////////////////////////////////////
//
//

static EQP_Errors CADI7393_readRegister(CADI7393 *This, 
										TQP_UINT8 regAddr, 
										PTQP_UINT8 pValue
										)
{
    return (II2C_WriteThenRead(This->m_pI2C, 
							   This->m_cI2CDevAddr,	//device address
							   1,					//write size
							   &regAddr,			//write buffer
							   1,					//read size
							   pValue				//read buffer
							   ));
}



static EQP_Errors CADI7393_writeRegister(CADI7393 *This, 
										 TQP_UINT8 regAddr, 
										 TQP_UINT8 value
										 )
{
    TQP_UINT8 buffer[2];

	buffer[0] = regAddr;
	buffer[1] = value;

    return (II2C_Write(This->m_pI2C, 
					   This->m_cI2CDevAddr,	//device address
					   2,					//write size
					   buffer				//write buffer
					   ));
}



static EQP_Errors CADI7393_WriteBlock(CADI7393 *This, 
									  TQP_UINT32 *pRegTable
									  )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	TQP_UINT32	dwTotalReg = 0;

	// get total number of registers to write
	//
	while (-1 != pRegTable[dwTotalReg]) 
	{
		dwTotalReg++;
	}

	dwTotalReg >>= 1;

	// do multiple writes
	//
	while (dwTotalReg)
	{
		qpStatus = CADI7393_writeRegister(This,
										  (TQP_UINT8)pRegTable[0], 
										  (TQP_UINT8)pRegTable[1]
										  );
		if (!QP_SUCCESS(qpStatus))
			return (qpStatus);

		pRegTable += 2;
		dwTotalReg--;
	}

	return (qpStatus);
}



/////////////////////////////////////////////////////////////////////////////
//
//


// IGenericDevice
//
static EQP_Errors CADI7393_InitDevice(IVideoEncoder *This_p, 
									  QP_DEVICE_CALLBACK pFuncCallback,
									  TQP_PVOID context,
									  TQP_PVOID pInitData
									  )
{
	PQPVIDENCODER_INITDATA	pVidEncInitData = (PQPVIDENCODER_INITDATA)pInitData;
	VIDEO_STANDARD			standard;
	CVideoEncoder			*pVideoEncoder = GET_INHERITED_OBJECT(CVideoEncoder, m_iVideoEncoder, This_p);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CADI7393_InitDevice()\n"
				 ));

	if (CObject_IsInitialized(&pVideoEncoder->m_Object))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
					 "CADI7393_InitDevice() Already Initialized. Skipping...\n"
					 ));
		return (QPERR_SUCCESS);
	}

	// save callback address and context
	//
	pVideoEncoder->m_pDeviceCallback = pFuncCallback;
	pVideoEncoder->m_callbackContext = context;

	// default standard
	//
	if (pVidEncInitData)
	{
		standard = pVidEncInitData->standard;

		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
					 "CADI7393_InitDevice() input standard(%d), remembered standard(%d)\n",
					 standard, 
					 pVideoEncoder->m_VidStandard
					 ));
	}
	else
	{
		standard = pVideoEncoder->m_VidStandard;
	}

	IVideoEncoder_SetVideoStandard(This_p,
								   standard
								   );

	pVideoEncoder->m_fVideoEncoderEnabled = QP_TRUE;

	// CObject Init
	//
	CObject_Init(&pVideoEncoder->m_Object);

	return (QPERR_SUCCESS);
}



// IVideoEncoder
//

// Enables encoder device.
static EQP_Errors CADI7393_Enable(IVideoEncoder *This_p)
{
/*
	TQP_UINT8		ucReg;
	EQP_Errors		qpStatus;
*/
	CVideoEncoder	*pVideoEncoder = GET_INHERITED_OBJECT(CVideoEncoder, m_iVideoEncoder, This_p);
	//CADI7393		*This = GET_INHERITED_OBJECT(CADI7393, m_VideoEncoder, pVideoEncoder);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CADI7393_Enable() ****** ADI7393 doesn't support this feature! ******\n"
				 ));

/*
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CADI7393_Enable() ****** writeRegister() failed to write ADI7393 registers! ******\n" 
					 ));
		return (QPERR_FAIL);
	}
*/

	pVideoEncoder->m_fVideoEncoderEnabled = QP_TRUE;
	return (QPERR_SUCCESS);
}



// Disables encoder device.
static EQP_Errors CADI7393_Disable(IVideoEncoder *This_p)
{
/*
	TQP_UINT8		ucReg;
	EQP_Errors		qpStatus;
*/
	CVideoEncoder	*pVideoEncoder = GET_INHERITED_OBJECT(CVideoEncoder, m_iVideoEncoder, This_p);
	//CADI7393		*This = GET_INHERITED_OBJECT(CADI7393, m_VideoEncoder, pVideoEncoder);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CADI7393_Disable() ****** ADI7393 doesn't support this feature! ******\n"
				 ));

/*
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CADI7393_Disable() ****** writeRegister failed to write ADI7393 registers! ******\n"
					 ));
		return (QPERR_FAIL);
	}
*/

	pVideoEncoder->m_fVideoEncoderEnabled = QP_FALSE;
	return (QPERR_SUCCESS);
}



// Sets video system of the encoder.
//
static EQP_Errors CADI7393_SetVideoStandard(IVideoEncoder *This_p, 
											VIDEO_STANDARD standard
											)
{
	static TQP_UINT32	dw7393RegValsNtsc[] = 
	{
    0x17, 0x02,
    0x00, 0x1C,
    0x01, 0x10,
    0x30, 0x00,
    0x31, 0x01,
    -1,   -1
	};

	static TQP_UINT32	dw7393RegValsPal[] = 
	{
    0x17, 0x02,
    0x00, 0x1C,
    0x01, 0x10,
    0x30, 0x18,
    0x31, 0x01,
    -1,   -1
	};

	static TQP_UINT32	dw7393RegVals720p[] = 
	{
    0x17, 0x02,
    0x00, 0x1C,
    0x01, 0x10,
    0x30, 0x28,
    0x31, 0x01,
    -1,   -1
	};

	static TQP_UINT32	dw7393RegVals1080i[] = 
	{
    0x17, 0x02,
    0x00, 0x1C,
    0x01, 0x10,
    0x30, 0x6C,
    0x31, 0x01,
    -1,   -1
	};

	TQP_UINT32		*pVals;
	EQP_Errors		qpStatus;
	CVideoEncoder	*pVideoEncoder = GET_INHERITED_OBJECT(CVideoEncoder, m_iVideoEncoder, This_p);
	CADI7393		*This = GET_INHERITED_OBJECT(CADI7393, m_VideoEncoder, pVideoEncoder);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CADI7393_SetVideoStandard() standard(%d)\n",
				 standard
				 ));

	if (VIDEO_STANDARD_NTSC == standard)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
					 "CADI7393_SetVideoStandard() NTSC\n"
					 ));
		pVals = dw7393RegValsNtsc;
	}
	else if (VIDEO_STANDARD_PAL == standard)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
					 "CADI7393_SetVideoStandard() PAL\n"
					 ));
		pVals = dw7393RegValsPal;
	}
	else if (VIDEO_STANDARD_720P == standard)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
					 "CADI7393_SetVideoStandard() 720p\n"
					 ));
		pVals = dw7393RegVals720p;
	}
	else if (VIDEO_STANDARD_1080I == standard)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
					 "CADI7393_SetVideoStandard() 1080i\n"
					 ));
		pVals = dw7393RegVals1080i;
	}
	else
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CADI7393_SetVideoStandard() Unknown Video Standard: %d \n", 
					 standard
					 ));
		return (QPERR_PARMS);
	}

	// save the video standard
	//
	pVideoEncoder->m_VidStandard = standard;

	// Initialize 7128 with the default value
	//
	qpStatus = CADI7393_WriteBlock(This,
								   pVals
								   );

	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CADI7393_SetVideoStandard() WriteBlock Failed!\n"
					 ));
		return (QPERR_FAIL);
	}

	return (QPERR_SUCCESS);
}



// constructor
//
CADI7393 * CADI7393_Constructor(CADI7393 *pADI7393,
								CObject *pParent,
								TQP_UINT32 dwAttributes,
								II2C *pI2C,
								TQP_UINT8 cI2CDevAddr
								)
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CADI7393_Constructor()\n"
				 ));

	if (pADI7393)
	{
		CVideoEncoder	*pVideoEncoder;

		pVideoEncoder = CVideoEncoder_Constructor(&pADI7393->m_VideoEncoder, 
												  pParent,
												  dwAttributes
												  );
		if (!pVideoEncoder)
		{
			pADI7393 = QP_NULL;
		}
		else
		{
			// fill in the function table
			//

			// IGenericDevice
			pADI7393->m_VideoEncoder.m_iVideoEncoder.InitDevice = CADI7393_InitDevice;

			// IVideoEncoder
			pADI7393->m_VideoEncoder.m_iVideoEncoder.Enable = CADI7393_Enable;
			pADI7393->m_VideoEncoder.m_iVideoEncoder.Disable = CADI7393_Disable;
			pADI7393->m_VideoEncoder.m_iVideoEncoder.SetVideoStandard = CADI7393_SetVideoStandard;

			// initialize members
			//
			pADI7393->m_pI2C = pI2C;
			pADI7393->m_cI2CDevAddr = cI2CDevAddr;
		}
	}

	return (pADI7393);
}



// destructor
//
void CADI7393_Destructor(CADI7393 *This)
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CADI7393_Destructor()\n"
				 ));

	CVideoEncoder_Destructor(&This->m_VideoEncoder);
}


