///////////////////////////////////////////////////////////////////////////////
//     
//
// Cadi9889.c
//
// Description:
//
//  ad9889B implementation
//
// History:
//	February 12, 2010 created
//
// Authors: Frank Wei
//
///////////////////////////////////////////////////////////////////////////////


#include "cadi9889.h"
#include "util.h"
#include "qpdbgmgr.h"

extern TQP_DbgMCompID	g_QPPeripheralDBGCompID;


/////////////////////////////////////////////////////////////////////////////
//
//

static EQP_Errors CADI9889_readRegister(CADI9889 *This,
										TQP_UINT8 regAddr,
										PTQP_UINT8 pValue
										)
{
    return (II2C_WriteThenRead(This->m_pI2C,
							   This->m_cI2CDevAddr,	//device address
							   1,					//write size
							   &regAddr,			//write buffer
							   1,					//read size
							   pValue				//read buffer
							   ));
}



static EQP_Errors CADI9889_writeRegister(CADI9889 *This,
										 TQP_UINT8 regAddr,
										 TQP_UINT8 value
										 )
{
    TQP_UINT8 buffer[2];

	buffer[0] = regAddr;
	buffer[1] = value;

    return (II2C_Write(This->m_pI2C,
					   This->m_cI2CDevAddr,	//device address
					   2,					//write size
					   buffer				//write buffer
					   ));
}



static EQP_Errors CADI9889_WriteBlock(CADI9889 *This,
									  TQP_UINT32 *pRegTable
									  )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	TQP_UINT32	dwTotalReg = 0;

	// get total number of registers to write
	//
	while (-1 != pRegTable[dwTotalReg])
	{
		dwTotalReg++;
	}

	dwTotalReg >>= 1;

	// do multiple writes
	//
	while (dwTotalReg)
	{
		qpStatus = CADI9889_writeRegister(This,
										  (TQP_UINT8)pRegTable[0],
										  (TQP_UINT8)pRegTable[1]
										  );
		if (!QP_SUCCESS(qpStatus))
			return (qpStatus);

		pRegTable += 2;
		dwTotalReg--;
	}

	return (qpStatus);
}



/////////////////////////////////////////////////////////////////////////////
//
//


// IGenericDevice
//
static EQP_Errors CADI9889_InitDevice(IVideoEncoder *This_p,
									  QP_DEVICE_CALLBACK pFuncCallback,
									  TQP_PVOID context,
									  TQP_PVOID pInitData
									  )
{
	PQPVIDENCODER_INITDATA	pVidEncInitData = (PQPVIDENCODER_INITDATA)pInitData;
	VIDEO_STANDARD			standard;
	CVideoEncoder			*pVideoEncoder = GET_INHERITED_OBJECT(CVideoEncoder, m_iVideoEncoder, This_p);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CADI9889_InitDevice()\n"
				 ));

	if (CObject_IsInitialized(&pVideoEncoder->m_Object))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
					 "CADI9889_InitDevice() Already Initialized. Skipping...\n"
					 ));
		return (QPERR_SUCCESS);
	}

	// save callback address and context
	//
	pVideoEncoder->m_pDeviceCallback = pFuncCallback;
	pVideoEncoder->m_callbackContext = context;

	// default standard
	//
	if (pVidEncInitData)
	{
		standard = pVidEncInitData->standard;

		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
					 "CADI9889_InitDevice() input standard(%d), remembered standard(%d)\n",
					 standard,
					 pVideoEncoder->m_VidStandard
					 ));
	}
	else
	{
		standard = pVideoEncoder->m_VidStandard;
	}

	IVideoEncoder_SetVideoStandard(This_p,
								   standard
								   );

	pVideoEncoder->m_fVideoEncoderEnabled = QP_TRUE;

	// CObject Init
	//
	CObject_Init(&pVideoEncoder->m_Object);

	return (QPERR_SUCCESS);
}



// IVideoEncoder
//

// Enables encoder device.
static EQP_Errors CADI9889_Enable(IVideoEncoder *This_p)
{
	CVideoEncoder	*pVideoEncoder = GET_INHERITED_OBJECT(CVideoEncoder, m_iVideoEncoder, This_p);
	//CADI9889		*This = GET_INHERITED_OBJECT(CADI9889, m_VideoEncoder, pVideoEncoder);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CADI9889_Enable() ****** not implemented! ******\n"
				 ));

	pVideoEncoder->m_fVideoEncoderEnabled = QP_TRUE;
	return (QPERR_SUCCESS);
}



// Disables encoder device.
static EQP_Errors CADI9889_Disable(IVideoEncoder *This_p)
{
	CVideoEncoder	*pVideoEncoder = GET_INHERITED_OBJECT(CVideoEncoder, m_iVideoEncoder, This_p);
	//CADI9889		*This = GET_INHERITED_OBJECT(ADI9889, m_VideoEncoder, pVideoEncoder);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CADI9889_Disable() ****** not implemented! ******\n"
				 ));


	pVideoEncoder->m_fVideoEncoderEnabled = QP_FALSE;
	return (QPERR_SUCCESS);
}



// Sets video system of the encoder.
//
static EQP_Errors CADI9889_SetVideoStandard(IVideoEncoder *This_p,
											VIDEO_STANDARD standard
											)
{
	// same settings for all standards
	static TQP_UINT32	dw9889RegVals[] =
	{
		0x1,0x0,
		0x2,0x18,
		0x3,0x0,
		0x7,0x0,
		0x8,0x0,
		0x9,0x0,
		0x0A,0x0D,	//I2S 256fs
		0x0B,0x0E,
		0x0C,0x04,
		0x0D,0x18,	//24bit
		0x0E,0x1,
		0x0F,0x13,
		0x10,0x25,
		0x11,0x37,
		0x12,0x0,
		0x13,0x0,
		0x14,0x0,
		0x15,0x22,	//48K, YCbCr 4:2:2
		//0x15,0x02,	//44.1K, YCbCr 4:2:2
		0x16,0x40,	//YCbCr 4:4:4 out
		0x17,0x0A,
		0x18,0x6,
		0x19,0x62,
		0x1A,0x4,
		0x1B,0xA8,
		0x1C,0x0,
		0x1D,0x0,
		0x1E,0x1C,
		0x1F,0x84,
		0x20,0x1C,
		0x21,0xBF,
		0x22,0x4,
		0x23,0xA8,
		0x24,0x1E,
		0x25,0x70,
		0x26,0x2,
		0x27,0x1E,
		0x28,0x0,
		0x29,0x0,
		0x2A,0x4,
		0x2B,0xA8,
		0x2C,0x8,
		0x2D,0x12,
		0x2E,0x1B,
		0x2F,0xAC,
		0x30,0x0,
		0x31,0x0,
		0x32,0x0,
		0x33,0x0,
		0x34,0x0,
		0x35,0x0,
		0x36,0x0,
		0x37,0x0,
		0x38,0x0,
		0x39,0x0,
		0x3A,0x0,
		0x3B,0x80,	//0x3B,0x81, CSC
		0x3C,0x0,
		0x40,0x0,
		0x41,0x10,
		0x43,0xA0,	//EDID
		0x44,0xF8,
		0x45,0x20,	//YCbCr
		0x46,0x0,
		0x47,0x0,
		0x48,0x0,
		0x49,0x0,
		0x4A,0x0,
		0x4B,0x0,
		0x4C,0x0,
		0x4D,0x0,
		0x4E,0x0,
		0x4F,0x0,
		0x50,0x0,
		0x51,0x0,
		0x52,0x0,
		0x53,0x0,
		0x54,0x0,
		0x55,0x0,
		0x56,0x0,
		0x57,0x0,
		0x58,0x0,
		0x59,0x0,
		0x5A,0x0,
		0x5B,0x0,
		0x5C,0x0,
		0x5D,0x0,
		0x5E,0x0,
		0x5F,0x0,
		0x60,0x0,
		0x61,0x0,
		0x62,0x0,
		0x63,0x0,
		0x64,0x0,
		0x65,0x0,
		0x66,0x0,
		0x67,0x0,
		0x68,0x0,
		0x69,0x0,
		0x6A,0x0,
		0x6B,0x0,
		0x6C,0x0,
		0x6D,0x0,
		0x6E,0x0,
		0x6F,0x0,
		0x70,0x0,
		0x71,0x0,
		0x72,0x0,
		0x73,0x0,
		0x74,0x0,
		0x75,0x0,
		0x76,0x0,
		0x77,0x0,
		0x78,0x0,
		0x79,0x0,
		0x7A,0x0,
		0x7B,0x0,
		0x7C,0x0,
		0x7D,0x0,
		0x7E,0x0,
		0x7F,0x0,
		0x80,0x0,
		0x81,0x0,
		0x82,0x0,
		0x83,0x0,
		0x84,0x0,
		0x85,0x0,
		0x86,0x0,
		0x87,0x0,
		0x88,0x0,
		0x89,0x0,
		0x8A,0x0,
		0x8B,0x0,
		0x8C,0x0,
		0x8D,0x0,
		0x8E,0x0,
		0x8F,0x0,
		0x90,0x0,
		0x91,0x0,
		0x92,0x0,
		0x93,0x0,
		0x94,0xC0,
		0x95,0x0,
		0x96,0x20,
		0x97,0x0,		//0x97,4,
		0x98,0x3,
		0x99,0x2,
		0x9A,0x0,
		0x9B,0x18,
		0x9C,0x38,
		0x9D,0x61,
		0xA1,0x0,
		0xA2,0x87,
		0xA3,0x87,
		0xA4,0x8,
		0xA5,0xC0,
		0xA6,0x0,
		0xA7,0x0,
		0xA8,0x0,
		0xA9,0x0,
		0xAA,0x0,
		0xAB,0x0,
		0xAF,0x16,
		0xB9,0x0,
		0xBA,0x60,
		0xBB,0x0,
		0xC4,0x0,
		0xC9,0x3,
		0xCA,0x0,
		0xCB,0x0,
		0xCC,0x2,
		0xFA,0x7D,
		0xFB,0xAA,
		0xFC,0x1C,
		0xFD,0x0,
		0xFE,0xB0,
		0xFF,0x0,
		-1,   -1
	};

	TQP_UINT32		*pVals;
	EQP_Errors		qpStatus;
	CVideoEncoder	*pVideoEncoder = GET_INHERITED_OBJECT(CVideoEncoder, m_iVideoEncoder, This_p);
	CADI9889		*This = GET_INHERITED_OBJECT(CADI9889, m_VideoEncoder, pVideoEncoder);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CADI9889_SetVideoStandard() standard(%d)\n",
				 standard
				 ));

	if (VIDEO_STANDARD_720P == standard)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
					 "CADI9889_SetVideoStandard() 720p\n"
					 ));
		pVals = dw9889RegVals;
	}
	else if (VIDEO_STANDARD_1080I == standard)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
					 "CADI9889_SetVideoStandard() 1080i\n"
					 ));
		pVals = dw9889RegVals;
	}
	else if (VIDEO_STANDARD_1080P == standard)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
					 "CADI9889_SetVideoStandard() 1080p\n"
					 ));
		pVals = dw9889RegVals;
	}
	else
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CADI9889_SetVideoStandard() Unknown Video Standard: %d \n",
					 standard
					 ));
		return (QPERR_PARMS);
	}

	// save the video standard
	//
	pVideoEncoder->m_VidStandard = standard;

	// Initialize 9889 with the default value
	//
	qpStatus = CADI9889_WriteBlock(This,
								   pVals
								   );

	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CADI9889_SetVideoStandard() WriteBlock Failed!\n"
					 ));
		return (QPERR_FAIL);
	}

	return (QPERR_SUCCESS);
}



// constructor
//
CADI9889 * CADI9889_Constructor(CADI9889 *pADI9889,
								CObject *pParent,
								TQP_UINT32 dwAttributes,
								II2C *pI2C,
								TQP_UINT8 cI2CDevAddr
								)
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CADI9889_Constructor()\n"
				 ));

	if (pADI9889)
	{
		CVideoEncoder	*pVideoEncoder;

		pVideoEncoder = CVideoEncoder_Constructor(&pADI9889->m_VideoEncoder,
												  pParent,
												  dwAttributes
												  );
		if (!pVideoEncoder)
		{
			pADI9889 = QP_NULL;
		}
		else
		{
			// fill in the function table
			//

			// IGenericDevice
			pADI9889->m_VideoEncoder.m_iVideoEncoder.InitDevice = CADI9889_InitDevice;

			// IVideoEncoder
			pADI9889->m_VideoEncoder.m_iVideoEncoder.Enable = CADI9889_Enable;
			pADI9889->m_VideoEncoder.m_iVideoEncoder.Disable = CADI9889_Disable;
			pADI9889->m_VideoEncoder.m_iVideoEncoder.SetVideoStandard = CADI9889_SetVideoStandard;

			// initialize members
			//
			pADI9889->m_pI2C = pI2C;
			pADI9889->m_cI2CDevAddr = cI2CDevAddr;
		}
	}

	return (pADI9889);
}



// destructor
//
void CADI9889_Destructor(CADI9889 *This)
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CADI9889_Destructor()\n"
				 ));

	CVideoEncoder_Destructor(&This->m_VideoEncoder);
}


