///////////////////////////////////////////////////////////////////////////////
//        
//
// Caudcodec.c
//
// Description: 
//
//  Audio codec implementation
// 
// History:
//	Aug 28, 2006	(dt) created
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#include "caudcodec.h"
#include "util.h"
#include "qpdbgmgr.h"


extern TQP_DbgMCompID	g_QPPeripheralDBGCompID;


// IGenericDevice
//
static EQP_Errors CAudioCodec_InitDevice(IAudioCodec *This_p,
										 QP_DEVICE_CALLBACK pFuncCallback,
										 TQP_PVOID context,
										 TQP_PVOID pInitData
										 )
{
	CAudioCodec	*This =	GET_INHERITED_OBJECT(CAudioCodec, m_iAudCodec, This_p);


	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CAudioCodec_InitDevice()\n"
				 ));

	// save callback address and context
	//
	This->m_pDeviceCallback = pFuncCallback;
	This->m_callbackContext = context;

	// CObject Init
	//
	CObject_Init(&This->m_Object);
	return (QPERR_SUCCESS);
}



static EQP_Errors CAudioCodec_Release(IAudioCodec *This_p)
{
	CAudioCodec	*This =	GET_INHERITED_OBJECT(CAudioCodec, m_iAudCodec, This_p);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CAudioCodec_Release()\n"
				 ));

	This->m_pDeviceCallback = QP_NULL;
	This->m_callbackContext = QP_NULL;
	CObject_Done(&This->m_Object);
	return (QPERR_SUCCESS);
}



static EQP_Errors CAudioCodec_Reset(IAudioCodec *This_p) 
{
	return (QPERR_SUCCESS);
}



static EQP_Errors CAudioCodec_Set(IAudioCodec *This_p, 
								  TQP_REFGUID guid,
								  TQP_OBJECT_HANDLE hIndex,
								  TQP_UINT32 dwCode,
								  TQP_PVOID pInput,
								  TQP_PVOID pOutput,
								  TQP_UINT32 dwSize
								  )
{
	return (QPERR_NOTIMPL);
}



static EQP_Errors CAudioCodec_Get(IAudioCodec *This_p,
								  TQP_REFGUID guid,
								  TQP_OBJECT_HANDLE hIndex,
								  TQP_UINT32 dwCode,
								  TQP_PVOID pInput,
								  TQP_PVOID pOutput,
								  PTQP_UINT32 pdwSizeGot
								  )
{
	return (QPERR_NOTIMPL);
}



// IAudioCodec
//

// Select the audio source.
static EQP_Errors CAudioCodec_SelectSource(IAudioCodec *This_p, 
										   AIN_SOURCE src
										   ) 
{
	CAudioCodec	*This =	GET_INHERITED_OBJECT(CAudioCodec, m_iAudCodec, This_p);

	This->m_audioSource = src; 
	return (QPERR_SUCCESS);
}



// Get the audio source.
static EQP_Errors CAudioCodec_GetSource(IAudioCodec *This_p,
										PAIN_SOURCE psrc
										) 
{
	CAudioCodec	*This =	GET_INHERITED_OBJECT(CAudioCodec, m_iAudCodec, This_p);

	*psrc = This->m_audioSource; 
	return (QPERR_SUCCESS);
}



// Get Attenuation Range for the Device
static EQP_Errors CAudioCodec_GetVolumeRange(IAudioCodec *This_p,
											 int *pnDefault,
											 int *pnMin,
											 int *pnMax
											 )
{
	CAudioCodec	*This =	GET_INHERITED_OBJECT(CAudioCodec, m_iAudCodec, This_p);

	if ((QP_NULL == pnDefault) ||
		(QP_NULL == pnMin) ||
		(QP_NULL == pnMax)
		)
	{
		return (QPERR_PARMS);
	}

	*pnDefault = This->m_nVolumeDef;
	*pnMin = This->m_nVolumeMin;
	*pnMax = This->m_nVolumeMax;

	return (QPERR_SUCCESS);
}



// Set Volume
static EQP_Errors CAudioCodec_SetVolume(IAudioCodec *This_p,
										int nLeft,
										int nRight
										)
{
	CAudioCodec	*This =	GET_INHERITED_OBJECT(CAudioCodec, m_iAudCodec, This_p);

	This->m_nVolumeL = nLeft;
	This->m_nVolumeR = nRight;
	return (QPERR_SUCCESS);
}



static EQP_Errors CAudioCodec_GetVolume(IAudioCodec *This_p,
										int *pnLeft,
										int *pnRight
										)
{
	CAudioCodec	*This =	GET_INHERITED_OBJECT(CAudioCodec, m_iAudCodec, This_p);

	*pnLeft = This->m_nVolumeL;
	*pnRight = This->m_nVolumeR;
	return (QPERR_SUCCESS);
}



// Set Mute
static EQP_Errors CAudioCodec_SetMute(IAudioCodec *This_p,
									  TQP_BOOL fMuteLeft,
									  TQP_BOOL fMuteRight
									  )
{
	CAudioCodec	*This =	GET_INHERITED_OBJECT(CAudioCodec, m_iAudCodec, This_p);

	This->m_bMuteL = fMuteLeft;
	This->m_bMuteR = fMuteRight;
	return (QPERR_SUCCESS);
}



static EQP_Errors CAudioCodec_GetMute(IAudioCodec *This_p,
									  TQP_BOOL *pfMuteLeft,
									  TQP_BOOL *pfMuteRight
									  )
{
	CAudioCodec	*This =	GET_INHERITED_OBJECT(CAudioCodec, m_iAudCodec, This_p);

	*pfMuteLeft = This->m_bMuteL;
	*pfMuteRight = This->m_bMuteR;
	return (QPERR_SUCCESS);
}



// constructor
//
CAudioCodec * CAudioCodec_Constructor(CAudioCodec *pAudCodec,
									  CObject *pParent,
									  TQP_UINT32 dwAttributes,
									  ITuner *pTuner,
									  ITVAudio *pTvAudio
									  )
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CAudioCodec_Constructor()\n"
				 ));

	if (pAudCodec)
	{
		CObject_Constructor(&pAudCodec->m_Object, 
							pParent, 
		 					dwAttributes
		 					);

		// fill in the function table
		//

		// IGenericDevice
		pAudCodec->m_iAudCodec.InitDevice = CAudioCodec_InitDevice;
		pAudCodec->m_iAudCodec.Release = CAudioCodec_Release;
		pAudCodec->m_iAudCodec.Reset = CAudioCodec_Reset;
		pAudCodec->m_iAudCodec.Set = CAudioCodec_Set;
		pAudCodec->m_iAudCodec.Get = CAudioCodec_Get;

		// IAudioCodec
		pAudCodec->m_iAudCodec.SelectSource = CAudioCodec_SelectSource;
		pAudCodec->m_iAudCodec.GetSource = CAudioCodec_GetSource;
		pAudCodec->m_iAudCodec.GetVolumeRange = CAudioCodec_GetVolumeRange;
		pAudCodec->m_iAudCodec.SetVolume = CAudioCodec_SetVolume;
		pAudCodec->m_iAudCodec.GetVolume = CAudioCodec_GetVolume;
		pAudCodec->m_iAudCodec.SetMute = CAudioCodec_SetMute;
		pAudCodec->m_iAudCodec.GetMute = CAudioCodec_GetMute;

		// initialize members
		//
		pAudCodec->m_pTuner = pTuner;
		pAudCodec->m_pTvAudio = pTvAudio;
		pAudCodec->m_pDeviceCallback = QP_NULL;
		pAudCodec->m_callbackContext = QP_NULL;

		pAudCodec->m_audioSource = AUDINVALID;
		pAudCodec->m_bMuteL = QP_FALSE;
		pAudCodec->m_bMuteR = QP_FALSE;
	}

	return (pAudCodec);
}


// destructor
//
void CAudioCodec_Destructor(CAudioCodec *This)
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CAudioCodec_Destructor()\n"
				 ));

	if (CObject_IsInitialized(&This->m_Object))
		QPGenericDevice_Release(&This->m_iAudCodec);

	CObject_Destructor(&This->m_Object);
}




