///////////////////////////////////////////////////////////////////////////////
//     
//
// cfm31.c
//
// Description:
//
//  FM31  implementation
//
// History:
//	Nov 20, 2011 created
//
// Authors: CY Liu
//
///////////////////////////////////////////////////////////////////////////////


#include "cfm31.h"
#include "qptimemgr.h"
#include "qpdbgmgr.h"
#include "util.h"
#include "cqlcodeclib.h"
#include "ci2c.h"

/////////////////////////////////////////////////////////////////////////////
//
//

extern TQP_DbgMCompID	g_QPPeripheralDBGCompID;


/////////////////////////////////////////////////////////////////////////////
//
//
static EQP_Errors CFM31_WriteBlock(CFM31 *This, TQP_UINT8 length,
		TQP_UINT8 data_0, TQP_UINT8 data_1, TQP_UINT8 data_2, TQP_UINT8 data_3,
		TQP_UINT8 data_4, TQP_UINT8 data_5, TQP_UINT8 data_6)
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	TQP_UINT8 buffer[7];

	buffer[0] = data_0;
	buffer[1] = data_1;
	buffer[2] = data_2;
	buffer[3] = data_3;
	buffer[4] = data_4;
	buffer[5] = data_5;
	buffer[6] = data_6;

    qpStatus = II2C_Write(This->m_pI2C, This->m_cI2CDevAddr, length, buffer);
	return (qpStatus);
}

static EQP_Errors CFM31_ClockInit(CFM31 * This)
{
	EQP_Errors	qpStatus;
	TQP_UINT32	dwRegister;
	TQP_UINT32  dwRegB0=0;
	TQP_UINT32  dwRegB4=0;
	TQP_UINT32  dwReg30=0;
    CI2C *pCI2C = GET_INHERITED_OBJECT(CI2C, m_ii2c, This->m_pI2C);

	qpStatus = QPHCI_RegisterRead(&pCI2C->m_pMpegCodec->m_hci, 0xB0, &dwRegB0 );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterRead(&pCI2C->m_pMpegCodec->m_hci, 0xB4, &dwRegB4 );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterRead(&pCI2C->m_pMpegCodec->m_hci, 0x30, &dwReg30 );

	/* Step1: Program CPU to output clock on AXCLK pin*/
	/* set 0xB0 to 0xe023f */
	dwRegister = 0xe023f;
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&pCI2C->m_pMpegCodec->m_hci, 0xB0, dwRegister );
	/* set 0xB4 to 0x418021f */
	dwRegister = 0x418021f;
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&pCI2C->m_pMpegCodec->m_hci, 0xB4, dwRegister );
	/* set 0x30 bit 16 */
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterRead(&pCI2C->m_pMpegCodec->m_hci, 0x30, &dwRegister );
	dwRegister = dwRegister | (1 << 16);
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&pCI2C->m_pMpegCodec->m_hci, 0x30, dwRegister );

	/* Step2: Reset FM31 by toggle GPIO 4*/
	/* set gpio 4 bit directoion to output */
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterRead(&pCI2C->m_pMpegCodec->m_hci, 0x610, &dwRegister);
	dwRegister = dwRegister | (1 << 4);
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&pCI2C->m_pMpegCodec->m_hci, 0x610, dwRegister );
	/* set gpio 4, value 0 */
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterRead(&pCI2C->m_pMpegCodec->m_hci, 0x614, &dwRegister );
	dwRegister = dwRegister & (~(1 << 4));
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&pCI2C->m_pMpegCodec->m_hci, 0x614, dwRegister );
	QPTMDelayMilliS(10);			//10ms (FM31 spec.: 100us (min.)
	/* set gpio 4, value 1 */
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterRead(&pCI2C->m_pMpegCodec->m_hci, 0x614, &dwRegister );
	dwRegister = dwRegister | (1 << 4);
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&pCI2C->m_pMpegCodec->m_hci, 0x614, dwRegister );
	QPTMDelayMilliS(50);

	/* Step3: Do a simple write to check it work or not */
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x22, 0xCF, 0x00, 0x00);
	if (!QP_SUCCESS(qpStatus))
	{
		/* restore original value */
		QPHCI_RegisterWrite(&pCI2C->m_pMpegCodec->m_hci, 0xB0, dwRegB0 );
		QPHCI_RegisterWrite(&pCI2C->m_pMpegCodec->m_hci, 0xB4, dwRegB4 );
		QPHCI_RegisterWrite(&pCI2C->m_pMpegCodec->m_hci, 0x30, dwReg30 );
	}
	return (qpStatus);
}

static EQP_Errors CFM31_EnableI2SClock(CFM31 * This)
{
	TQP_UINT32	dwRegister;
    CI2C *pCI2C = GET_INHERITED_OBJECT(CI2C, m_ii2c, This->m_pI2C);
	/* Programming CPU "Global Enable and Gated Clock For Codec" register, set bit 8, 9, clear bit 26 */
	QPHCI_RegisterRead(&pCI2C->m_pMpegCodec->m_hci, 0, &dwRegister );
	dwRegister = (dwRegister & (~(1<<26))) | (1<<8) | (1<<9);
	QPHCI_RegisterWrite(&pCI2C->m_pMpegCodec->m_hci, 0, dwRegister );
	/* Programming CPU "Audio Output Control" register */
    dwRegister = 0x16;
	QPHCI_RegisterWrite(&pCI2C->m_pMpegCodec->m_hci, 0x304, dwRegister );

    dwRegister = 0;
	QPHCI_RegisterRead(&pCI2C->m_pMpegCodec->m_hci, 0x304, &dwRegister);
	return (QPERR_SUCCESS);
}

/////////////////////////////////////////////////////////////////////////////
//
//


// IGenericDevice
//
static EQP_Errors CFM31_InitDevice(IAudioCodec *This_p,
									 QP_DEVICE_CALLBACK pFuncCallback,
									 TQP_PVOID context,
									 TQP_PVOID pInitData
									 )
{
    CAudioCodec *pAudioCodec = GET_INHERITED_OBJECT(CAudioCodec, m_iAudCodec, This_p);
    CFM31      *This = GET_INHERITED_OBJECT(CFM31, m_AudioCodec, pAudioCodec);
	EQP_Errors	qpStatus;

    QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
                 "CFM31_InitDevice()\n"
                ));

    if (CObject_IsInitialized(&pAudioCodec->m_Object))
    {
        QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
                     "CFM31_InitDevice() Already Initialized. Skipping...\n"
                    ));
      return (QPERR_SUCCESS);
    }

    // save callback address and context
    //
    pAudioCodec->m_pDeviceCallback = pFuncCallback;
    pAudioCodec->m_callbackContext = context;

	//Program PL330 for FM31 Clock
	qpStatus = CFM31_ClockInit(This);

#define ONE_MIC
#define AEC_OFF
#define NOISE_SUPPRESSION_ON
	if (QP_SUCCESS(qpStatus))
	{
		//FM31 init register
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x22, 0xCF, 0x00, 0x00);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0x04, 0x00, 0xf8);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0x0D, 0x08, 0x00);
#ifndef ONE_MIC
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0x11, 0x00, 0x02);
#else
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0x11, 0x01, 0x01);
#endif
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0x12, 0x00, 0x90);

#ifndef AEC_OFF
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0x15, 0x03, 0x17);
#else
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0x15, 0x00, 0x17);
#endif
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0x16, 0x00, 0x40);
#ifdef NOISE_SUPPRESSION_ON
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0x17, 0x15, 0x00);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0x18, 0x0f, 0xff);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0x19, 0x0A, 0xff);
#else
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0x17, 0x7f, 0xff);
#endif
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0x2C, 0x00, 0x21);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0x3A, 0x01, 0x80);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0x3B, 0x01, 0x00);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0x44, 0x02, 0x50);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0x52, 0x40, 0x00);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0x53, 0x30, 0x00);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0x8B, 0x20, 0x00);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0xD9, 0x20, 0x00);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0xDA, 0x30, 0x00);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0xDB, 0x32, 0x00);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0xDC, 0x64, 0x00);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0xF7, 0x00, 0x48);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0xFB, 0x05, 0x5F);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0x1A, 0x00, 0x00);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0xB1, 0x2E, 0x00);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0x35, 0x00, 0x06);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0x01, 0x00, 0x02);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0x36, 0x00, 0x02);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0x41, 0x00, 0x10);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0x91, 0x00, 0x05);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0xFC, 0x00, 0x05);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0xB2, 0x34, 0x00);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0xB4, 0x40, 0x00);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0xB5, 0x10, 0x00);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0x1D, 0x00, 0x12);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0x0B, 0x00, 0x03);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0xF9, 0x00, 0x00);
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0x08, 0x00, 0x5F);
		/* run */
		CFM31_WriteBlock(This, 7, 0xfc, 0xf3, 0x3b, 0x23, 0x0C, 0x00, 0x00);
		//QPTMDelayMilliS(300);
		// Enable CPU 16KHz I2S Clock
	    CFM31_EnableI2SClock(This);
	}
	else {
        QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
                     "CFM31_InitDevice() Can not Detect FM31 Chip, Skipping...\n"
                    ));
	}
    // CObject Init
    //
    CObject_Init(&pAudioCodec->m_Object);

    return (QPERR_SUCCESS);
}


// constructor
//
CFM31 * CFM31_Constructor(CFM31 *pFM31,
								CObject *pParent,
								TQP_UINT32 dwAttributes,
								ITuner *pTuner,
								ITVAudio *pTvAudio,
								II2C *pI2C,
								TQP_UINT8 cI2CDevAddr
								)
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CFM31_Constructor()\n"
				 ));

	if (pFM31)
	{
		CAudioCodec	*pAudioCodec;

		pAudioCodec = CAudioCodec_Constructor(&pFM31->m_AudioCodec,
												  pParent,
												  dwAttributes,
												  pTuner,
												  pTvAudio
												  );

		if (!pAudioCodec)
		{
			pFM31 = QP_NULL;
		}
		else
		{
			// fill in the function table
			//

			// IGenericDevice
			pFM31->m_AudioCodec.m_iAudCodec.InitDevice = CFM31_InitDevice;

			// initialize members
			//
			pFM31->m_pI2C = pI2C;
			pFM31->m_cI2CDevAddr = cI2CDevAddr;
		}
	}

	return (pFM31);
}



// destructor
//
void CFM31_Destructor(CFM31 *This)
{
    QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CFM31_Destructor()\n"
				 ));
    CAudioCodec_Destructor(&This->m_AudioCodec);
}

