///////////////////////////////////////////////////////////////////////////////
//        
//
// Ci2c.h
//
// Description: 
//
//  I2C provider implementation
// 
// History:
//	Aug 28, 2006	(dt) created
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __CI2C_H__
#define __CI2C_H__

#if defined(QP_LINXKER)
#include "qlautoconf.h"
#endif

#include "qpi2c.h"
#if (!defined(QP_LINXKER) && !defined(_QP_LINUX) && !defined(_QP_MACOSX)) || defined(CONFIG_PL330SOC)
#include "cgpio_i2c.h"
#endif
#include "cobject.h"
#include "cusbcntl.h"
#include "qptimemgr.h"
#ifdef __cplusplus
extern "C" {
#endif


#ifndef __CQLCODEC_FWD_DEFINED__
#define __CQLCODEC_FWD_DEFINED__
typedef struct CQLCodec CQLCodec;
#endif //__CQLCODEC_FWD_DEFINED__



typedef struct CI2C
{
	// public CObject
	//
	CObject			m_Object;

	// public II2C
	//
	II2C			m_ii2c;

	// access type
	//
	QPI2C_TYPE		m_type;
	TQP_UINT32		m_clk;
	TQP_UINT32		m_data;

	CUsbCntl		*m_pUsbCntl;
	CQLCodec		*m_pMpegCodec;

	/* hw_i2c_in_use, 
	** Only for master mode's callback function in vsync
	** Because in this callback function can not enable critical session protection
	** The operation mode is, 
	**   Other processes to set m_hw_i2c_in_use while they are using h/w i2c bus.
	**   And callback function to check m_hw_i2c_in_use to see whether it can use the h/w i2c bus. 
	*/
	TQP_UINT32		m_hw_i2c_in_use;

#if (!defined(_QP_QPSOS) && !defined(QP_LINXKER) && !defined(_QP_LINUX) && !defined(_QP_MACOSX)) || defined(CONFIG_PL330SOC)
	/* gpioi2cdev, 
	** move from static allocation to here, to have multiple card support in slave mode.
	*/  
	struct gpio_i2c_device gpioi2cdev;
#endif	

} CI2C;



// constructor
//
CI2C * CI2C_Constructor(CI2C *pI2c, 
						CObject *pParent,
						TQP_UINT32 dwAttributes,
						QPI2C_TYPE type,
						TQP_UINT32 clk,
						TQP_UINT32 data,
						CUsbCntl *pUsbCntl,
						CQLCodec *pMpegCodec
						);

// destructor
//
void CI2C_Destructor(CI2C *This);

EQP_Errors _CI2CQl201_Read(CI2C *This,
                           TQP_UINT8 cDeviceAddr,
                           int nRdSize,
                           PTQP_UINT8 pRdBuf,
                           int nWrSize,
                           PTQP_UINT8 pWrBuf,
                           TQP_BOOL fCritical
                           );

EQP_Errors _CI2CQl201_Write(CI2C *This, 
						    TQP_UINT8 cDeviceAddr, 
						    int nSize, 
						    PTQP_UINT8 pBuf,
						    TQP_BOOL fCritical
							);

#ifdef __cplusplus
}
#endif

#endif //__CI2C_H__


