///////////////////////////////////////////////////////////////////////////////
//     
//
// Cit6603.c
//
// Description:
//
//  it6603 implementation
//
// History:
//	Aug 13, 2010 created
//
// Authors: Frank Wei
//
///////////////////////////////////////////////////////////////////////////////

#if defined(QP_LINXKER) || defined (_QP_LINUX) || defined (_QP_MACOSX)
#include "qlautoconf.h"
#endif
#include "cit6603.h"
#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
#include "ctvp7002.h"
#include "ctw9910.h"
#endif
#include "qptimemgr.h"
#include "qpdbgmgr.h"
#include "util.h"
#include "cqlcodeclib.h"
#include "cqlcodec.h"


/////////////////////////////////////////////////////////////////////////////
//
//

extern TQP_DbgMCompID	g_QPPeripheralDBGCompID;


/////////////////////////////////////////////////////////////////////////////
//
//


static EQP_Errors CIT6603_readRegister(CIT6603 *This,
									   TQP_UINT8 regAddr,
									   PTQP_UINT8 pValue
									   )
{
    return (II2C_WriteThenRead(This->m_pI2C,
							   This->m_cI2CDevAddr,	//device address
							   1,					//write size
							   &regAddr,			//write buffer
							   1,					//read size
							   pValue				//read buffer
							   ));
}



static EQP_Errors CIT6603_writeRegister(CIT6603 *This,
										TQP_UINT8 regAddr,
										TQP_UINT8 value
										)
{
    TQP_UINT8 buffer[2];

	buffer[0] = regAddr;
	buffer[1] = value;

    return (II2C_Write(This->m_pI2C,
					   This->m_cI2CDevAddr,	//device address
					   2,					//write size
					   buffer				//write buffer
					   ));
}


static EQP_Errors CIT6603_WriteBlock(CIT6603 *This,
									 TQP_UINT32 *pRegTable
									 )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	TQP_UINT32	dwTotalReg = 0;

	// get total number of registers to write
	//
	while (-1 != pRegTable[dwTotalReg])
	{
		dwTotalReg++;
	}

	dwTotalReg >>= 1;

	// do multiple writes
	//
	while (dwTotalReg)
	{
		qpStatus = CIT6603_writeRegister(This,
										 (TQP_UINT8)pRegTable[0],
										 (TQP_UINT8)pRegTable[1]
										 );
		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CIT6603_WriteBlock() ****** writeRegister failed! status(%d) ******\n",
						 qpStatus
						 ));
			break;
		}

		pRegTable += 2;
		dwTotalReg--;
	}

	return (qpStatus);
}


/////////////////////////////////////////////////////////////////////////////
//
//


// IGenericDevice
//
static EQP_Errors CIT6603_InitDevice(IVideoDecoder *This_p,
									 QP_DEVICE_CALLBACK pFuncCallback,
									 TQP_PVOID context,
									 TQP_PVOID pInitData
									 )
{
    TQP_UINT8     buffer[2];
    //TQP_UINT8     ucRegAddr;
    TQP_UINT8     ucValue;
    EQP_Errors    qpStatus = QPERR_SUCCESS;

    CVideoDecoder *pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
    CQLCodecLib   *pQLCodecLib = GET_INHERITED_OBJECT(CQLCodecLib, m_Object, pVideoDecoder->m_Object.m_pParent);
    CIT6603       *This = GET_INHERITED_OBJECT(CIT6603, m_VideoDecoder, pVideoDecoder);

	//P0 16bit YPbPr 4:2:2 in pass through
    static TQP_UINT32 dwIT6603HDMIRegVals[] =
    {
	//0x06,0x00,
	//0x05,0x10,
	0x05,0x81,
	0x07,0x0C,	//port0
	0x16,0x0F,
	0x17,0x85,
	0x18,0x07,
	0x8C,0x00,

	0x0F,0x00,	//DEFAULT
	0x08,0xAE,
	0x3B,0x40,
	0x68,0x03,
	0x6B,0x11,
	0x6C,0x00,
	0x93,0x43,
	0x94,0x4F,
	0x95,0x87,
	0x96,0x33,

	0x1D,0x20,
	0x75,0x60,  //24bit stand I2S, change to 0xE0 to invert left & right channel (was 0x60)
	0x3D,0x40,	//YUV422
	0x1A,0x17,
	0x1B,0x60,	//16bit
	0x1C,0x10,
	0x78,0xC1,	//27M 256fs
	0x89,0xFF,
	0x05,0x00,
	//0x11,0x09,	//int HDCP     // Move to initial code, only need do these for IT6603
	//0x11,0x19,

	0x9B,0x80,	//rdrom reset?
	0x9B,0xC0,
	0x9B,0x80,
	0x9B,0xC0,
	0x9B,0x80,
	0x9B,0xC0,
	0x9B,0x80,
	0x9B,0xC0,
	0x9B,0x80,
	0x9B,0xC0,
	0x9B,0x80,
	0x9B,0xC0,
	0x9B,0x80,
	0x9B,0xC0,
	0x9B,0x80,
	0x9B,0xC0,
	0x9B,0x00,

	0x05,0x01,	//sw reset
	0x89,0xFF,	//dis-mute tristate all
	0x05,0x00,
	0x07,0x0C,	//terminator off port0
	0x07,0x00,	//terminator on#videostate to pwrdn
	0x73,0x00,
	0x89,0x80,

	0x20,0x04,
	0x21,0x10,
	0x22,0x80,
	0x23,0x10,
	0x87,0x00,

	//for raw audio
	//0x77,0xa0,
	//0x88,0x10,
        -1,-1
    };

    QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
                 "%s(): multiType(%d)\n",
                 __FUNCTION__,
                 pQLCodecLib->m_InitData.vidDecoderInitData.multiType
                 ));

    if (CObject_IsInitialized(&pVideoDecoder->m_Object))
    {
        QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
                     "CIT6603_InitDevice() Already Initialized. Skipping...\n"
                    ));
      return (QPERR_SUCCESS);
    }

    // save callback address and context
    //
    pVideoDecoder->m_pDeviceCallback = pFuncCallback;
    pVideoDecoder->m_callbackContext = context;

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
	// if has multiple video input types, disable others
	//
    if (MULTIPLE_VIDDECODER_TYPE1 == pQLCodecLib->m_InitData.vidDecoderInitData.multiType) {
    	// power down TVP7002
    	buffer[0] = 0x17;
    	buffer[1] = 0x03;
    	II2C_Write(This->m_pI2C, TVP7002_DEVICE_ADDR/*device address*/, 2/*write size*/, buffer/*write buffer*/);

    	// power down TW9910
    	buffer[0] = 0x03;
    	buffer[1] = 0x06;
    	II2C_Write(This->m_pI2C, TW9910_DEVICE_ADDR/*device address*/, 2/*write size*/, buffer/*write buffer*/);

		DelayMilliSeconds(100);
    }
#endif

    //power ON
    buffer[0] = 0x06;
    buffer[1] = 0x00;
    II2C_Write(This->m_pI2C, IT6603_DEVICE_ADDR, 2/*write size*/, buffer/*write buffer*/);

    //msleep(3);

    //Reset to default
    buffer[0] = 0x05;
    buffer[1] = 0x10;
    II2C_Write(This->m_pI2C, IT6603_DEVICE_ADDR, 2/*write size*/, buffer/*write buffer*/);

    //msleep(3);

    // Initialize IT6603 with the default value
    //
    CIT6603_WriteBlock(This, dwIT6603HDMIRegVals);

	//set registers when detect RAW audio input from Reg 0x9c Bit1
    if (!CIT6603_readRegister(This, 0x9c, &ucValue)) {
		if ((ucValue & 0x02)  == 0x2) {	//compressed
			CIT6603_writeRegister(This, 0x77, 0xa0);
			CIT6603_writeRegister(This, 0x88, 0x10);
		}
    } else {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
			"CIT6603_InitDevice() : Error when checking audio channel status\n"));
    }

    qpStatus = CIT6603_readRegister (This,                        // read IT660x REG04 for reversion
                                     0x04,
                                     &buffer[1]);

    QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
                 "IT660x Rev = %x\n",
                 buffer[1]
                 ));


    if (buffer[1] == 0xa2)                                       // Old version, need HPCP init
    {
        buffer[0] = 0x11;
        buffer[1] = 0x09;
        II2C_Write(This->m_pI2C,
                   IT6603_DEVICE_ADDR,
                   2,              //write size
                   buffer          //write buffer
                  );

        buffer[0] = 0x11;
        buffer[1] = 0x19;
        II2C_Write(This->m_pI2C,
                   IT6603_DEVICE_ADDR,
                   2,              //write size
                   buffer          //write buffer
                  );
    }

    // program the video decoder with settings read from the registry
    //
    QPVideoDecoder_SetVideoSystem(This_p,
                                  pVideoDecoder->m_VidStandard
                                 );
    QPVideoDecoder_SelectVideoSource(This_p,
                                  pVideoDecoder->m_VidSource
                                 );

    QPVideoDecoder_SetBrightness(This_p,
                                 pVideoDecoder->m_nBrightness
                                );
    QPVideoDecoder_SetContrast(This_p,
                               pVideoDecoder->m_nContrast
                              );
    QPVideoDecoder_SetSaturation(This_p,
                                 pVideoDecoder->m_nSaturation
                                );
    QPVideoDecoder_SetHue(This_p,
                          pVideoDecoder->m_nHue
                         );
    QPVideoDecoder_SetSharpness(This_p,
                                pVideoDecoder->m_nSharpness
                               );

    // CObject Init
    //
    CObject_Init(&pVideoDecoder->m_Object);

    return (QPERR_SUCCESS);
}



// IVideoDecoder
//
static EQP_Errors CIT6603_SetVideoRes(IVideoDecoder *This_p,
									  TQP_UINT32 dwWid,
				  					  TQP_UINT32 dwHgt,
				  					  TQP_UINT32 dwScanFmt,
				  					  TQP_UINT32 dwFps
									  )
{
	EQP_Errors		error = QPERR_SUCCESS;
    CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CIT6603			*This = GET_INHERITED_OBJECT(CIT6603, m_VideoDecoder, pVideoDecoder);
	CQLCodecLib		*pQLCodecLib = GET_INHERITED_OBJECT(CQLCodecLib, m_Object, pVideoDecoder->m_Object.m_pParent);
	CQLCodec 		*pQLCodec = pQLCodecLib->m_pMpgCodec;
	TQP_UINT8		value;


    static TQP_UINT32 dwRegValsYuvToYuv[] = {
		0x20,0x04,
		0x21,0x10,
		0x22,0x80,
		0x23,0x10,
		0x87,0x00,
        -1,-1
    };

    static TQP_UINT32 dwRegValsRgbToYuv[] = {
		0x20, 0x06,
		0x3d, 0x4c,
		0x21, 0x00,
		0x22, 0x80,
		0x23, 0x00,
		0x24, 0xb8,
		0x25, 0x05,
		0x26, 0xb4,
		0x27, 0x01,
		0x28, 0x93,
		0x29, 0x00,
		0x2a, 0x49,
		0x2b, 0x3c,
		0x2c, 0x18,
		0x2d, 0x04,
		0x2e, 0x9f,
		0x2f, 0x3f,
		0x30, 0xd9,
		0x31, 0x3c,
		0x32, 0x10,
		0x33, 0x3f,
		0x34, 0x18,
		0x35, 0x04,
        -1,-1
    };

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CIT6603_SetVideoRes() : Video Res Set to %dx %d\n",
				 dwWid,
				 dwHgt
				 ));

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
	if (MULTIPLE_VIDDECODER_TYPE1 == pQLCodecLib->m_InitData.vidDecoderInitData.multiType) {
		TQP_UINT32 reg;

		// configure audio switch
		// low for C85343 (TVP7002/TW9910)
		// high for IT6604

		// configure gpio 12 to output
		QPHCI_RegisterRead(&pQLCodec->m_hci, 0x610, &reg);
		reg = reg | (1 << 12);
		QPHCI_RegisterWrite(&pQLCodec->m_hci, 0x610, reg);

		// gpio 12 to high state
		QPHCI_RegisterRead(&pQLCodec->m_hci, 0x614, &reg);
		reg = reg | (1 << 12);
		QPHCI_RegisterWrite(&pQLCodec->m_hci, 0x614, reg);
	}
#endif

	error = CIT6603_readRegister(This, 0xae, &value);
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
		 "CIT6603_SetVideoRes() : CIT6603_readRegister(0xae) value=0x%08x error=%d\n", value, error));

	if (!error) {
		if ((((value & 0x60) >> 5) == 0x1/*yuv422*/) || (((value & 0x60) >> 5) == 0x2/*yuv444*/)) {
			/* input is yuv422 or yuv444 */
			CIT6603_WriteBlock(This, dwRegValsYuvToYuv);
		} else {
			/* input is rgb */
			CIT6603_WriteBlock(This, dwRegValsRgbToYuv);
		}
	} else {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			 "CIT6603_SetVideoRes() : Error in checking video InfoFrame status\n"));
	}

	//set registers when detect RAW audio input
	error = CIT6603_readRegister(This, 0x9c, &value);
	if (!error) {
		if ((value & 0x02)  == 0x2) {	//compressed
			CIT6603_writeRegister(This, 0x77, 0xa0);
			CIT6603_writeRegister(This, 0x88, 0x10);
		} else {		//default
			CIT6603_writeRegister(This, 0x77, 0x20);
			CIT6603_writeRegister(This, 0x88, 0x00);
		}
	} else {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
			 "CIT6603_SetVideoRes() : Error in checking audio channel status\n"));
	}

	return (error);
}


static EQP_Errors CIT6603_IsHDCPContent(IVideoDecoder *This_p,
										TQP_UINT32 *pnIsHDCP)
{
	EQP_Errors		qpStatus = QPERR_SUCCESS;
    CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CIT6603			*This = GET_INHERITED_OBJECT(CIT6603, m_VideoDecoder, pVideoDecoder);
	TQP_UINT8		ucValue;

#if 1
	*pnIsHDCP = 0;

	qpStatus = CIT6603_readRegister(This, 0x15, &ucValue);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CIT6603_SetTriState(): ****** readRegister failed! status(%d) ******\n",
					 qpStatus
					 ));
    } else {
		if ((ucValue & 0x03)  == 0x3) {
			/*HDCP on*/
			*pnIsHDCP = 1;
		}
    }

#else
	/* test code */
	static unsigned int unCount = 0;
	static unsigned int unSwitch = 0;

	if (unCount++ >= 60) {
		unCount = 0;
		unSwitch = unSwitch ^ 0x1;
	}

	*pnIsHDCP = unSwitch;
#endif

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
			     "CIT6603_IsHDCPContent(): *pnIsHDCP(%d) return(%d)\n",
			     *pnIsHDCP,
			     qpStatus
			     ));

	return (qpStatus);
}


static EQP_Errors CIT6603_SetTriState(IVideoDecoder *This_p,
									  TQP_UINT32 nTriState
									  )
{
	EQP_Errors		qpStatus = QPERR_SUCCESS;
    CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CIT6603			*This = GET_INHERITED_OBJECT(CIT6603, m_VideoDecoder, pVideoDecoder);
	TQP_UINT8		ucValue;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
			     "%CIT6603_SetTriState(): enter, nTriState(%d)\n", nTriState));

	if (nTriState) {
		ucValue = 0xff;
	} else {
		ucValue = 0x80;
	}

	qpStatus = CIT6603_writeRegister(This, 0x89, ucValue);
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CIT6603_SetTriState(): ****** writeRegister failed! status(%d) ******\n",
					 qpStatus
					 ));
	}

	return (qpStatus);
}



// constructor
//
CIT6603 * CIT6603_Constructor(CIT6603 *pIT6603,
								CObject *pParent,
								TQP_UINT32 dwAttributes,
								ITuner *pTuner,
								ITVAudio *pTvAudio,
								II2C *pI2C,
								TQP_UINT8 cI2CDevAddr
								)
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CIT6603_Constructor()\n"
				 ));

	if (pIT6603)
	{
		CVideoDecoder	*pVideoDecoder;

		pVideoDecoder = CVideoDecoder_Constructor(&pIT6603->m_VideoDecoder,
												  pParent,
												  dwAttributes,
												  pTuner,
												  pTvAudio
												  );

		if (!pVideoDecoder)
		{
			pIT6603 = QP_NULL;
		}
		else
		{
			// fill in the function table
			//

			// IGenericDevice
			pIT6603->m_VideoDecoder.m_iVideoDecoder.InitDevice = CIT6603_InitDevice;

			// IVideoDecoder
			pIT6603->m_VideoDecoder.m_iVideoDecoder.SetVideoRes = CIT6603_SetVideoRes;

			// to filter out HDCP content
			pIT6603->m_VideoDecoder.m_iVideoDecoder.IsHDCPContent = CIT6603_IsHDCPContent;
			pIT6603->m_VideoDecoder.m_iVideoDecoder.SetTriState = CIT6603_SetTriState;


			// initialize members
			//
			pIT6603->m_pI2C = pI2C;
			pIT6603->m_cI2CDevAddr = cI2CDevAddr;
		}
	}

	return (pIT6603);
}



// destructor
//
void CIT6603_Destructor(CIT6603 *This)
{
    //TQP_UINT8     buffer[2];
    //TQP_UINT8     ucRegAddr;
    //TQP_UINT8     ucValue;

    QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CIT6603_Destructor()\n"
				 ));


    QPTMDelayMilliS(10);   // delay 10ms

    CVideoDecoder_Destructor(&This->m_VideoDecoder);
}

