///////////////////////////////////////////////////////////////////////////////
//        
//
// cov5642.h
//
// Description: 
//
//  ov5642 declaration
// 
// History:
//	September 14, 2010 created
//
// Authors: Xing Pan
//
///////////////////////////////////////////////////////////////////////////////


#ifndef __COV5642_H__
#define __COV5642_H__

#include "cviddecoder.h"
#include "qpi2c.h"
#include "cthread.h"

#ifdef __cplusplus
extern "C" {
#endif


/////////////////////////////////////////////////////////////////////////////
//
//

#define OV5642_DEVICE_ADDR		0x3c

#define OV5642_POWER_GPIOBIT	0			//gpio0
#define OV5642_RESET_GPIOBIT	1			//gpio1


#define  OV5642_CMD_MAIN		0x3024
#define  OV5642_CMD_TAG	    	0x3025
#define  OV5642_CMD_PARA3		0x5082
#define  OV5642_CMD_PARA2		0x5083
#define  OV5642_CMD_PARA1		0x5084
#define  OV5642_CMD_PARA0		0x5085
#define  OV5642_STA_ZONE		0x3026
#define  OV5642_STA_FOCUS		0x3027
                            	
#define  OV5642_S_FIRWARE 		0xFF 		//Firmware is downloaded and not run.
#define  OV5642_S_STARTUP 		0xFA 		//Firmware is initializing.
#define  OV5642_S_ERROR 		0xFE 		//Unknown Firmware state
#define  OV5642_S_DRVICERR 		0xEE 		//Driver IC is not found.
#define  OV5642_S_IDLE 			0x00 		//Idle state, lens is located at the furthest position.
#define  OV5642_S_FOCUSING 		0x01 		//Auto Focus is running.
#define  OV5642_S_FOCUSED 		0x02 		//Auto Focus is stopped.
#define  OV5642_S_CAPTURE 		0x12 		//Read to capture
#define  OV5642_S_STEP 			0x20 		//Manual mode


#define  OV5642_MAINCMD_ENABLE_OVERLAY		0x01
#define  OV5642_MAINCMD_DISABLE_OVERLAY		0x02
#define  OV5642_MAINCMD_SINGLE_FOCUS_MODE	0x03
#define  OV5642_MAINCMD_CONST_FOCUS_MODE	0x04
#define  OV5642_MAINCMD_STEP_MODE			0x05
#define  OV5642_MAINCMD_PAUSED				0x06
#define  OV5642_MAINCMD_RETURN_IDEL			0x08
#define  OV5642_MAINCMD_SET_ZONE_MODE		0x10
#define  OV5642_MAINCMD_UPDATE_ZONE			0x12
#define  OV5642_MAINCMD_SET_MOTOR_MODE		0x20
#define  OV5642_MAINCMD_SET_SCAN_MODE		0x30

#define RAW_MJPEG_FRAMERATE		((1<<VIDEO_DATATYPE_RAW)|(1<<VIDEO_DATATYPE_JPEG))
#define RAWDATA_FRAMERATE		((1<<VIDEO_DATATYPE_RAW))
#define MJPEG_FRAMERATE			((1<<VIDEO_DATATYPE_JPEG))



/////////////////////////////////////////////////////////////////////////////
//
//
typedef enum _OV5642_ResCategory
{   
    OV5642_RC_UNDEFINE	= 0,		
    OV5642_RC_VGA	,				
    OV5642_RC_720P	,				
    OV5642_RC_1080P	,			
    OV5642_RC_5MEGA	,
    OV5642_RC_NUM			
} OV5642_ResCategory, *POV5642_ResCategory;

typedef struct _OV5642_PLLDIVIDER
{
	TQP_UINT8 				divs;
	TQP_UINT8 				divm;
	TQP_UINT8 				divp;
}OV5642_PLLDIVIDER, * POV5642_PLLDIVIDER;

typedef struct _OV5642_FRAMERATETABLE
{
	OV5642_ResCategory 		category;
	TQP_UINT8				supportflag;
	TQP_UINT8 				framerate;
	OV5642_PLLDIVIDER 		divider;
}OV5642_FRAMERATETABLE;

typedef struct COV5642 
{
	// public CVideoDecoder
	//
	CVideoDecoder			m_VideoDecoder;

	// COV5642
	II2C					*m_pI2C;				// I2C interface
	TQP_UINT8				m_cI2CDevAddr;			// i2c device address
	OV5642_ResCategory		m_TarResCategory;		//target resolution category
	OV5642_ResCategory		m_CurResCategory;		//resolution category
	VIDEO_DATATYPE			m_OvDataType;			
	TQP_OSMEvtgrpID			m_EvtEmuTask;
	TQP_OSMEvtgrpID			m_EvtEmuTaskReply;

} COV5642;


COV5642 * COV5642_Constructor(COV5642 *pOV5642, 
							  CObject *pParent,
							  TQP_UINT32 dwAttributes,
							  ITuner *pTuner,
							  ITVAudio *pTvAudio,
							  II2C *pI2C,
							  TQP_UINT8 cI2CDevAddr
							  );
void COV5642_Destructor(COV5642 *This);

#ifdef __cplusplus
}
#endif

#endif //__CADI7441_H__


