///////////////////////////////////////////////////////////////////////////////
//        
//
// Cpe1005.c
//
// Description: 
//
//  AcuteLogic camera module PE1005 implementation
// 
// History:
//      April 30, 2008 created
//
// Authors: Andy Yu
//
///////////////////////////////////////////////////////////////////////////////

#include "cpe1005.h"
#include "qpdbgmgr.h"
#include "qptimemgr.h"
#include "util.h"
//#include <stdio.h>
/////////////////////////////////////////////////////////////////////////////
//
//

#define	PE1005_BRIGHTNESS_MIN				0x00
#define	PE1005_BRIGHTNESS_MAX				0xFF
#define	PE1005_BRIGHTNESS_DEFAULT			0x80

#define	PE1005_CONTRAST_MIN				0x00
#define	PE1005_CONTRAST_MAX				0x7F
#define	PE1005_CONTRAST_DEFAULT				0x44

#define	PE1005_SATURATION_MIN				0x00
#define	PE1005_SATURATION_MAX				0x7F
#define	PE1005_SATURATION_DEFAULT			0x40

#define	PE1005_HUE_MIN					(-128)	// 0x80
#define	PE1005_HUE_MAX					(+127)	// 0x7F
#define	PE1005_HUE_DEFAULT				0x00

#define	PE1005_SHARPNESS_MIN				0
#define	PE1005_SHARPNESS_MAX				15
#define	PE1005_SHARPNESS_DEFAULT			8

extern TQP_DbgMCompID g_QPPeripheralDBGCompID;

/////////////////////////////////////////////////////////////////////////////
//
//

/////////////////////////////////////////////////////////////////////////////
//
//

static EQP_Errors CPE1005_InitIO(CPE1005 * This)
{
	EQP_Errors qpStatus;
	TQP_UINT8 buffer[2];

	// on QL300 EVB, use PCA9535 to control IO pins of PE1005
	// clear all output pins
	This->m_PCA9535Gout0 = 0x00;
	buffer[0] = 0x02;
	buffer[1] = This->m_PCA9535Gout0;
	qpStatus = II2C_Write(This->m_pI2C, 0x24,	//device address
			      2,	//write size
			      buffer	//write buffer
	    );
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_InitIO() ****** II2C_Write() failed! ******\n"));
		return (QPERR_FAIL);
	}

	This->m_PCA9535Gout1 = 0x00;
	buffer[0] = 0x03;
	buffer[1] = This->m_PCA9535Gout1;
	qpStatus = II2C_Write(This->m_pI2C, 0x24,	//device address
			      2,	//write size
			      buffer	//write buffer
	    );
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_InitIO() ****** II2C_Write() failed! ******\n"));
		return (QPERR_FAIL);
	}
	// configure in/out
	// PON      = G0.0   (output to PE1005)
	// ShutDown = G1.3   (output to PE1005)
	// HRSTX    = G1.4   (output to PE1005)
	// INT_IN   = G1.0   (output to PE1005)
	// POWER    = G1.7   (output)
	// INT_OUT  = G0.1   (input from PE1005)
	buffer[0] = 0x06;
	buffer[1] = ~(PE1005_PON);
	qpStatus = II2C_Write(This->m_pI2C, 0x24,	//device address
			      2,	//write size
			      buffer	//write buffer
	    );
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_InitIO() ****** II2C_Write() failed! ******\n"));
		return (QPERR_FAIL);
	}

	buffer[0] = 0x07;
	buffer[1] = (TQP_UINT8)(~(PE1005_INT_IN | PE1005_SHUTDOWN | PE1005_HRSTX | PE1005_POWER));
	qpStatus = II2C_Write(This->m_pI2C, 0x24,	//device address
			      2,	//write size
			      buffer	//write buffer
	    );
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_InitIO() ****** II2C_Write() failed! ******\n"));
		return (QPERR_FAIL);
	}

	return (QPERR_SUCCESS);
}

static EQP_Errors CPE1005_POWER_On(CPE1005 * This)
{
	EQP_Errors qpStatus;
	TQP_UINT8 buffer[2];

	// on QL300 EVB, use PCA9535 to control IO pins of PE1005
	// POWER = G1.7 (trun on PE1005 4V & 2.8V)
	This->m_PCA9535Gout1 |= PE1005_POWER;
	buffer[0] = 0x03;
	buffer[1] = This->m_PCA9535Gout1;
	qpStatus = II2C_Write(This->m_pI2C, 0x24,	//device address
			      2,	//write size
			      buffer	//write buffer
	    );

	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_POWER_On() ****** II2C_Write() failed! ******\n"));
		return (QPERR_FAIL);
	}

	return (QPERR_SUCCESS);
}

//static 
EQP_Errors CPE1005_POWER_Off(CPE1005 * This)
{
	EQP_Errors qpStatus;
	TQP_UINT8 buffer[2];

	// on QL300 EVB, use PCA9535 to control IO pins of PE1005
	// POWER = G1.7 (turn off PE1005 power)
	This->m_PCA9535Gout1 &= ~(PE1005_POWER);
	buffer[0] = 0x03;
	buffer[1] = This->m_PCA9535Gout1;
	qpStatus = II2C_Write(This->m_pI2C, 0x24,	//device address
			      2,	//write size
			      buffer	//write buffer
	    );

	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_POWER_Off() ****** II2C_Write() failed! ******\n"));
		return (QPERR_FAIL);
	}

	return (QPERR_SUCCESS);
}

static EQP_Errors CPE1005_PON_On(CPE1005 * This)
{
	EQP_Errors qpStatus;
	TQP_UINT8 buffer[2];

	// on QL300 EVB, use PCA9535 to control IO pins of PE1005
	// PON = G0.0 (output to PE1005)
	This->m_PCA9535Gout0 |= PE1005_PON;
	buffer[0] = 0x02;
	buffer[1] = This->m_PCA9535Gout0;
	qpStatus = II2C_Write(This->m_pI2C, 0x24,	//device address
			      2,	//write size
			      buffer	//write buffer
	    );

	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_PON_On() ****** II2C_Write() failed! ******\n"));
		return (QPERR_FAIL);
	}

	return (QPERR_SUCCESS);
}

static EQP_Errors CPE1005_PON_Off(CPE1005 * This)
{
	EQP_Errors qpStatus;
	TQP_UINT8 buffer[2];

	// on QL300 EVB, use PCA9535 to control IO pins of PE1005
	// PON = G0.0 (output to PE1005)
	This->m_PCA9535Gout0 &= ~(PE1005_PON);
	buffer[0] = 0x02;
	buffer[1] = This->m_PCA9535Gout0;
	qpStatus = II2C_Write(This->m_pI2C, 0x24,	//device address
			      2,	//write size
			      buffer	//write buffer
	    );

	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_PON_Off() ****** II2C_Write() failed! ******\n"));
		return (QPERR_FAIL);
	}

	return (QPERR_SUCCESS);
}

static EQP_Errors CPE1005_ShutDown_On(CPE1005 * This)
{
	EQP_Errors qpStatus;
	TQP_UINT8 buffer[2];

	// on QL300 EVB, use PCA9535 to control IO pins of PE1005
	// ShutDown = G1.3 (output to PE1005)
	This->m_PCA9535Gout1 |= PE1005_SHUTDOWN;
	buffer[0] = 0x03;
	buffer[1] = This->m_PCA9535Gout1;
	qpStatus = II2C_Write(This->m_pI2C, 0x24,	//device address
			      2,	//write size
			      buffer	//write buffer
	    );

	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_ShutDown_On() ****** II2C_Write() failed! ******\n"));
		return (QPERR_FAIL);
	}

	return (QPERR_SUCCESS);
}

static EQP_Errors CPE1005_ShutDown_Off(CPE1005 * This)
{
	EQP_Errors qpStatus;
	TQP_UINT8 buffer[2];

	// on QL300 EVB, use PCA9535 to control IO pins of PE1005
	// ShutDown = G1.3 (output to PE1005)
	This->m_PCA9535Gout1 &= ~(PE1005_SHUTDOWN);
	buffer[0] = 0x03;
	buffer[1] = This->m_PCA9535Gout1;
	qpStatus = II2C_Write(This->m_pI2C, 0x24,	//device address
			      2,	//write size
			      buffer	//write buffer
	    );

	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_ShutDown_Off() ****** II2C_Write() failed! ******\n"));
		return (QPERR_FAIL);
	}

	return (QPERR_SUCCESS);
}

static EQP_Errors CPE1005_HRSTX_On(CPE1005 * This)
{
	EQP_Errors qpStatus;
	TQP_UINT8 buffer[2];

	// on QL300 EVB, use PCA9535 to control IO pins of PE1005
	// HRSTX = ? (output to PE1005)
	This->m_PCA9535Gout1 |= PE1005_HRSTX;
	buffer[0] = 0x03;
	buffer[1] = This->m_PCA9535Gout1;
	qpStatus = II2C_Write(This->m_pI2C, 0x24,	//device address
			      2,	//write size
			      buffer	//write buffer
	    );

	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_ShutDown_Off() ****** II2C_Write() failed! ******\n"));
		return (QPERR_FAIL);
	}

	return (QPERR_SUCCESS);
}

static EQP_Errors CPE1005_HRSTX_Off(CPE1005 * This)
{
	EQP_Errors qpStatus;
	TQP_UINT8 buffer[2];

	// on QL300 EVB, use PCA9535 to control IO pins of PE1005
	// HRSTX = ? (output to PE1005)
	This->m_PCA9535Gout1 &= ~(PE1005_HRSTX);
	buffer[0] = 0x03;
	buffer[1] = This->m_PCA9535Gout1;
	qpStatus = II2C_Write(This->m_pI2C, 0x24,	//device address
			      2,	//write size
			      buffer	//write buffer
	    );

	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_ShutDown_Off() ****** II2C_Write() failed! ******\n"));
		return (QPERR_FAIL);
	}

	return (QPERR_SUCCESS);
}

static EQP_Errors CPE1005_INT_IN_On(CPE1005 * This)
{
	EQP_Errors qpStatus;
	TQP_UINT8 buffer[2];

	// on QL300 EVB, use PCA9535 to control IO pins of PE1005
	// INT_IN = G1.0 (output to PE1005)
	This->m_PCA9535Gout1 |= PE1005_INT_IN;
	buffer[0] = 0x03;
	buffer[1] = This->m_PCA9535Gout1;
	qpStatus = II2C_Write(This->m_pI2C, 0x24,	//device address
			      2,	//write size
			      buffer	//write buffer
	    );

	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_INT_IN_On() ****** II2C_Write() failed! ******\n"));
		return (QPERR_FAIL);
	}

	return (QPERR_SUCCESS);
}

static EQP_Errors CPE1005_INT_IN_Off(CPE1005 * This)
{
	EQP_Errors qpStatus;
	TQP_UINT8 buffer[2];

	// on QL300 EVB, use PCA9535 to control IO pins of PE1005
	// INT_IN = G1.0 (output to PE1005)
	This->m_PCA9535Gout1 &= ~(PE1005_INT_IN);
	buffer[0] = 0x03;
	buffer[1] = This->m_PCA9535Gout1;
	qpStatus = II2C_Write(This->m_pI2C, 0x24,	//device address
			      2,	//write size
			      buffer	//write buffer
	    );

	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_INT_IN_Off() ****** II2C_Write() failed! ******\n"));
		return (QPERR_FAIL);
	}

	return (QPERR_SUCCESS);
}

static EQP_Errors CPE1005_INT_OUT_Check(CPE1005 * This)
{
	EQP_Errors qpStatus;
	TQP_UINT8 ucRegAddr;
	TQP_UINT8 ucValue;

	// on QL300 EVB, use PCA9535 to control IO pins of PE1005
	// INT_OUT = G0.1 (input from PE1005)
	ucRegAddr = 0x00;
	qpStatus = II2C_WriteThenRead(This->m_pI2C, 0x24,	//device address
				      1,	//write size
				      &ucRegAddr,	//write buffer
				      1,	//read size
				      &ucValue	//read buffer
	    );

	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_INT_OUT_Check() ****** II2C_WriteThenRead() failed! ******\n"));
		return (QPERR_FAIL);
	}

	if (ucValue & PE1005_INT_OUT) {
		// received INT_OUT
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
			      "CPE1005_INT_OUT_Check() ****** got INT_OUT! ******\n"));
		return (QPERR_SUCCESS);
	}

	return (QPERR_FAIL);
}

/////////////////////////////////////////////////////////////////////////////
//
//

static EQP_Errors CPE1005_SetCheckSum(CPE1005 * This)
{
	TQP_UINT16 usPacketSum = 0;
	TQP_UINT8 i = 0;

	for (i = 0; i < 42; i++) {
		usPacketSum += This->m_CmdPacket.Read[i];
	}

	This->m_CmdPacket.stPakAll.usPakChkSum = 0xFFFF - usPacketSum;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
		      "CPE1005_SetCheckSum() %04x %04x %04x\n",
		      This->m_CmdPacket.stPakAll.usPakChkSum, usPacketSum, (0xFFFF - usPacketSum)
		     ));

	return QPERR_SUCCESS;
}

static EQP_Errors CPE1005_ChkCheckSum(CPE1005 * This)
{
	TQP_UINT16 usPacketSum = 0;
	TQP_UINT8 i = 0;

	for (i = 0; i < 42; i++) {
		usPacketSum += This->m_ResPacket.Read[i];
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
		      "CPE1005_ChkCheckSum() %04x %04x %04x\n",
		      This->m_ResPacket.stPakAll.usPakChkSum, usPacketSum, (0xFFFF - usPacketSum)
		     ));

	if (This->m_ResPacket.stPakAll.usPakChkSum != (0xFFFF - usPacketSum)) {
		return (QPERR_FAIL);
	}

	return (QPERR_SUCCESS);
}

static void DebugPrintPacketData(TQP_UINT8 * data, TQP_INT32 length, char *title)
{
	int i;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
		      "%s\n",
		      title
		     ));

	for (i = 0; i < length; i++) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
		      "%02X ",
		      data[i]
		     ));
		if (((i + 1) % 10) == 0) {
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
			      "\n"
		     	));
		}
	}
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
		"\n"
     	));
}

static EQP_Errors CPE1005_SendCommand(CPE1005 * This)
{
	EQP_Errors qpStatus;
	TQP_UINT8 i = 0, retry = 0;
	TQP_UINT8 ucHighByte = 0, ucLowByte = 0;
	TQP_UINT8 aI2C[44];

	QPTMDelayMilliS(30);	// delay 20 ms

	CPE1005_SetCheckSum(This);

	// Andy test endian
	//This->m_CmdPacket.stPakAll.stCmdPrm.stDVCMode.RSV1 = 0x1234;

      SEND_COMMAND:

#if 1
	// endian
	for (i = 0; i < 44; i++) {
		ucHighByte = This->m_CmdPacket.Read[i];
		ucLowByte = This->m_CmdPacket.Read[(i + 1)];
		aI2C[i++] = ucLowByte;
		aI2C[i] = ucHighByte;
	}
#endif

	DebugPrintPacketData(aI2C, 44, "Send command packet:");

	qpStatus = II2C_Write(This->m_pI2C, This->m_cI2CDevAddr,	//device address
			      44,	//write size
			      aI2C	//write buffer
	    );
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "II2C_Write() failed to send command to PE1005!\n"));
		return (QPERR_FAIL);
	}

RETRY_COMMAND:

	QPTMDelayMilliS(5);	// delay 1 ms

#if 0				// Andy test: for EVB
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
		      "II2C_Read() AAA! ******\n"));
	i = 0;
	qpStatus = II2C_WriteThenRead(This->m_pI2C, This->m_cI2CDevAddr,	//device address
				      1,	//write size
				      &i,	//write buffer
				      44,	//read size
				      aI2C	//read buffer
	    );
#else // for Q.Cam
	qpStatus = II2C_Read(This->m_pI2C, This->m_cI2CDevAddr,	//device address
			     44,	//write size
			     aI2C	//read buffer
	    );
#endif
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "II2C_Read() failed to get response from PE1005!\n"));
		return (QPERR_FAIL);
	}
#if 1
	// endian
	for (i = 0; i < 44; i++) {
		ucHighByte = aI2C[i];
		ucLowByte = aI2C[(i + 1)];
		This->m_ResPacket.Read[i++] = ucLowByte;
		This->m_ResPacket.Read[i] = ucHighByte;
	}
#endif

	DebugPrintPacketData(aI2C, 44, "Response Packet:");

	qpStatus = CPE1005_ChkCheckSum(This);
	if (!QP_SUCCESS(qpStatus)) {
		// PE1005 sends wrong response packet
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "checksum error on response packet from PE1005!\n"));
		//return (QPERR_FAIL);
	}

	if (This->m_ResPacket.stPakAll.ucRetry == 0x01) {
		// PE1005 needs retry
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "Cretry?\n"));
		goto RETRY_COMMAND;	// retry
	}
	// response packet is valid right now

	if (This->m_ResPacket.stPakAll.stPakHeader.stPakHead.PRS == PRS_PCS_ERROR) {
		// PE1005 reports checksum error on command packet
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "II2C_Read() got checksum error from PE1005! %x\n",
			      This->m_ResPacket.stPakAll.stPakHeader.stPakHead.PRS));
#if 1
		if (retry < 6) {
			retry++;
			goto SEND_COMMAND;
		} else {
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
				      "retry out of checksum error from PE1005! %x\n",
				      This->m_ResPacket.stPakAll.stPakHeader.stPakHead.PRS));
			return (QPERR_FAIL);
		}
#endif
	}
#if 1				// Andy test: just report but don't care it according to ADL
	if ((This->m_ResPacket.stPakAll.stCmdHeader.Read & RET_CMDEXEC_MASK) != RET_CMDEXEC_OK) {
		// PE1005 reports something wrong
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "II2C_Read() got command error (0x%x) from PE1005!\n",
			      This->m_ResPacket.stPakAll.stCmdHeader.Read));
		//return (QPERR_FAIL);
	}

	if (This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.ALT !=
	    This->m_ResPacket.stPakAll.stPakHeader.stPakHead.ALT) {
		// PE1005 ALT bit wrong
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "II2C_Read() got ALT error from PE1005!\n"));
		//return (QPERR_FAIL);
	}
#endif

	// host inversed PE1005's ALT bit when protocol is completed
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.ALT =
	    !(This->m_ResPacket.stPakAll.stPakHeader.stPakHead.ALT);

	return (QPERR_SUCCESS);
}

/////////////////////////////////////////////////////////////////////////////
//
//

// IVideoDecoder
//

// PE1005 Modec Change Sequence
static EQP_Errors CPE1005_SeqModeChange(IVideoDecoder * This_p)
{
	CVideoDecoder *pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CPE1005 *This = GET_INHERITED_OBJECT(CPE1005, m_VideoDecoder, pVideoDecoder);
	EQP_Errors qpStatus;
	TQP_UINT8 i = 0, retry = 0;

#if 1				// Andy test: set DVC mode
	// setup camera mode as idle
	for (i = 2; i < 44; i++) {
		This->m_CmdPacket.Read[i] = 0;
	}

	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.Reserved2 = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.PRS = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.Reserved1 = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.SR = 0;

	This->m_CmdPacket.stPakAll.ucRetry = 0;
	This->m_CmdPacket.stPakAll.Reserved1 = 0;

	This->m_CmdPacket.stPakAll.stCmdHeader.Read = 0x0202;
	This->m_CmdPacket.stPakAll.usCmdPrmSize = 0x0002;

	This->m_CmdPacket.stPakAll.stCmdPrm.Read[0] = 0x01;	// CAMMD = DVC mode
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[1] = 0x00;

	qpStatus = CPE1005_SendCommand(This);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_SeqModeChange() failed to set DVC mode to PE1005!\n"));
		return (QPERR_FAIL);
	}
#endif

#if 0				// Andy test: set DVC mode provided by ADL
	// setup camera mode as idle
	for (i = 2; i < 44; i++) {
		This->m_CmdPacket.Read[i] = 0;
	}

	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.Reserved2 = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.PRS = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.Reserved1 = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.SR = 0;

	This->m_CmdPacket.stPakAll.ucRetry = 0;
	This->m_CmdPacket.stPakAll.Reserved1 = 0;

	This->m_CmdPacket.stPakAll.stCmdHeader.Read = 0x0200;
	This->m_CmdPacket.stPakAll.usCmdPrmSize = 0x0020;

	This->m_CmdPacket.stPakAll.stCmdPrm.Read[0] = 0xF8;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[1] = 0x34;

	This->m_CmdPacket.stPakAll.stCmdPrm.Read[2] = 0x00;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[3] = 0x60;

	This->m_CmdPacket.stPakAll.stCmdPrm.Read[6] = 0x26;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[7] = 0x42;

	This->m_CmdPacket.stPakAll.stCmdPrm.Read[14] = 0x83;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[15] = 0x00;

	This->m_CmdPacket.stPakAll.stCmdPrm.Read[16] = 0x50;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[17] = 0x52;

	qpStatus = CPE1005_SendCommand(This);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_SeqModeChange() ****** CPE1005_SendCommand() failed to set DVC mode to PE1005! ******\n"));
		return (QPERR_FAIL);
	}
#endif

	QPTMDelayMilliS(20);	// delay 20 ms

	// do mode change
	for (i = 2; i < 44; i++) {
		This->m_CmdPacket.Read[i] = 0;
	}

	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.Reserved2 = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.PRS = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.Reserved1 = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.SR = 0;

	This->m_CmdPacket.stPakAll.ucRetry = 0;
	This->m_CmdPacket.stPakAll.Reserved1 = 0;

	This->m_CmdPacket.stPakAll.stCmdHeader.Read = 0x0018;
	This->m_CmdPacket.stPakAll.usCmdPrmSize = 0x0002;

	This->m_CmdPacket.stPakAll.stCmdPrm.Read[0] = 0x01;	// INT: mode change
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[1] = 0x00;

	qpStatus = CPE1005_SendCommand(This);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_SeqModeChange() failed to change mode to PE1005!\n"));
		return (QPERR_FAIL);
	} else {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
			      "CPE1005_SeqModeChange() send command OK! ******\n"));
	}

	// set INT_IN
	qpStatus = CPE1005_INT_IN_On(This);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_SeqModeChange() failed to set INT_IN of PE1005!\n"));
		return (QPERR_FAIL);
	} else {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
			      "CPE1005_SeqModeChange() CPE1005_INT_IN_On OK!\n"));
	}

CHECK_INT_OUT:
	// check INT_OUT
	qpStatus = CPE1005_INT_OUT_Check(This);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
			      "CPE1005_SeqModeChange() checking INT_OUT from PE1005!\n"));
		if (retry++ > 20) {
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
				      "CPE1005_SeqModeChange() failed to check INT_OUT from PE1005!\n"));
			return (QPERR_FAIL);
		}
		QPTMDelayMilliS(500);	// check period 500 ms
		goto CHECK_INT_OUT;
	}
	// clear INT_IN
	qpStatus = CPE1005_INT_IN_Off(This);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_SeqModeChange() failed to clear INT_IN of PE1005!\n"));
		return (QPERR_FAIL);
	}

	return (QPERR_SUCCESS);
}

// PE1005 Shutdown Camera System Sequence
//static 
EQP_Errors CPE1005_SeqShutdownCamera(IVideoDecoder * This_p)
{
	CVideoDecoder *pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CPE1005 *This = GET_INHERITED_OBJECT(CPE1005, m_VideoDecoder, pVideoDecoder);
	EQP_Errors qpStatus;
	TQP_UINT8 i = 0, retry = 0;

	// setup camera mode as idle
	for (i = 2; i < 44; i++) {
		This->m_CmdPacket.Read[i] = 0;
	}

	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.Reserved2 = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.PRS = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.Reserved1 = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.SR = 0;

	This->m_CmdPacket.stPakAll.ucRetry = 0;
	This->m_CmdPacket.stPakAll.Reserved1 = 0;

	This->m_CmdPacket.stPakAll.stCmdHeader.Read = 0x0202;
	This->m_CmdPacket.stPakAll.usCmdPrmSize = 0x0002;

	This->m_CmdPacket.stPakAll.stCmdPrm.Read[0] = 0x00;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[1] = 0x00;	// CAMMD = IDLE mode

	qpStatus = CPE1005_SendCommand(This);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_SeqShutdownCamera() ****** CPE1005_SendCommand() failed to change mode to PE1005! ******\n"));
		return (QPERR_FAIL);
	}
	// do mode change
	for (i = 2; i < 44; i++) {
		This->m_CmdPacket.Read[i] = 0;
	}

	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.Reserved2 = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.PRS = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.Reserved1 = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.SR = 0;

	This->m_CmdPacket.stPakAll.ucRetry = 0;
	This->m_CmdPacket.stPakAll.Reserved1 = 0;

	This->m_CmdPacket.stPakAll.stCmdHeader.Read = 0x0018;
	This->m_CmdPacket.stPakAll.usCmdPrmSize = 0x0002;

	This->m_CmdPacket.stPakAll.stCmdPrm.Read[0] = 0x00;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[1] = 0x01;	// INT: mode change

	qpStatus = CPE1005_SendCommand(This);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_SeqShutdownCamera() ****** CPE1005_SendCommand() failed to change mode to PE1005! ******\n"));
		return (QPERR_FAIL);
	}
	// set INT_IN
	qpStatus = CPE1005_INT_IN_On(This);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_SeqModeChange() ****** CPE1005_INT_IN_On() failed to set INT_IN of PE1005! ******\n"));
		return (QPERR_FAIL);
	} else {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
			      "CPE1005_SeqModeChange() CPE1005_INT_IN_On OK! ******\n"));
	}

      CHECK_INT_OUT:
	// check INT_OUT
	qpStatus = CPE1005_INT_OUT_Check(This);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
			      "CPE1005_SeqModeChange() ****** checking INT_OUT from PE1005! ******\n"));
		if (retry++ > 20) {
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
				      "CPE1005_SeqModeChange() ****** CPE1005_INT_OUT_Check() failed to check INT_OUT from PE1005! ******\n"));
			return (QPERR_FAIL);
		}
		QPTMDelayMilliS(500);	// check period 500 ms
		goto CHECK_INT_OUT;
	}
	// clear INT_IN
	qpStatus = CPE1005_INT_IN_Off(This);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_SeqModeChange() ****** CPE1005_INT_IN_Off() failed to clear INT_IN of PE1005! ******\n"));
		return (QPERR_FAIL);
	}

	return (QPERR_SUCCESS);
}

// PE1005 Get Firmware Version
//static
EQP_Errors CPE1005_GetFWVersion(IVideoDecoder * This_p)
{
	CVideoDecoder *pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CPE1005 *This = GET_INHERITED_OBJECT(CPE1005, m_VideoDecoder, pVideoDecoder);
	EQP_Errors qpStatus;
	TQP_UINT16 usType = 0, usVer = 0, usYear = 0, usDate = 0;
	TQP_UINT8 i = 0;

	// initialize 44-byte command packet
	for (i = 2; i < 44; i++) {
		This->m_CmdPacket.Read[i] = 0;
	}

	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.Reserved2 = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.PRS = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.Reserved1 = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.SR = 0;

	This->m_CmdPacket.stPakAll.ucRetry = 0;
	This->m_CmdPacket.stPakAll.Reserved1 = 0;

	This->m_CmdPacket.stPakAll.stCmdHeader.Read = 0x0015;
	This->m_CmdPacket.stPakAll.usCmdPrmSize = 0x0000;

	qpStatus = CPE1005_SendCommand(This);

	// firmware information
	usType =
	    ((TQP_UINT16) This->m_ResPacket.Read[4] << 8) + (TQP_UINT16) This->m_ResPacket.Read[5];
	usVer =
	    ((TQP_UINT16) This->m_ResPacket.Read[6] << 8) + (TQP_UINT16) This->m_ResPacket.Read[7];
	usYear =
	    ((TQP_UINT16) This->m_ResPacket.Read[8] << 8) + (TQP_UINT16) This->m_ResPacket.Read[9];
	usDate =
	    ((TQP_UINT16) This->m_ResPacket.Read[10] << 8) +
	    (TQP_UINT16) This->m_ResPacket.Read[11];

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
		      "CPE1005_CmdGetFWVersion() ****** TYPE(%04x) VER(%04x) YEAR(%04x) DATE(%04x)! ******\n",
		      usType, usVer, usYear, usDate));

	return (qpStatus);
}

static EQP_Errors CPE1005_SetVideoRes(IVideoDecoder * This_p, VIDEO_STANDARD standard)
{
	CVideoDecoder *pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CPE1005 *This = GET_INHERITED_OBJECT(CPE1005, m_VideoDecoder, pVideoDecoder);

	EQP_Errors qpStatus;
	TQP_UINT8 i = 0, retry = 0;

//return (QPERR_SUCCESS);

	if ((VIDEO_STANDARD_720P != standard) && (VIDEO_STANDARD_1080I != standard)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_SetVideoRes() Unknown Video Standard: %d \n", standard));
		return (QPERR_PARMS);
	}
	// initialize 44-byte command packet
	for (i = 2; i < 44; i++) {
		This->m_CmdPacket.Read[i] = 0;
	}

	// set frame rate
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.Reserved2 = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.PRS = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.Reserved1 = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.SR = 0;

	This->m_CmdPacket.stPakAll.ucRetry = 0;
	This->m_CmdPacket.stPakAll.Reserved1 = 0;

	This->m_CmdPacket.stPakAll.stCmdHeader.Read = 0x0200;
	This->m_CmdPacket.stPakAll.usCmdPrmSize = 0x0020;

	// 720 60P:
	// 00 00 00 00 02 00 00 20 - 2D F8 40 00 00 51 52 66
	// 00 54 00 00 00 00 00 EA - 92 A0 00 00 00 00 00 00
	// 00 00 00 00 00 00 00 00 - 00 00 FA FF                                                                                                            

	// 720 60P: old settings
	// 00 00 00 00 02 00 00 20 - 2D F8 00 00 00 00 42 26 
	// 00 00 80 5D 00 05 3B 83 - 52 50 00 00 06 F3 05 00
	// 01 F9 04 AA 61 66 00 24 - 00 00 F8 7D
	if (VIDEO_STANDARD_720P == standard) {
		This->m_CmdPacket.stPakAll.stCmdPrm.Read[1] = 0x2D;	// 1280x720 60P
	} else if (VIDEO_STANDARD_1080I == standard) {
		This->m_CmdPacket.stPakAll.stCmdPrm.Read[1] = 0x34;	// 1920x1080 30P
	}
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[0] = 0xF8;	//0xF8;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[2] = 0x00;	//0x00;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[3] = 0x40;	//0x00;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[4] = 0x51;	//0x00;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[5] = 0x00;	//0x00;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[6] = 0x66;	//0x26;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[7] = 0x52;	//0x42;

	This->m_CmdPacket.stPakAll.stCmdPrm.Read[8] = 0x54;	//0x00;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[9] = 0x00;	//0x00;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[10] = 0x00;	//0x5D;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[11] = 0x00;	//0x80;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[12] = 0x00;	//0x05;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[13] = 0x00;	//0x00;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[14] = 0xEA;	//0x83;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[15] = 0x00;	//0x3B;

	This->m_CmdPacket.stPakAll.stCmdPrm.Read[16] = 0xA0;	//0x50;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[17] = 0x92;	//0x52;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[18] = 0x00;	//0x00;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[19] = 0x00;	//0x00;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[21] = 0x00;	//0xF3;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[22] = 0x00;	//0x06;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[22] = 0x00;	//0x00;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[23] = 0x00;	//0x05;

	This->m_CmdPacket.stPakAll.stCmdPrm.Read[24] = 0x00;	//0xF9;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[25] = 0x00;	//0x01;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[26] = 0x00;	//0xAA;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[27] = 0x00;	//0x04;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[28] = 0x00;	//0x66;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[29] = 0x00;	//0x61;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[30] = 0x00;	//0x24;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[31] = 0x00;	//0x00;

	qpStatus = CPE1005_SendCommand(This);

	QPTMDelayMilliS(20);	// delay 20 ms

	// do mode change
	for (i = 2; i < 44; i++) {
		This->m_CmdPacket.Read[i] = 0;
	}

	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.Reserved2 = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.PRS = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.Reserved1 = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.SR = 0;

	This->m_CmdPacket.stPakAll.ucRetry = 0;
	This->m_CmdPacket.stPakAll.Reserved1 = 0;

	This->m_CmdPacket.stPakAll.stCmdHeader.Read = 0x0018;
	This->m_CmdPacket.stPakAll.usCmdPrmSize = 0x0002;

	This->m_CmdPacket.stPakAll.stCmdPrm.Read[0] = 0x01;	// INT: mode change
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[1] = 0x00;

	qpStatus = CPE1005_SendCommand(This);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_SeqModeChange() ****** CPE1005_SendCommand() failed to change mode to PE1005! ******\n"));
		return (QPERR_FAIL);
	} else {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
			      "CPE1005_SeqModeChange() send command OK! ******\n"));
	}

	// set INT_IN
	qpStatus = CPE1005_INT_IN_On(This);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_SeqModeChange() ****** CPE1005_INT_IN_On() failed to set INT_IN of PE1005! ******\n"));
		return (QPERR_FAIL);
	} else {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
			      "CPE1005_SeqModeChange() CPE1005_INT_IN_On OK! ******\n"));
	}

      CHECK_INT_OUT:
	// check INT_OUT
	qpStatus = CPE1005_INT_OUT_Check(This);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
			      "CPE1005_SeqModeChange() ****** checking INT_OUT from PE1005! ******\n"));
		if (retry++ > 20) {
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
				      "CPE1005_SeqModeChange() ****** CPE1005_INT_OUT_Check() failed to check INT_OUT from PE1005! ******\n"));
			return (QPERR_FAIL);
		}
		QPTMDelayMilliS(500);	// check period 500 ms
		goto CHECK_INT_OUT;
	}
	// clear INT_IN
	qpStatus = CPE1005_INT_IN_Off(This);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_SeqModeChange() ****** CPE1005_INT_IN_Off() failed to clear INT_IN of PE1005! ******\n"));
		return (QPERR_FAIL);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
		      "CPE1005_SetFrameRate() : PE1005 frame size set OK\n"));

	return (QPERR_SUCCESS);
}

//static 
EQP_Errors CPE1005_SetFrameRate(IVideoDecoder * This_p, TQP_UINT32 dwRate)
{
	CVideoDecoder *pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CPE1005 *This = GET_INHERITED_OBJECT(CPE1005, m_VideoDecoder, pVideoDecoder);

	EQP_Errors qpStatus;
	TQP_UINT8 i = 0, retry = 0;

	// initialize 44-byte command packet
	for (i = 2; i < 44; i++) {
		This->m_CmdPacket.Read[i] = 0;
	}

	// get current camera settings
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.Reserved2 = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.PRS = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.Reserved1 = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.SR = 0;

	This->m_CmdPacket.stPakAll.ucRetry = 0;
	This->m_CmdPacket.stPakAll.Reserved1 = 0;

	This->m_CmdPacket.stPakAll.stCmdHeader.Read = 0x0201;
	This->m_CmdPacket.stPakAll.usCmdPrmSize = 0x0000;

	qpStatus = CPE1005_SendCommand(This);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
		      "CPE1005_SetFrameRate() : PE1005 Frame Rate Set to %dx\n", dwRate));

	// initialize 44-byte command packet
	for (i = 2; i < 44; i++) {
		This->m_CmdPacket.Read[i] = 0;
	}

	// copy other fields of CMD_STAT
	for (i = 8; i < 39; i++) {
		This->m_CmdPacket.Read[i] = This->m_ResPacket.Read[i];
	}

	// set frame rate
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.Reserved2 = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.PRS = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.Reserved1 = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.SR = 0;

	This->m_CmdPacket.stPakAll.ucRetry = 0;
	This->m_CmdPacket.stPakAll.Reserved1 = 0;

	This->m_CmdPacket.stPakAll.stCmdHeader.Read = 0x0200;
	This->m_CmdPacket.stPakAll.usCmdPrmSize = 0x0020;

	This->m_CmdPacket.stPakAll.stCmdPrm.Read[1] &= 0xF8;	// bit 0~2
	//This->m_CmdPacket.stPakAll.stCmdPrm.Read[1] |= 0x01;   // for 15fps, note: 1080p minimum 15fps
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[1] |= 0x04;	// for 30 fps

	qpStatus = CPE1005_SendCommand(This);

	QPTMDelayMilliS(20);	// delay 20 ms

	// do mode change
	for (i = 2; i < 44; i++) {
		This->m_CmdPacket.Read[i] = 0;
	}

	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.Reserved2 = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.PRS = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.Reserved1 = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.SR = 0;

	This->m_CmdPacket.stPakAll.ucRetry = 0;
	This->m_CmdPacket.stPakAll.Reserved1 = 0;

	This->m_CmdPacket.stPakAll.stCmdHeader.Read = 0x0018;
	This->m_CmdPacket.stPakAll.usCmdPrmSize = 0x0002;

	This->m_CmdPacket.stPakAll.stCmdPrm.Read[0] = 0x01;	// INT: mode change
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[1] = 0x00;

	qpStatus = CPE1005_SendCommand(This);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_SeqModeChange() ****** CPE1005_SendCommand() failed to change mode to PE1005! ******\n"));
		return (QPERR_FAIL);
	} else {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
			      "CPE1005_SeqModeChange() send command OK! ******\n"));
	}

	// set INT_IN
	qpStatus = CPE1005_INT_IN_On(This);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_SeqModeChange() ****** CPE1005_INT_IN_On() failed to set INT_IN of PE1005! ******\n"));
		return (QPERR_FAIL);
	} else {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
			      "CPE1005_SeqModeChange() CPE1005_INT_IN_On OK! ******\n"));
	}

      CHECK_INT_OUT:
	// check INT_OUT
	qpStatus = CPE1005_INT_OUT_Check(This);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
			      "CPE1005_SeqModeChange() ****** checking INT_OUT from PE1005! ******\n"));
		if (retry++ > 20) {
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
				      "CPE1005_SeqModeChange() ****** CPE1005_INT_OUT_Check() failed to check INT_OUT from PE1005! ******\n"));
			return (QPERR_FAIL);
		}
		QPTMDelayMilliS(500);	// check period 500 ms
		goto CHECK_INT_OUT;
	}
	// clear INT_IN
	qpStatus = CPE1005_INT_IN_Off(This);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_SeqModeChange() ****** CPE1005_INT_IN_Off() failed to clear INT_IN of PE1005! ******\n"));
		return (QPERR_FAIL);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
		      "CPE1005_SetFrameRate() : PE1005 Frame Rate Set OK\n"));

	return (QPERR_SUCCESS);
}


static EQP_Errors CPE1005_GetLensRoomPos(IVideoDecoder *This_p, PTQP_INT16 psCurrent, PTQP_INT16 psMin, PTQP_INT16 psMax)
{
    //CVideoDecoder *pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
    //CPE1005       *This = GET_INHERITED_OBJECT(CPE1005, m_VideoDecoder, pVideoDecoder);

    psMin = (PTQP_INT16)-36;
    psMax = (PTQP_INT16)36;

    return (QPERR_SUCCESS);
}

static EQP_Errors CPE1005_SetLensRoomPos(IVideoDecoder *This_p, TQP_INT16 sCurrent)
{
	CVideoDecoder *pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CPE1005 *This = GET_INHERITED_OBJECT(CPE1005, m_VideoDecoder, pVideoDecoder);

	EQP_Errors qpStatus;
	TQP_UINT8 i = 0;

	// initialize 44-byte command packet
	for (i = 2; i < 44; i++) {
		This->m_CmdPacket.Read[i] = 0;
	}

	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.Reserved2 = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.PRS = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.Reserved1 = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.SR = 0;

	This->m_CmdPacket.stPakAll.ucRetry = 0;
	This->m_CmdPacket.stPakAll.Reserved1 = 0;

	This->m_CmdPacket.stPakAll.stCmdHeader.Read = 0x040E;
	This->m_CmdPacket.stPakAll.usCmdPrmSize = 0x0004;

	This->m_CmdPacket.stPakAll.stCmdPrm.Read[0] = 0x00;	// specify zoom speed mode, always 0
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[1] = 0x00;

#if 0
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[2] = 0x10;	// zoom parameter
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[3] = 0x00;
#else // to wide direction
    //This->m_CmdPacket.stPakAll.stCmdPrm.Read[2]      = 0xEF;   // zoom parameter
    //This->m_CmdPacket.stPakAll.stCmdPrm.Read[3]      = 0xFF;
    {
        short *psPos = (short *)&This->m_CmdPacket.stPakAll.stCmdPrm.Read[2];
        *psPos = (TQP_INT16)sCurrent;
    }
#endif

	qpStatus = CPE1005_SendCommand(This);

    QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
                  "CPE1005_SetOpticalZoom() : Start optical zoom to %dx\n",
                  sCurrent
                ));

    QPTMDelayMilliS(1200);   // delay 1 seconds for zoom

	// stop zoom

	// initialize 44-byte command packet
	for (i = 2; i < 44; i++) {
		This->m_CmdPacket.Read[i] = 0;
	}

	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.Reserved2 = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.PRS = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.Reserved1 = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.SR = 0;

	This->m_CmdPacket.stPakAll.ucRetry = 0;
	This->m_CmdPacket.stPakAll.Reserved1 = 0;

	This->m_CmdPacket.stPakAll.stCmdHeader.Read = 0x040E;
	This->m_CmdPacket.stPakAll.usCmdPrmSize = 0x0004;

	This->m_CmdPacket.stPakAll.stCmdPrm.Read[0] = 0x00;	// specify zoom speed mode, always 0
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[1] = 0x00;

	This->m_CmdPacket.stPakAll.stCmdPrm.Read[2] = 0x00;	// stop zoom
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[3] = 0x00;

	qpStatus = CPE1005_SendCommand(This);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
		      "CPE1005_SetOpticalZoom() : Optical zoom completes\n"));

	return qpStatus;
}

// PE1005 Startup Camera System Sequence
static EQP_Errors CPE1005_SeqStartupCamera(IVideoDecoder * This_p)
{
	//CVideoDecoder *pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	//CPE1005 *This = GET_INHERITED_OBJECT(CPE1005, m_VideoDecoder, pVideoDecoder);
	EQP_Errors qpStatus;
	//TQP_UINT8 i = 0;

	VIDEO_STANDARD standard = VIDEO_STANDARD_720P;

	QPTMDelayMilliS(3000);	// MUST delay 3 secs, wait for PE1005 internal power on sequence completes

	// configure camera
	qpStatus = CPE1005_SeqModeChange(This_p);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_SeqStartupCamera() ****** CPE1005_SeqModeChange() failed to change mode of PE1005! ******\n"));
		return (QPERR_FAIL);
	} else {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
			      "CPE1005_SeqModeChange() OK! ******\n"));
	}

#if 1				// do optical zoom
	QPTMDelayMilliS(1000);	// delay 1 secs for latch

    qpStatus = CPE1005_SetLensRoomPos(This_p, 0);
    if (!QP_SUCCESS(qpStatus)) {
        QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
                     "CPE1005_SetOpticalZoom() ****** failed to do zoom control of PE1005! ******\n"
                    ));
        return (QPERR_FAIL);
    }
    else
    {
        QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
                     "CPE1005_SetOpticalZoom() OK! ******\n"
                    ));
    }
#endif

#if 0				// do frame rate
	QPTMDelayMilliS(1000);	// delay 1 secs for latch

	qpStatus = CPE1005_SetFrameRate(This_p, 0);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_SetFrameRate() ****** failed to set frame rate of PE1005! ******\n"));
		return (QPERR_FAIL);
	} else {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
			      "CPE1005_SetFrameRate() OK! ******\n"));
	}
#endif

#if 1				// do resolution
	QPTMDelayMilliS(1000);	// delay 1 secs for latch

	qpStatus = CPE1005_SetVideoRes(This_p, standard);	// set default 1920x1080
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_SetVideoRes() ****** failed to set frame size of PE1005! ******\n"));
		return (QPERR_FAIL);
	} else {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
			      "CPE1005_SetVideoRes() OK! ******\n"));
	}
#endif

#if 0				// try sense command
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
		      "Test sense command start! ******\n"));

	//QPTMDelayMilliS(3000);   // delay 3 secs for latch

	for (i = 2; i < 44; i++) {
		This->m_CmdPacket.Read[i] = 0;
	}

	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.Reserved2 = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.PRS = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.Reserved1 = 0;
	This->m_CmdPacket.stPakAll.stPakHeader.stPakHead.SR = 0;

	This->m_CmdPacket.stPakAll.ucRetry = 0;
	This->m_CmdPacket.stPakAll.Reserved1 = 0;

	This->m_CmdPacket.stPakAll.stCmdHeader.Read = 0x0201;
	This->m_CmdPacket.stPakAll.usCmdPrmSize = 0x0000;

	This->m_CmdPacket.stPakAll.stCmdPrm.Read[0] = 0x00;
	This->m_CmdPacket.stPakAll.stCmdPrm.Read[1] = 0x00;

	qpStatus = CPE1005_SendCommand(This);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "Test sense command() ****** CPE1005_SendCommand() failed to change mode to PE1005! ******\n"));
		return (QPERR_FAIL);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
		      "Test sense command OK! ******\n"));
#endif

	return (QPERR_SUCCESS);
}

// IGenericDevice
//
static EQP_Errors CPE1005_InitDevice(IVideoDecoder * This_p,
				     QP_DEVICE_CALLBACK pFuncCallback,
				     TQP_PVOID context, TQP_PVOID pInitData)
{
	EQP_Errors qpStatus;
	//TQP_UINT8 buffer[2];	// Andy test
	//TQP_UINT8 regAddr;	// Andy test
	//TQP_UINT8 Value;	// Andy test

	CVideoDecoder *pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CPE1005 *This = GET_INHERITED_OBJECT(CPE1005, m_VideoDecoder, pVideoDecoder);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
		      "CPE1005_InitDevice()\n"));

	if (CObject_IsInitialized(&pVideoDecoder->m_Object)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
			      "CPE1005_InitDevice() Already Initialized. Skipping...\n"));
		return (QPERR_SUCCESS);
	}
	// save callback address and context
	//
	pVideoDecoder->m_pDeviceCallback = pFuncCallback;
	pVideoDecoder->m_callbackContext = context;

	qpStatus = CPE1005_InitIO(This);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_InitDevice() ****** CPE1005_InitIO() failed to initial GPIO! ******\n"));
		return (QPERR_FAIL);
	}
	// Do power down sequence first to prevent from abnormal behavior due to last wrong status
	// HRSTX (L) --> SHUT_DOWN (L) --> PON (L)

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
		      "CPE1005_InitDevice() ****** Power off first! ******\n"));

	qpStatus = CPE1005_HRSTX_Off(This);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_InitIO() ****** CPE1005_PON_Off() failed! ******\n"));
		return (QPERR_FAIL);
	}

	QPTMDelayMilliS(20);	// delay 20 ms

	qpStatus = CPE1005_ShutDown_Off(This);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_InitIO() ****** CPE1005_PON_Off() failed! ******\n"));
		return (QPERR_FAIL);
	}

	QPTMDelayMilliS(20);	// delay 20 ms

	qpStatus = CPE1005_PON_Off(This);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_InitIO() ****** CPE1005_PON_Off() failed! ******\n"));
		return (QPERR_FAIL);
	}

	QPTMDelayMilliS(1000);	// delay 1 sec

	// Power on sequence
	// 4V & 2.8V ready --> PON (H) --> Shut_down (H) --> HRSTX (H) --> 300ms --> Reset OK
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
		      "CPE1005_InitDevice() ****** Power on PE1005! ******\n"));

	qpStatus = CPE1005_POWER_On(This);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_InitDevice() ****** CPE1005_POWER_On() failed! ******\n"));
		return (QPERR_FAIL);
	}

	QPTMDelayMilliS(1000);	// delay 1s

	qpStatus = CPE1005_PON_On(This);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_InitDevice() ****** CPE1005_PON_On() failed! ******\n"));
		return (QPERR_FAIL);
	}

	QPTMDelayMilliS(10);	// delay 10 ms

	qpStatus = CPE1005_ShutDown_On(This);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_InitDevice() ****** CPE1005_PON_On() failed! ******\n"));
		return (QPERR_FAIL);
	}

	QPTMDelayMilliS(10);	// delay 10 ms

	qpStatus = CPE1005_HRSTX_On(This);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_InitDevice() ****** CPE1005_PON_On() failed! ******\n"));
		return (QPERR_FAIL);
	}
#if 0
	// Initialize PE1005...
	//

	// program the video decoder with settings read from the registry
	//
	QPVideoDecoder_SetVideoSystem(This_p, pVideoDecoder->m_VidStandard);
	QPVideoDecoder_SelectVideoSource(This_p, pVideoDecoder->m_VidSource);

	QPVideoDecoder_SetBrightness(This_p, pVideoDecoder->m_nBrightness);
	QPVideoDecoder_SetContrast(This_p, pVideoDecoder->m_nContrast);
	QPVideoDecoder_SetSaturation(This_p, pVideoDecoder->m_nSaturation);
	QPVideoDecoder_SetHue(This_p, pVideoDecoder->m_nHue);
	QPVideoDecoder_SetSharpness(This_p, pVideoDecoder->m_nSharpness);
#endif

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
		      "CPE1005_InitDevice() OK! \n"));

	qpStatus = CPE1005_SeqStartupCamera(This_p);
	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_InitDevice() ****** CPE1005_SeqStartupCamera() failed! ******\n"));
		return (QPERR_FAIL);
	}

	if (!QP_SUCCESS(qpStatus)) {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
			      "CPE1005_ShutDown_Off() ****** II2C_Write() failed! ******\n"));
		return (QPERR_FAIL);
	}
	// CObject Init
	//
	CObject_Init(&pVideoDecoder->m_Object);

	return (QPERR_SUCCESS);
}

// constructor
//
CPE1005 *CPE1005_Constructor(CPE1005 * pPE1005,
			     CObject * pParent,
			     TQP_UINT32 dwAttributes,
			     ITuner * pTuner,
			     ITVAudio * pTvAudio, II2C * pI2C, TQP_UINT8 cI2CDevAddr)
{
	TQP_UINT8 i = 0;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
		      "CPE1005_Constructor()\n"));

	if (pPE1005) {
		CVideoDecoder *pVideoDecoder;

		pVideoDecoder = CVideoDecoder_Constructor(&pPE1005->m_VideoDecoder,
							  pParent, dwAttributes, pTuner, pTvAudio);

		if (!pVideoDecoder) {
			pPE1005 = QP_NULL;
		} else {
			// fill in the function table
			//

			// IGenericDevice
			pPE1005->m_VideoDecoder.m_iVideoDecoder.InitDevice = CPE1005_InitDevice;

            // IVideoDecoder
            pPE1005->m_VideoDecoder.m_iVideoDecoder.SetVideoSystem = CPE1005_SetVideoRes;
            
            // 2009.0213 by jkglee
            pPE1005->m_VideoDecoder.m_iVideoDecoder.GetLensRoomPos = CPE1005_GetLensRoomPos;
            pPE1005->m_VideoDecoder.m_iVideoDecoder.SetLensRoomPos = CPE1005_SetLensRoomPos;

			// initialize members
			//
			pPE1005->m_pI2C = pI2C;
			pPE1005->m_cI2CDevAddr = cI2CDevAddr;

			// inherited members
			//
			pPE1005->m_VideoDecoder.m_nBrightnessDef = PE1005_BRIGHTNESS_DEFAULT;
			pPE1005->m_VideoDecoder.m_nBrightnessMin = PE1005_BRIGHTNESS_MIN;
			pPE1005->m_VideoDecoder.m_nBrightnessMax = PE1005_BRIGHTNESS_MAX;
			pPE1005->m_VideoDecoder.m_nBrightness = PE1005_BRIGHTNESS_DEFAULT;
			pPE1005->m_VideoDecoder.m_nContrastDef = PE1005_CONTRAST_DEFAULT;
			pPE1005->m_VideoDecoder.m_nContrastMin = PE1005_CONTRAST_MIN;
			pPE1005->m_VideoDecoder.m_nContrastMax = PE1005_CONTRAST_MAX;
			pPE1005->m_VideoDecoder.m_nContrast = PE1005_CONTRAST_DEFAULT;
			pPE1005->m_VideoDecoder.m_nSaturationDef = PE1005_SATURATION_DEFAULT;
			pPE1005->m_VideoDecoder.m_nSaturationMin = PE1005_SATURATION_MIN;
			pPE1005->m_VideoDecoder.m_nSaturationMax = PE1005_SATURATION_MAX;
			pPE1005->m_VideoDecoder.m_nSaturation = PE1005_SATURATION_DEFAULT;
			pPE1005->m_VideoDecoder.m_nHueDef = PE1005_HUE_DEFAULT;
			pPE1005->m_VideoDecoder.m_nHueMin = PE1005_HUE_MIN;
			pPE1005->m_VideoDecoder.m_nHueMax = PE1005_HUE_MAX;
			pPE1005->m_VideoDecoder.m_nHue = PE1005_HUE_DEFAULT;
			pPE1005->m_VideoDecoder.m_nSharpnessDef = PE1005_SHARPNESS_DEFAULT;
			pPE1005->m_VideoDecoder.m_nSharpnessMin = PE1005_SHARPNESS_MIN;
			pPE1005->m_VideoDecoder.m_nSharpnessMax = PE1005_SHARPNESS_MAX;
			pPE1005->m_VideoDecoder.m_nSharpness = PE1005_SHARPNESS_DEFAULT;
		}

		// initialize 44-byte command packet
		for (i = 0; i < 44; i++) {
			pPE1005->m_CmdPacket.Read[i] = 0;
		}
	}

	return (pPE1005);
}

// destructor
//
void CPE1005_Destructor(CPE1005 * This)
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
		      "CPE1005_Destructor()\n"));

	CVideoDecoder_Destructor(&This->m_VideoDecoder);
}
