///////////////////////////////////////////////////////////////////////////////
//        
//
// Csil9034.h
//
// Description: 
//
//  SIL9034 implementation
// 
// History:
//	May 12, 2008 created
//
// Authors: Andy Yu
//
///////////////////////////////////////////////////////////////////////////////


#include "csil9034.h"
#include "util.h"
#include "qpdbgmgr.h"

extern TQP_DbgMCompID	g_QPPeripheralDBGCompID;


/////////////////////////////////////////////////////////////////////////////
//
//

static EQP_Errors CSIL9034_readRegister(CSIL9034 *This, 
										TQP_UINT8 regAddr, 
										PTQP_UINT8 pValue
										)
{
    return (II2C_WriteThenRead(This->m_pI2C, 
							   This->m_cI2CDevAddr,	//device address
							   1,					//write size
							   &regAddr,			//write buffer
							   1,					//read size
							   pValue				//read buffer
							   ));
}



static EQP_Errors CSIL9034_writeRegister(CSIL9034 *This, 
										 TQP_UINT8 regAddr, 
										 TQP_UINT8 value
										 )
{
    TQP_UINT8 buffer[2];

	buffer[0] = regAddr;
	buffer[1] = value;

    return (II2C_Write(This->m_pI2C, 
					   This->m_cI2CDevAddr,	//device address
					   2,					//write size
					   buffer				//write buffer
					   ));
}



static EQP_Errors CSIL9034_WriteBlock(CSIL9034 *This, 
									  TQP_UINT32 *pRegTable
									  )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	TQP_UINT32	dwTotalReg = 0;

	// get total number of registers to write
	//
	while (-1 != pRegTable[dwTotalReg]) 
	{
		dwTotalReg++;
	}

	dwTotalReg >>= 1;

	// do multiple writes
	//
	while (dwTotalReg)
	{
		qpStatus = CSIL9034_writeRegister(This,
										  (TQP_UINT8)pRegTable[0], 
										  (TQP_UINT8)pRegTable[1]
										  );
		if (!QP_SUCCESS(qpStatus))
			return (qpStatus);

		pRegTable += 2;
		dwTotalReg--;
	}

	return (qpStatus);
}



/////////////////////////////////////////////////////////////////////////////
//
//


// IGenericDevice
//
static EQP_Errors CSIL9034_InitDevice(IVideoEncoder *This_p, 
									  QP_DEVICE_CALLBACK pFuncCallback,
									  TQP_PVOID context,
									  TQP_PVOID pInitData
									  )
{
	PQPVIDENCODER_INITDATA	pVidEncInitData = (PQPVIDENCODER_INITDATA)pInitData;
	VIDEO_STANDARD			standard;
	CVideoEncoder			*pVideoEncoder = GET_INHERITED_OBJECT(CVideoEncoder, m_iVideoEncoder, This_p);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CSIL9034_InitDevice()\n"
				 ));

	if (CObject_IsInitialized(&pVideoEncoder->m_Object))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
					 "CSIL9034_InitDevice() Already Initialized. Skipping...\n"
					 ));
		return (QPERR_SUCCESS);
	}

	// save callback address and context
	//
	pVideoEncoder->m_pDeviceCallback = pFuncCallback;
	pVideoEncoder->m_callbackContext = context;

	// default standard
	//
	if (pVidEncInitData)
	{
		standard = pVidEncInitData->standard;

		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
					 "CSIL9034_InitDevice() input standard(%d), remembered standard(%d)\n",
					 standard, 
					 pVideoEncoder->m_VidStandard
					 ));
	}
	else
	{
		standard = pVideoEncoder->m_VidStandard;
	}

	IVideoEncoder_SetVideoStandard(This_p,
								   standard
								   );

	pVideoEncoder->m_fVideoEncoderEnabled = QP_TRUE;

	// CObject Init
	//
	CObject_Init(&pVideoEncoder->m_Object);

	return (QPERR_SUCCESS);
}



// IVideoEncoder
//

// Enables encoder device.
static EQP_Errors CSIL9034_Enable(IVideoEncoder *This_p)
{
/*
	TQP_UINT8		ucReg;
	EQP_Errors		qpStatus;
*/
	CVideoEncoder	*pVideoEncoder = GET_INHERITED_OBJECT(CVideoEncoder, m_iVideoEncoder, This_p);
	//CSIL9034		*This = GET_INHERITED_OBJECT(CSIL9034, m_VideoEncoder, pVideoEncoder);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CSIL9034_Enable() ****** not implemented! ******\n"
				 ));

/*
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CSIL9034_Enable() ****** writeRegister() failed to write SIL9034 registers! ******\n" 
					 ));
		return (QPERR_FAIL);
	}
*/

	pVideoEncoder->m_fVideoEncoderEnabled = QP_TRUE;
	return (QPERR_SUCCESS);
}



// Disables encoder device.
static EQP_Errors CSIL9034_Disable(IVideoEncoder *This_p)
{
/*
	TQP_UINT8		ucReg;
	EQP_Errors		qpStatus;
*/
	CVideoEncoder	*pVideoEncoder = GET_INHERITED_OBJECT(CVideoEncoder, m_iVideoEncoder, This_p);
	//CSIL9034		*This = GET_INHERITED_OBJECT(CSIL9034, m_VideoEncoder, pVideoEncoder);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CSIL9034_Disable() ****** not implemented! ******\n"
				 ));

/*
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CSIL9034_Disable() ****** writeRegister failed to write SIL9034 registers! ******\n"
					 ));
		return (QPERR_FAIL);
	}
*/

	pVideoEncoder->m_fVideoEncoderEnabled = QP_FALSE;
	return (QPERR_SUCCESS);
}



// Sets video system of the encoder.
//
static EQP_Errors CSIL9034_SetVideoStandard(IVideoEncoder *This_p, 
											VIDEO_STANDARD standard
											)
{
	// same settings for all standards
	static TQP_UINT32	dw9034RegVals[] = 
	{
    0x08,0x35,
    0x0F,0x04,
    0x32,0x04,
    0x33,0x00,
    0x34,0x19,
    0x36,0x00,
    0x37,0x05,
    0x38,0xD0,
    0x39,0x05,
    0x82,0x20,
    0x48,0x60,
    0x4A,0x3C,
    0x75,0xFF,
    0x76,0xFF,
    0x77,0xFF,
    0x70,0xFF,
    0x71,0xFF,
    0x72,0xFF,
    0x73,0xFF,
		-1,   -1
	};

	TQP_UINT32		*pVals;
	EQP_Errors		qpStatus;
	CVideoEncoder	*pVideoEncoder = GET_INHERITED_OBJECT(CVideoEncoder, m_iVideoEncoder, This_p);
	CSIL9034		*This = GET_INHERITED_OBJECT(CSIL9034, m_VideoEncoder, pVideoEncoder);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CSIL9034_SetVideoStandard() standard(%d)\n",
				 standard
				 ));

	if (VIDEO_STANDARD_720P == standard)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
					 "CSIL9034_SetVideoStandard() 720p\n"
					 ));
		pVals = dw9034RegVals;
	}
	else if (VIDEO_STANDARD_1080I == standard)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
					 "CSIL9034_SetVideoStandard() 1080i\n"
					 ));
		pVals = dw9034RegVals;
	}
	else
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CSIL9034_SetVideoStandard() Unknown Video Standard: %d \n", 
					 standard
					 ));
		return (QPERR_PARMS);
	}

	// save the video standard
	//
	pVideoEncoder->m_VidStandard = standard;

	// Initialize 9034 with the default value
	//
	qpStatus = CSIL9034_WriteBlock(This,
								   pVals
								   );

	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CSIL9034_SetVideoStandard() WriteBlock Failed!\n"
					 ));
		return (QPERR_FAIL);
	}

	return (QPERR_SUCCESS);
}



// constructor
//
CSIL9034 * CSIL9034_Constructor(CSIL9034 *pSIL9034,
								CObject *pParent,
								TQP_UINT32 dwAttributes,
								II2C *pI2C,
								TQP_UINT8 cI2CDevAddr
								)
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CSIL9034_Constructor()\n"
				 ));

	if (pSIL9034)
	{
		CVideoEncoder	*pVideoEncoder;

		pVideoEncoder = CVideoEncoder_Constructor(&pSIL9034->m_VideoEncoder, 
												  pParent,
												  dwAttributes
												  );
		if (!pVideoEncoder)
		{
			pSIL9034 = QP_NULL;
		}
		else
		{
			// fill in the function table
			//

			// IGenericDevice
			pSIL9034->m_VideoEncoder.m_iVideoEncoder.InitDevice = CSIL9034_InitDevice;

			// IVideoEncoder
			pSIL9034->m_VideoEncoder.m_iVideoEncoder.Enable = CSIL9034_Enable;
			pSIL9034->m_VideoEncoder.m_iVideoEncoder.Disable = CSIL9034_Disable;
			pSIL9034->m_VideoEncoder.m_iVideoEncoder.SetVideoStandard = CSIL9034_SetVideoStandard;

			// initialize members
			//
			pSIL9034->m_pI2C = pI2C;
			pSIL9034->m_cI2CDevAddr = cI2CDevAddr;
		}
	}

	return (pSIL9034);
}



// destructor
//
void CSIL9034_Destructor(CSIL9034 *This)
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CSIL9034_Destructor()\n"
				 ));

	CVideoEncoder_Destructor(&This->m_VideoEncoder);
}


