///////////////////////////////////////////////////////////////////////////////
//        
//
// Ctuner.c
//
// Description: 
//
//  TV tuner implementation
// 
// History:
//	Aug 28, 2006	(dt) created
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#include "ctuner.h"
#include "util.h"
#include "qpdbgmgr.h"
#include "qpviddecoder.h"

extern TQP_DbgMCompID	g_QPPeripheralDBGCompID;


// IGenericDevice
//
static EQP_Errors CTuner_InitDevice(ITuner *This_p, 
									QP_DEVICE_CALLBACK pFuncCallback,
									TQP_PVOID context,
									TQP_PVOID pInitData
									)
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTuner::InitDevice()\n"
				 ));


	if (CObject_IsInitialized(&This->m_Object))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
					 "CTuner::InitDevice() Already Initialized. Skipping...\n"
					 ));
		return (QPERR_SUCCESS);
	}

	// save callback address and context
	//
	This->m_pDeviceCallback = pFuncCallback;
	This->m_callbackContext = context;

	// Init the Tuner
	//
	if (QPPROPERTY_TUNER_MODE_TV == This->m_ulTunerMode)
	{
		QPTuner_SetFrequency(This_p,
							 This->m_ulCurrentFrequency, 
							 This->m_ulCurrentCountry
							 );
	}
	else
	{
		QPTuner_SetFMFrequency(This_p,
							   This->m_ulFMCurrentFrequency, 
							   This->m_ulCurrentCountry
							   );
	}


	// CObject Init
	//
	CObject_Init(&This->m_Object);

	return (QPERR_SUCCESS);
}



static EQP_Errors CTuner_Release(ITuner *This_p)
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTuner_Release()\n"
				 ));

	This->m_pDeviceCallback = QP_NULL;
	This->m_callbackContext = QP_NULL;

	// CObject Done
	//
	CObject_Done(&This->m_Object);

	return (QPERR_SUCCESS);
}



static EQP_Errors CTuner_Reset(ITuner *This_p) 
{
	return (QPERR_SUCCESS);
}



static EQP_Errors CTuner_Set(ITuner *This_p,
							 TQP_REFGUID guid,
							 TQP_OBJECT_HANDLE hIndex,
							 TQP_UINT32 dwCode,
							 TQP_PVOID pInput,
							 TQP_PVOID pOutput,
							 TQP_UINT32 dwSize
							 )
{
	return (QPERR_NOTIMPL);
}



static EQP_Errors CTuner_Get(ITuner *This_p,
							 TQP_REFGUID guid,
							 TQP_OBJECT_HANDLE hIndex,
							 TQP_UINT32 dwCode,
							 TQP_PVOID pInput,
							 TQP_PVOID pOutput,
							 PTQP_UINT32 pdwSizeGot
							 )
{
	return (QPERR_NOTIMPL);
}



// ITuner
//
static TQP_BOOL CTuner_SetFrequency(ITuner *This_p,
									TQP_UINT32 ulFrequency, 
									TQP_UINT32 ulCountry
									)
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	This->m_ulCurrentFrequency = ulFrequency; 
	return (QP_TRUE);
}



static TQP_BOOL CTuner_SetFMFrequency(ITuner *This_p,
									  TQP_UINT32 ulFrequency, 
									  TQP_UINT32 ulCountry
									  ) 
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	This->m_ulFMCurrentFrequency = ulFrequency; 
	return (QP_TRUE);
}



static TQP_BOOL CTuner_GetPLLOffsetAndBusyStatus(ITuner *This_p,
												 TQP_BOOL *pbPllLocked,	// Returns PLL locked flag here
												 PTQP_INT32 plPllOffset	// Returns PLL offset flag here
												 )
{
	*pbPllLocked = QP_TRUE;
	*plPllOffset = 0;
	return (QP_TRUE);
}



static TQP_BOOL CTuner_GetSignalStrength(ITuner *This_p,
										 TQP_BOOL *pbPllLocked, 
										 PTQP_UINT32 pulSignalStrenth
										 )
{
	*pbPllLocked = QP_TRUE;
	*pulSignalStrenth = 100;
	return (QP_TRUE);
}



static void CTuner_SetMode(ITuner *This_p,
						   TQP_UINT32 mode
						   ) 
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	This->m_ulTunerMode = mode;
}



static TQP_UINT32 CTuner_GetMode(ITuner *This_p)	// Get Tuner Mode
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	return (This->m_ulTunerMode);
}



static TQP_BOOL CTuner_SetStandard(ITuner *This_p,
								   TQP_UINT32 standard
								   ) 
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	This->m_ulCurrentStandard = standard; 
	return (QP_TRUE);
}



static TQP_UINT32 CTuner_GetStandard(ITuner *This_p) 
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	return (QPPROPERTY_TUNER_MODE_TV == This->m_ulTunerMode ? This->m_ulCurrentStandard : This->m_ulFMStandardsSupported);
}



static TQP_BOOL CTuner_SetCountry(ITuner *This_p,
								  TQP_UINT32 country
								  ) 
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	This->m_ulCurrentCountry = country; 
	return (QP_TRUE);
}



static TQP_UINT32 CTuner_GetCountry(ITuner *This_p) 
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	return (This->m_ulCurrentCountry);
}



static TQP_UINT32 CTuner_GetSupportedModes(ITuner *This_p) 
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	return (This->m_ulSupportedModes);
}



static TQP_UINT32 CTuner_GetSupportedStandards(ITuner *This_p,
											   TQP_UINT32 mode
											   ) 
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	return (QPPROPERTY_TUNER_MODE_TV == mode ? This->m_ulStandardsSupported : This->m_ulFMStandardsSupported);
}



static TQP_UINT32 CTuner_GetMinFrequency(ITuner *This_p,
										 TQP_UINT32 mode
										 ) 
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	return (QPPROPERTY_TUNER_MODE_TV == mode ? This->m_ulMinFrequency : This->m_ulFMMinFrequency);
}



static TQP_UINT32 CTuner_GetMaxFrequency(ITuner *This_p,
										 TQP_UINT32 mode
										 ) 
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	return (QPPROPERTY_TUNER_MODE_TV == mode ? This->m_ulMaxFrequency : This->m_ulFMMaxFrequency);
}



static TQP_UINT32 CTuner_GetNumInputs(ITuner *This_p,
									  TQP_UINT32 mode
									  ) 
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	return (This->m_ulNumberOfInputs);
}



static TQP_UINT32 CTuner_GetTuningGranularity(ITuner *This_p,
											  TQP_UINT32 mode
											  ) 
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	return (QPPROPERTY_TUNER_MODE_TV == mode ? This->m_ulTuningGranularity : This->m_ulFMTuningGranularity);
}



static TQP_UINT32 CTuner_GetSettlingTime(ITuner *This_p,
										 TQP_UINT32 mode
										 ) 
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	return (QPPROPERTY_TUNER_MODE_TV == mode ? This->m_ulSettlingTime : This->m_ulFMSettlingTime);
}



static TQP_UINT32 CTuner_GetTuningStrategy(ITuner *This_p,
										   TQP_UINT32 mode
										   ) 
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	return (QPPROPERTY_TUNER_MODE_TV == mode ? This->m_ulStrategy : This->m_ulFMStrategy);
}



static TQP_UINT32 CTuner_GetTuningStrategy2(ITuner *This_p) 
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	return (QPPROPERTY_TUNER_MODE_TV == This->m_ulTunerMode ? This->m_ulStrategy : This->m_ulFMStrategy);
}



static TQP_UINT32 CTuner_GetFrequency(ITuner *This_p) 
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	return (QPPROPERTY_TUNER_MODE_TV == This->m_ulTunerMode ? This->m_ulCurrentFrequency : This->m_ulFMCurrentFrequency);
}



static TQP_UINT32 CTuner_GetLastFrequency(ITuner *This_p) 
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	return (This->m_ulLastFrequency);
}



static TQP_UINT32 CTuner_GetTuningFlags(ITuner *This_p) 
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	return (This->m_ulTuningFlags);
}



static TQP_UINT32 CTuner_GetVideoSubChannel(ITuner *This_p) 
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	return (This->m_ulVideoSubChannel);
}



static TQP_UINT32 CTuner_GetAudioSubChannel(ITuner *This_p) 
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	return (This->m_ulAudioSubChannel);
}



static TQP_UINT32 CTuner_GetChannel(ITuner *This_p) 
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	return (This->m_ulChannel);
}



static void CTuner_SetLastFrequency(ITuner *This_p,
									TQP_UINT32 ulLastFreq
									) 
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	This->m_ulLastFrequency = ulLastFreq;
}



static void CTuner_SetTuningFlags(ITuner *This_p,
								  TQP_UINT32 ulTuningFlags
								  ) 
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	This->m_ulTuningFlags = ulTuningFlags;
}


static void CTuner_SetVideoSubChannel(ITuner *This_p,
									  TQP_UINT32 ulVidSubChannel
									  ) 
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	This->m_ulVideoSubChannel = ulVidSubChannel;
}



static void CTuner_SetAudioSubChannel(ITuner *This_p,
									  TQP_UINT32 ulAudSubChannel
									  ) 
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	This->m_ulAudioSubChannel = ulAudSubChannel;
}



static void CTuner_SetChannel(ITuner *This_p,
							  TQP_UINT32 ulChannel
							  ) 
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	This->m_ulChannel = ulChannel;
}



static TQP_UINT32 CTuner_GetInputIndex(ITuner *This_p) 
{
	CTuner	*This =	GET_INHERITED_OBJECT(CTuner, m_iTuner, This_p);

	return (This->m_ulTunerInput);
}



// constructor
//
CTuner * CTuner_Constructor(CTuner *pTuner, 
							CObject *pParent,
							TQP_UINT32 dwAttributes
							)
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTuner_Constructor()\n"
				 ));

	if (pTuner)
	{
		CObject_Constructor(&pTuner->m_Object, 
							pParent, 
							dwAttributes
							);

		// fill in the function table
		//

		// IGenericDevice
		pTuner->m_iTuner.InitDevice = CTuner_InitDevice;
		pTuner->m_iTuner.Release = CTuner_Release;
		pTuner->m_iTuner.Reset = CTuner_Reset;
		pTuner->m_iTuner.Set = CTuner_Set;
		pTuner->m_iTuner.Get = CTuner_Get;

		// ITuner
		pTuner->m_iTuner.SetFrequency = CTuner_SetFrequency;
		pTuner->m_iTuner.SetFMFrequency = CTuner_SetFMFrequency;
		pTuner->m_iTuner.GetPLLOffsetAndBusyStatus = CTuner_GetPLLOffsetAndBusyStatus;
		pTuner->m_iTuner.GetSignalStrength = CTuner_GetSignalStrength;
		pTuner->m_iTuner.SetMode = CTuner_SetMode;
		pTuner->m_iTuner.GetMode = CTuner_GetMode;
		pTuner->m_iTuner.SetStandard = CTuner_SetStandard;
		pTuner->m_iTuner.GetStandard = CTuner_GetStandard;
		pTuner->m_iTuner.SetCountry = CTuner_SetCountry;
		pTuner->m_iTuner.GetCountry = CTuner_GetCountry;
		pTuner->m_iTuner.GetSupportedModes = CTuner_GetSupportedModes;
		pTuner->m_iTuner.GetSupportedStandards = CTuner_GetSupportedStandards;
		pTuner->m_iTuner.GetMinFrequency = CTuner_GetMinFrequency;
		pTuner->m_iTuner.GetMaxFrequency = CTuner_GetMaxFrequency;
		pTuner->m_iTuner.GetNumInputs = CTuner_GetNumInputs;
		pTuner->m_iTuner.GetTuningGranularity = CTuner_GetTuningGranularity;
		pTuner->m_iTuner.GetSettlingTime = CTuner_GetSettlingTime;
		pTuner->m_iTuner.GetTuningStrategy = CTuner_GetTuningStrategy;
		pTuner->m_iTuner.GetTuningStrategy2 = CTuner_GetTuningStrategy2;
		pTuner->m_iTuner.GetFrequency = CTuner_GetFrequency;
		pTuner->m_iTuner.GetLastFrequency = CTuner_GetLastFrequency;
		pTuner->m_iTuner.GetTuningFlags = CTuner_GetTuningFlags;
		pTuner->m_iTuner.GetVideoSubChannel = CTuner_GetVideoSubChannel;
		pTuner->m_iTuner.GetAudioSubChannel = CTuner_GetAudioSubChannel;
		pTuner->m_iTuner.GetChannel = CTuner_GetChannel;
		pTuner->m_iTuner.SetLastFrequency = CTuner_SetLastFrequency;
		pTuner->m_iTuner.SetTuningFlags = CTuner_SetTuningFlags;
		pTuner->m_iTuner.SetVideoSubChannel = CTuner_SetVideoSubChannel;
		pTuner->m_iTuner.SetAudioSubChannel = CTuner_SetAudioSubChannel;
		pTuner->m_iTuner.SetChannel = CTuner_SetChannel;
		pTuner->m_iTuner.GetInputIndex = CTuner_GetInputIndex;

		// initialize members
		//
		pTuner->m_pDeviceCallback = QP_NULL;
		pTuner->m_callbackContext = QP_NULL;

		pTuner->m_ulCurrentFrequency = 55250000L;
		pTuner->m_ulNumberOfStandards = 1;
		pTuner->m_ulStandardsSupported = VIDEO_STANDARD_NTSC;
		pTuner->m_ulCurrentStandard = VIDEO_STANDARD_NTSC;
		pTuner->m_ulNumberOfInputs = 1;
		pTuner->m_ulMinFrequency = 55250000L;
		pTuner->m_ulMaxFrequency = 801250000L;
		pTuner->m_ulIntermediateFrequency = 45750000L;
		pTuner->m_ulIntermediateFrequencyJapen = 58750000L;
		pTuner->m_ulSupportedModes = QPPROPERTY_TUNER_MODE_TV;
		pTuner->m_ulTunerMode = QPPROPERTY_TUNER_MODE_TV;
		pTuner->m_ulTuningGranularity = 62500L;
		pTuner->m_ulSettlingTime = 100;
		pTuner->m_ulStrategy = QP_TUNER_STRATEGY_DRIVER_TUNES;
		pTuner->m_ulTunerInput = 0L;
		
		pTuner->m_ulFMCurrentFrequency = 87500000L;
		pTuner->m_ulFMStandardsSupported = 0;
		pTuner->m_ulFMMinFrequency = 87500000L;
		pTuner->m_ulFMMaxFrequency = 108000000L;
		pTuner->m_ulFMIntermediateFrequency = 10700000L;
		pTuner->m_ulFMTuningGranularity = 50000L;
		pTuner->m_ulFMSettlingTime = 100;
		pTuner->m_ulFMStrategy = QP_TUNER_STRATEGY_DRIVER_TUNES;
		
		pTuner->m_ulLastFrequency = 55250000L;
		pTuner->m_ulTuningFlags = 0;
		pTuner->m_ulVideoSubChannel = 0;
		pTuner->m_ulAudioSubChannel = 0;
		pTuner->m_ulChannel = 0;
		pTuner->m_ulCurrentCountry = 1;
		
		pTuner->m_ulSoundCarrierFreq = TUNER_SIF_4_5_MHZ;
	}

	return (pTuner);
}



// destructor
//
void CTuner_Destructor(CTuner *This)
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTuner_Destructor()\n"
				 ));

	if (CObject_IsInitialized(&This->m_Object))
		QPGenericDevice_Release(&This->m_iTuner);

	CObject_Destructor(&This->m_Object);
}

