///////////////////////////////////////////////////////////////////////////////
//        
//
// Ctvaudio.c
//
// Description: 
//
//  TV Audio implementation
// 
// History:
//	Aug 28, 2006	(dt) created
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#include "ctvaudio.h"
#include "util.h"
#include "qpdbgmgr.h"

extern TQP_DbgMCompID	g_QPPeripheralDBGCompID;


// IGenericDevice
//
static EQP_Errors CTVAudio_InitDevice(ITVAudio *This_p, 
									  QP_DEVICE_CALLBACK pFuncCallback,
									  TQP_PVOID context,
									  TQP_PVOID pInitData
									  )
{
	CTVAudio	*This =	GET_INHERITED_OBJECT(CTVAudio, m_iTvAudio, This_p);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTVAudio_InitDevice()\n"
				 ));

	This->m_pDeviceCallback = pFuncCallback;
	This->m_callbackContext = context;
	CObject_Init(&This->m_Object);
	return (QPERR_SUCCESS);
}



static EQP_Errors CTVAudio_Release(ITVAudio *This_p)
{
	CTVAudio	*This =	GET_INHERITED_OBJECT(CTVAudio, m_iTvAudio, This_p);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTVAudio_Release()\n"
				 ));

	This->m_pDeviceCallback = QP_NULL;
	This->m_callbackContext = QP_NULL;
	CObject_Done(&This->m_Object);
	return (QPERR_SUCCESS);
}



static EQP_Errors CTVAudio_Reset(ITVAudio *This_p) 
{
	return (QPERR_SUCCESS);
}



static EQP_Errors CTVAudio_Set(ITVAudio *This_p, 
							   TQP_REFGUID guid,
							   TQP_OBJECT_HANDLE hIndex,
							   TQP_UINT32 dwCode,
							   TQP_PVOID pInput,
							   TQP_PVOID pOutput,
							   TQP_UINT32 dwSize
							   )
{
	return (QPERR_NOTIMPL);
}



static EQP_Errors CTVAudio_Get(ITVAudio *This_p, 
							   TQP_REFGUID guid,
							   TQP_OBJECT_HANDLE hIndex,
							   TQP_UINT32 dwCode,
							   TQP_PVOID pInput,
							   TQP_PVOID pOutput,
							   PTQP_UINT32 pdwSizeGot
							   )
{
	return (QPERR_NOTIMPL);
}



// ITVAudio
//
static EQP_Errors CTVAudio_SetMode(ITVAudio *This_p, 
								   TQP_UINT32 ulMode
								   ) 
{	
	CTVAudio	*This =	GET_INHERITED_OBJECT(CTVAudio, m_iTvAudio, This_p);

	This->m_ulMode = ulMode;
	return (QPERR_SUCCESS);
}



static EQP_Errors CTVAudio_GetMode(ITVAudio *This_p, 
								   PTQP_UINT32 pulMode
								   )
{	
	CTVAudio	*This =	GET_INHERITED_OBJECT(CTVAudio, m_iTvAudio, This_p);

	*pulMode = This->m_ulMode;
	return (QPERR_SUCCESS);
}



static EQP_Errors CTVAudio_GetSupportedModes(ITVAudio *This_p, 
											 PTQP_UINT32 pulMode
											 )
{	
	CTVAudio	*This =	GET_INHERITED_OBJECT(CTVAudio, m_iTvAudio, This_p);

	*pulMode = This->m_ulSupportedModes;
	return (QPERR_SUCCESS);
}



static EQP_Errors CTVAudio_GetAvailableModes(ITVAudio *This_p, 
											 PTQP_UINT32 pulMode
											 )
{	
	CTVAudio	*This =	GET_INHERITED_OBJECT(CTVAudio, m_iTvAudio, This_p);

	*pulMode = This->m_ulAvailableModes;
	return (QPERR_SUCCESS);
}



static EQP_Errors CTVAudio_SelectInput(ITVAudio *This_p, 
									   ETVAUDIO_INPUT eInput
									   ) 
{
	CTVAudio	*This =	GET_INHERITED_OBJECT(CTVAudio, m_iTvAudio, This_p);

	This->m_audioSource = eInput;
	return (QPERR_SUCCESS);
}



static EQP_Errors CTVAudio_GetInput(ITVAudio *This_p, 
									PETVAUDIO_INPUT pInput
									) 
{
	CTVAudio	*This =	GET_INHERITED_OBJECT(CTVAudio, m_iTvAudio, This_p);

	*pInput = This->m_audioSource; 
	return (QPERR_SUCCESS);
}



static EQP_Errors CTVAudio_MuteAudio(ITVAudio *This_p)
{
	CTVAudio	*This =	GET_INHERITED_OBJECT(CTVAudio, m_iTvAudio, This_p);

	if (!This->m_bMuted)
	{
		This->m_bMuted = QP_TRUE;
	}

	return (QPERR_SUCCESS);
}



static EQP_Errors CTVAudio_UnmuteAudio(ITVAudio *This_p)
{
	CTVAudio	*This =	GET_INHERITED_OBJECT(CTVAudio, m_iTvAudio, This_p);

	if (This->m_bMuted)
	{
		This->m_bMuted = QP_FALSE;
	}

	return (QPERR_SUCCESS);
}



// constructor
//
CTVAudio * CTVAudio_Constructor(CTVAudio *pTVAudio, 
								CObject *pParent,
								TQP_UINT32 dwAttributes
								)
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTVAudio_Constructor()\n"
				 ));

	if (pTVAudio)
	{
		CObject_Constructor(&pTVAudio->m_Object, 
							pParent, 
							dwAttributes
							);
		// fill in the function table
		//

		// IGenericDevice
		pTVAudio->m_iTvAudio.InitDevice = CTVAudio_InitDevice;
		pTVAudio->m_iTvAudio.Release = CTVAudio_Release;
		pTVAudio->m_iTvAudio.Reset = CTVAudio_Reset;
		pTVAudio->m_iTvAudio.Set = CTVAudio_Set;
		pTVAudio->m_iTvAudio.Get = CTVAudio_Get;

		// ITVAudio
		pTVAudio->m_iTvAudio.SetMode = CTVAudio_SetMode;
		pTVAudio->m_iTvAudio.GetMode = CTVAudio_GetMode;
		pTVAudio->m_iTvAudio.GetSupportedModes = CTVAudio_GetSupportedModes;
		pTVAudio->m_iTvAudio.GetAvailableModes = CTVAudio_GetAvailableModes;
		pTVAudio->m_iTvAudio.SelectInput = CTVAudio_SelectInput;
		pTVAudio->m_iTvAudio.GetInput = CTVAudio_GetInput;
		pTVAudio->m_iTvAudio.MuteAudio = CTVAudio_MuteAudio;
		pTVAudio->m_iTvAudio.UnmuteAudio = CTVAudio_UnmuteAudio;

		// initialize members
		//
		pTVAudio->m_pDeviceCallback = QP_NULL;
		pTVAudio->m_callbackContext = QP_NULL;

		pTVAudio->m_bMuted = QP_FALSE;

		pTVAudio->m_ulSupportedModes = TVAUDIO_MODE_MONO |
									   TVAUDIO_MODE_STEREO |
									   TVAUDIO_MODE_LANG_A |
									   TVAUDIO_MODE_LANG_B
									   ;
		pTVAudio->m_ulAvailableModes = TVAUDIO_MODE_MONO |
									   TVAUDIO_MODE_STEREO |
									   TVAUDIO_MODE_LANG_A
									   ;
		pTVAudio->m_ulMode = TVAUDIO_MODE_MONO |
							 TVAUDIO_MODE_LANG_A
							 ;
	}

	return (pTVAudio);
}



// destructor
//
void CTVAudio_Destructor(CTVAudio *This)
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTVAudio_Destructor()\n"
				 ));

	if (CObject_IsInitialized(&This->m_Object))
		QPGenericDevice_Release(&This->m_iTvAudio);

	CObject_Destructor(&This->m_Object);
}

