///////////////////////////////////////////////////////////////////////////////
//        
//
// ctvp7002.c
//
// Description: 
//   TVP7002 implementation
// 
// History:
//	 June 14, 2010 created
//
// Authors: Jack Lee
//
///////////////////////////////////////////////////////////////////////////////


#include "ctvp7002.h"
#include "cit6603.h"
#include "ctw9910.h"
#include "qptimemgr.h"
#include "qpdbgmgr.h"
#include "util.h"
#include "cqlcodeclib.h"
#include "cqlcodec.h"


/////////////////////////////////////////////////////////////////////////////
//
//

#define	TVP7002_BRIGHTNESS_MIN				0x00   // not supported
#define	TVP7002_BRIGHTNESS_MAX				0xFF   // not supported
#define	TVP7002_BRIGHTNESS_DEFAULT			0x80   // not supported

#define	TVP7002_CONTRAST_MIN				0x00   // not supported
#define	TVP7002_CONTRAST_MAX				0x7F   // not supported
#define	TVP7002_CONTRAST_DEFAULT			0x44   // not supported

#define	TVP7002_SATURATION_MIN				0x00   // not supported
#define	TVP7002_SATURATION_MAX				0x7F   // not supported
#define	TVP7002_SATURATION_DEFAULT			0x40   // not supported

#define	TVP7002_HUE_MIN						(-128)	// 0x80   // not supported
#define	TVP7002_HUE_MAX						(+127)	// 0x7F   // not supported
#define	TVP7002_HUE_DEFAULT					0x00   // not supported

#define	TVP7002_SHARPNESS_MIN				0      // not supported
#define	TVP7002_SHARPNESS_MAX				15     // not supported
#define	TVP7002_SHARPNESS_DEFAULT			8      // not supported


extern TQP_DbgMCompID	g_QPPeripheralDBGCompID;

#if 0
static TQP_UINT8 dwTVP7002RegVals_1024x768x60Hz[] = 
{
 	// 1024_768_60Hz
	TVP7002_DEVICE_ADDR, 0x01, 0x54, // H-PLL FEEDBACK DIVIDER MSB  
	TVP7002_DEVICE_ADDR, 0x02, 0x00, // H-PLL FEEDBACK DIVIDER LSB  
	TVP7002_DEVICE_ADDR, 0x03, 0x58, // H-PLL CONTROL               
	TVP7002_DEVICE_ADDR, 0x04, 0x80, // H-PLL PHASE SELECT          
	TVP7002_DEVICE_ADDR, 0x05, 0x06, // CLAMP START                 
	TVP7002_DEVICE_ADDR, 0x06, 0x10, // CLAMP WIDTH                 
	TVP7002_DEVICE_ADDR, 0x07, 0x60, // HSOUT OUTPUT WIDTH          
	TVP7002_DEVICE_ADDR, 0x08, 0x80, // BLU FINE GAIN            
	TVP7002_DEVICE_ADDR, 0x09, 0x80, // GRN FINE GAIN            
	TVP7002_DEVICE_ADDR, 0x0A, 0x80, // RED FINE GAIN            
	TVP7002_DEVICE_ADDR, 0x0B, 0x80, // BLU FINE OFFSET          
	TVP7002_DEVICE_ADDR, 0x0C, 0x80, // GRN FINE OFFSET          
	TVP7002_DEVICE_ADDR, 0x0D, 0x80, // RED FINE OFFSET          
	TVP7002_DEVICE_ADDR, 0x0E, 0x24, // SYNC CONTROL 1              
	TVP7002_DEVICE_ADDR, 0x0F, 0x2A, // H-PLL AND CLAMP CONTROL     
	TVP7002_DEVICE_ADDR, 0x10, 0x58, // SYNC ON GREEN THRESHOLD     
	TVP7002_DEVICE_ADDR, 0x11, 0x40, // SYNC SEPERATOR THRESHOLD    
	TVP7002_DEVICE_ADDR, 0x12, 0x01, // H-PLL PRE-COAST             
	TVP7002_DEVICE_ADDR, 0x13, 0x00, // H-PLL POST-COAST            
	TVP7002_DEVICE_ADDR, 0x15, 0x02, // OUTPUT FORMATTER            
	TVP7002_DEVICE_ADDR, 0x16, 0x11, // MISC CONTROL 1              
	TVP7002_DEVICE_ADDR, 0x17, 0x10, // MISC CONTROL 2              
	TVP7002_DEVICE_ADDR, 0x18, 0x11, // MISC CONTROL 3              
	TVP7002_DEVICE_ADDR, 0x19, 0xEA, // INPUT MUX SELECT 1          
	TVP7002_DEVICE_ADDR, 0x1A, 0xC2, // INPUT MUX SELECT 2          
	TVP7002_DEVICE_ADDR, 0x1B, 0x77, // BLU AND GRN COARSE GAIN  
	TVP7002_DEVICE_ADDR, 0x1C, 0x07, // RED COARSE GAIN          
	TVP7002_DEVICE_ADDR, 0x1D, 0x00, // FINE OFFSET LSB             
	TVP7002_DEVICE_ADDR, 0x1E, 0x00, // BLU COARSE OFFSET        
	TVP7002_DEVICE_ADDR, 0x1F, 0x00, // GRN COARSE OFFSET        
	TVP7002_DEVICE_ADDR, 0x20, 0x00, // RED COARSE OFFSET        
	TVP7002_DEVICE_ADDR, 0x21, 0x0D, // HSOUT OUTPUT START          
	TVP7002_DEVICE_ADDR, 0x22, 0x00, // MISC CONTROL 4              
	TVP7002_DEVICE_ADDR, 0x26, 0x80, // AUTO LEVEL CONTROL ENABLE   
	TVP7002_DEVICE_ADDR, 0x28, 0x53, // AUTO LEVEL CONTROL FILTER   
	TVP7002_DEVICE_ADDR, 0x29, 0x08, // RESERVED                
	TVP7002_DEVICE_ADDR, 0x2A, 0x03, // FINE CLAMP CONTROL          
	TVP7002_DEVICE_ADDR, 0x2B, 0x00, // POWER CONTROL               
	TVP7002_DEVICE_ADDR, 0x2C, 0x50, // ADC SETUP                   
	TVP7002_DEVICE_ADDR, 0x2D, 0x00, // COARSE CLAMP CONTROL        
	TVP7002_DEVICE_ADDR, 0x2E, 0x80, // SOG CLAMP CONTROL           
	TVP7002_DEVICE_ADDR, 0x2F, 0x0C, // RGB COARSE CLAMP CONTROL    
	TVP7002_DEVICE_ADDR, 0x30, 0x04, // SOG COARSE CLAMP CONTROL    
	TVP7002_DEVICE_ADDR, 0x31, 0x18, // AUTO LEVEL CONTROL PLACEMENT
	TVP7002_DEVICE_ADDR, 0x34, 0x07, // MACROVISION STRIPPER WIDTH  
	TVP7002_DEVICE_ADDR, 0x35, 0x01, // VSYNC ALIGNMENT             
	TVP7002_DEVICE_ADDR, 0x36, 0x00, // SYNC BYPASS                 
	TVP7002_DEVICE_ADDR, 0x3D, 0x06, // LINE LENGTH TOLERANCE       
	TVP7002_DEVICE_ADDR, 0x3F, 0x0F, // VIDEO BANDWIDTH CONTROL     
	TVP7002_DEVICE_ADDR, 0x40, 0x32, // AVID START PIXEL LSB        
	TVP7002_DEVICE_ADDR, 0x41, 0x01, // AVID START PIXEL MSB        
	TVP7002_DEVICE_ADDR, 0x42, 0x32, // AVID STOP PIXEL LSB         
	TVP7002_DEVICE_ADDR, 0x43, 0x05, // AVID STOP PIXEL MSB         
	TVP7002_DEVICE_ADDR, 0x44, 0x03, // VBLK START LINE OFFSET (F0) 
	TVP7002_DEVICE_ADDR, 0x45, 0x03, // VBLK START LINE OFFSET (F1) 
	TVP7002_DEVICE_ADDR, 0x46, 0x24, // VBLK DURATION (F0)          
	TVP7002_DEVICE_ADDR, 0x47, 0x26, // VBLK DURATION (F1)          
	TVP7002_DEVICE_ADDR, 0x48, 0x00, // F-BIT START LINE OFFSET (F0)
	TVP7002_DEVICE_ADDR, 0x49, 0x00, // F-BIT START LINE OFFSET (F1)
	TVP7002_DEVICE_ADDR, 0x4A, 0xE3, // 1ST CSC COEFFICIENT LSB     
	TVP7002_DEVICE_ADDR, 0x4B, 0x16, // 1ST CSC COEFFICIENT MSB     
	TVP7002_DEVICE_ADDR, 0x4C, 0x4F, // 2ND CSC COEFFICIENT LSB     
	TVP7002_DEVICE_ADDR, 0x4D, 0x02, // 2ND CSC COEFFICIENT MSB     
	TVP7002_DEVICE_ADDR, 0x4E, 0xCE, // 3RD CSC COEFFICIENT LSB     
	TVP7002_DEVICE_ADDR, 0x4F, 0x06, // 3RD CSC COEFFICIENT MSB     
	TVP7002_DEVICE_ADDR, 0x50, 0xAB, // 4TH CSC COEFFICIENT LSB     
	TVP7002_DEVICE_ADDR, 0x51, 0xF3, // 4TH CSC COEFFICIENT MSB     
	TVP7002_DEVICE_ADDR, 0x52, 0x00, // 5TH CSC COEFFICIENT LSB     
	TVP7002_DEVICE_ADDR, 0x53, 0x10, // 5TH CSC COEFFICIENT MSB     
	TVP7002_DEVICE_ADDR, 0x54, 0x55, // 6TH CSC COEFFICIENT LSB     
	TVP7002_DEVICE_ADDR, 0x55, 0xFC, // 6TH CSC COEFFICIENT MSB     
	TVP7002_DEVICE_ADDR, 0x56, 0x78, // 7TH CSC COEFFICIENT LSB     
	TVP7002_DEVICE_ADDR, 0x57, 0xF1, // 7TH CSC COEFFICIENT MSB     
	TVP7002_DEVICE_ADDR, 0x58, 0x88, // 8TH CSC COEFFICIENT LSB     
	TVP7002_DEVICE_ADDR, 0x59, 0xFE, // 8TH CSC COEFFICIENT MSB     
	TVP7002_DEVICE_ADDR, 0x5A, 0x00, // 9TH CSC COEFFICIENT LSB     
	TVP7002_DEVICE_ADDR, 0x5B, 0x10, // 9TH CSC COEFFICIENT MSB    
	0xff, 0xff, 0xff
};
#endif

#if 0
static TQP_UINT8 dwTVP7002RegVals_1280x720x60Hz[] = 
{
 	// 1280_720_60Hz
	TVP7002_DEVICE_ADDR, 0x01, 0x69, // PLL DIVIDE MSB              
	TVP7002_DEVICE_ADDR, 0x02, 0x20, // PLL DIVIDE LSB              
	TVP7002_DEVICE_ADDR, 0x03, 0xA0, // PLL CONTROL                 
	TVP7002_DEVICE_ADDR, 0x04, 0x80, // PHASE SELECT                
	TVP7002_DEVICE_ADDR, 0x05, 0x32, // CLAMP START                 
	TVP7002_DEVICE_ADDR, 0x06, 0x20, // CLAMP WIDTH                 
	TVP7002_DEVICE_ADDR, 0x07, 0x28, // HSYNC OUTPUT WIDTH          
	TVP7002_DEVICE_ADDR, 0x08, 0x00, // BLU FINE GAIN            
	TVP7002_DEVICE_ADDR, 0x09, 0x00, // GRN FINE GAIN            
	TVP7002_DEVICE_ADDR, 0x0A, 0x00, // RED FINE GAIN            
	TVP7002_DEVICE_ADDR, 0x0B, 0x80, // BLU FINE OFFSET          
	TVP7002_DEVICE_ADDR, 0x0C, 0x80, // GRN FINE OFFSET          
	TVP7002_DEVICE_ADDR, 0x0D, 0x80, // RED FINE OFFSET          
	TVP7002_DEVICE_ADDR, 0x0E, 0x24, // SYNC CONTROL                
	TVP7002_DEVICE_ADDR, 0x0F, 0x22, // PLL AND CLAMP CONTROL       
	TVP7002_DEVICE_ADDR, 0x10, 0x58, // SYNC ON GREEN THRESHOLD ;5D    
	TVP7002_DEVICE_ADDR, 0x11, 0x40, // SYNC SEPERATOR THRESHOLD    
	TVP7002_DEVICE_ADDR, 0x12, 0x01, // PRE-COAST                   
	TVP7002_DEVICE_ADDR, 0x13, 0x00, // POST-COAST                  
	TVP7002_DEVICE_ADDR, 0x15, 0x02,// OUTPUT FORMATTER            
	TVP7002_DEVICE_ADDR, 0x16, 0x11, // TEST REGISTER 
	TVP7002_DEVICE_ADDR, 0x17, 0x10, // DATA ENABLE 
	TVP7002_DEVICE_ADDR, 0x18, 0x11, // MISC CONTROL 3 CLOCK POLARITY           
	TVP7002_DEVICE_ADDR, 0x19, 0xEA, // INPUT MUX SELECT 1          
	TVP7002_DEVICE_ADDR, 0x1A, 0xC2, // INPUT MUX SELECT 2          
	TVP7002_DEVICE_ADDR, 0x1B, 0x77, // BLU AND GREEN COARSE GAIN
	TVP7002_DEVICE_ADDR, 0x1C, 0x07, // RED COARSE GAIN          
	TVP7002_DEVICE_ADDR, 0x1D, 0x00, // FINE OFFSET LSB             
	TVP7002_DEVICE_ADDR, 0x1E, 0x10, // BLU COARSE OFFSET        
	TVP7002_DEVICE_ADDR, 0x1F, 0x10, // GRN COARSE OFFSET        
	TVP7002_DEVICE_ADDR, 0x20, 0x10, // RED COARSE OFFSET        
	TVP7002_DEVICE_ADDR, 0x21, 0x42, // HSYNC OUTPUT START          
	TVP7002_DEVICE_ADDR, 0x22, 0x00, // LOOP RESISTOR CONTROL       
	TVP7002_DEVICE_ADDR, 0x26, 0x80, // ALC ENABLE                  
	TVP7002_DEVICE_ADDR, 0x28, 0x53, // ALC FILTER                  
	TVP7002_DEVICE_ADDR, 0x29, 0x08, // RESERVED 	
	TVP7002_DEVICE_ADDR, 0x2A, 0x03, // FINE CLAMP                  
	TVP7002_DEVICE_ADDR, 0x2B, 0x00, // POWER CONTROL               
	TVP7002_DEVICE_ADDR, 0x2C, 0x50, // TRIM CONTROL                
	TVP7002_DEVICE_ADDR, 0x2D, 0x00, // COARSE CLAMP CONTROL        
	TVP7002_DEVICE_ADDR, 0x2E, 0x80, // SOG CLAMP                   
	TVP7002_DEVICE_ADDR, 0x2F, 0x0C, // RGB COARSE CLAMP CONTROL	
	TVP7002_DEVICE_ADDR, 0x30, 0x04, // SOG COARSE CLAMP CONTROL    
	TVP7002_DEVICE_ADDR, 0x31, 0x5A, // ALC PLACEMENT  
	TVP7002_DEVICE_ADDR, 0x34, 0x07, // Macrovision Stripper Width
	TVP7002_DEVICE_ADDR, 0x35, 0x01,
	TVP7002_DEVICE_ADDR, 0x36, 0x00, // Sync Bypass
	TVP7002_DEVICE_ADDR, 0x3D, 0x06, // Line Length Tolerance (Pixel Tolerance)
	TVP7002_DEVICE_ADDR, 0x3F, 0x0F, //Video B/W control
	TVP7002_DEVICE_ADDR, 0x40, 0x32, // AVID Start  313 (300+13)
	TVP7002_DEVICE_ADDR, 0x41, 0x01, // AVID Start
	TVP7002_DEVICE_ADDR, 0x42, 0x39, // AVID Stop  1593 (313 + 1280)
	TVP7002_DEVICE_ADDR, 0x43, 0x06, // AVID Stop
	TVP7002_DEVICE_ADDR, 0x44, 0x05, // VBLK F0 Offset
	TVP7002_DEVICE_ADDR, 0x45, 0x05, // VBLK F1 Offset
	TVP7002_DEVICE_ADDR, 0x46, 0x1D, // VBLK F0 Duration 30 lines
	TVP7002_DEVICE_ADDR, 0x47, 0x1E, // VBLK F1 Duration     
	0xff, 0xff, 0xff
};
#endif 

#if 0
static TQP_UINT8 dwTVP7002RegVals_1920x1080x60iHz[] = 
{
 	// 1920_1080_60iHz
	TVP7002_DEVICE_ADDR, 0x01, 0x89, // H-PLL FEEDBACK DIVIDER MSB  
	TVP7002_DEVICE_ADDR, 0x02, 0x80, // H-PLL FEEDBACK DIVIDER LSB  
	TVP7002_DEVICE_ADDR, 0x03, 0xA0, // H-PLL CONTROL               
	TVP7002_DEVICE_ADDR, 0x04, 0x80, // H-PLL PHASE SELECT          
	TVP7002_DEVICE_ADDR, 0x05, 0x32, // CLAMP START                 
	TVP7002_DEVICE_ADDR, 0x06, 0x20, // CLAMP WIDTH                 
	TVP7002_DEVICE_ADDR, 0x07, 0x60, // HSOUT OUTPUT WIDTH          
	TVP7002_DEVICE_ADDR, 0x08, 0x80, // BLU FINE GAIN            
	TVP7002_DEVICE_ADDR, 0x09, 0x80, // GRN FINE GAIN            
	TVP7002_DEVICE_ADDR, 0x0A, 0x80, // RED FINE GAIN            
	TVP7002_DEVICE_ADDR, 0x0B, 0x80, // BLU FINE OFFSET          
	TVP7002_DEVICE_ADDR, 0x0C, 0x80, // GRN FINE OFFSET          
	TVP7002_DEVICE_ADDR, 0x0D, 0x80, // RED FINE OFFSET          
	TVP7002_DEVICE_ADDR, 0x0E, 0x20, // SYNC CONTROL 1              
	TVP7002_DEVICE_ADDR, 0x0F, 0x22, // H-PLL AND CLAMP CONTROL     
	TVP7002_DEVICE_ADDR, 0x10, 0x58, // SYNC ON GREEN THRESHOLD     
	TVP7002_DEVICE_ADDR, 0x11, 0x40, // SYNC SEPERATOR THRESHOLD    
	TVP7002_DEVICE_ADDR, 0x12, 0x01, // H-PLL PRE-COAST             
	TVP7002_DEVICE_ADDR, 0x13, 0x00, // H-PLL POST-COAST            
	TVP7002_DEVICE_ADDR, 0x15, 0x02, // OUTPUT FORMATTER            
	TVP7002_DEVICE_ADDR, 0x16, 0x11, // MISC CONTROL 1              
	TVP7002_DEVICE_ADDR, 0x17, 0x10, // MISC CONTROL 2              
	TVP7002_DEVICE_ADDR, 0x18, 0x11, // MISC CONTROL 3              
	TVP7002_DEVICE_ADDR, 0x19, 0xEA, // INPUT MUX SELECT 1          
	TVP7002_DEVICE_ADDR, 0x1A, 0xC2, // INPUT MUX SELECT 2          
	TVP7002_DEVICE_ADDR, 0x1B, 0x55, // BLU AND GRN COARSE GAIN  
	TVP7002_DEVICE_ADDR, 0x1C, 0x05, // RED COARSE GAIN          
	TVP7002_DEVICE_ADDR, 0x1D, 0x00, // FINE OFFSET LSB             
	TVP7002_DEVICE_ADDR, 0x1E, 0x00, // BLU COARSE OFFSET        
	TVP7002_DEVICE_ADDR, 0x1F, 0x00, // GRN COARSE OFFSET        
	TVP7002_DEVICE_ADDR, 0x20, 0x00, // RED COARSE OFFSET        
	TVP7002_DEVICE_ADDR, 0x21, 0x39, // HSOUT OUTPUT START          
	TVP7002_DEVICE_ADDR, 0x22, 0x00, // MISC CONTROL 4              
	TVP7002_DEVICE_ADDR, 0x26, 0x80, // AUTO LEVEL CONTROL ENABLE   
	TVP7002_DEVICE_ADDR, 0x28, 0x53, // AUTO LEVEL CONTROL FILTER   
	TVP7002_DEVICE_ADDR, 0x29, 0x08, // RESERVED                
	TVP7002_DEVICE_ADDR, 0x2A, 0x03, // FINE CLAMP CONTROL          
	TVP7002_DEVICE_ADDR, 0x2B, 0x00, // POWER CONTROL               
	TVP7002_DEVICE_ADDR, 0x2C, 0x50, // ADC SETUP                   
	TVP7002_DEVICE_ADDR, 0x2D, 0x00, // COARSE CLAMP CONTROL        
	TVP7002_DEVICE_ADDR, 0x2E, 0x80, // SOG CLAMP CONTROL           
	TVP7002_DEVICE_ADDR, 0x2F, 0x0C, // RGB COARSE CLAMP CONTROL    
	TVP7002_DEVICE_ADDR, 0x30, 0x04, // SOG COARSE CLAMP CONTROL    
	TVP7002_DEVICE_ADDR, 0x31, 0x5A, // AUTO LEVEL CONTROL PLACEMENT
	TVP7002_DEVICE_ADDR, 0x34, 0x07, // MACROVISION STRIPPER WIDTH  
	TVP7002_DEVICE_ADDR, 0x35, 0x01, // VSYNC ALIGNMENT             
	TVP7002_DEVICE_ADDR, 0x36, 0x00, // SYNC BYPASS                 
	TVP7002_DEVICE_ADDR, 0x3D, 0x06, // LINE LENGTH TOLERANCE       
	TVP7002_DEVICE_ADDR, 0x3F, 0x0F, // VIDEO BANDWIDTH CONTROL     
	TVP7002_DEVICE_ADDR, 0x40, 0xE0, // AVID START PIXEL LSB        
	TVP7002_DEVICE_ADDR, 0x41, 0x00, // AVID START PIXEL MSB        
	TVP7002_DEVICE_ADDR, 0x42, 0x70, // AVID STOP PIXEL LSB         
	TVP7002_DEVICE_ADDR, 0x43, 0x08, // AVID STOP PIXEL MSB         
	TVP7002_DEVICE_ADDR, 0x44, 0x02, // VBLK START LINE OFFSET (F0) 
	TVP7002_DEVICE_ADDR, 0x45, 0x02, // VBLK START LINE OFFSET (F1) 
	TVP7002_DEVICE_ADDR, 0x46, 0x16, // VBLK DURATION (F0)          
	TVP7002_DEVICE_ADDR, 0x47, 0x17, // VBLK DURATION (F1)          
	TVP7002_DEVICE_ADDR, 0x48, 0x00, // F-BIT START LINE OFFSET (F0)
	TVP7002_DEVICE_ADDR, 0x49, 0x00, // F-BIT START LINE OFFSET (F1)
	TVP7002_DEVICE_ADDR, 0x4A, 0xE3, // 1ST CSC COEFFICIENT LSB     
	TVP7002_DEVICE_ADDR, 0x4B, 0x16, // 1ST CSC COEFFICIENT MSB     
	TVP7002_DEVICE_ADDR, 0x4C, 0x4F, // 2ND CSC COEFFICIENT LSB     
	TVP7002_DEVICE_ADDR, 0x4D, 0x02, // 2ND CSC COEFFICIENT MSB     
	TVP7002_DEVICE_ADDR, 0x4E, 0xCE, // 3RD CSC COEFFICIENT LSB     
	TVP7002_DEVICE_ADDR, 0x4F, 0x06, // 3RD CSC COEFFICIENT MSB     
	TVP7002_DEVICE_ADDR, 0x50, 0xAB, // 4TH CSC COEFFICIENT LSB     
	TVP7002_DEVICE_ADDR, 0x51, 0xF3, // 4TH CSC COEFFICIENT MSB     
	TVP7002_DEVICE_ADDR, 0x52, 0x00, // 5TH CSC COEFFICIENT LSB     
	TVP7002_DEVICE_ADDR, 0x53, 0x10, // 5TH CSC COEFFICIENT MSB     
	TVP7002_DEVICE_ADDR, 0x54, 0x55, // 6TH CSC COEFFICIENT LSB     
	TVP7002_DEVICE_ADDR, 0x55, 0xFC, // 6TH CSC COEFFICIENT MSB     
	TVP7002_DEVICE_ADDR, 0x56, 0x78, // 7TH CSC COEFFICIENT LSB     
	TVP7002_DEVICE_ADDR, 0x57, 0xF1, // 7TH CSC COEFFICIENT MSB     
	TVP7002_DEVICE_ADDR, 0x58, 0x88, // 8TH CSC COEFFICIENT LSB     
	TVP7002_DEVICE_ADDR, 0x59, 0xFE, // 8TH CSC COEFFICIENT MSB     
	TVP7002_DEVICE_ADDR, 0x5A, 0x00, // 9TH CSC COEFFICIENT LSB     
	TVP7002_DEVICE_ADDR, 0x5B, 0x10, // 9TH CSC COEFFICIENT MSB    
	0xff, 0xff, 0xff
};
#endif

static TQP_UINT8 dwTVP7002RegVals_720x480x60i[] =
{
	TVP7002_DEVICE_ADDR, 0x01, 0x35,
	TVP7002_DEVICE_ADDR, 0x02, 0xa0,
	TVP7002_DEVICE_ADDR, 0x03, 0x18,
	TVP7002_DEVICE_ADDR, 0x04, 0x80,
	TVP7002_DEVICE_ADDR, 0x05, 0x06,
	TVP7002_DEVICE_ADDR, 0x06, 0x10,
	TVP7002_DEVICE_ADDR, 0x07, 0x3e,
//	TVP7002_DEVICE_ADDR, 0x08, 0x00,
//	TVP7002_DEVICE_ADDR, 0x09, 0x00,
//	TVP7002_DEVICE_ADDR, 0x0a, 0x00,
	TVP7002_DEVICE_ADDR, 0x0b, 0x80,
	TVP7002_DEVICE_ADDR, 0x0c, 0x90,
	TVP7002_DEVICE_ADDR, 0x0d, 0x80,
	TVP7002_DEVICE_ADDR, 0x0e, 0x3f,
	TVP7002_DEVICE_ADDR, 0x0f, 0x2e,
	TVP7002_DEVICE_ADDR, 0x10, 0x5d,
	TVP7002_DEVICE_ADDR, 0x11, 0x47,
	TVP7002_DEVICE_ADDR, 0x12, 0x03,
	TVP7002_DEVICE_ADDR, 0x13, 0x03,

	TVP7002_DEVICE_ADDR, 0x15, 0x46,
	TVP7002_DEVICE_ADDR, 0x16, 0x11,
	TVP7002_DEVICE_ADDR, 0x17, 0x10,
	TVP7002_DEVICE_ADDR, 0x18, 0x00,
	TVP7002_DEVICE_ADDR, 0x19, 0x00,
	TVP7002_DEVICE_ADDR, 0x1a, 0x1f,
//	TVP7002_DEVICE_ADDR, 0x1b, 0x77,
//	TVP7002_DEVICE_ADDR, 0x1c, 0x07,
//	TVP7002_DEVICE_ADDR, 0x1d, 0x00,
//	TVP7002_DEVICE_ADDR, 0x1e, 0x10,
//	TVP7002_DEVICE_ADDR, 0x1f, 0x10,
//	TVP7002_DEVICE_ADDR, 0x20, 0x10,
	TVP7002_DEVICE_ADDR, 0x21, 0x27,
	TVP7002_DEVICE_ADDR, 0x22, 0x00,

	TVP7002_DEVICE_ADDR, 0x26, 0x80,

	TVP7002_DEVICE_ADDR, 0x28, 0x53,
//	TVP7002_DEVICE_ADDR, 0x29, 0x08,
	TVP7002_DEVICE_ADDR, 0x2a, 0x87,
	TVP7002_DEVICE_ADDR, 0x2b, 0x00,
	TVP7002_DEVICE_ADDR, 0x2c, 0x50,
	TVP7002_DEVICE_ADDR, 0x2d, 0x00,
	TVP7002_DEVICE_ADDR, 0x2e, 0x80,
//	TVP7002_DEVICE_ADDR, 0x2f, 0x0c,
//	TVP7002_DEVICE_ADDR, 0x30, 0x04,
	TVP7002_DEVICE_ADDR, 0x31, 0x18,

	TVP7002_DEVICE_ADDR, 0x34, 0x83,
	TVP7002_DEVICE_ADDR, 0x35, 0x01,
//	TVP7002_DEVICE_ADDR, 0x36, 0x00,

//	TVP7002_DEVICE_ADDR, 0x3d, 0x06,

	TVP7002_DEVICE_ADDR, 0x3f, 0x0f,
	TVP7002_DEVICE_ADDR, 0x40, 0x95,
	TVP7002_DEVICE_ADDR, 0x41, 0x00,
	TVP7002_DEVICE_ADDR, 0x42, 0x12,
	TVP7002_DEVICE_ADDR, 0x43, 0x00,
	TVP7002_DEVICE_ADDR, 0x44, 0x01,
	TVP7002_DEVICE_ADDR, 0x45, 0x00,
	TVP7002_DEVICE_ADDR, 0x46, 0x14,
	TVP7002_DEVICE_ADDR, 0x47, 0x14,
	TVP7002_DEVICE_ADDR, 0x48, 0x02,
	TVP7002_DEVICE_ADDR, 0x49, 0x01
};

static TQP_UINT8 dwTVP7002RegVals_720x480x60p[] =
{
	TVP7002_DEVICE_ADDR, 0x01, 0x35,
	TVP7002_DEVICE_ADDR, 0x02, 0xa0,
	TVP7002_DEVICE_ADDR, 0x03, 0x18,
	TVP7002_DEVICE_ADDR, 0x04, 0x80,
	TVP7002_DEVICE_ADDR, 0x05, 0x06,
	TVP7002_DEVICE_ADDR, 0x06, 0x10,
	TVP7002_DEVICE_ADDR, 0x07, 0x3e,
//	TVP7002_DEVICE_ADDR, 0x08, 0x00,
//	TVP7002_DEVICE_ADDR, 0x09, 0x00,
//	TVP7002_DEVICE_ADDR, 0x0a, 0x00,
	TVP7002_DEVICE_ADDR, 0x0b, 0x80,
	TVP7002_DEVICE_ADDR, 0x0c, 0x90,
	TVP7002_DEVICE_ADDR, 0x0d, 0x80,
	TVP7002_DEVICE_ADDR, 0x0e, 0x3f,
	TVP7002_DEVICE_ADDR, 0x0f, 0x2e,
	TVP7002_DEVICE_ADDR, 0x10, 0x5d,
	TVP7002_DEVICE_ADDR, 0x11, 0x64,
	TVP7002_DEVICE_ADDR, 0x12, 0x03,
	TVP7002_DEVICE_ADDR, 0x13, 0x03,

	TVP7002_DEVICE_ADDR, 0x15, 0x46,
	TVP7002_DEVICE_ADDR, 0x16, 0x11,
	TVP7002_DEVICE_ADDR, 0x17, 0x10,
	TVP7002_DEVICE_ADDR, 0x18, 0x00,
	TVP7002_DEVICE_ADDR, 0x19, 0x00,
	TVP7002_DEVICE_ADDR, 0x1a, 0xcf,
//	TVP7002_DEVICE_ADDR, 0x1b, 0x77,
//	TVP7002_DEVICE_ADDR, 0x1c, 0x07,
//	TVP7002_DEVICE_ADDR, 0x1d, 0x00,
//	TVP7002_DEVICE_ADDR, 0x1e, 0x10,
//	TVP7002_DEVICE_ADDR, 0x1f, 0x10,
//	TVP7002_DEVICE_ADDR, 0x20, 0x10,
	TVP7002_DEVICE_ADDR, 0x21, 0x22,
	TVP7002_DEVICE_ADDR, 0x22, 0x00,

	TVP7002_DEVICE_ADDR, 0x26, 0x80,

	TVP7002_DEVICE_ADDR, 0x28, 0x53,
//	TVP7002_DEVICE_ADDR, 0x29, 0x08,
	TVP7002_DEVICE_ADDR, 0x2a, 0x87,
	TVP7002_DEVICE_ADDR, 0x2b, 0x00,
	TVP7002_DEVICE_ADDR, 0x2c, 0x50,
	TVP7002_DEVICE_ADDR, 0x2d, 0x00,
	TVP7002_DEVICE_ADDR, 0x2e, 0x80,
//	TVP7002_DEVICE_ADDR, 0x2f, 0x0c,
//	TVP7002_DEVICE_ADDR, 0x30, 0x04,
	TVP7002_DEVICE_ADDR, 0x31, 0x18,

	TVP7002_DEVICE_ADDR, 0x34, 0x43,
	TVP7002_DEVICE_ADDR, 0x35, 0x01,
//	TVP7002_DEVICE_ADDR, 0x36, 0x00,

//	TVP7002_DEVICE_ADDR, 0x3d, 0x06,

	TVP7002_DEVICE_ADDR, 0x3f, 0x01,
	TVP7002_DEVICE_ADDR, 0x40, 0x97,
	TVP7002_DEVICE_ADDR, 0x41, 0x00,
	TVP7002_DEVICE_ADDR, 0x42, 0x12,
	TVP7002_DEVICE_ADDR, 0x43, 0x00,
	TVP7002_DEVICE_ADDR, 0x44, 0x07,
	TVP7002_DEVICE_ADDR, 0x45, 0x07,
	TVP7002_DEVICE_ADDR, 0x46, 0x2c,
	TVP7002_DEVICE_ADDR, 0x47, 0x2c
};

static TQP_UINT8 dwTVP7002RegVals_720x576x50i[] =
{
	TVP7002_DEVICE_ADDR, 0x01, 0x36,
	TVP7002_DEVICE_ADDR, 0x02, 0x00,
	TVP7002_DEVICE_ADDR, 0x03, 0x28,
	TVP7002_DEVICE_ADDR, 0x04, 0x80,
	TVP7002_DEVICE_ADDR, 0x05, 0x06,
	TVP7002_DEVICE_ADDR, 0x06, 0x10,
	TVP7002_DEVICE_ADDR, 0x07, 0x3f,
//	TVP7002_DEVICE_ADDR, 0x08, 0x00,
//	TVP7002_DEVICE_ADDR, 0x09, 0x00,
//	TVP7002_DEVICE_ADDR, 0x0a, 0x00,
	TVP7002_DEVICE_ADDR, 0x0b, 0x80,
	TVP7002_DEVICE_ADDR, 0x0c, 0x90,
	TVP7002_DEVICE_ADDR, 0x0d, 0x80,
	TVP7002_DEVICE_ADDR, 0x0e, 0x3f,
	TVP7002_DEVICE_ADDR, 0x0f, 0x2e,
	TVP7002_DEVICE_ADDR, 0x10, 0x5d,
	TVP7002_DEVICE_ADDR, 0x11, 0x47,
	TVP7002_DEVICE_ADDR, 0x12, 0x03,
	TVP7002_DEVICE_ADDR, 0x13, 0x03,

	TVP7002_DEVICE_ADDR, 0x15, 0x46,
	TVP7002_DEVICE_ADDR, 0x16, 0x11,
	TVP7002_DEVICE_ADDR, 0x17, 0x10,
	TVP7002_DEVICE_ADDR, 0x18, 0x00,
	TVP7002_DEVICE_ADDR, 0x19, 0x00,
	TVP7002_DEVICE_ADDR, 0x1a, 0x1f,
//	TVP7002_DEVICE_ADDR, 0x1b, 0x77,
//	TVP7002_DEVICE_ADDR, 0x1c, 0x07,
//	TVP7002_DEVICE_ADDR, 0x1d, 0x00,
//	TVP7002_DEVICE_ADDR, 0x1e, 0x10,
//	TVP7002_DEVICE_ADDR, 0x1f, 0x10,
//	TVP7002_DEVICE_ADDR, 0x20, 0x10,
	TVP7002_DEVICE_ADDR, 0x21, 0x23,
	TVP7002_DEVICE_ADDR, 0x22, 0x00,

	TVP7002_DEVICE_ADDR, 0x26, 0x80,

	TVP7002_DEVICE_ADDR, 0x28, 0x53,
//	TVP7002_DEVICE_ADDR, 0x29, 0x08,
	TVP7002_DEVICE_ADDR, 0x2a, 0x87,
	TVP7002_DEVICE_ADDR, 0x2b, 0x00,
	TVP7002_DEVICE_ADDR, 0x2c, 0x50,
	TVP7002_DEVICE_ADDR, 0x2d, 0x00,
	TVP7002_DEVICE_ADDR, 0x2e, 0x80,
//	TVP7002_DEVICE_ADDR, 0x2f, 0x0c,
//	TVP7002_DEVICE_ADDR, 0x30, 0x04,
	TVP7002_DEVICE_ADDR, 0x31, 0x18,

	TVP7002_DEVICE_ADDR, 0x34, 0x83,
	TVP7002_DEVICE_ADDR, 0x35, 0x01,
//	TVP7002_DEVICE_ADDR, 0x36, 0x00,

//	TVP7002_DEVICE_ADDR, 0x3d, 0x06,

	TVP7002_DEVICE_ADDR, 0x3f, 0x0f,
	TVP7002_DEVICE_ADDR, 0x40, 0x9d,
	TVP7002_DEVICE_ADDR, 0x41, 0x00,
	TVP7002_DEVICE_ADDR, 0x42, 0x10,
	TVP7002_DEVICE_ADDR, 0x43, 0x00,
	TVP7002_DEVICE_ADDR, 0x44, 0x01,
	TVP7002_DEVICE_ADDR, 0x45, 0x00,
	TVP7002_DEVICE_ADDR, 0x46, 0x17,
	TVP7002_DEVICE_ADDR, 0x47, 0x17
};

static TQP_UINT8 dwTVP7002RegVals_720x576x50p[] =
{
	TVP7002_DEVICE_ADDR, 0x01, 0x36,
	TVP7002_DEVICE_ADDR, 0x02, 0x00,
	TVP7002_DEVICE_ADDR, 0x03, 0x18,
	TVP7002_DEVICE_ADDR, 0x04, 0x80,
	TVP7002_DEVICE_ADDR, 0x05, 0x06,
	TVP7002_DEVICE_ADDR, 0x06, 0x10,
	TVP7002_DEVICE_ADDR, 0x07, 0x40,
//	TVP7002_DEVICE_ADDR, 0x08, 0x00,
//	TVP7002_DEVICE_ADDR, 0x09, 0x00,
//	TVP7002_DEVICE_ADDR, 0x0a, 0x00,
	TVP7002_DEVICE_ADDR, 0x0b, 0x80,
	TVP7002_DEVICE_ADDR, 0x0c, 0x90,
	TVP7002_DEVICE_ADDR, 0x0d, 0x80,
	TVP7002_DEVICE_ADDR, 0x0e, 0x3f,
	TVP7002_DEVICE_ADDR, 0x0f, 0x2e,
	TVP7002_DEVICE_ADDR, 0x10, 0x5d,
	TVP7002_DEVICE_ADDR, 0x11, 0x64,
	TVP7002_DEVICE_ADDR, 0x12, 0x03,
	TVP7002_DEVICE_ADDR, 0x13, 0x03,

	TVP7002_DEVICE_ADDR, 0x15, 0x46,
	TVP7002_DEVICE_ADDR, 0x16, 0x11,
	TVP7002_DEVICE_ADDR, 0x17, 0x10,
	TVP7002_DEVICE_ADDR, 0x18, 0x00,
	TVP7002_DEVICE_ADDR, 0x19, 0x00,
	TVP7002_DEVICE_ADDR, 0x1a, 0x1f,
//	TVP7002_DEVICE_ADDR, 0x1b, 0x77,
//	TVP7002_DEVICE_ADDR, 0x1c, 0x07,
//	TVP7002_DEVICE_ADDR, 0x1d, 0x00,
//	TVP7002_DEVICE_ADDR, 0x1e, 0x10,
//	TVP7002_DEVICE_ADDR, 0x1f, 0x10,
//	TVP7002_DEVICE_ADDR, 0x20, 0x10,
	TVP7002_DEVICE_ADDR, 0x21, 0x27,
	TVP7002_DEVICE_ADDR, 0x22, 0x00,

	TVP7002_DEVICE_ADDR, 0x26, 0x80,

	TVP7002_DEVICE_ADDR, 0x28, 0x53,
//	TVP7002_DEVICE_ADDR, 0x29, 0x08,
	TVP7002_DEVICE_ADDR, 0x2a, 0x87,
	TVP7002_DEVICE_ADDR, 0x2b, 0x00,
	TVP7002_DEVICE_ADDR, 0x2c, 0x50,
	TVP7002_DEVICE_ADDR, 0x2d, 0x00,
	TVP7002_DEVICE_ADDR, 0x2e, 0x80,
//	TVP7002_DEVICE_ADDR, 0x2f, 0x0c,
//	TVP7002_DEVICE_ADDR, 0x30, 0x04,
	TVP7002_DEVICE_ADDR, 0x31, 0x18,

	TVP7002_DEVICE_ADDR, 0x34, 0x43,
	TVP7002_DEVICE_ADDR, 0x35, 0x01,
//	TVP7002_DEVICE_ADDR, 0x36, 0x00,

//	TVP7002_DEVICE_ADDR, 0x3d, 0x06,

	TVP7002_DEVICE_ADDR, 0x3f, 0x01,
	TVP7002_DEVICE_ADDR, 0x40, 0x9f,
	TVP7002_DEVICE_ADDR, 0x41, 0x00,
	TVP7002_DEVICE_ADDR, 0x42, 0x17,
	TVP7002_DEVICE_ADDR, 0x43, 0x00,
	TVP7002_DEVICE_ADDR, 0x44, 0x04,
	TVP7002_DEVICE_ADDR, 0x45, 0x04,
	TVP7002_DEVICE_ADDR, 0x46, 0x31,
	TVP7002_DEVICE_ADDR, 0x47, 0x31
};

static TQP_UINT8 dwTVP7002RegVals_1280x720x50p[] =
{
	TVP7002_DEVICE_ADDR, 0x01, 0x7b,
	TVP7002_DEVICE_ADDR, 0x02, 0xc0,
	TVP7002_DEVICE_ADDR, 0x03, 0x98,
	TVP7002_DEVICE_ADDR, 0x04, 0x80,
	TVP7002_DEVICE_ADDR, 0x05, 0x32,
	TVP7002_DEVICE_ADDR, 0x06, 0x20,
	TVP7002_DEVICE_ADDR, 0x07, 0x28,
//	TVP7002_DEVICE_ADDR, 0x08, 0x00,
//	TVP7002_DEVICE_ADDR, 0x09, 0x00,
//	TVP7002_DEVICE_ADDR, 0x0a, 0x00,
	TVP7002_DEVICE_ADDR, 0x0b, 0x80,
	TVP7002_DEVICE_ADDR, 0x0c, 0x90,
	TVP7002_DEVICE_ADDR, 0x0d, 0x80,
	TVP7002_DEVICE_ADDR, 0x0e, 0x3f,
	TVP7002_DEVICE_ADDR, 0x0f, 0x2e,
	TVP7002_DEVICE_ADDR, 0x10, 0x5d,
	TVP7002_DEVICE_ADDR, 0x11, 0x40,
	TVP7002_DEVICE_ADDR, 0x12, 0x01,
	TVP7002_DEVICE_ADDR, 0x13, 0x00,
       
	TVP7002_DEVICE_ADDR, 0x15, 0x46,
	TVP7002_DEVICE_ADDR, 0x16, 0x11, // 0x16[4], CbCr align
	TVP7002_DEVICE_ADDR, 0x17, 0x10,
	TVP7002_DEVICE_ADDR, 0x18, 0x00, // 0x18[0], CLK POL
	TVP7002_DEVICE_ADDR, 0x19, 0x00,
	TVP7002_DEVICE_ADDR, 0x1a, 0xcf,
//	TVP7002_DEVICE_ADDR, 0x1b, 0x77,
//	TVP7002_DEVICE_ADDR, 0x1c, 0x07,
//	TVP7002_DEVICE_ADDR, 0x1d, 0x00,
//	TVP7002_DEVICE_ADDR, 0x1e, 0x10,
//	TVP7002_DEVICE_ADDR, 0x1f, 0x10,
//	TVP7002_DEVICE_ADDR, 0x20, 0x10,
	TVP7002_DEVICE_ADDR, 0x21, 0x53,
	TVP7002_DEVICE_ADDR, 0x22, 0x00,

	TVP7002_DEVICE_ADDR, 0x26, 0x80,

	TVP7002_DEVICE_ADDR, 0x28, 0x53,
//	TVP7002_DEVICE_ADDR, 0x29, 0x08,
	TVP7002_DEVICE_ADDR, 0x2a, 0x87,
	TVP7002_DEVICE_ADDR, 0x2b, 0x00,
	TVP7002_DEVICE_ADDR, 0x2c, 0x50,
	TVP7002_DEVICE_ADDR, 0x2d, 0x00,
	TVP7002_DEVICE_ADDR, 0x2e, 0x80,
//	TVP7002_DEVICE_ADDR, 0x2f, 0x0c,
//	TVP7002_DEVICE_ADDR, 0x30, 0x04,
	TVP7002_DEVICE_ADDR, 0x31, 0x5a,

	TVP7002_DEVICE_ADDR, 0x34, 0x07,
	TVP7002_DEVICE_ADDR, 0x35, 0x01,
//	TVP7002_DEVICE_ADDR, 0x36, 0x00,

//	TVP7002_DEVICE_ADDR, 0x3d, 0x06,

	TVP7002_DEVICE_ADDR, 0x3f, 0x0f,
	TVP7002_DEVICE_ADDR, 0x40, 0x4b, // AVID Start pixel
	TVP7002_DEVICE_ADDR, 0x41, 0x01,
	TVP7002_DEVICE_ADDR, 0x42, 0x53, // AVID Stop pixel
	TVP7002_DEVICE_ADDR, 0x43, 0x06,
	TVP7002_DEVICE_ADDR, 0x44, 0x05,
	TVP7002_DEVICE_ADDR, 0x45, 0x05,
	TVP7002_DEVICE_ADDR, 0x46, 0x1e,
	TVP7002_DEVICE_ADDR, 0x47, 0x1e
};

static TQP_UINT8 dwTVP7002RegVals_1280x720x60p[] =
{
	TVP7002_DEVICE_ADDR, 0x01, 0x67,
	TVP7002_DEVICE_ADDR, 0x02, 0x20,
	TVP7002_DEVICE_ADDR, 0x03, 0xa0,
	TVP7002_DEVICE_ADDR, 0x04, 0x80,
	TVP7002_DEVICE_ADDR, 0x05, 0x32,
	TVP7002_DEVICE_ADDR, 0x06, 0x20,
	TVP7002_DEVICE_ADDR, 0x07, 0x29,
//	TVP7002_DEVICE_ADDR, 0x08, 0x00,
//	TVP7002_DEVICE_ADDR, 0x09, 0x00,
//	TVP7002_DEVICE_ADDR, 0x0a, 0x00,
	TVP7002_DEVICE_ADDR, 0x0b, 0x80,
	TVP7002_DEVICE_ADDR, 0x0c, 0x90,
	TVP7002_DEVICE_ADDR, 0x0d, 0x80,
	TVP7002_DEVICE_ADDR, 0x0e, 0x3f,
	TVP7002_DEVICE_ADDR, 0x0f, 0x2e,
	TVP7002_DEVICE_ADDR, 0x10, 0x5d,
	TVP7002_DEVICE_ADDR, 0x11, 0x40,
	TVP7002_DEVICE_ADDR, 0x12, 0x01, //0x12, H-PLL Pre-Coast, for default
	TVP7002_DEVICE_ADDR, 0x13, 0x00,

	TVP7002_DEVICE_ADDR, 0x15, 0x46,
	TVP7002_DEVICE_ADDR, 0x16, 0x11, //0x16[4], CbCr align
	TVP7002_DEVICE_ADDR, 0x17, 0x10,
	TVP7002_DEVICE_ADDR, 0x18, 0x00, //0x18[0], CLK POL
	TVP7002_DEVICE_ADDR, 0x19, 0x00,
	TVP7002_DEVICE_ADDR, 0x1a, 0xcf,
//	TVP7002_DEVICE_ADDR, 0x1b, 0x77,
//	TVP7002_DEVICE_ADDR, 0x1c, 0x07,
//	TVP7002_DEVICE_ADDR, 0x1d, 0x00,
//	TVP7002_DEVICE_ADDR, 0x1e, 0x10,
//	TVP7002_DEVICE_ADDR, 0x1f, 0x10,
//	TVP7002_DEVICE_ADDR, 0x20, 0x10,
	TVP7002_DEVICE_ADDR, 0x21, 0x53,
	TVP7002_DEVICE_ADDR, 0x22, 0x00,

	TVP7002_DEVICE_ADDR, 0x26, 0x80,
       
	TVP7002_DEVICE_ADDR, 0x28, 0x53,
//	TVP7002_DEVICE_ADDR, 0x29, 0x08,
	TVP7002_DEVICE_ADDR, 0x2a, 0x87,
	TVP7002_DEVICE_ADDR, 0x2b, 0x00,
	TVP7002_DEVICE_ADDR, 0x2c, 0x50,
	TVP7002_DEVICE_ADDR, 0x2d, 0x00,
	TVP7002_DEVICE_ADDR, 0x2e, 0x80,
//	TVP7002_DEVICE_ADDR, 0x2f, 0x0c,
//	TVP7002_DEVICE_ADDR, 0x30, 0x04,
	TVP7002_DEVICE_ADDR, 0x31, 0x5a,

	TVP7002_DEVICE_ADDR, 0x34, 0x07,
	TVP7002_DEVICE_ADDR, 0x35, 0x01,
//	TVP7002_DEVICE_ADDR, 0x36, 0x00,

//	TVP7002_DEVICE_ADDR, 0x3d, 0x06,

	TVP7002_DEVICE_ADDR, 0x3f, 0x0f,
	TVP7002_DEVICE_ADDR, 0x40, 0x49, //AVID Start pixel
	TVP7002_DEVICE_ADDR, 0x41, 0x01,
	TVP7002_DEVICE_ADDR, 0x42, 0x50, //AVID Stop pixel
	TVP7002_DEVICE_ADDR, 0x43, 0x06,
	TVP7002_DEVICE_ADDR, 0x44, 0x06,
	TVP7002_DEVICE_ADDR, 0x45, 0x06,
	TVP7002_DEVICE_ADDR, 0x46, 0x1e,
	TVP7002_DEVICE_ADDR, 0x47, 0x1e
};

static TQP_UINT8 dwTVP7002RegVals_1920x1080x50i[] = 
{
	TVP7002_DEVICE_ADDR, 0x01, 0xa5,
	TVP7002_DEVICE_ADDR, 0x02, 0x00,
	TVP7002_DEVICE_ADDR, 0x03, 0x90, // H-PLL Control, datasheet Recommend
	TVP7002_DEVICE_ADDR, 0x04, 0x80,
	TVP7002_DEVICE_ADDR, 0x05, 0x32,
	TVP7002_DEVICE_ADDR, 0x06, 0x20,
	TVP7002_DEVICE_ADDR, 0x07, 0x2c,
//	TVP7002_DEVICE_ADDR, 0x08, 0x00,
//	TVP7002_DEVICE_ADDR, 0x09, 0x00,
//	TVP7002_DEVICE_ADDR, 0x0a, 0x00,
	TVP7002_DEVICE_ADDR, 0x0b, 0x80,
	TVP7002_DEVICE_ADDR, 0x0c, 0x90,
	TVP7002_DEVICE_ADDR, 0x0d, 0x80,
	TVP7002_DEVICE_ADDR, 0x0e, 0x3f,
	TVP7002_DEVICE_ADDR, 0x0f, 0x2e,
	TVP7002_DEVICE_ADDR, 0x10, 0x5d,
	TVP7002_DEVICE_ADDR, 0x11, 0x40,
	TVP7002_DEVICE_ADDR, 0x12, 0x00,
	TVP7002_DEVICE_ADDR, 0x13, 0x00,

	TVP7002_DEVICE_ADDR, 0x15, 0x46,
	TVP7002_DEVICE_ADDR, 0x16, 0x11, // 0x16[4], CbCr align
	TVP7002_DEVICE_ADDR, 0x17, 0x10,
	TVP7002_DEVICE_ADDR, 0x18, 0x00, // 0x18[0], CLK POL
	TVP7002_DEVICE_ADDR, 0x19, 0x00,
	TVP7002_DEVICE_ADDR, 0x1a, 0xcf,
//	TVP7002_DEVICE_ADDR, 0x1b, 0x77,
//	TVP7002_DEVICE_ADDR, 0x1c, 0x07,
//	TVP7002_DEVICE_ADDR, 0x1d, 0x00,
//	TVP7002_DEVICE_ADDR, 0x1e, 0x10,
//	TVP7002_DEVICE_ADDR, 0x1f, 0x10,
//	TVP7002_DEVICE_ADDR, 0x20, 0x10,
	TVP7002_DEVICE_ADDR, 0x21, 0x4c, // HSOUT Output Start
	TVP7002_DEVICE_ADDR, 0x22, 0x00,

	TVP7002_DEVICE_ADDR, 0x26, 0x80,

	TVP7002_DEVICE_ADDR, 0x28, 0x53,
//	TVP7002_DEVICE_ADDR, 0x29, 0x08,
	TVP7002_DEVICE_ADDR, 0x2a, 0x07,
	TVP7002_DEVICE_ADDR, 0x2b, 0x00,
	TVP7002_DEVICE_ADDR, 0x2c, 0x50,
	TVP7002_DEVICE_ADDR, 0x2d, 0x00,
	TVP7002_DEVICE_ADDR, 0x2e, 0x80,
//	TVP7002_DEVICE_ADDR, 0x2f, 0x0c,
//	TVP7002_DEVICE_ADDR, 0x30, 0x04,
	TVP7002_DEVICE_ADDR, 0x31, 0x5a,

	TVP7002_DEVICE_ADDR, 0x34, 0x07,
	TVP7002_DEVICE_ADDR, 0x35, 0x01,
//	TVP7002_DEVICE_ADDR, 0x36, 0x00,

//	TVP7002_DEVICE_ADDR, 0x3d, 0x06,

	TVP7002_DEVICE_ADDR, 0x3f, 0x0f,
	TVP7002_DEVICE_ADDR, 0x40, 0x0a, // AVID Start pixel
	TVP7002_DEVICE_ADDR, 0x41, 0x01,
	TVP7002_DEVICE_ADDR, 0x42, 0x92, // AVID Stop pixel
	TVP7002_DEVICE_ADDR, 0x43, 0x08,
	TVP7002_DEVICE_ADDR, 0x44, 0x02,
	TVP7002_DEVICE_ADDR, 0x45, 0x02,
	TVP7002_DEVICE_ADDR, 0x46, 0x15,
	TVP7002_DEVICE_ADDR, 0x47, 0x16,
	TVP7002_DEVICE_ADDR, 0x48, 0x00,
	TVP7002_DEVICE_ADDR, 0x49, 0x00
};	

static TQP_UINT8 dwTVP7002RegVals_1920x1080x50p[] = 
{
	TVP7002_DEVICE_ADDR, 0x01, 0xa5,
	TVP7002_DEVICE_ADDR, 0x02, 0x00,
	TVP7002_DEVICE_ADDR, 0x03, 0xd8,
	TVP7002_DEVICE_ADDR, 0x04, 0x80,
	TVP7002_DEVICE_ADDR, 0x05, 0x32,
	TVP7002_DEVICE_ADDR, 0x06, 0x20,
	TVP7002_DEVICE_ADDR, 0x07, 0x2c,
//	TVP7002_DEVICE_ADDR, 0x08, 0x00,
//	TVP7002_DEVICE_ADDR, 0x09, 0x00,
//	TVP7002_DEVICE_ADDR, 0x0a, 0x00,
	TVP7002_DEVICE_ADDR, 0x0b, 0x80,
	TVP7002_DEVICE_ADDR, 0x0c, 0x90,
	TVP7002_DEVICE_ADDR, 0x0d, 0x80,
	TVP7002_DEVICE_ADDR, 0x0e, 0x3f,
	TVP7002_DEVICE_ADDR, 0x0f, 0x2e,
	TVP7002_DEVICE_ADDR, 0x10, 0x5d,
	TVP7002_DEVICE_ADDR, 0x11, 0x40,
	TVP7002_DEVICE_ADDR, 0x12, 0x00,
	TVP7002_DEVICE_ADDR, 0x13, 0x00,

	TVP7002_DEVICE_ADDR, 0x15, 0x42,
	TVP7002_DEVICE_ADDR, 0x16, 0x11, // 0x16[4], CbCr align
	TVP7002_DEVICE_ADDR, 0x17, 0x10,
	TVP7002_DEVICE_ADDR, 0x18, 0x00, // 0x18[0], CLK POL
	TVP7002_DEVICE_ADDR, 0x19, 0x00,
	TVP7002_DEVICE_ADDR, 0x1a, 0xcf,
//	TVP7002_DEVICE_ADDR, 0x1b, 0x77,
//	TVP7002_DEVICE_ADDR, 0x1c, 0x07,
//	TVP7002_DEVICE_ADDR, 0x1d, 0x00,
//	TVP7002_DEVICE_ADDR, 0x1e, 0x10,
//	TVP7002_DEVICE_ADDR, 0x1f, 0x10,
//	TVP7002_DEVICE_ADDR, 0x20, 0x10,
	TVP7002_DEVICE_ADDR, 0x21, 0x57, // HSOUT Output Start
	TVP7002_DEVICE_ADDR, 0x22, 0x00,

	TVP7002_DEVICE_ADDR, 0x26, 0x80,

	TVP7002_DEVICE_ADDR, 0x28, 0x53,
//	TVP7002_DEVICE_ADDR, 0x29, 0x08,
	TVP7002_DEVICE_ADDR, 0x2a, 0x07,
	TVP7002_DEVICE_ADDR, 0x2b, 0x00,
	TVP7002_DEVICE_ADDR, 0x2c, 0x50,
	TVP7002_DEVICE_ADDR, 0x2d, 0x00,
	TVP7002_DEVICE_ADDR, 0x2e, 0x80,
//	TVP7002_DEVICE_ADDR, 0x2f, 0x0c,
//	TVP7002_DEVICE_ADDR, 0x30, 0x04,
	TVP7002_DEVICE_ADDR, 0x31, 0x5a,

	TVP7002_DEVICE_ADDR, 0x34, 0x07,
	TVP7002_DEVICE_ADDR, 0x35, 0x80,
//	TVP7002_DEVICE_ADDR, 0x36, 0x00,

//	TVP7002_DEVICE_ADDR, 0x3d, 0x06,

	TVP7002_DEVICE_ADDR, 0x3f, 0x0f,
	TVP7002_DEVICE_ADDR, 0x40, 0x07, // AVID Start pixel
	TVP7002_DEVICE_ADDR, 0x41, 0x01,
	TVP7002_DEVICE_ADDR, 0x42, 0x8f, // AVID Stop pixel
	TVP7002_DEVICE_ADDR, 0x43, 0x08,
	TVP7002_DEVICE_ADDR, 0x44, 0x04,
	TVP7002_DEVICE_ADDR, 0x45, 0x04,
	TVP7002_DEVICE_ADDR, 0x46, 0x2a,
	TVP7002_DEVICE_ADDR, 0x47, 0x2a,
	TVP7002_DEVICE_ADDR, 0x48, 0x00,
	TVP7002_DEVICE_ADDR, 0x49, 0x00
};

static TQP_UINT8 dwTVP7002RegVals_1920x1080x60i[] = 
{
	TVP7002_DEVICE_ADDR, 0x01, 0x89,
	TVP7002_DEVICE_ADDR, 0x02, 0x80,
	TVP7002_DEVICE_ADDR, 0x03, 0x98,
	TVP7002_DEVICE_ADDR, 0x04, 0x80,
	TVP7002_DEVICE_ADDR, 0x05, 0x32,
	TVP7002_DEVICE_ADDR, 0x06, 0x20,
	TVP7002_DEVICE_ADDR, 0x07, 0x2c,
//	TVP7002_DEVICE_ADDR, 0x08, 0x00,
//	TVP7002_DEVICE_ADDR, 0x09, 0x00,
//	TVP7002_DEVICE_ADDR, 0x0a, 0x00,
	TVP7002_DEVICE_ADDR, 0x0b, 0x80,
	TVP7002_DEVICE_ADDR, 0x0c, 0x90,
	TVP7002_DEVICE_ADDR, 0x0d, 0x80,
	TVP7002_DEVICE_ADDR, 0x0e, 0x3f,
	TVP7002_DEVICE_ADDR, 0x0f, 0x2e,
	TVP7002_DEVICE_ADDR, 0x10, 0x5d,
	TVP7002_DEVICE_ADDR, 0x11, 0x40,
	TVP7002_DEVICE_ADDR, 0x12, 0x00,
	TVP7002_DEVICE_ADDR, 0x13, 0x00,

	TVP7002_DEVICE_ADDR, 0x15, 0x46,
	TVP7002_DEVICE_ADDR, 0x16, 0x11, //0x16[4], CbCr align
	TVP7002_DEVICE_ADDR, 0x17, 0x10,
	TVP7002_DEVICE_ADDR, 0x18, 0x00, //0x18[0], CLK POL
	TVP7002_DEVICE_ADDR, 0x19, 0x00,
	TVP7002_DEVICE_ADDR, 0x1a, 0xcf,
//	TVP7002_DEVICE_ADDR, 0x1b, 0x77,
//	TVP7002_DEVICE_ADDR, 0x1c, 0x07,
//	TVP7002_DEVICE_ADDR, 0x1d, 0x00,
//	TVP7002_DEVICE_ADDR, 0x1e, 0x10,
//	TVP7002_DEVICE_ADDR, 0x1f, 0x10,
//	TVP7002_DEVICE_ADDR, 0x20, 0x10,
	TVP7002_DEVICE_ADDR, 0x21, 0x55, //HSOUT Output Start
	TVP7002_DEVICE_ADDR, 0x22, 0x00,

	TVP7002_DEVICE_ADDR, 0x26, 0x80,

	TVP7002_DEVICE_ADDR, 0x28, 0x53,
//	TVP7002_DEVICE_ADDR, 0x29, 0x08,
	TVP7002_DEVICE_ADDR, 0x2a, 0x07,
	TVP7002_DEVICE_ADDR, 0x2b, 0x00,
	TVP7002_DEVICE_ADDR, 0x2c, 0x50,
	TVP7002_DEVICE_ADDR, 0x2d, 0x00,
	TVP7002_DEVICE_ADDR, 0x2e, 0x80,
//	TVP7002_DEVICE_ADDR, 0x2f, 0x0c,
//	TVP7002_DEVICE_ADDR, 0x30, 0x04,
	TVP7002_DEVICE_ADDR, 0x31, 0x5a,

	TVP7002_DEVICE_ADDR, 0x34, 0x07,
	TVP7002_DEVICE_ADDR, 0x35, 0x01,
//	TVP7002_DEVICE_ADDR, 0x36, 0x00,

//	TVP7002_DEVICE_ADDR, 0x3d, 0x06,

	TVP7002_DEVICE_ADDR, 0x3f, 0x0f,
	TVP7002_DEVICE_ADDR, 0x40, 0x09, //AVID Start pixel
	TVP7002_DEVICE_ADDR, 0x41, 0x01,
	TVP7002_DEVICE_ADDR, 0x42, 0x91,
	TVP7002_DEVICE_ADDR, 0x43, 0x08,
	TVP7002_DEVICE_ADDR, 0x44, 0x02,
	TVP7002_DEVICE_ADDR, 0x45, 0x02,
	TVP7002_DEVICE_ADDR, 0x46, 0x16,
	TVP7002_DEVICE_ADDR, 0x47, 0x17,
	TVP7002_DEVICE_ADDR, 0x48, 0x00,
	TVP7002_DEVICE_ADDR, 0x49, 0x00
};

static TQP_UINT8 dwTVP7002RegVals_1920x1080x60p[] = 
{
	TVP7002_DEVICE_ADDR, 0x01, 0x89,
	TVP7002_DEVICE_ADDR, 0x02, 0x80,
	TVP7002_DEVICE_ADDR, 0x03, 0xe0,
	TVP7002_DEVICE_ADDR, 0x04, 0x80,
	TVP7002_DEVICE_ADDR, 0x05, 0x32,
	TVP7002_DEVICE_ADDR, 0x06, 0x20,
	TVP7002_DEVICE_ADDR, 0x07, 0x2c,
//	TVP7002_DEVICE_ADDR, 0x08, 0x00,
//	TVP7002_DEVICE_ADDR, 0x09, 0x00,
//	TVP7002_DEVICE_ADDR, 0x0a, 0x00,
	TVP7002_DEVICE_ADDR, 0x0b, 0x80,
	TVP7002_DEVICE_ADDR, 0x0c, 0x90,
	TVP7002_DEVICE_ADDR, 0x0d, 0x80,
	TVP7002_DEVICE_ADDR, 0x0e, 0x3f,
	TVP7002_DEVICE_ADDR, 0x0f, 0x2e,
	TVP7002_DEVICE_ADDR, 0x10, 0x5d,
	TVP7002_DEVICE_ADDR, 0x11, 0x40,
	TVP7002_DEVICE_ADDR, 0x12, 0x00,
	TVP7002_DEVICE_ADDR, 0x13, 0x00,
	TVP7002_DEVICE_ADDR, 0x15, 0x42, // 0x15[2]= 0x0, CbCr order, base on board layout issue
	TVP7002_DEVICE_ADDR, 0x16, 0x11, // 0x16[4]= 0x1, CbCr alignment normal operration
	TVP7002_DEVICE_ADDR, 0x17, 0x10,
	TVP7002_DEVICE_ADDR, 0x18, 0x00, // 0x18[0]= 0x0, CLK POL(PCLK)rising edge, base on board layout issue
	TVP7002_DEVICE_ADDR, 0x19, 0x00,
	TVP7002_DEVICE_ADDR, 0x1a, 0xcf,
//	TVP7002_DEVICE_ADDR, 0x1b, 0x77,
//	TVP7002_DEVICE_ADDR, 0x1c, 0x07,
//	TVP7002_DEVICE_ADDR, 0x1d, 0x00,
//	TVP7002_DEVICE_ADDR, 0x1e, 0x10,
//	TVP7002_DEVICE_ADDR, 0x1f, 0x10,
//	TVP7002_DEVICE_ADDR, 0x20, 0x10,
	TVP7002_DEVICE_ADDR, 0x21, 0x55, // HSOUT Output Start
	TVP7002_DEVICE_ADDR, 0x22, 0x00,

	TVP7002_DEVICE_ADDR, 0x26, 0x80,

	TVP7002_DEVICE_ADDR, 0x28, 0x53,
//	TVP7002_DEVICE_ADDR, 0x29, 0x08,
	TVP7002_DEVICE_ADDR, 0x2a, 0x07,
	TVP7002_DEVICE_ADDR, 0x2b, 0x00,
	TVP7002_DEVICE_ADDR, 0x2c, 0x50,
	TVP7002_DEVICE_ADDR, 0x2d, 0x00,
	TVP7002_DEVICE_ADDR, 0x2e, 0x80,
//	TVP7002_DEVICE_ADDR, 0x2f, 0x0c,
//	TVP7002_DEVICE_ADDR, 0x30, 0x04,
	TVP7002_DEVICE_ADDR, 0x31, 0x5a,

	TVP7002_DEVICE_ADDR, 0x34, 0x07,
	TVP7002_DEVICE_ADDR, 0x35, 0x80,
//	TVP7002_DEVICE_ADDR, 0x36, 0x00,

//	TVP7002_DEVICE_ADDR, 0x3d, 0x06,

	TVP7002_DEVICE_ADDR, 0x3f, 0x0f,
	TVP7002_DEVICE_ADDR, 0x40, 0x07, // AVID Start pixel
	TVP7002_DEVICE_ADDR, 0x41, 0x01,
	TVP7002_DEVICE_ADDR, 0x42, 0x8f, // AVID Stop pixel
	TVP7002_DEVICE_ADDR, 0x43, 0x08,
	TVP7002_DEVICE_ADDR, 0x44, 0x03,
	TVP7002_DEVICE_ADDR, 0x45, 0x03,
	TVP7002_DEVICE_ADDR, 0x46, 0x2c,
	TVP7002_DEVICE_ADDR, 0x47, 0x2c,
	TVP7002_DEVICE_ADDR, 0x48, 0x00,
	TVP7002_DEVICE_ADDR, 0x49, 0x00
};	

/////////////////////////////////////////////////////////////////////////////
//
//


static EQP_Errors CTVP7002_readRegister(CTVP7002 *This,
									   TQP_UINT8 regAddr, 
									   PTQP_UINT8 pValue
									   )
{
    return (II2C_WriteThenRead(This->m_pI2C,
							   This->m_cI2CDevAddr,	//device address
							   1,					//write size
							   &regAddr,			//write buffer
							   1,					//read size
							   pValue				//read buffer
							   ));
}



static EQP_Errors CTVP7002_writeRegister(CTVP7002 *This,
										TQP_UINT8 regAddr, 
										TQP_UINT8 value
										)
{
    TQP_UINT8 buffer[2];

	buffer[0] = regAddr;
	buffer[1] = value;

    return (II2C_Write(This->m_pI2C, 
					   This->m_cI2CDevAddr,	//device address
					   2,					//write size
					   buffer				//write buffer
					   ));
}


static EQP_Errors CTVP7002_WriteBlock(CTVP7002 *This, TQP_UINT8 *pRegTable)
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	
	// get total number of registers to write
	//
	while (0xff != pRegTable[0])
	{
		qpStatus = II2C_Write(This->m_pI2C, 
					   pRegTable[0],	//device address
					   2,				//write size
					   pRegTable+1		//write buffer
					   );										 
		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CTVP7002_WriteBlock() ****** writeRegister failed! status(%d) ******\n",
						 qpStatus
						 ));
			break;
		}

		pRegTable += 3;
	}

	return (qpStatus);
}



/////////////////////////////////////////////////////////////////////////////
//
//


// IGenericDevice
//
EQP_Errors CTVP7002_InitDevice(IVideoDecoder *This_p, 
							   QP_DEVICE_CALLBACK pFuncCallback,
							   TQP_PVOID context,
							   TQP_PVOID pInitData
							   )
{
    TQP_UINT8     buffer[2];
    TQP_UINT8     ucRegAddr;
    TQP_UINT8     ucValue;

    CVideoDecoder *pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
    CQLCodecLib   *pQLCodecLib = GET_INHERITED_OBJECT(CQLCodecLib, m_Object, pVideoDecoder->m_Object.m_pParent);
    CTVP7002      *This = GET_INHERITED_OBJECT(CTVP7002, m_VideoDecoder, pVideoDecoder);
    
    QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
                 "%s(): multiType(%d)\n",
                 __FUNCTION__,
                 pQLCodecLib->m_InitData.vidDecoderInitData.multiType
                 ));
                
    if (CObject_IsInitialized(&pVideoDecoder->m_Object))
    {
        QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
                     "CTVP7002_InitDevice() Already Initialized. Skipping...\n" 
                    ));
      return (QPERR_SUCCESS);
    }
	
    // save callback address and context
    //
    pVideoDecoder->m_pDeviceCallback = pFuncCallback;
    pVideoDecoder->m_callbackContext = context;

	// if has multiple video input types, disable others
	//
    if (MULTIPLE_VIDDECODER_TYPE1 == pQLCodecLib->m_InitData.vidDecoderInitData.multiType) {
    	// power down IT6604
    	buffer[0] = 0x89;
    	buffer[1] = 0xff;
    	II2C_Write(This->m_pI2C, IT6603_DEVICE_ADDR/*device address*/, 2/*write size*/, buffer/*write buffer*/);
    	
    	// power down TW9910
    	buffer[0] = 0x03;
    	buffer[1] = 0x06;
    	II2C_Write(This->m_pI2C, TW9910_DEVICE_ADDR/*device address*/, 2/*write size*/, buffer/*write buffer*/);
				   
		DelayMilliSeconds(100);		   
    }

    // Initialize TVP7002 with the default value
    //
    CTVP7002_WriteBlock(This, dwTVP7002RegVals_1280x720x60p);

    // program the video decoder with settings read from the registry
    //
    QPVideoDecoder_SetVideoSystem(This_p, pVideoDecoder->m_VidStandard);        
    QPVideoDecoder_SelectVideoSource(This_p, pVideoDecoder->m_VidSource);

    QPVideoDecoder_SetBrightness(This_p, pVideoDecoder->m_nBrightness);
    QPVideoDecoder_SetContrast(This_p, pVideoDecoder->m_nContrast);
    QPVideoDecoder_SetSaturation(This_p, pVideoDecoder->m_nSaturation);
    QPVideoDecoder_SetHue(This_p, pVideoDecoder->m_nHue);
    QPVideoDecoder_SetSharpness(This_p, pVideoDecoder->m_nSharpness);

    // CObject Init
    //
    CObject_Init(&pVideoDecoder->m_Object);

    return (QPERR_SUCCESS);
}



// IVideoDecoder
//
EQP_Errors CTVP7002_SetVideoRes(IVideoDecoder *This_p,
							 	TQP_UINT32 dwWid,
				  				TQP_UINT32 dwHgt,
				  				TQP_UINT32 dwScanFmt,
				  				TQP_UINT32 dwFps
				  				)
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CTVP7002		*This = GET_INHERITED_OBJECT(CTVP7002, m_VideoDecoder, pVideoDecoder);
	CQLCodecLib		*pQLCodecLib = GET_INHERITED_OBJECT(CQLCodecLib, m_Object, pVideoDecoder->m_Object.m_pParent);
	CQLCodec 		*pQLCodec = pQLCodecLib->m_pMpgCodec;	
	TQP_UINT8		*pVals;
	EQP_Errors		qpStatus;
	
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CTVP7002_SetVideoRes() : Video Res Set to %dx %d\n",
				 dwWid,
				 dwHgt
				 ));
				 
	//return (QPERR_SUCCESS);
	
	if (MULTIPLE_VIDDECODER_TYPE1 == pQLCodecLib->m_InitData.vidDecoderInitData.multiType) {
		TQP_UINT32 reg;	
		
		// configure audio switch 
		// low for C85343 (TVP7002/TW9910)
		// high for IT6604
		
		// configure gpio 12 to output
		QPHCI_RegisterRead(&pQLCodec->m_hci, 0x610, &reg);
		reg = reg | (1 << 12);
		QPHCI_RegisterWrite(&pQLCodec->m_hci, 0x610, reg);		
		
		// gpio 12 to low state
		QPHCI_RegisterRead(&pQLCodec->m_hci, 0x614, &reg);
		reg = reg & ~(1 << 12);
		QPHCI_RegisterWrite(&pQLCodec->m_hci, 0x614, reg);				
	}
	
	if (((720 == dwWid) && (480 == dwHgt)) || ((640 == dwWid) && (480 == dwHgt))) {
		if (SCAN_FORMAT_INTERLACE == dwScanFmt) {			
			pVals = dwTVP7002RegVals_720x480x60i;
		} else {
			pVals = dwTVP7002RegVals_720x480x60p;
		}
	} else if ((720 == dwWid) && (576 == dwHgt)) {
		if (SCAN_FORMAT_INTERLACE == dwScanFmt) {
			pVals = dwTVP7002RegVals_720x576x50i;
		} else {
			pVals = dwTVP7002RegVals_720x576x50p;
		}
	} else if ((1280 == dwWid) && (720 == dwHgt)) {
		if (50 == dwFps) {
			pVals = dwTVP7002RegVals_1280x720x50p;
		} else {
			pVals = dwTVP7002RegVals_1280x720x60p;
		}
	} else if ((1920 == dwWid) && (1080 == dwHgt)) {
		if (SCAN_FORMAT_INTERLACE == dwScanFmt) {			
			if (25 == dwFps) {
				pVals = dwTVP7002RegVals_1920x1080x50i;
			} else {
				pVals = dwTVP7002RegVals_1920x1080x60i;
			}
		} else {
			if (50 == dwFps) {
				pVals = dwTVP7002RegVals_1920x1080x50p;
			} else {
				pVals = dwTVP7002RegVals_1920x1080x60p;
			}
		}
	} else {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "%s(): Unsupported video format resolution(%dx%d) scan_format(%d) fps(%d)\n", 
					 __FUNCTION__, dwWid, dwHgt, dwScanFmt, dwFps
					 ));
		return (QPERR_PARMS);
	}
	
	qpStatus = CTVP7002_WriteBlock(This, pVals);
	
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "%s(): ****** WriteBlock() failed! ******\n",
					 __FUNCTION__
					 ));
		return (QPERR_FAIL);
	}		
				 
	DelayMilliSeconds(150);
	
	return (QPERR_SUCCESS);
}



// constructor
//
CTVP7002 * CTVP7002_Constructor(CTVP7002 *pTVP7002, 
								CObject *pParent,
								TQP_UINT32 dwAttributes,
								ITuner *pTuner,
								ITVAudio *pTvAudio,
								II2C *pI2C,
								TQP_UINT8 cI2CDevAddr
								)
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTVP7002_Constructor()\n"
				 ));

	if (pTVP7002)
	{
		CVideoDecoder	*pVideoDecoder;

		pVideoDecoder = CVideoDecoder_Constructor(&pTVP7002->m_VideoDecoder, 
												  pParent,
												  dwAttributes,
												  pTuner,
												  pTvAudio
												  );

		if (!pVideoDecoder)
		{
			pTVP7002 = QP_NULL;
		}
		else
		{
			// fill in the function table
			//

			// IGenericDevice
			pTVP7002->m_VideoDecoder.m_iVideoDecoder.InitDevice = CTVP7002_InitDevice;

			// IVideoDecoder
			pTVP7002->m_VideoDecoder.m_iVideoDecoder.SetVideoRes = CTVP7002_SetVideoRes;

			// initialize members
			//
			pTVP7002->m_pI2C = pI2C;
			pTVP7002->m_cI2CDevAddr = cI2CDevAddr;

			// inherited members
			//
			pTVP7002->m_VideoDecoder.m_nBrightnessDef = TVP7002_BRIGHTNESS_DEFAULT;
			pTVP7002->m_VideoDecoder.m_nBrightnessMin = TVP7002_BRIGHTNESS_MIN;
			pTVP7002->m_VideoDecoder.m_nBrightnessMax = TVP7002_BRIGHTNESS_MAX;
			pTVP7002->m_VideoDecoder.m_nBrightness = TVP7002_BRIGHTNESS_DEFAULT;
			pTVP7002->m_VideoDecoder.m_nContrastDef = TVP7002_CONTRAST_DEFAULT; 
			pTVP7002->m_VideoDecoder.m_nContrastMin = TVP7002_CONTRAST_MIN;
			pTVP7002->m_VideoDecoder.m_nContrastMax = TVP7002_CONTRAST_MAX;
			pTVP7002->m_VideoDecoder.m_nContrast = TVP7002_CONTRAST_DEFAULT;
			pTVP7002->m_VideoDecoder.m_nSaturationDef = TVP7002_SATURATION_DEFAULT;
			pTVP7002->m_VideoDecoder.m_nSaturationMin = TVP7002_SATURATION_MIN;
			pTVP7002->m_VideoDecoder.m_nSaturationMax = TVP7002_SATURATION_MAX;
			pTVP7002->m_VideoDecoder.m_nSaturation = TVP7002_SATURATION_DEFAULT;
			pTVP7002->m_VideoDecoder.m_nHueDef = TVP7002_HUE_DEFAULT;
			pTVP7002->m_VideoDecoder.m_nHueMin = TVP7002_HUE_MIN;
			pTVP7002->m_VideoDecoder.m_nHueMax = TVP7002_HUE_MAX;
			pTVP7002->m_VideoDecoder.m_nHue = TVP7002_HUE_DEFAULT;
			pTVP7002->m_VideoDecoder.m_nSharpnessDef = TVP7002_SHARPNESS_DEFAULT;
			pTVP7002->m_VideoDecoder.m_nSharpnessMin = TVP7002_SHARPNESS_MIN;
			pTVP7002->m_VideoDecoder.m_nSharpnessMax = TVP7002_SHARPNESS_MAX;
			pTVP7002->m_VideoDecoder.m_nSharpness = TVP7002_SHARPNESS_DEFAULT;
		}
	}

	return (pTVP7002);
}



// destructor
//
void CTVP7002_Destructor(CTVP7002 *This)
{
    TQP_UINT8     buffer[2];
    TQP_UINT8     ucRegAddr;
    TQP_UINT8     ucValue;
    
    QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTVP7002_Destructor()\n"
				 ));

    CVideoDecoder_Destructor(&This->m_VideoDecoder);
}

