///////////////////////////////////////////////////////////////////////////////
//        
//
// ctvp7002vh.c
//
// Description: 
//   TVP7002VH implementation
// 
// History:
//	 October 10, 2013 created
//
// Authors: Jack Lee
//
///////////////////////////////////////////////////////////////////////////////

#include "ctvp7002vh.h"
#include "qptimemgr.h"
#include "qpdbgmgr.h"
#include "util.h"
#include "cqlcodeclib.h"
#include "cqlcodec.h"


/////////////////////////////////////////////////////////////////////////////
//
//

#define	TVP7002VH_BRIGHTNESS_MIN			0x00   // not supported
#define	TVP7002VH_BRIGHTNESS_MAX			0xFF   // not supported
#define	TVP7002VH_BRIGHTNESS_DEFAULT		0x80   // not supported

#define	TVP7002VH_CONTRAST_MIN				0x00   // not supported
#define	TVP7002VH_CONTRAST_MAX				0x7F   // not supported
#define	TVP7002VH_CONTRAST_DEFAULT			0x44   // not supported

#define	TVP7002VH_SATURATION_MIN			0x00   // not supported
#define	TVP7002VH_SATURATION_MAX			0x7F   // not supported
#define	TVP7002VH_SATURATION_DEFAULT		0x40   // not supported

#define	TVP7002VH_HUE_MIN					(-128)	// 0x80   // not supported
#define	TVP7002VH_HUE_MAX					(+127)	// 0x7F   // not supported
#define	TVP7002VH_HUE_DEFAULT				0x00   // not supported

#define	TVP7002VH_SHARPNESS_MIN				0      // not supported
#define	TVP7002VH_SHARPNESS_MAX				15     // not supported
#define	TVP7002VH_SHARPNESS_DEFAULT			8      // not supported


extern TQP_DbgMCompID	g_QPPeripheralDBGCompID;

static TQP_UINT8 dwTVP7002VHRegVals_1280x720x50Hz[] = 
{
	TVP7002VH_DEVICE_ADDR, 0x01, 0x7b,
	TVP7002VH_DEVICE_ADDR, 0x02, 0xc0,
	TVP7002VH_DEVICE_ADDR, 0x03, 0x98,
	TVP7002VH_DEVICE_ADDR, 0x04, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x05, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x06, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x07, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x08, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x09, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x0a, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x0b, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0c, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0d, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0e, 0x24,
	TVP7002VH_DEVICE_ADDR, 0x0f, 0x2e,
	TVP7002VH_DEVICE_ADDR, 0x10, 0x58,
	TVP7002VH_DEVICE_ADDR, 0x11, 0x40,
	TVP7002VH_DEVICE_ADDR, 0x12, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x13, 0x00,
      
	TVP7002VH_DEVICE_ADDR, 0x15, 0x02,
	TVP7002VH_DEVICE_ADDR, 0x16, 0x11,
	TVP7002VH_DEVICE_ADDR, 0x17, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x18, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x19, 0xc0,
	TVP7002VH_DEVICE_ADDR, 0x1a, 0xc2,
	TVP7002VH_DEVICE_ADDR, 0x1b, 0x77,
	TVP7002VH_DEVICE_ADDR, 0x1c, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x1d, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x1e, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x1f, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x20, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x21, 0x3a,
	TVP7002VH_DEVICE_ADDR, 0x22, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x26, 0x80,
       
	TVP7002VH_DEVICE_ADDR, 0x28, 0x53,
	TVP7002VH_DEVICE_ADDR, 0x29, 0x08,
	TVP7002VH_DEVICE_ADDR, 0x2a, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x2b, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x2c, 0x50,
	TVP7002VH_DEVICE_ADDR, 0x2d, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x2e, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x2f, 0x0c,
	TVP7002VH_DEVICE_ADDR, 0x30, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x31, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x32, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x33, 0x60,
	TVP7002VH_DEVICE_ADDR, 0x34, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x35, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x36, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x3d, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x3e, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x3f, 0x0f,
       
	TVP7002VH_DEVICE_ADDR, 0x40, 0x24,
	TVP7002VH_DEVICE_ADDR, 0x41, 0x01,
       
	TVP7002VH_DEVICE_ADDR, 0x42, 0x24,
	TVP7002VH_DEVICE_ADDR, 0x43, 0x06,
      
	TVP7002VH_DEVICE_ADDR, 0x44, 0x03,
	TVP7002VH_DEVICE_ADDR, 0x45, 0x03,
	TVP7002VH_DEVICE_ADDR, 0x46, 0x1c,
	TVP7002VH_DEVICE_ADDR, 0x47, 0x1c
};

static TQP_UINT8 dwTVP7002VHRegVals_1920x1080x50Hz[] = 
{
	TVP7002VH_DEVICE_ADDR, 0x01, 0xa5,
	TVP7002VH_DEVICE_ADDR, 0x02, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x03, 0xd8,
	TVP7002VH_DEVICE_ADDR, 0x04, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x05, 0x32,
	TVP7002VH_DEVICE_ADDR, 0x06, 0x20,
	TVP7002VH_DEVICE_ADDR, 0x07, 0x2c,
	TVP7002VH_DEVICE_ADDR, 0x08, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x09, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x0a, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x0b, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0c, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0d, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0e, 0x24,
	TVP7002VH_DEVICE_ADDR, 0x0f, 0x2e,
	TVP7002VH_DEVICE_ADDR, 0x10, 0x58,
	TVP7002VH_DEVICE_ADDR, 0x11, 0x40,
	TVP7002VH_DEVICE_ADDR, 0x12, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x13, 0x00,
      
	TVP7002VH_DEVICE_ADDR, 0x15, 0x02,
	TVP7002VH_DEVICE_ADDR, 0x16, 0x11,
	TVP7002VH_DEVICE_ADDR, 0x17, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x18, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x19, 0xc0,
	TVP7002VH_DEVICE_ADDR, 0x1a, 0xc2,
	TVP7002VH_DEVICE_ADDR, 0x1b, 0x77,
	TVP7002VH_DEVICE_ADDR, 0x1c, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x1d, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x1e, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x1f, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x20, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x21, 0x57,
	TVP7002VH_DEVICE_ADDR, 0x22, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x26, 0x80,
       
	TVP7002VH_DEVICE_ADDR, 0x28, 0x53,
	TVP7002VH_DEVICE_ADDR, 0x29, 0x08,
	TVP7002VH_DEVICE_ADDR, 0x2a, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x2b, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x2c, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x2d, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x2e, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x2f, 0x0c,
	TVP7002VH_DEVICE_ADDR, 0x30, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x31, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x32, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x33, 0x60,
	TVP7002VH_DEVICE_ADDR, 0x34, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x35, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x36, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x3d, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x3e, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x3f, 0x0f,
       
	TVP7002VH_DEVICE_ADDR, 0x40, 0xdd,
	TVP7002VH_DEVICE_ADDR, 0x41, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x42, 0x60,
	TVP7002VH_DEVICE_ADDR, 0x43, 0x08,
      
	TVP7002VH_DEVICE_ADDR, 0x44, 0x03,
	TVP7002VH_DEVICE_ADDR, 0x45, 0x03,
	TVP7002VH_DEVICE_ADDR, 0x46, 0x2b,
	TVP7002VH_DEVICE_ADDR, 0x47, 0x2b
};

static TQP_UINT8 dwTVP7002VHRegVals_720x480x60Hz[] = 
{
	TVP7002VH_DEVICE_ADDR, 0x01, 0x35,
	TVP7002VH_DEVICE_ADDR, 0x02, 0xa0,
	TVP7002VH_DEVICE_ADDR, 0x03, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x04, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x05, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x06, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x07, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x08, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x09, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x0a, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x0b, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0c, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0d, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0e, 0x24,
	TVP7002VH_DEVICE_ADDR, 0x0f, 0x2e,
	TVP7002VH_DEVICE_ADDR, 0x10, 0x58,
	TVP7002VH_DEVICE_ADDR, 0x11, 0x64,
	TVP7002VH_DEVICE_ADDR, 0x12, 0x03,
	TVP7002VH_DEVICE_ADDR, 0x13, 0x03,
      
	TVP7002VH_DEVICE_ADDR, 0x15, 0x02,
	TVP7002VH_DEVICE_ADDR, 0x16, 0x11,
	TVP7002VH_DEVICE_ADDR, 0x17, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x18, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x19, 0xc0,
	TVP7002VH_DEVICE_ADDR, 0x1a, 0xc2,
	TVP7002VH_DEVICE_ADDR, 0x1b, 0x77,
	TVP7002VH_DEVICE_ADDR, 0x1c, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x1d, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x1e, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x1f, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x20, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x21, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x22, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x26, 0x80,
       
	TVP7002VH_DEVICE_ADDR, 0x28, 0x53,
	TVP7002VH_DEVICE_ADDR, 0x29, 0x08,
	TVP7002VH_DEVICE_ADDR, 0x2a, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x2b, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x2c, 0x50,
	TVP7002VH_DEVICE_ADDR, 0x2d, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x2e, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x2f, 0x0c,
	TVP7002VH_DEVICE_ADDR, 0x30, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x31, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x32, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x33, 0x60,
	TVP7002VH_DEVICE_ADDR, 0x34, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x35, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x36, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x3d, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x3e, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x3f, 0x0f,
       
	TVP7002VH_DEVICE_ADDR, 0x40, 0x9a,
	TVP7002VH_DEVICE_ADDR, 0x41, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x42, 0x12,
	TVP7002VH_DEVICE_ADDR, 0x43, 0x00,
      
	TVP7002VH_DEVICE_ADDR, 0x44, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x45, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x46, 0x2b,
	TVP7002VH_DEVICE_ADDR, 0x47, 0x2b
};

static TQP_UINT8 dwTVP7002VHRegVals_720x576x50Hz[] = 
{
	TVP7002VH_DEVICE_ADDR, 0x01, 0x36,
	TVP7002VH_DEVICE_ADDR, 0x02, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x03, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x04, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x05, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x06, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x07, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x08, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x09, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x0a, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x0b, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0c, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0d, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0e, 0x24,
	TVP7002VH_DEVICE_ADDR, 0x0f, 0x2e,
	TVP7002VH_DEVICE_ADDR, 0x10, 0x58,
	TVP7002VH_DEVICE_ADDR, 0x11, 0x64,
	TVP7002VH_DEVICE_ADDR, 0x12, 0x03,
	TVP7002VH_DEVICE_ADDR, 0x13, 0x03,
      
	TVP7002VH_DEVICE_ADDR, 0x15, 0x02,
	TVP7002VH_DEVICE_ADDR, 0x16, 0x11,
	TVP7002VH_DEVICE_ADDR, 0x17, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x18, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x19, 0xc0,
	TVP7002VH_DEVICE_ADDR, 0x1a, 0xc2,
	TVP7002VH_DEVICE_ADDR, 0x1b, 0x77,
	TVP7002VH_DEVICE_ADDR, 0x1c, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x1d, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x1e, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x1f, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x20, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x21, 0x20,
	TVP7002VH_DEVICE_ADDR, 0x22, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x26, 0x80,
       
	TVP7002VH_DEVICE_ADDR, 0x28, 0x53,
	TVP7002VH_DEVICE_ADDR, 0x29, 0x08,
	TVP7002VH_DEVICE_ADDR, 0x2a, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x2b, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x2c, 0x50,
	TVP7002VH_DEVICE_ADDR, 0x2d, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x2e, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x2f, 0x0c,
	TVP7002VH_DEVICE_ADDR, 0x30, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x31, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x32, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x33, 0x60,
	TVP7002VH_DEVICE_ADDR, 0x34, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x35, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x36, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x3d, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x3e, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x3f, 0x0f,
       
	TVP7002VH_DEVICE_ADDR, 0x40, 0xa3,
	TVP7002VH_DEVICE_ADDR, 0x41, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x42, 0x17,
	TVP7002VH_DEVICE_ADDR, 0x43, 0x00,
      
	TVP7002VH_DEVICE_ADDR, 0x44, 0x02,
	TVP7002VH_DEVICE_ADDR, 0x45, 0x02,
	TVP7002VH_DEVICE_ADDR, 0x46, 0x2e,
	TVP7002VH_DEVICE_ADDR, 0x47, 0x2e
};

static TQP_UINT8 dwTVP7002VHRegVals_640x480x60Hz[] = 
{
	TVP7002VH_DEVICE_ADDR, 0x01, 0x32,
	TVP7002VH_DEVICE_ADDR, 0x02, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x03, 0x20,
	TVP7002VH_DEVICE_ADDR, 0x04, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x05, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x06, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x07, 0x60,
	TVP7002VH_DEVICE_ADDR, 0x08, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x09, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x0a, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x0b, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0c, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0d, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0e, 0x24,
	TVP7002VH_DEVICE_ADDR, 0x0f, 0x2e,
	TVP7002VH_DEVICE_ADDR, 0x10, 0x58,
	TVP7002VH_DEVICE_ADDR, 0x11, 0x40,
	TVP7002VH_DEVICE_ADDR, 0x12, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x13, 0x00,
      
	TVP7002VH_DEVICE_ADDR, 0x15, 0x02,
	TVP7002VH_DEVICE_ADDR, 0x16, 0x11,
	TVP7002VH_DEVICE_ADDR, 0x17, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x18, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x19, 0xc0,
	TVP7002VH_DEVICE_ADDR, 0x1a, 0xc2,
	TVP7002VH_DEVICE_ADDR, 0x1b, 0x77,
	TVP7002VH_DEVICE_ADDR, 0x1c, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x1d, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x1e, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x1f, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x20, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x21, 0x3a,
	TVP7002VH_DEVICE_ADDR, 0x22, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x26, 0x80,
       
	TVP7002VH_DEVICE_ADDR, 0x28, 0x53,
	TVP7002VH_DEVICE_ADDR, 0x29, 0x08,
	TVP7002VH_DEVICE_ADDR, 0x2a, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x2b, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x2c, 0x50,
	TVP7002VH_DEVICE_ADDR, 0x2d, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x2e, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x2f, 0x0c,
	TVP7002VH_DEVICE_ADDR, 0x30, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x31, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x32, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x33, 0x60,
	TVP7002VH_DEVICE_ADDR, 0x34, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x35, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x36, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x3d, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x3e, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x3f, 0x0f,
       
	TVP7002VH_DEVICE_ADDR, 0x40, 0xb0,
	TVP7002VH_DEVICE_ADDR, 0x41, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x42, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x43, 0x00,
      
	TVP7002VH_DEVICE_ADDR, 0x44, 0x08,
	TVP7002VH_DEVICE_ADDR, 0x45, 0x08,
	TVP7002VH_DEVICE_ADDR, 0x46, 0x2b,
	TVP7002VH_DEVICE_ADDR, 0x47, 0x2b
};

static TQP_UINT8 dwTVP7002VHRegVals_800x600x60Hz[] = 
{
	TVP7002VH_DEVICE_ADDR, 0x01, 0x42,
	TVP7002VH_DEVICE_ADDR, 0x02, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x03, 0x58,
	TVP7002VH_DEVICE_ADDR, 0x04, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x05, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x06, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x07, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x08, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x09, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x0a, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x0b, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0c, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0d, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0e, 0x24,
	TVP7002VH_DEVICE_ADDR, 0x0f, 0x2e,
	TVP7002VH_DEVICE_ADDR, 0x10, 0x58,
	TVP7002VH_DEVICE_ADDR, 0x11, 0x40,
	TVP7002VH_DEVICE_ADDR, 0x12, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x13, 0x00,
      
	TVP7002VH_DEVICE_ADDR, 0x15, 0x02,
	TVP7002VH_DEVICE_ADDR, 0x16, 0x11,
	TVP7002VH_DEVICE_ADDR, 0x17, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x18, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x19, 0xc0,
	TVP7002VH_DEVICE_ADDR, 0x1a, 0xc2,
	TVP7002VH_DEVICE_ADDR, 0x1b, 0x77,
	TVP7002VH_DEVICE_ADDR, 0x1c, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x1d, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x1e, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x1f, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x20, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x21, 0x3a,
	TVP7002VH_DEVICE_ADDR, 0x22, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x26, 0x80,
       
	TVP7002VH_DEVICE_ADDR, 0x28, 0x53,
	TVP7002VH_DEVICE_ADDR, 0x29, 0x08,
	TVP7002VH_DEVICE_ADDR, 0x2a, 0x87,
	TVP7002VH_DEVICE_ADDR, 0x2b, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x2c, 0x50,
	TVP7002VH_DEVICE_ADDR, 0x2d, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x2e, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x2f, 0x0c,
	TVP7002VH_DEVICE_ADDR, 0x30, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x31, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x32, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x33, 0x60,
	TVP7002VH_DEVICE_ADDR, 0x34, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x35, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x36, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x3d, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x3e, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x3f, 0x0f,
       
	TVP7002VH_DEVICE_ADDR, 0x40, 0xf8,
	TVP7002VH_DEVICE_ADDR, 0x41, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x42, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x43, 0x04,
      
	TVP7002VH_DEVICE_ADDR, 0x44, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x45, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x46, 0x1a,
	TVP7002VH_DEVICE_ADDR, 0x47, 0x1a
};

static TQP_UINT8 dwTVP7002VHRegVals_1024x768x60Hz[] = 
{
	TVP7002VH_DEVICE_ADDR, 0x01, 0x54,
	TVP7002VH_DEVICE_ADDR, 0x02, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x03, 0x58,
	TVP7002VH_DEVICE_ADDR, 0x04, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x05, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x06, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x07, 0x88,
	TVP7002VH_DEVICE_ADDR, 0x08, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x09, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x0a, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x0b, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0c, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0d, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0e, 0x24,
	TVP7002VH_DEVICE_ADDR, 0x0f, 0x2e,
	TVP7002VH_DEVICE_ADDR, 0x10, 0x58,
	TVP7002VH_DEVICE_ADDR, 0x11, 0x40,
	TVP7002VH_DEVICE_ADDR, 0x12, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x13, 0x00,
      
	TVP7002VH_DEVICE_ADDR, 0x15, 0x02,
	TVP7002VH_DEVICE_ADDR, 0x16, 0x11,
	TVP7002VH_DEVICE_ADDR, 0x17, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x18, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x19, 0xc0,
	TVP7002VH_DEVICE_ADDR, 0x1a, 0xc2,
	TVP7002VH_DEVICE_ADDR, 0x1b, 0x77,
	TVP7002VH_DEVICE_ADDR, 0x1c, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x1d, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x1e, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x1f, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x20, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x21, 0x3a,
	TVP7002VH_DEVICE_ADDR, 0x22, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x26, 0x80,
       
	TVP7002VH_DEVICE_ADDR, 0x28, 0x53,
	TVP7002VH_DEVICE_ADDR, 0x29, 0x08,
	TVP7002VH_DEVICE_ADDR, 0x2a, 0x87,
	TVP7002VH_DEVICE_ADDR, 0x2b, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x2c, 0x50,
	TVP7002VH_DEVICE_ADDR, 0x2d, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x2e, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x2f, 0x0c,
	TVP7002VH_DEVICE_ADDR, 0x30, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x31, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x32, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x33, 0x60,
	TVP7002VH_DEVICE_ADDR, 0x34, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x35, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x36, 0x00,

	TVP7002VH_DEVICE_ADDR, 0x3d, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x3e, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x3f, 0x0f,
       
	TVP7002VH_DEVICE_ADDR, 0x40, 0x42,
	TVP7002VH_DEVICE_ADDR, 0x41, 0x01,
       
	TVP7002VH_DEVICE_ADDR, 0x42, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x43, 0x00,
      
	TVP7002VH_DEVICE_ADDR, 0x44, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x45, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x46, 0x24,
	TVP7002VH_DEVICE_ADDR, 0x47, 0x24
};

static TQP_UINT8 dwTVP7002VHRegVals_1280x720x60Hz[] = 
{
	TVP7002VH_DEVICE_ADDR, 0x01, 0x67,
	TVP7002VH_DEVICE_ADDR, 0x02, 0x20,
	TVP7002VH_DEVICE_ADDR, 0x03, 0xa0,
	TVP7002VH_DEVICE_ADDR, 0x04, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x05, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x06, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x07, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x08, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x09, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x0a, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x0b, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0c, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0d, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0e, 0x24,
	TVP7002VH_DEVICE_ADDR, 0x0f, 0x2e,
	TVP7002VH_DEVICE_ADDR, 0x10, 0x58,
	TVP7002VH_DEVICE_ADDR, 0x11, 0x40,
	TVP7002VH_DEVICE_ADDR, 0x12, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x13, 0x00,
      
	TVP7002VH_DEVICE_ADDR, 0x15, 0x02,
	TVP7002VH_DEVICE_ADDR, 0x16, 0x11,
	TVP7002VH_DEVICE_ADDR, 0x17, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x18, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x19, 0xc0,
	TVP7002VH_DEVICE_ADDR, 0x1a, 0xc2,
	TVP7002VH_DEVICE_ADDR, 0x1b, 0x77,
	TVP7002VH_DEVICE_ADDR, 0x1c, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x1d, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x1e, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x1f, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x20, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x21, 0x3a,
	TVP7002VH_DEVICE_ADDR, 0x22, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x26, 0x80,
       
	TVP7002VH_DEVICE_ADDR, 0x28, 0x53,
	TVP7002VH_DEVICE_ADDR, 0x29, 0x08,
	TVP7002VH_DEVICE_ADDR, 0x2a, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x2b, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x2c, 0x50,
	TVP7002VH_DEVICE_ADDR, 0x2d, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x2e, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x2f, 0x0c,
	TVP7002VH_DEVICE_ADDR, 0x30, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x31, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x32, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x33, 0x60,
	TVP7002VH_DEVICE_ADDR, 0x34, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x35, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x36, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x3d, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x3e, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x3f, 0x0f,
       
	TVP7002VH_DEVICE_ADDR, 0x40, 0x24,
	TVP7002VH_DEVICE_ADDR, 0x41, 0x01,
       
	TVP7002VH_DEVICE_ADDR, 0x42, 0x2c,
	TVP7002VH_DEVICE_ADDR, 0x43, 0x06,
      
	TVP7002VH_DEVICE_ADDR, 0x44, 0x05,
	TVP7002VH_DEVICE_ADDR, 0x45, 0x05,
	TVP7002VH_DEVICE_ADDR, 0x46, 0x1d,
	TVP7002VH_DEVICE_ADDR, 0x47, 0x1d
};

static TQP_UINT8 dwTVP7002VHRegVals_1280x768x60Hz[] = 
{
	TVP7002VH_DEVICE_ADDR, 0x01, 0x68,
	TVP7002VH_DEVICE_ADDR, 0x02, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x03, 0xa0,
	TVP7002VH_DEVICE_ADDR, 0x04, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x05, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x06, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x07, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x08, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x09, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x0a, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x0b, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0c, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0d, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0e, 0x24,
	TVP7002VH_DEVICE_ADDR, 0x0f, 0x2e,
	TVP7002VH_DEVICE_ADDR, 0x10, 0x58,
	TVP7002VH_DEVICE_ADDR, 0x11, 0x40,
	TVP7002VH_DEVICE_ADDR, 0x12, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x13, 0x00,
      
	TVP7002VH_DEVICE_ADDR, 0x15, 0x02,
	TVP7002VH_DEVICE_ADDR, 0x16, 0x11,
	TVP7002VH_DEVICE_ADDR, 0x17, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x18, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x19, 0xc0,
	TVP7002VH_DEVICE_ADDR, 0x1a, 0xc2,
	TVP7002VH_DEVICE_ADDR, 0x1b, 0x77,
	TVP7002VH_DEVICE_ADDR, 0x1c, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x1d, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x1e, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x1f, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x20, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x21, 0x3a,
	TVP7002VH_DEVICE_ADDR, 0x22, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x26, 0x80,
       
	TVP7002VH_DEVICE_ADDR, 0x28, 0x53,
	TVP7002VH_DEVICE_ADDR, 0x29, 0x08,
	TVP7002VH_DEVICE_ADDR, 0x2a, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x2b, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x2c, 0x50,
	TVP7002VH_DEVICE_ADDR, 0x2d, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x2e, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x2f, 0x0c,
	TVP7002VH_DEVICE_ADDR, 0x30, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x31, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x32, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x33, 0x60,
	TVP7002VH_DEVICE_ADDR, 0x34, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x35, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x36, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x3d, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x3e, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x3f, 0x0f,
       
	TVP7002VH_DEVICE_ADDR, 0x40, 0x5a,
	TVP7002VH_DEVICE_ADDR, 0x41, 0x01,
       
	TVP7002VH_DEVICE_ADDR, 0x42, 0x5e,
	TVP7002VH_DEVICE_ADDR, 0x43, 0x06,
      
	TVP7002VH_DEVICE_ADDR, 0x44, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x45, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x46, 0x1c,
	TVP7002VH_DEVICE_ADDR, 0x47, 0x1c
};

static TQP_UINT8 dwTVP7002VHRegVals_1280x960x60Hz[] = 
{
	TVP7002VH_DEVICE_ADDR, 0x01, 0x70,
	TVP7002VH_DEVICE_ADDR, 0x02, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x03, 0x98,
	TVP7002VH_DEVICE_ADDR, 0x04, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x05, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x06, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x07, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x08, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x09, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x0a, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x0b, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0c, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0d, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0e, 0x24,
	TVP7002VH_DEVICE_ADDR, 0x0f, 0x2e,
	TVP7002VH_DEVICE_ADDR, 0x10, 0x58,
	TVP7002VH_DEVICE_ADDR, 0x11, 0x40,
	TVP7002VH_DEVICE_ADDR, 0x12, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x13, 0x00,
      
	TVP7002VH_DEVICE_ADDR, 0x15, 0x02,
	TVP7002VH_DEVICE_ADDR, 0x16, 0x11,
	TVP7002VH_DEVICE_ADDR, 0x17, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x18, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x19, 0xc0,
	TVP7002VH_DEVICE_ADDR, 0x1a, 0xc2,
	TVP7002VH_DEVICE_ADDR, 0x1b, 0x77,
	TVP7002VH_DEVICE_ADDR, 0x1c, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x1d, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x1e, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x1f, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x20, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x21, 0x3a,
	TVP7002VH_DEVICE_ADDR, 0x22, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x26, 0x80,
       
	TVP7002VH_DEVICE_ADDR, 0x28, 0x53,
	TVP7002VH_DEVICE_ADDR, 0x29, 0x08,
	TVP7002VH_DEVICE_ADDR, 0x2a, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x2b, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x2c, 0x50,
	TVP7002VH_DEVICE_ADDR, 0x2d, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x2e, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x2f, 0x0c,
	TVP7002VH_DEVICE_ADDR, 0x30, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x31, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x32, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x33, 0x60,
	TVP7002VH_DEVICE_ADDR, 0x34, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x35, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x36, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x3d, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x3e, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x3f, 0x0f,
       
	TVP7002VH_DEVICE_ADDR, 0x40, 0xc2,
	TVP7002VH_DEVICE_ADDR, 0x41, 0x01,
       
	TVP7002VH_DEVICE_ADDR, 0x42, 0xc6,
	TVP7002VH_DEVICE_ADDR, 0x43, 0x06,
      
	TVP7002VH_DEVICE_ADDR, 0x44, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x45, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x46, 0x26,
	TVP7002VH_DEVICE_ADDR, 0x47, 0x26
};

static TQP_UINT8 dwTVP7002VHRegVals_1280x1024x60Hz[] = 
{
	TVP7002VH_DEVICE_ADDR, 0x01, 0x69,
	TVP7002VH_DEVICE_ADDR, 0x02, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x03, 0xa0,
	TVP7002VH_DEVICE_ADDR, 0x04, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x05, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x06, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x07, 0x70,
	TVP7002VH_DEVICE_ADDR, 0x08, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x09, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x0a, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x0b, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0c, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0d, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0e, 0x24,
	TVP7002VH_DEVICE_ADDR, 0x0f, 0x2e,
	TVP7002VH_DEVICE_ADDR, 0x10, 0x58,
	TVP7002VH_DEVICE_ADDR, 0x11, 0x40,
	TVP7002VH_DEVICE_ADDR, 0x12, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x13, 0x00,
      
	TVP7002VH_DEVICE_ADDR, 0x15, 0x02,
	TVP7002VH_DEVICE_ADDR, 0x16, 0x11,
	TVP7002VH_DEVICE_ADDR, 0x17, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x18, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x19, 0xc0,
	TVP7002VH_DEVICE_ADDR, 0x1a, 0xc2,
	TVP7002VH_DEVICE_ADDR, 0x1b, 0x77,
	TVP7002VH_DEVICE_ADDR, 0x1c, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x1d, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x1e, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x1f, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x20, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x21, 0x3a,
	TVP7002VH_DEVICE_ADDR, 0x22, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x26, 0x80,
       
	TVP7002VH_DEVICE_ADDR, 0x28, 0x53,
	TVP7002VH_DEVICE_ADDR, 0x29, 0x08,
	TVP7002VH_DEVICE_ADDR, 0x2a, 0x87,
	TVP7002VH_DEVICE_ADDR, 0x2b, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x2c, 0x50,
	TVP7002VH_DEVICE_ADDR, 0x2d, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x2e, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x2f, 0x0c,
	TVP7002VH_DEVICE_ADDR, 0x30, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x31, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x32, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x33, 0x60,
	TVP7002VH_DEVICE_ADDR, 0x34, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x35, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x36, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x3d, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x3e, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x3f, 0x0f,
       
	TVP7002VH_DEVICE_ADDR, 0x40, 0x84,
	TVP7002VH_DEVICE_ADDR, 0x41, 0x01,
       
	TVP7002VH_DEVICE_ADDR, 0x42, 0x86,
	TVP7002VH_DEVICE_ADDR, 0x43, 0x06,
      
	TVP7002VH_DEVICE_ADDR, 0x44, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x45, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x46, 0x28,
	TVP7002VH_DEVICE_ADDR, 0x47, 0x28
};

static TQP_UINT8 dwTVP7002VHRegVals_1360x768x60Hz[] = 
{
	TVP7002VH_DEVICE_ADDR, 0x01, 0x70,
	TVP7002VH_DEVICE_ADDR, 0x02, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x03, 0x98,
	TVP7002VH_DEVICE_ADDR, 0x04, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x05, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x06, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x07, 0x70,
	TVP7002VH_DEVICE_ADDR, 0x08, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x09, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x0a, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x0b, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0c, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0d, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0e, 0x36,
	TVP7002VH_DEVICE_ADDR, 0x0f, 0x2e,
	TVP7002VH_DEVICE_ADDR, 0x10, 0x58,
	TVP7002VH_DEVICE_ADDR, 0x11, 0x40,
	TVP7002VH_DEVICE_ADDR, 0x12, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x13, 0x00,
      
	TVP7002VH_DEVICE_ADDR, 0x15, 0x02,
	TVP7002VH_DEVICE_ADDR, 0x16, 0x11,
	TVP7002VH_DEVICE_ADDR, 0x17, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x18, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x19, 0xc0,
	TVP7002VH_DEVICE_ADDR, 0x1a, 0xc2,
	TVP7002VH_DEVICE_ADDR, 0x1b, 0x77,
	TVP7002VH_DEVICE_ADDR, 0x1c, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x1d, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x1e, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x1f, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x20, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x21, 0x3a,
	TVP7002VH_DEVICE_ADDR, 0x22, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x26, 0x80,
       
	TVP7002VH_DEVICE_ADDR, 0x28, 0x53,
	TVP7002VH_DEVICE_ADDR, 0x29, 0x08,
	TVP7002VH_DEVICE_ADDR, 0x2a, 0x87,
	TVP7002VH_DEVICE_ADDR, 0x2b, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x2c, 0x50,
	TVP7002VH_DEVICE_ADDR, 0x2d, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x2e, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x2f, 0x0c,
	TVP7002VH_DEVICE_ADDR, 0x30, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x31, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x32, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x33, 0x60,
	TVP7002VH_DEVICE_ADDR, 0x34, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x35, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x36, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x3d, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x3e, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x3f, 0x0f,
       
	TVP7002VH_DEVICE_ADDR, 0x40, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x41, 0x01,
       
	TVP7002VH_DEVICE_ADDR, 0x42, 0xd2,
	TVP7002VH_DEVICE_ADDR, 0x43, 0x06,
      
	TVP7002VH_DEVICE_ADDR, 0x44, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x45, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x46, 0x1b,
	TVP7002VH_DEVICE_ADDR, 0x47, 0x1b
};

static TQP_UINT8 dwTVP7002VHRegVals_1366x768x60Hz[] = 
{
	TVP7002VH_DEVICE_ADDR, 0x01, 0x70,
	TVP7002VH_DEVICE_ADDR, 0x02, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x03, 0x98,
	TVP7002VH_DEVICE_ADDR, 0x04, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x05, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x06, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x07, 0x8f,
	TVP7002VH_DEVICE_ADDR, 0x08, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x09, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x0a, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x0b, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0c, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0d, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0e, 0x36,
	TVP7002VH_DEVICE_ADDR, 0x0f, 0x2e,
	TVP7002VH_DEVICE_ADDR, 0x10, 0x58,
	TVP7002VH_DEVICE_ADDR, 0x11, 0x40,
	TVP7002VH_DEVICE_ADDR, 0x12, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x13, 0x00,
      
	TVP7002VH_DEVICE_ADDR, 0x15, 0x02,
	TVP7002VH_DEVICE_ADDR, 0x16, 0x11,
	TVP7002VH_DEVICE_ADDR, 0x17, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x18, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x19, 0xc0,
	TVP7002VH_DEVICE_ADDR, 0x1a, 0xc2,
	TVP7002VH_DEVICE_ADDR, 0x1b, 0x77,
	TVP7002VH_DEVICE_ADDR, 0x1c, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x1d, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x1e, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x1f, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x20, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x21, 0x3a,
	TVP7002VH_DEVICE_ADDR, 0x22, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x26, 0x80,
       
	TVP7002VH_DEVICE_ADDR, 0x28, 0x53,
	TVP7002VH_DEVICE_ADDR, 0x29, 0x08,
	TVP7002VH_DEVICE_ADDR, 0x2a, 0x87,
	TVP7002VH_DEVICE_ADDR, 0x2b, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x2c, 0x50,
	TVP7002VH_DEVICE_ADDR, 0x2d, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x2e, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x2f, 0x0c,
	TVP7002VH_DEVICE_ADDR, 0x30, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x31, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x32, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x33, 0x60,
	TVP7002VH_DEVICE_ADDR, 0x34, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x35, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x36, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x3d, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x3e, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x3f, 0x0f,
       
	TVP7002VH_DEVICE_ADDR, 0x40, 0x7e,
	TVP7002VH_DEVICE_ADDR, 0x41, 0x01,
       
	TVP7002VH_DEVICE_ADDR, 0x42, 0xda,
	TVP7002VH_DEVICE_ADDR, 0x43, 0x06,
      
	TVP7002VH_DEVICE_ADDR, 0x44, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x45, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x46, 0x19,
	TVP7002VH_DEVICE_ADDR, 0x47, 0x19
};

static TQP_UINT8 dwTVP7002VHRegVals_1400x1050x60Hz[] = 
{
	TVP7002VH_DEVICE_ADDR, 0x01, 0x74,
	TVP7002VH_DEVICE_ADDR, 0x02, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x03, 0x98,
	TVP7002VH_DEVICE_ADDR, 0x04, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x05, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x06, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x07, 0x90,
	TVP7002VH_DEVICE_ADDR, 0x08, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x09, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x0a, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x0b, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0c, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0d, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0e, 0x24,
	TVP7002VH_DEVICE_ADDR, 0x0f, 0x2e,
	TVP7002VH_DEVICE_ADDR, 0x10, 0x58,
	TVP7002VH_DEVICE_ADDR, 0x11, 0x40,
	TVP7002VH_DEVICE_ADDR, 0x12, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x13, 0x00,
      
	TVP7002VH_DEVICE_ADDR, 0x15, 0x02,
	TVP7002VH_DEVICE_ADDR, 0x16, 0x11,
	TVP7002VH_DEVICE_ADDR, 0x17, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x18, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x19, 0xc0,
	TVP7002VH_DEVICE_ADDR, 0x1a, 0xc2,
	TVP7002VH_DEVICE_ADDR, 0x1b, 0x77,
	TVP7002VH_DEVICE_ADDR, 0x1c, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x1d, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x1e, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x1f, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x20, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x21, 0x3a,
	TVP7002VH_DEVICE_ADDR, 0x22, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x26, 0x80,
       
	TVP7002VH_DEVICE_ADDR, 0x28, 0x53,
	TVP7002VH_DEVICE_ADDR, 0x29, 0x08,
	TVP7002VH_DEVICE_ADDR, 0x2a, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x2b, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x2c, 0x50,
	TVP7002VH_DEVICE_ADDR, 0x2d, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x2e, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x2f, 0x0c,
	TVP7002VH_DEVICE_ADDR, 0x30, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x31, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x32, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x33, 0x60,
	TVP7002VH_DEVICE_ADDR, 0x34, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x35, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x36, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x3d, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x3e, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x3f, 0x0f,
       
	TVP7002VH_DEVICE_ADDR, 0x40, 0xa4,
	TVP7002VH_DEVICE_ADDR, 0x41, 0x01,
       
	TVP7002VH_DEVICE_ADDR, 0x42, 0x1c,
	TVP7002VH_DEVICE_ADDR, 0x43, 0x07,
      
	TVP7002VH_DEVICE_ADDR, 0x44, 0x03,
	TVP7002VH_DEVICE_ADDR, 0x45, 0x03,
	TVP7002VH_DEVICE_ADDR, 0x46, 0x26,
	TVP7002VH_DEVICE_ADDR, 0x47, 0x26
};

static TQP_UINT8 dwTVP7002VHRegVals_1920x1080x60Hz[] = 
{
	TVP7002VH_DEVICE_ADDR, 0x01, 0x89,
	TVP7002VH_DEVICE_ADDR, 0x02, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x03, 0xe0,
	TVP7002VH_DEVICE_ADDR, 0x04, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x05, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x06, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x07, 0x2c,
	TVP7002VH_DEVICE_ADDR, 0x08, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x09, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x0a, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x0b, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0c, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0d, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x0e, 0x24,
	TVP7002VH_DEVICE_ADDR, 0x0f, 0x2e,
	TVP7002VH_DEVICE_ADDR, 0x10, 0x58,
	TVP7002VH_DEVICE_ADDR, 0x11, 0x40,
	TVP7002VH_DEVICE_ADDR, 0x12, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x13, 0x00,
      
	TVP7002VH_DEVICE_ADDR, 0x15, 0x02,
	TVP7002VH_DEVICE_ADDR, 0x16, 0x11,
	TVP7002VH_DEVICE_ADDR, 0x17, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x18, 0x10,
	TVP7002VH_DEVICE_ADDR, 0x19, 0xc0,
	TVP7002VH_DEVICE_ADDR, 0x1a, 0xc2,
	TVP7002VH_DEVICE_ADDR, 0x1b, 0x77,
	TVP7002VH_DEVICE_ADDR, 0x1c, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x1d, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x1e, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x1f, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x20, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x21, 0x0d,
	TVP7002VH_DEVICE_ADDR, 0x22, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x26, 0x80,
       
	TVP7002VH_DEVICE_ADDR, 0x28, 0x53,
	TVP7002VH_DEVICE_ADDR, 0x29, 0x08,
	TVP7002VH_DEVICE_ADDR, 0x2a, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x2b, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x2c, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x2d, 0x00,
	TVP7002VH_DEVICE_ADDR, 0x2e, 0x80,
	TVP7002VH_DEVICE_ADDR, 0x2f, 0x0c,
	TVP7002VH_DEVICE_ADDR, 0x30, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x31, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x32, 0x18,
	TVP7002VH_DEVICE_ADDR, 0x33, 0x60,
	TVP7002VH_DEVICE_ADDR, 0x34, 0x07,
	TVP7002VH_DEVICE_ADDR, 0x35, 0x01,
	TVP7002VH_DEVICE_ADDR, 0x36, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x3d, 0x06,
	TVP7002VH_DEVICE_ADDR, 0x3e, 0x04,
	TVP7002VH_DEVICE_ADDR, 0x3f, 0x0f,
       
	TVP7002VH_DEVICE_ADDR, 0x40, 0xdd,
	TVP7002VH_DEVICE_ADDR, 0x41, 0x00,
       
	TVP7002VH_DEVICE_ADDR, 0x42, 0x61,
	TVP7002VH_DEVICE_ADDR, 0x43, 0x08,
      
	TVP7002VH_DEVICE_ADDR, 0x44, 0x03,
	TVP7002VH_DEVICE_ADDR, 0x45, 0x03,
	TVP7002VH_DEVICE_ADDR, 0x46, 0x2b,
	TVP7002VH_DEVICE_ADDR, 0x47, 0x2b
};

/////////////////////////////////////////////////////////////////////////////
//
//


static EQP_Errors CTVP7002VH_readRegister(CTVP7002VH *This,
										  TQP_UINT8 regAddr, 
										  PTQP_UINT8 pValue
										  )
{
    return (II2C_WriteThenRead(This->m_pI2C,
							   This->m_cI2CDevAddr,	//device address
							   1,					//write size
							   &regAddr,			//write buffer
							   1,					//read size
							   pValue				//read buffer
							   ));
}



static EQP_Errors CTVP7002VH_writeRegister(CTVP7002VH *This,
										   TQP_UINT8 regAddr, 
										   TQP_UINT8 value
										   )
{
    TQP_UINT8 buffer[2];

	buffer[0] = regAddr;
	buffer[1] = value;

    return (II2C_Write(This->m_pI2C, 
					   This->m_cI2CDevAddr,	//device address
					   2,					//write size
					   buffer				//write buffer
					   ));
}


static EQP_Errors CTVP7002VH_WriteBlock(CTVP7002VH *This, TQP_UINT8 *pRegTable)
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	
	// get total number of registers to write
	//
	while (0xff != pRegTable[0])
	{
		qpStatus = II2C_Write(This->m_pI2C, 
					   pRegTable[0],	//device address
					   2,				//write size
					   pRegTable+1		//write buffer
					   );										 
		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CTVP7002VH_WriteBlock() ****** writeRegister failed! status(%d) ******\n",
						 qpStatus
						 ));
			break;
		}

		pRegTable += 3;
	}

	return (qpStatus);
}



/////////////////////////////////////////////////////////////////////////////
//
//


// IGenericDevice
//
EQP_Errors CTVP7002VH_InitDevice(IVideoDecoder *This_p, 
								 QP_DEVICE_CALLBACK pFuncCallback,
								 TQP_PVOID context,
								 TQP_PVOID pInitData
								 )
{
    TQP_UINT8     buffer[2];
    TQP_UINT8     ucRegAddr;
    TQP_UINT8     ucValue;

    CVideoDecoder *pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
    CQLCodecLib   *pQLCodecLib = GET_INHERITED_OBJECT(CQLCodecLib, m_Object, pVideoDecoder->m_Object.m_pParent);
    CTVP7002VH      *This = GET_INHERITED_OBJECT(CTVP7002VH, m_VideoDecoder, pVideoDecoder);
    
    QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
                 "%s(): multiType(%d)\n",
                 __FUNCTION__,
                 pQLCodecLib->m_InitData.vidDecoderInitData.multiType
                 ));
                
    if (CObject_IsInitialized(&pVideoDecoder->m_Object))
    {
        QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
                     "CTVP7002VH_InitDevice() Already Initialized. Skipping...\n" 
                    ));
      return (QPERR_SUCCESS);
    }
	
    // save callback address and context
    //
    pVideoDecoder->m_pDeviceCallback = pFuncCallback;
    pVideoDecoder->m_callbackContext = context;

    // Initialize TVP7002VH with the default value
    //
    CTVP7002VH_WriteBlock(This, dwTVP7002VHRegVals_1280x720x60Hz);

    // program the video decoder with settings read from the registry
    //
    QPVideoDecoder_SetVideoSystem(This_p, pVideoDecoder->m_VidStandard);        
    QPVideoDecoder_SelectVideoSource(This_p, pVideoDecoder->m_VidSource);

    QPVideoDecoder_SetBrightness(This_p, pVideoDecoder->m_nBrightness);
    QPVideoDecoder_SetContrast(This_p, pVideoDecoder->m_nContrast);
    QPVideoDecoder_SetSaturation(This_p, pVideoDecoder->m_nSaturation);
    QPVideoDecoder_SetHue(This_p, pVideoDecoder->m_nHue);
    QPVideoDecoder_SetSharpness(This_p, pVideoDecoder->m_nSharpness);

    // CObject Init
    //
    CObject_Init(&pVideoDecoder->m_Object);

    return (QPERR_SUCCESS);
}



// IVideoDecoder
//
EQP_Errors CTVP7002VH_SetVideoRes(IVideoDecoder *This_p,
								  TQP_UINT32 dwWid,
				  				  TQP_UINT32 dwHgt,
				  				  TQP_UINT32 dwScanFmt,
				  				  TQP_UINT32 dwFps
				  				  )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CTVP7002VH		*This = GET_INHERITED_OBJECT(CTVP7002VH, m_VideoDecoder, pVideoDecoder);
	CQLCodecLib		*pQLCodecLib = GET_INHERITED_OBJECT(CQLCodecLib, m_Object, pVideoDecoder->m_Object.m_pParent);
	CQLCodec 		*pQLCodec = pQLCodecLib->m_pMpgCodec;	
	TQP_UINT8		*pVals;
	EQP_Errors		qpStatus;
	
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CTVP7002VH_SetVideoRes() : Video Res Set to %dx %d\n",
				 dwWid,
				 dwHgt
				 ));
				 
	//return (QPERR_SUCCESS);

	if ((640 == dwWid) && (480 == dwHgt)) {
		pVals = dwTVP7002VHRegVals_640x480x60Hz;
	} else if ((720 == dwWid) && (480 == dwHgt)) {
		pVals = dwTVP7002VHRegVals_720x480x60Hz;
	} else if ((720 == dwWid) && (576 == dwHgt)) {
		pVals = dwTVP7002VHRegVals_720x576x50Hz;
	} else if ((800 == dwWid) && (600 == dwHgt)) {
		pVals = dwTVP7002VHRegVals_800x600x60Hz;
	} else if ((1024 == dwWid) && (768 == dwHgt)) {
		pVals = dwTVP7002VHRegVals_1024x768x60Hz;
	} else if ((1280 == dwWid) && (720 == dwHgt)) {
		if (50 == dwFps) {
			pVals = dwTVP7002VHRegVals_1280x720x50Hz;
		} else {
			pVals = dwTVP7002VHRegVals_1280x720x60Hz;
		}
	} else if ((1280 == dwWid) && (768 == dwHgt)) {
		pVals = dwTVP7002VHRegVals_1280x768x60Hz;
	} else if ((1280 == dwWid) && (960 == dwHgt)) {
		pVals = dwTVP7002VHRegVals_1280x960x60Hz;
	} else if ((1280 == dwWid) && (1024 == dwHgt)) {
		pVals = dwTVP7002VHRegVals_1280x1024x60Hz;		
	} else if ((1360 == dwWid) && (768 == dwHgt)) {
		pVals = dwTVP7002VHRegVals_1360x768x60Hz;
	} else if ((1366 == dwWid) && (768 == dwHgt)) {
		pVals = dwTVP7002VHRegVals_1366x768x60Hz;		
	} else if ((1400 == dwWid) && (1050 == dwHgt)) {
		pVals = dwTVP7002VHRegVals_1400x1050x60Hz;		
	} else if ((1920 == dwWid) && (1080 == dwHgt)) {
		if (50 == dwFps) {
			pVals = dwTVP7002VHRegVals_1920x1080x50Hz;
		} else {
			pVals = dwTVP7002VHRegVals_1920x1080x60Hz;
		}
	} else {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "%s(): Unsupported video format resolution(%dx%d) scan_format(%d) fps(%d)\n", 
					 __FUNCTION__, dwWid, dwHgt, dwScanFmt, dwFps
					 ));
		return (QPERR_PARMS);
	}
	
	qpStatus = CTVP7002VH_WriteBlock(This, pVals);
	
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "%s(): ****** WriteBlock() failed! ******\n",
					 __FUNCTION__
					 ));
		return (QPERR_FAIL);
	}		
				 
	DelayMilliSeconds(150);
	
	return (QPERR_SUCCESS);
}



// constructor
//
CTVP7002VH * CTVP7002VH_Constructor(CTVP7002VH *pTVP7002VH, 
									CObject *pParent,
									TQP_UINT32 dwAttributes,
									ITuner *pTuner,
									ITVAudio *pTvAudio,
									II2C *pI2C,
									TQP_UINT8 cI2CDevAddr
									)
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTVP7002VH_Constructor()\n"
				 ));

	if (pTVP7002VH)
	{
		CVideoDecoder	*pVideoDecoder;

		pVideoDecoder = CVideoDecoder_Constructor(&pTVP7002VH->m_VideoDecoder, 
												  pParent,
												  dwAttributes,
												  pTuner,
												  pTvAudio
												  );

		if (!pVideoDecoder)
		{
			pTVP7002VH = QP_NULL;
		}
		else
		{
			// fill in the function table
			//

			// IGenericDevice
			pTVP7002VH->m_VideoDecoder.m_iVideoDecoder.InitDevice = CTVP7002VH_InitDevice;

			// IVideoDecoder
			pTVP7002VH->m_VideoDecoder.m_iVideoDecoder.SetVideoRes = CTVP7002VH_SetVideoRes;

			// initialize members
			//
			pTVP7002VH->m_pI2C = pI2C;
			pTVP7002VH->m_cI2CDevAddr = cI2CDevAddr;

			// inherited members
			//
			pTVP7002VH->m_VideoDecoder.m_nBrightnessDef = TVP7002VH_BRIGHTNESS_DEFAULT;
			pTVP7002VH->m_VideoDecoder.m_nBrightnessMin = TVP7002VH_BRIGHTNESS_MIN;
			pTVP7002VH->m_VideoDecoder.m_nBrightnessMax = TVP7002VH_BRIGHTNESS_MAX;
			pTVP7002VH->m_VideoDecoder.m_nBrightness = TVP7002VH_BRIGHTNESS_DEFAULT;
			pTVP7002VH->m_VideoDecoder.m_nContrastDef = TVP7002VH_CONTRAST_DEFAULT; 
			pTVP7002VH->m_VideoDecoder.m_nContrastMin = TVP7002VH_CONTRAST_MIN;
			pTVP7002VH->m_VideoDecoder.m_nContrastMax = TVP7002VH_CONTRAST_MAX;
			pTVP7002VH->m_VideoDecoder.m_nContrast = TVP7002VH_CONTRAST_DEFAULT;
			pTVP7002VH->m_VideoDecoder.m_nSaturationDef = TVP7002VH_SATURATION_DEFAULT;
			pTVP7002VH->m_VideoDecoder.m_nSaturationMin = TVP7002VH_SATURATION_MIN;
			pTVP7002VH->m_VideoDecoder.m_nSaturationMax = TVP7002VH_SATURATION_MAX;
			pTVP7002VH->m_VideoDecoder.m_nSaturation = TVP7002VH_SATURATION_DEFAULT;
			pTVP7002VH->m_VideoDecoder.m_nHueDef = TVP7002VH_HUE_DEFAULT;
			pTVP7002VH->m_VideoDecoder.m_nHueMin = TVP7002VH_HUE_MIN;
			pTVP7002VH->m_VideoDecoder.m_nHueMax = TVP7002VH_HUE_MAX;
			pTVP7002VH->m_VideoDecoder.m_nHue = TVP7002VH_HUE_DEFAULT;
			pTVP7002VH->m_VideoDecoder.m_nSharpnessDef = TVP7002VH_SHARPNESS_DEFAULT;
			pTVP7002VH->m_VideoDecoder.m_nSharpnessMin = TVP7002VH_SHARPNESS_MIN;
			pTVP7002VH->m_VideoDecoder.m_nSharpnessMax = TVP7002VH_SHARPNESS_MAX;
			pTVP7002VH->m_VideoDecoder.m_nSharpness = TVP7002VH_SHARPNESS_DEFAULT;
		}
	}

	return (pTVP7002VH);
}



// destructor
//
void CTVP7002VH_Destructor(CTVP7002VH *This)
{
    TQP_UINT8     buffer[2];
    TQP_UINT8     ucRegAddr;
    TQP_UINT8     ucValue;
    
    QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTVP7002VH_Destructor()\n"
				 ));

    CVideoDecoder_Destructor(&This->m_VideoDecoder);
}

