///////////////////////////////////////////////////////////////////////////////
//        
//
// cttw9910.c
//
// Description: 
//   TW9910 implementation
// 
// History:
//	 April 4, 2013 created
//
// Authors: Jack Lee
//
///////////////////////////////////////////////////////////////////////////////


#include "ctw9910.h"
#include "ctvp7002.h"
#include "cit6603.h"
#include "qptimemgr.h"
#include "qpdbgmgr.h"
#include "util.h"
#include "cqlcodeclib.h"
#include "cqlcodec.h"


/////////////////////////////////////////////////////////////////////////////
//
//

#define	TW9910_BRIGHTNESS_MIN			0x00   // not supported
#define	TW9910_BRIGHTNESS_MAX			0xFF   // not supported
#define	TW9910_BRIGHTNESS_DEFAULT		0x80   // not supported

#define	TW9910_CONTRAST_MIN				0x00   // not supported
#define	TW9910_CONTRAST_MAX				0x7F   // not supported
#define	TW9910_CONTRAST_DEFAULT			0x44   // not supported

#define	TW9910_SATURATION_MIN			0x00   // not supported
#define	TW9910_SATURATION_MAX			0x7F   // not supported
#define	TW9910_SATURATION_DEFAULT		0x40   // not supported

#define	TW9910_HUE_MIN					(-128)	// 0x80   // not supported
#define	TW9910_HUE_MAX					(+127)	// 0x7F   // not supported
#define	TW9910_HUE_DEFAULT				0x00   // not supported

#define	TW9910_SHARPNESS_MIN			0      // not supported
#define	TW9910_SHARPNESS_MAX			15     // not supported
#define	TW9910_SHARPNESS_DEFAULT		8      // not supported

extern TQP_DbgMCompID	g_QPPeripheralDBGCompID;

static TQP_UINT8 dwTW9910RegVals_NTSC[] = 
{
	// Initialize TW9910 with the default value
	TW9910_DEVICE_ADDR, 0x06, 0x00,
	TW9910_DEVICE_ADDR, 0x1a, 0x4f,
	TW9910_DEVICE_ADDR, 0x03, 0xa2,
	TW9910_DEVICE_ADDR, 0x05, 0x01,
	TW9910_DEVICE_ADDR, 0x08, 0x13,
	TW9910_DEVICE_ADDR, 0x09, 0xf2,
	TW9910_DEVICE_ADDR, 0x19, 0x57,
	TW9910_DEVICE_ADDR, 0x1a, 0x0f,
	TW9910_DEVICE_ADDR, 0x1b, 0xc0,
	TW9910_DEVICE_ADDR, 0x29, 0x03,
	TW9910_DEVICE_ADDR, 0x55, 0x00,
	TW9910_DEVICE_ADDR, 0x6b, 0x26,
	TW9910_DEVICE_ADDR, 0x6c, 0x36,
	TW9910_DEVICE_ADDR, 0x6d, 0xF0,
	TW9910_DEVICE_ADDR, 0x6e, 0x28,
	TW9910_DEVICE_ADDR, 0x1a, 0x0f,
	TW9910_DEVICE_ADDR, 0x06, 0x80,
	TW9910_DEVICE_ADDR, 0x10, 0x00,
	TW9910_DEVICE_ADDR, 0x11, 0x5C,
	TW9910_DEVICE_ADDR, 0x13, 0x80,
	TW9910_DEVICE_ADDR, 0x14, 0x80,
	TW9910_DEVICE_ADDR, 0x2D, 0x16,

	// TW9910 NTSC
	TW9910_DEVICE_ADDR, 0x06, 0x00,
	TW9910_DEVICE_ADDR, 0x1a, 0x4f,
	TW9910_DEVICE_ADDR, 0x03, 0xa2,
	TW9910_DEVICE_ADDR, 0x05, 0x01,
	TW9910_DEVICE_ADDR, 0x08, 0x13,
	TW9910_DEVICE_ADDR, 0x09, 0xf2,
	TW9910_DEVICE_ADDR, 0x19, 0x57,
	TW9910_DEVICE_ADDR, 0x1a, 0x0f,
	TW9910_DEVICE_ADDR, 0x1b, 0xc0,
	TW9910_DEVICE_ADDR, 0x29, 0x03,
	TW9910_DEVICE_ADDR, 0x55, 0x00,
	TW9910_DEVICE_ADDR, 0x6b, 0x26,
	TW9910_DEVICE_ADDR, 0x6c, 0x36,
	TW9910_DEVICE_ADDR, 0x6d, 0xF0,
	TW9910_DEVICE_ADDR, 0x6e, 0x28,
	TW9910_DEVICE_ADDR, 0x1a, 0x0f,
	TW9910_DEVICE_ADDR, 0x06, 0x80,
	TW9910_DEVICE_ADDR, 0x1c, 0x00,
	0xff, 0xff, 0xff
};

static TQP_UINT8 dwTW9910RegVals_PAL[] = 
{
	// Initialize TW9910 with the default value
	TW9910_DEVICE_ADDR, 0x06, 0x00,
	TW9910_DEVICE_ADDR, 0x1a, 0x4f,
	TW9910_DEVICE_ADDR, 0x03, 0xa2,
	TW9910_DEVICE_ADDR, 0x05, 0x01,
	TW9910_DEVICE_ADDR, 0x08, 0x13,
	TW9910_DEVICE_ADDR, 0x09, 0xf2,
	TW9910_DEVICE_ADDR, 0x19, 0x57,
	TW9910_DEVICE_ADDR, 0x1a, 0x0f,
	TW9910_DEVICE_ADDR, 0x1b, 0xc0,
	TW9910_DEVICE_ADDR, 0x29, 0x03,
	TW9910_DEVICE_ADDR, 0x55, 0x00,
	TW9910_DEVICE_ADDR, 0x6b, 0x26,
	TW9910_DEVICE_ADDR, 0x6c, 0x36,
	TW9910_DEVICE_ADDR, 0x6d, 0xF0,
	TW9910_DEVICE_ADDR, 0x6e, 0x28,
	TW9910_DEVICE_ADDR, 0x1a, 0x0f,
	TW9910_DEVICE_ADDR, 0x06, 0x80,
	TW9910_DEVICE_ADDR, 0x10, 0x00,
	TW9910_DEVICE_ADDR, 0x11, 0x5C,
	TW9910_DEVICE_ADDR, 0x13, 0x80,
	TW9910_DEVICE_ADDR, 0x14, 0x80,
	TW9910_DEVICE_ADDR, 0x2D, 0x16,

	// TW9910 PAL
	TW9910_DEVICE_ADDR, 0x06, 0x00,
	TW9910_DEVICE_ADDR, 0x1a, 0x4f,
	TW9910_DEVICE_ADDR, 0x03, 0xa2,
	TW9910_DEVICE_ADDR, 0x05, 0x01,
	TW9910_DEVICE_ADDR, 0x08, 0x19,
	TW9910_DEVICE_ADDR, 0x09, 0x1f,
	TW9910_DEVICE_ADDR, 0x19, 0x57,
	TW9910_DEVICE_ADDR, 0x1a, 0x0f,
	TW9910_DEVICE_ADDR, 0x1b, 0xc0,
	TW9910_DEVICE_ADDR, 0x29, 0x03,
	TW9910_DEVICE_ADDR, 0x55, 0x00,
	TW9910_DEVICE_ADDR, 0x6b, 0x26,
	TW9910_DEVICE_ADDR, 0x6c, 0x36,
	TW9910_DEVICE_ADDR, 0x6d, 0xf0,
	TW9910_DEVICE_ADDR, 0x6e, 0x28,
	TW9910_DEVICE_ADDR, 0x1a, 0x0f,
	TW9910_DEVICE_ADDR, 0x06, 0x80,
	TW9910_DEVICE_ADDR, 0x1c, 0x01,
	0xff, 0xff, 0xff
};


/////////////////////////////////////////////////////////////////////////////
//
//


static EQP_Errors CTW9910_readRegister(CTW9910 *This, TQP_UINT8 regAddr, PTQP_UINT8 pValue)
{
    return (II2C_WriteThenRead(This->m_pI2C,
							   This->m_cI2CDevAddr,	//device address
							   1,					//write size
							   &regAddr,			//write buffer
							   1,					//read size
							   pValue				//read buffer
							   ));
}



static EQP_Errors CTW9910_writeRegister(CTW9910 *This, TQP_UINT8 regAddr, TQP_UINT8 value)
{
    TQP_UINT8 buffer[2];

	buffer[0] = regAddr;
	buffer[1] = value;

    return (II2C_Write(This->m_pI2C, 
					   This->m_cI2CDevAddr,	//device address
					   2,					//write size
					   buffer				//write buffer
					   ));
}


static EQP_Errors CTW9910_WriteBlock(CTW9910 *This, TQP_UINT8 *pRegTable)
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	
	// get total number of registers to write
	//
	while (0xff != pRegTable[0])
	{
		qpStatus = II2C_Write(This->m_pI2C, pRegTable[0]/*device address*/, 2/*write size*/, pRegTable+1/*write buffer*/);
		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CTW9910_WriteBlock() ****** writeRegister failed! status(%d) ******\n",
						 qpStatus
						 ));
			break;
		}

		pRegTable += 3;
	}

	return (qpStatus);
}



/////////////////////////////////////////////////////////////////////////////
//
//


// IGenericDevice
//
EQP_Errors CTW9910_InitDevice(IVideoDecoder *This_p, 
							  QP_DEVICE_CALLBACK pFuncCallback,
							  TQP_PVOID context,
							  TQP_PVOID pInitData
							  )
{
    TQP_UINT8     buffer[2];
    //TQP_UINT8     ucRegAddr;
    //TQP_UINT8     ucValue;

    CVideoDecoder *pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
    CQLCodecLib   *pQLCodecLib = GET_INHERITED_OBJECT(CQLCodecLib, m_Object, pVideoDecoder->m_Object.m_pParent);
    CTW9910      *This = GET_INHERITED_OBJECT(CTW9910, m_VideoDecoder, pVideoDecoder);
    
    QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
                 "%s(): multiType(%d)\n",
                 __FUNCTION__,
                 pQLCodecLib->m_InitData.vidDecoderInitData.multiType
                 ));
                
    if (CObject_IsInitialized(&pVideoDecoder->m_Object))
    {
        QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
                     "CTW9910_InitDevice() Already Initialized. Skipping...\n" 
                    ));
      return (QPERR_SUCCESS);
    }
	
    // save callback address and context
    //
    pVideoDecoder->m_pDeviceCallback = pFuncCallback;
    pVideoDecoder->m_callbackContext = context;

	// if has multiple video input types, disable others
	//
    if (MULTIPLE_VIDDECODER_TYPE1 == pQLCodecLib->m_InitData.vidDecoderInitData.multiType) {
    	// power down IT6604
    	buffer[0] = 0x89;
    	buffer[1] = 0xff;
    	II2C_Write(This->m_pI2C, IT6603_DEVICE_ADDR/*device address*/, 2/*write size*/, buffer/*write buffer*/);
    	
    	// power down TVP7002
    	buffer[0] = 0x17;
    	buffer[1] = 0x03;
    	II2C_Write(This->m_pI2C, TVP7002_DEVICE_ADDR/*device address*/, 2/*write size*/, buffer/*write buffer*/);
				   
		DelayMilliSeconds(100);		   
    }

    // Initialize TW9910 with the default value
    //
    CTW9910_WriteBlock(This, dwTW9910RegVals_NTSC);

    // program the video decoder with settings read from the registry
    //
    QPVideoDecoder_SetVideoSystem(This_p, pVideoDecoder->m_VidStandard);        
    QPVideoDecoder_SelectVideoSource(This_p, pVideoDecoder->m_VidSource);

    QPVideoDecoder_SetBrightness(This_p, pVideoDecoder->m_nBrightness);
    QPVideoDecoder_SetContrast(This_p, pVideoDecoder->m_nContrast);
    QPVideoDecoder_SetSaturation(This_p, pVideoDecoder->m_nSaturation);
    QPVideoDecoder_SetHue(This_p, pVideoDecoder->m_nHue);
    QPVideoDecoder_SetSharpness(This_p, pVideoDecoder->m_nSharpness);

    // CObject Init
    //
    CObject_Init(&pVideoDecoder->m_Object);

    return (QPERR_SUCCESS);
}



// IVideoDecoder
//
EQP_Errors CTW9910_SetVideoRes(IVideoDecoder *This_p,
							   TQP_UINT32 dwWid,
				  			   TQP_UINT32 dwHgt,
				  			   TQP_UINT32 dwScanFmt,
				  			   TQP_UINT32 dwFps
				  			   )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CTW9910			*This = GET_INHERITED_OBJECT(CTW9910, m_VideoDecoder, pVideoDecoder);
	CQLCodecLib		*pQLCodecLib = GET_INHERITED_OBJECT(CQLCodecLib, m_Object, pVideoDecoder->m_Object.m_pParent);
	CQLCodec 		*pQLCodec = pQLCodecLib->m_pMpgCodec;	
	TQP_UINT8		*pVals;
	EQP_Errors		qpStatus;
	
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CTW9910_SetVideoRes() : Video Res Set to %dx %d\n",
				 dwWid,
				 dwHgt
				 ));
				 
	//return (QPERR_SUCCESS);
	
	if (MULTIPLE_VIDDECODER_TYPE1 == pQLCodecLib->m_InitData.vidDecoderInitData.multiType) {
		TQP_UINT32 reg;	
		
		// configure audio switch 
		// low for C85343 (TVP7002/TW9910)
		// high for IT6604
		
		// configure gpio 12 to output
		QPHCI_RegisterRead(&pQLCodec->m_hci, 0x610, &reg);
		reg = reg | (1 << 12);
		QPHCI_RegisterWrite(&pQLCodec->m_hci, 0x610, reg);		
		
		// gpio 12 to low state
		QPHCI_RegisterRead(&pQLCodec->m_hci, 0x614, &reg);
		reg = reg & ~(1 << 12);
		QPHCI_RegisterWrite(&pQLCodec->m_hci, 0x614, reg);				
	}	
	
	if ((720 == dwWid) && (480 == dwHgt) && (SCAN_FORMAT_INTERLACE == dwScanFmt)) {
		pVals = dwTW9910RegVals_NTSC;
		pQLCodecLib->m_pMpgCodec->m_ulViuSyncCode1 = 0xF1F1F1DA;
		pQLCodecLib->m_pMpgCodec->m_ulViuSyncCode2 = 0xB6F1F1B6;                 
	} else if ((720 == dwWid) && (576 == dwHgt) && (SCAN_FORMAT_INTERLACE == dwScanFmt)) {
		pVals = dwTW9910RegVals_PAL;
		pQLCodecLib->m_pMpgCodec->m_ulViuSyncCode1 = 0x000000F1;
		pQLCodecLib->m_pMpgCodec->m_ulViuSyncCode2 = 0xB6F1F100;
	} else {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "%s(): Unsupported video format resolution(%dx%d) scan_format(%d) fps(%d)\n", 
					 __FUNCTION__, dwWid, dwHgt, dwScanFmt, dwFps
					 ));
		return (QPERR_PARMS);
	}
	
	qpStatus = CTW9910_WriteBlock(This, pVals);
	
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "%s(): ****** WriteBlock() failed! ******\n",
					 __FUNCTION__
					 ));
		return (QPERR_FAIL);
	}
	
	// FIXME, need twice writing to make it work
	
	qpStatus = CTW9910_WriteBlock(This, pVals);
	
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "%s(): ****** WriteBlock() failed! ******\n",
					 __FUNCTION__
					 ));
		return (QPERR_FAIL);
	}	
				 
	DelayMilliSeconds(50);
	
	return (QPERR_SUCCESS);
}



// constructor
//
CTW9910 * CTW9910_Constructor(CTW9910 *pTW9910, 
							  CObject *pParent,
							  TQP_UINT32 dwAttributes,
							  ITuner *pTuner,
							  ITVAudio *pTvAudio,
							  II2C *pI2C,
							  TQP_UINT8 cI2CDevAddr
							  )
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTW9910_Constructor()\n"
				 ));

	if (pTW9910)
	{
		CVideoDecoder	*pVideoDecoder;

		pVideoDecoder = CVideoDecoder_Constructor(&pTW9910->m_VideoDecoder, 
												  pParent,
												  dwAttributes,
												  pTuner,
												  pTvAudio
												  );

		if (!pVideoDecoder)
		{
			pTW9910 = QP_NULL;
		}
		else
		{
			// fill in the function table
			//

			// IGenericDevice
			pTW9910->m_VideoDecoder.m_iVideoDecoder.InitDevice = CTW9910_InitDevice;

			// IVideoDecoder
			pTW9910->m_VideoDecoder.m_iVideoDecoder.SetVideoRes = CTW9910_SetVideoRes;

			// initialize members
			//
			pTW9910->m_pI2C = pI2C;
			pTW9910->m_cI2CDevAddr = cI2CDevAddr;

			// inherited members
			//
			pTW9910->m_VideoDecoder.m_nBrightnessDef = TW9910_BRIGHTNESS_DEFAULT;
			pTW9910->m_VideoDecoder.m_nBrightnessMin = TW9910_BRIGHTNESS_MIN;
			pTW9910->m_VideoDecoder.m_nBrightnessMax = TW9910_BRIGHTNESS_MAX;
			pTW9910->m_VideoDecoder.m_nBrightness = TW9910_BRIGHTNESS_DEFAULT;
			pTW9910->m_VideoDecoder.m_nContrastDef = TW9910_CONTRAST_DEFAULT; 
			pTW9910->m_VideoDecoder.m_nContrastMin = TW9910_CONTRAST_MIN;
			pTW9910->m_VideoDecoder.m_nContrastMax = TW9910_CONTRAST_MAX;
			pTW9910->m_VideoDecoder.m_nContrast = TW9910_CONTRAST_DEFAULT;
			pTW9910->m_VideoDecoder.m_nSaturationDef = TW9910_SATURATION_DEFAULT;
			pTW9910->m_VideoDecoder.m_nSaturationMin = TW9910_SATURATION_MIN;
			pTW9910->m_VideoDecoder.m_nSaturationMax = TW9910_SATURATION_MAX;
			pTW9910->m_VideoDecoder.m_nSaturation = TW9910_SATURATION_DEFAULT;
			pTW9910->m_VideoDecoder.m_nHueDef = TW9910_HUE_DEFAULT;
			pTW9910->m_VideoDecoder.m_nHueMin = TW9910_HUE_MIN;
			pTW9910->m_VideoDecoder.m_nHueMax = TW9910_HUE_MAX;
			pTW9910->m_VideoDecoder.m_nHue = TW9910_HUE_DEFAULT;
			pTW9910->m_VideoDecoder.m_nSharpnessDef = TW9910_SHARPNESS_DEFAULT;
			pTW9910->m_VideoDecoder.m_nSharpnessMin = TW9910_SHARPNESS_MIN;
			pTW9910->m_VideoDecoder.m_nSharpnessMax = TW9910_SHARPNESS_MAX;
			pTW9910->m_VideoDecoder.m_nSharpness = TW9910_SHARPNESS_DEFAULT;
		}
	}

	return (pTW9910);
}



// destructor
//
void CTW9910_Destructor(CTW9910 *This)
{
    //TQP_UINT8     buffer[2];
    //TQP_UINT8     ucRegAddr;
    //TQP_UINT8     ucValue;
    
    QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CTW9910_Destructor()\n"
				 ));

    CVideoDecoder_Destructor(&This->m_VideoDecoder);
}

