///////////////////////////////////////////////////////////////////////////////
//     
//
// Cviddecoder.c
//
// Description:
//
//  Audio codec implementation
//
// History:
//	Aug 28, 2006	(dt) created
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#include "cviddecoder.h"
#include "util.h"
#include "qpdbgmgr.h"

#define	VIDDEC_BRIGHTNESS_MIN					0x00
#define	VIDDEC_BRIGHTNESS_MAX					0xFF
#define	VIDDEC_BRIGHTNESS_DEFAULT				0x80

#define	VIDDEC_CONTRAST_MIN						0x00
#define	VIDDEC_CONTRAST_MAX						0x7F
#define	VIDDEC_CONTRAST_DEFAULT					0x44

#define	VIDDEC_SATURATION_MIN					0x00
#define	VIDDEC_SATURATION_MAX					0x7F
#define	VIDDEC_SATURATION_DEFAULT				0x40

#define	VIDDEC_HUE_MIN							(-128)
#define	VIDDEC_HUE_MAX							(+127)
#define	VIDDEC_HUE_DEFAULT						0x00

#define	VIDDEC_SHARPNESS_MIN					0
#define	VIDDEC_SHARPNESS_MAX					15
#define	VIDDEC_SHARPNESS_DEFAULT				8

#define	VIDDEC_EDGE_ENHANCEMENT_MIN				0x00
#define	VIDDEC_EDGE_ENHANCEMENT_MAX				0xFF
#define	VIDDEC_EDGE_ENHANCEMENT_DEFAULT			0x80

#define	VIDDEC_GAMMA_MIN						0x00
#define	VIDDEC_GAMMA_MAX						0xFF
#define	VIDDEC_GAMMA_DEFAULT					0x80

#define	VIDDEC_LOW_LIGHT_BOOST_MIN				0x00
#define	VIDDEC_LOW_LIGHT_BOOST_MAX				0xFF
#define	VIDDEC_LOW_LIGHT_BOOST_DEFAULT			0x80

#define	VIDDEC_EXPOSURE_MIN						0x00
#define	VIDDEC_EXPOSURE_MAX						0xFF
#define	VIDDEC_EXPOSURE_DEFAULT					0x80

#define	VIDDEC_AUTO_EXPOSURE_PRIORITY_MIN		0
#define	VIDDEC_AUTO_EXPOSURE_PRIORITY_MAX		1
#define	VIDDEC_AUTO_EXPOSURE_PRIORITY_DEFAULT	1

#define	VIDDEC_WHITE_BALANCE_MIN				0x00
#define	VIDDEC_WHITE_BALANCE_MAX				0xFF
#define	VIDDEC_WHITE_BALANCE_DEFAULT			0x80

#define	VIDDEC_FOCUS_MIN						0x00
#define	VIDDEC_FOCUS_MAX						0xFF
#define	VIDDEC_FOCUS_DEFAULT					0x80

#define	VIDDEC_POWER_LINE_FREQ_MIN				0
#define	VIDDEC_POWER_LINE_FREQ_MAX				2
#define	VIDDEC_POWER_LINE_FREQ_DEFAULT			2


extern TQP_DbgMCompID	g_QPPeripheralDBGCompID;


// IGenericDevice
//
static EQP_Errors CVideoDecoder_InitDevice(IVideoDecoder *This_p,
										   QP_DEVICE_CALLBACK pFuncCallback,
										   TQP_PVOID context,
										   TQP_PVOID pInitData
										   )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CVideoDecoder_InitDevice()\n"
				 ));

	// save callback address and context
	//
	This->m_pDeviceCallback = pFuncCallback;
	This->m_callbackContext = context;

	This->m_bVideoStarted = QP_FALSE;


	// CObject Init
	//
	CObject_Init(&This->m_Object);

	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_Release(IVideoDecoder *This_p)
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CVideoDecoder_Release()\n"
				 ));

	This->m_pDeviceCallback = QP_NULL;
	This->m_callbackContext = QP_NULL;
	CObject_Done(&This->m_Object);
	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_Reset(IVideoDecoder *This_p)
{
	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_Set(IVideoDecoder *This_p,
									TQP_REFGUID guid,
									TQP_OBJECT_HANDLE hIndex,
									TQP_UINT32 dwCode,
									TQP_PVOID pInput,
									TQP_PVOID pOutput,
									TQP_UINT32 dwSize
									)
{
	return (QPERR_NOTIMPL);
}



static EQP_Errors CVideoDecoder_Get(IVideoDecoder *This_p,
									TQP_REFGUID guid,
									TQP_OBJECT_HANDLE hIndex,
									TQP_UINT32 dwCode,
									TQP_PVOID pInput,
									TQP_PVOID pOutput,
									PTQP_UINT32 pdwSizeGot
									)
{
	return (QPERR_NOTIMPL);
}



// IVideoDecoder
//
static EQP_Errors CVideoDecoder_SelectVideoSource(IVideoDecoder *This_p,
												  VIDEO_SOURCE source
												  )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	This->m_VidSource = source;
	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_SetVideoSystem(IVideoDecoder *This_p,
											   VIDEO_STANDARD standard
											   )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	This->m_VidStandard = standard;
	return (QPERR_SUCCESS);
}

static EQP_Errors CVideoDecoder_SetVideoDataType(IVideoDecoder *This_p,
											   VIDEO_DATATYPE datatype
											   )
{
//	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);


	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_SetVcrTiming(IVideoDecoder *This_p,
											 TQP_UINT32 enable
											 )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	This->m_VCRTiming = enable;
	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_SetOutputEnabled(IVideoDecoder *This_p,
												 TQP_UINT32 enable
												 )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	This->m_OutputEnable = enable;
	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_GetVideoSource(IVideoDecoder *This_p,
											   PVIDEO_SOURCE pSource
											   )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	*pSource = This->m_VidSource;
	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_GetVideoSystem(IVideoDecoder *This_p,
											   PVIDEO_STANDARD pStandard
											   )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	*pStandard = This->m_VidStandard;
	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_SetVideoRes(IVideoDecoder *This_p,
											TQP_UINT32 dwWid,
											TQP_UINT32 dwHgt,
											TQP_UINT32 dwScanFmt,
											TQP_UINT32 dwFps
											)
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	This->m_dwWidth = dwWid;
	This->m_dwHeight = dwHgt;
	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_GetVcrTiming(IVideoDecoder *This_p,
											 TQP_UINT32 *pEnable
											 )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	*pEnable = This->m_VCRTiming;
	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_GetOutputEnabled(IVideoDecoder *This_p,
												 TQP_UINT32 *pEnable
												 )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	*pEnable = This->m_OutputEnable;
	return (QPERR_SUCCESS);
}



// Get Decoder Brightness range.
static EQP_Errors CVideoDecoder_GetBrightnessRange(IVideoDecoder *This_p,
												   int *pnDefault,
												   int *pnMin,
												   int *pnMax
												   )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	if ((QP_NULL == pnDefault) ||
		(QP_NULL == pnMin) ||
		(QP_NULL == pnMax)
		)
		return (QPERR_PARMS);

	*pnDefault = This->m_nBrightnessDef;
	*pnMin = This->m_nBrightnessMin;
	*pnMax = This->m_nBrightnessMax;

	return (QPERR_SUCCESS);
}



// Get Decoder Brightness
static EQP_Errors CVideoDecoder_GetBrightness(IVideoDecoder *This_p,
											  int *pnParamValue
											  )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	*pnParamValue = This->m_bMuted ? This->m_nSavedBrightness : This->m_nBrightness;
	return (QPERR_SUCCESS);
}



// Set Decoder Brightness
static EQP_Errors CVideoDecoder_SetBrightness(IVideoDecoder *This_p,
											  int nParamValue
											  )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	This->m_nBrightness = nParamValue;
	return (QPERR_SUCCESS);
}



// Get Decoder Contrast range.
static EQP_Errors CVideoDecoder_GetContrastRange(IVideoDecoder *This_p,
												 int *pnDefault,
												 int *pnMin,
												 int *pnMax
												 )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	if ((QP_NULL == pnDefault) ||
		(QP_NULL == pnMin) ||
		(QP_NULL == pnMax)
		)
		return (QPERR_PARMS);

	*pnDefault = This->m_nContrastDef;
	*pnMin = This->m_nContrastMin;
	*pnMax = This->m_nContrastMax;

	return (QPERR_SUCCESS);
}



// Get Decoder Contrast
static EQP_Errors CVideoDecoder_GetContrast(IVideoDecoder *This_p,
											int *pnParamValue
											)
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	*pnParamValue = This->m_bMuted ? This->m_nSavedContrast : This->m_nContrast;
	return (QPERR_SUCCESS);
}



// Set Decoder Contrast
static EQP_Errors CVideoDecoder_SetContrast(IVideoDecoder *This_p,
											int nParamValue
											)
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	This->m_nContrast = nParamValue;
	return (QPERR_SUCCESS);
}



// Get Decoder Saturation range.
static EQP_Errors CVideoDecoder_GetSaturationRange(IVideoDecoder *This_p,
												   int *pnDefault,
												   int *pnMin,
												   int *pnMax
												   )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	if ((QP_NULL == pnDefault) ||
		(QP_NULL == pnMin) ||
		(QP_NULL == pnMax)
		)
		return (QPERR_PARMS);

	*pnDefault = This->m_nSaturationDef;
	*pnMin = This->m_nSaturationMin;
	*pnMax = This->m_nSaturationMax;

	return (QPERR_SUCCESS);
}



// Get Decoder Saturation
static EQP_Errors CVideoDecoder_GetSaturation(IVideoDecoder *This_p,
											  int *pnParamValue
											  )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	*pnParamValue = This->m_bMuted ? This->m_nSavedSaturation : This->m_nSaturation;
	return (QPERR_SUCCESS);
}



// Set Decoder Saturation
static EQP_Errors CVideoDecoder_SetSaturation(IVideoDecoder *This_p,
											  int nParamValue
											  )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	This->m_nSaturation = nParamValue;
	return (QPERR_SUCCESS);
}



// Get Decoder Hue range.
static EQP_Errors CVideoDecoder_GetHueRange(IVideoDecoder *This_p,
											int *pnDefault,
											int *pnMin,
											int *pnMax
											)
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	if ((QP_NULL == pnDefault) ||
		(QP_NULL == pnMin) ||
		(QP_NULL == pnMax)
		)
		return (QPERR_PARMS);

	*pnDefault = This->m_nHueDef;
	*pnMin = This->m_nHueMin;
	*pnMax = This->m_nHueMax;

	return (QPERR_SUCCESS);
}



// Get Decoder Hue
static EQP_Errors CVideoDecoder_GetHue(IVideoDecoder *This_p,
									   int *pnParamValue
									   )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	*pnParamValue = This->m_bMuted ? This->m_nSavedHue : This->m_nHue;
	return (QPERR_SUCCESS);
}



// Set Decoder Hue
static EQP_Errors CVideoDecoder_SetHue(IVideoDecoder *This_p,
									   int nParamValue
									   )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	This->m_nHue = nParamValue;
	return (QPERR_SUCCESS);
}



// Get Decoder Sharpness range.
static EQP_Errors CVideoDecoder_GetSharpnessRange(IVideoDecoder *This_p,
												  int *pnDefault,
												  int *pnMin,
												  int *pnMax
												  )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	if ((QP_NULL == pnDefault) ||
		(QP_NULL == pnMin) ||
		(QP_NULL == pnMax)
		)
		return (QPERR_PARMS);

	*pnDefault = This->m_nSharpnessDef;
	*pnMin = This->m_nSharpnessMin;
	*pnMax = This->m_nSharpnessMax;

	return (QPERR_SUCCESS);
}



// Get Decoder Sharpness
static EQP_Errors CVideoDecoder_GetSharpness(IVideoDecoder *This_p,
											 int *pnParamValue
											 )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	*pnParamValue = This->m_nSharpness;
	return (QPERR_SUCCESS);
}



// Set Decoder Sharpness
static EQP_Errors CVideoDecoder_SetSharpness(IVideoDecoder *This_p,
											 int nParamValue
											 )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	This->m_nSharpness = nParamValue;
	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_DetectCopyProtection(IVideoDecoder *This_p,
													 TQP_BOOL *pCopyProtected,
													 PTQP_UINT8 pStatus
													 )
{
	*pCopyProtected = QP_FALSE;
	*pStatus = 0;
	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_DetectVideoStandard(IVideoDecoder *This_p,
													PVIDEO_STANDARD pStandard,
													PTQP_UINT32 pulSignalLocked,
													PTQP_UINT32 pulNumberOfLines
													)
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	*pulSignalLocked = 1;
	*pStandard = This->m_VidStandard;
	*pulNumberOfLines = (VIDEO_STANDARD_NTSC == This->m_VidStandard) ? 525 : 625;
	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_MuteVideo(IVideoDecoder *This_p)
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	if (!This->m_bMuted)
	{
		This->m_nSavedBrightness = This->m_nBrightness;
		This->m_nSavedContrast = This->m_nContrast;
		This->m_nSavedSaturation = This->m_nSaturation;
		This->m_nSavedHue = This->m_nHue;

		//
		//
		QPVideoDecoder_SetBrightness(This_p, This->m_nBrightnessMin);
		QPVideoDecoder_SetContrast(This_p, This->m_nContrastMin);
		QPVideoDecoder_SetSaturation(This_p, This->m_nSaturationMin);
		QPVideoDecoder_SetHue(This_p, This->m_nHueMin);

		This->m_bMuted = QP_TRUE;
	}

	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_UnmuteVideo(IVideoDecoder *This_p)
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	if (This->m_bMuted)
	{
		This->m_bMuted = QP_FALSE;

		QPVideoDecoder_SetBrightness(This_p, This->m_nSavedBrightness);
		QPVideoDecoder_SetContrast(This_p,This->m_nSavedContrast);
		QPVideoDecoder_SetSaturation(This_p,This->m_nSavedSaturation);
		QPVideoDecoder_SetHue(This_p,This->m_nSavedHue);
	}

	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_IsVideoMuted(IVideoDecoder *This_p,
											 TQP_BOOL *pbMuted
											 )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	*pbMuted = This->m_bMuted;
	return (QPERR_SUCCESS);
}



// sensor control extension
//

// edge enhancement
static EQP_Errors CVideoDecoder_GetEdgeEnhancementRange(IVideoDecoder *This_p,
														int *pnDefault,
			  											int *pnMin,
			  											int *pnMax
			  											)
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	if ((QP_NULL == pnDefault) ||
		(QP_NULL == pnMin) ||
		(QP_NULL == pnMax)
		)
		return (QPERR_PARMS);

	*pnDefault = This->m_nEdgeEnhancementDef;
	*pnMin = This->m_nEdgeEnhancementMin;
	*pnMax = This->m_nEdgeEnhancementMax;

	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_GetEdgeEnhancement(IVideoDecoder *This_p,
												   int *pnParamValue
												   )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	*pnParamValue = This->m_nEdgeEnhancement;
	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_SetEdgeEnhancement(IVideoDecoder *This_p,
												   int nParamValue
												   )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	This->m_nEdgeEnhancement = nParamValue;
	return (QPERR_SUCCESS);
}



// gamma
static EQP_Errors CVideoDecoder_GetGammaRange(IVideoDecoder *This_p,
											  int *pnDefault,
			  								  int *pnMin,
			  								  int *pnMax
			  								  )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	if ((QP_NULL == pnDefault) ||
		(QP_NULL == pnMin) ||
		(QP_NULL == pnMax)
		)
		return (QPERR_PARMS);

	*pnDefault = This->m_nGammaDef;
	*pnMin = This->m_nGammaMin;
	*pnMax = This->m_nGammaMax;

	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_GetGamma(IVideoDecoder *This_p,
										 int *pnParamValue
										 )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	*pnParamValue = This->m_nGamma;
	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_SetGamma(IVideoDecoder *This_p,
										 int nParamValue
										 )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	This->m_nGamma = nParamValue;
	return (QPERR_SUCCESS);
}



// low light boost
static EQP_Errors CVideoDecoder_GetLowLightBoostRange(IVideoDecoder *This_p,
													  int *pnDefault,
			  										  int *pnMin,
			  										  int *pnMax
			  										  )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	if ((QP_NULL == pnDefault) ||
		(QP_NULL == pnMin) ||
		(QP_NULL == pnMax)
		)
		return (QPERR_PARMS);

	*pnDefault = This->m_nLowLightBoostDef;
	*pnMin = This->m_nLowLightBoostMin;
	*pnMax = This->m_nLowLightBoostMax;

	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_GetLowLightBoost(IVideoDecoder *This_p,
												 int *pnParamValue
												 )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	*pnParamValue = This->m_nLowLightBoost;
	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_SetLowLightBoost(IVideoDecoder *This_p,
												 int nParamValue
												 )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	This->m_nLowLightBoost = nParamValue;
	return (QPERR_SUCCESS);
}



// exposure
static EQP_Errors CVideoDecoder_GetExposureRange(IVideoDecoder *This_p,
												 int *pnDefault,
			  									 int *pnMin,
			  									 int *pnMax
			  									 )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	if ((QP_NULL == pnDefault) ||
		(QP_NULL == pnMin) ||
		(QP_NULL == pnMax)
		)
		return (QPERR_PARMS);

	*pnDefault = This->m_nExposureDef;
	*pnMin = This->m_nExposureMin;
	*pnMax = This->m_nExposureMax;

	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_GetExposure(IVideoDecoder *This_p,
											int *pnParamValue
											)
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	*pnParamValue = This->m_nExposure;
	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_SetExposure(IVideoDecoder *This_p,
											int nParamValue
											)
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	This->m_nExposure = nParamValue;
	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_SetAutoExposureEnabled(IVideoDecoder *This_p,
													   TQP_UINT32 enable
													   )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	This->m_AutoExposureEnable = enable;
	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_GetAutoExposureEnabled(IVideoDecoder *This_p,
													   TQP_UINT32 *enable
													   )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	*enable = This->m_AutoExposureEnable;
	return (QPERR_SUCCESS);
}



// auto exposure priority
static EQP_Errors CVideoDecoder_GetAutoExposurePriorityRange(IVideoDecoder *This_p,
															 int *pnDefault,
			  												 int *pnMin,
			  												 int *pnMax
			  												 )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	if ((QP_NULL == pnDefault) ||
		(QP_NULL == pnMin) ||
		(QP_NULL == pnMax)
		)
		return (QPERR_PARMS);

	*pnDefault = This->m_nAutoExposurePriorityDef;
	*pnMin = This->m_nAutoExposurePriorityMin;
	*pnMax = This->m_nAutoExposurePriorityMax;

	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_GetAutoExposurePriority(IVideoDecoder *This_p,
														int *pnParamValue
														)
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	*pnParamValue = This->m_nAutoExposurePriority;
	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_SetAutoExposurePriority(IVideoDecoder *This_p,
														int nParamValue
														)
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	This->m_nAutoExposurePriority = nParamValue;
	return (QPERR_SUCCESS);
}



// white balance
static EQP_Errors CVideoDecoder_GetWhiteBalanceRange(IVideoDecoder *This_p,
													 int *pnDefault,
			  										 int *pnMin,
			  										 int *pnMax
			  										 )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	if ((QP_NULL == pnDefault) ||
		(QP_NULL == pnMin) ||
		(QP_NULL == pnMax)
		)
		return (QPERR_PARMS);

	*pnDefault = This->m_nWhiteBalanceDef;
	*pnMin = This->m_nWhiteBalanceMin;
	*pnMax = This->m_nWhiteBalanceMax;

	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_GetWhiteBalance(IVideoDecoder *This_p,
												int *pnParamValue
												)
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	*pnParamValue = This->m_nWhiteBalance;
	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_SetWhiteBalance(IVideoDecoder *This_p,
												int nParamValue
												)
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	This->m_nWhiteBalance = nParamValue;
	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_SetAutoWhiteBalanceEnabled(IVideoDecoder *This_p,
														   TQP_UINT32 enable
														   )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	This->m_AutoWhiteBalanceEnable = enable;
	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_GetAutoWhiteBalanceEnabled(IVideoDecoder *This_p,
														   TQP_UINT32 *enable
														   )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	*enable = This->m_AutoWhiteBalanceEnable;
	return (QPERR_SUCCESS);
}



// focus
static EQP_Errors CVideoDecoder_GetFocusRange(IVideoDecoder *This_p,
											  int *pnDefault,
			  								  int *pnMin,
			  								  int *pnMax
			  								  )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	if ((QP_NULL == pnDefault) ||
		(QP_NULL == pnMin) ||
		(QP_NULL == pnMax)
		)
		return (QPERR_PARMS);

	*pnDefault = This->m_nFocusDef;
	*pnMin = This->m_nFocusMin;
	*pnMax = This->m_nFocusMax;

	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_GetFocus(IVideoDecoder *This_p,
										 int *pnParamValue
										 )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	*pnParamValue = This->m_nFocus;
	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_SetFocus(IVideoDecoder *This_p,
										 int nParamValue
										 )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	This->m_nFocus = nParamValue;
	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_SetAutoFocusEnabled(IVideoDecoder *This_p,
													TQP_UINT32 enable
													)
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	This->m_AutoFocusEnable = enable;
	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_GetAutoFocusEnabled(IVideoDecoder *This_p,
													TQP_UINT32 *enable
													)
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	*enable = This->m_AutoFocusEnable;
	return (QPERR_SUCCESS);
}



// power line frequency
static EQP_Errors CVideoDecoder_GetPowerLineFreqRange(IVideoDecoder *This_p,
													  int *pnDefault,
			  										  int *pnMin,
			  										  int *pnMax
			  										  )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	if ((QP_NULL == pnDefault) ||
		(QP_NULL == pnMin) ||
		(QP_NULL == pnMax)
		)
		return (QPERR_PARMS);

	*pnDefault = This->m_nPowerLineFreqDef;
	*pnMin = This->m_nPowerLineFreqMin;
	*pnMax = This->m_nPowerLineFreqMax;

	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_GetPowerLineFreq(IVideoDecoder *This_p,
												 int *pnParamValue
												 )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	*pnParamValue = This->m_nPowerLineFreq;
	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_SetPowerLineFreq(IVideoDecoder *This_p,
												 int nParamValue
												 )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	This->m_nPowerLineFreq = nParamValue;
	return (QPERR_SUCCESS);
}


static EQP_Errors CVideoDecoder_IsHDCPContent(IVideoDecoder *This_p,
											  TQP_UINT32 *pnIsHDCP)
{
	*pnIsHDCP = 0;
	return (QPERR_SUCCESS);
}


static EQP_Errors CVideoDecoder_SetTriState(IVideoDecoder *This_p,
											TQP_UINT32 nTriState
											)
{
	return (QPERR_SUCCESS);
}



// start/stop video
//
static EQP_Errors CVideoDecoder_StartVideo(IVideoDecoder *This_p,
										   TQP_UINT32 fps
										   )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	This->m_bVideoStarted = QP_TRUE;
	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_StopVideo(IVideoDecoder *This_p)
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	This->m_bVideoStarted = QP_FALSE;
	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoDecoder_IsVideoStarted(IVideoDecoder *This_p,
											   TQP_UINT32 *pbStarted
											   )
{
	CVideoDecoder	*This =	GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);

	*pbStarted = This->m_bVideoStarted;
	return (QPERR_SUCCESS);
}



// constructor
//
CVideoDecoder * CVideoDecoder_Constructor(CVideoDecoder *pVideoDecoder,
										  CObject *pParent,
										  TQP_UINT32 dwAttributes,
										  ITuner *pTuner,
										  ITVAudio *pTvAudio
										  )
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CVideoDecoder_Constructor()\n"
				 ));

	if (pVideoDecoder)
	{
		CObject_Constructor(&pVideoDecoder->m_Object,
							pParent,
							dwAttributes
							);

		// fill in the function table
		//

		// IGenericDevice
		pVideoDecoder->m_iVideoDecoder.InitDevice = CVideoDecoder_InitDevice;
		pVideoDecoder->m_iVideoDecoder.Release = CVideoDecoder_Release;
		pVideoDecoder->m_iVideoDecoder.Reset = CVideoDecoder_Reset;
		pVideoDecoder->m_iVideoDecoder.Set = CVideoDecoder_Set;
		pVideoDecoder->m_iVideoDecoder.Get = CVideoDecoder_Get;

		// IVideoDecoder
		pVideoDecoder->m_iVideoDecoder.SelectVideoSource = CVideoDecoder_SelectVideoSource;
		pVideoDecoder->m_iVideoDecoder.SetVideoSystem = CVideoDecoder_SetVideoSystem;
		pVideoDecoder->m_iVideoDecoder.SetVideoDataType = CVideoDecoder_SetVideoDataType;
		pVideoDecoder->m_iVideoDecoder.SetVcrTiming = CVideoDecoder_SetVcrTiming;
		pVideoDecoder->m_iVideoDecoder.SetOutputEnabled = CVideoDecoder_SetOutputEnabled;
		pVideoDecoder->m_iVideoDecoder.GetVideoSource = CVideoDecoder_GetVideoSource;
		pVideoDecoder->m_iVideoDecoder.GetVideoSystem = CVideoDecoder_GetVideoSystem;
		pVideoDecoder->m_iVideoDecoder.SetVideoRes = CVideoDecoder_SetVideoRes;
		pVideoDecoder->m_iVideoDecoder.GetVcrTiming = CVideoDecoder_GetVcrTiming;
		pVideoDecoder->m_iVideoDecoder.GetOutputEnabled = CVideoDecoder_GetOutputEnabled;
		pVideoDecoder->m_iVideoDecoder.GetBrightnessRange = CVideoDecoder_GetBrightnessRange;
		pVideoDecoder->m_iVideoDecoder.GetBrightness = CVideoDecoder_GetBrightness;
		pVideoDecoder->m_iVideoDecoder.SetBrightness = CVideoDecoder_SetBrightness;
		pVideoDecoder->m_iVideoDecoder.GetContrastRange = CVideoDecoder_GetContrastRange;
		pVideoDecoder->m_iVideoDecoder.GetContrast = CVideoDecoder_GetContrast;
		pVideoDecoder->m_iVideoDecoder.SetContrast = CVideoDecoder_SetContrast;
		pVideoDecoder->m_iVideoDecoder.GetSaturationRange = CVideoDecoder_GetSaturationRange;
		pVideoDecoder->m_iVideoDecoder.GetSaturation = CVideoDecoder_GetSaturation;
		pVideoDecoder->m_iVideoDecoder.SetSaturation = CVideoDecoder_SetSaturation;
		pVideoDecoder->m_iVideoDecoder.GetHueRange = CVideoDecoder_GetHueRange;
		pVideoDecoder->m_iVideoDecoder.GetHue = CVideoDecoder_GetHue;
		pVideoDecoder->m_iVideoDecoder.SetHue = CVideoDecoder_SetHue;
		pVideoDecoder->m_iVideoDecoder.GetSharpnessRange = CVideoDecoder_GetSharpnessRange;
		pVideoDecoder->m_iVideoDecoder.GetSharpness = CVideoDecoder_GetSharpness;
		pVideoDecoder->m_iVideoDecoder.SetSharpness = CVideoDecoder_SetSharpness;
		pVideoDecoder->m_iVideoDecoder.DetectCopyProtection = CVideoDecoder_DetectCopyProtection;
		pVideoDecoder->m_iVideoDecoder.DetectVideoStandard = CVideoDecoder_DetectVideoStandard;
		pVideoDecoder->m_iVideoDecoder.MuteVideo = CVideoDecoder_MuteVideo;
		pVideoDecoder->m_iVideoDecoder.UnmuteVideo = CVideoDecoder_UnmuteVideo;
		pVideoDecoder->m_iVideoDecoder.IsVideoMuted = CVideoDecoder_IsVideoMuted;
		pVideoDecoder->m_iVideoDecoder.GetEdgeEnhancementRange = CVideoDecoder_GetEdgeEnhancementRange;
		pVideoDecoder->m_iVideoDecoder.GetEdgeEnhancement = CVideoDecoder_GetEdgeEnhancement;
		pVideoDecoder->m_iVideoDecoder.SetEdgeEnhancement = CVideoDecoder_SetEdgeEnhancement;
		pVideoDecoder->m_iVideoDecoder.GetGammaRange = CVideoDecoder_GetGammaRange;
		pVideoDecoder->m_iVideoDecoder.GetGamma = CVideoDecoder_GetGamma;
		pVideoDecoder->m_iVideoDecoder.SetGamma = CVideoDecoder_SetGamma;
		pVideoDecoder->m_iVideoDecoder.GetLowLightBoostRange = CVideoDecoder_GetLowLightBoostRange;
		pVideoDecoder->m_iVideoDecoder.GetLowLightBoost = CVideoDecoder_GetLowLightBoost;
		pVideoDecoder->m_iVideoDecoder.SetLowLightBoost = CVideoDecoder_SetLowLightBoost;
		pVideoDecoder->m_iVideoDecoder.GetExposureRange = CVideoDecoder_GetExposureRange;
		pVideoDecoder->m_iVideoDecoder.GetExposure = CVideoDecoder_GetExposure;
		pVideoDecoder->m_iVideoDecoder.SetExposure = CVideoDecoder_SetExposure;
		pVideoDecoder->m_iVideoDecoder.SetAutoExposureEnabled = CVideoDecoder_SetAutoExposureEnabled;
		pVideoDecoder->m_iVideoDecoder.GetAutoExposureEnabled = CVideoDecoder_GetAutoExposureEnabled;
		pVideoDecoder->m_iVideoDecoder.GetAutoExposurePriorityRange = CVideoDecoder_GetAutoExposurePriorityRange;
		pVideoDecoder->m_iVideoDecoder.GetAutoExposurePriority = CVideoDecoder_GetAutoExposurePriority;
		pVideoDecoder->m_iVideoDecoder.SetAutoExposurePriority = CVideoDecoder_SetAutoExposurePriority;
		pVideoDecoder->m_iVideoDecoder.GetWhiteBalanceRange = CVideoDecoder_GetWhiteBalanceRange;
		pVideoDecoder->m_iVideoDecoder.GetWhiteBalance = CVideoDecoder_GetWhiteBalance;
		pVideoDecoder->m_iVideoDecoder.SetWhiteBalance = CVideoDecoder_SetWhiteBalance;
		pVideoDecoder->m_iVideoDecoder.SetAutoWhiteBalanceEnabled = CVideoDecoder_SetAutoWhiteBalanceEnabled;
		pVideoDecoder->m_iVideoDecoder.GetAutoWhiteBalanceEnabled = CVideoDecoder_GetAutoWhiteBalanceEnabled;
		pVideoDecoder->m_iVideoDecoder.GetFocusRange = CVideoDecoder_GetFocusRange;
		pVideoDecoder->m_iVideoDecoder.GetFocus = CVideoDecoder_GetFocus;
		pVideoDecoder->m_iVideoDecoder.SetFocus = CVideoDecoder_SetFocus;
		pVideoDecoder->m_iVideoDecoder.SetAutoFocusEnabled = CVideoDecoder_SetAutoFocusEnabled;
		pVideoDecoder->m_iVideoDecoder.GetAutoFocusEnabled = CVideoDecoder_GetAutoFocusEnabled;
		pVideoDecoder->m_iVideoDecoder.GetPowerLineFreqRange = CVideoDecoder_GetPowerLineFreqRange;
		pVideoDecoder->m_iVideoDecoder.GetPowerLineFreq = CVideoDecoder_GetPowerLineFreq;
		pVideoDecoder->m_iVideoDecoder.SetPowerLineFreq = CVideoDecoder_SetPowerLineFreq;
		pVideoDecoder->m_iVideoDecoder.StartVideo = CVideoDecoder_StartVideo;
		pVideoDecoder->m_iVideoDecoder.StopVideo = CVideoDecoder_StopVideo;
		pVideoDecoder->m_iVideoDecoder.IsVideoStarted = CVideoDecoder_IsVideoStarted;

		pVideoDecoder->m_iVideoDecoder.IsHDCPContent = CVideoDecoder_IsHDCPContent;
		pVideoDecoder->m_iVideoDecoder.SetTriState = CVideoDecoder_SetTriState;

		// initialize members
		//
		pVideoDecoder->m_pDeviceCallback = QP_NULL;
		pVideoDecoder->m_callbackContext = QP_NULL;
		pVideoDecoder->m_pTuner = pTuner;
		pVideoDecoder->m_pTvAudio = pTvAudio;
		pVideoDecoder->m_VidSource = VIDEO_SOURCE_CVIDEO1;
		pVideoDecoder->m_VidStandard = VIDEO_STANDARD_NTSC;
		pVideoDecoder->m_OutputEnable = QP_TRUE;
		pVideoDecoder->m_VCRTiming = QP_FALSE;
		pVideoDecoder->m_bMuted = QP_FALSE;
		pVideoDecoder->m_dwWidth = 1280;
		pVideoDecoder->m_dwHeight = 720;

		pVideoDecoder->m_nBrightnessDef = VIDDEC_BRIGHTNESS_DEFAULT;
		pVideoDecoder->m_nBrightnessMin = VIDDEC_BRIGHTNESS_MIN;
		pVideoDecoder->m_nBrightnessMax = VIDDEC_BRIGHTNESS_MAX;
		pVideoDecoder->m_nBrightness = VIDDEC_BRIGHTNESS_DEFAULT;

		pVideoDecoder->m_nContrastDef = VIDDEC_CONTRAST_DEFAULT;
		pVideoDecoder->m_nContrastMin = VIDDEC_CONTRAST_MIN;
		pVideoDecoder->m_nContrastMax = VIDDEC_CONTRAST_MAX;
		pVideoDecoder->m_nContrast = VIDDEC_CONTRAST_DEFAULT;

		pVideoDecoder->m_nSaturationDef = VIDDEC_SATURATION_DEFAULT;
		pVideoDecoder->m_nSaturationMin = VIDDEC_SATURATION_MIN;
		pVideoDecoder->m_nSaturationMax = VIDDEC_SATURATION_MAX;
		pVideoDecoder->m_nSaturation = VIDDEC_SATURATION_DEFAULT;

		pVideoDecoder->m_nHueDef = VIDDEC_HUE_DEFAULT;
		pVideoDecoder->m_nHueMin = VIDDEC_HUE_MIN;
		pVideoDecoder->m_nHueMax = VIDDEC_HUE_MAX;
		pVideoDecoder->m_nHue = VIDDEC_HUE_DEFAULT;

		pVideoDecoder->m_nSharpnessDef = VIDDEC_SHARPNESS_DEFAULT;
		pVideoDecoder->m_nSharpnessMin = VIDDEC_SHARPNESS_MIN;
		pVideoDecoder->m_nSharpnessMax = VIDDEC_SHARPNESS_MAX;
		pVideoDecoder->m_nSharpness = VIDDEC_SHARPNESS_DEFAULT;

		pVideoDecoder->m_nEdgeEnhancementDef = VIDDEC_EDGE_ENHANCEMENT_DEFAULT;
		pVideoDecoder->m_nEdgeEnhancementMin = VIDDEC_EDGE_ENHANCEMENT_MIN;
		pVideoDecoder->m_nEdgeEnhancementMax = VIDDEC_EDGE_ENHANCEMENT_MAX;
		pVideoDecoder->m_nEdgeEnhancement = VIDDEC_EDGE_ENHANCEMENT_DEFAULT;

		pVideoDecoder->m_nGammaDef = VIDDEC_GAMMA_DEFAULT;
		pVideoDecoder->m_nGammaMin = VIDDEC_GAMMA_MIN;
		pVideoDecoder->m_nGammaMax = VIDDEC_GAMMA_MAX;
		pVideoDecoder->m_nGamma = VIDDEC_GAMMA_DEFAULT;

		pVideoDecoder->m_nLowLightBoostDef = VIDDEC_LOW_LIGHT_BOOST_DEFAULT;
		pVideoDecoder->m_nLowLightBoostMin = VIDDEC_LOW_LIGHT_BOOST_MIN;
		pVideoDecoder->m_nLowLightBoostMax = VIDDEC_LOW_LIGHT_BOOST_MAX;
		pVideoDecoder->m_nLowLightBoost = VIDDEC_LOW_LIGHT_BOOST_DEFAULT;

		pVideoDecoder->m_nExposureDef = VIDDEC_EXPOSURE_DEFAULT;
		pVideoDecoder->m_nExposureMin = VIDDEC_EXPOSURE_MIN;
		pVideoDecoder->m_nExposureMax = VIDDEC_EXPOSURE_MAX;
		pVideoDecoder->m_nExposure = VIDDEC_EXPOSURE_DEFAULT;
		pVideoDecoder->m_AutoExposureEnable = QP_TRUE;

		pVideoDecoder->m_nAutoExposurePriorityDef = VIDDEC_AUTO_EXPOSURE_PRIORITY_DEFAULT;
		pVideoDecoder->m_nAutoExposurePriorityMin = VIDDEC_AUTO_EXPOSURE_PRIORITY_MIN;
		pVideoDecoder->m_nAutoExposurePriorityMax = VIDDEC_AUTO_EXPOSURE_PRIORITY_MAX;
		pVideoDecoder->m_nAutoExposurePriority = VIDDEC_AUTO_EXPOSURE_PRIORITY_DEFAULT;

		pVideoDecoder->m_nWhiteBalanceDef = VIDDEC_WHITE_BALANCE_DEFAULT;
		pVideoDecoder->m_nWhiteBalanceMin = VIDDEC_WHITE_BALANCE_MIN;
		pVideoDecoder->m_nWhiteBalanceMax = VIDDEC_WHITE_BALANCE_MAX;
		pVideoDecoder->m_nWhiteBalance = VIDDEC_WHITE_BALANCE_DEFAULT;
		pVideoDecoder->m_AutoWhiteBalanceEnable = QP_TRUE;

		pVideoDecoder->m_nFocusDef = VIDDEC_FOCUS_DEFAULT;
		pVideoDecoder->m_nFocusMin = VIDDEC_FOCUS_MIN;
		pVideoDecoder->m_nFocusMax = VIDDEC_FOCUS_MAX;
		pVideoDecoder->m_nFocus = VIDDEC_FOCUS_DEFAULT;
		pVideoDecoder->m_AutoFocusEnable = QP_FALSE;

		pVideoDecoder->m_nPowerLineFreqDef = VIDDEC_POWER_LINE_FREQ_DEFAULT;
		pVideoDecoder->m_nPowerLineFreqMin = VIDDEC_POWER_LINE_FREQ_MIN;
		pVideoDecoder->m_nPowerLineFreqMax = VIDDEC_POWER_LINE_FREQ_MAX;
		pVideoDecoder->m_nPowerLineFreq = VIDDEC_POWER_LINE_FREQ_DEFAULT;

		pVideoDecoder->m_bVideoStarted = QP_FALSE;
	}

	return (pVideoDecoder);
}



// destructor
//
void CVideoDecoder_Destructor(CVideoDecoder *This)
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CVideoDecoder_Destructor()\n"
				 ));

	if (CObject_IsInitialized(&This->m_Object))
		QPGenericDevice_Release(&This->m_iVideoDecoder);

	CObject_Destructor(&This->m_Object);
}

