///////////////////////////////////////////////////////////////////////////////
//        
//
// Cviddecoder.h
//
// Description: 
//
//  Video decoder implementation
// 
// History:
//	Sep 5, 2006	(dt) created
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#ifndef __CVIDDECODER_H__
#define __CVIDDECODER_H__


#include "cobject.h"
#include "qpviddecoder.h"
#include "qptuner.h"
#include "qptvaudio.h"


#ifdef __cplusplus
extern "C" {
#endif


typedef struct CVideoDecoder
{
	// public CObject
	//
	CObject				m_Object;

	// public IVideoDecoder
	//
	IVideoDecoder		m_iVideoDecoder;

	// CVideoDecoder
	//
	QP_DEVICE_CALLBACK	m_pDeviceCallback;					// codec client call back function
	TQP_PVOID			m_callbackContext;					// codec client call back function context

	ITuner				*m_pTuner;
	ITVAudio			*m_pTvAudio;

	VIDEO_SOURCE		m_VidSource;						// video source
	VIDEO_STANDARD		m_VidStandard;						// video standard
	TQP_UINT32			m_OutputEnable;						// decoder output enabled
	TQP_UINT32			m_VCRTiming;						// vcr timing enabled

	TQP_UINT32			m_dwWidth;
	TQP_UINT32			m_dwHeight;

	int					m_nBrightnessDef;
	int					m_nBrightnessMin;
	int					m_nBrightnessMax;
	int					m_nBrightness;

	int					m_nContrastDef;
	int					m_nContrastMin;
	int					m_nContrastMax;
	int					m_nContrast;

	int					m_nSaturationDef;
	int					m_nSaturationMin;
	int					m_nSaturationMax;
	int					m_nSaturation;

	int					m_nHueDef;
	int					m_nHueMin;
	int					m_nHueMax;
	int					m_nHue;

	int					m_nSharpnessDef;
	int					m_nSharpnessMin;
	int					m_nSharpnessMax;
	int					m_nSharpness;


	TQP_BOOL			m_bMuted;
	int					m_nSavedBrightness;
	int					m_nSavedContrast;
	int					m_nSavedSaturation;
	int					m_nSavedHue;

	int					m_nEdgeEnhancementDef;
	int					m_nEdgeEnhancementMin;
	int					m_nEdgeEnhancementMax;
	int					m_nEdgeEnhancement;

	int					m_nGammaDef;
	int					m_nGammaMin;
	int					m_nGammaMax;
	int					m_nGamma;

	int					m_nLowLightBoostDef;
	int					m_nLowLightBoostMin;
	int					m_nLowLightBoostMax;
	int					m_nLowLightBoost;

	int					m_nExposureDef;
	int					m_nExposureMin;
	int					m_nExposureMax;
	int					m_nExposure;
	TQP_UINT32			m_AutoExposureEnable;

	int					m_nAutoExposurePriorityDef;
	int					m_nAutoExposurePriorityMin;
	int					m_nAutoExposurePriorityMax;
	int					m_nAutoExposurePriority;

	int					m_nWhiteBalanceDef;
	int					m_nWhiteBalanceMin;
	int					m_nWhiteBalanceMax;
	int					m_nWhiteBalance;
	TQP_UINT32			m_AutoWhiteBalanceEnable;

	int					m_nFocusDef;
	int					m_nFocusMin;
	int					m_nFocusMax;
	int					m_nFocus;
	TQP_UINT32			m_AutoFocusEnable;

	int					m_nPowerLineFreqDef;
	int					m_nPowerLineFreqMin;
	int					m_nPowerLineFreqMax;
	int					m_nPowerLineFreq;

	TQP_BOOL			m_bVideoStarted;

} CVideoDecoder;


// constructor
//
CVideoDecoder * CVideoDecoder_Constructor(CVideoDecoder *pVideoDecoder, 
										  CObject *pParent,
										  TQP_UINT32 dwAttributes,
										  ITuner *pTuner,
										  ITVAudio *pTvAudio
										  );

// destructor
//
void CVideoDecoder_Destructor(CVideoDecoder *This);

#ifdef __cplusplus
}
#endif

#endif //__CVIDDECODER_H__


