///////////////////////////////////////////////////////////////////////////////
//        
//
// Cvidencoder.c
//
// Description: 
//
//  Video encoder implementation
// 
// History:
//	Sep 5, 2006	(dt) created
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#include "cvidencoder.h"
#include "util.h"
#include "qpdbgmgr.h"

extern TQP_DbgMCompID	g_QPPeripheralDBGCompID;


// IGenericDevice
//
static EQP_Errors CVideoEncoder_InitDevice(IVideoEncoder *This_p, 
										   QP_DEVICE_CALLBACK pFuncCallback,
										   TQP_PVOID context,
										   TQP_PVOID pInitData
										   )
{
	CVideoEncoder	*This = GET_INHERITED_OBJECT(CVideoEncoder, m_iVideoEncoder, This_p);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CVideoEncoder_InitDevice()\n"
				 ));

	// save callback address and context
	//
	This->m_pDeviceCallback = pFuncCallback;
	This->m_callbackContext = context;

	This->m_fVideoEncoderEnabled = QP_TRUE;

	// CObject Init
	//
	CObject_Init(&This->m_Object);

	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoEncoder_Release(IVideoEncoder *This_p)
{
	CVideoEncoder	*This = GET_INHERITED_OBJECT(CVideoEncoder, m_iVideoEncoder, This_p);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CVideoEncoder_Release()\n"
				 ));

	This->m_pDeviceCallback = QP_NULL;
	This->m_callbackContext = QP_NULL;
	CObject_Done(&This->m_Object);
	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoEncoder_Reset(IVideoEncoder *This_p) 
{
	return (QPERR_SUCCESS);
}



static EQP_Errors CVideoEncoder_Set(IVideoEncoder *This_p,
									TQP_REFGUID guid,
									TQP_OBJECT_HANDLE hIndex,
									TQP_UINT32 dwCode,
									TQP_PVOID pInput,
									TQP_PVOID pOutput,
									TQP_UINT32 dwSize
									)
{
	return (QPERR_NOTIMPL);
}



static EQP_Errors CVideoEncoder_Get(IVideoEncoder *This_p,
									TQP_REFGUID guid,
									TQP_OBJECT_HANDLE hIndex,
									TQP_UINT32 dwCode,
									TQP_PVOID pInput,
									TQP_PVOID pOutput,
									PTQP_UINT32 pdwSizeGot
									)
{
	return (QPERR_NOTIMPL);
}



// IVideoEncoder
//

// Enables encoder device.
static EQP_Errors CVideoEncoder_Enable(IVideoEncoder *This_p) 
{
	CVideoEncoder	*This = GET_INHERITED_OBJECT(CVideoEncoder, m_iVideoEncoder, This_p);

	This->m_fVideoEncoderEnabled = QP_TRUE; 
	return (QPERR_SUCCESS);
}



// Disables encoder device.
static EQP_Errors CVideoEncoder_Disable(IVideoEncoder *This_p) 
{
	CVideoEncoder	*This = GET_INHERITED_OBJECT(CVideoEncoder, m_iVideoEncoder, This_p);

	This->m_fVideoEncoderEnabled = QP_FALSE; 
	return (QPERR_SUCCESS);
}



// Get encoder status.
static EQP_Errors CVideoEncoder_IsEnabled(IVideoEncoder *This_p,
										  TQP_BOOL *pfEnabled
										  ) 
{
	CVideoEncoder	*This = GET_INHERITED_OBJECT(CVideoEncoder, m_iVideoEncoder, This_p);

	*pfEnabled = This->m_fVideoEncoderEnabled;
	return (QPERR_SUCCESS);
}



// Sets video system of the encoder.
static EQP_Errors CVideoEncoder_SetVideoStandard(IVideoEncoder *This_p,
												 VIDEO_STANDARD standard
												 ) 
{
	CVideoEncoder	*This = GET_INHERITED_OBJECT(CVideoEncoder, m_iVideoEncoder, This_p);

	This->m_VidStandard = standard; 
	return (QPERR_SUCCESS);
}



// Get video system of the encoder.
static EQP_Errors CVideoEncoder_GetVideoStandard(IVideoEncoder *This_p,
												 PVIDEO_STANDARD pStandard
												 )
{
	CVideoEncoder	*This = GET_INHERITED_OBJECT(CVideoEncoder, m_iVideoEncoder, This_p);

	*pStandard = This->m_VidStandard;
	return (QPERR_SUCCESS);
}



// constructor
//
CVideoEncoder * CVideoEncoder_Constructor(CVideoEncoder *pVideoEncoder, 
										  CObject *pParent,
										  TQP_UINT32 dwAttributes
										  )
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CVideoEncoder_Constructor()\n"
				 ));

	if (pVideoEncoder)
	{
		CObject_Constructor(&pVideoEncoder->m_Object, 
							pParent, 
							dwAttributes
							);

		// fill in the function table
		//

		// IGenericDevice
		pVideoEncoder->m_iVideoEncoder.InitDevice = CVideoEncoder_InitDevice;
		pVideoEncoder->m_iVideoEncoder.Release = CVideoEncoder_Release;
		pVideoEncoder->m_iVideoEncoder.Reset = CVideoEncoder_Reset;
		pVideoEncoder->m_iVideoEncoder.Set = CVideoEncoder_Set;
		pVideoEncoder->m_iVideoEncoder.Get = CVideoEncoder_Get;

		// IVideoEncoder
		pVideoEncoder->m_iVideoEncoder.Enable = CVideoEncoder_Enable;
		pVideoEncoder->m_iVideoEncoder.Disable = CVideoEncoder_Disable;
		pVideoEncoder->m_iVideoEncoder.IsEnabled = CVideoEncoder_IsEnabled;
		pVideoEncoder->m_iVideoEncoder.SetVideoStandard = CVideoEncoder_SetVideoStandard;
		pVideoEncoder->m_iVideoEncoder.GetVideoStandard = CVideoEncoder_GetVideoStandard;

		// initialize members
		//
		pVideoEncoder->m_pDeviceCallback = QP_NULL;
		pVideoEncoder->m_callbackContext = QP_NULL;
		pVideoEncoder->m_VidStandard = VIDEO_STANDARD_NTSC;
		pVideoEncoder->m_fVideoEncoderEnabled = QP_TRUE;
	}

	return (pVideoEncoder);
}



// destructor
//
void CVideoEncoder_Destructor(CVideoEncoder *This)
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CVideoEncoder_Destructor()\n"
				 ));

	if (CObject_IsInitialized(&This->m_Object))
		QPGenericDevice_Release(&This->m_iVideoEncoder);

	CObject_Destructor(&This->m_Object);
}

