///////////////////////////////////////////////////////////////////////////////
//        
//
// CVidInputAuto1.c
//
// Description: 
//   CVidInputAuto1 implementation
// 
// History:
//	 Sep. 23, 2010 created
//
// Authors: Jack Lee
//
///////////////////////////////////////////////////////////////////////////////


#include "cvidinputauto1.h"
#include "ctvp7002.h"
#include "cadv7614.h"
#include "qptimemgr.h"
#include "qpdbgmgr.h"
#include "util.h"
#include "cqlcodeclib.h"
#include "ci2c.h"


EQP_Errors CADV7614_InitDevice(IVideoDecoder *This_p, 
							   QP_DEVICE_CALLBACK pFuncCallback,
							   TQP_PVOID context,
							   TQP_PVOID pInitData
							   );
EQP_Errors CADV7614_SetVideoRes(IVideoDecoder *This_p,
							 	TQP_UINT32 dwWid,
				  				TQP_UINT32 dwHgt,
				  				TQP_UINT32 dwScanFmt,
				  				TQP_UINT32 dwFps
				  				);
EQP_Errors CTVP7002_InitDevice(IVideoDecoder *This_p, 
							   QP_DEVICE_CALLBACK pFuncCallback,
							   TQP_PVOID context,
							   TQP_PVOID pInitData
							   );
EQP_Errors CTVP7002_SetVideoRes(IVideoDecoder *This_p,
							 	TQP_UINT32 dwWid,
				  				TQP_UINT32 dwHgt,
				  				TQP_UINT32 dwScanFmt,
				  				TQP_UINT32 dwFps
				  				);


/////////////////////////////////////////////////////////////////////////////
//
//

#define	VIDINPUTAUTO1_BRIGHTNESS_MIN			0x00   // not supported
#define	VIDINPUTAUTO1_BRIGHTNESS_MAX			0xFF   // not supported
#define	VIDINPUTAUTO1_BRIGHTNESS_DEFAULT		0x80   // not supported

#define	VIDINPUTAUTO1_CONTRAST_MIN				0x00   // not supported
#define	VIDINPUTAUTO1_CONTRAST_MAX				0x7F   // not supported
#define	VIDINPUTAUTO1_CONTRAST_DEFAULT			0x44   // not supported

#define	VIDINPUTAUTO1_SATURATION_MIN			0x00   // not supported
#define	VIDINPUTAUTO1_SATURATION_MAX			0x7F   // not supported
#define	VIDINPUTAUTO1_SATURATION_DEFAULT		0x40   // not supported

#define	VIDINPUTAUTO1_HUE_MIN					(-128)	// 0x80   // not supported
#define	VIDINPUTAUTO1_HUE_MAX					(+127)	// 0x7F   // not supported
#define	VIDINPUTAUTO1_HUE_DEFAULT				0x00   // not supported

#define	VIDINPUTAUTO1_SHARPNESS_MIN				0      // not supported
#define	VIDINPUTAUTO1_SHARPNESS_MAX				15     // not supported
#define	VIDINPUTAUTO1_SHARPNESS_DEFAULT			8      // not supported


extern TQP_DbgMCompID	g_QPPeripheralDBGCompID;

/////////////////////////////////////////////////////////////////////////////
//
//


// IGenericDevice
//
static EQP_Errors CVidInputAuto1_InitDevice(IVideoDecoder *This_p, 
											QP_DEVICE_CALLBACK pFuncCallback,
											TQP_PVOID context,
											TQP_PVOID pInitData
											)
{
    CVideoDecoder *pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
    CVidInputAuto1 *This = GET_INHERITED_OBJECT(CVidInputAuto1, m_VideoDecoder, pVideoDecoder);
    CI2C *pCI2C = GET_INHERITED_OBJECT(CI2C, m_ii2c, This->m_pI2C);

	TQP_UINT32	regF8 = 0;
	QPHCI_RegisterRead(&pCI2C->m_pMpegCodec->m_hci, 0xf8, &regF8);
					   
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputAuto1_InitDevice() regF8=0x%08x\n", regF8
				 ));
				 
	switch (regF8)
	{
		case 0x08: //VD4(adv7614)
			This->m_VideoDecoder.m_iVideoDecoder.InitDevice = CADV7614_InitDevice;
			This->m_VideoDecoder.m_iVideoDecoder.SetVideoRes = CADV7614_SetVideoRes;
			This->m_cI2CDevAddr = ADV7614_DEVICE_ADDR_IO;
			return CADV7614_InitDevice(This_p, 
									   pFuncCallback,
									   context,
									   pInitData
									   );						
			break;
		case 0x0C: //FP2(tvp7002)
			This->m_VideoDecoder.m_iVideoDecoder.InitDevice = CTVP7002_InitDevice;
			This->m_VideoDecoder.m_iVideoDecoder.SetVideoRes = CTVP7002_SetVideoRes;
			This->m_cI2CDevAddr = TVP7002_DEVICE_ADDR;
			return CTVP7002_InitDevice(This_p, 
									   pFuncCallback,
									   context,
									   pInitData
									   );
			break;
		case 0x0A: //VD2(dummy)
		default:
			break;	
	}	
	
	CObject_Init(&pVideoDecoder->m_Object);
    
    return (QPERR_SUCCESS);
}



// IVideoDecoder
//
static EQP_Errors CVidInputAuto1_SetVideoRes(IVideoDecoder *This_p,
									  		 TQP_UINT32 dwWid,
				  					  		 TQP_UINT32 dwHgt,
				  					  		 TQP_UINT32 dwScanFmt,
				  					  		 TQP_UINT32 dwFps
				  					  		 )
{
	return (QPERR_SUCCESS);
}



// constructor
//
CVidInputAuto1 * CVidInputAuto1_Constructor(CVidInputAuto1 *pVidInputAuto1, 
											CObject *pParent,
											TQP_UINT32 dwAttributes,
											ITuner *pTuner,
											ITVAudio *pTvAudio,
											II2C *pI2C,
											TQP_UINT8 cI2CDevAddr
											)
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CVidInputAuto1_Constructor()\n"
				 ));

	if (pVidInputAuto1)
	{
		CVideoDecoder	*pVideoDecoder;

		pVideoDecoder = CVideoDecoder_Constructor(&pVidInputAuto1->m_VideoDecoder, 
												  pParent,
												  dwAttributes,
												  pTuner,
												  pTvAudio
												  );

		if (!pVideoDecoder)
		{
			pVidInputAuto1 = QP_NULL;
		}
		else
		{
			// fill in the function table
			//

			// IGenericDevice
			pVidInputAuto1->m_VideoDecoder.m_iVideoDecoder.InitDevice = CVidInputAuto1_InitDevice;

			// IVideoDecoder
			pVidInputAuto1->m_VideoDecoder.m_iVideoDecoder.SetVideoRes = CVidInputAuto1_SetVideoRes;

			// initialize members
			//
			pVidInputAuto1->m_pI2C = pI2C;
			pVidInputAuto1->m_cI2CDevAddr = cI2CDevAddr;

			// inherited members
			//
			pVidInputAuto1->m_VideoDecoder.m_nBrightnessDef = VIDINPUTAUTO1_BRIGHTNESS_DEFAULT;
			pVidInputAuto1->m_VideoDecoder.m_nBrightnessMin = VIDINPUTAUTO1_BRIGHTNESS_MIN;
			pVidInputAuto1->m_VideoDecoder.m_nBrightnessMax = VIDINPUTAUTO1_BRIGHTNESS_MAX;
			pVidInputAuto1->m_VideoDecoder.m_nBrightness = VIDINPUTAUTO1_BRIGHTNESS_DEFAULT;
			pVidInputAuto1->m_VideoDecoder.m_nContrastDef = VIDINPUTAUTO1_CONTRAST_DEFAULT; 
			pVidInputAuto1->m_VideoDecoder.m_nContrastMin = VIDINPUTAUTO1_CONTRAST_MIN;
			pVidInputAuto1->m_VideoDecoder.m_nContrastMax = VIDINPUTAUTO1_CONTRAST_MAX;
			pVidInputAuto1->m_VideoDecoder.m_nContrast = VIDINPUTAUTO1_CONTRAST_DEFAULT;
			pVidInputAuto1->m_VideoDecoder.m_nSaturationDef = VIDINPUTAUTO1_SATURATION_DEFAULT;
			pVidInputAuto1->m_VideoDecoder.m_nSaturationMin = VIDINPUTAUTO1_SATURATION_MIN;
			pVidInputAuto1->m_VideoDecoder.m_nSaturationMax = VIDINPUTAUTO1_SATURATION_MAX;
			pVidInputAuto1->m_VideoDecoder.m_nSaturation = VIDINPUTAUTO1_SATURATION_DEFAULT;
			pVidInputAuto1->m_VideoDecoder.m_nHueDef = VIDINPUTAUTO1_HUE_DEFAULT;
			pVidInputAuto1->m_VideoDecoder.m_nHueMin = VIDINPUTAUTO1_HUE_MIN;
			pVidInputAuto1->m_VideoDecoder.m_nHueMax = VIDINPUTAUTO1_HUE_MAX;
			pVidInputAuto1->m_VideoDecoder.m_nHue = VIDINPUTAUTO1_HUE_DEFAULT;
			pVidInputAuto1->m_VideoDecoder.m_nSharpnessDef = VIDINPUTAUTO1_SHARPNESS_DEFAULT;
			pVidInputAuto1->m_VideoDecoder.m_nSharpnessMin = VIDINPUTAUTO1_SHARPNESS_MIN;
			pVidInputAuto1->m_VideoDecoder.m_nSharpnessMax = VIDINPUTAUTO1_SHARPNESS_MAX;
			pVidInputAuto1->m_VideoDecoder.m_nSharpness = VIDINPUTAUTO1_SHARPNESS_DEFAULT;
		}
	}

	return (pVidInputAuto1);
}



// destructor
//
void CVidInputAuto1_Destructor(CVidInputAuto1 *This)
{
    TQP_UINT8     buffer[2];
    TQP_UINT8     ucRegAddr;
    TQP_UINT8     ucValue;
    
    QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CVidInputAuto1_Destructor()\n"
				 ));

    CVideoDecoder_Destructor(&This->m_VideoDecoder);
}

