///////////////////////////////////////////////////////////////////////////////
//     
//
// Cvidinputusbfw.c
//
// Description:
//
//  Video input device handled in the USB firmware
//
// History:
//	Aug 7, 2009	(dt) created
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#include "cvidinputusbfw.h"
#include "qpdbgmgr.h"
#include "util.h"
#include "iqlusbfw.h"


/////////////////////////////////////////////////////////////////////////////
//
//

#define	VIDINPUTUSBFW_BRIGHTNESS_MIN				(-64)
#define	VIDINPUTUSBFW_BRIGHTNESS_MAX				64
#define	VIDINPUTUSBFW_BRIGHTNESS_DEFAULT			0x00

#define	VIDINPUTUSBFW_CONTRAST_MIN					0x00
#define	VIDINPUTUSBFW_CONTRAST_MAX					64
#define	VIDINPUTUSBFW_CONTRAST_DEFAULT				32

#define	VIDINPUTUSBFW_SATURATION_MIN				0x00
#define	VIDINPUTUSBFW_SATURATION_MAX				128
#define	VIDINPUTUSBFW_SATURATION_DEFAULT			64

#define	VIDINPUTUSBFW_HUE_MIN						(-128)	// 0x80
#define	VIDINPUTUSBFW_HUE_MAX						127		// 0x7F
#define	VIDINPUTUSBFW_HUE_DEFAULT					0x00

#define	VIDINPUTUSBFW_SHARPNESS_MIN					0
#define	VIDINPUTUSBFW_SHARPNESS_MAX					6
#define	VIDINPUTUSBFW_SHARPNESS_DEFAULT				3

#define	VIDINPUTUSBFW_EDGE_ENHANCEMENT_MIN			0x00
#define	VIDINPUTUSBFW_EDGE_ENHANCEMENT_MAX			0xFF
#define	VIDINPUTUSBFW_EDGE_ENHANCEMENT_DEFAULT		0x80

#define	VIDINPUTUSBFW_GAMMA_MIN						75
#define	VIDINPUTUSBFW_GAMMA_MAX						125
#define	VIDINPUTUSBFW_GAMMA_DEFAULT					100

#define	VIDINPUTUSBFW_LOW_LIGHT_BOOST_MIN			0
#define	VIDINPUTUSBFW_LOW_LIGHT_BOOST_MAX			2
#define	VIDINPUTUSBFW_LOW_LIGHT_BOOST_DEFAULT		1

#define	VIDINPUTUSBFW_EXPOSURE_MIN					0x00
#define	VIDINPUTUSBFW_EXPOSURE_MAX					0xFF
#define	VIDINPUTUSBFW_EXPOSURE_DEFAULT				0x80

#define	VIDINPUTUSBFW_WHITE_BALANCE_MIN				2800
#define	VIDINPUTUSBFW_WHITE_BALANCE_MAX				6500
#define	VIDINPUTUSBFW_WHITE_BALANCE_DEFAULT			4600

#define	VIDINPUTUSBFW_FOCUS_MIN						1
#define	VIDINPUTUSBFW_FOCUS_MAX						20
#define	VIDINPUTUSBFW_FOCUS_DEFAULT					7


extern TQP_DbgMCompID	g_QPPeripheralDBGCompID;




/////////////////////////////////////////////////////////////////////////////
//
//

#pragma pack(push, 1)

typedef struct _USBFW_CAMERA_CNTL_CMD
{
	USBFW_COMMAND_UVC_HEADER	hdr;
	unsigned char				data[8];
} USBFW_CAMERA_CNTL_CMD, *PUSBFW_CAMERA_CNTL_CMD;



#pragma pack(pop)



int short2int(short val)
{
	return (val > 0 ? (int)val : -((int)-val));
}



short int2short(int val)
{
	return (val > 0 ? (short)val : -((short)-val));
}



static EQP_Errors CVidInputUsbFW_SetCommand(CVidInputUsbFW *This,
											unsigned char unit,
											unsigned char selector,
											unsigned short len,
											unsigned char *pData
											)
{
	EQP_Errors				qpStatus;
	USBFW_CAMERA_CNTL_CMD	cmd;
	unsigned short			i;

	if (QP_NULL == This->m_pUsbCntl)
	{
		return (QPERR_INVALID);
	}

	if (!pData)
	{
		return (QPERR_PARMS);
	}

	// setup UVC header
	//
	cmd.hdr.type = QLUSBFW_UVC_CONTROL;
	cmd.hdr.req = SET_CUR;
	cmd.hdr.wunit.unit = unit;
	cmd.hdr.wunit.zero = 0;
	cmd.hdr.wselector.selector = selector;
	cmd.hdr.wselector.zero = 0;
	cmd.hdr.length = len;

	// data
	//
	for (i = 0; i < len; i++)
	{
		cmd.data[i] = pData[i];
	}

	// send the command
	//
	qpStatus = CUsbCntl_GenericCmd(This->m_pUsbCntl,
								   (PTQP_UINT8)&cmd,
								   sizeof(USBFW_COMMAND_UVC_HEADER) + len,
								   QP_NULL,
								   0,
								   QP_TRUE
								   );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_SetCommand() GenericCmd Failed Status(%d)\n",
					 qpStatus
					 ));
	}
	return (qpStatus);
}



static EQP_Errors CVidInputUsbFW_GetCommand(CVidInputUsbFW *This,
											unsigned char req,
											unsigned char unit,
											unsigned char selector,
											unsigned short len,
											unsigned char *pData
											)
{
	EQP_Errors					qpStatus;
	USBFW_COMMAND_UVC_HEADER	hdr;
	//unsigned short				i;

	if (QP_NULL == This->m_pUsbCntl)
	{
		return (QPERR_INVALID);
	}

	if (!pData)
	{
		return (QPERR_PARMS);
	}

	// setup UVC header
	//
	hdr.type = QLUSBFW_UVC_CONTROL;
	hdr.req = req;
	hdr.wunit.unit = unit;
	hdr.wunit.zero = 0;
	hdr.wselector.selector = selector;
	hdr.wselector.zero = 0;
	hdr.length = len;

	// send the command
	//
	qpStatus = CUsbCntl_GenericCmd(This->m_pUsbCntl,
								   (PTQP_UINT8)&hdr,
								   sizeof(USBFW_COMMAND_UVC_HEADER),
								   pData,
								   len,
								   QP_TRUE
								   );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetCommand() GenericCmd Failed Status(%d)\n",
					 qpStatus
					 ));
	}
	return (qpStatus);
}






/////////////////////////////////////////////////////////////////////////////
//
//


// IGenericDevice
//
static EQP_Errors CVidInputUsbFW_InitDevice(IVideoDecoder *This_p,
											QP_DEVICE_CALLBACK pFuncCallback,
											TQP_PVOID context,
											TQP_PVOID pInitData
											)
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	//CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CVidInputUsbFW_InitDevice()\n"
				 ));

	if (CObject_IsInitialized(&pVideoDecoder->m_Object))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
					 "CVidInputUsbFW_InitDevice() Already Initialized. Skipping...\n"
					 ));
		return (QPERR_SUCCESS);
	}

	// save callback address and context
	//
	pVideoDecoder->m_pDeviceCallback = pFuncCallback;
	pVideoDecoder->m_callbackContext = context;

	// init the video input device
	//
	// ...
	pVideoDecoder->m_bVideoStarted = QP_FALSE;

	// CObject Init
	//
	CObject_Init(&pVideoDecoder->m_Object);

	return (QPERR_SUCCESS);
}



// IVideoDecoder
//
static EQP_Errors CVidInputUsbFW_SetVideoRes(IVideoDecoder *This_p,
											 TQP_UINT32 dwWid,
				  							 TQP_UINT32 dwHgt,
				  							 TQP_UINT32 dwScanFmt,
				  							 TQP_UINT32 dwFps
				  							 )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	//CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	EQP_Errors		qpStatus = QPERR_SUCCESS;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_SetVideoRes() : Video Res Set to %dx %d, status(%d)\n",
				 dwWid,
				 dwHgt,
				 qpStatus
				 ));
	pVideoDecoder->m_dwWidth = dwWid;
	pVideoDecoder->m_dwHeight = dwHgt;

	return (qpStatus);
}


// Get Decoder Brightness range.
static EQP_Errors CVidInputUsbFW_GetBrightnessRange(IVideoDecoder *This_p,
													int *pnDefault,
													int *pnMin,
													int *pnMax
													)
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	short			val;
	EQP_Errors		qpStatus;

	if ((QP_NULL == pnDefault) ||
		(QP_NULL == pnMin) ||
		(QP_NULL == pnMax)
		)
		return (QPERR_PARMS);

	// get minimum
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_MIN,
										 UVC_UNIT_PU,
										 PU_BRIGHTNESS_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetBrightnessRange() CVidInputUsbFW_GetCommand(GET_MIN) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnMin = pVideoDecoder->m_nBrightnessMin = short2int(val);

	// get maximum
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_MAX,
										 UVC_UNIT_PU,
										 PU_BRIGHTNESS_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetBrightnessRange() CVidInputUsbFW_GetCommand(GET_MAX) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnMax = pVideoDecoder->m_nBrightnessMax = short2int(val);

	// get default
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_DEF,
										 UVC_UNIT_PU,
										 PU_BRIGHTNESS_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetBrightnessRange() CVidInputUsbFW_GetCommand(GET_DEF) Failed Status(%d)!\n",
					 qpStatus

					 ));
		return (qpStatus);
	}

	*pnDefault = pVideoDecoder->m_nBrightnessDef = short2int(val);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_GetBrightnessRange() min(%d) max(%d) def(%d)\n",
				 *pnMin, *pnMax, *pnDefault
				 ));

	return (QPERR_SUCCESS);
}



// Get Decoder Brightness
static EQP_Errors CVidInputUsbFW_GetBrightness(IVideoDecoder *This_p,
											   int *pnParamValue
											   )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	short			val;
	EQP_Errors		qpStatus;

	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_CUR,
										 UVC_UNIT_PU,
										 PU_BRIGHTNESS_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetBrightness() CVidInputUsbFW_GetCommand(GET_CUR) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnParamValue = short2int(val);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_GetBrightness() ret(%d) val(%d)\n",
				 *pnParamValue,
				 val
				 ));

	return (QPERR_SUCCESS);
}



// Set Decoder Brightness
static EQP_Errors CVidInputUsbFW_SetBrightness(IVideoDecoder *This_p,
											   int nParamValue
											   )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	short			val = int2short(nParamValue);
	EQP_Errors		qpStatus;

	qpStatus = CVidInputUsbFW_SetCommand(This,
										 UVC_UNIT_PU,
										 PU_BRIGHTNESS_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_SetBrightness() CVidInputUsbFW_SetCommand() Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_SetBrightness() param(%d) val(%d)\n",
				 nParamValue,
				 val
				 ));

	pVideoDecoder->m_nBrightness = nParamValue;
	return (QPERR_SUCCESS);
}



// Get Decoder Contrast range.
static EQP_Errors CVidInputUsbFW_GetContrastRange(IVideoDecoder *This_p,
												  int *pnDefault,
												  int *pnMin,
												  int *pnMax
												  )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	short			val;
	EQP_Errors		qpStatus;

	if ((QP_NULL == pnDefault) ||
		(QP_NULL == pnMin) ||
		(QP_NULL == pnMax)
		)
		return (QPERR_PARMS);

	// get minimum
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_MIN,
										 UVC_UNIT_PU,
										 PU_CONTRAST_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetContrastRange() CVidInputUsbFW_GetCommand(GET_MIN) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnMin = pVideoDecoder->m_nContrastMin = short2int(val);

	// get maximum
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_MAX,
										 UVC_UNIT_PU,
										 PU_CONTRAST_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetContrastRange() CVidInputUsbFW_GetCommand(GET_MAX) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnMax = pVideoDecoder->m_nContrastMax = short2int(val);

	// get default
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_DEF,
										 UVC_UNIT_PU,
										 PU_CONTRAST_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetContrastRange() CVidInputUsbFW_GetCommand(GET_DEF) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnDefault = pVideoDecoder->m_nContrastDef = short2int(val);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_GetContrastRange() min(%d) max(%d) def(%d)\n",
				 *pnMin, *pnMax, *pnDefault
				 ));

	return (QPERR_SUCCESS);
}



// Get Decoder Contrast
static EQP_Errors CVidInputUsbFW_GetContrast(IVideoDecoder *This_p,
											 int *pnParamValue
											 )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	short			val;
	EQP_Errors		qpStatus;

	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_CUR,
										 UVC_UNIT_PU,
										 PU_CONTRAST_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetContrast() CVidInputUsbFW_GetCommand(GET_CUR) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnParamValue = short2int(val);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_GetContrast() ret(%d) val(%d)\n",
				 *pnParamValue,
				 val
				 ));

	return (QPERR_SUCCESS);
}



// Set Decoder Contrast
static EQP_Errors CVidInputUsbFW_SetContrast(IVideoDecoder *This_p,
											 int nParamValue
											 )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	short			val = int2short(nParamValue);
	EQP_Errors		qpStatus;

	qpStatus = CVidInputUsbFW_SetCommand(This,
										 UVC_UNIT_PU,
										 PU_CONTRAST_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_SetContrast() CVidInputUsbFW_SetCommand() Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	pVideoDecoder->m_nContrast = nParamValue;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_SetContrast() param(%d) val(%d)\n",
				 nParamValue,
				 val
				 ));

	return (QPERR_SUCCESS);
}



// Get Decoder Saturation range.
static EQP_Errors CVidInputUsbFW_GetSaturationRange(IVideoDecoder *This_p,
													int *pnDefault,
													int *pnMin,
													int *pnMax
													)
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	short			val;
	EQP_Errors		qpStatus;

	if ((QP_NULL == pnDefault) ||
		(QP_NULL == pnMin) ||
		(QP_NULL == pnMax)
		)
		return (QPERR_PARMS);

	// get minimum
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_MIN,
										 UVC_UNIT_PU,
										 PU_SATURATION_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetSaturationRange() CVidInputUsbFW_GetCommand(GET_MIN) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnMin = pVideoDecoder->m_nSaturationMin = short2int(val);

	// get maximum
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_MAX,
										 UVC_UNIT_PU,
										 PU_SATURATION_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetSaturationRange() CVidInputUsbFW_GetCommand(GET_MAX) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnMax = pVideoDecoder->m_nSaturationMax = short2int(val);

	// get default
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_DEF,
										 UVC_UNIT_PU,
										 PU_SATURATION_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetSaturationRange() CVidInputUsbFW_GetCommand(GET_DEF) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnDefault = pVideoDecoder->m_nSaturationDef = short2int(val);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_GetSaturationRange() min(%d) max(%d) def(%d)\n",
				 *pnMin, *pnMax, *pnDefault
				 ));

	return (QPERR_SUCCESS);
}



// Get Decoder Saturation
static EQP_Errors CVidInputUsbFW_GetSaturation(IVideoDecoder *This_p,
											   int *pnParamValue
											   )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	short			val;
	EQP_Errors		qpStatus;

	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_CUR,
										 UVC_UNIT_PU,
										 PU_SATURATION_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetSaturation() CVidInputUsbFW_GetCommand(GET_CUR) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnParamValue = short2int(val);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_GetSaturation() ret(%d) val(%d)\n",
				 *pnParamValue,
				 val
				 ));

	return (QPERR_SUCCESS);
}



// Set Decoder Saturation
static EQP_Errors CVidInputUsbFW_SetSaturation(IVideoDecoder *This_p,
											   int nParamValue
											   )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	short			val = int2short(nParamValue);
	EQP_Errors		qpStatus;

	qpStatus = CVidInputUsbFW_SetCommand(This,
										 UVC_UNIT_PU,
										 PU_SATURATION_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_SetSaturation() CVidInputUsbFW_SetCommand() Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	pVideoDecoder->m_nSaturation = nParamValue;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_SetSaturation() param(%d) val(%d)\n",
				 nParamValue,
				 val
				 ));

	return (QPERR_SUCCESS);
}



// Get Decoder Hue range.
static EQP_Errors CVidInputUsbFW_GetHueRange(IVideoDecoder *This_p,
											 int *pnDefault,
											 int *pnMin,
											 int *pnMax
											 )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	short			val;
	EQP_Errors		qpStatus;

	if ((QP_NULL == pnDefault) ||
		(QP_NULL == pnMin) ||
		(QP_NULL == pnMax)
		)
		return (QPERR_PARMS);

	// get minimum
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_MIN,
										 UVC_UNIT_PU,
										 PU_HUE_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetHueRange() CVidInputUsbFW_GetCommand(GET_MIN) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnMin = pVideoDecoder->m_nHueMin = short2int(val);

	// get maximum
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_MAX,
										 UVC_UNIT_PU,
										 PU_HUE_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetHueRange() CVidInputUsbFW_GetCommand(GET_MAX) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnMax = pVideoDecoder->m_nHueMax = short2int(val);

	// get default
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_DEF,
										 UVC_UNIT_PU,
										 PU_HUE_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetHueRange() CVidInputUsbFW_GetCommand(GET_DEF) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnDefault = pVideoDecoder->m_nHueDef = short2int(val);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_GetHueRange() min(%d) max(%d) def(%d)\n",
				 *pnMin, *pnMax, *pnDefault
				 ));

	return (QPERR_SUCCESS);
}



// Get Decoder Hue
static EQP_Errors CVidInputUsbFW_GetHue(IVideoDecoder *This_p,
										int *pnParamValue
										)
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	short			val;
	EQP_Errors		qpStatus;

	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_CUR,
										 UVC_UNIT_PU,
										 PU_HUE_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetHue() CVidInputUsbFW_GetCommand(GET_CUR) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnParamValue = short2int(val);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_GetHue() ret(%d) val(%d)\n",
				 *pnParamValue,
				 val
				 ));

	return (QPERR_SUCCESS);
}



// Set Decoder Hue
static EQP_Errors CVidInputUsbFW_SetHue(IVideoDecoder *This_p,
										int nParamValue
										)
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	short			val = int2short(nParamValue);
	EQP_Errors		qpStatus;

	qpStatus = CVidInputUsbFW_SetCommand(This,
										 UVC_UNIT_PU,
										 PU_HUE_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_SetHue() CVidInputUsbFW_SetCommand() Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	pVideoDecoder->m_nHue = nParamValue;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_SetHue() param(%d) val(%d)\n",
				 nParamValue,
				 val
				 ));

	return (QPERR_SUCCESS);
}



// Get Decoder Sharpness range.
static EQP_Errors CVidInputUsbFW_GetSharpnessRange(IVideoDecoder *This_p,
												   int *pnDefault,
												   int *pnMin,
												   int *pnMax
												   )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	short			val;
	EQP_Errors		qpStatus;

	if ((QP_NULL == pnDefault) ||
		(QP_NULL == pnMin) ||
		(QP_NULL == pnMax)
		)
		return (QPERR_PARMS);

	// get minimum
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_MIN,
										 UVC_UNIT_PU,
										 PU_SHARPNESS_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetSharpnessRange() CVidInputUsbFW_GetCommand(GET_MIN) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnMin = pVideoDecoder->m_nSharpnessMin = short2int(val);

	// get maximum
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_MAX,
										 UVC_UNIT_PU,
										 PU_SHARPNESS_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetSharpnessRange() CVidInputUsbFW_GetCommand(GET_MAX) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnMax = pVideoDecoder->m_nSharpnessMax = short2int(val);

	// get default
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_DEF,
										 UVC_UNIT_PU,
										 PU_SHARPNESS_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetSharpnessRange() CVidInputUsbFW_GetCommand(GET_DEF) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnDefault = pVideoDecoder->m_nSharpnessDef = short2int(val);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_GetSharpnessRange() min(%d) max(%d) def(%d)\n",
				 *pnMin, *pnMax, *pnDefault
				 ));

	return (QPERR_SUCCESS);
}



// Get Decoder Sharpness
static EQP_Errors CVidInputUsbFW_GetSharpness(IVideoDecoder *This_p,
											  int *pnParamValue
											  )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	short			val;
	EQP_Errors		qpStatus;

	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_CUR,
										 UVC_UNIT_PU,
										 PU_SHARPNESS_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetSharpness() CVidInputUsbFW_GetCommand(GET_CUR) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnParamValue = short2int(val);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_GetSharpness() ret(%d) val(%d)\n",
				 *pnParamValue,
				 val
				 ));

	return (QPERR_SUCCESS);
}



// Set Decoder Sharpness
static EQP_Errors CVidInputUsbFW_SetSharpness(IVideoDecoder *This_p,
											  int nParamValue
											  )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	short			val = int2short(nParamValue);
	EQP_Errors		qpStatus;

	qpStatus = CVidInputUsbFW_SetCommand(This,
										 UVC_UNIT_PU,
										 PU_SHARPNESS_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_SetSharpness() CVidInputUsbFW_SetCommand() Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	pVideoDecoder->m_nSharpness = nParamValue;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_SetSharpness() param(%d) val(%d)\n",
				 nParamValue,
				 val
				 ));

	return (QPERR_SUCCESS);
}



// edge enhancement
static EQP_Errors CVidInputUsbFW_GetEdgeEnhancementRange(IVideoDecoder *This_p,
														 int *pnDefault,
			  											 int *pnMin,
			  											 int *pnMax
			  											 )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	//CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);

	if ((QP_NULL == pnDefault) ||
		(QP_NULL == pnMin) ||
		(QP_NULL == pnMax)
		)
		return (QPERR_PARMS);

	*pnDefault = pVideoDecoder->m_nEdgeEnhancementDef;
	*pnMin = pVideoDecoder->m_nEdgeEnhancementMin;
	*pnMax = pVideoDecoder->m_nEdgeEnhancementMax;

	return (QPERR_SUCCESS);
}



static EQP_Errors CVidInputUsbFW_GetEdgeEnhancement(IVideoDecoder *This_p,
													int *pnParamValue
													)
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	//CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);

	*pnParamValue = pVideoDecoder->m_nEdgeEnhancement;
	return (QPERR_SUCCESS);
}



static EQP_Errors CVidInputUsbFW_SetEdgeEnhancement(IVideoDecoder *This_p,
													int nParamValue
													)
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	//CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);

	pVideoDecoder->m_nEdgeEnhancement = nParamValue;
	return (QPERR_SUCCESS);
}



// gamma
static EQP_Errors CVidInputUsbFW_GetGammaRange(IVideoDecoder *This_p,
											   int *pnDefault,
			  								   int *pnMin,
			  								   int *pnMax
			  								   )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	short			val;
	EQP_Errors		qpStatus;

	if ((QP_NULL == pnDefault) ||
		(QP_NULL == pnMin) ||
		(QP_NULL == pnMax)
		)
		return (QPERR_PARMS);

	// get minimum
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_MIN,
										 UVC_UNIT_PU,
										 PU_GAMMA_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetGammaRange() CVidInputUsbFW_GetCommand(GET_MIN) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnMin = pVideoDecoder->m_nGammaMin = short2int(val);

	// get maximum
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_MAX,
										 UVC_UNIT_PU,
										 PU_GAMMA_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetGammaRange() CVidInputUsbFW_GetCommand(GET_MAX) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnMax = pVideoDecoder->m_nGammaMax = short2int(val);

	// get default
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_DEF,
										 UVC_UNIT_PU,
										 PU_GAMMA_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetGammaRange() CVidInputUsbFW_GetCommand(GET_DEF) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnDefault = pVideoDecoder->m_nGammaDef = short2int(val);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_GetGammaRange() min(%d) max(%d) def(%d)\n",
				 *pnMin, *pnMax, *pnDefault
				 ));

	return (QPERR_SUCCESS);
}



static EQP_Errors CVidInputUsbFW_GetGamma(IVideoDecoder *This_p,
										  int *pnParamValue
										  )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	short			val;
	EQP_Errors		qpStatus;

	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_CUR,
										 UVC_UNIT_PU,
										 PU_GAMMA_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetGamma() CVidInputUsbFW_GetCommand(GET_CUR) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnParamValue = short2int(val);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_GetGamma() ret(%d) val(%d)\n",
				 *pnParamValue,
				 val
				 ));

	return (QPERR_SUCCESS);
}



static EQP_Errors CVidInputUsbFW_SetGamma(IVideoDecoder *This_p,
										  int nParamValue
										  )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	short			val = int2short(nParamValue);
	EQP_Errors		qpStatus;

	qpStatus = CVidInputUsbFW_SetCommand(This,
										 UVC_UNIT_PU,
										 PU_GAMMA_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_SetGamma() CVidInputUsbFW_SetCommand() Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	pVideoDecoder->m_nGamma = nParamValue;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_SetGamma() param(%d) val(%d)\n",
				 nParamValue,
				 val
				 ));

	return (QPERR_SUCCESS);
}



// low light boost
static EQP_Errors CVidInputUsbFW_GetLowLightBoostRange(IVideoDecoder *This_p,
													   int *pnDefault,
			  										   int *pnMin,
			  										   int *pnMax
			  										   )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	short			val;
	EQP_Errors		qpStatus;

	if ((QP_NULL == pnDefault) ||
		(QP_NULL == pnMin) ||
		(QP_NULL == pnMax)
		)
		return (QPERR_PARMS);

	// get minimum
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_MIN,
										 UVC_UNIT_PU,
										 PU_BACKLIGHT_COMPENSATION_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetLowLightBoostRange() CVidInputUsbFW_GetCommand(GET_MIN) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnMin = pVideoDecoder->m_nLowLightBoostMin = short2int(val);

	// get maximum
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_MAX,
										 UVC_UNIT_PU,
										 PU_BACKLIGHT_COMPENSATION_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetLowLightBoostRange() CVidInputUsbFW_GetCommand(GET_MAX) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnMax = pVideoDecoder->m_nLowLightBoostMax = short2int(val);

	// get default
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_DEF,
										 UVC_UNIT_PU,
										 PU_BACKLIGHT_COMPENSATION_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetLowLightBoostRange() CVidInputUsbFW_GetCommand(GET_DEF) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnDefault = pVideoDecoder->m_nLowLightBoostDef = short2int(val);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_GetLowLightBoostRange() min(%d) max(%d) def(%d)\n",
				 *pnMin, *pnMax, *pnDefault
				 ));

	return (QPERR_SUCCESS);
}



static EQP_Errors CVidInputUsbFW_GetLowLightBoost(IVideoDecoder *This_p,
												  int *pnParamValue
												  )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	short			val;
	EQP_Errors		qpStatus;

	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_CUR,
										 UVC_UNIT_PU,
										 PU_BACKLIGHT_COMPENSATION_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetLowLightBoost() CVidInputUsbFW_GetCommand(GET_CUR) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnParamValue = short2int(val);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_GetLowLightBoost() ret(%d) val(%d)\n",
				 *pnParamValue,
				 val
				 ));

	return (QPERR_SUCCESS);
}



static EQP_Errors CVidInputUsbFW_SetLowLightBoost(IVideoDecoder *This_p,
												  int nParamValue
												  )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	short			val = int2short(nParamValue);
	EQP_Errors		qpStatus;

	qpStatus = CVidInputUsbFW_SetCommand(This,
										 UVC_UNIT_PU,
										 PU_BACKLIGHT_COMPENSATION_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_SetLowLightBoost() CVidInputUsbFW_SetCommand() Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	pVideoDecoder->m_nLowLightBoost = nParamValue;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_SetLowLightBoost() param(%d) val(%d)\n",
				 nParamValue,
				 val
				 ));

	return (QPERR_SUCCESS);
}



// exposure
static EQP_Errors CVidInputUsbFW_GetExposureRange(IVideoDecoder *This_p,
												  int *pnDefault,
			  									  int *pnMin,
			  									  int *pnMax
			  									  )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	int				val;
	EQP_Errors		qpStatus;

	if ((QP_NULL == pnDefault) ||
		(QP_NULL == pnMin) ||
		(QP_NULL == pnMax)
		)
		return (QPERR_PARMS);

	// get minimum
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_MIN,
										 UVC_UNIT_CT,
										 CT_EXPOSURE_TIME_ABSOLUTE_CONTROL,
										 4,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetExposureRange() CVidInputUsbFW_GetCommand(GET_MIN) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnMin = pVideoDecoder->m_nExposureMin = val;

	// get maximum
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_MAX,
										 UVC_UNIT_CT,
										 CT_EXPOSURE_TIME_ABSOLUTE_CONTROL,
										 4,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetExposureRange() CVidInputUsbFW_GetCommand(GET_MAX) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnMax = pVideoDecoder->m_nExposureMax = val;

	// get default
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_DEF,
										 UVC_UNIT_CT,
										 CT_EXPOSURE_TIME_ABSOLUTE_CONTROL,
										 4,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetExposureRange() CVidInputUsbFW_GetCommand(GET_DEF) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnDefault = pVideoDecoder->m_nExposureDef = val;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_GetExposureRange() min(%d) max(%d) def(%d)\n",
				 *pnMin, *pnMax, *pnDefault
				 ));

	return (QPERR_SUCCESS);
}



static EQP_Errors CVidInputUsbFW_GetExposure(IVideoDecoder *This_p,
											 int *pnParamValue
											 )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	int				val;
	EQP_Errors		qpStatus;

	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_CUR,
										 UVC_UNIT_CT,
										 CT_EXPOSURE_TIME_ABSOLUTE_CONTROL,
										 4,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetExposure() CVidInputUsbFW_GetCommand(GET_CUR) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnParamValue = val;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_GetExposure() ret(%d) val(%d)\n",
				 *pnParamValue,
				 val
				 ));

	return (QPERR_SUCCESS);
}



static EQP_Errors CVidInputUsbFW_SetExposure(IVideoDecoder *This_p,
											 int nParamValue
											 )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	int				val = nParamValue;
	EQP_Errors		qpStatus;

	qpStatus = CVidInputUsbFW_SetCommand(This,
										 UVC_UNIT_CT,
										 CT_EXPOSURE_TIME_ABSOLUTE_CONTROL,
										 4,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_SetExposure() CVidInputUsbFW_SetCommand() Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	pVideoDecoder->m_nExposure = nParamValue;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_SetExposure() param(%d) val(%d)\n",
				 nParamValue,
				 val
				 ));

	return (QPERR_SUCCESS);
}



static EQP_Errors CVidInputUsbFW_SetAutoExposureEnabled(IVideoDecoder *This_p,
														TQP_UINT32 enable
														)
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	unsigned char	val;
	EQP_Errors		qpStatus;

	// UVC spec 4.2.2.1.2
	//
	val = enable ? 0x08 : 0x1;

	qpStatus = CVidInputUsbFW_SetCommand(This,
										 UVC_UNIT_CT,
										 CT_AE_MODE_CONTROL,
										 1,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_SetAutoExposureEnabled() CVidInputUsbFW_SetCommand() Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	pVideoDecoder->m_AutoExposureEnable = enable;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_SetAutoExposureEnabled() param(%d) val(%d)\n",
				 enable,
				 val
				 ));

	return (QPERR_SUCCESS);
}



static EQP_Errors CVidInputUsbFW_GetAutoExposureEnabled(IVideoDecoder *This_p,
														TQP_UINT32 *enable
														)
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	unsigned char	val;
	EQP_Errors		qpStatus;

	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_CUR,
										 UVC_UNIT_CT,
										 CT_AE_MODE_CONTROL,
										 1,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetAutoExposureEnabled() CVidInputUsbFW_GetCommand(GET_CUR) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*enable = (0 != (val & 0x08));

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_GetAutoExposureEnabled() param(%d) val(%d)\n",
				 *enable,
				 val
				 ));

	return (QPERR_SUCCESS);
}



// auto exposure priority
static EQP_Errors CVidInputUsbFW_GetAutoExposurePriorityRange(IVideoDecoder *This_p,
															  int *pnDefault,
			  												  int *pnMin,
			  												  int *pnMax
			  												  )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	unsigned char	val;
	EQP_Errors		qpStatus;

	if ((QP_NULL == pnDefault) ||
		(QP_NULL == pnMin) ||
		(QP_NULL == pnMax)
		)
		return (QPERR_PARMS);

	*pnMin = pVideoDecoder->m_nAutoExposurePriorityMin;
	*pnMax = pVideoDecoder->m_nAutoExposurePriorityMax;

	// get default
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_DEF,
										 UVC_UNIT_CT,
										 CT_AE_PRIORITY_CONTROL,
										 1,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetAutoExposurePriorityRange() CVidInputUsbFW_GetCommand(GET_DEF) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnDefault = pVideoDecoder->m_nAutoExposurePriorityDef = (int)val;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_GetAutoExposurePriorityRange() min(%d) max(%d) def(%d)\n",
				 *pnMin, *pnMax, *pnDefault
				 ));

	return (QPERR_SUCCESS);
}



static EQP_Errors CVidInputUsbFW_GetAutoExposurePriority(IVideoDecoder *This_p,
														 int *pnParamValue
														 )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	unsigned char	val;
	EQP_Errors		qpStatus;

	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_CUR,
										 UVC_UNIT_CT,
										 CT_AE_PRIORITY_CONTROL,
										 1,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetAutoExposurePriority() CVidInputUsbFW_GetCommand(GET_CUR) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnParamValue = (int)val;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_GetAutoExposurePriority() ret(%d) val(%d)\n",
				 *pnParamValue,
				 val
				 ));

	return (QPERR_SUCCESS);
}



static EQP_Errors CVidInputUsbFW_SetAutoExposurePriority(IVideoDecoder *This_p,
														 int nParamValue
														 )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	unsigned char	val = (unsigned char)nParamValue;
	EQP_Errors		qpStatus;

	qpStatus = CVidInputUsbFW_SetCommand(This,
										 UVC_UNIT_CT,
										 CT_AE_PRIORITY_CONTROL,
										 1,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_SetAutoExposurePriority() CVidInputUsbFW_SetCommand() Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	pVideoDecoder->m_nAutoExposurePriority = nParamValue;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_SetAutoExposurePriority() param(%d) val(%d)\n",
				 nParamValue,
				 val
				 ));

	return (QPERR_SUCCESS);
}



// white balance
static EQP_Errors CVidInputUsbFW_GetWhiteBalanceRange(IVideoDecoder *This_p,
													  int *pnDefault,
			  										  int *pnMin,
			  										  int *pnMax
			  										  )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	short			val;
	EQP_Errors		qpStatus;

	if ((QP_NULL == pnDefault) ||
		(QP_NULL == pnMin) ||
		(QP_NULL == pnMax)
		)
		return (QPERR_PARMS);

	// get minimum
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_MIN,
										 UVC_UNIT_PU,
										 PU_WHITE_BALANCE_TEMPERATURE_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetWhiteBalanceRange() CVidInputUsbFW_GetCommand(GET_MIN) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnMin = pVideoDecoder->m_nWhiteBalanceMin = short2int(val);

	// get maximum
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_MAX,
										 UVC_UNIT_PU,
										 PU_WHITE_BALANCE_TEMPERATURE_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetWhiteBalanceRange() CVidInputUsbFW_GetCommand(GET_MAX) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnMax = pVideoDecoder->m_nWhiteBalanceMax = short2int(val);

	// get default
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_DEF,
										 UVC_UNIT_PU,
										 PU_WHITE_BALANCE_TEMPERATURE_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetWhiteBalanceRange() CVidInputUsbFW_GetCommand(GET_DEF) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnDefault = pVideoDecoder->m_nWhiteBalanceDef = short2int(val);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_GetWhiteBalanceRange() min(%d) max(%d) def(%d)\n",
				 *pnMin, *pnMax, *pnDefault
				 ));

	return (QPERR_SUCCESS);
}



static EQP_Errors CVidInputUsbFW_GetWhiteBalance(IVideoDecoder *This_p,
												 int *pnParamValue
												 )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	short			val;
	EQP_Errors		qpStatus;

	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_CUR,
										 UVC_UNIT_PU,
										 PU_WHITE_BALANCE_TEMPERATURE_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetWhiteBalance() CVidInputUsbFW_GetCommand(GET_CUR) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnParamValue = short2int(val);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_GetWhiteBalance() ret(%d) val(%d)\n",
				 *pnParamValue,
				 val
				 ));

	return (QPERR_SUCCESS);
}



static EQP_Errors CVidInputUsbFW_SetWhiteBalance(IVideoDecoder *This_p,
												 int nParamValue
												 )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	short			val = int2short(nParamValue);
	EQP_Errors		qpStatus;

	qpStatus = CVidInputUsbFW_SetCommand(This,
										 UVC_UNIT_PU,
										 PU_WHITE_BALANCE_TEMPERATURE_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_SetWhiteBalance() CVidInputUsbFW_SetCommand() Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	pVideoDecoder->m_nWhiteBalance = nParamValue;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_SetWhiteBalance() param(%d) val(%d)\n",
				 nParamValue,
				 val
				 ));

	return (QPERR_SUCCESS);
}



static EQP_Errors CVidInputUsbFW_SetAutoWhiteBalanceEnabled(IVideoDecoder *This_p,
															TQP_UINT32 enable
															)
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	unsigned char	val;
	EQP_Errors		qpStatus;

	// UVC spec 4.2.2.3.12
	//
	val = enable ? 1 : 0;

	qpStatus = CVidInputUsbFW_SetCommand(This,
										 UVC_UNIT_PU,
										 PU_WHITE_BALANCE_TEMPERATURE_AUTO_CONTROL,
										 1,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_SetAutoWhiteBalanceEnabled() CVidInputUsbFW_SetCommand() Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	pVideoDecoder->m_AutoWhiteBalanceEnable = enable;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_SetAutoWhiteBalanceEnabled() param(%d) val(%d)\n",
				 enable,
				 val
				 ));

	return (QPERR_SUCCESS);
}



static EQP_Errors CVidInputUsbFW_GetAutoWhiteBalanceEnabled(IVideoDecoder *This_p,
															TQP_UINT32 *enable
															)
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	unsigned char	val;
	EQP_Errors		qpStatus;

	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_CUR,
										 UVC_UNIT_PU,
										 PU_WHITE_BALANCE_TEMPERATURE_AUTO_CONTROL,
										 1,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetAutoWhiteBalanceEnabled() CVidInputUsbFW_GetCommand(GET_CUR) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*enable = (1 == val);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_GetAutoWhiteBalanceEnabled() param(%d) val(%d)\n",
				 *enable,
				 val
				 ));

	return (QPERR_SUCCESS);
}



// focus
static EQP_Errors CVidInputUsbFW_GetFocusRange(IVideoDecoder *This_p,
											   int *pnDefault,
			  								   int *pnMin,
			  								   int *pnMax
			  								   )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	short			val;
	EQP_Errors		qpStatus;

	if ((QP_NULL == pnDefault) ||
		(QP_NULL == pnMin) ||
		(QP_NULL == pnMax)
		)
		return (QPERR_PARMS);

	// get minimum
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_MIN,
										 UVC_UNIT_CT,
										 CT_FOCUS_ABSOLUTE_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetFocusRange() CVidInputUsbFW_GetCommand(GET_MIN) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnMin = pVideoDecoder->m_nFocusMin = short2int(val);

	// get maximum
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_MAX,
										 UVC_UNIT_CT,
										 CT_FOCUS_ABSOLUTE_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetFocusRange() CVidInputUsbFW_GetCommand(GET_MAX) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnMax = pVideoDecoder->m_nFocusMax = short2int(val);

	// get default
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_DEF,
										 UVC_UNIT_CT,
										 CT_FOCUS_ABSOLUTE_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetFocusRange() CVidInputUsbFW_GetCommand(GET_DEF) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnDefault = pVideoDecoder->m_nFocusDef = short2int(val);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_GetFocusRange() min(%d) max(%d) def(%d)\n",
				 *pnMin, *pnMax, *pnDefault
				 ));

	return (QPERR_SUCCESS);
}



static EQP_Errors CVidInputUsbFW_GetFocus(IVideoDecoder *This_p,
										  int *pnParamValue
										  )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	short			val;
	EQP_Errors		qpStatus;

	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_CUR,
										 UVC_UNIT_CT,
										 CT_FOCUS_ABSOLUTE_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetFocus() CVidInputUsbFW_GetCommand(GET_CUR) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnParamValue = short2int(val);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_GetFocus() ret(%d) val(%d)\n",
				 *pnParamValue,
				 val
				 ));

	return (QPERR_SUCCESS);
}



static EQP_Errors CVidInputUsbFW_SetFocus(IVideoDecoder *This_p,
										  int nParamValue
										  )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	short			val = int2short(nParamValue);
	EQP_Errors		qpStatus;

	qpStatus = CVidInputUsbFW_SetCommand(This,
										 UVC_UNIT_CT,
										 CT_FOCUS_ABSOLUTE_CONTROL,
										 2,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_SetFocus() CVidInputUsbFW_SetCommand() Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	pVideoDecoder->m_nFocus = nParamValue;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_SetFocus() param(%d) val(%d)\n",
				 nParamValue,
				 val
				 ));

	return (QPERR_SUCCESS);
}



static EQP_Errors CVidInputUsbFW_SetAutoFocusEnabled(IVideoDecoder *This_p,
													 TQP_UINT32 enable
													 )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	unsigned char	val;
	EQP_Errors		qpStatus;

	// UVC spec 4.2.2.1.8
	//
	val = enable ? 1 : 0;

	qpStatus = CVidInputUsbFW_SetCommand(This,
										 UVC_UNIT_CT,
										 CT_FOCUS_AUTO_CONTROL,
										 1,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_SetAutoFocusEnabled() CVidInputUsbFW_SetCommand() Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	pVideoDecoder->m_AutoFocusEnable = enable;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_SetAutoFocusEnabled() param(%d) val(%d)\n",
				 enable,
				 val
				 ));

	return (QPERR_SUCCESS);
}



static EQP_Errors CVidInputUsbFW_GetAutoFocusEnabled(IVideoDecoder *This_p,
													 TQP_UINT32 *enable
													 )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	unsigned char	val;
	EQP_Errors		qpStatus;

	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_CUR,
										 UVC_UNIT_CT,
										 CT_FOCUS_AUTO_CONTROL,
										 1,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetAutoFocusEnabled() CVidInputUsbFW_GetCommand(GET_CUR) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*enable = (1 == val);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_GetAutoFocusEnabled() param(%d) val(%d)\n",
				 *enable,
				 val
				 ));

	return (QPERR_SUCCESS);
}



// power line frequency
static EQP_Errors CVidInputUsbFW_GetPowerLineFreqRange(IVideoDecoder *This_p,
													   int *pnDefault,
			  										   int *pnMin,
			  										   int *pnMax
			  										   )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	unsigned char	val;
	EQP_Errors		qpStatus;

	if ((QP_NULL == pnDefault) ||
		(QP_NULL == pnMin) ||
		(QP_NULL == pnMax)
		)
		return (QPERR_PARMS);

	*pnMin = pVideoDecoder->m_nPowerLineFreqMin;
	*pnMax = pVideoDecoder->m_nPowerLineFreqMax;

	// get default
	//
	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_DEF,
										 UVC_UNIT_PU,
										 PU_POWER_LINE_FREQUENCY_CONTROL,
										 1,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetPowerLineFreqRange() CVidInputUsbFW_GetCommand(GET_DEF) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnDefault = pVideoDecoder->m_nPowerLineFreqDef = (int)val;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_GetPowerLineFreqRange() min(%d) max(%d) def(%d)\n",
				 *pnMin, *pnMax, *pnDefault
				 ));

	return (QPERR_SUCCESS);
}



static EQP_Errors CVidInputUsbFW_GetPowerLineFreq(IVideoDecoder *This_p,
												  int *pnParamValue
												  )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	unsigned char	val;
	EQP_Errors		qpStatus;

	qpStatus = CVidInputUsbFW_GetCommand(This,
										 GET_CUR,
										 UVC_UNIT_PU,
										 PU_POWER_LINE_FREQUENCY_CONTROL,
										 1,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_GetPowerLineFreq() CVidInputUsbFW_GetCommand(GET_CUR) Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	*pnParamValue = (int)val;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_GetPowerLineFreq() ret(%d) val(%d)\n",
				 *pnParamValue,
				 val
				 ));

	return (QPERR_SUCCESS);
}



static EQP_Errors CVidInputUsbFW_SetPowerLineFreq(IVideoDecoder *This_p,
												  int nParamValue
												  )
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	unsigned char	val = (unsigned char)nParamValue;
	EQP_Errors		qpStatus;

	qpStatus = CVidInputUsbFW_SetCommand(This,
										 UVC_UNIT_PU,
										 PU_POWER_LINE_FREQUENCY_CONTROL,
										 1,
										 (unsigned char *)&val
										 );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CVidInputUsbFW_SetPowerLineFreq() CVidInputUsbFW_SetCommand() Failed Status(%d)!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	pVideoDecoder->m_nPowerLineFreq = nParamValue;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
				 "CVidInputUsbFW_SetPowerLineFreq() param(%d) val(%d)\n",
				 nParamValue,
				 val
				 ));

	return (QPERR_SUCCESS);
}



// start/stop video
//
static EQP_Errors CVidInputUsbFW_StartVideo(IVideoDecoder *This_p,
											TQP_UINT32 fps
											)
{
	CVideoDecoder				*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW				*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	EQP_Errors		            qpStatus = QPERR_SUCCESS;
	USBFW_COMMAND_START_VIDEO	usbcmd;
	unsigned char	ret = 0xaa;

	if (QP_NULL == This->m_pUsbCntl)
	{
		return (QPERR_INVALID);
	}

	ENTER_CRITICAL(&pVideoDecoder->m_Object)

	if (!pVideoDecoder->m_bVideoStarted)
	{
		// setup USB command header
		//
		usbcmd.type = QLUSBFW_UVC_FC3_STARTVIDEO;
		usbcmd.fps = (unsigned char)fps;
		usbcmd.videoType = QLUSBFW_UVC_FC3_START_VIDEO_H264;
		usbcmd.width = (unsigned short)pVideoDecoder->m_dwWidth;
		usbcmd.height = (unsigned short)pVideoDecoder->m_dwHeight;

		// send the command
		//
		qpStatus = CUsbCntl_GenericCmd(This->m_pUsbCntl,
									   (PTQP_UINT8)&usbcmd,
									   sizeof(USBFW_COMMAND),
									   &ret,
									   1,
									   QP_TRUE
									   );
		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CVidInputUsbFW_StartVideo() GenericCmd Failed Status(%d)\n",
						 qpStatus
						 ));
		}
		else
		{
			pVideoDecoder->m_bVideoStarted = QP_TRUE;

			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
						 "CVidInputUsbFW_StartVideo() (%d x %d) fps(%d) ret(%d)\n",
						 pVideoDecoder->m_dwWidth,
						 pVideoDecoder->m_dwHeight,
						 fps,
						 ret
						 ));
		}
	}

	LEAVE_CRITICAL(&pVideoDecoder->m_Object)

	return (qpStatus);
}



static EQP_Errors CVidInputUsbFW_StopVideo(IVideoDecoder *This_p)
{
	CVideoDecoder	*pVideoDecoder = GET_INHERITED_OBJECT(CVideoDecoder, m_iVideoDecoder, This_p);
	CVidInputUsbFW	*This = GET_INHERITED_OBJECT(CVidInputUsbFW, m_VideoDecoder, pVideoDecoder);
	EQP_Errors		qpStatus = QPERR_SUCCESS;
	USBFW_COMMAND	usbcmd;
	unsigned char	ret = 0xaa;

	if (QP_NULL == This->m_pUsbCntl)
	{
		return (QPERR_INVALID);
	}

	ENTER_CRITICAL(&pVideoDecoder->m_Object)

	if (pVideoDecoder->m_bVideoStarted)
	{
		// setup USB command header
		//
		usbcmd.cmd_type = QLUSBFW_UVC_FC3_STOPVIDEO;
		usbcmd.cmd_rw = 1;
		usbcmd.cmd_data_len = 0;
		usbcmd.pad.addr32 = 0;

		// send the command
		//
		qpStatus = CUsbCntl_GenericCmd(This->m_pUsbCntl,
									   (PTQP_UINT8)&usbcmd,
									   sizeof(USBFW_COMMAND),
									   &ret,
									   1,
									   QP_TRUE
									   );
		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CVidInputUsbFW_StopVideo() GenericCmd Failed Status(%d)\n",
						 qpStatus
						 ));
		}
		else
		{
			pVideoDecoder->m_bVideoStarted = QP_FALSE;
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_WARN),
						 "CVidInputUsbFW_StopVideo() ret(%d)\n",
						 ret
						 ));
		}
	}

	LEAVE_CRITICAL(&pVideoDecoder->m_Object)

	return (qpStatus);
}



// constructor
//
CVidInputUsbFW * CVidInputUsbFW_Constructor(CVidInputUsbFW *pVidInputUsbFW,
											CObject *pParent,
											TQP_UINT32 dwAttributes,
											ITuner *pTuner,
											ITVAudio *pTvAudio,
											CUsbCntl *pUsbCntl
											)
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CVidInputUsbFW_Constructor()\n"
				 ));

	if (pVidInputUsbFW)
	{
		CVideoDecoder	*pVideoDecoder;

		pVideoDecoder = CVideoDecoder_Constructor(&pVidInputUsbFW->m_VideoDecoder,
												  pParent,
												  dwAttributes,
												  pTuner,
												  pTvAudio
												  );

		if (!pVideoDecoder)
		{
			pVidInputUsbFW = QP_NULL;
		}
		else
		{
			// fill in the function table
			//

			// IGenericDevice
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.InitDevice = CVidInputUsbFW_InitDevice;

			// IVideoDecoder
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.SetVideoRes = CVidInputUsbFW_SetVideoRes;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.GetBrightnessRange = CVidInputUsbFW_GetBrightnessRange;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.GetBrightness = CVidInputUsbFW_GetBrightness;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.SetBrightness = CVidInputUsbFW_SetBrightness;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.GetContrastRange = CVidInputUsbFW_GetContrastRange;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.GetContrast = CVidInputUsbFW_GetContrast;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.SetContrast = CVidInputUsbFW_SetContrast;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.GetSaturationRange = CVidInputUsbFW_GetSaturationRange;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.GetSaturation = CVidInputUsbFW_GetSaturation;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.SetSaturation = CVidInputUsbFW_SetSaturation;
			////pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.GetHueRange = CVidInputUsbFW_GetHueRange;
			////pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.GetHue = CVidInputUsbFW_GetHue;
			////pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.SetHue = CVidInputUsbFW_SetHue;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.GetSharpnessRange = CVidInputUsbFW_GetSharpnessRange;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.GetSharpness = CVidInputUsbFW_GetSharpness;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.SetSharpness = CVidInputUsbFW_SetSharpness;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.GetEdgeEnhancementRange = CVidInputUsbFW_GetEdgeEnhancementRange;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.GetEdgeEnhancement = CVidInputUsbFW_GetEdgeEnhancement;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.SetEdgeEnhancement = CVidInputUsbFW_SetEdgeEnhancement;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.GetGammaRange = CVidInputUsbFW_GetGammaRange;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.GetGamma = CVidInputUsbFW_GetGamma;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.SetGamma = CVidInputUsbFW_SetGamma;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.GetLowLightBoostRange = CVidInputUsbFW_GetLowLightBoostRange;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.GetLowLightBoost = CVidInputUsbFW_GetLowLightBoost;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.SetLowLightBoost = CVidInputUsbFW_SetLowLightBoost;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.GetExposureRange = CVidInputUsbFW_GetExposureRange;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.GetExposure = CVidInputUsbFW_GetExposure;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.SetExposure = CVidInputUsbFW_SetExposure;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.SetAutoExposureEnabled = CVidInputUsbFW_SetAutoExposureEnabled;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.GetAutoExposureEnabled = CVidInputUsbFW_GetAutoExposureEnabled;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.GetAutoExposurePriorityRange = CVidInputUsbFW_GetAutoExposurePriorityRange;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.GetAutoExposurePriority = CVidInputUsbFW_GetAutoExposurePriority;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.SetAutoExposurePriority = CVidInputUsbFW_SetAutoExposurePriority;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.GetWhiteBalanceRange = CVidInputUsbFW_GetWhiteBalanceRange;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.GetWhiteBalance = CVidInputUsbFW_GetWhiteBalance;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.SetWhiteBalance = CVidInputUsbFW_SetWhiteBalance;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.SetAutoWhiteBalanceEnabled = CVidInputUsbFW_SetAutoWhiteBalanceEnabled;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.GetAutoWhiteBalanceEnabled = CVidInputUsbFW_GetAutoWhiteBalanceEnabled;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.GetFocusRange = CVidInputUsbFW_GetFocusRange;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.GetFocus = CVidInputUsbFW_GetFocus;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.SetFocus = CVidInputUsbFW_SetFocus;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.SetAutoFocusEnabled = CVidInputUsbFW_SetAutoFocusEnabled;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.GetAutoFocusEnabled = CVidInputUsbFW_GetAutoFocusEnabled;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.GetPowerLineFreqRange = CVidInputUsbFW_GetPowerLineFreqRange;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.GetPowerLineFreq = CVidInputUsbFW_GetPowerLineFreq;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.SetPowerLineFreq = CVidInputUsbFW_SetPowerLineFreq;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.StartVideo = CVidInputUsbFW_StartVideo;
			pVidInputUsbFW->m_VideoDecoder.m_iVideoDecoder.StopVideo = CVidInputUsbFW_StopVideo;

			// initialize members
			//
			pVidInputUsbFW->m_pUsbCntl = pUsbCntl;

			// inherited members
			//
			pVidInputUsbFW->m_VideoDecoder.m_nBrightnessDef = VIDINPUTUSBFW_BRIGHTNESS_DEFAULT;
			pVidInputUsbFW->m_VideoDecoder.m_nBrightnessMin = VIDINPUTUSBFW_BRIGHTNESS_MIN;
			pVidInputUsbFW->m_VideoDecoder.m_nBrightnessMax = VIDINPUTUSBFW_BRIGHTNESS_MAX;
			pVidInputUsbFW->m_VideoDecoder.m_nBrightness = VIDINPUTUSBFW_BRIGHTNESS_DEFAULT;
			pVidInputUsbFW->m_VideoDecoder.m_nContrastDef = VIDINPUTUSBFW_CONTRAST_DEFAULT;
			pVidInputUsbFW->m_VideoDecoder.m_nContrastMin = VIDINPUTUSBFW_CONTRAST_MIN;
			pVidInputUsbFW->m_VideoDecoder.m_nContrastMax = VIDINPUTUSBFW_CONTRAST_MAX;
			pVidInputUsbFW->m_VideoDecoder.m_nContrast = VIDINPUTUSBFW_CONTRAST_DEFAULT;
			pVidInputUsbFW->m_VideoDecoder.m_nSaturationDef = VIDINPUTUSBFW_SATURATION_DEFAULT;
			pVidInputUsbFW->m_VideoDecoder.m_nSaturationMin = VIDINPUTUSBFW_SATURATION_MIN;
			pVidInputUsbFW->m_VideoDecoder.m_nSaturationMax = VIDINPUTUSBFW_SATURATION_MAX;
			pVidInputUsbFW->m_VideoDecoder.m_nSaturation = VIDINPUTUSBFW_SATURATION_DEFAULT;
			pVidInputUsbFW->m_VideoDecoder.m_nHueDef = VIDINPUTUSBFW_HUE_DEFAULT;
			pVidInputUsbFW->m_VideoDecoder.m_nHueMin = VIDINPUTUSBFW_HUE_MIN;
			pVidInputUsbFW->m_VideoDecoder.m_nHueMax = VIDINPUTUSBFW_HUE_MAX;
			pVidInputUsbFW->m_VideoDecoder.m_nHue = VIDINPUTUSBFW_HUE_DEFAULT;
			pVidInputUsbFW->m_VideoDecoder.m_nSharpnessDef = VIDINPUTUSBFW_SHARPNESS_DEFAULT;
			pVidInputUsbFW->m_VideoDecoder.m_nSharpnessMin = VIDINPUTUSBFW_SHARPNESS_MIN;
			pVidInputUsbFW->m_VideoDecoder.m_nSharpnessMax = VIDINPUTUSBFW_SHARPNESS_MAX;
			pVidInputUsbFW->m_VideoDecoder.m_nSharpness = VIDINPUTUSBFW_SHARPNESS_DEFAULT;
			pVidInputUsbFW->m_VideoDecoder.m_nEdgeEnhancementDef = VIDINPUTUSBFW_EDGE_ENHANCEMENT_DEFAULT;
			pVidInputUsbFW->m_VideoDecoder.m_nEdgeEnhancementMin = VIDINPUTUSBFW_EDGE_ENHANCEMENT_MIN;
			pVidInputUsbFW->m_VideoDecoder.m_nEdgeEnhancementMax = VIDINPUTUSBFW_EDGE_ENHANCEMENT_MAX;
			pVidInputUsbFW->m_VideoDecoder.m_nEdgeEnhancement = VIDINPUTUSBFW_EDGE_ENHANCEMENT_DEFAULT;
			pVidInputUsbFW->m_VideoDecoder.m_nGammaDef = VIDINPUTUSBFW_GAMMA_DEFAULT;
			pVidInputUsbFW->m_VideoDecoder.m_nGammaMin = VIDINPUTUSBFW_GAMMA_MIN;
			pVidInputUsbFW->m_VideoDecoder.m_nGammaMax = VIDINPUTUSBFW_GAMMA_MAX;
			pVidInputUsbFW->m_VideoDecoder.m_nGamma = VIDINPUTUSBFW_GAMMA_DEFAULT;
			pVidInputUsbFW->m_VideoDecoder.m_nLowLightBoostDef = VIDINPUTUSBFW_LOW_LIGHT_BOOST_DEFAULT;
			pVidInputUsbFW->m_VideoDecoder.m_nLowLightBoostMin = VIDINPUTUSBFW_LOW_LIGHT_BOOST_MIN;
			pVidInputUsbFW->m_VideoDecoder.m_nLowLightBoostMax = VIDINPUTUSBFW_LOW_LIGHT_BOOST_MAX;
			pVidInputUsbFW->m_VideoDecoder.m_nLowLightBoost = VIDINPUTUSBFW_LOW_LIGHT_BOOST_DEFAULT;
			pVidInputUsbFW->m_VideoDecoder.m_nExposureDef = VIDINPUTUSBFW_EXPOSURE_DEFAULT;
			pVidInputUsbFW->m_VideoDecoder.m_nExposureMin = VIDINPUTUSBFW_EXPOSURE_MIN;
			pVidInputUsbFW->m_VideoDecoder.m_nExposureMax = VIDINPUTUSBFW_EXPOSURE_MAX;
			pVidInputUsbFW->m_VideoDecoder.m_nExposure = VIDINPUTUSBFW_EXPOSURE_DEFAULT;
			pVidInputUsbFW->m_VideoDecoder.m_AutoExposureEnable = QP_TRUE;
			pVidInputUsbFW->m_VideoDecoder.m_nWhiteBalanceDef = VIDINPUTUSBFW_WHITE_BALANCE_DEFAULT;
			pVidInputUsbFW->m_VideoDecoder.m_nWhiteBalanceMin = VIDINPUTUSBFW_WHITE_BALANCE_MIN;
			pVidInputUsbFW->m_VideoDecoder.m_nWhiteBalanceMax = VIDINPUTUSBFW_WHITE_BALANCE_MAX;
			pVidInputUsbFW->m_VideoDecoder.m_nWhiteBalance = VIDINPUTUSBFW_WHITE_BALANCE_DEFAULT;
			pVidInputUsbFW->m_VideoDecoder.m_AutoWhiteBalanceEnable = QP_TRUE;
			pVidInputUsbFW->m_VideoDecoder.m_nFocusDef = VIDINPUTUSBFW_FOCUS_DEFAULT;
			pVidInputUsbFW->m_VideoDecoder.m_nFocusMin = VIDINPUTUSBFW_FOCUS_MIN;
			pVidInputUsbFW->m_VideoDecoder.m_nFocusMax = VIDINPUTUSBFW_FOCUS_MAX;
			pVidInputUsbFW->m_VideoDecoder.m_nFocus = VIDINPUTUSBFW_FOCUS_DEFAULT;
			pVidInputUsbFW->m_VideoDecoder.m_AutoFocusEnable = QP_FALSE;
		}
	}

	return (pVidInputUsbFW);
}



// destructor
//
void CVidInputUsbFW_Destructor(CVidInputUsbFW *This)
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPPeripheralDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CVidInputUsbFW_Destructor()\n"
				 ));

	CVideoDecoder_Destructor(&This->m_VideoDecoder);
}

