///////////////////////////////////////////////////////////////////////////////
//        
//
// Qpi2c.h
//
// Description: 
//
//  I2C provider interface
// 
// History:
//	Aug 28, 2006	(dt) created
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __QPI2C_H__
#define __QPI2C_H__

#include "qptypes.h"

#ifdef __cplusplus
extern "C" {
#endif


/////////////////////////////////////////////////////////////////////////////
//
//


// i2c access type
//
typedef enum _QPI2C_TYPE
{
	QPI2C_TYPE_DUMMY = 0,
	QPI2C_TYPE_USE_USB = 1,
	QPI2C_TYPE_USE_QLCODEC = 2,
	QPI2C_TYPE_USE_201_EMULATION = 3,
	QPI2C_TYPE_SOC_INTERNAL_HW = 4,
	QPI2C_TYPE_SOC_INTERNAL_GPIO = 5,
	QPI2C_TYPE_USE_USB_SWI2C = 6
} QPI2C_TYPE, *PQPI2C_TYPE;

#define QPI2C_TYPE_FIRST	QPI2C_TYPE_USE_USB
#define QPI2C_TYPE_LAST		QPI2C_TYPE_USE_USB_SWI2C

typedef struct _QPI2C_INITDATA
{
	QPI2C_TYPE	type;
	TQP_UINT32	clk;
	TQP_UINT32	data;
} QPI2C_INITDATA, *PQPI2C_INITDATA;



/////////////////////////////////////////////////////////////////////////////
//
//


#ifndef __II2C_FWD_DEFINED__
#define __II2C_FWD_DEFINED__
typedef struct II2C II2C;
#endif //__II2C_FWD_DEFINED__

    struct II2C
    {
		// II2C
		EQP_Errors (*Read)(II2C *This, 
						   TQP_UINT8 cDeviceAddr, 
						   int nSize, 
						   PTQP_UINT8 pBuf
						   );
		EQP_Errors (*Write)(II2C *This, 
							TQP_UINT8 cDeviceAddr, 
							int nSize, 
							PTQP_UINT8 pBuf
							);
		EQP_Errors (*WriteThenRead)(II2C *This, 
									TQP_UINT8 cDeviceAddr, 
									int nWriteSize, 
									const PTQP_UINT8 pWriteBuf,
									int nReadSize,
									PTQP_UINT8 pReadBuf
									);
    };

#define II2C_Read(This, cDeviceAddr, nSize, pBuf) \
    (This)->Read(This, cDeviceAddr, nSize, pBuf)

#define II2C_Write(This, cDeviceAddr, nSize, pBuf) \
    (This)->Write(This, cDeviceAddr, nSize, pBuf)

#define II2C_WriteThenRead(This, cDeviceAddr, nWriteSize, pWriteBuf, nReadSize, pReadBuf) \
    (This)->WriteThenRead(This, cDeviceAddr, nWriteSize, pWriteBuf, nReadSize, pReadBuf)


#ifdef __cplusplus
}
#endif

#endif //__QPI2C_H__

