///////////////////////////////////////////////////////////////////////////////
//        
//
// Saa7115regs.h
//
// Description: 
//
//  SAA7115 Register definations
// 
// History:
//	Sep 5, 2006	(dt) created for fellini project
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#ifndef __SAA7115REGS_H__
#define __SAA7115REGS_H__

#include "qptypes.h"

/////////////////////////////////////////////////////////////////////////////
//
//

// Registers Sub Address 0x00 to 0x1F Video Decoder
//

#define SAA7115_REG_CHIP_VERSION				0x00
#define SAA7115_REG_ANALOG_INPUT_CONTROL_0		0x01
#define SAA7115_REG_ANALOG_INPUT_CONTROL_1		0x02
#define SAA7115_REG_ANALOG_INPUT_CONTROL_2		0x03
#define SAA7115_REG_ANALOG_INPUT_CONTROL_3		0x04
#define SAA7115_REG_ANALOG_INPUT_CONTROL_4		0x05
#define SAA7115_REG_HORIZONTAL_SYNC_START		0x06
#define SAA7115_REG_HORIZONTAL_SYNC_STOP		0x07
#define SAA7115_REG_SYNC_CONTROL				0x08
#define SAA7115_REG_LUMINANCE_CONTROL			0x09
#define SAA7115_REG_LUMINANCE_BRIGHTNESS		0x0A
#define SAA7115_REG_LUMINANCE_CONTRAST			0x0B
#define SAA7115_REG_CHROMA_SATURATION			0x0C
#define SAA7115_REG_CHROMA_HUE					0x0D
#define SAA7115_REG_CHROMA_CONTROL_1			0x0E
#define SAA7115_REG_CHROMA_GAIN_CONTROL			0x0F
#define SAA7115_REG_CHROMA_CONTROL_2			0x10
#define SAA7115_REG_MODE_DELAY_CONTROL			0x11
#define SAA7115_REG_RT_SIGNAL_CONTROL			0x12
#define SAA7115_REG_RT_XPORT_OUTPUT_CONTROL		0x13
#define SAA7115_REG_ANALOG_ADC_CONTROL			0x14
#define SAA7115_REG_VGATE_START					0x15
#define SAA7115_REG_VGATE_STOP					0x16
#define SAA7115_REG_MISC_VGATE_MSB				0x17
#define SAA7115_REG_RAW_DATA_GAIN				0x18
#define SAA7115_REG_RAW_DATA_OFFSET				0x19
#define SAA7115_REG_COLORKILLER_THRESHOLD		0x1A
#define SAA7115_REG_MISC_TVVCRDET				0x1B
#define SAA7115_REG_ENHANCED_COMB_CTRL_1		0x1C
#define SAA7115_REG_ENHANCED_COMB_CTRL_2		0x1D
#define SAA7115_REG_STATUS_BYTE_DECODER_1		0x1E
#define SAA7115_REG_STATUS_BYTE_DECODER_2		0x1F


#define SAA7115_REG_VID_DECODER_BEGIN			SAA7115_REG_CHIP_VERSION


#pragma pack(1)

typedef union _SAA7115_CHIP_VERSION
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	ChipID:4;						// Bits 3 - 0
		TQP_UINT8	ChipVersion:4;					// Bits 7 - 4
	} Reg;
} SAA7115_CHIP_VERSION, *PSAA7115_CHIP_VERSION;


typedef union _SAA7115_ANALOG_INPUT_CONTROL_0
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	IDEL:4;							// Bits 3 - 0
		TQP_UINT8	GUDL:2;							// Bits 5 - 4
		TQP_UINT8	WPOFF:1;						// Bits 6
		TQP_UINT8	AOSL2:1;						// Bits 7
	} Reg;
} SAA7115_ANALOG_INPUT_CONTROL_0, *PSAA7115_ANALOG_INPUT_CONTROL_0;


typedef union _SAA7115_ANALOG_INPUT_CONTROL_1
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	MODE:4;							// Bits 3 - 0
		TQP_UINT8	Reserved:2;						// Bits 5 - 4
		TQP_UINT8	FUSE:2;							// Bits 7 - 6
	} Reg;
} SAA7115_ANALOG_INPUT_CONTROL_1, *PSAA7115_ANALOG_INPUT_CONTROL_1;


typedef union _SAA7115_ANALOG_INPUT_CONTROL_2
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	GAI18:1;						// Bits 0
		TQP_UINT8	GAI28:1;						// Bits 1
		TQP_UINT8	GAFIX:1;						// Bits 2
		TQP_UINT8	HOLDG:1;						// Bits 3
		TQP_UINT8	CPOFF:1;						// Bits 4
		TQP_UINT8	VBSL:1;							// Bits 5
		TQP_UINT8	HLNRS:1;						// Bits 6
		TQP_UINT8	Reserved:1;						// Bits 7
	} Reg;
} SAA7115_ANALOG_INPUT_CONTROL_2, *PSAA7115_ANALOG_INPUT_CONTROL_2;


typedef union _SAA7115_SYNC_CONTROL
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	VNOI:2;							// Bits 1 - 0
		TQP_UINT8	HPLL:1;							// Bits 2
		TQP_UINT8	HTC:2;							// Bits 4 - 3
		TQP_UINT8	FOET:1;							// Bits 5
		TQP_UINT8	FSEL:1;							// Bits 6
		TQP_UINT8	AUFD:1;							// Bits 7
	} Reg;
} SAA7115_SYNC_CONTROL, *PSAA7115_SYNC_CONTROL;


typedef union _SAA7115_LUMINANCE_CONTROL
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	LUFI:4;							// Bits 3 - 0
		TQP_UINT8	LUBW:1;							// Bits 4
		TQP_UINT8	LDEL:1;							// Bits 5
		TQP_UINT8	YCOMB:1;						// Bits 6
		TQP_UINT8	BYPS:1;							// Bits 7
	} Reg;
} SAA7115_LUMINANCE_CONTROL, *PSAA7115_LUMINANCE_CONTROL;


typedef union _SAA7115_CHROMA_CONTROL_1
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	CCOMB:1;						// Bits 0
		TQP_UINT8	AUTO0:1;						// Bits 1
		TQP_UINT8	FCTC:1;							// Bits 2
		TQP_UINT8	DCVF:1;							// Bits 3
		TQP_UINT8	CSTD:3;							// Bits 6 - 4
		TQP_UINT8	CDTO:1;							// Bits 7
	} Reg;
} SAA7115_CHROMA_CONTROL_1, *PSAA7115_CHROMA_CONTROL_1;


typedef union _SAA7115_CHROMA_CONTROL_2
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	LCBW:3;							// Bits 2 - 0
		TQP_UINT8	CHBW:1;							// Bits 3
		TQP_UINT8	OFFV1:2;						// Bits 5 - 4
		TQP_UINT8	OFFU1:2;						// Bits 7 - 6
	} Reg;
} SAA7115_CHROMA_CONTROL_2, *PSAA7115_CHROMA_CONTROL_2;


typedef union _SAA7115_MODE_DELAY_CONTROL
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	YDEL:3;							// Bits 2 - 0
		TQP_UINT8	RTP0:1;							// Bits 3
		TQP_UINT8	HDEL:2;							// Bits 5 - 4
		TQP_UINT8	RTP1:1;							// Bits 6
		TQP_UINT8	COLO:1;							// Bits 7
	} Reg;
} SAA7115_MODE_DELAY_CONTROL, *PSAA7115_MODE_DELAY_CONTROL;


typedef union _SAA7115_RT_SIGNAL_CONTROL
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	RTSE0:4;						// Bits 3 - 0
		TQP_UINT8	RTSE1:4;						// Bits 7 - 4
	} Reg;
} SAA7115_RT_SIGNAL_CONTROL, *PSAA7115_RT_SIGNAL_CONTROL;


typedef union _SAA7115_RT_XPORT_OUTPUT_CONTROL
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	OFTS:3;							// Bits 2 - 0
		TQP_UINT8	HLSEL:1;						// Bits 3
		TQP_UINT8	XRVS:2;							// Bits 5 - 4
		TQP_UINT8	XRHS:1;							// Bits 6
		TQP_UINT8	RTCE:1;							// Bits 7
	} Reg;
} SAA7115_RT_XPORT_OUTPUT_CONTROL, *PSAA7115_RT_XPORT_OUTPUT_CONTROL;


typedef union _SAA7115_ANALOG_ADC_CONTROL
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	APCK:2;							// Bits 1 - 0
		TQP_UINT8	AUTO1:1;						// Bits 2
		TQP_UINT8	XTOUTE:1;						// Bits 3
		TQP_UINT8	AOSL:2;							// Bits 5 - 4
		TQP_UINT8	UPTCV:1;						// Bits 6
		TQP_UINT8	CM99:1;							// Bits 7
	} Reg;
} SAA7115_ANALOG_ADC_CONTROL, *PSAA7115_ANALOG_ADC_CONTROL;


typedef union _SAA7115_MISC_VGATE_MSB
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	VSTA8:1;						// Bits 0
		TQP_UINT8	VSTO8:1;						// Bits 1
		TQP_UINT8	VGPS:1;							// Bits 2
		TQP_UINT8	LATY:3;							// Bits 5 - 3
		TQP_UINT8	LLC2E:1;						// Bits 6
		TQP_UINT8	LLCE:1;							// Bits 7
	} Reg;
} SAA7115_MISC_VGATE_MSB, *PSAA7115_MISC_VGATE_MSB;


typedef union _SAA7115_COLORKILLER_THRESHOLD
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	STHR:4;							// Bits 3 - 0
		TQP_UINT8	QTHR:4;							// Bits 7 - 4
	} Reg;
} SAA7115_COLORKILLER_THRESHOLD, *PSAA7115_COLORKILLER_THRESHOLD;


typedef union _SAA7115_MISC_TVVCRDET
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	FSQC:1;							// Bits 0
		TQP_UINT8	ACOL:1;							// Bits 1
		TQP_UINT8	Reserved1:2;					// Bits 3 - 2
		TQP_UINT8	OFTS3:1;						// Bits 4
		TQP_UINT8	Reserved2:1;					// Bits 5
		TQP_UINT8	ATVT:2;							// Bits 7 - 6
	} Reg;
} SAA7115_MISC_TVVCRDET, *PSAA7115_MISC_TVVCRDET;


typedef union _SAA7115_ENHANCED_COMB_CTRL_1
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	CMBT:2;							// Bits 1 - 0
		TQP_UINT8	MEDG:2;							// Bits 3 - 2
		TQP_UINT8	VEDG:2;							// Bits 5 - 4
		TQP_UINT8	HODG:2;							// Bits 7 - 6
	} Reg;
} SAA7115_ENHANCED_COMB_CTRL_1, *PSAA7115_ENHANCED_COMB_CTRL_1;


typedef union _SAA7115_ENHANCED_COMB_CTRL_2
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	VEDT:2;							// Bits 1 - 0
		TQP_UINT8	Reserved:6;						// Bits 7 - 2
	} Reg;
} SAA7115_ENHANCED_COMB_CTRL_2, *PSAA7115_ENHANCED_COMB_CTRL_2;


typedef union _SAA7115_STATUS_BYTE_DECODER
{
	TQP_UINT16		Read;
	struct
	{
		TQP_UINT16	DCSTD:2;						// Bits 1 - 0
		TQP_UINT16	WIPA:1;							// Bits 2
		TQP_UINT16	GLIMB:1;						// Bits 3
		TQP_UINT16	GLIMT:1;						// Bits 4
		TQP_UINT16	SLTCA:1;						// Bits 5
		TQP_UINT16	HLCK:1;							// Bits 6
		TQP_UINT16	NFLD:1;							// Bits 7
		TQP_UINT16	RDCAP:1;						// Bits 8
		TQP_UINT16	COPRO:1;						// Bits 9
		TQP_UINT16	COLSTR:1;						// Bits 10
		TQP_UINT16	TYPE3:1;						// Bits 11
		TQP_UINT16	STTB:1;							// Bits 12
		TQP_UINT16	FIDT:1;							// Bits 13
		TQP_UINT16	HLVLN:1;						// Bits 14
		TQP_UINT16	INTL:1;							// Bits 15
	} Reg;
} SAA7115_STATUS_BYTE_DECODER, *PSAA7115_STATUS_BYTE_DECODER;


#pragma pack()


#define	SAA7115_BRIGHTNESS_MIN					0x00
#define	SAA7115_BRIGHTNESS_MAX					0xFF
#define	SAA7115_BRIGHTNESS_DEFAULT				0x80

#define	SAA7115_CONTRAST_MIN					0x00
#define	SAA7115_CONTRAST_MAX					0x7F
#define	SAA7115_CONTRAST_DEFAULT				0x44

#define	SAA7115_SATURATION_MIN					0x00
#define	SAA7115_SATURATION_MAX					0x7F
#define	SAA7115_SATURATION_DEFAULT				0x40

#define	SAA7115_HUE_MIN							(-128)	// 0x80
#define	SAA7115_HUE_MAX							(+127)	// 0x7F
#define	SAA7115_HUE_DEFAULT						0x00

#define	SAA7115_SHARPNESS_MIN					0
#define	SAA7115_SHARPNESS_MAX					15
#define	SAA7115_SHARPNESS_DEFAULT				8


typedef enum _SAA7115_INPUT_MODE
{
	SAA7115_INPUT_MODE_0 = 0x00,	// 0 0 0 0 AI11           _PIN_20
	SAA7115_INPUT_MODE_1 = 0x01,	// 0 0 0 1 AI12           _PIN_18
	SAA7115_INPUT_MODE_2 = 0x02,	// 0 0 1 0 AI21           _PIN_16
	SAA7115_INPUT_MODE_3 = 0x03,	// 0 0 1 1 AI22           _PIN_14
	SAA7115_INPUT_MODE_4 = 0x04,	// 0 1 0 0 AI23           _PIN_12
	SAA7115_INPUT_MODE_5 = 0x05,	// 0 1 0 1 AI24           _PIN_10
	SAA7115_INPUT_MODE_6 = 0x06,	// 0 1 1 0 Y-AI11, C-AI21 _PIN_20_16
	SAA7115_INPUT_MODE_7 = 0x07,	// 0 1 1 1 Y-AI12, C-AI22 _PIN_18_14
	SAA7115_INPUT_MODE_8 = 0x08,	// 1 0 0 0 Y-AI11, C-AI21 _PIN_20_16
	SAA7115_INPUT_MODE_9 = 0x09		// 1 0 0 1 Y-AI12, C-AI22 _PIN_18_14
} SAA7115_INPUT_MODE, *PSAA7115_INPUT_MODE;



/////////////////////////////////////////////////////////////////////////////
//
//

// Registers Sub Address 0x30 to 0x3F Audio Clock Generation
//

#define SAA7115_REG_AUDIO_CLK_CYCLE_PER_FIELD_0	0x30
#define SAA7115_REG_AUDIO_CLK_CYCLE_PER_FIELD_1	0x31
#define SAA7115_REG_AUDIO_CLK_CYCLE_PER_FIELD_2	0x32
#define SAA7115_REG_AUDIO_CLK_INCREMENT_0		0x34
#define SAA7115_REG_AUDIO_CLK_INCREMENT_1		0x35
#define SAA7115_REG_AUDIO_CLK_INCREMENT_2		0x36
#define SAA7115_REG_AUDIO_CLK_RATIO_AMXCLK_ASCLK	0x38
#define SAA7115_REG_AUDIO_CLK_RATIO_ASCLK_AMXCLK	0x39
#define SAA7115_REG_AUDIO_CLK_CONTROL			0x3A


#define SAA7115_REG_AUDIO_CLK_BEGIN				SAA7115_REG_AUDIO_CLK_CYCLE_PER_FIELD_0



#pragma pack(1)

typedef union _SAA7115_AUDIO_CLK_CONTROL
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	SCPH:1;							// Bits 0
		TQP_UINT8	LRPH:1;							// Bits 1
		TQP_UINT8	AMVR:1;							// Bits 2
		TQP_UINT8	APLL:1;							// Bits 3
		TQP_UINT8	Reserved:2;						// Bits 5 - 4
		TQP_UINT8	CGCDIV:1;						// Bits 6
		TQP_UINT8	UCGC:1;							// Bits 7
	} Reg;
} SAA7115_AUDIO_CLK_CONTROL, *PSAA7115_AUDIO_CLK_CONTROL;

#pragma pack()



/////////////////////////////////////////////////////////////////////////////
//
//

// Registers Sub Address 0x40 to 0x7F VBI Data Slicer
//

#define SAA7115_REG_VBI_AC1						0x40
#define SAA7115_REG_VBI_LCR2					0x41
#define SAA7115_REG_VBI_LCR3					0x42
#define SAA7115_REG_VBI_LCR4					0x43
#define SAA7115_REG_VBI_LCR5					0x44
#define SAA7115_REG_VBI_LCR6					0x45
#define SAA7115_REG_VBI_LCR7					0x46
#define SAA7115_REG_VBI_LCR8					0x47
#define SAA7115_REG_VBI_LCR9					0x48
#define SAA7115_REG_VBI_LCR10					0x49
#define SAA7115_REG_VBI_LCR11					0x4A
#define SAA7115_REG_VBI_LCR12					0x4B
#define SAA7115_REG_VBI_LCR13					0x4C
#define SAA7115_REG_VBI_LCR14					0x4D
#define SAA7115_REG_VBI_LCR15					0x4E
#define SAA7115_REG_VBI_LCR16					0x4F
#define SAA7115_REG_VBI_LCR17					0x50
#define SAA7115_REG_VBI_LCR18					0x51
#define SAA7115_REG_VBI_LCR19					0x52
#define SAA7115_REG_VBI_LCR20					0x53
#define SAA7115_REG_VBI_LCR21					0x54
#define SAA7115_REG_VBI_LCR22					0x55
#define SAA7115_REG_VBI_LCR23					0x56
#define SAA7115_REG_VBI_LCR24					0x57
#define SAA7115_REG_VBI_FC						0x58
#define SAA7115_REG_VBI_HOFF					0x59
#define SAA7115_REG_VBI_VOFF					0x5A
#define SAA7115_REG_VBI_HVOFF					0x5B
#define SAA7115_REG_VBI_OUTPUT_MODE				0x5D
#define SAA7115_REG_VBI_SDID					0x5E


#define SAA7115_REG_VBI_BEGIN					SAA7115_REG_VBI_AC1


#pragma pack(1)

typedef union _SAA7115_VBI_AC1
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	Reserved:4;						// Bits 3 - 0
		TQP_UINT8	HUNT_N:1;						// Bits 4
		TQP_UINT8	FCE:1;							// Bits 5
		TQP_UINT8	HAM_N:1;						// Bits 6
		TQP_UINT8	CHKWSS:1;						// Bits 7
	} Reg;
} SAA7115_VBI_AC1, *PSAA7115_VBI_AC1;


typedef union _SAA7115_VBI_LCR
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	Field2:4;						// Bits 3 - 0
		TQP_UINT8	Field1:4;						// Bits 7 - 4
	} Reg;
} SAA7115_VBI_LCR, *PSAA7115_VBI_LCR;


typedef union _SAA7115_VBI_HVOFF
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	HOFF:3;							// Bits 2 - 0
		TQP_UINT8	Reserved1:1;					// Bits 3
		TQP_UINT8	VOFF:1;							// Bits 4
		TQP_UINT8	VEP:1;							// Bits 5
		TQP_UINT8	Reserved2:1;					// Bits 6
		TQP_UINT8	FOFF:1;							// Bits 7
	} Reg;
} SAA7115_VBI_HVOFF, *PSAA7115_VBI_HVOFF;


typedef union _SAA7115_VBI_OUTPUT_MODE
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	SLDOM:5;						// Bits 4 - 0
		TQP_UINT8	Reserved:3;						// Bits 7 - 5
	} Reg;
} SAA7115_VBI_OUTPUT_MODE, *PSAA7115_VBI_OUTPUT_MODE;


typedef union _SAA7115_VBI_SDID
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	SDID:6;							// Bits 5 - 0
		TQP_UINT8	FC7V:1;							// Bits 6
		TQP_UINT8	FC8V:1;							// Bits 7
	} Reg;
} SAA7115_VBI_SDID, *PSAA7115_VBI_SDID;


#pragma pack()


/////////////////////////////////////////////////////////////////////////////
//
//

// Registers Sub Address 0x80 to 0x8F Global Settings
//

#define SAA7115_REG_GLOBAL_CONTROL_1			0x80
#define SAA7115_REG_GLOBAL_VSYN_FIELD_ID		0x81
#define SAA7115_REG_GLOBAL_XPORT_DELAY_ENABLE	0x83
#define SAA7115_REG_GLOBAL_IPORT_DEFINATIONS	0x84
#define SAA7115_REG_GLOBAL_IPORT_POLARITIES		0x85
#define SAA7115_REG_GLOBAL_IPORT_FIFO			0x86
#define SAA7115_REG_GLOBAL_IPORT_DELAY_ENABLE	0x87
#define SAA7115_REG_GLOBAL_POWER_SAVE			0x88
#define SAA7115_REG_GLOBAL_SCALER_STATUS		0x8F


#define SAA7115_REG_GLOBAL_BEGIN				SAA7115_REG_GLOBAL_CONTROL_1


#pragma pack(1)

typedef union _SAA7115_GLOBAL_CONTROL_1
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	ICKS:4;							// Bits 3 - 0
		TQP_UINT8	TEA:1;							// Bits 4
		TQP_UINT8	TEB:1;							// Bits 5
		TQP_UINT8	Reserved:1;						// Bits 6
		TQP_UINT8	CMOD:1;							// Bits 7
	} Reg;
} SAA7115_GLOBAL_CONTROL_1, *PSAA7115_GLOBAL_CONTROL_1;


typedef union _SAA7115_GLOBAL_VSYN_FIELD_ID
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	V_EAV:2;						// Bits 1 - 0
		TQP_UINT8	FTIME:1;						// Bits 2
		TQP_UINT8	Reserved:5;						// Bits 7 - 3
	} Reg;
} SAA7115_GLOBAL_VSYN_FIELD_ID, *PSAA7115_GLOBAL_VSYN_FIELD_ID;


typedef union _SAA7115_GLOBAL_XPORT_DELAY_ENABLE
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	XPE:2;							// Bits 1 - 0
		TQP_UINT8	XRQT:1;							// Bits 2
		TQP_UINT8	Reserved1:1;					// Bits 3
		TQP_UINT8	XPCK:2;							// Bits 5 - 4
		TQP_UINT8	Reserved2:2;					// Bits 7 - 6
	} Reg;
} SAA7115_GLOBAL_XPORT_DELAY_ENABLE, *PSAA7115_GLOBAL_XPORT_DELAY_ENABLE;


typedef union _SAA7115_GLOBAL_IPORT_DEFINATIONS
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	IDH:2;							// Bits 1 - 0
		TQP_UINT8	IDV:2;							// Bits 3 - 2
		TQP_UINT8	IDG0:2;							// Bits 5 - 4
		TQP_UINT8	IDG1:2;							// Bits 7 - 6
	} Reg;
} SAA7115_GLOBAL_IPORT_DEFINATIONS, *PSAA7115_GLOBAL_IPORT_DEFINATIONS;


typedef union _SAA7115_GLOBAL_IPORT_POLARITIES
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	IDQP:1;							// Bits 0
		TQP_UINT8	IRHP:1;							// Bits 1
		TQP_UINT8	IRVP:1;							// Bits 2
#if 1
		TQP_UINT8	IGP1P:1;						// Bits 3
		TQP_UINT8	IGP0P:1;						// Bits 4
#else
		TQP_UINT8	IGP0P:1;						// Bits 3
		TQP_UINT8	IGP1P:1;						// Bits 4
#endif
		TQP_UINT8	ILLV:1;							// Bits 5
		TQP_UINT8	ISWP0:1;						// Bits 6
		TQP_UINT8	ISWP1:1;						// Bits 7
	} Reg;
} SAA7115_GLOBAL_IPORT_POLARITIES, *PSAA7115_GLOBAL_IPORT_POLARITIES;


typedef union _SAA7115_GLOBAL_IPORT_FIFO
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	FEL:2;							// Bits 1 - 0
		TQP_UINT8	FFL:2;							// Bits 3 - 2
		TQP_UINT8	IDG02:1;						// Bits 4
		TQP_UINT8	IDG12:1;						// Bits 5
		TQP_UINT8	VITX:1;							// Bits 6
		TQP_UINT8	IMPAK:1;						// Bits 7
	} Reg;
} SAA7115_GLOBAL_IPORT_FIFO, *PSAA7115_GLOBAL_IPORT_FIFO;


typedef union _SAA7115_GLOBAL_IPORT_DELAY_ENABLE
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	IPE:2;							// Bits 1 - 0
		TQP_UINT8	Reserved:2;						// Bits 3 - 2
		TQP_UINT8	IPCK:4;							// Bits 7 - 4
	} Reg;
} SAA7115_GLOBAL_IPORT_DELAY_ENABLE, *PSAA7115_GLOBAL_IPORT_DELAY_ENABLE;


typedef union _SAA7115_GLOBAL_POWER_SAVE
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	SLM:4;							// Bits 3 - 0
		TQP_UINT8	DPROG:1;						// Bits 4
		TQP_UINT8	SWRST:1;						// Bits 5
		TQP_UINT8	CH1EN:1;						// Bits 6
		TQP_UINT8	CH2EN:1;						// Bits 7
	} Reg;
} SAA7115_GLOBAL_POWER_SAVE, *PSAA7115_GLOBAL_POWER_SAVE;


typedef union _SAA7115_GLOBAL_SCALER_STATUS
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	FIDSCO:1;						// Bits 0
		TQP_UINT8	FIDSCI:1;						// Bits 1
		TQP_UINT8	ERR_OF:1;						// Bits 2
		TQP_UINT8	PRDON:1;						// Bits 3
		TQP_UINT8	FFOV:1;							// Bits 4
		TQP_UINT8	FFIL:1;							// Bits 5
		TQP_UINT8	ITRI:1;							// Bits 6
		TQP_UINT8	XTRI:1;							// Bits 7
	} Reg;
} SAA7115_GLOBAL_SCALER_STATUS, *PSAA7115_GLOBAL_SCALER_STATUS;


#pragma pack()


/////////////////////////////////////////////////////////////////////////////
//
//

// Registers Sub Address 0x90 to 0xBF Task A
//

#define SAA7115_REG_TASKA_CONTROL				0x90
#define SAA7115_REG_TASKA_XPORT_FORMAT_CONFIG	0x91
#define SAA7115_REG_TASKA_XPORT_INPUT_DEFINITIONS	0x92
#define SAA7115_REG_TASKA_IPORT_FORMAT_CONFIG	0x93
#define SAA7115_REG_TASKA_HORZ_INPUT_START_1	0x94
#define SAA7115_REG_TASKA_HORZ_INPUT_START_2	0x95
#define SAA7115_REG_TASKA_HORZ_INPUT_LENGTH_1	0x96
#define SAA7115_REG_TASKA_HORZ_INPUT_LENGTH_2	0x97
#define SAA7115_REG_TASKA_VERT_INPUT_START_1	0x98
#define SAA7115_REG_TASKA_VERT_INPUT_START_2	0x99
#define SAA7115_REG_TASKA_VERT_INPUT_LENGTH_1	0x9A
#define SAA7115_REG_TASKA_VERT_INPUT_LENGTH_2	0x9B
#define SAA7115_REG_TASKA_HORZ_OUTPUT_LENGTH_1	0x9C
#define SAA7115_REG_TASKA_HORZ_OUTPUT_LENGTH_2	0x9D
#define SAA7115_REG_TASKA_VERT_OUTPUT_LENGTH_1	0x9E
#define SAA7115_REG_TASKA_VERT_OUTPUT_LENGTH_2	0x9F
#define SAA7115_REG_TASKA_HORZ_PRESCALING		0xA0
#define SAA7115_REG_TASKA_ACCUMULATION_LENGTH	0xA1
#define SAA7115_REG_TASKA_PRESCALER_CONTROL		0xA2
#define SAA7115_REG_TASKA_LUMINANCE_BRIGHTNESS	0xA4
#define SAA7115_REG_TASKA_LUMINANCE_CONTRAST	0xA5
#define SAA7115_REG_TASKA_CHROMA_SATURATION		0xA6
#define SAA7115_REG_TASKA_HORZ_SCALING_INC_LUMA_1	0xA8
#define SAA7115_REG_TASKA_HORZ_SCALING_INC_LUMA_2	0xA9
#define SAA7115_REG_TASKA_HORZ_PHASE_OFFSET_LUMA	0xAA
#define SAA7115_REG_TASKA_HORZ_SCALING_INC_CHROMA_1	0xAC
#define SAA7115_REG_TASKA_HORZ_SCALING_INC_CHROMA_2	0xAD
#define SAA7115_REG_TASKA_HORZ_PHASE_OFFSET_CHROMA	0xAE
#define SAA7115_REG_TASKA_VERT_SCALING_INC_LUMA_1	0xB0
#define SAA7115_REG_TASKA_VERT_SCALING_INC_LUMA_2	0xB1
#define SAA7115_REG_TASKA_VERT_SCALING_INC_CHROMA_1	0xB2
#define SAA7115_REG_TASKA_VERT_SCALING_INC_CHROMA_2	0xB3
#define SAA7115_REG_TASKA_VERT_SCALING_MODE_CONTROL	0xB4
#define SAA7115_REG_TASKA_VERT_PHASE_OFFSET_CHROMA0	0xB8
#define SAA7115_REG_TASKA_VERT_PHASE_OFFSET_CHROMA1	0xB9
#define SAA7115_REG_TASKA_VERT_PHASE_OFFSET_CHROMA2	0xBA
#define SAA7115_REG_TASKA_VERT_PHASE_OFFSET_CHROMA3	0xBB
#define SAA7115_REG_TASKA_VERT_PHASE_OFFSET_LUMA0	0xBC
#define SAA7115_REG_TASKA_VERT_PHASE_OFFSET_LUMA1	0xBD
#define SAA7115_REG_TASKA_VERT_PHASE_OFFSET_LUMA2	0xBE
#define SAA7115_REG_TASKA_VERT_PHASE_OFFSET_LUMA3	0xBF


#define SAA7115_REG_TASKA_BEGIN					SAA7115_REG_TASKA_CONTROL


#pragma pack(1)

typedef union _SAA7115_TASKA_CONTROL
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	STRC:2;							// Bits 1 - 0
		TQP_UINT8	RPTSK:1;						// Bits 2
		TQP_UINT8	FSKP:3;							// Bits 5 - 3
		TQP_UINT8	OFIDC:1;						// Bits 6
		TQP_UINT8	CONLH:1;						// Bits 7
	} Reg;
} SAA7115_TASKA_CONTROL, *PSAA7115_TASKA_CONTROL;


typedef union _SAA7115_TASKA_XPORT_FORMAT_CONFIG
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	FSC:3;							// Bits 2 - 0
		TQP_UINT8	SCRQE:1;						// Bits 3
		TQP_UINT8	SCSRC:2;						// Bits 5 - 4
		TQP_UINT8	HLDFV:1;						// Bits 6
		TQP_UINT8	CONLV:1;						// Bits 7
	} Reg;
} SAA7115_TASKA_XPORT_FORMAT_CONFIG, *PSAA7115_TASKA_XPORT_FORMAT_CONFIG;


typedef union _SAA7115_TASKA_XPORT_INPUT_DEFINITIONS
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	XCKS:1;							// Bits 0
		TQP_UINT8	XDQ:1;							// Bits 1
		TQP_UINT8	XDH:1;							// Bits 2
		TQP_UINT8	XCODE:1;						// Bits 3
		TQP_UINT8	XDV0:1;							// Bits 4
		TQP_UINT8	XDV1:1;							// Bits 5
		TQP_UINT8	XFDH:1;							// Bits 6
		TQP_UINT8	XFDV:1;							// Bits 7
	} Reg;
} SAA7115_TASKA_XPORT_INPUT_DEFINITIONS, *PSAA7115_TASKA_XPORT_INPUT_DEFINITIONS;


typedef union _SAA7115_TASKA_IPORT_FORMAT_CONFIG
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	FSI:3;							// Bits 2 - 0
		TQP_UINT8	FOI:2;							// Bits 4 - 3
		TQP_UINT8	FYSK:1;							// Bits 5
		TQP_UINT8	INS80:1;						// Bits 6
		TQP_UINT8	ICODE:1;						// Bits 7
	} Reg;
} SAA7115_TASKA_IPORT_FORMAT_CONFIG, *PSAA7115_TASKA_IPORT_FORMAT_CONFIG;


typedef union _SAA7115_TASKA_PRESCALER_CONTROL
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	XDCG:3;							// Bits 2 - 0
		TQP_UINT8	XC2_1:1;						// Bits 3
		TQP_UINT8	PFY:2;							// Bits 5 - 4
		TQP_UINT8	PFUV:2;							// Bits 7 - 6
	} Reg;
} SAA7115_TASKA_PRESCALER_CONTROL, *PSAA7115_TASKA_PRESCALER_CONTROL;


typedef union _SAA7115_TASKA_VERT_SCALING_MODE_CONTROL
{
	TQP_UINT8		Read;
	struct
	{
		TQP_UINT8	YMODE:1;						// Bits 0
		TQP_UINT8	Reserved1:3;					// Bits 3 - 1
		TQP_UINT8	YMIR:1;							// Bits 4
		TQP_UINT8	Reserved2:3;					// Bits 7 - 5
	} Reg;
} SAA7115_TASKA_VERT_SCALING_MODE_CONTROL, *PSAA7115_TASKA_VERT_SCALING_MODE_CONTROL;

#pragma pack()



/////////////////////////////////////////////////////////////////////////////
//
//

// Registers Sub Address 0xC0 to 0xEF Task B
//

#define SAA7115_REG_TASKB_CONTROL				0xC0
#define SAA7115_REG_TASKB_XPORT_FORMAT_CONFIG	0xC1
#define SAA7115_REG_TASKB_XPORT_INPUT_DEFINITIONS	0xC2
#define SAA7115_REG_TASKB_IPORT_FORMAT_CONFIG	0xC3
#define SAA7115_REG_TASKB_HORZ_INPUT_START_1	0xC4
#define SAA7115_REG_TASKB_HORZ_INPUT_START_2	0xC5
#define SAA7115_REG_TASKB_HORZ_INPUT_LENGTH_1	0xC6
#define SAA7115_REG_TASKB_HORZ_INPUT_LENGTH_2	0xC7
#define SAA7115_REG_TASKB_VERT_INPUT_START_1	0xC8
#define SAA7115_REG_TASKB_VERT_INPUT_START_2	0xC9
#define SAA7115_REG_TASKB_VERT_INPUT_LENGTH_1	0xCA
#define SAA7115_REG_TASKB_VERT_INPUT_LENGTH_2	0xCB
#define SAA7115_REG_TASKB_HORZ_OUTPUT_LENGTH_1	0xCC
#define SAA7115_REG_TASKB_HORZ_OUTPUT_LENGTH_2	0xCD
#define SAA7115_REG_TASKB_VERT_OUTPUT_LENGTH_1	0xCE
#define SAA7115_REG_TASKB_VERT_OUTPUT_LENGTH_2	0xCF
#define SAA7115_REG_TASKB_HORZ_PRESCALING		0xD0
#define SAA7115_REG_TASKB_ACCUMULATION_LENGTH	0xD1
#define SAA7115_REG_TASKB_PRESCALER_CONTROL		0xD2
#define SAA7115_REG_TASKB_LUMINANCE_BRIGHTNESS	0xD4
#define SAA7115_REG_TASKB_LUMINANCE_CONTRAST	0xD5
#define SAA7115_REG_TASKB_CHROMA_SATURATION		0xD6
#define SAA7115_REG_TASKB_HORZ_SCALING_INC_LUMA_1	0xD8
#define SAA7115_REG_TASKB_HORZ_SCALING_INC_LUMA_2	0xD9
#define SAA7115_REG_TASKB_HORZ_PHASE_OFFSET_LUMA	0xDA
#define SAA7115_REG_TASKB_HORZ_SCALING_INC_CHROMA_1	0xDC
#define SAA7115_REG_TASKB_HORZ_SCALING_INC_CHROMA_2	0xDD
#define SAA7115_REG_TASKB_HORZ_PHASE_OFFSET_CHROMA	0xDE
#define SAA7115_REG_TASKB_VERT_SCALING_INC_LUMA_1	0xE0
#define SAA7115_REG_TASKB_VERT_SCALING_INC_LUMA_2	0xE1
#define SAA7115_REG_TASKB_VERT_SCALING_INC_CHROMA_1	0xE2
#define SAA7115_REG_TASKB_VERT_SCALING_INC_CHROMA_2	0xE3
#define SAA7115_REG_TASKB_VERT_SCALING_MODE_CONTROL	0xE4
#define SAA7115_REG_TASKB_VERT_PHASE_OFFSET_CHROMA0	0xE8
#define SAA7115_REG_TASKB_VERT_PHASE_OFFSET_CHROMA1	0xE9
#define SAA7115_REG_TASKB_VERT_PHASE_OFFSET_CHROMA2	0xEA
#define SAA7115_REG_TASKB_VERT_PHASE_OFFSET_CHROMA3	0xEB
#define SAA7115_REG_TASKB_VERT_PHASE_OFFSET_LUMA0	0xEC
#define SAA7115_REG_TASKB_VERT_PHASE_OFFSET_LUMA1	0xED
#define SAA7115_REG_TASKB_VERT_PHASE_OFFSET_LUMA2	0xEE
#define SAA7115_REG_TASKB_VERT_PHASE_OFFSET_LUMA3	0xEF


#define SAA7115_REG_TASKB_BEGIN					SAA7115_REG_TASKB_CONTROL



#endif //__SAA7115REGS_H__

