///////////////////////////////////////////////////////////////////////////////
//     
//
// Cthread.c
//
// Description:
//
//	Generic thread functions
//
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#include "cthread.h"
#include "qpdbgmgr.h"
#ifndef QP_LINXKER
#include <string.h>
#endif //!QP_LINXKER
#ifdef QP_LINXKER
#include <linux/version.h>
#endif //QP_LINXKER

extern TQP_DbgMCompID	g_QPCodecDBGCompID;


// thread proc
//
static void CThread_StaticThreadProc(void * Context)
{
	CThread	*pThread = (CThread *)Context;

	if (pThread->m_bRemoveUserSpaceMapping)
	{
		// daemonize Linux thread in the base thread proc
		// instead of scattered all over the place
		// currently only enabled for 331 based development
#ifdef QP_LINXKER
#if LINUX_VERSION_CODE < KERNEL_VERSION(3,7,0)
#ifdef __LINUX24__
		sprintf(current->comm, "%s", pThread->m_szThreadName);
		daemonize();
#else //!__LINUX24__
		daemonize(pThread->m_szThreadName);
		current->flags |= PF_NOFREEZE;
#endif //__LINUX24__
#endif // LINUX_VERSION_CODE < KERNEL_VERSION(3,7,0)
		//set_user_nice(current, -10);
#endif //QP_LINXKER
	}

	// thread function
	//
	pThread->m_ThreadProc(pThread);
}



static TQP_BOOL CThread_ThreadExit(CThread *This)
{
	EQP_Errors	qpStatus;
	TQP_UINT32	ulEvents;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CThread_ThreadExit()\n"
				 ));

	QPOSMClearEvtgrp(This->m_EvtReply,
					 THREAD_EVENT_EXIT_DONE
					 );
	QPOSMSetEvtgrp(This->m_EvtWait,
				   THREAD_EVENT_EXIT
				   );
	qpStatus = QPOSMWaitEvtgrp(This->m_EvtReply,
							   THREAD_EVENT_EXIT_DONE,
							   &ulEvents,
							   2000
							   );
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CThread_ThreadExit() Status(%d)\n",
				 qpStatus
				 ));

	return (QP_SUCCESS(qpStatus));
}



CThread	* CThread_Constructor(CThread *pThread,
							  char *Name,
							  TQP_PVOID context,
							  TQP_UINT32 priority,
							  TQP_OSMEvtgrpID evtWait,
							  TQP_OSMEvtgrpID evtReply,
							  TQP_BOOL bRemoveUserSpaceMapping
							  )
{
	if (pThread)
	{
		pThread->m_threadID = QP_NULL;

		pThread->m_context = context;
		pThread->m_priority = priority;
		pThread->m_EvtWait = evtWait;
		pThread->m_EvtReply = evtReply;
		pThread->m_bRemoveUserSpaceMapping = bRemoveUserSpaceMapping;

		strcpy(pThread->m_szThreadName,
			   Name
			   );

		pThread->ThreadExit = CThread_ThreadExit;
	}
	return (pThread);
}



void CThread_Destructor(CThread *This)
{
	// make sure the thread is destroyed
	//
	CThread_ThreadDone(This);
}



// Thread creation and clean up
//
TQP_BOOL CThread_ThreadInit(CThread *This)
{
	TQP_OSMThreadParms	threadParam;
	EQP_Errors			qpStatus;

	threadParam.procP = CThread_StaticThreadProc;
	threadParam.parm = (void *)This;
	threadParam.stackP = QP_NULL;
	threadParam.stackSize = 8192;
	threadParam.priority = This->m_priority;
	strcpy(threadParam.name,
		   This->m_szThreadName
		   );

	qpStatus = QPOSMCreateThread(&This->m_threadID,
								 &threadParam
								 );

	return (QP_SUCCESS(qpStatus));
}



TQP_BOOL CThread_ThreadDone(CThread *This)
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;


	if (This->m_threadID)
	{
		This->ThreadExit(This);

		// put minimum delay here just to be on the safe side.
		// OS manager already synchronize thread exit and
		// thread deletion as of Nov 13th, 2006
		//
		QPTMDelayMilliS(15);

		// call OS manager delete thread function
		//
		qpStatus = QPOSMDeleteThread(This->m_threadID);

		// nil the thread id regardless
		This->m_threadID = QP_NULL;

		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CThread_ThreadDone() QPOSMDeleteThread failed status(%d)!!!\n",
						 qpStatus
						 ));
		}
	}

	// succeed it anyway
	//
	return (QP_TRUE);
}



TQP_BOOL CThread_IsThreadInited(CThread *This)
{
	return (QP_NULL != This->m_threadID);
}





