///////////////////////////////////////////////////////////////////////////////
//        
//
// Util.c
//
// Description: 
//
//  Handy utilities
// 
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#ifdef _QP_WINKER
#include <wdm.h>
#endif //_QP_WINKER


#include "util.h"
#include "qposmgr.h"
#include "qptimemgr.h"
#include "qpmemmgr.h"
#include "qpdbgmgr.h"
#include "qpmpgcodec.h"
#ifndef QP_LINXKER
#include <string.h>
#else //QP_LINXKER
#include <linux/vmalloc.h>
#include <linux/pagemap.h>
#include <asm/page.h>
#include <asm/pgtable.h>
#endif //!QP_LINXKER

extern TQP_DbgMCompID	g_QPCodecDBGCompID;
static int s_alloc_count = 0;


void DelayMilliSeconds(TQP_UINT32 Milliseconds)
{
	QPTMDelayMilliS(Milliseconds);
}



void DelayMicroSeconds(TQP_UINT32 Microseconds)
{
	QPTMDelayMicroS(Microseconds);
}



TQP_INT64 GetSystemTimeUtil(void) 
{
	return (QPTMGetSysTimeMilliS());
}



TQP_INT64 GetSystemTimeUtilUSec(void)
{
	return (QPTMGetSysTimeMicroS());
}


// in msec
//
void BeginTimer(TQP_INT64 *pTimeto)
{
	TQP_INT64	curTime;

	curTime = GetSystemTimeUtil();
	*pTimeto += curTime;
}


TQP_BOOL Timeout(TQP_INT64 Timeto, 
				 TQP_UINT32 delay
				 )
{
	TQP_INT64	curTime;

	curTime = GetSystemTimeUtil();

	if (delay)
		DelayMilliSeconds(delay);

	return (curTime >= Timeto);
}



TQP_INT32 BytesSwap(TQP_UINT8 *pSource,	// Pointer to source data
					TQP_UINT8 *pTarget,	// Pointer to hold the target data, NULL if inplace swapping is required
					TQP_INT32 lSize		// Size of buffers in bytes.
					)
{
	TQP_INT32	lBytesSwapped = 0;
	TQP_INT32	lRemainder = 0;

	// Make it 4 bytes aligned.

	if ((lSize < 4 /* sizeof(int)*/) || 
		(QP_NULL == pSource)
		)
	{
		return (lBytesSwapped);
	}

	lRemainder = lSize % 4 /*sizeof(int)*/;
	lSize -= lRemainder;

	// We can use the bswap function for x86.
	if ((pSource == pTarget) || 
		(QP_NULL == pTarget)
		)
	{
#if defined(_X86_) && _X86_
		_asm
		{
			mov ecx, lSize;
			shr ecx, 2;
			mov esi, pSource;
			mov edi, pTarget;
loop1:
			lodsd
			bswap eax;
			stosd
			loop loop1
		}

		lBytesSwapped = lSize;
#else //!_X86_
		while (lBytesSwapped < lSize)
		{
			TQP_UINT8 ucTmp;
			ucTmp = (TQP_UINT8) *pSource;
		    *pSource = (TQP_UINT8) *(pSource + 3);
			*(pSource + 3) = ucTmp;

			ucTmp = *(pSource + 1);
			*(pSource + 1) = (TQP_UINT8) *(pSource + 2);
			*(pSource + 2) = ucTmp;

			pSource += 4 /*sizeof(int)*/;
			lBytesSwapped += 4 /*sizeof(int)*/;
		}
#endif //_X86_
	}
	else
	{
#if defined(_X86_) && _X86_
		_asm
		{
			mov ecx, lSize;
			shr ecx, 2;
			mov esi, pSource;
			mov edi, pTarget;
loop2:
			lodsd
			bswap eax;
			stosd
			loop loop2
		}

		lBytesSwapped = lSize;
#else //!_X86_
		while (lBytesSwapped < lSize)
		{
			*pTarget = *(pSource + 3);
			*(pTarget + 1) = *(pSource + 2);
			*(pTarget + 2) = *(pSource + 1);
			*(pTarget + 3) = *pSource;
			pSource += 4 /*sizeof(int)*/;
			pTarget += 4 /*sizeof(int)*/;
			lBytesSwapped += 4 /*sizeof(int)*/;
		}
#endif //_X86_
	}

	return (lBytesSwapped);
}


TQP_BOOL IsEqualTQPGUID(TQP_REFGUID rguid1, 
						TQP_REFGUID rguid2
						)
{ 
	return ((TQP_BOOL)!memcmp(rguid1, 
							  rguid2, 
							  sizeof(TQP_GUID)
							  )); 
}



void * QPMMMalloc2Ex(unsigned long bytes)
{
	void	*pBuf;

#ifdef _QP_WINKER
	pBuf = ExAllocatePoolWithTag(NonPagedPool, 
								 bytes,
								 'vrdq'
								 );
    if (pBuf)
	{
        RtlZeroMemory(pBuf, 
					  bytes
					  );
		s_alloc_count++;
	}
#else //!_QP_WINKER

	TQP_UINT32	memType = SetMemType(QPMM_ATTR_FALSE,QPMM_ATTR_ANY,QPMM_ATTR_ANY,QPMM_ALIGN_CACHE);

	pBuf = QPMMMalloc2(&memType, 
					   bytes
					   );
	if (pBuf)
	{
		memset(pBuf,
			   0,
			   bytes
			   );
		s_alloc_count++;
	}
#endif
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_EXTRACE),
				 "QPMMMalloc2Ex() ptr(0x%x) cnt(%d)\n", 
				 pBuf,
				 s_alloc_count
				 ));
	return (pBuf);
}



void QPMMFree2Ex(void * memP)
{
#ifdef _QP_WINKER
	if (memP)
	{
		s_alloc_count--;
		ExFreePoolWithTag(memP,
						  'vrdq'
						  );
	}
#else //!_QP_WINKER
	if (memP)
	{
		s_alloc_count--;
		QPMMFree2(memP);
	}
#endif //_QP_WINKER

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_EXTRACE),
				 "QPMMFree2Ex() ptr(0x%x) cnt(%d)\n",
				 memP,
				 s_alloc_count
				 ));
}



TQP_BOOL IsSafeToCall()
{
#ifdef _QP_WINKER
	return (KeGetCurrentIrql() < DISPATCH_LEVEL);
#else //!_QP_WINKER
	return (QP_TRUE);
#endif //_QP_WINKER
}


void * PLvmalloc(unsigned long bytes)
{
#ifdef QP_LINXKER
	return (vmalloc_32(bytes));
#else //!QP_LINXKER
	return (QPMMMalloc2Ex(bytes));
#endif //QP_LINXKER
}



void PLvfree(void * memP)
{
#ifdef QP_LINXKER
	vfree(memP);
#else //!QP_LINXKER
	QPMMFree2Ex(memP);
#endif //QP_LINXKER

}



