/*************************************************************************/
/**      **/
/**            **/
/**                **/
/**              **/
/**                                                                     **/
/**                               **/
/**                                                  **/
/*************************************************************************/

#include "qlautoconf.h"

#include <stddef.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <time.h>
#include <signal.h>
#ifndef __APPLE__
#include <mntent.h>
#include <asm/types.h>
#endif // __APPLE__
#include <sys/types.h>
#include <errno.h>
#include <arpa/inet.h>


#ifndef __APPLE__
#include <sys/socket.h>
#include <linux/socket.h>
#endif
#include <netinet/in.h>
#include <sys/un.h>
#include <sys/stat.h>

#include <pthread.h>
#include <search.h>

#ifdef CONFIG_ENABLE_TESTAPPS_SDL
#include <SDL/SDL.h>
#include <SDL/SDL_thread.h>
#endif

#include "qptypes.h"
#include "pldevioctl.h"
#include "qlutils.h"
#include "qlstrm.h"
#include "mp4_file.h"

#include "qlprofile.h"

#ifndef CONFIG_HOST_PLATFORM_USERSPACE
#include <linux/videodev2.h>
#include "uvcvideo.h"
#else
#include "userpl330.h"
#include "uservideodev2.h"

#define ioctl(dev, cmd, arg)  plv4l_ioctl(dev, cmd, arg)
#endif // CONFIG_HOST_PLATFORM_USERSPACE


#ifdef	SUPPORT_PL3XX_OSD
#include "qlosdfunc.h"
#endif

#define MODULE_NAME			"plstrm"
#define MODULE_VERSION		"1.0"

#define MAX_RETRY_COUNT		75

#define MAX_UDP_PACKET_LEN	1472
#define MAX_TCP_PACKET_LEN	1460

#define	SUPPORT_HLS_FILE

#ifndef CONFIG_HOST_PLATFORM_USERSPACE
#define CONFIG_ENABLE_MOTION_DETECT
#define CONFIG_ENABLE_PL_AUDIO_AGC
#else
#undef CONFIG_ENABLE_MOTION_DETECT
#undef CONFIG_ENABLE_PL_AUDIO_AGC
#endif

#ifdef CONFIG_ENABLE_MOTION_DETECT
#include "qlmd.h"
#define	MD_CFG_FILE		"./md.cfg"
PTQLMDInfo		pMDInfo=NULL;
#endif
#ifdef CONFIG_ENABLE_PL_AUDIO_AGC
#include "qlvagc.h"
#define	VAGC_CFG_FILE		"./vagc.cfg"
PTQLVAGCInfo	pVAGCInfo=NULL;
#endif

#define MP4_PTS_RESET_TO_ZERO
#define MP4FILE_MAX_LEN  	255

#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
#define	MP4_TMP_FILE		"/tmp/mp4.tmp"
#define O_DIRECT_BUFFER_SIZE		((512+64)*1024)
#define O_DIRECT_BUFFER_THRESHOLD	(512*1024)
#endif

#if defined (__APPLE__)
#define FILEPOS_ASSIGN(x,y)   { x = y; }
#else
#define FILEPOS_ASSIGN(x,y)   { x.__pos = y; }
#endif

extern int do_set(DEVHANDLE hDev,
			char *pszItem,
			char *pszValue
			);
extern int qlconf(component_t *pComponent,
		   TQP_OBJECT_HANDLE hTaskUpStream,
		   char *scrfilename
		   );

struct qlapp_v4l_buf_info
{
	char				*addr;
	unsigned int		size;

	// for userptr
	struct v4l2_buffer	stV4lBuf;
	int					sent;
};

#ifndef max
#define max(a,b)        (((a) < (b)) ? (b) : (a))
#endif

#ifdef	SUPPORT_HLS_FILE
int compar (const void *a,const void *b)
{
	return (strcasecmp((char *) a, (char *) b));
}
#endif

volatile unsigned int g_unCtrlCReceived = 0;
volatile unsigned int g_unKillReceived = 0;
unsigned int g_unEnableLoop = 0;
unsigned int g_unEnableLoopCnt = 0;
unsigned int g_unEnableLoopNeedCnt = 0;
unsigned int g_unEnableLoopDelay = 0;
unsigned int g_unNoUserInput = 0;
unsigned int g_unReducePrintf = 0;
unsigned int g_unUseODirect = 0;


static uint64_t getTimeUs(void) {
	struct timeval tv;
	uint64_t ret;
	gettimeofday(&tv, NULL);
	ret = (uint64_t)tv.tv_sec * 1000000 + tv.tv_usec;
	return ret;
}


static void mb2rasframe(
    unsigned char *outputbuf_y,
    unsigned char *outputbuf_u,
    unsigned char *outputbuf_v,
    unsigned char *y_outbuf,
    unsigned char *c_outbuf,
    int H, int V,
    int ByteSwapFlag,
    unsigned int output_buffer_size)
{
    int             xmb, ymb, i, j;
    int             LSize, CSize;
    unsigned int    uiTemp;
    unsigned char   *buf;
    unsigned int    OutTotalSizeFrame;
    unsigned char   *y_start,*c_start;
    int             xmbcnt, ymbcnt;

    y_start = y_outbuf;
    c_start = c_outbuf;

    // for Y

    xmbcnt = H / 16;
    ymbcnt = V / 16;

    OutTotalSizeFrame = 0;

    ////////////////////////////////////////////// Luma
    for (ymb = 0; ymb < ymbcnt; ymb++) {
        for (xmb = 0; xmb < xmbcnt; xmb++) {
            for (j = 0; j < 16; j++) {
                buf = outputbuf_y + H * (ymb * 16 + j) + xmb * 16;

                memmove(buf, y_outbuf, 16);
                y_outbuf += 16;

                if (ByteSwapFlag) {
                    // do byte swapping
                    for (i = 0; i < 4; i++) {
#ifdef _X86_
                        _asm {
                            mov esi, buf;
                            mov edi, buf;
                            lodsd
                            bswap eax;
                            stosd
                        }
                        buf += 4;
#else //!_X86_
                        uiTemp =
                            (buf[i * 4] & 0xff) |
                            ((buf[i *4 + 1] << 8) & 0xff00) |
                            ((buf[i * 4 + 2] << 16) & 0xff0000) |
                            ((buf[i * 4 + 3] << 24) & 0xff000000);

                        buf[i * 4] = (uiTemp >> 24) & 0xff;
                        buf[i * 4 + 1] = (uiTemp >> 16) & 0xff;
                        buf[i * 4 + 2] = (uiTemp >> 8) & 0xff;
                        buf[i * 4 + 3] = (uiTemp)&0xff;
#endif //_X86_
                    }
                }
                OutTotalSizeFrame += 16;
            }
        }
    }


    ////////////////////////////////////////////// Chroma
    LSize = H * V;
    CSize = H * V / 4;

    for (ymb = 0; ymb < ymbcnt; ymb++) {
        for(xmb = 0; xmb < xmbcnt; xmb++) {
            for(j = 0; j < 8; j++) {
#if 1
                buf = outputbuf_v + ((H / 2) * (ymb * 8 + j)) + (xmb * 8);
#else
                buf = outputbuf_u + ((H / 2) * (ymb * 8 + j)) + (xmb * 8);
#endif
                memmove(buf, c_outbuf, 8);
                c_outbuf += 8;

                if (ByteSwapFlag) {
                    // do byte swapping
                    for (i = 0; i < 2; i++) {
#ifdef _X86_
                        _asm {
                            mov esi, buf;
                            mov edi, buf;
                            lodsd
                            bswap eax;
                            stosd
                        }
                        buf += 4;
#else //!_X86_
                        uiTemp =
                            (buf[i * 4] & 0xff) |
                            ((buf[i * 4 + 1] << 8) & 0xff00) |
                            ((buf[i * 4 + 2] << 16) & 0xff0000) |
                            ((buf[i * 4 + 3] << 24) & 0xff000000);

                        buf[i * 4] = (uiTemp >> 24) & 0xff;
                        buf[i * 4 + 1] = (uiTemp >> 16) & 0xff;
                        buf[i * 4 + 2] = (uiTemp >> 8) & 0xff;
                        buf[i * 4 + 3] = (uiTemp) & 0xff;
#endif //_X86_
                    }
                }
                OutTotalSizeFrame += 8;
            }

            for (j = 0; j < 8; j++) {
#if 1
                buf = outputbuf_u + ((H / 2) * (ymb * 8 + j)) + (xmb * 8);
#else
                buf = outputbuf_v + ((H / 2) * (ymb * 8 + j)) + (xmb * 8);
#endif
                memmove(buf, c_outbuf, 8);
                c_outbuf += 8;

                if (ByteSwapFlag) {
                    // do byte swapping
                    for (i = 0; i < 2; i++) {
#ifdef _X86_
                        _asm {
                            mov esi, buf;
                            mov edi, buf;
                            lodsd
                            bswap eax;
                            stosd
                        }

                        buf += 4;
#else //!_X86_
                        uiTemp =
                            (buf[i * 4] & 0xff) |
                            ((buf[i * 4 + 1] << 8) & 0xff00) |
                            ((buf[i * 4 + 2] << 16) & 0xff0000) |
                            ((buf[i * 4 + 3] << 24) & 0xff000000);

                        buf[i * 4] = (uiTemp >> 24) & 0xff;
                        buf[i * 4 + 1] = (uiTemp >> 16) & 0xff;
                        buf[i * 4 + 2] = (uiTemp >> 8) & 0xff;
                        buf[i * 4 + 3] = (uiTemp) & 0xff;
#endif //_X86_
                    }
                }
                OutTotalSizeFrame += 8;
            }
        }
    }
}


#ifdef CONFIG_ENABLE_TESTAPPS_SDL
static unsigned int		unSdlVideoFlags =  SDL_ANYFORMAT | SDL_DOUBLEBUF | SDL_RESIZABLE;
static pthread_mutex_t	g_sdlSurfaceLock = PTHREAD_MUTEX_INITIALIZER;
static SDL_Surface		*g_psdlSurface = NULL;
static SDL_Overlay		*g_psdlOverlay = NULL;


static int Plstream_CreateOverlay(unsigned int ulWidth,
								  unsigned int ulHeight
								  )
{
	int	err = 0;

	pthread_mutex_lock(&g_sdlSurfaceLock);

	if (!g_psdlSurface)
	{
		g_psdlSurface = SDL_SetVideoMode(ulWidth, ulHeight, 0, unSdlVideoFlags);
		if (!g_psdlSurface)
		{
			fprintf(stderr, "SDL: could not set video mode - exiting\n");
			err = 1;
			goto Plstream_CreateOverlay_Exit;
		}
	}
	if (!g_psdlOverlay)
	{
		g_psdlOverlay = SDL_CreateYUVOverlay(ulWidth, ulHeight, SDL_YV12_OVERLAY, g_psdlSurface);
		if (!g_psdlOverlay)
		{
			fprintf(stderr, "SDL: could not create yuv overlay - exiting\n");
			err = 1;
			goto Plstream_CreateOverlay_Exit;
		}
	}

Plstream_CreateOverlay_Exit:

	pthread_mutex_unlock(&g_sdlSurfaceLock);
	return (err);
}


#define _DISPLAY_ALL		0
#define _DISPLAY_TOP		1
#define _DISPLAY_BOTTOM		2

static void Plstream_DisplayFrame(unsigned int ulWidth,
								  unsigned int ulHeight,
								  unsigned char *pBuf,
								  int displayMode,
								  unsigned int dataType
								  )
{
	SDL_Rect		sdlRect;
	unsigned char	*pucaYBuff;
	unsigned char	*pucaUBuff;
	unsigned char	*pucaVBuff;

	if (!g_unCtrlCReceived)
	{
		pucaYBuff = (unsigned char *)pBuf;
		pucaUBuff = (unsigned char *)(pBuf + ulWidth * ulHeight);
		pucaVBuff = (unsigned char *)(pBuf + ((ulWidth * ulHeight * 5) / 4));

		pthread_mutex_lock(&g_sdlSurfaceLock);

		if (!SDL_LockYUVOverlay(g_psdlOverlay))
		{
			if (QP_YUV_DATA_TYPE_QYUV == dataType)
			{
				mb2rasframe((void *)g_psdlOverlay->pixels[0],
							(void *)g_psdlOverlay->pixels[1],
							(void *)g_psdlOverlay->pixels[2],
							(void*)pucaYBuff,
							(void*)pucaUBuff,
							ulWidth,
							ulHeight,
							1,
							(ulWidth * ulHeight * 3) / 2
							);
			}
			else
			{
				memcpy(g_psdlOverlay->pixels[0], pucaYBuff, ulWidth * ulHeight);
				memcpy(g_psdlOverlay->pixels[1], pucaUBuff, (ulWidth * ulHeight) / 4);
				memcpy(g_psdlOverlay->pixels[2], pucaVBuff, (ulWidth * ulHeight) / 4);
			}

			SDL_UnlockYUVOverlay(g_psdlOverlay);

			switch (displayMode)
			{
				case _DISPLAY_TOP:
					sdlRect.x = 0;
					sdlRect.y = 0;
					sdlRect.w = ulWidth;
					sdlRect.h = ulHeight / 2;
					break;

				case _DISPLAY_BOTTOM:
					sdlRect.x = 0;
					sdlRect.y = ulHeight / 2;
					sdlRect.w = ulWidth;
					sdlRect.h = ulHeight / 2;
					break;

				case _DISPLAY_ALL:
				default:
					sdlRect.x = 0;
					sdlRect.y = 0;
					sdlRect.w = ulWidth;
					sdlRect.h = ulHeight;
					break;
			}

			SDL_DisplayYUVOverlay(g_psdlOverlay, &sdlRect);
		}
		else
		{
			printf("%s() SDL_LockYUVOverlay() failed %d %s\n", __FUNCTION__, errno, strerror(errno));
		}

		pthread_mutex_unlock(&g_sdlSurfaceLock);
	}
}

#endif //CONFIG_ENABLE_TESTAPPS_SDL


#ifdef ENABLE_APP_LINUX_PROFILING
static int enable_fw_profiling(int dev)
{
    QPCODEC_DIAG_MEMORY_STRUCT qlmemrw;
    #define FWPROFILING_BUF_ENABLE_ADDRESS 0x2f10ac

	if (dev > 0)
	{
		int err;

		qlmemrw.address = FWPROFILING_BUF_ENABLE_ADDRESS;
		qlmemrw.num_dwords = 1;
		qlmemrw.value[0] = 1;

		err = ioctl(dev, PLDEV_DIAG_IOCTL_MEM_WRITE, &qlmemrw);
		if (!err)
			return 0;
	}
	return 1;
}
#endif


static void SigIntHanlder(int Signal)
{
    /*signal(SIGALRM, Signal);*/
    g_unCtrlCReceived = 1;
    return;
}

static void SigStopHanlder(int Signal)
{
    printf("%s()\n", __FUNCTION__);
    return;
}

static void SigContHanlder(int Signal)
{
    printf("%s()\n", __FUNCTION__);
    return;
}


static void SigPipeHanlder(int Signal)
{
    printf("%s()\n", __FUNCTION__);
    return;
}

static void SigTermHanlder(int Signal)
{
    printf("%s()\n", __FUNCTION__);
    g_unCtrlCReceived = 1;
    g_unKillReceived = 1;
    return;
}

void changemode(int dir)
{
	static struct termios	oldt, newt;

	if (dir == 1)
	{
		tcgetattr(STDIN_FILENO, &oldt);
		newt = oldt;
		newt.c_lflag &= ~(ICANON | ECHO);
		tcsetattr(STDIN_FILENO, TCSANOW, &newt);
	}
	else
	{
		tcsetattr(STDIN_FILENO, TCSANOW, &oldt);
	}
}



int kbhit(void)
{
	if (g_unNoUserInput) {
		return 0;
	} else {
		struct timeval	tv;
		fd_set			rdfs;

		tv.tv_sec = 0;
		tv.tv_usec = 0;

		FD_ZERO(&rdfs);
		FD_SET(STDIN_FILENO, &rdfs);

		select(STDIN_FILENO + 1, &rdfs, NULL, NULL, &tv);
		return FD_ISSET(STDIN_FILENO, &rdfs);
	}
}


void usage(int nExit)
{
    printf("\n");
    printf("  [%s version %s]\n", MODULE_NAME, MODULE_VERSION);
    printf("  -------------------------------------------------\n");
    printf("  %s <command1> <arg1> <arg2> ... [+ <command2> <arg1> <arg2> ...] \n", MODULE_NAME);
    printf("\n");
    printf("  stream command,\n");
    printf("    enc                     => encode.\n");
    printf("    dec                     => decode.\n");
    printf("\n");
    printf("  arg for configuration,\n");
    printf("    config <file_name>      => configuration script file.\n");
    printf("\n");
    printf("  arg for bus selection,\n");
    printf("    bus                     => bus type. Could be 0(USB)/1(PCIe).\n");
    printf("\n");
    printf("  arg for card selection,\n");
    printf("    card                    => Card number. Could be 0/1/2/3, default is 0.\n");
    printf("\n");
    printf("  arg for stream media,\n");
    printf("    ifile <file_name>       => input from file.\n");
    printf("    iudp                    => input from UDP.\n");
    printf("    itcp                    => input from TCP.\n");
    printf("    idgram <server_path>      => input from Unix domain socket (SOCK_DGRAM).\n");
    printf("    istream <server_path>     => input from Unix domain socket (SOCK_STREAM).\n");
    printf("    iuvcraw <device_node>   => input from UVC RAW device.\n");
    printf("    iuvcmjpeg <device_node> => input from UVC MJPEG device.\n");
    printf("    ofile <file_name>       => output to file.\n");
    printf("    doutfile <file_name>    => dump output stream to file.\n");
    printf("    dinfile <file_name>     => dump input  stream to file.\n");
	printf("    omp4file <file_name>    => output as mp4 file. \n");
    printf("    oudp <udp_addr>         => output to udp.\n");
    printf("    otcp <tcp_addr>         => output to tcp. \n");
    printf("    odgram <server_path> <client_path>   => output to Unix domain socket (SOCK_DGRAM).\n");
    printf("    ostream <server_path> <client_path>  => output to Unix domain socket (SOCK_STREAM).\n");
    printf("    opack                   => Unix domain socket output with 4 byte header.\n");
    printf("    ipack                   => Unix domain socket input with 4 byte header.\n");
    printf("    osdl <display_mode>     => output to SDL. 0: full window, 1: top, 2: bottom\n");
    printf("    null                    => output to null.\n");
    printf("    iport <port_num>        => input port number for TCP/UDP streaming.\n");
    printf("    oport <port_num>        => output port number for TCP/UDP streaming.\n");
    printf("    uvcfps <fps>            => define uvc frame rate.\n");
    printf("    uvcpreview              => enable uvc preview.\n");
    printf("    separate                => save each jpeg frame to one file.\n");
    printf("    loop                    => repeat send source if input from file.\n");
    printf("    osd <1/0>               => turn on/off osd in encoding mode.\n");
    printf("    limit <size in KBytes>  => maximum output file size\n");
	printf("    saveoriginal			=> save original ps file for mp4\n");
	printf("    runloop	<n> <delay in sec> => loop running plstrm <n> times, n=0 means infinitely until got SIGTERM\n");
#ifdef ENABLE_APP_LINUX_PROFILING
	printf("    applog <file_name>      => enable app time log and save to file\n");
#else
	printf("    applog <file_name>      => dummy function, not enable profiling support\n");
#endif
    printf("\n");
    printf("  examples, \n");
    printf("    %s enc ofile output.mpg\n", MODULE_NAME);
    printf("    %s card 1 dec ifile input.mpg\n", MODULE_NAME);
    printf("\n");

    if (nExit) {
        exit(nExit);
    }
}

void ScaleUp960x720To1280x720(char* pcDes,char* pcSrc)
{
    int nI,nJ,nXOffset;
    unsigned short usTemp;

    for(nI=0;nI<720;nI++) {
        nXOffset= 0;
        for (nJ=0;nJ<960*2;nJ+=12) {
            *(pcDes+nXOffset+1280*2*nI)= *(pcSrc+nI*960*2+nJ);
            *(pcDes+nXOffset+1280*2*nI+1)= *(pcSrc+nI*960*2+nJ+1);
            usTemp= *(pcSrc+nI*960*2+nJ+2);
            *(pcDes+nXOffset+1280*2*nI+2)= (unsigned char)usTemp;
            *(pcDes+nXOffset+1280*2*nI+3)= *(pcSrc+nI*960*2+nJ+3);
            usTemp= *(pcSrc+nI*960*2+nJ+2);
            *(pcDes+nXOffset+1280*2*nI+4)= (unsigned char)usTemp;
            *(pcDes+nXOffset+1280*2*nI+5)= *(pcSrc+nI*960*2+nJ+5);
            *(pcDes+nXOffset+1280*2*nI+6)= *(pcSrc+nI*960*2+nJ+4);
            *(pcDes+nXOffset+1280*2*nI+7)= *(pcSrc+nI*960*2+nJ+3);

            *(pcDes+nXOffset+1280*2*nI+8)= *(pcSrc+nI*960*2+nJ+6);
            *(pcDes+nXOffset+1280*2*nI+9)= *(pcSrc+nI*960*2+nJ+9);
            usTemp= *(pcSrc+nI*960*2+nJ+8);
            *(pcDes+nXOffset+1280*2*nI+10)= usTemp;
            *(pcDes+nXOffset+1280*2*nI+11)= *(pcSrc+nI*960*2+nJ+7);
            usTemp= *(pcSrc+nI*960*2+nJ+8);
            *(pcDes+nXOffset+1280*2*nI+12)= usTemp;
            *(pcDes+nXOffset+1280*2*nI+13)= *(pcSrc+nI*960*2+nJ+9);
            *(pcDes+nXOffset+1280*2*nI+14)= *(pcSrc+nI*960*2+nJ+10);
            *(pcDes+nXOffset+1280*2*nI+15)= *(pcSrc+nI*960*2+nJ+11);

            nXOffset+= 16;
        }
    }
}


void YUV422ProgTo420Plane(char* pcDes,char* pcSrc,int nWidth,int nHeight)
{
    int nI, nJ;
    int nVOffset = nWidth*nHeight;
    int nUOffset = nWidth*nHeight+(nWidth*nHeight>>2);

    for(nI=0;nI<nHeight;nI++) {
        for(nJ=0;nJ<nWidth*2;nJ+=4) {
            //Y Plane
            *(pcDes+nI*nWidth+(nJ>>1))= *(pcSrc+nI*nWidth*2+nJ+0);
            *(pcDes+nI*nWidth+(nJ>>1)+1)= *(pcSrc+nI*nWidth*2+nJ+2);
            //U Plane
            if (!(nI&0x01))
                *(pcDes+nUOffset+(nJ>>2)+(nI>>1)*(nWidth>>1))= *(pcSrc+nI*nWidth*2+nJ+1);
            //V Plane
            if (!(nI&0x01))
                *(pcDes+nVOffset+(nJ>>2)+(nI>>1)*(nWidth>>1))= *(pcSrc+nI*nWidth*2+nJ+3);
        }
    }
}

char *generate_filename_byindex(char *oriname, int id)
{
	char *outname;
	int  i,idx;
	char str[16];
	idx = (int) strlen(oriname);
	outname = (char *) malloc(idx+60);
	memset(outname, 0, idx+60);
	for (i=idx-1;i>=0;i--)
	{
		if (oriname[i]=='.')
			break;
	}
	if (i>0)
		strncpy(outname, oriname, i);
	else
		strcpy(outname, oriname);
	snprintf(str, 16, ".%05d", id);
	strcat(outname, str);
    if (i>0)
		strncat(outname, &oriname[i], idx-i);
	return outname;
}

/////////////////////////////////////////////////////////////////////////////
//
//
void fix_data_packet(unsigned char *pBuf, uint32_t *pbyteused, unsigned int packetno, component_t * pComponent)
{
	if ((COMPONENT_TYPE_ENCODE == pComponent->devType) &&
		(COMPONENT_PORT_COMP_AUD_OUT == pComponent->ports[STREAM_DIR_OUT].portType) &&
		(AUDIO_TYPE_SILK == pComponent->audControl.audio_type))
	{
		int done = 0, nHeaderLen = 0, nFrameLen, nPaddingBytes;
		if (packetno==1)
		{
			nHeaderLen = 9;
		}
		while (!done) {
			nFrameLen = pBuf[nHeaderLen] + pBuf[nHeaderLen+1]*256;
			nPaddingBytes = *pbyteused  - nHeaderLen - nFrameLen - 2;
			if (nPaddingBytes < 0)
			{
				/* this is not a full frame, it shell never happen */
				done = 1;
				fprintf(stderr,"silk data error: packet too short (%u)(%u)(%u)(%d)\n",*pbyteused, pBuf[nHeaderLen],pBuf[nHeaderLen+1],nFrameLen);
			}
			else if (nPaddingBytes <=3 )
			{
				/* this is a single frame, 0-3 with padding bytes */
				*pbyteused -= nPaddingBytes;
				done = 1;
				//printf("silk data with nPaddingBytes=%d len=%d\n", nPaddingBytes, *pbyteused);
			}
			else {	 /* nPaddingBytes > 3, there have another frame(s) */
				nHeaderLen = nHeaderLen + nFrameLen + 2;
				//printf("silk data with nPaddingBytes=%d still %d\n", nPaddingBytes%4, nPaddingBytes);
			 	if (nPaddingBytes>0)
				{	/* move data */
					memcpy(&pBuf[nHeaderLen],
							&pBuf[nHeaderLen + nPaddingBytes%4],
							nPaddingBytes - nPaddingBytes%4);
					*pbyteused -= nPaddingBytes%4;
				}
				//printf("len=%d\n", *pbyteused);
			}
		}
	}
}


int check_stream_header(unsigned char *pBuf, uint32_t byteused, component_t * pComponent)
{
    //printf("Check header %d %d %02x %02x %02x (%02x %02x %02x %02x) \n", pComponent->sysControl.Artesa.stream_type, ARTESA_STREAM_TYPE_TRANSPORT, pBuf[0],pBuf[1],pBuf[2],pBuf[14],pBuf[15],pBuf[16],pBuf[17]);
	switch (pComponent->sysControl.Artesa.stream_type)
	{
		case ARTESA_STREAM_TYPE_PROGRAM:
			{
				if (byteused>18)
				{
					if (
					   /*(pBuf[0]==0x00) &&
						(pBuf[1]==0x00)	&&
						(pBuf[2]==0x01) &&
						(pBuf[3]==0xBA)	&& */
						(pBuf[14]==0x00) &&
						(pBuf[15]==0x00) &&
						(pBuf[16]==0x01) &&
						(pBuf[17]==0xBB)
						)
					{  	/* This is a System header */
						return 0;
					}
				}
			}
			break;
		case ARTESA_STREAM_TYPE_TRANSPORT:
			{
				int pos=0;
				while ( (pos+3) < byteused)
				{
					if (
						(pBuf[pos]==0x47) &&
						(pBuf[pos+1]==0x40)	&&
						(pBuf[pos+2]==0x00)
						)
					{  	/* This is a PAT table */
						return pos;
					}
					else
						pos+= 188;
				}
			}
			break;
		case ARTESA_STREAM_TYPE_ELEMENTARY:
			{
				if (byteused>5)
				{
					char NAL_TYPE;
					NAL_TYPE = pBuf[4] & 0x1F;
					/* check is IDR or I frame */
					if ((NAL_TYPE==7) || (NAL_TYPE==8) || (NAL_TYPE==5))
					{  	/* This is a IDR frame for our chip */
						return 0;
					}
				}
			}
			break;
	}
	return -1;
}

QT_TRANSCODE_INFO * do_mp4_localinit(FILE ** pfpOutput,
                FILE ** pfdMp4vstts, FILE ** pfdMp4vstsz, FILE ** pfdMp4vstco, FILE ** pfdMp4vstsc,
                FILE ** pfdMp4astts, FILE ** pfdMp4astsz, FILE ** pfdMp4astco, FILE ** pfdMp4astsc,
                component_t *pComponent, ps_mux_t *ppsState, char *filename, int FirstStream)
{
	QT_TRANSCODE_INFO *pqtinfo;
	char *FNamemp4;

	*pfpOutput = fopen(filename, "wb+");
	if (*pfpOutput == NULL)
	{
		printf("%s() Unable to open mp4 file %s.\n", __FUNCTION__, filename);
		return NULL;
	}


#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
	filename = MP4_TMP_FILE;
#endif

	FNamemp4 = (char *) malloc(strlen(filename)+10);
	if (FNamemp4==NULL)
	{
		printf("%s() LINE %d: no memory\n", __FUNCTION__, __LINE__);
		return NULL;
	}
	strcpy(FNamemp4, filename);
	strcat(FNamemp4, "vss");
	*pfdMp4vstts = fopen( FNamemp4, "w+" );
	if( *pfdMp4vstts == NULL )
	{
	    printf("Unable to open file %s.\n",FNamemp4);
		free(FNamemp4);
		return NULL;
	}
	strcpy(FNamemp4, filename);
	strcat(FNamemp4, "vsz");
	*pfdMp4vstsz = fopen( FNamemp4, "w+" );
	if( *pfdMp4vstsz == NULL )
	{
	    printf("Unable to open file %s.\n",FNamemp4);
		free(FNamemp4);
		return NULL;
	}
	strcpy(FNamemp4, filename);
	strcat(FNamemp4, "vco");
	*pfdMp4vstco = fopen( FNamemp4, "w+" );
	if( *pfdMp4vstco == NULL )
	{
	    printf("Unable to open file %s.\n",FNamemp4);
		free(FNamemp4);
		return NULL;
	}
	strcpy(FNamemp4, filename);
	strcat(FNamemp4, "vsc");
	*pfdMp4vstsc = fopen( FNamemp4, "w+" );
	if( *pfdMp4vstsc == NULL )
	{
	    printf("Unable to open file %s.\n",FNamemp4);
		free(FNamemp4);
		return NULL;
	}

//#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
//	filename = MP4_TMP_FILE;
//#endif

	strcpy(FNamemp4, filename);
	strcat(FNamemp4, "ass");
	*pfdMp4astts = fopen( FNamemp4, "w+" );
	if( *pfdMp4astts == NULL )
	{
	    printf("Unable to open file %s.\n",FNamemp4);
		free(FNamemp4);
		return NULL;
	}
	strcpy(FNamemp4, filename);
	strcat(FNamemp4, "asz");
	*pfdMp4astsz = fopen( FNamemp4, "w+" );
	if( *pfdMp4astsz == NULL )
	{
	    printf("Unable to open file %s.\n",FNamemp4);
		free(FNamemp4);
		return NULL;
	}
	strcpy(FNamemp4, filename);
	strcat(FNamemp4, "aco");
	*pfdMp4astco = fopen( FNamemp4, "w+" );
	if( *pfdMp4astco == NULL )
	{
	    printf("Unable to open file %s.\n",FNamemp4);
		free(FNamemp4);
		return NULL;
	}
	strcpy(FNamemp4, filename);
	strcat(FNamemp4, "asc");
	*pfdMp4astsc = fopen( FNamemp4, "w+" );
	if( *pfdMp4astsc == NULL )
	{
	    printf("Unable to open file %s.\n",FNamemp4);
		free(FNamemp4);
		return NULL;
	}
	pqtinfo = mp4_init(*pfpOutput, *pfdMp4vstts, *pfdMp4vstsz, *pfdMp4vstco, *pfdMp4vstsc, *pfdMp4astts, *pfdMp4astsz, *pfdMp4astco, *pfdMp4astsc, pComponent);
	if (pqtinfo==NULL)
	{
	    printf("Unable to do mp4 init\n");
		free(FNamemp4);
		return NULL;
	}
	//pqtinfo = pqtinfo_static;
	memset(ppsState, 0, sizeof(ps_mux_t));
	ppsState->Startaddr = (unsigned char *)malloc( PS_BUF_SIZE );
	ppsState->Endaddr = ppsState->Startaddr+PS_BUF_SIZE;
	ppsState->temp = 0;
	ppsState->mp4State = 0;
	ppsState->overLen = 0;
	ppsState->nOffset = 0;
	ppsState->nPESHeadLen = 0;
	ppsState->nLength = 0;
	ppsState->LenVideoFrame = 0;
	ppsState->LenAudioFrame = 0;
	ppsState->RdCnt = 0;
	ppsState->RdPtr= ppsState->Startaddr;
	ppsState->WrCnt = 0;
	ppsState->WrPtr = ppsState->Startaddr;
	ppsState->aBuff = (unsigned char *)malloc( AUDIO_BUF_SIZE );
	ppsState->aWrPtr = ppsState->aBuff;
#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
	ppsState->vBuff = (unsigned char *)malloc( VIDEO_BUF_SIZE );
	ppsState->vWrPtr = ppsState->vBuff;
#endif
	pqtinfo->acfg.i_pts_start = 0;
	pqtinfo->acfg.i_pts = 0;
	pqtinfo->vcfg.i_pts = 0;

    ppsState->nAudioCount = 0;
    ppsState->nVideoCount = 0;
    ppsState->rCnt = 0;
	ppsState->nFirstStream = FirstStream;
	ppsState->pts_offset = 0;
	ppsState->ptsa_done = 0;

	if ((ppsState->Startaddr==NULL)||(ppsState->aBuff==NULL)
#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
		||(ppsState->vBuff==NULL)
#endif
		)
	{
		printf("%s() LINE %d: no memory\n", __FUNCTION__, __LINE__);
		if (ppsState->Startaddr)
		{
			free(ppsState->Startaddr);
			ppsState->Startaddr = NULL;
		}
		if (ppsState->aBuff)
		{
			free(ppsState->aBuff);
			ppsState->aBuff = NULL;
            ppsState->aWrPtr = NULL;
		}
#if defined(CONFIG_HOST_PLATFORM_PL330SOC)		
		if (ppsState->vBuff)
		{
			free(ppsState->vBuff);
			ppsState->vBuff = NULL;
		}
#endif
		free(pqtinfo);
		free(FNamemp4);
		return NULL;
	}
	free(FNamemp4);
	return pqtinfo;
}

int do_mp4_process_data(unsigned char * pBuf, unsigned int Buflen, component_t *pComponent, ps_mux_t *ppsState, QT_TRANSCODE_INFO *pqtinfo, unsigned char *CacheSps)
{
    unsigned char	caBuffer[32];
	fpos_t filepos;

	if (!pqtinfo)
		return 1;
	if((PS_BUF_SIZE+ppsState->RdCnt-ppsState->WrCnt)>Buflen)
	{
		ppsState->nLength = (unsigned int)(ppsState->Endaddr - ppsState->WrPtr);
		if(ppsState->nLength<Buflen)
		{
			memcpy((char*)ppsState->WrPtr, (char*)pBuf, ppsState->nLength);
			pBuf += ppsState->nLength;
			ppsState->nLength = Buflen-ppsState->nLength;
			ppsState->WrPtr = ppsState->Startaddr;
			memcpy((char*)ppsState->WrPtr, (char*)pBuf, ppsState->nLength);
			ppsState->WrPtr += ppsState->nLength;
			ppsState->WrCnt += Buflen;
		}
		else
		{
#if 1
			if ((ppsState->nVideoCount==0)&&(ppsState->WrCnt==0))
			{
			 	//printf("====>process first frame \n");
				if (ppsState->nFirstStream)
				{
					memcpy(CacheSps, &pBuf[0x4B], 0x70);
					memcpy((char*)ppsState->WrPtr, (char*)pBuf, Buflen);
					ppsState->WrPtr += Buflen;
					ppsState->WrCnt += Buflen;
				}
				else {
					int peslen;
					peslen = ((int)(pBuf[0x3C])*256) + (int)(pBuf[0x3D]);
					peslen += 0x70;
					pBuf[0x3C] = ( peslen >> 8 ) &0xff;
					pBuf[0x3D] = peslen & 0xff;
					memcpy(ppsState->WrPtr, pBuf, 0x4B);
					ppsState->WrPtr += 0x4B;
					memcpy(ppsState->WrPtr, CacheSps, 0x70);
					ppsState->WrPtr += 0x70;
					memcpy(ppsState->WrPtr, &pBuf[0x4b], Buflen - 0x4b);
					ppsState->WrPtr += (Buflen - 0x4b);
					ppsState->WrCnt = ppsState->WrCnt + Buflen + 0x70;
				}
			}
			else
#endif
			{
				memcpy((char*)ppsState->WrPtr, (char*)pBuf, Buflen);
				ppsState->WrPtr += Buflen;
				ppsState->WrCnt += Buflen;
			}
		}
	}
	else
	{
		printf("%s: buffer overflow RdCnt=%d, WrCnt=%d, PS_BUF_SIZE=%d, new_frame_len=%d, ppsState->mp4State=0x%x\n",
			__FUNCTION__, ppsState->RdCnt, ppsState->WrCnt, PS_BUF_SIZE, Buflen, ppsState->mp4State);
	}

	switch(ppsState->mp4State)
	{
		case 0xC1:
			if((ppsState->WrCnt-ppsState->RdCnt)<ppsState->overLen)
			{
				break;
			}
			ppsState->mp4State = 0x00;
			ppsState->nLength = ppsState->overLen;
			if(ppsState->WrPtr>ppsState->RdPtr)
			{
				//fwrite(ppsState->RdPtr, 1, ppsState->nLength, pqtinfo->mp4_handle /*fpOutput*/);
				memcpy((char*)ppsState->aWrPtr, (char*)ppsState->RdPtr, ppsState->nLength);
				ppsState->RdPtr += ppsState->nLength;
				ppsState->aWrPtr += ppsState->nLength;
			}
			else
			{
				ppsState->nOffset = (unsigned int) (ppsState->Endaddr-ppsState->RdPtr);
				if(ppsState->nLength>ppsState->nOffset)
				{
					//fwrite(ppsState->RdPtr, 1, ppsState->nOffset, pqtinfo->mp4_handle /*fpOutput*/);
					memcpy((char*)ppsState->aWrPtr, (char*)ppsState->RdPtr, ppsState->nOffset);
					ppsState->aWrPtr += ppsState->nOffset;
					ppsState->nOffset = ppsState->nLength-ppsState->nOffset;
					ppsState->RdPtr = ppsState->Startaddr;
					//fwrite(ppsState->RdPtr, 1, ppsState->nOffset, pqtinfo->mp4_handle /*fpOutput*/);
					memcpy((char*)ppsState->aWrPtr, (char*)ppsState->RdPtr, ppsState->nOffset);
					ppsState->RdPtr += ppsState->nOffset;
					ppsState->aWrPtr += ppsState->nOffset;
				}
				else
				{
					//fwrite(ppsState->RdPtr, 1, ppsState->nLength, pqtinfo->mp4_handle /*fpOutput*/);
					memcpy((char*)ppsState->aWrPtr, (char*)ppsState->RdPtr, ppsState->nLength);
					ppsState->RdPtr += ppsState->nLength;
					ppsState->aWrPtr += ppsState->nLength;
				}
			}
			if(ppsState->RdPtr >= ppsState->Endaddr)
			{
				ppsState->temp = (unsigned int)(ppsState->RdPtr - ppsState->Endaddr);
				ppsState->RdPtr = ppsState->Startaddr + ppsState->temp;
			}
			ppsState->RdCnt += ppsState->nLength;
		break;
		case 0xE1:
			if((ppsState->WrCnt-ppsState->RdCnt)<ppsState->overLen)
			{
				break;
			}
			ppsState->mp4State = 0x00;
			ppsState->nLength = ppsState->overLen;
			if(ppsState->WrPtr>ppsState->RdPtr)
			{
#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
				fwrite(ppsState->RdPtr, 1, ppsState->nLength, pqtinfo->mp4_handle/*fpOutput*/);
#else
				memcpy((char*)ppsState->vWrPtr, (char*)ppsState->RdPtr, ppsState->nLength);
				ppsState->vWrPtr += ppsState->nLength;
#endif
				ppsState->RdPtr += ppsState->nLength;
			}
			else
			{
				ppsState->nOffset = (unsigned int) (ppsState->Endaddr-ppsState->RdPtr);
				if(ppsState->nLength>ppsState->nOffset)
				{
#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
					fwrite(ppsState->RdPtr, 1, ppsState->nOffset, pqtinfo->mp4_handle/*fpOutput*/);
#else
					memcpy((char*)ppsState->vWrPtr, (char*)ppsState->RdPtr, ppsState->nOffset);
					ppsState->vWrPtr += ppsState->nOffset;
#endif
					ppsState->nOffset = ppsState->nLength-ppsState->nOffset;
					ppsState->RdPtr = ppsState->Startaddr;
#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
					fwrite(ppsState->RdPtr, 1, ppsState->nOffset, pqtinfo->mp4_handle/*fpOutput*/);
#else
					memcpy((char*)ppsState->vWrPtr, (char*)ppsState->RdPtr, ppsState->nOffset);
					ppsState->vWrPtr += ppsState->nOffset;
#endif
					ppsState->RdPtr += ppsState->nOffset;
				}
				else
				{
#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
					fwrite(ppsState->RdPtr, 1, ppsState->nLength, pqtinfo->mp4_handle/*fpOutput*/);
#else
					memcpy((char*)ppsState->vWrPtr, (char*)ppsState->RdPtr, ppsState->nLength);
					ppsState->vWrPtr += ppsState->nLength;
#endif
					ppsState->RdPtr += ppsState->nLength;
				}
			}
			if(ppsState->RdPtr >= ppsState->Endaddr)
			{
				ppsState->temp = (unsigned int) (ppsState->RdPtr - ppsState->Endaddr);
				ppsState->RdPtr = ppsState->Startaddr + ppsState->temp;
			}
			ppsState->RdCnt += ppsState->nLength;
		break;
	}

	if(ppsState->mp4State == 0x00)
	{
		while((ppsState->RdCnt+0x40)<ppsState->WrCnt)
		{
			ppsState->RdPtr0 = ppsState->RdPtr;
			for(ppsState->temp = 0; ppsState->temp<4; ppsState->temp++)
			{
				caBuffer[ppsState->temp] = *ppsState->RdPtr0++;
				if(ppsState->RdPtr0 >= ppsState->Endaddr)
					ppsState->RdPtr0 = ppsState->Startaddr;
			}
			//printf("Pos:%x, Type:%x\n", ppsState->RdCnt, caBuffer[3]);
			if ((caBuffer[0] == 0x0) || (caBuffer[1] == 0x0) || (caBuffer[2] == 0x1))
			{
				switch (caBuffer[3])
				{
					case 0xBA:
						for(ppsState->temp = 0; ppsState->temp<10; ppsState->temp++)
						{
							caBuffer[ppsState->temp] = *ppsState->RdPtr0++;
							if(ppsState->RdPtr0 >= ppsState->Endaddr)
							{
								ppsState->RdPtr0 = ppsState->Startaddr;
							}
						}
						ppsState->nLength = (unsigned int)(caBuffer[9] & 0x7);
						//printf("BA Pos:%x, Len: %x\n", ppsState->RdCnt, caBuffer[9]);
						ppsState->RdPtr += (14+ppsState->nLength);
						if(ppsState->RdPtr >= ppsState->Endaddr)
						{
							ppsState->temp = (unsigned int) (ppsState->RdPtr - ppsState->Endaddr);
							ppsState->RdPtr = ppsState->Startaddr + ppsState->temp;
						}
						ppsState->RdCnt += (14+ppsState->nLength);
					break;
					case 0xBB:
					case 0xBC:
					case 0xBE:
						//printf("BB Pos:%x\n", ppsState->RdCnt);
						for(ppsState->temp = 0; ppsState->temp<2; ppsState->temp++)
						{
							caBuffer[ppsState->temp] = *ppsState->RdPtr0++;
							if(ppsState->RdPtr0 >= ppsState->Endaddr)
								ppsState->RdPtr0 = ppsState->Startaddr;
						}
						ppsState->nLength = ((int)(caBuffer[0])*256) + (int)(caBuffer[1]);
						ppsState->RdPtr += (6+ppsState->nLength);
						if(ppsState->RdPtr >= ppsState->Endaddr)
						{
							ppsState->temp = (unsigned int) (ppsState->RdPtr - ppsState->Endaddr);
							ppsState->RdPtr = ppsState->Startaddr + ppsState->temp;
						}
						ppsState->RdCnt += (6+ppsState->nLength);
					break;
					case 0xC0:
						//printf("C0 Pos:%x\n", ppsState->RdCnt);
						for(ppsState->temp = 0; ppsState->temp<10; ppsState->temp++)
						{
							caBuffer[ppsState->temp] = *ppsState->RdPtr0++;
							if(ppsState->RdPtr0 >= ppsState->Endaddr)
								ppsState->RdPtr0 = ppsState->Startaddr;
						}
						ppsState->nLength = ((int)(caBuffer[0])*256) + (int)(caBuffer[1]);
						ppsState->nPESHeadLen = (int)(caBuffer[4]);
						ppsState->nLength -= (3+ppsState->nPESHeadLen);
						if ((caBuffer[2] & 0x04) == 0x04)
						{
							ppsState->temp = (caBuffer[5]&0x0e)<<29;
							ppsState->temp |= (caBuffer[6]&0xff)<<22;
							ppsState->temp |= (caBuffer[7]&0xfe)<<14;
							ppsState->temp |= (caBuffer[8]&0xff)<<7;
							ppsState->temp |= (caBuffer[9]>>1)&0x7f;
							pqtinfo->acfg.i_pts = ppsState->temp;
#ifdef MP4_PTS_RESET_TO_ZERO
							if (ppsState->nFirstStream!=1)
							{
								if (ppsState->ptsa_done==0)
								{
									if (((int) ppsState->pts_offset - (int) ppsState->temp ) > 0)
									{
										ppsState->temp = 0;
									}
									else { /* temp > offset, take care pts overrun case */
										ppsState->ptsa_done = 1;
										ppsState->temp = ppsState->temp - ppsState->pts_offset;
									}
								}
								else {
									ppsState->temp = ppsState->temp - ppsState->pts_offset;
								}
							}
#endif
							if(ppsState->nAudioCount ==0)
							{
							    pqtinfo->acfg.i_pts_start = ppsState->temp;
								pqtinfo->acfg.i_pts_back = pqtinfo->acfg.i_pts;
							}
							if(pqtinfo->dVideo == 1)
							{
								pqtinfo->dVideo = 0;
								if(ppsState->LenVideoFrame)
								{
									if(ppsState->nVideoCount == 1)
									{
									    ppsState->LenVideoFrame -= 0x70;
									}
									//printf("A Pos: %x\n", pqtinfo->file_offset-ppsState->LenVideoFrame);
#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
									/* update frame length, and output to file */
									if(ppsState->nVideoCount == 1) {
										qlput4byte( ppsState->vBuff+0x70, ppsState->LenVideoFrame-4);
									} else {
										qlput4byte( ppsState->vBuff, ppsState->LenVideoFrame-4);
									}
									fwrite(ppsState->vBuff, 1, ppsState->vWrPtr-ppsState->vBuff, pqtinfo->mp4_handle/*fpOutput*/);
									ppsState->vWrPtr = ppsState->vBuff;
#else
									rewind(pqtinfo->mp4_handle );
									FILEPOS_ASSIGN(filepos, pqtinfo->file_offset-ppsState->LenVideoFrame);
									//if(fseek( (void *)pqtinfo->mp4_handle, pqtinfo->file_offset-ppsState->LenVideoFrame, SEEK_SET ))
									if(fsetpos(pqtinfo->mp4_handle, &filepos))
									{
									    printf( "cannot seek ttt:%lx, %x\n", pqtinfo->file_offset, ppsState->LenVideoFrame );
									}

									qlput4byte( &caBuffer[0], ppsState->LenVideoFrame-4);
									fwrite(caBuffer, 1, 4, pqtinfo->mp4_handle/*fpOutput*/);

									rewind(pqtinfo->mp4_handle );
									FILEPOS_ASSIGN(filepos, pqtinfo->file_offset);
									if(fsetpos(pqtinfo->mp4_handle, &filepos))
									{
									    printf( "cannot seek ttt2:%lx, %x\n", pqtinfo->file_offset, ppsState->LenVideoFrame );
									}
#endif

									//fseek( (void *)pqtinfo->mp4_handle, pqtinfo->file_offset, SEEK_SET );
									qlput4byte( &pqtinfo->vstszp[pqtinfo->nvstsz], ppsState->LenVideoFrame);
									ppsState->LenVideoFrame = 0;
									pqtinfo->nvstsz += 4;
									if( pqtinfo->nvstsz == QMP_S_STSZP )
									{
									    fwrite( pqtinfo->vstszp, 1, pqtinfo->nvstsz, (void *)pqtinfo->fp_vstszp);
									    pqtinfo->nvstszp += pqtinfo->nvstsz;
									    pqtinfo->nvstsz = 0;
									}
									pqtinfo->vcfg.frames++;
									pqtinfo->vstsc ++;
								}
								if(pqtinfo->vstsc)
								{
									qlput4byte( &pqtinfo->vstscp[pqtinfo->nvstsc], pqtinfo->vChunkCnt);
									pqtinfo->nvstsc += 4;
									qlput4byte( &pqtinfo->vstscp[pqtinfo->nvstsc], pqtinfo->vstsc );
									pqtinfo->nvstsc += 4;
									qlput4byte( &pqtinfo->vstscp[pqtinfo->nvstsc], 1 );
									pqtinfo->nvstsc += 4;
									pqtinfo->vstsc = 0;
									if( pqtinfo->nvstsc == QMP_S_STSCP )
									{
										fwrite( pqtinfo->vstscp, 1, pqtinfo->nvstsc, (void *)pqtinfo->fp_vstscp);
										pqtinfo->nvstscp += pqtinfo->nvstsc;
										pqtinfo->nvstsc = 0;
									}
								}

								//printf("Audio offset: %x\n", pqtinfo->file_offset);
								ppsState->aWrPtr = ppsState->aBuff;
							}
							if(ppsState->LenAudioFrame>7)
							{
								ppsState->nOffset = 0;
								ppsState->aFrameCnt = 0;
								for(ppsState->temp = 0x04; ppsState->temp<(ppsState->LenAudioFrame-3); ppsState->temp++)
								{
									if((ppsState->aBuff[ppsState->temp] == 0xff) && (ppsState->aBuff[ppsState->temp+1] == 0xf9)
									    && ((ppsState->aBuff[ppsState->temp+2] == 0x60)||(ppsState->aBuff[ppsState->temp+2] == 0x54)||
									        (ppsState->aBuff[ppsState->temp+2] == 0x50)||(ppsState->aBuff[ppsState->temp+2] == 0x4c))
									    && ((ppsState->aBuff[ppsState->temp+3] == 0x80)||(ppsState->aBuff[ppsState->temp+3] == 0x40)))
									{
										ppsState->aFrameSize[0] = ppsState->temp - ppsState->nOffset-7;
									}
									else
									{
										if (ppsState->temp==(ppsState->LenAudioFrame-4))
										{
											/* last byte, set FrameSize */
											ppsState->aFrameSize[0] = ppsState->LenAudioFrame-ppsState->nOffset-7;
										}
										else
											continue;
									}
									/* output audio data */
									{
									    qlput4byte( &pqtinfo->astcop[pqtinfo->nastco], (TQP_UINT32) pqtinfo->file_offset );
									    pqtinfo->nastco += 4;
									    if( pqtinfo->nastco == QMP_S_STCOP )
									    {
									        fwrite( pqtinfo->astcop, 1, pqtinfo->nastco, (void *)pqtinfo->fp_astcop);
									        pqtinfo->nastcop += pqtinfo->nastco;
									        pqtinfo->nastco = 0;
									    }
									    pqtinfo->aChunkCnt++;

									    fwrite((char *)&ppsState->aBuff[ppsState->nOffset+7], 1, ppsState->aFrameSize[0], pqtinfo->mp4_handle/*fpOutput*/);
									    pqtinfo->file_offset += ppsState->aFrameSize[0];
									    qlput4byte( &pqtinfo->astszp[pqtinfo->nastsz], ppsState->aFrameSize[0]);
									    pqtinfo->nastsz += 4;
									    if( pqtinfo->nastsz == QMP_S_STSZP )
									    {
									        fwrite( pqtinfo->astszp, 1, pqtinfo->nastsz, (void *)pqtinfo->fp_astszp);
									        pqtinfo->nastszp += pqtinfo->nastsz;
									        pqtinfo->nastsz = 0;
									    }
									    pqtinfo->acfg.frames ++;
									    pqtinfo->astsc ++;

									    qlput4byte( &pqtinfo->astscp[pqtinfo->nastsc], pqtinfo->aChunkCnt);
									    pqtinfo->nastsc += 4;
									    qlput4byte( &pqtinfo->astscp[pqtinfo->nastsc], pqtinfo->astsc );
									    pqtinfo->nastsc += 4;
									    qlput4byte( &pqtinfo->astscp[pqtinfo->nastsc], 1 );
									    pqtinfo->nastsc += 4;
									    pqtinfo->astsc = 0;
									    if( pqtinfo->nastsc == QMP_S_STSCP )
									    {
									        fwrite( pqtinfo->astscp, 1, pqtinfo->nastsc, (void *)pqtinfo->fp_astscp);
									        pqtinfo->nastscp += pqtinfo->nastsc;
									        pqtinfo->nastsc = 0;
									    }
									}

									ppsState->nOffset = ppsState->temp;
									ppsState->aFrameCnt++;
								}

						     	//printf("[aFrameCnt(C0):%d, ppsState->LenAudioFrame=%d]\n",ppsState->aFrameCnt,ppsState->LenAudioFrame);
								ppsState->LenAudioFrame = 0;
								ppsState->aWrPtr = ppsState->aBuff;
							}
							else if(ppsState->LenAudioFrame)
							{
								printf("mp4: err: LenAudioFrame=%d\n",ppsState->LenAudioFrame);
							}

							if(ppsState->nAudioCount !=0)
							{
								if(ppsState->aFrameCnt >0)
								{
									for(ppsState->temp = 0x01; ppsState->temp<ppsState->aFrameCnt; ppsState->temp++)
									{
										qlput4byte( &pqtinfo->asttsp[pqtinfo->nastts], 1);
							    		pqtinfo->nastts += 4;
							    		qlput4byte( &pqtinfo->asttsp[pqtinfo->nastts], 0x400);
							    		pqtinfo->nastts += 4;
							    		pqtinfo->acfg.i_pts_back += (QT_TIME_SCALE*1024/pqtinfo->acfg.samplingrate);
							    		if( pqtinfo->nastts == QMP_S_STTSP )
							    		{
							    			fwrite( pqtinfo->asttsp, 1, pqtinfo->nastts, (void *)pqtinfo->fp_asttsp);
							    			pqtinfo->nasttsp += pqtinfo->nastts;
							    			pqtinfo->nastts = 0;
							    		}
									}
									qlput4byte( &pqtinfo->asttsp[pqtinfo->nastts], 1);
							    	pqtinfo->nastts += 4;
									ppsState->nOffset = ((pqtinfo->acfg.i_pts-pqtinfo->acfg.i_pts_back)*pqtinfo->acfg.samplingrate)/QT_TIME_SCALE;
									if(ppsState->nOffset < 0x3FF)
										ppsState->nOffset = 0x400;
#if 0
									if(ppsState->nOffset > 0x410)
										printf("Audio Pos: %x\n", ppsState->nOffset);
#endif
							    	qlput4byte( &pqtinfo->asttsp[pqtinfo->nastts], ppsState->nOffset);
							    	pqtinfo->nastts += 4;
							    	pqtinfo->acfg.i_pts_back = pqtinfo->acfg.i_pts;
							    	if( pqtinfo->nastts == QMP_S_STTSP )
							    	{
							    		fwrite( pqtinfo->asttsp, 1, pqtinfo->nastts, (void *)pqtinfo->fp_asttsp);
							    		pqtinfo->nasttsp += pqtinfo->nastts;
							    		pqtinfo->nastts = 0;
							    	}
								}
							}
							ppsState->nAudioCount ++;
							pqtinfo->dAudio = 1;
						    //printf("Audio Pos: %x\n", pqtinfo->file_offset);
						}
						ppsState->LenAudioFrame += ppsState->nLength;
						ppsState->RdPtr += (9+ppsState->nPESHeadLen);
						ppsState->RdCnt += (9+ppsState->nPESHeadLen);
						if(pqtinfo->acfg.i_pts == 0)
						{
						    ppsState->LenAudioFrame = 0;
						    ppsState->nAudioCount = 0;
						    ppsState->aWrPtr = ppsState->aBuff;
						    ppsState->RdPtr += ppsState->nLength;
						}
						else
						{
							if(ppsState->RdPtr >= ppsState->Endaddr)
							{
							    ppsState->temp = (unsigned int) (ppsState->RdPtr - ppsState->Endaddr);
							    ppsState->RdPtr = ppsState->Startaddr + ppsState->temp;
							}
							if((ppsState->WrCnt-ppsState->RdCnt)<ppsState->nLength)
							{
							    ppsState->overLen = ppsState->nLength;
							    ppsState->mp4State = 0xC1;
							    break;
							}
							if(ppsState->WrPtr>ppsState->RdPtr)
							{
							    //fwrite(ppsState->RdPtr, 1, ppsState->nLength, pqtinfo->mp4_handle /*fpOutput*/);
							    memcpy((char*)ppsState->aWrPtr, (char*)ppsState->RdPtr, ppsState->nLength);
							    ppsState->RdPtr += ppsState->nLength;
							    ppsState->aWrPtr += ppsState->nLength;
							}
							else
							{
							    ppsState->nOffset = (unsigned int) (ppsState->Endaddr -ppsState->RdPtr);
							    if(ppsState->nLength > ppsState->nOffset)
							    {
							        //fwrite(ppsState->RdPtr, 1, ppsState->nOffset, pqtinfo->mp4_handle /*fpOutput*/);
							        memcpy((char*)ppsState->aWrPtr, (char*)ppsState->RdPtr, ppsState->nOffset);
							        ppsState->aWrPtr += ppsState->nOffset;
							        ppsState->nOffset = ppsState->nLength -ppsState->nOffset;
							        ppsState->RdPtr = ppsState->Startaddr;
							        //fwrite(ppsState->RdPtr, 1, ppsState->nOffset, pqtinfo->mp4_handle /*fpOutput*/);
							        memcpy((char*)ppsState->aWrPtr, (char*)ppsState->RdPtr, ppsState->nOffset);
							        ppsState->RdPtr += ppsState->nOffset;
							        ppsState->aWrPtr += ppsState->nOffset;
							    }
							    else
							    {
							        //fwrite(ppsState->RdPtr, 1, ppsState->nLength, pqtinfo->mp4_handle /*fpOutput*/);
							        memcpy((char*)ppsState->aWrPtr, (char*)ppsState->RdPtr, ppsState->nLength);
							        ppsState->RdPtr += ppsState->nLength;
							        ppsState->aWrPtr += ppsState->nLength;
							    }
							}
						}
						if(ppsState->RdPtr >= ppsState->Endaddr)
						{
						    ppsState->temp = (unsigned int) (ppsState->RdPtr - ppsState->Endaddr);
						    ppsState->RdPtr = ppsState->Startaddr + ppsState->temp;
						}
						ppsState->RdCnt += ppsState->nLength;
					break;
					case 0xE0:
						//printf("E0 Pos:%x\n", ppsState->RdCnt);
						for(ppsState->temp = 0; ppsState->temp<10; ppsState->temp++)
						{
						    caBuffer[ppsState->temp] = *ppsState->RdPtr0++;
						    if(ppsState->RdPtr0 >= ppsState->Endaddr)
						    {
						        ppsState->RdPtr0 = ppsState->Startaddr;
						    }
						}
						ppsState->nLength = ((int)(caBuffer[0])*256) + (int)(caBuffer[1]);
						ppsState->nPESHeadLen = (int)(caBuffer[4]);
						ppsState->nLength -= (3+ppsState->nPESHeadLen);

						if ((caBuffer[2] & 0x04) == 0x04) /* data_alignment_indicator */
						{
							ppsState->temp = (caBuffer[5]&0x0e)<<29;
							ppsState->temp |= (caBuffer[6]&0xff)<<22;
							ppsState->temp |= (caBuffer[7]&0xfe)<<14;
							ppsState->temp |= (caBuffer[8]&0xff)<<7;
							ppsState->temp |= (caBuffer[9]>>1)&0x7f;
#ifdef MP4_PTS_RESET_TO_ZERO
							if (ppsState->nFirstStream!=1)
							{
								if(ppsState->nVideoCount ==0)
								{
									ppsState->pts_offset = ppsState->temp;
								}
								ppsState->temp = ppsState->temp - ppsState->pts_offset;
							}
#endif
							if(ppsState->nVideoCount ==0)
							{
								qlput4byte( &pqtinfo->vstcop[pqtinfo->nvstco], (TQP_UINT32) (pqtinfo->file_offset+0x70) );
								pqtinfo->nvstco = 4;
								pqtinfo->vChunkCnt = 1;
							}
							else
							{
							    qlput4byte( &pqtinfo->vsttsp[pqtinfo->nvstts], 1);
							    pqtinfo->nvstts += 4;
							    qlput4byte( &pqtinfo->vsttsp[pqtinfo->nvstts], (ppsState->temp-pqtinfo->vcfg.i_pts) );
							    pqtinfo->nvstts += 4;
							    pqtinfo->vcfg.i_pts = ppsState->temp;
							    if( pqtinfo->nvstts == QMP_S_STTSP )
							    {
							        fwrite( pqtinfo->vsttsp, 1, pqtinfo->nvstts, (void *)pqtinfo->fp_vsttsp);
							        pqtinfo->nvsttsp += pqtinfo->nvstts;
							        pqtinfo->nvstts = 0;
							    }
							}
							pqtinfo->dVideo = 1;
							if(pqtinfo->dAudio)
							{
								pqtinfo->dAudio = 0;
								if(ppsState->LenAudioFrame>7)
								{
									ppsState->nOffset = 0;
									ppsState->aFrameCnt = 0;
									for(ppsState->temp = 0x04; ppsState->temp<(ppsState->LenAudioFrame-3); ppsState->temp++)
									{
										if((ppsState->aBuff[ppsState->temp] == 0xff) && (ppsState->aBuff[ppsState->temp+1] == 0xf9)
									    	&& ((ppsState->aBuff[ppsState->temp+2] == 0x60)||(ppsState->aBuff[ppsState->temp+2] == 0x54)||
									        	(ppsState->aBuff[ppsState->temp+2] == 0x50)||(ppsState->aBuff[ppsState->temp+2] == 0x4c))
									    	&& ((ppsState->aBuff[ppsState->temp+3] == 0x80)||(ppsState->aBuff[ppsState->temp+3] == 0x40)))
										{
											ppsState->aFrameSize[0] = ppsState->temp - ppsState->nOffset-7;
										}
										else
										{
											if (ppsState->temp==(ppsState->LenAudioFrame-4))
											{
												/* last byte, set FrameSize */
												ppsState->aFrameSize[0] = ppsState->LenAudioFrame-ppsState->nOffset-7;
											}
											else
												continue;
										}
										/* output audio data */
										{
										    qlput4byte( &pqtinfo->astcop[pqtinfo->nastco], (TQP_UINT32) pqtinfo->file_offset );
										    pqtinfo->nastco += 4;
										    if( pqtinfo->nastco == QMP_S_STCOP )
										    {
										        fwrite( pqtinfo->astcop, 1, pqtinfo->nastco, (void *)pqtinfo->fp_astcop);
										        pqtinfo->nastcop += pqtinfo->nastco;
										        pqtinfo->nastco = 0;
										    }
										    pqtinfo->aChunkCnt++;

										    fwrite((char *)&ppsState->aBuff[ppsState->nOffset+7], 1, ppsState->aFrameSize[0], pqtinfo->mp4_handle /*fpOutput*/);
										    pqtinfo->file_offset += ppsState->aFrameSize[0];
										    qlput4byte( &pqtinfo->astszp[pqtinfo->nastsz], ppsState->aFrameSize[0]);
										    pqtinfo->nastsz += 4;
										    if( pqtinfo->nastsz == QMP_S_STSZP )
										    {
										        fwrite( pqtinfo->astszp, 1, pqtinfo->nastsz, (void *)pqtinfo->fp_astszp);
										        pqtinfo->nastszp += pqtinfo->nastsz;
										        pqtinfo->nastsz = 0;
										    }
										    pqtinfo->acfg.frames ++;
										    pqtinfo->astsc ++;

										    qlput4byte( &pqtinfo->astscp[pqtinfo->nastsc], pqtinfo->aChunkCnt);
										    pqtinfo->nastsc += 4;
										    qlput4byte( &pqtinfo->astscp[pqtinfo->nastsc], pqtinfo->astsc );
										    pqtinfo->nastsc += 4;
										    qlput4byte( &pqtinfo->astscp[pqtinfo->nastsc], 1 );
										    pqtinfo->nastsc += 4;
										    pqtinfo->astsc = 0;
										    if( pqtinfo->nastsc == QMP_S_STSCP )
										    {
										        fwrite( pqtinfo->astscp, 1, pqtinfo->nastsc, (void *)pqtinfo->fp_astscp);
										        pqtinfo->nastscp += pqtinfo->nastsc;
										        pqtinfo->nastsc = 0;
										    }
										}

										ppsState->nOffset = ppsState->temp;
										ppsState->aFrameCnt++;
									}

								     	//printf("[aFrameCnt(E0):%d, ppsState->LenAudioFrame=%d]\n",ppsState->aFrameCnt,ppsState->LenAudioFrame);
									ppsState->LenAudioFrame = 0;
									ppsState->aWrPtr = ppsState->aBuff;
								}
								else if(ppsState->LenAudioFrame) {
									printf("mp4: err: LenAudioFrame=%d\n",ppsState->LenAudioFrame);
								}

								//printf("Video offset: %x\n", pqtinfo->file_offset);
								qlput4byte( &pqtinfo->vstcop[pqtinfo->nvstco], (TQP_UINT32) pqtinfo->file_offset );
								pqtinfo->nvstco += 4;
								if( pqtinfo->nvstco == QMP_S_STCOP )
								{
									fwrite( pqtinfo->vstcop, 1, pqtinfo->nvstco, (void *)pqtinfo->fp_vstcop);
									pqtinfo->nvstcop += pqtinfo->nvstco;
									pqtinfo->nvstco = 0;
								}
								pqtinfo->vChunkCnt++;
							}
							//printf("Video Pos: %x\n", pqtinfo->file_offset);

							if(ppsState->LenVideoFrame)
							{
								if(ppsState->nVideoCount == 1)
								{
									ppsState->LenVideoFrame -= 0x70;
								}

								//printf("V Pos: %x\n", pqtinfo->file_offset-ppsState->LenVideoFrame);

#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
								/* update frame length, and output to file */
								if(ppsState->nVideoCount == 1) {
									qlput4byte( ppsState->vBuff+0x70, ppsState->LenVideoFrame-4);
								} else {
									qlput4byte( ppsState->vBuff, ppsState->LenVideoFrame-4);
								}
								fwrite(ppsState->vBuff, 1, ppsState->vWrPtr-ppsState->vBuff, pqtinfo->mp4_handle/*fpOutput*/);
								ppsState->vWrPtr = ppsState->vBuff;
#else
								//if(fseek( (void *)pqtinfo->mp4_handle, pqtinfo->file_offset-ppsState->LenVideoFrame, SEEK_SET ))
								rewind( (void *)pqtinfo->mp4_handle );
								FILEPOS_ASSIGN(filepos, pqtinfo->file_offset - ppsState->LenVideoFrame);
								if(fsetpos((void *)pqtinfo->mp4_handle, &filepos))
								{
									printf( "cannot seek ttt\n" );
								}
								qlput4byte( &caBuffer[0], ppsState->LenVideoFrame-4);
								fwrite(caBuffer, 1, 4, pqtinfo->mp4_handle /*fpOutput*/);
								rewind(pqtinfo->mp4_handle );
								FILEPOS_ASSIGN(filepos, pqtinfo->file_offset);
								if(fsetpos(pqtinfo->mp4_handle, &filepos))
								{
								    printf( "cannot seek ttt2(end):%lx, %x\n", pqtinfo->file_offset, ppsState->LenVideoFrame );
								}
#endif

								//fseek( (void *)pqtinfo->mp4_handle, pqtinfo->file_offset, SEEK_SET );
								qlput4byte( &pqtinfo->vstszp[pqtinfo->nvstsz], ppsState->LenVideoFrame);
								ppsState->LenVideoFrame = 0;
								pqtinfo->nvstsz += 4;
								if( pqtinfo->nvstsz == QMP_S_STSZP )
								{
									fwrite( pqtinfo->vstszp, 1, pqtinfo->nvstsz, (void *)pqtinfo->fp_vstszp);
									pqtinfo->nvstszp += pqtinfo->nvstsz;
									pqtinfo->nvstsz = 0;
								}
								pqtinfo->vcfg.frames++;
								pqtinfo->vstsc ++;
							}
							ppsState->nVideoCount ++;
						}
						pqtinfo->file_offset += ppsState->nLength;
						ppsState->LenVideoFrame += ppsState->nLength;
						ppsState->RdPtr += (9+ppsState->nPESHeadLen);
						if(ppsState->RdPtr >= ppsState->Endaddr)
						{
							ppsState->temp = (unsigned int) (ppsState->RdPtr - ppsState->Endaddr);
							ppsState->RdPtr = ppsState->Startaddr + ppsState->temp;
						}
						ppsState->RdCnt += (9+ppsState->nPESHeadLen);
						if((ppsState->WrCnt-ppsState->RdCnt)<ppsState->nLength)
						{
							ppsState->overLen = ppsState->nLength;
							ppsState->mp4State = 0xE1;
							break;
						}
						if(ppsState->WrPtr>ppsState->RdPtr)
						{
#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
							fwrite(ppsState->RdPtr, 1, ppsState->nLength, pqtinfo->mp4_handle /*fpOutput*/);
#else
							memcpy((char*)ppsState->vWrPtr, (char*)ppsState->RdPtr, ppsState->nLength);
							ppsState->vWrPtr += ppsState->nLength;
#endif
							ppsState->RdPtr += ppsState->nLength;
						}
						else
						{
							ppsState->nOffset = (unsigned int) (ppsState->Endaddr -ppsState->RdPtr);
							if(ppsState->nLength > ppsState->nOffset)
							{
#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
								fwrite(ppsState->RdPtr, 1, ppsState->nOffset, pqtinfo->mp4_handle /*fpOutput*/);
#else
								memcpy((char*)ppsState->vWrPtr, (char*)ppsState->RdPtr, ppsState->nOffset);
								ppsState->vWrPtr += ppsState->nOffset;
#endif
								ppsState->nOffset = ppsState->nLength -ppsState->nOffset;
								ppsState->RdPtr = ppsState->Startaddr;
#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
								fwrite(ppsState->RdPtr, 1, ppsState->nOffset, pqtinfo->mp4_handle /*fpOutput*/);
#else
								memcpy((char*)ppsState->vWrPtr, (char*)ppsState->RdPtr, ppsState->nOffset);
								ppsState->vWrPtr += ppsState->nOffset;
#endif
								ppsState->RdPtr += ppsState->nOffset;
							}
							else
							{
#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
								fwrite(ppsState->RdPtr, 1, ppsState->nLength, pqtinfo->mp4_handle /*fpOutput*/);
#else
								memcpy((char*)ppsState->vWrPtr, (char*)ppsState->RdPtr, ppsState->nLength);
								ppsState->vWrPtr += ppsState->nLength;
#endif
								ppsState->RdPtr += ppsState->nLength;
							}
						}
						if(ppsState->RdPtr >= ppsState->Endaddr)
						{
							ppsState->temp = (unsigned int) (ppsState->RdPtr - ppsState->Endaddr);
							ppsState->RdPtr = ppsState->Startaddr + ppsState->temp;
						}
						ppsState->RdCnt += ppsState->nLength;
						break;
					default:
						ppsState->RdPtr += 4;
						if(ppsState->RdPtr >= ppsState->Endaddr)
						{
							ppsState->temp = (unsigned int) (ppsState->RdPtr - ppsState->Endaddr);
							ppsState->RdPtr = ppsState->Startaddr + ppsState->temp;
						}
						ppsState->RdCnt += 4;
					break;
				}
			}
			else
			{
				ppsState->RdPtr += 4;
				printf("RdCnt: %x, WrCnt: %x, End: %p Len: %x (%x %x %x %x)\n",
					ppsState->RdCnt, ppsState->WrCnt, ppsState->Endaddr, ppsState->nLength, caBuffer[0],caBuffer[1],caBuffer[2],caBuffer[3]);
				if(ppsState->RdPtr >= ppsState->Endaddr)
				{
					ppsState->temp = (unsigned int) (ppsState->RdPtr - ppsState->Endaddr);
					ppsState->RdPtr = ppsState->Startaddr + ppsState->temp;
				}
				ppsState->RdCnt += 4;
				ppsState->rCnt = 0xffffffff;
				break;
			}
			if(ppsState->mp4State)
			{
				//printf("mp4State:%x, ppsState->overLen:%x\n", ppsState->mp4State, ppsState->overLen);
				break;
			}
		}
	}
	return 0;
}


int do_mp4_localfinal(FILE ** pfpOutput,
                FILE ** pfdMp4vstts, FILE ** pfdMp4vstsz, FILE ** pfdMp4vstco, FILE ** pfdMp4vstsc,
                FILE ** pfdMp4astts, FILE ** pfdMp4astsz, FILE ** pfdMp4astco, FILE ** pfdMp4astsc,
                component_t *pComponent, ps_mux_t *ppsState, QT_TRANSCODE_INFO *pqtinfo, char *filename)
{
	char *FNamemp4;

    if (!pqtinfo)
		return 1;

#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
	filename = MP4_TMP_FILE;
#endif

	FNamemp4 = (char *) malloc(strlen(filename)+10);
	if (FNamemp4==NULL)
	{
		printf("%s() LINE %d: no memory\n", __FUNCTION__, __LINE__);
		return 1;
	}

    if (ppsState->LenAudioFrame>7)
    {
        ppsState->nOffset = 0;
        ppsState->aFrameCnt = 0;
        for(ppsState->temp = 0x04; ppsState->temp<(ppsState->LenAudioFrame-3); ppsState->temp++)
        {
            if((ppsState->aBuff[ppsState->temp] == 0xff) && (ppsState->aBuff[ppsState->temp+1] == 0xf9)
                && ((ppsState->aBuff[ppsState->temp+2] == 0x60)||(ppsState->aBuff[ppsState->temp+2] == 0x54)||
                      (ppsState->aBuff[ppsState->temp+2] == 0x50)||(ppsState->aBuff[ppsState->temp+2] == 0x4c))
                && ((ppsState->aBuff[ppsState->temp+3] == 0x80)||(ppsState->aBuff[ppsState->temp+3] == 0x40)))
            {
                ppsState->aFrameSize[0] = ppsState->temp - ppsState->nOffset-7;
			}
			else
			{
				if (ppsState->temp==(ppsState->LenAudioFrame-4))
				{
					/* last byte, set FrameSize */
					ppsState->aFrameSize[0] = ppsState->LenAudioFrame-ppsState->nOffset-7;
				}
				else
					continue;
			}
			/* output audio data */
			{
	            qlput4byte( &pqtinfo->astcop[pqtinfo->nastco], (TQP_UINT32) pqtinfo->file_offset );
	            pqtinfo->nastco += 4;
	            if( pqtinfo->nastco == QMP_S_STCOP )
	            {
	                fwrite( pqtinfo->astcop, 1, pqtinfo->nastco, (void *)pqtinfo->fp_astcop);
	                pqtinfo->nastcop += pqtinfo->nastco;
	                pqtinfo->nastco = 0;
	            }
	            pqtinfo->aChunkCnt++;

	            fwrite((char *)&ppsState->aBuff[ppsState->nOffset+7], 1, ppsState->aFrameSize[0], pqtinfo->mp4_handle /*fpOutput*/);
	            pqtinfo->file_offset += ppsState->aFrameSize[0];
	            qlput4byte( &pqtinfo->astszp[pqtinfo->nastsz], ppsState->aFrameSize[0]);
	            pqtinfo->nastsz += 4;
	            if( pqtinfo->nastsz == QMP_S_STSZP )
	            {
	                fwrite( pqtinfo->astszp, 1, pqtinfo->nastsz, (void *)pqtinfo->fp_astszp);
	                pqtinfo->nastszp += pqtinfo->nastsz;
	                pqtinfo->nastsz = 0;
	            }
	            pqtinfo->acfg.frames ++;
	            pqtinfo->astsc ++;

	            qlput4byte( &pqtinfo->astscp[pqtinfo->nastsc], pqtinfo->aChunkCnt);
	            pqtinfo->nastsc += 4;
	            qlput4byte( &pqtinfo->astscp[pqtinfo->nastsc], pqtinfo->astsc );
	            pqtinfo->nastsc += 4;
	            qlput4byte( &pqtinfo->astscp[pqtinfo->nastsc], 1 );
	            pqtinfo->nastsc += 4;
	            pqtinfo->astsc = 0;
	            if( pqtinfo->nastsc == QMP_S_STSCP )
	            {
	                fwrite( pqtinfo->astscp, 1, pqtinfo->nastsc, (void *)pqtinfo->fp_astscp);
	                pqtinfo->nastscp += pqtinfo->nastsc;
	                pqtinfo->nastsc = 0;
	            }
			}

            ppsState->nOffset = ppsState->temp;
            ppsState->aFrameCnt++;
        }

		//printf("[aFrameCnt(G0):%d, ppsState->LenAudioFrame=%d]\n",ppsState->aFrameCnt,ppsState->LenAudioFrame);
        ppsState->LenAudioFrame = 0;
        ppsState->aWrPtr = ppsState->aBuff;
    }
	else if(ppsState->LenAudioFrame)
	{
		printf("mp4: err: LenAudioFrame=%d\n",ppsState->LenAudioFrame);
	}
	if(ppsState->aFrameCnt >0)
	{
		for(ppsState->temp = 0x00; ppsState->temp<ppsState->aFrameCnt; ppsState->temp++)
		{
			qlput4byte( &pqtinfo->asttsp[pqtinfo->nastts], 1);
			pqtinfo->nastts += 4;
			qlput4byte( &pqtinfo->asttsp[pqtinfo->nastts], 0x400);
			pqtinfo->nastts += 4;
			if( pqtinfo->nastts == QMP_S_STTSP )
			{
				fwrite( pqtinfo->asttsp, 1, pqtinfo->nastts, (void *)pqtinfo->fp_asttsp);
				pqtinfo->nasttsp += pqtinfo->nastts;
				pqtinfo->nastts = 0;
			}
		}
	}

	fflush(*pfdMp4vstts);
	fflush(*pfdMp4vstsz);
	fflush(*pfdMp4vstco);
	fflush(*pfdMp4vstsc);
	fflush(*pfdMp4astts);
	fflush(*pfdMp4astsz);
	fflush(*pfdMp4astco);
	fflush(*pfdMp4astsc);
	fflush(*pfpOutput);
    mp4_fini(pqtinfo);
    if(ppsState->vBuff)
        free(ppsState->vBuff);
    if(ppsState->aBuff)
        free(ppsState->aBuff);
    if(ppsState->Startaddr)
        free(ppsState->Startaddr);
    if(*pfdMp4vstts)
    {
        fclose(*pfdMp4vstts);
        *pfdMp4vstts = 0;
        strcpy(FNamemp4, filename);
        strcat(FNamemp4, "vss");
        remove(FNamemp4);
    }
    if(*pfdMp4vstsz)
    {
        fclose(*pfdMp4vstsz);
        *pfdMp4vstsz = 0;
        strcpy(FNamemp4, filename);
        strcat(FNamemp4, "vsz");
        remove(FNamemp4);
    }
    if(*pfdMp4vstco)
    {
        fclose(*pfdMp4vstco);
        *pfdMp4vstco = 0;
        strcpy(FNamemp4, filename);
        strcat(FNamemp4, "vco");
        remove(FNamemp4);
    }
    if(*pfdMp4vstsc)
    {
        fclose(*pfdMp4vstsc);
        *pfdMp4vstsc = 0;
        strcpy(FNamemp4, filename);
        strcat(FNamemp4, "vsc");
        remove(FNamemp4);
    }

//#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
//	filename = MP4_TMP_FILE;
//#endif

    if(*pfdMp4astts)
    {
        fclose(*pfdMp4astts);
        *pfdMp4astts = 0;
        strcpy(FNamemp4, filename);
        strcat(FNamemp4, "ass");
        remove(FNamemp4);
    }
    if(*pfdMp4astsz)
    {
        fclose(*pfdMp4astsz);
        *pfdMp4astsz = 0;
        strcpy(FNamemp4, filename);
        strcat(FNamemp4, "asz");
        remove(FNamemp4);
    }
    if(*pfdMp4astco)
    {
        fclose(*pfdMp4astco);
        *pfdMp4astco = 0;
        strcpy(FNamemp4, filename);
        strcat(FNamemp4, "aco");
        remove(FNamemp4);
    }
    if(*pfdMp4astsc)
    {
        fclose(*pfdMp4astsc);
        *pfdMp4astsc = 0;
        strcpy(FNamemp4, filename);
        strcat(FNamemp4, "asc");
        remove(FNamemp4);
    }
	free(FNamemp4);
	if (*pfpOutput)
	{
		fclose(*pfpOutput);
		*pfpOutput = 0;
	}
	return 0;
}

void test_streamout_userptr(component_t *pComponent)
{
	int							lErr = 0;
	FILE						*fpOutput = NULL;
	FILE						*fpOutputBackup = NULL;

	struct v4l2_capability		stCap;
	struct v4l2_requestbuffers	stReq;
	struct qlapp_v4l_buf_info	*stAppV4lBuf= 0;
	struct v4l2_buffer			stV4lBuf;
	int							nV4lBufCnt;

	int							nRetryCnt = 0;
	unsigned int				ulXferBufCnt = 0;
	unsigned int				ulXferFullBufCnt = 1;

	unsigned int				unStopTime = 0;

	char						*pcaYUVBuff = NULL;
	unsigned int				ulYUVBuffSize;
	unsigned int				ulWidth;
	unsigned int				ulHeight;

	int							nSocket = -1;
    struct sockaddr_in			stToAddr;
	struct sockaddr_un 			stToUn;
    unsigned char				*pszNameTemp = NULL;

	int							frame_size;
	int							frame_nb;

	printf("%s() [\n", __FUNCTION__);

	if (COMPONENT_TYPE_ENCODE == pComponent->devType)
	{
		ulWidth = pComponent->ulWidth;
		ulHeight = pComponent->ulHeight;
	}
	else
	{
		ulWidth = pComponent->decDispAttrib.outputSize.Attrib.x;
		ulHeight = pComponent->decDispAttrib.outputSize.Attrib.y;
	}

	switch (pComponent->ports[STREAM_DIR_OUT].portType)
	{
		case COMPONENT_PORT_YUV_OUT:
			frame_nb = 2;
			frame_size = (ulWidth * ulHeight * pComponent->ports[STREAM_DIR_OUT].openFormat.yuv.nBitCount) / 8;
			break;

		case COMPONENT_PORT_COMP_OUT:
            if (pComponent->ports[STREAM_DIR_OUT].openFormat.vid.nDataType == QP_COMP_VID_DATA_TYPE_MJPEG)
            {
                frame_nb = 8;
                frame_size = 1024 * 1024;
                break;
            }
            /* go through */
		default:
			frame_nb = 32;
			frame_size = 64 * 1024;
			break;
	}

    /***********************************************
    ** 1> Open output file descriptor
    ***********************************************/
    if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_TCP_OUT)
	{
		int	curFlags;
		int	nRetryCnt = 0;

		nSocket = socket(AF_INET, SOCK_STREAM, 0);
		if (nSocket < 0)
		{
			printf("%s() Failed to create socket.\n", __FUNCTION__);
			goto FUNC_END;
		}

		stToAddr.sin_addr.s_addr = inet_addr((const char *)pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr);
		stToAddr.sin_port = htons(pComponent->ports[STREAM_DIR_OUT].unPort);
		stToAddr.sin_family = AF_INET;

		curFlags = fcntl(nSocket, F_GETFL, 0);
		fcntl(nSocket, F_SETFL, curFlags | O_NONBLOCK);

		printf("%s() Connecting to %s .", __FUNCTION__, pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr);
		while (connect(nSocket, (void *)&stToAddr, sizeof(stToAddr)) < 0)
		{
			if (pComponent->ports[STREAM_DIR_OUT].unCtrlCReceived)
			{
				goto FUNC_END;
			}
			fflush(stdout);
			usleep(1000);
			if (nRetryCnt++ == 1000)
			{
				nRetryCnt = 0;
				printf(".");
			}
		}
		printf("\n%s() Got connected\n", __FUNCTION__);
	}
	else if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_UDP_OUT)
	{
		int	curFlags;

		nSocket = socket(AF_INET, SOCK_DGRAM, 0);
		if (nSocket < 0)
		{
            printf("%s() Failed to create socket.\n", __FUNCTION__);
			goto FUNC_END;
        }

		stToAddr.sin_addr.s_addr = inet_addr((const char *)pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr);
		stToAddr.sin_port = htons(pComponent->ports[STREAM_DIR_OUT].unPort);
		stToAddr.sin_family = AF_INET;

		curFlags = fcntl(nSocket, F_GETFL, 0);
		fcntl(nSocket, F_SETFL, curFlags | O_NONBLOCK);
	}
    else if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_STREAM_OUT)
	{
		int	curFlags;
		int	nRetryCnt = 0;
		int	nAddrLen = 0;

		nSocket = socket(AF_UNIX, SOCK_STREAM, 0);
		if (nSocket < 0)
		{
			printf("%s() Failed to create socket.\n", __FUNCTION__);
			goto FUNC_END;
		}

		/* bind client address */
		unlink((const char *)pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr1);
		memset(&stToUn, 0, sizeof(stToUn));
		stToUn.sun_family = AF_UNIX;
		strncpy(stToUn.sun_path, (const char *)pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr1, sizeof(stToUn.sun_path) - 1);
		nAddrLen = offsetof(struct sockaddr_un, sun_path) + (int)strlen((const char *)pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr1);

		curFlags = fcntl(nSocket, F_GETFL, 0);
		fcntl(nSocket, F_SETFL, curFlags | O_NONBLOCK);

		printf("%s() bind to %s .\n", __FUNCTION__, pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr1);
		if (bind(nSocket, (struct sockaddr *)&stToUn, nAddrLen) < 0)
		{
			printf("%s() Failed to bind socket. errno=%d\n", __FUNCTION__, errno);
			goto FUNC_END;
		}

		/* connect to server address */
		memset(&stToUn, 0, sizeof(stToUn));
		stToUn.sun_family = AF_UNIX;
		strncpy(stToUn.sun_path, (const char *)pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr, sizeof(stToUn.sun_path) - 1);
		nAddrLen = offsetof(struct sockaddr_un, sun_path) + (int)strlen((const char *)pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr);

		curFlags = fcntl(nSocket, F_GETFL, 0);
		fcntl(nSocket, F_SETFL, curFlags | O_NONBLOCK);

		printf("%s() Connecting to %s .", __FUNCTION__, pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr);
		while (connect(nSocket, (struct sockaddr *)&stToUn, nAddrLen) < 0)
		{
			if (pComponent->ports[STREAM_DIR_OUT].unCtrlCReceived)
			{
				goto FUNC_END;
			}
			fflush(stdout);
			usleep(1000);
			if (nRetryCnt++ == 1000)
			{
				nRetryCnt = 0;
				printf(".");
			}
		}
		printf("\n%s() Got connected\n", __FUNCTION__);
	}
	else if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_DGRAM_OUT)
	{
		int	curFlags;
		int	nAddrLen;

		nSocket = socket(AF_UNIX, SOCK_DGRAM, 0);
		if (nSocket < 0)
		{
			printf("%s() Failed to create socket.\n", __FUNCTION__);
			goto FUNC_END;
		}

		/* bind client address */
		unlink((const char *)pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr1);
		memset(&stToUn, 0, sizeof(stToUn));
		stToUn.sun_family = AF_UNIX;
		strncpy(stToUn.sun_path, (const char *)pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr1, sizeof(stToUn.sun_path) - 1);
		nAddrLen = offsetof(struct sockaddr_un, sun_path) + (int)strlen((const char *)pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr1);

		curFlags = fcntl(nSocket, F_GETFL, 0);
		fcntl(nSocket, F_SETFL, curFlags | O_NONBLOCK);

		printf("%s() bind to %s .\n", __FUNCTION__, pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr1);
		if (bind(nSocket, (struct sockaddr *)&stToUn, nAddrLen) < 0)
		{
			printf("%s() Failed to bind socket. errno=%d\n", __FUNCTION__, errno);
			goto FUNC_END;
		}

		memset(&stToUn, 0, sizeof(stToUn));
		stToUn.sun_family = AF_UNIX;
		strncpy(stToUn.sun_path, (const char *)pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr, sizeof(stToUn.sun_path) - 1);
		nAddrLen = offsetof(struct sockaddr_un, sun_path) + (int)strlen((const char *)pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr);

#ifdef CONFIG_ENABLE_TESTAPPS_SDL
	}
	else if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_SDL_OUT)
	{
		if (Plstream_CreateOverlay(ulWidth, ulHeight))
		{
		 	goto FUNC_END;
		}
#endif //CONFIG_ENABLE_TESTAPPS_SDL
	}
	else if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_FILE_OUT)
	{
		if (pComponent->ports[STREAM_DIR_OUT].unJPEGSeparate)
		{
			/* only open when got data */
		}
		else
		{
			fpOutput = fopen(pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr, "wb");
			if (fpOutput == NULL)
			{
				printf("%s() Unable to open file %s.\n", __FUNCTION__, pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr);
				lErr = 1;
				goto FUNC_END;
			}
		}
	}
	else if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_NULL_OUT)
	{
		// output to null
	}
	else
	{
		printf("%s() Unknown media type %d.\n", __FUNCTION__, pComponent->ports[STREAM_DIR_OUT].eMediaType);
		goto FUNC_END;
	}

	if (pComponent->ports[STREAM_DIR_OUT].eMediaType2 == MEDIA_FILE_OUT)
	{
		fpOutputBackup = fopen(pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr2, "wb");
		if (fpOutputBackup == NULL)
		{
			printf("%s() Unable to open backup file %s.\n", __FUNCTION__, pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr2);
			lErr = 1;
			goto FUNC_END;
		}
	}

	/***********************************************
	** 2> Create v4l buffers
	***********************************************/

	stAppV4lBuf = malloc(frame_nb * sizeof(struct qlapp_v4l_buf_info));
	if (!stAppV4lBuf)
	{
		printf("%s() Unable to allocate memory for V4L app structure \n", __FUNCTION__);
		lErr = 4;
		goto FUNC_END;
	}

	memset(stAppV4lBuf, 0, frame_nb * sizeof(struct qlapp_v4l_buf_info));

	for (nV4lBufCnt = 0; nV4lBufCnt < frame_nb; nV4lBufCnt++)
	{
		stAppV4lBuf[nV4lBufCnt].addr = malloc(frame_size);
		if (!stAppV4lBuf[nV4lBufCnt].addr)
		{
			printf("%s() Unable to allocate memory for V4L app structure \n", __FUNCTION__);
			lErr = 4;
			goto FUNC_END;
		}
		stAppV4lBuf[nV4lBufCnt].size = frame_size;
		stAppV4lBuf[nV4lBufCnt].sent = 0;

		stAppV4lBuf[nV4lBufCnt].stV4lBuf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
		stAppV4lBuf[nV4lBufCnt].stV4lBuf.memory = V4L2_MEMORY_USERPTR;
		stAppV4lBuf[nV4lBufCnt].stV4lBuf.sequence = pComponent->ports[STREAM_DIR_OUT].portType;
		stAppV4lBuf[nV4lBufCnt].stV4lBuf.index = nV4lBufCnt;
		stAppV4lBuf[nV4lBufCnt].stV4lBuf.length = frame_size;
		stAppV4lBuf[nV4lBufCnt].stV4lBuf.m.userptr = (unsigned long)stAppV4lBuf[nV4lBufCnt].addr;
	}

	/***********************************************
	** 3> Stream on
	***********************************************/
	while (!pComponent->ports[STREAM_DIR_OUT].unCtrlCReceived)
	{
		if (pComponent->ports[STREAM_DIR_OUT].unStarted)
		{
			/***********************************************
			** QBUF, send all the buffers to driver
			***********************************************/
			for (nV4lBufCnt = 0; nV4lBufCnt < frame_nb; nV4lBufCnt++)
			{
				if (!stAppV4lBuf[nV4lBufCnt].sent)
				{
					stAppV4lBuf[nV4lBufCnt].stV4lBuf.bytesused = 0;
					lErr = ioctl(pComponent->hDev, VIDIOC_QBUF, &stAppV4lBuf[nV4lBufCnt].stV4lBuf);
					if (lErr)
					{
						printf("%s() QBUF ioctl failed %d %s\n", __FUNCTION__, errno, strerror(errno));
						if (errno == EAGAIN)
						{
							lErr = 0;
						}
						break;
					}
					else
					{
						stAppV4lBuf[nV4lBufCnt].sent = 1;
					}
				}
			}

	        #ifdef CONFIG_ENABLE_TESTAPPS_USE_SELECT
	        //fprintf(stderr, "==before out select t=%llu\n",getTimeUs());
			{
		        fd_set readfds;
		        struct timeval tv;
		        int r;

		        FD_ZERO (&readfds);
		        FD_SET (pComponent->hDev, &readfds);

		        /* Timeout. */
		        tv.tv_sec =  1;
		        tv.tv_usec = 0;

		        r = select ((pComponent->hDev) + 1, &readfds, NULL, NULL, &tv);
		        if (r<=0) {
		            //fprintf (stderr, "out select timeout t=%llu\n",getTimeUs());
		            continue;
		        }
	            //fprintf(stderr, "==out select pass t=%llu\n",getTimeUs());
			}
			#endif
			/***********************************************
			** DQBUF, get buffer from driver
			***********************************************/

			stV4lBuf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
			stV4lBuf.memory = V4L2_MEMORY_USERPTR;
			stV4lBuf.sequence = pComponent->ports[STREAM_DIR_OUT].portType;

	        //APP_PROFILING_LOG(EVENT_APP_D_OUT_START, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt+1, 0);
			lErr = ioctl(pComponent->hDev, VIDIOC_DQBUF, &stV4lBuf);
			if (!lErr)
			{
				static unsigned long nPreviousPrintTime = 0;
				unsigned long nTime = 0;
				char *pBuf;

                nRetryCnt = 0;

				// clear sent flag
				stAppV4lBuf[stV4lBuf.index].sent = 0;

				nTime = (unsigned long) time(NULL);

				ulXferBufCnt++;

				if (stV4lBuf.reserved & QP_KSSTREAM_HEADER_OPTIONSF_PREROLL ) {
                    APP_PROFILING_LOG(EVENT_APP_D_OUT_INCOMPLETE_FRAME, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt,ulXferBufCnt, stV4lBuf.bytesused);
				}

	            APP_PROFILING_LOG(EVENT_APP_D_OUT, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);

				if ((!g_unReducePrintf) || ((nTime - nPreviousPrintTime) > 1)) {
					printf("\t\t\t\t\t\t\t encXferBufCnt[%lld]: %8u %8u %8u 0x%x 0x%08x t=%lu\r", (long long) pComponent->hDev, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused, stV4lBuf.reserved, stV4lBuf.flags, nTime);
					nPreviousPrintTime = nTime;
				}

				pBuf = stAppV4lBuf[stV4lBuf.index].addr;

				/* remove pending zero for word boundary if necessary */
				fix_data_packet((unsigned char *)pBuf, &stV4lBuf.bytesused, ulXferBufCnt, pComponent);

                if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_TCP_OUT)
				{
					int	nSent = 0;
					int	nTotalSent = 0;
					int	nToBeSent = 0;

			        //APP_PROFILING_LOG(EVENT_APP_S_OUT, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
					if (pComponent->ports[STREAM_DIR_OUT].unPackData)
					{
						unsigned char sbuf[5];
						sbuf[0] = (stV4lBuf.bytesused >> 24) & 0xFF;
						sbuf[1] = (stV4lBuf.bytesused >> 16) & 0xFF;
						sbuf[2] = (stV4lBuf.bytesused >> 8) & 0xFF;
						sbuf[3] = (stV4lBuf.bytesused) & 0xFF;
						while (nTotalSent < 4)
						{
							nToBeSent = 4 - nTotalSent;
							nSent = (int) send(nSocket, (void *) &sbuf[nTotalSent], nToBeSent, 0);
							if (nSent <= 0)
							{
								if (pComponent->ports[STREAM_DIR_OUT].unCtrlCReceived)
								{
									break;
								}
								else
								{
									if (nSent<0) /* error */
									{
										if ((errno==EAGAIN)||(errno==EINTR)||(errno==ENOBUFS))
											continue;
										else {	/* maybe the connect already broken? */
											usleep(10000);
										}
									}
									continue;
								}
							}
							nTotalSent += nSent;
						}
					}
					nTotalSent = 0;
					while (nTotalSent < (int)stV4lBuf.bytesused)
					{
						nToBeSent = stV4lBuf.bytesused - nTotalSent;
						if (nToBeSent > MAX_TCP_PACKET_LEN)
							nToBeSent = MAX_TCP_PACKET_LEN;
						nSent = (int) send(nSocket, (void *)(pBuf + nTotalSent), nToBeSent, 0);

						if (nSent <= 0)
						{
							if (pComponent->ports[STREAM_DIR_OUT].unCtrlCReceived)
							{
								break;
							}
							else
							{
								if (nSent<0) /* error */
								{
									if ((errno==EAGAIN)||(errno==EINTR)||(errno==ENOBUFS))
										continue;
									else {	/* maybe the connect already broken? */
										usleep(10000);
									}
								}
								continue;
							}
						}
						if (nTotalSent==0)
					        APP_PROFILING_LOG(EVENT_APP_S_OUT_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, nSent);
						nTotalSent += nSent;
					}
			        APP_PROFILING_LOG(EVENT_APP_S_OUT_FINISH, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
				}
				else if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_UDP_OUT)
				{
					int	nSent = 0;
					int	nTotalSent = 0;
					int	nToBeSent = 0;

			        //APP_PROFILING_LOG(EVENT_APP_S_OUT, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
					if (pComponent->ports[STREAM_DIR_OUT].unPackData)
					{
						unsigned char sbuf[5];
						sbuf[0] = (stV4lBuf.bytesused >> 24) & 0xFF;
						sbuf[1] = (stV4lBuf.bytesused >> 16) & 0xFF;
						sbuf[2] = (stV4lBuf.bytesused >> 8) & 0xFF;
						sbuf[3] = (stV4lBuf.bytesused) & 0xFF;
						while (nTotalSent < 4)
						{
							nToBeSent = 4 - nTotalSent;
							nSent = (int) sendto(nSocket,
										   (void *)&sbuf[nTotalSent],
										   nToBeSent,
										   0,
										   (void *)&stToAddr,
										   sizeof(struct sockaddr_in)
										   );
							if (nSent <= 0)
							{
								if (pComponent->ports[STREAM_DIR_OUT].unCtrlCReceived)
								{
									break;
								}
								else
								{
									if (nSent<0) /* error */
									{
										if ((errno==EAGAIN)||(errno==EINTR)||(errno==ENOBUFS))
											continue;
										else {	/* maybe the connect already broken? */
											usleep(10000);
										}
									}
									continue;
								}
							}
							nTotalSent += nSent;
						}
					}
					nTotalSent = 0;
					while (nTotalSent < (int)stV4lBuf.bytesused)
					{
						nToBeSent = stV4lBuf.bytesused - nTotalSent;
						if (nToBeSent > MAX_UDP_PACKET_LEN)
							nToBeSent = MAX_UDP_PACKET_LEN;
						nSent = (int) sendto(nSocket,
									   (void *)(pBuf + nTotalSent),
									   nToBeSent,
									   0,
									   (void *)&stToAddr,
									   sizeof(struct sockaddr_in)
									   );

						if (nSent <= 0)
						{
							if (pComponent->ports[STREAM_DIR_OUT].unCtrlCReceived)
							{
								break;
							}
							else
							{
								if (nSent<0) /* error */
								{
									if ((errno==EAGAIN)||(errno==EINTR)||(errno==ENOBUFS))
										continue;
									else {	/* maybe the connect already broken? */
										usleep(10000);
									}
								}
								continue;
							}
						}
						//printf("nSent=%d\n", nSent);
						if (nTotalSent==0)
					        APP_PROFILING_LOG(EVENT_APP_S_OUT_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, nSent);
						nTotalSent += nSent;
					}
			        APP_PROFILING_LOG(EVENT_APP_S_OUT_FINISH, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
				}
                else if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_STREAM_OUT)
				{
					int	nSent = 0;
					int	nTotalSent = 0;
					int	nToBeSent = 0;

			        //APP_PROFILING_LOG(EVENT_APP_S_OUT, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
					if (pComponent->ports[STREAM_DIR_OUT].unPackData)
					{
						unsigned char sbuf[5];
						sbuf[0] = (stV4lBuf.bytesused >> 24) & 0xFF;
						sbuf[1] = (stV4lBuf.bytesused >> 16) & 0xFF;
						sbuf[2] = (stV4lBuf.bytesused >> 8) & 0xFF;
						sbuf[3] = (stV4lBuf.bytesused) & 0xFF;
						while (nTotalSent < 4)
						{
							nToBeSent = 4 - nTotalSent;
							nSent = (int) send(nSocket, (void *)&sbuf[nTotalSent], nToBeSent, 0);
							if (nSent <= 0)
							{
								if (pComponent->ports[STREAM_DIR_OUT].unCtrlCReceived)
								{
									break;
								}
								else
								{
									if (nSent<0) /* error */
									{
										if ((errno==EAGAIN)||(errno==EINTR)||(errno==ENOBUFS))
											continue;
										else {	/* maybe the connect already broken? */
											usleep(10000);
										}
									}
									continue;
								}
							}
							nTotalSent += nSent;
						}
					}
					nTotalSent = 0;
					while (nTotalSent < (int)stV4lBuf.bytesused)
					{
						nToBeSent = stV4lBuf.bytesused - nTotalSent;
						if (!pComponent->ports[STREAM_DIR_OUT].unPackData)
						{
							if (nToBeSent > MAX_TCP_PACKET_LEN)
								nToBeSent = MAX_TCP_PACKET_LEN;
						}
						nSent = (int) send(nSocket, (void *)(pBuf + nTotalSent), nToBeSent, 0);

						if (nSent <= 0)
						{
							if (pComponent->ports[STREAM_DIR_OUT].unCtrlCReceived)
							{
								break;
							}
							else
							{
								if (nSent<0) /* error */
								{
									if ((errno==EAGAIN)||(errno==EINTR)||(errno==ENOBUFS))
										continue;
									else {	/* maybe the connect already broken? */
										usleep(10000);
									}
								}
								continue;
							}
						}
						if (nTotalSent==0)
					        APP_PROFILING_LOG(EVENT_APP_S_OUT_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, nSent);
						nTotalSent += nSent;
					}
			        APP_PROFILING_LOG(EVENT_APP_S_OUT_FINISH, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
				}
				else if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_DGRAM_OUT)
				{
					int	nSent = 0;
					int	nTotalSent = 0;
					int	nToBeSent = 0;
					int	nAddrLen;

					nAddrLen = offsetof(struct sockaddr_un, sun_path) + (int) strlen(pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr);

			        //APP_PROFILING_LOG(EVENT_APP_S_OUT, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
					if (pComponent->ports[STREAM_DIR_OUT].unPackData)
					{
						unsigned char sbuf[5];
						sbuf[0] = (stV4lBuf.bytesused >> 24) & 0xFF;
						sbuf[1] = (stV4lBuf.bytesused >> 16) & 0xFF;
						sbuf[2] = (stV4lBuf.bytesused >> 8) & 0xFF;
						sbuf[3] = (stV4lBuf.bytesused) & 0xFF;
						while (nTotalSent < 4)
						{
							nToBeSent = 4 - nTotalSent;
							nSent = (int) sendto(nSocket,
										   (void *)&sbuf[nTotalSent],
										   nToBeSent,
										   0,
										   (void *)&stToUn,
										   nAddrLen
										   );
							if (nSent <= 0)
							{
								if (pComponent->ports[STREAM_DIR_OUT].unCtrlCReceived)
								{
									break;
								}
								else
								{
									if (nSent<0) /* error */
									{
										if ((errno==EAGAIN)||(errno==EINTR)||(errno==ENOBUFS))
											continue;
										else {	/* maybe the connect already broken? */
											usleep(10000);
										}
									}
									continue;
								}
							}
							nTotalSent += nSent;
						}
					}
					nTotalSent = 0;
					while (nTotalSent < (int)stV4lBuf.bytesused)
					{
						nToBeSent = stV4lBuf.bytesused - nTotalSent;
						if (!pComponent->ports[STREAM_DIR_OUT].unPackData)
						{
							if (nToBeSent > MAX_UDP_PACKET_LEN)
								nToBeSent = MAX_UDP_PACKET_LEN;
						}
						nSent = (int) sendto(nSocket,
									   (void *)(pBuf + nTotalSent),
									   nToBeSent,
									   0,
									   (void *)&stToUn,
									   nAddrLen
									   );

						if (nSent <= 0)
						{
							if (pComponent->ports[STREAM_DIR_OUT].unCtrlCReceived)
							{
								break;
							}
							else
							{
								if (nSent<0) /* error */
								{
									if ((errno==EAGAIN)||(errno==EINTR)||(errno==ENOBUFS))
										continue;
									else {	/* maybe the connect already broken? */
										usleep(10000);
									}
								}
								continue;
							}
						}
						//printf("nSent=%d\n", nSent);
						if (nTotalSent==0)
					        APP_PROFILING_LOG(EVENT_APP_S_OUT_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, nSent);
						nTotalSent += nSent;
					}
			        APP_PROFILING_LOG(EVENT_APP_S_OUT_FINISH, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
#ifdef CONFIG_ENABLE_TESTAPPS_SDL
				}
				else if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_SDL_OUT)
				{
			        //APP_PROFILING_LOG(EVENT_APP_S_OUT, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
					if (pComponent->ports[STREAM_DIR_OUT].openFormat.yuv.nDataType==QP_YUV_DATA_TYPE_YUYV)
					{
						if (stV4lBuf.bytesused == (ulWidth * ulHeight * 2))
						{
							if (pcaYUVBuff==NULL)
							{
								pcaYUVBuff = (unsigned char*)malloc((ulWidth * ulHeight * 3)/2);
							}
							if (pcaYUVBuff)
							{
								YUV422ProgTo420Plane(pcaYUVBuff, pBuf, ulWidth, ulHeight);
								Plstream_DisplayFrame(ulWidth,
												  ulHeight,
												  (unsigned char *)pcaYUVBuff,
												  pComponent->ports[STREAM_DIR_OUT].displayType,
												  QP_YUV_DATA_TYPE_YV12
												  );
							}
						}
                    }
					else
					{
						if (stV4lBuf.bytesused == ((ulWidth * ulHeight * 3) / 2))
						{
								Plstream_DisplayFrame(ulWidth,
													  ulHeight,
													  (unsigned char *)pBuf,
													  pComponent->ports[STREAM_DIR_OUT].displayType,
													  pComponent->ports[STREAM_DIR_OUT].openFormat.yuv.nDataType
													  );
						}
					}
			        APP_PROFILING_LOG(EVENT_APP_S_OUT_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
			        APP_PROFILING_LOG(EVENT_APP_S_OUT_FINISH, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
#endif //CONFIG_ENABLE_TESTAPPS_SDL
				}
				else if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_FILE_OUT)
				{
			        //APP_PROFILING_LOG(EVENT_APP_S_OUT, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
					if (pComponent->ports[STREAM_DIR_OUT].unJPEGSeparate)
					{
					    char *pszNameTemp;
						pszNameTemp = generate_filename_byindex(pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr, ulXferBufCnt);
						fpOutput = fopen(pszNameTemp, "wb");
						if (pszNameTemp) {
							free(pszNameTemp);
							pszNameTemp = NULL;
						}
					}

					if (fpOutput)
                        fwrite((void*)pBuf, 1, stV4lBuf.bytesused, fpOutput);

			        APP_PROFILING_LOG(EVENT_APP_S_OUT_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
			        APP_PROFILING_LOG(EVENT_APP_S_OUT_FINISH, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
					if (pComponent->ports[STREAM_DIR_OUT].unJPEGSeparate)
					{
						if (fpOutput) {
                            fclose(fpOutput);
	                        fpOutput = NULL;
						}
					}
				}
				else if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_NULL_OUT)
				{
			        //APP_PROFILING_LOG(EVENT_APP_S_OUT, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
			        APP_PROFILING_LOG(EVENT_APP_S_OUT_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
			        APP_PROFILING_LOG(EVENT_APP_S_OUT_FINISH, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
				}
				fflush(stdout);

				if (pComponent->ports[STREAM_DIR_OUT].eMediaType2 == MEDIA_FILE_OUT)
				{
					if (fpOutputBackup)
                        fwrite((void*)pBuf, 1, stV4lBuf.bytesused, fpOutputBackup);
				}

				if (stV4lBuf.reserved & QP_KSSTREAM_HEADER_OPTIONSF_PREROLL ) {
				}
				else  {
					ulXferFullBufCnt++;
				}
			}
			else
			{
				if (errno == EAGAIN)
				{
		            //APP_PROFILING_LOG(EVENT_APP_D_OUT_NODATA, (TQP_UINT32) pComponent->hDev, pComponent->devType, 0, 0, 0);
					lErr = 0;
				}
				else
				{
		            APP_PROFILING_LOG(EVENT_APP_D_OUT_FAILED, (TQP_UINT32) pComponent->hDev, pComponent->devType,ulXferFullBufCnt, ulXferBufCnt+1, 0);
					//printf("\r%s()  DQBUF failed(%d) errno(%d)\n", __FUNCTION__, lErr, errno);
				}

				usleep(1000);
			}

			if (pComponent->ports[STREAM_DIR_OUT].unCtrlCReceived)
			{
				printf("\n\n%s() CTRL+C received.\n", __FUNCTION__);
				break;
			}

		}
	} //while(!lErr)


FUNC_END:

	printf("\n");

	usleep(10000);

	if (stAppV4lBuf)
	{
		for (nV4lBufCnt = 0; nV4lBufCnt < frame_nb; nV4lBufCnt++)
		{
			if (stAppV4lBuf[nV4lBufCnt].addr)
			{
				free(stAppV4lBuf[nV4lBufCnt].addr);
				stAppV4lBuf[nV4lBufCnt].addr = NULL;
			}
		}
		free(stAppV4lBuf);
	}

	if (fpOutput)
	{
		fclose(fpOutput);
	}

	if (fpOutputBackup)
	{
		fclose(fpOutputBackup);
	}

    if (nSocket != -1)
	{
		close(nSocket);
	}

	if (pcaYUVBuff)
	{
		free(pcaYUVBuff);
	}

	if (pszNameTemp)
	{
		free(pszNameTemp);
	}

	pComponent->ports[STREAM_DIR_OUT].unStarted = 0;

	printf("%s() ]\n", __FUNCTION__);


    //return lErr;
}



void test_streamin_userptr(component_t *pComponent)
{
	int							lErr = 0;

	FILE						*fpInput = NULL;
	FILE						*fpOutputBackup = NULL;

	int							videoInInited = 0;

	int							nType = V4L2_BUF_TYPE_VIDEO_CAPTURE;

	struct v4l2_capability		stCap;
	struct v4l2_requestbuffers	stReq;
	struct qlapp_v4l_buf_info	*stAppV4lBuf = 0;
	struct v4l2_buffer			stV4lBuf;
	struct v4l2_buffer			*pstV4lBuf = NULL;
	int							nV4lBufCnt;

	int							nRetryCnt = 0;
	unsigned int				ulXferBufCnt = 0;

	unsigned int				unStopTime = 0;

	unsigned int				ulIOBlockSize = 64 * 1024;

	unsigned int				ulInWidth, ulInHeight;
	unsigned int				ulOutWidth, ulOutHeight;

	int							nSocket = -1;
	struct sockaddr_in			stFromAddr;

	unsigned char				*pucaUvcBuffer = NULL;

	unsigned int				unUvcFrameRepeatCnt = 0;

	unsigned int				ulPTS = 0;
	int							nNeedGetDataLen = 1;
	int							nDataLen = 0;

	int							frame_size;
	int							frame_nb;

	printf("%s() [\n", __FUNCTION__);

	if (COMPONENT_TYPE_ENCODE == pComponent->devType)
	{
		ulInWidth = pComponent->picResolution.Artesa.in_pic_width;
		ulInHeight = pComponent->picResolution.Artesa.in_pic_height;

		ulOutWidth = pComponent->outResolution.out_pic_width;
		ulOutHeight = pComponent->outResolution.out_pic_height;
	}
	else
	{
		ulOutWidth = pComponent->decDispAttrib.outputSize.Attrib.x;
		ulOutHeight = pComponent->decDispAttrib.outputSize.Attrib.y;
		ulInWidth = ulOutWidth;
		ulInHeight = ulOutHeight;
	}

	switch (pComponent->ports[STREAM_DIR_IN].portType)
	{
		case COMPONENT_PORT_YUV_IN:
			frame_nb = 2;
			ulIOBlockSize = frame_size = (ulInWidth * ulInHeight * 3) / 2;
			break;

		case COMPONENT_PORT_PCM_IN:
			frame_nb = 32;
			ulIOBlockSize = frame_size = 8 * 1024;
			break;

		case COMPONENT_PORT_COMP_IN:
		default:
			frame_nb = 32;
			ulIOBlockSize = frame_size = 64 * 1024;
			break;
	}

	/***********************************************
	** 1> Open output file descriptor
	***********************************************/
	if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_TCP_IN)
	{
		int				curFlags;
		int				nListenSocket;
		fd_set			readset;
		struct timeval	timeout;
		socklen_t 		nAddrLen = 0;
		int				nReuseFlag = 1;

		nListenSocket = socket(AF_INET, SOCK_STREAM, 0);
		if (nListenSocket < 0)
		{
			printf("%s() Failed to create socket.\n", __FUNCTION__);
			goto FUNC_END;
		}

		if (setsockopt(nListenSocket, SOL_SOCKET, SO_REUSEADDR, (const char*)&nReuseFlag, sizeof nReuseFlag) < 0)
		{
			printf("%s() Failed to setsockopt(SO_REUSEADDR).\n", __FUNCTION__);
			goto FUNC_END;
		}

		stFromAddr.sin_addr.s_addr = htonl(INADDR_ANY);
		stFromAddr.sin_port = htons(pComponent->ports[STREAM_DIR_IN].unPort);
		stFromAddr.sin_family = AF_INET;

		curFlags = fcntl(nListenSocket, F_GETFL, 0);
		fcntl(nListenSocket, F_SETFL, curFlags | O_NONBLOCK);

		if (bind(nListenSocket, (void *)&stFromAddr, sizeof(stFromAddr)) < 0)
		{
			printf("%s() Failed to bind socket. errno=%d\n", __FUNCTION__, errno);
			goto FUNC_END;
		}

		if (listen(nListenSocket, 1) < 0)
		{
			printf("%s() Failed to listen. errno=%d\n", __FUNCTION__, errno);
			goto FUNC_END;
		}

		printf("%s() Waiting for connection .", __FUNCTION__);
		fflush(stdout);

		while (1)
		{
			FD_ZERO(&readset);
			FD_SET(nListenSocket, &readset);

			timeout.tv_sec = 1;
			timeout.tv_usec = 0;

			if (select(nListenSocket + 1, &readset, NULL, NULL, &timeout) >= 0)
			{
				if (FD_ISSET(nListenSocket, &readset))
				{
					nSocket = accept(nListenSocket, (void *)&stFromAddr, &nAddrLen);
					curFlags = fcntl(nSocket, F_GETFL, 0);
					fcntl(nSocket, F_SETFL, curFlags | O_NONBLOCK);
					printf("\n%s() Got connection from %s. \n", __FUNCTION__, inet_ntoa(stFromAddr.sin_addr));
					break;
                }
            }
			if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
			{
				goto FUNC_END;
			}
			printf(".");
			fflush(stdout);
			usleep(1000);
		}
		fflush(stdout);
	}
#ifndef CONFIG_HOST_PLATFORM_USERSPACE
	else if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_UDP_IN)
	{
		int	curFlags;

		nSocket = socket(AF_INET, SOCK_DGRAM, 0);
		if (nSocket < 0)
		{
			printf("%s() Failed to create socket.\n", __FUNCTION__);
			goto FUNC_END;
		}
		stFromAddr.sin_addr.s_addr = htonl(INADDR_ANY);
		stFromAddr.sin_port = htons(pComponent->ports[STREAM_DIR_IN].unPort);
		stFromAddr.sin_family = AF_INET;

		curFlags = fcntl(nSocket, F_GETFL, 0);
		fcntl(nSocket, F_SETFL, curFlags | O_NONBLOCK);

		if (bind(nSocket, (void *)&stFromAddr, sizeof(stFromAddr)) < 0)
		{
			printf("%s() Failed to bind socket. errno=%d\n", __FUNCTION__, errno);
			goto FUNC_END;
		}
	}
	else if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_STREAM_IN)
	{
		int				curFlags;
		int				nListenSocket;
		fd_set			readset;
		struct timeval	timeout;
		int				nAddrLen;
		struct sockaddr_un stFromUn;
		struct stat statbuf;

		nListenSocket = socket(AF_UNIX, SOCK_STREAM, 0);
		if (nListenSocket < 0)
		{
			printf("%s() Failed to create domain socket.\n", __FUNCTION__);
			goto FUNC_END;
		}

		unlink(pComponent->ports[STREAM_DIR_IN].pszNameOrAddr);
		memset(&stFromUn, 0, sizeof(stFromUn));
		stFromUn.sun_family = AF_UNIX;
		strncpy(stFromUn.sun_path, pComponent->ports[STREAM_DIR_IN].pszNameOrAddr, sizeof(stFromUn.sun_path) - 1);
		nAddrLen = offsetof(struct sockaddr_un, sun_path) + strlen(pComponent->ports[STREAM_DIR_IN].pszNameOrAddr);

		curFlags = fcntl(nListenSocket, F_GETFL, 0);
		fcntl(nListenSocket, F_SETFL, curFlags | O_NONBLOCK);

		if (bind(nListenSocket, (struct sockaddr *)&stFromUn, nAddrLen) < 0)
		{
			printf("%s() Failed to bind socket. errno=%d\n", __FUNCTION__, errno);
			goto FUNC_END;
		}

		if (listen(nListenSocket, 1) < 0)
		{
			printf("%s() Failed to listen. errno=%d\n", __FUNCTION__, errno);
			goto FUNC_END;
		}

		printf("%s() Waiting for connection .", __FUNCTION__);
		fflush(stdout);

		while (1)
		{
			FD_ZERO(&readset);
			FD_SET(nListenSocket, &readset);

			timeout.tv_sec = 1;
			timeout.tv_usec = 0;

			if (select(nListenSocket + 1, &readset, NULL, NULL, &timeout) >= 0)
			{
				if (FD_ISSET(nListenSocket, &readset))
				{
					nAddrLen = sizeof(struct sockaddr_un); // this is important point for linux */
					nSocket = accept(nListenSocket, (struct sockaddr *)&stFromUn, &nAddrLen);
					curFlags = fcntl(nSocket, F_GETFL, 0);
					fcntl(nSocket, F_SETFL, curFlags | O_NONBLOCK);

					/* optional, obtain the client's uid from its calling address */
					nAddrLen -= offsetof(struct sockaddr_un, sun_path);  /* len of pathname */
					stFromUn.sun_path[nAddrLen] = 0; /* null terminate string */
					if (stat(stFromUn.sun_path, &statbuf) < 0)
					{
						/* error */
						close(nSocket);
						printf("\n%s() Got wrong connection from %s, retry it\n", __FUNCTION__, stFromUn.sun_path);
						continue;
					}
					else {
						if (S_ISSOCK(statbuf.st_mode) )
						{
							//if (uidptr != NULL) *uidptr = statbuf.st_uid;    /* return uid of caller */
							unlink(stFromUn.sun_path);       /* we're done with pathname now */
							printf("\n%s() Got connection from %s. \n", __FUNCTION__, stFromUn.sun_path);
							close(nListenSocket); // close listen socket
							break;
						}
						close(nSocket);
						printf("\n%s() Got wrong connection from %s, not a socket, retry it\n", __FUNCTION__, stFromUn.sun_path);
						continue;
					}
                }
            }
			if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
			{
				goto FUNC_END;
			}
			printf(".");
			fflush(stdout);
			usleep(1000);
		}
		fflush(stdout);
	}
	else if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_DGRAM_IN)
	{
		int	curFlags;
		struct sockaddr_un stFromUn;
		int nAddrLen;

		nSocket = socket(AF_UNIX, SOCK_DGRAM, 0);
		if (nSocket < 0)
		{
			printf("%s() Failed to create socket.\n", __FUNCTION__);
			goto FUNC_END;
		}

		unlink(pComponent->ports[STREAM_DIR_IN].pszNameOrAddr);
		memset(&stFromUn, 0, sizeof(stFromUn));
		stFromUn.sun_family = AF_UNIX;
		strncpy(stFromUn.sun_path, pComponent->ports[STREAM_DIR_IN].pszNameOrAddr, sizeof(stFromUn.sun_path) - 1);
		nAddrLen = offsetof(struct sockaddr_un, sun_path) + strlen(pComponent->ports[STREAM_DIR_IN].pszNameOrAddr);

		curFlags = fcntl(nSocket, F_GETFL, 0);
		fcntl(nSocket, F_SETFL, curFlags | O_NONBLOCK);

		if (bind(nSocket, (struct sockaddr *)&stFromUn, sizeof(stFromUn)) < 0)
		{
			printf("%s() Failed to bind socket. errno=%d\n", __FUNCTION__, errno);
			goto FUNC_END;
		}
	}
	else if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_UVCRAW_IN)
	{
		if (pComponent->ports[STREAM_DIR_IN].unUvcFrameRepeat < 1)
		{
            pComponent->ports[STREAM_DIR_IN].unUvcFrameRepeat = 1;
		}

		if (pComponent->ports[STREAM_DIR_IN].unUvcFps < 1)
		{
			pComponent->ports[STREAM_DIR_IN].unUvcFps = 30;
		}

		if ((lErr = uvc_open_device(pComponent->ports[STREAM_DIR_IN].pszNameOrAddr)) < 0)
		{
			goto FUNC_END;
		}
		else
		{
			videoInInited = 1;
		}

		if ((lErr = uvc_init_device(V4L2_PIX_FMT_YUYV, ulInWidth, ulInHeight, pComponent->ports[STREAM_DIR_IN].unUvcFps)) < 0)
		{
			goto FUNC_END;
		}

		if ((lErr = uvc_start_capturing()) < 0)
		{
			goto FUNC_END;
		}

		pucaUvcBuffer = (unsigned char*)malloc(ulInWidth * ulInHeight * 2);
		if (!pucaUvcBuffer)
		{
			lErr = 1;
			goto FUNC_END;
		}

#ifdef CONFIG_ENABLE_TESTAPPS_SDL
        if (pComponent->ports[STREAM_DIR_IN].unUvcPreview)
		{
			if (Plstream_CreateOverlay(ulInWidth, ulInHeight))
			{
		 		goto FUNC_END;
			}
		}
#endif //CONFIG_ENABLE_TESTAPPS_SDL
	}
	else if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_UVCMJPEG_IN)
	{
		if (pComponent->ports[STREAM_DIR_IN].unUvcFrameRepeat < 1)
		{
			pComponent->ports[STREAM_DIR_IN].unUvcFrameRepeat = 1;
		}

		if (pComponent->ports[STREAM_DIR_IN].unUvcFps < 1)
		{
			pComponent->ports[STREAM_DIR_IN].unUvcFps = 30;
		}

		if ((lErr = uvc_open_device(pComponent->ports[STREAM_DIR_IN].pszNameOrAddr)) < 0)
		{
			goto FUNC_END;
		}
		else
		{
			videoInInited = 1;
		}

		if ((lErr = uvc_init_device(V4L2_PIX_FMT_MJPEG, ulInWidth, ulInHeight, pComponent->ports[STREAM_DIR_IN].unUvcFps)) < 0)
		{
			goto FUNC_END;
		}

		if ((lErr = uvc_start_capturing()) < 0)
		{
			goto FUNC_END;
		}

#ifdef CONFIG_ENABLE_TESTAPPS_SDL
		if (pComponent->ports[STREAM_DIR_IN].unUvcPreview)
		{
			if (Plstream_CreateOverlay(ulInWidth, ulInHeight))
			{
		 		goto FUNC_END;
			}
		}
#endif //CONFIG_ENABLE_TESTAPPS_SDL
	}
#endif // CONFIG_HOST_PLATFORM_USERSPACE
	else if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_FILE_IN)
	{
		fpInput = fopen(pComponent->ports[STREAM_DIR_IN].pszNameOrAddr, "rb");
		if (fpInput == NULL)
		{
            printf("%s() Unable to open file %s.\n", __FUNCTION__, pComponent->ports[STREAM_DIR_IN].pszNameOrAddr);
			lErr = 1;
			goto FUNC_END;
		}
	}
	else
	{
		goto FUNC_END;
	}

	if (pComponent->ports[STREAM_DIR_IN].eMediaType2 == MEDIA_FILE_OUT)
	{
		fpOutputBackup = fopen(pComponent->ports[STREAM_DIR_IN].pszNameOrAddr2, "wb");
		if (fpOutputBackup == NULL)
		{
			printf("%s() Unable to open backup file %s.\n", __FUNCTION__, pComponent->ports[STREAM_DIR_IN].pszNameOrAddr2);
			lErr = 1;
			goto FUNC_END;
		}
	}

	/***********************************************
	** 2> Create v4l buffers
	***********************************************/
	stAppV4lBuf = malloc(frame_nb * sizeof(struct qlapp_v4l_buf_info));
	if (!stAppV4lBuf)
	{
		printf("%s() Unable to allocate memory for V4L app structure \n", __FUNCTION__);
		lErr = 4;
		goto FUNC_END;
	}

	memset(stAppV4lBuf, 0, frame_nb * sizeof(struct qlapp_v4l_buf_info));

	for (nV4lBufCnt = 0; nV4lBufCnt < frame_nb; nV4lBufCnt++)
	{
		stAppV4lBuf[nV4lBufCnt].addr = malloc(frame_size);
		if (!stAppV4lBuf[nV4lBufCnt].addr)
		{
			printf("%s() Unable to allocate memory for V4L app structure \n", __FUNCTION__);
			lErr = 4;
			goto FUNC_END;
		}
		stAppV4lBuf[nV4lBufCnt].size = frame_size;
		stAppV4lBuf[nV4lBufCnt].sent = 0;

		stAppV4lBuf[nV4lBufCnt].stV4lBuf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
		stAppV4lBuf[nV4lBufCnt].stV4lBuf.memory = V4L2_MEMORY_USERPTR;
		stAppV4lBuf[nV4lBufCnt].stV4lBuf.sequence = pComponent->ports[STREAM_DIR_IN].portType;
		stAppV4lBuf[nV4lBufCnt].stV4lBuf.index = nV4lBufCnt;
		stAppV4lBuf[nV4lBufCnt].stV4lBuf.length = frame_size;
		stAppV4lBuf[nV4lBufCnt].stV4lBuf.m.userptr = (unsigned long)stAppV4lBuf[nV4lBufCnt].addr;
	}


	/***********************************************
	** 3> Stream on
	***********************************************/
	while (!pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
	{
		if (pComponent->ports[STREAM_DIR_IN].unStarted)
		{
			/***********************************************
			** DQBUF, get buffer from driver
			***********************************************/
		    #ifdef CONFIG_ENABLE_TESTAPPS_USE_SELECT
	        //fprintf(stderr, "==before test_streamin_userptr select t=%llu\n",getTimeUs());
			{
		        fd_set writefds;
		        struct timeval tv;
		        int r;

		        FD_ZERO (&writefds);
		        FD_SET (pComponent->hDev, &writefds);

		        /* Timeout. */
		        tv.tv_sec =  1;
		        tv.tv_usec = 0;

		        r = select ((pComponent->hDev) + 1,NULL, &writefds, NULL, &tv);

		        if (r<=0) {
		            //fprintf (stderr, "test_streamin_userptr select timeout t=%llu\n",getTimeUs());
		            continue;
		        }
	            //fprintf(stderr, "==test_streamin_userptr select pass t=%llu\n",getTimeUs());
			}
			#endif
	        APP_PROFILING_LOG(EVENT_APP_D_IN_START, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, 0);
			for (nV4lBufCnt = 0; nV4lBufCnt < frame_nb; nV4lBufCnt++)
			{
				if (stAppV4lBuf[nV4lBufCnt].sent)
				{
					stV4lBuf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
					stV4lBuf.memory = V4L2_MEMORY_USERPTR;
					stV4lBuf.sequence = pComponent->ports[STREAM_DIR_IN].portType;

					lErr = ioctl(pComponent->hDev, VIDIOC_DQBUF, &stV4lBuf);
					if (!lErr)
					{
						stAppV4lBuf[stV4lBuf.index].sent = 0;
					}
					else
					{
						break;
					}
				}
			}

			if (lErr)
			{
				if (errno == EAGAIN)
				{
					lErr = 0;
				}
			}

			/***********************************************
			** get empty buffer and read data
			***********************************************/
			pstV4lBuf = NULL;

			for (nV4lBufCnt = 0; nV4lBufCnt < frame_nb; nV4lBufCnt++)
			{
				if (!stAppV4lBuf[nV4lBufCnt].sent)
				{
					pstV4lBuf = &stAppV4lBuf[nV4lBufCnt].stV4lBuf;
					break;
				}
			}

			if (pstV4lBuf)
			{
				unsigned char *pBuf;

				nRetryCnt = 0;

				pBuf = (unsigned char *)stAppV4lBuf[pstV4lBuf->index].addr;

RETRY:
				if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_TCP_IN)
				{
					int				nReceived = 0;
					int				nTotalReceived = 0;
					unsigned int	ulAddrLen = 0;

		            APP_PROFILING_LOG(EVENT_APP_S_IN, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, 0);
					if (pComponent->ports[STREAM_DIR_IN].unPackData)
					{
						int ulIOBlockSizeReal;
						int	nIOError = 0;

						if (nNeedGetDataLen)
						{
							while (nTotalReceived < 4)
							{
								int datatoget = 4 - nTotalReceived;
								nReceived = (int) recv(nSocket, (void *)(pBuf + nTotalReceived), datatoget, 0);

								if (nReceived <= 0)
								{
									if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
									{
										nIOError = 1;
										break;
									}
									else
									{
										if (EAGAIN == errno)
										{
											usleep(1000);
											continue;
										}
										else
										{
											nIOError = 1;
											break;
										}
									}
								}
								else
								{
									nTotalReceived += nReceived;
								}
							}
							if (nIOError==0)
								nDataLen = pBuf[0]<< 24 | pBuf[1]<<16 | pBuf[2] << 8 | pBuf[3];
						}
						if (nIOError==0)
						{
							if (nDataLen > ulIOBlockSize)
								ulIOBlockSizeReal = ulIOBlockSize;
							else
								ulIOBlockSizeReal = nDataLen;

							nTotalReceived = 0;
							while (nTotalReceived < (int)ulIOBlockSizeReal)
							{
								int datatoget = ulIOBlockSizeReal - nTotalReceived;
								nReceived = (int) recv(nSocket, (void *)(pBuf + nTotalReceived), datatoget, 0);

								if (nReceived <= 0)
								{
									if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
									{
										break;
									}
									else
									{
										if (EAGAIN == errno)
										{
											usleep(1000);
											continue;
										}
										else
										{
											break;
										}
									}
								}
								else
								{
									if (nTotalReceived==0)
										APP_PROFILING_LOG(EVENT_APP_S_IN_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, nReceived);

									nTotalReceived += nReceived;
								}
							}
							pstV4lBuf->bytesused = nTotalReceived;
							nDataLen -=  nTotalReceived;
							if (nDataLen<=0)
								nNeedGetDataLen = 1;
						}
						else {
							/* socket failed or unCtrlCReceived */
						}
					}
					else
					{
						while (nTotalReceived  < (int)ulIOBlockSize)
						{
							int datatoget = ulIOBlockSize - nTotalReceived;
							if (datatoget > MAX_TCP_PACKET_LEN)
								datatoget = MAX_TCP_PACKET_LEN;
							nReceived = (int) recv(nSocket, (void *)(pBuf + nTotalReceived), datatoget, 0);

							if (nReceived <= 0)
							{
								if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
								{
									break;
								}
								else
								{
									if (EAGAIN == errno)
									{
										usleep(1000);
										continue;
									}
									else
									{
										break;
									}
								}
							}
							else
							{
								if (nTotalReceived==0)
									APP_PROFILING_LOG(EVENT_APP_S_IN_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, nReceived);
								nTotalReceived += nReceived;
							}

							if (nReceived < datatoget)
							{
								break;
							}
						}
						pstV4lBuf->bytesused = nTotalReceived;
					}
		            APP_PROFILING_LOG(EVENT_APP_S_IN_FINISH, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, pstV4lBuf->bytesused);
				}
				else if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_UDP_IN)
				{
					int	nReceived = 0;
					int	nTotalReceived = 0;
					socklen_t nAddrLen = 0;

		            APP_PROFILING_LOG(EVENT_APP_S_IN, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, 0);
					if (pComponent->ports[STREAM_DIR_IN].unPackData)
					{
						int ulIOBlockSizeReal;
						int nIOError = 0;

						if (nNeedGetDataLen)
						{
							while (nTotalReceived < 4)
							{
								int datatoget = 4 - nTotalReceived;
								nReceived = (int) recvfrom(nSocket,
											 (void *)(pBuf + nTotalReceived),
											 datatoget,
											 0,
											 (void *)&stFromAddr,
											 &nAddrLen
											 );

								if (nReceived <= 0)
								{
									if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
									{
										nIOError = 1;
										break;
									}
									else
									{
                                        if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
                                        {
                                            break;
                                        }
                                        else
                                        {
                                            usleep(1000);
                                            continue;
                                        }
									}
								}
								else
								{
									nTotalReceived += nReceived;
								}
							}
							if (nIOError==0)
								nDataLen = pBuf[0]<< 24 | pBuf[1]<<16 | pBuf[2] << 8 | pBuf[3];
						}
						if (nIOError==0)
						{
							if (nDataLen > ulIOBlockSize)
								ulIOBlockSizeReal = ulIOBlockSize;
							else
								ulIOBlockSizeReal = nDataLen;

							nTotalReceived = 0;
							while (nTotalReceived < (int)ulIOBlockSizeReal)
							{
								int datatoget = ulIOBlockSizeReal - nTotalReceived;
								nReceived = (int) recvfrom(nSocket,
													 (void *)(pBuf + nTotalReceived),
													 datatoget,
													 0,
													 (void *)&stFromAddr,
													 &nAddrLen
													 );

								if (nReceived <= 0)
								{
                                    if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
                                    {
                                        break;
                                    }
                                    else
                                    {
                                        usleep(1000);
                                        continue;
                                    }
								}
								else
								{
									if (nTotalReceived==0)
										APP_PROFILING_LOG(EVENT_APP_S_IN_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, nReceived);

									nTotalReceived += nReceived;
								}
							}
							pstV4lBuf->bytesused = nTotalReceived;
							nDataLen -= nTotalReceived;
							if (nDataLen<=0)
								nNeedGetDataLen = 1;
						}
						else {
							/* socket failed or unCtrlCReceived */
						}
					}
					else
					{
						while (nTotalReceived < (int)ulIOBlockSize)
						{
							int datatoget = ulIOBlockSize - nTotalReceived;
							if (datatoget > MAX_UDP_PACKET_LEN)
								datatoget = MAX_UDP_PACKET_LEN;
							nReceived = (int) recvfrom(nSocket,
												 (void *)(pBuf + nTotalReceived),
												 datatoget,
												 0,
												 (void *)&stFromAddr,
												 &nAddrLen
												 );

							if (nReceived <= 0)
							{
								if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
								{
									break;
								}
								else
								{
									usleep(1000);
									continue;
								}
							}
							if (nTotalReceived==0)
								APP_PROFILING_LOG(EVENT_APP_S_IN_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, nReceived);

							nTotalReceived += nReceived;

							if (nReceived < MAX_UDP_PACKET_LEN)
							{
								break;
							}
						}
						pstV4lBuf->bytesused = nTotalReceived;
					}
		            APP_PROFILING_LOG(EVENT_APP_S_IN_FINISH, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, pstV4lBuf->bytesused);
				}
				else if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_STREAM_IN)
				{
					int				nReceived = 0;
					int				nTotalReceived = 0;
					unsigned int	ulAddrLen = 0;

		            APP_PROFILING_LOG(EVENT_APP_S_IN, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, 0);
					if (pComponent->ports[STREAM_DIR_IN].unPackData)
					{
						int ulIOBlockSizeReal;
						int	nIOError = 0;

						if (nNeedGetDataLen)
						{
							while (nTotalReceived < 4)
							{
								int datatoget = 4 - nTotalReceived;
								nReceived = (int) recv(nSocket, (void *)(pBuf + nTotalReceived), datatoget, 0);

								if (nReceived <= 0)
								{
									if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
									{
										nIOError = 1;
										break;
									}
									else
									{
										if (EAGAIN == errno)
										{
											usleep(1000);
											continue;
										}
										else
										{
											nIOError = 1;
											break;
										}
									}
								}
								else
								{
									nTotalReceived += nReceived;
								}
							}
							if (nIOError==0)
								nDataLen = pBuf[0]<< 24 | pBuf[1]<<16 | pBuf[2] << 8 | pBuf[3];
						}
						if (nIOError==0)
						{
							if (nDataLen > ulIOBlockSize)
								ulIOBlockSizeReal = ulIOBlockSize;
							else
								ulIOBlockSizeReal = nDataLen;

							nTotalReceived = 0;
							while (nTotalReceived < (int)ulIOBlockSizeReal)
							{
								int datatoget = ulIOBlockSizeReal - nTotalReceived;
								nReceived = (int) recv(nSocket, (void *)(pBuf + nTotalReceived), datatoget, 0);

								if (nReceived <= 0)
								{
									if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
									{
										break;
									}
									else
									{
										if (EAGAIN == errno)
										{
											usleep(1000);
											continue;
										}
										else
										{
											break;
										}
									}
								}
								else
								{
									if (nTotalReceived==0)
										APP_PROFILING_LOG(EVENT_APP_S_IN_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, nReceived);

									nTotalReceived += nReceived;
								}
							}
							pstV4lBuf->bytesused = nTotalReceived;
							nDataLen -=  nTotalReceived;
							if (nDataLen<=0)
								nNeedGetDataLen = 1;
						}
						else {
							/* socket failed or unCtrlCReceived */
						}
					}
					else
					{
						while (nTotalReceived < (int)ulIOBlockSize)
						{
							int datatoget = ulIOBlockSize - nTotalReceived;
							if (datatoget > MAX_TCP_PACKET_LEN)
								datatoget = MAX_TCP_PACKET_LEN;
							nReceived = (int) recv(nSocket, (void *)(pBuf + nTotalReceived), datatoget, 0);

							if (nReceived <= 0)
							{
								if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
								{
									break;
								}
								else
								{
									if (EAGAIN == errno)
									{
										usleep(1000);
										continue;
									}
									else
									{
										break;
									}
								}
							}
							else
							{
								if (nTotalReceived==0)
									APP_PROFILING_LOG(EVENT_APP_S_IN_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, nReceived);
								nTotalReceived += nReceived;
							}

							if (nReceived < datatoget)
							{
								break;
							}
						}
						pstV4lBuf->bytesused = nTotalReceived;
		            }
		            APP_PROFILING_LOG(EVENT_APP_S_IN_FINISH, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, pstV4lBuf->bytesused);
				}
				else if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_DGRAM_IN)
				{
					int	nReceived = 0;
					int	nTotalReceived = 0;
					socklen_t nAddrLen = 0;
					struct sockaddr_un stFromUn;

		            APP_PROFILING_LOG(EVENT_APP_S_IN, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, 0);
					if (pComponent->ports[STREAM_DIR_IN].unPackData)
					{
						int ulIOBlockSizeReal;
						int nIOError = 0;

						if (nNeedGetDataLen)
						{
							while (nTotalReceived < 4)
							{
								int datatoget = 4 - nTotalReceived;
								nReceived = (int) recvfrom(nSocket,
											 (void *)(pBuf + nTotalReceived),
											 datatoget,
											 0,
											 (void *)&stFromUn,
											 &nAddrLen
											 );

								if (nReceived <= 0)
								{
									if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
									{
										nIOError = 1;
										break;
									}
									else
									{
                                        if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
                                        {
                                            break;
                                        }
                                        else
                                        {
                                            usleep(1000);
                                            continue;
                                        }
									}
								}
								else
								{
									nTotalReceived += nReceived;
								}
							}
							if (nIOError==0)
								nDataLen = pBuf[0]<< 24 | pBuf[1]<<16 | pBuf[2] << 8 | pBuf[3];
						}
						if (nIOError==0)
						{
							if (nDataLen > ulIOBlockSize)
								ulIOBlockSizeReal = ulIOBlockSize;
							else
								ulIOBlockSizeReal = nDataLen;

							nTotalReceived = 0;
							while (nTotalReceived < (int)ulIOBlockSizeReal)
							{
								int datatoget = ulIOBlockSizeReal - nTotalReceived;
								nReceived = (int) recvfrom(nSocket,
													 (void *)(pBuf + nTotalReceived),
													 datatoget,
													 0,
													 (void *)&stFromUn,
													 &nAddrLen
													 );

								if (nReceived <= 0)
								{
                                    if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
                                    {
                                        break;
                                    }
                                    else
                                    {
                                        usleep(1000);
                                        continue;
                                    }
								}
								else
								{
									if (nTotalReceived==0)
										APP_PROFILING_LOG(EVENT_APP_S_IN_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, nReceived);

									nTotalReceived += nReceived;
								}
							}
							pstV4lBuf->bytesused = nTotalReceived;
							nDataLen -= nTotalReceived;
							if (nDataLen<=0)
								nNeedGetDataLen = 1;
						}
						else {
							/* socket failed or unCtrlCReceived */
						}
					}
					else
					{
						while (nTotalReceived < (int)ulIOBlockSize)
						{
							int datatoget = ulIOBlockSize - nTotalReceived;
							if (datatoget > MAX_UDP_PACKET_LEN)
								datatoget = MAX_UDP_PACKET_LEN;
							nReceived = (int) recvfrom(nSocket,
												 (void *)(pBuf + nTotalReceived),
												 datatoget,
												 0,
												 (void *)&stFromUn,
												 &nAddrLen
												 );

							if (nReceived <= 0)
							{
								if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
								{
									break;
								}
								else
								{
									usleep(1000);
									continue;
								}
							}
							if (nTotalReceived==0)
								APP_PROFILING_LOG(EVENT_APP_S_IN_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, nReceived);

							nTotalReceived += nReceived;

							if (nReceived < datatoget)
							{
								break;
							}
						}
						pstV4lBuf->bytesused = nTotalReceived;
					}
		            APP_PROFILING_LOG(EVENT_APP_S_IN_FINISH, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, pstV4lBuf->bytesused);
				}
#ifndef CONFIG_HOST_PLATFORM_USERSPACE
				else if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_UVCRAW_IN)
				{
		            APP_PROFILING_LOG(EVENT_APP_S_IN, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, 0);
					if (unUvcFrameRepeatCnt == 0)
					{
						unsigned int unDataSize = ulInWidth*ulInHeight*2;
						if (uvc_read_frame(pucaUvcBuffer, &unDataSize) == 0)
						{
							YUV422ProgTo420Plane(pBuf, pucaUvcBuffer, ulInWidth, ulInHeight);
							pstV4lBuf->bytesused += ulIOBlockSize;
							if (pComponent->ports[STREAM_DIR_IN].unUvcFrameRepeat > 1)
							{
								memcpy(pucaUvcBuffer, (void*)pBuf, ulIOBlockSize);
							}
						}
					}
					else
					{
						memcpy((void*)pBuf, pucaUvcBuffer, ulIOBlockSize);
						pstV4lBuf->bytesused = ulIOBlockSize;
                    }
					unUvcFrameRepeatCnt = (unUvcFrameRepeatCnt + 1) % pComponent->ports[STREAM_DIR_IN].unUvcFrameRepeat;

#ifdef CONFIG_ENABLE_TESTAPPS_SDL
					if (pComponent->ports[STREAM_DIR_IN].unUvcPreview)
					{
						Plstream_DisplayFrame(ulInWidth,
											  ulInHeight,
											  (unsigned char *)pBuf,
											  _DISPLAY_ALL,
											  QP_YUV_DATA_TYPE_YV12
											  );
					}
#endif //CONFIG_ENABLE_TESTAPPS_SDL
					APP_PROFILING_LOG(EVENT_APP_S_IN_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, pstV4lBuf->bytesused);
		            APP_PROFILING_LOG(EVENT_APP_S_IN_FINISH, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, pstV4lBuf->bytesused);
				}
				else if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_UVCMJPEG_IN)
				{
					unsigned int unDataSize = stV4lBuf.length;
		            APP_PROFILING_LOG(EVENT_APP_S_IN, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, 0);
					if (uvc_read_frame(pBuf, &unDataSize) == 0)
					{
						pstV4lBuf->bytesused = unDataSize;
					}
#ifdef CONFIG_ENABLE_TESTAPPS_SDL
					if (pComponent->ports[STREAM_DIR_IN].unUvcPreview)
					{
						Plstream_DisplayFrame(ulInWidth,
											  ulInHeight,
											  (unsigned char *)pBuf,
											  _DISPLAY_ALL,
											  QP_YUV_DATA_TYPE_YV12
											  );
					}
#endif //CONFIG_ENABLE_TESTAPPS_SDL
					APP_PROFILING_LOG(EVENT_APP_S_IN_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, pstV4lBuf->bytesused);
		            APP_PROFILING_LOG(EVENT_APP_S_IN_FINISH, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, pstV4lBuf->bytesused);
				}
#endif // CONFIG_HOST_PLATFORM_USERSPACE
				else if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_FILE_IN)
				{
		            APP_PROFILING_LOG(EVENT_APP_S_IN, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, 0);
					pstV4lBuf->bytesused = (uint32_t)fread((void*)pBuf, 1, ulIOBlockSize, fpInput);
					APP_PROFILING_LOG(EVENT_APP_S_IN_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, pstV4lBuf->bytesused);
		            APP_PROFILING_LOG(EVENT_APP_S_IN_FINISH, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, pstV4lBuf->bytesused);
				}


				if ((pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_FILE_IN) &&
					(pstV4lBuf->bytesused != ulIOBlockSize)
					)
				{
#if 0
					/* EOF */
					pComponent->ports[STREAM_DIR_IN].unCtrlCReceived = 1;
#else
					if ((pComponent->ports[STREAM_DIR_IN].portType == COMPONENT_PORT_COMP_IN) ||
						(pComponent->ports[STREAM_DIR_IN].portType == COMPONENT_PORT_YUV_IN)
						)
					{
						fseek(fpInput, 0, SEEK_SET);
						//// goto RETRY;
					}
					else
					{
						if (pComponent->ports[STREAM_DIR_IN].unFileLoop)
                        {
                            fseek(fpInput, 0, SEEK_SET);
                            printf("%s() read file again\n", __FUNCTION__);
                        }
                        else
                            pComponent->ports[STREAM_DIR_IN].unCtrlCReceived = 1;
					}
#endif
				}

				if (pstV4lBuf->bytesused != 0)
				{
					memcpy(&stV4lBuf, pstV4lBuf, sizeof(struct v4l2_buffer));

					if (pComponent->ports[STREAM_DIR_IN].unUserPTS)
					{
                        /* assign PTS, top 31 bits of 33 bits in 90KHz measurement */
					    if ((pComponent->ports[STREAM_DIR_IN].unUvcFps>0) &&
					        ((pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_UVCMJPEG_IN)||
						     (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_UVCRAW_IN)))
                        {
                            ulPTS += (90000/pComponent->ports[STREAM_DIR_IN].unUvcFps)>>2;
                        }
                        else
                            ulPTS += (90000/30)>>2;
						ulPTS &= 0x7fffffff;
						stV4lBuf.reserved = ulPTS;
	                    stV4lBuf.flags |= QP_BUFDESC_FLAG_PTS;
					}
					else
					{
						/* not use PTS */
						ulPTS = 0;
						stV4lBuf.reserved = ulPTS;
	                    stV4lBuf.flags &= ~QP_BUFDESC_FLAG_PTS;
					}

					if (pComponent->ports[STREAM_DIR_IN].eMediaType2 == MEDIA_FILE_OUT)
					{
						if (fpOutputBackup)
                       		fwrite((void*)pBuf, 1, stV4lBuf.bytesused, fpOutputBackup);
					}

					/***********************************************
					** QBUF, put data to driver
					***********************************************/
					lErr = ioctl(pComponent->hDev, VIDIOC_QBUF, &stV4lBuf);
					if (lErr)
					{
			            APP_PROFILING_LOG(EVENT_APP_D_IN_FAILED, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, stV4lBuf.bytesused);
						printf("%s() QBUF ioctl failed %d %s\n", __FUNCTION__, errno, strerror(errno));
						if (errno == EAGAIN)
						{
							lErr = 0;
						}
					}
					else
					{
			            APP_PROFILING_LOG(EVENT_APP_D_IN, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, stV4lBuf.bytesused);
						printf(" decXferBufCnt[%lld]: %8u %8u 0x%x 0x%x \r", (long long) pComponent->hDev, ulXferBufCnt++, stV4lBuf.bytesused, stV4lBuf.reserved, stV4lBuf.flags);
						stAppV4lBuf[stV4lBuf.index].sent = 1;
					}
				}
				else
				{
					if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
					{
						printf("\n\n%s() CTRL+C received.\n", __FUNCTION__);
						break;
					}

					usleep(1000);
					goto RETRY;
				}

				fflush(stdout);
			}

			if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
			{
				printf("\n\n%s() CTRL+C received.\n", __FUNCTION__);
				break;
			}

			if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_FILE_IN)
			{
				usleep(30000);
			}
			else
			{
				usleep(1000);
			}
		}
    } //while(!lErr)


FUNC_END:

	printf("\n");

	usleep(10000);

	if (stAppV4lBuf)
	{
		for (nV4lBufCnt = 0; nV4lBufCnt < frame_nb; nV4lBufCnt++)
		{
			if (stAppV4lBuf[nV4lBufCnt].addr)
			{
				free(stAppV4lBuf[nV4lBufCnt].addr);
				stAppV4lBuf[nV4lBufCnt].addr = NULL;
			}
		}
		free(stAppV4lBuf);
	}

	if (fpInput)
	{
		fclose(fpInput);
	}

	if (fpOutputBackup)
	{
		fclose(fpOutputBackup);
	}

    if (nSocket != -1)
	{
		close(nSocket);
	}

#ifndef CONFIG_HOST_PLATFORM_USERSPACE
	if (videoInInited)
	{
		uvc_stop_capturing();
		uvc_uninit_device();
		uvc_close_device();
	}

    if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_UVCRAW_IN)
	{
		if (pucaUvcBuffer)
		{
			free(pucaUvcBuffer);
		}
	}
#endif // CONFIG_HOST_PLATFORM_USERSPACE
    pComponent->ports[STREAM_DIR_IN].unStarted = 0;

    //return lErr;
	printf("%s() ]\n", __FUNCTION__);
}



/////////////////////////////////////////////////////////////////////////////
//
//


void test_streamout(component_t *pComponent)
{
	int							lErr = 0;
#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
	int							fd = 0;
	unsigned char 				*pucaODirectBuffer = NULL;
	unsigned int				unODirectBufferOffset = 0;
#endif
	FILE						*fpOutput = NULL;
	FILE						*fpOutputBackup = NULL;
	FILE						*fpOutputDbg = NULL;
	FILE						*fdMp4vstts = 0, *fdMp4vstsz = 0, *fdMp4vstco = 0, *fdMp4vstsc = 0, *fdMp4astts = 0, *fdMp4astsz = 0, *fdMp4astco = 0, *fdMp4astsc = 0;
//	fpos_t filepos;
//	char						*FNamemp4=NULL;

	struct v4l2_capability		stCap;
	struct v4l2_requestbuffers	stReq;
	struct qlapp_v4l_buf_info	*stAppV4lBuf= 0;
	struct v4l2_buffer			stV4lBuf;
//#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
#if 0
	struct v4l2_buffer			stV4lBufDummy;
#endif
	int							nV4lBufCnt;

	int							nRetryCnt = 0;
	unsigned int				ulXferBufCnt = 0;
	unsigned int				ulXferFullBufCnt = 1;
	unsigned long 				ulXferRawByteCnt = 0;
	unsigned long 				ulXferOutByteCnt = 0;
	unsigned int                nXferFileCnt = 0;
	int							nFrameEnd = 1;
	unsigned int				unStopTime = 0;

	char						*pcaYUVBuff = NULL;
	unsigned int				ulYUVBuffSize;
	unsigned int				ulWidth;
	unsigned int				ulHeight;

	int							nSocket = -1;
    struct sockaddr_in			stToAddr;
	struct sockaddr_un 			stToUn;
    ps_mux_t					psState;
	unsigned char 				*pcaMP4CacheSps=NULL;
//    unsigned int				nAudioCount=0, nVideoCount=0, rCnt=0;
    QT_TRANSCODE_INFO 			*pqtinfo=NULL;

	char						*pstFName = NULL;
#ifdef	SUPPORT_HLS_FILE
	FILE						*fpHLSTag = 0;
    char				        *pstTemp = NULL;
	unsigned int				hlsIndex = 1;
	unsigned int				hlsIndexNum = 5;	//min 4
	unsigned int				hlsPeriod = 3;
	unsigned long 				pCurTag = 0;
	int						i;
	unsigned int 				roundFlag = 0;
	unsigned int				patFlag = 0;
	#define	HLS_MAX_NUM	999
	#define	HLS_EXP_NUM	2	//min 1

	unsigned long				curTime = 0;
	unsigned long				tmp = 0x004047;
	char *p = 0;
    char *key_code 			= (char *)&tmp;
	int 						nomemory = 0;
#endif

#ifdef	SUPPORT_VI_OSD
	QPCODEC_VIOSD pQLVIOSD;

	pComponent->ports[STREAM_DIR_OUT].gViuAddressOK = 0;
	pComponent->ports[STREAM_DIR_OUT].gViuosdOn = 0;

    if((pComponent->ports[STREAM_DIR_OUT].portType==COMPONENT_PORT_COMP_OUT) ||
      (pComponent->ports[STREAM_DIR_OUT].portType==COMPONENT_PORT_YUV_OUT))
    {
	    if (pComponent->ports[STREAM_DIR_OUT].unVIOSDOn)
	        pComponent->ports[STREAM_DIR_OUT].gViuosdOn = QLVIOSD_Init(pComponent->hDev, &pQLVIOSD);
    }

#endif

	printf("%s() [\n", __FUNCTION__);

	if (COMPONENT_TYPE_ENCODE == pComponent->devType)
	{
		ulWidth = pComponent->ulWidth;
		ulHeight = pComponent->ulHeight;
	}
	else
	{
		ulWidth = pComponent->decDispAttrib.outputSize.Attrib.x;
		ulHeight = pComponent->decDispAttrib.outputSize.Attrib.y;
	}

	stReq.count = 0;

    /***********************************************
    ** 1> Open output file descriptor
    ***********************************************/
    if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_TCP_OUT)
	{
		int	curFlags;
		int	nRetryCnt = 0;

		nSocket = socket(AF_INET, SOCK_STREAM, 0);
		if (nSocket < 0)
		{
			printf("%s() Failed to create socket.\n", __FUNCTION__);
			goto FUNC_END;
		}

		stToAddr.sin_addr.s_addr = inet_addr(pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr);
		stToAddr.sin_port = htons(pComponent->ports[STREAM_DIR_OUT].unPort);
		stToAddr.sin_family = AF_INET;

		curFlags = fcntl(nSocket, F_GETFL, 0);
		fcntl(nSocket, F_SETFL, curFlags | O_NONBLOCK);

		printf("%s() Connecting to %s .", __FUNCTION__, pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr);
		while (connect(nSocket, (void *)&stToAddr, sizeof(stToAddr)) < 0)
		{
			if (pComponent->ports[STREAM_DIR_OUT].unCtrlCReceived)
			{
				goto FUNC_END;
			}
			fflush(stdout);
			usleep(1000);
			if (nRetryCnt++ == 1000)
			{
				nRetryCnt = 0;
				printf(".");
			}
		}
		printf("\n%s() Got connected\n", __FUNCTION__);
	}
	else if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_UDP_OUT)
	{
		int	curFlags;

		nSocket = socket(AF_INET, SOCK_DGRAM, 0);
		if (nSocket < 0)
		{
            printf("%s() Failed to create socket.\n", __FUNCTION__);
			goto FUNC_END;
        }

		stToAddr.sin_addr.s_addr = inet_addr((const char *)pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr);
		stToAddr.sin_port = htons(pComponent->ports[STREAM_DIR_OUT].unPort);
		stToAddr.sin_family = AF_INET;

		curFlags = fcntl(nSocket, F_GETFL, 0);
		fcntl(nSocket, F_SETFL, curFlags | O_NONBLOCK);
    }
	else if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_STREAM_OUT)
	{
		int	curFlags;
		int	nRetryCnt = 0;
		int	nAddrLen;

		nSocket = socket(AF_UNIX, SOCK_STREAM, 0);
		if (nSocket < 0)
		{
			printf("%s() Failed to create socket.\n", __FUNCTION__);
			goto FUNC_END;
		}

		/* bind client address */
		unlink(pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr1);
		memset(&stToUn, 0, sizeof(stToUn));
		stToUn.sun_family = AF_UNIX;
		strncpy(stToUn.sun_path, pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr1, sizeof(stToUn.sun_path) - 1);
		nAddrLen = offsetof(struct sockaddr_un, sun_path) + (int) strlen(pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr1);

		curFlags = fcntl(nSocket, F_GETFL, 0);
		fcntl(nSocket, F_SETFL, curFlags | O_NONBLOCK);

		printf("%s() bind to %s .\n", __FUNCTION__, pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr1);
		if (bind(nSocket, (struct sockaddr *)&stToUn, nAddrLen) < 0)
		{
			printf("%s() Failed to bind socket. errno=%d\n", __FUNCTION__, errno);
			goto FUNC_END;
		}

		/* connect to server address */
		memset(&stToUn, 0, sizeof(stToUn));
		stToUn.sun_family = AF_UNIX;
		strncpy(stToUn.sun_path, pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr, sizeof(stToUn.sun_path) - 1);
		nAddrLen = offsetof(struct sockaddr_un, sun_path) + (int) strlen(pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr);

		curFlags = fcntl(nSocket, F_GETFL, 0);
		fcntl(nSocket, F_SETFL, curFlags | O_NONBLOCK);

		printf("%s() Connecting to %s .", __FUNCTION__, pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr);
		while (connect(nSocket, (struct sockaddr *)&stToUn, nAddrLen) < 0)
		{
			if (pComponent->ports[STREAM_DIR_OUT].unCtrlCReceived)
			{
				goto FUNC_END;
			}
			fflush(stdout);
			usleep(1000);
			if (nRetryCnt++ == 1000)
			{
				nRetryCnt = 0;
				printf(".");
			}
		}
		printf("\n%s() Got connected\n", __FUNCTION__);
	}
	else if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_DGRAM_OUT)
	{
		int	curFlags;
		int	nAddrLen;


		nSocket = socket(AF_UNIX, SOCK_DGRAM, 0);
		if (nSocket < 0)
		{
			printf("%s() Failed to create socket.\n", __FUNCTION__);
			goto FUNC_END;
		}

		/* bind client address */
		unlink(pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr1);
		memset(&stToUn, 0, sizeof(stToUn));
		stToUn.sun_family = AF_UNIX;
		strncpy(stToUn.sun_path, pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr1, sizeof(stToUn.sun_path) - 1);
		nAddrLen = offsetof(struct sockaddr_un, sun_path) + (int) strlen(pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr1);

		curFlags = fcntl(nSocket, F_GETFL, 0);
		fcntl(nSocket, F_SETFL, curFlags | O_NONBLOCK);

		printf("%s() bind to %s .\n", __FUNCTION__, pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr1);
		if (bind(nSocket, (struct sockaddr *)&stToUn, nAddrLen) < 0)
		{
			printf("%s() Failed to bind socket. errno=%d\n", __FUNCTION__, errno);
			goto FUNC_END;
		}

		memset(&stToUn, 0, sizeof(stToUn));
		stToUn.sun_family = AF_UNIX;
		strncpy(stToUn.sun_path, pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr, sizeof(stToUn.sun_path) - 1);
		nAddrLen = offsetof(struct sockaddr_un, sun_path) + (int) strlen(pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr);

#ifdef CONFIG_ENABLE_TESTAPPS_SDL
	}
	else if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_SDL_OUT)
	{
		if (Plstream_CreateOverlay(ulWidth, ulHeight))
		{
		 	goto FUNC_END;
		}
#endif //CONFIG_ENABLE_TESTAPPS_SDL
	}
	else if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_FILE_OUT)
	{
		if (pComponent->ports[STREAM_DIR_OUT].unJPEGSeparate)
		{
			/* only open when got data */
		}
		else
		{
#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
			if (g_unUseODirect) {
				unlink(pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr);
				fd = open(pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr, O_DIRECT | O_CREAT | O_RDWR | O_LARGEFILE);
				fpOutput = fdopen(fd, "w+b");

				pucaODirectBuffer = (unsigned char *)malloc(O_DIRECT_BUFFER_SIZE);
				if (pucaODirectBuffer==NULL)
				{
					nomemory = 1;
					lErr = 1;
					goto FUNC_END;
				}
				unODirectBufferOffset = 0;
			} else {
				fpOutput = fopen(pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr, "wb");
			}
#else
			fpOutput = fopen(pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr, "wb");
#endif
            if (fpOutput == NULL)
            {
                printf("%s() Unable to open file %s.\n", __FUNCTION__, pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr);
                lErr = 1;
                goto FUNC_END;
            }
		}
	}
#ifdef	SUPPORT_HLS_FILE
	else if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_HLS_FILE_OUT)
	{
		pstTemp = malloc(strlen(pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr)+30);
		pstFName = malloc(strlen(pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr)+30);
		if ((pstTemp==NULL)||(pstFName==NULL))
		{
			nomemory = 1;
			lErr = 1;
			goto FUNC_END;
		}
		strcpy(pstTemp, pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr);
		strcpy(pstFName, pstTemp);

		if(strrchr(pstTemp, '/') != 0)
			strcpy(pstFName, strrchr(pstTemp, '/')+1);
		printf("%s -%s\n", pstTemp, pstFName);

		strcat(pstTemp, ".m3u8");
		fpHLSTag = fopen(pstTemp, "w+");

		if (fpHLSTag == NULL)
		{
			printf("%s() Unable to open file %s.\n", __FUNCTION__, pstTemp);
			lErr = 1;
			goto FUNC_END;
		}

		sprintf(pstTemp, "%s_%03d.ts",pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr, hlsIndex);
		fpOutput = fopen(pstTemp, "w+");
		if (fpOutput == NULL)
		{
			printf("Warning::Unable to open file %s.\n", pstTemp);
			goto FUNC_END;
		}

		if (pComponent->ports[STREAM_DIR_OUT].unPort > 0)	//use port number for test only
			hlsPeriod = pComponent->ports[STREAM_DIR_OUT].unPort;

		i = pComponent->inputControl.Artesa.out_frame_rate/pComponent->gopLoopFltr.Artesa.intra_period;
		hlsPeriod = hlsPeriod*(i>1? i:1);

		sprintf(pstTemp,"#EXTM3U\n");
		fputs(pstTemp, fpHLSTag);
		sprintf(pstTemp,"#EXT-X-TARGETDURATION:%d\n", hlsPeriod);
		fputs(pstTemp, fpHLSTag);
		sprintf(pstTemp,"#EXT-X-ALLOW-CACHE:YES\n");
		fputs(pstTemp, fpHLSTag);

		pCurTag = ftell(fpHLSTag);
		curTime = time((time_t*)NULL);
	}
#endif
	else if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_MP4FILE_OUT)
	{
		if (pComponent->ports[STREAM_DIR_OUT].unSaveOriginal==1)
		{
			char *nametmp;
			nametmp = (char *) malloc(strlen(pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr)+10);
			if (nametmp==NULL)
			{
				printf("%s() LINE:%d: no memory\n", __FUNCTION__,__LINE__);
				lErr = 1;
				goto FUNC_END;
			}
			strcpy(nametmp, pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr);
			strcat(nametmp, ".ps");
			fpOutputDbg = fopen(nametmp, "wb");
			if (fpOutputDbg == NULL)
			{
				printf("%s() Unable to open mp4.ps file %s.\n", __FUNCTION__, nametmp);
				lErr = 1;
                free(nametmp);
				goto FUNC_END;
			}
			free(nametmp);
		}

		pstFName = malloc(strlen(pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr)+10);
		if (pstFName==NULL)
		{
			printf("%s() LINE:%d: no memory\n", __FUNCTION__,__LINE__);
			lErr = 1;
			goto FUNC_END;
		}
		strcpy(pstFName, pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr);
		pcaMP4CacheSps = malloc(0x80);
		pqtinfo = do_mp4_localinit(&fpOutput, &fdMp4vstts, &fdMp4vstsz, &fdMp4vstco, &fdMp4vstsc,
                &fdMp4astts, &fdMp4astsz, &fdMp4astco, &fdMp4astsc, pComponent, &psState, pstFName, 1);
		if (!pqtinfo || !fpOutput || !fdMp4vstts || !fdMp4vstsz || !fdMp4vstco || !fdMp4vstsc ||
                !fdMp4astts ||!fdMp4astsz || !fdMp4astco || !fdMp4astsc)
		{
		    printf("%s() Unable to do mp4 init\n", __FUNCTION__);
		    lErr = 1;
		    goto FUNC_END;
		}
	}
	else if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_NULL_OUT)
	{
		// output to null
	}
	else
	{
		printf("%s() Unknown media type %d.\n", __FUNCTION__, pComponent->ports[STREAM_DIR_OUT].eMediaType);
		goto FUNC_END;
	}

	if (pComponent->ports[STREAM_DIR_OUT].eMediaType2 == MEDIA_FILE_OUT)
	{
		fpOutputBackup = fopen(pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr2, "wb");
        if (fpOutputBackup == NULL)
        {
			printf("%s() Unable to open backup file %s.\n", __FUNCTION__, pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr2);
            lErr = 1;
            goto FUNC_END;
		}
	}

#ifdef CONFIG_ENABLE_MOTION_DETECT
	if(pComponent->ports[STREAM_DIR_OUT].portType == COMPONENT_PORT_ACTIVITY_OUT)
	{
		if(QLMD_Initial(pComponent, &pMDInfo)<0)
		{
			printf("Motion detect Initial failed\n");
			goto FUNC_END;
		}
		else if(QLMD_Config(pMDInfo, MD_CFG_FILE)<0)
		{
			printf("Motion detect config failed\n");
			goto FUNC_END;
		}
	}
#endif

	/***********************************************
	** 3> Create mmap
	***********************************************/
	stReq.count = pComponent->ports[STREAM_DIR_OUT].portType;
	stReq.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
	stReq.memory = V4L2_MEMORY_MMAP;
	lErr = ioctl(pComponent->hDev, VIDIOC_REQBUFS, &stReq);
	if (lErr)
	{
		printf("%s() V4L Request Buffer %d %s\n", __FUNCTION__, errno, strerror(errno));
		lErr = 3;
		goto FUNC_END;
	}
	else
	{
		stAppV4lBuf = malloc(stReq.count * sizeof(struct qlapp_v4l_buf_info));
		if (!stAppV4lBuf)
		{
			printf("%s() Unable to allocate memory for V4L app structure \n", __FUNCTION__);
			lErr = 4;
			goto FUNC_END;
		}

		for (nV4lBufCnt = 0; nV4lBufCnt < stReq.count; nV4lBufCnt++)
		{
			stV4lBuf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
			stV4lBuf.memory = V4L2_MEMORY_MMAP;
			stV4lBuf.sequence = pComponent->ports[STREAM_DIR_OUT].portType;
			stV4lBuf.index = nV4lBufCnt;

			lErr = ioctl(pComponent->hDev, VIDIOC_QUERYBUF, &stV4lBuf);
			if (!lErr)
			{
				stAppV4lBuf[nV4lBufCnt].size = stV4lBuf.length;
#ifndef CONFIG_HOST_PLATFORM_USERSPACE
				stAppV4lBuf[nV4lBufCnt].addr = mmap(0, stV4lBuf.length, PROT_READ | PROT_WRITE, MAP_SHARED, pComponent->hDev, stV4lBuf.m.offset);
#else
				//After do VIDIOC_QUERYBUF, the stV4lBuf.sequence will be clear to 0, and port type will be put in 4 most significant bits of stV4lBuf.m.offset
				stV4lBuf.sequence = pComponent->ports[STREAM_DIR_OUT].portType;
				stAppV4lBuf[nV4lBufCnt].addr = plv4l_mmap(pComponent->hDev, stV4lBuf.sequence, stV4lBuf.m.offset);
#endif
				if ((stAppV4lBuf[nV4lBufCnt].addr == (void *)0) || (stAppV4lBuf[nV4lBufCnt].addr == (void *)-1))
				{
					printf("%s() V4L mmap failed index=%d \n", __FUNCTION__, nV4lBufCnt);
					lErr = 5;
					goto FUNC_END;
				}
			}
			else
			{
				printf("%s() VIDIOC_QUERYBUF failed index=%d \n", __FUNCTION__, nV4lBufCnt);
				lErr = 5;
				goto FUNC_END;
			}
		}
	}

	/***********************************************
	** 5> Stream on
	***********************************************/
	while (!pComponent->ports[STREAM_DIR_OUT].unCtrlCReceived)
	{
		if (pComponent->ports[STREAM_DIR_OUT].unStarted)
		{
	        #ifdef CONFIG_ENABLE_TESTAPPS_USE_SELECT
	        //fprintf(stderr, "==test_streamout before out select t=%llu\n",getTimeUs());
			{
		        fd_set readfds;
		        struct timeval tv;
		        int r;

		        FD_ZERO (&readfds);
		        FD_SET (pComponent->hDev, &readfds);

		        /* Timeout. */
		        tv.tv_sec =  1;
		        tv.tv_usec = 0;

		        r = select ((pComponent->hDev) + 1, &readfds, NULL, NULL, &tv);
		        if (r<=0) {
		            //fprintf (stderr, "test_streamout out select timeout t=%llu\n",getTimeUs());
		            continue;
		        }
	            //fprintf(stderr, "test_streamout out select pass t=%llu\n",getTimeUs());
	            if (!FD_ISSET(pComponent->hDev, &readfds))
	            {
                    //fprintf(stderr, "test_streamout not ours %d\n",pComponent->hDev);
		            continue;
	            }
			}
			#endif
			/***********************************************
			** DQBUF, get buffer of output data from driver
			***********************************************/
			stV4lBuf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
			stV4lBuf.memory = V4L2_MEMORY_MMAP;
			stV4lBuf.sequence = pComponent->ports[STREAM_DIR_OUT].portType;

	        //APP_PROFILING_LOG(EVENT_APP_D_OUT_START, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt+1, 0);
			lErr = ioctl(pComponent->hDev, VIDIOC_DQBUF, &stV4lBuf);
			if (!lErr)
			{
				static unsigned long nPreviousPrintTime = 0;
				unsigned long nTime = 0;
				char *pBuf;

                nRetryCnt = 0;

				nTime = (unsigned long) time(NULL);

				ulXferBufCnt++;

				if (stV4lBuf.reserved & QP_KSSTREAM_HEADER_OPTIONSF_PREROLL ) {
                    APP_PROFILING_LOG(EVENT_APP_D_OUT_INCOMPLETE_FRAME, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
				}
	            APP_PROFILING_LOG(EVENT_APP_D_OUT, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);

				if ((!g_unReducePrintf) || ((nTime - nPreviousPrintTime) > 1)) {
					printf("\t\t\t\t\t\t\t encXferBufCnt[%lld]: %8u %8u %8u 0x%x f=0x%08x t=%lu\r", (long long) pComponent->hDev, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused, stV4lBuf.reserved, stV4lBuf.flags, nTime);
    	            /* stV4lBuf.flags: bit 31: PTS valid or not, bit 0-30: Put the PTS (90KHz) */
        	        /* stV4lBuf.reserved: Put the OptionsFlags field, e.g. 0x00000002 (QP_KSSTREAM_HEADER_OPTIONSF_PREROLL) means this frame need append with next frame to become a full frame */
					nPreviousPrintTime = nTime;
				}

				ulXferRawByteCnt += stV4lBuf.bytesused;

				pBuf = (char *) stAppV4lBuf[stV4lBuf.index].addr; //Load mapped address
#ifndef CONFIG_HOST_PLATFORM_ASPEN
				msync((void*)pBuf, stV4lBuf.bytesused, MS_SYNC);
#endif
                #if 0
				printf("Get Encodeddatalen=%d, F[%x %x %x %x %x %x] D[%02X %02X %02X %02X %02X %02X]\n",
				  stV4lBuf.bytesused, stV4lBuf.index, stV4lBuf.type, stV4lBuf.flags, stV4lBuf.sequence, stV4lBuf.length, stV4lBuf.reserved,
				  (unsigned char) pBuf[0],(unsigned char) pBuf[1],(unsigned char) pBuf[2], (unsigned char) pBuf[3],(unsigned char) pBuf[4],(unsigned char) pBuf[5]);
                #endif
				/* remove pending zero for word boundary if necessary */
				fix_data_packet((unsigned char *)pBuf, &stV4lBuf.bytesused, ulXferBufCnt, pComponent);

#ifdef CONFIG_ENABLE_PL_AUDIO_AGC
				if ((pComponent->ports[STREAM_DIR_OUT].gAudAgcOn) && pVAGCInfo )
					QLVAGC_AgcUpdate(pVAGCInfo, ulXferRawByteCnt);
#endif

#ifdef CONFIG_ENABLE_MOTION_DETECT
				if(pComponent->ports[STREAM_DIR_OUT].portType == COMPONENT_PORT_ACTIVITY_OUT)
				{
					QLMD_GetResult(pMDInfo, pBuf, stV4lBuf.bytesused);
				}
#endif
                if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_TCP_OUT)
				{
					int	nSent = 0;
					int	nTotalSent = 0;
					int	nToBeSent = 0;

			        //APP_PROFILING_LOG(EVENT_APP_S_OUT, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
					if (pComponent->ports[STREAM_DIR_OUT].unPackData)
					{
						unsigned char sbuf[5];
						sbuf[0] = (stV4lBuf.bytesused >> 24) & 0xFF;
						sbuf[1] = (stV4lBuf.bytesused >> 16) & 0xFF;
						sbuf[2] = (stV4lBuf.bytesused >> 8) & 0xFF;
						sbuf[3] = (stV4lBuf.bytesused) & 0xFF;
						while (nTotalSent < 4)
						{
							nToBeSent = 4 - nTotalSent;
							nSent = (int) send(nSocket, (void *) &sbuf[nTotalSent], nToBeSent, 0);
							if (nSent <= 0)
							{
								if (pComponent->ports[STREAM_DIR_OUT].unCtrlCReceived)
								{
									break;
								}
								else
								{
									if (nSent<0) /* error */
									{
										if ((errno==EAGAIN)||(errno==EINTR)||(errno==ENOBUFS))
											continue;
										else {	/* maybe the connect already broken? */
											usleep(10000);
										}
									}
									continue;
								}
							}
							nTotalSent += nSent;
						}
					}
					nTotalSent = 0;
					while (nTotalSent < (int)stV4lBuf.bytesused)
					{
						nToBeSent = stV4lBuf.bytesused - nTotalSent;
						if (nToBeSent > MAX_TCP_PACKET_LEN)
							nToBeSent = MAX_TCP_PACKET_LEN;
						nSent = (int) send(nSocket, (void *)(pBuf + nTotalSent), nToBeSent, 0);
						if (nSent <= 0)
						{
							if (pComponent->ports[STREAM_DIR_OUT].unCtrlCReceived)
							{
								break;
							}
							else
							{
								if (nSent<0) /* error */
								{
									if ((errno==EAGAIN)||(errno==EINTR)||(errno==ENOBUFS))
										continue;
									else {	/* maybe the connect already broken? */
										usleep(10000);
									}
								}
								continue;
							}
						}
						if (nTotalSent==0)
					        APP_PROFILING_LOG(EVENT_APP_S_OUT_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, nSent);
						nTotalSent += nSent;
					}
			        APP_PROFILING_LOG(EVENT_APP_S_OUT_FINISH, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
				}
				else if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_UDP_OUT)
				{
					int	nSent = 0;
					int	nTotalSent = 0;
					int	nToBeSent = 0;

			        //APP_PROFILING_LOG(EVENT_APP_S_OUT, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
					if (pComponent->ports[STREAM_DIR_OUT].unPackData)
					{
						unsigned char sbuf[5];
						sbuf[0] = (stV4lBuf.bytesused >> 24) & 0xFF;
						sbuf[1] = (stV4lBuf.bytesused >> 16) & 0xFF;
						sbuf[2] = (stV4lBuf.bytesused >> 8) & 0xFF;
						sbuf[3] = (stV4lBuf.bytesused) & 0xFF;
						while (nTotalSent < 4)
						{
							nToBeSent = 4 - nTotalSent;
							nSent = (int) sendto(nSocket,
										   (void *)&sbuf[nTotalSent],
										   nToBeSent,
										   0,
										   (void *)&stToAddr,
										   sizeof(struct sockaddr_in)
										   );
							if (nSent <= 0)
							{
								if (pComponent->ports[STREAM_DIR_OUT].unCtrlCReceived)
								{
									break;
								}
								else
								{
									if (nSent<0) /* error */
									{
										if ((errno==EAGAIN)||(errno==EINTR)||(errno==ENOBUFS))
											continue;
										else {	/* maybe the connect already broken? */
											usleep(10000);
										}
									}
									continue;
								}
							}
							nTotalSent += nSent;
						}
					}
					nTotalSent = 0;
					while (nTotalSent < (int)stV4lBuf.bytesused)
					{
						nToBeSent = stV4lBuf.bytesused - nTotalSent;
						if (nToBeSent > MAX_UDP_PACKET_LEN)
							nToBeSent = MAX_UDP_PACKET_LEN;
						nSent = (int) sendto(nSocket,
									   (void *)(pBuf + nTotalSent),
									   nToBeSent,
									   0,
									   (void *)&stToAddr,
									   sizeof(struct sockaddr_in)
									   );

						if (nSent <= 0)
						{
							if (pComponent->ports[STREAM_DIR_OUT].unCtrlCReceived)
							{
								break;
							}
							else
							{
								if (nSent<0) /* error */
								{
									if ((errno==EAGAIN)||(errno==EINTR)||(errno==ENOBUFS))
										continue;
									else {	/* maybe the connect already broken? */
										usleep(10000);
									}
								}
								continue;
							}
						}
						//printf("nSent=%d\n", nSent);
						if (nTotalSent==0)
					        APP_PROFILING_LOG(EVENT_APP_S_OUT_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, nSent);
						nTotalSent += nSent;
					}
			        APP_PROFILING_LOG(EVENT_APP_S_OUT_FINISH, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
				}
                else if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_STREAM_OUT)
				{
					int	nSent = 0;
					int	nTotalSent = 0;
					int	nToBeSent = 0;

			        //APP_PROFILING_LOG(EVENT_APP_S_OUT, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
					if (pComponent->ports[STREAM_DIR_OUT].unPackData)
					{
						unsigned char sbuf[5];
						sbuf[0] = (stV4lBuf.bytesused >> 24) & 0xFF;
						sbuf[1] = (stV4lBuf.bytesused >> 16) & 0xFF;
						sbuf[2] = (stV4lBuf.bytesused >> 8) & 0xFF;
						sbuf[3] = (stV4lBuf.bytesused) & 0xFF;
						while (nTotalSent < 4)
						{
							nToBeSent = 4 - nTotalSent;
							nSent = (int) send(nSocket, (void *)&sbuf[nTotalSent], nToBeSent, 0);
							if (nSent <= 0)
							{
								if (pComponent->ports[STREAM_DIR_OUT].unCtrlCReceived)
								{
									break;
								}
								else
								{
									if (nSent<0) /* error */
									{
										if ((errno==EAGAIN)||(errno==EINTR)||(errno==ENOBUFS))
											continue;
										else {	/* maybe the connect already broken? */
											usleep(10000);
										}
									}
									continue;
								}
							}
							nTotalSent += nSent;
						}
					}
					nTotalSent = 0;
					while (nTotalSent < (int)stV4lBuf.bytesused)
					{
						nToBeSent = stV4lBuf.bytesused - nTotalSent;
						if (!pComponent->ports[STREAM_DIR_OUT].unPackData)
						{
							if (nToBeSent > MAX_TCP_PACKET_LEN)
								nToBeSent = MAX_TCP_PACKET_LEN;
						}
						nSent = (int) send(nSocket, (void *)(pBuf + nTotalSent), nToBeSent, 0);

						if (nSent <= 0)
						{
							if (pComponent->ports[STREAM_DIR_OUT].unCtrlCReceived)
							{
								break;
							}
							else
							{
								if (nSent<0) /* error */
								{
									if ((errno==EAGAIN)||(errno==EINTR)||(errno==ENOBUFS))
										continue;
									else {	/* maybe the connect already broken? */
										usleep(10000);
									}
								}
								continue;
							}
						}
						if (nTotalSent==0)
					        APP_PROFILING_LOG(EVENT_APP_S_OUT_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, nSent);
						nTotalSent += nSent;
					}
			        APP_PROFILING_LOG(EVENT_APP_S_OUT_FINISH, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
				}
				else if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_DGRAM_OUT)
				{
					int	nSent = 0;
					int	nTotalSent = 0;
					int	nToBeSent = 0;
					int	nAddrLen;

					nAddrLen = offsetof(struct sockaddr_un, sun_path) + (int)strlen(pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr);

			        //APP_PROFILING_LOG(EVENT_APP_S_OUT, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
					if (pComponent->ports[STREAM_DIR_OUT].unPackData)
					{
						unsigned char sbuf[5];
						sbuf[0] = (stV4lBuf.bytesused >> 24) & 0xFF;
						sbuf[1] = (stV4lBuf.bytesused >> 16) & 0xFF;
						sbuf[2] = (stV4lBuf.bytesused >> 8) & 0xFF;
						sbuf[3] = (stV4lBuf.bytesused) & 0xFF;
						while (nTotalSent < 4)
						{
							nToBeSent = 4 - nTotalSent;
							nSent = (int) sendto(nSocket,
										   (void *)&sbuf[nTotalSent],
										   nToBeSent,
										   0,
										   (void *)&stToUn,
										   nAddrLen
										   );
							if (nSent <= 0)
							{
								if (pComponent->ports[STREAM_DIR_OUT].unCtrlCReceived)
								{
									break;
								}
								else
								{
									if (nSent<0) /* error */
									{
										if ((errno==EAGAIN)||(errno==EINTR)||(errno==ENOBUFS))
											continue;
										else {	/* maybe the connect already broken? */
											usleep(10000);
										}
									}
									continue;
								}
							}
							nTotalSent += nSent;
						}
					}
					nTotalSent = 0;
					while (nTotalSent < (int)stV4lBuf.bytesused)
					{
						nToBeSent = stV4lBuf.bytesused - nTotalSent;
						if (!pComponent->ports[STREAM_DIR_OUT].unPackData)
						{
							if (nToBeSent > MAX_UDP_PACKET_LEN)
								nToBeSent = MAX_UDP_PACKET_LEN;
						}
						nSent = (int) sendto(nSocket,
									   (void *)(pBuf + nTotalSent),
									   nToBeSent,
									   0,
									   (void *)&stToUn,
									   nAddrLen
									   );

						if (nSent <= 0)
						{
							if (pComponent->ports[STREAM_DIR_OUT].unCtrlCReceived)
							{
								break;
							}
							else
							{
								if (nSent<0) /* error */
								{
									if ((errno==EAGAIN)||(errno==EINTR)||(errno==ENOBUFS))
										continue;
									else {	/* maybe the connect already broken? */
										usleep(10000);
									}
								}
								continue;
							}
						}
						//printf("nSent=%d\n", nSent);
						if (nTotalSent==0)
					        APP_PROFILING_LOG(EVENT_APP_S_OUT_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, nSent);
						nTotalSent += nSent;
					}
			        APP_PROFILING_LOG(EVENT_APP_S_OUT_FINISH, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
#ifdef CONFIG_ENABLE_TESTAPPS_SDL
				}
				else if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_SDL_OUT)
				{
			        //APP_PROFILING_LOG(EVENT_APP_S_OUT, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
					if (pComponent->ports[STREAM_DIR_OUT].openFormat.yuv.nDataType==QP_YUV_DATA_TYPE_YUYV)
					{
						if (stV4lBuf.bytesused == (ulWidth * ulHeight * 2))
						{
							if (pcaYUVBuff==NULL)
							{
								pcaYUVBuff = (unsigned char*)malloc((ulWidth * ulHeight * 3)/2);
							}
							if (pcaYUVBuff)
							{
								YUV422ProgTo420Plane(pcaYUVBuff, pBuf, ulWidth, ulHeight);
								Plstream_DisplayFrame(ulWidth,
												  ulHeight,
												  (unsigned char *)pcaYUVBuff,
												  pComponent->ports[STREAM_DIR_OUT].displayType,
												  QP_YUV_DATA_TYPE_YV12
												  );
							}
						}
                    }
					else
					{
						if (stV4lBuf.bytesused == ((ulWidth * ulHeight * 3) / 2))
						{
								Plstream_DisplayFrame(ulWidth,
													  ulHeight,
													  (unsigned char *)pBuf,
													  pComponent->ports[STREAM_DIR_OUT].displayType,
													  pComponent->ports[STREAM_DIR_OUT].openFormat.yuv.nDataType
													  );
						}
					}
			        APP_PROFILING_LOG(EVENT_APP_S_OUT_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
			        APP_PROFILING_LOG(EVENT_APP_S_OUT_FINISH, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
#endif //CONFIG_ENABLE_TESTAPPS_SDL
				}
				else if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_FILE_OUT)
				{
			        //APP_PROFILING_LOG(EVENT_APP_S_OUT, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
					if (pComponent->ports[STREAM_DIR_OUT].unUpLimit)
					{	/* check if large then limit */
						char *pBufTmp;
						unsigned int len;
						int framebegin = 1;

						pBufTmp = pBuf;
						len = stV4lBuf.bytesused;

						if (nFrameEnd)
							framebegin = 1;
						else
							framebegin = 0;

						if ((stV4lBuf.reserved & 0x02)==0x02)
							nFrameEnd = 0;
						else
							nFrameEnd = 1;

						if ((framebegin)&&(ulXferOutByteCnt>=pComponent->ports[STREAM_DIR_OUT].unUpLimit))
						{
							/* need to seek header */
							int	ret;
                #if 0
				printf("Get Encodeddatalen= D[%02X %02X %02X %02X %02X %02X]\n",
				  (unsigned char) pBuf[0],(unsigned char) pBuf[1],(unsigned char) pBuf[2], (unsigned char) pBuf[3],(unsigned char) pBuf[4],(unsigned char) pBuf[5]);
                #endif
							ret = check_stream_header((unsigned char *)pBuf, stV4lBuf.bytesused, pComponent);
							if (ret>=0)
							{  	/* header found, close previous stream */
                                char *pszNameTemp;

								if (ret>0)
								{  /* there have some data belong to previous file */
									if (fpOutput) {
								#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
											if (g_unUseODirect) {
												memcpy(pucaODirectBuffer+unODirectBufferOffset, pBufTmp, ret);
												unODirectBufferOffset += ret;
												if (unODirectBufferOffset > O_DIRECT_BUFFER_THRESHOLD) {
													fwrite((void*)pucaODirectBuffer, 1, O_DIRECT_BUFFER_THRESHOLD, fpOutput);
													ulXferOutByteCnt += O_DIRECT_BUFFER_THRESHOLD;
													unODirectBufferOffset -= O_DIRECT_BUFFER_THRESHOLD;
													memcpy(pucaODirectBuffer, pucaODirectBuffer+O_DIRECT_BUFFER_THRESHOLD, unODirectBufferOffset);
												}
											} else {
				                	        	fwrite((void*)pBufTmp, 1, ret, fpOutput);
				                    	    	ulXferOutByteCnt += ret;
											}
								#else
					                        fwrite((void*)pBufTmp, 1, ret, fpOutput);
					                       	ulXferOutByteCnt += ret;
								#endif
											pBufTmp += ret;
											len -= ret;
									}
								}
								if (fpOutput)
									fclose(fpOutput);
								nXferFileCnt++;
								pszNameTemp = generate_filename_byindex(pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr, nXferFileCnt);
								printf("\n %s() LINE %d: Create Next File %s , pos=%lu\n", __FUNCTION__,__LINE__,
									pszNameTemp, ulXferOutByteCnt);
								fpOutput = fopen(pszNameTemp, "wb");
								free(pszNameTemp);
								pszNameTemp = NULL;
								ulXferOutByteCnt = 0;
							}
						}
						if ((fpOutput) && (len>0)) {
					#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
								if (g_unUseODirect) {
									memcpy(pucaODirectBuffer+unODirectBufferOffset, pBufTmp, len);
									unODirectBufferOffset += len;
									if (unODirectBufferOffset > O_DIRECT_BUFFER_THRESHOLD) {
										fwrite((void*)pucaODirectBuffer, 1, O_DIRECT_BUFFER_THRESHOLD, fpOutput);
										ulXferOutByteCnt += O_DIRECT_BUFFER_THRESHOLD;
										unODirectBufferOffset -= O_DIRECT_BUFFER_THRESHOLD;
										memcpy(pucaODirectBuffer, pucaODirectBuffer+O_DIRECT_BUFFER_THRESHOLD, unODirectBufferOffset);
									}
								} else {
	                	        	fwrite((void*)pBufTmp, 1, len, fpOutput);
	                    	    	ulXferOutByteCnt += len;
								}
					#else
		                        fwrite((void*)pBufTmp, 1, len, fpOutput);
		                       	ulXferOutByteCnt += len;
					#endif
						}
					}
					else
					{
						if (pComponent->ports[STREAM_DIR_OUT].unJPEGSeparate)
						{
						    char *pszNameTemp;
							pszNameTemp = generate_filename_byindex(pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr, ulXferBufCnt);
							fpOutput = fopen(pszNameTemp, "wb");
							if (pszNameTemp) {
								free(pszNameTemp);
								pszNameTemp = NULL;
							}
							ulXferOutByteCnt = 0;
						}
						if (fpOutput) {
	#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
							if (g_unUseODirect) {
								memcpy(pucaODirectBuffer+unODirectBufferOffset, pBuf, stV4lBuf.bytesused);
								unODirectBufferOffset += stV4lBuf.bytesused;
								if (unODirectBufferOffset > O_DIRECT_BUFFER_THRESHOLD) {
									fwrite((void*)pucaODirectBuffer, 1, O_DIRECT_BUFFER_THRESHOLD, fpOutput);
									ulXferOutByteCnt += O_DIRECT_BUFFER_THRESHOLD;
									unODirectBufferOffset -= O_DIRECT_BUFFER_THRESHOLD;
									memcpy(pucaODirectBuffer, pucaODirectBuffer+O_DIRECT_BUFFER_THRESHOLD, unODirectBufferOffset);
								}
							} else {
	                        	fwrite((void*)pBuf, 1, stV4lBuf.bytesused, fpOutput);
	                        	ulXferOutByteCnt += stV4lBuf.bytesused;
							}
	#else
	                        fwrite((void*)pBuf, 1, stV4lBuf.bytesused, fpOutput);
	                       	ulXferOutByteCnt += stV4lBuf.bytesused;
	#endif
						}
						if (pComponent->ports[STREAM_DIR_OUT].unJPEGSeparate)
						{
							if (fpOutput) {
	                            fclose(fpOutput);
		                        fpOutput = NULL;
							}
						}
					}
			        APP_PROFILING_LOG(EVENT_APP_S_OUT_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
			        APP_PROFILING_LOG(EVENT_APP_S_OUT_FINISH, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
				}
	#ifdef	SUPPORT_HLS_FILE
				else if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_HLS_FILE_OUT)
				{
			        //APP_PROFILING_LOG(EVENT_APP_S_OUT, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
					if(time((time_t*)NULL) - curTime >= hlsPeriod){	//sometime data not frame base???
						    size_t keylen = stV4lBuf.bytesused;
 							p = (char *)lfind(key_code, pBuf, &keylen, 1, compar);
							if (p != NULL){
								//printf("Find the key code %x-%x-%x-%x\n", *p, *(p+1),*(p+2),*(p+3));
								//printf("addr pBuf= %x p=%x\n", pBuf, p);

								patFlag = 1;
								if(fpOutput){
						            fwrite((void*)pBuf, 1, (p-pBuf), fpOutput);
									ulXferOutByteCnt += (p-pBuf);
									fflush(fpOutput);
									if(fclose(fpOutput) != 0) printf("Warning::Unable to close file no.%d\n",hlsIndex);
									fpOutput = NULL;
									printf("index=%d, period=%d \n",hlsIndex, hlsPeriod);
									curTime = time((time_t*)NULL);
								}
							}
					}

					if(fpOutput == NULL)
					{
						if(fpHLSTag)
						{
							if ((i = hlsIndex-hlsIndexNum) >= 0)
							{
								fseek(fpHLSTag, pCurTag, SEEK_SET);

								i = hlsIndex-hlsIndexNum+1;
								sprintf(pstTemp,"#EXT-X-MEDIA-SEQUENCE:%03d\n", i);
								fputs(pstTemp, fpHLSTag);

								for (i=i+1; i < hlsIndex; i++){
									sprintf(pstTemp, "#EXTINF:%d,\n", hlsPeriod);
									fputs(pstTemp, fpHLSTag);

									sprintf(pstTemp, "%s_%03d.ts\n", pstFName, i);
									fputs(pstTemp, fpHLSTag);
								}
								roundFlag = 0;
							}
							else if(roundFlag == 1)
							{
								fseek(fpHLSTag, pCurTag, SEEK_SET);

								i = hlsIndex + HLS_MAX_NUM -hlsIndexNum+1;
								sprintf(pstTemp,"#EXT-X-MEDIA-SEQUENCE:%03d\n", i);

								fputs(pstTemp, fpHLSTag);

								for (i=i+1; i <= HLS_MAX_NUM; i++){
									sprintf(pstTemp, "#EXTINF:%d,\n", hlsPeriod);
									fputs(pstTemp, fpHLSTag);

									sprintf(pstTemp, "%s_%03d.ts\n", pstFName, i);
									fputs(pstTemp, fpHLSTag);
								}

								for (i = 1; i < hlsIndex; i++){
									sprintf(pstTemp, "#EXTINF:%d,\n", hlsPeriod);
									fputs(pstTemp, fpHLSTag);

									sprintf(pstTemp, "%s_%03d.ts\n", pstFName, i);
									fputs(pstTemp, fpHLSTag);
								}
							}else{
								fseek(fpHLSTag, pCurTag, SEEK_SET);
								sprintf(pstTemp,"#EXT-X-MEDIA-SEQUENCE:%03d\n", 0);
								fputs(pstTemp, fpHLSTag);

								for (i = 1; i < hlsIndex; i++){
									sprintf(pstTemp, "#EXTINF:%d,\n", hlsPeriod);
									fputs(pstTemp, fpHLSTag);

									sprintf(pstTemp, "%s_%03d.ts\n", pstFName, i);
									fputs(pstTemp, fpHLSTag);
								}
							}

							sprintf(pstTemp, "#EXTINF:%d,\n", hlsPeriod);
							fputs(pstTemp, fpHLSTag);

							sprintf(pstTemp, "%s_%03d.ts\n", pstFName, hlsIndex);
							fputs(pstTemp, fpHLSTag);
							fflush(fpHLSTag);
						}

						//remove the expired file
						if((i = hlsIndex - hlsIndexNum) > HLS_EXP_NUM){
							sprintf(pstTemp, "%s_%03d.ts",pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr, i-HLS_EXP_NUM);
							if (access(pstTemp, F_OK) == 0)remove(pstTemp);
						}else{
							sprintf(pstTemp, "%s_%03d.ts",pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr, HLS_MAX_NUM + (i-HLS_EXP_NUM));
							if (access(pstTemp, F_OK) == 0)remove(pstTemp);
						}

						if(++hlsIndex > HLS_MAX_NUM){
							hlsIndex = 1;
							roundFlag = 1;
						}

						sprintf(pstTemp, "%s_%03d.ts",pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr, hlsIndex);
						fpOutput = fopen(pstTemp, "w+");
						if (fpOutput == NULL)
						{
							printf("Warning::Unable to open file %s.\n", pstTemp);
							goto FUNC_END;
						}
					}

					if(fpOutput)
					{
						if(patFlag == 1){
               				fwrite((void*)p, 1, (stV4lBuf.bytesused-(p-pBuf)), fpOutput);
							ulXferOutByteCnt += (stV4lBuf.bytesused-(p-pBuf));
							patFlag = 0;
						}else
               				fwrite((void*)pBuf, 1, stV4lBuf.bytesused, fpOutput);
							ulXferOutByteCnt += stV4lBuf.bytesused;
					}
			        APP_PROFILING_LOG(EVENT_APP_S_OUT_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
			        APP_PROFILING_LOG(EVENT_APP_S_OUT_FINISH, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
				}
	#endif

				else if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_MP4FILE_OUT)
				{
			        //APP_PROFILING_LOG(EVENT_APP_S_OUT, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
					if (pComponent->ports[STREAM_DIR_OUT].unSaveOriginal==1)
					{
           				if (fpOutputDbg)
           					fwrite((void*)pBuf, 1, stV4lBuf.bytesused, fpOutputDbg);
					}

					if (pComponent->ports[STREAM_DIR_OUT].unUpLimit)
					{	/* check if large then limit */
						int framebegin;

						if (nFrameEnd)
							framebegin = 1;
						else
							framebegin = 0;

						if ((stV4lBuf.reserved & 0x02)==0x02)
							nFrameEnd = 0;
						else
							nFrameEnd = 1;

						if ((framebegin)&&(ulXferOutByteCnt>=pComponent->ports[STREAM_DIR_OUT].unUpLimit))
						{
							/* need to seek header */
							int	ret;
							ret = check_stream_header((unsigned char *)pBuf, stV4lBuf.bytesused, pComponent);
							if (ret<0)
							{
								fprintf(stderr,"%s() LINE %d: not beginning\n", __FUNCTION__,__LINE__);
							}
							else
							{  	/* header found, close previous stream */
								ret = do_mp4_localfinal(&fpOutput, &fdMp4vstts, &fdMp4vstsz, &fdMp4vstco, &fdMp4vstsc,
						                &fdMp4astts, &fdMp4astsz, &fdMp4astco, &fdMp4astsc, pComponent, &psState, pqtinfo, pstFName);
								if (pqtinfo && ret) {
									free(pqtinfo);
									pqtinfo = NULL;
								}
								else  {
									/* pqtinfo 's memory is already free in mp4_fini() */
									pqtinfo = NULL;
								}
								//if (pqtinfo || fpOutput || fdMp4vstts || fdMp4vstsz || fdMp4vstco || fdMp4vstsc ||
					            //    fdMp4astts || fdMp4astsz || fdMp4astco || fdMp4astsc)
								//{
								//    printf("\n\n %s() LINE %d: Unable to do mp4 finalize!!\n", __FUNCTION__,__LINE__);
								//}
								if (pstFName) {
									free(pstFName);
									pstFName = NULL;
								}
								if (fpOutput)
									fclose(fpOutput);
								nXferFileCnt++;
								pstFName = generate_filename_byindex(pComponent->ports[STREAM_DIR_OUT].pszNameOrAddr, nXferFileCnt);
								printf("\n %s() LINE %d: Create Next MP4 File %s , pos=%lu\n", __FUNCTION__,__LINE__, pstFName, ulXferOutByteCnt);
								pqtinfo = do_mp4_localinit(&fpOutput, &fdMp4vstts, &fdMp4vstsz, &fdMp4vstco, &fdMp4vstsc,
					                &fdMp4astts, &fdMp4astsz, &fdMp4astco, &fdMp4astsc, pComponent, &psState, pstFName, 0);
								if (!pqtinfo)
								{
								    printf("%s() Unable to do mp4 init\n", __FUNCTION__);
								}
								ulXferOutByteCnt = 0;
							}
						}
					}

					do_mp4_process_data((unsigned char *)pBuf, stV4lBuf.bytesused, pComponent, &psState, pqtinfo, pcaMP4CacheSps);
					ulXferOutByteCnt += stV4lBuf.bytesused;
			        APP_PROFILING_LOG(EVENT_APP_S_OUT_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
			        APP_PROFILING_LOG(EVENT_APP_S_OUT_FINISH, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
				}

				else if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_NULL_OUT)
				{
			        //APP_PROFILING_LOG(EVENT_APP_S_OUT, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
			        APP_PROFILING_LOG(EVENT_APP_S_OUT_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
			        APP_PROFILING_LOG(EVENT_APP_S_OUT_FINISH, (TQP_UINT32) pComponent->hTask, pComponent->devType, ulXferFullBufCnt, ulXferBufCnt, stV4lBuf.bytesused);
				}

				if (pComponent->ports[STREAM_DIR_OUT].eMediaType2 == MEDIA_FILE_OUT)
				{
					if (fpOutputBackup)
                        fwrite((void*)pBuf, 1, stV4lBuf.bytesused, fpOutputBackup);
				}

				if (stV4lBuf.reserved & QP_KSSTREAM_HEADER_OPTIONSF_PREROLL ) {
				}
				else {
					ulXferFullBufCnt++;
				}
				/***********************************************
				** QBUF, put empty buffer back to driver
				***********************************************/
//#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
#if 0
				/* to work-around the problem that cause driver error "QBUF unknown sequence" */
				memcpy((unsigned char *)&stV4lBufDummy, (unsigned char *)&stV4lBuf, sizeof(struct v4l2_buffer));
#endif
				lErr = ioctl(pComponent->hDev, VIDIOC_QBUF, &stV4lBuf);
				if(lErr)
				{
					if (errno == EAGAIN)
					{
						lErr = 0;
					}
					else
					{
						printf("%s() QBUF failed lErr = %d, errno = %d(%s)\n", __FUNCTION__, lErr, errno,strerror(errno));
					}
				}

				fflush(stdout);
			}
			else
			{
				if (errno == EAGAIN)
				{
		            //APP_PROFILING_LOG(EVENT_APP_D_OUT_NODATA, (TQP_UINT32) pComponent->hDev, pComponent->devType, 0, 0, 0);
					lErr = 0;
				}
				else
				{
		            APP_PROFILING_LOG(EVENT_APP_D_OUT_FAILED, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, 0, 0);
					//printf("\n%s()  DQBUF failed(%d) errno(%d)\n", __FUNCTION__, lErr, errno);
				}
				usleep(1000);
			}

			if (pComponent->ports[STREAM_DIR_OUT].unCtrlCReceived)
			{
				printf("\n\n%s() CTRL+C received.\n", __FUNCTION__);
				break;
			}
#ifdef	SUPPORT_VI_OSD
			if(pComponent->ports[STREAM_DIR_OUT].gViuosdOn)
			{
				if(pComponent->ports[STREAM_DIR_OUT].gViuAddressOK == 0)
		    	{
//		    		printf("VI-OSD \n");
		    		printf("VI-OSD %d \n", g_unCtrlCReceived );
		    		pComponent->ports[STREAM_DIR_OUT].gViuAddressOK = QLVIOSD_Start(pComponent->hDev, &pQLVIOSD);
		    		if(pComponent->ports[STREAM_DIR_OUT].gViuAddressOK == 0xff)
		    		{
		    		    pComponent->ports[STREAM_DIR_OUT].gViuosdOn = 0;
		    		    pComponent->ports[STREAM_DIR_OUT].gViuAddressOK = 0;
		    		    QLVIOSD_Stop(pComponent->hDev, &pQLVIOSD);
		    		}
				}
				else
				{
				    QLVIOSD_Service(pComponent->hDev, &pQLVIOSD);
				}
			}
#endif
		}
		else
		{
			usleep(30000);
		//	printf("pComponent->ports[STREAM_DIR_OUT].unStarted = %d\n", pComponent->ports[STREAM_DIR_OUT].unStarted);
		}
	} //while(!lErr)


FUNC_END:

	printf("\n");

	usleep(10000);

#ifdef	SUPPORT_VI_OSD
	if(pComponent->ports[STREAM_DIR_OUT].gViuosdOn)
	{
		pComponent->ports[STREAM_DIR_OUT].gViuosdOn = 0;
		QLVIOSD_Stop(pComponent->hDev, &pQLVIOSD);
	}
#endif
	if (stAppV4lBuf)
	{
#ifndef CONFIG_HOST_PLATFORM_USERSPACE
		for(nV4lBufCnt = 0; nV4lBufCnt < stReq.count; nV4lBufCnt++)
		{
			if (stAppV4lBuf[nV4lBufCnt].addr)
			{
				munmap(stAppV4lBuf[nV4lBufCnt].addr, stAppV4lBuf[nV4lBufCnt].size);
			}
		}
#endif
		free(stAppV4lBuf);
	}

	if (pComponent->ports[STREAM_DIR_OUT].eMediaType == MEDIA_MP4FILE_OUT)
	{
        int	ret;
		ret = do_mp4_localfinal(&fpOutput, &fdMp4vstts, &fdMp4vstsz, &fdMp4vstco, &fdMp4vstsc,
                &fdMp4astts, &fdMp4astsz, &fdMp4astco, &fdMp4astsc, pComponent, &psState, pqtinfo, pstFName);
		if (pqtinfo && ret) {
			free(pqtinfo);
			pqtinfo = NULL;
		}
		else  {
			/* pqtinfo 's memory is already free in mp4_fini() */
			pqtinfo = NULL;
		}
		if (pcaMP4CacheSps)
        {
			free(pcaMP4CacheSps);
            pcaMP4CacheSps = NULL;
        }

		//if (pqtinfo || fpOutput || fdMp4vstts || fdMp4vstsz || fdMp4vstco || fdMp4vstsc ||
        //        fdMp4astts || fdMp4astsz || fdMp4astco || fdMp4astsc)
		//{
		//    printf("\n\n %s() Unable to do mp4 finalize!!\n", __FUNCTION__);
		//}
		if (pstFName) {
			free(pstFName);
			pstFName = NULL;
		}
	}

#ifdef CONFIG_ENABLE_MOTION_DETECT
	if(pMDInfo)
	{
		QLMD_Release(pMDInfo);
	}
#endif

#ifdef CONFIG_ENABLE_PL_AUDIO_AGC
	if ((pComponent->ports[STREAM_DIR_OUT].gAudAgcOn) && pVAGCInfo)
	{
	    QLVAGC_AgcOnOff(pVAGCInfo, 0);
		QLVAGC_HwClose(pVAGCInfo);
		QLVAGC_Release(pVAGCInfo);
		pVAGCInfo = NULL;
	}
#endif

#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
	if (pucaODirectBuffer) {
		if (unODirectBufferOffset) {
			if (fpOutput) {
				fwrite((void*)pucaODirectBuffer, 1, unODirectBufferOffset, fpOutput);
				ulXferOutByteCnt += unODirectBufferOffset;
			}
		}
		free(pucaODirectBuffer);
	};
#endif
    
    if (pcaMP4CacheSps)
    {
        free(pcaMP4CacheSps);
    }

	if (fpOutput)
	{
		fclose(fpOutput);
	}

	if (fpOutputBackup)
	{
		fclose(fpOutputBackup);
	}

	if (fpOutputDbg)
	{
		fclose(fpOutputDbg);
	}

    if (nSocket != -1)
	{
		close(nSocket);
	}

	if (pcaYUVBuff)
	{
		free(pcaYUVBuff);
	}

#ifdef	SUPPORT_HLS_FILE
	if(fpHLSTag)
	{
		sprintf(pstTemp,"#EXT-X-ENDLIST\n");
		fputs(pstTemp, fpHLSTag);
		fclose(fpHLSTag);
	}
	if(pstTemp)
		free(pstTemp);
	if(pstFName)
		free(pstFName);
#endif
//	if (FNamemp4)
//		free(FNamemp4);

	pComponent->ports[STREAM_DIR_OUT].unStarted = 0;

	printf("%s() ]\n", __FUNCTION__);


    //return lErr;
}



void test_streamin(component_t *pComponent)
{
	int							lErr = 0;

	FILE						*fpInput = 0;
	FILE						*fpOutputBackup = NULL;

	int							videoInInited = 0;

	int							nType = V4L2_BUF_TYPE_VIDEO_CAPTURE;

	struct v4l2_capability		stCap;
	struct v4l2_requestbuffers	stReq;
	struct qlapp_v4l_buf_info	*stAppV4lBuf = 0;
	struct v4l2_buffer			stV4lBuf;
	int							nV4lBufCnt;

	int							nRetryCnt = 0;
	unsigned int				ulXferBufCnt = 0;

	unsigned int				unStopTime = 0;

	unsigned int				ulIOBlockSize = 64 * 1024;

	unsigned int				ulInWidth, ulInHeight;
	unsigned int				ulOutWidth, ulOutHeight;

	int							nSocket = -1;
	struct sockaddr_in			stFromAddr;

	unsigned char				*pucaUvcBuffer = NULL;

	unsigned int				unUvcFrameRepeatCnt = 0;

	unsigned int				ulPTS = 0;
	int							nNeedGetDataLen = 1;
	int							nDataLen = 0;

	printf("%s() [\n", __FUNCTION__);

	if (COMPONENT_TYPE_ENCODE == pComponent->devType)
	{
		ulInWidth = pComponent->picResolution.Artesa.in_pic_width;
		ulInHeight = pComponent->picResolution.Artesa.in_pic_height;

		ulOutWidth = pComponent->outResolution.out_pic_width;
		ulOutHeight = pComponent->outResolution.out_pic_height;
	}
	else
	{
		ulOutWidth = pComponent->decDispAttrib.outputSize.Attrib.x;
		ulOutHeight = pComponent->decDispAttrib.outputSize.Attrib.y;
		ulInWidth = ulOutWidth;
		ulInHeight = ulOutHeight;
	}

	if (COMPONENT_PORT_YUV_IN == pComponent->ports[STREAM_DIR_IN].portType)
	{
		ulIOBlockSize = (ulInWidth * ulInHeight * 3) / 2;
	}
	else
	{
		ulIOBlockSize = 256 * 1024;
	}


	/***********************************************
	** 1> Open output file descriptor
	***********************************************/
	if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_TCP_IN)
	{
		int				curFlags;
		int				nListenSocket;
		fd_set			readset;
		struct timeval	timeout;
		socklen_t		nAddrLen = 0;
		int				nReuseFlag = 1;

		nListenSocket = socket(AF_INET, SOCK_STREAM, 0);
		if (nListenSocket < 0)
		{
			printf("%s() Failed to create socket.\n", __FUNCTION__);
			goto FUNC_END;
		}

		if (setsockopt(nListenSocket, SOL_SOCKET, SO_REUSEADDR, (const char*)&nReuseFlag, sizeof nReuseFlag) < 0)
		{
			printf("%s() Failed to setsockopt(SO_REUSEADDR).\n", __FUNCTION__);
			goto FUNC_END;
		}

		stFromAddr.sin_addr.s_addr = htonl(INADDR_ANY);
		stFromAddr.sin_port = htons(pComponent->ports[STREAM_DIR_IN].unPort);
		stFromAddr.sin_family = AF_INET;

		curFlags = fcntl(nListenSocket, F_GETFL, 0);
		fcntl(nListenSocket, F_SETFL, curFlags | O_NONBLOCK);

		if (bind(nListenSocket, (void *)&stFromAddr, sizeof(stFromAddr)) < 0)
		{
			printf("%s() Failed to bind socket. errno=%d\n", __FUNCTION__, errno);
			goto FUNC_END;
		}

		if (listen(nListenSocket, 1) < 0)
		{
			printf("%s() Failed to listen. errno=%d\n", __FUNCTION__, errno);
			goto FUNC_END;
		}

		printf("%s() Waiting for connection .", __FUNCTION__);
		fflush(stdout);

		while (1)
		{
			FD_ZERO(&readset);
			FD_SET(nListenSocket, &readset);

			timeout.tv_sec = 1;
			timeout.tv_usec = 0;

			if (select(nListenSocket + 1, &readset, NULL, NULL, &timeout) >= 0)
			{
				if (FD_ISSET(nListenSocket, &readset))
				{
					nSocket = accept(nListenSocket, (void *)&stFromAddr, &nAddrLen);

					curFlags = fcntl(nSocket, F_GETFL, 0);
					fcntl(nSocket, F_SETFL, curFlags | O_NONBLOCK);

					printf("\n%s() Got connection from %s. \n", __FUNCTION__, inet_ntoa(stFromAddr.sin_addr));
					break;
                }
            }
			if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
			{
				goto FUNC_END;
			}
			printf(".");
			fflush(stdout);
			usleep(1000);
		}
		fflush(stdout);
	}
#ifndef CONFIG_HOST_PLATFORM_USERSPACE
	else if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_UDP_IN)
	{
		int	curFlags;

		nSocket = socket(AF_INET, SOCK_DGRAM, 0);
		if (nSocket < 0)
		{
			printf("%s() Failed to create socket.\n", __FUNCTION__);
			goto FUNC_END;
		}
		stFromAddr.sin_addr.s_addr = htonl(INADDR_ANY);
		stFromAddr.sin_port = htons(pComponent->ports[STREAM_DIR_IN].unPort);
		stFromAddr.sin_family = AF_INET;

		curFlags = fcntl(nSocket, F_GETFL, 0);
		fcntl(nSocket, F_SETFL, curFlags | O_NONBLOCK);

		if (bind(nSocket, (void *)&stFromAddr, sizeof(stFromAddr)) < 0)
		{
			printf("%s() Failed to bind socket. errno=%d\n", __FUNCTION__, errno);
			goto FUNC_END;
		}
	}
	else if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_STREAM_IN)
	{
		int				curFlags;
		int				nListenSocket;
		fd_set			readset;
		struct timeval	timeout;
		int				nAddrLen = 0;
		struct sockaddr_un stFromUn;
		struct stat statbuf;

		nListenSocket = socket(AF_UNIX, SOCK_STREAM, 0);
		if (nListenSocket < 0)
		{
			printf("%s() Failed to create domain socket.\n", __FUNCTION__);
			goto FUNC_END;
		}

		unlink(pComponent->ports[STREAM_DIR_IN].pszNameOrAddr);
		memset(&stFromUn, 0, sizeof(stFromUn));
		stFromUn.sun_family = AF_UNIX;
		strncpy(stFromUn.sun_path, pComponent->ports[STREAM_DIR_IN].pszNameOrAddr, sizeof(stFromUn.sun_path) - 1);
		nAddrLen = offsetof(struct sockaddr_un, sun_path) + strlen(pComponent->ports[STREAM_DIR_IN].pszNameOrAddr);

		curFlags = fcntl(nListenSocket, F_GETFL, 0);
		fcntl(nListenSocket, F_SETFL, curFlags | O_NONBLOCK);

		if (bind(nListenSocket, (struct sockaddr *)&stFromUn, nAddrLen) < 0)
		{
			printf("%s() Failed to bind socket. errno=%d\n", __FUNCTION__, errno);
			goto FUNC_END;
		}

		if (listen(nListenSocket, 1) < 0)
		{
			printf("%s() Failed to listen. errno=%d\n", __FUNCTION__, errno);
			goto FUNC_END;
		}

		printf("%s() Waiting for connection .", __FUNCTION__);
		fflush(stdout);

		while (1)
		{
			FD_ZERO(&readset);
			FD_SET(nListenSocket, &readset);

			timeout.tv_sec = 1;
			timeout.tv_usec = 0;

			if (select(nListenSocket + 1, &readset, NULL, NULL, &timeout) >= 0)
			{
				if (FD_ISSET(nListenSocket, &readset))
				{
					nAddrLen = sizeof(struct sockaddr_un); // this is important point for linux */
					nSocket = accept(nListenSocket, (struct sockaddr *)&stFromUn, &nAddrLen);
					curFlags = fcntl(nSocket, F_GETFL, 0);
					fcntl(nSocket, F_SETFL, curFlags | O_NONBLOCK);

					/* optional, obtain the client's uid from its calling address */
					nAddrLen -= offsetof(struct sockaddr_un, sun_path);  /* len of pathname */
					stFromUn.sun_path[nAddrLen] = 0; /* null terminate string */
					if (stat(stFromUn.sun_path, &statbuf) < 0)
					{
						/* error */
						close(nSocket);
						printf("\n%s() Got wrong connection from %s, retry it\n", __FUNCTION__, stFromUn.sun_path);
						continue;
					}
					else {
						if (S_ISSOCK(statbuf.st_mode) )
						{
							//if (uidptr != NULL) *uidptr = statbuf.st_uid;    /* return uid of caller */
							unlink(stFromUn.sun_path);       /* we're done with pathname now */
							printf("\n%s() Got connection from %s. \n", __FUNCTION__, stFromUn.sun_path);
							close(nListenSocket); // close listen socket
							break;
						}
						close(nSocket);
						printf("\n%s() Got wrong connection from %s, not a socket, retry it\n", __FUNCTION__, stFromUn.sun_path);
						continue;
					}
                }
            }
			if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
			{
				goto FUNC_END;
			}
			printf(".");
			fflush(stdout);
			usleep(1000);
		}
		fflush(stdout);
	}
	else if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_DGRAM_IN)
	{
		int	curFlags;
		struct sockaddr_un stFromUn;
		int nAddrLen;

		nSocket = socket(AF_UNIX, SOCK_DGRAM, 0);
		if (nSocket < 0)
		{
			printf("%s() Failed to create socket.\n", __FUNCTION__);
			goto FUNC_END;
		}

		unlink(pComponent->ports[STREAM_DIR_IN].pszNameOrAddr);
		memset(&stFromUn, 0, sizeof(stFromUn));
		stFromUn.sun_family = AF_UNIX;
		strncpy(stFromUn.sun_path, pComponent->ports[STREAM_DIR_IN].pszNameOrAddr, sizeof(stFromUn.sun_path) - 1);
		nAddrLen = offsetof(struct sockaddr_un, sun_path) + strlen(pComponent->ports[STREAM_DIR_IN].pszNameOrAddr);

		curFlags = fcntl(nSocket, F_GETFL, 0);
		fcntl(nSocket, F_SETFL, curFlags | O_NONBLOCK);

		if (bind(nSocket, (struct sockaddr *)&stFromUn, sizeof(stFromUn)) < 0)
		{
			printf("%s() Failed to bind socket. errno=%d\n", __FUNCTION__, errno);
			goto FUNC_END;
		}
	}
	else if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_UVCRAW_IN)
	{
		if (pComponent->ports[STREAM_DIR_IN].unUvcFrameRepeat < 1)
		{
            pComponent->ports[STREAM_DIR_IN].unUvcFrameRepeat = 1;
		}

		if (pComponent->ports[STREAM_DIR_IN].unUvcFps < 1)
		{
			pComponent->ports[STREAM_DIR_IN].unUvcFps = 30;
		}

		if ((lErr = uvc_open_device(pComponent->ports[STREAM_DIR_IN].pszNameOrAddr)) < 0)
		{
			goto FUNC_END;
		}
		else
		{
			videoInInited = 1;
		}

		if ((lErr = uvc_init_device(V4L2_PIX_FMT_YUYV, ulInWidth, ulInHeight, pComponent->ports[STREAM_DIR_IN].unUvcFps)) < 0)
		{
			goto FUNC_END;
		}

		if ((lErr = uvc_start_capturing()) < 0)
		{
			goto FUNC_END;
		}

		pucaUvcBuffer = (unsigned char*)malloc(ulInWidth * ulInHeight * 2);
		if (!pucaUvcBuffer)
		{
			lErr = 1;
			goto FUNC_END;
		}

#ifdef CONFIG_ENABLE_TESTAPPS_SDL
        if (pComponent->ports[STREAM_DIR_IN].unUvcPreview)
		{
			if (Plstream_CreateOverlay(ulInWidth, ulInHeight))
			{
		 		goto FUNC_END;
			}
		}
#endif //CONFIG_ENABLE_TESTAPPS_SDL
	}
	else if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_UVCMJPEG_IN)
	{
		if (pComponent->ports[STREAM_DIR_IN].unUvcFrameRepeat < 1)
		{
			pComponent->ports[STREAM_DIR_IN].unUvcFrameRepeat = 1;
		}

		if (pComponent->ports[STREAM_DIR_IN].unUvcFps < 1)
		{
			pComponent->ports[STREAM_DIR_IN].unUvcFps = 30;
		}

		if ((lErr = uvc_open_device(pComponent->ports[STREAM_DIR_IN].pszNameOrAddr)) < 0)
		{
			goto FUNC_END;
		}
		else
		{
			videoInInited = 1;
		}

		if ((lErr = uvc_init_device(V4L2_PIX_FMT_MJPEG, ulInWidth, ulInHeight, pComponent->ports[STREAM_DIR_IN].unUvcFps)) < 0)
		{
			goto FUNC_END;
		}

		if ((lErr = uvc_start_capturing()) < 0)
		{
			goto FUNC_END;
		}

#ifdef CONFIG_ENABLE_TESTAPPS_SDL
		if (pComponent->ports[STREAM_DIR_IN].unUvcPreview)
		{
			if (Plstream_CreateOverlay(ulInWidth, ulInHeight))
			{
		 		goto FUNC_END;
			}
		}
#endif //CONFIG_ENABLE_TESTAPPS_SDL
	}
#endif // CONFIG_HOST_PLATFORM_USERSPACE
	else if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_FILE_IN)
	{
		fpInput = fopen(pComponent->ports[STREAM_DIR_IN].pszNameOrAddr, "rb");
		if (fpInput == NULL)
		{
            printf("%s() Unable to open file %s.\n", __FUNCTION__, pComponent->ports[STREAM_DIR_IN].pszNameOrAddr);
			lErr = 1;
			goto FUNC_END;
		}
	}
	else
	{
		goto FUNC_END;
	}

	if (pComponent->ports[STREAM_DIR_IN].eMediaType2 == MEDIA_FILE_OUT)
	{
		fpOutputBackup = fopen(pComponent->ports[STREAM_DIR_IN].pszNameOrAddr2, "wb");
		if (fpOutputBackup == NULL)
		{
			printf("%s() Unable to open backup file %s.\n", __FUNCTION__, pComponent->ports[STREAM_DIR_IN].pszNameOrAddr2);
			lErr = 1;
			goto FUNC_END;
		}
	}

	/***********************************************
	** 3> Create mmap
	***********************************************/

	printf("%s() before mmap....\n", __FUNCTION__);


	stReq.count = pComponent->ports[STREAM_DIR_IN].portType;
	stReq.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
	stReq.memory = V4L2_MEMORY_MMAP;
	lErr = ioctl(pComponent->hDev, VIDIOC_REQBUFS, &stReq);
	if (lErr)
	{
		printf("%s() V4L Request Buffer %d %s\n", __FUNCTION__, errno, strerror(errno));
		lErr = 3;
		goto FUNC_END;
	}
	else
	{
		printf("%s() VIDIOC_REQBUFS count(%d)\n", __FUNCTION__, stReq.count);

		stAppV4lBuf = malloc(stReq.count * sizeof(struct qlapp_v4l_buf_info));
		if (!stAppV4lBuf)
		{
			printf("%s() Unable to allocate memory for V4L app structure \n", __FUNCTION__);
			lErr = 4;
			goto FUNC_END;
		}
        
        memset(stAppV4lBuf, 0, stReq.count * sizeof(struct qlapp_v4l_buf_info));

		for (nV4lBufCnt = 0; nV4lBufCnt < stReq.count; nV4lBufCnt++)
		{
			stV4lBuf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
			stV4lBuf.memory = V4L2_MEMORY_MMAP;
			stV4lBuf.sequence = pComponent->ports[STREAM_DIR_IN].portType;
			stV4lBuf.index = nV4lBufCnt;

			lErr = ioctl(pComponent->hDev, VIDIOC_QUERYBUF, &stV4lBuf);
			if (!lErr)
			{
				stAppV4lBuf[nV4lBufCnt].size = stV4lBuf.length;
#ifndef CONFIG_HOST_PLATFORM_USERSPACE
				stAppV4lBuf[nV4lBufCnt].addr = mmap(0, stV4lBuf.length, PROT_READ | PROT_WRITE, MAP_SHARED, pComponent->hDev, stV4lBuf.m.offset);
#else
				//After do VIDIOC_QUERYBUF, the stV4lBuf.sequence will be clear to 0, and port type will be put in 4 most significant bits of stV4lBuf.m.offset
				stV4lBuf.sequence = pComponent->ports[STREAM_DIR_IN].portType;
				stAppV4lBuf[nV4lBufCnt].addr = plv4l_mmap(pComponent->hDev, stV4lBuf.sequence, stV4lBuf.m.offset);
#endif
				if ((stAppV4lBuf[nV4lBufCnt].addr == (void *)0) || (stAppV4lBuf[nV4lBufCnt].addr == (void *)-1))
				{
					printf("%s() V4L mmap failed index=%d \n", __FUNCTION__, nV4lBufCnt);
					lErr = 5;
					goto FUNC_END;
				}
			}
			else
			{
				printf("%s() VIDIOC_QUERYBUF failed index=%d \n", __FUNCTION__, nV4lBufCnt);
				lErr = 5;
				goto FUNC_END;
			}
		}

		if (ulIOBlockSize > stAppV4lBuf[0].size)
		{
			ulIOBlockSize = stAppV4lBuf[0].size;
		}
	}

	/***********************************************
	** 5> Stream on
	***********************************************/
	while (!pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
	{
		if (pComponent->ports[STREAM_DIR_IN].unStarted)
		{
			/***********************************************
			** DQBUF, get empty buffer from driver
			***********************************************/
	        #ifdef CONFIG_ENABLE_TESTAPPS_USE_SELECT
	        //fprintf(stderr, "==before test_streamin select t=%llu\n",getTimeUs());
			{
		        fd_set writefds;
		        struct timeval tv;
		        int r;

		        FD_ZERO (&writefds);
		        FD_SET (pComponent->hDev, &writefds);

		        /* Timeout. */
		        tv.tv_sec =  1;
		        tv.tv_usec = 0;

		        r = select ((pComponent->hDev) + 1,NULL, &writefds, NULL, &tv);

		        if (r<=0) {
		            //fprintf (stderr, "test_streamin select timeout t=%llu\n",getTimeUs());
		            continue;
		        }
	            //fprintf(stderr, "==test_streamin select pass t=%llu\n",getTimeUs());
			}
			#endif
			stV4lBuf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
			stV4lBuf.memory = V4L2_MEMORY_MMAP;
			stV4lBuf.sequence = pComponent->ports[STREAM_DIR_IN].portType;
	        APP_PROFILING_LOG(EVENT_APP_D_IN_START, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, 0);
			lErr = ioctl(pComponent->hDev, VIDIOC_DQBUF, &stV4lBuf);
			if (!lErr)
			{
				unsigned char *pBuf;

				nRetryCnt = 0;

				pBuf = (unsigned char *) stAppV4lBuf[stV4lBuf.index].addr; //Load mapped address
				//msync((void*)pBuf, stV4lBuf.length, MS_SYNC);

RETRY:
				if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_TCP_IN)
				{
					int				nReceived = 0;
					int				nTotalReceived = 0;
					unsigned int	ulAddrLen = 0;

		            APP_PROFILING_LOG(EVENT_APP_S_IN, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, 0);
					if (pComponent->ports[STREAM_DIR_IN].unPackData)
					{
						int ulIOBlockSizeReal;
						int	nIOError = 0;

						if (nNeedGetDataLen)
						{
							while (nTotalReceived < 4)
							{
								int datatoget = 4 - nTotalReceived;
								nReceived = (int)recv(nSocket, (void *)(pBuf + nTotalReceived), datatoget, 0);

								if (nReceived <= 0)
								{
									if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
									{
										nIOError = 1;
										break;
									}
									else
									{
										if (EAGAIN == errno)
										{
											usleep(1000);
											continue;
										}
										else
										{
											nIOError = 1;
											break;
										}
									}
								}
								else
								{
									nTotalReceived += nReceived;
								}
							}
							if (nIOError==0)
								nDataLen = pBuf[0]<< 24 | pBuf[1]<<16 | pBuf[2] << 8 | pBuf[3];
						}
						if (nIOError==0)
						{
							if (nDataLen > ulIOBlockSize)
								ulIOBlockSizeReal = ulIOBlockSize;
							else
								ulIOBlockSizeReal = nDataLen;

							nTotalReceived = 0;
							while (nTotalReceived < (int)ulIOBlockSizeReal)
							{
								int datatoget = ulIOBlockSizeReal - nTotalReceived;
								nReceived = (int)recv(nSocket, (void *)(pBuf + nTotalReceived), datatoget, 0);

								if (nReceived <= 0)
								{
									if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
									{
										break;
									}
									else
									{
										if (EAGAIN == errno)
										{
											usleep(1000);
											continue;
										}
										else
										{
											break;
										}
									}
								}
								else
								{
									if (nTotalReceived==0)
										APP_PROFILING_LOG(EVENT_APP_S_IN_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, nReceived);

									nTotalReceived += nReceived;
								}
							}
							stV4lBuf.bytesused = nTotalReceived;
							nDataLen -=  nTotalReceived;
							if (nDataLen<=0)
								nNeedGetDataLen = 1;
						}
						else {
							/* socket failed or unCtrlCReceived */
						}
					}
					else
					{
						while (nTotalReceived < (int)ulIOBlockSize)
						{
							int datatoget = ulIOBlockSize - nTotalReceived;
							if (datatoget > MAX_TCP_PACKET_LEN)
								datatoget = MAX_TCP_PACKET_LEN;
							nReceived = (int)recv(nSocket, (void *)(pBuf + nTotalReceived), datatoget, 0);

							if (nReceived <= 0)
							{
								if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
								{
									break;
								}
								else
								{
									if (EAGAIN == errno)
									{
										usleep(1000);
										continue;
									}
									else
									{
										break;
									}
								}
							}
							else
							{
								if (nTotalReceived==0)
									APP_PROFILING_LOG(EVENT_APP_S_IN_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, nReceived);
								nTotalReceived += nReceived;
							}

							if (nReceived < datatoget)
							{
								break;
							}
						}
						stV4lBuf.bytesused = nTotalReceived;
					}
		            APP_PROFILING_LOG(EVENT_APP_S_IN_FINISH, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, stV4lBuf.bytesused);
				}
				else if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_UDP_IN)
				{
					int	nReceived = 0;
					int	nTotalReceived = 0;
					socklen_t nAddrLen = 0;

		            APP_PROFILING_LOG(EVENT_APP_S_IN, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, 0);
					if (pComponent->ports[STREAM_DIR_IN].unPackData)
					{
						int ulIOBlockSizeReal;
						int nIOError = 0;

						if (nNeedGetDataLen)
						{
							while (nTotalReceived < 4)
							{
								int datatoget = 4 - nTotalReceived;
								nReceived = (int)recvfrom(nSocket,
											 (void *)(pBuf + nTotalReceived),
											 datatoget,
											 0,
											 (void *)&stFromAddr,
											 &nAddrLen
											 );
								if (nReceived <= 0)
								{
									if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
									{
										nIOError = 1;
										break;
									}
									else
									{
                                        if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
                                        {
                                            break;
                                        }
                                        else
                                        {
                                            usleep(1000);
                                            continue;
                                        }
									}
								}
								else
								{
									nTotalReceived += nReceived;
								}
							}
							if (nIOError==0)
								nDataLen = pBuf[0]<< 24 | pBuf[1]<<16 | pBuf[2] << 8 | pBuf[3];
						}
						if (nIOError==0)
						{
							if (nDataLen > ulIOBlockSize)
								ulIOBlockSizeReal = ulIOBlockSize;
							else
								ulIOBlockSizeReal = nDataLen;

							nTotalReceived = 0;
							while (nTotalReceived < (int)ulIOBlockSizeReal)
							{
								int datatoget = ulIOBlockSizeReal - nTotalReceived;
								nReceived = (int)recvfrom(nSocket,
													 (void *)(pBuf + nTotalReceived),
													 datatoget,
													 0,
													 (void *)&stFromAddr,
													 &nAddrLen
													 );

								if (nReceived <= 0)
								{
                                    if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
                                    {
                                        break;
                                    }
                                    else
                                    {
                                        usleep(1000);
                                        continue;
                                    }
								}
								else
								{
									if (nTotalReceived==0)
										APP_PROFILING_LOG(EVENT_APP_S_IN_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, nReceived);

									nTotalReceived += nReceived;
								}
							}
							stV4lBuf.bytesused = nTotalReceived;
							nDataLen -= nTotalReceived;
							if (nDataLen<=0)
								nNeedGetDataLen = 1;
						}
						else {
							/* socket failed or unCtrlCReceived */
						}
					}
					else
					{
						while (nTotalReceived < (int)ulIOBlockSize)
						{
							int datatoget = ulIOBlockSize - nTotalReceived;
							if (datatoget > MAX_UDP_PACKET_LEN)
								datatoget = MAX_UDP_PACKET_LEN;
							nReceived = (int)recvfrom(nSocket,
												 (void *)(pBuf + nTotalReceived),
												 datatoget,
												 0,
												 (void *)&stFromAddr,
												 &nAddrLen
												 );

							if (nReceived <= 0)
							{
								if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
								{
									break;
								}
								else
								{
									usleep(1000);
									continue;
								}
							}
							if (nTotalReceived==0)
								APP_PROFILING_LOG(EVENT_APP_S_IN_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, nReceived);

							nTotalReceived += nReceived;

							if (nReceived < datatoget)
							{
								break;
							}
						}
						stV4lBuf.bytesused = nTotalReceived;
					}
		            APP_PROFILING_LOG(EVENT_APP_S_IN_FINISH, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, stV4lBuf.bytesused);
				}
				else if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_STREAM_IN)
				{
					int				nReceived = 0;
					int				nTotalReceived = 0;
					unsigned int	ulAddrLen = 0;

		            APP_PROFILING_LOG(EVENT_APP_S_IN, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, 0);
					if (pComponent->ports[STREAM_DIR_IN].unPackData)
					{
						int ulIOBlockSizeReal;
						int	nIOError = 0;

						if (nNeedGetDataLen)
						{
							while (nTotalReceived < 4)
							{
								int datatoget = 4 - nTotalReceived;
								nReceived = (int)recv(nSocket, (void *)(pBuf + nTotalReceived), datatoget, 0);

								if (nReceived <= 0)
								{
									if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
									{
										nIOError = 1;
										break;
									}
									else
									{
										if (EAGAIN == errno)
										{
											usleep(1000);
											continue;
										}
										else
										{
											nIOError = 1;
											break;
										}
									}
								}
								else
								{
									nTotalReceived += nReceived;
								}
							}
							if (nIOError==0)
								nDataLen = pBuf[0]<< 24 | pBuf[1]<<16 | pBuf[2] << 8 | pBuf[3];
						}
						if (nIOError==0)
						{
							if (nDataLen > ulIOBlockSize)
								ulIOBlockSizeReal = ulIOBlockSize;
							else
								ulIOBlockSizeReal = nDataLen;

							nTotalReceived = 0;
							while (nTotalReceived < (int)ulIOBlockSizeReal)
							{
								int datatoget = ulIOBlockSizeReal - nTotalReceived;
								nReceived = (int)recv(nSocket, (void *)(pBuf + nTotalReceived), datatoget, 0);

								if (nReceived <= 0)
								{
									if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
									{
										break;
									}
									else
									{
										if (EAGAIN == errno)
										{
											usleep(1000);
											continue;
										}
										else
										{
											break;
										}
									}
								}
								else
								{
									if (nTotalReceived==0)
										APP_PROFILING_LOG(EVENT_APP_S_IN_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, nReceived);

									nTotalReceived += nReceived;
								}
							}
							stV4lBuf.bytesused = nTotalReceived;
							nDataLen -=  nTotalReceived;
							if (nDataLen<=0)
								nNeedGetDataLen = 1;
						}
						else {
							/* socket failed or unCtrlCReceived */
						}
					}
					else
					{
						while (nTotalReceived < (int)ulIOBlockSize)
						{
							int datatoget = ulIOBlockSize - nTotalReceived;
							if (datatoget > MAX_TCP_PACKET_LEN)
								datatoget = MAX_TCP_PACKET_LEN;
							nReceived = (int)recv(nSocket, (void *)(pBuf + nTotalReceived), datatoget, 0);

							if (nReceived <= 0)
							{
								if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
								{
									break;
								}
								else
								{
									if (EAGAIN == errno)
									{
										usleep(1000);
										continue;
									}
									else
									{
										break;
									}
								}
							}
							else
							{
								if (nTotalReceived==0)
									APP_PROFILING_LOG(EVENT_APP_S_IN_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, nReceived);

								nTotalReceived += nReceived;
							}

							if (nReceived < datatoget)
							{
								break;
							}
						}
						stV4lBuf.bytesused = nTotalReceived;
					}
		            APP_PROFILING_LOG(EVENT_APP_S_IN_FINISH, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, stV4lBuf.bytesused);
				}
				else if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_DGRAM_IN)
				{
					int	nReceived = 0;
					int	nTotalReceived = 0;
					socklen_t nAddrLen = 0;
					struct sockaddr_un stFromUn;

		            APP_PROFILING_LOG(EVENT_APP_S_IN, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, 0);
					if (pComponent->ports[STREAM_DIR_IN].unPackData)
					{
						int ulIOBlockSizeReal;
						int nIOError = 0;

						if (nNeedGetDataLen)
						{
							while (nTotalReceived < 4)
							{
								int datatoget = 4 - nTotalReceived;
								nReceived = (int)recvfrom(nSocket,
											 (void *)(pBuf + nTotalReceived),
											 datatoget,
											 0,
											 (void *)&stFromUn,
											 &nAddrLen
											 );

								if (nReceived <= 0)
								{
									if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
									{
										nIOError = 1;
										break;
									}
									else
									{
                                        if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
                                        {
                                            break;
                                        }
                                        else
                                        {
                                            usleep(1000);
                                            continue;
                                        }
									}
								}
								else
								{
									nTotalReceived += nReceived;
								}
							}
							if (nIOError==0)
								nDataLen = pBuf[0]<< 24 | pBuf[1]<<16 | pBuf[2] << 8 | pBuf[3];
						}
						if (nIOError==0)
						{
							if (nDataLen > ulIOBlockSize)
								ulIOBlockSizeReal = ulIOBlockSize;
							else
								ulIOBlockSizeReal = nDataLen;

							nTotalReceived = 0;
							while (nTotalReceived < (int)ulIOBlockSizeReal)
							{
								int datatoget = ulIOBlockSizeReal - nTotalReceived;
								nReceived = (int)recvfrom(nSocket,
													 (void *)(pBuf + nTotalReceived),
													 datatoget,
													 0,
													 (void *)&stFromUn,
													 &nAddrLen
													 );

								if (nReceived <= 0)
								{
                                    if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
                                    {
                                        break;
                                    }
                                    else
                                    {
                                        usleep(1000);
                                        continue;
                                    }
								}
								else
								{
									if (nTotalReceived==0)
										APP_PROFILING_LOG(EVENT_APP_S_IN_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, nReceived);

									nTotalReceived += nReceived;
								}
							}
							stV4lBuf.bytesused = nTotalReceived;
							nDataLen -= nTotalReceived;
							if (nDataLen<=0)
								nNeedGetDataLen = 1;
						}
						else {
							/* socket failed or unCtrlCReceived */
						}
					}
					else
					{
						while (nTotalReceived < (int)ulIOBlockSize)
						{
							int datatoget = ulIOBlockSize - nTotalReceived;
							if (datatoget > MAX_UDP_PACKET_LEN)
								datatoget = MAX_UDP_PACKET_LEN;
							nReceived = (int)recvfrom(nSocket,
												 (void *)(pBuf + nTotalReceived),
												 datatoget,
												 0,
												 (void *)&stFromUn,
												 &nAddrLen
												 );

							if (nReceived <= 0)
							{
								if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
								{
									break;
								}
								else
								{
									usleep(1000);
									continue;
								}
							}
							if (nTotalReceived==0)
								APP_PROFILING_LOG(EVENT_APP_S_IN_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, nReceived);
							nTotalReceived += nReceived;

							if (nReceived < datatoget)
							{
								break;
							}
						}
						stV4lBuf.bytesused = nTotalReceived;
					}
		            APP_PROFILING_LOG(EVENT_APP_S_IN_FINISH, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, stV4lBuf.bytesused);
				}
#ifndef CONFIG_HOST_PLATFORM_USERSPACE
				else if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_UVCRAW_IN)
				{
		            APP_PROFILING_LOG(EVENT_APP_S_IN, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, 0);
					if (unUvcFrameRepeatCnt == 0)
					{
						unsigned int unDataSize = ulInWidth*ulInHeight*2;
						if (uvc_read_frame(pucaUvcBuffer, &unDataSize) == 0)
						{
							YUV422ProgTo420Plane(pBuf, pucaUvcBuffer, ulInWidth, ulInHeight);
							stV4lBuf.bytesused += ulIOBlockSize;
							if (pComponent->ports[STREAM_DIR_IN].unUvcFrameRepeat > 1)
							{
								memcpy(pucaUvcBuffer, (void*)pBuf, ulIOBlockSize);
							}
						}
					}
					else
					{
						memcpy((void*)pBuf, pucaUvcBuffer, ulIOBlockSize);
						stV4lBuf.bytesused = ulIOBlockSize;
                    }

					unUvcFrameRepeatCnt = (unUvcFrameRepeatCnt + 1) % pComponent->ports[STREAM_DIR_IN].unUvcFrameRepeat;

#ifdef CONFIG_ENABLE_TESTAPPS_SDL
					if (pComponent->ports[STREAM_DIR_IN].unUvcPreview)
					{
						Plstream_DisplayFrame(ulInWidth,
											  ulInHeight,
											  (unsigned char *)pBuf,
											  _DISPLAY_ALL,
											  QP_YUV_DATA_TYPE_YV12
											  );
					}
#endif //CONFIG_ENABLE_TESTAPPS_SDL
					APP_PROFILING_LOG(EVENT_APP_S_IN_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, stV4lBuf.bytesused);
		            APP_PROFILING_LOG(EVENT_APP_S_IN_FINISH, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, stV4lBuf.bytesused);
				}
				else if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_UVCMJPEG_IN)
				{
					unsigned int unDataSize = stV4lBuf.length;
		            APP_PROFILING_LOG(EVENT_APP_S_IN, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, 0);
					if (uvc_read_frame(pBuf, &unDataSize) == 0)
					{
						stV4lBuf.bytesused = unDataSize;
					}
#ifdef CONFIG_ENABLE_TESTAPPS_SDL
					if (pComponent->ports[STREAM_DIR_IN].unUvcPreview)
					{
						Plstream_DisplayFrame(ulInWidth,
											  ulInHeight,
											  (unsigned char *)pBuf,
											  _DISPLAY_ALL,
											  QP_YUV_DATA_TYPE_YV12
											  );
					}
#endif //CONFIG_ENABLE_TESTAPPS_SDL
					APP_PROFILING_LOG(EVENT_APP_S_IN_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, stV4lBuf.bytesused);
		            APP_PROFILING_LOG(EVENT_APP_S_IN_FINISH, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, stV4lBuf.bytesused);
				}
#endif // CONFIG_HOST_PLATFORM_USERSPACE
				else if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_FILE_IN)
				{
		            APP_PROFILING_LOG(EVENT_APP_S_IN, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, 0);
					stV4lBuf.bytesused = (TQP_UINT32) fread((void*)pBuf, 1, ulIOBlockSize, fpInput);
					APP_PROFILING_LOG(EVENT_APP_S_IN_FIRST, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, stV4lBuf.bytesused);
		            APP_PROFILING_LOG(EVENT_APP_S_IN_FINISH, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, stV4lBuf.bytesused);
				}


				if ((pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_FILE_IN) &&
					(stV4lBuf.bytesused != ulIOBlockSize)
					)
				{
#if 0
					/* EOF */
					pComponent->ports[STREAM_DIR_IN].unCtrlCReceived = 1;
#else
					if ((pComponent->ports[STREAM_DIR_IN].portType == COMPONENT_PORT_COMP_IN) ||
						(pComponent->ports[STREAM_DIR_IN].portType == COMPONENT_PORT_YUV_IN)
						)
					{
						fseek(fpInput, 0, SEEK_SET);
						//// goto RETRY;
					}
					else
					{
						if (pComponent->ports[STREAM_DIR_IN].unFileLoop)
                        {
                            fseek(fpInput, 0, SEEK_SET);
                            printf("%s() read file again\n", __FUNCTION__);
                        }
                        else
                            pComponent->ports[STREAM_DIR_IN].unCtrlCReceived = 1;
					}
#endif
				}

				if (stV4lBuf.bytesused != 0)
				{
					if (pComponent->ports[STREAM_DIR_IN].unUserPTS)
					{
                        /* assign PTS, top 31 bits of 33 bits in 90KHz measurement */
					    if ((pComponent->ports[STREAM_DIR_IN].unUvcFps>0) &&
					     ((pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_UVCMJPEG_IN)||
						 (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_UVCRAW_IN)))
                        {
                            ulPTS += (90000/pComponent->ports[STREAM_DIR_IN].unUvcFps)>>2;
                        }
                        else
                            ulPTS += (90000/30)>>2;
						ulPTS &= 0x7fffffff;
						stV4lBuf.reserved = ulPTS;
	                    stV4lBuf.flags |= QP_BUFDESC_FLAG_PTS;
					}
					else
					{
						/* not use PTS */
						ulPTS = 0;
						stV4lBuf.reserved = ulPTS;
	                    stV4lBuf.flags &= ~QP_BUFDESC_FLAG_PTS;
					}

					if (pComponent->ports[STREAM_DIR_IN].eMediaType2 == MEDIA_FILE_OUT)
					{
						if (fpOutputBackup)
                       		fwrite((void*)pBuf, 1, stV4lBuf.bytesused, fpOutputBackup);
					}
					/***********************************************
					** QBUF, put data back to driver
					***********************************************/
#ifndef CONFIG_HOST_PLATFORM_ASPEN
					msync((void*)pBuf, stV4lBuf.length, MS_SYNC);
#endif

					lErr = ioctl(pComponent->hDev, VIDIOC_QBUF, &stV4lBuf);
					if (lErr)
					{
			            APP_PROFILING_LOG(EVENT_APP_D_IN_FAILED, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, stV4lBuf.bytesused);
						printf("%s() QBUF ioctl failed %d %s\n", __FUNCTION__, errno, strerror(errno));
					}
					else
					{
			            APP_PROFILING_LOG(EVENT_APP_D_IN, (TQP_UINT32) pComponent->hTask, pComponent->devType, 0, ulXferBufCnt+1, stV4lBuf.bytesused);
						printf(" decXferBufCnt[%lld]: %8u %8u 0x%x 0x%x \n", (long long) pComponent->hDev, ulXferBufCnt++, stV4lBuf.bytesused, stV4lBuf.reserved, stV4lBuf.flags);
					}
				}
				else
				{
					if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
					{
						printf("\n\n%s() CTRL+C received.\n", __FUNCTION__);
						break;
					}

					usleep(1000);
					goto RETRY;
				}
				fflush(stdout);
			}
			else
			{
				if (errno == EAGAIN)
				{
					lErr = 0;
				}
				usleep(1000);
			}

			if (pComponent->ports[STREAM_DIR_IN].unCtrlCReceived)
			{
				printf("\n\n%s() CTRL+C received.\n", __FUNCTION__);
				break;
			}

			if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_FILE_IN)
			{
				usleep(30000);
			}
			else
			{
				usleep(1000);
			}
		}
    } //while(!lErr)


FUNC_END:

	printf("\n");

	usleep(10000);

	if (stAppV4lBuf)
	{
#ifndef CONFIG_HOST_PLATFORM_USERSPACE
		for (nV4lBufCnt = 0; nV4lBufCnt < stReq.count; nV4lBufCnt++)
		{
			if (stAppV4lBuf[nV4lBufCnt].addr)
			{
				munmap(stAppV4lBuf[nV4lBufCnt].addr, stAppV4lBuf[nV4lBufCnt].size);
			}
		}
#endif
		free(stAppV4lBuf);
	}

	if (fpInput)
	{
		fclose(fpInput);
	}

	if (fpOutputBackup)
	{
		fclose(fpOutputBackup);
	}

    if (nSocket != -1)
	{
		close(nSocket);
	}

#ifndef CONFIG_HOST_PLATFORM_USERSPACE
	if (videoInInited)
	{
		uvc_stop_capturing();
		uvc_uninit_device();
		uvc_close_device();
	}

    if (pComponent->ports[STREAM_DIR_IN].eMediaType == MEDIA_UVCRAW_IN)
	{
		if (pucaUvcBuffer)
		{
			free(pucaUvcBuffer);
		}
	}

#endif // CONFIG_HOST_PLATFORM_USERSPACE
    pComponent->ports[STREAM_DIR_IN].unStarted = 0;

    //return lErr;
	printf("%s() ]\n", __FUNCTION__);
}


/////////////////////////////////////////////////////////////////////////////
//
//

#define MAX_SUPPORTED_COMPONENTS	5

#define HAS_ARG_NUM(argc, argnow, need) ((nArgNow + need) < argc)

extern QPCODEC_OSD_MEM_CONFIG osdMem;

#ifdef	SUPPORT_VO_OSD
int trans_mem_address(unsigned int dev)
{
	int	 lErr = 0;
	QLDEV_OSD_PARAMETER osd;

	lErr = ioctl(dev, PLDEV_CNTL_IOCTL_GET_OSD_MEMORY, &osdMem);

	osd.value = osdMem.memStartAddress;
	osd.type = PARAM_SET_ADDRESS;
	lErr = lErr? lErr: ioctl(dev, PLDEV_DIAG_IOCTL_OSD_CONFIG, &osd);

	return lErr;
}
#endif

int  change_framerate(DEVHANDLE dev)
{
	static int fps=10;
	int	 lErr = 0;
	QPCODEC_INPUT_CONTROL inputControl;

    fps++;
    if (fps>30) fps=10;
    lErr = ioctl(dev, PLDEV_CNTL_IOCTL_GET_INPUT_CONTROL, &inputControl);
	inputControl.Artesa.out_frame_rate = fps;
	lErr = (0 != lErr) ? lErr : ioctl(dev, PLDEV_CNTL_IOCTL_SET_INPUT_CONTROL, &inputControl);
	printf("\n===> change_framerate to %d fps\n", fps);
	return lErr;
}

int  change_bitrate(DEVHANDLE dev)
{
	static int rate = 500;
	int	 lErr = 0;
	QPCODEC_RATE_CONTROL rateControl;

	if (rate<1000) {
		rate += 100;
	}
	else {
		rate += 1000;
	}
	if (rate>6000) rate=500;
	lErr = ioctl(dev, PLDEV_CNTL_IOCTL_GET_RATE_CONTROL, &rateControl);
    rateControl.Artesa.avg_bit_rate = rate;
	lErr = (0 != lErr) ? lErr : ioctl(dev, PLDEV_CNTL_IOCTL_SET_RATE_CONTROL, &rateControl);
	printf("\n===> change_bitrate to %d kbps\n", rate);
	return lErr;
}

int  change_audio_bitrate(DEVHANDLE dev)
{
	static int rate = 8000;
	int	 lErr = 0;
	QPCODEC_AUDIO_CONTROL_PARAM audControl;

	rate += 8000;

	if (rate>32000) rate=8000;
	lErr = ioctl(dev, PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM, &audControl);
    audControl.bit_rate = rate/1000;
	lErr = (0 != lErr) ? lErr : ioctl(dev, PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM, &audControl);
	printf("\n===> change audio bitrate to %d kbps\n", rate/1000);
	return lErr;
}

int  change_audio_sample_rate(DEVHANDLE dev)
{
	static int rate = 0;
	int	 lErr = 0;
	QPCODEC_AUDIO_CONTROL_PARAM audControl;
	char str[8];

	rate ++;

	if (rate>14) rate=0;
	switch(rate)
	{
		case 0: strcpy(str,"44.1");break;
		case 1: strcpy(str,"48");break;
		case 2: strcpy(str,"32");break;
		case 4: strcpy(str,"22.5");break;
		case 5: strcpy(str,"24");break;
		case 6: strcpy(str,"16");break;
		case 8: strcpy(str,"8");break;
		default:
			strcpy(str,"Other");
			break;
	}
	lErr = ioctl(dev, PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_PARAM, &audControl);
    	audControl.sample_rate = rate;
	lErr = (0 != lErr) ? lErr : ioctl(dev, PLDEV_CNTL_IOCTL_SET_AUDIO_CONTROL_PARAM, &audControl);
	printf("\n===> change audio sample rate to %s KHz\n", str);
	return lErr;
}

int  get_raw_audio_type(DEVHANDLE dev)
{
	int	 lErr = 0;
	QPCODEC_AUDIO_CONTROL_EX audControlEx;

	lErr = ioctl(dev, PLDEV_CNTL_IOCTL_GET_AUDIO_CONTROL_EX, &audControlEx);
	printf("\n===> Get raw audio type (1=ac3 2=aac 3=aac2 4=dts) = %d\n", audControlEx.raw_audio_type);
	return lErr;
}

int  insert_i_frame(DEVHANDLE dev)
{
	int	 lErr = 0;

	lErr = ioctl(dev, PLDEV_CNTL_IOCTL_SET_INSERT_I_FRAME, NULL);
	printf("\n===> insert_i_frame called on dev=%lld\n", (long long) dev);
	return lErr;
}

int  change_aiu_volume(DEVHANDLE dev)
{
	TQP_UINT32 Volume = 8;
	TQP_UINT32 oriVolume;
	int	 lErr = 0;

	lErr = ioctl(dev, PLDEV_CNTL_IOCTL_GET_ENC_AUD_INPUT_VOL, &oriVolume);
	if (oriVolume==0)
		Volume = 8;
	else
		Volume = oriVolume-1;
	lErr = ioctl(dev, PLDEV_CNTL_IOCTL_SET_ENC_AUD_INPUT_VOL, &Volume);
	printf("\n===> change_aiu_volume from %d to %d called ret=%d\n", oriVolume, Volume, lErr);
	return lErr;
}

int  change_aou_volume(DEVHANDLE dev)
{
	TQP_UINT32 Volume = 8;
	TQP_UINT32 oriVolume;
	int	 lErr = 0;

	lErr = ioctl(dev, PLDEV_CNTL_IOCTL_GET_DEC_AUD_OUTPUT_VOL, &oriVolume);
	if (oriVolume==0)
		Volume = 8;
	else
		Volume = oriVolume-1;
	lErr = ioctl(dev, PLDEV_CNTL_IOCTL_SET_DEC_AUD_OUTPUT_VOL, &Volume);
	printf("\n===> change_aou_volume from %d to %d called ret=%d\n", oriVolume, Volume, lErr);
	return lErr;
}

int  insert_jpeg_frame_request(DEVHANDLE dev)
{
	int	 lErr = 0;

	lErr = ioctl(dev, PLDEV_CNTL_IOCTL_SET_ENC_GET_A_FRAME, NULL);
	printf("\n===> insert_jpeg_frame_request called on dev=%lld\n", (long long) dev);
	return lErr;
}



#define IT6604_ADDRESS 0x48
struct _i2c_debug_reg_array {
    uint8_t subaddress;
    const char *info;
};

static struct _i2c_debug_reg_array it6604_debug_reg_list[] = {
 {02, "device id, = 0x23" },
 {03, "device id, = 0x60" },
 {04, "version, = 0xa3" },
 {0x10, "b.5=0 or b.7=0 HDMI not insert, b4=0(DVI) 1:(HDMI mode)" },
 {0x12, "hdcp status" },
 {0x58, "video status: b3:video stable, b1:interlace" },
 {0x65, "video input: b0:hsync b1:vsync b2:HDMI(in avmute)" },
 {0x9c, "audio status" },
 {0xae, "avi info frame byte 1" },
 {0x1b, "driver set = 0x60" },
 {0x1c, "driver set = 0x10" },
 {0x3d, "driver set = 0x40" },
 {0x77, "=0xa0 (audio compressed)" },
 {0x88, "=0x10 (audio compressed)" },
 {0x20, "driver set yuv(0x04) rgb(0x06)" },
 {0,0}
};

int dump_frontend_regs(DEVHANDLE dev)
{
	int lErr = 0;
	TQP_UINT32 value;
	uint8_t retvalue;

    lErr = ioctl(dev, PLDEV_DIAG_IOCTL_VIDEO_DECODER_GET, &value);
	printf("\ncurrent frontend type=%d ret=%d %s\n", value, lErr, (lErr)?"failed":"");
	if (!lErr) {
		if (value==14) {
            int i = 0;
			printf("dump it6603/it6604 registers\n");
			while (it6604_debug_reg_list[i].subaddress!=0) {
				/* note: it6604 need use swi2c */
				lErr = pl_i2cnb_get(dev, 1, IT6604_ADDRESS, 1, &it6604_debug_reg_list[i].subaddress, 1, &retvalue);
                if (!lErr) {
                    printf("addr[0x%x]=0x%x; %s\n", it6604_debug_reg_list[i].subaddress,
                           retvalue, it6604_debug_reg_list[i].info);
                }
                else {
                     printf("addr[0x%x]= read err=%d\n", it6604_debug_reg_list[i].subaddress, lErr);
                }
                i++;
			}
		}
		else {
			printf("dump frontend type=%d not implemented\n", value);
		}
	}
	return lErr;
}

int main(int argc,
		 char* argv[]
		 )
{
	int			lErr = 0;
	int			nArgNow = 1;
	int			nCmdIdx = 0;
	int			nHas2ndCmd;
	component_t	component[MAX_SUPPORTED_COMPONENTS];
	int			nType = V4L2_BUF_TYPE_VIDEO_CAPTURE;
	int			i, j;
	int			ch;
    TQP_OBJECT_HANDLE hTask;

#ifdef CONFIG_ENABLE_TESTAPPS_SDL
	const SDL_VideoInfo	*sdlVideoInfo = NULL;
	int				needSdl = 0;
	int				sdlInited = 0;
	unsigned int	ulWidth = 48; /*640*/
	unsigned int	ulHeight = 48; /*480*/
#endif //CONFIG_ENABLE_TESTAPPS_SDL
#ifdef ENABLE_APP_LINUX_PROFILING
#ifdef CONFIG_ENABLE_TESTAPPS_PROFILING_ALWAYS
	unsigned int unAppLog = 1;
#else
	unsigned int unAppLog = 0;
#endif
	unsigned char applogname[128]="prof_app.dat";
#endif

	if (argc<2)
		usage(1);

	signal(SIGINT, SigIntHanlder);
	signal(SIGSTOP, SigStopHanlder);
	signal(SIGCONT, SigContHanlder);
	signal(SIGTERM, SigTermHanlder);

//	signal(SIGPIPE, SigPipeHanlder);  // not define, keep compatible with old version

#ifdef CONFIG_HOST_PLATFORM_USERSPACE
	plusb_init_module(1);
#endif

LOOP_START:
   	nArgNow = 1;
    nCmdIdx = 0;
	lErr = 0;
	g_unCtrlCReceived = 0;

	memset(&component[0],
		   0,
		   MAX_SUPPORTED_COMPONENTS * sizeof(component_t)
		   );

HAS_2ND_CMD:

	nHas2ndCmd = 0;

	component[nCmdIdx].busType = QPHCI_BUS_TOTAL_BUSES;
#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
    component[nCmdIdx].ports[STREAM_DIR_OUT].unVIOSDOn = 1;
#endif

    while (nArgNow < argc)
	{
        if (!strcasecmp(argv[nArgNow],"ENC"))
		{
			nArgNow++;
			component[nCmdIdx].function.functionEx.function = CODEC_OPEN_EX_ENCODE;
			component[nCmdIdx].devType = COMPONENT_TYPE_ENCODE;
		}
		else if (!strcasecmp(argv[nArgNow],"DEC"))
		{
			nArgNow++;
			component[nCmdIdx].function.functionEx.function = CODEC_OPEN_EX_DECODE;
			component[nCmdIdx].devType = COMPONENT_TYPE_DECODE;
        }
		else if (!strcasecmp(argv[nArgNow],"BUS"))
		{
			if (!HAS_ARG_NUM(argc, nArgNow, 1))
				break;
			nArgNow++;
			component[nCmdIdx].busType = atoi(argv[nArgNow++]);
        }
		else if (!strcasecmp(argv[nArgNow],"CARD"))
		{
			if (!HAS_ARG_NUM(argc, nArgNow, 1))
				break;
			nArgNow++;
			component[nCmdIdx].devInstance = atoi(argv[nArgNow++]);
        }
		else if (!strcasecmp(argv[nArgNow],"CONFIG"))
		{
			if (!HAS_ARG_NUM(argc, nArgNow, 1))
				break;
			nArgNow++;
			component[nCmdIdx].pszScriptName = argv[nArgNow++];
		}
		else if (!strcasecmp(argv[nArgNow],"IFILE"))
		{
			if (!HAS_ARG_NUM(argc, nArgNow, 1))
				break;
			nArgNow++;
			component[nCmdIdx].ports[STREAM_DIR_IN].eMediaType = MEDIA_FILE_IN;
			component[nCmdIdx].ports[STREAM_DIR_IN].pszNameOrAddr = argv[nArgNow++];
		}
		else if (!strcasecmp(argv[nArgNow],"OFILE"))
		{
			if (!HAS_ARG_NUM(argc, nArgNow, 1))
				break;
			nArgNow++;
			component[nCmdIdx].ports[STREAM_DIR_OUT].eMediaType = MEDIA_FILE_OUT;
			component[nCmdIdx].ports[STREAM_DIR_OUT].pszNameOrAddr = argv[nArgNow++];
		}
		else if (!strcasecmp(argv[nArgNow],"DOUTFILE"))
		{
			if (!HAS_ARG_NUM(argc, nArgNow, 1))
				break;
			nArgNow++;
			component[nCmdIdx].ports[STREAM_DIR_OUT].eMediaType2 = MEDIA_FILE_OUT;
			component[nCmdIdx].ports[STREAM_DIR_OUT].pszNameOrAddr2 = argv[nArgNow++];
		}
		else if (!strcasecmp(argv[nArgNow],"DINFILE"))
		{
			if (!HAS_ARG_NUM(argc, nArgNow, 1))
				break;
			nArgNow++;
			component[nCmdIdx].ports[STREAM_DIR_IN].eMediaType2 = MEDIA_FILE_OUT;
			component[nCmdIdx].ports[STREAM_DIR_IN].pszNameOrAddr2 = argv[nArgNow++];
		}
#ifdef	SUPPORT_HLS_FILE
		else if (!strcasecmp(argv[nArgNow],"OHLSFILE"))
		{
			if (!HAS_ARG_NUM(argc, nArgNow, 1))
				break;
			nArgNow++;
			component[nCmdIdx].ports[STREAM_DIR_OUT].eMediaType = MEDIA_HLS_FILE_OUT;
			component[nCmdIdx].ports[STREAM_DIR_OUT].pszNameOrAddr = argv[nArgNow++];
		}
#endif
		else if (!strcasecmp(argv[nArgNow],"IUDP"))
		{
            nArgNow++;
            component[nCmdIdx].ports[STREAM_DIR_IN].eMediaType = MEDIA_UDP_IN;
        }
		else if (!strcasecmp(argv[nArgNow],"OUDP"))
		{
			if (!HAS_ARG_NUM(argc, nArgNow, 1))
				break;
			nArgNow++;
			component[nCmdIdx].ports[STREAM_DIR_OUT].eMediaType = MEDIA_UDP_OUT;
			component[nCmdIdx].ports[STREAM_DIR_OUT].pszNameOrAddr = argv[nArgNow++];
		}
		else if (!strcasecmp(argv[nArgNow],"ITCP"))
		{
			nArgNow++;
			component[nCmdIdx].ports[STREAM_DIR_IN].eMediaType = MEDIA_TCP_IN;
		}
		else if (!strcasecmp(argv[nArgNow],"OTCP"))
		{
			if (!HAS_ARG_NUM(argc, nArgNow, 1))
				break;
			nArgNow++;
			component[nCmdIdx].ports[STREAM_DIR_OUT].eMediaType = MEDIA_TCP_OUT;
			component[nCmdIdx].ports[STREAM_DIR_OUT].pszNameOrAddr = argv[nArgNow++];
        }
		else if (!strcasecmp(argv[nArgNow],"IDGRAM"))
		{
			if (!HAS_ARG_NUM(argc, nArgNow, 1))
				break;
            nArgNow++;
            component[nCmdIdx].ports[STREAM_DIR_IN].eMediaType = MEDIA_DGRAM_IN;
			component[nCmdIdx].ports[STREAM_DIR_IN].pszNameOrAddr = argv[nArgNow++];
        }
		else if (!strcasecmp(argv[nArgNow],"ODGRAM"))
		{
			if (!HAS_ARG_NUM(argc, nArgNow, 2))
				break;
			nArgNow++;
			component[nCmdIdx].ports[STREAM_DIR_OUT].eMediaType = MEDIA_DGRAM_OUT;
			component[nCmdIdx].ports[STREAM_DIR_OUT].pszNameOrAddr = argv[nArgNow++];
			component[nCmdIdx].ports[STREAM_DIR_OUT].pszNameOrAddr1 = argv[nArgNow++];
		}
		else if (!strcasecmp(argv[nArgNow],"ISTREAM"))
		{
			if (!HAS_ARG_NUM(argc, nArgNow, 1))
				break;
            nArgNow++;
            component[nCmdIdx].ports[STREAM_DIR_IN].eMediaType = MEDIA_STREAM_IN;
			component[nCmdIdx].ports[STREAM_DIR_IN].pszNameOrAddr = argv[nArgNow++];
        }
		else if (!strcasecmp(argv[nArgNow],"OSTREAM"))
		{
			if (!HAS_ARG_NUM(argc, nArgNow, 2))
				break;
			nArgNow++;
			component[nCmdIdx].ports[STREAM_DIR_OUT].eMediaType = MEDIA_STREAM_OUT;
			component[nCmdIdx].ports[STREAM_DIR_OUT].pszNameOrAddr = argv[nArgNow++];
			component[nCmdIdx].ports[STREAM_DIR_OUT].pszNameOrAddr1 = argv[nArgNow++];
		}
		else if (!strcasecmp(argv[nArgNow],"IPACK"))
		{
			nArgNow++;
			component[nCmdIdx].ports[STREAM_DIR_IN].unPackData = 1;
        }
		else if (!strcasecmp(argv[nArgNow],"OPACK"))
		{
			nArgNow++;
			component[nCmdIdx].ports[STREAM_DIR_OUT].unPackData = 1;
		}
		else if (!strcasecmp(argv[nArgNow],"IUVCRAW"))
		{
			if (!HAS_ARG_NUM(argc, nArgNow, 1))
				break;
			nArgNow++;
			component[nCmdIdx].ports[STREAM_DIR_IN].eMediaType = MEDIA_UVCRAW_IN;
			component[nCmdIdx].ports[STREAM_DIR_IN].pszNameOrAddr = argv[nArgNow++];
		}
		else if (!strcasecmp(argv[nArgNow],"IUVCMJPEG"))
		{
            if (!HAS_ARG_NUM(argc, nArgNow, 1))
				break;
            nArgNow++;
            component[nCmdIdx].ports[STREAM_DIR_IN].eMediaType = MEDIA_UVCMJPEG_IN;
            component[nCmdIdx].ports[STREAM_DIR_IN].pszNameOrAddr = argv[nArgNow++];
        }
		else if (!strcasecmp(argv[nArgNow],"OSDL"))
		{
            if (!HAS_ARG_NUM(argc, nArgNow, 1))
				break;
            nArgNow++;
            component[nCmdIdx].ports[STREAM_DIR_OUT].eMediaType = MEDIA_SDL_OUT;
            component[nCmdIdx].ports[STREAM_DIR_OUT].displayType = atoi(argv[nArgNow++]);
        }
		else if (!strcasecmp(argv[nArgNow],"IPORT"))
		{
            if (!HAS_ARG_NUM(argc, nArgNow, 1))
				break;
            nArgNow++;
            component[nCmdIdx].ports[STREAM_DIR_IN].unPort = atoi(argv[nArgNow++]);
        }
		else if (!strcasecmp(argv[nArgNow],"OPORT"))
		{
            if (!HAS_ARG_NUM(argc, nArgNow, 1))
				break;
            nArgNow++;
            component[nCmdIdx].ports[STREAM_DIR_OUT].unPort = atoi(argv[nArgNow++]);
        }
		else if (!strcasecmp(argv[nArgNow],"UVCFPS"))
		{
            if (!HAS_ARG_NUM(argc, nArgNow, 1))
				break;
            nArgNow++;
            if (strchr(argv[nArgNow], '*'))
			{
                component[nCmdIdx].ports[STREAM_DIR_IN].unUvcFrameRepeat = atoi(strchr(argv[nArgNow], '*') + 1);
            }
            component[nCmdIdx].ports[STREAM_DIR_IN].unUvcFps = atoi(argv[nArgNow++]);
        }
		else if (!strcasecmp(argv[nArgNow],"UVCPREVIEW"))
		{
            nArgNow++;
            component[nCmdIdx].ports[STREAM_DIR_IN].unUvcPreview = 1;
        }
		else if (!strcasecmp(argv[nArgNow],"SEPARATE"))
		{
            nArgNow++;
            component[nCmdIdx].ports[STREAM_DIR_OUT].unJPEGSeparate = 1;
        }
		else if (!strcasecmp(argv[nArgNow],"USERPTS"))
		{
            nArgNow++;
            component[nCmdIdx].ports[STREAM_DIR_IN].unUserPTS = 1;
        }
		else if (!strcasecmp(argv[nArgNow],"OMP4FILE"))
		{
			if (!HAS_ARG_NUM(argc, nArgNow, 1))
				break;
			nArgNow++;
			component[nCmdIdx].ports[STREAM_DIR_OUT].eMediaType = MEDIA_MP4FILE_OUT;
			component[nCmdIdx].ports[STREAM_DIR_OUT].pszNameOrAddr = argv[nArgNow++];
		}
		else if (!strcasecmp(argv[nArgNow],"NULL"))
		{
			nArgNow++;
			component[nCmdIdx].ports[STREAM_DIR_OUT].eMediaType = MEDIA_NULL_OUT;
		}
		else if (!strcasecmp(argv[nArgNow],"FORCEFULLBUFFER"))
		{
			nArgNow++;
			component[nCmdIdx].ports[STREAM_DIR_OUT].ulForceFullBuffer = COMPONENT_PORT_TYPE_FORCE_FULL_BUFFER;
		}
		else if (!strcasecmp(argv[nArgNow],"NOUSERINPUT"))
		{
            nArgNow++;
			g_unNoUserInput = 1;
        }
		else if (!strcasecmp(argv[nArgNow],"REDUCEPRINTF"))
		{
            nArgNow++;
			g_unReducePrintf = 1;
        }
#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
		else if (!strcasecmp(argv[nArgNow],"USEODIRECT"))
		{
            nArgNow++;
			g_unUseODirect = 1;
        }
#endif
#ifdef CONFIG_ENABLE_PL_AUDIO_AGC
		else if (!strcasecmp(argv[nArgNow],"AGCON"))
		{
            nArgNow++;
			component[nCmdIdx].ports[STREAM_DIR_OUT].gAudAgcOn = 1;
        }
#endif
		else if (!strcasecmp(argv[nArgNow],"LOOP"))
		{
            nArgNow++;
            component[nCmdIdx].ports[STREAM_DIR_IN].unFileLoop = 1;
        }
		else if (!strcasecmp(argv[nArgNow],"OSD"))
		{
			if (!HAS_ARG_NUM(argc, nArgNow, 1))
				break;
			nArgNow++;
			if (atoi(argv[nArgNow]))
				component[nCmdIdx].ports[STREAM_DIR_OUT].unVIOSDOn = 1;
			else
				component[nCmdIdx].ports[STREAM_DIR_OUT].unVIOSDOn = 0;
			nArgNow++;
        }
		else if (!strcasecmp(argv[nArgNow],"LIMIT"))
		{
			int lmt;
			if (!HAS_ARG_NUM(argc, nArgNow, 1))
				break;
			nArgNow++;
			lmt = atoi(argv[nArgNow]);
			if (lmt>0)
				component[nCmdIdx].ports[STREAM_DIR_OUT].unUpLimit = (unsigned long) lmt*1024;
			else
				component[nCmdIdx].ports[STREAM_DIR_OUT].unUpLimit = 1000000000;
			nArgNow++;
        }
		else if (!strcasecmp(argv[nArgNow],"APPLOG"))
		{
			if (!HAS_ARG_NUM(argc, nArgNow, 1))
				break;
			nArgNow++;
#ifdef ENABLE_APP_LINUX_PROFILING
			unAppLog = 1;
			strncpy(applogname, argv[nArgNow], sizeof(applogname));
#endif
			nArgNow++;
		}
		else if (!strcasecmp(argv[nArgNow],"SAVEORIGINAL"))
		{
            nArgNow++;
            component[nCmdIdx].ports[STREAM_DIR_OUT].unSaveOriginal = 1;
        }
		else if (!strcasecmp(argv[nArgNow],"RUNLOOP"))
		{
			if (!HAS_ARG_NUM(argc, nArgNow, 2))
				break;
			nArgNow++;
			g_unEnableLoop = 1;
			g_unEnableLoopNeedCnt = atoi(argv[nArgNow++]);
			g_unEnableLoopDelay = atoi(argv[nArgNow++]);
        }
		else if (!strcasecmp(argv[nArgNow],"+"))
		{
            nArgNow++;
            nHas2ndCmd = 1;
            break;
        }
		else
		{
            nArgNow++;
        }
    }
#ifndef CONFIG_HOST_PLATFORM_USERSPACE
	// lookup and open the device
	lErr = lookup_video_device_node(component[nCmdIdx].devInstance,
									component[nCmdIdx].devType,
									component[nCmdIdx].busType,
									component[nCmdIdx].szDevName
									);
#endif // CONFIG_HOST_PLATFORM_USERSPACE
	if (0 == lErr)
	{
		#ifdef ENABLE_APP_LINUX_PROFILING
        if (unAppLog)
	        APP_PROFILING_LOG_INIT();
		#endif

#ifndef CONFIG_HOST_PLATFORM_USERSPACE
		component[nCmdIdx].hDev = open(component[nCmdIdx].szDevName,
									   O_RDWR
									   );
		if (component[nCmdIdx].hDev <= 0)
#else
		component[nCmdIdx].hDev = (DEVHANDLE) plv4l_open(component[nCmdIdx].devInstance, component[nCmdIdx].devType);
		if (component[nCmdIdx].hDev == 0)
#endif // CONFIG_HOST_PLATFORM_USERSPACE
		{
			printf("Unable to Open %s.\n", component[nCmdIdx].szDevName);

            APP_PROFILING_LOG(EVENT_APP_OPEN_FAILED, (TQP_UINT32) component[nCmdIdx].hDev, component[i].devType, 0, 0, 0);

			lErr = 1;
			goto FUNC_END;
		}
		else
		{
			PLDEV_INFO	devInfo;

            APP_PROFILING_LOG(EVENT_APP_OPEN, (TQP_UINT32) component[nCmdIdx].hDev, component[i].devType, 0, 0, 0);

			devInfo.busType = PLDRV_VERSION_CODE;   // used for api version checking
			devInfo.type = PLIOCTLAPI_VERSION_CODE; // used for api version checking

			lErr = ioctl(component[nCmdIdx].hDev,
						 PLDEV_STRM_IOCTL_DEVICE_INFO,
						 &devInfo
						 );
			if (!lErr)
			{
				component[nCmdIdx].hTask = devInfo.hTask;
				component[nCmdIdx].busType = devInfo.busType;
	            APP_PROFILING_LOG(EVENT_APP_OPEN_MAPTO_CH, (TQP_UINT32) component[nCmdIdx].hDev, component[i].devType, component[nCmdIdx].hTask, component[nCmdIdx].hTask, component[nCmdIdx].hTask);
			}
			else
			{
				if (lErr==QPERR_CTRL_VERSION)
				{
					printf("ioctl PLDEV_STRM_IOCTL_DEVICE_INFO APP/Driver Version Not Matched\n");
				}
				else {
					printf("ioctl PLDEV_STRM_IOCTL_DEVICE_INFO failed.\n");
				}
				lErr = 1;
				goto FUNC_END;
			}
		}
	}
	else
	{
		printf("Unable to find device.\n");
		lErr = 1;
		goto FUNC_END;
	}

	// get the configuration
    if (nCmdIdx<=0)
    	hTask = TQP_NULL_HANDLE;
    else if (nCmdIdx==1)
    	hTask = component[0].hTask;
	else
		hTask = component[1].hTask;
	lErr = qlconf(&component[nCmdIdx],
				  hTask,
				  component[nCmdIdx].pszScriptName
				  );
	if (lErr)
	{
		printf("Unable to config device.\n");
		lErr = 1;
		goto FUNC_END;
	}

	for (i = 0; i < MAX_STREAM_DIR; i++)
	{
		if ((component[nCmdIdx].ports[i].portType >= 0) &&
			(component[nCmdIdx].ports[i].portType < MAX_COMPONENT_PORT)
			)
		{
			// open port
			//
			COMPONENT_PORT_OPEN	port_open;

			memcpy(&port_open.format,
				   &component[nCmdIdx].ports[i].openFormat,
				   sizeof(COMPONENT_PORT_OPEN_FORMAT)
				   );
		#ifdef CONFIG_ENABLE_TESTAPPS_USERBUF
            if (component[nCmdIdx].busType==QPHCI_BUS_PCI)
            {
			// try to open with userptr
			port_open.type = component[nCmdIdx].ports[i].portType | component[nCmdIdx].ports[i].ulForceFullBuffer;
			port_open.memory = V4L2_MEMORY_USERPTR;

            APP_PROFILING_LOG(EVENT_APP_PORTOPEN, (TQP_UINT32) component[nCmdIdx].hTask, component[nCmdIdx].devType, port_open.type, port_open.memory, port_open.type);

			lErr = ioctl(component[nCmdIdx].hDev,
						 PLDEV_STRM_IOCTL_PORT_OPEN,
						 &port_open
						 );
            }
            else {
				// try to open with mmap first, driver only support userptr in PCIE */
				port_open.type = component[nCmdIdx].ports[i].portType | component[nCmdIdx].ports[i].ulForceFullBuffer;
				port_open.memory = V4L2_MEMORY_MMAP;

                APP_PROFILING_LOG(EVENT_APP_PORTOPEN, (TQP_UINT32) component[nCmdIdx].hTask,  component[nCmdIdx].devType, port_open.type, port_open.memory, port_open.type);

				lErr = ioctl(component[nCmdIdx].hDev,
							 PLDEV_STRM_IOCTL_PORT_OPEN,
							 &port_open
							 );
            }
			if (lErr)
			{
				printf("ioctl(PLDEV_STRM_IOCTL_PORT_OPEN) component(%d) type(%u) failed %d %s\n", nCmdIdx, port_open.type, errno, strerror(errno));
				if (EPERM == errno)
				{
					// open with mmap
					port_open.type = component[nCmdIdx].ports[i].portType | component[nCmdIdx].ports[i].ulForceFullBuffer;
					port_open.memory = V4L2_MEMORY_MMAP;

                    APP_PROFILING_LOG(EVENT_APP_PORTOPEN, (TQP_UINT32) component[nCmdIdx].hTask,  component[nCmdIdx].devType, port_open.type, port_open.memory, port_open.type);

					lErr = ioctl(component[nCmdIdx].hDev,
								 PLDEV_STRM_IOCTL_PORT_OPEN,
								 &port_open
								 );
					if (lErr)
					{
						printf("ioctl(PLDEV_STRM_IOCTL_PORT_OPEN) component(%d) type(%u) failed %d %s\n", nCmdIdx, port_open.type, errno, strerror(errno));
						goto FUNC_END;
					}
				}
				else
				{
					goto FUNC_END;
				}
			}
		#else
			{
				// open with mmap
				port_open.type = component[nCmdIdx].ports[i].portType | component[nCmdIdx].ports[i].ulForceFullBuffer;
				port_open.memory = V4L2_MEMORY_MMAP;

                APP_PROFILING_LOG(EVENT_APP_PORTOPEN, (TQP_UINT32) component[nCmdIdx].hTask,  component[nCmdIdx].devType, port_open.type, port_open.memory, port_open.type);

				lErr = ioctl(component[nCmdIdx].hDev,
							 PLDEV_STRM_IOCTL_PORT_OPEN,
							 &port_open
							 );
				if (lErr)
				{
					printf("ioctl(PLDEV_STRM_IOCTL_PORT_OPEN) component(%d) type(%u) failed %d %s\n", nCmdIdx, port_open.type, errno, strerror(errno));
					goto FUNC_END;
				}
			}
		#endif
			printf("ioctl(PLDEV_STRM_IOCTL_PORT_OPEN) component(%d) type(%u) succeed\n", nCmdIdx, port_open.type);

			component[nCmdIdx].ports[i].memory = port_open.memory;
			component[nCmdIdx].ports[i].opened = QP_TRUE;
			component[nCmdIdx].ports[i].unSentStopCmd = 0;
			component[nCmdIdx].ports[i].unStarted = 0;
		}
	}

	/* qlconfDump(&component[nCmdIdx]); */
	/* check command is acceptable or not */
	{
		if (component[nCmdIdx].ports[STREAM_DIR_OUT].unJPEGSeparate)
		{
			if ((component[nCmdIdx].ports[STREAM_DIR_OUT].eMediaType == MEDIA_FILE_OUT) &&
				(component[nCmdIdx].ports[STREAM_DIR_OUT].portType==COMPONENT_PORT_COMP_OUT) &&
				(component[nCmdIdx].ports[STREAM_DIR_OUT].openFormat.vid.nDataType == QP_COMP_VID_DATA_TYPE_MJPEG))
			{
				/* configure is correct */
			}
			else {
				component[nCmdIdx].ports[STREAM_DIR_OUT].unJPEGSeparate = 0; /* config not accept */
				printf("[separate] is not allowed for steam %d\n", nCmdIdx);
			}
		}
		if (component[nCmdIdx].ports[STREAM_DIR_OUT].unUpLimit)
		{
			if ((component[nCmdIdx].devType==COMPONENT_TYPE_ENCODE) &&
				(component[nCmdIdx].ports[STREAM_DIR_OUT].portType==COMPONENT_PORT_COMP_OUT) &&
				(component[nCmdIdx].ports[STREAM_DIR_OUT].openFormat.vid.nDataType == QP_COMP_VID_DATA_TYPE_H264) &&
				((component[nCmdIdx].ports[STREAM_DIR_OUT].eMediaType == MEDIA_FILE_OUT)||
				 (component[nCmdIdx].ports[STREAM_DIR_OUT].eMediaType == MEDIA_MP4FILE_OUT)))
			{
				printf("set limit file check point @%lu\n", component[nCmdIdx].ports[STREAM_DIR_OUT].unUpLimit);
				/* configure is correct */
			}
			else {
				component[nCmdIdx].ports[STREAM_DIR_OUT].unUpLimit = 0; /* config not accept */
				printf("[limit] is not allowed for steam %d\n", nCmdIdx);
			}
		}
	#ifdef CONFIG_ENABLE_PL_AUDIO_AGC
		if (component[nCmdIdx].ports[STREAM_DIR_OUT].gAudAgcOn)
		{
			if (QLVAGC_Initial(&component[nCmdIdx], &pVAGCInfo)<0)
			{
				printf("Voice AGC Initial failed\n");
				goto FUNC_END;
			}
			else if(QLVAGC_Config(pVAGCInfo, VAGC_CFG_FILE)<0)
			{
				printf("Voice AGC config failed\n");
				goto FUNC_END;
			}
			if (pVAGCInfo)
			{
				QLVAGC_HwOpen(pVAGCInfo, 0);
				QLVAGC_AgcOnOff(pVAGCInfo, 1);
			}
			if (component[nCmdIdx].ports[STREAM_DIR_OUT].unStarted)
				fprintf(stderr,"==>error, set agc after stream on\n");
			else
				fprintf(stderr,"==>ok, set agc before stream on\n");
	    }
	#endif
	}

#ifdef ENABLE_APP_LINUX_PROFILING
    {
        if (unAppLog) {
            enable_fw_profiling(component[nCmdIdx].hDev);
        }
    }
#endif

	if (nHas2ndCmd)
	{
		nCmdIdx++;
		goto HAS_2ND_CMD;
	}

#ifdef CONFIG_ENABLE_TESTAPPS_SDL

	// initialize SDL
	//
	for (i = 0; i < MAX_SUPPORTED_COMPONENTS; i++)
	{
		if ((component[i].hDev > 0) &&
			(component[i].ports[STREAM_DIR_OUT].portType >= 0) &&
			(component[i].ports[STREAM_DIR_OUT].portType < MAX_COMPONENT_PORT) &&
			component[i].ports[STREAM_DIR_OUT].opened
			)
		{
			if (MEDIA_SDL_OUT == component[i].ports[STREAM_DIR_OUT].eMediaType)
			{
				needSdl = 1;
				ulWidth = max(ulWidth, component[i].ulWidth);
				ulHeight = max(ulHeight, component[i].ulHeight);
			}
		}
	}

	if (needSdl)
	{
		printf("Init SDL (%u x %u)\n", ulWidth, ulHeight);

		if (SDL_Init(SDL_INIT_VIDEO))
		{
			fprintf(stderr, "Could not initialize SDL - %s\n", SDL_GetError());
			goto FUNC_END;
		}
		sdlInited = 1;
		sdlVideoInfo = SDL_GetVideoInfo();

		if (sdlVideoInfo->hw_available)
		{
			printf("Hardware surfaces are available (%dk video memory)\n", sdlVideoInfo->video_mem);
			unSdlVideoFlags |= SDL_HWSURFACE;
		}

		if (sdlVideoInfo->blit_hw)
		{
			printf("Copy blits between hardware surfaces are accelerated\n");
			unSdlVideoFlags |= SDL_ASYNCBLIT;
		}

		if (!(unSdlVideoFlags & SDL_HWSURFACE))
		{
			unSdlVideoFlags |= SDL_SWSURFACE;
		}

		if (Plstream_CreateOverlay(ulWidth, ulHeight))
		{
	 		goto FUNC_END;
		}
	}

#endif //CONFIG_ENABLE_TESTAPPS_SDL

	// create the streaming threads
	//
	for (i = 0; i < MAX_SUPPORTED_COMPONENTS; i++)
	{
		for (j = 0; j < MAX_STREAM_DIR; j++)
		{
			if ((component[i].hDev > 0) &&
				(component[i].ports[j].portType >= 0) &&
				(component[i].ports[j].portType < MAX_COMPONENT_PORT) &&
				component[i].ports[j].opened
				)
			{
				// create streaming thread
				//
				if (STREAM_DIR_IN == j)
				{
					if (COMPONENT_PORT_VIRTUAL_IN != component[i].ports[j].portType)
					{
						if (V4L2_MEMORY_USERPTR == component[i].ports[j].memory)
						{
							lErr = pthread_create(&component[i].ports[j].threadId,
												  NULL,
												  (void *)test_streamin_userptr,
												  &component[i]
												  );
						}
						else
						{
							lErr = pthread_create(&component[i].ports[j].threadId,
												  NULL,
												  (void *)test_streamin,
												  &component[i]
												  );
						}
						if (!lErr)
						{
							component[i].ports[j].ulThreadCreated = 1;
						}
					}
				}
				else
				{
					if (COMPONENT_PORT_VIRTUAL_OUT != component[i].ports[j].portType)
					{
						if (V4L2_MEMORY_USERPTR == component[i].ports[j].memory)
						{
							lErr = pthread_create(&component[i].ports[j].threadId,
												  NULL,
												  (void *)test_streamout_userptr,
												  &component[i]
												  );
						}
						else
						{
							lErr = pthread_create(&component[i].ports[j].threadId,
												  NULL,
												  (void *)test_streamout,
												  &component[i]
												  );
						}
						if (!lErr)
						{
							component[i].ports[j].ulThreadCreated = 1;
						}
					}
				}
			}
		}
	}

	// wait for 100 msec
	usleep(100000);

	// acquire
	//
	for (i = MAX_SUPPORTED_COMPONENTS - 1; i >= 0; i--)
	{
		if (component[i].hDev > 0)
		{
			printf("acquire(%d) hDev(%lld)\n", i, (long long) component[i].hDev);

            APP_PROFILING_LOG(EVENT_APP_ACQUIRE, (TQP_UINT32) component[i].hTask, component[i].devType, 0, 0, 0);

			lErr = ioctl(component[i].hDev, PLDEV_STRM_IOCTL_ACQUIRE, 0);
		}
	}

	// start streaming
	//
	for (i = MAX_SUPPORTED_COMPONENTS - 1; i >= 0; i--)
	{
		if (component[i].hDev > 0)
		{
			printf("start(%d) hDev(%lld)\n", i, (long long) component[i].hDev);

            APP_PROFILING_LOG(EVENT_APP_START, (TQP_UINT32) component[i].hTask, component[i].devType, 0, 0, i);

			lErr = ioctl(component[i].hDev, VIDIOC_STREAMON, &nType);
			if (!lErr)
			{
				for (j = 0; j < MAX_STREAM_DIR; j++)
				{
					if (component[i].ports[j].opened)
					{
						component[i].ports[j].unStarted = 1;
					}
				}
			}
		}
	}


	// wait for user input
	//
	changemode(1);

CHECK_USER_INPUT:
	while ((!kbhit())&&(!g_unCtrlCReceived))
	{
		usleep(30000);
	}

	if(g_unCtrlCReceived)
		goto STOP_FUNC;

	ch = getchar();

	if(ch != EOF)
	{
		printf("\nGot %c\n", ch);
	}
	if ((ch == 'q') ||
		(ch == 'Q') ||
		(ch == 0x03)
		)
	{
		g_unCtrlCReceived = 1;
        if (ch == 'Q')
        {
			g_unKillReceived = 1;
        }
	}
	else
	{
		switch(ch){
		case 'g':		//allocate mem and transfer to osd-driver
				//Lookup component by script file
				for (i = 0; i < MAX_SUPPORTED_COMPONENTS; i++)
				{
					if (component[i].hDev > 0 && (!strcasecmp(component[i].pszScriptName, "vou_on.cfg")))
						break;
				}

				if (i < MAX_SUPPORTED_COMPONENTS)
				{
					nCmdIdx = i;
					do_set(component[nCmdIdx].hDev, "OsdMemory-Command", "request");
					do_set(component[nCmdIdx].hDev, "OsdMemory-Size", "1MB");
				#ifdef SUPPORT_VO_OSD
					trans_mem_address(component[nCmdIdx].hDev);
				#endif
				}
				else
					printf("It is not the component for vou_on.cfg\n");

				break;

			case 'f':		//free osd memory
				if (component[i].hDev > 0 && (!strcasecmp(component[i].pszScriptName, "vou_on.cfg")))
				{
					do_set(component[nCmdIdx].hDev, "OsdMemory-Command", "free");
				#ifdef SUPPORT_VO_OSD
					trans_mem_address(component[nCmdIdx].hDev);
				#endif
				}
				break;

			case 'r':		//change framerate
				//Lookup component
				for (i = 0; i < MAX_SUPPORTED_COMPONENTS; i++)
				{
					if ((component[i].hDev > 0) && (COMPONENT_TYPE_ENCODE ==component[i].devType))
						break;
				}

				if (i < MAX_SUPPORTED_COMPONENTS)
				{
					change_framerate(component[i].hDev);
				}
				else
				{
					printf("No component for encoding\n");
				}
				break;

			case 'b':		//change bitrate
				//Lookup component
				for (i = 0; i < MAX_SUPPORTED_COMPONENTS; i++)
				{
					if ((component[i].hDev > 0) && (COMPONENT_TYPE_ENCODE ==component[i].devType))
						break;
				}

				if (i < MAX_SUPPORTED_COMPONENTS)
				{
					change_bitrate(component[i].hDev);
				}
				else
				{
					printf("No component for encoding\n");
				}
				break;

			case 'a':		//change audio bitrate
				//Lookup component
				for (i = 0; i < MAX_SUPPORTED_COMPONENTS; i++)
				{
					if ((component[i].hDev > 0) && (COMPONENT_TYPE_ENCODE ==component[i].devType))
						break;
				}

				if (i < MAX_SUPPORTED_COMPONENTS)
				{
					change_audio_bitrate(component[i].hDev);
				}
				else
				{
					printf("No component for encoding\n");
				}
				break;

			case 's':		//change audio sampling freq
				//Lookup component
				for (i = 0; i < MAX_SUPPORTED_COMPONENTS; i++)
				{
					if ((component[i].hDev > 0) && (COMPONENT_TYPE_ENCODE ==component[i].devType))
						break;
				}

				if (i < MAX_SUPPORTED_COMPONENTS)
				{
					change_audio_sample_rate(component[i].hDev);
				}
				else
				{
					printf("No component for encoding\n");
				}
				break;

			case 't':		//get raw audio type
				//Lookup component
				for (i = 0; i < MAX_SUPPORTED_COMPONENTS; i++)
				{
					if ((component[i].hDev > 0) && (COMPONENT_TYPE_ENCODE ==component[i].devType))
						break;
				}

				if (i < MAX_SUPPORTED_COMPONENTS)
				{
					get_raw_audio_type(component[i].hDev);
				}
				else
				{
					printf("No component for encoding\n");
				}
				break;

			case 'i':		//Insert I frame
				//Lookup component
				for (i = 0; i < MAX_SUPPORTED_COMPONENTS; i++)
				{
					if ((component[i].hDev > 0) && (COMPONENT_TYPE_ENCODE ==component[i].devType))
						break;
				}

				if (i < MAX_SUPPORTED_COMPONENTS)
				{
					insert_i_frame(component[i].hDev);
				}
				else
				{
					printf("No component for encoding\n");
				}
				break;

			case 'd':		//get one demand frame
				{
					int found=0;
					//Lookup component
					for (i = 0; i < MAX_SUPPORTED_COMPONENTS; i++)
					{
						if ((component[i].hDev > 0) && (component[i].ports[STREAM_DIR_OUT].unJPEGOnDemand == 1))
						{
							insert_jpeg_frame_request(component[i].hDev);
							found = 1;
						}
					}
					if (!found)
					{
            	        printf("No component supports insert_jpeg_frame_request\n");
					}
				}
				break;

			case 'c':
				{
#ifdef CONFIG_ENABLE_MOTION_DETECT
					for (i = 0; i < MAX_SUPPORTED_COMPONENTS; i++)
					{
						if ((component[i].hDev > 0)&&(component[i].ports[STREAM_DIR_OUT].portType == COMPONENT_PORT_ACTIVITY_OUT))
						{
							if(QLMD_Config(pMDInfo, MD_CFG_FILE)<0)
							{
								printf("Motion detect config failed\n");
							}
						}
					}
#endif
				}
				break;

			case 'l':		//Aiu Volume Up
				//Lookup component
				for (i = 0; i < MAX_SUPPORTED_COMPONENTS; i++)
				{
					if ((component[i].hDev > 0) && (COMPONENT_TYPE_ENCODE ==component[i].devType))
						break;
				}

				if (i < MAX_SUPPORTED_COMPONENTS)
				{
					change_aiu_volume(component[i].hDev);
				}
				else
				{
					printf("No component for volume up\n");
				}
				break;

			case 'm':		//Aou Volume Down
				//Lookup component
				for (i = 0; i < MAX_SUPPORTED_COMPONENTS; i++)
				{
					if ((component[i].hDev > 0) && (COMPONENT_TYPE_DECODE ==component[i].devType))
						break;
				}

				if (i < MAX_SUPPORTED_COMPONENTS)
				{
					change_aou_volume(component[i].hDev);
				}
				else
				{
					printf("No component for volume down\n");
				}
				break;

				case 'w':		//Front end register dump
					//Lookup component
					for (i = 0; i < MAX_SUPPORTED_COMPONENTS; i++)
					{
						if (component[i].hDev > 0)
							break;
					}

					if (i < MAX_SUPPORTED_COMPONENTS)
					{
						dump_frontend_regs(component[i].hDev);
					}
					else
					{
						printf("No component for register dump\n");
					}
					break;
		}

		goto CHECK_USER_INPUT;
	}

STOP_FUNC:

	changemode(0);
	//printf("==>stop===\n");

#if 1
#ifdef	SUPPORT_VI_OSD
	/* If gViuosdOn on and not start yet, wait it finish start process or maximum 3 sec, to avoid driver race condition */
	for (i = 0; i < MAX_SUPPORTED_COMPONENTS; i++)
	{
		if (component[i].hDev > 0)
		{
		    if ((component[i].ports[STREAM_DIR_OUT].opened) && (component[i].ports[STREAM_DIR_OUT].gViuosdOn))
		    {
		        int loop = 0;
		        while ((component[i].ports[STREAM_DIR_OUT].gViuAddressOK==0) && (loop<300))
		        {
                    //fprintf(stderr, "hDev(%d) Wait VI-OSD Start!\n", component[i].hDev);
                    usleep(10000);
                    loop++;
		        }
		    }
		}
	}
#endif
#endif
	// stop streaming
	//
	for (i = MAX_SUPPORTED_COMPONENTS - 1; i >= 0; i--)
	{
		if (component[i].hDev > 0)
		{
			printf("stop(%d) hDev(%lld)\n", i, (long long) component[i].hDev);

            APP_PROFILING_LOG(EVENT_APP_STOP, (TQP_UINT32) component[i].hTask, component[i].devType, 0, 0, i);

			ioctl(component[i].hDev, VIDIOC_STREAMOFF, &nType);
			for (j = 0; j < MAX_STREAM_DIR; j++)
			{
				if (component[i].ports[j].opened)
				{
					component[i].ports[j].unSentStopCmd = 1;
				}
			}
		}
	}

	// wait for 100 msec
	////usleep(100000);

	// propagate the signal to all the threads
	//
	for (i = MAX_SUPPORTED_COMPONENTS - 1; i >= 0; i--)
	{
		for (j = 0; j < MAX_STREAM_DIR; j++)
		{
			if (component[i].ports[j].ulThreadCreated)
			{
				printf("%s() set component(%d) port(%d) unCtrlCReceived\n", __FUNCTION__, i, j);
				component[i].ports[j].unCtrlCReceived = 1;
			}
		}
	}

	// wait for 100 msec
	usleep(100000);

FUNC_END:

	printf("%s() FUNC_END\n", __FUNCTION__);

	for (i = 0; i < MAX_SUPPORTED_COMPONENTS; i++)
	{
		for (j = 0; j < MAX_STREAM_DIR; j++)
		{
			// close ports
			if (component[i].ports[j].opened)
			{
				printf("close component(%d) port (%d)\n", i, j);

                APP_PROFILING_LOG(EVENT_APP_PORTCLOSE, (TQP_UINT32) component[i].hTask, component[i].devType, component[i].ports[j].portType, 0, 0);

				ioctl(component[i].hDev,
					  PLDEV_STRM_IOCTL_PORT_CLOSE,
					  &component[i].ports[j].portType
					  );
				component[i].ports[j].opened = QP_FALSE;
			}
		}

		// close device
		if (component[i].hDev > 0)
		{
			printf("close component(%d) hDev(%lld)\n", i, (long long) component[i].hDev);

            APP_PROFILING_LOG(EVENT_APP_CLOSE, (TQP_UINT32) component[i].hTask, component[i].devType, 0, 0, 0);
#ifndef CONFIG_HOST_PLATFORM_USERSPACE
			close(component[i].hDev);
#else
			plv4l_close(component[i].hDev);
#endif
			component[i].hDev = 0;
		}
	}


	for (i = 0; i < MAX_SUPPORTED_COMPONENTS; i++)
	{
		for (j = 0; j < MAX_STREAM_DIR; j++)
		{
			if (component[i].ports[j].ulThreadCreated)
			{
				pthread_join(component[i].ports[j].threadId,
							 NULL
							 );
				component[i].ports[j].ulThreadCreated = 0;
			}
		}
	}

#ifdef CONFIG_ENABLE_TESTAPPS_SDL
    pthread_mutex_destroy(&g_sdlSurfaceLock);
	if (g_psdlOverlay)
	{
		SDL_FreeYUVOverlay(g_psdlOverlay);
	}
	if (g_psdlSurface)
	{
		SDL_FreeSurface(g_psdlSurface);
	}
	if (sdlInited)
	{
		SDL_Quit();
	}
#endif //CONFIG_ENABLE_TESTAPPS_SDL

#ifdef ENABLE_APP_LINUX_PROFILING
	if (unAppLog) {
	    APP_PORFILING_LOG_DUMP(applogname);
    	APP_PORFILING_LOG_FINT();
	}
#endif

	g_unEnableLoopCnt++;
	if ((g_unEnableLoop) &&
		(g_unKillReceived==0) &&
		((g_unEnableLoopNeedCnt==0) || ((g_unEnableLoopNeedCnt!=0) && (g_unEnableLoopCnt <= g_unEnableLoopNeedCnt)))) {
		int i;
		for (i=g_unEnableLoopDelay;i>=0;i--) {
			printf("==> next start in %d sec [%d/%d]==[%d]\n", i, g_unEnableLoopCnt, g_unEnableLoopNeedCnt, g_unEnableLoopDelay);
			if (g_unKillReceived)
				break;
			sleep(1);
		}
		if (g_unKillReceived==0)
			goto LOOP_START;
	}

#ifdef CONFIG_HOST_PLATFORM_USERSPACE
	plusb_fini_module();
#endif

	return (lErr);
}

