///////////////////////////////////////////////////////////////////////////////
//     
//
// Cqlcodec.c
//
// Description:
//
//  implementation of the codec chip
//
// History:
//	Aug 4, 2006	(dt) initial check in
//
// Authors: (dt) David Tao
//
///////////////////////////////////////////////////////////////////////////////


#include "cqlcodec.h"
#include "cqlcodeclib.h"
#include "qpmemmgr.h"
#include "qpdbgmgr.h"
#include "util.h"
#ifndef QP_LINXKER
#include <string.h>
#endif //!QP_LINXKER
#include "codeclib.h"
#include "usblib.h"
#include "qpdevdiag.h"

#if defined(QP_LINXKER) || defined(_QP_LINUX) || defined(_QP_MACOSX)
#include "qlautoconf.h"
#endif //QP_LINXKER

extern TQP_DbgMCompID	g_QPCodecDBGCompID;
extern TQP_UINT32		g_syncModeTable[19];



/////////////////////////////////////////////////////////////////////////////
//
//

#define QL201_VID_FW_START_ADDR		0x0
#define QL201_AUD_FW_START_ADDR		0x100000


/////////////////////////////////////////////////////////////////////////////
//
//

// IMpegCodec
//
EQP_Errors CQLCodec_AllocEncodeTask(IMpegCodec *This_p,
									PTQP_OBJECT_HANDLE phTask
									);

EQP_Errors CQLCodec_AllocDecodeTask(IMpegCodec *This_p,
									PTQP_OBJECT_HANDLE phTask
									);
EQP_Errors CQLCodec_ReleaseTask(IMpegCodec *This_p,
								TQP_OBJECT_HANDLE hTask
								);
EQP_Errors CQLCodec_Open(IMpegCodec *This_p,
						 TQP_OBJECT_HANDLE hTask,
						 TQP_UINT32 dwFlags,
						 TQP_PVOID pDataFormat,
						 PTQP_OBJECT_HANDLE phStream,
						 QP_DEVICE_CALLBACK pFuncCallback,
						 TQP_PVOID context
						 );
EQP_Errors CQLCodec_Close(IMpegCodec *This_p,
						  TQP_OBJECT_HANDLE hStream
						  );
EQP_Errors CQLCodec_Start(IMpegCodec *This_p,
						  TQP_OBJECT_HANDLE hStream
						  );
EQP_Errors CQLCodec_Stop(IMpegCodec *This_p,
						 TQP_OBJECT_HANDLE hStream
						 );
EQP_Errors CQLCodec_Acquire(IMpegCodec *This_p,
							TQP_OBJECT_HANDLE hStream
							);
EQP_Errors CQLCodec_Pause(IMpegCodec *This_p,
						  TQP_OBJECT_HANDLE hStream
						  );
EQP_Errors CQLCodec_Step(IMpegCodec *This_p,
						 TQP_OBJECT_HANDLE hStream,
						 TQP_UINT32 nb
						 );
EQP_Errors CQLCodec_SetRate(IMpegCodec *This_p,
							TQP_OBJECT_HANDLE hStream,
							TQP_INT32 lNewRate
							);
EQP_Errors CQLCodec_GetRate(IMpegCodec *This_p,
							TQP_OBJECT_HANDLE hStream,
							PTQP_INT32 plRate
							);
EQP_Errors CQLCodec_BeginFlush(IMpegCodec *This_p,
							   TQP_OBJECT_HANDLE hStream
							   );
EQP_Errors CQLCodec_Flush(IMpegCodec *This_p,
						  TQP_OBJECT_HANDLE hStream
						  );
EQP_Errors CQLCodec_EndFlush(IMpegCodec *This_p,
							 TQP_OBJECT_HANDLE hStream
							 );
EQP_Errors CQLCodec_AddBuffer(IMpegCodec *This_p,
							  TQP_OBJECT_HANDLE hStream,
							  PQP_BUFFER_DESCRIPTOR pBufDesc
							  );
EQP_Errors CQLCodec_CancelBuffer(IMpegCodec *This_p,
								 TQP_OBJECT_HANDLE hStream,
								 PQP_BUFFER_DESCRIPTOR pBufDesc
								 );
EQP_Errors CQLCodec_TimeoutBuffer(IMpegCodec *This_p,
								  TQP_OBJECT_HANDLE hStream,
								  PQP_BUFFER_DESCRIPTOR pBufDesc
								  );
EQP_Errors CQLCodec_GetTime(IMpegCodec *This_p,
							TQP_OBJECT_HANDLE hStream,
		  					PTQP_UINT64 pTime
		  					);
EQP_Errors CQLCodec_XferData(IMpegCodec *This_p,
							 TQP_OBJECT_HANDLE hStream,
							 PQP_BUFFER_DESCRIPTOR pBufDesc
							 );



/////////////////////////////////////////////////////////////////////////////
//
//

// Interrupt
//
void CQLCodec_InterruptHandler(void * refCon);
void CQLCodec_InterruptHandlerUSB(void * refCon);



/////////////////////////////////////////////////////////////////////////////
//
//

static void CQLCodec_LoadDefaultSettings(CQLCodec *This)
{
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		// Encoder register defination
		//
		This->m_ENC_REG_MESSAGE = REG_TO_ARM_MESSAGE;
		This->m_ENC_REG_SYSTEM_CONTROL = REG_TO_ARM_1;
		This->m_ENC_REG_PICTURE_RESOLUTION = REG_TO_ARM_2;
		This->m_ENC_REG_INPUT_CONTROL = REG_TO_ARM_3;
		This->m_ENC_REG_RATE_CONTROL = REG_TO_ARM_4;
		This->m_ENC_REG_BIT_RATE = REG_TO_ARM_5;
		This->m_ENC_REG_FILTER_CONTROL = REG_TO_ARM_6;
		This->m_ENC_REG_GOP_LOOP_FILTER = REG_TO_ARM_7;
		This->m_ENC_REG_OUT_PIC_RESOLUTION = REG_TO_ARM_8;
		This->m_ENC_REG_ET_CONTROL = REG_TO_ARM_9;
		This->m_ENC_REG_BLOCK_SIZE = REG_TO_ARM_9;
		This->m_ENC_REG_AUDIO_CONTROL_PARAM = REG_TO_ARM_10;
		This->m_ENC_REG_AUDIO_CONTROL_EX = REG_TO_ARM_11;

		// 656 output configuration
		//
		This->m_VOEnable = QP_TRUE;
		This->m_VOUMode = QPCODEC_VOUMODE_NTSC_INTERLACED;
		This->m_VOUStartPixel = 1;
		This->m_VOUStartLine = 0;

		// I2S output configuration
		//
		This->m_AOEnable = QP_TRUE;

		This->m_AOControls.Read = 0;
		This->m_AOControls.Reg.ao_en = 1;
		This->m_AOControls.Reg.ao_msb = AO_MSB_MSB;
		This->m_AOControls.Reg.lrclk_i = AO_LRCLK_I_LEFT_LOW;
		This->m_AOControls.Reg.bclk_i = AO_BCLK_I_FALLING_EDGE;	// 0
		This->m_AOControls.Reg.ao_i2s = AO_I2S_PROGRAM;
		This->m_AOControls.Reg.ao_rj = AO_RJ_LEFT_JUSTIFIED;
		This->m_AOControls.Reg.ao_s = AO_MODE_MASTER;

		This->m_AudControls.Read = 0;
		This->m_AudControls.Reg.audio_type = AUDIO_TYPE_AAC;
		This->m_AudControls.Reg.ch_num = 2;
		This->m_AudControls.Reg.sample_rate = SAMPLE_RATE_48K;

		This->m_AIVolume = 8;
		This->m_AOVolume = 8;
	}
	else
	{
		// Encoder register defination
		//
		This->m_ENC_REG_MESSAGE = REG_TO_ARM_MESSAGE;
		This->m_ENC_REG_SYSTEM_CONTROL = REG_TO_ARM_1;
		This->m_ENC_REG_PICTURE_RESOLUTION = REG_TO_ARM_2;
		This->m_ENC_REG_INPUT_CONTROL = REG_TO_ARM_3;
		This->m_ENC_REG_RATE_CONTROL = REG_TO_ARM_4;
		This->m_ENC_REG_BIT_RATE = REG_TO_ARM_5;
		This->m_ENC_REG_FILTER_CONTROL = REG_TO_ARM_6;
		This->m_ENC_REG_GOP_LOOP_FILTER = REG_TO_ARM_7;
		This->m_ENC_REG_ET_CONTROL = REG_TO_ARM_8;
		This->m_ENC_REG_OUT_PIC_RESOLUTION = REG_TO_ARM_9;
		This->m_ENC_REG_AUDIO_CONTROL_PARAM = REG_TO_ARM_10;
		This->m_ENC_REG_AUDIO_CONTROL_EX = REG_TO_ARM_11;


		// 656 output configuration
		//
		This->m_VOEnable = QP_TRUE;
		This->m_VOUMode = QPCODEC_VOUMODE_NTSC_INTERLACED;
		This->m_VOUStartPixel = 1;
		This->m_VOUStartLine = 0;

		// I2S output configuration
		//
		This->m_AOEnable = QP_TRUE;

		This->m_AOControls.Read = 0;
		This->m_AOControls.Reg.ao_en = 1;
		This->m_AOControls.Reg.ao_msb = AO_MSB_MSB;
		This->m_AOControls.Reg.lrclk_i = AO_LRCLK_I_LEFT_LOW;
		This->m_AOControls.Reg.bclk_i = AO_BCLK_I_FALLING_EDGE;	// 0
		This->m_AOControls.Reg.ao_i2s = AO_I2S_PROGRAM;
		This->m_AOControls.Reg.ao_rj = AO_RJ_LEFT_JUSTIFIED;
		This->m_AOControls.Reg.ao_s = AO_MODE_MASTER;

		This->m_AudControls.Read = 0;
		This->m_AudControls.Reg.audio_type = AUDIO_TYPE_AAC;
		This->m_AudControls.Reg.ch_num = 2;
		This->m_AudControls.Reg.sample_rate = SAMPLE_RATE_48K;

		This->m_AIVolume = 8;
		This->m_AOVolume = 8;
	}
}



/////////////////////////////////////////////////////////////////////////////
//
//


static EQP_Errors CQLCodec_RegisterISR(CQLCodec *This,
									   QPHCI_BUS BusType,
									   CPCIeCntl *pPCIeCntl
									   )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;

	This->m_bInterruptAttached = QP_FALSE;

	switch (BusType)
	{
		case QPHCI_BUS_PCI:
#ifdef _QP_WINKER
			qpStatus = CPCIeCntl_RegisterISR(pPCIeCntl);
			This->m_bInterruptAttached = QP_SUCCESS(qpStatus);
#endif //_QP_WINKER
			break;
		case QPHCI_BUS_USB:
		case QPHCI_BUS_201_EMULATION:
			qpStatus = QPOSMRegisterISR(&This->m_isrID,
										CQLCodec_InterruptHandlerUSB,
										(void *)This,
										This->m_interruptNumber
										);
			This->m_bInterruptAttached = QP_SUCCESS(qpStatus);
			break;
		case QPHCI_BUS_CUSTOM_1:
		case QPHCI_BUS_INTERNAL:
		default:
			qpStatus = QPOSMRegisterISR(&This->m_isrID,
										CQLCodec_InterruptHandler,
										(void *)This,
										This->m_interruptNumber
										);
			This->m_bInterruptAttached = QP_SUCCESS(qpStatus);
	}

	return (qpStatus);

}



static EQP_Errors CQLCodec_UnregisterISR(CQLCodec *This,
										 QPHCI_BUS BusType,
										 CPCIeCntl *pPCIeCntl
										 )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;

	switch (BusType)
	{
		case QPHCI_BUS_PCI:
#ifdef _QP_WINKER
			qpStatus = CPCIeCntl_UnregisterISR(pPCIeCntl);
#endif //_QP_WINKER
			break;
		case QPHCI_BUS_USB:
		case QPHCI_BUS_CUSTOM_1:
		case QPHCI_BUS_201_EMULATION:
		case QPHCI_BUS_INTERNAL:
		default:
			QPOSMUnregisterISR(This->m_isrID);
	}

	return (qpStatus);
}



/////////////////////////////////////////////////////////////////////////////
//
//

// call back function
//
EQP_Errors CQLCodec_DeviceCallback(CQLCodec *This,
								   TQP_UINT32 dwCode,
								   TQP_PVOID pParam
								   )
{
	if (This->m_pDeviceCallback &&
		This->m_callbackContext
		)
	{
		return (This->m_pDeviceCallback(This->m_callbackContext,
										dwCode,
										pParam
										));
	}
	else
	{
		return (QPERR_FAIL);
	}
}



// initialize memory controller
//
EQP_Errors CQLCodec_InitializeMemory(CQLCodec *This)
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_InitializeMemory() type(%d) size(%dMb)\n",
				 This->m_MemType,
				 This->m_MemSize
				 ));

	if ((QPHCI_BUS_INTERNAL == This->m_hci.m_bus_type) &&
		(QPHCI_MODE_MASTER == This->m_hci.m_access_mode)
		)
	{
		return (QPERR_SUCCESS);
	}

	if ((QPHCI_BUS_USB == This->m_hci.m_bus_type) &&
		(QPHCI_MODE_DIRECT == This->m_hci.m_access_mode) &&
		(QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
		)
	{
		return (QPERR_SUCCESS);
	}

	if ((This->m_MemType < QPCODEC_MEM_TYPE_FIRST) ||
		(This->m_MemType > QPCODEC_MEM_TYPE_LAST)
		)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CQLCodec_InitializeMemory() failed invalid type(%d)\n",
					 This->m_MemType
					 ));
		return (QPERR_PARMS);
	}

	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		if (QPCODEC_CHIP_ARTESA == This->m_ChipType)
		{
			TQP_UINT32	regF1C;

			if (QLCODEC_VER_QL330_B0 == This->m_ChipVersion.Read)
			{
				TQP_UINT32 regF14;
				TQP_UINT32 mem0;
				TQP_UINT32 regCas = 7;
				TQP_UINT32 regRas = 2;

				// detect CAS
				regF14 = (regRas << 16) | regCas;
				QPHCI_RegisterWrite(&This->m_hci,
									0xf14,
									regF14
									);
				QPHCI_MemoryWrite(&This->m_hci,
								  0,
								  regF14
								  );
				while (regCas > 3)
				{
					QPHCI_MemoryWrite(&This->m_hci,
									  0x1 << (regCas+6),
									  regCas-1
									  );
					regCas--;
				}
				QPHCI_MemoryRead(&This->m_hci,
								 0,
								 &mem0
								 );
				regCas = mem0 & 0xf;

				// detect RAS
				regF14 = (regRas << 16) | regCas;
				QPHCI_RegisterWrite(&This->m_hci,
									0xf14,
									regF14
									);
				QPHCI_MemoryWrite(&This->m_hci,
								  0,
								  regRas
								  );
				while (regRas > 1)
				{
					QPHCI_MemoryWrite(&This->m_hci,
									  0x1 << (regCas+21),
									  regRas-1
									  );
					regRas--;
				}
				QPHCI_MemoryRead(&This->m_hci,
								 0,
								 &mem0
								 );
				regRas = mem0 & 0xf;


				// set register 0xf14
				regF14 = (regRas << 16) | regCas;
				QPHCI_RegisterWrite(&This->m_hci,
									0xf14,
									regF14
									);
			}
			else
			{
				switch (This->m_MemSize)
				{
					case 128:
						// set 128Mb to 8 colume [2:0]=3
						// set 128Mb to 12 row [17:16]=1
						//
						QPHCI_RegisterWrite(&This->m_hci,
											0xf14,
											0x00010003
											);
						break;

					case 256:
						// set 256Mb to 9 colume [2:0]=4
						// set 256Mb to 12 row [17:16]=1
						//
						QPHCI_RegisterWrite(&This->m_hci,
											0xf14,
											0x00010004
											);
						break;

					case 512:
						// set 512Mb to 9 colume [2:0]=4
						// set 512Mb to 13 row [17:16]=2
						//
						QPHCI_RegisterWrite(&This->m_hci,
											0xf14,
											0x00020004
											);
						break;

					case 1024:
						// set 1024Mb to 10 colume [2:0]=5
						// set 1024Mb to 13 row [17:16]=2
						//
						QPHCI_RegisterWrite(&This->m_hci,
											0xf14,
											0x00020005
											);
						break;

					default:
						QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
									 "CQLCodec_InitializeMemory() failed invalid size(%d)\n",
									 This->m_MemSize
									 ));
						return (QPERR_PARMS);
				}
			}

			// clear bit [9:8] of register F1C
			QPHCI_RegisterRead(&This->m_hci,
							   0xf1c,
							   &regF1C
							   );
			QPHCI_RegisterWrite(&This->m_hci,
								0xf1c,
								regF1C & 0xFFFFFCFF
								);
		}
		else
		{
			switch (This->m_MemSize)
			{
				case 64:
					// set 64Mb to 8 colume [2:0]=3
					// set 64Mb to 12 row [17:16]=1
					//
					QPHCI_RegisterWrite(&This->m_hci,
										0xf14,
										0x00010103
										);
					break;

				case 128:
					// set 128Mb to 8 colume [2:0]=4
					// set 128Mb to 12 row [17:16]=1
					//
					QPHCI_RegisterWrite(&This->m_hci,
										0xf14,
										0x00010004
										);
					break;

				case 256:
					// set 256Mb to 9 colume [2:0]=4
					// set 256Mb to 12 row [17:16]=2
					//
					QPHCI_RegisterWrite(&This->m_hci,
										0xf14,
										0x00020004
										);
					break;

				case 512:
					// set 512Mb to 9 colume [2:0]=5
					// set 512Mb to 13 row [17:16]=2
					//
					QPHCI_RegisterWrite(&This->m_hci,
										0xf14,
										0x00020005
										);
					break;

				case 1024:
					// set 1024Mb to 10 colume [2:0]=5
					// set 1024Mb to 13 row [17:16]=3
					//
					QPHCI_RegisterWrite(&This->m_hci,
										0xf14,
										0x00030005
										);
					break;

				default:
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
								 "CQLCodec_InitializeMemory() failed invalid size(%d)\n",
								 This->m_MemSize
								 ));
					return (QPERR_PARMS);
			}
		}

		// Andy 2008.09.05: set refresh time
		QPHCI_RegisterWrite(&This->m_hci,
							0xf04,
							0x0D03110B
							);

		// set CAS latency to 3 for mDDR
		QPHCI_RegisterWrite(&This->m_hci,
							0xf08,
							0x00000003
							);

		if (QPCODEC_MEM_TYPE_mDDR == This->m_MemType)
		{
			if ((QLCODEC_VER_QL330_A0 == This->m_ChipVersion.Read) ||
				(QLCODEC_VER_QL330_A1 == This->m_ChipVersion.Read)
				)
			{
				// set dk-dqs delay to zero for 330A0 & 330A1
				QPHCI_RegisterWrite(&This->m_hci,
									0xf40,
									0
									);
			}
			else if (QLCODEC_VER_QL330_B0 == This->m_ChipVersion.Read)
			{
				// set dk-dqs delay to two for 330B0
				QPHCI_RegisterWrite(&This->m_hci,
									0xf40,
									2
									);
			}
		}

		if (QPCODEC_CHIP_ARTESA_VERTEX_5 == This->m_ChipType)
		{
			QPHCI_RegisterWrite(&This->m_hci,
								0xf10,
								0x01240080
								);
		}
		else
		{
			// refresh count; must < 7.8us
			// @166MHz
			//
			QPHCI_RegisterWrite(&This->m_hci,
								0xf10,
								0x05140080
								////0x01240080
								);
		}

		// issue MEM init sequence by set [0]=1
		QPHCI_RegisterWrite(&This->m_hci,
							0xf18,
							0x1
							);

		if (QPCODEC_CHIP_ARTESA_VERTEX_5 == This->m_ChipType)
		{
//			QPHCI_RegisterWrite(&This->m_hci,
//								0x60,
//								0x00ff00ff//0x3f003f//0x100010
//								);

			QPHCI_RegisterWrite(&This->m_hci,
								0x60,
								0x00200020//0x3f003f//0x100010
								);


			// for running scripts
			QPHCI_WriteHciRegister(&This->m_hci,
								   0x3,
								   0x0
								   );
			QPHCI_RegisterWrite(&This->m_hci,
								0x44,
								0x0
								);
			QPHCI_RegisterWrite(&This->m_hci,
								0x0,
								0xC000
								);
		}
		else if (QPCODEC_CHIP_ARTESA_VERTEX_4 == This->m_ChipType)
		{
			QPHCI_RegisterWrite(&This->m_hci,
								0x60,
								0x1a001a
								);
		}
	}
	else
	{
		switch (This->m_MemSize)
		{
			case 64:
				// set 64Mb to 8 colume [2:0]=3
				// set 64Mb to 12 row [17:16]=1
				//
				QPHCI_RegisterWrite(&This->m_hci,
									0xf14,
									0x00010103
									);
				break;

			case 128:
				// set 128Mb to 9 colume [2:0]=4
				// set 128Mb to 12 row [17:16]=1
				//
				QPHCI_RegisterWrite(&This->m_hci,
									0xf14,
									0x00010104
									);
				break;

			case 256:
				// set 256Mb to 9 colume [2:0]=4
				// set 256Mb to 13 row [17:16]=2
				//
				QPHCI_RegisterWrite(&This->m_hci,
									0xf14,
									0x00020104
									);
				break;

			case 512:
				// set 512Mb to 10 colume [2:0]=5
				// set 512Mb to 13 row [17:16]=2
				//
				QPHCI_RegisterWrite(&This->m_hci,
									0xf14,
									0x00020105
									);
				break;

			case 1024:
				// set 1024Mb to 10 colume [2:0]=5
				// set 1024Mb to 14 row [17:16]=3
				//
				QPHCI_RegisterWrite(&This->m_hci,
									0xf14,
									0x00030105
									);
				break;

			default:
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodec_InitializeMemory() failed invalid size(%d)\n",
							 This->m_MemSize
							 ));
				return (QPERR_PARMS);
		}

		if (QPCODEC_MEM_TYPE_mDDR == This->m_MemType)
		{
			// set CAS latency to 3 for mDDR
			QPHCI_RegisterWrite(&This->m_hci,
								0xf08,
								0x00000003
								);
			// set dk-dqs delay, default to 4
			QPHCI_RegisterWrite(&This->m_hci,
								0xf40,
								0x00000004
								);
		}

		// issue MEM init sequence by set [0]=1
		QPHCI_RegisterWrite(&This->m_hci,
							0xf18,
							0x1
							);
	}

	DelayMilliSeconds(100);

	return (QPERR_SUCCESS);
}



EQP_Errors CQLCodec_AOSwitch(CQLCodec *This,
							 TQP_BOOL bOn
							 )
{
	EQP_Errors	qpStatus;
	TQP_UINT32	reg50;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_AOSwitch() (%d)\n",
				  bOn
				  ));

	qpStatus = QPHCI_RegisterRead(&This->m_hci,
								  0x50,
								  &reg50
								  );
	if (QP_SUCCESS(qpStatus))
	{
		// turn on/off AO
		//
		if (bOn)
			reg50 &= ~0x2;
		else
			reg50 |= 0x2;

		qpStatus = QPHCI_RegisterWrite(&This->m_hci,
									   0x50,
									   reg50
									   );
	}

	return (qpStatus);
}



EQP_Errors CQLCodec_VOSwitch(CQLCodec *This,
							 TQP_BOOL bOn
							 )
{
	EQP_Errors	qpStatus;
	TQP_UINT32	reg50;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_VOSwitch (%d)\n",
				 bOn
				 ));

	qpStatus = QPHCI_RegisterRead(&This->m_hci,
								  0x50,
								  &reg50
								  );
	if (QP_SUCCESS(qpStatus))
	{
		// turn on/off VO
		//
		if (bOn)
			reg50 &= ~0x4;
		else
			reg50 |= 0x4;

		qpStatus = QPHCI_RegisterWrite(&This->m_hci,
									   0x50,
									   reg50
									   );
	}
	return (qpStatus);
}



EQP_Errors CQLCodec_SetGPIODefaults(CQLCodec *This)
{
	EQP_Errors	qpStatus;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_SetGPIODefaults() dir(0x%08x) val(0x%08x)\n",
				  This->m_GPIODirections,
				  This->m_GPIOValues
				  ));

	qpStatus = QPHCI_RegisterWrite(&This->m_hci,
								   0x610,
								   This->m_GPIODirections
								   );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  0x614,
																	  This->m_GPIOValues
																	  );
	return (qpStatus);
}



EQP_Errors CQLCodec_PowerUp(CQLCodec *This)
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;

	if ((QPHCI_BUS_INTERNAL == This->m_hci.m_bus_type) &&
		(QPHCI_MODE_MASTER == This->m_hci.m_access_mode)
		)
	{
		This->m_PowerState = QLCODEC_PWR_STATE_D0;
		return (qpStatus);
	}

	// artesa bring up
	//
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		return (QPERR_SUCCESS);
	}

	ENTER_CRITICAL(&This->m_Object)

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		// reset the chip to bring back other default settings
		//
		qpStatus = QPHCI_ResetArm(&This->m_hci,
				     			  QP_FALSE
				     			  );

		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_ReInit(&This->m_hci);
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : CQLCodec_InitializeMemory(This);
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : CQLCodec_AOSwitch(This,
																		This->m_AOEnable
																		);
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : CQLCodec_VOSwitch(This,
																		This->m_VOEnable
																		);
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : CQLCodec_SetGPIODefaults(This);

		DelayMilliSeconds(5);

		// start arm running
		//
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_ResetArm(&This->m_hci,
																	 QP_TRUE
																	 );
		if (QP_SUCCESS(qpStatus))
			This->m_PowerState = QLCODEC_PWR_STATE_D0;
	}

	LEAVE_CRITICAL(&This->m_Object)

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "CQLCodec_PowerUp status(%d) power state(%d)\n",
				 qpStatus,
				 This->m_PowerState
				 ));
	return (qpStatus);
}



EQP_Errors CQLCodec_PowerDown(CQLCodec *This)
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	TQP_UINT32	dwReg = 0;

	if ((QPHCI_BUS_INTERNAL == This->m_hci.m_bus_type) &&
		(QPHCI_MODE_MASTER == This->m_hci.m_access_mode)
		)
	{
		return (qpStatus);
	}

	// artesa bring up
	//
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		return (QPERR_SUCCESS);
	}

	ENTER_CRITICAL(&This->m_Object)

	if (QLCODEC_PWR_STATE_D0 == This->m_PowerState)
	{
		qpStatus = QPHCI_EnableArm(&This->m_hci,
								   QP_FALSE
								   );

		// clear mail box tokens
		//
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  REG_TO_ARM_MESSAGE_STATUS,
																		  0
																		  );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  REG_TO_HOST_MESSAGE_STATUS,
																		  0
																		  );

		// toggle reset and gating the clocks
		//
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterRead(&This->m_hci,
																		 0x0,
																		 &dwReg
																		 );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  0x0,
																		  dwReg & 0xE0000000
																		  );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																		  0x0,
																		  (dwReg & 0xE0000000) | 0x01FFFFFF
																		  );
		This->m_PowerState = QLCODEC_PWR_STATE_D1;
	}

	LEAVE_CRITICAL(&This->m_Object)

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "CQLCodec_PowerDown status(%d) power state(%d)\n",
				 qpStatus,
				 This->m_PowerState
				 ));
	return (qpStatus);

}



/////////////////////////////////////////////////////////////////////////////
//
//

// IGenericDevice
//
static EQP_Errors CQLCodec_InitDevice(IMpegCodec *This_p,
									  QP_DEVICE_CALLBACK pFuncCallback,
									  TQP_PVOID context,
									  TQP_PVOID pInitData
									  )
{
	EQP_Errors				qpStatus;
	PQPMPGCODEC_INITDATA	pMpgCodecInitData = (PQPMPGCODEC_INITDATA)pInitData;
	CQLCodec				*This =	GET_INHERITED_OBJECT(CQLCodec, m_iMpegCodec, This_p);
	TQP_UINT32				reg;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_InitDevice() This_p(0x%x) This(0x%x) initData(0x%x)\n",
				 This_p,
				 This,
				 pMpgCodecInitData
				 ));

	if (CObject_IsInitialized(&This->m_Object))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
					 "CQLCodec_InitDevice() Already Initialized. Skipping...\n"
					 ));
		return (QPERR_SUCCESS);
	}

	// remember initialization data
	//
	if (pMpgCodecInitData)
	{
		This->m_MemType = pMpgCodecInitData->MemType;
		This->m_MemSize = pMpgCodecInitData->MemSize;
		This->m_ChipType = pMpgCodecInitData->ChipType;
		This->m_VerFwAPI = pMpgCodecInitData->VerFwAPI;
		This->m_FwIntMode = pMpgCodecInitData->FwIntMode;
		This->m_FwFixedMode = pMpgCodecInitData->FwFixedMode;
		This->m_GPIODirections = pMpgCodecInitData->GPIODirections;
		This->m_GPIOValues = pMpgCodecInitData->GPIOValues;
		This->m_ErrorRecovery = pMpgCodecInitData->ErrorRecovery;
		This->m_Pll4 = pMpgCodecInitData->Pll4;
		This->m_Pll5 = pMpgCodecInitData->Pll5;
		This->m_UseArtesaExt340 = pMpgCodecInitData->UseArtesaExt340;
		This->m_FilterOutHDCPContent = pMpgCodecInitData->FilterOutHDCPContent;

		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "%s() UseArtesaExt340(%d)\n", __FUNCTION__, This->m_UseArtesaExt340));

		if (QPHCI_MODE_MASTER != pMpgCodecInitData->AccessMode)
		{
			TQP_PFMAttrStruct		attr;

			switch (This->m_ChipType)
			{
				case QPCODEC_CHIP_BERINGER:
					attr.chip.type = QPPF_CHIPTYPE_BERINGER;
					break;
				case QPCODEC_CHIP_ARTESA_VERTEX_4:
					attr.chip.type = QPPF_CHIPTYPE_ARTESA_VIRTEX_4;
					break;
				case QPCODEC_CHIP_ARTESA_VERTEX_5:
					attr.chip.type = QPPF_CHIPTYPE_ARTESA_VIRTEX_5;
					break;
				case QPCODEC_CHIP_ARTESA:
				default:
					attr.chip.type = QPPF_CHIPTYPE_ARTESA;
					break;
			}

			QPPFMSetAttr(QPPF_ACODE_CHIP,
						 &attr
						 );
		}
	}
    else
    {
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
                      "CQLCodec_InitDevice() No initial data, return error...\n"
                      ));
		return (QPERR_FAIL);
    }

	// get the build in firmware
	//
	if (!This->m_bVideoFWUpdated)
	{
		if (pMpgCodecInitData &&
			pMpgCodecInitData->bUseExtFW &&
			pMpgCodecInitData->pVideoFW &&
			pMpgCodecInitData->VidFWSize
			)
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_InitDevice() config to use external Video FW\n"
				 ));

			// use external firmware
			//
			This->m_pVideoFW = (PTQP_UINT8)QPMMMalloc2Ex(pMpgCodecInitData->VidFWSize);
			if (This->m_pVideoFW)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
					 "CQLCodec_InitDevice() load external Video FW\n"
					 ));

				memcpy(This->m_pVideoFW,
					   pMpgCodecInitData->pVideoFW,
					   pMpgCodecInitData->VidFWSize
					   );
				This->m_QL201FWSize = pMpgCodecInitData->VidFWSize;
				This->m_bVideoFWUpdated = QP_TRUE;
			}
			else
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
					"CQLCodec_InitDevice() load internal Video FW\n"
					));

				// use build in firmware instead
				This->m_QL201FWSize = 0;
				QPPFMGetVideoFW(&This->m_pVideoFW,
								&This->m_QL201FWSize
								);
			}
		}
		else
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_InitDevice() config to use internal Video FW\n"
				 ));

			// build in firmware
			//
			QPPFMGetVideoFW(&This->m_pVideoFW,
							&This->m_QL201FWSize
							);
		}
	}

	if (!This->m_bAudioFWUpdated)
	{
		if (pMpgCodecInitData &&
			pMpgCodecInitData->bUseExtFW &&
			pMpgCodecInitData->pAudioFW &&
			pMpgCodecInitData->AudFWSize
			)
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_InitDevice() config to use external Audio FW\n"
				 ));

			// use external firmware
			//
			This->m_pAudioFW = (PTQP_UINT8)QPMMMalloc2Ex(pMpgCodecInitData->AudFWSize);
			if (This->m_pAudioFW)
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_InitDevice() load external Audio FW\n"
				 ));

				memcpy(This->m_pAudioFW,
					   pMpgCodecInitData->pAudioFW,
					   pMpgCodecInitData->AudFWSize
					   );
				This->m_QL201AudFWSize = pMpgCodecInitData->AudFWSize;
				This->m_bAudioFWUpdated = QP_TRUE;
			}
			else
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_InitDevice() load internal Audio FW\n"
				 ));

				// use build in firmware instead
				This->m_QL201AudFWSize = 0;
				QPPFMGetAudioFW(&This->m_pAudioFW,
								&This->m_QL201AudFWSize
								);
			}
		}
		else
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_InitDevice() config to use internal Audio FW\n"
				 ));

			// build in firmware
			//
			QPPFMGetAudioFW(&This->m_pAudioFW,
							&This->m_QL201AudFWSize
							);
		}
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_InitDevice() vid_FW(0x%x) size(%d) aud_FW(0x%x) size(%d)\n",
				 This->m_pVideoFW,
				 This->m_QL201FWSize,
				 This->m_pAudioFW,
				 This->m_QL201AudFWSize
				 ));

	// load default settings based on chip type
	//
	CQLCodec_LoadDefaultSettings(This);

	// Initialize HCI
	//
	if (pMpgCodecInitData)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
					 "\r\n******************** HCI settings ***************************\n"
					 ));
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
					 "\r\n\r\n  AccessMode(%d) Bus(%d) reg_base(0x%x) mem_base(0x%x) page(0x%x)\r\n\r\n",
					 pMpgCodecInitData->AccessMode,
					 pMpgCodecInitData->BusType,
					 pMpgCodecInitData->RegisterBase,
					 pMpgCodecInitData->MemoryBase,
					 pMpgCodecInitData->PageSize
					 ));
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
					 "************************************************************\r\n\r\n"
					 ));
		qpStatus = QPHCI_Init(&This->m_hci,
							  pMpgCodecInitData->AccessMode,
							  pMpgCodecInitData->BusType,
							  (PTQP_UINT8)pMpgCodecInitData->RegisterBase,
							  (PTQP_UINT8)pMpgCodecInitData->MemoryBase,
							  pMpgCodecInitData->PageSize,
							  This,
							  This->m_pUsbCntl,
							  This->m_pPCIeCntl
							  );
	}
	else
	{
		TQP_PVOID	regBase = (TQP_PVOID) 0x80000000;
		TQP_PVOID	memBase = (TQP_PVOID) 0;

		qpStatus = QPHCI_Init(&This->m_hci,
							  QPHCI_MODE_DIRECT,
							  QPHCI_BUS_CUSTOM_1,
							  (PTQP_UINT8)regBase,
							  (PTQP_UINT8)memBase,
							  0x100000,
							  This,
							  This->m_pUsbCntl,
							  This->m_pPCIeCntl
							  );
	}

	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CQLCodec_InitDevice() QPHCI_Init failed status(%d)\n",
					 qpStatus
					 ));
		This->m_bHCIInited = QP_FALSE;
		return (qpStatus);
	}
	else
	{
		This->m_bHCIInited = QP_TRUE;
	}

	// read chip version
	//
	qpStatus = QPHCI_RegisterRead(&This->m_hci,
								  QLCODEC_REG_CHIP_VERSION,
								  &reg
								  );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CQLCodec_InitDevice() read chip version failed status(%d)\n",
					 qpStatus
					 ));
		QPHCI_Done(&This->m_hci);
		This->m_bHCIInited = QP_FALSE;
		return (qpStatus);
	}
	else
	{
		This->m_ChipVersion.Read = reg;
		This->m_hci.m_ulChipVer = This->m_ChipVersion.Read;
	}

	// if we can read from chip version register, use it for chip type
	//
	if ((pMpgCodecInitData) && (!pMpgCodecInitData->bDontInitHW))
	{
		// initialize memory controller
		//
		CQLCodec_InitializeMemory(This);

		// init dummy ARM loop
		//
		QPHCI_InitArmLoop(&This->m_hci);

		// set GPIO defaults
		//
		CQLCodec_SetGPIODefaults(This);
	}

	// install interrupt handler
	//
	if (pMpgCodecInitData)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
					 "\r\n******************** interrupt number ***************************\n"
					 ));
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
					 "\r\n\r\n  Interrupt Number(%d)\r\n\r\n",
					 pMpgCodecInitData->InterruptNumber
					 ));
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
					 "************************************************************\r\n\r\n"
					 ));
		This->m_interruptNumber = pMpgCodecInitData->InterruptNumber;
	}

	// register ISR for the interrupt
	//
	qpStatus = CQLCodec_RegisterISR(This,
									pMpgCodecInitData->BusType,
									This->m_pPCIeCntl
									);
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CQLCodec_InitDevice() QPOSMRegisterIntHdlr failed status(%d)\n",
					 qpStatus
					 ));
		QPHCI_Done(&This->m_hci);
		This->m_bHCIInited = QP_FALSE;
		return (qpStatus);
	}

	if (pMpgCodecInitData)
	{
		// initial VI configuration
		//
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
					 "\r\n******************** VIU settings ***************************\n"
					 ));
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
					 "\r\n\r\n  ViuMode(%d) ViuFormat(%d) start_pixel(%d) start_line(%d) ClkEdge(%d) sync_code1(0x%x) sync_code2(0x%x)\r\n\r\n",
					 pMpgCodecInitData->VIUMode,
					 pMpgCodecInitData->VIUFormat,
					 pMpgCodecInitData->VIUStartPixel,
					 pMpgCodecInitData->VIUStartLine,
					 pMpgCodecInitData->ClkEdge,
					 pMpgCodecInitData->VIUSyncCode1,
					 pMpgCodecInitData->VIUSyncCode2
					 ));
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
					 "************************************************************\r\n\r\n"
					 ));

		This->m_VIUMode = pMpgCodecInitData->VIUMode;
		This->m_VIUFormat = pMpgCodecInitData->VIUFormat;
		This->m_ClkEdge = pMpgCodecInitData->ClkEdge;
		This->m_VIUStartPixel = pMpgCodecInitData->VIUStartPixel;
		This->m_VIUStartLine = pMpgCodecInitData->VIUStartLine;
		This->m_ulViuSyncCode1 = pMpgCodecInitData->VIUSyncCode1;
		This->m_ulViuSyncCode2 = pMpgCodecInitData->VIUSyncCode2;

		// initial AI configuration
		//
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
					 "\r\n******************** AI settings ***************************\n"
					 ));
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
					 "\r\n\r\n  ai_msb(%d) lr_clk(%d) bclk(%d) i2s_mode(%d) ar_rj(%d) ai_m(%d)\r\n\r\n",
					 pMpgCodecInitData->AIControls_ai_msb,
					 pMpgCodecInitData->AIControls_lrclk_i,
					 pMpgCodecInitData->AIControls_bclk_i,
					 pMpgCodecInitData->AIControls_ai_i2s,
					 pMpgCodecInitData->AIControls_ai_rj,
					 pMpgCodecInitData->AIControls_ai_m
					 ));
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
					 "************************************************************\r\n\r\n"
					 ));

		This->m_AIControls_ai_msb = pMpgCodecInitData->AIControls_ai_msb;
		This->m_AIControls_lrclk_i = pMpgCodecInitData->AIControls_lrclk_i;
		This->m_AIControls_bclk_i = pMpgCodecInitData->AIControls_bclk_i;
		This->m_AIControls_ai_i2s = pMpgCodecInitData->AIControls_ai_i2s;
		This->m_AIControls_ai_rj = pMpgCodecInitData->AIControls_ai_rj;
		This->m_AIControls_ai_m = pMpgCodecInitData->AIControls_ai_m;

		// setup initial VO configuration
		//
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
					 "\r\n******************** VOU settings ***************************\n"
					 ));
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
					 "\r\n\r\n  VOUEnable(%d) VOUMode(%d) start_pixel(%d) start_line(%d)\r\n\r\n",
					 pMpgCodecInitData->VOEnable,
					 pMpgCodecInitData->VOUMode,
					 pMpgCodecInitData->VOUStartPixel,
					 pMpgCodecInitData->VOUStartLine
					 ));
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
					 "************************************************************\r\n\r\n"
					 ));

		This->m_VOEnable = pMpgCodecInitData->VOEnable;
		This->m_VOUMode = pMpgCodecInitData->VOUMode;
		This->m_VOUStartPixel = pMpgCodecInitData->VOUStartPixel;
		This->m_VOUStartLine = pMpgCodecInitData->VOUStartLine;

		// setup initial AO configuration
		//
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
					 "\r\n******************** AO settings ***************************\n"
					 ));
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
					 "\r\n\r\n  AOEnable(%d) msb(%d) lr_clk(%d) bclk(%d) i2s_mode(%d) ar_rj(%d) ao_s(%d)\r\n\r\n",
					 pMpgCodecInitData->AOEnable,
					 pMpgCodecInitData->AOControls_ao_msb,
					 pMpgCodecInitData->AOControls_lrclk_i,
					 pMpgCodecInitData->AOControls_bclk_i,
					 pMpgCodecInitData->AOControls_ao_i2s,
					 pMpgCodecInitData->AOControls_ao_rj,
					 pMpgCodecInitData->AOControls_ao_s
					 ));
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
					 "************************************************************\r\n\r\n"
					 ));

		This->m_AOEnable = pMpgCodecInitData->AOEnable;
		This->m_AOControls_ao_msb = pMpgCodecInitData->AOControls_ao_msb;
		This->m_AOControls_lrclk_i = pMpgCodecInitData->AOControls_lrclk_i;
		This->m_AOControls_bclk_i = pMpgCodecInitData->AOControls_bclk_i;
		This->m_AOControls_ao_i2s = pMpgCodecInitData->AOControls_ao_i2s;
		This->m_AOControls_ao_rj = pMpgCodecInitData->AOControls_ao_rj;
		This->m_AOControls_ao_s = pMpgCodecInitData->AOControls_ao_s;
	}

#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
	if (!pMpgCodecInitData->bDontInitHW) {
#endif
		qpStatus = CQLCodec_AOSwitch(This,
									 This->m_AOEnable
									 );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : CQLCodec_VOSwitch(This,
																		This->m_VOEnable
																		);
#if !defined(CONFIG_HOST_PLATFORM_PL330SOC)
	}
#endif

	// save callback address and context
	//
	This->m_pDeviceCallback = pFuncCallback;
	This->m_callbackContext = context;

#ifdef _QPCODEC_CALLBACK_IN_THREAD

	// create emulated interrupt thread
	//
	if (!CThread_IsThreadInited(&This->m_Thread))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
					 "CQLCodec_InitDevice() CThread_ThreadInit()\n"
					 ));
		CThread_ThreadInit(&This->m_Thread);
	}
#endif //_QPCODEC_CALLBACK_IN_THREAD

	// CObject Init
	//
	CObject_Init(&This->m_Object);

	return (qpStatus);
}



static EQP_Errors CQLCodec_Release(IMpegCodec *This_p)
{
	CQLCodec	*This = GET_INHERITED_OBJECT(CQLCodec, m_iMpegCodec, This_p);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_Release()\n"
				 ));

	// close all the opened channels
	//
	// clean up
	//
	while (0 != CObjectMgr_GetNumberOfObjects(This->m_pChannelMgr))
	{
		CChannel	*pChannel = (CChannel *)CObjectMgr_GetObjectByIndex(This->m_pChannelMgr,
															0
															);
		if (pChannel)
		{
			CQLCodec_Close(&This->m_iMpegCodec,
						   pChannel->m_hChannel
						   );
		}
	}

	// make sure all interrupts are disabled before
	// unregister ISR
	//
	QPHCI_DisableInterrupt(&This->m_hci,
						   HCI_INTERRUPT_ARM | HCI_INTERRUPT_P2M | HCI_INTERRUPT_M2P
						   );

	// un-install interrupt handler
	//
	if (This->m_bInterruptAttached)
	{
		CQLCodec_UnregisterISR(This,
							   This->m_hci.m_bus_type,
							   This->m_pPCIeCntl
							   );
		This->m_bInterruptAttached = QP_FALSE;
	}

#ifdef _QPCODEC_CALLBACK_IN_THREAD
	if (CThread_IsThreadInited(&This->m_Thread))
	{
		CThread_ThreadDone(&This->m_Thread);
	}
#endif //_QPCODEC_CALLBACK_IN_THREAD

	// HCI done
	//
	if (This->m_bHCIInited)
	{
		QPHCI_Done(&This->m_hci);
		This->m_bHCIInited = QP_FALSE;
	}

	// power state
	//
	This->m_PowerState = QLCODEC_PWR_STATE_D3;

	// clear callback function
	//
	This->m_pDeviceCallback = QP_NULL;
	This->m_callbackContext = QP_NULL;

	// CObject Done
	//
	CObject_Done(&This->m_Object);

	return (QPERR_SUCCESS);
}



static EQP_Errors CQLCodec_Reset(IMpegCodec *This_p)
{
	EQP_Errors	qpStatus;
	CQLCodec	*This =	GET_INHERITED_OBJECT(CQLCodec, m_iMpegCodec, This_p);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_Reset()\n"
				 ));

	// clear mail box tokens
	//
	qpStatus = QPHCI_RegisterWrite(&This->m_hci,
								   REG_TO_ARM_MESSAGE_STATUS,
								   0
								   );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  REG_TO_HOST_MESSAGE_STATUS,
																	  0
																	  );

	// reset the chip to bring back other default settings
	//
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_ResetArm(&This->m_hci,
			     												 QP_FALSE
			     												 );

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_ReInit(&This->m_hci);
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : CQLCodec_InitializeMemory(This);
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : CQLCodec_AOSwitch(This,
																	This->m_AOEnable
																	);
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : CQLCodec_VOSwitch(This,
																	This->m_VOEnable
																	);
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : CQLCodec_SetGPIODefaults(This);

	DelayMilliSeconds(5);

	// start arm running
	//
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_ResetArm(&This->m_hci,
																 QP_TRUE
																 );

	return (qpStatus);
}



static EQP_Errors CQLCodec_Set(IMpegCodec *This_p,
							   TQP_REFGUID guid,
							   TQP_OBJECT_HANDLE hIndex,
							   TQP_UINT32 dwCode,
							   TQP_PVOID pInput,
							   TQP_PVOID pOutput,
							   TQP_UINT32 dwSize
							   )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	CQLCodec	*This =	GET_INHERITED_OBJECT(CQLCodec, m_iMpegCodec, This_p);
	PTASK_DATA	pTaskData;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_Set() hIndex(%d)\n",
				 hIndex
				 ));

	if ((hIndex >= MAX_TASKS) ||
		!This->m_pTask->m_TaskData[hIndex].valid
		)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "CQLCodec_Set() invalid task\n"
					 ));
		return (QPERR_PARMS);
	}

	pTaskData = &This->m_pTask->m_TaskData[hIndex];

	if (IsEqualTQPGUID((TQP_REFGUID)&PROPSETID_QPCODEC_CONTROL,
					   guid
					   ))
	{
		switch (dwCode)
		{
			// Encoder
			//
			case QPCODEC_PROP_SYSTEM_CONTROL:
			{
				PQPCODEC_SYSTEM_CONTROL	pSystemControl = (PQPCODEC_SYSTEM_CONTROL)pInput;

				if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Set() QPCODEC_PROP_SYSTEM_CONTROL stream type(%d) stream data(%d) profile(%d) level(%d) ff_mod(%d)\r\nspsr_freq(%d) v_mode(%d) cabac_init(%d) ver(%d) xfer_mode(%d)\n",
								 pSystemControl->Artesa.stream_type,
								 pSystemControl->Artesa.stream_data,
								 pSystemControl->Artesa.profile,
								 pSystemControl->Artesa.level,
								 pSystemControl->Artesa.ff_mode,
								 pSystemControl->Artesa.spsr_freq,
								 pSystemControl->Artesa.v_mode,
								 pSystemControl->Artesa.cabac_init_idc,
								 pSystemControl->Artesa.version,
								 pSystemControl->Artesa.xfer_mode
								 ));

					pTaskData->m_systemControl.Read = 0;

					pTaskData->m_systemControl.RegArtesa.stream_type = pSystemControl->Artesa.stream_type;
					pTaskData->m_systemControl.RegArtesa.stream_data = pSystemControl->Artesa.stream_data;
					pTaskData->m_systemControl.RegArtesa.profile = pSystemControl->Artesa.profile;
					pTaskData->m_systemControl.RegArtesa.level = pSystemControl->Artesa.level;
					pTaskData->m_systemControl.RegArtesa.ff_mod = pSystemControl->Artesa.ff_mode;
					pTaskData->m_systemControl.RegArtesa.spsr_freq = pSystemControl->Artesa.spsr_freq;
					pTaskData->m_systemControl.RegArtesa.v_mode = pSystemControl->Artesa.v_mode;
					pTaskData->m_systemControl.RegArtesa.cabac_init_idc = pSystemControl->Artesa.cabac_init_idc;
					pTaskData->m_systemControl.RegArtesa.version = pSystemControl->Artesa.version;
					pTaskData->m_systemControl.RegArtesa.xfer_mode = pSystemControl->Artesa.xfer_mode;
				}
				else
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Set() QPCODEC_PROP_SYSTEM_CONTROL stream type(%d) profile(%d) level(%d)cap_mod(%d) ff_mod(%d)\r\nspsr_freq(%d) v_mode(%d) cabac_init(%d) ver(%d)\n",
								 pSystemControl->Beringer.stream_type,
								 pSystemControl->Beringer.profile,
								 pSystemControl->Beringer.level,
								 pSystemControl->Beringer.cap_mode,
								 pSystemControl->Beringer.ff_mode,
								 pSystemControl->Beringer.spsr_freq,
								 pSystemControl->Beringer.v_mode,
								 pSystemControl->Beringer.cabac_init_idc,
								 pSystemControl->Beringer.version
								 ));

					pTaskData->m_systemControl.Read = 0;

					pTaskData->m_systemControl.Reg.stream_type = pSystemControl->Beringer.stream_type;
					pTaskData->m_systemControl.Reg.profile = pSystemControl->Beringer.profile;
					pTaskData->m_systemControl.Reg.level = pSystemControl->Beringer.level;
					pTaskData->m_systemControl.Reg.cap_mod = pSystemControl->Beringer.cap_mode;
					pTaskData->m_systemControl.Reg.ff_mod = pSystemControl->Beringer.ff_mode;
					pTaskData->m_systemControl.Reg.spsr_freq = pSystemControl->Beringer.spsr_freq;
					pTaskData->m_systemControl.Reg.v_mode = pSystemControl->Beringer.v_mode;
					pTaskData->m_systemControl.Reg.cabac_init_idc = pSystemControl->Beringer.cabac_init_idc;
					pTaskData->m_systemControl.Reg.version = pSystemControl->Beringer.version;
				}
				break;
			}

			case QPCODEC_PROP_PICTURE_RESOLUTION:
			{
				PQPCODEC_PICTURE_RESOLUTION	pPictureResoultion = (PQPCODEC_PICTURE_RESOLUTION)pInput;
				//CQLCodecLib					*pQLCodecLib = GET_INHERITED_OBJECT(CQLCodecLib, m_Object, This->m_Object.m_pParent);

				if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Set() QPCODEC_PROP_PICTURE_RESOLUTION in_width(%d) in_height(%d)",
								 pPictureResoultion->Artesa.in_pic_width,
								 pPictureResoultion->Artesa.in_pic_height
								 ));
					pTaskData->m_pictureResolution.Read = 0;

					pTaskData->m_pictureResolution.RegArtesa.in_pic_width = pPictureResoultion->Artesa.in_pic_width;
					pTaskData->m_pictureResolution.RegArtesa.in_pic_height = pPictureResoultion->Artesa.in_pic_height;
				}
				else
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Set() QPCODEC_PROP_PICTURE_RESOLUTION width(%d) v_scale(%d) height(%d) h_scale(%d)",
								 pPictureResoultion->Beringer.pic_width,
								 pPictureResoultion->Beringer.v_scale,
								 pPictureResoultion->Beringer.pic_height,
								 pPictureResoultion->Beringer.h_scale
								 ));
					pTaskData->m_pictureResolution.Read = 0;

					pTaskData->m_pictureResolution.Reg.pic_width = pPictureResoultion->Beringer.pic_width;
					pTaskData->m_pictureResolution.Reg.v_scale = pPictureResoultion->Beringer.v_scale;
					pTaskData->m_pictureResolution.Reg.pic_height = pPictureResoultion->Beringer.pic_height;
					pTaskData->m_pictureResolution.Reg.h_scale = pPictureResoultion->Beringer.h_scale;
				}

#if 0
				if (pQLCodecLib->m_pVidDecoder)
				{
					qpStatus = QPVideoDecoder_SetVideoRes(&pQLCodecLib->m_pVidDecoder->m_iVideoDecoder,
														  pTaskData->m_pictureResolution.Reg.pic_width,
														  pTaskData->m_pictureResolution.Reg.pic_height
														  );
				}
#endif
				break;
			}

			case QPCODEC_PROP_INPUT_CONTROL:
			{
				PQPCODEC_INPUT_CONTROL	pInputControl = (PQPCODEC_INPUT_CONTROL)pInput;

				if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Set() QPCODEC_PROP_INPUT_CONTROL scan(%d) src_mode(%d) sync_mode(%d:%d) data_type(%d) \r\n in_frame_rate(%d) out_frame_rate(%d) force_60p_in(%d) pd_en(%d) ck_edge(%d) bytes_swap(%d)\n",
								 pInputControl->Artesa.scan_format,
								 pInputControl->Artesa.src_mode,
								 pInputControl->Artesa.sync_mode,
								 g_syncModeTable[pInputControl->Artesa.sync_mode],
								 pInputControl->Artesa.data_type,
								 pInputControl->Artesa.in_frame_rate,
								 pInputControl->Artesa.out_frame_rate,
								 pInputControl->Artesa.force_60p_in,
								 pInputControl->Artesa.pd_en,
								 pInputControl->Artesa.ck_edge,
								 pInputControl->Artesa.bytes_swap
								 ));

					pTaskData->m_inputControl.Read = 0;

					pTaskData->m_inputControl.RegArtesa.scan_format = pInputControl->Artesa.scan_format;
					pTaskData->m_inputControl.RegArtesa.src_mode = pInputControl->Artesa.src_mode;
					pTaskData->m_syncMode = pInputControl->Artesa.sync_mode;
					pTaskData->m_inputControl.RegArtesa.sync_mode = g_syncModeTable[pTaskData->m_syncMode];
					pTaskData->m_inputControl.RegArtesa.data_type = pInputControl->Artesa.data_type;
					pTaskData->m_inputControl.RegArtesa.in_frame_rate = pInputControl->Artesa.in_frame_rate;
					pTaskData->m_inputControl.RegArtesa.out_frame_rate = pInputControl->Artesa.out_frame_rate;
					pTaskData->m_inputControl.RegArtesa.force_60p_in = pInputControl->Artesa.force_60p_in;
					pTaskData->m_inputControl.RegArtesa.pd_en = pInputControl->Artesa.pd_en;
					pTaskData->m_inputControl.RegArtesa.ck_edge = pInputControl->Artesa.ck_edge;
					pTaskData->m_inputControl.RegArtesa.bytes_swap = pInputControl->Artesa.bytes_swap;
				}
				else
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Set() QPCODEC_PROP_INPUT_CONTROL mode(%d) src(%d) dst(%d) bl_e(%d) lines(%d)\r\n frame_rate(%d) out_fr(%d) fmt(%d) pd_en(%d) ck_edge(%d)\n",
								 pInputControl->Beringer.mode,
								 pInputControl->Beringer.src,
								 pInputControl->Beringer.dst,
								 pInputControl->Beringer.bl_e,
								 pInputControl->Beringer.black_line_num,
								 pInputControl->Beringer.in_frame_rate,
								 pInputControl->Beringer.out_frame_rate,
								 pInputControl->Beringer.fmt,
								 pInputControl->Beringer.pd_en,
								 pInputControl->Beringer.ck_edge
								 ));

					pTaskData->m_inputControl.Read = 0;

					pTaskData->m_inputControl.Reg.mode = pInputControl->Beringer.mode;
					pTaskData->m_inputControl.Reg.src = pInputControl->Beringer.src;
					pTaskData->m_inputControl.Reg.dst = pInputControl->Beringer.dst;
					pTaskData->m_inputControl.Reg.bl_e = pInputControl->Beringer.bl_e;
					pTaskData->m_inputControl.Reg.black_line_num = pInputControl->Beringer.black_line_num;
					pTaskData->m_inputControl.Reg.in_frame_rate = pInputControl->Beringer.in_frame_rate;
					pTaskData->m_inputControl.Reg.out_frame_rate = pInputControl->Beringer.out_frame_rate;
					pTaskData->m_inputControl.Reg.fmt = pInputControl->Beringer.fmt;
					pTaskData->m_inputControl.Reg.pd_en = pInputControl->Beringer.pd_en;
					pTaskData->m_inputControl.Reg.ck_edge = pInputControl->Beringer.ck_edge;
				}

				if (TASK_STATE_IDLE != pTaskData->m_State)
				{
					CQLCodec_UpdateEncoderConfig(This,
												 hIndex,
												 QP_TRUE
												 );
				}
				break;
			}

			case QPCODEC_PROP_RATE_CONTROL:
			{
				PQPCODEC_RATE_CONTROL	pRateControl = (PQPCODEC_RATE_CONTROL)pInput;

				if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Set() QPCODEC_PROP_RATE_CONTROL bitrate(%d) qp_update(%d) mode(%d) fixed(%d) vbr(%d) intra_refresh_mode(%d)\n",
								 pRateControl->Artesa.avg_bit_rate,
								 pRateControl->Artesa.qp_update_freq,
								 pRateControl->Artesa.mode,
								 pRateControl->Artesa.fixed,
								 pRateControl->Artesa.vbr,
								 pRateControl->Artesa.intra_refresh_mode
								 ));
					pTaskData->m_rateControl.Read = 0;

					pTaskData->m_rateControl.RegArtesa.avg_bit_rate = pRateControl->Artesa.avg_bit_rate;
					pTaskData->m_rateControl.RegArtesa.qp_update_freq = pRateControl->Artesa.qp_update_freq;
					pTaskData->m_rateControl.RegArtesa.mode = pRateControl->Artesa.mode;
					pTaskData->m_rateControl.RegArtesa.fixed = pRateControl->Artesa.fixed;
					pTaskData->m_rateControl.RegArtesa.vbr = pRateControl->Artesa.vbr;
					pTaskData->m_rateControl.RegArtesa.intra_refresh_mode = pRateControl->Artesa.intra_refresh_mode;
				}
				else
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Set() QPCODEC_PROP_RATE_CONTROL bitrate(%d) qp_update(%d) vbr(%d)\n",
								 pRateControl->Beringer.avg_bit_rate,
								 pRateControl->Beringer.qp_update_freq,
								 pRateControl->Beringer.vbr
								 ));
					pTaskData->m_rateControl.Read = 0;

					pTaskData->m_rateControl.Reg.avg_bit_rate = pRateControl->Beringer.avg_bit_rate;
					pTaskData->m_rateControl.Reg.qp_update_freq = pRateControl->Beringer.qp_update_freq;
					pTaskData->m_rateControl.Reg.vbr = pRateControl->Beringer.vbr;
				}

				if (TASK_STATE_IDLE != pTaskData->m_State)
				{
					CQLCodec_UpdateEncoderConfig(This,
												 hIndex,
												 QP_TRUE
												 );
				}
				break;
			}

			case QPCODEC_PROP_RATE_CONTROL_EX:
			{
				PRATE_CONTROL_EX	pRateControlEx = (PRATE_CONTROL_EX)pInput;

				if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Set() QPCODEC_PROP_RATE_CONTROL_EX base_qp_length(%d) local_activity_on(%d)\n",
								 pRateControlEx->Artesa.base_qp_length,
								 pRateControlEx->Artesa.local_activity_on
								 ));
					pTaskData->m_rateControlEx.Read = 0;

					pTaskData->m_rateControlEx.Artesa.base_qp_length = pRateControlEx->Artesa.base_qp_length;
					pTaskData->m_rateControlEx.Artesa.local_activity_on = pRateControlEx->Artesa.local_activity_on;
					pTaskData->m_rateControlEx.Artesa.motion_detect_interval = pRateControlEx->Artesa.motion_detect_interval;
				}
				break;
			}

			case QPCODEC_PROP_VBR_BIT_RATE:
			{
				PQPCODEC_VBR_BIT_RATE	pBitRate = (PQPCODEC_VBR_BIT_RATE)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_VBR_BIT_RATE min(%d) max(%d)\n",
							 pBitRate->min_bit_rate,
							 pBitRate->max_bit_rate
							 ));
				pTaskData->m_bitRate.Read = 0;

				pTaskData->m_bitRate.Reg.min_bit_rate = pBitRate->min_bit_rate;
				pTaskData->m_bitRate.Reg.max_bit_rate = pBitRate->max_bit_rate;

				if (TASK_STATE_IDLE != pTaskData->m_State)
				{
					CQLCodec_UpdateEncoderConfig(This,
												 hIndex,
												 QP_TRUE
												 );
				}
				break;
			}

			case QPCODEC_PROP_FILTER_CONTROL:
			{
				PQPCODEC_FILTER_CONTROL	pFilterControl = (PQPCODEC_FILTER_CONTROL)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_FILTER_CONTROL start_pixel(%d) en(%d) strength(%d) start_line(%d) dis_skp(%d)\n",
							 pFilterControl->start_pixel,
							 pFilterControl->fl_en,
							 pFilterControl->filter_strength,
							 pFilterControl->start_line,
							 pFilterControl->dis_skp
							 ));

				pTaskData->m_filterControl.Read = 0;

				pTaskData->m_filterControl.Reg.start_pixel = pFilterControl->start_pixel;
				pTaskData->m_filterControl.Reg.fl_en = pFilterControl->fl_en;
				pTaskData->m_filterControl.Reg.filter_strength = pFilterControl->filter_strength;
				pTaskData->m_filterControl.Reg.start_line = pFilterControl->start_line;
				pTaskData->m_filterControl.Reg.dis_skp = pFilterControl->dis_skp;

				if (TASK_STATE_IDLE != pTaskData->m_State)
				{
					CQLCodec_UpdateEncoderConfig(This,
												 hIndex,
												 QP_TRUE
												 );
				}
				break;
			}

			case QPCODEC_PROP_GOP_LOOP_FILTER:
			{
				PQPCODEC_GOP_LOOP_FILTER	pGopLfltr = (PQPCODEC_GOP_LOOP_FILTER)pInput;

				if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Set() QPCODEC_PROP_GOP_LOOP_FILTER i_period(%d) lp_fltr(%d) dis_lf_idc(%d) \r\nalpha_offset(%d) beta_offset(%d) B_num(%d) ref_n(%d) t8x8(%d) ref_p(%d) IDR(%d)\n",
								 pGopLfltr->Artesa.intra_period,
								 pGopLfltr->Artesa.lp_filter,
								 pGopLfltr->Artesa.dis_lf_idc,
								 pGopLfltr->Artesa.alpha_offset,
								 pGopLfltr->Artesa.beta_offset,
								 pGopLfltr->Artesa.B_num,
								 pGopLfltr->Artesa.ref_n,
								 pGopLfltr->Artesa.t8x8,
								 pGopLfltr->Artesa.ref_p,
								 pGopLfltr->Artesa.IDR
								 ));

					pTaskData->m_gopLoopFilter.Read = 0;

					pTaskData->m_gopLoopFilter.RegArtesa.intra_period = pGopLfltr->Artesa.intra_period;
					pTaskData->m_gopLoopFilter.RegArtesa.lp_filter = pGopLfltr->Artesa.lp_filter;
					pTaskData->m_gopLoopFilter.RegArtesa.dis_lf_idc = pGopLfltr->Artesa.dis_lf_idc;
					pTaskData->m_gopLoopFilter.RegArtesa.alpha_offset = pGopLfltr->Artesa.alpha_offset;
					pTaskData->m_gopLoopFilter.RegArtesa.beta_offset = pGopLfltr->Artesa.beta_offset;
					pTaskData->m_gopLoopFilter.RegArtesa.B_num = pGopLfltr->Artesa.B_num;
					pTaskData->m_gopLoopFilter.RegArtesa.ref_n = pGopLfltr->Artesa.ref_n;
					pTaskData->m_gopLoopFilter.RegArtesa.t8x8 = pGopLfltr->Artesa.t8x8;
					pTaskData->m_gopLoopFilter.RegArtesa.ref_p = pGopLfltr->Artesa.ref_p;
					pTaskData->m_gopLoopFilter.RegArtesa.IDR = pGopLfltr->Artesa.IDR;
				}
				else
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Set() QPCODEC_PROP_GOP_LOOP_FILTER i_period(%d) lp_fltr(%d) dis_lf_idc(%d) \r\nalpha_offset(%d) beta_offset(%d) B_num(%d) IDR(%d)\n",
								 pGopLfltr->Beringer.intra_period,
								 pGopLfltr->Beringer.lp_filter,
								 pGopLfltr->Beringer.dis_lf_idc,
								 pGopLfltr->Beringer.alpha_offset,
								 pGopLfltr->Beringer.beta_offset,
								 pGopLfltr->Beringer.B_num,
								 pGopLfltr->Beringer.IDR
								 ));

					pTaskData->m_gopLoopFilter.Read = 0;

					pTaskData->m_gopLoopFilter.Reg.intra_period = pGopLfltr->Beringer.intra_period;
					pTaskData->m_gopLoopFilter.Reg.lp_filter = pGopLfltr->Beringer.lp_filter;
					pTaskData->m_gopLoopFilter.Reg.dis_lf_idc = pGopLfltr->Beringer.dis_lf_idc;
					pTaskData->m_gopLoopFilter.Reg.alpha_offset = pGopLfltr->Beringer.alpha_offset;
					pTaskData->m_gopLoopFilter.Reg.beta_offset = pGopLfltr->Beringer.beta_offset;
					pTaskData->m_gopLoopFilter.Reg.B_num = pGopLfltr->Beringer.B_num;
					pTaskData->m_gopLoopFilter.Reg.IDR = pGopLfltr->Beringer.IDR;
				}

				if (TASK_STATE_IDLE != pTaskData->m_State)
				{
					CQLCodec_UpdateEncoderConfig(This,
												 hIndex,
												 QP_TRUE
												 );
				}
				break;
			}

			case QPCODEC_PROP_ET_CONTROL:
			{
				PQPCODEC_ET_CONTROL	pEtControl = (PQPCODEC_ET_CONTROL)pInput;

				if (QPCODEC_CHIP_BERINGER == This->m_ChipType)
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Set() QPCODEC_PROP_ET_CONTROL threshold_offset(%d) max(%d) et(%d) et_sad(%d) ch_data_type(%d) ref_num(%d)\n",
								 pEtControl->et_threshold_offset,
								 pEtControl->et_threshold_max,
								 pEtControl->et,
								 pEtControl->et_sad_threshold,
								 pEtControl->channel_data_type,
								 pEtControl->ref_num
								 ));

					pTaskData->m_etControl.Read = 0;

					pTaskData->m_etControl.Reg.et_threshold_offset = pEtControl->et_threshold_offset;
					pTaskData->m_etControl.Reg.et_threshold_max = pEtControl->et_threshold_max;
					pTaskData->m_etControl.Reg.et = pEtControl->et;
					pTaskData->m_etControl.Reg.et_sad_threshold = pEtControl->et_sad_threshold;
					pTaskData->m_etControl.Reg.data_channel_type = pEtControl->channel_data_type;
					pTaskData->m_etControl.Reg.ref_num = pEtControl->ref_num;
				}
				else
				{
					qpStatus = QPERR_NOTSUPP;
				}
				break;
			}

			case QPCODEC_PROP_OUT_PICTURE_RESOLUTION:
			{
				PQPCODEC_OUT_PICTURE_RESOLUTION	pPictureResoultion = (PQPCODEC_OUT_PICTURE_RESOLUTION)pInput;

				if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Set() QPCODEC_PROP_OUT_PICTURE_RESOLUTION in_width(%d) in_height(%d)",
								 pPictureResoultion->out_pic_width,
								 pPictureResoultion->out_pic_height
								 ));
					pTaskData->m_outPictureResolution.Read = 0;

					pTaskData->m_outPictureResolution.RegArtesa.out_pic_width = pPictureResoultion->out_pic_width;
					pTaskData->m_outPictureResolution.RegArtesa.out_pic_height = pPictureResoultion->out_pic_height;
				}
				else
				{
					qpStatus = QPERR_NOTSUPP;
				}
				break;
			}

			case QPCODEC_PROP_BLOCK_SIZE:
			{
				PQPCODEC_BLOCK_SIZE	pBlockSize = (PQPCODEC_BLOCK_SIZE)pInput;

				if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
				{
					if (This->m_UseArtesaExt340) {
						QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
									 "CQLCodec_Set() QPCODEC_PROP_BLOCK_SIZE block_size(%d)kb slice_group(%d) multi_slice_mode(%d) low_latency_mode(%d)",
									 pBlockSize->block_size,
									 pBlockSize->slice_group,
									 pBlockSize->multi_slice_mode,
									 pBlockSize->low_latency_mode
									 ));
						pTaskData->m_blkXferSize.Read = 0;

						pTaskData->m_blkXferSize.RegArtesaExt340.block_size = pBlockSize->block_size;
						pTaskData->m_blkXferSize.RegArtesaExt340.slice_limit = pBlockSize->slice_group;
						pTaskData->m_blkXferSize.RegArtesaExt340.multi_slice_mode = pBlockSize->multi_slice_mode;
						pTaskData->m_blkXferSize.RegArtesaExt340.low_latency_mode = pBlockSize->low_latency_mode;
					} else {
						QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
									 "CQLCodec_Set() QPCODEC_PROP_BLOCK_SIZE block_size(%d)kb slice_group(%d)",
									 pBlockSize->block_size,
									 pBlockSize->slice_group
									 ));
						pTaskData->m_blkXferSize.Read = 0;

						pTaskData->m_blkXferSize.RegArtesa.block_size = pBlockSize->block_size;
						pTaskData->m_blkXferSize.RegArtesa.slice_group = pBlockSize->slice_group;
					}
				}
				else
				{
					qpStatus = QPERR_NOTSUPP;
				}
				break;
			}

			case QPCODEC_PROP_AUDIO_CONTROL_PARAM:
			{
				PQPCODEC_AUDIO_CONTROL_PARAM	pAudCntlParam = (PQPCODEC_AUDIO_CONTROL_PARAM)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_AUDIO_CONTROL_PARAM bit_rate(%d) ch_sel(%d) sample_rate(%d) aou_pass_en(%d) ai_msb(%d) \r\nlrclk_i(%d) bclk_i(%d) ai_i2s(%d) ai_rj(%d) ai_m(%d) ch_num(%d) pts(%d) disAi(%d) raw(%d) audio_type(%d)\n",
							 pAudCntlParam->bit_rate,
							 pAudCntlParam->ch_sel,
							 pAudCntlParam->sample_rate,
							 pAudCntlParam->aou_pass_en,
							 pAudCntlParam->ai_msb,
							 pAudCntlParam->lrclk_i,
							 pAudCntlParam->bclk_i,
							 pAudCntlParam->ai_i2s,
							 pAudCntlParam->ai_rj,
							 pAudCntlParam->ai_m,
							 pAudCntlParam->ch_num,
							 pAudCntlParam->pts,
							 pAudCntlParam->disAi,
							 pAudCntlParam->raw,
							 pAudCntlParam->audio_type
							 ));

				pTaskData->m_audioControlParam.Read = 0;

				pTaskData->m_audioControlParam.Reg.bit_rate = pAudCntlParam->bit_rate;
				pTaskData->m_audioControlParam.Reg.ch_sel = pAudCntlParam->ch_sel;
				pTaskData->m_audioControlParam.Reg.sample_rate = pAudCntlParam->sample_rate;
				pTaskData->m_audioControlParam.Reg.aou_pass_en = pAudCntlParam->aou_pass_en;
				pTaskData->m_audioControlParam.Reg.ai_msb = pAudCntlParam->ai_msb;
				pTaskData->m_audioControlParam.Reg.lrclk_i = pAudCntlParam->lrclk_i;
				pTaskData->m_audioControlParam.Reg.bclk_i = pAudCntlParam->bclk_i;
				pTaskData->m_audioControlParam.Reg.ai_i2s = pAudCntlParam->ai_i2s;
				pTaskData->m_audioControlParam.Reg.ai_rj = pAudCntlParam->ai_rj;
#ifdef CONFIG_PL_AI_FIX_IN_MASTER
				pTaskData->m_audioControlParam.Reg.ai_m = AI_M_MASTER;
#else
				pTaskData->m_audioControlParam.Reg.ai_m = pAudCntlParam->ai_m;
#endif
				pTaskData->m_audioControlParam.Reg.ch_num = pAudCntlParam->ch_num;
				pTaskData->m_audioControlParam.Reg.pts = pAudCntlParam->pts;
				pTaskData->m_audioControlParam.Reg.disAi = pAudCntlParam->disAi;
				pTaskData->m_audioControlParam.Reg.raw = pAudCntlParam->raw;
				pTaskData->m_audioControlParam.Reg.audio_type = pAudCntlParam->audio_type;

				if (TASK_STATE_IDLE != pTaskData->m_State)
				{
					CQLCodec_UpdateEncoderConfig(This,
												 hIndex,
												 QP_TRUE
												 );
				}
				break;
			}

			case QPCODEC_PROP_AUDIO_CONTROL_EX:
			{
				PQPCODEC_AUDIO_CONTROL_EX	pAudCntlEx = (PQPCODEC_AUDIO_CONTROL_EX)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_AUDIO_CONTROL_EX aac_ver(%d) h_type(%d) TNS(%d) \r\nIS(%d) PNS(%d) MS(%d) cutoff(%d)\r\n mode(%d) block_size(%d) sample_cnt(%d) frame_len(%d) downsample(%d) lr_swap(%d)\r\n",
							 pAudCntlEx->aac_ver,
							 pAudCntlEx->h_type,
							 pAudCntlEx->TNS,
							 pAudCntlEx->IS,
							 pAudCntlEx->PNS,
							 pAudCntlEx->MS,
							 pAudCntlEx->cutoff_freq,
							 pAudCntlEx->mode,
							 pAudCntlEx->block_size,
							 pAudCntlEx->sample_cnt,
							 pAudCntlEx->frame_len,
							 pAudCntlEx->downsample,
							 pAudCntlEx->lr_swap
							 ));

				pTaskData->m_audioControlExAAC.Read = 0;
				pTaskData->m_audioControlExAAC.Reg.aac_ver = pAudCntlEx->aac_ver;
				pTaskData->m_audioControlExAAC.Reg.h_type = pAudCntlEx->h_type;
				pTaskData->m_audioControlExAAC.Reg.TNS = pAudCntlEx->TNS;
				pTaskData->m_audioControlExAAC.Reg.IS = pAudCntlEx->IS;
				pTaskData->m_audioControlExAAC.Reg.PNS = pAudCntlEx->PNS;
				pTaskData->m_audioControlExAAC.Reg.MS = pAudCntlEx->MS;
				pTaskData->m_audioControlExAAC.Reg.cutoff_freq = pAudCntlEx->cutoff_freq;
				pTaskData->m_audioControlExAAC.Reg.downsample = pAudCntlEx->downsample;
				pTaskData->m_audioControlExAAC.Reg.lr_swap = pAudCntlEx->lr_swap;

				pTaskData->m_audioControlExG711.Read = 0;
				pTaskData->m_audioControlExG711.Reg.mode = pAudCntlEx->mode;
				pTaskData->m_audioControlExG711.Reg.block_size = pAudCntlEx->block_size;
				pTaskData->m_audioControlExG711.Reg.downsample = pAudCntlEx->downsample;
				pTaskData->m_audioControlExG711.Reg.lr_swap = pAudCntlEx->lr_swap;
#if defined(CONFIG_HOST_PLATFORM_PL330SOC)
				// mastermode can not use 10ms
				if (pTaskData->m_audioControlExG711.Reg.block_size < BLOCK_SIZE_160)
					pTaskData->m_audioControlExG711.Reg.block_size = BLOCK_SIZE_160;
#endif


				pTaskData->m_audioControlExLPCM.Read = 0;
				pTaskData->m_audioControlExLPCM.Reg.sample_cnt = pAudCntlEx->sample_cnt;
				pTaskData->m_audioControlExLPCM.Reg.downsample = pAudCntlEx->downsample;
				pTaskData->m_audioControlExLPCM.Reg.lr_swap = pAudCntlEx->lr_swap;

				pTaskData->m_audioControlExSILK.Read = 0;
				pTaskData->m_audioControlExSILK.Reg.frame_len = pAudCntlEx->frame_len;
				pTaskData->m_audioControlExSILK.Reg.downsample = pAudCntlEx->downsample;
				pTaskData->m_audioControlExSILK.Reg.lr_swap = pAudCntlEx->lr_swap;

				break;
			}

			case QPCODEC_PROP_REL_CUR_VID_BUF:
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_REL_CUR_VID_BUF\n"));
				qpStatus = QPFWENCAPI_ReleaseCurBuf(This,
													(TQP_UINT32) hIndex
													);
				break;
			}

			case QPCODEC_PROP_INSERT_I_FRAME:
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_INSERT_I_FRAME\n"));
				qpStatus = QPFWENCAPI_InsertIFrame(This,
												   (TQP_UINT32) hIndex
												   );
				break;
			}

			case QPCODEC_PROP_INSERT_USER_DATA:
			{
				PQPCODEC_INSERT_USER_DATA	pInsertUserData = (PQPCODEC_INSERT_USER_DATA)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_INSERT_USER_DATA numUserData(%d)\n",
							 pInsertUserData->numUserData
							 ));
				qpStatus = QPFWENCAPI_InsertUserData(This,
													 (TQP_UINT32) hIndex,
													 pInsertUserData->numUserData,
													 &pInsertUserData->UserData[0]
													 );
				break;
			}

			case QPCODEC_PROP_MP4_VID_NUM_BLOCK:
			{
				PQPCODEC_MP4_VID_NUM_BLOCK	pMP4VidBlockNumber = (PQPCODEC_MP4_VID_NUM_BLOCK)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_MP4_VID_NUM_BLOCK num(%d)\n",
							 pMP4VidBlockNumber->num
							 ));
				qpStatus = QPFWENCAPI_SetMP4VideoBlockNumber(This,
															 (TQP_UINT32) hIndex,
															 pMP4VidBlockNumber->num
															 );
				if (QP_SUCCESS(qpStatus))
				{
					pTaskData->m_MP4VideoBlockNumber = pMP4VidBlockNumber->num;
				}
				break;
			}

			case QPCODEC_PROP_AUDIO_ENHANCEMENT:
			{
				PQPCODEC_AUDIO_ENHANCEMENT	pAudEnhancement = (PQPCODEC_AUDIO_ENHANCEMENT)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_AUDIO_ENHANCEMENT gain1(%d) gain2(%d) add(%d) sub(%d) att1(%d) att2(%d) lgain(%d) rgain(%d)\n",
							 pAudEnhancement->gain1,
							 pAudEnhancement->gain2,
							 pAudEnhancement->add,
							 pAudEnhancement->sub,
							 pAudEnhancement->att1,
							 pAudEnhancement->att2,
							 pAudEnhancement->lgain,
							 pAudEnhancement->rgain
							 ));

				qpStatus = QPFWENCAPI_SetAudioEnhancement(This,
														  (TQP_UINT32) hIndex,
														  pAudEnhancement->gain1,
														  pAudEnhancement->gain2,
														  pAudEnhancement->add,
														  pAudEnhancement->sub,
														  pAudEnhancement->att1,
														  pAudEnhancement->att2,
														  pAudEnhancement->lgain,
														  pAudEnhancement->rgain
														  );
				if (QP_SUCCESS(qpStatus))
				{
					pTaskData->m_audioEnhancement.gain1 = pAudEnhancement->gain1;
					pTaskData->m_audioEnhancement.gain2 = pAudEnhancement->gain2;
					pTaskData->m_audioEnhancement.add = pAudEnhancement->add;
					pTaskData->m_audioEnhancement.sub = pAudEnhancement->sub;
					pTaskData->m_audioEnhancement.att1 = pAudEnhancement->att1;
					pTaskData->m_audioEnhancement.att2 = pAudEnhancement->att2;
					pTaskData->m_audioEnhancement.lgain = pAudEnhancement->lgain;
					pTaskData->m_audioEnhancement.rgain = pAudEnhancement->rgain;
				}
				break;
			}

			case QPCODEC_PROP_ENC_STOP_MODE:
			{
				PQPCODEC_ENC_STOP_MODE	pStopMode = (PQPCODEC_ENC_STOP_MODE)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_ENC_STOP_MODE mode(%d)\n",
							 pStopMode->stop_mode
							 ));
				pTaskData->m_EncStopMode = pStopMode->stop_mode;
				break;
			}

			case QPCODEC_PROP_INDEX_CAP_FREQ:
			{
				PQPCODEC_INDEX_CAP_FREQ	pIndexFreq = (PQPCODEC_INDEX_CAP_FREQ)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_INDEX_CAP_FREQ freq(%d)\n",
							 pIndexFreq->freq
							 ));
				if ((pIndexFreq->freq >= 8) &&
					(pIndexFreq->freq <= 128)
					)
				{
					pTaskData->m_EncIndexCapFreq = pIndexFreq->freq;
				}
				break;
			}

			case QPCODEC_PROP_ENC_VID_PADDING:
			{
				PQPCODEC_ENC_VID_PADDING	pVidPadding = (PQPCODEC_ENC_VID_PADDING)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_ENC_VID_PADDING enable(%d)\n",
							 pVidPadding->Enable
							 ));
				pTaskData->m_bEncEnableVidPadding = pVidPadding->Enable;
				break;
			}

			case QPCODEC_PROP_ENC_FREEZE_VIDEO:
			{
				PQPCODEC_ENC_FREEZE_VIDEO	pFreezeVid = (PQPCODEC_ENC_FREEZE_VIDEO)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_ENC_FREEZE_VIDEO enable(%d)\n",
							 pFreezeVid->Enable
							 ));
				qpStatus = QPFWENCAPI_FreezeVideo(This,
												  (TQP_UINT32) hIndex,
												  pFreezeVid->Enable
												  );
				if (QP_SUCCESS(qpStatus))
				{
					pTaskData->m_bEncVidFrozen = pFreezeVid->Enable;
				}
				break;
			}


			case QPCODEC_PROP_ENC_STILL_VIDEO_INPUT:
			{
				PQPCODEC_ENC_STILL_VIDEO_INPUT	pStillVid = (PQPCODEC_ENC_STILL_VIDEO_INPUT)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_ENC_STILL_VIDEO_INPUT enable(%d)\n",
							 pStillVid->Enable
							 ));
				qpStatus = QPFWENCAPI_StillVideoInput(This,
													  (TQP_UINT32) hIndex,
													  pStillVid->Enable
													  );
				if (QP_SUCCESS(qpStatus))
				{
					pTaskData->m_bEncVidStillInput = pStillVid->Enable;
				}
				break;
			}

			case QPCODEC_PROP_ENC_CAP_MODE:
			{
				PQPCODEC_ENC_CAP_MODE	pCapMode = (PQPCODEC_ENC_CAP_MODE)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_ENC_CAP_MODE capMode(%d) trigMode(0x%x) gpioPin(%d)\n",
							 pCapMode->capMode,
							 pCapMode->trigMode,
							 pCapMode->gpioPin
							 ));
				pTaskData->m_EncCapMode.capMode = pCapMode->capMode;
				pTaskData->m_EncCapMode.trigMode = pCapMode->trigMode;
				pTaskData->m_EncCapMode.gpioPin = pCapMode->gpioPin;

				if (TASK_STATE_IDLE != pTaskData->m_State)
				{
					qpStatus = QPFWENCAPI_SetEncMode(This,
													 (TQP_UINT32) hIndex,
													 pTaskData->m_EncCapMode.capMode,
													 pTaskData->m_EncCapMode.trigMode,
													 pTaskData->m_EncCapMode.gpioPin
													 );
				}
				break;
			}

			case QPCODEC_PROP_ENC_GET_A_FRAME:
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_ENC_GET_A_FRAME\n"));
				qpStatus = QPFWENCAPI_GetAFrame(This,
												(TQP_UINT32) hIndex
												);
				break;
			}

			case QPCODEC_PROP_ENC_MJPEG_QUALITY:
			{
				PTQP_INT32	pQuality = (PTQP_INT32)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodec_Set() QPCODEC_PROP_ENC_MJPEG_QUALITY quality(%d)\n",
							 *pQuality
							 ));
				if ((*pQuality >= 0) &&
					(*pQuality <= 15)
					)
				{
					pTaskData->m_EncMjpegQuality = (TQP_UINT32)*pQuality;

					if (TASK_STATE_IDLE != pTaskData->m_State)
					{
						qpStatus = QPFWENCAPI_SetMJPEGQuality(This,
	 														  (TQP_UINT32) hIndex,
															  pTaskData->m_EncMjpegQuality
															  );
					}
				}
				break;
			}

			case QPCODEC_PROP_ENC_MJPEG_FRAME_BUF:
			{
				PTQP_INT32	pFrameBufNb = (PTQP_INT32)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodec_Set() QPCODEC_PROP_ENC_MJPEG_FRAME_BUF nb(%d)\n",
							 *pFrameBufNb
							 ));
				if ((*pFrameBufNb >= 0) &&
					(*pFrameBufNb <= 30)
					)
				{
					pTaskData->m_EncMjpegFrameBuffer = (TQP_UINT32)*pFrameBufNb;
				}
				break;
			}

			case QPCODEC_PROP_EXTERNAL_TRIGGER_TO_SYNC:
			{
				PQPCODEC_EXTERNAL_TRIGGER_MODE pExternalTriggerToSync = (PQPCODEC_EXTERNAL_TRIGGER_MODE)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodec_Set() QPCODEC_PROP_EXTERNAL_TRIGGER_TO_SYNC Enable(%d) gpioPin(%d)\n",
							 pExternalTriggerToSync->Enable,
							 pExternalTriggerToSync->gpioPin
							 ));
				pTaskData->m_ExternalTriggerToSync.Enable = pExternalTriggerToSync->Enable;
				pTaskData->m_ExternalTriggerToSync.gpioPin = pExternalTriggerToSync->gpioPin;

				break;
			}

			case QPCODEC_PROP_PTS_COUNTER_RESET:
			{
				PQPCODEC_PTS_COUNTER_RESET pPTSCounterReset = (PQPCODEC_PTS_COUNTER_RESET)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodec_Set() PQPCODEC_PTS_COUNTER_RESET Enable(%d) gpioPin(%d) immediate(%d)\n",
							 pPTSCounterReset->Enable,
							 pPTSCounterReset->gpioPin,
							 pPTSCounterReset->immediate
							 ));
				pTaskData->m_PTSCounterReset.Enable = pPTSCounterReset->Enable;
				pTaskData->m_PTSCounterReset.gpioPin = pPTSCounterReset->gpioPin;
				pTaskData->m_PTSCounterReset.immediate = pPTSCounterReset->immediate;

				break;
			}

			case QPCODEC_PROP_DECIMATION:
			{
				PQPCODEC_DECIMATION_FACTOR pDecimationFactor = (PQPCODEC_DECIMATION_FACTOR)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodec_Set() QPCODEC_PROP_DECIMATION input_format(%d) output_format(%d) h_numerator(%d) h_denominator(%d) v_numerator(%d) v_denominator(%d)\n",
							 pDecimationFactor->input_format,
							 pDecimationFactor->output_format,
							 pDecimationFactor->scale_factor.factor.h_numerator,
							 pDecimationFactor->scale_factor.factor.h_denominator,
							 pDecimationFactor->scale_factor.factor.v_numerator,
							 pDecimationFactor->scale_factor.factor.v_denominator
							 ));
				pTaskData->m_RawVideoDecimationFactor.input_format = pDecimationFactor->input_format;
				pTaskData->m_RawVideoDecimationFactor.output_format = pDecimationFactor->output_format;
				pTaskData->m_RawVideoDecimationFactor.scale_factor.read = pDecimationFactor->scale_factor.read;

				break;
			}

			case QPCODEC_PROP_DEINTERLACE:
			{
				PTQP_INT32	pMode = (PTQP_INT32)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodec_Set() QPCODEC_PROP_DEINTERLACE mode(%d)\n",
							 *pMode
							 ));

				if ((*pMode >= 0) &&
					(*pMode <= 2)
					)
				{
					pTaskData->m_DeinterlaceMode = (TQP_UINT32)*pMode;
				}
				break;
			}

			case QPCODEC_PROP_LARGE_COMPRESS_BUFFER_CONTROL:
			{
				PTQP_INT32	pControl = (PTQP_INT32)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodec_Set() QPCODEC_PROP_LARGE_COMPRESS_BUFFER_CONTROL (0x%08x)\n",
							 *pControl
							 ));

				pTaskData->m_LargeCompressBufferControl = (TQP_UINT32)*pControl;
				break;
			}

			case QPCODEC_PROP_ENABLE_LOW_BITRATE_MODE:
			{
				PTQP_INT32	pControl = (PTQP_INT32)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodec_Set() QPCODEC_PROP_ENABLE_LOW_BITRATE_MODE (0x%08x)\n",
							 *pControl
							 ));

				pTaskData->m_EnableLowBitrateMode = (TQP_UINT32)*pControl;
				break;
			}

			case QPCODEC_PROP_ENABLE_AV_DISCARD_CONTROL:
			{
				PTQP_INT32	pControl = (PTQP_INT32)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodec_Set() QPCODEC_PROP_ENABLE_AV_DISCARD_CONTROL (0x%08x)\n",
							 *pControl
							 ));

				pTaskData->m_EnableAVDiscardControl = (TQP_UINT32)*pControl;
				break;
			}

			case QPCODEC_PROP_USE_SW_PTS:
			{
				PTQP_INT32	pControl = (PTQP_INT32)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodec_Set() QPCODEC_PROP_USE_SW_PTS (0x%08x)\n",
							 *pControl
							 ));

				pTaskData->m_UseSWPTS = (TQP_UINT32)*pControl;
				break;
			}

			case QPCODEC_PROP_VIU_SYN_CCODE_1: // [H.P.LIN] peter
			{
				PTQP_INT32	pControl = (PTQP_INT32)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodec_Set() QPCODEC_PROP_VIU_SYN_CCODE_1 (0x%08x)\n",
							 *pControl
							 ));

				This->m_ulViuSyncCode1 = (TQP_UINT32)*pControl;
				break;
			}

			case QPCODEC_PROP_VIU_SYN_CCODE_2: // [H.P.LIN] peter
			{
				PTQP_INT32	pControl = (PTQP_INT32)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodec_Set() QPCODEC_PROP_VIU_SYN_CCODE_2 (0x%08x)\n",
							 *pControl
							 ));

				This->m_ulViuSyncCode2 = (TQP_UINT32)*pControl;
				break;
			}

			case QPCODEC_PROP_ENC_ALL_SETTINGS:
			{
				PQPCODEC_ENC_ALL_SETTINGS	pAll = (PQPCODEC_ENC_ALL_SETTINGS)pInput;

				pTaskData->m_systemControl.Read = pAll->EncSystemControl;
				pTaskData->m_pictureResolution.Read = pAll->EncPictureResolution;
				pTaskData->m_inputControl.Read = pAll->EncInputControl;
				pTaskData->m_syncMode = pAll->EncSyncMode;
				pTaskData->m_rateControl.Read = pAll->EncRateControl;
				pTaskData->m_rateControlEx.Read = pAll->EncRateControlEx;
				pTaskData->m_bitRate.Read = pAll->EncBitRate;
				pTaskData->m_filterControl.Read = pAll->EncFilterControl;
				pTaskData->m_gopLoopFilter.Read = pAll->EncGopLoopFilter;
				pTaskData->m_etControl.Read = pAll->EncEtControl;
				pTaskData->m_outPictureResolution.Read = pAll->EncOutPictureResolution;
				pTaskData->m_blkXferSize.Read = pAll->EncBlockXferSize;
				pTaskData->m_audioControlParam.Read = pAll->EncAudioControlParam;
#ifdef CONFIG_PL_AI_FIX_IN_MASTER
				pTaskData->m_audioControlParam.Reg.ai_m = AI_M_MASTER;
#endif
				pTaskData->m_audioControlExAAC.Read = pAll->EncAudioControlExAAC;
				pTaskData->m_audioControlExG711.Read = pAll->EncAudioControlExG711;
				pTaskData->m_audioControlExLPCM.Read = pAll->EncAudioControlExLPCM;
				pTaskData->m_EncIndexCapFreq = pAll->EncIndexCapFreq;
				pTaskData->m_MP4VideoBlockNumber = pAll->EncMP4VideoBlockNumber;
				pTaskData->m_EncStopMode = pAll->EncStopMode;
				pTaskData->m_bEncEnableVidPadding = pAll->EncEnableVidPadding;
				pTaskData->m_EncMjpegQuality = pAll->EncMjpegQuality;
				pTaskData->m_EncMjpegFrameBuffer = pAll->EncMjpegFrameBuffer;
				pTaskData->m_ExternalTriggerToSync.Enable = pAll->EncExternalTriggerToSyncEnable;
				pTaskData->m_ExternalTriggerToSync.gpioPin = pAll->EncExternalTriggerToSyncGpioPin;
				pTaskData->m_PTSCounterReset.Enable = pAll->EncPTSResetByTriggerEnable;
				pTaskData->m_PTSCounterReset.gpioPin = pAll->EncPTSResetByTriggerGpioPin;
				pTaskData->m_PTSCounterReset.immediate = pAll->EncPTSResetByTriggerImmediate;
				pTaskData->m_RawVideoDecimationFactor.input_format = pAll->EncDecimationInputFormat;
				pTaskData->m_RawVideoDecimationFactor.output_format = pAll->EncDecimationOutputFormat;
				pTaskData->m_RawVideoDecimationFactor.scale_factor.read = pAll->EncDecimationScaleFactor;

				pTaskData->m_DeinterlaceMode = pAll->DeinterlaceMode;
				pTaskData->m_audioControlExSILK.Read = pAll->EncAudioControlExSILK;

				pTaskData->m_LargeCompressBufferControl = pAll->LargeCompressBufferControl;
				pTaskData->m_EnableLowBitrateMode = pAll->EnableLowBitrateMode;

				pTaskData->m_EnableAVDiscardControl = pAll->EnableAVDiscardControl;
				pTaskData->m_UseSWPTS = pAll->UseSWPTS;

				break;
			}

			// Decoder
			//
			case QPCODEC_PROP_DEC_STREAM_ID:
			{
				PTQP_UINT32	pStreamID = (PTQP_UINT32)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_DEC_STREAM_ID (%d)\n",
							 *pStreamID
							 ));

				pTaskData->m_streamID.Reg.stream_type = *pStreamID;
				break;
			}

			case QPCODEC_PROP_DEC_OUTPUT_MODE:
			{
				PTQP_UINT32	pOutputMode = (PTQP_UINT32)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_DEC_OUTPUT_MODE (0x%x)\n",
							 *pOutputMode
							 ));

				pTaskData->m_outputMode = *pOutputMode;
				break;
			}

			case QPCODEC_PROP_DEC_RESOLUTION:
			{
				PQPCODEC_PICTURE_RESOLUTION	pPicRes = (PQPCODEC_PICTURE_RESOLUTION)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_DEC_RESOLUTION width(%d) height(%d)\n",
							 pPicRes->Beringer.pic_width,
							 pPicRes->Beringer.pic_height
							 ));

				if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
				{
					pTaskData->m_decodeResolution.Reg.pic_height = pPicRes->Artesa.in_pic_height;
					pTaskData->m_decodeResolution.Reg.pic_width = pPicRes->Artesa.in_pic_width;
				}
				else
				{
					pTaskData->m_decodeResolution.Reg.pic_height = pPicRes->Beringer.pic_height;
					pTaskData->m_decodeResolution.Reg.pic_width = pPicRes->Beringer.pic_width;
				}
				break;
			}

			case QPCODEC_PROP_DEC_STOP_MODE:
			{
				PQPCODEC_DEC_STOP_MODE	pStopMode = (PQPCODEC_DEC_STOP_MODE)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_DEC_STOP_MODE stop_mode(%d) disp_mode(%d)\n",
							 pStopMode->stop_mode,
							 pStopMode->stop_disp_mode
							 ));
				pTaskData->m_decStopMode = pStopMode->stop_mode;
				pTaskData->m_decStopDispMode = pStopMode->stop_disp_mode;
				break;
			}

			case QPCODEC_PROP_DEC_TRICK_MODE:
			{
				PQPCODEC_DEC_TRICK_MODE	pTrickMode = (PQPCODEC_DEC_TRICK_MODE)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_DEC_TRICK_MODE spd(%d) dir(%d) step(%d) PTS(%d) disp_pref(%d)\n",
							 pTrickMode->playSpeed,
							 pTrickMode->playDirection,
							 pTrickMode->step,
							 pTrickMode->PTS,
							 pTrickMode->fieldPreference
							 ));

				// remember settings and send the command to firmware
				//
				qpStatus = QPFWDECAPI_SetTrickMode(This,
												   (TQP_UINT32) hIndex,
												   pTrickMode->playSpeed,
												   pTrickMode->playDirection,
												   pTrickMode->step,
												   pTrickMode->PTS,
												   pTrickMode->fieldPreference
												   );
				if (QP_SUCCESS(qpStatus))
				{
					pTaskData->m_decTrickPlaySpeed = pTrickMode->playSpeed;
					pTaskData->m_decTrickPlayDirection = pTrickMode->playDirection;
					pTaskData->m_decTrickFieldPreference = pTrickMode->fieldPreference;
				}
				break;
			}

			case QPCODEC_PROP_DEC_DISP_ATTRIB:
			{
				PQPCODEC_DEC_DISP_ATTRIB	pDispAttrib = (PQPCODEC_DEC_DISP_ATTRIB)pInput;
				CQLCodecLib					*pQLCodecLib = GET_INHERITED_OBJECT(CQLCodecLib, m_Object, This->m_Object.m_pParent);

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_DEC_DISP_ATTRIB TVSystem(%d) in_size(0x%x) out_size(0x%x) in_orig(0x%x) out_orig(0x%x) ForceRemote(%d)\n",
							 pDispAttrib->TVSystem,
							 pDispAttrib->inputSize.Read,
							 pDispAttrib->outputSize.Read,
							 pDispAttrib->inputOrigin.Read,
							 pDispAttrib->outputOrigin.Read,
							 pDispAttrib->ForceRemote
							 ));

				// remember settings and send the command to firmware
				//
				qpStatus = QPFWDECAPI_SetDisplayAttrib(This,
													   (TQP_UINT32) hIndex,
                                                       (pDispAttrib->ForceRemote << 0x8) | (pDispAttrib->TVSystem & 0xFF),
													   pDispAttrib->inputSize.Read,
													   pDispAttrib->outputSize.Read,
													   pDispAttrib->inputOrigin.Read,
													   pDispAttrib->outputOrigin.Read
													   );
				if (QP_SUCCESS(qpStatus))
				{
					pTaskData->m_decDispTVSystem = pDispAttrib->TVSystem;
					pTaskData->m_decDispForceRemote = pDispAttrib->ForceRemote;
					pTaskData->m_decDispInputSize.Read = pDispAttrib->inputSize.Read;
					pTaskData->m_decDispOutputSize.Read = pDispAttrib->outputSize.Read;
					pTaskData->m_decDispInputOrigin.Read = pDispAttrib->inputOrigin.Read;
					pTaskData->m_decDispOutputOrigin.Read = pDispAttrib->outputOrigin.Read;

					// config video encoder
					if (pQLCodecLib->m_pVidEncoder)
					{
						VIDEO_STANDARD	std = VIDEO_STANDARD_INVALID;

						switch (pDispAttrib->TVSystem)
						{
							case QPDEC_DISP_ATTRIB_576i:
								std = VIDEO_STANDARD_PAL;
								break;
							case QPDEC_DISP_ATTRIB_480i:
								std = VIDEO_STANDARD_NTSC;
								break;
							case QPDEC_DISP_ATTRIB_720p:
								std = VIDEO_STANDARD_720P;
								break;
							case QPDEC_DISP_ATTRIB_1080i:
								std = VIDEO_STANDARD_1080I;
								break;
							case QPDEC_DISP_ATTRIB_1080p:
								std = VIDEO_STANDARD_1080P;
								break;
							case QPDEC_DISP_ATTRIB_480p:
							default:
								break;
						}

						if (VIDEO_STANDARD_INVALID != std)
						{
							qpStatus = IVideoEncoder_SetVideoStandard(&pQLCodecLib->m_pVidEncoder->m_iVideoEncoder,
																	  std
																	  );
                            if (!QP_SUCCESS(qpStatus))
                            {
                                QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
                                "CQLCodec_Set() QPCODEC_PROP_DEC_DISP_ATTRIB IVideoEncoder_SetVideoStandard set %d fail, ignore it\n",
                                std
                                ));
                                qpStatus = QPERR_SUCCESS;
                            }
						}
					}
				}

				break;
			}

			case QPCODEC_PROP_DEC_DISP_MODE:
			{
				PQPCODEC_DEC_DISP_MODE	pDispMode = (PQPCODEC_DEC_DISP_MODE)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodec_Set() QPCODEC_PROP_DEC_DISP_MODE operation(%d) dispMode(%d)\n",
							 pDispMode->operation,
							 pDispMode->displayMode
							 ));

				// remember settings and send the command to firmware
				//
				qpStatus = QPFWDECAPI_SetDisplayMode(This,
													 (TQP_UINT32) hIndex,
													 pDispMode->operation,
													 pDispMode->displayMode
													 );

				if (QP_SUCCESS(qpStatus))
				{
					pTaskData->m_decDispMode = pDispMode->displayMode;
				}
				break;
			}

			case QPCODEC_PROP_DEC_AUDIO_CONTROL:
			{
				PQPCODEC_DEC_AUDIO_CONTROL	pAudControls = (PQPCODEC_DEC_AUDIO_CONTROL)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_DEC_AUDIO_CONTROL audio_type(%d) ch_num(%d) sample_rate(%d) mode(%d) upsample(%d)\n",
							 pAudControls->audio_type,
							 pAudControls->ch_num,
							 pAudControls->sample_rate,
							 pAudControls->mode,
							 pAudControls->upsample
							 ));
				This->m_AudControls.Read = 0;
				This->m_AudControls.Reg.audio_type = pAudControls->audio_type;
				This->m_AudControls.Reg.ch_num = pAudControls->ch_num;
				This->m_AudControls.Reg.sample_rate = pAudControls->sample_rate;
				This->m_AudControls.Reg.mode = pAudControls->mode;
				This->m_AudControls.Reg.upsample = pAudControls->upsample;
				break;
			}

			case QPCODEC_PROP_DEC_DATA_XFER_CONTROL:
			{
				PQPCODEC_DEC_DATA_XFER_CONTROL	pXferMode = (PQPCODEC_DEC_DATA_XFER_CONTROL)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_DEC_DATA_XFER_CONTROL xfer_mode(%d) minimum_size(%d)\n",
							 pXferMode->xfer_mode,
							 pXferMode->minimum_size
							 ));
				pTaskData->m_decXferMode = pXferMode->xfer_mode;
				pTaskData->m_decXferMinimumSize = pXferMode->minimum_size;
				break;
			}

			case QPCODEC_PROP_DEC_ALL_SETTINGS:
			{
				PQPCODEC_DEC_ALL_SETTINGS	pAll = (PQPCODEC_DEC_ALL_SETTINGS)pInput;

				pTaskData->m_streamID.Read = pAll->streamID;
				pTaskData->m_outputMode = pAll->outputMode;
				pTaskData->m_decodeResolution.Read = pAll->decodeResolution;
				pTaskData->m_decStopMode = pAll->decStopMode;
  				pTaskData->m_decStopDispMode = pAll->decStopDispMode;
  				pTaskData->m_decDispMode = pAll->decDispMode;
  				pTaskData->m_decDispTVSystem = pAll->decDispTVSystem;
  				pTaskData->m_decDispForceRemote = pAll->decDispForceRemote;
  				pTaskData->m_decDispInputSize.Read = pAll->decDispInputSize;
  				pTaskData->m_decDispOutputSize.Read = pAll->decDispOutputSize;
  				pTaskData->m_decDispInputOrigin.Read = pAll->decDispInputOrigin;
  				pTaskData->m_decDispOutputOrigin.Read = pAll->decDispOutputOrigin;
				pTaskData->m_decXferMode = pAll->decXferMode;
				pTaskData->m_decXferMinimumSize = pAll->decXferMinimumSize;
				break;
			}

			// Misc
			//
			case QPCODEC_PROP_BEGIN_CHANNEL_CHANGE:
			case QPCODEC_PROP_END_CHANNEL_CHANGE:
				break;


			case QPCODEC_PROP_656_OUT_CNTL:
			{
				PQPCODEC_OUT_CNTL	pOutCntl = (PQPCODEC_OUT_CNTL)pInput;

				qpStatus = CQLCodec_VOSwitch(This,
											 pOutCntl->Enable
											 );
				if (QP_SUCCESS(qpStatus))
					This->m_VOEnable = pOutCntl->Enable;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_656_OUT_CNTL (%d) status(%d)\n",
							 pOutCntl->Enable,
							 qpStatus
							 ));
				break;
			}

			case QPCODEC_PROP_I2S_OUT_CNTL:
			{
				PQPCODEC_OUT_CNTL	pOutCntl = (PQPCODEC_OUT_CNTL)pInput;

				qpStatus = CQLCodec_AOSwitch(This,
											 pOutCntl->Enable
											 );
				if (QP_SUCCESS(qpStatus))
					This->m_AOEnable = pOutCntl->Enable;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodec_Set() QPCODEC_PROP_I2S_OUT_CNTL enable(%d) status(%d)\n",
							 pOutCntl->Enable,
							 qpStatus
							 ));
				break;
			}

			case QPCODEC_PROP_VOU_CONFIG:
			{
				PQPCODEC_VOU_CONFIG	pVOUConfig = (PQPCODEC_VOU_CONFIG)pInput;

				This->m_VOUMode = pVOUConfig->VOUMode;
				This->m_VOUStartPixel = pVOUConfig->VOUStartPixel;
				This->m_VOUStartLine = pVOUConfig->VOUStartLine;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_VOU_CONFIG VOUMode(%d) start_pixel(%d) start_line(%d)\n",
							 This->m_VOUMode,
							 This->m_VOUStartPixel,
							 This->m_VOUStartLine
							 ));
				break;
			}

			case QPCODEC_PROP_AOU_CONFIG:
			{
				PQPCODEC_AOU_CONFIG	pAOUConfig = (PQPCODEC_AOU_CONFIG)pInput;

				This->m_AOControls.Reg.ao_en = pAOUConfig->ao_en;
				This->m_AOControls.Reg.ao_msb = pAOUConfig->ao_msb;
				This->m_AOControls.Reg.lrclk_i = pAOUConfig->lrclk_i;
				This->m_AOControls.Reg.bclk_i = pAOUConfig->bclk_i;
				This->m_AOControls.Reg.ao_i2s = pAOUConfig->ao_i2s;
				This->m_AOControls.Reg.ao_rj = pAOUConfig->ao_rj;
#ifdef CONFIG_PL_AO_FIX_IN_SLAVE
				This->m_AOControls.Reg.ao_s = AO_MODE_SLAVE;
#else
				This->m_AOControls.Reg.ao_s = pAOUConfig->ao_s;
#endif

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_AOU_CONFIG ao_en(%d) ao_msb(%d) lrclk_i(%d) bclk_i(%d) ao_i2s(%d) ao_rj(%d) ao_s(%d)\n",
							 This->m_AOControls.Reg.ao_en,
							 This->m_AOControls.Reg.ao_msb,
							 This->m_AOControls.Reg.lrclk_i,
							 This->m_AOControls.Reg.bclk_i,
							 This->m_AOControls.Reg.ao_i2s,
							 This->m_AOControls.Reg.ao_rj,
							 This->m_AOControls.Reg.ao_s
							 ));
				break;
			}

			case QPCODEC_PROP_POWER_STATE:
			{
				PQPCODEC_POWER_STATE	pPowerState = (PQPCODEC_POWER_STATE)pInput;

				if (!CQLCodec_IsCodecIdle(This))
				{
					qpStatus = QPERR_INVALID;
				}
				else
				{
					if (pPowerState->On)
					{
						qpStatus = CQLCodec_PowerUp(This);
					}
					else
					{
						qpStatus = CQLCodec_PowerDown(This);
					}
				}

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_POWER_STATE on(%d) status(%d)\n",
							 pPowerState->On,
							 qpStatus
							 ));
				break;
			}

			case QPCODEC_PROP_ERROR_RECOVERY:
			{
				PQPCODEC_ERROR_RECOVERY	pErrorRecovery = (PQPCODEC_ERROR_RECOVERY)pInput;

				This->m_ErrorRecovery = pErrorRecovery->Level;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_ERROR_RECOVERY level(%d) \n",
							 pErrorRecovery->Level
							 ));
				break;
			}

			// GPIO
			//
			case QPCODEC_GPIO_BIT_DIRECTION:
			{
				PGPIO_BIT_DIRECTION	pBitDirection = (PGPIO_BIT_DIRECTION)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_GPIO_BIT_DIRECTION bit(%d) dir(%d) \n",
							 pBitDirection->bBitNumber,
							 pBitDirection->Direction
							 ));

				qpStatus = CQLCodec_SetGPIOBitDirection(This,
														pBitDirection
														);
				break;
			}

			case QPCODEC_GPIO_BIT_VALUE:
			{
				PGPIO_BIT_VALUE	pBitValue = (PGPIO_BIT_VALUE)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_GPIO_BIT_VALUE bit(%d) val(%d) \n",
							 pBitValue->bBitNumber,
							 pBitValue->bValue
							 ));

				qpStatus = CQLCodec_SetGPIOBitValue(This,
													pBitValue
													);
				break;
			}

			// Codec
			//
			case QPCODEC_PROP_SYS_FUNCTION:
			{
				PQPCODEC_SYS_FUNCTION	pSysFunction = (PQPCODEC_SYS_FUNCTION)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_SYS_FUNCTION function(%d)\n",
							 pSysFunction->function
							 ));
				pTaskData->m_codec_function.function = pSysFunction->function;
				break;
			}

			case QPCODEC_PROP_SYS_LINK:
			{
				PQPCODEC_SYS_LINK	pSysLink = (PQPCODEC_SYS_LINK)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_SYS_LINK video_input(%d) video_in_ch(%d) video_output(%d) video_out_ch(%d) audio_input(%d) audio_in_ch(%d) audio_output(%d) audio_out_ch(%d)\n",
							 pSysLink->video_input,
							 pSysLink->video_in_ch,
							 pSysLink->video_output,
							 pSysLink->video_out_ch,
							 pSysLink->audio_input,
							 pSysLink->audio_in_ch,
							 pSysLink->audio_output,
							 pSysLink->audio_out_ch
							 ));

				pTaskData->m_video_input = pSysLink->video_input;
				pTaskData->m_video_in_ch = pSysLink->video_in_ch;
				pTaskData->m_video_output = pSysLink->video_output;
				pTaskData->m_video_out_ch = pSysLink->video_out_ch;
				pTaskData->m_audio_input = pSysLink->audio_input;
				pTaskData->m_audio_in_ch = pSysLink->audio_in_ch;
				pTaskData->m_audio_output = pSysLink->audio_output;
				pTaskData->m_audio_out_ch = pSysLink->audio_out_ch;
				break;
			}

			case QPCODEC_PROP_SYS_CONFIG:
			{
				PQPCODEC_SYS_CONFIG	pSysConfig = (PQPCODEC_SYS_CONFIG)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodec_Set() QPCODEC_PROP_SYS_CONFIG version(%d) fixed mode(%d)\n",
							 pSysConfig->version,
							 pSysConfig->fixedMode
							 ));
				This->m_VerFwAPI = pSysConfig->version;
				This->m_FwFixedMode = pSysConfig->fixedMode;
				break;
			}

			case QPCODEC_PROP_OSD_MEMORY:
			{
				PQPCODEC_OSD_MEM_CONFIG	pOsdMemConfig = (PQPCODEC_OSD_MEM_CONFIG)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodec_Set() QPCODEC_PROP_OSD_MEMORY command(0x%x) size(0x%x) \n",
							 pOsdMemConfig->command,
							 pOsdMemConfig->memSize
							 ));


				// send the command to firmware
				//
				qpStatus = QPFWDECAPI_GetVouOsdMem(This,
												   (TQP_UINT32) hIndex,
												   pOsdMemConfig->command,
												   pOsdMemConfig->memSize
												   );
				break;
			}

			case QPCODEC_PROP_ENC_OSD_SETTINGS:
			{
				PTQP_UINT32	pbVIOSDen = (PTQP_UINT32)pInput;
				qpStatus = QPFWENCAPI_SetVIOSD(This,
												(TQP_UINT32) hIndex,
												*pbVIOSDen
											   );

					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Set() QPCODEC_PROP_ENC_OSD_SETTINGS  bVIOSDEn(0x%x) status(%d)\n",
								 *pbVIOSDen,
								 qpStatus
								 ));
				break;
			}

			case QPCODEC_PROP_AUDIO_OUTPUT_VOLUME:
			{
				PTQP_UINT32	pVol = (PTQP_UINT32)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_AUDIO_OUTPUT_VOLUME vol(%u)\n",
							 *pVol
							 ));

				This->m_AOVolume = *pVol;

				qpStatus = QPFWDECAPI_SetAudioOutputVolume(This,
														   (TQP_UINT32) hIndex,
														   *pVol
														   );
				break;
			}

			case QPCODEC_PROP_AUDIO_INPUT_VOLUME:
			{
				PTQP_UINT32	pVol = (PTQP_UINT32)pInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Set() QPCODEC_PROP_AUDIO_INPUT_VOLUME vol(%d)\n",
							 *pVol
							 ));

				This->m_AIVolume = *pVol;

				qpStatus = QPFWENCAPI_SetAudioInputVolume(This,
														   (TQP_UINT32) hIndex,
														   *pVol
														   );
				break;
			}

			default:
				break;
		}
	}
	else
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "CQLCodec_Set() unknown property\n"
					 ));
		qpStatus = QPERR_NOTIMPL;
	}

	return (qpStatus);
}



static EQP_Errors CQLCodec_Get(IMpegCodec *This_p,
							   TQP_REFGUID guid,
							   TQP_OBJECT_HANDLE hIndex,
							   TQP_UINT32 dwCode,
							   TQP_PVOID pInput,
							   TQP_PVOID pOutput,
							   PTQP_UINT32 pdwSizeGot
							   )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	CQLCodec	*This =	GET_INHERITED_OBJECT(CQLCodec, m_iMpegCodec, This_p);
	PTASK_DATA	pTaskData;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_Get() hIndex(%d)\n",
				 hIndex
				 ));

	if ((hIndex >= MAX_TASKS) ||
		!This->m_pTask->m_TaskData[hIndex].valid
		)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "CQLCodec_Get() invalid task\n"
					 ));
		return (QPERR_PARMS);
	}

	pTaskData = &This->m_pTask->m_TaskData[hIndex];

	if (IsEqualTQPGUID((TQP_REFGUID)&PROPSETID_QPCODEC_CONTROL,
					   guid
					   ))
	{
		switch (dwCode)
		{
			// Encoder
			//
			case QPCODEC_PROP_SYSTEM_CONTROL:
			{
				PQPCODEC_SYSTEM_CONTROL	pSystemControl = (PQPCODEC_SYSTEM_CONTROL)pOutput;

				if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
				{
					pSystemControl->Artesa.stream_type = pTaskData->m_systemControl.RegArtesa.stream_type;
					pSystemControl->Artesa.stream_data = pTaskData->m_systemControl.RegArtesa.stream_data;
					pSystemControl->Artesa.profile = pTaskData->m_systemControl.RegArtesa.profile;
					pSystemControl->Artesa.level = pTaskData->m_systemControl.RegArtesa.level;
					pSystemControl->Artesa.ff_mode = pTaskData->m_systemControl.RegArtesa.ff_mod;
					pSystemControl->Artesa.spsr_freq = pTaskData->m_systemControl.RegArtesa.spsr_freq;
					pSystemControl->Artesa.v_mode = pTaskData->m_systemControl.RegArtesa.v_mode;
					pSystemControl->Artesa.cabac_init_idc = pTaskData->m_systemControl.RegArtesa.cabac_init_idc;
					pSystemControl->Artesa.version = pTaskData->m_systemControl.RegArtesa.version;
					pSystemControl->Artesa.xfer_mode = pTaskData->m_systemControl.RegArtesa.xfer_mode;

					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Get() QPCODEC_PROP_SYSTEM_CONTROL stream type(%d) stream data(%d) profile(%d) level(%d) ff_mod(%d)\r\nspsr_freq(%d) v_mode(%d) cabac_init(%d) ver(%d) xfer_mode(%d)\n",
								 pSystemControl->Artesa.stream_type,
								 pSystemControl->Artesa.stream_data,
								 pSystemControl->Artesa.profile,
								 pSystemControl->Artesa.level,
								 pSystemControl->Artesa.ff_mode,
								 pSystemControl->Artesa.spsr_freq,
								 pSystemControl->Artesa.v_mode,
								 pSystemControl->Artesa.cabac_init_idc,
								 pSystemControl->Artesa.version,
								 pSystemControl->Artesa.xfer_mode
								 ));
				}
				else
				{
					pSystemControl->Beringer.stream_type = pTaskData->m_systemControl.Reg.stream_type;
					pSystemControl->Beringer.profile = pTaskData->m_systemControl.Reg.profile;
					pSystemControl->Beringer.level = pTaskData->m_systemControl.Reg.level;
					pSystemControl->Beringer.cap_mode = pTaskData->m_systemControl.Reg.cap_mod;
					pSystemControl->Beringer.ff_mode = pTaskData->m_systemControl.Reg.ff_mod;
					pSystemControl->Beringer.spsr_freq = pTaskData->m_systemControl.Reg.spsr_freq;
					pSystemControl->Beringer.v_mode = pTaskData->m_systemControl.Reg.v_mode;
					pSystemControl->Beringer.cabac_init_idc = pTaskData->m_systemControl.Reg.cabac_init_idc;
					pSystemControl->Beringer.version = pTaskData->m_systemControl.Reg.version;

					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Get() QPCODEC_PROP_SYSTEM_CONTROL stream type(%d) profile(%d) level(%d)\r\n cap_mode(%d) ff_mode(%d) spsr_freq(%d) v_mode(%d) cabac_init(%d) ver(%d)\n",
								 pSystemControl->Beringer.stream_type,
								 pSystemControl->Beringer.profile,
								 pSystemControl->Beringer.level,
								 pSystemControl->Beringer.cap_mode,
								 pSystemControl->Beringer.ff_mode,
								 pSystemControl->Beringer.spsr_freq,
								 pSystemControl->Beringer.v_mode,
								 pSystemControl->Beringer.cabac_init_idc,
								 pSystemControl->Beringer.version
								 ));
				}

				*pdwSizeGot = sizeof(QPCODEC_SYSTEM_CONTROL);
				break;
			}

			case QPCODEC_PROP_PICTURE_RESOLUTION:
			{
				PQPCODEC_PICTURE_RESOLUTION	pPictureResoultion = (PQPCODEC_PICTURE_RESOLUTION)pOutput;

				if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
				{
					pPictureResoultion->Artesa.in_pic_width = pTaskData->m_pictureResolution.RegArtesa.in_pic_width;
					pPictureResoultion->Artesa.in_pic_height = pTaskData->m_pictureResolution.RegArtesa.in_pic_height;

					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Get() QPCODEC_PROP_PICTURE_RESOLUTION in_width(%d) in_height(%d)",
								 pPictureResoultion->Artesa.in_pic_width,
								 pPictureResoultion->Artesa.in_pic_height
								 ));
				}
				else
				{
					pPictureResoultion->Beringer.pic_width = pTaskData->m_pictureResolution.Reg.pic_width;
					pPictureResoultion->Beringer.h_scale = pTaskData->m_pictureResolution.Reg.h_scale;
					pPictureResoultion->Beringer.pic_height = pTaskData->m_pictureResolution.Reg.pic_height;
					pPictureResoultion->Beringer.v_scale = pTaskData->m_pictureResolution.Reg.v_scale;

					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Get() QPCODEC_PROP_PICTURE_RESOLUTION width(%d) height(%d) h_scale(%d) v_scale(%d)\n",
								 pPictureResoultion->Beringer.pic_width,
								 pPictureResoultion->Beringer.pic_height,
								 pPictureResoultion->Beringer.h_scale,
								 pPictureResoultion->Beringer.v_scale
								 ));
				}

				*pdwSizeGot = sizeof(QPCODEC_PICTURE_RESOLUTION);
				break;
			}

			case QPCODEC_PROP_INPUT_CONTROL:
			{
				PQPCODEC_INPUT_CONTROL	pInputControl = (PQPCODEC_INPUT_CONTROL)pOutput;

				if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
				{
					pInputControl->Artesa.scan_format = pTaskData->m_inputControl.RegArtesa.scan_format;
					pInputControl->Artesa.src_mode = pTaskData->m_inputControl.RegArtesa.src_mode;
					pInputControl->Artesa.sync_mode = pTaskData->m_syncMode;
					pInputControl->Artesa.data_type = pTaskData->m_inputControl.RegArtesa.data_type;
					pInputControl->Artesa.in_frame_rate = pTaskData->m_inputControl.RegArtesa.in_frame_rate;
					pInputControl->Artesa.out_frame_rate = pTaskData->m_inputControl.RegArtesa.out_frame_rate;
					pInputControl->Artesa.force_60p_in = pTaskData->m_inputControl.RegArtesa.force_60p_in;
					pInputControl->Artesa.pd_en = pTaskData->m_inputControl.RegArtesa.pd_en;
					pInputControl->Artesa.ck_edge = pTaskData->m_inputControl.RegArtesa.ck_edge;
					pInputControl->Artesa.bytes_swap = pTaskData->m_inputControl.RegArtesa.bytes_swap;

					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Get() QPCODEC_PROP_INPUT_CONTROL scan(%d) src_mode(%d) sync_mode(%d:%d) data_type(%d) \r\n in_frame_rate(%d) out_frame_rate(%d) force_60p_in(%d) pd_en(%d) ck_edge(%d) bytes_swap(%d)\n",
								 pInputControl->Artesa.scan_format,
								 pInputControl->Artesa.src_mode,
								 pInputControl->Artesa.sync_mode,
								 pTaskData->m_inputControl.RegArtesa.sync_mode,
								 pInputControl->Artesa.data_type,
								 pInputControl->Artesa.in_frame_rate,
								 pInputControl->Artesa.out_frame_rate,
								 pInputControl->Artesa.force_60p_in,
								 pInputControl->Artesa.pd_en,
								 pInputControl->Artesa.ck_edge,
								 pInputControl->Artesa.bytes_swap
								 ));
				}
				else
				{
					pInputControl->Beringer.mode = pTaskData->m_inputControl.Reg.mode;
					pInputControl->Beringer.src = pTaskData->m_inputControl.Reg.src;
					pInputControl->Beringer.dst = pTaskData->m_inputControl.Reg.dst;
					pInputControl->Beringer.bl_e = pTaskData->m_inputControl.Reg.bl_e;
					pInputControl->Beringer.black_line_num = pTaskData->m_inputControl.Reg.black_line_num;
					pInputControl->Beringer.in_frame_rate = pTaskData->m_inputControl.Reg.in_frame_rate;
					pInputControl->Beringer.out_frame_rate = pTaskData->m_inputControl.Reg.out_frame_rate;
					pInputControl->Beringer.fmt = pTaskData->m_inputControl.Reg.fmt;
					pInputControl->Beringer.pd_en = pTaskData->m_inputControl.Reg.pd_en;
					pInputControl->Beringer.ck_edge = pTaskData->m_inputControl.Reg.ck_edge;

					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Get() QPCODEC_PROP_INPUT_CONTROL mode(%d) src(%d) dst(%d) bl_e(%d) lines(%d)\r\n frame_rate(%d) out_fr(%d) fmt(%d) pd_en(%d) ck_edge(%d)\n",
								 pInputControl->Beringer.mode,
								 pInputControl->Beringer.src,
								 pInputControl->Beringer.dst,
								 pInputControl->Beringer.bl_e,
								 pInputControl->Beringer.black_line_num,
								 pInputControl->Beringer.in_frame_rate,
								 pInputControl->Beringer.out_frame_rate,
								 pInputControl->Beringer.fmt,
								 pInputControl->Beringer.pd_en,
								 pInputControl->Beringer.ck_edge
								 ));
				}

				*pdwSizeGot = sizeof(QPCODEC_INPUT_CONTROL);
				break;
			}

			case QPCODEC_PROP_RATE_CONTROL:
			{
				PQPCODEC_RATE_CONTROL	pRateControl = (PQPCODEC_RATE_CONTROL)pOutput;

				if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
				{
					pRateControl->Artesa.avg_bit_rate = pTaskData->m_rateControl.RegArtesa.avg_bit_rate;
					pRateControl->Artesa.qp_update_freq = pTaskData->m_rateControl.RegArtesa.qp_update_freq;
					pRateControl->Artesa.mode = pTaskData->m_rateControl.RegArtesa.mode;
					pRateControl->Artesa.fixed = pTaskData->m_rateControl.RegArtesa.fixed;
					pRateControl->Artesa.vbr = pTaskData->m_rateControl.RegArtesa.vbr;
					pRateControl->Artesa.intra_refresh_mode = pTaskData->m_rateControl.RegArtesa.intra_refresh_mode;

					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Get() QPCODEC_PROP_RATE_CONTROL bitrate(%d) qp_update(%d) mode(%d) fixed(%d) vbr(%d) intra_refresh_mode(%d)\n",
								 pRateControl->Artesa.avg_bit_rate,
								 pRateControl->Artesa.qp_update_freq,
								 pRateControl->Artesa.mode,
								 pRateControl->Artesa.fixed,
								 pRateControl->Artesa.vbr,
								 pRateControl->Artesa.intra_refresh_mode
								 ));
				}
				else
				{
					pRateControl->Beringer.avg_bit_rate = pTaskData->m_rateControl.Reg.avg_bit_rate;
					pRateControl->Beringer.qp_update_freq = pTaskData->m_rateControl.Reg.qp_update_freq;
					pRateControl->Beringer.vbr = pTaskData->m_rateControl.Reg.vbr;

					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Get() QPCODEC_PROP_RATE_CONTROL bitrate(%d) qp_update(%d) vbr(%d)\n",
								 pRateControl->Beringer.avg_bit_rate,
								 pRateControl->Beringer.qp_update_freq,
								 pRateControl->Beringer.vbr
								 ));
				}

				*pdwSizeGot = sizeof(QPCODEC_RATE_CONTROL);
				break;
			}

			case QPCODEC_PROP_RATE_CONTROL_EX:
			{
				PRATE_CONTROL_EX	pRateControlEx = (PRATE_CONTROL_EX)pOutput;

				if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
				{
					pRateControlEx->Artesa.base_qp_length = pTaskData->m_rateControlEx.Artesa.base_qp_length;
					pRateControlEx->Artesa.local_activity_on = pTaskData->m_rateControlEx.Artesa.local_activity_on;
					pRateControlEx->Artesa.motion_detect_interval = pTaskData->m_rateControlEx.Artesa.motion_detect_interval;

					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Get() QPCODEC_PROP_RATE_CONTROL_EX base_qp_length(%d) local_activity_on(%d)\n",
								 pRateControlEx->Artesa.base_qp_length,
								 pRateControlEx->Artesa.local_activity_on
								 ));
				}

				*pdwSizeGot = sizeof(RATE_CONTROL_EX);
				break;
			}

			case QPCODEC_PROP_VBR_BIT_RATE:
			{
				PQPCODEC_VBR_BIT_RATE	pBitRate = (PQPCODEC_VBR_BIT_RATE)pOutput;

				pBitRate->min_bit_rate = pTaskData->m_bitRate.Reg.min_bit_rate;
				pBitRate->max_bit_rate = pTaskData->m_bitRate.Reg.max_bit_rate;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_VBR_BIT_RATE min(%d) max(%d)\n",
							 pBitRate->min_bit_rate,
							 pBitRate->max_bit_rate
							 ));

				*pdwSizeGot = sizeof(QPCODEC_VBR_BIT_RATE);
				break;
			}

			case QPCODEC_PROP_FILTER_CONTROL:
			{
				PQPCODEC_FILTER_CONTROL	pFilterControl = (PQPCODEC_FILTER_CONTROL)pOutput;

				pFilterControl->start_pixel = pTaskData->m_filterControl.Reg.start_pixel;
				pFilterControl->fl_en = pTaskData->m_filterControl.Reg.fl_en;
				pFilterControl->filter_strength = pTaskData->m_filterControl.Reg.filter_strength;
				pFilterControl->start_line = pTaskData->m_filterControl.Reg.start_line;
				pFilterControl->dis_skp = pTaskData->m_filterControl.Reg.dis_skp;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_FILTER_CONTROL start_pixel(%d) en(%d) strength(%d) start_line(%d) dis_skp(%d)\n",
							 pFilterControl->start_pixel,
							 pFilterControl->fl_en,
							 pFilterControl->filter_strength,
							 pFilterControl->start_line,
							 pFilterControl->dis_skp
							 ));

				*pdwSizeGot = sizeof(QPCODEC_FILTER_CONTROL);
				break;
			}

			case QPCODEC_PROP_GOP_LOOP_FILTER:
			{
				PQPCODEC_GOP_LOOP_FILTER	pGopLfltr = (PQPCODEC_GOP_LOOP_FILTER)pOutput;

				if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
				{
					pGopLfltr->Artesa.intra_period = pTaskData->m_gopLoopFilter.RegArtesa.intra_period;
					pGopLfltr->Artesa.lp_filter = pTaskData->m_gopLoopFilter.RegArtesa.lp_filter;
					pGopLfltr->Artesa.dis_lf_idc = pTaskData->m_gopLoopFilter.RegArtesa.dis_lf_idc;
					pGopLfltr->Artesa.alpha_offset = pTaskData->m_gopLoopFilter.RegArtesa.alpha_offset;
					pGopLfltr->Artesa.beta_offset = pTaskData->m_gopLoopFilter.RegArtesa.beta_offset;
					pGopLfltr->Artesa.B_num = pTaskData->m_gopLoopFilter.RegArtesa.B_num;
					pGopLfltr->Artesa.ref_n = pTaskData->m_gopLoopFilter.RegArtesa.ref_n;
					pGopLfltr->Artesa.t8x8 = pTaskData->m_gopLoopFilter.RegArtesa.t8x8;
					pGopLfltr->Artesa.ref_p = pTaskData->m_gopLoopFilter.RegArtesa.ref_p;
					pGopLfltr->Artesa.IDR = pTaskData->m_gopLoopFilter.RegArtesa.IDR;

					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Set() QPCODEC_PROP_GOP_LOOP_FILTER i_period(%d) lp_fltr(%d) dis_lf_idc(%d) \r\nalpha_offset(%d) beta_offset(%d) B_num(%d) ref_n(%d) t8x8(%d) ref_p(%d) IDR(%d)\n",
								 pGopLfltr->Artesa.intra_period,
								 pGopLfltr->Artesa.lp_filter,
								 pGopLfltr->Artesa.dis_lf_idc,
								 pGopLfltr->Artesa.alpha_offset,
								 pGopLfltr->Artesa.beta_offset,
								 pGopLfltr->Artesa.B_num,
								 pGopLfltr->Artesa.ref_n,
								 pGopLfltr->Artesa.t8x8,
								 pGopLfltr->Artesa.ref_p,
								 pGopLfltr->Artesa.IDR
								 ));
				}
				else
				{
					pGopLfltr->Beringer.intra_period = pTaskData->m_gopLoopFilter.Reg.intra_period;
					pGopLfltr->Beringer.lp_filter = pTaskData->m_gopLoopFilter.Reg.lp_filter;
					pGopLfltr->Beringer.dis_lf_idc = pTaskData->m_gopLoopFilter.Reg.dis_lf_idc;
					pGopLfltr->Beringer.alpha_offset = pTaskData->m_gopLoopFilter.Reg.alpha_offset;
					pGopLfltr->Beringer.beta_offset = pTaskData->m_gopLoopFilter.Reg.beta_offset;
					pGopLfltr->Beringer.B_num = pTaskData->m_gopLoopFilter.Reg.B_num;
					pGopLfltr->Beringer.IDR = pTaskData->m_gopLoopFilter.Reg.IDR;

					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Get() QPCODEC_PROP_GOP_LOOP_FILTER i_period(%d) lp_fltr(%d) dis_lf_idc(%d) \r\nalpha_offset(%d) beta_offset(%d) B_num(%d) IDR(%d)\n",
								 pGopLfltr->Beringer.intra_period,
								 pGopLfltr->Beringer.lp_filter,
								 pGopLfltr->Beringer.dis_lf_idc,
								 pGopLfltr->Beringer.alpha_offset,
								 pGopLfltr->Beringer.beta_offset,
								 pGopLfltr->Beringer.B_num,
								 pGopLfltr->Beringer.IDR
								 ));
				}

				*pdwSizeGot = sizeof(QPCODEC_GOP_LOOP_FILTER);
				break;
			}

			case QPCODEC_PROP_ET_CONTROL:
			{
				PQPCODEC_ET_CONTROL	pEtControl = (PQPCODEC_ET_CONTROL)pOutput;

				pEtControl->et_threshold_offset = pTaskData->m_etControl.Reg.et_threshold_offset;
				pEtControl->et_threshold_max = pTaskData->m_etControl.Reg.et_threshold_max;
				pEtControl->et = pTaskData->m_etControl.Reg.et;
				pEtControl->et_sad_threshold = pTaskData->m_etControl.Reg.et_sad_threshold;
				pEtControl->channel_data_type = pTaskData->m_etControl.Reg.data_channel_type;
				pEtControl->ref_num = pTaskData->m_etControl.Reg.ref_num;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_ET_CONTROL threshold_offset(%d) max(%d) et(%d) et_sad(%d)\n",
							 pEtControl->et_threshold_offset,
							 pEtControl->et_threshold_max,
							 pEtControl->et,
							 pEtControl->et_sad_threshold
							 ));

				*pdwSizeGot = sizeof(QPCODEC_ET_CONTROL);
				break;
			}

			case QPCODEC_PROP_OUT_PICTURE_RESOLUTION:
			{
				PQPCODEC_OUT_PICTURE_RESOLUTION	pPictureResoultion = (PQPCODEC_OUT_PICTURE_RESOLUTION)pOutput;

				if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
				{
					pPictureResoultion->out_pic_width = pTaskData->m_outPictureResolution.RegArtesa.out_pic_width;
					pPictureResoultion->out_pic_height = pTaskData->m_outPictureResolution.RegArtesa.out_pic_height;

					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Get() QPCODEC_PROP_OUT_PICTURE_RESOLUTION in_width(%d) in_height(%d)",
								 pPictureResoultion->out_pic_width,
								 pPictureResoultion->out_pic_height
								 ));
					*pdwSizeGot = sizeof(QPCODEC_OUT_PICTURE_RESOLUTION);
				}
				else
				{
					TQP_UINT32	w, h;

					w = pTaskData->m_pictureResolution.Reg.pic_width;
					h = pTaskData->m_pictureResolution.Reg.pic_height;

					switch (pTaskData->m_pictureResolution.Reg.h_scale)
					{
						case FILTER_MODE_1_2:
							if (w == 720)
								w = 352;
							else
								w = w >> 1;
							break;

						case FILTER_MODE_1_4:
							if (w == 720)
								w = 176;
							else
								w = w >> 2;
							break;

						default:
							break;
					}

					switch (pTaskData->m_pictureResolution.Reg.v_scale)
					{
						case FILTER_MODE_1_2:
							h = h >> 1;
							break;

						case FILTER_MODE_1_4:
							if (h == 480)
								h = 144;
							else
								h = h >> 2;
							break;

						default:
							break;
					}

					pPictureResoultion->out_pic_width = w;
					pPictureResoultion->out_pic_height = h;

					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Get() QPCODEC_PROP_OUT_PICTURE_RESOLUTION in_width(%d) in_height(%d)",
								 pPictureResoultion->out_pic_width,
								 pPictureResoultion->out_pic_height
								 ));
					*pdwSizeGot = sizeof(QPCODEC_OUT_PICTURE_RESOLUTION);
				}
				break;
			}

			case QPCODEC_PROP_BLOCK_SIZE:
			{
				PQPCODEC_BLOCK_SIZE	pBlockSize = (PQPCODEC_BLOCK_SIZE)pOutput;

				if (This->m_UseArtesaExt340) {
					pBlockSize->block_size = pTaskData->m_blkXferSize.RegArtesaExt340.block_size;
					pBlockSize->slice_group = pTaskData->m_blkXferSize.RegArtesaExt340.slice_limit;
					pBlockSize->multi_slice_mode = pTaskData->m_blkXferSize.RegArtesaExt340.multi_slice_mode;
					pBlockSize->low_latency_mode = pTaskData->m_blkXferSize.RegArtesaExt340.low_latency_mode;

					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
								 "CQLCodec_Get() QPCODEC_PROP_BLOCK_SIZE block_size(%d)kb slice_group(%d) multi_slice_mode(%d) low_latency_mode(%d)",
								 pBlockSize->block_size,
								 pBlockSize->slice_group,
								 pBlockSize->multi_slice_mode,
								 pBlockSize->low_latency_mode
								 ));
				} else {
					pBlockSize->block_size = pTaskData->m_blkXferSize.RegArtesa.block_size;
					pBlockSize->slice_group = pTaskData->m_blkXferSize.RegArtesa.slice_group;
					pBlockSize->multi_slice_mode = 0;
					pBlockSize->low_latency_mode = 0;

					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
								 "CQLCodec_Get() QPCODEC_PROP_BLOCK_SIZE block_size(%d)kb slice_group(%d)",
								 pBlockSize->block_size,
								 pBlockSize->slice_group
								 ));
				}
				*pdwSizeGot = sizeof(QPCODEC_BLOCK_SIZE);
				break;
			}

			case QPCODEC_PROP_AUDIO_CONTROL_PARAM:
			{
				PQPCODEC_AUDIO_CONTROL_PARAM	pAudCntlParam = (PQPCODEC_AUDIO_CONTROL_PARAM)pOutput;

					pAudCntlParam->bit_rate = pTaskData->m_audioControlParam.Reg.bit_rate;
					pAudCntlParam->ch_sel = pTaskData->m_audioControlParam.Reg.ch_sel;
					pAudCntlParam->sample_rate = pTaskData->m_audioControlParam.Reg.sample_rate;
					pAudCntlParam->aou_pass_en = pTaskData->m_audioControlParam.Reg.aou_pass_en;
					pAudCntlParam->ai_msb = pTaskData->m_audioControlParam.Reg.ai_msb;
					pAudCntlParam->lrclk_i = pTaskData->m_audioControlParam.Reg.lrclk_i;
					pAudCntlParam->bclk_i = pTaskData->m_audioControlParam.Reg.bclk_i;
					pAudCntlParam->ai_i2s = pTaskData->m_audioControlParam.Reg.ai_i2s;
					pAudCntlParam->ai_rj = pTaskData->m_audioControlParam.Reg.ai_rj;
					pAudCntlParam->ai_m = pTaskData->m_audioControlParam.Reg.ai_m;
					pAudCntlParam->ch_num = pTaskData->m_audioControlParam.Reg.ch_num;
					pAudCntlParam->pts = pTaskData->m_audioControlParam.Reg.pts;
					pAudCntlParam->disAi = pTaskData->m_audioControlParam.Reg.disAi;
					pAudCntlParam->raw = pTaskData->m_audioControlParam.Reg.raw;
					pAudCntlParam->audio_type = pTaskData->m_audioControlParam.Reg.audio_type;

					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Get() QPCODEC_PROP_AUDIO_CONTROL_PARAM bit_rate(%d) ch_sel(%d) sample_rate(%d) aou_pass_en(%d) ai_msb(%d) \r\nlrclk_i(%d) bclk_i(%d) ai_i2s(%d) ai_rj(%d) ai_m(%d) ch_num(%d) pts(%d) disAi(%d) raw(%d) audio_type(%d)\n",
								 pAudCntlParam->bit_rate,
								 pAudCntlParam->ch_sel,
								 pAudCntlParam->sample_rate,
								 pAudCntlParam->aou_pass_en,
								 pAudCntlParam->ai_msb,
								 pAudCntlParam->lrclk_i,
								 pAudCntlParam->bclk_i,
								 pAudCntlParam->ai_i2s,
								 pAudCntlParam->ai_rj,
								 pAudCntlParam->ai_m,
								 pAudCntlParam->ch_num,
								 pAudCntlParam->pts,
								 pAudCntlParam->disAi,
								 pAudCntlParam->raw,
								 pAudCntlParam->audio_type
								 ));

				*pdwSizeGot = sizeof(QPCODEC_AUDIO_CONTROL_PARAM);
				break;
			}

			case QPCODEC_PROP_AUDIO_CONTROL_EX:
			{
				PQPCODEC_AUDIO_CONTROL_EX	pAudCntlEx = (PQPCODEC_AUDIO_CONTROL_EX)pOutput;

				pAudCntlEx->aac_ver = pTaskData->m_audioControlExAAC.Reg.aac_ver;
				pAudCntlEx->h_type = pTaskData->m_audioControlExAAC.Reg.h_type;
				pAudCntlEx->TNS = pTaskData->m_audioControlExAAC.Reg.TNS;
				pAudCntlEx->IS = pTaskData->m_audioControlExAAC.Reg.IS;
				pAudCntlEx->PNS = pTaskData->m_audioControlExAAC.Reg.PNS;
				pAudCntlEx->MS = pTaskData->m_audioControlExAAC.Reg.MS;
				pAudCntlEx->cutoff_freq = pTaskData->m_audioControlExAAC.Reg.cutoff_freq;

				pAudCntlEx->mode = pTaskData->m_audioControlExG711.Reg.mode;
				pAudCntlEx->block_size = pTaskData->m_audioControlExG711.Reg.block_size;

				pAudCntlEx->sample_cnt = pTaskData->m_audioControlExLPCM.Reg.sample_cnt;
				pAudCntlEx->raw_audio_type = pTaskData->m_audioControlExLPCM.Reg.raw_type;

				pAudCntlEx->frame_len = pTaskData->m_audioControlExSILK.Reg.frame_len;
				pAudCntlEx->downsample = pTaskData->m_audioControlExLPCM.Reg.downsample;
				pAudCntlEx->lr_swap = pTaskData->m_audioControlExLPCM.Reg.lr_swap;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_AUDIO_CONTROL_EX aac_ver(%d) h_type(%d) TNS(%d) \r\nIS(%d) PNS(%d) MS(%d) cutoff(%d)\r\n mode(%d) block_size(%d) sample_cnt(%d) frame_len(%d) downsample(%d) lr_swap(%d)\r\n",
							 pAudCntlEx->aac_ver,
							 pAudCntlEx->h_type,
							 pAudCntlEx->TNS,
							 pAudCntlEx->IS,
							 pAudCntlEx->PNS,
							 pAudCntlEx->MS,
							 pAudCntlEx->cutoff_freq,
							 pAudCntlEx->mode,
							 pAudCntlEx->block_size,
							 pAudCntlEx->sample_cnt,
							 pAudCntlEx->frame_len,
							 pAudCntlEx->downsample,
							 pAudCntlEx->lr_swap
							 ));

				*pdwSizeGot = sizeof(QPCODEC_AUDIO_CONTROL_EX);
				break;
			}

			case QPCODEC_PROP_VIOSD_TABLE_ADDR:
			{
				PQPCODEC_VIOSD	pFontTxtOut = (PQPCODEC_VIOSD)pOutput;

				if (!pFontTxtOut)
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
								 "CQLCodec_Get() QPCODEC_PROP_VIOSD_TABLE_ADDR NULL_PTR!! pOutput(0x%x)\n",
								 pOutput
								 ));
					qpStatus = QPERR_PARMS;
				}
				else
				{
					qpStatus = QPFWENCAPI_GetViosdTableaddr(This,
															   (TQP_UINT32) hIndex,
															   &pFontTxtOut->FontTableAddr,
															   &pFontTxtOut->TextListAddr,
															   &pFontTxtOut->TimestampAddr
															   );

					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Get() QPCODEC_PROP_VIOSD_TABLE_ADDR font_addr(0x%x) txtAddr(0x%x) timeAddr(0x%x) status(%d)\n",
								 pFontTxtOut->FontTableAddr,
								 pFontTxtOut->TextListAddr,
								 pFontTxtOut->TimestampAddr,
								 qpStatus
								 ));
				}

				*pdwSizeGot = QP_SUCCESS(qpStatus) ? sizeof(QPCODEC_VIOSD) : 0;
				break;
			}

			case QPCODEC_PROP_GET_CUR_VID_BUF_INFO:
			{
				PQPCODEC_GET_CUR_VID_BUF_INFO	pCurVidBufIn = (PQPCODEC_GET_CUR_VID_BUF_INFO)pInput;
				PQPCODEC_GET_CUR_VID_BUF_INFO	pCurVidBufOut = (PQPCODEC_GET_CUR_VID_BUF_INFO)pOutput;

				if (!pCurVidBufIn ||
					!pCurVidBufOut
					)
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
								 "CQLCodec_Get() QPCODEC_PROP_GET_CUR_VID_BUF_INFO NULL_PTR!! pInput(0x%x) pOutput(0x%x)\n",
								 pInput,
								 pOutput
								 ));
					qpStatus = QPERR_PARMS;
				}
				else
				{
					qpStatus = QPFWENCAPI_GetCurVidBufInfo(This,
														   (TQP_UINT32) hIndex,
														   pCurVidBufIn->HoldBuffer,
														   &pCurVidBufOut->VidBufLumaAddr,
														   &pCurVidBufOut->VidBufChromaAddr,
														   &pCurVidBufOut->FrameLABufAddr,
														   &pCurVidBufOut->TopLABufAddr,
														   &pCurVidBufOut->BottomLABufAddr,
														   &pCurVidBufOut->VidBufPTS
														   );

					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Get() QPCODEC_PROP_GET_CUR_VID_BUF_INFO hold_buf(%d) LumaAddr(0x%x) chroma_addr(0x%x) frame_la_addr(0x%x) top_la_addr(0x%x) bot_la_addr(0x%x) pts(%d) status(%d)\n",
								 pCurVidBufIn->HoldBuffer,
								 pCurVidBufOut->VidBufLumaAddr,
								 pCurVidBufOut->VidBufChromaAddr,
								 pCurVidBufOut->FrameLABufAddr,
								 pCurVidBufOut->TopLABufAddr,
								 pCurVidBufOut->BottomLABufAddr,
								 pCurVidBufOut->VidBufPTS,
								 qpStatus
								 ));
				}

				*pdwSizeGot = QP_SUCCESS(qpStatus) ? sizeof(QPCODEC_GET_CUR_VID_BUF_INFO) : 0;
				break;
			}

			case QPCODEC_PROP_MP4_VID_NUM_BLOCK:
			{
				PQPCODEC_MP4_VID_NUM_BLOCK	pMP4VidBlockNumber = (PQPCODEC_MP4_VID_NUM_BLOCK)pOutput;

				pMP4VidBlockNumber->num = pTaskData->m_MP4VideoBlockNumber;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_MP4_VID_NUM_BLOCK num(%d)\n",
							 pMP4VidBlockNumber->num
							 ));

				*pdwSizeGot = sizeof(QPCODEC_MP4_VID_NUM_BLOCK);
				break;
			}

			case QPCODEC_PROP_AUDIO_ENHANCEMENT:
			{
				PQPCODEC_AUDIO_ENHANCEMENT	pAudEnhancement = (PQPCODEC_AUDIO_ENHANCEMENT)pOutput;

				pAudEnhancement->gain1 = pTaskData->m_audioEnhancement.gain1;
				pAudEnhancement->gain2 = pTaskData->m_audioEnhancement.gain2;
				pAudEnhancement->add = pTaskData->m_audioEnhancement.add;
				pAudEnhancement->sub = pTaskData->m_audioEnhancement.sub;
				pAudEnhancement->att1 = pTaskData->m_audioEnhancement.att1;
				pAudEnhancement->att2 = pTaskData->m_audioEnhancement.att2;
				pAudEnhancement->lgain = pTaskData->m_audioEnhancement.lgain;
				pAudEnhancement->rgain = pTaskData->m_audioEnhancement.rgain;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_AUDIO_ENHANCEMENT gain1(%d) gain2(%d) add(%d) sub(%d) att1(%d) att2(%d) lgain(%d) rgain(%d)\n",
							 pAudEnhancement->gain1,
							 pAudEnhancement->gain2,
							 pAudEnhancement->add,
							 pAudEnhancement->sub,
							 pAudEnhancement->att1,
							 pAudEnhancement->att2,
							 pAudEnhancement->lgain,
							 pAudEnhancement->rgain
							 ));

				*pdwSizeGot = sizeof(QPCODEC_AUDIO_ENHANCEMENT);
				break;
			}

			case QPCODEC_PROP_ENC_STOP_MODE:
			{
				PQPCODEC_ENC_STOP_MODE	pStopMode = (PQPCODEC_ENC_STOP_MODE)pOutput;

				pStopMode->stop_mode = pTaskData->m_EncStopMode;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_ENC_STOP_MODE mode(%d)\n",
							 pStopMode->stop_mode
							 ));

				*pdwSizeGot = sizeof(QPCODEC_ENC_STOP_MODE);
				break;
			}


			case QPCODEC_PROP_INDEX_CAP_FREQ:
			{
				PQPCODEC_INDEX_CAP_FREQ	pIndexFreq = (PQPCODEC_INDEX_CAP_FREQ)pOutput;

				pIndexFreq->freq = pTaskData->m_EncIndexCapFreq;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_INDEX_CAP_FREQ freq(%d)\n",
							 pIndexFreq->freq
							 ));

				*pdwSizeGot = sizeof(QPCODEC_INDEX_CAP_FREQ);
				break;
			}

			case QPCODEC_PROP_ENC_VID_PADDING:
			{
				PQPCODEC_ENC_VID_PADDING	pVidPadding = (PQPCODEC_ENC_VID_PADDING)pOutput;

				pVidPadding->Enable = pTaskData->m_bEncEnableVidPadding;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_ENC_VID_PADDING Enable(%d)\n",
							 pVidPadding->Enable
							 ));

				*pdwSizeGot = sizeof(QPCODEC_ENC_VID_PADDING);
				break;
			}

			case QPCODEC_PROP_ENC_FREEZE_VIDEO:
			{
				PQPCODEC_ENC_FREEZE_VIDEO	pFreezeVid = (PQPCODEC_ENC_FREEZE_VIDEO)pOutput;

				pFreezeVid->Enable = pTaskData->m_bEncVidFrozen;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_ENC_VID_PADDING Enable(%d)\n",
							 pFreezeVid->Enable
							 ));

				*pdwSizeGot = sizeof(QPCODEC_ENC_FREEZE_VIDEO);
				break;
			}

			case QPCODEC_PROP_ENC_STILL_VIDEO_INPUT:
			{
				PQPCODEC_ENC_STILL_VIDEO_INPUT	pStillVid = (PQPCODEC_ENC_STILL_VIDEO_INPUT)pOutput;

				pStillVid->Enable = pTaskData->m_bEncVidStillInput;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_ENC_STILL_VIDEO_INPUT Enable(%d)\n",
							 pStillVid->Enable
							 ));

				*pdwSizeGot = sizeof(QPCODEC_ENC_STILL_VIDEO_INPUT);
				break;
			}

			case QPCODEC_PROP_ENC_CAP_MODE:
			{
				PQPCODEC_ENC_CAP_MODE	pCapMode = (PQPCODEC_ENC_CAP_MODE)pOutput;

				pCapMode->capMode = pTaskData->m_EncCapMode.capMode;
				pCapMode->trigMode = pTaskData->m_EncCapMode.trigMode;
				pCapMode->gpioPin = pTaskData->m_EncCapMode.gpioPin;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_ENC_CAP_MODE capMode(%d) trigMode(0x%x) gpioPin(%d)\n",
							 pCapMode->capMode,
							 pCapMode->trigMode,
							 pCapMode->gpioPin
							 ));

				*pdwSizeGot = sizeof(QPCODEC_ENC_CAP_MODE);
				break;
			}

			case QPCODEC_PROP_ENC_MJPEG_QUALITY:
			{
				PTQP_UINT32	pQuality = (PTQP_UINT32)pOutput;

				*pQuality = pTaskData->m_EncMjpegQuality;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodec_Get() QPCODEC_PROP_ENC_MJPEG_QUALITY Quality(%d)\n",
							 *pQuality
							 ));

				*pdwSizeGot = sizeof(TQP_UINT32);
				break;
			}

			case QPCODEC_PROP_ENC_MJPEG_FRAME_BUF:
			{
				PTQP_UINT32	pFrameBufNb = (PTQP_UINT32)pOutput;

				*pFrameBufNb = pTaskData->m_EncMjpegFrameBuffer;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodec_Get() QPCODEC_PROP_ENC_MJPEG_FRAME_BUF nb(%d)\n",
							 *pFrameBufNb
							 ));

				*pdwSizeGot = sizeof(TQP_UINT32);
				break;
			}

			case QPCODEC_PROP_EXTERNAL_TRIGGER_TO_SYNC:
			{
				PQPCODEC_EXTERNAL_TRIGGER_MODE pExternalTriggerToSync = (PQPCODEC_EXTERNAL_TRIGGER_MODE)pOutput;

				pExternalTriggerToSync->Enable = pTaskData->m_ExternalTriggerToSync.Enable;
				pExternalTriggerToSync->gpioPin = pTaskData->m_ExternalTriggerToSync.gpioPin;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodec_Get() QPCODEC_PROP_EXTERNAL_TRIGGER_TO_SYNC Enable(%d) gpioPin(%d)\n",
							 pExternalTriggerToSync->Enable,
							 pExternalTriggerToSync->gpioPin
							 ));

				*pdwSizeGot = sizeof(QPCODEC_PROP_EXTERNAL_TRIGGER_TO_SYNC);
				break;
			}

			case QPCODEC_PROP_PTS_COUNTER_RESET:
			{
				PQPCODEC_PTS_COUNTER_RESET pPTSCounterReset = (PQPCODEC_PTS_COUNTER_RESET)pOutput;

				pPTSCounterReset->Enable = pTaskData->m_PTSCounterReset.Enable;
				pPTSCounterReset->gpioPin = pTaskData->m_PTSCounterReset.gpioPin;
				pPTSCounterReset->immediate = pTaskData->m_PTSCounterReset.immediate;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodec_Get() PQPCODEC_PTS_COUNTER_RESET Enable(%d) gpioPin(%d) immediate(%d)\n",
							 pPTSCounterReset->Enable,
							 pPTSCounterReset->gpioPin,
							 pPTSCounterReset->immediate
							 ));

				*pdwSizeGot = sizeof(PQPCODEC_PTS_COUNTER_RESET);
				break;
			}

			case QPCODEC_PROP_DECIMATION:
			{
				PQPCODEC_DECIMATION_FACTOR pDecimationFactor = (PQPCODEC_DECIMATION_FACTOR)pOutput;

				pDecimationFactor->input_format = pTaskData->m_RawVideoDecimationFactor.input_format;
				pDecimationFactor->output_format = pTaskData->m_RawVideoDecimationFactor.output_format;
				pDecimationFactor->scale_factor.read = pTaskData->m_RawVideoDecimationFactor.scale_factor.read;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodec_Get() QPCODEC_PROP_DECIMATION input_format(%d) output_format(%d) h_numerator(%d) h_denominator(%d) v_numerator(%d) v_denominator(%d)\n",
							 pDecimationFactor->input_format,
							 pDecimationFactor->output_format,
							 pDecimationFactor->scale_factor.factor.h_numerator,
							 pDecimationFactor->scale_factor.factor.h_denominator,
							 pDecimationFactor->scale_factor.factor.v_numerator,
							 pDecimationFactor->scale_factor.factor.v_denominator
							 ));

				*pdwSizeGot = sizeof(QPCODEC_DECIMATION_FACTOR);

				break;
			}

			case QPCODEC_PROP_DEINTERLACE:
			{
				PTQP_UINT32	pMode = (PTQP_UINT32)pOutput;

				*pMode = pTaskData->m_DeinterlaceMode;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodec_Get() QPCODEC_PROP_DEINTERLACE mode(%d)\n",
							 *pMode
							 ));

				*pdwSizeGot = sizeof(TQP_UINT32);
				break;
			}

			case QPCODEC_PROP_LARGE_COMPRESS_BUFFER_CONTROL:
			{
				PTQP_UINT32	pControl = (PTQP_UINT32)pOutput;

				*pControl = pTaskData->m_LargeCompressBufferControl;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodec_Get() QPCODEC_PROP_LARGE_COMPRESS_BUFFER_CONTROL (0x%08x)\n",
							 *pControl
							 ));

				*pdwSizeGot = sizeof(TQP_UINT32);
				break;
			}

			case QPCODEC_PROP_ENABLE_LOW_BITRATE_MODE:
			{
				PTQP_UINT32	pControl = (PTQP_UINT32)pOutput;

				*pControl = pTaskData->m_EnableLowBitrateMode;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodec_Get() QPCODEC_PROP_ENABLE_LOW_BITRATE_MODE (0x%08x)\n",
							 *pControl
							 ));

				*pdwSizeGot = sizeof(TQP_UINT32);
				break;
			}

			case QPCODEC_PROP_ENABLE_AV_DISCARD_CONTROL:
			{
				PTQP_UINT32	pControl = (PTQP_UINT32)pOutput;

				*pControl = pTaskData->m_EnableAVDiscardControl;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodec_Get() QPCODEC_PROP_ENABLE_AV_DISCARD_CONTROL (0x%08x)\n",
							 *pControl
							 ));

				*pdwSizeGot = sizeof(TQP_UINT32);
				break;
			}

			case QPCODEC_PROP_USE_SW_PTS:
			{
				PTQP_UINT32	pControl = (PTQP_UINT32)pOutput;

				*pControl = pTaskData->m_UseSWPTS;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodec_Get() QPCODEC_PROP_USE_SW_PTS (0x%08x)\n",
							 *pControl
							 ));

				*pdwSizeGot = sizeof(TQP_UINT32);
				break;
			}


			case QPCODEC_PROP_VIU_SYN_CCODE_1: // [H.P.LIN] peter
			{
				PTQP_UINT32	pControl = (PTQP_UINT32)pOutput;

				*pControl = This->m_ulViuSyncCode1;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodec_Get() QPCODEC_PROP_VIU_SYN_CCODE_1 (0x%08x)\n",
							 *pControl
							 ));

				*pdwSizeGot = sizeof(TQP_UINT32);
				break;
			}

			case QPCODEC_PROP_VIU_SYN_CCODE_2: // [H.P.LIN] peter
			{
				PTQP_UINT32	pControl = (PTQP_UINT32)pOutput;

				*pControl = This->m_ulViuSyncCode2;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodec_Get() QPCODEC_PROP_VIU_SYN_CCODE_2 (0x%08x)\n",
							 *pControl
							 ));

				*pdwSizeGot = sizeof(TQP_UINT32);
				break;
			}

			case QPCODEC_PROP_ENC_ALL_SETTINGS:
			{
				PQPCODEC_ENC_ALL_SETTINGS	pAll = (PQPCODEC_ENC_ALL_SETTINGS)pOutput;

				pAll->EncSystemControl = pTaskData->m_systemControl.Read;
				pAll->EncPictureResolution = pTaskData->m_pictureResolution.Read;
				pAll->EncInputControl = pTaskData->m_inputControl.Read;
				pAll->EncSyncMode = pTaskData->m_syncMode;
				pAll->EncRateControl = pTaskData->m_rateControl.Read;
				pAll->EncRateControlEx = pTaskData->m_rateControlEx.Read;
				pAll->EncBitRate = pTaskData->m_bitRate.Read;
				pAll->EncFilterControl = pTaskData->m_filterControl.Read;
				pAll->EncGopLoopFilter = pTaskData->m_gopLoopFilter.Read;
				pAll->EncEtControl = pTaskData->m_etControl.Read;
				pAll->EncOutPictureResolution = pTaskData->m_outPictureResolution.Read;
				pAll->EncBlockXferSize = pTaskData->m_blkXferSize.Read;
				pAll->EncAudioControlParam = pTaskData->m_audioControlParam.Read;
				pAll->EncAudioControlExAAC = pTaskData->m_audioControlExAAC.Read;
				pAll->EncAudioControlExG711 = pTaskData->m_audioControlExG711.Read;
				pAll->EncAudioControlExLPCM = pTaskData->m_audioControlExLPCM.Read;
				pAll->EncIndexCapFreq = pTaskData->m_EncIndexCapFreq;
				pAll->EncMP4VideoBlockNumber = pTaskData->m_MP4VideoBlockNumber;
				pAll->EncStopMode = pTaskData->m_EncStopMode;
				pAll->EncEnableVidPadding = pTaskData->m_bEncEnableVidPadding;
				pAll->EncMjpegQuality = pTaskData->m_EncMjpegQuality;
				pAll->EncMjpegFrameBuffer = pTaskData->m_EncMjpegFrameBuffer;
				pAll->EncExternalTriggerToSyncEnable =pTaskData->m_ExternalTriggerToSync.Enable;
				pAll->EncExternalTriggerToSyncGpioPin = pTaskData->m_ExternalTriggerToSync.gpioPin;
				pAll->EncPTSResetByTriggerEnable = pTaskData->m_PTSCounterReset.Enable;
				pAll->EncPTSResetByTriggerGpioPin = pTaskData->m_PTSCounterReset.gpioPin;
				pAll->EncPTSResetByTriggerImmediate = pTaskData->m_PTSCounterReset.immediate;
				pAll->EncDecimationInputFormat = pTaskData->m_RawVideoDecimationFactor.input_format;
				pAll->EncDecimationOutputFormat = pTaskData->m_RawVideoDecimationFactor.output_format;
				pAll->EncDecimationScaleFactor = pTaskData->m_RawVideoDecimationFactor.scale_factor.read;

				pAll->DeinterlaceMode = pTaskData->m_DeinterlaceMode;
				pAll->EncAudioControlExSILK = pTaskData->m_audioControlExSILK.Read;

				pAll->LargeCompressBufferControl = pTaskData->m_LargeCompressBufferControl;
				pAll->EnableLowBitrateMode = pTaskData->m_EnableLowBitrateMode;

				pAll->EnableAVDiscardControl = pTaskData->m_EnableAVDiscardControl;
				pAll->UseSWPTS = pTaskData->m_UseSWPTS;

				*pdwSizeGot = sizeof(QPCODEC_ENC_ALL_SETTINGS);
				break;
			}

			// Decoder
			//
			case QPCODEC_PROP_DEC_STREAM_ID:
			{
				PTQP_UINT32	pStreamID = (PTQP_UINT32)pOutput;

				*pStreamID = pTaskData->m_streamID.Reg.stream_type;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_DEC_STREAM_ID stream_id(%d)\n",
							 *pStreamID
							 ));

				*pdwSizeGot = sizeof(TQP_UINT32);
				break;
			}

			case QPCODEC_PROP_DEC_OUTPUT_MODE:
			{
				PTQP_UINT32	pOutputMode = (PTQP_UINT32)pOutput;

				*pOutputMode = pTaskData->m_outputMode;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_DEC_OUTPUT_MODE outputMode(0x%x)\n",
							 *pOutputMode
							 ));

				*pdwSizeGot = sizeof(TQP_UINT32);
				break;
			}

			case QPCODEC_PROP_DEC_RESOLUTION:
			{
				PQPCODEC_PICTURE_RESOLUTION	pPicRes = (PQPCODEC_PICTURE_RESOLUTION)pOutput;

				if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
				{
					pPicRes->Artesa.in_pic_height = pTaskData->m_decodeResolution.Reg.pic_height;
					pPicRes->Artesa.in_pic_width = pTaskData->m_decodeResolution.Reg.pic_width;
				}
				else
				{
					pPicRes->Beringer.pic_height = pTaskData->m_decodeResolution.Reg.pic_height;
					pPicRes->Beringer.pic_width = pTaskData->m_decodeResolution.Reg.pic_width;
				}

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_DEC_RESOLUTION (%d x %d)\n",
							 pPicRes->Beringer.pic_height,
							 pPicRes->Beringer.pic_width
							 ));

				*pdwSizeGot = sizeof(QPCODEC_PICTURE_RESOLUTION);
				break;
			}

			case QPCODEC_PROP_DEC_STOP_MODE:
			{
				PQPCODEC_DEC_STOP_MODE	pStopMode = (PQPCODEC_DEC_STOP_MODE)pOutput;

				pStopMode->stop_mode = pTaskData->m_decStopMode;
				pStopMode->stop_disp_mode = pTaskData->m_decStopDispMode;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_DEC_STOP_MODE stop_mode(%d) disp_mode(%d)\n",
							 pStopMode->stop_mode,
							 pStopMode->stop_disp_mode
							 ));

				*pdwSizeGot = sizeof(QPCODEC_DEC_STOP_MODE);
				break;
			}

			case QPCODEC_PROP_DEC_TRICK_MODE:
			{
				PQPCODEC_DEC_TRICK_MODE	pTrickMode = (PQPCODEC_DEC_TRICK_MODE)pOutput;

				pTrickMode->playSpeed = pTaskData->m_decTrickPlaySpeed;
				pTrickMode->playDirection = pTaskData->m_decTrickPlayDirection;
				pTrickMode->fieldPreference = pTaskData->m_decTrickFieldPreference;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_DEC_TRICK_MODE spd(%d) dir(%d) field_pref(%d)\n",
							 pTrickMode->playSpeed,
							 pTrickMode->playDirection,
							 pTrickMode->fieldPreference
							 ));

				*pdwSizeGot = sizeof(QPCODEC_DEC_TRICK_MODE);
				break;
			}


			case QPCODEC_PROP_DEC_DISP_ATTRIB:
			{
				PQPCODEC_DEC_DISP_ATTRIB	pDispAttrib = (PQPCODEC_DEC_DISP_ATTRIB)pOutput;

				pDispAttrib->TVSystem = pTaskData->m_decDispTVSystem;
				pDispAttrib->ForceRemote = pTaskData->m_decDispForceRemote;
				pDispAttrib->inputSize.Read = pTaskData->m_decDispInputSize.Read;
				pDispAttrib->outputSize.Read = pTaskData->m_decDispOutputSize.Read;
				pDispAttrib->inputOrigin.Read = pTaskData->m_decDispInputOrigin.Read;
				pDispAttrib->outputOrigin.Read = pTaskData->m_decDispOutputOrigin.Read;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_DEC_DISP_ATTRIB tv_sys(%d) in_size(0x%x) out_size(0x%x) in_orig(0x%x) out_orig(0x%x) ForceRemote(%d)\n",
							 pDispAttrib->TVSystem,
							 pDispAttrib->inputSize.Read,
							 pDispAttrib->outputSize.Read,
							 pDispAttrib->inputOrigin.Read,
							 pDispAttrib->outputOrigin.Read,
							 pDispAttrib->ForceRemote
							 ));

				*pdwSizeGot = sizeof(QPCODEC_DEC_DISP_ATTRIB);
				break;
			}

			case QPCODEC_PROP_DEC_DISP_MODE:
			{
				PQPCODEC_DEC_DISP_MODE	pDispMode = (PQPCODEC_DEC_DISP_MODE)pOutput;

				pDispMode->displayMode = pTaskData->m_decDispMode;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_DEC_DISP_MODE displayMode(%d)\n",
							 pDispMode->displayMode
							 ));

				*pdwSizeGot = sizeof(QPCODEC_DEC_DISP_MODE);
				break;
			}

			case QPCODEC_PROP_DEC_GET_PLAY_INFO:
			{
				PQPCODEC_DEC_GET_PLAY_INFO	pPlayInfoIn = (PQPCODEC_DEC_GET_PLAY_INFO)pInput;
				PQPCODEC_DEC_GET_PLAY_INFO	pPlayInfoOut = (PQPCODEC_DEC_GET_PLAY_INFO)pOutput;

				if (!pPlayInfoIn ||
					!pPlayInfoOut
					)
				{
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
								 "CQLCodec_Get() QPCODEC_PROP_DEC_GET_PLAY_INFO NULL PTR!! pInput(0x%x) pOutput(0x%x)\n",
								 pInput,
								 pOutput
								 ));
					qpStatus = QPERR_PARMS;
				}
				else
				{
					qpStatus = QPFWDECAPI_GetPlayInfo(This,
													  (TQP_UINT32) hIndex,
													  pPlayInfoIn->infoType
													  );

					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Get() QPCODEC_PROP_DEC_GET_PLAY_INFO type(%d) status(%d)\n",
								 pPlayInfoIn->infoType,
								 qpStatus
								 ));
				}

				if (QP_SUCCESS(qpStatus))
				{
					switch (pPlayInfoIn->infoType)
					{
						case QPDEC_INFO_TYPE_GET_SHARE_MEM:
							pPlayInfoOut->INFO.GET_SHARE_MEM.wrMem = pTaskData->m_decFWSharedMemWr;
							pPlayInfoOut->INFO.GET_SHARE_MEM.rdMem = pTaskData->m_decFWSharedMemRd;
							break;

						default:
							pPlayInfoOut->INFO.ALL.param1 = 0;
							pPlayInfoOut->INFO.ALL.param2 = 0;
							break;
					}

					*pdwSizeGot = sizeof(QPCODEC_DEC_GET_PLAY_INFO);
				}
				else
				{
					*pdwSizeGot = 0;
				}
				break;
			}

			case QPCODEC_PROP_DEC_AUDIO_CONTROL:
			{
				PQPCODEC_DEC_AUDIO_CONTROL	pAudControls = (PQPCODEC_DEC_AUDIO_CONTROL)pOutput;

				pAudControls->audio_type = This->m_AudControls.Reg.audio_type;
				pAudControls->ch_num = This->m_AudControls.Reg.ch_num;
				pAudControls->sample_rate = This->m_AudControls.Reg.sample_rate;
				pAudControls->mode = This->m_AudControls.Reg.mode;
				pAudControls->upsample = This->m_AudControls.Reg.upsample;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_DEC_AUDIO_CONTROL audio_type(%d) ch_num(%d) sample_rate(%d) mode(%d) upsample(%d)\n",
							 pAudControls->audio_type,
							 pAudControls->ch_num,
							 pAudControls->sample_rate,
							 pAudControls->mode,
							 pAudControls->upsample
							 ));
				*pdwSizeGot = sizeof(QPCODEC_DEC_AUDIO_CONTROL);
				break;
			}

			case QPCODEC_PROP_DEC_DATA_XFER_CONTROL:
			{
				PQPCODEC_DEC_DATA_XFER_CONTROL	pXferMode = (PQPCODEC_DEC_DATA_XFER_CONTROL)pOutput;

				pXferMode->xfer_mode = pTaskData->m_decXferMode;
				pXferMode->minimum_size = pTaskData->m_decXferMinimumSize;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_DEC_DATA_XFER_CONTROL xfer_mode(%d) minimum_size(%d)\n",
							 pXferMode->xfer_mode,
							 pXferMode->minimum_size
							 ));

				*pdwSizeGot = sizeof(QPCODEC_DEC_DATA_XFER_CONTROL);
				break;
			}

			case QPCODEC_PROP_DEC_ALL_SETTINGS:
			{
				PQPCODEC_DEC_ALL_SETTINGS	pAll = (PQPCODEC_DEC_ALL_SETTINGS)pOutput;

				pAll->streamID = pTaskData->m_streamID.Read;
				pAll->outputMode = pTaskData->m_outputMode;
				pAll->decodeResolution = pTaskData->m_decodeResolution.Read;
				pAll->decStopMode = pTaskData->m_decStopMode;
  				pAll->decStopDispMode = pTaskData->m_decStopDispMode;
  				pAll->decDispMode = pTaskData->m_decDispMode;
  				pAll->decDispTVSystem = pTaskData->m_decDispTVSystem;
  				pAll->decDispForceRemote = pTaskData->m_decDispForceRemote;
  				pAll->decDispInputSize = pTaskData->m_decDispInputSize.Read;
  				pAll->decDispOutputSize = pTaskData->m_decDispOutputSize.Read;
  				pAll->decDispInputOrigin = pTaskData->m_decDispInputOrigin.Read;
  				pAll->decDispOutputOrigin = pTaskData->m_decDispOutputOrigin.Read;
				pAll->decXferMode = pTaskData->m_decXferMode;
				pAll->decXferMinimumSize = pTaskData->m_decXferMinimumSize;

				*pdwSizeGot = sizeof(QPCODEC_DEC_ALL_SETTINGS);
				break;
			}

			case QPCODEC_PROP_ENCODER_STATUS:
				break;

			case QPCODEC_PROP_DECODER_STATUS:
				break;


			case QPCODEC_PROP_656_OUT_CNTL:
			{
				PQPCODEC_OUT_CNTL	pOutCntl = (PQPCODEC_OUT_CNTL)pOutput;
				TQP_UINT32			reg50;

				qpStatus = QPHCI_RegisterRead(&This->m_hci,
											  0x50,
											  &reg50
											  );
				if (QP_SUCCESS(qpStatus))
				{
					pOutCntl->Enable = (0 == (reg50 & 0x4));
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Get() QPCODEC_PROP_656_OUT_CNTL (%d)\n",
								 pOutCntl->Enable
								 ));
					*pdwSizeGot = sizeof(QPCODEC_OUT_CNTL);
				}
				break;
			}

			case QPCODEC_PROP_I2S_OUT_CNTL:
			{
				PQPCODEC_OUT_CNTL	pOutCntl = (PQPCODEC_OUT_CNTL)pOutput;
				TQP_UINT32			reg50;

				qpStatus = QPHCI_RegisterRead(&This->m_hci,
											  0x50,
											  &reg50
											  );
				if (QP_SUCCESS(qpStatus))
				{
					pOutCntl->Enable = (0 == (reg50 & 0x2));
					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
								 "CQLCodec_Get() QPCODEC_PROP_I2S_OUT_CNTL (%d)\n",
								 pOutCntl->Enable
								 ));
					*pdwSizeGot = sizeof(QPCODEC_OUT_CNTL);
				}
				break;
			}

			case QPCODEC_PROP_VOU_CONFIG:
			{
				PQPCODEC_VOU_CONFIG	pVOUConfig = (PQPCODEC_VOU_CONFIG)pOutput;

				pVOUConfig->VOUMode = This->m_VOUMode;
				pVOUConfig->VOUStartPixel = This->m_VOUStartPixel;
				pVOUConfig->VOUStartLine = This->m_VOUStartLine;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_VOU_CONFIG VOUMode(%d) start_pixel(%d) start_line(%d)\n",
							 pVOUConfig->VOUMode,
							 pVOUConfig->VOUStartPixel,
							 pVOUConfig->VOUStartLine
							 ));
				*pdwSizeGot = sizeof(QPCODEC_VOU_CONFIG);
				break;
			}

			case QPCODEC_PROP_AOU_CONFIG:
			{
				PQPCODEC_AOU_CONFIG	pAOUConfig = (PQPCODEC_AOU_CONFIG)pOutput;

				pAOUConfig->ao_en = This->m_AOControls.Reg.ao_en;
				pAOUConfig->ao_msb = This->m_AOControls.Reg.ao_msb;
				pAOUConfig->lrclk_i = This->m_AOControls.Reg.lrclk_i;
				pAOUConfig->bclk_i = This->m_AOControls.Reg.bclk_i;
				pAOUConfig->ao_i2s = This->m_AOControls.Reg.ao_i2s;
				pAOUConfig->ao_rj = This->m_AOControls.Reg.ao_rj;
				pAOUConfig->ao_s = This->m_AOControls.Reg.ao_s;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_AOU_CONFIG ao_en(%d) ao_msb(%d) lrclk_i(%d) bclk_i(%d) ao_i2s(%d) ao_rj(%d) ao_s(%d)\n",
							 pAOUConfig->ao_en,
							 pAOUConfig->ao_msb,
							 pAOUConfig->lrclk_i,
							 pAOUConfig->bclk_i,
							 pAOUConfig->ao_i2s,
							 pAOUConfig->ao_rj,
							 pAOUConfig->ao_s
							 ));
				*pdwSizeGot = sizeof(QPCODEC_AOU_CONFIG);
				break;
			}

			case QPCODEC_PROP_POWER_STATE:
			{
				PQPCODEC_POWER_STATE	pPowerState = (PQPCODEC_POWER_STATE)pOutput;

				pPowerState->On = (QLCODEC_PWR_STATE_D0 == This->m_PowerState);

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_POWER_STATE on(%d) power state(%d)\n",
							 pPowerState->On,
							 This->m_PowerState
							 ));
				*pdwSizeGot = sizeof(QPCODEC_POWER_STATE);
				break;
			}

			case QPCODEC_PROP_ERROR_RECOVERY:
			{
				PQPCODEC_ERROR_RECOVERY	pErrorRecovery = (PQPCODEC_ERROR_RECOVERY)pOutput;

				pErrorRecovery->Level = This->m_ErrorRecovery;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_ERROR_RECOVERY level(%d)\n",
							 pErrorRecovery->Level
							 ));
				*pdwSizeGot = sizeof(QPCODEC_ERROR_RECOVERY);
				break;
			}

			case QPCODEC_PROP_USE_ARTESA_EXT_340:
			{
				PTQP_UINT32	pUseArtesaExt340 = (PTQP_UINT32)pOutput;

				*pUseArtesaExt340 = This->m_UseArtesaExt340;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_USE_ARTESA_EXT_340 (%d)\n",
							 *pUseArtesaExt340
							 ));
				*pdwSizeGot = sizeof(TQP_UINT32);
				break;
			}

			case QPCODEC_PROP_FILTER_OUT_HDCP_CONTENT:
			{
				PTQP_UINT32	pFilterOutHDCPContent = (PTQP_UINT32)pOutput;

				*pFilterOutHDCPContent = This->m_FilterOutHDCPContent;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_FILTER_OUT_HDCP_CONTENT (%d)\n",
							 *pFilterOutHDCPContent
							 ));
				*pdwSizeGot = sizeof(TQP_UINT32);
				break;
			}

			case QPCODEC_IS_CODEC_ERROR:
			{
				PTQP_UINT32	pIsCodecError = (PTQP_UINT32)pOutput;

				*pIsCodecError = CQLCodec_IsCodecError(This) | This->m_HDCPState;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodec_Get() QPCODEC_IS_CODEC_ERROR (%d)\n",
							 *pIsCodecError
							 ));
				*pdwSizeGot = sizeof(TQP_UINT32);
				break;
			}

			case QPCODEC_GPIO_BIT_DIRECTION:
			{
				PGPIO_BIT_DIRECTION	pBitDirectionIn = (PGPIO_BIT_DIRECTION)pInput;
				PGPIO_BIT_DIRECTION	pBitDirectionOut = (PGPIO_BIT_DIRECTION)pOutput;

				pBitDirectionOut->bBitNumber = pBitDirectionIn->bBitNumber;

				qpStatus = CQLCodec_GetGPIOBitDirection(This,
														pBitDirectionOut
														);
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_GPIO_BIT_DIRECTION bit(%d) dir(%d)\n",
							 pBitDirectionOut->bBitNumber,
							 pBitDirectionOut->Direction
							 ));

				*pdwSizeGot = sizeof(GPIO_BIT_DIRECTION);
				break;
			}

			case QPCODEC_GPIO_BIT_VALUE:
			{
				PGPIO_BIT_VALUE	pBitValueIn = (PGPIO_BIT_VALUE)pInput;
				PGPIO_BIT_VALUE	pBitValueOut = (PGPIO_BIT_VALUE)pOutput;

				pBitValueOut->bBitNumber = pBitValueIn->bBitNumber;

				qpStatus = CQLCodec_GetGPIOBitValue(This,
													pBitValueOut
													);
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_GPIO_BIT_VALUE bit(%d) val(%d)\n",
							 pBitValueOut->bBitNumber,
							 pBitValueOut->bValue
							 ));

				*pdwSizeGot = sizeof(GPIO_BIT_VALUE);
				break;
			}

			// Codec
			//
			case QPCODEC_PROP_SYS_FUNCTION:
			{
				PQPCODEC_SYS_FUNCTION	pSysFunction = (PQPCODEC_SYS_FUNCTION)pOutput;

				pSysFunction->function = pTaskData->m_codec_function.function;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_SYS_FUNCTION function(%d)\n",
							 pSysFunction->function
							 ));

				*pdwSizeGot = sizeof(QPCODEC_SYS_FUNCTION);
				break;
			}

			case QPCODEC_PROP_SYS_LINK:
			{
				PQPCODEC_SYS_LINK	pSysLink = (PQPCODEC_SYS_LINK)pOutput;

				pSysLink->video_input = pTaskData->m_video_input;
				pSysLink->video_in_ch = pTaskData->m_video_in_ch;
				pSysLink->video_output = pTaskData->m_video_output;
				pSysLink->video_out_ch = pTaskData->m_video_out_ch;
				pSysLink->audio_input = pTaskData->m_audio_input;
				pSysLink->audio_in_ch = pTaskData->m_audio_in_ch;
				pSysLink->audio_output = pTaskData->m_audio_output;
				pSysLink->audio_out_ch = pTaskData->m_audio_out_ch;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_SYS_LINK video_input(%d) video_in_ch(%d) video_output(%d) video_out_ch(%d) audio_input(%d) audio_in_ch(%d) audio_output(%d) audio_out_ch(%d)\n",
							 pSysLink->video_input,
							 pSysLink->video_in_ch,
							 pSysLink->video_output,
							 pSysLink->video_out_ch,
							 pSysLink->audio_input,
							 pSysLink->audio_in_ch,
							 pSysLink->audio_output,
							 pSysLink->audio_out_ch
							 ));

				*pdwSizeGot = sizeof(QPCODEC_SYS_LINK);
				break;
			}

			case QPCODEC_PROP_SYS_CONFIG:
			{
				PQPCODEC_SYS_CONFIG	pSysConfig = (PQPCODEC_SYS_CONFIG)pOutput;

				pSysConfig->version = This->m_VerFwAPI;
				pSysConfig->fixedMode = This->m_FwFixedMode;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_SYS_CONFIG version(%d) fixed mode(%d)\n",
							 pSysConfig->version,
							 pSysConfig->fixedMode
							 ));
				*pdwSizeGot = sizeof(QPCODEC_SYS_CONFIG);
				break;
			}

			case QPCODEC_PROP_OSD_MEMORY:
			{
				PQPCODEC_OSD_MEM_CONFIG	pOsdMemConfig = (PQPCODEC_OSD_MEM_CONFIG)pOutput;

				pOsdMemConfig->memStartAddress = pTaskData->m_decFWSharedMemWr;
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_OSD_MEMORY start_addr(0x%x) size(0x%x) \n",
							 pOsdMemConfig->memStartAddress,
							 pOsdMemConfig->memSize
							 ));

				*pdwSizeGot = sizeof(QPCODEC_OSD_MEM_CONFIG);
				break;
			}

			case QPCODEC_PROP_CTRL_VERSION:
			{
				PQPCODEC_CTRL_VERSION pCtrlVersion = (PQPCODEC_CTRL_VERSION)pOutput;

				pCtrlVersion->major = QPDEVCNTL_VER_MAJOR;
				pCtrlVersion->minor = QPDEVCNTL_VER_MINOR;
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
							 "CQLCodec_Get() QPCODEC_PROP_CTRL_VERSION major.minor(%d.%d)\n",
							 pCtrlVersion->major,
							 pCtrlVersion->minor
							 ));

				*pdwSizeGot = sizeof(QPCODEC_CTRL_VERSION);
				break;
			}

			case QPCODEC_PROP_AUDIO_OUTPUT_VOLUME:
			{
				PTQP_UINT32	pVol = (PTQP_UINT32)pOutput;

				*pVol = This->m_AOVolume;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_AUDIO_OUTPUT_VOLUME vol(%d)\n",
							 *pVol
							 ));

				*pdwSizeGot = sizeof(TQP_UINT32);

				break;
			}

			case QPCODEC_PROP_AUDIO_INPUT_VOLUME:
			{
				PTQP_UINT32	pVol = (PTQP_UINT32)pOutput;

				*pVol = This->m_AIVolume;

				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
							 "CQLCodec_Get() QPCODEC_PROP_AUDIO_INPUT_VOLUME vol(%d)\n",
							 *pVol
							 ));

				break;
			}

			default:
				break;
		}
	}
	else
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "CQLCodec_Get() unknown property\n"
					 ));
		qpStatus = QPERR_NOTIMPL;
	}

	return (qpStatus);
}


// CQLCodec
//


EQP_Errors CQLCodec_UpdateMiscConfig(CQLCodec *This,
									 TQP_OBJECT_HANDLE hTask
									 )
{
	EQP_Errors			qpStatus = QPERR_SUCCESS;
	QP_VBI_DATAFORMAT	VBIFormat;
	PTASK_DATA			pTaskData;


	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		return (QPERR_SUCCESS);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_UpdateMiscConfig()\n"
				 ));

	pTaskData = &This->m_pTask->m_TaskData[hTask];

	if (!pTaskData->m_bAcquired)
	{
		qpStatus = QPFWCODECAPI_SystemLink(This,
										   (TQP_UINT32) hTask,
										   pTaskData->m_video_input,
										   pTaskData->m_video_in_ch,
										   pTaskData->m_video_output,
										   pTaskData->m_video_out_ch,
										   pTaskData->m_audio_input,
										   pTaskData->m_audio_in_ch,
										   pTaskData->m_audio_output,
										   pTaskData->m_audio_out_ch
										   );
		if (QP_SUCCESS(qpStatus))
		{
			pTaskData->m_bAcquired = QP_TRUE;

		}
	}

	if (QPCODEC_CHIP_ARTESA_FPGA & This->m_ChipType)
	{
		return (QPERR_SUCCESS);
	}

	// set the encode mode
	//
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_SetEncMode(This,
	 																	(TQP_UINT32) hTask,
																		pTaskData->m_EncCapMode.capMode,
																		pTaskData->m_EncCapMode.trigMode,
																		pTaskData->m_EncCapMode.gpioPin
																		);


	// set the MJPEG quality
	//
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_SetMJPEGQuality(This,
	 																		 (TQP_UINT32) hTask,
																			 pTaskData->m_EncMjpegQuality
																			 );

	// set the MJPEG internal frame buffer bumber
	//
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_SetMJPEGFrameBuffer(This,
	 																			 (TQP_UINT32) hTask,
																				 pTaskData->m_EncMjpegFrameBuffer
																				 );

	// set external trigger to sync mode
	//
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_SetExternalTriggerToSync(This,
	 																			 (TQP_UINT32) hTask,
																				 pTaskData->m_ExternalTriggerToSync.Enable,
																				 pTaskData->m_ExternalTriggerToSync.gpioPin
																				 );

	// set PTS counter reset to 0 vis GPIO trigger
	//
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_SetPTSResetByTrigger(This,
	 																			 (TQP_UINT32) hTask,
																				 pTaskData->m_PTSCounterReset.Enable,
																				 pTaskData->m_PTSCounterReset.gpioPin,
																				 pTaskData->m_PTSCounterReset.immediate
																				 );

	// set raw video decimation
	//
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_SetRawVideoDecimation(This,
	 																			   (TQP_UINT32) hTask,
																				   pTaskData->m_RawVideoDecimationFactor.input_format,
																				   pTaskData->m_RawVideoDecimationFactor.output_format,
																				   pTaskData->m_RawVideoDecimationFactor.scale_factor.read
																				   );

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_SetDeinterlaceMode(This,
	 																			(TQP_UINT32) hTask,
																				pTaskData->m_DeinterlaceMode
																				);

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_SetRateControlEx(This,
	 																		  (TQP_UINT32) hTask,
																			  pTaskData->m_rateControlEx.Artesa.base_qp_length,
																			  pTaskData->m_rateControlEx.Artesa.local_activity_on,
																			  pTaskData->m_rateControlEx.Artesa.motion_detect_interval
																			  );

	if (pTaskData->m_LargeCompressBufferControl & QPCODEC_LARGE_COMPRESS_BUFFER_CONTROL_ENABLE)
	{
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_SetLargeCompressBufferControl(This,
				 																			   (TQP_UINT32) hTask,
																							   pTaskData->m_LargeCompressBufferControl
																							   );
	}

	if (pTaskData->m_EnableLowBitrateMode)
	{
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_SetLowBitrateMode(This,
	 																			   (TQP_UINT32) hTask,
																				   pTaskData->m_EnableLowBitrateMode
																				   );
	}

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_SetAVDiscardControl(This,
 																				 (TQP_UINT32) hTask,
																				 pTaskData->m_EnableAVDiscardControl
																				 );

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_SetUseSWPTS(This,
 																		 (TQP_UINT32) hTask,
																		 pTaskData->m_UseSWPTS
																		 );

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_SetViuSyncCode(This,
																			(TQP_UINT32) hTask,
																			This->m_ulViuSyncCode1,
																			This->m_ulViuSyncCode2
																			);

	// artesa firmware currently can not handle many misc commands
	//
	if (QPCODEC_CHIP_ARTESA_ALL & This->m_ChipType)
	{
		return (qpStatus);
	}

	// index capture
	//
	if (QP_SUCCESS(qpStatus))
	{
		if (pTaskData->pChannel[TASK_DATA_TYPE_INDEX])
		{
			qpStatus = QPFWENCAPI_SetIndexCapture(This,
												  (TQP_UINT32) hTask,
												  pTaskData->m_EncIndexCapFreq
												  );
		}
		else
		{
			qpStatus = QPFWENCAPI_SetIndexCapture(This,
												  (TQP_UINT32) hTask,
												  0
												  );
		}
	}

	// VBI capture
	//
	if (QP_SUCCESS(qpStatus))
	{
		TQP_BOOL	bVBI = QP_FALSE;

		if (pTaskData->pChannel[TASK_DATA_TYPE_VBI])
		{
			CVBIOutChannel	*pVbiOutChannel = GET_INHERITED_OBJECT(CVBIOutChannel, m_Channel, pTaskData->pChannel[TASK_DATA_TYPE_VBI]);

			bVBI = CVBIOutChannel_GetVBIFormat(pVbiOutChannel,
											   &VBIFormat
											   );
		}

		if (bVBI)
		{
			qpStatus = QPFWENCAPI_SetVBIInfo(This,
											 (TQP_UINT32) hTask,
											 1,
											 VBIFormat.Top_StartLine,
											 VBIFormat.Top_EndLine - VBIFormat.Top_StartLine + 1,
 											 VBIFormat.Top_StartPixel,
											 VBIFormat.Top_SamplesPerLine,
											 VBIFormat.Bottom_StartLine,
											 VBIFormat.Bottom_EndLine - VBIFormat.Bottom_StartLine + 1,
  											 VBIFormat.Bottom_StartPixel,
											 VBIFormat.Bottom_SamplesPerLine
											 );
		}
		else
		{
			qpStatus = QPFWENCAPI_SetVBIInfo(This,
											 (TQP_UINT32) hTask,
											 0,
											 0,
											 0,
											 0,
											 0,
											 0,
											 0,
											 0,
											 0
											 );
		}
	}

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_SetMP4VideoBlockNumber(This,
																					(TQP_UINT32) hTask,
																					pTaskData->m_MP4VideoBlockNumber
																					);
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_SetAudioEnhancement(This,
																				 (TQP_UINT32) hTask,
																				 pTaskData->m_audioEnhancement.gain1,
																				 pTaskData->m_audioEnhancement.gain2,
																				 pTaskData->m_audioEnhancement.add,
																				 pTaskData->m_audioEnhancement.sub,
																				 pTaskData->m_audioEnhancement.att1,
																				 pTaskData->m_audioEnhancement.att2,
																				 pTaskData->m_audioEnhancement.lgain,
																				 pTaskData->m_audioEnhancement.rgain
																				 );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_EnableVidPadding(This,
																			  (TQP_UINT32) hTask,
																			  pTaskData->m_bEncEnableVidPadding
																			  );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_StillVideoInput(This,
																			 (TQP_UINT32) hTask,
																			 pTaskData->m_bEncVidStillInput
																			 );
	// initially video is not frozen
	//
	pTaskData->m_bEncVidFrozen = QP_FALSE;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "%CQLCodec_UpdateMiscConfig() Status(0x%x)\n",
				 qpStatus
				 ));

	return (qpStatus);
}



EQP_Errors CQLCodec_UpdateEncoderConfig(CQLCodec *This,
										TQP_OBJECT_HANDLE hTask,
										TQP_BOOL bGrabMailbox
										)
{
	EQP_Errors	qpStatus;
	PTASK_DATA	pTaskData;

	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		return (QPERR_SUCCESS);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_UpdateEncoderConfig()\n"
				 ));

	if (bGrabMailbox)
	{
		qpStatus = QPFWAPI_MailboxReady(This,
										500
										);
		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CQLCodec_UpdateEncoderConfig() QPFWAPI_MailboxReady() failed(%d)\n",
						 qpStatus
						 ));
			return (qpStatus);
		}
	}

	pTaskData = &This->m_pTask->m_TaskData[hTask];

	qpStatus = QPFWENCAPI_SetSystemControl(This,
			     						   pTaskData->m_systemControl.Read
			     						   );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_SetPictureResolution(This,
																				  pTaskData->m_pictureResolution.Read
																				  );
#if 0
	if (QP_SUCCESS(qpStatus))
	{
		CQLCodecLib	*pQLCodecLib = GET_INHERITED_OBJECT(CQLCodecLib, m_Object, This->m_Object.m_pParent);

		if (pQLCodecLib->m_pVidDecoder)
		{
			qpStatus = QPVideoDecoder_SetVideoRes(&pQLCodecLib->m_pVidDecoder->m_iVideoDecoder,
												  pTaskData->m_pictureResolution.Reg.pic_width,
												  pTaskData->m_pictureResolution.Reg.pic_height,
												  pTaskData->m_inputControl.RegArtesa.scan_format
												  );
		}
	}
#endif

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_SetInputControl(This,
																			 pTaskData->m_inputControl.Read
																			 );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_SetRateControl(This,
																			pTaskData->m_rateControl.Read
																			);
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_SetVBRBitRate(This,
																		   pTaskData->m_bitRate.Read
																		   );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_SetFilterControl(This,
																			  pTaskData->m_filterControl.Read
																			  );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_SetGOPLoopFilter(This,
																			  pTaskData->m_gopLoopFilter.Read
																			  );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_SetETControl(This,
																		  pTaskData->m_etControl.Read
																		  );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_SetBlockSize(This,
																		  pTaskData->m_blkXferSize.Read
																		  );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_SetOutPictureResolution(This,
																					 pTaskData->m_outPictureResolution.Read
																					 );
	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_SetAudioControlParameters(This,
																					   pTaskData->m_audioControlParam.Read
																					   );
	switch(pTaskData->m_audioControlParam.Reg.audio_type)
	{
		case AUDIO_TYPE_G711:
		case AUDIO_TYPE_G729:
			qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_SetAudioControlExtension(This,
																							  pTaskData->m_audioControlExG711.Read
																							  );
			break;
		case AUDIO_TYPE_LPCM:
		case AUDIO_TYPE_RAW:
		    pTaskData->m_audioControlExLPCM.Reg.raw_type = 0; /* clear raw_type information in previous streaming */
			qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_SetAudioControlExtension(This,
																							  pTaskData->m_audioControlExLPCM.Read
																							  );
			break;
		case AUDIO_TYPE_SILK:
			qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_SetAudioControlExtension(This,
																							  pTaskData->m_audioControlExSILK.Read
																							  );
			break;
		case AUDIO_TYPE_AAC:
		default:
			qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_SetAudioControlExtension(This,
																							  pTaskData->m_audioControlExAAC.Read
																							  );
	}


	if (bGrabMailbox)
	{
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPFWENCAPI_UpdateConfig(This,
																			  (TQP_UINT32) hTask
																			  );


		QPFWAPI_MailboxDone(This);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "%CQLCodec_UpdateEncoderConfig() Status(0x%x)\n",
				 qpStatus
				 ));
	return (qpStatus);
}



EQP_Errors CQLCodec_UpdateDecoderConfig(CQLCodec *This,
										TQP_OBJECT_HANDLE hTask
										)
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
	PTASK_DATA	pTaskData;


	if (QLCODEC_PWR_STATE_D0 != This->m_PowerState)
	{
		return (QPERR_SUCCESS);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_UpdateDecoderConfig()\n"
				 ));

	pTaskData = &This->m_pTask->m_TaskData[hTask];


	if (!pTaskData->m_bAcquired)
	{
		qpStatus = QPFWCODECAPI_SystemLink(This,
										   (TQP_UINT32) hTask,
										   pTaskData->m_video_input,
										   pTaskData->m_video_in_ch,
										   pTaskData->m_video_output,
										   pTaskData->m_video_out_ch,
										   pTaskData->m_audio_input,
										   pTaskData->m_audio_in_ch,
										   pTaskData->m_audio_output,
										   pTaskData->m_audio_out_ch
										   );
		if (QP_SUCCESS(qpStatus))
		{
			pTaskData->m_bAcquired = QP_TRUE;

		}
	}


	qpStatus = QPFWDECAPI_SetDisplayAttrib(This,
										   (TQP_UINT32) hTask,
										   (pTaskData->m_decDispForceRemote << 0x8) | (pTaskData->m_decDispTVSystem & 0xFF),
										   pTaskData->m_decDispInputSize.Read,
										   pTaskData->m_decDispOutputSize.Read,
										   pTaskData->m_decDispInputOrigin.Read,
										   pTaskData->m_decDispOutputOrigin.Read
										   );

	if (QP_SUCCESS(qpStatus))
	{
		CQLCodecLib	*pQLCodecLib = GET_INHERITED_OBJECT(CQLCodecLib, m_Object, This->m_Object.m_pParent);

		// config video encoder
		if ((SYS_LINK_VID_OUT_VOU == pTaskData->m_video_output) &&
			pQLCodecLib->m_pVidEncoder
			)
		{
			VIDEO_STANDARD	std = VIDEO_STANDARD_INVALID;

			switch (pTaskData->m_decDispTVSystem)
			{
				case QPDEC_DISP_ATTRIB_576i:
					std = VIDEO_STANDARD_PAL;
					break;
				case QPDEC_DISP_ATTRIB_480i:
					std = VIDEO_STANDARD_NTSC;
					break;
				case QPDEC_DISP_ATTRIB_720p:
					std = VIDEO_STANDARD_720P;
					break;
				case QPDEC_DISP_ATTRIB_1080i:
					std = VIDEO_STANDARD_1080I;
					break;
                case QPDEC_DISP_ATTRIB_1080p:
					std = VIDEO_STANDARD_1080P;
					break;
				case QPDEC_DISP_ATTRIB_480p:
				default:
					break;
			}

			if (VIDEO_STANDARD_INVALID != std)
			{
				qpStatus = IVideoEncoder_SetVideoStandard(&pQLCodecLib->m_pVidEncoder->m_iVideoEncoder,
														  std
														  );
                if (!QP_SUCCESS(qpStatus))
                {
                    QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
                        "CQLCodec_UpdateDecoderConfig() IVideoEncoder_SetVideoStandard set %d fail, ignore it\n",
                        std
                    ));
                    qpStatus = QPERR_SUCCESS;
                }
			}
		}
	}


////	qpStatus = QPFWDECAPI_SetDisplayMode(This,
////										 QPDEC_DISP_MODE_NORMAL,
////  		  								 This->m_decDispMode
////  		  								 );

	return (qpStatus);
}



// Firmware download
//
EQP_Errors CQLCodec_FWDownload(CQLCodec *This,
							   PTQP_UINT8 pFwBuffer,
							   TQP_UINT32 ulFWSize,
							   TQP_BOOL doReset,
							   TQP_UINT32 ulStartAddr,
							   TQP_BOOL bCheckState,
							   TQP_BOOL bVerify
							   )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
#ifdef _QP_ANDROID
	TQP_UINT32	dwDMASize = min((TQP_UINT32)(16 * 1024), QPHCI_GetMaxDMASize(&This->m_hci));
#else
	TQP_UINT32	dwDMASize = min((TQP_UINT32)(32 * 1024), QPHCI_GetMaxDMASize(&This->m_hci));
#endif

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_FWDownload() start(0x%x) size(%d) dma(%d)\n",
				 ulStartAddr,
				 ulFWSize,
				 dwDMASize
				 ));

	// can not download firmware in master mode
	//
	if ((QPHCI_BUS_INTERNAL == This->m_hci.m_bus_type) &&
		(QPHCI_MODE_MASTER == This->m_hci.m_access_mode)
		)
	{
		return (qpStatus);
	}

	// do not download if any channel is running
	//
	if (bCheckState)
	{
		if (!CQLCodec_IsCodecIdle(This))
			return (qpStatus);
	}

	// hold ARM in reset
	//
	if (doReset)
	{
		qpStatus = QPHCI_ResetArm(&This->m_hci,
								  QP_FALSE
								  );
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_ReInit(&This->m_hci);
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : CQLCodec_InitializeMemory(This);
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : CQLCodec_AOSwitch(This,
																		This->m_AOEnable
																		);
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : CQLCodec_VOSwitch(This,
																		This->m_VOEnable
																		);
		qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : CQLCodec_SetGPIODefaults(This);

		DelayMilliSeconds(1);
	}

	if (QL201_AUD_FW_START_ADDR == ulStartAddr)
	{
#if 1
		// make sure ARC is disabled if this is audio fw
		//
		if (doReset)
		{
			TQP_UINT32 dwRegister;
			QPHCI_RegisterRead(&This->m_hci,
							   0,
							   &dwRegister
							   );

			QPHCI_RegisterWrite(&This->m_hci,
								0,
								dwRegister & ~(0x2000)//0
								);

			DelayMilliSeconds(1);
		}
#endif
	}
	else
	{
		CQLCodecLib	*pQLCodecLib = GET_INHERITED_OBJECT(CQLCodecLib, m_Object, This->m_Object.m_pParent);
#ifndef _QP_QPSOS
		TQP_UINT32	USBLibSharedMemory;

		if (QPSOSGetOSMajorVersion((pFwBuffer + 0x100)) >= 3)
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
						 "CQLCodec_FWDownload() QPSOS3 or greater\n"
						 ));
			USBLibSharedMemory = PHY_USB_AREA;
		}
		else
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
						 "CQLCodec_FWDownload() QPSOS2\n"
						 ));
			USBLibSharedMemory = FLAT_USBAREA;
		}

#if 0   /* for debug only */
		if (This->m_FwFixedMode==0) {
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CQLCodec_FWDownload() Let FW use static memory allcoation\n"
						 ));
		}
		else {
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CQLCodec_FWDownload() Let FW use dynamic memory allcoation\n"
						 ));
		}
#endif
		// init firmware configuration
		//
		QPHCI_MemoryWrite(&This->m_hci,
						  CONFIG_SIGN_FIXED_MODE_PHY_ADDR,
						  This->m_FwFixedMode
						  );
		QPHCI_MemoryWrite(&This->m_hci,
						  CONFIG_SIGN_INT_MODE_PHY_ADDR,
						  This->m_FwIntMode
						  );

		// init USB firmware configuration
		//
		QPHCI_MemoryWrite(&This->m_hci,
						  USBLibSharedMemory + FIELD_OFFSET(struct usb_system_configs, useMode),
						  QPPF_USBLIBGADGETMODE_FX2CLONE
						  );
#endif //!_QP_QPSOS


		// change PCIE module i2c address and system PLL to 165MHz to work
		// around QL330 I2C issue if we are using QL330 as i2c provider
		//
		if ((QPI2C_TYPE_USE_QLCODEC == pQLCodecLib->m_InitData.i2cInitData.type) ||
			(QPI2C_TYPE_SOC_INTERNAL_HW == pQLCodecLib->m_InitData.i2cInitData.type)
			)
		{
			// change pcie phy's i2c debug address from 0x10 to 0x15.
			QPHCI_RegisterWrite(&This->m_hci,
								0xB78,
								0x150000
								);
		}

		// change PCIE module ARM clock to 261MHz
		if (QPHCI_BUS_PCI == This->m_hci.m_bus_type)
		{
			// change pll4 setting
			if (This->m_Pll4)
			{
				QPHCI_RegisterWrite(&This->m_hci,
									0xC8,
									This->m_Pll4
									);
			}
			else
			{
				if (QLCODEC_VER_QL330_B0 == This->m_hci.m_ulChipVer)
				{
					QPHCI_RegisterWrite(&This->m_hci,
										0xC8,
										0x30130
										);
				}
				else
				{

					QPHCI_RegisterWrite(&This->m_hci,
										0xC8,
										0x20236
										);
				}
			}

			// change pll5 setting
			if (This->m_Pll5)
			{
				QPHCI_RegisterWrite(&This->m_hci,
									0xCC,
									This->m_Pll5
									);
			}
			else
			{
				QPHCI_RegisterWrite(&This->m_hci,
									0xCC,
									0x10239
									);
			}
		}

		// clear the mail box
		//
		QPHCI_RegisterWrite(&This->m_hci,
							REG_TO_ARM_MESSAGE_STATUS,
							0
							);
	}

	// download firmware
	//
	if (QP_SUCCESS(qpStatus))
	{
		TQP_UINT32		ulThisWrite;
		TQP_UINT32		ulOffset = 0;
		PTQP_UINT8		pBuf = (PTQP_UINT8)QPMMMalloc2Ex(dwDMASize);

#if 1
		// starting from FW1.00.0009 arc code will
		// zero all the data area in the firmwware
		// itself

		// zero init buffer
		//
		memset(pBuf,
			   0,
			   dwDMASize
			   );

		// zero all the non-initialized audio memory
		//
		if (QL201_AUD_FW_START_ADDR == ulStartAddr)
		{
			if (pBuf)
			{
				TQP_UINT32	ulFWSizeDwodAligned = ulFWSize & 0xFFFFFFFC;
				TQP_UINT32	ulZero = 460324;

				if (ulZero >= ulFWSizeDwodAligned)
				{
					ulZero -= ulFWSizeDwodAligned;
				}
				else
				{
					ulZero = 0;
				}

				ulOffset = ulFWSizeDwodAligned;

				// write zero to the audio memory
				//
				while (ulZero)
				{
					ulThisWrite = min(ulZero, dwDMASize);

					qpStatus = CQLCodec_StartDMAWrite(This,
													  (ulStartAddr + ulOffset) >> 2,
													  pBuf,
													  ulThisWrite,
													  QP_FALSE,	// swap
													  QP_TRUE,	// sync
													  DMA_TRANSFER_MODE_LINEAR | DMA_BUFFER_MODE_COMMON,
													  0,
													  0,
													  QP_TRUE
													  );
					if (!QP_SUCCESS(qpStatus))
					{
						break;
					}
					ulZero -= ulThisWrite;
					ulOffset += ulThisWrite;
				}

				ulOffset = 0;
			}
		}
#endif //0

		while (QP_SUCCESS(qpStatus) &&
			   ulFWSize
			   )
		{
			ulThisWrite = min(ulFWSize, dwDMASize);

			qpStatus = CQLCodec_StartDMAWrite(This,
											  (ulStartAddr + ulOffset) >> 2,
											  pFwBuffer + ulOffset,
											  ulThisWrite,
											  QP_FALSE,
											  QP_TRUE,
											  DMA_TRANSFER_MODE_LINEAR | DMA_BUFFER_MODE_COMMON,
											  0,
											  0,
											  QP_TRUE
											  );
			if (bVerify &&
				pBuf
				)
			{
				if (QP_SUCCESS(qpStatus))
				{
					DelayMilliSeconds(1);

					qpStatus = CQLCodec_StartDMARead(This,
													 (ulStartAddr + ulOffset) >> 2,
													 pBuf,
													 ulThisWrite,
													 QP_FALSE,
													 QP_TRUE,
													 DMA_TRANSFER_MODE_LINEAR | DMA_BUFFER_MODE_COMMON,
													 0,
													 0,
													 QP_TRUE
													 );
				}

				if (QP_SUCCESS(qpStatus))
				{
					TQP_UINT32	ulCmp = 0;

					ulCmp = memcmp(pFwBuffer + ulOffset,
								   pBuf,
								   ulThisWrite
								   );

					if (0 != ulCmp)
					{
						QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
									 "CQLCodec_FWDownload() memcmp failed(%d) offset (%d) len (%d)\n",
									 ulCmp, ulOffset, ulThisWrite
									 ));
						qpStatus = QPERR_FAIL;
						#if 0
						{
							int i;
							QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
									"%s() FW DUMP: :",
									__FUNCTION__
							));

							for (i = 0; i < 10; i++) {
								QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
										"%#1x:		%#1x:",
										*(pBuf + i),
										*(pFwBuffer + ulOffset + i)
								));
							}

							QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
									"\n"));

							for (i = 0; i < ulThisWrite; i++) {
								if (*(pFwBuffer + ulOffset + i) != *(pBuf + i)) {
									QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
									 		"CQLCodec_FWDownload() differ at pos (%d) fw(%d) buf(%d)\n",
									 		i, *(pFwBuffer + ulOffset + i), *(pBuf + i)
									 		));
									break;
								}
							}
						}
						#endif
					}
				}
			}

			if (!QP_SUCCESS(qpStatus))
				break;

			ulOffset += ulThisWrite;
			ulFWSize -= ulThisWrite;
		}

		if (pBuf)
		{
			QPMMFree2Ex(pBuf);
		}
	}

	// ARM running
	//
	if (doReset)
	{
		if (QP_SUCCESS(qpStatus))
		{
			DelayMilliSeconds(250);

			qpStatus = QPHCI_ResetArm(&This->m_hci,
									  QP_TRUE
									  );

			DelayMilliSeconds(1);
		}
	}
	else
	{
		DelayMilliSeconds(1);
	}

	return (qpStatus);
}



EQP_Errors CQLCodec_FWDownloadAll(CQLCodec *This,
								  TQP_BOOL bCheckState,
								  TQP_BOOL bVerify
								  )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;

	// can not download firmware in master mode
	//
	if ((QPHCI_BUS_INTERNAL == This->m_hci.m_bus_type) &&
		(QPHCI_MODE_MASTER == This->m_hci.m_access_mode)
		)
	{
		return (qpStatus);
	}

	// do not download if any channel is running
	//
	if (bCheckState)
	{
		if (!CQLCodec_IsCodecIdle(This))
			return (qpStatus);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "CQLCodec_FWDownloadAll() checkState(%d) verify(%d)\n",
				 bCheckState,
				 bVerify
				 ));

	ENTER_CRITICAL(&This->m_Object)

	// hold arm in reset
	//
	QPHCI_ResetArm(&This->m_hci,
				   QP_FALSE
				   );
	QPHCI_ReInit(&This->m_hci);
	CQLCodec_InitializeMemory(This);
	CQLCodec_AOSwitch(This,
					  This->m_AOEnable
					  );
	CQLCodec_VOSwitch(This,
					  This->m_VOEnable
					  );
	CQLCodec_SetGPIODefaults(This);

//	DelayMilliSeconds(1);
	DelayMilliSeconds(50);

#if 1
	{
		// disable arc hardware module
		//

		TQP_UINT32 dwRegister;
		QPHCI_RegisterRead(&This->m_hci,
						   0,
						   &dwRegister
						   );

		QPHCI_RegisterWrite(&This->m_hci,
							0,
							dwRegister & ~(0x2000)
							);

		DelayMicroSeconds(1);
	}
#endif

	// download audio microcode
	//
	qpStatus = CQLCodec_FWDownload(This,
								   This->m_pAudioFW,
								   This->m_QL201AudFWSize,
								   QP_FALSE,
								   QL201_AUD_FW_START_ADDR,
								   QP_FALSE,
								   bVerify
								   );

	if (!QP_SUCCESS(qpStatus))
	{
		LEAVE_CRITICAL(&This->m_Object)

		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CQLCodec_FWDownloadAll() FWDownload Audio FW Failed status(%d)!!!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	DelayMilliSeconds(1);

	// download video microcode
	//
	qpStatus = CQLCodec_FWDownload(This,
								   This->m_pVideoFW,
								   This->m_QL201FWSize,
								   QP_FALSE,
								   QL201_VID_FW_START_ADDR,
								   QP_FALSE,
								   bVerify
								   );

	if (!QP_SUCCESS(qpStatus))
	{
		LEAVE_CRITICAL(&This->m_Object)

		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CQLCodec_FWDownloadAll() FWDownload video FW Failed status(%d)!!!\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	DelayMicroSeconds(500);

	// enable arm
	//
	QPHCI_ResetArm(&This->m_hci,
				   QP_TRUE
				   );

	// wait a little bit for firmware to initialize
	//
	DelayMilliSeconds(150);

	LEAVE_CRITICAL(&This->m_Object)

#if 1
	// restore the Driver AIAOVolume setting value because firmware reload
	This->m_AIVolume = 8;
	This->m_AOVolume = 8;
#endif

	return (qpStatus);
}



EQP_Errors CQLCodec_FWUpdate(CQLCodec *This,
							 PTQP_UINT8 pFwBuffer,
							 TQP_UINT32 ulFWSize,
							 TQP_UINT32 ulStartAddr
							 )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;

	ENTER_CRITICAL(&This->m_Object)

	if (QL201_VID_FW_START_ADDR == ulStartAddr)
	{
		if (This->m_pVideoFW)
		{
			// delete the previously allocated buffer
			//
			if (This->m_bVideoFWUpdated)
				QPMMFree2Ex(This->m_pVideoFW);
			else
				QPPFMGetVideoFW(QP_NULL,
								QP_NULL
								);
		}

		This->m_pVideoFW = (PTQP_UINT8)QPMMMalloc2Ex(ulFWSize);
		if (This->m_pVideoFW)
		{
			memcpy(This->m_pVideoFW,
				   pFwBuffer,
				   ulFWSize
				   );
			This->m_QL201FWSize = ulFWSize;
			This->m_bVideoFWUpdated = QP_TRUE;
		}
		else
		{
			// use build in firmware
			This->m_QL201FWSize = 0;

			qpStatus = QPPFMGetVideoFW(&This->m_pVideoFW,
									   &This->m_QL201FWSize
									   );
			This->m_bVideoFWUpdated = QP_FALSE;
		}
	}
	else if (QL201_AUD_FW_START_ADDR == ulStartAddr)
	{
		if (This->m_pAudioFW)
		{
			// delete the previous allocated buffer
			//
			if (This->m_bAudioFWUpdated)
				QPMMFree2Ex(This->m_pAudioFW);
			else
				QPPFMGetAudioFW(QP_NULL,
								QP_NULL
								);
		}

		This->m_pAudioFW = (PTQP_UINT8)QPMMMalloc2Ex(ulFWSize);
		if (This->m_pAudioFW)
		{
			memcpy(This->m_pAudioFW,
				   pFwBuffer,
				   ulFWSize
				   );
			This->m_QL201AudFWSize = ulFWSize;
			This->m_bAudioFWUpdated = QP_TRUE;
		}
		else
		{
			This->m_QL201AudFWSize = 0;
			qpStatus = QPPFMGetAudioFW(&This->m_pAudioFW,
									   &This->m_QL201AudFWSize
									   );
			This->m_bAudioFWUpdated = QP_FALSE;
		}
	}
	else
	{
		qpStatus = QPERR_PARMS;
	}

	LEAVE_CRITICAL(&This->m_Object)

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_FWUpdate() status(%d)\n",
				 qpStatus
				 ));

	return (qpStatus);
}



EQP_Errors CQLCodec_FWSwitchMode(CQLCodec *This,
								 TQP_UINT32 ulMode,
								 TQP_BOOL bCheckState
								 )
{
	EQP_Errors	qpStatus = QPERR_SUCCESS;
#ifndef _QP_QPSOS
	TQP_UINT32	USBLibSharedMemory;
	TQP_UINT32	QPSOSVersionBlk[2];
#endif //!_QP_QPSOS

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "CQLCodec_FWSwitchMode() mode(%d) bCheckState(%d)\n",
				 ulMode,
				 bCheckState
				 ));

	if (QXU_FIRMWARE_UVC != ulMode)
	{
		return (qpStatus);
	}

	// can not switch mode if not use our own USB
	//
	if ((QPHCI_BUS_USB != This->m_hci.m_bus_type) ||
		(QPHCI_MODE_DIRECT != This->m_hci.m_access_mode)
		)
	{
		return (qpStatus);
	}

	// do not switch mode if any channel is running
	//
	if (bCheckState)
	{
		if (!CQLCodec_IsCodecIdle(This))
			return (qpStatus);
	}

#ifndef _QP_QPSOS
	QPHCI_MemoryRead(&This->m_hci,
					 0x100,
					 &QPSOSVersionBlk[0]
					 );
	QPHCI_MemoryRead(&This->m_hci,
					 0x104,
					 &QPSOSVersionBlk[1]
					 );

	if (QPSOSGetOSMajorVersion(&QPSOSVersionBlk[0]) >= 3)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "CQLCodec_FWSwitchMode() QPSOS3 or greater\n"
					 ));
		USBLibSharedMemory = PHY_USB_AREA;
	}
	else
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "CQLCodec_FWSwitchMode() QPSOS2\n"
					 ));
		USBLibSharedMemory = FLAT_USBAREA;
	}

	// init firmware configuration
	//
	QPHCI_MemoryWrite(&This->m_hci,
					  CONFIG_SIGN_INT_MODE_PHY_ADDR,
					  QPPF_CODECLIBINTMODE_SW1_ISR	// SW1
					  );

	// init USB firmware configuration
	//
	QPHCI_MemoryWrite(&This->m_hci,
					  USBLibSharedMemory + FIELD_OFFSET(struct usb_system_configs, useMode),
					  QPPF_USBLIBGADGETMODE_UVC
					  );

#endif //!_QP_QPSOS

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "CQLCodec_FWSwitchMode() QPHCI_ResetArm(0)\n"
				 ));

	// hold ARM in reset
	//
	qpStatus = QPHCI_ResetArm(&This->m_hci,
							  QP_FALSE
							  );
	DelayMilliSeconds(1);

	// clear the mail box
	//
	QPHCI_RegisterWrite(&This->m_hci,
						REG_TO_ARM_MESSAGE_STATUS,
						0
						);

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "CQLCodec_FWSwitchMode() QPHCI_ResetArm(1)\n"
				 ));

	// ARM running
	//
	qpStatus = QPHCI_ResetArm(&This->m_hci,
							  QP_TRUE
							  );
	return (qpStatus);
}



#ifdef _QPCODEC_USE_EXTERNAL_HOST_DMA_FUNC

// declare function prototypes for
// QPX_Wrap_WriteDMA and QPX_Wrap_ReadDMA
//
extern void QPX_Wrap_WriteDMA(void * src, unsigned long size, int bSync);
extern void QPX_Wrap_ReadDMA(void * dst, unsigned long size, int bSync);

#endif //_QPCODEC_USE_EXTERNAL_HOST_DMA_FUNC




// DMA
//
EQP_Errors CQLCodec_StartDMAWrite(CQLCodec *This,
								  TQP_UINT32 ulAtmAddr,
								  PTQP_UINT8 pHostAddr,
								  TQP_UINT32 ulLength,
								  TQP_BOOL bSwap,
								  TQP_BOOL bSync,
								  TQP_UINT32 ulXferMode,
								  TQP_UINT32 ulPicWidth,
								  TQP_UINT32 ulPicHeight,
								  TQP_BOOL IsFrameMode
								  )
{
	EQP_Errors	qpStatus;
	TQP_UINT32	ulEvents;

	if ((0 == ulLength) ||
		(QP_NULL == pHostAddr)
		)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "CQLCodec_StartDMAWrite() Param error ulLength(%d), pHostAddr(0x%x)\n",
					 ulLength,
					 pHostAddr
					 ));
		return (QPERR_PARMS);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, /*QPDBGM_LEVEL_WARN*/QPDBGM_LEVEL_TRACE),
				 "CQLCodec_StartDMAWrite() Arm(0x%x) Host(0x%x) len(%d) swap(%d) sync(%d) xferMode(%d) w(%d) h(%d) frame(%d)\n",
				 ulAtmAddr,
				 pHostAddr,
				 ulLength,
				 bSwap,
				 bSync,
				 ulXferMode,
				 ulPicWidth,
				 ulPicHeight,
				 IsFrameMode
				 ));

	if (bSync)
	{
		if ((QPHCI_BUS_USB != This->m_hci.m_bus_type) &&
			(QPHCI_BUS_201_EMULATION != This->m_hci.m_bus_type) &&
			(QPHCI_BUS_PCI != This->m_hci.m_bus_type)
			)
		{
			// clear any outstanding P2M interrupt
			//
			QPHCI_ClearInterrupt(&This->m_hci,
								 HCI_INTERRUPT_P2M
								 );

			// enable P2M interrupt
			//
			QPHCI_EnableInterrupt(&This->m_hci,
								  HCI_INTERRUPT_P2M
								  );
		}

		// clear P2M done
		//
		QPOSMClearEvtgrp(This->m_EvtSyncDma,
						 QLCODEC_EVENT_DMA_SYNC_WRITE_DONE
						 );
	}

	// start QL201 DMA
	//
	qpStatus = QPHCI_StartDMAWrite(&This->m_hci,
								   ulAtmAddr,
								   ulLength >> 2,
								   pHostAddr,
								   ulXferMode,
								   ulPicWidth / 16,
								   0,
								   bSwap ? (DMA_DATA_SWAP_BYTE | DMA_DATA_SWAP_WORD) : 0,
								   IsFrameMode
								   );

	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CQLCodec_StartDMAWrite() QPHCI_StartDMAWrite() Failed status(%d)\n",
					 qpStatus
					 ));
	}
#ifndef CONFIG_HOST_PLATFORM_STR8133_REF
	else
	{
		// start host DMA
		//
		if (QPHCI_BUS_CUSTOM_1 == This->m_hci.m_bus_type)
		{
			// to use external host DMA function
			// define _QPCODEC_USE_EXTERNAL_HOST_DMA_FUNC compile time switch
			// in the platform.h
			//
#ifdef _QPCODEC_USE_EXTERNAL_HOST_DMA_FUNC
			QPX_Wrap_WriteDMA(pHostAddr, ulLength, (int)bSync);
#else //!_QPCODEC_USE_EXTERNAL_HOST_DMA_FUNC
			// write the data to the streaming port
			// 16 bit at a time
			//
			PTQP_UINT32	ptr = (PTQP_UINT32)pHostAddr;
			TQP_UINT32	i;
			TQP_UINT32	ulDMALength = ulLength >> 2;
			TQP_PVOID	StreamPort;

			qpStatus = QPHCI_GetStreamingPortAddress(&This->m_hci,
	 												 (TQP_PVOID *)&StreamPort
	 												 );

			if (QP_SUCCESS(qpStatus))
			{
				for (i = 0; i < ulDMALength; i++)
				{
					*((volatile TQP_UINT16 *)StreamPort) = (TQP_UINT16)(*ptr & 0x0000ffff);
					*((volatile TQP_UINT16 *)StreamPort) = (TQP_UINT16)(*ptr >> 16);
					ptr++;
				}
			}
#endif //_QPCODEC_USE_EXTERNAL_HOST_DMA_FUNC
		}
	}
#endif // !CONFIG_HOST_PLATFORM_STR8133_REF

	if (bSync)
	{
		// wait for P2M done
		//
		if (QP_SUCCESS(qpStatus))
		{
			qpStatus = QPOSMWaitEvtgrp(This->m_EvtSyncDma,
									   QLCODEC_EVENT_DMA_SYNC_WRITE_DONE,
									   &ulEvents,
									   2000
									   );

			if (!QP_SUCCESS(qpStatus))
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodec_StartDMAWrite() QPOSMWaitEvtgrp() Failed status(%d)\n",
							 qpStatus
							 ));
			}
			else
			{
				QPHCI_DMAWriteDone(&This->m_hci);
			}
		}

		if ((QPHCI_BUS_USB != This->m_hci.m_bus_type) &&
			(QPHCI_BUS_201_EMULATION != This->m_hci.m_bus_type) &&
			(QPHCI_BUS_PCI != This->m_hci.m_bus_type)
			)
		{
			// disable P2M interrupt
			//
			QPHCI_DisableInterrupt(&This->m_hci,
								   HCI_INTERRUPT_P2M
								   );
		}

		QPOSMClearEvtgrp(This->m_EvtSyncDma,
						 QLCODEC_EVENT_DMA_SYNC_WRITE_DONE
						 );
	}
#ifdef _QP_REDUCE_DMA_DONE_TASK_SWITCH
	else
	{
		if ((QPHCI_BUS_CUSTOM_1 == This->m_hci.m_bus_type) ||
			(QPHCI_BUS_201_EMULATION == This->m_hci.m_bus_type)
			)
		{
			if (QP_SUCCESS(qpStatus))
			{
				This->m_pTask->CompleteData(This->m_pTask,
											CHANNEL_DIR_WRITE
											);
			}
		}

	}
#endif //_QP_REDUCE_DMA_DONE_TASK_SWITCH

	return (qpStatus);
}



EQP_Errors CQLCodec_StartDMARead(CQLCodec *This,
								 TQP_UINT32 ulAtmAddr,
								 PTQP_UINT8 pHostAddr,
								 TQP_UINT32 ulLength,
								 TQP_BOOL bSwap,
								 TQP_BOOL bSync,
								 TQP_UINT32 ulXferMode,
								 TQP_UINT32 ulPicWidth,
								 TQP_UINT32 ulPicHeight,
								 TQP_BOOL IsFrameMode
								 )
{
	EQP_Errors	qpStatus;
	TQP_UINT32	ulEvents;

	if ((0 == ulLength) ||
		(QP_NULL == pHostAddr)
		)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "CQLCodec_StartDMARead() Param error ulLength(%d), pHostAddr(0x%x)\n",
					 ulLength,
					 pHostAddr
					 ));
		return (QPERR_PARMS);
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, /*QPDBGM_LEVEL_WARN*/QPDBGM_LEVEL_TRACE),
				 "CQLCodec_StartDMARead() Arm(0x%x) Host(0x%lx) len(%d) swap(%d) sync(%d) xferMode(%d) w(%d) h(%d) frame(%d)\n",
				 ulAtmAddr,
				 pHostAddr,
				 ulLength,
				 bSwap,
				 bSync,
				 ulXferMode,
				 ulPicWidth,
				 ulPicHeight,
				 IsFrameMode
				 ));

	if (bSync)
	{
		if ((QPHCI_BUS_USB != This->m_hci.m_bus_type) &&
			(QPHCI_BUS_201_EMULATION != This->m_hci.m_bus_type) &&
			(QPHCI_BUS_PCI != This->m_hci.m_bus_type)
			)
		{
			// clear any outstanding M2P interrupt
			//
			QPHCI_ClearInterrupt(&This->m_hci,
								 HCI_INTERRUPT_M2P
								 );

			// enable M2P interrupt
			//
			QPHCI_EnableInterrupt(&This->m_hci,
								  HCI_INTERRUPT_M2P
								  );
		}

		// clear M2P done
		//
		QPOSMClearEvtgrp(This->m_EvtSyncDma,
						 QLCODEC_EVENT_DMA_SYNC_READ_DONE
						 );
	}

	// start QL201 DMA
	//
	qpStatus = QPHCI_StartDMARead(&This->m_hci,
								  ulAtmAddr,
								  ulLength >> 2,
								  pHostAddr,
								  ulXferMode,
								  ulPicWidth / 16,
								  0,
								  bSwap ? (DMA_DATA_SWAP_BYTE | DMA_DATA_SWAP_WORD) : 0,
								  IsFrameMode
								  );

	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
					 "CQLCodec_StartDMARead() QPHCI_StartDMARead() Failed status(%d)\n",
					 qpStatus
					 ));
	}
#ifndef CONFIG_HOST_PLATFORM_STR8133_REF
	else
	{
		// start host DMA
		//
		if (QPHCI_BUS_CUSTOM_1 == This->m_hci.m_bus_type)
		{
			// to use external host DMA function
			// define _QPCODEC_USE_EXTERNAL_HOST_DMA_FUNC compile time switch
			// in the platform.h
			//
#ifdef _QPCODEC_USE_EXTERNAL_HOST_DMA_FUNC
			QPX_Wrap_ReadDMA(pHostAddr, ulLength, (int)bSync);
#else //!_QPCODEC_USE_EXTERNAL_HOST_DMA_FUNC
			// read the data from the streaming port
			// 16 bit at a time
			//
			PTQP_UINT32	ptr = (PTQP_UINT32)pHostAddr;
			TQP_UINT32	i;
			TQP_UINT32	ulDMALength = ulLength >> 2;
			TQP_PVOID	StreamPort;
			TQP_UINT16	vp1, vp2;

			qpStatus = QPHCI_GetStreamingPortAddress(&This->m_hci,
	 												 (TQP_PVOID *)&StreamPort
	 												 );
			if (QP_SUCCESS(qpStatus))
			{
				for (i = 0; i < ulDMALength; i++)
				{
					vp1 = *((volatile TQP_UINT16 *)StreamPort);
					vp2 = *((volatile TQP_UINT16 *)StreamPort);
					*ptr++= (vp1) | (vp2 << 16);
				}
			}
#endif //_QPCODEC_USE_EXTERNAL_HOST_DMA_FUNC
		}
	}
#endif // !CONFIG_HOST_PLATFORM_STR8133_REF

	if (bSync)
	{
		// wait for M2P done
		//
		if (QP_SUCCESS(qpStatus))
		{
			qpStatus = QPOSMWaitEvtgrp(This->m_EvtSyncDma,
									   QLCODEC_EVENT_DMA_SYNC_READ_DONE,
									   &ulEvents,
									   2000
									   );

			if (!QP_SUCCESS(qpStatus))
			{
				QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
							 "CQLCodec_StartDMARead() QPOSMWaitEvtgrp() Failed status(%d)\n",
							 qpStatus
							 ));
			}
			else
			{
				QPHCI_DMAReadDone(&This->m_hci);

				QPHCI_CopyFromCommonBuffer(&This->m_hci,
										   pHostAddr,
										   ulLength
										   );
			}
		}

		if ((QPHCI_BUS_USB != This->m_hci.m_bus_type) &&
			(QPHCI_BUS_201_EMULATION != This->m_hci.m_bus_type) &&
			(QPHCI_BUS_PCI != This->m_hci.m_bus_type)
			)
		{
			// disable M2P interrupt
			//
			QPHCI_DisableInterrupt(&This->m_hci,
								   HCI_INTERRUPT_M2P
								   );
		}

		QPOSMClearEvtgrp(This->m_EvtSyncDma,
						 QLCODEC_EVENT_DMA_SYNC_READ_DONE
						 );
	}
#ifdef _QP_REDUCE_DMA_DONE_TASK_SWITCH
	else
	{
		if ((QPHCI_BUS_CUSTOM_1 == This->m_hci.m_bus_type) ||
			(QPHCI_BUS_201_EMULATION == This->m_hci.m_bus_type)
			)
		{
			if (QP_SUCCESS(qpStatus))
			{
				This->m_pTask->CompleteData(This->m_pTask,
											CHANNEL_DIR_READ
											);
			}
		}
	}
#endif //_QP_REDUCE_DMA_DONE_TASK_SWITCH


	return (qpStatus);
}



EQP_Errors CQLCodec_SetGPIOBitDirection(CQLCodec *This,
										PGPIO_BIT_DIRECTION pBitDirection
										)
{
	EQP_Errors	qpStatus;
	TQP_UINT32	dwRegister;
	TQP_UINT32	dwMask;

	if (pBitDirection->bBitNumber > 15)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "CQLCodec_SetGPIOBitDirection() Invalid GPIO number(%d)\n",
					 pBitDirection->bBitNumber
					 ));
		return (QPERR_PARMS);
	}

	qpStatus = QPHCI_RegisterRead(&This->m_hci,
								  0x610,
								  &dwRegister
								  );
	dwMask = ~(1 << pBitDirection->bBitNumber);

	dwRegister = (dwRegister & dwMask) |
				 (pBitDirection->Direction << pBitDirection->bBitNumber)
				 ;

	qpStatus = !QP_SUCCESS(qpStatus) ? qpStatus : QPHCI_RegisterWrite(&This->m_hci,
																	  0x610,
																	  dwRegister
																	  );
	return (qpStatus);
}



EQP_Errors CQLCodec_GetGPIOBitDirection(CQLCodec *This,
										PGPIO_BIT_DIRECTION pBitDirection
										)
{
	EQP_Errors	qpStatus;
	TQP_UINT32	dwRegister;
	TQP_UINT32	dwMask;

	if (pBitDirection->bBitNumber > 15)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "CQLCodec_GetGPIOBitDirection() Invalid GPIO number(%d)\n",
					 pBitDirection->bBitNumber
					 ));
		return (QPERR_PARMS);
	}

	qpStatus = QPHCI_RegisterRead(&This->m_hci,
								  0x610,
								  &dwRegister
								  );
	dwMask = (1 << pBitDirection->bBitNumber);

	pBitDirection->Direction = (dwRegister & dwMask) ? GPIO_OUTPUT : GPIO_INPUT;

	return (qpStatus);
}



EQP_Errors CQLCodec_SetGPIOBitValue(CQLCodec *This,
									PGPIO_BIT_VALUE pBitValue
									)
{
	EQP_Errors	qpStatus;
	TQP_UINT32	dwRegister;
	TQP_UINT32	dwMask;

	if (pBitValue->bBitNumber > 15)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "CQLCodec_SetGPIOBitValue() Invalid GPIO number(%d)\n",
					 pBitValue->bBitNumber
					 ));
		return (QPERR_PARMS);
	}

	// Read the GPIO value
	//
	qpStatus = QPHCI_RegisterRead(&This->m_hci,
								  0x614,
								  &dwRegister
								  );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "CQLCodec_SetGPIOBitValue() QPHCI_RegisterRead() failed(%d)\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	dwMask = ~(1 << pBitValue->bBitNumber);

	dwRegister = (dwRegister & dwMask) |
				 (pBitValue->bValue << pBitValue->bBitNumber)
				 ;

	// Write the value to GPIO write data port
	//
	qpStatus = QPHCI_RegisterWrite(&This->m_hci,
								   0x614,
								   dwRegister
								   );
	return (qpStatus);
}



EQP_Errors CQLCodec_GetGPIOBitValue(CQLCodec *This,
									PGPIO_BIT_VALUE pBitValue
									)
{
	EQP_Errors	qpStatus;
	TQP_UINT32	dwRegister;
	TQP_UINT32	dwMask;
	TQP_UINT32	dwDirection;

	if (pBitValue->bBitNumber > 15)
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "CQLCodec_GetGPIOBitValue() Invalid GPIO number(%d)\n",
					 pBitValue->bBitNumber
					 ));
		return (QPERR_PARMS);
	}

	// Read the GPIO direction
	//
	qpStatus = QPHCI_RegisterRead(&This->m_hci,
								  0x610,
								  &dwDirection
								  );
	if (!QP_SUCCESS(qpStatus))
	{
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
					 "CQLCodec_GetGPIOBitValue() QPHCI_RegisterRead() failed(%d)\n",
					 qpStatus
					 ));
		return (qpStatus);
	}

	if (dwDirection & (1 << pBitValue->bBitNumber))
	{
		// write direction
		//
		qpStatus = QPHCI_RegisterRead(&This->m_hci,
									  0x614,
									  &dwRegister
									  );
	}
	else
	{
		// read direction
		//
		qpStatus = QPHCI_RegisterRead(&This->m_hci,
									  0x618,
									  &dwRegister
									  );
	}

	dwMask = (1 << pBitValue->bBitNumber);

	pBitValue->bValue = (dwRegister & dwMask) ? 1 : 0;

	return (qpStatus);
}



// helper
//
TQP_BOOL CQLCodec_IsCodecIdle(CQLCodec *This)
{
	TQP_BOOL	bIdle = QP_TRUE;
	int			i;

	for (i = 0; i < MAX_TASKS; i++)
	{
		if (TASK_STATE_IDLE != CTask_GetTaskState(This->m_pTask,
												  i
												  ))
		{
			bIdle = QP_FALSE;
			break;
		}
	}

	return (bIdle);
}



TQP_BOOL CQLCodec_IsCodecIdleWithoutDMAPending(CQLCodec *This)
{
	TQP_BOOL	bIdle = QP_TRUE;
	int			i;

	for (i = 0;  i < MAX_CHANNEL_DIRECTION; i++)
	{
		if (This->m_pTask->m_pPending[i])
		{
			bIdle = QP_FALSE;
			break;
		}
	}

	if (bIdle)
	{
		for (i = 0; i < MAX_TASKS; i++)
		{
			if (TASK_STATE_IDLE != CTask_GetTaskState(This->m_pTask,
													  i
													  ))
			{
				bIdle = QP_FALSE;
				break;
			}
		}
	}

	return (bIdle);
}



TQP_BOOL CQLCodec_IsCodecOpen(CQLCodec *This)
{
	TQP_BOOL	bOpen = QP_FALSE;
	int			i;

	for (i = 0; i < MAX_TASKS; i++)
	{
		if (CTask_isOpened(This->m_pTask,
						   i
						   ))
		{
			bOpen = QP_TRUE;
			break;
		}
	}

	return (bOpen);
}



void CQLCodec_ClrCodecError(CQLCodec *This)
{
	int	i;

	for (i = 0; i < MAX_TASKS; i++)
	{
		This->m_pTask->m_TaskData[i].m_Error = QPERR_SUCCESS;
	}
}



TQP_BOOL CQLCodec_IsCodecError(CQLCodec *This)
{
	int	i;

	for (i = 0; i < MAX_TASKS; i++)
	{
		if (This->m_pTask->m_TaskData[i].valid &&
			(QPERR_SUCCESS != This->m_pTask->m_TaskData[i].m_Error)
			)
		{
			return (QP_TRUE);
		}
	}

	return (QP_FALSE);
}



#ifdef _QPCODEC_CALLBACK_IN_THREAD

// CThread
//

// thread proc
//
static int CQLCodec_ThreadProc(CThread *This_p)
{
	EQP_Errors	qpStatus;
	TQP_UINT32	ulFlaggedEvents = 0;
	CQLCodec	*This = (CQLCodec *)This_p->m_context;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_ThreadProc() ENTER (0x%x) (0x%x)\n",
				 This_p,
				 This
				 ));

	while (QP_TRUE)
	{
		// wait on event
		//
		qpStatus = QPOSMWaitEvtgrp(This_p->m_EvtWait,
								   QLCODEC_EVENT_CALLBACK_THREAD_MASK,
								   &ulFlaggedEvents,
								   -1
								   );
		QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, /*QPDBGM_LEVEL_WARN*/QPDBGM_LEVEL_TRACE),
					 "CQLCodec_ThreadProc() thread woke up status(%d)\n",
					 qpStatus
					 ));

		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CQLCodec_ThreadProc() QPOSMWaitEvtgrp (%d)\n",
						 qpStatus
						 ));
			if (QPERR_TIMEOUT != qpStatus)
			{
				break;
			}
		}

		if (ulFlaggedEvents & THREAD_EVENT_EXIT)
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
						 "CQLCodec_ThreadProc() THREAD_EVENT_EXIT - BYE\n"
						 ));
			QPOSMSetEvtgrp(This_p->m_EvtReply,
						   THREAD_EVENT_EXIT_DONE
						   );
			break;
		}

		if (ulFlaggedEvents & QLCODEC_EVENT_CALLBACK_THREAD_DATA)
		{
			TQP_INT32	i;
			TQP_BOOL	Checks[MAX_CHANNELID];
			TQP_BOOL	bCheck;

			// check for callback
			//
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, /*QPDBGM_LEVEL_WARN*/QPDBGM_LEVEL_TRACE),
						 "CQLCodec_ThreadProc() QLCODEC_EVENT_CALLBACK_THREAD_DATA[\n"
						 ));

			for (i = 0; i < MAX_CHANNELID; i++)
			{
				Checks[i] = QP_TRUE;
			}

			do
			{
				bCheck = QP_FALSE;

				for (i = 0; i < MAX_CHANNELID; i++)
				{
					if (Checks[i])
					{
						Checks[i] = CChannel_GetDeviceCallback(This->m_pChannels[i]);
					}

					QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_EXTRACE),
								 "CQLCodec_ThreadProc() CChannel_GetDeviceCallback() bCheck(%d) Checks[%d](%d)\n",
								 bCheck,
								 i,
								 Checks[i]
								 ));
					bCheck |= Checks[i];
				}

			} while (bCheck);

			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, /*QPDBGM_LEVEL_WARN*/QPDBGM_LEVEL_TRACE),
						 "CQLCodec_ThreadProc() QLCODEC_EVENT_CALLBACK_THREAD_DATA]\n"
						 ));
		}
	}

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_WARN),
				 "CQLCodec_ThreadProc() See Ya!\n"
				 ));
	return (0);
}



TQP_BOOL CQLCodec_IsCallbackPending(CQLCodec *This)
{
	TQP_INT32	i;
	TQP_BOOL	bPending = QP_FALSE;

	for (i = 0; i < MAX_CHANNELID; i++)
	{
		bPending |= CChannel_CheckDeviceCallback(This->m_pChannels[i]);
	}

	return (bPending);
}


#endif //_QPCODEC_CALLBACK_IN_THREAD




// constructor
//
CQLCodec * CQLCodec_Constructor(CQLCodec *pQLCodec,
								CObject *pParent,
								TQP_UINT32 dwAttributes,
								CUsbCntl *pUsbCntl,
								CPCIeCntl *pPCIeCntl
								)
{
	EQP_Errors	qpStatus;

	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_Constructor()\n"
				 ));

	if (pQLCodec)
	{
#ifdef _QPCODEC_CALLBACK_IN_THREAD
		CThread	*pThread;
#endif //_QPCODEC_CALLBACK_IN_THREAD

		// CObject
		//
		CObject_Constructor(&pQLCodec->m_Object,
							pParent,
							dwAttributes
		  					);

		// fill in the function table
		//

		// IGenericDevice
		//
		pQLCodec->m_iMpegCodec.InitDevice = CQLCodec_InitDevice;
		pQLCodec->m_iMpegCodec.Release = CQLCodec_Release;
		pQLCodec->m_iMpegCodec.Reset = CQLCodec_Reset;
		pQLCodec->m_iMpegCodec.Set = CQLCodec_Set;
		pQLCodec->m_iMpegCodec.Get = CQLCodec_Get;

		// IMpegCodec
		//
		pQLCodec->m_iMpegCodec.AllocEncodeTask = CQLCodec_AllocEncodeTask;
		pQLCodec->m_iMpegCodec.AllocDecodeTask = CQLCodec_AllocDecodeTask;
		pQLCodec->m_iMpegCodec.ReleaseTask = CQLCodec_ReleaseTask;
		pQLCodec->m_iMpegCodec.Open = CQLCodec_Open;
		pQLCodec->m_iMpegCodec.Close = CQLCodec_Close;
		pQLCodec->m_iMpegCodec.Start = CQLCodec_Start;
		pQLCodec->m_iMpegCodec.Stop = CQLCodec_Stop;
		pQLCodec->m_iMpegCodec.Acquire = CQLCodec_Acquire;
		pQLCodec->m_iMpegCodec.Pause = CQLCodec_Pause;
		pQLCodec->m_iMpegCodec.Step = CQLCodec_Step;
		pQLCodec->m_iMpegCodec.SetRate = CQLCodec_SetRate;
		pQLCodec->m_iMpegCodec.GetRate = CQLCodec_GetRate;
		pQLCodec->m_iMpegCodec.BeginFlush = CQLCodec_BeginFlush;
		pQLCodec->m_iMpegCodec.Flush = CQLCodec_Flush;
		pQLCodec->m_iMpegCodec.EndFlush = CQLCodec_EndFlush;
		pQLCodec->m_iMpegCodec.AddBuffer = CQLCodec_AddBuffer;
		pQLCodec->m_iMpegCodec.CancelBuffer = CQLCodec_CancelBuffer;
		pQLCodec->m_iMpegCodec.TimeoutBuffer = CQLCodec_TimeoutBuffer;
		pQLCodec->m_iMpegCodec.GetTime = CQLCodec_GetTime;
		pQLCodec->m_iMpegCodec.XferData = CQLCodec_XferData;


		// HCI
		//
		pQLCodec->m_bHCIInited = QP_FALSE;

		// initialize member variables
		//
		pQLCodec->m_pUsbCntl = pUsbCntl;
		pQLCodec->m_pPCIeCntl = pPCIeCntl;

		pQLCodec->m_pDeviceCallback = QP_NULL;
		pQLCodec->m_callbackContext = QP_NULL;

		pQLCodec->m_pTask = QP_NULL;

		// get build in firmware
		//
		pQLCodec->m_pVideoFW = QP_NULL;
		pQLCodec->m_pAudioFW = QP_NULL;
		pQLCodec->m_QL201FWSize = 0;
		pQLCodec->m_QL201AudFWSize = 0;
		pQLCodec->m_bVideoFWUpdated = QP_FALSE;
		pQLCodec->m_bAudioFWUpdated = QP_FALSE;

		// interrupt
		//
		pQLCodec->m_bInterruptAttached = QP_FALSE;
		pQLCodec->m_interruptNumber = QPOS_ISR_CODEC;
		pQLCodec->m_isrID = QP_NULL;

		// memory settings
		//
		pQLCodec->m_MemType = QPCODEC_MEM_TYPE_mDDR;
		pQLCodec->m_MemSize = 512;	// in units of Mb

		// chip type
		//
		pQLCodec->m_ChipType = QPCODEC_CHIP_BERINGER;

		// api version
		//
		pQLCodec->m_VerFwAPI = 0;

		// firmware configuration
		//
		pQLCodec->m_FwIntMode = QPPF_CODECLIBINTMODE_HIU_ISR;	// HIU_ISR
		pQLCodec->m_FwFixedMode = 0;	// fixed mode

		// GPIO settings
		//
		pQLCodec->m_GPIODirections = 0;
		pQLCodec->m_GPIOValues = 0;

		// Mutexs and other synchronization objects
		//
		pQLCodec->m_SemaphoreFWAPI = QP_NULL;
		pQLCodec->m_EvtTask = QP_NULL;
		pQLCodec->m_EvtTaskReply = QP_NULL;
		pQLCodec->m_EvtSyncDma = QP_NULL;
#ifdef _QPCODEC_CALLBACK_IN_THREAD
		pQLCodec->m_EvtCallbackThread = QP_NULL;
		pQLCodec->m_EvtCallbackThreadReply = QP_NULL;
#endif //_QPCODEC_CALLBACK_IN_THREAD

		// power state
		//
		pQLCodec->m_PowerState = QLCODEC_PWR_STATE_D3;

		// error recovery
		//
		pQLCodec->m_ErrorRecovery = QPCODEC_ERROR_RECOVERY_ON_ERROR;

		// usb default mode
		//
		pQLCodec->m_USBDefaultMode = QPPF_USBLIBGADGETMODE_FX2CLONE;

		// use artesa ext 340
		pQLCodec->m_UseArtesaExt340 = 0;

		// filter out HDCP content
		pQLCodec->m_FilterOutHDCPContent = 0;

		// create synchronization objects for task
		//
		qpStatus = QPOSMCreateEvtgrp(&pQLCodec->m_EvtTask,
									 0
									 );
		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CQLCodec_Constructor() QPOSMCreateEvtgrp() m_EvtTask Failed status(%d)!!!!\n",
						 qpStatus
						 ));
			CQLCodec_Destructor(pQLCodec);
			return (QP_NULL);
		}

		qpStatus = QPOSMCreateEvtgrp(&pQLCodec->m_EvtTaskReply,
									 0
									 );
		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CQLCodec_Constructor() QPOSMCreateEvtgrp() m_EvtTaskReply Failed status(%d)!!!!\n",
						 qpStatus
						 ));
			CQLCodec_Destructor(pQLCodec);
			return (QP_NULL);
		}

		// create streaming task
		//
		pQLCodec->m_pTask = (CTask *)QPMMMalloc2Ex(sizeof(CTask));
		if (!CTask_Constructor(pQLCodec->m_pTask,
							   &pQLCodec->m_Object,
							   OBJECT_CRITICAL_LIGHT,
							   QPOS_THRPRI_HIGHEST,
							   pQLCodec,
							   pQLCodec->m_EvtTask,
							   pQLCodec->m_EvtTaskReply
							   ))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CQLCodec_Constructor() CTask_Constructor Failed!!!!\n"
						 ));
			CQLCodec_Destructor(pQLCodec);
			return (QP_NULL);
		}

		// create channel manager
		//
		pQLCodec->m_pChannelMgr = (CObjectMgr *)QPMMMalloc2Ex(sizeof(CObjectMgr));
		if (!CObjectMgr_Constructor(pQLCodec->m_pChannelMgr,
									&pQLCodec->m_Object,
									OBJECT_CRITICAL_HEAVY,
									QP_NULL
									))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CQLCodec_Constructor() CObjectMgr_Constructor Failed!!!!\n"
						 ));
			CQLCodec_Destructor(pQLCodec);
			return (QP_NULL);
		}

		// Create synchronization objects
		//
		qpStatus = QPOSMCreateSem(&pQLCodec->m_SemaphoreFWAPI,
								  1,
								  1
								  );
		if (!QP_SUCCESS(qpStatus))
		{
			pQLCodec->m_SemaphoreFWAPI = QP_NULL;
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CQLCodec_Constructor() QPOSMCreateSem() Failed status(%d)!!!!\n",
						 qpStatus
						 ));
			CQLCodec_Destructor(pQLCodec);
			return (QP_NULL);
		}

		qpStatus = QPOSMCreateEvtgrp(&pQLCodec->m_EvtSyncDma,
									 0
									 );
		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CQLCodec_Constructor() QPOSMCreateEvtgrp() m_EvtSyncDma Failed status(%d)!!!!\n",
						 qpStatus
						 ));
			CQLCodec_Destructor(pQLCodec);
			return (QP_NULL);
		}

#ifdef _QPCODEC_CALLBACK_IN_THREAD

		qpStatus = QPOSMCreateEvtgrp(&pQLCodec->m_EvtCallbackThread,
									 0
									 );
		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CQLCodec_Constructor() QPOSMCreateEvtgrp() m_EvtCallbackThread Failed status(%d)!!!!\n",
						 qpStatus
						 ));
			CQLCodec_Destructor(pQLCodec);
			return (QP_NULL);
		}

		qpStatus = QPOSMCreateEvtgrp(&pQLCodec->m_EvtCallbackThreadReply,
									 0
									 );
		if (!QP_SUCCESS(qpStatus))
		{
			QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_ERROR),
						 "CQLCodec_Constructor() QPOSMCreateEvtgrp() m_EvtCallbackThreadReply Failed status(%d)!!!!\n",
						 qpStatus
						 ));
			CQLCodec_Destructor(pQLCodec);
			return (QP_NULL);
		}

		pThread = CThread_Constructor(&pQLCodec->m_Thread,
									  "CQLCodec_ThreadProc",
									  (TQP_PVOID)pQLCodec,
									  QP_CODECTHR_PRI,
									  pQLCodec->m_EvtCallbackThread,
									  pQLCodec->m_EvtCallbackThreadReply,
									  QP_TRUE
									  );
		if (!pThread)
		{
			CObject_Destructor(&pQLCodec->m_Object);
			return (QP_NULL);
		}

		// fill in the function table
		//

		// CThread
		pQLCodec->m_Thread.m_ThreadProc = CQLCodec_ThreadProc;

#endif //_QPCODEC_CALLBACK_IN_THREAD
	}

	return (pQLCodec);
}



void CQLCodec_Destructor(CQLCodec *This)
{
	QPDbgMPrintf((QPDBGMCOMPLVL(g_QPCodecDBGCompID, QPDBGM_LEVEL_TRACE),
				 "CQLCodec_Destructor()\n"
				 ));

	QPGenericDevice_Release(&This->m_iMpegCodec);

	// clean up
	//
	if (This->m_pChannelMgr)
	{
		CObjectMgr_Destructor(This->m_pChannelMgr);
		QPMMFree2Ex((void *)This->m_pChannelMgr);
		This->m_pChannelMgr = QP_NULL;
	}

	// destroy encoder and decoder task
	//
	if (This->m_pTask)
	{
		CTask_Destructor(This->m_pTask);
		QPMMFree2Ex((void *)This->m_pTask);
		This->m_pTask = QP_NULL;
	}

	// destroy synchronization objects
	//
	if (This->m_SemaphoreFWAPI)
	{
		QPOSMDeleteSem(This->m_SemaphoreFWAPI);
		This->m_SemaphoreFWAPI = QP_NULL;
	}

	if (This->m_EvtTask)
	{
		QPOSMDeleteEvtgrp(This->m_EvtTask);
		This->m_EvtTask = QP_NULL;
	}

	if (This->m_EvtTaskReply)
	{
		QPOSMDeleteEvtgrp(This->m_EvtTaskReply);
		This->m_EvtTaskReply = QP_NULL;
	}

	if (This->m_EvtSyncDma)
	{
		QPOSMDeleteEvtgrp(This->m_EvtSyncDma);
		This->m_EvtSyncDma = QP_NULL;
	}

#ifdef _QPCODEC_CALLBACK_IN_THREAD
	if (This->m_EvtCallbackThread)
	{
		QPOSMDeleteEvtgrp(This->m_EvtCallbackThread);
		This->m_EvtCallbackThread = QP_NULL;
	}

	if (This->m_EvtCallbackThreadReply)
	{
		QPOSMDeleteEvtgrp(This->m_EvtCallbackThreadReply);
		This->m_EvtCallbackThreadReply = QP_NULL;
	}
#endif //_QPCODEC_CALLBACK_IN_THREAD


	if (This->m_pVideoFW)
	{
		// delete the previously allocated buffer
		//
		if (This->m_bVideoFWUpdated)
			QPMMFree2Ex(This->m_pVideoFW);
		else
			QPPFMGetVideoFW(QP_NULL,
							QP_NULL
							);
	}

	This->m_pVideoFW = QP_NULL;
	This->m_QL201FWSize = 0;
	This->m_bVideoFWUpdated = QP_FALSE;


	if (This->m_pAudioFW)
	{
		// delete the previous allocated buffer
		//
		if (This->m_bAudioFWUpdated)
			QPMMFree2Ex(This->m_pAudioFW);
		else
			QPPFMGetAudioFW(QP_NULL,
							QP_NULL
							);
	}

	This->m_pAudioFW = QP_NULL;
	This->m_QL201AudFWSize = 0;
	This->m_bAudioFWUpdated = QP_FALSE;

#ifdef _QPCODEC_CALLBACK_IN_THREAD
	// make sure we destroy the thread
	//
	CThread_ThreadDone(&This->m_Thread);
	CThread_Destructor(&This->m_Thread);
#endif //_QPCODEC_CALLBACK_IN_THREAD

	// CObject
	//
	CObject_Destructor(&This->m_Object);
}





